/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.util.BodyClickListener");

(function() {
	
dojo.require("jazz.util.array");

var array = jazz.util.array;
var _bodyClickHandlers = new Array();
var _clickEvent = null;

/*
 * PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * future with minimal or no warning.
 * 
 * @since 0.7
 */
jazz.util.BodyClickListener.register = function(func, priority) {
	/*	summary:
	 * 		register a function to be alerted whenever a link is clicked on the page.  
	 * 		Provides the opportunity to veto the link click.  
	 *	func: function that is notified when a link is clicked. Return a shouldVeto boolean.  
	 *		true to ignore link click, false or null to continue.
	 *	priority: higher number = higher priority 
	 *	return: handler object.
	 */
	if (_bodyClickHandlers.length == 0) {
		_clickEvent = dojo.connect(dojo.body(), "onclick", null, _handleBodyClick);
	}
	var newHandler = {f: func, p: priority};
	array.insertByPriority(_bodyClickHandlers, newHandler);
	return newHandler;
};

jazz.util.BodyClickListener.unregister = function(handler) {
	/*	summary:
	 * 		unregister your body click listener
	 * 	handler:  the handler object, returned by the register function
	 * 	return: success boolean
	 */
	var success = array.removeByReference(_bodyClickHandlers, handler);
	if (_bodyClickHandlers.length == 0) {
		dojo.disconnect(_clickEvent);
	} 
	return success;
};

jazz.util.BodyClickListener.askForVetos = function(href) {
	for (var i = 0; i < _bodyClickHandlers.length; i++) {
		var shouldVeto = _bodyClickHandlers[i].f(href);
		if (shouldVeto) {
			return true;
		}
	}
	return false;
};

function _handleBodyClick(e) {
	if (e.altKey || e.ctrlKey || e.shiftKey) {
		return;
	}
	if (e.target) {
		var href, node = e.target;
		while (node) {
			if (node.tagName === "A") {
				href = node.href;
				break;
			}
			node = node.parentNode;
		}
		if (!href) return;
	} else {
		return;
	}
	if (jazz.util.BodyClickListener.askForVetos(href)) {
		dojo.stopEvent(e);
	}
}

})();