/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.ButtonGroup");

(function(){

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.util.ui");

/**
 * Constructs a new ButtonGroup that equalizes the width of buttons.  This can be
 * defined in a template or constructed dynamically.
 * 
 * @name jazz.ui.ButtonGroup
 * @class <p>A widget that equalizes the width of buttons.</p>
 * 	<p>PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * 	future with minimal or no warning.</p>
 * @constructor
 * @extends dijit._Widget
 * @extends dijit._Templated
 */ 

dojo.declare("jazz.ui.ButtonGroup", [dijit._Widget, dijit._Templated],  /** @lends jazz.ui.ButtonGroup.prototype */ {

	/**
	 * Specify an array of <button> dom nodes to be equalized. Can also be specified in a template:
	 *   ex. <div dojotype="jazz.ui.ButtonGroup"><button>My Button</button><button>My Wider Button</button></div>
	 *   
	 * @type Array
	 * @default null
	 */
	buttons: null,

	// Internals
	
	templateString: "<div dojoattachpoint=\"containerNode\"></div>",
	
	postCreate: function() {
		if (!this.buttons) {
			var buttons = dojo.query("> button", this.domNode);
			if (buttons && buttons.length > 0) {
				this.buttons = buttons;
				this._updateWidths();
			}
		}
	},
	_setButtonsAttr: function(buttons) {
		if (!dojo.isArray(buttons)) {
			console.error("[ButtonGroup] buttons argument must be an array");
			return;
		}
		dojo.empty(this.domNode);
		for (var i = 0; i < buttons.length; i++) {
			dojo.place(buttons[i], this.domNode);
		}
		this.buttons = buttons;
		this._updateWidths();
	},
	_updateWidths: function() {
		var buttons = this.buttons;
		var strings = []; 
		for (var i = 0; i < buttons.length; i++) {
			strings.push(buttons[i].innerHTML);
		}
		var width = this._getMaxWidth(strings);
		for (var j = 0; j < buttons.length; j++) {
			buttons[j].style.width = width + "px";
		}
	},
	_getMaxWidth: function(strings) {
		var button = dojo.create("button", {}, document.body);
		var maxWidth = 0;
		for (var i = 0; i < strings.length; i++) {
			button.innerHTML = strings[i];
			var width = button.clientWidth;
			if (width > maxWidth) maxWidth = width;
		}
		document.body.removeChild(button);
		return maxWidth;
	}
});
})();