/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("jazz.opensocial.Gadget");
	
	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");
	dojo.require("dojo._base.json");
	
	dojo.require("jazz.opensocial.GadgetMetadata");
	dojo.require("opensocial.internal.container.core");
	dojo.require("opensocial.internal.container.rpc");
	dojo.require("opensocial.internal.container.pubsub");
	
	var GADGET_RENDER_URL = net.jazz.ajax._contextRoot + net.jazz.ajax._webuiPrefix + 'services/gadgetRender';
	
	dojo.declare("jazz.opensocial.Gadget", [dijit._Widget, dijit._Templated], {
		/**
		 * summary:
		 * 	Widget used to host an OpenSocial Gadget.
		 * 
		 * description:
		 * 	An IFrame is used to embed an OpenSocial Gadget
		 * 
		 * parameters:
		 * 	url (String)
		 * 		The url of the gadget.
		 * 
		 * 	moduleId (Integer)
		 * 		An identifier for this gadget instance.
		 * 
		 * 	secureToken (String)
		 * 		A signed string identifying the complete context in which this gadget resides.
		 * 
		 * 	[prefs] (Object)
		 * 		Optional parameter containing the value of previously-set User Preferences.
		 */
		
		templatePath: dojo.moduleUrl("jazz.opensocial", "Gadget.html"),
		
		_url: null,
		_moduleId: null,
		_secureToken: null,
		_title:null,
		_lang: null,
		_view: null,
		
		constructor: function(args) {
			this._url = _ExtractFromArgs(args, 'url');
			this._moduleId = _ExtractFromArgs(args, 'moduleId', 0);
			this._view = _ExtractFromArgs(args, 'view', 'default');
			this._secureToken = _ExtractFromArgs(args, 'token', 'MISSING_TOKEN');
			this._prefs = _ExtractFromArgs(args, 'prefs', {});
		},
		
		postCreate: function() {
			this.inherited(arguments);
			_RunInOrder(
					[this._requestRenderURL,
					this._loadPrereqs,
					this._initFrame,
					this._renderCurrentView],
					this);
		},
		
		// to be overriden by consumers, will be called when the gadget metadata is available
		onGadgetMetadata: function(metadata) {
		},
		
		onGadgetPrefs: function(delta) {
			dojo.mixin(this._prefs, delta);
		},
		
		setGadgetHeight: function(height) {
			this._iFrame.style.height = height + 'px';
		},
		
		getTitle: function() {
			return this._title;
		},
		
		setTitle: function(title) {
			if (this._title == title)
				return;
			this._title = title;
			this.onTitle(title);
		},
		
		onTitle: function(title) {
			/*
			 * summary:
			 * 	User defined function to handle title updates.
			 * tags:
			 * 	callback 
			 */
		},
		
		onLocation: function(url) {
			this._iFrame.src = url;
		},
		
		navigateTo: function(/* String */ view, /*optional*/ params) {
			this._view = view;
			this._viewParams = params;
			this._renderCurrentView();
		},
		
		_initFrame: function() {
			//ISSUE why does Shindig define proprietary API in the OpenSocial namespace?
			//ISSUE Why is this even necessary?  Should be done for free by core features.
			gadgets.rpc.setRelayUrl(this.id,
				_GetParentUrl() + '/gadgets/files/container/rpc_relay.html');
			
			if (!this._scrolling)
				this._iFrame.scrolling = 'no';
			
			this.setTitle(this._initialTitle);
		},
		
		_renderCurrentView: function() {
			var url = this._renderBase + "/web/services/gadgetRender" +
					'?container=default' +
					'&lang=default' +
					'&country=DEFAULT' +
					'&view=' + this._view +
					'&url=' + encodeURIComponent(this._url) +
					'&nocache=true' +
					'&parent=' + encodeURIComponent(_GetParentUrl()) +
					'&st=' + encodeURIComponent(this._secureToken);
			if (djConfig.isDebug)
				url += '&debug=true';
			var prefs = this._prefs;
			for (var key in prefs) {
				if (!prefs.hasOwnProperty(key))
					continue;
				var value = prefs[key];
				
				//ISSUE: Shindig's prefs.js doesn't currently decode preference keys
				key = encodeURIComponent(key);
				
				url += '&up_' + key + '=' + encodeURIComponent(value);
			}
			
			var h = this._viewInfo[this._view].h;
			
			if (h > 0)
				this.setGadgetHeight(h);
			
			this.onLocation(url);
		},
		
		_requestRenderURL: function(callback) {
			function processMetadata(metadata) {
				this._renderUrl = metadata._iframe
				//		+ "&debug=1"
				//		+ "&nocache=true"
						+ "&parent=" + encodeURIComponent(_GetParentUrl())
						+ "&st=" + encodeURIComponent(this._secureToken);
				this._initialTitle = metadata.getTitle();
				this._secureToken = metadata.getSecureToken();
				this._renderBase = metadata.getRenderBase();
				
				var info = {}, v, views = metadata.getViews();
				for (var i in views) {
					v = views[i];
					info[v.name] = {h: v.h}
				}
				// if there is no 'default' view, fall back to 'home'
				if (!info["default"] && info["home"] && this._view === "default")
					this._view = "home";
				this._viewInfo = info;
				this._scrolling = metadata.getScrolling();
				this._secureToken = metadata.getSecureToken();
				this.onGadgetMetadata(metadata);
				callback();
			}
			
			new jazz.opensocial.GadgetMetadata({
				url: this._url,
				moduleId: this._moduleId,
				callback: processMetadata,
				callbackScope: this
			});
			return callback; //Indicate async by returning "true"
		},
		
		_loadPrereqs: function() {
			//TODO - make sure the container-side javascript features have been loaded
		}
	}); //END dojo.declare

	dojo.provide("jazz.opensocial.Gadget");

	function _GetParentUrl() {
		return document.location.protocol + '//' + document.location.host;
	}
	
	gadgets.rpc.register('resize_iframe',
		function _CallbackSetHeight(height) {
			var gadget = dijit.byId(this.f);
			gadget.setGadgetHeight(height);
		});
	
	gadgets.rpc.register('set_title',
		function _CallbackSetTitle(title) {
			var gadget = dijit.byId(this.f);
			gadget.setTitle(title);
		});
	
	gadgets.rpc.register('requestNavigateTo',
		function _CallbackRequestNavigateTo(view, params) {
			var gadget = dijit.byId(this.f);
			gadget.navigateTo(view, params);
		});
	
	gadgets.rpc.register('set_pref',
		function _CallbackSetPrefs(editToken, name, value) {
			var gadget = dijit.byId(this.f);
			var prefs = {};
			for (var i = 1; i < arguments.length; i += 2)
				prefs[arguments[i]] = arguments[i + 1];
			gadget.onGadgetPrefs(prefs);
		});
	
	gadgets.pubsubrouter.init(function(id) {
			return 'sender unavailable';
		});
	
	/**
	 * Calls a sequence of functions where each function is possibly asynchronous.  All of the functions
	 * accept a single parameter which is a callback.  If the function is synchronous, it simply returns
	 * with a value logically equivalent to false.  Asynchronous functions must return true and then
	 * invoke the provided callback.
	 * @param {Object} funcs
	 * @param {Object} defaultScope
	 */
	function _RunInOrder(funcs, defaultScope) {
		var i = 0;
		function loop() {
			while (i < funcs.length) {
				var scope = defaultScope || window;
				if (!dojo.isFunction(funcs[i])) 
					scope = funcs[i++];
				if (funcs[i++].call(scope, loop)) 
					return;
			}
		}
		loop();
	}
	
	function _ExtractFromArgs(/*Map*/args, /*String*/ id, /*Object*/ optDefault) {
		if (!(id in args)) {
			if (optDefault === undefined) 
				throw new Error('Missing required argument: "' + id + '"');
			return optDefault;
		}
		try {
			return args[id];
		}
		finally {
			delete args[id];
		}
	}
	
})();
