/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.require("jazz.app.proxy");
	dojo.require("jazz.client.xhr");
	dojo.require("dojo._base.json");
	
	var URL_METADATA = net.jazz.ajax._contextRoot + net.jazz.ajax._webuiPrefix + "services/gadgetMetadata";
	
	var CONTEXT = {
		country: "default",
		language: "default",
		view: "profile",
		container: "default",
		debug: true
	};
	
	function _ExtractFromArgs(/*Map*/args, /*String*/ id, /*Object*/ optDefault){
		if (!(id in args)) {
			if (optDefault === undefined) 
				throw new Error('Missing required argument: "' + id + '"');
			return optDefault;
		}
		try {
			return args[id];
		}
		finally {
			delete args[id];
		}
	}
	
	dojo.declare("jazz.opensocial.GadgetMetadata", null, {
		/**
		 * summary:
		 * 	Provides access to data from a gadget's declaration.
		 * 
		 * description:
		 * 	The data is fetched asynchronously, and a callback is invoked when the metadata is
		 * 	available.  The implementation may batch the fetching of metadata for multiple
		 * 	instances.  There is no guarantee to the order in which callbacks are invoked.
		 * 
		 * parameters:
		 * 	url (String)
		 * 		The url of the gadget.
		 * 
		 * 	callback (Function)
		 * 		A callback function to be invoked when the metadata is available.  For convenience,
		 * 		<code>this</code> is provided as the only argument.
		 * 
		 * 	[callbackScope] (Object)
		 * 		An optional scope in which to call the callback function. The default
		 * 		value is <code>window</code>.
		 * 
		 * 	[moduleId] 
		 * 		Optional parameter which clients should not use.  The module id has no
		 * 		affect on metadata.
		 * 
		 * 	[prefs]
		 * 		Optional parameter which clients should not use.  User prefs have no impact on 
		 * 		metadata.
		 */
		
		_url: null,
		_thumbnail: null,
		_title: null,
		_required: null,
		_optional: null,
		_vScroll: null,
		_st: null,
		
	//Not really metadata, only used during rendering a gadget
		_iframe: null,
		_id: 0,
		
		constructor: function(args) {
			this._url = _ExtractFromArgs(args, 'url');
			this._callback = _ExtractFromArgs(args, 'callback');
			this._callbackScope = _ExtractFromArgs(args, 'callbackScope', null);
			this._id = _ExtractFromArgs(args, 'moduleId', 0);
			this._prefs = _ExtractFromArgs(args, 'prefs', {});
			
			var request = {
				context: CONTEXT,
				gadgets: [{
					url: this._url,
					moduleId: this._id,
					prefs: this._prefs
				}]
			}
			
			var args = {
				url: URL_METADATA,
				headers : {
					"Content-Type" : "text/json"
				},
				postData: dojo.toJson(request)
			}
			
			var dfd = jazz.client.rawXhrPost(args);
			var self = this;
			dfd.addBoth(function(result) {
				var meta = dojo.fromJson(result);
				meta = meta.gadgets[0];
				self._processMetadata(meta);
			});
		},
		
		_processMetadata: function(meta) {
			this._title = meta.title;
			this._desc = meta.description;
			this._thumbnail = meta.thumbnail;
			//debugger;
			this._domain = meta.domain;
			this._height = meta.height;
			this._scrolling = meta.scrolling;
			this._st = meta.st;
			this._views = [];
			for (v in meta.views)
				this._views.push({
					name:v,
					w:meta.views[v].preferredWidth,
					h:meta.views[v].preferredHeight});
			this._required = meta.features;
			this._userPrefs = meta.userPrefs;
			
			this._callback.call(this._callbackScope, this);
		},
		
		getDescription: function() {
			return this._desc;
		},
		
		getHeight: function() {
			return this._height;
		},
		
		getScrolling: function() {
			return this._scrolling;
		},
		
		getViews: function() {
			return this._views;
		},
		
		getThumbnailUrl: function(){
			/**
			 * summary:
			 * 	Returns the URL (as a String) for the thumbnail if specified
			 */
			return this._thumbnail;
		},
		
		getTitle: function() {
			/**
			 * summary:
			 * 	Returns the Default Title as a String if specified
			 */
			return this._title;
		},
		
		getSecureToken : function() {
			return this._st;
		},
		
		getUserPrefs: function() {
			return this._userPrefs;
		}, 
		
		getRenderBase : function() {
			return (this._domain || "") + net.jazz.ajax._contextRoot;
		}
	});

	dojo.provide("jazz.opensocial.GadgetMetadata");
	
})();
