/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.ImagePreview");

dojo.require("jazz.ui.HoverPopup");

dojo.require("dojo.i18n");
dojo.requireLocalization("jazz.ui", "ImagePreviewMessages");

(function() {

var messages = dojo.i18n.getLocalization("jazz.ui", "ImagePreviewMessages");

/**
 * Constructs a new ImagePreview
 * @name jazz.ui.ImagePreview
 * @class <p>Shows an image in a HoverPopup on hover of a node. Commonly used on links to 
 *  images so they can be shown in place. The max width and height of the widget are 80%
 *  of the viewport width.</p>
 * 	<p>PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * 	future with minimal or no warning.</p>
 * @constructor
 * @extends jazz.ui.HoverPopup
 * @since 0.7
 */ 
dojo.declare("jazz.ui.ImagePreview", jazz.ui.HoverPopup, /** @lends jazz.ui.ImagePreview.prototype */{
	/**
	 * The url for the image. 
	 * @field 
	 * @type String
	 */
	src: null,
	
	/**
	 * The filename of the image to show in the top of the ImagePreview. 
	 * @field 
	 * @type String
	 */
	filename: null,
	
	/**
	 * If true, the title and open links in the preview will cause the image to open in a
	 * new window instead of the current one.
	 * @field 
	 * @type Boolean
	 */
	openInNew: true,
	 
	/**
	 * A function that will be registered as the click handler for the title and open links
	 * in the preview. This allows consumers to prevent the default link behavior and
	 * completely customize how the widget handles link clicks.
	 * @field 
	 * @type Function
	 */
	onOpenClick: null,

	/** @private */
	createContents: function(tip) {
		var root = dojo.create("div", {className: "jazz-ui-ImagePreview"});
		var header = dojo.create("div", {className: "header"}, root);
		
		var titleDiv = dojo.create("div", {className: "title"}, header);
		var title = dojo.create("a", {href: this.src}, titleDiv);
		title.appendChild(document.createTextNode(this.filename));
		
		var openDiv = dojo.create("div", {className: "open"}, header);
		var open = dojo.create("a", {innerHTML: messages.open, href: this.src}, openDiv);
		if (this.openInNew) {
			title.target = open.target = "_new";
		}
		if (this.onOpenClick) {
			this.connect(open,"onclick",this.onOpenClick);
			this.connect(title,"onclick",this.onOpenClick);
		}
		
		this._contentArea = dojo.create("div", {className: "loading", innerHTML: messages.loading}, root);
		this._imageTag = document.createElement("img");
		this._imageTag.src = this.src;
		this.managedConnect(this._imageTag, "onclick", this, this.close);
		this.poll();
		
		return root;
	},
	
	/** @private */
	poll: function() {
		if (this._imageTag.complete) {
			var div = document.createElement("div");
			div.style.position = "absolute";
			div.style.visibility = "hidden";
			div.style.left = "-1000px";
			div.style.top = "0";
			div.appendChild(this._imageTag);
			document.body.appendChild(div);
			var width = div.clientWidth;
			var height = div.clientHeight;
			document.body.removeChild(div);
			div.removeChild(this._imageTag);
			this._contentArea.style.width = width + "px";
			this._contentArea.style.height = height + "px";
			if (this._contentArea.firstChild) {
				this._contentArea.replaceChild(this._imageTag, this._contentArea.firstChild);
				while (this._contentArea.firstChild.nextSibling) 
					this._contentArea.removeChild(this._contentArea.firstChild.nextSibling);
			}
			else {
				this._contentArea.appendChild(this._imageTag);
			}
			this.position(this._target);
		}
		else 
			setTimeout(dojo.hitch(this, this.poll), 200);
	},
	
	/** @private */
	updateDimensions: function(v) {
		this.maxWidth = parseInt(v.w * 0.8);
		this.maxHeight = parseInt(v.h * 0.8);
	}

});

})();
