/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.app.JazzUI");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("net.jazz.ajax.ui.PageContainer");
dojo.require("jazz.app.internal.Banner");
dojo.require("jazz.app.internal.Navbar");
dojo.require("jazz.app.internal.Footer");
dojo.require("jazz.util.ResizeNotifier");
dojo.require("jazz.app.internal.ProgressIndicator");

(function() {

var PageContainer = net.jazz.ajax.ui.PageContainer;

/*
 * This class is deprecated. Use jazz.ui.PageTemplate instead.
 * If you use this, your application will not conform to jazz style guidelines.
 */

dojo.declare("jazz.app.JazzUI", [dijit._Widget, dijit._Templated], {
	/*
	 * 	summary:
	 * 		UI widget for Jazz-based Web UIs.
	 * 
	 *  description: 
	 *   	Contains three internal sub-widgets:
	 *   		1) banner - the banner at the top of the page.
	 *   		2) navbar - the navbar below the header, above the content area.  Defaults 
	 *   			to page registry links if they exist. 
	 *   		3) footer: widget at the bottom of the page.  
	 *   			Defined in template.
	 *   	Do not call or manipulate these widgets directly. Use APIs or references on JazzUI.
	 *   
	 *   DOM node references (use these to add custom dom to the layout):
	 *   	- bannerProductTitle, bannerRightText
	 *    	- navbarLeft, navbarRight
	 *   	- footerLeft, footerMid, footerRight
	 *   
	 *   since:
	 *   	0.7
	 */
	
	init: function(args) {
		var buttons = args.buttons;
		for (var i = 0; buttons && i < buttons.length; i++) {
			if (buttons[i].pageId && buttons[i].suppressPage) {
				args.suppressedPages.push(buttons[i].pageId);
			}
		}
		this._banner.init(args);
		this._navbar.init(args);
		this._footer.init(args.footer);
	
		this._resizeHandler = jazz.util.ResizeNotifier.register({context: this, callback: this._onResize, force: true, priority: 0});
		setTimeout(dojo.hitch(this,this._onResize),0);
	},
	
	_pinMinibar : function (pin)
	{
		this._minibarCell.style.width = this._minibar.getWidth()+"px";	
		jazz.util.ResizeNotifier.poke();
	},
	
	setContent: function(widget) {
		// summary:
		//		Sets the content area domNode. Pass a widget or an object with a
		//		domNode property.
		// since:
		//		0.7
		this._pageContainer._viewPage(widget);
	},
	
	// private functions
	
	templatePath: dojo.moduleUrl("jazz.app", "internal/templates/JazzUI.html"),
	
	postCreate: function() {	
		dojo.deprecated("jazz.app.JazzUI is deprecated.", "Use jazz.ui.PageTemplate instead.", "3.0.1");					
		this._banner = new jazz.app.internal.Banner({}, this._banner);
		this._navbar = new jazz.app.internal.Navbar({}, this._navbar);
		this._footer = new jazz.app.internal.Footer({}, this._footer);
		
		this._progressIndicator = new jazz.app.internal.ProgressIndicator({});
		this._content.appendChild(this._progressIndicator.domNode);
		
		this._pageContainer = new PageContainer({});
		this._content.appendChild(this._pageContainer.domNode);
		
		//setup dom node references for product customization
		this.bannerProductTitle = this._banner._productTitle; 	
		this.bannerRightText = this._banner._rightText;
		
		this.navbarLeft = this._navbar._leftLinks;
		this.navbarRight = this._navbar._rightLinks;
		
		this.footerLeft = this._footer._leftLogo;
		this.footerRight = this._footer._rightLogo;
		this.footerMid = this._footer._footerLinks;
	},
	_onResize: function() {
		var height = dojo.position(document.documentElement).h - dojo.position(this._banner.domNode).h - dojo.position(this._navbar.domNode).h - dojo.position(this._footer.domNode).h - 8;
		this._content.style[(dojo.isIE >= 8) ? "minHeight" : "height"] = height + "px";
	},
	_skipToContent: function(e) {
		e.preventDefault();
		this._content.focus();
	},
	destroy: function() {
		jazz.util.ResizeNotifier.unregister(this._resizeHandler);
		if (this._banner) this._banner.destroy();
		if (this._navbar) this._navbar.destroy();
		if (this._footer) this._footer.destroy();
		if (this._progressIndicator) this._progressIndicator.destroy();
		if (this._pageContainer) this._pageContainer.destroy();
		this.inherited(arguments);
	}
});
})();