/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.ListBox");

/*
 * Imports 
 */
dojo.require("dojo.i18n"); 
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.requireLocalization("jazz.ui", "ListBoxMessages");

(function() {

/*
 * Aliases 
 */

/*
 * TODOs 
 */
// Feature: Listener framework
// Support replace operation if ID is already defined

/*
 * Class 
 *
 * PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * future with minimal or no warning.
 * 
 * @since 0.7
 */
dojo.declare("jazz.ui.ListBox",
	[dijit._Widget, dijit._Templated], {

	/*
	 * Properties
	 */	
	width: "",
	height: "250px",
	margin: "5px",
	
	/*
	 * Specify the entry field to use when sorting list box entries
	 * Options: "label", "value", "id", "type"
	 */		
	sortField: "label",	
	
	/*
	 * Specify the sort order of the list box entries
	 * Options: 
	 * 0:  Ascending 
	 * 1:  Descending
	 * 2:  None
	 */		
	sortOrder: 0,	
	
	/* 
 	 * Specify the callback function to use when an entry is double-clicked
 	 * 
 	 * Function (entry)
 	 * 
 	 * @param Object entry
 	 *				. String label
 	 * 				. String value
 	 * 				. String id
 	 * 				. Number type	 
 	 */
	ondblClick: null,

	/* 
 	 * Specify the callback function to use when an entry is clicked
 	 * 
 	 * Function (entry)
 	 * 
 	 * @param Object entry
 	 *				. String label
 	 * 				. String value
 	 * 				. String id
 	 * 				. Number type	 
 	 */
	onClick: null,

	/*
	 * Specify whether to allow multiple selections using the CTRL and SHIFT
	 * keys
	 */	
	allowMultipleSelections: true,

	/*
 	 * Specify icons for list box entries by providing a map of entry types 
 	 * associated with icon pathes.
 	 * 
 	 * Array iconMap : [Number entryType] = String iconPath
 	 */	
	iconMap: [],

	/*
	 * Templates
	 */	
	templatePath: dojo.moduleUrl("jazz.ui", "templates/ListBox.html"),
	
	/*
	 * Static Variable Container 
	 */
	_static: function() {}, 	

	/*
 	 * Constructor
 	 * 
 	 * @param Object args
 	 * 
 	 * @return ListBox
 	 */
	constructor: function(args) {
		
		// Messages //------------------------------------------------------------->
		
		this.messages = dojo.i18n.getLocalization("jazz.ui", "ListBoxMessages");
		this.msgLoading = this.messages.msgLoading;
		
	},
	
	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 * @overrides super
 	 */	
	postCreate: function() {
		// Call super
		this.inherited(arguments);
		
		// Static Variables //------------------------------------------------->
		this._initializeStaticConstants();

		// Instance Variables //----------------------------------------------->
		this._domMap = {};
		this._dataMap = {};
		this._dataArray = [];
		this._selected = {};

		this._entryCount = 0;

		this._loadingFlag = false;
		
		this._lastSelectedEntryID = null;

		// Initialization //--------------------------------------------------->
		
		if (dojo.isIE) {
			this._selectHandler = this.connect(this.domNode, "onselectstart", this._handleEventSelectStart);
		}

		if (this.height) {
			this.domNode.style.height = this.height;
		}
		if (this.width) {
			this.domNode.style.width = this.width;
		}
		if (this.margin) {
			this.domNode.style.margin = this.margin;
		}
		
	},

	/*
	 * Private Method
	 *
	 * @param void
	 * @return void
	 */
	_focusFirst: function() {
		this._contentArea.firstChild.focus();
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_initializeStaticConstants: function() {
		// Track the number of viewlet instances
		if (this._static._instanceCount == undefined) {
			this._static._instanceCount = 1;
			// Proceed to initialize static constants
		} else {
			this._static._instanceCount++;
			// Static constants already initialized
			return;
		}

		this._static._CLASSNAME = "jazz.ui.ListBox";
		
		this._static._SORT_ORDER_ASCENDING = 0;
		this._static._SORT_ORDER_DESCENDING = 1;
		this._static._SORT_ORDER_NONE = 2;
		
	},

	/*
 	 * Public Method
 	 * 
 	 * @param Object entry
 	 * 				. String label
 	 * 				. String value
 	 * 				. String id
 	 * 				. Number type
 	 * @return void
 	 */	
	addEntry: function(entry) {
		// Validate list		
		if (this._isObjectDefined(this.domNode) === false) {
			return;
		}

		var id = entry.id;
		// Check if the ID is already in use
		if (this._isObjectDefined(this._dataMap[id])) {
			// Ignore
			return;
		}

		// Create DOM representation
		// Using an anchor instead of a div to allow :hover without IE performance degradation
		var entryNode = document.createElement("a");
		entryNode.href = "#";
		dojo.style(entryNode, 'display', 'block');
		entryNode.setAttribute("entryid", id);
		entryNode.setAttribute("tabindex", -1);
		dojo.addClass(entryNode, "entry");
		this.connect(entryNode, "onmousedown", this._handleEventSelectEntry);
		this._addIcon(entryNode, this.iconMap[entry.type]);
		this._addLabel(entryNode, entry.label);

		// Add double-click listener
		if (this.ondblClick !== null) {
			this.connect(entryNode, "ondblclick", this._handleEventDoubleClickEntry);
		}

		// Add click listener
		if (this.onClick) {
			this.connect(entryNode, "onclick", this._handleEventClickEntry);
		} else {
			this.connect(entryNode, "onclick", dojo.stopEvent);
		}

		// Add to data map
		this._dataMap[id] = entry;
		// Add to DOM map		
		this._domMap[id] = entryNode;
		// Add to DOM itself
		this._addNode(this._contentArea, entryNode, entry);
		// Add to selection map
		this._selected[id] = false;
		// Update entry count
		this._entryCount++;
	},
	
	sort: function(comparator) {
		// disable automatic sorting
		this.sortOrder = this._static._SORT_ORDER_NONE;
		// sort using the provided comparator
		this._dataArray.sort(comparator);
		// reorder the children off-dom
		this.domNode.removeChild(this._contentArea);
		for (var i = 0; i < this._dataArray.length; i++) {
			this._contentArea.appendChild(this._domMap[this._dataArray[i].id]);
		}
		this.domNode.appendChild(this._contentArea);
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param Element parent
 	 * @param String iconPath
 	 * @return void
 	 */	
	_addIcon: function(parent, iconPath) {
		
		if (this._isStringDefined(iconPath) === false) {
			return;	
		}
		
		var icon = document.createElement("span");
		dojo.addClass(icon, "entry-icon");
		var image = document.createElement("img");
		dojo.addClass(image, "entry-image");
		image.setAttribute("src", iconPath);
		image.setAttribute("alt", this.messages.msgIcon);
		
		icon.appendChild(image);	
		parent.appendChild(icon);	
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param Element parent
 	 * @param String labelText
 	 * @return void
 	 */	
	_addLabel: function(parent, labelText) {
		var label = document.createElement("span");
		dojo.addClass(label, "entry-label");
		label.innerHTML = labelText;
		
		parent.appendChild(label);
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Element parent
 	 * @param Element targetChild
 	 * @param Object targetEntry
 	 * 				. String label
 	 * 				. String value
 	 * 				. String id
 	 * 				. Number type
 	 * 
 	 * @return void
 	 */	
	_addNode: function(parent, targetChild, targetEntry) {

		if (parent.childNodes.length === 0) {
			// Only one entry
			parent.appendChild(targetChild);
			this._dataArray.push(targetEntry);
			return;
		}
		
		if (this.sortOrder === this._static._SORT_ORDER_ASCENDING) {
			// Ascending
			this._addNodeInOrder(parent, targetChild, targetEntry, this._ascendingCompareFunction);
		} else if (this.sortOrder === this._static._SORT_ORDER_DESCENDING) {
			// Descending
			this._addNodeInOrder(parent, targetChild, targetEntry, this._descendingCompareFunction);
		} else {
			// None
			parent.appendChild(targetChild);
			this._dataArray.push(targetEntry);
		}		

	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Element parent
 	 * @param Element targetChild
 	 * @param Object targetEntry
 	 * 				. String label
 	 * 				. String value
 	 * 				. String id
 	 * 				. Number type
 	 * @param Function compareFunction(
 	 * 						Object currentEntry,
 	 *								. String label
 	 * 								. String value
 	 * 								. String id
 	 * 								. Number type
 	 * 						Object targetEntry
 	 * 								. String label
 	 * 								. String value
 	 * 								. String id
 	 * 								. Number type
 	 * 					)
 	 * 
 	 * @return void
 	 */	
	_addNodeInOrder: function(parent, targetChild, targetEntry, compareFunction) {

		var children = parent.childNodes;

		// Binary insert
		var low = 0;
		var high = children.length;
		while (low < high) {
			var mid = Math.floor((low + high) / 2);
			
			var currentChild = children[mid];
			var currentEntry = this._dataMap[dojo.attr(currentChild, "entryid")];
			
			if (compareFunction(this, currentEntry, targetEntry)) {
				low = mid + 1; 
			} else {
				high = mid; 
			}
		}
		
		if (this._isObjectDefined(children[low])) {
			parent.insertBefore(targetChild, children[low]);
			this._dataArray.splice(low,0,targetEntry);
		} else {
			parent.appendChild(targetChild);
			this._dataArray.push(targetEntry);
		}

	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object context
 	 * @param Object currentEntry
 	 * 				. String label
 	 * 				. String value
 	 * 				. String id
 	 * 				. Number type
 	 * @param Object targetEntry
 	 * 				. String label
 	 * 				. String value
 	 * 				. String id
 	 * 				. Number type
 	 * 
 	 * @return void
 	 */		
	_ascendingCompareFunction: function(context, currentEntry, targetEntry) {
		if (currentEntry[context.sortField] < targetEntry[context.sortField]) {
			return true;
		}
		return false;
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object context
 	 * @param Object currentEntry
 	 * 				. String label
 	 * 				. String value
 	 * 				. String id
 	 * 				. Number type
 	 * @param Object targetEntry
 	 * 				. String label
 	 * 				. String value
 	 * 				. String id
 	 * 				. Number type
 	 * 
 	 * @return void
 	 */		
	_descendingCompareFunction: function(context, currentEntry, targetEntry) {
		if (currentEntry[context.sortField] > targetEntry[context.sortField]) {
			return true;
		}
		return false;
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return boolean
 	 */	
	_handleEventSelectEntry: function(event) {
		if (!event) {
			return false;
		}
		dojo.stopEvent(event);
		// Get target
		if (this._hasProperty(event, "currentTarget") === false) {
			return false;
		}
		var target = event.currentTarget;
		if (this._isObjectDefined(target) === false) {
			return false;
		}
		// Get ID
		var id = dojo.attr(target, "entryid");
		// Get entry
		var entry = this._dataMap[id];
		// Determine selection
		var doSelect = true;
		if (this._isSelected(entry)) {
			doSelect = false;
		}
		
		// Clear previous selection unless CTRL is being pressed
		if (!this.allowMultipleSelections || 
			!event.ctrlKey) {
			this.removeAllSelections();
		}
		
		// Do selection
		this._setSelected(entry, doSelect);

		if (this.allowMultipleSelections && 
			event.shiftKey) {
			// Perform multi-selection
			this._doMultiSelectShiftKey(id);
		} else {
			// Remember this entry as the last selection
			this._lastSelectedEntryID = id;
		}
		
		return true;
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param String targetSelectedEntryID
 	 * 
 	 * @return void
 	 */	
	_doMultiSelectShiftKey: function(targetSelectedEntryID) {
		// Get current entry DOM nodes
		var children = this._contentArea.childNodes;

		if (children.length <= 1) {
			// Not enough entries to do multi-select
			return;
		}
		// Entries >= 2
		
		if (this.getEntry(this._lastSelectedEntryID) === null) {
			// No last selected entry
			// Use the first entry as the last selected entry
			this._lastSelectedEntryID = children[0].entryid;
		}

		if (this._lastSelectedEntryID === targetSelectedEntryID) {
			// No range to select, the start and end entries are the same
			// Select the single entry
			this._setSelected(this.getEntry(targetSelectedEntryID), true);
			return;
		}

		var foundLastSelectedEntry = false;
		var foundTargetSelectedEntry = false;
		var doSelect = false;
		
		// Find the start and end entries and select the intermediate range
		for (var index = 0; index < children.length; index++) {
			// Get the ID of the current entry
			var currentEntryID = dojo.attr(children[index], 'entryid');
			
			if (currentEntryID === this._lastSelectedEntryID) {
				// Found the start or end entry of the selection range
				foundLastSelectedEntry = true;
				// Select it
				this._setSelected(this.getEntry(currentEntryID), true);
				// Set flag to select all intermediate entries encountered
				// after this
				doSelect = true;
				// Exit if this is the end entry of the selection range
				if (foundTargetSelectedEntry) {
					break;
				}
			} else if (currentEntryID === targetSelectedEntryID) {
				// Found the start or end entry of the selection range
				foundTargetSelectedEntry = true;
				// Select it
				this._setSelected(this.getEntry(currentEntryID), true);
				// Set flag to select all intermediate entries encountered
				// after this
				doSelect = true;
				// Exit if this is the end entry of the selection range
				if (foundLastSelectedEntry) {
					break;
				}
			} else if (doSelect) {
				// Start entry of the selection range was found 
				// Select intermediate entry
				this._setSelected(this.getEntry(currentEntryID), true);
			}
		}
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return boolean
 	 */	
	_handleEventDoubleClickEntry: function(event) {
		dojo.stopEvent(event);

		// Get target
		if (this._hasProperty(event, "currentTarget") === false) {
			return false;
		}
		var target = event.currentTarget;
		if (this._isObjectDefined(target) === false) {
			return false;
		}

		// Get ID
		var id = dojo.attr(target, "entryid");

		// Get entry
		var entry = this._dataMap[id];
		
		// Execute callback
		this.ondblClick(entry);

		return true;
	},		

	/*
 	 * @param Event event
 	 * 
 	 * @return boolean
 	 */	
	_handleEventClickEntry: function(event) {
		
		if (event) {
			dojo.stopEvent(event);
		} else {
			return false;
		}

		// Get current target
		var currentTarget = event.currentTarget;
		if (!currentTarget) {
			return false;
		}

		// Get ID
		var id = dojo.attr(currentTarget, "entryid");
		if (!id) {
			return false;
		}

		// Get entry
		var entry = this._dataMap[id];
		if (!entry) {
			return false;
		}
		
		// Execute callback
		this.onClick(entry);

		return true;
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return boolean
 	 */	
	_handleEventSelectStart: function(event) {
		if (event) {
			dojo.stopEvent(event);
		}
		// Prevent text selection on IE during multi-selection
		return false;
	},
	
	/*
 	 * Public Method
 	 * 
 	 * @param Array entries : Object
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 * @return void
 	 */	
	addEntries: function(entries) {
		for (var index = 0; index < entries.length; index++) {
			this.addEntry(entries[index]);
		}
	},	

	/*
 	 * Public Method
 	 * 
 	 * @param Object entry
 	 * 				. String label
 	 * 				. String value
 	 * 				. String id
 	 * 				. Number type
 	 * @return void
 	 */	
	removeEntry: function(entry) {
		// Validate list		
		if (this._isObjectDefined(this.domNode) === false) {
			return;
		}
		
		var id = entry.id;
		// Check if this operation is applicable
		if (this._isObjectDefined(this._dataMap[id]) === false) {
			// NO-OP
			return;
		}
		// Remove from the ordered representation
		for (var i = 0; i < this._dataArray.length; i++) {
			if (this._dataArray[i].id === id) {
				this._dataArray.splice(i,1);
				break;
			}
		}
		// Remove from list
		var node = this._domMap[id];
		this._contentArea.removeChild(node);
		// Remove from data map
		this._dataMap[id] = undefined;
		delete this._dataMap[id];
		// Remove from DOM map		
		this._domMap[id] = undefined;
		delete this._domMap[id];
		// Remove from selection map
		this._selected[id] = undefined;
		delete this._selected[id];
		// Update entry count
		this._entryCount--;

	},

	/*
 	 * Public Method
 	 * 
 	 * @param Array entries : Object
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 * @return void
 	 */	
	removeEntries: function(entries) {
		for (var index = 0; index < entries.length; index++) {
			this.removeEntry(entries[index]);
		}
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	removeAllEntries: function() {
		for (var id in this._dataMap) {
			this.removeEntry(this._dataMap[id]);
		}
	},

	/*
 	 * Public Method
 	 * 
 	 * @param Object entry
 	 * 				. String label
 	 * 				. String value
 	 * 				. String id
 	 * 				. Number type
 	 * @param Boolean show
 	 * 
 	 * @return void
 	 */	
	showEntry: function(entry, show) {
		// Validate list		
		if (this._isObjectDefined(this.domNode) === false) {
			return;
		}
		
		var id = entry.id;
		// Check if this operation is applicable
		if (this._isObjectDefined(this._dataMap[id]) === false) {
			// NO-OP
			return;
		}

		var node = this._domMap[id];
		this._updateVisibility(node, show);	
		
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return Number
 	 */	
	getEntryCount: function() {
		return this._entryCount;
	},
	
	/*
 	 * Public Method
 	 * 
 	 * @param String id
 	 * @return Object
 	 * 				. String label
 	 * 				. String value
 	 * 				. String id
 	 * 				. Number type
 	 */	
	getEntry: function(id) {
		if (this._isObjectDefined(this._dataMap[id]) === false) {
			return null;
		}
		return this._dataMap[id];
	},	

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return Array : Object
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 */	
	getAllEntries: function() {
		return dojo.clone(this._dataArray);
	},	
	
	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return Array : Object
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 */	
	getSelection: function() {
		var entryList = new Array();
		for (var i = 0; i < this._dataArray.length; i++) {
			if (this._selected[this._dataArray[i].id]) {
				entryList.push(this._dataArray[i]);
			}
		}
		return entryList;
	},		
	
	/*
 	 * Public Method
 	 * 
 	 * @param Array entries : Object
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 * @param boolean select
 	 * @return void
 	 */	
	setSelection: function(entries, select) {
		for (var index = 0; index < entries.length; index++) {
			this._setSelected(entries[index], select);
		}
	},		

	/*
 	 * @param Object entry
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 * @param boolean select
 	 * @return void
 	 */	
	_setSelected: function(entry, select) {
		var id = entry.id;
		// Check if this operation is applicable
		if (this._isObjectDefined(this._dataMap[id]) === false) {
			// NO-OP
			return;
		}
		// Check if a toggle is necessary
		if (this._selected[id] == select) {
			// NO-OP
			return;
		}
		
		// Store selection		
		this._selected[id] = select;
		// Update UI
		var node = this._domMap[id];
		if (select) {
			dojo.addClass(node, "j-selected");
		} else {
			dojo.removeClass(node, "j-selected");
		}
		
	},	

	/*
 	 * @param Object entry
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 * @return boolean
 	 */	
	_isSelected: function(entry) {
		if (this._selected[entry.id]) {
			return true;
		}
		return false;
	},	

	/*
 	 * Public Method
	 *
 	 * @param void
 	 * @return void
 	 */	
	removeAllSelections: function() {
		for (var id in this._selected) {
			if (this._selected[id]) {
				// Get entry
				var entry = this._dataMap[id];
				// Unselect it
				this._setSelected(entry, false);
			}
		}
	},	

	/*
 	 * Public Method
 	 * 
 	 * @param boolean show
 	 * @return void
 	 */	
	showLoading: function(show) {
		if (this._isObjectDefined(this.domNode) === false) {
			// Not initialized
			return;
		} else if (this._loadingFlag === show) {
			// Action already done
			return;
		} else if (show) {
			// Show message area
			this._showMessageArea();
		} else {
			// Show content area
			this._showContentArea();
		}
		
		// Update flag
		this._loadingFlag = show;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_showMessageArea: function() {
		// Hide the content area
		this._updateVisibility(this._contentArea, false);
		// Show the message area		
		this._updateVisibility(this._messageArea, true);		
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_showContentArea: function() {
		// Hide the message area
		this._updateVisibility(this._messageArea, false);	
		// Show the content area
		this._updateVisibility(this._contentArea, true);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Element node
 	 * @param boolean show
 	 * @return void
 	 */	
	_updateVisibility: function(node, show) {
		if (show) {
			// Show
			if (node.style.display != "block") {
				node.style.display = "block";
			}
		} else {
			// Hide
			if (node.style.display != "none") {
				node.style.display = "none";
			}
		}
	},
		
	/*
 	 * Private Method
 	 * 
 	 * @param Object variable
 	 * @return boolean
 	 */	
	_isObjectDefined: function(variable) {
	    if (typeof(variable) != 'object') {
			return false;
		} else if (variable === null) {
			return false;
		}
		return true;
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param String text
 	 * @return boolean
 	 */	
	_isStringDefined: function(text) {
	    if (typeof(text) != 'string') {
			return false;
		} else if (text.length === 0) {
			return false;
		}		
		return true;
	},			
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object variable
 	 * @param String property
 	 * @return boolean
 	 */	
	_hasProperty: function(variable, property) {
		if (property in variable) {
			return true;
		}
		return false;
	},		
	
	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 * @overrides super
 	 */		
	destroy: function() {
		// Disconnect handlers
		if (this._selectHandler) {
			this.disconnect(this._selectHandler);
			delete this._selectHandler;
			this._selectHandler = null;
		}
		// Call super
		this.inherited(arguments);
	},
	
	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return String
 	 */		
	toString: function() {
		return this._static._CLASSNAME;
	}		
	
});

})();
