/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("jazz.ui.MenuPopup");

dojo.require("dijit._Widget");
dojo.require("jazz.util.moduleResource");
dojo.require("dojo.i18n");
dojo.requireLocalization("jazz.ui", "MenuPopupMessages");

var messages = dojo.i18n.getLocalization("jazz.ui", "MenuPopupMessages"),
	_currentOpen = null, 
	_currentOver = null, 
	_deferred = null;

dojo.declare("jazz.ui.MenuPopup", [dijit._Widget], {

	align: "left",
	around: null,
	cacheMenu: false,
	caretSrc: "jazz.ui/graphics/Toolbar/white-caret.gif",
	caretSrcDirty: "jazz.ui/graphics/Toolbar/blue-caret.gif",
	destroyOnClose: false,
	group: "",
	injectCaret: false,
	menuClass: "",
	menuProvider: null,
	openClass: "menu-open",
	openOnHover: false,
	openOnCaretClick: false,
	
	openDelay: 300,
	closeDelay: 800,
	closeBufferPx: 5,
	
	_menuOpen: false,

	postCreate: function() {
		dojo.addClass(this.domNode, "jazz-ui-MenuPopup");
		if (this.group || this.openOnHover) this.connect(this.domNode, "onmouseover", this._onMouseOver);
		this.connect(this.domNode, "onclick", this._onClickHandler);
		this.connect(this.domNode, "onkeypress", this._handleKeypress);
		
		if (this.injectCaret && this.caretSrc){
			this._caret = dojo.create("img", {
				"class": "caret",
				src: jazz.util.moduleResource(this.caretSrc)
			}, this.focusNode||this.domNode);
			if (this.openOnCaretClick) {
				this.connect(this._caret, "onclick", this._onCaretClick);
			}
		}
		
		if (!this.around) {
			this.around = this.domNode;
		}
		dijit.setWaiState(this.domNode, "haspopup", "true");
	},
	
	open: function() {
		if (_currentOpen) {
			if (_currentOpen == this) {  // menu is already open.
				if (!this.openOnHover) this.close(false);
				return;
			} else {
				dojo.hitch(_currentOpen, _currentOpen.close)(false);	
			}
		}
		_currentOver = this;
		this.disconnect(this._mouseMoveEvent);
		this._mouseMoveEvent = null;
		
		if (this._menuOpen) {
			dojo.hitch(this._currentMenu, this._currentMenu.onOpen)();
			return;
		}
		if (_deferred) {
			_deferred.cancel();
		}
		if (this.cacheMenu && this._currentMenu) {
			var menu = this._currentMenu;  //there's a menu from last time. Re-use it.
		} else {
			try {
				menu = this.menuProvider(this);
				if (menu instanceof dojo.Deferred) {
					menu.addCallback(dojo.hitch(this,this._deferredOpen));
					this._deferred = _deferred = menu;
					this.domNode.style.cursor = "progress";
					return;
				}
			} catch(e) {
				console.error(e);
			}
			if (!menu) return;
			this._currentMenu = menu;
			dojo.addClass(menu.domNode, "jazz-ui-MenuPopup-menu");
			dojo.addClass(menu.domNode, this.menuClass);
			dijit.setWaiState(menu.domNode, "labelledby", this.id);
			dijit.setWaiRole(menu.domNode, "menu");
		}
		var exec = dojo.hitch(this, this.close);
		this.domNode.focus();
		dijit.popup.open({
			popup: menu, 
			parent: this, 
			around: this.around, 
			orient: (this.align == "right") ? {"BR":"TR"}: {"BL":"TL"}, 
			onExecute: exec,
			onCancel: exec
		});
		if (menu.onReflow) this._reflow = this.connect(menu, "onReflow", this._onResize);
		if (!this._resizeHandle) {
			this._resizeHandle = jazz.util.ResizeNotifier.register({
				context: this,
				callback: this._onResize,
				force: true,
				publishEvents: false
			});
		}
		this._menuOpen = true;
		_currentOpen = this;
		dojo.addClass(this.around,this.openClass);
		this._bodyClickEvent = this.connect(document.documentElement, "onclick", this._bodyClick);
		if (menu.startup) menu.startup();
		dijit.setWaiState(this.domNode, "expanded", "true");
	},

	destroy: function() {
		this.close(false);
		delete this.around;
		this.inherited(arguments);
	},
	close: function(focus) {
		var currentMenu = this._currentMenu;
		if (currentMenu && this._menuOpen) {
			if (currentMenu.domNode) {
				dijit.popup.close(currentMenu);
			}
			if (this._resizeHandle) {
				jazz.util.ResizeNotifier.unregister(this._resizeHandle);
				delete this._resizeHandle;
			}
			dijit.setWaiState(this.domNode, "expanded", "false");
			this._menuOpen = false;
			if (this.destroyOnClose) {
				if (currentMenu.domNode && currentMenu.destroyRecursive) {
					currentMenu.destroyRecursive();
				}
				this._currentMenu = null;
			}
			dojo.removeClass(this.around, this.openClass);
			this.disconnect(this._mouseMoveEvent);
			this.disconnect(this._bodyClickEvent);
			this.disconnect(this._reflow);
			clearTimeout(this._closeTimeout);
			clearTimeout(this._overTimeout);
			this._bodyClickEvent = null;
			this._mouseMoveEvent = null;
			this._closeTimeout = null;
			this._overTimeout = null;
			this._reflow = null;
			_currentOpen = null;
			if (focus === false) return;
			dijit.focus(this.focusNode||this.domNode);
		}
	},

	setDirty: function(dirty) {
		if (!this.injectCaret) {
			console.warn("[MenuPopup] setDirty not supported when injectCaret is false");
			return;
		}
		if (dirty && !this._caretWrapper) {
			this._caretWrapper = dojo.create("span", {
				"class": "caret-wrapper",
				title: messages.unsaved
			}, this.domNode);
			dojo.place(this._caret,this._caretWrapper);
			this._caret.src = jazz.util.moduleResource(this.caretSrcDirty);
		} else if (!dirty && this._caretWrapper) {
			this._caret.src = jazz.util.moduleResource(this.caretSrc);
			dojo.place(this._caret,this.focusNode||this.domNode);
			this.domNode.removeChild(this._caretWrapper);
			delete this._caretWrapper;
		}
	},
	
	// private functions
	
	_onMouseOver: function(event) {
		dojo.stopEvent(event);
		if (_currentOpen) {
			if (_currentOpen == this || _currentOpen.group !== this.group) {
				return;
			}
			dojo.hitch(_currentOpen, _currentOpen.close)(false);
			this.open();
			
			//prevent accidental closing of the menu when clicking on a recently opened menu anchor.
			this._preventClose = true;
			setTimeout(dojo.hitch(this, function(){delete this._preventClose}),500);
		} else if (this.openOnHover && !this._overTimeout) {
			_currentOver = this;
			this._overTimeout = setTimeout(dojo.hitch(this,this._deferredOpen),this.openDelay);
		}
		if (this.openOnHover) {
			if (!this._mouseMoveEvent) this._mouseMoveEvent = this.connect(dojo.doc, "onmousemove", this._mouseMove);
		}
	},
	
	_mouseMove: function(event) {
		var stayOpen = false;
		if (this._menuOpen) {
			var pc = dojo.position(this._currentMenu.domNode),
				mouseX = event.clientX,
				mouseY = event.clientY,
				BUFFER = this.closeBufferPx;
			if (pc.x - BUFFER <= mouseX && pc.x + pc.w + BUFFER >= mouseX && pc.y - BUFFER <= mouseY && pc.y + pc.h + BUFFER >= mouseY) {
				stayOpen = true;
			}
		}
		if ((this._menuOpen && stayOpen) || dojo.isDescendant(event.target, this.around)) {
			//stay open
			clearTimeout(this._closeTimeout);
			this._closeTimeout = null;
			return;	
		} else {
			//close
			if (this._menuOpen) {
				var canDismiss = this._currentMenu.canDismiss;
				if (dojo.isFunction(canDismiss)){
					if (dojo.hitch(this._currentMenu, canDismiss)() === false) {
						//the popup doesn't want to be dismissed yet.
						return;
					}
				}
				// wait delay and kill popup.
				if (!this._closeTimeout) {
					this._closeTimeout = setTimeout(dojo.hitch(this,this._deferredKill), this.closeDelay);	
				}
			} else {
				this.disconnect(this._mouseMoveEvent);
				this._mouseMoveEvent = null;
				clearTimeout(this._overTimeout);
				this._overTimeout = null;
			}
		}		
	},
	
	_deferredKill: function() {
		this.disconnect(this._mouseMoveEvent);
		this._mouseMoveEvent = null;
		this.close();
	},
	
	_onCaretClick: function(event) {
		this._onClickHandler(event, true);
	},
	
	_onClickHandler: function(event, caretClick) {
		if (!this.openOnCaretClick || caretClick) {
			if (event) dojo.stopEvent(event);
			if (this._preventClose) return;
			this.open();
		} else {
			if (this._preventClose) return;
			this.close(false);
			clearTimeout(this._overTimeout);
			this._overTimeout = null;
		}
	},
	
	_deferredOpen: function(event) {
		clearTimeout(this._overTimeout);
		this._overTimeout = null;
		this.domNode.style.cursor = "";
		if (_currentOver && _currentOver != this) {
			// the timeout fired here, but the mouse is over another menu
			if (_deferred && _deferred == this._deferred) {
				_deferred = null;
			}
			dojo.hitch(_currentOver, _currentOver.open)(); // open the other menu
		} else {
			this.open();
		}
		_currentOver = null;
	},
	_bodyClick: function(event){
		if (dojo.isDescendant(event.target, this.around) || dojo.isDescendant(event.target, this._currentMenu.domNode)) {
			return;
		}
		this.close(false);
	},
	_handleKeypress: function(e) {
		var k = dojo.keys;
		switch (e.charOrCode) {
			case k.UP_ARROW:
			case k.DOWN_ARROW:
			case k.SPACE:
				if (!this._menuOpen) this.open();
				e.preventDefault();
				break;
		}
		switch (e.charOrCode) {
			case k.UP_ARROW:
				this._currentMenu.focus("end");
				break;
			case k.DOWN_ARROW:
				this._currentMenu.focus("start");
				break;
			case k.TAB:
				if (this._menuOpen) {
					this._currentMenu.focus();
					e.preventDefault();
				}
				break;
			case k.ESCAPE:
			case k.LEFT_ARROW:
			case k.RIGHT_ARROW:
				if (this._menuOpen) this.close();
				break;
		}
	},
	_onResize: function(e){
		var menu = this._currentMenu;
		if (menu && menu.domNode && this._menuOpen) {
			var node = menu.domNode.parentNode;
			if (!node) return;
			var pos = dojo.position(node, true);
			var aroundPos = dojo.position(this.around, true);
			if (this.align == "right") {
				var left = Math.max(0, aroundPos.x + aroundPos.w - pos.w);
			} else {
				var w = dojo.position(document.body).w;
				if (aroundPos.x + pos.w > w) {
					left = w - pos.w;
				} else {
					left = aroundPos.x;	
				}
			}
			var top = aroundPos.y + aroundPos.h;
			node.style.top = top + "px";
			node.style.left = left + "px";
		}
	}
});
dojo.mixin(jazz.ui.MenuPopup, {
	closeAll: function() {
		if (_currentOpen) dojo.hitch(_currentOpen,_currentOpen.close)();
	}
});
})();