/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rdm.fronting.server.web.migration.rrm.MigrationStatusStep");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.ui.Button");
dojo.require("jazz.ui.wizard.WizardStepHeader");
dojo.require("jazz.ui.wizard.WizardStep");
dojo.require("jazz.ui.styledBox.TrimStyles");
dojo.require("jazz.ui.StyledBox");
dojo.require("dojo.i18n");

(function() {
dojo.declare("com.ibm.rdm.fronting.server.web.migration.rrm.MigrationStatusStep", 
    [jazz.ui.wizard.WizardStep, dijit._Widget, dijit._Templated], {

    templatePath: dojo.moduleUrl("com.ibm.rdm.fronting.server.web.migration.rrm", "MigrationStatusStep-template.html"),
    widgetsInTemplate: true,
    innerBoxTrim:  jazz.ui.styledBox.TrimStyles.EDITOR,
    migrationInfo: undefined,
    serverInfo: undefined,
    isFinished: false,
    isFailure: false,
    
    constructor: function() {
       this.messages = messages = { migrationLaunchButtonLabel: "launch migration",
                                    currentRelease: "",
                                    requiredRelease: ""  };
        this.stepTitle = "Migration Status";
    },

    postCreate: function() {
        this.header = new jazz.ui.wizard.WizardStepHeader({stepTitle: this.stepTitle});
        this.stepHeaderArea.appendChild(this.header.domNode);
    },
    
    // called by the wizard
    focusStep: function() {
        this.isFinished = false;
        this.isFailure = false;
        this.initializeMigration();
    },
    
    // Submits the migration task to the migration service.  This returns a task URI that 
    //  is used to monitor progress. 
    initializeMigration: function() {
        this.migrationInfo["async"] = "true";
        
        var self = this;
        
        if (!this.hasProcessed) {      
            jazz.client.rawXhrPost({
                url: this.serverInfo.rrmMigrationServiceUrl,
                headers: {
                    "DoorsRP-Request-Type": "private", //$NON-NLS-1$
                    "Milestone-Migration" : "true" //$NON-NLS-1$
                },
                content: this.migrationInfo,
                load: function(response, ioargs) {
                    self.displayStatus(ioargs.xhr.getResponseHeader("Location"));
                },
                error: function(error, ioargs) {
                   switch (ioargs.xhr.status) {
                    case 412:
                       self.migrationStatusInfoBox.innerHTML = "Migration currently in progress. Please try again."; 
                       break;
                    default:
                       var message = "<p>" + error + "</p><p> Use the 'back' button to re-launch the migration. Please try again.</p>"; 
                       self.migrationStatusInfoBox.innerHTML = message;
                       console.log(error);
                       break;
                    }
                    self.isFailure = true;
                    self.triggerButtonStateUpdate();
                }
             }); 
        } else {
            this.migrationLaunchButton.disabled = false;
        }
     },
         
    // displays the current task progress message from the task URI
    // This gets called recursively until it receives an error or the task is complete.     
    displayStatus: function(taskLocation) {     
        var self = this;
        
        this._currentMessage = "";
        
        var result = jazz.client.xhrGet({
            url: taskLocation,
            sync: true,
            handleAs: "xml",
            headers: {
                "Accept": "*",
                "DoorsRP-Request-Type": "private" 
            },
            load: function(responseObject, ioargs){
                 var taskProgress;
                 for (var x = 0; x < responseObject.documentElement.childNodes.length; x++) {
                    var childNode = responseObject.documentElement.childNodes[x];
                    if (childNode.nodeType == 1) {
                        for (var y = 0; y < childNode.childNodes.length; y++) {
                            var gchildNode = childNode.childNodes[y];
                            if (gchildNode.nodeType == 1) {
                                if("rm:taskProgress" == gchildNode.nodeName){
                                    taskProgress = gchildNode;
                                }
                            }
                        }
                    }
                }
                
                var textNode = taskProgress.childNodes[0];
                var status = textNode.nodeValue;
                if("Error" != status){
                    switch (ioargs.xhr.status) {
                        default:
                            var msg = "<p>This operation may take between a few minutes and more than an hour. The 'Finish' button will activate upon completion.</p>";
                                msg += "<p>N.B. RRC should not be accessed until this is complete.</p>" 
                            self.migrationStatusMessageBox.innerHTML = msg;
                            
                            if(self._currentMessage == status){
                                self.migrationStatusInfoBox.innerHTML += ".";
                            } else {
                                self.migrationStatusInfoBox.innerHTML = status.replace(/\n/gi, "<br>") + "%";
                            }
                            self._currentMessage = status;
                            break;
                    } 
                    var segments = status.split(" ");
                    if("100" != segments[segments.length-1]){
                        setTimeout(function() { 
                           self.displayStatus(taskLocation); 
                        }, 2000);
                    } else {
                      self.migrationStatusMessageBox.innerHTML = "";
                      var msg = status.replace(/\n/gi, "<br>") + "%";
                      
                      //msg += "<p><b>N.B.</b> For this particular migration, the number of resource updates should be very low if not zero. ";
                      //msg += "Additionally, the errors noted above are not of enough consequence to prevent the correct functioning of RRC.  It is ";
                      //msg += "however recommended to retain the migration archive for reference.</p>";
                      
                      msg += "<p>Migration is now complete.</p>";
                      self.migrationStatusInfoBox.innerHTML = msg;
                      self.isFinished = true;
                      self.triggerButtonStateUpdate();
                    }
                }
            },
            error: function(error, ioargs){
                switch (ioargs.xhr.status) {
                      default:
                          console.log(error);
                          break;
                } 
            }
        });
    },
    //
    // net.jazz.ui.Wizard callbacks
    //
    canFlipToNextPage: function() {
        return false;
    },
    
    canFlipToPreviousPage: function() {
        return this.isFailure;
    },
    
    canFinish: function() {
        return this.isFinished;
    }
});

})();