/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.PagedTable");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.ui.toolbar.Button");
dojo.require("jazz.util.i18n");
dojo.require("dojo.i18n");
dojo.requireLocalization("jazz.ui", "PagedTableMessages");

/*
 * PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * future with minimal or no warning.
 */

(function(){

var messages = dojo.i18n.getLocalization("jazz.ui", "PagedTableMessages");

var PENDING_UPDATE_FOOTER = 1;
var PENDING_UPDATE_REFRESH = 2;
var PENDING_UPDATE_RESET = 3;

var HASH_ENTRIES = 1;
var HASH_DETAILS = 2;
var HASH_START_INDEX = 100000;
var HASH_ITEMS_PER_PAGE = 10;

var createHash = function(startIndex, itemsPerPage, entriesReady, detailsReady) {
	return (entriesReady ? HASH_ENTRIES : 0) +
			(detailsReady ? HASH_DETAILS : 0) +
			(startIndex * HASH_START_INDEX) +
			(itemsPerPage * HASH_ITEMS_PER_PAGE);
};

dojo.declare("jazz.ui.PagedTable",	[dijit._Widget, dijit._Templated], {
	loadingText: messages.loading,
	clearOnFlush: true,
	
	itemsPerPage: 10,
	resultCount: -1,
	showDetails: false,
	
	setLoading: null,
	onHeightChange: null,
	onPageChange: null,
	
	fetch: function (/* int */ startIndex, /* int */ count, /* boolean */ entries, /* boolean */ details) {
		console.error("fetch has not been overriden.");
	},
	
	preRender: function(/* object[] */ entries) {
	},
	
	postRender: function() {
	},
	
	/* Node[] */ renderEntry: function(/* object */ entryData, /* object */ detailsData, /* boolean */ zoomed) {
		console.error("renderEntry has not been overriden.");
	},
	
	/* Node[] */ renderDetails: function(/* object */ detailsData) {
		return null;
	},
	
	setItemsPerPage: function(itemsPerPage) {
		this.itemsPerPage = parseInt(itemsPerPage);
		this._queueUpdate(PENDING_UPDATE_RESET);
	},
	
	setResultCount: function(resultCount) {
		this.resultCount = parseInt(resultCount);
		this._queueUpdate(PENDING_UPDATE_FOOTER);
	},
	
	setShowDetails: function(showDetails) {
		this.showDetails = showDetails;
		this._queueUpdate(PENDING_UPDATE_REFRESH);
	},
	
	setCurrentPage: function(pageNumber) {
		this.currentPage = pageNumber;
		this._queueUpdate(PENDING_UPDATE_REFRESH, true);
	},
	
	reset: function(/* boolean */ flush) {
		// clear pending updates in case of manual reset
		this._pendingUpdateType = 0;
		this.currentPage = 1;
		this._firePagedChanged();
		this.refresh(flush);
	},
	
	refresh: function(/* boolean */ flush) {
		// clear pending updates in case of manual refresh
		this._pendingUpdateType = 0;
		if (flush) {
			this._entries = new Array();
			this._details = new Array();
		}
		if (flush || this.showDetails) {
			this._clearHeight();
		}
		this._lastState = this.state;
		// store the current state for reference until the next refresh
		this.state = {
			showDetails: this.showDetails,
			itemsPerPage: parseInt(this.itemsPerPage)
		};
		var startIndex = (this.currentPage - 1) * this.itemsPerPage;
		var entriesReady = true;
		var detailsReady = true;
		if (this.resultCount == -1) {
			entriesReady = false;
			detailsReady = !this.showDetails;
		} else if (this.resultCount > 0 ){
			var initialPage = this.currentPage;
			while(startIndex >= this.resultCount && this.currentPage > 1) {
				this.currentPage--;
				startIndex = (this.currentPage - 1) * this.itemsPerPage;
			}
			if (initialPage !== this.currentPage)
				this._firePagedChanged();
			for (var i = startIndex; i < startIndex + parseInt(this.itemsPerPage) && i < this.resultCount; i++) {
				if (!this._entries[i]) 
					entriesReady = false;
				if (this.showDetails && !this._details[i]) 
					detailsReady = false;
				if (!entriesReady && (!this.showDetails || !detailsReady)) 
					break;
			}
		}
		delete this._isLoading;
		if (entriesReady && detailsReady) {
			this._render();
			this._fireHeightChange();
		}
		else {
			this._fireHeightChange();
			var hash = createHash(startIndex, parseInt(this.itemsPerPage), !entriesReady, !detailsReady);
			this._isLoading = hash;
			if (!this._pendingFetches[hash]) {
				var deferred = this.fetch(startIndex, parseInt(this.itemsPerPage), !entriesReady, !detailsReady);
				if (!deferred || !(deferred instanceof dojo.Deferred)) {
					console.error("fetch did not return a valid deferred");
				}
				this._pendingFetches[hash] = true;
				deferred.addCallback(dojo.hitch(this,function(result) {
					if (result && result.entries && result.entries.length > 0) {
						for (var i = 0; i < result.entries.length; i++)
							this._entries[startIndex + i] = result.entries[i];
					}
					if (result && result.details && result.details.length > 0) {
						for (var i = 0; i < result.details.length; i++)
							this._details[startIndex + i] = result.details[i];
					}
					if (this._isLoading === hash) {
						this._render();
						delete this._isLoading;
					}
					return result;
				}));
				deferred.addBoth(dojo.hitch(this,function(result) {
					delete this._pendingFetches[hash];
					this._setLoading(false);
					return result;
				}));
				deferred.addBoth(dojo.hitch(this,this._fireHeightChange));
			}
			if (this._isLoading) {
				this._setLoading(true, !this._initialLoad && !(flush && this.clearOnFlush));
			}
			delete this._initialLoad;
		}
		if (!this._isLoading) {
			this._setLoading(false);
		}
	},
	
	// INTERNAL PROPERTIES AND FUNCTIONS
	
	templatePath: dojo.moduleUrl("jazz.ui", "templates/PagedTable.html"),
	
	firstLinkImageSrc: dojo.moduleUrl("jazz.ui","graphics/PagedTable/beginning.gif").toString(),
	lastLinkImageSrc: dojo.moduleUrl("jazz.ui","graphics/PagedTable/end.gif").toString(),
	
	_pendingUpdateType: 0,
	
	constructor: function() {
		this.next = messages.next;
		this.prev = messages.prev;
		this.first = messages.first;
		this.last = messages.last;
	},
	
	postCreate: function() {
		var nodes = jazz.util.i18n.addMessage(this._pageNumbers, messages.pageCount);
		this._pageText = nodes[0];
		this._currentPageFooter = nodes[1];
		this._totalPagesFooter = nodes[2];
		this.currentPage = 1;

		this._prevButton = new jazz.ui.toolbar.Button({
			label: messages.prev,
			onClick: dojo.hitch(this, this._handlePrevPage),
			"class": "prevButton"
		});
		this._prevButton.placeAt(this._prev, "first");
		
		this._nextButton = new jazz.ui.toolbar.Button({
			label: messages.next,
			onClick: dojo.hitch(this, this._handleNextPage),
			"class": "nextButton"
		});
		this._nextButton.placeAt(this._next);

		this._firstButton = new jazz.ui.toolbar.Button({
			title: messages.first,
			imgSrc: this.firstLinkImageSrc,
			onClick: dojo.hitch(this, this._handleFirstPage),
			"class": "jumpToPageButton"
		});
		this._firstButton.placeAt(this._first);

		this._lastButton = new jazz.ui.toolbar.Button({
			title: messages.last,
			imgSrc: this.lastLinkImageSrc,
			onClick: dojo.hitch(this, this._handleLastPage),
			"class": "jumpToPageButton"
		});
		this._lastButton.placeAt(this._last);

		this.connect(this.domNode,'onmouseover',this._showControls);
		this.connect(this.domNode,'onmouseout',this._hideControls);
		
		this.connect(this._footerStart,'onfocus',dojo.hitch(this,this._footerEntered,true));
		this.connect(this._footerEnd,'onfocus',dojo.hitch(this,this._footerEntered,false));
		this.connect(this._firstButton.focusNode,'onblur',this._footerLinkBlur);
		this.connect(this._prevButton.focusNode,'onblur',this._footerLinkBlur);
		this.connect(this._nextButton.focusNode,'onblur',this._footerLinkBlur);
		this.connect(this._lastButton.focusNode,'onblur',this._footerLinkBlur);
		this.connect(this._firstButton.focusNode,'onfocus',this._footerLinkFocus);
		this.connect(this._prevButton.focusNode,'onfocus',this._footerLinkFocus);
		this.connect(this._nextButton.focusNode,'onfocus',this._footerLinkFocus);
		this.connect(this._lastButton.focusNode,'onfocus',this._footerLinkFocus);
		this._initialLoad = true;
		this._pendingFetches = {};
		this.refresh(true);
	},
	
	_render: function() {
		this._updateFooter();
		if (this.resultCount == 0) {
			return;
		}
		var startIndex = (this.currentPage - 1) * this.state.itemsPerPage;
		var endIndex = startIndex + this.state.itemsPerPage;
		if (!this._pageWidget) {
			this._pageWidget = new jazz.ui.PagedTable._Page({parent: this});
			dojo.place(this._pageWidget.domNode, this._loadingDiv, "before");
			this._pageWidget.setContent(this._entries.slice(startIndex, endIndex), this.state.showDetails ? this._details.slice(startIndex, endIndex) : null, this.state.showDetails);
		} else if (this._pageWidget.domNode) {
			var validate = this._checkHeight();
			this._pageWidget.setContent(this._entries.slice(startIndex, endIndex), this.state.showDetails? this._details.slice(startIndex, endIndex) : null, this.state.showDetails);
			if (validate)
				this._validateHeight();
		}
		delete this._lastState;
	},

	updateHeight: function() {
		this._checkHeight();
		this._doShrinkAnimation();
	},
	
	_checkHeight: function() {
		var height = this._pageWidget.domNode.offsetHeight;
		if (height === 0) {
			// if height is 0, widget is invisible or off dom, clear the height and set a flag to remind us to check again on next transition
			this._clearHeight();
			// return false to indicate that we do not need to validate the height afterward
			return false;
		} else {
			if (!this.state.showDetails && (!this._lastState || !this._lastState.showDetails) && (!this._currentHeight || height > this._currentHeight)) {
				this._currentHeight = height;
				this._contentArea.style.height = height + "px";
			}
		}
		// return a flag indicating if we need to validate the height after
		return !this.state.showDetails && (!this._lastState || !this._lastState.showDetails);
	},
	
	_validateHeight: function() {
		if (this._currentHeight && this._pageWidget.domNode.offsetHeight >= this._currentHeight) {
			this._clearHeight();
		}
	},
	
	_setLoading: function(loading, overlay) {
		if (this.setLoading)
			this.setLoading(loading);
		this._setLoadingMessage(loading, overlay);
	},
	
	_fireHeightChange: function() {
		if (this.onHeightChange)
			this.onHeightChange();
	},
	
	_setLoadingMessage: function(loading, overlay) {
		dojo.removeClass(this._contentArea, "loading-overlay");
		if (!loading) {
			dojo.addClass(this._loadingDiv, "hidden");
			if (this._pageWidget)
				dojo.removeClass(this._pageWidget.domNode, "hidden");
		} else {
			if (overlay) {
				dojo.addClass(this._contentArea, "loading-overlay");
			} else {
				this._updateFooter();
				if (this._pageWidget) 
					dojo.addClass(this._pageWidget.domNode, "hidden");
			}
			dojo.removeClass(this._loadingDiv, "hidden");
		}
	},
	
	_queueUpdate: function(type, force) {
		if (!this._pendingUpdate) {
			this._pendingUpdate = setTimeout(dojo.hitch(this,this._handleUpdate),0);
		}
		if (force || this._pendingUpdateType < type) {
			this._pendingUpdateType = type;
		}
	},
	
	_handleUpdate: function() {
		clearTimeout(this._pendingUpdate);
		delete this._pendingUpdate;
		switch(this._pendingUpdateType) {
			case PENDING_UPDATE_FOOTER:
				this._updateFooter();
				break;
			case PENDING_UPDATE_REFRESH:
				this.refresh(false);
				break;
			case PENDING_UPDATE_RESET:
				this.reset(false);
				break;
		}
		this._pendingUpdateType = 0;
	},
	
	_updateFooter: function() {
		var pageCount = this._getPageCount()
		this._totalPagesFooter.innerHTML = pageCount;
		this._updateCurrentPageFooter();
		if (pageCount > 1)
			dojo.removeClass(this._footer, "hidden");
		else
			dojo.addClass(this._footer, "hidden");
		this._fireHeightChange();
	},
	
	/*
	 * We have entered the footer so show the controls and then allow tabbing to continue.
	 */
	_footerEntered: function(forward){
		this._footerStart.tabIndex = -1;
		this._footerEnd.tabIndex = -1;
		this._showControls();
		if (forward) {
			if (this.currentPage == 1) {
				this._nextButton.focus();
			} else {
				this._firstButton.focus();
			}
		} else {
			if (this.currentPage == this._getPageCount()) {
				this._prevButton.focus();
			} else {
				this._lastButton.focus();
			}
		}
	},
	
	_footerLinkBlur: function() {
		// yield a call to hide the controls on blur of any footer link
		// if another footer link is the target of the focus transfer, the timeout will be cancelled
		this._footerExitTimeout = setTimeout(dojo.hitch(this,this._hideControls),0);
	},
	
	_footerLinkFocus: function() {
		// cancel the timeout to hide the footer since we are still inside it
		clearTimeout(this._footerExitTimeout);
	},
	
	_clearHeight: function() {
		this._contentArea.style.height = "";
		delete this._currentHeight;
		if (this._shrinkTimer) {
			clearTimeout(this._shrinkTimer);
			delete this._shrinkTimer;
		}
	},
	
	_handleNextPage: function(event) {
		dojo.stopEvent(event);
		if (this.currentPage == this._getPageCount()) {
			return;
		}
		this.currentPage++;
		this._postNavigate();
	},
	
	_handlePrevPage: function(event) {
		dojo.stopEvent(event);
		if (this.currentPage == 1) {
			return;
		}
		this.currentPage--;
		this._postNavigate();
	},
	
	_handleFirstPage: function(event) {
		dojo.stopEvent(event);
		this._first.blur();
		if (this.currentPage == 1) {
			return;
		}
		this.currentPage = 1;
		this._postNavigate();
	},
	
	_handleLastPage: function(event) {
		dojo.stopEvent(event);
		this._last.blur();
		if (this.currentPage == this._getPageCount()) {
			return;
		}
		this.currentPage = this._getPageCount();
		this._postNavigate();
	},
	
	_postNavigate: function() {
		this._updateCurrentPageFooter();
		this._updateControls();
		this._firePagedChanged();
		this.refresh(false);
	},
	
	_firePagedChanged: function() {
		if (this.onPageChange)
			this.onPageChange(this.currentPage);
	},
	
	_updateControls: function() {
		if (this.currentPage != 1) {
			this._firstButton.attr("imgSrc", this.firstLinkImageSrc);
			this._firstLinkDisabled = false;
			this._firstButton.attr("tabIndex", 0);
			this._firstButton.attr("enabled", true);
			this._prevButton.attr("enabled", true);
		} else {
			this._firstButton.attr("imgSrc", dojo.moduleUrl("jazz.ui","graphics/PagedTable/beginning-disabled.gif"));
			this._firstLinkDisabled = true;
			this._firstButton.attr("tabIndex", -1);
			this._firstButton.attr("enabled", false);
			this._prevButton.attr("enabled", false);
		}
		if (this.currentPage != this._getPageCount()) {
			this._lastButton.attr("imgSrc", this.lastLinkImageSrc);
			this._lastLinkDisabled = false;
			this._lastButton.attr("tabIndex", 0);
			this._lastButton.attr("enabled", true);
			this._nextButton.attr("enabled", true);
		} else {
			this._lastButton.attr("imgSrc", dojo.moduleUrl("jazz.ui","graphics/PagedTable/end-disabled.gif"));
			this._lastLinkDisabled = true;
			this._lastButton.attr("tabIndex", -1);
			this._lastButton.attr("enabled", false);
			this._nextButton.attr("enabled", false);
		}
	},
	
	_showControls: function() {
		this._updateControls();
		dojo.forEach([this._prev, this._next, this._first, this._last], function(control) {
			dojo.removeClass(control, "invisible");
		});
		if (this._shrinkTimer) {
			clearTimeout(this._shrinkTimer);
			delete this._shrinkTimer;
		}
	},
	
	_hideControls: function(e) {
		if (e && dojo.isDescendant(e.relatedTarget, this.domNode) === true)
			return;
		dojo.forEach([this._prev, this._next, this._first, this._last], function(control) {
			dojo.addClass(control, "invisible");
		});
		this._footerStart.tabIndex = 0;
		this._footerEnd.tabIndex = 0;
		if (!this.state.showDetails)
			this._shrinkTimer = setTimeout(dojo.hitch(this,this._doShrinkAnimation), 2500);
	},
	
	_doShrinkAnimation: function() {
		if (!this._pageWidget) {
			this._shrinkTimer = setTimeout(dojo.hitch(this,this._doShrinkAnimation), 100);
			return;
		}
		if (!this._currentHeight)
			return;
		delete this._shrinkTimer;
		var onEnd = dojo.hitch(this,function() {
			this._clearHeight();
			this._fireHeightChange();
		});
		// gaurd against a destroyed page widget
		var endHeight = this._pageWidget.domNode ? this._pageWidget.domNode.offsetHeight : 0;
		if (endHeight === 0) {
			onEnd();
			return;
		}
		var anim = dojo.animateProperty({node: this._contentArea, duration: 350, properties: {height: {start: this._currentHeight, end: endHeight, unit: "px"}}});
		this.connect(anim, "onAnimate", this._fireHeightChange);
		this.connect(anim, "onEnd", onEnd);
		setTimeout(function(){anim.play();}, 0);
	},
	
	_updateCurrentPageFooter: function() {
		this._currentPageFooter.innerHTML = this.currentPage;
	},
	
	_getPageCount: function() {
		return Math.ceil(this.resultCount / this.state.itemsPerPage);
	},
	
	uninitialize: function(){
		if (this._pageWidget) 
			this._pageWidget.destroy();
			
		if (this._prevButton) {
			this._prevButton.destroy();
			delete this._prevButton;
		}
		
		if (this._nextButton) {
			this._nextButton.destroy();
			delete this._nextButton;
		}

		if (this._firstButton) {
			this._firstButton.destroy();
			delete this._firstButton;
		}

		if (this._lastButton) {
			this._lastButton.destroy();
			delete this._lastButton;
		}
	}
});

dojo.declare("jazz.ui.PagedTable._Page",
	[dijit._Widget, dijit._Templated], {
	
	templateString: "<table class=\"pageTable\" cellspacing=\"0\" role=\"presentation\"><tbody dojoAttachPoint=\"tbody\"></tbody></table>",
	
	setContent: function(/* object[] */ entries, /* object[] */ details, /* boolean */ showDetails) {
		// this._currentNode is used to replace existing table nodes
		this._isShowingDetails = showDetails;
		this._currentNode = this.tbody.firstChild;
		while (this._currentNode && this._currentNode.nodeName == "#text")
			this._currentNode = this._currentNode.nextSibling;
		this.parent.preRender(entries);
		for (var i = 0; i < entries.length; i++) {
			this._doAppendReplaceNode(this._createRow(this.parent.renderEntry(entries[i], showDetails ? details[i] : null, showDetails)));
			if (details && details[i]) {
				this._doAppendReplaceNode(this._createRow(this.parent.renderDetails(details[i])));
			}
		}
		// remove any remaining old rows after populating the table
		while(this._currentNode) {
			var removeNode = this._currentNode;
			this._currentNode = this._currentNode.nextSibling;
			this.tbody.removeChild(removeNode);
		}
		if (dojo.isIE === 8) {
			// need to force a reflow in IE8 for some cases, see item 154811  :(
			dojo.style(this.domNode,{display:"none"});
			this.domNode.clientHeight;
			dojo.style(this.domNode,{display:""});
		}
		this.parent._fireHeightChange();
	},
	
	_createRow: function(/* Node[] */ columns) {
		var tr = document.createElement("tr");
		for (var i=0;i<columns.length;i++) {
			tr.appendChild(columns[i]);
		}
		return tr;
	},

	_doAppendReplaceNode: function(node) {
		if (this._currentNode) {
			var replaceNode = this._currentNode;
			this.tbody.replaceChild(node, replaceNode);
			this._currentNode = node.nextSibling;
		}
		else {
			this.tbody.appendChild(node);
		}
	},
	
	uninitialize: function() {
		while (this.tbody.firstChild)
			this.tbody.removeChild(this.tbody.firstChild);
	}

});

})();
