/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("jazz.ui.RemoteChooserContainer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("dojo.i18n");
dojo.requireLocalization("jazz.ui", "RemoteChooserContainerMessages");

var _PROTOCOL_WINDOWNAME = { 'OSLC1':'#oslc-windowName-1.0', 'OSLC2':'#oslc-core-windowName-1.0' },
	_PROTOCOL_POSTMSG = { 'OSLC1' : '#oslc-postMessage-1.0', 'OSLC2':'#oslc-core-postMessage-1.0' },
	_MESSAGE_PORT = "oslc-response:",
	_MESSAGE_RESIZE = "oslc-resize:",
	_PARAM_WIDTH = "oslc:hintWidth",
	_PARAM_HEIGHT = "oslc:hintHeight",

	isWindowName = dojo.isIE <= 7 || !window.postMessage,

	messages = dojo.i18n.getLocalization("jazz.ui", "RemoteChooserContainerMessages");

function _requiredArgument(/*Map*/ args, /*String*/ id) {
	if (!args[id])
		throw new Error('Missing required argument: "' + id + '"');
}

dojo.declare("jazz.ui.RemoteChooserContainer", [dijit._Widget, dijit._Templated], {
	
	//	summary:
	//		A wrapper widget for displaying and communicating with a webpage from another origin,
	//		which is hosted inside an IFrame. The domNode of this widget is undefined, but is
	//		guaranteed to be a block-level element with a relative width and height of 100%.
	//	example:
	//		var container = new jazz.ui.RemoteChooserContainer({
	//			remoteLocation: someRemoteUrl,
	//			onData: myCallback,
	//			onResize: myResizeCallback, // optional: function(newWidth, newHeight) 
	//          mode : ('OSLC1'|'OSLC2') // If not value provided assumes OSLC1 for backwards compatibility
	//		}, placeHolder);
	//	since:
	//		0.7
	
	//	remoteLocation: String
	//		The URL (as a string) of the remote webpage initially displayed
	remoteLocation: null,
	
	//	onData: Function
	//		The result callback function
	onData: null,
	
	//	onResize: [optional] Function(newWidth, newHeight)
	//		The resize callback function
	onResize: null,
	
	//	mode: [optional] String
	//		The OSLC spec that should be used. Defaults to 'OSLC1' only other valid value is 'OSLC2'.
	mode: 'OSLC1',
	
	_returnLocation: new dojo._Url(window.location, dojo.moduleUrl("dojo", "resources/blank.html")).toString(),

	templatePath: dojo.moduleUrl("jazz.ui", "templates/RemoteChooserContainer.html"),
	
	constructor: function(args) {
		_requiredArgument(args, 'remoteLocation');
		_requiredArgument(args, 'onData');
		if (args.mode && args.mode !== 'OSLC2' && args.mode !== 'OSLC1')
			delete args.mode; // ignore invalid modes
		this.loadingLabel = messages.loading;
	},
	
	postCreate: function() {
		if (isWindowName) {
			this.iframe.name = this._returnLocation;
			var protocol = _PROTOCOL_WINDOWNAME[this.mode];
		} else {
			this.connect(window,"onmessage",this._onMessage);
			protocol = _PROTOCOL_POSTMSG[this.mode];
		}
		this.iframe.src = this.remoteLocation + protocol;
	},
	
	_contentLoaded: function(e) {
		if (e.type === "load" || (this.iframe.readyState && this.iframe.readyState === "complete")) {
			if (this.loadingDiv) {
				this.domNode.removeChild(this.loadingDiv);
				delete this.loadingDiv;
			}
			if (isWindowName) {
				if (!this._finished) {
					try {
						if (this.iframe.contentWindow.location != this._returnLocation) 
							return;
						this._finished = true;
						this.onData(this.iframe.contentWindow.name);
					} 
					catch (e) {
						//Ignore location changes on the remote server
					};
				}
			}
		}
	},

	_onMessage: function(e) {
		try {
			if (e.source == this.iframe.contentWindow) {
				if (e.data.indexOf(_MESSAGE_PORT) == 0) {
					this.onData(e.data.substr(_MESSAGE_PORT.length));
				} else if (this.onResize && (e.data.indexOf(_MESSAGE_RESIZE) == 0)) {
					try {
						var newSize= dojo.fromJson(e.data.substr(_MESSAGE_RESIZE.length));
						if (newSize[_PARAM_WIDTH] || newSize[_PARAM_HEIGHT]) {
							this.onResize(newSize[_PARAM_WIDTH], newSize[_PARAM_HEIGHT]);
						}	
					} catch (e) {
						// ignore malformed replies
					}
				}
			}
		} catch(e) {
			// fail silently in case we cannot access the iframe's content window. See work item 159552.
		}
	}
});

})();