/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.app.web.RepositoryApplication"); //$NON-NLS-1$

dojo.require("jazz.app._AjaxApplication"); //$NON-NLS-1$
dojo.require("jazz.app.context"); //$NON-NLS-1$
dojo.require("jazz.app.auth"); //$NON-NLS-1$
dojo.require("jazz.util.html"); //$NON-NLS-1$
dojo.require("jazz.app.internal.applicationAbout"); //$NON-NLS-1$
dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$
dojo.require("net.jazz.ajax.ui.internal.PageRegistry"); //$NON-NLS-1$
dojo.require("com.ibm.team.dashboard.web.ui.DashboardMini"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.client.init.InitializationData"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.client.internal.RepositoryClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.client.session.Session"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.ui.internal.about.AboutJazzTeamServer"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.ui.internal.LicenseInfoWidget"); //$NON-NLS-1$
dojo.require("com.ibm.team.app.web.internal.welcome.WelcomeUtil"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.ui.internal.admin.AdminMenu"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.app.web.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$

var messages = dojo.i18n.getLocalization("com.ibm.team.app.web.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
var InitializationData = com.ibm.team.repository.web.client.init.InitializationData;
var RepositoryClient = com.ibm.team.repository.web.client.internal.RepositoryClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var Session = com.ibm.team.repository.web.client.session;
var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var JTS_IDENTIFIER = "http://jazz.net/application/jts";

dojo.declare("com.ibm.team.app.web.RepositoryApplication", jazz.app._AjaxApplication, { //$NON-NLS-1$

	autoRequestInitData: true,

	constructor: function(args) {
		if (this.autoRequestInitData) this.fetchInitData();
		
		jazz.app.auth.setLogoutUrl(net.jazz.ajax._contextRoot + "/service/com.ibm.team.repository.service.internal.ILogoutRestService");
		
		this.config.buttons.push({
			label: messages.admin,
			className: "admin-menu",
			align: "right",
			menuProvider: dojo.hitch(this,this._adminMenuProvider)
		});
		this.config.buttons.push({
			label: messages.help,
			className: "help-menu",
			align: "right",
			menuProvider: dojo.hitch(this,this._helpMenuProvider)
		});
		var appAbout = jazz.app.internal.applicationAbout.getCurrent() || {};
		var aboutStr = this._aboutStr = (appAbout.identifier == JTS_IDENTIFIER) ? messages.aboutServer : messages.aboutApp;
		var node = dojo.create("a", { href: "#", innerHTML: this.ui.attr("title"), title: aboutStr });
		dojo.connect(node, "onclick", this, this._openAboutBox);
		this.ui.attr("title", node);
	},
	fetchInitData: function() {
		RepositoryClient.getInitializationData(new ServiceResponseHandler(this, "_fetchInitDataSuccess", "_fetchInitDataFailure")); //$NON-NLS-1$ //$NON-NLS-2$
	},
	initContext: function() {
		try {
			var context = this.getCurrentContext();
			jazz.app.context.set(context);
			if (!context || !context.name) {
				this.ui.attr("navbar", null);
				this.ui.attr("searchProvider", null);
				if (context) {
					// show All Projects
					this.ui.attr("bannerTitle", messages.allProjects);
					net.jazz.ajax.ui.internal.PageRegistry.setDefaultPage("com.ibm.team.process.projectAreaList");
				} else {
					var urlContext = this._getContextFromUrl();
					this.ui.attr("bannerTitle", urlContext);
					var content = dojo.create("div",{ style: { paddingBottom: "1.8em" }}),
						adminLinkText = dojo.string.substitute("<a href=\"${0}\">${1}</a>", [ net.jazz.ajax._contextRoot + "/admin", messages.JazzAdminUI]); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
					dojo.create("div", { 
						innerHTML: dojo.string.substitute(messages.contextNotAuthorized, [ "<b>"+jazz.util.html.escape(urlContext)+"</b>" ]), 
						style: {paddingBottom: "8px"} 
					}, content);
					dojo.create("div", { 
						innerHTML: dojo.string.substitute(messages.AdminInvestigate, [ adminLinkText ])
					}, content);
					this.onFailure(null,content);
					return;
				}
			} else {
				this.ui.attr("bannerTitle", context.name);
			}
		} catch(e) {
			console.error(e);
			// redirect to '/web' if the URI isn't well-formed or projects are missing.
			this.onFailure(null, e.message);
			var href = net.jazz.ajax._contextRoot + "/web";
			if (window.location.search) {
				href += window.location.search;
			}
			location.href = href;
			return;
		}
		this.start();
	},
	/*
	 * returns: null for invalid project name
	 * 			{} for no context specified in URL
	 * 			{name, itemId, archived} for valid project
	 */
	getCurrentContext: function () {
		// set the global var for the current project area based on the last
		// segment of the document's URI
		var context = {},
			projectName = this._getContextFromUrl();
		try {
			var projectAreas = InitializationData.get("com.ibm.team.process.service.projectAreaWebUIInitializionData").projectAreas; //$NON-NLS-1$
			if (projectName) {
				//if we found a project name from the URL, match with project from server
				for (var i = 0; i < projectAreas.length; i++) {
					if (projectAreas[i].name == projectName) {
						context.itemId = projectAreas[i].itemId;
						context.name = projectAreas[i].name;
						context.archived = projectAreas[i].archived;
						context.uri = projectAreas[i].uri;
						break;
					}
				}
				if (!context.name) {
					return null;
				}
			}
		} catch (e) {
			console.error(e);
		}
		return context;
	},
	_fetchInitDataSuccess: function(initializationData) {
		try {
			Session.init(initializationData);
			var logoutUri = dojo.getObject("com.ibm.team.repository.web.logouturi");
			if (logoutUri) {
				this.config.logout = { href: logoutUri };
			}
			var contrib = Session.getAuthenticatedContributor();
			if (contrib) {
				this.config.user = {
					name: contrib.name,
					userId: contrib.userId
				};
			} else {
				this._handleNoContributor();
			}
		} catch (e) {
			document.title = messages.ErrorLoadingUser;			
			var content = dojo.create("div",{ style: { paddingBottom: "1.8em" }}),
				adminLinkText = dojo.string.substitute("<a href=\"${0}\">${1}</a>", [ net.jazz.ajax._contextRoot + "/admin", messages.JazzAdminUI]); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			dojo.create("div", { 
				innerHTML: messages.DatabaseError, 
				style: {paddingBottom: "8px"} 
			}, content);
			dojo.create("div", { 
				innerHTML: dojo.string.substitute(messages.AdminInvestigate, [ adminLinkText ])
			}, content);
			this.onFailure(e,content,true);
			return;
		}
		this.ui.attr("drawer", "com.ibm.team.dashboard.web.ui.DashboardMini");
		this.ui.attr("info", "com.ibm.team.repository.web.ui.internal.LicenseInfoWidget");
		this.initContext();
	},
	_fetchInitDataFailure: function(errorObject) {
		try {
			var context = this._getContextFromUrl();
			this.ui.attr("bannerTitle", context ? context : messages.allProjects);
		} catch (e) {
			// eat the error - 404
		}
		if (errorObject.status == 403) {
			this.ui.attr("homeMenuProvider", null);
			var dfd = jazz.app.auth.getUserRoles();
			dfd.addBoth(dojo.hitch(this,this._getUserRolesResponse));
			this._errObj = errorObject;
		} else if (errorObject.status !== 401) {
			this.onFailure(errorObject, dojo.string.substitute(messages.initDataError, [errorObject.message]), false);
		} else {
			this.onFailure(errorObject);
		}
	},
	_handleNoContributor: function () {
		// handle case where there is no DB so contributor isn't in init data
		var userId = Session.getAuthenticatedUserId();
		if (userId) {
			this.config.user = {
				name: userId,
				userId: userId
			};
		} else {
			// fake out the Logout path since we don't know the unauthorized user
			this.config.user = {
				name: "",
				userId: ""
			};
		}
	},
	_getUserRolesResponse: function(response) {
		this._handleNoContributor();
		var node = dojo.create("div", {innerHTML: (response && response.userId) ? dojo.string.substitute(messages.notAuthorized, ["<b>" + jazz.util.html.escape(response.userId) + "</b>"]) : messages.notAuthorized});
		this.onFailure(this._errObj, node);
	},
	_adminMenuProvider: function() {
		return new com.ibm.team.repository.web.ui.internal.admin.AdminMenu();
	},
	_helpMenuProvider: function() {
		var menu = new jazz.app._ComponentMenu();
		var helpuri = dojo.getObject("com.ibm.team.repository.web.helpuri");
		if (helpuri) {
			this.helpuri = helpuri;
			menu.addItemByArgs({label: messages.helpContents, icon: dojo.moduleUrl("jazz.app","graphics/JazzUI/icons/systemhelp.gif"), onClick: dojo.hitch(this,this._openHelpContents)});
		}
		menu.addItemByArgs({label: this._aboutStr, onClick: dojo.hitch(this,this._openAboutBox)});
		var dfd = com.ibm.team.app.web.internal.welcome.WelcomeUtil.isWelcomeEnabled();
		dfd.addCallback(dojo.hitch(this, function (enabled) {
			if (enabled) {
				menu.addItemByArgs({label: messages.welcome, onClick: dojo.hitch(this, this._openWelcomePage)});
			}
		}));
		return menu;
	},
	_openHelpContents: function(event) {
		if (this.helpuri)
			window.open(this.helpuri,'help');
	},
	_openAboutBox: function(e){
		if (e) e.preventDefault();
		var appAbout = jazz.app.internal.applicationAbout.getCurrent() || {},
			aboutStr = (appAbout.identifier == JTS_IDENTIFIER) ? messages.aboutServer : messages.aboutApp;
		new com.ibm.team.repository.web.ui.internal.about.AboutJazzTeamServer({dialogTitle: aboutStr});
	},
	_openWelcomePage: function(event) {
		location.href = "#action=com.ibm.team.app.viewWelcome";
	},
	_getContextFromUrl: function(){
		var pathname = window.location.pathname,
			segments = pathname.split('/web/projects');
		if (segments.length > 1) {
			var projectName = segments[1];
			if (projectName && projectName.indexOf("/") == 0) 
				projectName = projectName.substring(1);
			return decodeURIComponent(projectName);
		} else {
			var path = pathname.split('/web');
			if (path.length <= 1 || (path[1] != "" && path[1].indexOf("/projects") < 0)) {
				throw new Error(messages.fourOhFour);
				return;
			}
		}
		return null;
	}
});
})();