/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rdm.fronting.server.web.migration.rrm.RepositoryVersionCheckStep");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.ui.Button");
dojo.require("jazz.ui.wizard.WizardStepHeader");
dojo.require("jazz.ui.wizard.WizardStep");
dojo.require("jazz.ui.styledBox.TrimStyles");
dojo.require("jazz.ui.StyledBox");
dojo.require("dojo.i18n");
dojo.require("com.ibm.rdm.fronting.server.web.util.rrsutils");
dojo.require("jazz.client.xhr");

(function() {
dojo.declare("com.ibm.rdm.fronting.server.web.migration.rrm.RepositoryVersionCheckStep", 
    [jazz.ui.wizard.WizardStep, dijit._Widget, dijit._Templated], {

    templatePath: dojo.moduleUrl("com.ibm.rdm.fronting.server.web.migration.rrm", "RepositoryVersionCheckStep-template.html"),
    widgetsInTemplate: true,
    innerBoxTrim:  jazz.ui.styledBox.TrimStyles.EDITOR,
    migrationInfo: {}, 
    authenticationInfo: {}, 
    serverInfo: undefined,
    isFinished: false,
    
    constructor: function() {
       this.messages = messages = { description: "foo",
                                    setupName: "foo",
                                    migrationValidateButtonLabel: "validate",
                                    currentRelease: "",
                                    requiredRelease: ""  };
        this.stepTitle = "Repository version check";
    },

    postCreate: function() {
        this.header = new jazz.ui.wizard.WizardStepHeader({stepTitle: this.stepTitle});
        this.stepHeaderArea.appendChild(this.header.domNode);
    },

    // called by the wizard
    focusStep: function() {
        this.isCompleted = false;
        this.isFinished = false;
        //this.migrationValidateButton.disabled = false;
        this.extractAuthenticationInfo();
        this.checkRepositoryVersion();
    },
    
    // does a GET on the migration service which returns some JSON containing information about
    // which migrationKeys are avilable.  The value is added to this.migrationInfo.  The next step
    // (MigrationstatusStep.js) requires this value when it submits the migration task. 
    checkRepositoryVersion: function() {
        var self = this;
        
        /*
        There was special code added to this method to avoid the oAuth popup.
         - The request to the fronting server uses the 'skipInterceptors' parameter to prevent 
           the Foundation Web UI code from handling the OAuth dance which currently means the popup.
         - The 401 error is now handled by redirecting to the JF OAuth page, rewriting the callback URL 
           to get you back to this page.
         - The new request to this page will include an OAuth access token.  This is then taken from 
           the incoming query params and passed on to the new fronting server request.
        */
 
        var self = this;

        var getUrl = this.serverInfo.rrmMigrationServiceUrl + "?release=RRC3010&";
        for(key in this.authenticationInfo){
           if(key != "debug"){
             var nvp = key + "=" + this.authenticationInfo[key] + "&";
             getUrl += nvp;
           }
        }

        var result = jazz.client.xhrGet({
            url: getUrl,
            sync: true,
            handleAs: "json",
            skipInterceptors: ["jazz.oauth"],
            headers: {
                "Accept": "*",
                "DoorsRP-Request-Type": "private" 
            },
            load: function(responseObject, ioargs){
              if("true" != responseObject.migrationRequired){
                 self.SourceRepositoryBaseUri.innerHTML = "No migration required.";
                 self.isFinished = true;
                 self.triggerButtonStateUpdate();
               } else {
                   var key;
                   for(key in responseObject){
                        self.migrationInfo[key] = responseObject[key]; 
                   }
                   self.SourceRepositoryBaseUri.innerHTML = "Migration required. Select 'Next' to proceed.";
                  
                   //self.SourceRepositoryBaseUri.innerHTML = "Source repository URI: <input type=\"text\" style=\"width:300px;\" name=\"SourceRepositoryBaseUri\" value =\"" + 
                       //responseObject.sourceRepositoryBaseUri + "\"/>";                      
                   //self.SourceFrontingBaseUri.innerHTML = "Source fronting URI: <input type=\"text\" style=\"width:300px;\" name=\"SourceFrontingBaseUri\" value =\"" + 
                       //responseObject.sourceFrontingBaseUri + "\"/>";
                       
                   // TODO remove when setting source URIs is enabled    
                   self.isCompleted = true;
                   self.enableNext = true;
                   self.triggerButtonStateUpdate();      
                }  
            },
            error: function(error, ioargs){
                switch (ioargs.xhr.status) {
                case 401:
                    var oauthUrl = ioargs.xhr.getResponseHeader("X-jazz-web-oauth-url");
                    if(oauthUrl) {
                        if(error.jfsHttpException) {
                            error.jfsHttpException.message = "Authentication required; redirecting to login screen";
                        }
                        // yield to allow current callback to complete (e.g. to display
                        // brief error message before redirect)
                        setTimeout(function() {
                            // redirect to JTS OAuth authorize screen
                            location.href = com.ibm.rdm.fronting.server.web.util.rrsutils.getRewrittenOAuthUrl(oauthUrl);
                        }, 0);
                    }
                    break;   
                default:
                   console.log(error);
                   break;
                }
                self.isCompleted = false;
                self.triggerButtonStateUpdate();
            }           
        }); 
    },
    
    /*
    Remove the 'request_token_secret' and 'oauth_token' query params and redirect once more
    */
    extractAuthenticationInfo: function() {
        var curPage = location.href;
        var curPageUrl = new dojo._Url(curPage);
        
        //save the embedded step fragment
        var fragment = "#" + curPageUrl.fragment;
        var fragmentStart = curPage.indexOf(fragment);
        fullFragment = curPage.substr(fragmentStart);
        
        var nvps = com.ibm.rdm.fronting.server.web.util.rrsutils.getNVPs(curPageUrl.query);

        for(var i=0; i<nvps.length; i++) {
            this.authenticationInfo[nvps[i].n] = nvps[i].v;
        }  
    },
    // gets called with updated migration information
    // TODO currently not called
    initializeMigration: function(e) {
        dojo.stopEvent(e);
        this.migrationInfo["sourceRepositoryBaseUri"] = this.SourceRepositoryBaseUri.firstElementChild.value; 
        this.migrationInfo["sourceFrontingBaseUri"] = this.SourceFrontingBaseUri.firstElementChild.value; 
        this.isCompleted = true;
        this.enableNext = true;
        this.triggerButtonStateUpdate();
    },

    //
    // net.jazz.ui.Wizard callbacks
    //
    canFlipToNextPage: function() {
        return this.isCompleted;
    },
    
    canFinish: function() {
        return this.isFinished;
    }
});
})();