/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.ResourceLink");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.xml.DomParser");

dojo.require("jazz.ui.internal.ResourceLinkAggregator");
dojo.require("jazz.ui.HoverPopup");
dojo.require("jazz.util.html");
dojo.require("jazz.util.ui");
dojo.require("jazz.util.internal.login");
dojo.require("jazz.client.xhr");

dojo.require("dojo.i18n");
dojo.requireLocalization("jazz.ui.internal", "ResourceLinkMessages");

(function(){

var CSS_RL = "jazz-ui-ResourceLink";
var CSS_ARROW = "arrow";
var CT_COMPACT_RENDERING = "application/x-jazz-compact-rendering";
var CT_OSLC_COMPACT = "application/x-oslc-compact+xml";
var PREVIEW_RESIZE_MESSAGE_PREFIX = "jazz-compact-rendering-preview-height:";
var OSLC_PREVIEW_RESIZE_PREFIX = "oslc-preview-height:";
var XMLNS_DC_TERMS = "http://purl.org/dc/terms/";
var XMLNS_JP = "http://jazz.net/xmlns/prod/jazz/presentation/1.0/";
var XMLNS_RDF = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
var XMLNS_OSLC = "http://open-services.net/ns/core#";

var defaultLazyFetch = false;
var messages = dojo.i18n.getLocalization("jazz.ui.internal", "ResourceLinkMessages");

dojo.declare("jazz.ui.ResourceLink", dijit._Widget, {
    
	// summary:
	//		A hyperlink referencing a Jazz resource. We take the URL of the
	//		link's HREF attribute to determine the resource (Work Item, Plan,
	//		etc.) and then send back information to display a summary of the
	//		resource when the user hovers over the link. The summaries presented
	//		can have a small and large view depending on how much content the
	//		user wishes to see.
	//
	// parameters:
	//		uri: (String) The uri that will be used to find the available
	//				presentations. The URI can be relative ("/testURI") 
	//				or absolute ("http://localhost/testURI"). REQUIRED if 
	//				the given linkNode does not have a uri attached to 
	//				the HREF attribute.
	//		(optional) presentationUri: (String) A URL that will replace the
	//				link's HREF attribute, used to send a user to a different
	//				presentation of the given resource.
	//		(optional) label: (String) A string or HTML snippet that will be
	//				placed as the child node/inner text of the link/anchor node
	//				the Resource Link will be attached to.  If no srcNodeRef is 
	//				given, this argument is REQUIRED.
	//		(optional) useAbbreviation: (boolean) If set to true, the text of the
	//				anchor node will be replaced with the compact rendering's
	//				"jp:abbreviation" value. If none, exists then the widget will
	//				look at the value of the retainLinkText. If it is false, the
	//				widget will fall back to the full resource title and if it is
	//				true, it will leave the link text unchanged. By default, this
	//				false.
	//		(optional) retainLinkText: (boolean) If set to false, the text in the 
	//				srcNodeRef anchor node will change to the text of the compact 
	//				rendering's "dc:title" attribute. If set to true, the text of 
	//				the srcNodeRef anchor node will not change when the resource 
	//				link is created. By default, this parameter is set to false 
	//				and will change the text of the anchor node. Note, this
	//				parameter is ignored if useAbbreviation is true and the compact
	//				rendering document supplies a "jp:abbreviation".
	//		(optional) iconNode: (DOM Node reference or id) The DOM node in the page
	//				to that should be used to render the icon. If an <img> node is
	//				provided, the src attribute will be update on it directly. If any
	//				other node is provided, an <img> tag will be created and appended
	//				to it. This parameter can also be the string id of the desired
	//				node.
	//		(optional) lazyFetch: (boolean) If set to true, the Resource Link will
	//				will not fetch the compact rendering document until the user
	//				hovers over the link.  Also, if this option is set to true,
	//				the "retainLinkText" parameter will automatically be set to true,
	//				and the "iconNode" parameter will be ignored.  By default,
	//				this parameter is set to false, and the compact rendering document
	//				will be aggressively fetched when the page loads.
	//		(optional) srcNodeRef: (Anchor Node) The anchor node in the page
	//				that the Resource Link will be attached to. NOTE: if the uri
	//				parameter is not given, the href value of the given	node
	//				will be used in its place as the resource to reference.
	//
	// methods:
	//
	//		refresh: if the resource that the Resource Link is viewing has changed,
	//				one can call this method to retrieve the new compact rendering
	//				document and update the recource link hover with new data.
	//
	// usage:
	//		var navLink = new jazz.ui.ResourceLink({}, linkNode);
	//		-or-
    //		var navLink = new jazz.ui.ResourceLink(
	//				{
	//					uri: "aResourceuri", 
	//                  label: "Label for the Anchor Node"
	//				});
	//		-or-
	//		var navLink = new jazz.ui.ResourceLink(
	//				{
	//					uri: "aResourceuri", 
	//				},
	//				srcNodeRef);
	//		-or-
	//		var navLink = new jazz.ui.ResourceLink(
	//				{
	//					uri: "aResourceuri", 
	//					presentationUri: "anotherUrl"
	//				},
	//				srcNodeRef);
    //		-or-
	//		var navLink = new jazz.ui.ResourceLink(
	//				{
	//					uri: "aResourceuri", 
	//					presentationUri: "anotherUrl",
    //                  label: "New Label for the Anchor Node"
	//				},
	//				srcNodeRef);
    //		-or-
	//		var navLink = new jazz.ui.ResourceLink(
	//				{
	//					uri: "aResourceuri", 
	//					presentationUri: "anotherUrl",
    //                  label: "Label for the Anchor Node"
	//				});
	// since:
	//		0.7
	
	presentationUri: null,
	useAbbreviation: false,
	retainLinkText: false,
	iconNode: null,
	lazyFetch: null,
	
	/**
	 * Called when the compact rendering document has loaded. Will be called once per successful
	 * load in the event of calls to refresh.
	 */
	onLoad: function(){},
	
	/**
	 * Returns the title of the resource from the compact rendering document or null if no title
	 * is defined or the document has not been loaded yet.
	 */
	getTitle: function() {
		return this.popup && ((this.popup.compactRendering && this.popup.compactRendering.title) ||
							(this.popup.oslcCompact && this.popup.oslcCompact.title));
	},
	
	buildRendering: function(){
		if (!this.srcNodeRef && this.uri) {
			this.domNode = document.createElement("A");
			if (!this.label)
				throw new Error("ResourceLink REQUIRES a label parameter if a uri is given rather than a reference to an Anchor Node.");
		}
		else if (this.srcNodeRef && this.srcNodeRef.tagName === "A") {
			this.domNode = this.srcNodeRef;
			if (!this.uri)
				this.uri = this.domNode.href;
			this.target = this.srcNodeRef.target;
		}
		else
			throw new Error("ResourceLink REQUIRES that either the uri parameter or a valid reference to an Anchor Node is given");
		
		// check for non-http schemes
		var scheme = this.uri.match(/^[^\#\?]*\:\/\//);
		scheme = (scheme && scheme.length) ? scheme[0] : false;
		if (!scheme) {
			// there is no scheme, convert the relative uri to an absolute uri based on the document uri
			var baseUri = document.baseURI || document.URL; //document.URL is used to acquire the base URI in IE
			try {
				this.uri = new dojo._Url(baseUri, this.uri).toString();
			} catch (e) {
				// there was an error with the uri, disable the ResourceLink and render it as a normal link
				this._isInvalidUri = true;
			}
		} else if (scheme !== "http://" && scheme !== "https://") {
			// there was an invalid scheme, disable the ResourceLink and render it as a normal link
			this._isInvalidUri = true;
		}
		this.domNode.href = this.presentationUri || this.uri;
		if (this.label)
			this.domNode.innerHTML = jazz.util.html.removeScript(this.label);
		if (!this._isInvalidUri)
			this._styleLink();
	},
	
	postCreate: function() {
		if (this._isInvalidUri) return;
		var aggregatorId;
		if (this.iconNode)
			this.iconNode = dojo.byId(this.iconNode);
		if (this.lazyFetch || (defaultLazyFetch && this.lazyFetch !== false))
			this.lazyFetch = true;
		else
			aggregatorId = this._registerWithAggregator(this._batchedPresentationHandler, this._errorPresentation);

		this.popup = new net.jazz.ui._RichHover({
			around: this.domNode,
			uri: this.uri,
			presentationUri: this.presentationUri,
			lazyFetch: this.lazyFetch,
			onDisable: dojo.hitch(this,this._disable),
			onLoad: this.onLoad,
			aggregatorId: aggregatorId,
			target: this.target
		});
	},
	
	refresh: function() {
		this._registerWithAggregator(this._batchedPresentationHandler, this._errorRefresh);
	},
	
	_registerWithAggregator: function(callback, errback) {
		return jazz.ui.internal.resourceLinkAggreagtor().register(this.uri, dojo.hitch(this, callback), dojo.hitch(this, errback));
	},
	
	_styleLink: function () {
		dojo.addClass(this.domNode, CSS_RL);
		this.arrowNode = document.createElement("img");
		dojo.addClass(this.arrowNode, CSS_ARROW);
		this.arrowNode.src = dojo.moduleUrl("jazz.ui", "graphics/ResourceLink/rh-twistie.gif");
		this.domNode.appendChild(this.arrowNode);
	},
	
	_errorPresentation: function (err) {
		console.warn("ResourceLink Error (" + this.uri + "): " + err.message);
		this.popup.disable();
	},
	
	_errorRefresh: function (err) {
		console.warn("ResourceLink Refresh Error (" + this.uri + "): " + err.message);
	},
	
	_disable: function () {
		if (!this.domNode) return;
		if (this.arrowNode && this.arrowNode.parentNode == this.domNode) {
			this.domNode.removeChild(this.arrowNode);
			delete this.arrowNode;
		}
		dojo.removeClass(this.domNode, CSS_RL);
		delete this.popup;
	},
	
	_batchedPresentationHandler: function(result) {
		if (!this.domNode) return;
		if (result.oauthRequired) {
			this.popup.setOauthRequired(result.token);
		} else if (result.compactRendering) {
			// LEGACY
			result = result.compactRendering;
			processCompactRendering(result);
			if (result.abbreviation && this.useAbbreviation) {
				this._applyTitle(result.abbreviation);
			} else if (result.title && !this.retainLinkText) {
				this._applyTitle(result.title);
			}
			if (result.icon && this.iconNode) {
				var iconImg;
				if (this.iconNode.tagName === "IMG") {
					iconImg = this.iconNode;
				} else {
					iconImg = document.createElement("img");
					this.iconNode.appendChild(iconImg);
				}
				iconImg.src = result.icon;
			}
			this.popup.setCompactRendering(result);
		} else if (result.oslcCompact) {
			result = result.oslcCompact;
			processOslcCompact(result);
			if (result.shortTitle && this.useAbbreviation) {
				this._applyTitle(result.shortTitle);
			} else if (result.title && !this.retainLinkText) {
				this._applyTitle(result.title);
			}
			if (result.icon && this.iconNode) {
				var iconImg;
				if (this.iconNode.tagName === "IMG") {
					iconImg = this.iconNode;
				} else {
					iconImg = document.createElement("img");
					this.iconNode.appendChild(iconImg);
				}
				iconImg.src = result.icon;
			}
			this.popup.setOslcCompact(result);
		}
		this.onLoad();
	},
	
	_applyTitle: function(title) {
		if (this.arrowNode && this.arrowNode.parentNode)
			this.arrowNode.parentNode.removeChild(this.arrowNode);
		this.domNode.innerHTML = title;
		if (this.arrowNode)
			this.domNode.appendChild(this.arrowNode);
	},
	
	destroy: function() {
		if (this.popup)
			this.popup.destroy();
		this.inherited(arguments);
	}
	
});
jazz.ui.ResourceLink.setLazyFetch = function(bool) {
	defaultLazyFetch = bool;
};

dojo.declare("net.jazz.ui._RichHover", jazz.ui.HoverPopup, {
	customClass: "jazz-ui-ResourceHover",
	
	maxHeight: 10000,
	
	_attachArounds: function() {
		this.inherited(arguments);
		for (var i = 0; i < this._aroundNodes.length; i++) {
			this.connect(this._aroundNodes[i],"onclick",this._cancelOpen);
		}
	},
	
	_cancelOpen: function(e) {
		if(this._openTimer){
			clearTimeout(this._openTimer);
			delete this._openTimer;
		}
	},
	
	createContents: function() {
		if (!this.compactRendering) {
			if (this.lazyFetch) {
				this._fetchCompactRendering();
			}
			return dojo.create("div", {innerHTML: messages.loading});
		} else {
			return this._createPresentation(dojo.create("div"));
		}
	},
	
	close: function(e) {
		if (e)
			dojo.stopEvent(e);
		this.inherited(arguments);
	},
	
	onOpen: function() {
		this._openListeners = [];
		this._openListeners.push(this.connect(dojo.body(), "onkeypress", this._onKeyPress));
		this._openListeners.push(this.connect(dojo.body(), "onclick", this._onBodyClick));
		this._openListeners.push(this.connect(dojo, "stopEvent", this._onStopEvent));
		if (window.postMessage)
			this._openListeners.push(this.connect(window, "onmessage", this._onMessage));
		this._renderResult();
	},
	
	onClose: function() {
		this.programmatic = false;
		this.updateToggleLink(true);
		for (var i = 0; i < this._openListeners.length; i++)
			this.disconnect(this._openListeners[i]);
		if (this._disableOnClose)
			this.disable();
	},
	
	toggleSize: function(e) {
		dojo.stopEvent(e);
		this.programmatic = "block";
		this.updateToggleLink(this._isFullDisplay);
		if (this._isFullDisplay) {
			this.oslcCompact ? this.displaySmallSummary() : this.displaySmallSummaryLegacy();
		} else {
			this.oslcCompact ? this.displayLargeSummary() : this.displayLargeSummaryLegacy();
		}
		this.position();
	},
	
	updateToggleLink: function(showMore) {
		if (this._presentation) {
			dojo.addClass(this._presentation.toggleLink, showMore ? "showMore" : "showLess");
			dojo.removeClass(this._presentation.toggleLink, showMore ? "showLess" : "showMore");
			this._presentation.toggleLink.innerHTML = showMore ? messages.showMore : messages.showLess;
		}
	},
	
	displaySmallSummaryLegacy: function(){
		this._isFullDisplay = false;
		if (this.compactRendering.smallPreview) {
			// update the iframe dimensions early to give an accurate size for the HoverPopup.position function
			this._updateIFrameLegacy();
			this._presentation.hoverIframe.src = this.compactRendering.smallPreview;
		}
	},
	
	displayLargeSummaryLegacy: function(){
		this._isFullDisplay = true;
		if (this.compactRendering.largePreview) {
			// update the iframe dimensions early to give an accurate size for the HoverPopup.position function
			this._updateIFrameLegacy();
			this._presentation.hoverIframe.src = this.compactRendering.largePreview;
		}
	},
	
	displaySmallSummary: function(){
		this._isFullDisplay = false;
		if (this.oslcCompact.smallPreview) {
			// update the iframe dimensions early to give an accurate size for the HoverPopup.position function
			this._updateIFrame();
			this._presentation.hoverIframe.src = this.oslcCompact.smallPreview.document;
		}
	},
	
	displayLargeSummary: function(){
		this._isFullDisplay = true;
		if (this.oslcCompact.largePreview) {
			// update the iframe dimensions early to give an accurate size for the HoverPopup.position function
			this._updateIFrame();
			this._presentation.hoverIframe.src = this.oslcCompact.largePreview.document;
		}
	},
	
	disable: function() {
		if (this._isOpen) {
			this._renderError(messages.infoNotAvailable);
			this._disableOnClose = true;
			return;
		}
		if (this.onDisable)
			this.onDisable();
		this.destroy();
	},
	
	_renderResult: function() {
		if (this.oauthRequired) {
			if (this._presentation) {
				this._presentation.destroy();
				delete this._presentation;
			}
			this.content.innerHTML = "";
			var loginLink = jazz.util.internal.login.insertLoginLink(this.content, this.oauthToken);
			this.connect(loginLink, "onclick", dojo.hitch(this,this._fetchCompactRendering,false,true));
		} else if (this.compactRendering) {
			this._createPresentation();
			if (this.compactRendering.smallPreview) 
				this.displaySmallSummaryLegacy();
			else 
				this.displayLargeSummaryLegacy();
		} else if (this.oslcCompact) {
			this._createPresentation();
			if (this.oslcCompact.smallPreview) 
				this.displaySmallSummary();
			else 
				this.displayLargeSummary();
		}
	},
	
	_renderError: function(message) {
		if (this._presentation) {
			this._presentation.destroy();
			delete this._presentation;
		}
		this.content.innerHTML = message;
	},
	
	setOauthRequired: function(token) {
		this.oauthRequired = true;
		this.oauthToken = token;
		this._redraw();
	},
	
	setCompactRendering: function(cr) {
		this.oauthRequired = false;
		this.compactRendering = cr;
		if (!cr.showHover) {
			this.disable();
		} else {
			if (window.postMessage) {
				if (cr.initialSmallHeight && !this.dynamicSmallHeight)
					this.dynamicSmallHeight = cr.initialSmallHeight;
				if (cr.initialLargeHeight && !this.dynamicLargeHeight && !this.dynamicSmallHeight)
					this.dynamicLargeHeight = cr.initialLargeHeight;
			}
			this._updatePresentation();
			this._redraw();
		}
	},
	
	setOslcCompact: function(oslc) {
		this.oauthRequired = false;
		this.oslcCompact = oslc;
		if (!oslc.showHover) {
			this.disable();
		} else {
			if (window.postMessage) {
				if (oslc.smallPreview && oslc.smallPreview.initialHeight && !this.dynamicSmallHeight)
					this.dynamicSmallHeight = parseLength(oslc.smallPreview.initialHeight);
				if (oslc.largePreview && oslc.largePreview.initialHeight && !this.dynamicLargeHeight && !this.dynamicSmallHeight)
					this.dynamicLargeHeight = parseLength(oslc.largePreview.initialHeight);
			}
			this._updatePresentation();
			this._redraw();
		}
	},
	
	_redraw: function() {
		if (this._isOpen) {
			this._renderResult();
			this.position();
		}
	},
	
	_updatePresentation: function() {
		// this function happens to work in both legacy mode (compact rendering) and in oslc ui preview mode
		var cr = this.compactRendering || this.oslcCompact, pres = this._presentation;
		if (pres) {
			if (cr.title) {
				pres.titleLink.innerHTML = cr.title;
			}
			if (cr.icon) {
				pres.iconNode.src = cr.icon;
				dojo.removeClass(pres.iconLink, "hidden");
				dojo.addClass(pres.titleWrapper, "titleIndent");
			} else {
				pres.iconNode.src = "";
				dojo.addClass(pres.iconLink, "hidden");
				dojo.removeClass(pres.titleWrapper, "titleIndent");
			}
			if (cr.showExpand) {
				dojo.removeClass(pres.bottomSection, "hidden");
			} else {
				dojo.addClass(pres.bottomSection, "hidden");
			}
		}
	},
	
	_createPresentation: function(node) {
		node = node || this.content;
		if (!this.oauthRequired && !this._presentation) {
			this._presentation = new net.jazz.ui._RichHoverPresentation({
				close: dojo.hitch(this,this.close),
				toggleSize: dojo.hitch(this,this.toggleSize),
				target: this.target
			});
			this._updatePresentation();
			this._presentation.titleLink.href = this._presentation.iconLink.href = this.presentationUri || this.uri;
			node.innerHTML = "";
			node.appendChild(this._presentation.domNode);
		}
		return node;
	},
	
	// this function is used to get the document in lazy fetch mode
	// and to authenticate when oauth is required in batched mode
	_fetchCompactRendering: function(legacy, allowOauthInterceptor, e) {
		if (e) {
			e.preventDefault();
		}
		var args = {
			url: this.uri,
			headers: {
				accept: legacy ? CT_COMPACT_RENDERING : CT_OSLC_COMPACT + "," + CT_COMPACT_RENDERING + "; q=0.5"
			}
		};
		if (!allowOauthInterceptor) {
			args.skipInterceptors = ["jazz.oauth"];
		}
		var dfd = jazz.client.xhrGet(args);
		dfd.addCallback(dojo.hitch(this,function(dfd,response) {
			var doc;
			if (dfd.ioArgs.xhr.getResponseHeader("Content-Type").substring(0,CT_COMPACT_RENDERING.length) === CT_COMPACT_RENDERING) {
				doc = {compactRendering: parseCompactRenderingXML(dojox.xml.DomParser.parse(response))};
			} else if (dfd.ioArgs.xhr.getResponseHeader("Content-Type").substring(0,CT_OSLC_COMPACT.length) === CT_OSLC_COMPACT) {
				doc = {oslcCompact: parseOslcCompactXML(dojox.xml.DomParser.parse(response))};
			}
			// if this is not an oauth request, it is a lazy fetch, handle as lazy
			if (!allowOauthInterceptor && doc) {
				this._handleLazyFetchJson(doc);
			}
			// if there is a valid document, return it
			if (doc)
				return doc;
			// otherwise throw a bogus error to trigger Errback chain
			throw {};
		}, dfd));
		dfd.addErrback(dojo.hitch(this, function (err) {
			// if the deferred was cancelled, the user cancelled login, no action to take
			if (!err || err.dojoType !== "cancel") {
				if (err && err.status === 401) {
					// this is only for lazy fetch mode
					// when we are in regular mode and are using this path for auth we will either have authed or will hit err.dojoType === "cancel"
					var token = new dojo._Url(this.uri).authority;
					this.setOauthRequired(token);
					this.aggregatorId = jazz.ui.internal.resourceLinkAggreagtor().registerOauth(token, this.uri, dojo.hitch(this,this._handleLazyFetchJson), dojo.hitch(this,this.disable));
				} else if (!legacy) {
					this._fetchCompactRendering(true, allowOauthInterceptor);
				} else {
					this.disable();
				}
			}
			return err;
		}));
		if (allowOauthInterceptor) {
			// After we authenticate with OAuth, we poke the ResourceLinkAggregator to resend
		 	// an xhr request for all ResourceLinks that access the same server.
			dfd.addBoth(dojo.hitch(this, function(response) {
				// if the deferred was cancelled, the user did not login, do not poke the aggregator
				if (response && response.dojoType === "cancel")
					return response;
				var aggregator = jazz.ui.internal.resourceLinkAggreagtor();
				if (this.aggregatorId && response && (response.compactRendering || response.oslcCompact)) {
					// exclude ourself from the aggregator call since we already have the result
					var obj = aggregator.unregisterOauth(this.oauthToken, this.uri, this.aggregatorId);
					// react to the result
					obj.callback(response);
					delete this.aggregatorId;
				}
				aggregator.resendOauth(this.oauthToken);
				return response;
			}));
		}
	},
	
	_handleLazyFetchJson: function(response) {
		if (response.compactRendering) {
			processCompactRendering(response.compactRendering);
			this.setCompactRendering(response.compactRendering);
		}
		else if (response.oslcCompact) {
			processOslcCompact(response.oslcCompact);
			this.setOslcCompact(response.oslcCompact);
		}
		this.onLoad();
	},
	
	_updateIFrameLegacy: function() {
		var cr = this.compactRendering;
		if (!cr || !this._presentation)
			return;

		var v = dijit.getViewport();
			
		// update width first so we can measure height accurately
		
		// HoverPopup will treat a width greater than viewport - 60 as viewport - 60
		// update the effective widths so the iframe is sized according to the max
		// account for the 5 px horizontal padding around the iframe
		if (cr.smallWidth)
			this.effectiveSmallWidth = (v.w - 60 < cr.smallWidth + 5) ? v.w - 65 : cr.smallWidth;
		if (cr.largeWidth)
			this.effectiveLargeWidth = (v.w - 60 < cr.largeWidth + 5) ? v.w - 65 : cr.largeWidth;
		
		if (this.effectiveSmallWidth && this.effectiveSmallWidth + 5 > 500)
			this.maxWidth = this.effectiveSmallWidth + 5;
		if (this.effectiveLargeWidth && this.effectiveLargeWidth + 5 > 500 && (this.effectiveLargeWidth > this.effectiveSmallWidth || !this.effectiveSmallWidth))
			this.maxWidth = this.effectiveLargeWidth + 5;
			
		var w = this._isFullDisplay ? this.effectiveLargeWidth : this.effectiveSmallWidth;
		if (w) {
			this._presentation.loadingImage.style.width = this._presentation.hoverIframe.style.width = w + "px";
			if (dojo.isIE) {
				// IE needs explicit width on the divider and width 100% breaks it out the side
				// allow 4 px for margins on wrapper dix
				this._presentation.divider.style.width = (w - 4) + "px";
				this._presentation.bottomDivider.style.width = (w - 4) + "px";
			}
		}
		
		var heightOffset = this._presentation.titleBar.offsetHeight + 48;
		
		// Use dynamic heights if they're available
		var smallHeight = this.dynamicSmallHeight || cr.smallHeight;
		var largeHeight = this.dynamicLargeHeight || (cr.initialLargeHeight && this.dynamicSmallHeight) || cr.largeHeight;
		
		// Restrict the ResourceLink to 80% of the viewport height
		// update the effective heights so the iframe is sized according to the max
		// account for the 45 px (+ titleBar height) vertical padding/content around the iframe using heightOffset
		if (smallHeight)
			this.effectiveSmallHeight = (v.h * 0.8 < smallHeight + heightOffset) ? v.h * 0.8 - heightOffset : smallHeight;
		if (largeHeight)
			this.effectiveLargeHeight = (v.h * 0.8 < largeHeight + heightOffset) ? v.h * 0.8 - heightOffset : largeHeight;
		
		var h = this._isFullDisplay ? this.effectiveLargeHeight : this.effectiveSmallHeight;
		if (h) {
			this._presentation.loadingImage.style.height = this._presentation.hoverIframe.style.height = h + "px";
		}
	},
	
	_updateIFrame: function() {
		var oslc = this.oslcCompact;
		if (!oslc || !this._presentation)
			return;

		var v = dijit.getViewport();
			
		// update width first so we can measure height accurately
		
		// HoverPopup will treat a width greater than viewport - 60 as viewport - 60
		// update the effective widths so the iframe is sized according to the max
		// account for the 5 px horizontal padding around the iframe
		if (oslc.smallPreview) {
			var smallWidthHint = parseLength(oslc.smallPreview.hintWidth);
			this.effectiveSmallWidth = (v.w - 60 < smallWidthHint + 5) ? v.w - 65 : smallWidthHint;
			if (this.effectiveSmallWidth + 5 > 500)
				this.maxWidth = this.effectiveSmallWidth + 5;
		}
		if (oslc.largePreview) {
			var largeWidthHint = parseLength(oslc.largePreview.hintWidth);
			this.effectiveLargeWidth = (v.w - 60 < largeWidthHint + 5) ? v.w - 65 : largeWidthHint;
			if (this.effectiveLargeWidth + 5 > 500 && (this.effectiveLargeWidth > this.effectiveSmallWidth || !this.effectiveSmallWidth))
				this.maxWidth = this.effectiveLargeWidth + 5;
		}
		
		var w = this._isFullDisplay ? this.effectiveLargeWidth : this.effectiveSmallWidth;
		if (w) {
			this._presentation.loadingImage.style.width = this._presentation.hoverIframe.style.width = w + "px";
			if (dojo.isIE) {
				// IE needs explicit width on the divider and width 100% breaks it out the side
				// allow 4 px for margins on wrapper div
				this._presentation.divider.style.width = (w - 4) + "px";
				this._presentation.bottomDivider.style.width = (w - 4) + "px";
			}
		}
		
		var heightOffset = this._presentation.titleBar.offsetHeight + 48;
		// Restrict the ResourceLink to 80% of the viewport height
		// update the effective heights so the iframe is sized according to the max
		// account for the 45 px (+ titleBar height) vertical padding/content around the iframe using heightOffset
		
		// Use dynamic heights if they're available
		if (oslc.smallPreview) {
			var smallHeight = this.dynamicSmallHeight || parseLength(oslc.smallPreview.hintHeight);
			this.effectiveSmallHeight = (v.h * 0.8 < smallHeight + heightOffset) ? v.h * 0.8 - heightOffset : smallHeight;
		}
		if (oslc.largePreview) {
			var largeHeight = this.dynamicLargeHeight || (oslc.largePreview.initialHeight && this.dynamicSmallHeight) || parseLength(oslc.largePreview.hintHeight);
			this.effectiveLargeHeight = (v.h * 0.8 < largeHeight + heightOffset) ? v.h * 0.8 - heightOffset : largeHeight;
		}
		
		var h = this._isFullDisplay ? this.effectiveLargeHeight : this.effectiveSmallHeight;
		if (h) {
			this._presentation.loadingImage.style.height = this._presentation.hoverIframe.style.height = h + "px";
		}
	},
	
	_onKeyPress: function(e) {
		if(e.type == "keypress" && e.keyCode == dojo.keys.ESCAPE)
			this.close();
	},
	
	_onBodyClick: function(e) {
		var target = e && e.target;
		if (target && !dojo.isDescendant(target, jazz.ui._getMasterPopup().domNode))
			this.close();
	},
	
	_onStopEvent: function(e) {
		if (e.type == "click")
			this._onBodyClick(e);
	},
	
	_onMessage: function(e) {
		if (!(this.compactRendering || this.oslcCompact) || !this._presentation || e.source !== this._presentation.hoverIframe.contentWindow ||
				((this._isFullDisplay && typeof (this.compactRendering ? this.compactRendering.initialLargeHeight : this.oslcCompact.largePreview.initialHeight) === "undefined") ||
				(!this._isFullDisplay && (typeof (this.compactRendering ? this.compactRendering.initialSmallHeight : this.oslcCompact.smallPreview.initialHeight) === "undefined"))))
			return;
		var data = e.data, prefix = this.oslcCompact ? OSLC_PREVIEW_RESIZE_PREFIX : PREVIEW_RESIZE_MESSAGE_PREFIX;
		if (data && data.indexOf(prefix) === 0) {
			var newHeight = parseInt(data.substring(prefix.length), 10);
			if (this._isFullDisplay) {
				this.dynamicLargeHeight = newHeight;
			} else {
				this.dynamicSmallHeight = newHeight;
			}
			if (this._isOpen) {
				if (this.compactRendering)
					this._updateIFrameLegacy();
				else
					this._updateIFrame();
				this.position();
			}
		}
	},
	
	destroy: function() {
		if (this._presentation) {
			this._presentation.destroy();
			delete this._presentation;
		}
		this.inherited(arguments);
	}
});

dojo.declare("net.jazz.ui._RichHoverPresentation", [dijit._Widget, dijit._Templated], {
	templatePath: dojo.moduleUrl("jazz.ui", "templates/ResourceLink.html"),
	messages: messages,
	
	postCreate: function() {
		if (dojo.isIE){
			dojo.addClass(this.loadingImage, "hidden");
			dojo.removeClass(this.hoverIframe, "loadingIframe");
		}
		if (this.target)
			this.titleLink.target = this.target;
	},
	
	_contentLoaded: function(e){
		//Not checking for IE becaues there is a bug in IE where iframe contents are not shown
		//when "visibility: hidden" is removed from the iframe's CSS attribute
		if ( e.type === "load"){
			dojo.addClass(this.loadingImage, "hidden");
			dojo.removeClass(this.hoverIframe, "loadingIframe");
		}
	}
});

var processCompactRendering = function(results) {
	if (results.oauthRequired)
		return;
	
	if (results.title)
		results.title = jazz.util.html.removeScript(results.title);
	
	if (results.smallPreview || results.largePreview)
		results.showHover = true;
	
	if (results.smallPreview && results.largePreview)
		results.showExpand = true;
	
	initInt(results, "smallWidth", 300);
	initInt(results, "smallHeight", 200);
	initInt(results, "largeWidth", Math.max(300, results.smallWidth));
	initInt(results, "largeHeight", Math.max(300, results.smallHeight));
	initInt(results, "initialSmallHeight");
	initInt(results, "initialLargeHeight");
};
var initInt = function(obj, prop, def) {
	if (obj[prop] && (Math.ceil(obj[prop]) == Math.floor(obj[prop]))) 
		obj[prop] = parseInt(obj[prop], 10);
	else if (def)
		obj[prop] = def;
};

var processOslcCompact = function(results) {
	if (results.title)
		results.title = jazz.util.html.removeScript(results.title);
	
	if (results.smallPreview) {
		results.showHover = true;
		
		if (!results.smallPreview.hintWidth)
			results.smallPreview.hintWidth = "300px";
		if (!results.smallPreview.hintHeight)
			results.smallPreview.hintHeight = "200px";
	}
	
	if (results.largePreview) {
		if (results.showHover)
			results.showExpand = true;
		else
			results.showHover = true;
			
		if (!results.largePreview.hintWidth)
			results.largePreview.hintWidth = "300px";
		if (!results.largePreview.hintHeight)
			results.largePreview.hintHeight = "300px";
	}
};

var parseCompactRenderingXML = function(crXml) {
	var crObj = {}, pref_width, pref_height, init_height,
		titleNodes = crXml.getElementsByTagNameNS("title", XMLNS_DC_TERMS),
		iconNodes = crXml.getElementsByTagNameNS("icon", XMLNS_JP),
		smallPreviewNodes = crXml.getElementsByTagNameNS("smallPreview", XMLNS_JP),
		largePreviewNodes = crXml.getElementsByTagNameNS("largePreview", XMLNS_JP);
	if (titleNodes.length > 0 && titleNodes[0].childNodes.length > 0) {
		crObj.title = titleNodes[0].childNodes[0].nodeValue;
	}
	if (iconNodes.length > 0) {
		crObj.icon = iconNodes[0].getAttributeNS("resource", XMLNS_RDF);
	}
	if (smallPreviewNodes.length > 0) {
		crObj.smallPreview = smallPreviewNodes[0].getAttributeNS("resource", XMLNS_RDF);
		pref_width = parseInt(smallPreviewNodes[0].getAttributeNS("preferred_width", XMLNS_JP), 10);
		pref_height = parseInt(smallPreviewNodes[0].getAttributeNS("preferred_height", XMLNS_JP), 10);
		init_height = parseInt(smallPreviewNodes[0].getAttributeNS("initialHeight", XMLNS_JP), 10);
		if (pref_width) {
			crObj.smallWidth = pref_width;
		}
		if (pref_height) {
			crObj.smallHeight = pref_height;
		}
		if (init_height) {
			crObj.initialSmallHeight = init_height;
		}
	}
	if (largePreviewNodes.length > 0) {
		crObj.largePreview = largePreviewNodes[0].getAttributeNS("resource", XMLNS_RDF);
		pref_width = parseInt(largePreviewNodes[0].getAttributeNS("preferred_width", XMLNS_JP), 10);
		pref_height = parseInt(largePreviewNodes[0].getAttributeNS("preferred_height", XMLNS_JP), 10);
		init_height = parseInt(largePreviewNodes[0].getAttributeNS("initialHeight", XMLNS_JP), 10);
		if (pref_width) {
			crObj.largeWidth = pref_width;
		}
		if (pref_height) {
			crObj.largeHeight = pref_height;
		}
		if (init_height) {
			crObj.initialLargeHeight = init_height;
		}
	}
	return crObj;
};

var parseOslcCompactXML = function(oslcXml) {
	var oslcObj = {}, nodes,
		titleNodes = oslcXml.getElementsByTagNameNS("title", XMLNS_DC_TERMS),
		shortTitleNodes = oslcXml.getElementsByTagNameNS("shortTitle", XMLNS_OSLC),
		iconNodes = oslcXml.getElementsByTagNameNS("icon", XMLNS_OSLC),
		smallPreviewNodes = oslcXml.getElementsByTagNameNS("smallPreview", XMLNS_OSLC),
		largePreviewNodes = oslcXml.getElementsByTagNameNS("largePreview", XMLNS_OSLC);
	if (titleNodes.length > 0 && titleNodes[0].childNodes.length > 0) {
		oslcObj.title = titleNodes[0].childNodes[0].nodeValue;
	}
	if (shortTitleNodes.length > 0 && shortTitleNodes[0].childNodes.length > 0) {
		oslcObj.shortTitle = shortTitleNodes[0].childNodes[0].nodeValue;
	}
	if (iconNodes.length > 0) {
		oslcObj.icon = iconNodes[0].getAttributeNS("resource", XMLNS_RDF);
	}
	if (smallPreviewNodes.length > 0) {
		oslcObj.smallPreview = parseOslcPreviewXML(smallPreviewNodes[0]);
	}
	if (largePreviewNodes.length > 0) {
		oslcObj.largePreview = parseOslcPreviewXML(largePreviewNodes[0]);
	}
	return oslcObj;
};
var parseOslcPreviewXML = function(previewNode) {
	var result = {};
	result.document = previewNode.getElementsByTagNameNS("document", XMLNS_OSLC)[0].getAttributeNS("resource", XMLNS_RDF);
	parseOslcPreviewElement(previewNode, result, "hintWidth");
	parseOslcPreviewElement(previewNode, result, "hintHeight");
	parseOslcPreviewElement(previewNode, result, "initialHeight");
	return result;
};
var parseOslcPreviewElement = function(node, result, elementName) {
	var nodes = node.getElementsByTagNameNS(elementName, XMLNS_OSLC);
	if (nodes.length > 0 && nodes[0].childNodes.length > 0)
		result[elementName] = dojo.trim(nodes[0].childNodes[0].nodeValue);
};

var parseLength = function(length) {
	length = length.toLowerCase();
	var px = length.indexOf("px");
	var em = length.indexOf("em");
	var ex = length.indexOf("ex");
	length = parseFloat(length);
	if (em > 0) return Math.ceil(length * getEmSize());
	if (ex > 0) return Math.ceil(length * getExSize());
	return Math.ceil(length);
};

var _emSize, _exSize;
var getEmSize = function() {
	if (!_emSize) {
		_emSize = jazz.util.ui.measureEm(getTestNode());
		// clear the cache when finished rendering
		setTimeout(function() {
			_emSize = null;
		},0);
	}
	return _emSize;
};
var getExSize = function() {
	if (!_exSize) {
		_exSize = jazz.util.ui.measureEx(getTestNode());
		// clear the cache when finished rendering
		setTimeout(function() {
			_exSize = null;
		},0);
	}
	return _exSize;
};
var _testNode;
var getTestNode = function() {
	if (!_testNode) {
		_testNode = dojo.create("div",{style: {visibility: "hidden", fontSize: "133.3333333%"}},dojo.body());
	}
	return _testNode;
};

})();