/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.SimpleToolbar");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");

(function() {

/*
 * PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * future with minimal or no warning.
 */
dojo.setObject("jazz.ui.simpleToolbar.hoverStyles",{
	BLUE_BORDER: "DEPRECATED blue-border",
	BLUE: "DEPRECATED blue",
	ORANGE: "DEPRECATED orange",
	ORANGE_BORDER: "DEPRECATED orange-border",
	GRAY: "DEPRECATED gray",
	NONE: "DEPRECATED none"
});

dojo.setObject("jazz.ui.simpleToolbar.buttonType",{
	SIMPLE   : "DEPRECATED simple",
	TWO_STATE: "DEPRECATED twoState"
});

var HoverStyles = {
	BLUE_BORDER: "blue-border",
	BLUE: "blue",
	ORANGE: "orange",
	ORANGE_BORDER: "orange-border",
	GRAY: "gray",
	NONE: "none"
};
var ButtonTypes = {
	SIMPLE   : "simple",
	TWO_STATE: "twoState"
};

var CLASS_NAME = "jazz-ui-SimpleToolbar";

dojo.declare("jazz.ui.SimpleToolbar", [dijit._Widget, dijit._Templated], {
	
	hoverStyle: HoverStyles.ORANGE_BORDER,
	
	templateString: "<div class='"+CLASS_NAME+"'></div>",
	
	postCreate: function() {
		var hs = this.hoverStyle;
		if (typeof hs === "string" && hs.indexOf("DEPRECATED ") === 0) {
			dojo.deprecated("jazz.ui.simpleToolbar.hoverStyles", "Use jazz.ui.SimpleToolbar.HoverStyles instead");
			hs = hs.substring(11);
		}
		dojo.addClass(this.domNode,CLASS_NAME+"-"+hs);
		this.items = new Array();
		dijit.setWaiRole(this.domNode,"toolbar");
	},
	
	// Adds a basic toolbar button with a 16x16 icon and hover state.
	addButton: function(context, callback, args) {
		//args: label, imgSrc, index, className, href, showLabel
		if (!args || (!args.imgSrc && !args.className)) {
			console.error("Toolbar: Invalid arguments. Skipping.");
			return;
		}
		
		if (typeof args.buttonType === "string" && args.buttonType.indexOf("DEPRECATED ") === 0) {
			dojo.deprecated("jazz.ui.simpleToolbar.buttonType", "Use jazz.ui.SimpleToolbar.ButtonTypes instead");
			args.buttonType = args.buttonType.substring(11);
		}
		
		if (args.buttonType == ButtonTypes.TWO_STATE && !args.imgSrcPressed) {
			args.imgSrcPressed = args.imgSrc;
		}
		
		var button = dojo.create("a", {
			className: CLASS_NAME+"-button",
			href: args.href || "#",
			title: args.label
		});
		
		dijit.setWaiRole(button,"button");
		if (args.className) dojo.addClass(button, args.className);
		var img = dojo.create("img", {src: "" + (args.imgSrc || this._blankGif), alt: args.label }, button);
		args._img = img;
		
		if (args.label && args.showLabel) {
			var labelSpan = dojo.create("span", {"class": CLASS_NAME+"-label"}, button);
			labelSpan.appendChild(document.createTextNode(args.label));
		}
		
		if (callback) {
			this.connect(button, "onclick", function(event) {
				if (dojo.hasClass(event.currentTarget, CLASS_NAME + "-button-disabled")) {
					dojo.stopEvent(event);
					return;
				}
				if (args.buttonType == ButtonTypes.TWO_STATE) {
					if (dojo.hasClass(button, CLASS_NAME+"-button-pressed")) {
						img.src = args.imgSrc;
						dojo.removeClass(button, CLASS_NAME+"-button-pressed");
						event.pressed = false;
					} else {
						img.src = args.imgSrcPressed;
						dojo.addClass(button, CLASS_NAME+"-button-pressed");
						event.pressed = true;
					}
				}
				if (!args.href) event.preventDefault();
				dojo.hitch(context, callback)(event);
			});
		}
		
		return this._insert({domNode: button, _isButton: true, args: args}, args.index);
	},
	addSeparator: function(index) {
		return this._insert({domNode: dojo.create("div",{className: CLASS_NAME+"-separator"})}, index);
	},
	addWidget: function(widget, args) {
		// widget:  must have domNode
		// args:  isButton, index
		if (!args) args = {};
		var domNode = widget.domNode;
		if (args.isButton === true) {
			dojo.addClass(widget.domNode,CLASS_NAME+"-button");
		}
		return this._insert(widget, args.index);
	},
	getIndex: function(item) {
		for (var i = 0; i < this.items.length; i++) {
			if (this.items[i] === item) {
				return i;
			}
		}
		return -1;
	},
	getItem: function(index) {
		return this.items[index] || null;
	},
	removeItem: function(/*index or widget*/ widget) {
		var index = -1;
		if (typeof widget == "number") { 
			//index was passed, get widget
			index = widget;
			widget = this.getItem(index);
		} else {
			//widget was passed, get index
			index = this.getIndex(widget);
		}
		if (index >= 0 && widget && widget.domNode) {
			widget.domNode.parentNode.parentNode.removeChild(widget.domNode.parentNode);
			return this.items.splice(index,1);
		}
		return null;
	},
	hideItem: function(/*index or widget*/ widget) {
		if (typeof widget == "number") widget = this.getItem(widget);
		if (widget && widget.domNode && widget.domNode.parentNode) {
			widget.domNode.parentNode.style.display = "none";
		}
	},
	showItem: function(/*index or widget*/ widget) {
		if (typeof widget == "number") widget = this.getItem(widget);
		if (widget && widget.domNode && widget.domNode.parentNode) {
			widget.domNode.parentNode.style.display = "";
		}		
	},
	enableButton: function(/*index or widget*/ widget) {
		if (typeof widget == "number") widget = this.getItem(widget);
		if (widget && dojo.hasClass(widget.domNode,CLASS_NAME+"-button")) {
			dojo.removeClass(widget.domNode,CLASS_NAME+"-button-disabled");
			if (widget._isButton) {
				dijit.setWaiState(widget.domNode, "disabled", false);
				widget.domNode.href = "#";
			}
		}
	},
	isPressed : function (/*index or widget*/ widget) {
		if (typeof widget == "number") widget = this.getItem(widget);
		return dojo.hasClass(widget.domNode, CLASS_NAME+"-button-pressed");
	},
	setPressed : function (/*index or widget*/ widget, pressed) {
		var isPressed = this.isPressed(widget);
		if ((isPressed&&pressed) || (!(isPressed||pressed))) return;
		
		if (typeof widget == "number") widget = this.getItem(widget);

		if (pressed) {
			dojo.addClass(widget.domNode, CLASS_NAME + "-button-pressed");
			if (widget.args && widget.args._img && widget.args.imgSrcPressed) {
				widget.args._img.src = widget.args.imgSrcPressed;
			}
		} else {
			dojo.removeClass(widget.domNode, CLASS_NAME + "-button-pressed");
			if (widget.args && widget.args._img && widget.args.imgSrc) {
				widget.args._img.src = widget.args.imgSrc;
			} 
		}
	},
	disableButton: function(/*index or widget*/ widget) {
		if (typeof widget == "number") widget = this.getItem(widget);
		if (widget && dojo.hasClass(widget.domNode, CLASS_NAME+"-button")) {
			dojo.addClass(widget.domNode, CLASS_NAME+"-button-disabled");
			if (widget._isButton) {
				dijit.setWaiState(widget.domNode, "disabled", true);
				widget.domNode.removeAttribute("href");
			}
		}
	},
	updateButton: function(widget, args) {
		// args: label, imgSrc, className
		if (widget && widget.domNode && widget.domNode.firstChild && widget.domNode.firstChild.nodeName.toUpperCase() == "IMG") {
			var img = widget.domNode.firstChild;
			if (args.label || args.label === "") {
				img.alt = args.label;
				widget.domNode.title = args.label;
			}
			if (args.imgSrc)
				img.src = args.imgSrc;
			if (args.className) {
				// need to override the whole className to remove the old class
				widget.domNode.className = CLASS_NAME+"-button " + args.className;
			}
		}
	},
	_insert: function(widget, index) {
		if (!widget) return null;
		var div = dojo.create("div",{className: CLASS_NAME + "-item"});
		div.appendChild(widget.domNode);
		if (index != null && index < this.items.length) {
			this.domNode.insertBefore(div,this.items[index].domNode.parentNode);
			this.items.splice(index,0,widget);
		} else {
			this.domNode.appendChild(div);
			this.items.push(widget);
		}
		return widget;
	}
});
jazz.ui.SimpleToolbar.HoverStyles = HoverStyles;
jazz.ui.SimpleToolbar.ButtonTypes = ButtonTypes;
})();