/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.StyledBox");

dojo.require("jazz.ui.Menu");
dojo.require("jazz.ui.internal.ImgMenuItem");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.fx");
dojo.require("jazz.util.ResizeNotifier");
dojo.require("jazz.util.html");
dojo.require("jazz.ui.SimpleToolbar");
dojo.require("jazz.ui.styledBox.TrimStyles");
dojo.require("jazz.ui.MenuPopup")

dojo.requireLocalization("jazz.ui", "StyledBoxMessages");

(function() {
var ImgMenuItem = jazz.ui.internal.ImgMenuItem;

/**
 * @name jazz.ui.StyledBox.TrimStyles
 * @namespace <p>A set of constants for the {@link jazz.ui.StyledBox#trim} field.</p>
 * <p>PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * future with minimal or no warning.</p>
 */ 
var TrimStyles = /** @lends jazz.ui.StyledBox.TrimStyles */{
	/** 
	 * Blue trim.
	 * @constant
	 */
	BLUE: "jazz-ui-StyledBox-shadow jazz-ui-StyledBox-blue",
	/** 
	 * Blue trim with an alpha channel drop shadow.
	 * @constant
	 */
	BLUE_ALPHA_SHADOW: "jazz-ui-StyledBox-shadow jazz-ui-StyledBox-shadow-thick jazz-ui-StyledBox-blue",
	/** 
	 * Gray trim.
	 * @constant
	 */
	GRAY: "jazz-ui-StyledBox-shadow jazz-ui-StyledBox-gray",
	/** 
	 * Gold trim.
	 * @constant
	 */
	GOLD: "jazz-ui-StyledBox-shadow jazz-ui-StyledBox-gold",
	/** 
	 * Green trim.
	 * @constant
	 */
	GREEN: "jazz-ui-StyledBox-shadow jazz-ui-StyledBox-green",
	/** 
	 * Teal trim.
	 * @constant
	 */
	TEAL: "jazz-ui-StyledBox-shadow jazz-ui-StyledBox-teal",
	/** 
	 * Purple trim.
	 * @constant
	 */
	PURPLE: "jazz-ui-StyledBox-shadow jazz-ui-StyledBox-purple",
	/** 
	 * Orange trim.
	 * @constant
	 */
	ORANGE: "jazz-ui-StyledBox-shadow jazz-ui-StyledBox-orange",
	/** 
	 * No trim.
	 * @constant
	 */
	TRIMLESS: "jazz-ui-StyledBox-shadow jazz-ui-StyledBox-trimless",
	/** 
	 * Wizard style trim.
	 * @constant
	 */
	WIZARD: "jazz-ui-StyledBox-shadow jazz-ui-StyledBox-shadow-thick jazz-ui-StyledBox-wizard",
	/** 
	 * Editor style trim.
	 * @constant
	 */
	EDITOR: "jazz-ui-StyledBox-shadow jazz-ui-StyledBox-editor",
	/** 
	 * Editor style trim with a focus state.
	 * @constant
	 */
	EDITOR_FOCUS: "jazz-ui-StyledBox-shadow jazz-ui-StyledBox-editor jazz-ui-StyledBox-editor-focus",
	/** 
	 * Yellow editor style trim.
	 * @constant
	 */
	EDITOR_YELLOW: "jazz-ui-StyledBox-shadow jazz-ui-StyledBox-editor jazz-ui-StyledBox-editor-yellow",
	/** 
	 * Login dialog trim.
	 * @constant
	 */
	LOGIN: "jazz-ui-StyledBox-no-shadow jazz-ui-StyledBox-login"
};
var ResizeNotifier = jazz.util.ResizeNotifier;
var messages;

/**
 * Constructs a new StyledBox. All fields may be initialized in the widget constructor, but the provided
 * setters must be used to change them for existing instances.
 * @name jazz.ui.StyledBox
 * @class <p>A box widget for web UIs.</p>
 * 	<p>PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * 	future with minimal or no warning.</p>
 * @constructor
 * @extends dijit._Widget
 * @extends dijit._Templated
 * @extends dijit._Container
 * @since 0.7
 */ 
dojo.declare("jazz.ui.StyledBox",
	[dijit._Widget, dijit._Templated, dijit._Container], /** @lends jazz.ui.StyledBox.prototype */{

	/** @private */
	templatePath: dojo.moduleUrl("jazz.ui", "templates/StyledBox.html"),
	
	/**#@+ @field */
	/**
	 * DOM node for contents of StyledBox.
	 * @type DomNode
	 * @required
	 */
	contentNode: null,
	/**
	 * A custom classname to be added to the StyledBox wrapper div.
	 * @type String
	 * @default null
	 */
	"class": null,			//specify this value to add custom classname to box wrapper div
	/**
	 * Specify the width (eg. "400px", "40%"). This is the outer width of the StyledBox (including trim).
	 * Zero and empty string result in natural width (width: auto).
	 * @type String
	 * @default ""
	 */
	width: "",
	/**
	 * Specify the height. This is the inner height of the content area of the StyledBox (not including trim).
	 * Zero and empty string result in natural height (height: auto).
	 * @type String
	 * @default ""
	 */
	height: "",
	/**
	 * Specify the title of the box. 
	 * @type String
	 * @default ""
	 */
	primaryTitle: "",
	/**
	 * Specify the secondary title of the box. Appears beside primary title, in a lighter color 
	 * @type String
	 * @default ""
	 */
	secondaryTitle: "",
	/**
	 * Specify whether the title should be truncated with ellipsis' to fit in the header of the StyledBox. 
	 * @type Boolean
	 * @default true
	 */
	truncateTitle: true,
	/**
	 * Specifies a url for the title of the StyledBox to link to.
	 * @type String
	 * @default ""
	 */
	titleURL: "",
	/**
	 * Specify the trim style of the StyledBox. Valid values are available under {@link jazz.ui.StyledBox.TrimStyles}.
	 * @type String
	 * @default TRIMLESS
	 */
	trim: TrimStyles.TRIMLESS,
	/**
	 * Specify whether the header color should get darker on hover. 
	 * @type Boolean
	 * @default true
	 */
	headerHoverState: true,				//causes viewlet to change its rendering on mouse over (adds -hover class to box's root node).
	/**
	 * Specify whether the header should default to hover state color. This is a slightly darker rendering for the header color.
	 * If true, {@link jazz.ui.StyledBox#headerHoverState} is ignored.
	 * @type Boolean
	 * @default false
	 */
	headerDefaultToHoverState: false,  	//causes box to default to its hover state rendering (if true, headerHoverState value is ignored)
	/**
	 * Specify whether the StyledBox should have a solid background color. 
	 * @type Boolean
	 * @default false
	 */
	showBackground: false,
	/**
	 * Specify the path to a 16x16 icon to appear in the header of the box to the left of the title. 
	 * @type String
	 * @default ""
	 */
	iconPath: "",
	/**
	 * Specify menu entries for the StyledBox drop down menu. Array of objects in the form
	 * {label: "", iconClass: "", onClick: function(){}, index: 0}
	 * @type Object[]
	 */
	menuItems: null,
	/**
	 * Specify a function that will be called when the user tries to open the menu (if specified, a menu arrow is added to the toolbar
	 * in the StyledBox header). The function must instantiate and return a dijit.Menu. If specified, menuProvider supersedes {@link jazz.ui.StyledBox#menuItems}. 
	 * @type Function
	 */
	menuProvider: null,
	/**
	 * Specify whether the StyledBox can be closed by the user. 
	 * @type Boolean
	 * @default true
	 */
	closable: true,
	/**
	 * Specify whether the StyledBox can be collapsed. 
	 * @type Boolean
	 * @default false
	 */
	collapsible: false,
	/**
	 * Specify whether the StyledBox is collapsed by default. 
	 * @type Boolean
	 * @default false
	 */
	collapsed: false,
	/**
	 * Hides the header title, icon and bgcolor. Header buttons appear if defined. No effect when using TRIMLESS. Ignores {@link jazz.ui.StyledBox#collapsible}. 
	 * @type Boolean
	 * @default false
	 */
	headerless: false,
	/**
	 * Causes the box to destroy the widget passed in 'contentNode' on close (if it's a widget or the domNode for a widget).
	 * @type Boolean
	 * @default false
	 */
	destroyContents: false,

	/**
	 * Called when the StyledBox is closed. Return false in your function to veto the close. 
	 * @type Function
	 */
	onClose: null,		//in your handlers, return boolean false to veto action (close, expand, collapse).
	/**
	 * Called when the StyledBox is opened. 
	 * @type Function
	 */
	onOpen: null,
	/**
	 * Called when the StyledBox is collapsed. Return false in your function to veto collapsing. 
	 * @type Function
	 */
	onCollapse: null,
	/**
	 * Called when the StyledBox is expanded. Return false in your function to veto expanding. 
	 * @type Function
	 */
	onExpand: null,
	/**#@-*/

	/** @private */
	constructor: function() {
		this._messages = messages = dojo.i18n.getLocalization("jazz.ui", "StyledBoxMessages");
	},
	/** @private */
	postCreate: function() {
		this.setWidth();
		this.setHeight();
		
		//create toolbar widget
		this.toolbar = new jazz.ui.SimpleToolbar({hoverStyle: jazz.ui.SimpleToolbar.HoverStyles.NONE});
		this._headerButtons.appendChild(this.toolbar.domNode);
		
		if (!this.contentNode && this.getChildren().length == 0) this.contentNode = document.createElement("div");
		this._bodyContainer.appendChild(this.contentNode);
		this.setTitle(this.primaryTitle);
		this.setSecondaryTitle(this.secondaryTitle);
		this.setTitleURL(this.titleURL);
		this.setTrim(this.trim);
		if (this.showBackground) this.setBackground(this.showBackground);
		if (this.menuItems || this.menuProvider) {
			this._createMenuButton();
		} else if (this.headerless && !this.closable) {
			this._headerText.style.display="none";
		}
		this.setIcon();
		if (this.collapsible && !this.headerless) {
			this.connect(this._header,'ondblclick',this._toggleCollapse);
			this.connect(this._headerButtons,'ondblclick',dojo.stopEvent);
		}
		this._updateCollapsedState();
		this.setClosable(this.closable);
		if (this.onOpen && typeof this.onOpen == "function") this.onOpen();
	},
	_setupTruncate: function() {
		if (this.primaryTitle && !this.headerless && this.truncateTitle) {
			this._queueTruncate(true);
			if (!this._windowResizeEvent) {
				this._windowResizeEvent = ResizeNotifier.register({
					context: this,
					callback: this._queueTruncate
				});	
			}
		}
	},
	/**
	 * Setter for {@link jazz.ui.StyledBox#collapsed }. Controls whether or not the StyledBox is collapsed. 
	 * @param {Boolean} collapsed The new value.
	 * @param {Boolean} animate Indicates if the styled box should animate to the new state. Defaults to true if undefined.
	 */
	setCollapsed: function(collapsed, animate) {
		if (this.collapsed != collapsed) {
			this._toggleCollapse(null, animate);
		}
	},
	/**
	 * Setter for {@link jazz.ui.StyledBox#closable}. Controls whether the close icon is present in the top right corner of the StyledBox. 
	 * @param {Boolean} closable The new value.
	 */
	setClosable: function(closable) {
		this.disconnect(this._closeEvent);
		if (closable) {
			if (!this._closeButton) {
				this._closeButton = this.toolbar.addButton(this,this.close,{
					label: messages.close,
					className: "close"
				});
			}
			this._headerText.style.display="";
		} else {
			this.toolbar.removeItem(this._closeButton);
			this._closeButton = null;
		}
		this.closable = closable;
	},
	_toggleCollapse: function(event, animate) {
		if (event) dojo.stopEvent(event);
		if (this._collapseEvent) return;
		if (animate !== false) animate = true;
		var args = {node: this._middleSection, duration: 200};
		if (this.collapsed) {
			 //call onExpand callback, if exists
			if (this.onExpand && typeof this.onExpand == "function" && this.onExpand() === false) return;
			if (animate) {
				this._middleSection.style.height = "1px";
				this._collapseAnimation = dojo.fx.wipeIn(args);
			}
			dojo.removeClass(this._inner,"jazz-ui-StyledBox-collapsed");
		} else {
			//call onCollapse callback, if exists
			if (this.onCollapse && typeof this.onCollapse == "function" && this.onCollapse() === false) return;
			if (animate) {
				args.properties = {height: { end: 0 }};
				this._collapseAnimation = dojo.animateProperty(args);
				this._middleSection.style.overflow = "hidden";
			}
		}	
		if (animate) {
			this._collapseEvent = this.connect(this._collapseAnimation, "onEnd", this._onCollapseAnimationEnd);
			this._collapseAnimation.play();
		} else {
			this._onCollapseAnimationEnd();
		}
	},
	_onCollapseAnimationEnd: function(){
		this.disconnect(this._collapseEvent);
		delete this._collapseEvent;
		this.collapsed = !this.collapsed;
		this._updateCollapsedState();
		this._middleSection.style.overflow = "";
		dojo.publish("jazz.util.ResizeNotifier.resizeEvent");
	},
	_updateCollapsedState: function(){
		if (this._expandCollapseButton) {
			var index = this.toolbar.getIndex(this._expandCollapseButton);
			this.toolbar.removeItem(this._expandCollapseButton);
		}
		dijit.setWaiState(this._headerPrimaryLink||this._headerText, "expanded", !this.collapsed);
		if (this.collapsed) {
			dojo.addClass(this._inner,"jazz-ui-StyledBox-collapsed");
			if (this.collapsible) {
				this._expandCollapseButton = this.toolbar.addButton(this, this._toggleCollapse, {
					className: "expand",
					label: messages.expand,
					index: index == null ? 1 : index
				});
			}
		} else {
			dojo.removeClass(this._inner,"jazz-ui-StyledBox-collapsed");
			if (this.collapsible) {
				this._expandCollapseButton = this.toolbar.addButton(this, this._toggleCollapse, {
					className: "collapse",
					label: messages.collapse,
					index: index == null ? 1 : index
				});
			}
		}
	},
	/**
	 * Adds a menu item to the drop down.  
	 * @param {Object} args Object in the form {label: "", iconClass: "", onClick: function(), index: 0} where index is optional
	 */
	addMenuItem: function(args) {
		// args format: {/* string */ label, /* function */ onClick, /* string */ iconClass, /* int (optional)*/ index}
		if (this.menuProvider) {
			console.error("Cannot use addMenuItem when using a menuProvider.");
			return;
		}
		if (!this.menuItems) {
			this.menuItems = new Array();
			this._createMenuButton();
		}
		if (typeof args.index == "number") {
			this.menuItems.splice(args.index,0,args);
		} else {
			this.menuItems.push(args);	
		}
		// if the menu has already been created, insert it
		if (this._menu) {
			if (typeof args.index == "number") {
				this._menu.addChild(new ImgMenuItem(args), args.index);	
			} else {
				this._menu.addChild(new ImgMenuItem(args));
			}
		}
	},
	_createMenuButton: function() {
		if (!this._openMenuButton) {
			var button = this._openMenuButton = this.toolbar.addButton(null, null, {
				label: messages.menu,
				className: "menu",
				index: 0
			});
			this._menuPopup = new jazz.ui.MenuPopup({
				align: "right",
				menuProvider: dojo.hitch(this, this._createMenu)
			}, button.domNode);
		}
	},
	/**
	 * Closes the StyledBox and removes it from the DOM.
	 * @noparams
	 */
	close: function(event) {
		if (event) dojo.stopEvent(event);
		
		// call onClose callback function, if it exists
		if (this.onClose && typeof this.onClose == "function") {
			if (this.onClose() === false) return false;
		}
		
		if (dojo.isIE == 7) {
			this._onClose(); //IE7 does not fadeOut alpha PNG drop shadows properly (shows ugly black bars) - so just close it
		} else {
			//fadeOut
			var args = {node: this.domNode, duration: 200};
			var fadeOut = this._fadeOutAnim = dojo.fadeOut(args);
			this.connect(fadeOut, "onEnd", this._onClose);
			fadeOut.play();
			this._fadeOutTimeout = setTimeout(dojo.hitch(this,this._onClose),1200);
		}
	},
	/**
	 * Setter for {@link jazz.ui.StyledBox#iconPath}. Adds/updates the icon in the header of the StyledBox. 
	 * @param {String} iconPath The path to to the new icon.
	 */
	setIcon: function(iconPath) {
		if (!iconPath) iconPath = this.iconPath;
		if (!iconPath || this.headerless) {
			if (this._iconImage) {
				this._iconImage.parentNode.removeChild(this._iconImage);
				this._headerIcon.style.display = "";
				this._iconImage = null;
			}
			return;
		}
		if (this._iconImage) {
			this._iconImage.src = iconPath;
		} else {
			this._headerIcon.style.display = "block";
			var iconImage = document.createElement("img");
			iconImage.src = iconPath;
			iconImage.className = "icon-image";
			this._headerIcon.appendChild(iconImage);
			this._iconImage = iconImage;
		}
		this.iconPath = iconPath;
	},
	/**
	 * Setter for {@link jazz.ui.StyledBox#height}. 
	 * @param {String} height A string representing the new height, including units.
	 */
	setHeight: function(/*String with unit*/ height) {
		if (height) {
			this._bodyContainer.style.height = height;
			this.height = height;
		} else if (this.height) {
			this._bodyContainer.style.height = this.height;
		} else {
			this._bodyContainer.style.height = "";
		}
	},
	/**
	 * Setter for {@link jazz.ui.StyledBox#width}. 
	 * @param {String} width A string representing the new width, including units.
	 */
	setWidth: function(/*String with unit*/ width){
		if (width) {
			this.domNode.style.width = width;
			this.width = width;
		} else if (this.width){
			this.domNode.style.width = this.width;
		} else {
			this.domNode.style.width = "";
		}
	},
	/**
	 * Setter for {@link jazz.ui.StyledBox#primaryTitle}. 
	 * @param {String} title The new primary title.
	 */
	setTitle: function(/* string */ title) {
		if (this._headerPrimary.firstChild) {
			this._headerPrimary.replaceChild(document.createTextNode(title),this._headerPrimary.firstChild);	
		} else {
			this._headerPrimary.appendChild(document.createTextNode(title));
		}
		this.primaryTitle = title;
		this._setupTruncate();
	},
	/**
	 * Setter for {@link jazz.ui.StyledBox#secondaryTitle}. 
	 * @param {String} title The new secondary title.
	 */
	setSecondaryTitle: function(/* string */ title) {
		this._headerSecondary.innerHTML = title;
		this.secondaryTitle = title;
		this._setupTruncate();
	},
	/**
	 * Setter for {@link jazz.ui.StyledBox#titleURL}. The title will be shown
	 * as a hyperlink only when a valid URL is set.
	 * @param {String} url The new url for the title.
	 */
	setTitleURL: function(/* string */ url) {
		this.titleURL = url ? url : "";
		if (url) {
			if (!this._headerPrimaryLink) {
				this._headerPrimaryLink = dojo.create("a",null,this._headerPrimary.parentNode,"first");
				this._headerPrimaryLink.appendChild(this._headerPrimary);
				dijit.setWaiRole(this._headerPrimaryLink, "heading");
				dijit.setWaiState(this._headerPrimaryLink, "labelledby", this.id+"_label");
				dijit.setWaiState(this._headerPrimaryLink, "expanded", !this.collapsed);
				this._headerText.removeAttribute("role");
				this._headerText.tabIndex = "-1";
			}
			this._headerPrimaryLink.href = url;
		} else if (this._headerPrimaryLink) {
			this._headerPrimaryLink.parentNode.replaceChild(this._headerPrimary, this._headerPrimaryLink);
			delete this._headerPrimaryLink;
			this._headerText.tabIndex = "0";
			dijit.setWaiRole(this._headerText, "heading");
			dijit.setWaiState(this._headerText, "expanded", !this.collapsed);
		}
	},
	/**
	 * Setter for {@link jazz.ui.StyledBox#trim}. See {@link jazz.ui.StyledBox.TrimStyles} for trim constants.
	 * @param {String} url The new trim style.
	 */
	setTrim: function(trim) {
		if (typeof trim === "string" && trim.indexOf("DEPRECATED ") === 0) {
			dojo.deprecated("jazz.ui.styledBox.TrimStyles", "Use jazz.ui.StyledBox.TrimStyles instead");
			trim = trim.substring(11);
		}
		this.disconnect(this._addHoverHandle);
		this.disconnect(this._removeHoverHandle);
		if (this.trim == TrimStyles.TRIMLESS && trim != TrimStyles.TRIMLESS) {
			this.disconnect(this._headerLinkHandle);
		}
		if (this.trim != null) {
			dojo.removeClass(this.domNode,this.trim);
			dojo.removeClass(this._inner,this.trim);
			dojo.removeClass(this._inner,this.trim+"-hover");
		}
		dojo.removeClass(this._inner,"jazz-ui-StyledBox-hover");
		dojo.removeClass(this._inner,TrimStyles.TRIMLESS+"-showheader");
		dojo.addClass(this.domNode,trim);
		dojo.addClass(this._inner,trim);
		if (this.headerless && trim != TrimStyles.TRIMLESS) {
			dojo.addClass(this._inner,"jazz-ui-StyledBox-noheader");
		} else {
			dojo.removeClass(this._inner,"jazz-ui-StyledBox-noheader");
		}
		if (trim == TrimStyles.TRIMLESS) {
			this._headerLinkHandle = this.connect(this._headerLink,'onclick', this._toggleTrimlessHeader);
			this._addHoverHandle = this.connect(this.domNode,'onmouseover',this._addHoverClass);
			this._removeHoverHandle = this.connect(this.domNode,'onmouseout',this._removeHoverClass);
			this._headerVisible = false;
			if (this.collapsed)	this._toggleTrimlessHeader();
		} else if ((this.headerHoverState || this.headerless) && !this.headerDefaultToHoverState) {
			//if we want the header to change style on hover
			var node;
			if (this.headerless) 
				node = this.domNode;
			else 
				node = this._header; 
			this._addHoverHandle = this.connect(node,'onmouseover',this._addHoverClass);
			this._removeHoverHandle = this.connect(this.domNode,'onmouseout',this._removeHoverClass);			
		} else if (this.headerDefaultToHoverState) {
			dojo.addClass(this._inner,trim+"-hover");
			dojo.addClass(this._inner,"jazz-ui-StyledBox-hover");
		}
		this.trim = trim;
	},
	/**
	 * Setter for {@link jazz.ui.StyledBox#showBackground}.
	 * @param {Boolean} background The new showBackground state.
	 */
	setBackground: function(background) {
		if (background)
			dojo.addClass(this._contentWrapper,"showBackground");
		else
			dojo.removeClass(this._contentWrapper,"showBackground");
	},
	_toggleTrimlessHeader: function(event, kill) {
		if (event) event.preventDefault();
		if (this._headerVisible || kill) {
			dojo.removeClass(this._inner,TrimStyles.TRIMLESS+"-showheader");
			this._headerLink.innerHTML = messages.showHeader;
			this._headerLink.title = messages.showHeader;
			this._headerVisible = false;
		} else {
			dojo.addClass(this._inner,TrimStyles.TRIMLESS+"-showheader");
			this._headerLink.innerHTML = messages.hideHeader;
			this._headerLink.title = messages.hideHeader;
			this._headerVisible = true;
		}
	},
	_createMenu: function (){
		if (this.menuProvider) {
			var menu = this.menuProvider();
		} else {
			menu = new jazz.ui.Menu();
			// add known menu contributions
			if (this.menuItems) {
				for (var i=0;i<this.menuItems.length;i++) {
					if (this.menuItems[i].separator) {
						menu.addChild(new dijit.MenuSeparator());
						continue;
					}
					if (!this.menuItems[i].label) {
						console.error("Error creating menu:  Menu item at index "+i+" is missing a label.");
						continue;
					}
					menu.addChild(new ImgMenuItem(this.menuItems[i]));
				}
			}
		}
		if (menu instanceof dijit._Widget) {
			dojo.addClass(menu.domNode, "jazz-ui-StyledBox-menu");
			this._menuConnects = [
				this.connect(menu, "onOpen", this._onMenuOpen),
				this.connect(menu, "onClose", this._onMenuClose)
			];
			this._menu = menu;
			return menu;
		} else {
			throw new Error("Error creating menu.");
		}
	},
	_onMenuOpen: function(event) {
		this._addHoverClass(null, true);
	},
	_onMenuClose: function(event) {
		this._removeHoverClass(null, true);
		if (!this.menuProvider) {
			dojo.forEach(this._menuConnects, this.disconnect);
			delete this._menuConnects;
			this._menu.destroy();
		}
		this._menu = null;
	},
	_addHoverClass: function (event, force) {
		dojo.addClass(this._inner,this.trim+"-hover");
		dojo.addClass(this._inner,"jazz-ui-StyledBox-hover");
		if (force) {
			this.disconnect(this._removeHoverHandle);
			this._removeHoverHandle = null;
		} 
	},
	_removeHoverClass: function (event, force) {
		dojo.removeClass(this._inner,this.trim+"-hover");
		dojo.removeClass(this._inner,"jazz-ui-StyledBox-hover");
		dojo.addClass(this._inner,this.trim);
		if (force) {
			if (!this._removeHoverHandle) {
				this._removeHoverHandle = this.connect(this.domNode,'onmouseout',this._removeHoverClass);
			}
		} 
	},
	/*
	 * Handler for onresize truncate call.
	 * Use yielding so that UI is not hung in IE6/7 during truncate's while loop.
	 */ 
	_queueTruncate: function(force) {
		if (!this.truncateTimeout) {
			this.truncateTimeout = setTimeout(dojo.hitch(this,this._truncate,force),400);
		}
	},
	_truncate: function(force) {
		if (!this.primaryTitle || !this.truncateTitle || !this._header) return;
		this.truncateTimeout = null;
		var node = this._headerPrimary, 
			title = this.primaryTitle,
			htmlEscape = jazz.util.html.escape;
		// force is used when the title has been changed and the truncate algorithm needs to be run
		// even if the clientWidth is the same
		// force == true is used since this function is called from many places, some of which
		// pass in parameters
		if (force == true || (this._titleIsTruncated && this._header.clientWidth > this._prevWidth)) {
			node.innerHTML = htmlEscape(title);
			this._titleIsTruncated = false;
			if (node.title) {
				// Remove title attribute to prevent confusion over the accessible name
				node.removeAttribute("title");
			} 
		}
		var headerHeight = this._header.clientHeight;
		if (this._headerText.clientHeight > headerHeight + 5) {
			while (this._headerText.clientHeight > headerHeight + 5) {
				if (this._headerText.clientHeight > headerHeight * 2) {
					title = title.substring(0,title.length/2);
				} else {
					title = title.substring(0,title.length-2);
				}
				while (title.lastIndexOf(" ") == title.length-1)
					title = title.substring(0,title.length-1);
				node.innerHTML = htmlEscape(title) + "...";
				if (node.innerHTML.length <= 6) break;
			}
			node.title = this.primaryTitle;
			this._titleIsTruncated = true;
			this._prevWidth = this._header.clientWidth;
		}
	},
	_onClose: function() {
		if (this._closed) return;
		this._closed = true;
		clearTimeout(this._fadeOutTimeout);
		if (this._fadeOutAnim) {
			this._fadeOutAnim.stop();
			delete this._fadeOutAnim;
		}
		var widget = dijit.byId(this.contentNode.id);
		if (widget) {
			if (this.destroyContents) { // Destroy the content node widget for the dialog
				if (dojo.isFunction(widget.destroyRecursive)) {
				    widget.destroyRecursive();
				}
			} else {
				// remove widget from dom to ensure it isn't destroyed.
				var dom = widget.domNode;
				dom && dom.parentNode && dom.parentNode.removeChild(dom);
			}
		}
		this.destroy();
	},
	destroy: function() {
		ResizeNotifier.unregister(this._windowResizeEvent);
		if (this.toolbar) this.toolbar.destroy();
		if (this.truncateTimeout) {
			// cancel deferred title truncation
			clearTimeout(this.truncateTimeout);
			delete this.truncateTimeout;
		} 
		if (this._menuPopup) {
			this._menuPopup.destroy();
			delete this._menuPopup;
		}
		if (this._menu) {
			this._menu.destroy();
			delete this._menu;
		}
		this.inherited(arguments);
	}
});
jazz.ui.StyledBox.TrimStyles = TrimStyles

})();