/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function () {
dojo.provide("jazz.app.ThemableNode");

dojo.require("dijit._Widget");
dojo.require("jazz.client.xhr");
dojo.require("jazz.util.theming");

/*
 * PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * future with minimal or no warning.
 * 
 * @since 0.7
 */

dojo.declare("jazz.app.ThemableNode", [dijit._Widget], {
	constructor: function (args) {
		if ( !jazz.util.theming.isThemingLocal() ) return;
		if (args && args.targetNode && (args.htmlPath || args.widgetName)) {
			this.htmlPath = args.htmlPath;
			this.widgetName = args.widgetName;
			this.targetNode = args.targetNode;
			var self = this;
			// only make requests for theming content if there is an active theme
			jazz.util.theming.jsDfd.addCallback(dojo.hitch(this, function () {
				if ( this.widgetName && dojo.getObject(this.widgetName)) {
					var constructor = dojo.getObject(this.widgetName);
					var x = new constructor({},this.targetNode);
				} else {
					var dfd = jazz.client.xhrGet({
						url: jazz.util.theming.themeFileUrl(this.htmlPath).toString()
					});
					dfd.addCallback(dojo.hitch(this, this.handleContent));
					dfd.addErrback(dojo.hitch(this, this.handleErr));
					
				}
			}));
		} else {
			throw new Error("Please include a targetNode and either an htmlPath or a widgetName");
		}
	},
	
	handleContent: function (content) {
		if (content) {
			if (this.targetNode) {
				this.targetNode.innerHTML = content;
			} else {
				this.domNode.innerHTML = content;
			}			
		}
	},
	
	handleErr: function (error) {
		// Since not all theming styles will have custom content for a themable node,
		// we just ignore errors by default.  This method exists so users implementing
		// ThemableNode can connect to it in order to perform any custom error handling. 
	}
});
})();
