/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ensemble.Widget");

dojo.require("dijit._Widget");

(function() {

/**
 * A abstract Widget that can be placed in a WidgetPanel. The term Widget does not
 * mean dojo/dijit widget - it's a higher-level concept, sometimes called Gadget.
 * Provides a common interface to communicate with any type of widget.
 * 
 * @since 3.0
 */
dojo.declare("jazz.ensemble.Widget", [dijit._Widget], {
	
	/**
	 * {WidgetContext} The context in which the Widget is being hosted. Provided
	 * by the container, never null or undefined.
	 */
	context: null,
	
	/**
	 * {string} A string containing widget type-specific definition information,
	 * used to initialize the widget. Provided by the container if exists.
	 * e.g. "http://widgetprovider.host.com/widget"
	 */
	definition: null,
	
	/**
	 * {boolean} Whether or not the user is allowed to modify the widget state.
	 */
	readOnly: false,
	
	/**
	 * {string} An opaque universally unique id for the widget. No assumptions should
	 * be made about the structure of the id. Also, this unique id depends on the
	 * existence of a uuid for the containing WidgetPanel. If the WidgetPanel does
	 * not have a uuid, this field will be null.
	 * 
	 * @since 3.0.1.1
	 */
	uuid: null,
	
	/**
	 * Refreshes the widget. Called by the container when the user requests a
	 * refresh.
	 */
	refresh: function() {},
	
	/**
	 * Returns the default title for the widget as defined in its definition, etc.
	 */
	getDefaultTitle: function() { return ""; }
});

})();
