/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ensemble.WidgetContext");

(function() {

/**
 * Provides container facilities to a Widget. This is the interface through which
 * Widgets can communicate with their container or environment.
 * 
 * @since 3.0
 */
dojo.declare("jazz.ensemble.WidgetContext", null, {
	
	/**
	 * {object[]} Menu items to show in the container menu for this Widget. This is in
	 * addition to the default menu items provided by the container. Item objects have the
	 * following properties:
	 * 	- {string} label the text to show for the menu item
	 * 	- {function} onClick the function to call when the user activates the menu item.
	 * 	- (optional) {string} either an image URL or a css class name specifying
	 * 		a background-image property for the menu item's icon, or null for none.
	 * 		The use of CSS class is deprecated as it does not work when the CSS is loaded
	 * 		in a different frame.
	 */
	menuContributions: [],
	
	/**
	 * {object} An object containing arbitrary properties associated with
	 * this context. e.g. widget type-specific configuration properties.
	 */
	properties: {},
	
	/**
	 * Gets the widget's 16x16 icon URL, or undefined if there is none.
	 * 
	 * @return {string} url the 16x16 icon url
	 */
	getIcon: function() {},
	
	/**
	 * Returns the memento, which is a string containing arbitrary persisted data
	 * for the widget. Provided by the container if exists. Can be set via setMemento.
	 * 
	 * @return {string} the widget memento
	 */
	getMemento: function() {},
	
	/**
	 * Returns an object with string properties representing preference key-value
	 * pairs. Provided by the container. Can be set via setPreferences.
	 * 
	 * @return {object} the widget preferences
	 */
	getPreferences: function() {},
	
	/**
	 * Returns the widget container's current secondary title.
	 * 
	 * @return {string} the current widget's secondary title
	 */
	getSecondaryTitle: function() {},
	
	/**
	 * Returns the widget container's current title.
	 * 
	 * @return {string} the current widget title
	 */
	getTitle: function() {},
	
	/**
	 * Returns the widget container's toolbar, where additional toolbar items can
	 * be added, or undefined if the container has no toolbar.
	 * 
	 * @return {jazz.ui.SimpleToolbar} the container's toolbar
	 */
	getToolbar: function() {},
	
	/**
	 * Returns whether or not the widget state is shared among multiple users or not.
	 * Widgets may use this information to decide whether or not to allow user-
	 * specific personalization of the state.
	 * 
	 * @return {boolean} whether or not the widget state is shared or not
	 */
	isShared: function() {},
	
	/**
	 * Sets the callback handler to be called by the container when the user
	 * requests to edit content. The control(s) for editing content will only be
	 * shown if a handler is set.
	 * 
	 * @param {function} handler a function to be called when user requests
	 * 		to edit content
	 */
	setEditContentHandler: function(/** function */ handler) {},
	
	/**
	 * Sets the icon for the widget. This should be a URL to a 16x16 image.
	 * 
	 * @param {string} url the 16x16 icon url
	 */
	setIcon: function(/** string */ url) {},
	
	/**
	 * Sets the callback handler to be called by the container when the user
	 * requests to maximize the widget.
	 * 
	 * @param {function} handler a function to be called when user requests
	 * 		to maximize. The function will be passed one boolean param that will
	 *      be true if this is a request to maximize, and false otherwise.
	 */
	setMaximizeHandler: function(/** function */ handler) {},
	
	/**
	 * Sets the widget memento value to the given one. This is used to persist
	 * arbitrary data.
	 * 
	 * @param {string} memento the memento to set.
	 */
	setMemento: function(/** string */ memento) {},
	
	/**
	 * Sets the given menu items to show in the container menu. This is in addition
	 * to the default menu items provided by the container.
	 * 
	 * @param {object[]} items an array of items to show in the menu. Each item object
	 * 		has the following properties:
	 * 		- {string} label the text to show for the menu item
	 * 		- {function} onClick the function to call when the user activates the menu item.
	 * 		- (optional) {string} either an image URL or a css class name specifying
	 * 			a background-image property for the menu item's icon, or null for none.
	 * 			The use of CSS class is deprecated as it does not work when the CSS is loaded
	 * 			in a different frame.
	 */
	setMenuContributions: function(/** object[] */ items) {},
	
	/**
	 * Sets the new preference values for the widget. To be called by widgets
	 * when their settings are changed by the user.
	 * 
	 * @param {object} prefs an object with string properties representing
	 * 		preference key-value pairs.
	 */
	setPreferences: function(/** object */ prefs) {},
	
	/**
	 * Sets the callback handler to be called by the container when the container
	 * is about to start a maximize or restore operation.
	 * 
	 * @param {function} handler a function to be called when container is about to
	 *      be maximized or restored. The function will be passed one boolean param
	 *      that will be true if the container is about to maximize, and false
	 *      otherwise.
	 */
	setPreMaximizeHandler: function(/** function */ handler) {},
	
	/**
	 * Sets the new secondary title for the widget. This is shown after the
	 * main title, and is never persisted.
	 * 
	 * @param {string} title the new secondary widget title.
	 */
	setSecondaryTitle: function(/** string */ title) {},
	
	/**
	 * Sets the callback handler to be called by the container when the user
	 * requests to edit settings. The control for editing settings will only be
	 * shown if a handler is set.
	 * 
	 * @param {function} handler a function to be called when user requests
	 * 		to edit settings
	 */
	setSettingsHandler: function(/** function */ handler) {},
	
	/**
	 * Sets the new title for the widget.
	 * 
	 * @param {string} title the new widget title.
	 */
	setTitle: function(/** string */ title) {},
	
	/**
	 * Sets the URL for the widget container title link. The container will
	 * show the title as a hyperlink only when a valid URL is set.
	 * 
	 * @param {string} url the url for the title link
	 */
	setTitleURL: function(/** string */ url) {}
});

})();
