/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.Wizard");

dojo.require("jazz.ui.Dialog");
dojo.require("jazz.ui.StyledBox");
dojo.require("jazz.util.ui")
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("dojo.hash");
dojo.require("jazz.ui.wizard.WizardStepList");

dojo.requireLocalization("jazz.ui", "WizardMessages");

(function() {
	
var ui = jazz.util.ui;
var TrimStyles = jazz.ui.StyledBox.TrimStyles;
var DEFAULT_LENGTH = 48;
var PADDING = 20;
var STEPS_URI_PREFIX = "/steps/";

dojo.declare("jazz.ui.Wizard", dijit._Widget, {
	
	onFinish: null,
	onCancel: null,
	width: "944px",
	
	// passed on to wizard content
	contentArgs: {
		steps: [],
		showStepSummary: true,
		footerNode: null,
		headerNode: null,
		customProcessFinish: null, 
		customProcessCancel: null,
		includeCancel: null,
		stepListLayout: "horizontal",
		hideTrim: false,
		customPrefix: null
	},

	constructor: function(args) {
		dojo.mixin(this.contentArgs, args);
		this.contentArgs.container = this;
		this.isEmbedded = isEmbedded();
		this.contentWidget = new jazz.ui.WizardContent(this.contentArgs);
		var boxArgs = {
			contentNode: this.contentWidget.domNode,
			closable: false,
			closeOnEsc: false,
			destroyContents: true
		};
		if (args.fullPageContainer && isNode(args.fullPageContainer)) {
			this.containerNode = args.fullPageContainer;
			var includeCancel = args.includeCancel != null ? args.includeCancel : false;
			dojo.toggleClass(this.contentWidget.wizardCancelButton, "noDisplay", !includeCancel);
			dojo.toggleClass(this.contentWidget.wizardButtonArea, "dialogOnly",false);
			dojo.toggleClass(this.contentWidget.wizardPanelArea, "dialogOnly",false);
			
			if(this.isEmbedded || args.hideTrim) {
				this.containerNode.appendChild(this.contentWidget.domNode);				
			} else {
				boxArgs.trim = TrimStyles.WIZARD;
				this.viewBox = new jazz.ui.StyledBox(boxArgs);
				this.containerNode.appendChild(this.viewBox.domNode);
			}			
			this.contentWidget.resizeButtons(includeCancel);
		} else {
			var self = this;
			var includeCancel = args.includeCancel != null ? args.includeCancel : true;
			dojo.toggleClass(this.contentWidget.wizardCancelButton, "noDisplay", !includeCancel);
			boxArgs.onOpen = function() {
				self.contentWidget.resizeButtons(includeCancel);
			};
			boxArgs.primaryTitle = args.primaryTitle || "";
			boxArgs.width = args.width ? args.width : this.width;
			dojo.toggleClass(this.contentWidget.wizardHRline, "noDisplay", true);
			this.viewBox = new jazz.ui.Dialog(boxArgs);
			this.contentWidget._viewBox = this.viewBox;
		}
	},
	
	addStep: function(step) {
		this.contentWidget.addStep(step);
	},
	
	removeStep: function(step) {
		this.contentWidget.removeStep(step);
	},

	addStepGroup: function(group) {
		return this.contentWidget.addStepGroup(group);
	},
	
	removeGroup: function(group) {
		this.contentWidget.removeGroup(group);
	},
	
	selectStep: function(position) {
		this.contentWidget.selectStep(position);
	}, 
	
	checkForPendingStepRequest: function() {
		this.contentWidget.checkForPendingStepRequest();
	},
	
	close: function() {
		if(this.viewBox) {
			this.viewBox.close();
		}	
	},
	
	uninitialize: function(){
		this.contentWidget.destroyRecursive();
		var step;
		for(var i=0; i < this.contentArgs.steps.length; i++) {
			step = this.contentArgs.steps[i];
			if (dojo.isFunction(step.destroyRecursive)) {
			    step.destroyRecursive();
			}
		}
		this.inherited(arguments);
	}
	
});

dojo.declare("jazz.ui.WizardContent", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("jazz.ui", "templates/Wizard.html"),

	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("jazz.ui", "WizardMessages");
		this.isEmbedded = isEmbedded();
		this.validSteps = new Array();
		this.stepGroups = new Array();
		this.selectedStepPosition = 0;
		this._steps = [];
		this._isFirstLoad = true;
	},
	
	postCreate: function(){
		if (this.headerNode  && isNode(this.headerNode)) {
			this.wizardHeaderArea.appendChild(this.headerNode);
		}
		if (this.footerNode  && isNode(this.footerNode)) { 
			this.wizardFooterArea.appendChild(this.footerNode);
		}
		
		//Set the prefix for stepDOM node IDs
		this.domPrefix = this.customPrefix || "stepDOM_";
		
		this._stepList = new jazz.ui.wizard.WizardStepList({orientation: this.stepListLayout});
		this.wizardStepArea.appendChild(this._stepList.domNode);
		
		if (this.fullPageContainer && isNode(this.fullPageContainer)) {
			dojo.removeClass(this._stepList._wizardStepsWrapper,"dialogOnly");
			if(this.stepListLayout.toLowerCase() == "vertical") {
				dojo.addClass(this._stepList._wizardStepsWrapper, "vertical");
				dojo.addClass(this.wizardPanelArea, "vertical");
				dojo.addClass(this.wizardButtonArea, "vertical");		
			}
			if(this.isEmbedded) {
				dojo.subscribe("/dojo/hashchange", this, "checkNavigationStatus");
			} else {
				dojo.subscribe("/dojo/hashchange", this, "_hashChanged");
			}
		}
		
		if (!this.showStepSummary){
			this.hideStepList();
		}
		
		if(this.steps && this.steps.length > 0) {
			var self = this;
			dojo.forEach(this.steps, function(step){
				self.addStep(step);
			});
			
			var stepToSelect = 1;
			var index = window.location.hash.indexOf(STEPS_URI_PREFIX);
			if (index != -1) {
				var uriStep = this._getStepFromHash();
				if (this._isValidStep(uriStep)) {
					stepToSelect = parseInt(uriStep);
				}	
			} 		
			this.selectStep(stepToSelect);
		}
	},

	showStepList: function() {
		this._stepList.domNode.style.display = "block";
	},
	
	hideStepList: function() {
		this._stepList.domNode.style.display = "none";
	},
	
	getSelectedStep: function() {
		return this._steps[this.selectedStepPosition - 1];
	},
	
	addStep: function(step, groupNode) {
		this._steps.push(step);
		step.position = this._steps.length;

		//If the step is external, let's call a function to preload the contents of it's iframe.
		if(step.isExternal) {
			step.setStepPosition(step.position);
		}
		
		//Append the step to the DOM but keep it hidden
		step._stepDOM = dojo.create("div", { id: this.domPrefix + step.position, style: { display: "none" }});
		step._stepDOM.appendChild(step.domNode);
		this.wizardPanelArea.appendChild(step._stepDOM);
		
		//Add this step to the list of valid steps
		this.validSteps.push(step.position);

		step.wizard = this;
		if (step.inStepList) {
			step = this._stepList.addStep(step, groupNode);
		}	
		if (this._steps.length > 1) {
			dojo.toggleClass(this.wizardPreviousButton, "noDisplay", false);
			dojo.toggleClass(this.wizardNextButton, "noDisplay", false);
		}
		return step;
	},
	
	/* Allows for a step to be removed from the Wizard.
	 * 
	 * Known Issues: Causes step numbers to become out of order. This does not affect
	 * vertical step list wizards (since they don't show actual step numbers). But 
	 * this will cause problems for horizontal wizards.
	 * 
	 */
	
	removeStep: function(step) {
		if(step && step.position > 0) {
			//First remove the step domNode
			var domNode = dojo.byId(this.domPrefix + step.position);
			if(domNode) {
				this.wizardPanelArea.removeChild(domNode);
			}

			//Next, remove the step from the Step List
			//Note: Won't remove anything for horizontal step lists at this time.
			step.wizard = this;
			if (step.inStepList) {
				this._stepList.removeStep(step);		
			}
			
			//Remove the step from the list of valid steps
			var index = dojo.indexOf(this.validSteps, step.position);
			this.validSteps.splice(index, 1);
		}
	},
	
	removeGroup: function(group) {
		if(group && group._stepIndexes) {
			dojo.forEach(group._stepIndexes, function(x) {
				var step = this._steps[x - 1];
				this.removeStep(step);
			}, this);
			this._stepList.removeGroup(group);	
		}
	},
	
	/* Accepts a single parameter that contains a group name and an array of WizardStep objects.
	 * 
	 * {	name: "Rational Application Name",
	 * 		steps: [ WizardStep1, WizardStep2, WizardStep3 ]
	 * }
	 * 
	 * Note: Currently only Vertical Step Lists support step groups.
	 * 
	 * Returns: A handle to the group node it created in the step list
	 * 
	 */
	
	addStepGroup: function(group) {
		if(!group) return;
		if(group.name) {
			var node = this._stepList.addStepGroup(group.name);
		}
		var lastStep;
		var stepIndexes = new Array();
		if(group.steps && group.steps.length > 0) {
			dojo.forEach(group.steps, function(step){
				this.addStep(step, node);
				lastStep = step;
				stepIndexes.push(this._steps.length);
			}, this);
		}
		node._stepIndexes = stepIndexes;
		node._lastStepIndex = this._steps.length;
		this.stepGroups.push(node);

		//Remove the bottom-border on the last step in the group
		dojo.addClass(lastStep._stepListDOM, "lastStep");

		return node;
	},
	
	selectStep: function(stepPosition) {
		if (!this._isValidStep(stepPosition)){
			console.error(this.messages.stepNotFound);
			stepPosition = 1;
		}
		if (this.fullPageContainer && isNode(this.fullPageContainer)) {
			this.updateURI(stepPosition);
		} else {
			this._processSelectedStep(stepPosition);
		}	
	},
	
	_processSelectedStep: function (newStepPosition) {
		if (!this._isValidStep(newStepPosition)){
			console.error(this.messages.stepNotFound);
			newStepPosition = 1;
		}
		
		var previousStepPosition = this.selectedStepPosition;
		this.selectedStepPosition = newStepPosition;
		var selectedStep = this.getSelectedStep();

		/* Don't focus the step if it's already focused (useful for Embedded Steps where the hash is always changing) */
		if(previousStepPosition != newStepPosition) {
			selectedStep.focusStep();
		}
		
		/* External steps need to be notified whenever the hash changes so that they can
		 * communication with their iFrames. If the step is external, we will call the
		 * checkNavigationStatus() method which will handle this communication.
		 */
		if(selectedStep.isExternal) {
			selectedStep.checkNavigationStatus();
		}
		
		if(this.stepGroups.length > 0 && !this.isEmbedded) {
			this._updateGroupUI(previousStepPosition, newStepPosition);	
		}

		this.updateSummaryHighlight();
		this.displayStepContent();	
		this.updateNavigationControls();
	},
	
	_isValidStep: function (value) {
		if (isNaN(value)) {
			return false;
		}
		var testStep = parseInt(value);
		if (testStep > 0 && testStep <= this._steps.length) {
			return true;
		}
		return false;
	},
	
	_updateGroupUI: function(oldIndex, newIndex) {
		dojo.forEach(this.stepGroups, function(group) {
			if(oldIndex == group._lastStepIndex) {
				dojo.removeClass(group, "inProgress");
				dojo.addClass(group, "collapsed completed");
			}
			if(dojo.indexOf(group._stepIndexes,newIndex) != -1) {
				dojo.removeClass(group, "collapsed");
				dojo.addClass(group, "inProgress");
			}
		});
	},
	
	trySelectStep: function(stepPosition) {
		if (!this._isValidStep(stepPosition)){
			return false;
		}
		var step = this._steps[stepPosition - 1];
		var isInactive = dojo.hasClass(step, "inactiveStep");
		if (step.enabled && !isInactive) {
			this.selectStep(stepPosition);
			return true;
		}
		return false;
	},
	
	_hashChanged: function() {
		this.updateURI(null);
	},
	
	updateURI: function (stepNumber) {
		var uriStep = this._getStepFromHash();
		if(stepNumber) {
			var stepPosition = stepNumber;
			if(uriStep != stepPosition) {
				var newHash = "#" + STEPS_URI_PREFIX + stepPosition;
				self.location.hash = newHash;
				//Since we changed the hash, the _hashChanged will pick it up and handle processing the step automatically
				//But, if it's embedded.. we need to call it manually since it's hooked to checkNavigationStatus by default.
				if(this.isEmbedded) {
					this._hashChanged();	
				} 
				return;
			}
		} else {
			//If there was no step specified in the hash, default to 1
			var stepPosition = uriStep || 1;
		}
		//Check to make sure the step was not removed...
		var pos = dojo.indexOf(this.validSteps, stepPosition);
		if(pos === -1) {
			//User might be trying to load an embedded step.
			//Save the step number, and wait for the Application to
			//call checkForPendingStepRequest().
			this._hideChildren(this.wizardPanelArea);
			this.loadingDiv.style.display = "block";
			this.requestedStepNumber = stepPosition;
			
			//Backup Plan: If the caller never calls checkForPendingStepRequest()
			//This will prevent the user from "Loading..." forever.
			this.requestedStepTimeout = window.setTimeout(dojo.hitch(this, "pendingStepRequestTimeOut"), 10000); /* 10s */
			
		} else {
			this._processSelectedStep(stepPosition);
			if(this._isFirstLoad) {
				this.enablePreviousSteps(stepPosition);
			}
		}
		this._isFirstLoad = false;
	},
	
	checkForPendingStepRequest: function() {
		if(this.requestedStepTimeout) {
			window.clearTimeout(this.requestedStepTimeout);
		}
		if(this.requestedStepNumber) {
			var stepPosition = this.requestedStepNumber;
			var pos = dojo.indexOf(this.validSteps, stepPosition);
			if(pos === -1) {
				//User is trying to directly access a step that has been removed
				//Redirect them to the start of the wizard.
				stepPosition = 1;
			}
			this.requestedStepNumber = null;
			this._processSelectedStep(stepPosition);
			this.enablePreviousSteps(stepPosition);
		}
	},
	
	pendingStepRequestTimeOut: function() {
		//Just load the first step
		this._processSelectedStep(1);
	},

	//Mark all previous steps as enabled	
	enablePreviousSteps: function(stepPosition) {
		dojo.forEach(this._steps, function(step) {
			if(!step.enabled && step.position <= stepPosition) {
				this._stepList.enableStep(step);
			}
		}, this);
	},
	
	_getStepFromHash: function() {
		var index = self.location.hash.indexOf(STEPS_URI_PREFIX);
		if (index != -1) {
			var hashstr = self.location.hash.substr(1);
			var hparts = hashstr.split("&");
			for (var x = 0; x < hparts.length; x++) {
				var i = hparts[x].indexOf(STEPS_URI_PREFIX);
				if (i != -1) {
					return parseInt(hparts[x].substring(i + STEPS_URI_PREFIX.length));			
				}
			}
		}
	},
	
	updateSummaryHighlight: function(){
		this._stepList.selectStep(this.getSelectedStep());
	},
	
	updateNavigationControls: function(){
		var step = this.getSelectedStep();
		var canFlipPrev = step.canFlipToPreviousPage() && this.selectedStepPosition > 1;
		var canFlipNext = step.canFlipToNextPage() && this._steps.length > this.selectedStepPosition;
		var canFinish = step.canFinish();
		var canCancel = step.canCancel();
		
		if (this.isEmbedded) {
			//If this wizard is embedded, we should not update the buttons directly
			//but instead pipe the values into the URL of our parent.		
			//Multiplying the values by 1 to convert them to integers to simplify/obfuscate the URL
			
			var parentURL = getParentURL();
			if(!parentURL) {
				return;
			}

			//Can go Next or Prev should not depend on position in embedded wizard since it could only have 1 step...
			canFlipPrev = step.canFlipToPreviousPage();
			canFlipNext = step.canFlipToNextPage();
			
			//Get the dynamic height of the iFrame (default 1000px)
			var h = 1000;
			if(this.embedPanelArea && this.embedPanelArea.offsetHeight > 0) {
				h = this.embedPanelArea.offsetHeight;
			}
			parent.location.href = parentURL + "&h=" + h + "&cf=" + canFinish*1 + "&cn=" + canFlipNext*1 + "&cp=" + canFlipPrev*1 + "&cc=" + canCancel*1;

		} else {
			this.wizardPreviousButton.disabled = !canFlipPrev;
			this.wizardNextButton.disabled = !canFlipNext;
			this.wizardFinishButton.disabled = !canFinish;
			this.wizardCancelButton.disabled = !canCancel;
			dojo.toggleClass(this.wizardButtonArea, "noDisplay", !step.includeNavigationUI);
		}
	},
	
	displayStepContent: function() {
		if(this.isEmbedded) {
			dojo.empty(this.embedPanelArea);
			dojo.addClass(this.wizardContainer, "noDisplay");
			this.embedPanelArea.appendChild(this.getSelectedStep().domNode);
		} else {
			dojo.removeClass(this.wizardContainer, "noDisplay");
			this._hideChildren(this.wizardPanelArea);
			var st = dojo.byId(this.domPrefix + this.selectedStepPosition);
			if(st) {
				st.style.display = "block";
				this.updateDialogPosition();
			} else {
				//Basically, DOM is not yet loaded... we need to start an async task to check DOM and
				//show the step once the DOM is added to the page.				
				if(!this.DOMChecker) this.DOMChecker = window.setInterval(dojo.hitch(this, "_isDOMReady"), 30);
			}
		}
	},
	
	_isDOMReady: function() {
		//Checks to see if the DOM has been added to the page. If it has, then we can show the step the user selected 
		//This only happens when the entire wizard page is refreshed.
		//Usually this only runs once before the interval is cleared.
		var st = dojo.byId(this.domPrefix + this.selectedStepPosition);
		if (st) {
			st.style.display = "block";
			this.updateDialogPosition();
			window.clearInterval(this.DOMChecker);
			this.DOMChecker = null;
		} else {
			//Could be trying to load a step that was removed...
			this.updateURI();
		}
	},
	
	updateDialogPosition: function() {
		if(!this.fullPageContainer && this._viewBox) {
			this._viewBox.updatePosition();
		}
	},
	
	_hideChildren: function(node) {
		for(var x=0; x < node.childNodes.length; x++) {
			if(node.childNodes[x] && node.childNodes[x].style) {
				node.childNodes[x].style.display = "none";
			}
		}
	},
		
	onNavigationButtonClick: function(e) {
		var button = e.target;
		var step = this.getSelectedStep();
		if (button === this.wizardPreviousButton && step.canFlipToPreviousPage()) {
			step.onPrevious();
		} else if (button === this.wizardNextButton && step.canFlipToNextPage()) {
			step.onNext();
		} else if (button === this.wizardFinishButton && step.canFinish()) {
			step.onFinish();
		} else if (button === this.wizardCancelButton && step.canCancel()) {
			step.onCancel();
		}	
	},
	
	processPrevious: function() {
		if (!this.isEmbedded) {
			var pos = dojo.indexOf(this.validSteps, this.selectedStepPosition);
			this.selectStep(this.validSteps[pos-1]);
		} else if(getParentURL()) {
			//Send a message to the parent wizard that we are ready to go back a step. 
			parent.location.href = getParentURL() + "&pp=1";
		}
	}, 
	
	processNext: function() {
		if (!this.isEmbedded) {
			var pos = dojo.indexOf(this.validSteps, this.selectedStepPosition);
			this.selectStep(this.validSteps[pos+1]);
		} else if(getParentURL()) {
			//Send a message to the parent wizard that we are ready to advance the step. 			
			parent.location.href = getParentURL() + "&pn=1";			
		}
	}, 
	
	processFinish: function() {
		if (this.customProcessFinish && dojo.isFunction(this.customProcessFinish)) {
         	this.customProcessFinish();
         }
		 this.container.close();
	}, 
	
	processCancel: function() {
		if (this.customProcessCancel && dojo.isFunction(this.customProcessCancel)) {
         	this.customProcessCancel();
      	}
		this.container.close();
	},	

	resizeButtons: function(includeCancel) {
		var tester = document.createElement("div"); 
		document.body.appendChild(tester);
		
		var words = [messages.previous,messages.next, messages.finish];
		if (includeCancel) {
			words.push(messages.cancel);
		}
		var newWidth = ui.findMaxStringDisplayWidth(words, tester);
		
		if (newWidth > DEFAULT_LENGTH) {
			newWidth += PADDING;
			var newWidthStyle = newWidth + "pt";
			this.wizardPreviousButton.style.width = newWidthStyle;
			this.wizardNextButton.style.width = newWidthStyle;
			this.wizardFinishButton.style.width = newWidthStyle;
			if (includeCancel) {
				this.wizardCancelButton.style.width = newWidthStyle;
			}
		}
	},
	
	checkNavigationStatus: function() {
		/* Checks the URL to determine the state of the navigation controls */
		var qstr = self.location.hash.substr(1);
		var params = dojo.queryToObject(qstr);
		if(params && params.previous === "true") {
			this.getSelectedStep().onPrevious();
		} 
		else if(params && params.next === "true") {
			this.getSelectedStep().onNext();
		}
	},
	
	uninitialize: function(){
		this._stepList.destroyRecursive();
		
		this.inherited(arguments);
	}
});

function getParentURL() {
	var qstr = self.location.hash.substr(1);
	var params = dojo.queryToObject(qstr);
	var parentURL = params.parent;
	if(!parentURL) {
		try {
			url = parent.location.href;
			var i = url.indexOf("#");
			if(i) {
				var hash = url.substr(i);
				var hashParts = hash.split('&');
				hash = hashParts[0]; //only want the #/steps/18 part of the URL	
				parentURL = url.substr(0,i) + hash;
				//Add parent url back to the iFrame src since it was lost in a redirect
				self.location.hash = self.location.hash + "&parent=" + encodeURIComponent(parentURL);
			}
		} catch (e) {
			 // ignore: cross-domain browser security exception because we can't access the location
		};
		if (!parentURL) {
			console.warn("This embedded step cannot find its parent URL. Additional errors may occur.");  
			parentURL = "";
		} 
	}
	return parentURL;
}

function isNode(o){
	return o && o.nodeType && o.cloneNode;
}

function isEmbedded() {
	var qstr = self.location.search.substr(1);
	var params = dojo.queryToObject(qstr);
	return (params && params.embed === "true") ? true : false;
}

})();