/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("jazz.app._AjaxApplication"); //$NON-NLS-1$

dojo.require("jazz.ui.Menu"); //$NON-NLS-1$
dojo.require("jazz.ui.MenuPopup"); //$NON-NLS-1$
dojo.require("jazz.ui.PageTemplate"); //$NON-NLS-1$
dojo.require("jazz.ui.UnsupportedBrowserWarning"); //$NON-NLS-1$
dojo.require("jazz.app.internal.HomeMenu"); //$NON-NLS-1$
dojo.require("jazz.app.internal.CustomNavbar"); //$NON-NLS-1$
dojo.require("jazz.app.internal.Footer"); //$NON-NLS-1$
dojo.require("jazz.app.internal.BannerButtons"); //$NON-NLS-1$
dojo.require("jazz.app.internal.LoginLogoutArea"); //$NON-NLS-1$
dojo.require("jazz.app.ThemableNode"); //$NON-NLS-1$
dojo.require("jazz.app.discovery.PersonalDashboards"); //$NON-NLS-1$
dojo.require("jazz.app.internal.BannerSearch"); //$NON-NLS-1$
dojo.require("net.jazz.ajax.ui._Application"); //$NON-NLS-1$
dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$
dojo.require("net.jazz.ajax.ui.internal.PageRegistry"); //$NON-NLS-1$
dojo.require("net.jazz.ajax.ui.PageContainer"); //$NON-NLS-1$
dojo.require("net.jazz.ajax.core.internal.util"); //$NON-NLS-1$
dojo.require("jazz.app.internal.applicationAbout"); //$NON-NLS-1$
dojo.require("jazz.util.browserSupport"); //$NON-NLS-1$
dojo.require("jazz.util.theming"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("jazz.app.internal", "_AjaxApplicationMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var messages = dojo.i18n.getLocalization("jazz.app.internal", "_AjaxApplicationMessages"); //$NON-NLS-1$ //$NON-NLS-2$
var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var VIEW_PAGE = net.jazz.ajax.internal.events.viewPage;
var browserSupport = jazz.util.browserSupport;
var applicationAbout = jazz.app.internal.applicationAbout;

dojo.declare("jazz.app._AjaxApplication", net.jazz.ajax.ui._Application, { //$NON-NLS-1$

	name: "Rational Jazz Team Server",
	
	iconSmall: dojo.moduleUrl("jazz.app", "graphics/JazzUI/icons/Jazz_16.png"),
	
	iconLarge: null,

	config: {	// manipulate the config object before calling start().  
		
		showLoginLogout: true,		// Shows login/logout links in banner.
		
		usePageRegistry: true, 		// Decides whether app navbar should be prepopulated from the page registry.
		
		buttons: [],				// Push objects onto this array to define banner buttons.
									// Args: label, iconSrc, align, href, pageId, suppressPage, menuProvider
		
		navbarItems: []				// Push objects onto this array to define navbar items/menus.
									// Args: same as buttons
		
		// Additional parameters:
		//
		//   login, logout 			Replace the behaviour of the login/logout links.
		// 							this.config.login = { href: "/someUrl", callback: this.someFunction }
		//
		//
		//   user					Puts the username in the header and customizes menu url defaults. 
		//							{
		//								name: "John Doe", 
		//								userId: "jdoe", 
		//								admin: true, 
		//								menuItemsProvider: function,
		//								userSmallPreviewUrl: string
		//								userProfileUrl: string
		//								allProfilesUrl: string
		//								personalDashboardUrl: string
		//							}
		//
		//	 footer					args:  {
		//								leftLogo: { src: path to image, href (optional) },
		//								rightLogo: { same as above }
		//							}  // both optional
		//
	},

	ui: null,
	
	createHomeMenu: function() {
		if (!this._homeMenu){
			this._homeMenu = new jazz.app.internal.HomeMenu();
		}
		return this._homeMenu;
	},
	
	createUserMenu: function() {
		this._userMenuAdded = true;
		return new jazz.app.internal.LoginLogoutArea({
			config: this.config
		});
	},
	
	createBannerSearch: function() {
		return new jazz.app.internal.BannerSearch();
	},
	
	start: function(){
		if (this._deferStart) {
			return;
		} else {
			if (this.started) {
				return;
			}
		}	
		this.started = true;
		var config 	= this.config;
		
		jazz.util.theming.initialize();
		this._setupThemingHooks();
		
		this._navbar.init({
			navbarItems: config.navbarItems,
			usePageRegistry: config.usePageRegistry
		});
		this._footer.init(this.config.footer);
		
		this._setupUserButtons();
		
		this.ui.placeAt("net-jazz-ajax-WorkbenchRoot").startup();
		
		this.inherited(arguments);
	}, 

	onFailure: function(errorObj, message, showFinePrint) {
		this.workbench.setTitle("");
		this._setupUserButtons();
		if (errorObj && errorObj.status && errorObj.status == 401) {
			var string = messages.missingCredentials;
			var message = dojo.create("div");
			var buttonDiv = dojo.create("div", { style: { marginTop: "1em"} }, message);
			if (dojo.cookie("JazzFormAuth") != null && this.config.showLoginLogout) { //$NON-NLS-1$
				string += messages.loginAgain;
				var button = dojo.create("button", { innerHTML: messages.loginAgainButtonTxt }, buttonDiv);
				this._errorEvt = dojo.connect(button, "onclick", this, function (e) {
					dojo.stopEvent(e);
					var dfd = jazz.app.auth.logout();
					dfd.addCallback(function(){ window.location.reload(); });
				});
			} else {
				string += messages.basicAuthMessage;
				button = dojo.create("button", { innerHTML: messages.reload }, buttonDiv);
				this._errorEvt = dojo.connect(button, "onclick", this, function (e) {
					dojo.stopEvent(e);
					window.location.reload();
				});
			}
			dojo.place(document.createTextNode(string), message, "first");
		}
		dojo.publish(net.jazz.ajax.internal.events.viewErrorPage, [message, !errorObj || showFinePrint === false ? "" : errorObj.message, true]);
		this.ui.attr("navbar", null);
		this.ui.attr("searchProvider", null);
		this.ui.placeAt("net-jazz-ajax-WorkbenchRoot").startup();
		dojo.body().removeChild(dojo.byId("net-jazz-ajax-InitialLoadMessage"));
		this.started = true;
	},
	
	constructor: function(args) {
		this.workbench = args.workbench;
		this.workbench._setDefaultTitle(this.name);
		
		dojo.setObject("jazz.app.currentApplication", this);
		
		this._navbar = new jazz.app.internal.CustomNavbar();
		this._footer = new jazz.app.internal.Footer();
		var about = jazz.app.internal.applicationAbout.getCurrent() || {};
		var title = about.title || this.name;
		if (about.instanceName && about.instanceName != about.title) {
			title += " (" + about.instanceName + ")";
		}
		this.ui = new jazz.ui.PageTemplate({
			title: title,
			bannerIcon: this.iconLarge ? this.iconLarge.toString() : null,
			navbar: this._navbar,
			homeMenuProvider: dojo.hitch(this,this.createHomeMenu),
			searchProvider: this.createBannerSearch,
			footer: this._footer
		});
		
		this._setupHomeLink();	
		
		// shim old net.jazz.ajax ProgressIndicator events to new PageTemplate API (needed to preserve Workbench showMessage API).
		dojo.subscribe(net.jazz.ajax.internal.events.showMessage, this.ui, "showStatusMessage");
		dojo.subscribe(net.jazz.ajax.internal.events.hideMessageArea, this.ui, "hideStatusMessage");
		
		var support = browserSupport.getSupportLevel();
		if (support != browserSupport.SUPPORTED) {
			if (support == browserSupport.NOT_SUPPORTED) {
				this._deferStart = true;
				var warn = new jazz.ui.UnsupportedBrowserWarning();
				var contConnect = dojo.connect(warn, "onContinue", this, function (){
					dojo.disconnect(contConnect);
					warn.destroy();
					this._deferStart = false;
					if (!this._started) this.start();
				});
				document.body.appendChild(warn.domNode);
			}
			this._footer.showBrowserWarning();
		}
		
		this.ui.addChild(new net.jazz.ajax.ui.PageContainer());
		
		var currentPageId;
		dojo.subscribe(VIEW_PAGE, this, function(page) {
			if (page && page.id != currentPageId) {
				jazz.ui.MenuPopup.closeAll();
			}
			currentPageId = page.id;
			dojo.publish(jazz.ui.internal.drawer.events.POLL_FAST);		//tell the drawer widget to poll for heights faster for a bit when a viewPage event happens
		});
	},

	getHomeLink: function(){
		//TODO: remove this function - keeping it here in case there are callers in 3.0.1 products.
		dojo.deprecated("_AjaxApplication#getHomeLink");
		return "#";
	},
	
	getCurrentWorkbenchId: function() {
		return this.getApplicationId();
	},
	
	destroy: function() {
		this._errorEvt && dojo.disconnect(this._errorEvt);
		this._navbar && this._navbar.destroy();
		this._footer && this._footer.destroy();
		this.ui && this.ui.destroy();
	},
	
	_setupThemingHooks: function() {
		var node = dojo.create("div");
		this.ui.attr("aboveBanner", node);
		var themeNode = new jazz.app.ThemableNode({
			htmlPath: "html/aboveBanner.html",
			targetNode: node,
			widgetName : "jazz.theme.AboveBannerWidget"
		});
	},
	
	_setupUserButtons: function() {
		var buttons = this.config.buttons;
		if (buttons && this.config.showLoginLogout && !this._userMenuAdded) {
			buttons.splice(0, 0, this.createUserMenu());
			this.ui.attr("bannerRight", new jazz.app.internal.BannerButtons({
				buttons: this.config.buttons
			}));
		}
	},
	_setupHomeLink: function() {
		this.ui.attr("homeLink", jazz.app.discovery.PersonalDashboards.url);
	}
});
})();