/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function()
{
dojo.provide("jazz.iwidget._IWidget");

dojo.require("jazz.iwidget.Container");

/**
 * a mix in to provide IWidgetSupport for a class.
 *
 *
 *
 */
jazz.iwidget.mode = {
	VIEW: "view",
	EDIT: "edit",
	HELP: "help"
};

dojo.declare("jazz.iwidget._IWidget", null, {
	_iwidget: null,
	
	/**
	 * sets the attributes on the iWidget.
	 * This will only set attributes that actually exist in the object
	 * @param {Object} attributes : a
	 */
	setAttributes: function(map)
	{
		if (!this._iwidget || !map) return;
		
		var itemSets = this._iwidget.getAttributes();
		
		dojo.forEach(itemSets.getAllNames(), function(name)
		{
			if (map.hasOwnProperty(name)) itemSets.setItemValue(name, map[name]);
			else console.warn("jazz.iwidget._IWidget Warning: setting unknown attribute");
		});
		
		itemSets.save();
	},
	
	getAttributes: function()
	{
		if (!this._iwidget) return {};
		return _itemSet2Map(this._iwidget.getAttributes());
	},
	getDescriptorItems: function()
	{
		if (!this._iwidget) return {};
		return _itemSet2Map(this._iwidget.getIDescriptorItems());
	},
	
	destroy: function()
	{
		if (this._connects) {
			dojo.forEach(this._connects, dojo.disconnect)
			this._connects = [];
		}
		this._iwidget && this._iwidget.destroy();
		this._iwidget = null;
	},
	
	getSupportedModes: function()
	{
		return this._iwidget ? this._iwidget.getIWidgetDefinition().widgetDef.getSupportedModes() : [];
	},
	
	getCurrentMode: function()
	{
		return this._iwidget ? this._iwidget.getIDescriptorItems().getItemValue(iwConstants.iDescriptorItems.mode) : [];
	},
	
	isLoaded: function()
	{
		return this._iwidget && this._iwidget.isLoaded();
	},
	
	setMode: function(mode)
	{
		if (!this._iwidget) 
		{
			console.error("jazz.iwidget._IWidget: Error calling setMode: _iwidget==null");
			return;
		}
		
		iWidgetContainer.eventService.fireEvent(this._iwidget.getID(), iwConstants.EVENTS.onModeChanged, {
			"newMode": mode
		});
	},
	
	onLoad: function()
	{
		this.connect(this._iwidget.getAttributes(), "save", "onSaveAttributes");
	},
	
	onSaveAttributes: function()
	{
	},
	
	onModeChange: function(newMode)
	{
	},
	
	_createIWidget: function(node, url, prefs)
	{
	
		/**
		 * if the container is already loaded it will simply call the create widget function
		 * otherwise it will call it when it is done.
		 */
		var container = jazz.iwidget.getContainer();
		var self = this;
		container.load(function createWidget()
		{
			self._iwidget = container.createIWidget(node, url, prefs);
			
			if (self._iwidget.isLoaded()) self.onLoad();
			else self.connect(self._iwidget, "onLoad", "onLoad");
			
			/**
			 * Connecting to a private member is proabaly not the best
			 * way to handle this. But there doesn't seem to be an alternative
			 */
			self.connect(self._iwidget, "_handleModeChange", function()
			{
				self.onModeChange(self._iwidget.currentMode);
			});
		});
	},
	
	connect: function(o, f, tf)
	{
		if (!this._connects) this._connects = [];
		this._connects.push(dojo.connect(o, f, this, tf));
	}
});

var _itemSet2Map = function(itemSet)
{
	var map = {};
	
	if (itemSet) 
	{
		dojo.forEach(itemSet.getAllNames(), function(name)
		{
			map[name] = itemSet.getItemValue(name);
		});
	}
	return map;
};
})();
