/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("jazz.util.browserSupport");

/**
 * A utility that provides common API for determining whether the 
 * user is using a supported browser.
 */
jazz.util.browserSupport = {
	
	// Constants
	SUPPORTED: 1,
	TOLERATED: 2,
	NOT_SUPPORTED: 3,
	
	/*
	 * Call to determine whether the browser is supported. Returns one
	 * of the browserSupport constants defined above.
	 */
	getSupportLevel: function (){
		if (this.supported && dojo.some(this.supported, function(item){return item})) {
			return this.SUPPORTED;
		} else if (this.strictSupport || (this.unsupported && dojo.some(this.unsupported, function(item) { return item }))){
			return this.NOT_SUPPORTED;
		} else {
			return this.TOLERATED;
		}
	},
	
	/*
	 * Array of booleans that represent supported browsers.
	 * If any index of this array is true, SUPPORTED will be returned.
	 */
	supported: [
		dojo.isIE >= 7 && dojo.isIE <= 8,
		dojo.isFF == 3.6
	],
	/*
	 * Array of booleans that represent unsupported browsers.
	 * If any index of this array is true, NOT_SUPPORTED will be returned.
	 */	
	unsupported: [
		dojo.isIE <= 6,
		dojo.isFF <= 3
	],
	/*
	 * Strict Support Mode
	 * 
	 * Default false, all browsers that do not match either supported or unsupported 
	 * values will return TOLERATED. This is useful for displaying different
	 * levels of messaging to visitors using unclassified browsers, which include
	 * future browser releases or unsupported yet mostly functional browsers.
	 * 
	 * If true, only SUPPORTED or NOT_SUPPORTED will be enabled.
	 */
	strictSupport: false
	
};
})();
