/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("jazz.util.calendar");

/**
 * A shim to dojo that automatically selects the calendaring system based
 * on current locale.  
 * 
 * Overrides the datePackage on:
 *   - dijit.form.DateTextBox
 *   - dijit.Calendar
 *  
 * Exposes the following convenience API that automatically shims to the right date package:
 * 	 - jazz.util.calendar.date: the date package, equivalent to dojo.date or dojox.date.buddhist
 * 	 - jazz.util.calendar.datePackage: a string representing the current date package
 * 	 - jazz.util.calendar.Date: the date object for the current locale
 * 
 * Currently only supports the Thai Buddhist calendar.
 */

dojo.require("dojo.date.locale");
dojo.require("dijit.Calendar");
dojo.require("dijit.form.DateTextBox");

var isBuddhist = (dojo.locale.split("-")[0] == 'th');
	
dojo.requireIf(isBuddhist, "dojox.date.buddhist");
dojo.requireIf(isBuddhist, "dojox.date.buddhist.locale");

var calendar = jazz.util.calendar;
calendar.date = isBuddhist ? dojox.date.buddhist : dojo.date;
calendar.datePackage = isBuddhist ? "dojox.date.buddhist" : "dojo.date";
calendar.Date = isBuddhist ? dojox.date.buddhist.Date : Date;

if (isBuddhist) {
	dijit.Calendar.prototype.datePackage = calendar.datePackage;
	dijit.form.DateTextBox.prototype.datePackage = calendar.datePackage;
	var format = calendar.date.locale.format;
	calendar.date.locale.format = function(d,o) { return format(new dojox.date.buddhist.Date(d),o) };  //TODO: Push this into dojo.
}

})();