/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.client.internal.CountersClient");

dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");

(function() {

var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServiceRequest = com.ibm.team.repository.web.transport.ServiceRequest;

// methods
var GET_ALL_COUNTERS_METHOD = "getAllCounters";
var GET_GROUP_NAMES_METHOD = "getGroupNames";
var GET_GROUP_COUNTERS_METHOD = "getGroupCounters";

/**
 * Provides helper methods to fetch data from the repository package.
 */
com.ibm.team.repository.web.client.internal.CountersClient = {
	
	// service URI
	SERVICE_URI: "com.ibm.team.repository.service.internal.counters.ICountersRestService",

	/*
	 * @see com.ibm.team.repository.service.internal.counters.ICountersRestService#getAllCounters()
	 */
	getAllCounters: function(responseHandler) {
		CountersClient._invokeRepositoryService(responseHandler, GET_ALL_COUNTERS_METHOD);
	},
	
	/*
	 * @see com.ibm.team.repository.service.internal.counters.ICountersRestService#getGroupNames()
	 */
	getGroupNames: function(responseHandler) {
		CountersClient._invokeRepositoryService(responseHandler, GET_GROUP_NAMES_METHOD);
	},
	
	/*
	 * @see com.ibm.team.repository.service.internal.counters.ICountersRestService#getGroupCounters()
	 */
	getGroupCounters: function(responseHandler, group) {
		var args = {
			groupName: group
		}
		CountersClient._invokeRepositoryService(responseHandler, GET_GROUP_COUNTERS_METHOD, args);
	},	
	
	/*
	 * @see com.ibm.team.repository.service.internal.counters.ICountersRestService#getCountersInfo()
	 */
	getGroups: function(responseHandler) {
		CountersClient._invokeRepositoryService(responseHandler, GET_COUNTERS_INFO_METHOD);
	},	
	
	_invokeRepositoryService: function(responseHandler, method, appArgs) {
		// appArgs is optional, so test whether or not it was sent
		if (arguments.length == 2) {
			var serviceRequest= new ServiceRequest(
					CountersClient.SERVICE_URI,
					method);
		} else {
			serviceRequest= new ServiceRequest(
					CountersClient.SERVICE_URI,
					method,
					appArgs);
		}
		com.ibm.team.repository.web.transport.TeamServerClient.invokeService(
				serviceRequest,
				responseHandler);
	}
};

var CountersClient= com.ibm.team.repository.web.client.internal.CountersClient;

})();