/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.client.internal.OAuthAdminClient");

dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");

(function() {

/**
 * Provides helper methods to fetch data from the repository package.
 */
com.ibm.team.repository.web.client.internal.OAuthAdminClient= {
	
	OAUTH_SERVICE_URI: "com.ibm.team.repository.service.internal.oauth.IOAuthRestService",	
	
	REGISTER_CONSUMER_METHOD: "postRegisterNewConsumer",
	CONSUMER_TRUSTED_METHOD: "postSetConsumerTrusted",
	GET_PROVISIONAL_REGISTRATIONS_METHOD: "getProvisionalRegistrations",
	GET_PROVISIONAL_REGISTRATION: "getProvisionalRegistration",
	POST_ACCEPT_PROV_METHOD: "postAcceptProvisionalRegistration",
	POST_REQUEST_REMOTE_PROVISIONAL_METHOD: "postRequestRemoteProvisionalKey",
	GET_CONSUMERS_METHOD: "getConsumers",
	POST_DELETE_CONSUMER_METHOD: "postDeleteConsumer",
	GENERATE_SECRET: "getGenerateNewSecret",
	POST_SET_CONSUMER_ATTRIBUTES: "postSetConsumerAttributes",
	
	FRIENDS_SERVICE_URI: "com.ibm.team.repository.service.internal.discovery.IFriendsAdminRestService",
	APPLICATION_REG_SVC_URI: "com.ibm.team.repository.service.internal.discovery.IApplicationRegistrationFriendRestService" , 
	
	POST_TEST_NEW_FRIEND: "postTestNewFriend",
	POST_NEW_FRIEND: "postNewFriend",
	GET_FRIENDS_METHOD: "getFriends",
	POST_DELETE_FRIEND: "postDeleteFriend",
	GET_OAUTH_PROVISIONAL_REQUEST_URI: "getFriendsOAuthInfo",
	GET_FRIENDS_OAUTH_APPROVAL_URI: "getFriendsOAuthApprovalModuleUrl",
	POST_UPDATE_FRIEND: "postUpdateFriend",
	GET_LOCAL_APPS: "getLocallyInstalledApps",
	POST_REGISTER_APP: "postRegisterApp",
	POST_UNREGISTER_APP: "postUnregisterApp",
	POST_WARM_UP_LOCAL_APPS: "postWarmUpLocallyInstalledApps",
	GET_ABOUT_SERVICE_CONTENT: "getAboutServiceContent",

	getConsumers: function(responseHandler) {
		this._invokeService(responseHandler, this.GET_CONSUMERS_METHOD, this.OAUTH_SERVICE_URI);
	},
	
	deleteConsumer: function(responseHandler, consumerKey) {
		var args = {"consumerKey": consumerKey};
		this._invokeService(responseHandler, this.POST_DELETE_CONSUMER_METHOD, this.OAUTH_SERVICE_URI, args);
	},
	
	updateConsumer: function(responseHandler, args) {
		this._invokeService(responseHandler, this.POST_SET_CONSUMER_ATTRIBUTES, this.OAUTH_SERVICE_URI, args);
	},
	
	registerNewConsumer: function(responseHandler, args) {
		this._invokeService(responseHandler, this.REGISTER_CONSUMER_METHOD, this.OAUTH_SERVICE_URI, args);
	},
	
	generateSecret: function(responseHandler) {
		this._invokeService(responseHandler, this.GENERATE_SECRET, this.OAUTH_SERVICE_URI);
	},
	
	setConsumerTrusted: function(responseHandler, consumerKey, trusted) {
		var args = {"consumerKey": consumerKey,
		            "trusted": trusted};
		this._invokeService(responseHandler, this.CONSUMER_TRUSTED_METHOD, this.OAUTH_SERVICE_URI, args);
	},
	
	getProvisionalRegistrations: function(responseHandler){
		this._invokeService(responseHandler, this.GET_PROVISIONAL_REGISTRATIONS_METHOD, this.OAUTH_SERVICE_URI);
	},
	
	getProvisionalRegistrationByKey: function(responseHandler, key){
		var args = {"consumerKey": key};
		this._invokeService(responseHandler, this.GET_PROVISIONAL_REGISTRATION, this.OAUTH_SERVICE_URI, args);
	},
	
	acceptProvisionalRegistration: function(responseHandler, consumerName, consumerKey, trusted, accept) {
		var args = {"consumerKey": consumerKey,
		            "name": consumerName,
		            "trusted": trusted, 
					"accept": accept};
		this._invokeService(responseHandler, this.POST_ACCEPT_PROV_METHOD, this.OAUTH_SERVICE_URI, args);
	},
	
	requestRemoteProvisionalKey: function(responseHandler, args) {
		this._invokeService(responseHandler, this.POST_REQUEST_REMOTE_PROVISIONAL_METHOD, this.OAUTH_SERVICE_URI, args);
	},
	
	getOauthProvisionalRequestUri: function(responseHandler, args) {
		this._invokeService(responseHandler, this.GET_OAUTH_PROVISIONAL_REQUEST_URI, this.FRIENDS_SERVICE_URI, args);
	},
	
	getFriendsOauthApprovalUri: function(responseHandler, args) {
		this._invokeService(responseHandler, this.GET_FRIENDS_OAUTH_APPROVAL_URI, this.FRIENDS_SERVICE_URI, args);
	},

	testNewFriend: function(responseHandler, args) {
		this._invokeService(responseHandler, this.POST_TEST_NEW_FRIEND, this.FRIENDS_SERVICE_URI, args);
	},
	
	postNewFriend: function(responseHandler, args) {
		this._invokeService(responseHandler, this.POST_NEW_FRIEND, this.FRIENDS_SERVICE_URI, args);
	},
	
	getFriends: function(responseHandler, args) {
		this._invokeService(responseHandler, this.GET_FRIENDS_METHOD, this.FRIENDS_SERVICE_URI, args);
	},
	
	deleteFriend: function(responseHandler, args) {
		this._invokeService(responseHandler, this.POST_DELETE_FRIEND, this.FRIENDS_SERVICE_URI, args);
	},
	
	updateFriend: function(responseHandler, args) {
		this._invokeService(responseHandler, this.POST_UPDATE_FRIEND, this.FRIENDS_SERVICE_URI, args);
	},
	
	/* Registered Applications Step */
	getLocallyInstalledApps: function(responseHandler) {
		this._invokeService(responseHandler, this.GET_LOCAL_APPS, this.FRIENDS_SERVICE_URI);
	},
	
	postRegisterApp: function(responseHandler, args) {
		this._invokeService(responseHandler, this.POST_REGISTER_APP, this.APPLICATION_REG_SVC_URI, args);
	},
	
	postUnregisterApp: function(responseHandler, args) {
		this._invokeService(responseHandler, this.POST_UNREGISTER_APP, this.APPLICATION_REG_SVC_URI, args);
	},
	
	postWarmUpLocallyInstalledApps: function(responseHandler) {
		this._invokeService(responseHandler, this.POST_WARM_UP_LOCAL_APPS, this.FRIENDS_SERVICE_URI);
	},
	
	getAboutServiceContent: function(responseHandler, args){
		this._invokeService(responseHandler, this.GET_ABOUT_SERVICE_CONTENT, this.FRIENDS_SERVICE_URI, args);
	},

	_invokeService: function(responseHandler, method, service, appArgs) {
		// appArgs is optional, so test whether or not it was sent
		if (arguments.length == 3) {
			var serviceRequest= new com.ibm.team.repository.web.transport.ServiceRequest(
					service,
					method);
		} else {
			serviceRequest= new com.ibm.team.repository.web.transport.ServiceRequest(
					service,
					method,
					appArgs);
		}
		com.ibm.team.repository.web.transport.TeamServerClient.invokeService(
				serviceRequest,
				responseHandler);
	}
	
};

})();
