/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.client.internal.PersonalDashboardClient");

dojo.require("dojo.i18n");

dojo.require("jazz.app.context");

dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("jazz.app.events");
dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var getAuthenticatedContributor = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;

var PlatformUI = net.jazz.ajax.ui.PlatformUI;

var deferred, navInfo;

var messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

com.ibm.team.dashboard.web.client.internal.PersonalDashboardClient = {
	get: function() {
		if (deferred)
			return deferred;
		deferred = new dojo.Deferred();
		var serviceObj = {
			success: function(/* LeftNavDTO */ navDTO) {
				navInfo = navDTO;
				deferred.callback(navInfo);
			},
			failure: function(errorObj) {
				deferred.errback(errorObj);
			}
		};
		var userId = getAuthenticatedContributor().userId;
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		DashboardClient.getLeftNavInfo(jazz.app.context.get().name, srh, null, true);
		dojo.subscribe("/jazz/dashboard/rename", PersonalDashboardClient, PersonalDashboardClient._renameDashboard); // pass url, new title
		dojo.subscribe("/jazz/dashboard/delete", PersonalDashboardClient, PersonalDashboardClient._deleteDashboard); // pass url
		dojo.subscribe("/jazz/dashboard/share", PersonalDashboardClient, PersonalDashboardClient._shareDashboard); // pass url, shared
		dojo.subscribe("/jazz/dashboard/setDefault", PersonalDashboardClient, PersonalDashboardClient._setDefaultDashboard);
		return deferred;
	},
	
	/*
	 * Creates a new dashboard. Unlike newDashboard, it simply returns the deferred.
	 */
	createDashboard: function() {
		var dfd = new dojo.Deferred();
		var serviceObj = {
			success: function(/* ResponseDTO */ responseDTO) {
				switch(responseDTO.status) {
					case "OK":
						var dashboardBundle = responseDTO.data;
						var url = dashboardBundle.dashboardUrl;
						var id = url.substring(url.lastIndexOf("/") + 1);
						if (!navInfo.personalDashboards) {
							navInfo.personalDashboards = [];
						}
						navInfo.personalDashboards.push({
							id: id,
							itemId: id,
							url: url,
							title: responseDTO.msg,
							shared: false
						});
						dojo.publish("/jazz/dashboard/create",[url, responseDTO.msg]);
						dojo.publish(jazz.app.events.HOME_MENU_REFRESH);
						dfd.callback(url);
						break;
					case "ERROR":
						dfd.errback(new Error(responseDTO.msg));
						break;
				}
			},
			failure: function(/* errorObj */ e) {
				var error = new Error();
				error.message = e.status == 403 ? messages.createPermissionDenied : messages.createPersonalDashboardError;
				dojo.mixin(error, {"errorObj":e});
				dfd.errback(error);
			}
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		DashboardClient.createDashboard(null, DashboardConstants.SCOPE_CONTRIBUTOR, getAuthenticatedContributor().itemId, null, true, srh, null, true);
		return dfd;
	},
	
	/*
	 * Attempts to create a new dashboard and show it. Shows an error message
	 * if fails. If you need to handle errors, use createDashboard instead.
	 */
	newDashboard: function() {
		var deferred = PersonalDashboardClient.get();
		deferred.addCallback(function(info) {
			var serviceObj = {
				success: function(/* ResponseDTO */ responseDTO) {
					switch(responseDTO.status) {
						case "OK":
							var dashboardBundle = responseDTO.data;
							var url = dashboardBundle.dashboardUrl;
							var id = url.substring(url.lastIndexOf("/") + 1);
							if (!navInfo.personalDashboards) {
								navInfo.personalDashboards = [];
							}
							navInfo.personalDashboards.push({
								id: id,
								itemId: id,
								url: url,
								title: responseDTO.msg,
								shared: false
							});
							dojo.publish("/jazz/dashboard/create",[url, responseDTO.msg]);
							dojo.publish(jazz.app.events.HOME_MENU_REFRESH);
							var href = Util.getDashboardHref(dashboardBundle);
							if (PlatformUI && !PlatformUI.getWorkbench().shouldVetoNavigation(href)) {
								location.href = href;
							}
							break;
						case "ERROR":
							alert(responseDTO.msg);
							break;
					}
				},
				failure: function(/* Error */ e) {
					if (e.status == 403) {
						alert(messages.createPermissionDenied);
					}
					else {
						deferred.errback(e);
					}
				}
			};
			var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
			DashboardClient.createDashboard(null, DashboardConstants.SCOPE_CONTRIBUTOR, info.contributor.itemId, null, true, srh, null, true);
		});
		deferred.addErrback(function(errorObj) {
			if (errorObj.status == 403) {
				alert(messages.createPermissionDenied);
			}
			else {
				console.error(Constants.LeftNav.REMOTE_ERROR + errorObj.message, errorObj);
			}
		});
		return deferred;
	},
	
	_renameDashboard: function(url, title) {
		navInfo.personalDashboards[PersonalDashboardClient._findDashboard(url)].title = title;
	},
	
	_deleteDashboard: function(url) {
		navInfo.personalDashboards.splice(PersonalDashboardClient._findDashboard(url),1);
	},
	
	_shareDashboard: function(url, shared) {
		navInfo.personalDashboards[PersonalDashboardClient._findDashboard(url)].shared = shared;
	},
	
	_setDefaultDashboard: function (url) {
		var dbs = navInfo.personalDashboards;
		for (var i = 0; i < dbs.length; i++) {
			if (dbs[i].url === url) {
				dbs[i]["default"] = true;
			} else if (dbs[i]["default"]) {
				delete dbs[i]["default"];
			}
		}
	},
	
	_findDashboard: function(url) {
		var dbs = navInfo.personalDashboards;
		for (var i = 0; i < dbs.length; i++) {
			if (dbs[i].url === url)
				return i;
		}
	}
}
var PersonalDashboardClient = com.ibm.team.dashboard.web.client.internal.PersonalDashboardClient;

})();
