/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function () {
dojo.provide("com.ibm.team.repository.web.client.internal.ProvisioningClient");

dojo.require("jazz.app.auth");
dojo.require("jazz.client.xhr");
dojo.require("com.ibm.team.repository.web.transport.internal.ProvisionResponseHandler");
dojo.require("dojo.cookie");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.repository.web.client.internal","ProvisioningClientMessages");
var messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.client.internal","ProvisioningClientMessages");

com.ibm.team.repository.web.client.internal.ProvisioningClient = {
	
	connect: function(url, userid, password, provisionResponseHandler) {
		var connecturl = "connect?url="+url+"&userid="+userid+"&password="+password;
		this._invokeService(connecturl, provisionResponseHandler, arguments);
	},
	
	disconnect: function(provisionResponseHandler) {
		var url = "disconnect";
		this._invokeService(url, provisionResponseHandler, arguments);
	},
	
	installFeature: function(featureId, version, start, provisionResponseHandler) {
		var url = "installFeature?featureId="+featureId+"&version="+version+"&start="+start;
		this._invokeService(url, provisionResponseHandler, arguments);
	},
	
	startFeature: function(featureId, version, provisionResponseHandler) {
		var url = "startFeature?featureId="+featureId+"&version="+version;	
		this._invokeService(url, provisionResponseHandler, arguments);
	},
	
	stopFeature: function(featureId, version, provisionResponseHandler) {
		var url = "stopFeature?featureId="+featureId+"&version="+version;	
		this._invokeService(url, provisionResponseHandler, arguments);
	},
	
	startBundle: function(id, provisionResponseHandler) {
		var url = "startBundle?bundleId="+id;	
		this._invokeService(url, provisionResponseHandler, arguments);
	},
	
	stopBundle: function(id, provisionResponseHandler) {
		var url = "stopBundle?bundleId="+id;	
		this._invokeService(url, provisionResponseHandler, arguments);
	},
	
	startAll: function(provisionResponseHandler) {
		var url = "startAll";
		this._invokeService(url, provisionResponseHandler, arguments);
	},
	
	stopAll: function(provisionResponseHandler) {
		var url = "stopAll";
		this._invokeService(url, provisionResponseHandler, arguments);
	},
	
	removeFeature: function(featureId, version, provisionResponseHandler) {
		var url = "removeFeature?featureId="+featureId+"&version="+version;
		this._invokeService(url, provisionResponseHandler, arguments);
	},
	
	removeAll: function(provisionResponseHandler) {
		var url = "removeAll";
		this._invokeService(url, provisionResponseHandler, arguments);
	},
	
	getStatus: function(core, installed, available, featureId, version, provisionResponseHandler) {
		var url = "status?core="+core+"&installed="+installed+"&available="+available;
		if (featureId && version) {
			url +="&featureId="+featureId+"&version="+version;
		} 
		this._invokeService(url, provisionResponseHandler, arguments);
	},
	
	isRepositoryAvailable: function(provisionResponseHandler) {
		var url = "isRepositoryAvailable";
		this._invokeService(url, provisionResponseHandler, arguments);
	},
	
	getInstallLog: function(provisionResponseHandler) {
		var url = "status?installLog=true";
		this._invokeService(url, provisionResponseHandler, arguments);
	},
	
	getBundleStatus: function(bundleId, provisionResponseHandler) {
		var url = "status?bundleId="+bundleId;
		this._invokeService(url, provisionResponseHandler, arguments);
	},
		
	requestReset: function(provisionResponseHandler) {
		var url = "requestReset";
		this._invokeService(url, provisionResponseHandler, arguments);
	},
		
	reloadLogSettings: function(provisionResponseHandler) {
		var url = "reloadLogSettings";
		this._invokeService(url, provisionResponseHandler, arguments);
	},
		
	_invokeService: function(url, provisionResponseHandler, args) {
		var headers = {
			"accept": "text/json",
			"content-type": "application/x-www-form-urlencoded; charset=utf-8",
			"X-com-ibm-team-configuration-versions": "LATEST",
			"X-Jazz-CSRF-Prevent": dojo.cookie("JSESSIONID")
		};
		var xhrArgs = {
			url: dojo.baseUrl + "../../admin/cmd/" + url,
			handleAs: "json",
			headers: headers,
			preventCache: dojo.isIE,
			handle: function(response, ioArgs) {
				if (!(response instanceof Error)) {
					// success response
					provisionResponseHandler.doCallback(response, args);
				} else {
					// error response
					var errorObj = {
						status: ioArgs.xhr.status,
						statusText: ioArgs.xhr.statusText
					};
					if (ioArgs.xhr.status == 401 && ioArgs.args.url.match(/^.*\/isRepositoryAvailable.*$/)) {
						// Capture the scenario where the user cancels out of a Basic auth prompt and display a helpful message
						// rather than a blank "Loading..." screen
						var errMsg = messages.authRefresh; 
						alert(errMsg);
						errorObj.message = errMsg; 					
					}
					provisionResponseHandler.doErrorCallback(errorObj, args);
				}
			}
		};
		jazz.client.xhrGet(xhrArgs);
	}
};

})();