/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.client.internal.ServerStatusClient");

dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");

(function() {

var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServiceRequest = com.ibm.team.repository.web.transport.ServiceRequest;

// methods
var GET_COMPONENTS_METHOD = "getComponents";
var GET_COMPONENTS_WITH_SERVICE_CONFIG = "getComponentsWithServiceConfig";
var GET_SERVICE_STATUS_METHOD = "getServiceStatus";
var GET_ACTIVE_SERVICE_INFO_METHOD = "getActiveServiceInfo";
var POST_STATUS_FOR_SERVICES_METHOD = "postStatusForServices";
var GET_COMPONENTS_AND_STATUS = "getComponentsAndStatus";
var GET_SERVER_INFO_METHOD = "getServerInfo";
var GET_SERVICE_CONFIGURATION_METHOD = "getServiceConfiguration";
var POST_CONFIG_METHOD = "postConfigurationUpdates";
var POST_VALIDATION_METHOD = "postConfigurationValidation";
var GET_DATABASE_STATUS_METHOD = "getDatabaseStatus";
var GET_PRODUCT_INFO = "getAllProductInfo";
var POST_INITALIZE_DATABASE = "postInitalizeDatabase";
var GET_CREATION_STATUS = "getCreationStatus";
var GET_DIAGNOSTIC_RESULTS = "getAllDiagnosticResults";
var GET_NEW_DIAGNOSTIC_RESULT = "getNewDiagnosticResult";
var GET_PREVIOUS_DIAGNOSTIC_RESULT = "getPreviousDiagnosticResult";
var POST_CANCEL_DIAGNOSTIC = "postCancelDiagnostic";
var GET_IS_JAZZ_TEAM_SERVER = "getIsJazzTeamServer";
var GET_JAZZ_TEAM_SERVER_URI = "getJazzTeamServerURI";
var GET_CONFIGURED_JAZZ_TEAM_SERVER_URI = "getConfiguredJazzTeamServerURI";
var GET_SERVER_MODE_STRING = "getServerModeString";
var GET_CONFIGURED_JTS_STATUS = "getJtsConfigurationStatus";

/**
 * Provides helper methods to fetch data from server status service.
 * 
 * NOTE:
 * Internal dependencies on this client API are tracked in Task 43618.  
 * Be sure to notify subscribers of 43618 if there are incompatible changes
 * to the functions in this client which require adoption. 
 */
com.ibm.team.repository.web.client.internal.ServerStatusClient = {
	
	// service URI
	SERVICE_URI: "com.ibm.team.repository.service.internal.IServerStatusRestService",

	/*
	 * @see com.ibm.team.repository.service.internal.IServerStatusRestService.getComponents()
	 */
	getComponents: function(responseHandler) {
		ServerStatusClient._invokeRepositoryService(responseHandler, GET_COMPONENTS_METHOD, ServerStatusClient.SERVICE_URI);
	},
	
	/*
	 * @see com.ibm.team.repository.service.internal.IServerStatusRestService.getComponentsWithServiceConfig()
	 */
	 getComponentsWithServiceConfig: function (responseHandler) {
	 	ServerStatusClient._invokeRepositoryService(responseHandler, GET_COMPONENTS_WITH_SERVICE_CONFIG, ServerStatusClient.SERVICE_URI);
	 },
	
	/*
	 * @see com.ibm.team.repository.service.internal.IServerStatusRestService.getActiveServiceInfo()
	 */
	getActiveServiceInfo: function(responseHandler) {
		ServerStatusClient._invokeRepositoryService(responseHandler, GET_ACTIVE_SERVICE_INFO_METHOD, ServerStatusClient.SERVICE_URI);
	},	
	
	/*
	 * @see com.ibm.team.repository.service.internal.IServerStatusRestService.getServiceStatus(String)
	 */
	getServiceStatus: function(responseHandler, interfaceNameArg) {
		var args = {
			interfaceName: interfaceNameArg
		};
		ServerStatusClient._invokeRepositoryService(responseHandler, GET_SERVICE_STATUS_METHOD, ServerStatusClient.SERVICE_URI, args);
	},	
		
	/*
	 * @see com.ibm.team.repository.service.internal.IServerStatusRestService.postStatusForServices(ParamsServiceNames)
	 */
	postStatusForServiceList: function(responseHandler, services) {
		var appArgs= {
			jsonString: dojo.toJson(services)
		};

		ServerStatusClient._invokeRepositoryService(responseHandler, POST_STATUS_FOR_SERVICES_METHOD, ServerStatusClient.SERVICE_URI, appArgs);
	},	

	/*
	 * @see com.ibm.team.repository.service.internal.IServerStatusRestService.getServerInfo()
	 */
	getServerInfo: function(responseHandler) {
		ServerStatusClient._invokeRepositoryService(responseHandler, GET_SERVER_INFO_METHOD, ServerStatusClient.SERVICE_URI);
	},
	
	/*
	 * @see com.ibm.team.repository.service.internal.IServerStatusRestService.getDatabaseStatus()
	 */
	getDatabaseStatus: function(responseHandler) {
		ServerStatusClient._invokeRepositoryService(responseHandler, GET_DATABASE_STATUS_METHOD, ServerStatusClient.SERVICE_URI);
	},
	
	/*
	 * @see com.ibm.team.repository.service.internal.IServerStatusRestService.getComponentsAndStatus()
	 */
	getComponentsAndStatus: function(responseHandler) {
		ServerStatusClient._invokeRepositoryService(responseHandler, GET_COMPONENTS_AND_STATUS, ServerStatusClient.SERVICE_URI);
	},


// API to new Configuration service starts here.  
	CONFIG_SERVICE_URI: "com.ibm.team.repository.service.internal.IServerConfigurationRestService",

	/*
	 * @see com.ibm.team.repository.service.internal.IServerConfigurationRestService.getServiceConfiguration(String)
	 */
	getServiceConfiguration: function (responseHandler, implementationClassNameArg) {
		var args = {
			implementationClassName: implementationClassNameArg
		};
		ServerStatusClient._invokeRepositoryService(responseHandler, GET_SERVICE_CONFIGURATION_METHOD, ServerStatusClient.CONFIG_SERVICE_URI, args);
	},

	/*
	 * @see com.ibm.team.repository.service.internal.IServerConfigurationRestService.postConfigurationUpdates()
	 */
	postConfigurationUpdates: function(responseHandler, appArgs) {
		this._invokeRepositoryService(responseHandler, POST_CONFIG_METHOD, ServerStatusClient.CONFIG_SERVICE_URI, appArgs);
	},
	
	/*
	 * @see com.ibm.team.repository.service.internal.IServerConfigurationRestService.postConfigurationValidation()
	 */
	postConfigurationValidation: function(responseHandler, appArgs) {
		this._invokeRepositoryService(responseHandler, POST_VALIDATION_METHOD, ServerStatusClient.CONFIG_SERVICE_URI, appArgs);
	},

// API to new Product Registry service starts here.  
	PRODUCT_REGISTRY_SERVICE_URI: "com.ibm.team.repository.service.internal.IProductRegistryRestService",

	/*
	 * @see com.ibm.team.repository.service.internal.IProductRegistryRestService.getAllProductInfo()
	 */
	getAllProductInfo: function (responseHandler) {
		this._invokeRepositoryService(responseHandler, GET_PRODUCT_INFO, ServerStatusClient.PRODUCT_REGISTRY_SERVICE_URI);
	},	
	
// API to Database Creation service starts here.  
	DATABASE_CREATION_SERVICE_URI: "com.ibm.team.repository.service.internal.IDatabaseCreationRestService",
	DATABASES_CREATION_PROGRESS_SERVICE_URI: "com.ibm.team.repository.service.internal.IDatabaseCreationProgressRestService",

	/*
	 * @see com.ibm.team.repository.service.internal.IDatabaseCreationRestService.postInitalizeDatabase()
	 */
	postInitalizeDatabase: function (responseHandler) {
		this._invokeRepositoryService(responseHandler, POST_INITALIZE_DATABASE, ServerStatusClient.DATABASE_CREATION_SERVICE_URI);
	},
	
	/*
	 * @see com.ibm.team.repository.service.internal.IDatabaseCreationProgressRestService.getCreationStatus()
	 */
	getCreationStatus: function (responseHandler) {
		this._invokeRepositoryService(responseHandler, GET_CREATION_STATUS, ServerStatusClient.DATABASES_CREATION_PROGRESS_SERVICE_URI);
	},
	 
	
	
	_invokeRepositoryService: function(responseHandler, method, serviceUri, appArgs) {
		// appArgs is optional, so test whether or not it was sent
		if (arguments.length == 3) {
			var serviceRequest= new ServiceRequest(
					serviceUri,
					method);
		} else {
			serviceRequest= new ServiceRequest(
					serviceUri,
					method,
					appArgs);
		}
		com.ibm.team.repository.web.transport.TeamServerClient.invokeService(
				serviceRequest,
				responseHandler);
	},


// API to Server Diagnostics service starts here.
	DIAGNOSTICS_SERVICE_URI: "com.ibm.team.repository.service.internal.diagnostic.IDiagnosticRestService",

	getAllDiagnosticResults: function (responseHandler) {
		this._invokeRepositoryService(responseHandler, GET_DIAGNOSTIC_RESULTS, ServerStatusClient.DIAGNOSTICS_SERVICE_URI);
	},

	getNewDiagnosticResult: function (responseHandler, idArg) {
		var args = {
			id: idArg
		};
		this._invokeRepositoryService(responseHandler, GET_NEW_DIAGNOSTIC_RESULT, ServerStatusClient.DIAGNOSTICS_SERVICE_URI, args);
	},
	
	getPreviousDiagnosticResult: function (responseHandler, idArg) {
		var args = {
			id: idArg
		};
		this._invokeRepositoryService(responseHandler, GET_PREVIOUS_DIAGNOSTIC_RESULT, ServerStatusClient.DIAGNOSTICS_SERVICE_URI, args);
	},
	
	postCancelDiagnostic: function (responseHandler, idArg) {
		var args = {
			id: idArg
		};
		this._invokeRepositoryService(responseHandler, POST_CANCEL_DIAGNOSTIC, ServerStatusClient.DIAGNOSTICS_SERVICE_URI, args);
	},

// API to Server Description Rest Service
	SERVER_DESCRIPTION_SERVICE: "com.ibm.team.repository.service.IServerDescriptionRestService",

	getIsJazzTeamServer: function (responseHandler) {
		this._invokeRepositoryService(responseHandler, GET_IS_JAZZ_TEAM_SERVER, ServerStatusClient.SERVER_DESCRIPTION_SERVICE);
	},
	
	getConfiguredJazzTeamServerURI: function (responseHandler) {
		this._invokeRepositoryService(responseHandler, GET_CONFIGURED_JAZZ_TEAM_SERVER_URI, ServerStatusClient.SERVER_DESCRIPTION_SERVICE);
	},
	
	//Note: It's recommended that we use getConfiguredJazzTeamServerURI() instead.
	getJazzTeamServerURI: function (responseHandler) {
		this._invokeRepositoryService(responseHandler, GET_JAZZ_TEAM_SERVER_URI, ServerStatusClient.SERVER_DESCRIPTION_SERVICE);
	},
	
	getServerModeString: function (responseHandler) {
		this._invokeRepositoryService(responseHandler, GET_SERVER_MODE_STRING, ServerStatusClient.SERVER_DESCRIPTION_SERVICE);
	},

// API to JTS Configuration Service
	JTS_CONFIGURATION_SERVICE: "com.ibm.team.repository.service.compatibility.internal.IJtsConfigurationStateRestService",

	getJtsConfigurationStatus: function (responseHandler) {
		this._invokeRepositoryService(responseHandler, GET_CONFIGURED_JTS_STATUS, ServerStatusClient.JTS_CONFIGURATION_SERVICE);
	}

};

var ServerStatusClient= com.ibm.team.repository.web.client.internal.ServerStatusClient;

})();
