/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.client.internal.userRegistry");

dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");

// aliases and constants
var ServiceRequest = com.ibm.team.repository.web.transport.ServiceRequest;
var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
var THIS_PACKAGE = com.ibm.team.repository.web.client.internal.userRegistry;
var SERVICE_URI = "com.ibm.team.repository.service.internal.IExternalUserRegistryRestService";

//
// exported functions
//

/**
 * See IExternalUserRegistryRestService#computeLDAPQueries
 */
function computeLDAPQueries(srh, args) {
	validateArgs("computeLDAPQueries", args, ["basicProperties"]);
	TeamServerClient.invokeService(getServiceRequest("postComputeLDAPQueries", args), srh);
}
THIS_PACKAGE.computeLDAPQueries = computeLDAPQueries;

/**
 * See IExternalUserRegistryRestService#postSearchRegistry
 */
function searchRegistry(srh, args) {
	validateArgs("searchRegistry", args, ["searchText"]);
	TeamServerClient.invokeService(getServiceRequest("postSearchRegistry", args), srh);
}
THIS_PACKAGE.searchRegistry = searchRegistry;

function importUserFromRegistry(srh) {
	TeamServerClient.invokeService(getServiceRequest("postCreateUser"), srh);	
}
THIS_PACKAGE.importUserFromRegistry = importUserFromRegistry;


function getUserRegistryType(srh) {
	TeamServerClient.invokeService(getServiceRequest("getUserRegistryType"), srh);	
}
THIS_PACKAGE.getUserRegistryType = getUserRegistryType;
	

//
// internal functions
//

function getServiceRequest(method, args) {
	if (args) {
		return new ServiceRequest(SERVICE_URI, method, args);
	} else {
		return new ServiceRequest(SERVICE_URI, method);
	}
}

function validateArgs(fName, args, props) {
	for(var i=0; i<props.length; i++) {
		if(!args[props[i]]) {
			throw new Error(fName + " missing required parameter args." + props[i]);
		}
	}
}

})();
