/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.client.session.Session");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("com.ibm.team.repository.web.client.init.InitializationData");
dojo.require("net.jazz.ajax.ui.internal.PageRegistry");

(function() {

	var checkNotNull 		= net.jazz.ajax.toolbox.lang.checkNotNull;
	var checkNotEmpty 		= net.jazz.ajax.toolbox.lang.checkNotEmpty;
	var InitializationData 	= com.ibm.team.repository.web.client.init.InitializationData;

	function checkInit() {
		checkNotNull(com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR, "com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR");
	}
	
	com.ibm.team.repository.web.client.session = {
		init: function(initializationData) {
			InitializationData._load(initializationData);
			
			//read current contributor and uri from init data and set as global var
			var contributorInitData = InitializationData.get("com.ibm.team.repository.service.currentContributorInitializer");
			dojo.setObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR", contributorInitData ? contributorInitData.currentContributor : false);	//$NON-NLS-1$
			dojo.setObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR_URI", contributorInitData ? contributorInitData.currentContributorUri : false);	//$NON-NLS-1$
			dojo.setObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR_ROLES", contributorInitData ? contributorInitData.currentContributorRoles : false);	//$NON-NLS-1$
			
			//read config properties from init data and set as global vars
			var configProperties = InitializationData.get("com.ibm.team.repository.service.configPropertyInitializer"); //$NON-NLS-1$
			for (var propName in configProperties) {
				var propVal = configProperties[propName];
				dojo.setObject(propName, propVal);
			}
			
			//read the Server Mode and set as a global var
			var serverInfo = InitializationData.get("com.ibm.team.repository.service.serverInfoInitializer");
			dojo.setObject("com.ibm.team.repository.server.mode", serverInfo? serverInfo.mode : false);	//$NON-NLS-1$
			
			var hideUserGroups = InitializationData.get("com.ibm.team.repository.service.hideUserGroupAdminUIInitializationDataService");
			if (hideUserGroups){
				hideUserGroups.value = hideUserGroups.value === "true"? true:false;
			}
			dojo.setObject("com.ibm.team.repository.hideUserGroupAdminUI", hideUserGroups? hideUserGroups.value : true);	//$NON-NLS-1$
			
			//Process all removalPropertyIds found in the Page Extensions.
			//Look for each property in the Initialization Data. If a
			//property exists convert into a global property so that 
			//future net.jazz.ajax.ui.internal.PageRegistry.getPages
			//calls will remove the required pages
			dojo.forEach(net.jazz.ajax.ui.internal.PageRegistry.getPages(), function(page) {
				if (page.removalPropertyId != null) {
					var property = InitializationData.get(page.removalPropertyId);
					if (property != null && property.value != null) {
						dojo.setObject(page.removalPropertyId, property.value);
					}
				}
			});
		},
		/**
		 * Returns an object representing the current logged-in contributor or the Guest contributor if not logged in.
		 */
		getAuthenticatedContributor: function () {
			checkInit();
			return com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR;
		},
		
		/**
		 * Returns the URI (string) of the current logged-in contributor, or undefined
		 * if not available.
		 */
		getAuthenticatedContributorUri: function() {
			return com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR_URI;
		},
		
		/**
		 * Return true if the current user is logged-in (ie not Guest); false otherwise;
		 */
		isGuest: function() {
			checkInit();
			return com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR.itemId == "_OIi3MCDNEdu1A6FdF3RHyQ";
		},
		
		/**
		 * Return true if the current user is logged-in as the default ADMIN user; false otherwise;
		 */
		isDefaultAdmin: function() {
			checkInit();
			return com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR.userId == "ADMIN";
		},
		
		/**
		 * Returns a string object of the current logged-in userId using AUTHENTICATED_CONTRIBUTOR 
		 * if available, or whoami if available, or UNKNOWN if neither is available  
		 */
		getAuthenticatedUserId: function() {
			var testAUTH = com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR;
			if (testAUTH) {
				return testAUTH.userId;
			}
			var testWhoAmI = com.ibm.team.repository.web.client.internal.AUTHENTICATED_USERID;
			if (testWhoAmI) {
				return testWhoAmI;
			}
			return "UNKNOWN";	// don't externalize this string
		},

		/**
	 	 * Returns a link to the user profile page belonging to the authenticated
	 	 * user.
	 	 * 
	 	 * @return String
	 	 */		
		getUserProfileLink: function () {
			var contributor = com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR;
			if(!contributor) return null;
			return "#action=com.ibm.team.repository.viewUserProfile&itemId=" + contributor.itemId;
		}
	}
})();