/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.dnd.DraggableContainer");

dojo.require("jazz.ui.dnd.DraggableManager");

(function() {
	var draggableManager = jazz.ui.dnd.draggableManager();
	
	/*
	 * PROVISIONAL: This API is considered provisional and may be changed or removed in the
	 * future with minimal or no warning.
	 * 
	 * @since 0.7
	 */
	dojo.declare("jazz.ui.dnd.DraggableContainer", null, {
		constructor: function(node, params){
			this.node = node;
			this.accept = params.accept;
			this.widgets = new Array();
			this.overWidget = null;
			this.isSource = true;
			this.horizontal = false;
			
			this.events = [];
			this.events.push(dojo.connect(node, "ondragstart", this, this.dragStopEvent));
			this.events.push(dojo.connect(node, "onselectstart", this, this.dragStopEvent));
			this.events.push(dojo.connect(document.body,"ondragstart", this, this.dragStopEvent));
			this.events.push(dojo.connect(document.body,"onselectstart", this, this.dragStopEvent));
			
			dojo.addClass(this.node,"jazz-ui-dnd-DraggableContainer");
			
			this.registerContainer();
		},
		registerContainer: function() {  //seperate so that it can be overidden
			draggableManager.containers.push(this);
		},
		setEnabled: function(enabled) {
			this._disabled = !enabled;
			if (enabled) {
				dojo.removeClass(this.node,"jazz-ui-dnd-disabled");
			} else {
				dojo.addClass(this.node,"jazz-ui-dnd-disabled");
			}
		},
		dragStopEvent: function(event) {  //If viewlet is dragging, cancel text select and bubbling.
			if (event && (draggableManager.isDragging || draggableManager.fromContainer)) {
				dojo.stopEvent(event);
			}
		},
		/*
		 * Widget:
		 * 	dojo widget, or domNode wrapped in an object literal
		 * 	{domNode: domNode}
		 * Params:
		 * 	index: insertion index (optional - defaults to last)
		 * 	handle: domNode that initiates drag (optional - defaults to whole widget)
		 */
		add: function(widget, params) {
			var index = params.index;
			if (typeof(index) != "number" || index >= this.widgets.length) {
				index = this.widgets.length;
			}
			widget.type = params.type;
			widget.index = index;
			widget.handle = params.handle;
			if (!widget.handle) {
				widget.handle = widget.domNode;
			}
			if (index == this.widgets.length) {
				if (!this.node.lastChild || dojo.hasClass(this.node.lastChild,"dndItem")){
					this.node.appendChild(widget.domNode);
				} else {
					this.node.insertBefore(widget.domNode,this.node.lastChild);
				}	
			} else {
				this.node.insertBefore(widget.domNode, this.widgets[index].domNode);
			}
			this.widgets.splice(index,0,widget);
			widget.dndEvent = dojo.connect(widget.handle,"onmousedown",this,this.onMouseDown);
			
			dojo.addClass(widget.domNode,"dndItem");
			dojo.addClass(widget.handle,"jazz-ui-dnd-handle");
			this.updateIndices();
		},
		destroy: function() {
			while(this.events.length > 0) {
				dojo.disconnect(this.events.pop());
			}
			while (this.widgets.length > 0) {
				var widget = this.widgets.pop();
				if (widget.dndEvent) {
					dojo.disconnect(widget.dndEvent);
				}
				if (widget.destroy) {
					widget.destroy();	
				}
			}
			var containers = draggableManager.containers;
			for (var i = 0; i < containers.length; i++) {
				if (containers[i] == this) {
					containers.splice(i,1);
				}
			}
		},
		move: function(fromIndex, toContainer, toIndex) {
			var fromContainer = draggableManager.fromContainer;
			var overContainer = draggableManager.overContainer;
			
			var widget = fromContainer.widgets.splice(fromIndex,1)[0];
			widget.domNode.parentNode.removeChild(widget.domNode);
			overContainer.node.insertBefore(widget.domNode, this.getPlaceholder());
			overContainer.widgets.splice(toIndex,0,widget);
			
			var cols = [fromContainer, overContainer];
			for (var j = 0; j < cols.length; j++) {
				this.updateIndices(cols[j]);
				if (fromContainer == overContainer) break;
			}
		},
		remove: function(index){
			//remove widget from array
			var widget = this.widgets.splice(index,1)[0];
			widget.domNode.parentNode.removeChild(widget.domNode);
			this.updateIndices();
		},
		onMouseDown: function(event) {
			if (this._disabled) {
				return;
			}
			var e = event.currentTarget;
			for (var i = 0; i < this.widgets.length; i++) {
				if (e == this.widgets[i].handle) {
					var widget = this.widgets[i];
					break;
				}
			}
			if (widget) {
				this.onDndStart(widget, event);
				this.overWidget = widget;
			}
			this.dragStopEvent(event);
		},
		onMouseMove: function(event, index) {
			jazz.ui.dnd.autoScroll(event);
			this.dropAtEnd = false;
			var x = event.pageX;
			var y = event.pageY;
			if (index != null && !this.constrainAvatar) {
				this.overWidget = this.widgets[index];
				this.insertPlaceholder(true);
				return;
			}
			for (var i = 0; i < this.widgets.length; i++) {
				var widget = this.widgets[i];
				var overCoords = dojo._abs(widget.domNode, true);
				overCoords.w = widget.domNode.clientWidth;
				if ((this.constrainAvatar && this.constrainAvatar == "Y") || (widget != draggableManager.widget && (!this.horizontal || (x >= overCoords.x && x <= overCoords.x + overCoords.w)))) {
					overCoords.h = widget.domNode.offsetHeight + dojo.style(widget.domNode, "marginTop") + dojo.style(widget.domNode, "marginBottom");
					if (this.constrainAvatar && this.constrainAvatar == "A")
						var row = Math.round((overCoords.y - dojo.style(widget.domNode, "marginTop") - draggableManager.containerCoords.y) / 27);
					if ((this.constrainAvatar && this.constrainAvatar == "X") || (y >= overCoords.y && y <= overCoords.y + overCoords.h) || (this.constrainAvatar && this.constrainAvatar == "A" && ((row == 0 && y < overCoords.y) || (row == draggableManager.maxRow && y > overCoords.y + overCoords.h)))) {
						if (this.horizontal) {
							if (draggableManager.atStart) {
								var before = true;
							} else if (draggableManager.atRowEnd) {
								var before = false;
							} else {
								var before = (x - overCoords.x) < (overCoords.w / 2);	
							}
						} else {
							var before = (y - overCoords.y) < (overCoords.h / 2);
						} 
						if (this.overWidget != widget) {
							this.overWidget = widget;
						}
						if (this.overWidget) {
							this.insertPlaceholder(before);
						}
						return;
					}
				}
			}
			var placeholder = this.getPlaceholder();
			if (placeholder) {
				var wc = draggableManager.widgetCoords;
				var pc = dojo.coords(placeholder, true);
				pc.w = placeholder.offsetWidth;
				pc.h = placeholder.offsetHeight + wc.marginTop + wc.marginBottom;
				if ((this.constrainAvatar && this.constrainAvatar == "Y") || !this.horizontal || (x >= pc.x - wc.marginLeft && x <= pc.x + pc.w)) {
					if ((this.constrainAvatar && this.constrainAvatar == "X") || (y >= pc.y - wc.marginTop - 3 && y <= pc.y + pc.h)) {
						return;
					}
				}
			}
			if (!this.horizontal || draggableManager.atEnd) {
				this.dropAtEnd = true;
				this.insertPlaceholder(false);
				this.overWidget = this.widgets[this.widgets.length - 1];
			}
		},
		onMouseUp: function(event) {
			if (draggableManager.isDragging) {
				var fromIndex = draggableManager.widget.index;
				if (draggableManager.overContainer.isSource) {
					var fromContainer = draggableManager.fromContainer;
					var overContainer = draggableManager.overContainer;
					var placeholder = this.getPlaceholder();
					if (placeholder){
						var toIndex = this.getPlaceholder().index;	
					} else {
						this.onDndStop(false);
						return;
					}
					try {
						if (fromContainer != overContainer || fromIndex != toIndex) {
							this.move(fromIndex, overContainer, toIndex);
						}	
					} catch (err) {
						this.onDndStop(false);
						console.error("Drag and drop error: ", err);
						return;
					}		
				} else {
					var toTarget = draggableManager.overContainer;
					var cancelDrag = toTarget.move(fromIndex, toTarget);
					if (cancelDrag) {
						draggableManager.fromContainer.onDndStop(false);
						return;
					}
				}
			}
			this.onDndStop(true);
		},
		onDndStart: function(widget, event) {
			if (widget.dndEvent) {
				dojo.disconnect(widget.dndEvent);	
			}
			dojo.addClass(document.body,"jazz-ui-dnd-moveCursor");
			widget.dndEvent = dojo.connect(document.body,"onmouseup",this,this.onMouseUp);
			draggableManager.initDrag(this, widget, event);
		},
		onDndStop: function(success) {
			var widget = draggableManager.widget;
			dojo.removeClass(document.body,"jazz-ui-dnd-moveCursor");
			if (widget) {
				if (widget.dndEvent) {
					dojo.disconnect(widget.dndEvent);
				}
				var container = success ? draggableManager.overContainer : this;
				widget.dndEvent = dojo.connect(widget.handle,"onmousedown",container,this.onMouseDown);				
			}
			draggableManager.stopDrag();
			this.clearPlaceholder();
		},
		insertPlaceholder: function(before){
			var widgetCoords = draggableManager.widgetCoords;
			
			this.height = widgetCoords.h - 2;
			if (this.height == 0) {
				this.height = 50;
			}
			this.clearPlaceholder();
			
			var placeholder = document.createElement("div");
			placeholder.id = "jazz-ui-dnd-placeholder";
			
			placeholder.style.height = this.height - (widgetCoords.marginTop + widgetCoords.marginBottom) + "px";
			if (this.horizontal) {
				placeholder.style.width = widgetCoords.w + "px";
			}
			var dec = 0;
			if (this == draggableManager.fromContainer && this.overWidget.index > draggableManager.widget.index) {
				dec = 1;
			}
			
			if (this.dropAtEnd) {
				if (!this.node.lastChild || dojo.hasClass(this.node.lastChild,"dndItem")){
					this.node.appendChild(placeholder);
				} else {
					this.node.insertBefore(placeholder,this.node.lastChild);
				}
				placeholder.index = this.widgets.length - dec;
				return;
			}
			if (before) {
				this.node.insertBefore(placeholder,this.overWidget.domNode);
				placeholder.index = this.overWidget.index - dec;
			} else {
				var next = this.overWidget.domNode.nextSibling;
				if (!next) {
					this.node.appendChild(placeholder);
				} else {
					this.node.insertBefore(placeholder,next);
				}
				placeholder.index = this.overWidget.index + 1 - dec;
			}
		},	
		clearPlaceholder: function() {
			var placeholder = this.getPlaceholder();
			if (placeholder) {
				placeholder.parentNode.removeChild(placeholder);
			}
		},
		getPlaceholder: function() {
			return dojo.byId("jazz-ui-dnd-placeholder");
		},
		updateIndices: function(col) {
			if (!col) {
				col = this;
			}
			for (var i = 0; i < col.widgets.length; i++) {
				col.widgets[i].index = i;
			}
		}
	});
	
	dojo.provide("jazz.ui.dnd.Avatar");
	
	jazz.ui.dnd.Avatar = function(manager, className){
		this.manager = manager;
		this.construct(className);
	};
	
	dojo.extend(jazz.ui.dnd.Avatar, {
		construct: function(className) {
			var a = dojo.doc.createElement("div");
			a.className = "jazz-ui-dnd-avatar dndAvatar";
			var classes = className.split(" ");
			for (var i = 0; i < classes.length; i++) {
				if(classes[i]) {
					dojo.addClass(a,classes[i]);
				}
			}
			a.id = "avatar";
			a.style.position = "absolute";
			a.style.zIndex = 10000;
			a.style.margin = "0px";
			a.style.width = "33%";
			var widget = this.manager.widget;
			var node = widget.domNode.cloneNode(true);
			a.style.width = this.manager.widgetCoords.w + "px";
			widget.domNode.style.display = "none";
			node.id = "";
			a.appendChild(node);
			this.node = a;
		},
		destroy: function(){
			dojo._destroyElement(this.node);
			if (this.manager.widget.domNode) {
				this.manager.widget.domNode.style.display = "";	
			}
			this.node = false;
		}
	});
	
	jazz.ui.dnd.TRIGGER_AUTOSCROLL = 32;
	jazz.ui.dnd.AUTOSCROLL_OFFSET = 16;
	
	jazz.ui.dnd.autoScroll = function(e){
		// summary: scrolls the window if the cursor is within TRIGGER_AUTOSCROLL pixels of the edge of the viewport
		var dx = 0;
		var dy = 0;
		var viewport = dijit.getViewport();
		var trigger = jazz.ui.dnd.TRIGGER_AUTOSCROLL;
		var offset = jazz.ui.dnd.AUTOSCROLL_OFFSET;
		
		var de = document.documentElement;
		var scrollTop = de.scrollTop ? de.scrollTop : document.body.scrollTop;
		var scrollLeft = de.scrollLeft ? de.scrollLeft : document.body.scrollLeft;
		
		if (e.clientX < trigger) { //if the mouse coord is less than TRIGGER_AUTOSCROLL
			dx = -offset;
		} else if (e.clientX > viewport.w - trigger && scrollLeft < draggableManager._scrollMaxX) {
			dx = offset;
		}
		
		if (e.clientY < trigger) {
			dy = -offset;
		} else if (e.clientY > viewport.h - trigger && scrollTop < draggableManager._scrollMaxY) {
			dy = offset;
		}
		
		window.scrollBy(dx, dy);

	};
})();