/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.examples.PaginationControlExamples");

/*
 * Imports 
 */
dojo.require("jazz.ui.PaginationControl"); 


(function() {

/*
 * Aliases 
 */
var PaginationControl = jazz.ui.PaginationControl;

/*
 * Class 
 */
jazz.ui.examples.PaginationControlExamples = {
	
	/*
 	 * @param void
 	 * 
 	 * @return Element
 	 */		
	_createExample1: function() {
		
		var container = this._createContainer("e1:click");
		
		var paginationControl = new PaginationControl({
			total: 50,
			increment: 6,
			onNextClick: dojo.hitch(this, function(widget, start, end) {
				console.debug("Start: " + start);
				console.debug("End:   " + end);
			}),
			onPreviousClick: dojo.hitch(this, function(widget, start, end) {
				console.debug("Start: " + start);
				console.debug("End:   " + end);
			})
		});

		container.appendChild(paginationControl.domNode);		
		
		return container;
	},

	/*
 	 * @param void
 	 * 
 	 * @return Element
 	 */		
	_createExample2: function() {
		
		var page = this._getPage();	
		
		var onNextClick = dojo.hitch(this, function(widget, start, end) {
			var pageNumber = "Page: " + Math.floor(end / widget.increment);
			page.innerHTML = pageNumber;
			console.debug("Next: " + pageNumber);
		});
		var onPreviousClick = dojo.hitch(this, function(widget, start, end) {
			var pageNumber = "Page: " + Math.floor(end / widget.increment);
			page.innerHTML = pageNumber;
			console.debug("Previous: " + pageNumber);
		});
		
		var total = 100;
		var increment = 8;
		var name = "Example2";
		
		var paginationControl1 = new PaginationControl({
			total: total,
			increment: increment,
			onNextClick: onNextClick,
			onPreviousClick: onPreviousClick,
			name: name
		});
		var paginationControl2 = new PaginationControl({
			total: total,
			increment: increment,
			onNextClick: onNextClick,
			onPreviousClick: onPreviousClick,
			name: name
		});
		
		var container = this._createContainer("e2:click");
		container.appendChild(paginationControl1.domNode);		
		container.appendChild(page);		
		container.appendChild(paginationControl2.domNode);		
		
		return container;
	},

	/*
 	 * @param void
 	 * 
 	 * @return Element
 	 */		
	_createExample3: function() {
		
		var container = this._createContainer("e3:href");
		
		var paginationControl = new PaginationControl({
			total: 50,
			increment: 6,
			getNextHref: dojo.hitch(this, function(widget, start, end) {
				console.debug("Next Href: Start: " + start + ", End:   " + end);
				var nextNum = end + 1;
				return "#test" + nextNum;
			}),
			getPreviousHref: dojo.hitch(this, function(widget, start, end) {
				console.debug("Previous Href: Start: " + start + ", End:   " + end);
				var previousNum = start - widget.increment;
				return "#test" + previousNum;
			})
		});

		container.appendChild(paginationControl.domNode);		
		
		return container;
	},

	/*
 	 * @param void
 	 * 
 	 * @return Element
 	 */		
	_createExample4: function() {
		
		var page = this._getPage();	
		
		var getNextHref = dojo.hitch(this, function(widget, start, end) {
			var pageNumber = "Page: " + Math.floor(end / widget.increment);
			page.innerHTML = pageNumber;
			console.debug("Next: " + pageNumber);
			var nextNum = end + 1;
			return "#test" + nextNum;
		});
		var getPreviousHref = dojo.hitch(this, function(widget, start, end) {
			var pageNumber = "Page: " + Math.floor(end / widget.increment);
			page.innerHTML = pageNumber;
			console.debug("Previous: " + pageNumber);
			var previousNum = start - widget.increment;
			return "#test" + previousNum;
		});
		
		var total = 100;
		var increment = 8;
		var name = "Example4";
		
		var paginationControl1 = new PaginationControl({
			total: total,
			increment: increment,
			getNextHref: getNextHref,
			getPreviousHref: getPreviousHref,
			name: name
		});
		var paginationControl2 = new PaginationControl({
			total: total,
			increment: increment,
			getNextHref: getNextHref,
			getPreviousHref: getPreviousHref,
			name: name
		});
		
		var container = this._createContainer("e4:href");
		container.appendChild(paginationControl1.domNode);		
		container.appendChild(page);		
		container.appendChild(paginationControl2.domNode);		
		
		return container;
	},

	/*
 	 * @param void
 	 * 
 	 * @return Element
 	 */		
	_createExample5: function() {
		
		var container = this._createContainer("e5:click");
		
		var paginationControl = new PaginationControl({
			total: 50,
			startPage: 2, // Page #3
			increment: 6,
			onNextClick: dojo.hitch(this, function(widget, start, end) {
				console.debug("Start: " + start);
				console.debug("End:   " + end);
			}),
			onPreviousClick: dojo.hitch(this, function(widget, start, end) {
				console.debug("Start: " + start);
				console.debug("End:   " + end);
			})
		});

		container.appendChild(paginationControl.domNode);		
		
		var button = this._createEnablementButton(paginationControl);
		container.appendChild(button);		
		
		return container;
	},

	/*
 	 * @param void
 	 * 
 	 * @return Element
 	 */		
	_createExample6: function() {
		
		var container = this._createContainer("e6:href");
		
		var paginationControl = new PaginationControl({
			total: 50,
			startPage: 5, // Page #6
			increment: 6,
			getNextHref: dojo.hitch(this, function(widget, start, end) {
				console.debug("Next Href: Start: " + start + ", End:   " + end);
				var nextNum = end + 1;
				return "#test" + nextNum;
			}),
			getPreviousHref: dojo.hitch(this, function(widget, start, end) {
				console.debug("Previous Href: Start: " + start + ", End:   " + end);
				var previousNum = start - widget.increment;
				return "#test" + previousNum;
			})
		});

		container.appendChild(paginationControl.domNode);		

		var button = this._createEnablementButton(paginationControl);
		container.appendChild(button);		
		
		return container;
	},

	/*
 	 * @param PaginationControl paginationControl
 	 * 
 	 * @return Element
 	 */		
	_createEnablementButton: function(paginationControl) {

		var button = document.createElement("button");
		button.innerHTML = "Disable";
		button.value = "-";
		dojo.connect(button, "onclick", function(event) {
			if (event) {
				dojo.stopEvent(event);
			} else {
				return false;
			}
			
			if (!event.currentTarget) {
				return false;
			}
			
			if (event.currentTarget.value === "-") {
				event.currentTarget.innerHTML = "Enable";
				event.currentTarget.value = "+";
				paginationControl.setEnabled(false);
			} else {
				event.currentTarget.innerHTML = "Disable";
				event.currentTarget.value = "-";
				paginationControl.setEnabled(true);
			}
			return true;			
		});
		
		return button;
	},

	/*
 	 * @param String label
 	 * 
 	 * @return Element
 	 */		
	_createContainer: function(label) {
		var container = document.createElement("div");
		container.style.width = "400px";
		container.style.border = "1px solid black";
		container.style.margin = "3px";
		container.style.padding = "15px";
		
		var textNode = document.createTextNode(label);
		container.appendChild(textNode);
		
		return container;
	},

	/*
 	 * @param void
 	 * 
 	 * @return Element
 	 */		
	_getPage: function() {
		var page = document.createElement("div");
		page.style.margin = "10px";
		page.style.padding = "25px";
		page.style.backgroundColor = "#CCCCCC";
		page.innerHTML = "Page: 1";	
		return page;
	},
	
	/*
 	 * @param Element parent
 	 * 
 	 * @return void
 	 */		
	create: function(parent) {
		
		var table = document.createElement("table");
		var tbody = document.createElement("tbody");

		var tr1 = document.createElement("tr");
		var td1 = document.createElement("td");
		var td2 = document.createElement("td");
		
		td1.appendChild(this._createExample1());
		td2.appendChild(this._createExample2());
		tr1.appendChild(td1);
		tr1.appendChild(td2);

		var tr2 = document.createElement("tr");
		var td3 = document.createElement("td");
		var td4 = document.createElement("td");

		td3.appendChild(this._createExample3());
		td4.appendChild(this._createExample4());
		tr2.appendChild(td3);
		tr2.appendChild(td4);

		var tr3 = document.createElement("tr");
		var td5 = document.createElement("td");
		var td6 = document.createElement("td");

		td5.appendChild(this._createExample5());
		td6.appendChild(this._createExample6());
		tr3.appendChild(td5);
		tr3.appendChild(td6);
		
		tbody.appendChild(tr1);
		tbody.appendChild(tr2);
		tbody.appendChild(tr3);

		table.appendChild(tbody);
		parent.appendChild(table);
	}

};


})();

