/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.examples.SwapListDialogExamples");

/*
 * Imports 
 */
dojo.require("jazz.ui.SwapListDialog"); 


(function() {

/*
 * Aliases 
 */
var SwapListDialog = jazz.ui.SwapListDialog;

/*
 * Class 
 */
jazz.ui.examples.SwapListDialogExamples = {
	
	iconMap: [
		"http://www.mozilla.com/favicon.ico",
		"http://www.ibm.com/favicon.ico",
		"http://www.apache.org/favicon.ico"
	],

	entries: [
				{
					label: "Arts &amp; Humanities",
					value: "value_1",
					id: "id_1",
					type: 0
				},	
				{
					label: "Business &amp; Economy",
					value: "value_2",
					id: "id_2",
					type: 1
				},	
				{
					label: "Education",
					value: "value_3",
					id: "id_3",
					type: 2
				},	
				{
					label: "Entertainment",
					value: "value_4",
					id: "id_4",
					type: 0
				},	
				{
					label: "Government",
					value: "value_5",
					id: "id_5",
					type: 1
				},	
				{
					label: "Health",
					value: "value_6",
					id: "id_6",
					type: 2
				},	
				{
					label: "News &amp; Media",
					value: "value_7",
					id: "id_7",
					type: 0
				},	
				{
					label: "Recreation &amp; Sports",
					value: "value_8",
					id: "id_8",
					type: 1
				},	
				{
					label: "Reference",
					value: "value_9",
					id: "id_9",
					type: 2
				},	
				{
					label: "Regional",
					value: "value_10",
					id: "id_10",
					type: 0
				},	
				{
					label: "Science",
					value: "value_11",
					id: "id_11",
					type: 1
				},	
				{
					label: "Social Science",
					value: "value_12",
					id: "id_12",
					type: 2
				},	
				{
					label: "Society &amp; Culture",
					value: "value_13",
					id: "id_13",
					type: 0
				},	
				{
					label: "Computers &amp; Internet",
					value: "value_14",
					id: "id_14",
					type: 1
				}
			],

	/*
 	 * @param void
 	 * 
 	 * @return SwapListDialog
 	 */		
	_createSwapListDialog1: function() {
		var dialog = new jazz.ui.SwapListDialog({});
		
		dialog.setSelectedEntries(this.entries.slice(0, 7));
		dialog.setUnselectedEntries(this.entries.slice(7));
		
		return dialog;
	},

	/*
 	 * @param void
 	 * 
 	 * @return SwapListDialog
 	 */		
	_createSwapListDialog2: function() {
		var dialog = new jazz.ui.SwapListDialog({
			iconMap: this.iconMap,
			onOK: function(dialog) {
				var selectedEntries = dialog.getSelectedEntries();
				console.debug(selectedEntries);
			},
			onCancel: function(dialog) {
				var unselectedEntries = dialog.getUnselectedEntries();
				console.debug(unselectedEntries);   
			}
		});
		
		dialog.setSelectedEntries(this.entries.slice(0, 7));
		dialog.setUnselectedEntries(this.entries.slice(7));
		
		return dialog;
	},

	/*
 	 * @param void
 	 * 
 	 * @return SwapListDialog
 	 */		
	_createSwapListDialog3: function() {
		var dialog = new jazz.ui.SwapListDialog({
			iconMap: this.iconMap,
			sortField: "type"
		});
		
		dialog.setSelectedEntries(this.entries.slice(0, 7));
		dialog.setUnselectedEntries(this.entries.slice(7));
		
		return dialog;
	},

	/*
 	 * @param void
 	 * 
 	 * @return SwapListDialog
 	 */		
	_createSwapListDialog4: function() {
		var dialog = new jazz.ui.SwapListDialog({
			width: "1000px",
			height: "235px"
		});
		
		dialog.setSelectedEntries(this.entries.slice(0, 7));
		dialog.setUnselectedEntries(this.entries.slice(7));
		
		return dialog;
	},

	/*
 	 * @param Element parent
 	 * 
 	 * @return void
 	 */		
	create: function(parent) {
		
		var div = document.createElement("div");
		div.style.margin = "10px";
		var context = this;
		
		// Example 1
		var b1 = document.createElement("button");
		b1.innerHTML = "Example 1";
		dojo.connect(b1, "onclick", function(event) {
			if (event) {
				dojo.stopEvent(event);
			}
			var sld1 = context._createSwapListDialog1();
			sld1.show();
		});

		// Example 2
		var b2 = document.createElement("button");
		b2.innerHTML = "Example 2";
		dojo.connect(b2, "onclick", function(event) {
			if (event) {
				dojo.stopEvent(event);
			}
			var sld2 = context._createSwapListDialog2();
			sld2.show();
		});

		// Example 3
		var b3 = document.createElement("button");
		b3.innerHTML = "Example 3";
		dojo.connect(b3, "onclick", function(event) {
			if (event) {
				dojo.stopEvent(event);
			}
			var sld3 = context._createSwapListDialog3();
			sld3.show();
			sld3.showLoading(true);
			setTimeout(function() {
				sld3.showLoading(false);
			}, 5000);
		});
		
		// Example 4
		var b4 = document.createElement("button");
		b4.innerHTML = "Example 4";
		dojo.connect(b4, "onclick", function(event) {
			if (event) {
				dojo.stopEvent(event);
			}
			var sld4 = context._createSwapListDialog4();
			sld4.show();
		});
		
		div.appendChild(b1);
		div.appendChild(b2);
		div.appendChild(b3);
		div.appendChild(b4);

		parent.appendChild(div);
	}

};

})();

