/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008-2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.util.i18n");

(function() {

jazz.util.i18n = {
	
	STRONG: "b",
	SPAN: "s",
	
	/*
	 * Takes a string of the form: "Start<0>link</0><b>bold</b>Middle<s2>span</s2><b1>bold</b1>End"
	 * and returns an array of the form:
	 * 		[{text: "Start"},
	 * 		{text: "link", id: 0, type: "a"},
	 * 		{text: "bold", type: "strong"},
	 * 		{text: "Middle"},
	 * 		{text: "span", id: 2, type: "span"},
	 * 		{text: "bold", id: 1, type: "strong"},
	 * 		{text: "End"}]
	 */
	split: function(message) {
		try {
			var result = [];
			if (typeof(message) == "string") {
				var frag = message;
				var index = frag.indexOf("<");
				while (index != -1) {
					if (index != 0) {
						result.push({
							text: frag.substring(0, index)
						});
					}
					frag = frag.substring(index+1);
					if (frag.charAt(0) == "/")
						throw "End tag with no start tag.";
					index = frag.indexOf(">");
					var id = frag.substring(0,index);
					frag = frag.substring(index + 1);
					if (id.charAt(id.length - 1) == "/") {
						id = id.substring(0,id.length-1);
						var text = "";
					}
					else {
						var endTag = "</" + id + ">";
						index = frag.indexOf(endTag);
						if (index == -1) 
							throw "Start tag with no end tag for id " + id + ".";
						var text = frag.substring(0, index);
						frag = frag.substring(index + endTag.length);
					}
					var r = this._parseId(id);
					r.text = text;
					result.push(r);
					index = frag.indexOf("<");
				}
				result.push({
					text: frag
				});
			}
			return result;
		}
		catch (e) {
			console.error("I18nUtil error in message \"" + message + "\":" + e);
		}
	},
	
	/*
	 * Takes a string of the format specified in the split function and builds the represented
	 * dom structure by appending to the specified node. Returns an array of references to nodes
	 * using their ids as the indecies. Nodes with no id are not returned.
	 */
	addMessage: function(node, message) {
		var r = [];
		var s = this.split(message);
		for (var i = 0; i < s.length; i++) {
			if (typeof s[i].type == "undefined") {
				node.appendChild(document.createTextNode(s[i].text));
			}
			else {
				var c = document.createElement(s[i].type);
				if (s[i].type == "a")
					c.href = "#";
				c.innerHTML = s[i].text;
				node.appendChild(c);
				if (s[i].id == 0 || s[i].id)
					r[s[i].id] = c;
			}
		}
		return r;
	},
	
	_parseId: function(id) {
		var c = id.charAt(0).toLowerCase();
		var result = {};
		if (c >= "a" && c <= "z") {
			switch(c) {
				case this.SPAN:
					result.type = "span";
					break;
				case this.STRONG:
					result.type = "strong";
					break;
			}
			id = id.substring(1);
			if (id.length > 0)
				result.id = id;
		} else {
			result.type = "a";
			result.id = id;
		}
		return result;
	}
	
};

})();
