/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
/**
 * This is all borrowed (and slightly modified) from Shindig code for the OpenSocial
 * dynamic-height feature.
 */
if (window.dojo) {
	dojo.provide("jazz.util.iframe");
} else {
	window.jazz = window.jazz || {};
	jazz.util = jazz.util || {};
	jazz.util.iframe = jazz.util.iframe || {};
}

(function() {

/**
 * Measures the height of the current document. Takes into account the fact that the viewport
 * may be taller than the document itself and will always return the true height of the content.
 */
jazz.util.iframe.getDocumentHeight = function() {
	var height;
	// Calculating inner content height is hard and different between
	// browsers rendering in Strict vs. Quirks mode.  We use a combination of
	// three properties within document.body and document.documentElement:
	// - scrollHeight
	// - offsetHeight
	// - clientHeight
	// These values differ significantly between browsers and rendering modes.
	// But there are patterns.  It just takes a lot of time and persistence
	// to figure out.
	
	// Get the height of the viewport
	var vh = getViewportDimensions().height;
	var body = document.body;
	var docEl = document.documentElement;
	if (document.compatMode === 'CSS1Compat' && docEl.scrollHeight) {
		// In Strict mode:
		// The inner content height is contained in either:
		//    document.documentElement.scrollHeight
		//    document.documentElement.offsetHeight
		// Based on studying the values output by different browsers,
		// use the value that's NOT equal to the viewport height found above.
		height = docEl.scrollHeight !== vh ? docEl.scrollHeight : docEl.offsetHeight;
	} else if (navigator.userAgent.indexOf('AppleWebKit') >= 0) {
		// In Webkit:
		// Property scrollHeight and offsetHeight will only increase in value.
		// This will incorrectly calculate reduced height of a gadget
		// (ie: made smaller).
		height = getHeightForWebkit();
	} else if (body && docEl) {
		// In Quirks mode:
		// documentElement.clientHeight is equal to documentElement.offsetHeight
		// except in IE.  In most browsers, document.documentElement can be used
		// to calculate the inner content height.
		// However, in other browsers (e.g. IE), document.body must be used
		// instead.  How do we know which one to use?
		// If document.documentElement.clientHeight does NOT equal
		// document.documentElement.offsetHeight, then use document.body.
		var sh = docEl.scrollHeight;
		var oh = docEl.offsetHeight;
		if (docEl.clientHeight !== oh) {
			sh = body.scrollHeight;
			oh = body.offsetHeight;
		}
		
		// Detect whether the inner content height is bigger or smaller
		// than the bounding box (viewport).  If bigger, take the larger
		// value.  If smaller, take the smaller value.
		if (sh > vh) {
			// Content is larger
			height = sh > oh ? sh : oh;
		} else {
			// Content is smaller
			height = sh < oh ? sh : oh;
		}
    }
	return height;
}

function getViewportDimensions() {
	var x,y;
	if (self.innerHeight) {
		// all except Explorer
		x = self.innerWidth;
		y = self.innerHeight;
	} else if (document.documentElement &&
	           document.documentElement.clientHeight) {
		// Explorer 6 Strict Mode
		x = document.documentElement.clientWidth;
		y = document.documentElement.clientHeight;
	} else if (document.body) {
		// other Explorers
		x = document.body.clientWidth;
		y = document.body.clientHeight;
	} else {
		x = 0;
		y = 0;
	}
	return {width: x, height: y};
}

/**
* Parse out the value (specified in px) for a CSS attribute of an element.
*
* @param {Object} elem the element with the attribute to look for.
* @param {String} attr the CSS attribute name of interest.
* @returns {int} the value of the px attr of the elem.
*/
function parseIntFromElemPxAttribute(elem, attr) {
	var style = window.getComputedStyle(elem, "");
	var value = style.getPropertyValue(attr);
	value.match(/^([0-9]+)/);
	return parseInt(RegExp.$1, 10);
}

/**
* For Webkit-based browsers, calculate the height of the gadget iframe by
* iterating through all elements in the gadget, starting with the body tag.
* It is not sufficient to only account body children elements, because
* CSS style position "float" may place a child element outside of the
* containing parent element. Not counting "float" elements may lead to
* undercounting.
*
* @returns {int} the height of the gadget.
*/
function getHeightForWebkit() {
	var result = 0;
	var queue = [ document.body ];
	
	while (queue.length > 0) {
		var elem = queue.shift();
		var children = elem.childNodes;
		
		for (var i = 0; i < children.length; i++) {
			var child = children[i];
			if (typeof child.offsetTop !== 'undefined' &&
				typeof child.scrollHeight !== 'undefined') {
				// scrollHeight already accounts for border-bottom, padding-bottom.
				var bottom = child.offsetTop + child.scrollHeight + parseIntFromElemPxAttribute(child, "margin-bottom");
				result = Math.max(result, bottom);
			}
			queue.push(child);
		}
	} 
	
	// Add border, padding and margin of the containing body.
	return result
	    + parseIntFromElemPxAttribute(document.body, "border-bottom")
	    + parseIntFromElemPxAttribute(document.body, "margin-bottom")
	    + parseIntFromElemPxAttribute(document.body, "padding-bottom");
}

})();