/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("jazz.app.internal.Banner");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.app.internal.BannerButton");
dojo.require("jazz.app.ThemableNode");
dojo.require("dijit.Menu");
dojo.require("jazz.app.internal.LoginLogoutArea");
dojo.require("jazz.ui.MenuPopup");

dojo.requireLocalization("jazz.app.internal", "NavbarMessages");

var LoginLogoutArea = jazz.app.internal.LoginLogoutArea;
var messages;

/*
 * Provides the web UI's header area.
 */
dojo.declare("jazz.app.internal.Banner", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("jazz.app", "internal/templates/Banner.html"),
	
	buttons: [],
	
	init: function(args) {
		if(args.showLoginLogout){
			this._rightText.appendChild((new LoginLogoutArea({config: args})).domNode);	
		}
		
		if (!args.productTitleImg) {
			args.productTitleImg = dojo.moduleUrl("jazz.app", "graphics/JazzUI/Jazz-Team-Server.png");
		}
		this._productTitle.innerHTML = "";
		var title = document.createElement("img");
		title.alt = args.productName;
		title.src = args.productTitleImg;
		this._productTitle.appendChild(title);
		
		var hasLeft = false;
		if (args.buttons) {
			this.buttons = args.buttons;
			for(var i = 0; i < this.buttons.length; i++) {
				this.buttons[i]._widget = this.createButton(this.buttons[i]);
				if (this.buttons[i].align && this.buttons[i].align === "left") {
					hasLeft = true;
				}
			}
		}
		if (!hasLeft && args.productIconLarge) {
			this._prodIcon.style.display = "block";
			var img = document.createElement("img");
			img.src = args.productIconLarge;
			this._prodIcon.appendChild(img);
		}
		this._setupBrandingHooks();
	},
	createButton: function(args) {
		// Creates a button, puts it in the CommonNav and returns a reference.
		// args: iconSrc, title, href, menuProvider, align: "left" (default) or "right"
		if (!args.iconSrc || !args.label) {
			console.error("Error creating button: Must pass iconSrc and label params"); //$NON-NLS-1$
			return;
		}
		var button;
		if(!args.align || args.align === "left") {
			button = new jazz.app.internal.BannerButton();
		} else {
			button = {domNode: document.createElement("a")};
		}
		var label = args.label ? args.label : "";
		var href = "";
		if (args.pageId) {
			href = "#action=jazz.viewPage&id=" + args.pageId;
			dojo.subscribe(net.jazz.ajax.internal.events.viewPage, this, function(page) {
				this._checkButtonHighlight(page, args.pageId, button.domNode);
			});
		} else {
			href = args.href ? args.href : "#";	
		}
		button.domNode.href = href;
		this.connect(button.domNode, "onclick", function(event) {
			if (!href || href === "#") {
				dojo.stopEvent(event);
				if (button._menuPopup) {
					button._menuPopup._mouseOver();
				}
			} else {
				var PlatformUI = dojo.getObject("net.jazz.ajax.ui.PlatformUI");
				if (PlatformUI && PlatformUI.getWorkbench) {
					var workbench = PlatformUI.getWorkbench();
				}
				if (!workbench || !workbench.shouldVetoNavigation(href)) {
					location.href = href;
				} else {
					dojo.stopEvent(event);
				}
			}
		});
		dojo.addClass(button.domNode, "menu");
		button.domNode.title = label;
		var icon = document.createElement("img");
		icon.alt = label;
		icon.src = args.iconSrc;
		if (button.content) {
			button.content.appendChild(icon);
		} else {
			button.domNode.appendChild(icon);	
		}
		if (args.align && args.align == "right") {
			this._rightButtons.appendChild(button.domNode);	
		} else {
			this._leftButtons.appendChild(button.domNode);
			this._leftButtons.style.display = "block";
		}
		if (args.menuProvider) {
			if (button.content) {
				button._menuPopup = new jazz.ui.MenuPopup({align: args.align, menuClass:"jazz-app-BannerButton-menu", menuProvider: args.menuProvider, around: button.domNode}, button.content);	
			} else {
				button._menuPopup = new jazz.ui.MenuPopup({align: args.align, menuClass:"jazz-app-Banner-menu", menuProvider: args.menuProvider, around: button.domNode}, button.domNode);
			}
		} else {
			dojo.addClass(button.domNode,"menu-narrow");
		}
		return button;
	},
	_checkButtonHighlight: function (page, pageId, button) {
		if (page.id === pageId) {
			dojo.addClass(button,"selected");
		} else {
			dojo.removeClass(button,"selected");
		}
	},
	_setupBrandingHooks: function () {
		console.debug("Entered Baner._setupBrandingHooks");
		new jazz.app.ThemableNode({
			targetNode: this._aboveBannerNode,
			htmlPath: "html/aboveBanner.html",
			widgetName : "jazz.theme.AboveBannerWidget"
		});		
	},
	constructor: function() {
		console.debug("instantiating banner...");
		this.messages = messages = dojo.i18n.getLocalization("jazz.app.internal", "NavbarMessages");
	}
});
})();