/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.app.internal.CustomNavbar");

(function() {

dojo.require("net.jazz.ajax.ui.PageList");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("jazz.ui.MenuPopup");

dojo.declare("jazz.app.internal.CustomNavbar", [dijit._Widget, dijit._Templated], {
	
	templateString: "<div class=\"jazz-app-CustomNavbar\"><div dojoattachpoint=\"_leftLinks\"></div><div class=\"right-links\" dojoattachpoint=\"_rightLinks\"></div></div>",
	
	init: function(args) {
		if (args.usePageRegistry) {
			this._pageList = new net.jazz.ajax.ui.PageList({menuPopupConstructor: jazz.ui.MenuPopup, applicationId: net.jazz.ajax.ui.PlatformUI._getApplicationId(), suppressedPages: args.suppressedPages}).placeAt(this._leftLinks);
		}
		if (args.navbarItems) {
			for(var i = 0; i < args.navbarItems.length; i++) {
				args.navbarItems[i]._widget = this._createItem(args.navbarItems[i]);
			}
		}
	},
	overridePageName: function(){
		return this._pageList ? this._pageList.overridePageName.apply(this._pageList, arguments) : false;
	},
	_createItem: function(args) {
		// Creates a link/menu, puts it in the navbar and returns a reference.
		// args: iconSrc, title, href, pageId, menuProvider
		if (!args.label) {
			console.error("Error creating button: Must pass label param"); //$NON-NLS-1$
			return;
		}
		var label = args.label ? args.label : "";
		var button = document.createElement("a");
		if (args.pageId) {
			args.href = "#action=jazz.viewPage&id=" + args.pageId;
			if(args.isDefault && window.location.hash.indexOf("action") < 0) {
				if (args.href.charAt(0) == "#") {
					dojo.hash(args.href, true);
				} else {
					window.location.replace(args.href);
				}
			}
			dojo.subscribe(net.jazz.ajax.internal.events.viewPage, this, function(page) {
				this._checkButtonHighlight(page, args.pageId, button);
			});
		}
		button.href = args.href ? args.href : "#";	
		if (!args.href && !args.menuProvider) {
			this.connect(button,"onclick",dojo.stopEvent);
			button.style.cursor = "default";
		}
		if (args.iconSrc) {
			var icon = document.createElement("img");
			icon.alt = label;
			icon.src = args.iconSrc;
			button.appendChild(icon);			
		}
		button.appendChild(document.createTextNode(label));
		if (args.align && args.align == "right") {
			this._rightLinks.appendChild(button);
		} else {
			this._leftLinks.appendChild(button);
		}
		if (args.rightPipe) {
			var pipe = document.createElement("div");
			pipe.className="pipe";
			pipe.innerHTML = "|";
			if (args.align && args.align == "right") {
				this._rightLinks.appendChild(pipe);
			} else {
				this._leftLinks.appendChild(pipe);
			}
		}
		if (args.selected) {
			dojo.addClass(button,"selected");
		}
		if (args.menuProvider) {
			new jazz.ui.MenuPopup({group: "jazz-app-Navbar", align: args.align, menuProvider: args.menuProvider, menuClass:"jazz-app-Navbar-menu", injectCaret: true, cacheMenu: true}, button);
		} else {
			dojo.addClass(button,"menu-narrow");
		}
		return button;
	},
	_checkButtonHighlight: function (page, pageId, button) {
		if (page.id === pageId) {
			dojo.addClass(button,"selected");
		} else {
			dojo.removeClass(button,"selected");
		}
	}
});
})();

