/*******************************************************************************
 * Licensed Materials - Property of IBM (c) Copyright IBM Corporation 2005, 2011,
 * 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("jazz.ui.internal.Drawer");

dojo.require("jazz.ui.MegaMenu");
dojo.require("jazz.ui.Toolbar");
dojo.require("jazz.ui.toolbar.MenuButton");
dojo.require("jazz.ui.toolbar.Button");
dojo.require("jazz.util.ResizeNotifier");

dojo.requireLocalization("jazz.ui.internal", "DrawerMessages");

(function(){

var MIN_WIDTH 			= 16,
	MAX_WIDTH 			= 250,
	ANIM_DURATION		= 300,
	DRAWER_VERT_PAD 	= 8,
	HEIGHT_POLL 		= 1500,
	HEIGHT_FAST_POLL	= 400,
	FAST_POLL_COUNT		= 8,
	LOCAL_STORAGE_KEY	= "jazz-ui-internal-Drawer";

dojo.setObject("jazz.ui.internal.Drawer.state",{
	UNDOCKED  		: "undocked",
	CLOSED    		: "closed",
	OPEN    		: "open",
	PINNED    		: "pinned",
	SLIDING_OPEN	: "closed sliding",
	SLIDING_CLOSED	: "open sliding"
});

var state = jazz.ui.internal.Drawer.state,
	ResizeNotifier = jazz.util.ResizeNotifier;

var _messages;

dojo.declare("jazz.ui.internal.Drawer", [dijit._Widget, dijit._Container, dijit._Templated], {
	templatePath		: dojo.moduleUrl("jazz.ui", "internal/templates/Drawer.html"),
	headerTitle			: "",
	tooltip				: "",
	widgetsInTemplate	: true,
	leftAnchor 			: null,
	rightAnchor 		: null,
	currentAnchor       : null,
	buttons             : null,
	menuItems			: null, //array of objects in the form {label (string), iconClass (string), onClick (function), index (int, optional)}
	matchNode			: null,
	contentMatchNode	: null,
	_toolbar			: null,
	_isOpen				: false,
	_isPinned			: false,
	_slideCloseFn       : null,  // cache the hitch
	_slideOpenFn        : null,  // cache the hitch
	_openAnim           : null,
	_closeAnim          : null,
	_openMenuButton     : null,
	_pollingCount       : 0,
	_heightTimer        : null,
	
	constructor: function() {
		if (!_messages) {
			_messages = this._messages = dojo.i18n.getLocalization("jazz.ui.internal", "DrawerMessages");
		}
	},
	
	postCreate: function(){
		try {
			// each context root (server) used to get a separate state, check for them and delete them
			var legacyLocalStorageKey = "jazz-ui-internal-Drawer" + (dojo.getObject("net.jazz.ajax._contextRoot") || "").replace(/[^a-z0-9]/gi, "-");
			var legacyStateJson = localStorage[legacyLocalStorageKey];
			var anchorName = null;
			var stateJson = localStorage[LOCAL_STORAGE_KEY];
			if (legacyStateJson) {
				// use the legacy state if we have no proper state defined yet
				if (!stateJson)
					stateJson = localStorage[LOCAL_STORAGE_KEY] = legacyStateJson;
				delete localStorage[legacyLocalStorageKey];
			}
			// read pinned & anchor state from local storage
			if (stateJson) {
				var state = dojo.fromJson(stateJson); 
				this._isPinned = this._isOpen = state.isPinned;
				anchorName = state.anchor;
			}
		}
		catch (e) {
			// browser doesn't support localStorage; ignore
		}
		
		this._headerText.appendChild(document.createTextNode(this.headerTitle));
		
		this._menuButton.attr("menuProvider", dojo.hitch(this, this._menuProvider));
		this._pinButton.attr("pressed", this._isPinned);
		/*
		 * cache the slide close function so we don't create a hitch each time.
		 */
		this._slideCloseFn = dojo.hitch(this, "_slideClosed");
		this._slideOpenFn  = dojo.hitch(this, "_slideOpen");
		
		this._backgroundIframe = new dijit.BackgroundIframe(this.domNode);
		
		this._initAnchor(anchorName);
		if (dojo.isIE) this.connect(this.domNode, "onselectstart", this._onSelectStart);
	},
	_onSelectStart: function(e) {
		var ca = this.currentAnchor;
		if (ca) switch(ca.getState()) {
			case state.CLOSED:
			case state.SLIDING_OPEN:
				e.preventDefault();
		}
	},
	startup: function() {
		if (!this.currentAnchor && this._initAnchor() === false) {
			console.error("Error creating Drawer: no anchor provided.");
			return;
		}
		if (this.matchNode) this._yieldHeightCheck();
		dojo.subscribe(jazz.ui.internal.drawer.events.POLL_FAST, this, "pollFast");
		if (this._isPinned) {
			this.domNode.tabIndex="-1";
			this.currentAnchor.setState(state.PINNED);
			this.onOpen();
			this.onPinChange(this._isPinned);
			this.domNode.title = "";
		}
	},
	_initAnchor: function(/* string */ anchorName) {
		if (!(this.leftAnchor && this.rightAnchor)) {
			return false;
		}
		var anchor;
		if (anchorName == "left" && this.leftAnchor) {
			anchor = this.leftAnchor;
		}
		else if (anchorName == "right" && this.rightAnchor) {
			anchor = this.rightAnchor;
		}
		this._attachAnchor(anchor||this.leftAnchor||this.rightAnchor);
	},
	onClose: function() {
		// override to do work on close
	},
	onOpen: function() {
		// override to do work on open
	},
	onPinChange: function(pinned) {
		
	},
	swapTo: function (anchor) {
		this._noBlur = true;
		var state = this.currentAnchor ? this.currentAnchor.getState() : state.PINNED;
		this._attachAnchor(anchor, state);
		this.onPinChange(this._isPinned);
		this._saveState();
		var item = this._menuButton;
		if (item && item.domNode) item.focus();
		this._noBlur = false;
	},
	_attachAnchor: function (anchor, newState) {
		if (!anchor || anchor == this.currentAnchor) return;
		this._menu && this._menu.destroy();
		this._menu = null;
		this.currentAnchor && this.currentAnchor.setState(state.UNDOCKED);  //hide empty anchor if exists
		anchor.attachDrawer(this, newState);	//attach without removing to reduce reflows
		if (anchor.getState() == state.CLOSED) this.domNode.title = this.tooltip;
		this.currentAnchor = anchor;
		this._menuButton.align = (anchor == this.rightAnchor) ? "right" : "left";
		this._yieldHeightCheck();
	},
	onClick: function(e) {
		if (this._isPinned)	return;
		!this._isOpen && dojo.stopEvent(e);
		clearTimeout(this._heightTimer);
		this._heightTimer = null;
		this._slideOpenFn();
	},
	_pin: function() {
		this._isPinned = !this._isPinned;
		
		this.onPinChange(this._isPinned);
		
		if (this._isPinned) {
			this.domNode.tabIndex="-1";
			this.currentAnchor.setState(state.PINNED);
			this.currentAnchor.setHeight();
		} else {
			this.domNode.tabIndex="0";
			this.currentAnchor.setState(state.OPEN);
			if (!this._bodyClick) this._bodyClick = this.connect(document.body, "onclick", this.onBlur);
		}
		this._saveState();
		this.checkHeight();
	},
	getWidth: function() {
		return this._isPinned ? MAX_WIDTH : MIN_WIDTH;
	},
	checkHeight: function() {
		if (this.matchNode && this._innerWrapper) { //if _innerWrapper is null, drawer has been destroyed.
			if (this._currentHeight) {
				if (this.contentMatchNode) {
					var contentH = dojo.marginBox(this.contentMatchNode).h;
					var matchH = dojo.marginBox(this.matchNode).h;
					if ((contentH != this._lastContentH && matchH != this._lastMatchH)) {
						this._currentHeight = null;
					}
					this._lastMatchH = matchH;
					this._lastContentH = contentH;
				}
				var testH = dojo.marginBox(this._testNode).h;
				if (testH != this._lastTestH) {
					this._currentHeight = null;
				}
				this._lastTestH = testH;
			} 
			
			// set heights
			if (!this._currentHeight) {
				// if panel is open, clear the height so we can test the natural height of panel contents
				// else, zero out the height so that it doesn't interfere with the height of matchNode
				this._innerWrapper.style.height = this._isOpen ? "" : "0";
				// when the drawer is sliding, we must also manage the height of the anchor itself
				var doAnchorHeight = !this._isPinned && this._isOpen;
				var forceExpand = this._isPinned || this._isOpen;
				
				doAnchorHeight && this.currentAnchor.setHeight();
				
				if (testH == null) testH = dojo.marginBox(this._testNode).h;
				
				// test heights
				var h = this._isOpen ? Math.max(dojo.marginBox(this.matchNode).h, testH + 4) : dojo.marginBox(this.matchNode).h;
				
				// set heights
				if (doAnchorHeight && h >= 2) {
					this.currentAnchor.setHeight(h + 3);
				}
				if (h >= DRAWER_VERT_PAD) {
					var height = h - DRAWER_VERT_PAD;
					this._innerWrapper.style.height = height + "px";
					if (forceExpand) {
						var forceH = Math.max(h-testH,0);
					}
					this.currentAnchor.domNode.style.marginBottom = forceExpand ? -forceH + "px" : -height + "px";
				}
				this._currentHeight = h;
			}
			// schedule the next loop (quickly for pollingCount < FAST_POLL_COUNT, else longer)
			clearTimeout(this._heightTimer);
			this._heightTimer = setTimeout(dojo.hitch(this, this.checkHeight), (this._pollingCount++ < FAST_POLL_COUNT) ? HEIGHT_FAST_POLL : HEIGHT_POLL);
		}
	},
	_yieldHeightCheck: function() {
		//check height after yielding the UI thread.
		clearTimeout(this._heightTimer);
		this._currentHeight = null;
		setTimeout(dojo.hitch(this,this.checkHeight),0);
	},
	pollFast: function() {
		// check the height more often for a few iterations, including right now.
		this._pollingCount = 0;
		this.checkHeight();
	},
	_slideOpen: function() {
		if (this._isOpen) return;
		this._isOpen = true;

		if (this._closeAnim) {
			this._closeAnim.stop();
			this._closeAnim = null;
		}
		
		this.currentAnchor.setState(state.SLIDING_OPEN);
		
		this.domNode.title = "";
		
		this._openAnim = dojo.animateProperty({
			node       : this.domNode,
			duration   : ANIM_DURATION,
			properties : { width: { start: MIN_WIDTH, end: MAX_WIDTH, unit: "px" }}
		});
		this.domNode.style.overflow = "hidden";
		
		this.connect(this._openAnim, "onEnd", function() {
			this.currentAnchor.setState(state.OPEN);
			this.domNode.style.overflow = "";
			this._openAnim = null;
			setTimeout(dojo.hitch(this,this.onOpen),0);
			this.pollFast();
		});
		this._openAnim.play();
		if (!this._bodyClick) this._bodyClick = this.connect(document.body, "onclick", this.onBlur);
	},
	_onFocus: function(event) {
		this.currentAnchor.addHover();
	},
	onBlur: function(event) {
		// if there is no event or the target is not the body, this is an onBlur
		if (!event || event.currentTarget != document.body) {
			// if there is no new focus, this isn't a "real" onBlur event as we didn't go anywhere
			// either the browser lost focus or we clicked a dijit menu item for a viewlet in IE
			var focus = dijit.getFocus();
			if (!focus.node) return; // ignore the call
			// FF gets confused when you tab to iframes for remote viewlets, wire up a fake event so we can check if it's a descendant
			if ((!dojo.isIE || dojo.isIE > 8) && focus.node.nodeName === "IFRAME") {
				event = {target: focus.node};
			}
		}
		
		// remove the hover class for all blurs
		this.currentAnchor.removeHover();
	
		//do not close if focus changed to a child node of the drawer, or if we're swapping sides
		if (this._noBlur || this._isPinned || (event && event.target && (dojo.isDescendant(event.target, this.domNode) || !dojo.isDescendant(event.target, document.body)))) return;
		
		//do not close if a hover popup is open
		var mp = dojo.query("body > #jazz_ui_internal__MasterPopup_0");
		if (mp && mp[0] && dojo.style(mp[0],"display") != "none") return;
		
		//do not close if a dialog is open
		var dialog = dojo.query("body > .jazz-ui-Dialog-absolute");
		if (dialog.length > 0) return;
		
		this._slideClosed();
	},
	onKeyPress: function(e) {
		if (e && (e.keyCode === dojo.keys.ENTER || e.charCode === dojo.keys.SPACE)) {
			this._slideOpen();
		}
	},
	_slideClosed: function() { 
		if (!this._isOpen) return;
		this._isOpen = false;
		
		if (this._openAnim) {
			this._openAnim.stop();
			this._openAnim = null;
		} 
		
		this.currentAnchor.setState(state.SLIDING_CLOSED);
		
		this._closeAnim = dojo.animateProperty({
			node       : this.domNode,
			duration   : ANIM_DURATION,
			properties : { width: {	start: MAX_WIDTH, end: MIN_WIDTH, unit: "px" } }
		});
		this.onClose();
		this.domNode.style.overflow = "hidden";
		
		this.connect(this._closeAnim, "onEnd", function() {
			this.domNode.style.overflow = "";
			this.domNode.title = this.tooltip;
			this.currentAnchor.setState(state.CLOSED);
			this._closeAnim = null;
			this.currentAnchor.setHeight();
			this.pollFast();
		});
		
		this._closeAnim.play();
	},
	
	addMenuItem: function(args) {
		if (!this.menuItems) {
			this.menuItems = [];
		}
		if (typeof args.index == "number") {
			this.menuItems.splice(args.index,0,args);
		} else {
			this.menuItems.push(args);	
		}
		// if the menu has already been created, insert it
		if (this._menu) {
			if (typeof args.index == "number") {
				this._menu.addItemByArgs(args, args.index);	
			} else {
				this._menu.addItemByArgs(args);
			}
		}
	},
	_createMenu: function (){
		this._menu && this._menu.destroy();
		this._menu = new jazz.ui.MegaMenu({"class": "jazz-ui-Drawer-menu"});

		if (this.leftAnchor) {
			var args = {label:_messages.dockOnLeft, onClick: dojo.hitch(this, "swapTo", this.leftAnchor)};
			if (this.leftAnchor == this.currentAnchor) args["class"] = "selected";
			this._menu.addItemByArgs(args);
		}
		if (this.rightAnchor) {
			args = {label:_messages.dockOnRight, onClick: dojo.hitch(this, "swapTo", this.rightAnchor)};
			if (this.rightAnchor == this.currentAnchor) args["class"] = "selected";
			this._menu.addItemByArgs(args);
		}

		// add known menu contributions
		if (this.menuItems) {
			this._menu.addSeparator();
			dojo.forEach(this.menuItems, function(item) {
				if (item.separator) {
					this._menu.addSeparator();
				} else if (!item.label) {
					console.error("Error creating menu:  Menu item at index "+i+" is missing a label.");
				} else {
					this._menu.addItemByArgs(item);
				}
			}, this);
		}
	},
	_menuProvider: function() {
		if (!this._menu) this._createMenu();
		return this._menu;
	},
	_saveState: function() {
		try {
			// save in local storage for now until we implement
			// 121162: Dashboard mini forgets about position
			localStorage[LOCAL_STORAGE_KEY] = dojo.toJson({
				isPinned: this._isPinned,
				anchor: this.currentAnchor == this.rightAnchor ? "right" : "left"
			});
		}
		catch (e) {
			// browser doesn't support localStorage; ignore and don't save
		}
	},
	destroy: function() {
		clearTimeout(this._heightTimer);
		this._menu && this._menu.destroy();
		this._backgroundIframe.destroy();
		this.inherited(arguments);
	}
});

dojo.declare("jazz.ui.internal.DrawerAnchor", [dijit._Widget, dijit._Templated], {
	templateString: "<div class='jazz-ui-DrawerAnchor undocked' dojoattachevent='onmouseover: addHover, onmouseout: removeHover'></div>",
	_state: state.UNDOCKED,
	_drawer: null,
	
	postCreate: function (){
		this.setState(state.CLOSED);
	},
	attachDrawer: function (drawer, attachState){
		this._drawer = drawer;
		attachState = attachState || state.CLOSED;
		this.setState(attachState);
		this.domNode.appendChild(drawer.domNode);
	},
	removeDrawer: function (drawer){
		this._drawer = null;
		this.domNode.removeChild(drawer.domNode);
		this.setState(state.UNDOCKED);
	},
	addHover: function(event) {
		if (this._state == state.CLOSED) dojo.addClass(this.domNode, "hover");
	},
	removeHover: function(event) {
		if (this._state == state.CLOSED) dojo.removeClass(this.domNode, "hover");
	},
	setState: function (newState){
		var classes = this.domNode.className.split(" ");
		var curStates = this._state.split(" ");
		if (this._state == state.CLOSED) curStates.push("hover");  //add "hover" to list of classes to be removed
		for (var i = 0; i < curStates.length; i++) {
			var curStateIndex = dojo.indexOf(classes,curStates[i]);
			if (curStateIndex >= 0) {
				classes.splice(curStateIndex,1);
			}			
		}
		classes.push(newState);
		this.domNode.className = classes.join(" "); 
		this._state = newState;
	},
	getState: function (){
		return this._state;
	},
	setHeight: function(h) {
		if (this._currentHeight != h) {
			this.domNode.style.height = (typeof h == "number") ? h + "px" : "";
			this._currentHeight = h;
		}
	}
});

dojo.declare("jazz.ui.internal.DrawerAnchorLeft", [jazz.ui.internal.DrawerAnchor], {
	postCreate: function(){dojo.addClass(this.domNode,"left");}
});

dojo.declare("jazz.ui.internal.DrawerAnchorRight", [jazz.ui.internal.DrawerAnchor], {
	postCreate: function(){dojo.addClass(this.domNode,"right");}
});

dojo.setObject("jazz.ui.internal.drawer.events.POLL_FAST","jazz.ui.internal.drawer.events.POLL_FAST");

})();