/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.app.internal.Footer");
(function() {

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("jazz.app.internal", "_AjaxApplicationMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var messages = dojo.i18n.getLocalization("jazz.app.internal", "_AjaxApplicationMessages"); //$NON-NLS-1$ //$NON-NLS-2$

/**
 * Provides the web UI's footer.
 */
dojo.declare("jazz.app.internal.Footer", [dijit._Widget, dijit._Templated], {
		
	templatePath: dojo.moduleUrl("jazz.app", "internal/templates/Footer.html"),
	
	init: function (args) {
		if (!args) {
			args = {};
		}
		if (!args.leftLogo) {
			args.leftLogo = { src: dojo.moduleUrl("jazz.app", "graphics/JazzUI/IBM-logo.png"), href: "http://www.ibm.com/rational/", alt: "IBM" };
		}
		if (!args.rightLogo) {
			args.rightLogo = { src: dojo.moduleUrl("jazz.app", "graphics/JazzUI/jazz-logo.png"), href: "http://jazz.net/", alt: "Jazz" };
		}
		
		this._makeLogo(this._leftLogo, args.leftLogo);
		this._makeLogo(this._rightLogo, args.rightLogo);
		this._makeLinks(this._footerLinks, args.links);
		
		this._setupBrandingHooks();
	},
	_makeLogo: function (node, model) {
		var img = document.createElement("img");
		img.src = model.src;
		img.alt = model.alt;
		if (model.href) {
			var a = document.createElement("a");
			a.href = model.href;
			a.appendChild(img);
			node.appendChild(a);
		} else {
			node.appendChild(img);
		}
	},
	_makeLinks: function(node, model) {
		if (dojo.isArray(model)) {
			for(var i = 0; i < model.length; i++) {
				var a = document.createElement("a");
				a.appendChild(document.createTextNode(model[i].label));
				a.href = model[i].href;
				node.appendChild(a);
				if (i < model.length -1) node.appendChild(document.createTextNode("  |  "));			
			}
		}
	},
	_setupBrandingHooks: function () {
		new jazz.app.ThemableNode({
			targetNode: this._leftLogo,
			htmlPath: "html/footerLeft.html",
			widgetName : "jazz.theme.FooterLeftWidget"
		});
		new jazz.app.ThemableNode({
			targetNode: this._footerLinks,
			htmlPath: "html/footerCenter.html",
			widgetName : "jazz.theme.FooterCenterWidget"
		});
		new jazz.app.ThemableNode({
			targetNode: this._rightLogo,
			htmlPath: "html/footerRight.html",
			widgetName : "jazz.theme.FooterRightWidget"
		});
	},
	showBrowserWarning: function(show) {
		if (show !== false) {
			if (!this._bw) this._bw = dojo.create('span', {
				innerHTML: messages.unsupportedFooter,
				"class": "j-hint j-small"
			},this._footerLinks, "first");
		} else if (this._bw) {
			this._footerLinks.removeChild(this._bw);
			this._bw = null;
		}
	}
});

})();
