/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.app.internal.HomeMenu");

(function(){
	
dojo.require("jazz.app._ComponentMenu");
dojo.require("jazz.app.context");
dojo.require("jazz.app.internal.applicationAbout");
dojo.require("jazz.app.internal.util.DiscoveredService");
dojo.require("jazz.app.internal.util.XhrUtil");
dojo.require("jazz.app.internal.schema.HomeMenu");
dojo.require("jazz.ensemble.internal.Schema");
dojo.require("jazz.ensemble.internal.util.Xml");
dojo.require("jazz.ui.internal.MenuItemGroup");
dojo.require("jazz.ui.toolbar.TwoStateButton")
dojo.require("jazz.util.internal.login");
//dojo.require("jazz.app.internal.lpa");
dojo.require("jazz.util.html");
dojo.require("dojo.string");
dojo.require("jazz.app.events");
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dojo.DeferredList"); //$NON-NLS-1$
dojo.require("jazz.app.entryPoints");
dojo.requireLocalization("jazz.app.internal", "ComponentMenuMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var namespace = jazz.ensemble.internal.util.Xml.namespace;
var messages = dojo.i18n.getLocalization("jazz.app.internal", "ComponentMenuMessages"); //$NON-NLS-1$ //$NON-NLS-2$
var DiscoveredService = jazz.app.internal.util.DiscoveredService;
var XhrUtil	= jazz.app.internal.util.XhrUtil;
var HomeMenuSchema = jazz.app.internal.schema.HomeMenu;
var applicationAbout = jazz.app.internal.applicationAbout;
var WrapHints = jazz.ui.MegaMenu.WrapHints;

var MENU_SECTION_DASHBOARD = "http://jazz.net/ns/ui/menuSection#dashboard";
var MENU_SECTION_PROJECT   = "http://jazz.net/ns/ui/menuSection#project";
var MENU_SECTION_ADMIN   = "http://jazz.net/ns/ui/menuSection#admin";

var MENU_USAGE_ALL    = "http://jazz.net/ns/ui/menuUsage#all";
var MENU_USAGE_ADMIN  = "http://jazz.net/ns/ui/menuUsage#admin";

var DASHBOARD_PATH 	  = "/dashboards/";
var PROXY_HEADER	  = "X-net-jazz-ajax-proxy";

// aggressively load home menu services from discovery
var serviceDeferred;
setTimeout(function() {
	if (!serviceDeferred) {
		serviceDeferred = DiscoveredService.loadDiscoveryServices(namespace.ju + "HomeMenu", HomeMenuSchema.discovery, true);
	}
},2000);

//var RELATED_IMG_WAIT = "jazz.ui/graphics/ResourceLink/spinner.gif"; 
//var RELATED_IMG 	 = "jazz.app/graphics/HomeMenu/show-assoc-proj_co.gif"; 

dojo.declare("jazz.app.internal.HomeMenu", jazz.app._ComponentMenu, {
	
	showFilter: true,
	
	//_href2Resource : {},
	//_relatedResources : null,
	_currentProject : null,
	
	postCreate: function() {
		dojo.addClass(this.domNode,"jazz-app-HomeMenu");
		this._groups = [];
		this._scopers = [];
		this._groupMap = [];
		this._fetchMenuContents();
		this.subscribe(jazz.app.events.contextSwitcher.deleteContext, this.refresh);
		this.subscribe(jazz.app.events.contextSwitcher.addContext, this.refresh);
		this.subscribe(jazz.app.events.HOME_MENU_REFRESH, this.refresh);
		/**
		 * setup related projects
		 */
/*
		jazz.app.internal.lpa.getUserProjectService().addCallback(dojo.hitch(this,function(service){
			if (!service)return;
			this._showRelatedBtn = new jazz.ui.toolbar.TwoStateButton({
				"imgSrc": RELATED_IMG,
				"title" : messages.showRelated
			});
			
			this.connect(this._showRelatedBtn, "onClick", function(){
				this._showRelated({
					show: this._showRelatedBtn.attr("pressed"),
					passive: false
				}); 
			});
			this._menu._contributionArea.appendChild(this._showRelatedBtn.domNode ); 
			setTimeout(dojo.hitch(this,function() {
				this._showRelated({show:true, passive: true}); 
			}),1);
		}));
*/
	},
	refresh: function() {
		this._currentGroup = null;
		this.clear();
		if (!this._timeout) this._timeout = setTimeout(dojo.hitch(this,this._fetchMenuContents,true),0);
	},
	_refreshHandler: function(e) {
		dojo.stopEvent(e);
		this.refresh();
	},
	addLoading: function() {
		var loading = this.inherited(arguments);
		loading.containerNode.innerHTML = "<div>"+loading.containerNode.innerHTML+"</div>";
		return loading;
	},
	_fetchMenuContents: function(force) {
		if (!this._loading) {
			this._loading = this.addLoading();
		}
		delete this._timeout;
		if (!serviceDeferred || force) {
			serviceDeferred = DiscoveredService.loadDiscoveryServices(namespace.ju + "HomeMenu", HomeMenuSchema.discovery, true);
		}
		serviceDeferred.addCallback(dojo.hitch(this, function(services){
			if (!services || !services.HomeMenu || services.HomeMenu.length == 0){
				throw new Error("No HomeMenu entries found.");	//throw to the errback code
				return;
			}
			var abouts = applicationAbout.get();
			var dashboards = [], other = [];
			var publisherMap = {};
			dojo.forEach(services.HomeMenu, function(service){
				if (service.menuSection && service.menuSection != MENU_SECTION_PROJECT) return;
				var about = abouts[service.applicationAbout];
				if (about && about.identifier) {
					var count = publisherMap[about.identifier] || 0;
					publisherMap[about.identifier] = ++count;
				}
			});
			dojo.forEach(services.HomeMenu, function(service){
				var about = abouts[service.applicationAbout];
				if (!about || about.error) about = {title: service.contextRoot};
				
				switch (service.menuSection){
					case MENU_SECTION_DASHBOARD:
						service.title = "<b>"+messages.personalDashboards+"</b>";
						dashboards.push(service);
						break;
					case MENU_SECTION_ADMIN:
						//admin menuSection no longer supported
						break;
					default:
						var title = "<b>" + about.title + "</b>";
						var map = publisherMap[about.identifier];
						if ((!map || map > 1) && service.publisher) {
							title += " (" + service.publisher + ")"
						}
						service.title = title;
						other.push(service);
						break;
				}
			},this);

			dashboards.sort(serviceSorter);
			other.sort(serviceSorter);
			
			var deferreds = [];
			dojo.forEach(dashboards, function(d){
				var resp = this._addMenuGroup(d, false);
				if (resp) deferreds.push(resp);
			}, this);
			dojo.forEach(other, function(p){
				var resp = this._addMenuGroup(p, (p.contextRoot != net.jazz.ajax._viewletWebUIRoot));
				if (resp) deferreds.push(resp);
			}, this);
			
			// hardcode admin links because we know about them (from jazz.app.entryPoints)
			this._addAdminGroup();
			
			var requestCallback = dojo.hitch(this,function(res) {
            	for (i = 0; i < this._groups.length; i++){
					var group = this._groups[i];
					if (group && group.attr && group.attr('hide')) {
						this._groups[i].attr("hide", false);
					}	            		
            	}
				this.remove(this._loading);
    			delete this._loading;
            	return res;
            });
			
			if (deferreds.length > 0){
				var dl = new dojo.DeferredList(deferreds);
	            dl.addCallback(requestCallback);
			} else {
				//no deferreds - show all sections right away.
            	requestCallback(services);
			}
		}));
		
		serviceDeferred.addErrback(dojo.hitch(this, function(err){
			console.warn("[HomeMenu] Discovery service error: ", dojo.toJson(err));
			if (this._loading) {
				this.remove(this._loading);
				delete this._loading;
			}			
			var messageItem = new jazz.ui.internal.NodeMenuItem({
				"class": "message-item",
				wrapHint: WrapHints.NO_WRAP,
				focusable: false
			});	
			var arr = [messageItem];
			var node = messageItem.containerNode;
			if (err.dojoType && err.dojoType == "popupBlocked") {
				dojo.place(document.createTextNode(err.message), node, "only");
			} else if (err.status == 403) {
				node.innerHTML = messages.forbidden;
			} else {
				node.innerHTML = messages.discoveryError;
			}
			arr.push(this._getRefreshLink());
			for (var i = 0; i < arr.length; i++) {
				this.add(arr[i]);
			}
		})); 
	},
	_addAdminGroup: function() {
		var deferred = new dojo.Deferred();
		var state = this._getState("admin"),
			collapsed = ("collapsed" in state) ? state.collapsed : collapsed,
			title = "<b>" + messages.administration + "</b>";

		var group = new jazz.ui.internal.MenuItemGroup({
			hide: true,
			menu: this,
			groupTitle: title,
			collapsible: true,
			collapsed: collapsed,
			onToggle: dojo.hitch(this, function(collapsed) {
				this._updateState("admin", {
					collapsed: collapsed
				});
			})
		});
		this._groups.push(group);

		var d = jazz.app.entryPoints.get(["LifecycleProjectAdmin", "JazzTeamServerHome"]);
		d.addCallback(dojo.hitch(this,function(arr) {
			var items = [];
			if (arr[0]) items.push({label: messages.lpa, href: arr[0], icon: dojo.moduleUrl('jazz.app','graphics/HomeMenu/lpa_obj.gif').toString()});
			if (arr[1]) items.push({label: messages.jtsHome, href: arr[1], icon: dojo.moduleUrl('jazz.app','graphics/HomeMenu/admin_16.png').toString()});
			group.attr("items", items);
			this._updateGroups(group);
			deferred.callback({items: items, group: group});
		}));
		return deferred;
	},
	_addMenuGroup: function(service, collapsed) {
		var state = this._getState(service.contextRoot+"-"+service.menuSection),
			scope = ("scope" in state) ? state.scope : SCOPE_MY,
			collapsed = ("collapsed" in state) ? state.collapsed : collapsed,
			currentApp = service.contextRoot ? dojo.string.substitute(messages.currentAppUrl,[service.contextRoot]) : messages.currentApp,
			title = service.title;
		
		var group = new jazz.ui.internal.MenuItemGroup({
			hide: true,
			menu: this,
			groupTitle: title,
			collapsible: true,
			collapsed: collapsed,
			onToggle: dojo.hitch(this, function(collapsed) {
				this._updateState(service.contextRoot+"-"+service.menuSection, {
					collapsed: collapsed
				});
			}),
			currentMessage: currentApp
		});
		this._groups.push(group);
		
		if (collapsed) {
			group.attr("itemProvider", this._getItemProvider(service, group, scope == SCOPE_ALL, !collapsed));
		} else {
			var deferred = this._getItems(service, group, scope == SCOPE_ALL, !collapsed);
		}
		if(service.menuSection == MENU_SECTION_PROJECT) {
			var scoper = new jazz.app.internal.homeMenu.ProjectScoper({
				scope: scope,
				showMyProjects: dojo.hitch(this, function() {
					group.attr("itemProvider", this._getItemProvider(service, group, false, !collapsed));
					this._updateState(service.contextRoot+"-"+service.menuSection, {
						scope: SCOPE_MY
					})
				}),
				showAllProjects: dojo.hitch(this, function() {
					group.attr("itemProvider", this._getItemProvider(service, group, true, !collapsed));
					this._updateState(service.contextRoot+"-"+service.menuSection, {
						scope: SCOPE_ALL
					})
				})
			});
			this._scopers.push(scoper);
			var item = new jazz.ui.internal.NodeMenuItem({wrapHint: WrapHints.NO_WRAP, "class": "project-scoper-item", focusable: true, focus: function() {
				if (!this.domNode) return;
				try {
					switch(scoper.scope) {
						case SCOPE_ALL:
							scoper._myLink.focus();
							break;
						case SCOPE_MY:
							scoper._allLink.focus();
					}					
				} catch (e) {
					// eat the error - sometimes focus throws an exception
				}
			}});
			scoper.placeAt(item.containerNode);
			group.attr("toolbar", item);
		}
		return deferred;
	},
	_getState: function(key) {
		try {
			var string = localStorage["jazz-app-internal-HomeMenu"];
			if (string) {
				var homeMenuData = dojo.fromJson(string);
				return homeMenuData[key] || {};
			} else {
				return {};
			}	
		} catch (e) {
			return {};
		}

	},
	_updateState: function(key, newState) {
		try {
			var homeMenuData = dojo.fromJson(localStorage["jazz-app-internal-HomeMenu"]);
			if (!homeMenuData) homeMenuData = {};
			var state = homeMenuData[key];
			if (state) {
				dojo.mixin(state, newState);
			} else {
				homeMenuData[key] = newState;
			}
			localStorage["jazz-app-internal-HomeMenu"] = dojo.toJson(homeMenuData);
		} catch (e) {
			// localStorage not supported... ignore
		}
	},
	_updateGroups: function(group) {
		for (var i = 0; i < this._groups.length; i++) {
			if (group == this._groups[i]) {
				this._groupMap[i] = true;
			}
		}
		var moveLoading = false;
		for (var i = 0; i < this._groupMap.length; i++) {
			if (!this._groupMap[i]) {
				if (moveLoading) break;
				else return;
			}
			var group = this._groups[i];
			if (group && group.attr && group.attr('hide')) {
				moveLoading = true;
				group.attr('hide', false);
			}
		}
		if (this._loading) {
			this.add(this._loading);
			dojo.addClass(this._loading.domNode,"message-item border-loading");
		}
	},
	_getItems: function(service, group, all, passive, forProvider) {
		var deferred = new dojo.Deferred();
		var servUrl = service.service;
		if (all) servUrl += ((servUrl.indexOf("?") >= 0) ? "&" : "?") + "all=true";
		var serviceDef = XhrUtil.getXml(servUrl, {schema: HomeMenuSchema.menuService, passive: passive});
		serviceDef.addCallback(dojo.hitch(this, function (doc){
			var about = applicationAbout.get()[service.applicationAbout];
			if (doc) {
				var items = this._getMenuItems(doc.MenuItems, service, all);
			} else {
				items = this._getServerLogin(group, service);
			}
			if (!forProvider) {
				group.attr('items', items);
				this._updateGroups(group);
			}
			deferred.callback({items: items, group: group});			
			this.updateSelected();
		}));
		serviceDef.addErrback(dojo.hitch(this, function(err) {
			console.warn("[HomeMenu] Service error: ", err.responseText||err.message, err);
			var messageItem = new jazz.ui.internal.NodeMenuItem({
				"class": "message-item",
				wrapHint: WrapHints.NO_WRAP,
				focusable: false
			});	
			var arr = [messageItem];
			var node = messageItem.containerNode;
			if (err.dojoType && err.dojoType == "popupBlocked") {
				dojo.place(document.createTextNode(err.message), node, "only");
				arr.push(this._getServerLogin(group, service)[0]);
			} else if (err.status == 403) {
				group.attr("toolbar", null);
				var ioArgs = err.ioArgs;
				if (ioArgs) {
					var header = ioArgs.xhr.getResponseHeader(PROXY_HEADER);
				}
				if (header) {
					node.innerHTML = dojo.string.substitute(messages.notConfigured,[jazz.util.html.escape(service.contextRoot), messages.jtsSetup]);
				} else {
					node.innerHTML = messages.forbidden;
				}
			} else {
				node.innerHTML = messages.unreachable;
			}
			if (!forProvider) {
				group.attr("items", arr);
				this._updateGroups(group);	
			}
			deferred.callback({items: arr, group: group});
		}));
		return deferred;
	},
	_getItemProvider: function(service, group, all, passive) {
		return dojo.hitch(this, function() {
			return this._getItems(service, group, all, passive, true);
		});
	},
	_getMenuItems: function(modelItems, service, showAll) {
		var items = [], all = [], admin = [], addDebug = false;
		var search = dojo.queryToObject(location.search.substring(1));
		if (search["debugDojo"] || search["debug"]) {
			addDebug = true;
			for (var x in search) {
				if (search.hasOwnProperty(x) && x != "debugDojo" && x != "debug") {
					delete search[x];
				}
			}
		}
		dojo.forEach(modelItems, function(item){
			var href = item.link;
			if (addDebug) {
				// if we're in debug mode, preserve debug flag in home menu links
				var urlObj = new dojo._Url(href);
				if (urlObj.query) {
					dojo.mixin(search, dojo.queryToObject(urlObj.query));
					href = href.split("?")[0];
				} else if (urlObj.fragment) {
					href = href.split("#")[0];
				}
				href += "?" + dojo.objectToQuery(search);
				if (urlObj.fragment) href += "#" + urlObj.fragment;
			}
			var itemObj = { label: jazz.util.html.escape(item.title), href: href, icon: item.icon || service.defaultIcon };
			/*
			if (item.link && item.resource) {
				this._href2Resource[href] = item.resource;
			} 
			if (this._isRelated(href))	itemObj["class"] = "related";
			*/
			switch (item.usage) {
				case MENU_USAGE_ALL:
					all.push(itemObj);
					break;
				case MENU_USAGE_ADMIN:
					admin.push(itemObj);  
					break;
				default:
					items.push(itemObj);
					break;
			}
		}, this);
		if (items.length == 0 && service.menuSection == MENU_SECTION_PROJECT) {
			var messageItem = new jazz.ui.internal.NodeMenuItem({
				"class": "message-item",
				wrapHint: WrapHints.NO_WRAP,
				focusable: false
			});	
			messageItem.containerNode.innerHTML = showAll ? messages.noProjects : messages.notAMemberProjects;
			items.push(messageItem);
		}
		dojo.forEach(all, 	function(i){ items.push(i); });
		dojo.forEach(admin, function(i){ items.push(i); });
		return items;
	},
	_getServerLogin: function(group, service) {
		var node = dojo.create("div", {"class": "login-item"});
		var loginLink = jazz.util.internal.login.insertLoginLink(node, false);
		this.connect(loginLink, "onclick", function(e){
			dojo.stopEvent(e);
			var state = this._getState(service.contextRoot+"-"+service.menuSection),
				scope = ("scope" in state) ? state.scope : SCOPE_MY;
			group.attr("itemProvider", this._getItemProvider(service, group, scope == SCOPE_ALL, false));
		});
		return [ node ];
	},
	_getRefreshLink: function() {
		var messageItem = new jazz.ui.internal.NodeMenuItem({
			"class": "message-item",
			wrapHint: WrapHints.NO_WRAP,
			focusable: false
		});	
		var a = dojo.create("a", {
			href: "#",
			innerHTML: messages.tryAgain
		}, messageItem.containerNode, "only");
		this.connect(a, "onclick", this._refreshHandler);
		return messageItem;
	},
/*
	_isRelated : function (href){
		// we have to return true|false otherwise dojo.toggleClass gets confused
		var resource=this._href2Resource[href];
		if (href&&resource&&this._relatedResources&&this._relatedResources[resource])
			return true;
		else
			return false;
	},
	_showRelated: function(args){
		this._showRelatedBtn.attr("pressed", args.show);
		dojo.toggleClass(this.domNode, "showRelated", args.show);
		if (args.show) {
			this._refreshRelated(args.passive);
		}
	},
	_refreshRelated: function(passive) { 
		if (!dojo.hasClass(this.domNode, "showRelated")) return;

		if (passive === undefined) passive = true;

		var project=jazz.app.context.get().itemId;
		if (this._relatedResources && (this._currentProject == project)) return;
		this._currentProject=null;
		this._relatedResources=null;

		// load the related projects for this project
		this._showRelatedBtn.attr("imgSrc", RELATED_IMG_WAIT);
		var relatedDeferred = jazz.app.internal.lpa.getRelatedProjects(project, passive);
		var _this = this;		
		 
		relatedDeferred.addCallback(function(related){
			if (!related) {
				_this._showRelatedBtn.attr("pressed", false);
				dojo.toggleClass(_this.domNode, "showRelated", false);
			}
			else {
				_this._currentProject=project;
				_this._relatedResources={};
				dojo.forEach(related, function(r){
					_this._relatedResources[r] = true;
				});
				dojo.forEach(_this.getItems(), function(item){
					dojo.toggleClass(item.domNode, "related", _this._isRelated(item.href));
				});
			}
			_this._showRelatedBtn.attr("imgSrc", RELATED_IMG);
		});
		relatedDeferred.addErrback(function (error){
			console.warn("[HomeMenu] Service error: ", error.responseText||error.message, error);
			_this._showRelatedBtn.attr("pressed", false);
			dojo.toggleClass(_this.domNode, "showRelated", false);
			_this._showRelatedBtn.attr("imgSrc", RELATED_IMG);
		});
	},
*/
	getSelected: function() {
		var context = jazz.app.context.get(),
			items = this.getItems(),
			item, group;
		for (var i = 0; i < items.length; i++) {
			item = items[i];
			group = item.__menuItemGroup;
			if (context.dashboard) {
				var href = item.href;
				if (href && href == context.dashboard) {
					if (group) {
						if (this._currentGroup) this._currentGroup.setCurrent(false);
						this._currentGroup = group;
						group.setCurrent(true);
					}
					return item;
				}
			} else {
				var locHref = new dojo._Url(location.href),
					itemHref = new dojo._Url(item.href);
				if (item.label !== context.name && href !== locHref && (locHref.authority != itemHref.authority || locHref.path !== itemHref.path)) {
					continue;
				}
				if (group) {
					if (this._currentGroup) this._currentGroup.setCurrent(false);
					this._currentGroup = group;
					group.setCurrent(true);
				}
				return item;
			}
		}
		//this._refreshRelated();
	},
	uninitialize: function() {
		while(this._groups.length > 0) this._groups.pop().destroy();
		while(this._scopers.length > 0) this._scopers.pop().destroy();
	}
});

dojo.provide("jazz.app.internal.homeMenu.ProjectScoper");

var SCOPE_MY = "my",
	SCOPE_ALL = "all";
	
dojo.declare("jazz.app.internal.homeMenu.ProjectScoper",[dijit._Widget, dijit._Templated, dijit._Contained], {
	templateString: "<div class=\"text-actions\"><a waiRole=\"menuitemradio\" tabIndex=\"-1\" dojoattachpoint=\"_myLink\" dojoattachevent=\"onkeydown: _showMyProjects, onclick: _showMyProjects\" href=\"#\">" + messages.myProjects + "</a><a dojoattachpoint=\"_allLink\" waiRole=\"menuitemradio\" dojoattachevent=\"onkeydown: _showAllProjects, onclick: _showAllProjects\" href=\"#\" tabIndex=\"-1\">" + messages.allProjects + "</a>",
	scope: SCOPE_MY,
	_showMyProjects: function(e) {
		if (e.type == "keydown" && e.keyCode != dojo.keys.ENTER) {
			return;	
		}
		dojo.stopEvent(e);
		this.attr("scope",SCOPE_MY);
		this.showMyProjects();
		e.type == "keydown" && this._allLink.focus();
	},
	_showAllProjects: function(e) {
		if (e.type == "keydown" && e.keyCode != dojo.keys.ENTER) {
			return;	
		}
		dojo.stopEvent(e);
		this.attr("scope",SCOPE_ALL);
		this.showAllProjects();
		e.type == "keydown" && this._myLink.focus();
	},
	_setScopeAttr: function(scope) {
		this._myLink.setAttribute("aria-checked",scope == SCOPE_MY);
		this._allLink.setAttribute("aria-checked",scope == SCOPE_ALL);
		dojo.toggleClass(this._myLink, "selected", scope == SCOPE_MY);
		dojo.toggleClass(this._allLink, "selected", scope == SCOPE_ALL);
		this.scope = scope;
	}
});

function serviceSorter(a, b){
	if (!a || !a.title) return 1;
	else if (!b || !b.title) return -1;
	else if (a.contextRoot == net.jazz.ajax._viewletWebUIRoot) return -1;
	else if (b.contextRoot == net.jazz.ajax._viewletWebUIRoot) return 1;
	else if (a.title.toLowerCase() == b.title.toLowerCase()) return 0;
	return (a.title.toLowerCase() < b.title.toLowerCase()) ? -1 : 1;
}

var EVENTS_ROOT = "jazz.app.events.contextSwitcher";
dojo.setObject(EVENTS_ROOT, {deleteContext: EVENTS_ROOT + ".deleteContext", addContext: EVENTS_ROOT + ".addContext"});

})();