/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ensemble.internal.LayoutConverter");

dojo.require("jazz.ensemble.internal.Schema");
dojo.require("jazz.ensemble.internal.util.XmlBindingFactory");

(function() {

var Schema = jazz.ensemble.internal.Schema;
var XmlBindingFactory = jazz.ensemble.internal.util.XmlBindingFactory;

jazz.ensemble.internal.LayoutConverter = {
	
	/*
	 * Converts from one layout data format to another. Modifies the given
	 * DOM element and its children.
	 */
	convert: function(/* string */ oldType, /* string */ newType, /* Element */ layoutDataElem) {
		var layoutData = XmlBindingFactory.createBinding(layoutDataElem, Schema.COLUMN_LAYOUT_DATA);
		var columnCollection = layoutData.getColumns();
		var columns = columnCollection.getColumns();
		
		var oldNum = _numColumns(oldType);
		var newNum = _numColumns(newType);
		
		if (newNum > oldNum) {
			// added column(s); add empty ones
			var numNewCols = newNum - oldNum;
			for (var i=0;i<numNewCols;i++) {
				var column = columnCollection.createColumn();
				var widgetCollection = column.createWidgets();
				widgetCollection.setParseType("Collection");
				column.setWidgets(widgetCollection);
				columns.push(column);
			}
		}
		else {
			// removed column(s); re-distribute widgets into remaining columns
			var displacedWidgets = [];
			for (var i=newNum;i<oldNum;i++) {
				var widgetCollection = columns[i].getWidgets();
				var widgets = widgetCollection.getWidgets();
				displacedWidgets = displacedWidgets.concat(widgets);
			}
			for (var i=0;i<newNum;i++) {
				var widgetCollection = columns[i].getWidgets();
				var widgets = widgetCollection.getWidgets();
				for (var j=i;j<displacedWidgets.length;j+=newNum) {
					widgets.push(displacedWidgets[j]);
				}
				widgetCollection.setWidgets(widgets);
			}
			// remove extra columns
			columns.splice(newNum, oldNum - newNum);
		}
		// re-balance widths; use integers
		var width = Math.floor(100 / columns.length);
		var remainder = 100 - (width*columns.length);
		for (var i=0;i<columns.length;i++) {
			// distribute remainder over first column(s), e.g. 34, 33, 33.
			columns[i].setWidth(width + (i < remainder ? 1 : 0));
		}
		columnCollection.setColumns(columns);
	}
};

/*
 * Returns the number of columns in the given layout type.
 */
function _numColumns(/* string */ type) {
	switch (type) {
		case "jazz.ensemble.layout.1column":
			return 1;
		case "jazz.ensemble.layout.2column":
			return 2;
		case "jazz.ensemble.layout.3column":
			return 3;
	}
}

})();
