/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("jazz.app.internal.LoginLogoutArea");

dojo.require("dojo.cookie");
dojo.require("jazz.client.xhr");
dojo.require("jazz.app.auth");
dojo.require("jazz.app.events");
dojo.require("jazz.util.html");
dojo.require("jazz.app.internal.UserMenu");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("jazz.ui.MenuPopup");
dojo.requireLocalization("jazz.app.internal", "LoginLogoutMessages");

var PlatformUI = net.jazz.ajax.ui.PlatformUI;

dojo.declare("jazz.app.internal.LoginLogoutArea", [dijit._Widget, dijit._Templated], {
	
	templateString: "<span class=\"jazz-app-loginLogoutArea\"></span>",

	config: null,	//passed into constructor
	
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("jazz.app.internal", "LoginLogoutMessages");
		this._prevUser = {};
	},
	
	postCreate: function() {
		if (this.config && this.config.user) {
			this.setLoggedIn();
		} else {
			this.setLoggedOut();
		}
		this.subscribe(jazz.app.events.USER_CHANGE, this._onUserChange);
	},
	
	uninitialize: function() {
		this._resetState();
	},
	
	setLoggedIn: function() {
		this._setMode("logout");
	},
	
	setLoggedOut: function() {
		this._setMode("login");
	},
	
	_onUserChange: function(userId) {
		var config = this.config;
		if (config && config.user) {
			if (config.user.userId != userId) {
				location.reload();
			}
		}
	},
	
	_resetState: function() {
		var self = this;
		dojo.forEach(["_clickEvent", "_userCallbackHandler"], function(eventName){
			if(self[eventName]) {
				self.disconnect(self[eventName]);
				delete self[eventName];
			}
		});
		dojo.empty(this.domNode);
	},
	
	_setMode: function(/*String*/mode) {
		
		// check for guest mode
		if (dojo.isObject(this.config.user) && this.config.user.itemId === "_OIi3MCDNEdu1A6FdF3RHyQ") {
			mode = "login";
		}
		
		// bail out early if in an invalid state
		if(mode !== "login" && mode !== "logout") {
			throw new Error("Invalid mode: " + mode);
		}
		
		// reset widget's state
		this._resetState();
		
		// set the UI based on state
		if(mode === "login") {
			this._showLoginOrRegister();
		} else if(mode === "logout") {
			this._showLoggedInUserAndLogoutControl();
		}
	},
	
	_showLoggedInUserAndLogoutControl: function() {
		var config = this.config;
		if (config.user) {
			var name = config.user.name || config.user.userId;
			if (name.length >= 28) name = name.substring(0,28) + "...";
			name = jazz.util.html.escape(name);
		} else {
		 	name = "";
		}
		var a11y = dojo.hasClass(document.body, "dijit_a11y");
		var node = dojo.create("a", {href: "#", innerHTML: "<span class=\"user-name\">"+ (a11y ? (this.messages.userProfile + " - " + name) : name) + "</span><span class=\"sprite-image user-menu\"></span>"});
		new jazz.ui.MenuPopup({
			align: "right",
			menuProvider: dojo.hitch(this, this._menuProvider),
			injectCaret: true,
			title: this.messages.userProfile,
			destroyOnClose: true,
			group: "jazz-app-BannerButtons"
		},node).placeAt(this.domNode);
	},
	
	_showLoginOrRegister: function() {
		// create "Log In or Register" link
		var a = dojo.create("a",{
			href: "#",
			"class": "login",
			innerHTML: this.messages.loginOrRegister
		});
		// attach the handling event
		this._clickEvent = this.connect(a, "onclick", function(e) {
			dojo.stopEvent(e);
			jazz.app.auth.login();
		});
		dojo.place(a, this.domNode);		
	},
	_menuProvider: function() {
		var items = this._getDefaultMenuItems(),
			config = this.config;	
		if(config && config.user) {
			if (config.user.menuItemsProvider) {
				var contribItems = config.user.menuItemsProvider();
				items.push(""); //separator
				items = items.concat(contribItems);
				items.push(""); //separator
			}
			if (config.user.userSmallPreviewUrl) {
				var smallPreviewUrl = config.user.userSmallPreviewUrl; 
			}
		}
		var args = {
			items: items,
			config: config
		};
		if (!smallPreviewUrl) {
			// try to get it from the session.
			try {
				var session = dojo.getObject("com.ibm.team.repository.web.client.session");
				if (session) {
					var userUri = session.getAuthenticatedContributorUri();
					if (userUri) smallPreviewUrl = userUri + "/smallPreview";
				}
			} catch(e) {
				// do nothing - session is not initialized - don't show a preview
			}
		}
		if (smallPreviewUrl) {
			args.userSmallPreviewUrl = smallPreviewUrl;
		}
		return new jazz.app.internal.UserMenu(args);
	},
	_getDefaultMenuItems: function() {
		var config = this.config,
			session = dojo.getObject("com.ibm.team.repository.web.client.session");
			
		if (config && config.user) {
			var userProfileUrl = config.user.userProfileUrl,
				allProfilesUrl = config.user.allProfilesUrl,
				personalDashboardUrl = config.user.personalDashboardUrl;
		}
		
		if (session && !userProfileUrl) userProfileUrl = session.getUserProfileLink();
		if (!allProfilesUrl) allProfilesUrl = getJtsRoot() + "/admin#action=jazz.viewPage&id=com.ibm.team.repository.users";
		if (!personalDashboardUrl) personalDashboardUrl = getJtsRoot() + "/dashboards";

		var items = [];
		if (userProfileUrl) {
			items.push({
				label: this.messages.viewMyProfile,
				href: userProfileUrl
			});
		}
		var roles = dojo.getObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR_ROLES");
		var isAdmin = false;
		if (roles) {
			for (var i=0; i < roles.length; i++) {
				if (roles[i] === "JazzAdmins") {
					isAdmin = true;
					break;
				}
			}
		}
		if (userProfileUrl && allProfilesUrl && isAdmin) {
			items.push({
				label: this.messages.viewOtherProfiles,
				href: allProfilesUrl
			});
		}
		if (personalDashboardUrl) {
			items.push({
				label: this.messages.openPersonalDash,
				href: personalDashboardUrl
			});
		}
		return items;
	}
});

function getJtsRoot() {
	try {
		var session = dojo.getObject("com.ibm.team.repository.web.client.session");
		if (session) {
			var userUri = session.getAuthenticatedContributorUri();
		}
		if (userUri) {
			var jts = userUri.split("/users/");
			if (jts.length) {
				return jts[0];
			}
		}
	} catch(e) {
		// eat the error
	}
	return net.jazz.ajax._viewletServiceRoot;
}

})();