/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("jazz.app.internal.LoginWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.CheckBox");
dojo.require("dijit.form.TextBox");
dojo.require("jazz.app.internal.applicationAbout");

dojo.require("jazz.util.internal.login");

dojo.requireLocalization("jazz.app.internal", "LoginWidgetMessages");
		
var about = jazz.app.internal.applicationAbout.getCurrent() || {};
var JTS_IDENTIFIER = "http://jazz.net/application/jts";

dojo.declare("jazz.app.internal.LoginWidget", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("jazz.app", "internal/templates/LoginWidget.html"),
	_rationalLogo: dojo.moduleUrl("jazz.app", "graphics/LoginWidget/logo-rtl-sm.gif"),
	_ibmLogo: dojo.moduleUrl("jazz.app", "graphics/LoginWidget/logo-ibm-sm.gif"),
	requestUrl: null,
	widgetsInTemplate: true,
	
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("jazz.app.internal", "LoginWidgetMessages");
	},
	
	postCreate: function() {
		this.inherited(arguments);
		this._origWindowTitle = document.title;
		document.title = this.messages.loginTitle;
		// show saved user ID, if present
		var userIdCookieVal = jazz.util.internal.login.getUserIdCookie();
		this.userId.attr('value', userIdCookieVal || "");
		this.rememberUserId.attr('value', userIdCookieVal ? true : false);
		
		//propagate any hash to the form so it doesn't get lost
		this.loginForm.action += jazz.util.internal.login.getLocationHash();
		
		// show error message, if necessary
		if(this.state === "login-failed") {
			this.loginErrorDiv.style.visibility = "visible";
			this.loginErrorDiv.appendChild(document.createTextNode(this.messages.invalidCredentials));
		}
		var hostname = window.location.hostname,
			realm = net.jazz.ajax._oauthRealmName;
		if (about.identifier == JTS_IDENTIFIER) {
			if (realm) {
				var string = dojo.string.substitute(this.messages.serverInfoWithRealm, ["<span>"+hostname+"</span>", "<span>"+realm+"</span>"]);
			} else {
				string = dojo.string.substitute(this.messages.serverInfo, ["<span>"+hostname+"</span>", "<span>"+realm+"</span>"]);
			}
		} else {
			var appName = about.title,
				instanceName = about.instanceName;
			if (!appName) {
				appName = "";
			} else if (instanceName && instanceName != appName) {
				appName = appName + " (" + instanceName + ")";
			}	
			if (realm) {
				var string = dojo.string.substitute(this.messages.appInfoWithRealm, [appName ? "<span>"+appName+"</span>": "", "<span class=\"no-wrap\">"+hostname+"</span>", "<span>"+realm+"</span>"]);
			} else {
				string = dojo.string.substitute(this.messages.appInfo, [appName ? "<span>"+appName+"</span>" : "", "<span class=\"no-wrap\">"+hostname+"</span>", "<span>"+realm+"</span>"]);
			}
		}
		this._serverInfo.innerHTML = string;
		this.focusNode = (this.state === "login-failed" || !this.userId.attr('value')) ? this.userId.domNode : this.password.domNode;
	},
	doLogIn: function(e) {
		var cookieVal = this.rememberUserId.attr('value') ? this.userId.attr('value') : "";
		jazz.util.internal.login.setUserIdCookie(cookieVal);
		
		if(!this.onSubmit) {
			return;
		}
		
		dojo.stopEvent(e);
		
		// run the callback in the global scope so that it doesn't have access to internal variables
		dojo.hitch(window, this.onSubmit, this.userId.attr('value'), this.password.attr('value'))();
		document.title = this._origWindowTitle;
	}
});
})();