/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.internal.MenuItemGroup");

(function() {

var WrapHints = jazz.ui.MegaMenu.WrapHints;
var LAST_ITEM_CLASS = "jazz-ui-MegaMenu-collapsible-last";
var HIDE_CLASS = "jazz-ui-MenuItemGroup-hide";

dojo.declare("jazz.ui.internal.MenuItemGroup", [dijit._Widget], {
	
	groupTitle: 	/* String */ "",
	itemProvider: 	/* function */ null,
	toolbar:		/* widget */ null,
	items: 			/* Object (args) */ null,
	menu: 			/* jazz.app._ComponentMenu */ null,
	insertIndex: 	/* number */ null,
	noItems:		/* domNode|String */ null,
	wrapHint:		/* wrapHint */ null,
	collapsible:	/* boolean */ false,
	collapsed:		/* boolean */ true,
	currentMessage: /* string */ "",
	hide:			/* boolean */ false,
	
	// Internal:
	
	_titleItem: null,
	_loadingItem: null,
	_noItem: null,
	
	addItemByArgs: function(item, pos) {
		if (!this.items) this.attr("items",[]);
		var index = this._getInsertIndex();
		if (pos > this.items.length) pos = this.items.length;
		if (typeof pos == "number") {
			var insertIndex = index + pos;
		} else if (typeof pos == "string") {
			if (pos == "end") insertIndex = index + this.items.length;
			else if (pos == "start") insertIndex = index;
		} else {
			insertIndex = this._getInsertIndex() + this.items.length;
		}
		return this._addItem(item,insertIndex); 
	},
	remove: function(item) {
		if (item) {
			if (!this.items) return;
			for (var i = 0; i < this.items.length; i++) {
				if (this.items[i] == item) {
					this.items.splice(i, 1);
					break;
				}
			}
			return this.menu.remove(item);
		} else {
			this.attr("items", null);
			this._removeLoadingItem();
			this._removeNoItem();
			if (this._titleItem) this.menu.remove(this._titleItem);
		}
	},
	setCurrent: function (current) {
		dojo.toggleClass(this._titleItem.domNode, "current", current);
		if (current) {
			this._currentArrow = dojo.create("div", {"class": "current-icon", title: this.currentMessage}, this._titleItem.containerNode, "first");
		} else {
			this._titleItem.containerNode.removeChild(this._currentArrow);
			this._currentArrow = null;
		}
	},
	_setToolbarAttr: function(toolbar) {
		if (this.toolbar != toolbar) {
			this.menu.remove(this.toolbar);
			this.toolbar = null;
		}
		if (toolbar) {
			var index = this._getInsertIndex();
			this.toolbar = this.menu.add(toolbar, index);
			dojo.toggleClass(this.toolbar.domNode, HIDE_CLASS, this.collapsed || this.hide);
			this.toolbar.attr("focusable", !this.collapsed);
		}
	},
	_setGroupTitleAttr: function(title) {
		this.groupTitle = title;
		if (!this._titleItem) {
			var args = {
				insertIndex: this._getInsertIndex()
			}
			if (this.wrapHint){
				args.wrapHint = this.wrapHint
			}			
			if (this.collapsible) {
				title = "<div><span class=\"twistie\"></span><div class=\"twistie-title\">"+title+"</div>";
				args.focusable = true;
				this._titleItem = this.menu.addTitle(title, args);
				dojo.addClass(this._titleItem.domNode,"jazz-ui-MegaMenu-collapsible");
				this.connect(this._titleItem.domNode,"onmouseover",function() {
					dojo.addClass(this._titleItem.domNode, "hover");
				});
				this.connect(this._titleItem.domNode,"onmouseout",function() {
					dojo.removeClass(this._titleItem.domNode, "hover");
				});
				this.connect(this._titleItem.domNode,"onclick",function() {
					this.attr("collapsed", !this.collapsed);
				});
				this.connect(this._titleItem.domNode,"onkeydown",function(e) {
					if (e.keyCode == dojo.keys.ENTER || e.keyCode == dojo.keys.SPACE) {
						this.attr("collapsed", !this.collapsed);
						dojo.stopEvent(e);
					}
				});
			} else {
				this._titleItem = this.menu.addTitle(title, args);
			}
		} else {
			this._titleItem.attr("label", title);
		}
	},
	_setWrapHintAttr: function(hint) {
		this.wrapHint = hint;
		if (this._titleItem) {
			this._titleItem.wrapHint = hint;
		}
	},
	_setItemProviderAttr: function(provider) {
		this.itemProvider = provider;
		this.items && this._setItemsAttr();
		if (!this.collapsible || !this.collapsed) {
			var items = provider();
			if (items instanceof dojo.getObject("dojo.Deferred")) {
				items.addCallback(dojo.hitch(this,this._deferredCallback));
				if (!this._loadingItem) {
					var index = this._getInsertIndex()
					this._loadingItem = this.menu.addLoading(index++);
					this._addLast(index);
				}
			}
			this._deferred = items;
		}
	},
	_deferredCallback: function(items) {
		this._deferred = null;
		if(items && items.items) {
			this._setItemsAttr(items.items);
		} else {
			this._setItemsAttr(items);
		}
	},
	_setNoItemsAttr: function(noItem) {
		if (!noItem) return;
		this._removeLoadingItem();
		if (!this.items || this.items.length == 0) {
			if (this._noItem) this._removeNoItem();
			this._noItem = this.menu.add(noItem, this._getInsertIndex());
		}
	},
	getIndexInParent: function() {
		return this._titleItem.getIndexInParent();
	},
	_getInsertIndex: function() {
		return this.toolbar ? this.toolbar.getIndexInParent() + 1 : this._titleItem ? this._titleItem.getIndexInParent() + 1 : (this.insertIndex == null) ? "end" : this.insertIndex;
	},
	_setHideAttr: function(hide) {
		if (!this.collapsible || !this.collapsed) this._hideContents(hide);
		if (this._titleItem) dojo.toggleClass(this._titleItem.domNode, HIDE_CLASS, hide);
		this.hide = hide;
	},
	_hideContents: function(hide) {
		if (this.items) {
			for (var i = 0; i < this.items.length; i++) {
				var item = this.items[i];
				if (item.domNode) {
					dojo.toggleClass(item.domNode, HIDE_CLASS, hide);
				}
			}		
		}
		if (this._loadingItem) dojo.toggleClass(this._loadingItem.domNode, HIDE_CLASS, hide);
		if (this.toolbar) dojo.toggleClass(this.toolbar.domNode, HIDE_CLASS, hide);		
		if (this._last) dojo.toggleClass(this._last.domNode, HIDE_CLASS, hide);
	},
	_setItemsAttr: function(items) {
		var curItems = this.items;
		if (curItems && curItems.length > 0) {
			for (var i = 0; i < curItems.length; i++) {
				var item = this.menu.remove(curItems[i]);
				if (item && item.destroy) item.destroy();
			}
			if (this._last) this.menu.remove(this._last.domNode);
		}
		var index = this._getInsertIndex();
		if (items == null) {
			this.items = null;
		} else {
			this._removeLoadingItem();
			this._removeNoItem();
			this.items = [];
			for (var j = 0; j < items.length; j++) {
				items[j] = this._addItem(items[j], index++);
			}
			this._addLast(index++);
			return this.items;
		}
	},
	_addLast: function(index) {
		if (this.collapsible && !this._last) {
			this._last = this.menu.add(dojo.create("div"), {
				insertIndex: index,
				wrapHint: WrapHints.NO_WRAP
			});
			dojo.addClass(this._last.domNode, LAST_ITEM_CLASS);
			if (this.collapsible && this.collapsed || this.hide) {
				dojo.addClass(this._last.domNode, HIDE_CLASS);
			}
		}
	},
	_setCollapsedAttr: function(collapsed) {
		if (!collapsed) collapsed = false;
		dojo.toggleClass(this._titleItem.domNode, "collapsed", collapsed);
		dijit.setWaiState(this._titleItem.focusNode, "expanded", !collapsed);
		if (this.collapsed === collapsed) return;
		this.collapsed = collapsed;
		this._hideContents(collapsed);
		if (this.toolbar) this.toolbar.attr("focusable", !collapsed);
		if (collapsed) {
			if (this._deferred && !this._loadingItem) {
				this._loadingItem = this.menu.addLoading(this._getInsertIndex());
			}
		} else {
			if (this._loadingItem) {
				this._removeLoadingItem();
			} else if (this.itemProvider) {
				this._setItemProviderAttr(this.itemProvider);
			}
		}
		this.menu._queueReflow && this.menu._queueReflow();
		this.onToggle&&this.onToggle(collapsed);
	},
	_addItem: function(item, index){
		if (!this.items) this.items = [];
		var insertIndex = this._getInsertIndex();
		if (index - insertIndex <= 2 && !item.wrapHint) {
			item.wrapHint = WrapHints.NO_WRAP;
		} 
		if (item.nodeType || item.domNode) {
			item = this.menu.add(item, {insertIndex: index, iconIndent: false});
			dojo.addClass(item.domNode, "no-icon");
		} else {
			item = this.menu.addItemByArgs(item, index);
		}
		if (this.collapsible && this.collapsed || this.hide) {
			dojo.addClass(item.domNode, HIDE_CLASS);
		}
		item.__menuItemGroup = this;
		this.items.splice(index,0,item);
		return item;
	},
	_removeLoadingItem: function() {
		if (this._loadingItem) {
			this.menu.remove(this._loadingItem).destroy();
			this._loadingItem = null;
		}
	},
	_removeNoItem: function() {
		if (this._noItem) {
			this.menu.remove(this._noItem).destroy();
			this._noItem = null;
		}
	},
	postCreate: function() { 
		if (!this.items) this.attr("items", null);
	}
});
})();