/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.app.internal.ProgressIndicator");

dojo.require("dijit._Widget");
dojo.require("jazz.util.ResizeNotifier");
dojo.require("jazz.util.html");

dojo.require("net.jazz.ajax.core.internal.util");

dojo.declare("jazz.app.internal.ProgressIndicator", dijit._Widget, {
	
	// summary:
	//		a widget for displaying progress messages
	//	description:
	//		This widget contains no public methods; simply instantiate it and
	//		place it within a Jazz Framework-based application. To style it,
	//		define a class with the selector "net-jazz-ajax-ProgressIndicator"
	//		and add	your desired rules.
	
	//
	// Internal methods; do not call directly
	//
	
	visible: false,
	
	postCreate: function() {
		dojo.deprecated("jazz.app.internal.ProgressIndicator is deprecated.", "Use built-in feature of jazz.ui.PageTemplate instead.", "3.0.1");
		
		dojo.addClass(this.domNode, "jazz-app-internal-ProgressIndicator");
		// subscribe to relevant events
		dojo.subscribe(net.jazz.ajax.internal.events.showMessage, this, "_showMessage");
		dojo.subscribe(net.jazz.ajax.internal.events.hideMessageArea, this, "_hideMessageArea");
		// create animations for fade-in and fade-out
		var args = {node: this.domNode, duration: 250};
		this.fadeInAnimation = dojo.fadeIn(args);
		this.fadeOutAnimation = dojo.fadeOut(args);
		
		this.connect(this.fadeOutAnimation,"onEnd",function() {
			this.domNode.style.display = "none";
		});
		
		// hide the node before first use
		this.domNode.style.display = "none";
	},
	updatePosition: function() {
		this.domNode.style.top = "";
		var contentArea = dojo.getObject("jazz.app.currentApplication.ui._content");
		var width = contentArea ? dojo.coords(contentArea).w : 0;
		var coords = dojo.coords(this.domNode);
		if (coords.y <= 3) {
			var scrollTop = document.documentElement.scrollTop;
			this.domNode.style.top = ((scrollTop ? scrollTop : document.body.scrollTop) + 2) + "px";
		}
		this.domNode.style.right = width ? (width / 2 + 18 - coords.w / 2) + "px" : "";
	},
	_showMessage: function(/*String*/ msg) {
		this.domNode.innerHTML = "";
		this.domNode.appendChild(document.createTextNode(msg + "..."));
		if (!this.visible) {
			if (this.fadeOutAnimation.status() == "playing") {
				this.fadeOutAnimation.stop();
			}
			this.domNode.style.display = "block";
			this.fadeInAnimation.play();
			if (!this._onscroll) this._onscroll = this.connect(window,"onscroll",this.updatePosition);	
			if (!this._onresize) this._onresize = jazz.util.ResizeNotifier.register({context: this, callback: this.updatePosition, force: true});
			this.visible = true;
		}
		this.updatePosition();
	},
	_hideMessageArea: function() {
		if(this.visible){
			if (this.fadeInAnimation.status() == "playing") {
				this.fadeInAnimation.stop();
			}
			this.fadeOutAnimation.play();
			this.disconnect(this._onscroll);
			jazz.util.ResizeNotifier.unregister(this._onresize);
			this._onscroll = this._onresize = null;
			this.visible = false;
		}
	},
	
	destroy: function() {
		if (this._onresize) jazz.util.ResizeNotifier.unregister(this._onresize);
		this.inherited(arguments);
	}
});