/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
dojo.provide("jazz.ui.internal.ResourceLinkAggregator");

dojo.require("jazz.client.xhr")

var uniqueId = 1;

jazz.ui.internal.resourceLinkAggreagtor = function () {
	if (!jazz.ui.internal._resourceLinkAggregator) {
		jazz.ui.internal._resourceLinkAggregator = new jazz.ui.internal.ResourceLinkAggregator();
	}
	return jazz.ui.internal._resourceLinkAggregator;
}

dojo.declare("jazz.ui.internal.ResourceLinkAggregator", null, {
	
	constructor: function () {
		this.reset();
		this.resetOauth();
	},
	
	reset: function () {
		this._resourceLinks = {};
		this._timeoutId = null;
	},
	
	resetOauth: function (token) {
		if (token && this._oauthRLs) {
			this._oauthRLs[token] = [];
		} else {
			this._oauthRLs = {};
		}
	},
	
	/**
	 * A ResourceLink uses this method to register itself with the ResourceLinkAggregator.  The
	 * ResourceLinkAggregator collects information about all of the ResourceLinks that register,
	 * and it sends a batch request for compact rendering information after the page finishes
	 * loading.
	 * 
	 * @param {String} uri - The resource URI that is associated with the ResourceLink being registered
	 * @param {Function} callback - This function is called after a successful response
	 * @param {Function} errback - This function is called if there is an error response
	 */
	register: function (uri, callback, errback, id) {
		id = id || uniqueId++;
		if (this._requestPending) {
			if (!this._registerQueue)
				this._registerQueue = [];
			this._registerQueue.push({uri: uri, callback: callback, errback: errback, id: id});
		} else {
			if (!this._resourceLinks[uri]) {
				this._resourceLinks[uri] = [];
			}
			this._resourceLinks[uri].push({
				callback: callback,
				errback: errback,
				id: id
			});
			
			if (!this._timeoutId) {
				// yield until the page finishes loading, allowing all the ResourceLinks on the
				// page to register before sending the batch request.
				this._timeoutId = setTimeout(dojo.hitch(this, this.sendRequest), 0);
			}
		}
		return id;
	},
	
	registerOauth: function(token, uri, callback, errback) {
		var rlObj = {
			callback: callback,
			errback: errback,
			id: uniqueId++
		};
		this._handleRegisterOauth(token, uri, rlObj);
		return rlObj.id;
	},
	
	unregisterOauth: function(token, uri, id) {
		if (this._oauthRLs[token] && this._oauthRLs[token][uri]) {
			arr = this._oauthRLs[token][uri];
			for (var i = 0; i < arr.length; i++) {
				if (arr[i].id === id) {
					return arr.splice(i,1)[0];
				}
			}
		}
		return false;
	},
	
	_handleRegisterOauth: function(token, uri, rlObj) {
		if (this._oauthRLs[token] == null) {
			this._oauthRLs[token] = {};
		}
		if (this._oauthRLs[token][uri] == null) {
			this._oauthRLs[token][uri] = [];
		}
		this._oauthRLs[token][uri].push(rlObj);
	},
		
	sendRequest: function () {
		this._requestPending = true;
		var uris = [];
		for (var i in this._resourceLinks) {
			uris.push(i);
		}
		
		var dfd = jazz.client.xhrPost({
			url: net.jazz.ajax._contextRoot + "/_compactRendering",
			handleAs: "json",
			content: {
				uri: uris
			}
		});
		dfd.addCallback(dojo.hitch(this, this.handleResponse, null));
		dfd.addErrback(dojo.hitch(this, this.handleError, null));
	},
	
	handleResponse: function (token, resp) {
		var rlObj = token ? this._oauthRLs[token] : this._resourceLinks;
		for (var uri in rlObj) {
			if (resp[uri]) {
				for (var i = 0; i < rlObj[uri].length; i++) {
					if (resp[uri]._err) {
						rlObj[uri][i].errback(resp[uri]._err);
					}
					else {
						if (resp[uri].oauthRequired) {
							if (token)
								var authFailed = true;
							else
								this._handleRegisterOauth(resp[uri].token, uri, rlObj[uri][i]);
						}
						rlObj[uri][i].callback(resp[uri]);
					}
				}
			}
		}
		// reset the status of the aggregator after sending back all the responses
		if (!token) {
			this.reset();
		} else if (!authFailed) {
			// only clear the oauth state for the token if we aren't still getting challenges (e.g. auth was successful)
			this.resetOauth(token);
		}
		this._requestPending = false;
		if (this._registerQueue) {
			for (var i = 0; i < this._registerQueue.length; i++) {
				this.register(this._registerQueue[i].uri, this._registerQueue[i].callback, this._registerQueue[i].errback, this._registerQueue[i].id);
			}
		}
		delete this._registerQueue;
	},
	
	handleError: function (token, err) {
		for (var uri in this._resourceLinks) {
			for (var i=0; i < this._resourceLinks[uri].length; i++) {
				this._resourceLinks[uri][i].errback(err);
			}
		}
		// reset the status of the aggregator after sending back all the responses
		if (token) {
			this.resetOauth(token);
		} else {
			this.reset();
		}
	},
	
	resendOauth: function (token) {
		var uris = [];
		if (this._oauthRLs[token]) {
			for (var i in this._oauthRLs[token]) {
				uris.push(i);
			}
			var dfd = jazz.client.xhrPost({
				url: net.jazz.ajax._contextRoot + "/_compactRendering",
				handleAs: "json",
				content: {
					uri: uris
				}
			});
			dfd.addCallback(dojo.hitch(this, this.handleResponse, token));
			dfd.addErrback(dojo.hitch(this, this.handleError, token));
		}
	}
});
})();
