/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ensemble.internal.Schema");

dojo.require("jazz.ensemble.internal.util.Xml");
dojo.require("jazz.ensemble.internal.util.XmlBindingFactory");

(function() {

var Xml = jazz.ensemble.internal.util.Xml;
var XmlBindingFactory = jazz.ensemble.internal.util.XmlBindingFactory;

var NS = {
	JW: "http://jazz.net/xmlns/prod/jazz/widget/1.0/"
};

var WIDGET = [
	{
		get: "getId",
		set: "setId",
		type: XmlBindingFactory.STRING,
		nodeType: XmlBindingFactory.ATTRIBUTE,
		attrName: "nodeID",
		attrNS: Xml.namespace.rdf
	},
	{
		get: "getTitle",
		set: "setTitle",
		type: XmlBindingFactory.STRING,
		nodeType: XmlBindingFactory.CHILD_ELEMENT_TEXT,
		elemName: "title",
		elemNS: Xml.namespace.dublin
	},
	{
		get: "isCustomTitle",
		set: "setCustomTitle",
		type: XmlBindingFactory.BOOLEAN,
		nodeType: XmlBindingFactory.CHILD_ELEMENT_TEXT,
		elemName: "customTitle",
		elemNS: NS.JW
	},
	{
		get: "getType",
		set: "setType",
		type: XmlBindingFactory.STRING,
		nodeType: XmlBindingFactory.CHILD_ELEMENT_TEXT,
		elemName: "type",
		elemNS: Xml.namespace.dublin
	},
	{
		get: "getDefinition",
		set: "setDefinition",
		type: XmlBindingFactory.STRING,
		nodeType: XmlBindingFactory.CHILD_ELEMENT_ATTRIBUTE,
		elemName: "definition",
		elemNS: NS.JW,
		attrName: "resource",
		attrNS: Xml.namespace.rdf
	},
	{
		get: "getIcon",
		set: "setIcon",
		type: XmlBindingFactory.STRING,
		nodeType: XmlBindingFactory.CHILD_ELEMENT_ATTRIBUTE,
		elemName: "icon",
		elemNS: NS.JW,
		attrName: "resource",
		attrNS: Xml.namespace.rdf
	},
	{
		get: "getPreferences",
		set: "setPreferences",
		create: "createPreferences",
		type: XmlBindingFactory.OBJECT,
		nodeType: XmlBindingFactory.CHILD_ELEMENT,
		elemName: "preferences",
		elemNS: NS.JW,
		schema: [
			{
				set: "setParseType",
				type: XmlBindingFactory.STRING,
				nodeType: XmlBindingFactory.ATTRIBUTE,
				attrName: "parseType",
				attrNS: Xml.namespace.rdf
			},
			{
				get: "getPreferences",
				set: "setPreferences",
				create: "createPreference",
				type: XmlBindingFactory.OBJECT,
				nodeType: XmlBindingFactory.CHILD_ELEMENT,
				elemName: "Preference",
				elemNS: NS.JW,
				multiValued: true,
				schema: [
					{
						get: "getName",
						set: "setName",
						type: XmlBindingFactory.STRING,
						nodeType: XmlBindingFactory.CHILD_ELEMENT_TEXT,
						elemName: "name",
						elemNS: NS.JW
					},
					{
						get: "getValue",
						set: "setValue",
						type: XmlBindingFactory.STRING,
						nodeType: XmlBindingFactory.CHILD_ELEMENT_TEXT,
						elemName: "value",
						elemNS: NS.JW
					}
				]
			}
		]
	},
	{
		get: "getMemento",
		set: "setMemento",
		type: XmlBindingFactory.STRING,
		nodeType: XmlBindingFactory.CHILD_ELEMENT_TEXT,
		elemName: "memento",
		elemNS: NS.JW
	},
	{
		get: "getShowBackground",
		set: "setShowBackground",
		type: XmlBindingFactory.BOOLEAN,
		nodeType: XmlBindingFactory.CHILD_ELEMENT_TEXT,
		elemName: "showBackground",
		elemNS: NS.JW
	},
	{
		get: "getTrim",
		set: "setTrim",
		type: XmlBindingFactory.STRING,
		nodeType: XmlBindingFactory.CHILD_ELEMENT_TEXT,
		elemName: "trim",
		elemNS: NS.JW
	},
	{
		get: "isCollapsed",
		set: "setCollapsed",
		type: XmlBindingFactory.BOOLEAN,
		nodeType: XmlBindingFactory.CHILD_ELEMENT_TEXT,
		elemName: "collapsed",
		elemNS: NS.JW
	}
];

var WIDGET_PANEL = [
	{
		// widgets property
		get: "getWidgets",
		set: "setWidgets",
		create: "createWidgets",
		type: XmlBindingFactory.OBJECT,
		nodeType: XmlBindingFactory.CHILD_ELEMENT,
		elemName: "widgets",
		elemNS: NS.JW,
		schema: [
			{
				set: "setParseType",
				type: XmlBindingFactory.STRING,
				nodeType: XmlBindingFactory.ATTRIBUTE,
				attrName: "parseType",
				attrNS: Xml.namespace.rdf
			},
			{
				// widget resources
				get: "getWidgets",
				set: "setWidgets",
				create: "createWidget",
				type: XmlBindingFactory.OBJECT,
				nodeType: XmlBindingFactory.CHILD_ELEMENT,
				elemName: "Widget",
				elemNS: NS.JW,
				multiValued: true,
				schema: WIDGET
			}
		]
	},
	{
		// layout property
		get: "getLayout",
		set: "setLayout",
		create: "createLayout",
		type: XmlBindingFactory.OBJECT,
		nodeType: XmlBindingFactory.CHILD_ELEMENT,
		elemName: "layout",
		elemNS: NS.JW,
		schema: [
			{
				// layout resource
				get: "getLayout",
				set: "setLayout",
				create: "createLayout",
				type: XmlBindingFactory.OBJECT,
				nodeType: XmlBindingFactory.CHILD_ELEMENT,
				elemName: "Layout",
				elemNS: NS.JW,
				schema: [
					{
						get: "getType",
						set: "setType",
						type: XmlBindingFactory.STRING,
						nodeType: XmlBindingFactory.CHILD_ELEMENT_TEXT,
						elemName: "type",
						elemNS: Xml.namespace.dublin
					},
					{
						// data property
						get: "getData",
						set: "setData",
						create: "createData",
						type: XmlBindingFactory.OBJECT,
						nodeType: XmlBindingFactory.CHILD_ELEMENT,
						elemName: "data",
						elemNS: NS.JW,
						chain: true,
						schema: [
							{
								/* Layout data schema depends on layout type - use late binding */
								get: "getLayoutData",
								set: "setLayoutData",
								create: "createLayoutData",
								type: XmlBindingFactory.NODE,
								nodeType: XmlBindingFactory.CHILD_ELEMENT,
								elemName: "LayoutData",
								elemNS: NS.JW
							}
						]
					}
				]
			}
		]
	}
];

var COLUMN_LAYOUT_DATA = [
	{
		get: "getColumns",
		set: "setColumns",
		create: "createColumns",
		type: XmlBindingFactory.OBJECT,
		nodeType: XmlBindingFactory.CHILD_ELEMENT,
		elemName: "columns",
		elemNS: NS.JW,
		schema: [
			{
				set: "setParseType",
				type: XmlBindingFactory.STRING,
				nodeType: XmlBindingFactory.ATTRIBUTE,
				attrName: "parseType",
				attrNS: Xml.namespace.rdf
			},
			{
				get: "getColumns",
				set: "setColumns",
				create: "createColumn",
				type: XmlBindingFactory.OBJECT,
				nodeType: XmlBindingFactory.CHILD_ELEMENT,
				elemName: "Column",
				elemNS: NS.JW,
				multiValued: true,
				schema: [
					{
						get: "getWidth",
						set: "setWidth",
						type: XmlBindingFactory.NUMBER,
						nodeType: XmlBindingFactory.CHILD_ELEMENT_TEXT,
						elemName: "width",
						elemNS: NS.JW
					},
					{
						get: "getWidgets",
						set: "setWidgets",
						create: "createWidgets",
						type: XmlBindingFactory.OBJECT,
						nodeType: XmlBindingFactory.CHILD_ELEMENT,
						elemName: "widgets",
						elemNS: NS.JW,
						schema: [
							{
								set: "setParseType",
								type: XmlBindingFactory.STRING,
								nodeType: XmlBindingFactory.ATTRIBUTE,
								attrName: "parseType",
								attrNS: Xml.namespace.rdf
							},
							{
								get: "getWidgets",
								set: "setWidgets",
								type: XmlBindingFactory.STRING,
								nodeType: XmlBindingFactory.CHILD_ELEMENT_ATTRIBUTE,
								attrName: "nodeID",
								attrNS: Xml.namespace.rdf,
								elemName: "Widget",
								elemNS: NS.JW,
								multiValued: true
							}
						]
					}
				]
			}
		]
	}
];

jazz.ensemble.internal.Schema = {
	WIDGET: WIDGET,
	WIDGET_PANEL: WIDGET_PANEL,
	COLUMN_LAYOUT_DATA: COLUMN_LAYOUT_DATA	
};

})();