/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.internal.Select");

dojo.require("jazz.ui.Menu");
dojo.require("jazz.ui.toolbar.MenuButton");

(function() {

/**
 * Constructs a new Select widget.
 * @name jazz.ui.internal.Select
 * @class <p>A rich select widget for web UIs.</p>
 * 	<p>PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * 	future with minimal or no warning.</p>
 * @constructor
 * @extends dijit._Widget
 * @extends dijit._Templated
 */
dojo.declare("jazz.ui.internal.Select", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("jazz.ui", "internal/templates/Select.html"),

	/**
	 * A reference to the object that is currently selected.
	 * @field
	 * @type Object
	 */
	selected: null,
	
	/**
	 * The default entry to be displayed if nothing is selected.
	 * @field
	 * @type Object
	 * @required
	 */
	defaultEntry: null,

	/**
	 * The menu items to display.  This should consist of an object where the
	 * keys are the titles of the fields and the values are arrays of objects,
	 * where each object should have a 'label' field and optionally an 'icon'
	 * field.
	 * @field
	 * @type Object
	 * @required
	 */
	menuItems: null,
	
	/**
	 * Callback which will be called with the newly selected value whenever the
	 * currently selected value changes.
	 * @field
	 * @type Function
	 */
	onChange: null,

	widgetsInTemplate: true,
	
	postCreate: function() {
		this._opened = false;
		this._menuButton.menuProvider = dojo.hitch(this, this._createMenu);
		this._menuButton.around = this.domNode;
		this.setSelected(this.defaultEntry);
		this.connect(this._selectLeft, "onclick", function(event) {
			this._menuButton.open();
			this._menuButton.focus();
		});
		this.connect(this.domNode, "ondragstart", dojo.stopEvent);
		this.connect(this.domNode, "onselectstart", dojo.stopEvent);
		this.connect(this.domNode, "onmouseover", dojo.hitch(this, this._toggleHoverBorder, true));
		this.connect(this.domNode, "onmouseout", dojo.hitch(this, this._toggleHoverBorder, false));
	},

	_toggleHoverBorder: function(active) {
		dojo.toggleClass(this.domNode, 'active', active || this._opened);
	},

	_addItemByArgs: function(menu, args, insertIndex) {
		var item = new jazz.ui.internal.ImgMenuItem(args);
		menu.addChild(item, {
			insertIndex: insertIndex,
			wrapHint: args.wrapHint
		});
		return item;
	},

	_addTitle: function(menu, title, args) {
		if (typeof args == "number") args = { insertIndex: args };
		var item = new jazz.ui.internal.NodeMenuItem({
			'label': title,
			'isTitle': true,
			'focusable': args && args.focusable
		});
		menu.addChild(item, args);
		return item;
	},
	
	_onMenuOpen: function() {
		this._opened = true;
		this._toggleHoverBorder(true);
		this._menu.domNode.style.width = "";
		var menuWidth = dojo.position(this._menu.domNode).w;
		var thisWidth = dojo.position(this.domNode).w;
		if (menuWidth < thisWidth) {
			dojo.style(this._menu.domNode, 'width', thisWidth + 'px');
		}
		if (this.selected && this.selected.item) {
			this._menu.setSelected(this.selected.item);
		}
	},

	_onMenuClose: function() {
		this._opened = false;
		this._toggleHoverBorder(false);
	},

	_createMenu: function() {
		if (!this._menu) {
			this._menu = new jazz.ui.Menu({
				selectable: true,
				onOpen: dojo.hitch(this, this._onMenuOpen),
				onClose: dojo.hitch(this, this._onMenuClose)
			});
			for (var title in this.menuItems) {
				var titleMenu = this._addTitle(this._menu, title);
				for (var elt in this.menuItems[title]) {
					// Need to use a closure here to be able to pass the
					// correct item to each callback
					var entry = this.menuItems[title][elt];
					entry.onClick = dojo.hitch(this, function(entry) {
						this._wasClicked(entry);
					}, entry);
					entry.item = this._addItemByArgs(this._menu, entry, elt);
				}
			}
		}
		return this._menu;
	},

	setSelected: function(entry) {
		if (!entry) return;
		this.selected = entry;
		this._selectLeft.innerHTML = entry.label;
		if ('icon' in entry) {
			dojo.create('img', {
				'src': entry.icon,
				'class': 'current-icon'
			}, this._selectLeft, 'first');
		}
		if (entry.item) {
			this._menu.setSelected(entry.item);
		}
	},

	_wasClicked: function(entry) {
		this.setSelected(entry);
		if (this.onChange) {
			this.onChange(entry);
		}
	},

	destroy: function() {
		if (this._menu) this._menu.destroy();
		this._menuButton.destroy();
		this.inherited(arguments);
	}

});

})();
