/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.internal.SwapList");

/*
 * Imports 
 */
dojo.require("dojo.i18n");  
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.ui.ListBox");
dojo.require("jazz.ui.FilterBox");

dojo.requireLocalization("jazz.ui.internal", "SwapListMessages");

/*
 * TODOs 
 */

(function() {

/*
 * Aliases 
 */
var ListBox = jazz.ui.ListBox;

/*
 * Class 
 *
 * PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * future with minimal or no warning.
 */
dojo.declare("jazz.ui.internal.SwapList",
	[dijit._Widget, dijit._Templated], {

	/*
	 * Properties
	 */	
	unselectedListTitleText: "",
	selectedListTitleText: "",
	listBoxWidth: "250px",
	listBoxHeight: "250px",

	/*
	 * Specify the entry field to use when sorting list box entries
	 * Options: "label", "value", "id", "type"
	 */		
	sortField: "label",	
	
	/*
	 * If true, the selected list will not be automatically sorted and there will
	 * be move up/down buttons to allow the user to reorder it.
	 */
	sortSelected: false,	

	/*  
 	 * Specify icons for selected (left) and unselected (right) bucket 
 	 * entries by providing a map of entry types associated with icon pathes.
 	 * 
 	 * Array iconMap : [Number entryType] = String iconPath
 	 */	
	iconMap: [],

	/*
	 * Internal
	 */
	_FILTERBOX_HEIGHT: 20,

	/*
	 * Templates
	 */	
	templatePath: dojo.moduleUrl("jazz.ui", "internal/templates/SwapList.html"),

	/*
 	 * Constructor
 	 * 
 	 * @param Object args
 	 * 
 	 * @return SwapList
 	 */
	constructor: function(args) {
		
		// Messages //------------------------------------------------------------->
		
		this.messages = dojo.i18n.getLocalization("jazz.ui.internal", "SwapListMessages");
		this.unselectedListTitleText = this.messages.msgAllItems;
		this.selectedListTitleText = this.messages.msgSelectedItems;
		this.msgAdd = this.messages.msgAdd;
		this.msgAddAll = this.messages.msgAddAll;
		this.msgRemove = this.messages.msgRemove;
		this.msgRemoveAll = this.messages.msgRemoveAll;
		this.msgMoveUp = this.messages.msgMoveUp;
		this.msgMoveDown = this.messages.msgMoveDown;
		this.msgOutOf = this.messages.msgOutOf;
		
	},
	
	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 * @overrides super
 	 */	
	postCreate: function() {
		// Call super
		this.inherited(arguments);
		
		// Instance Variables //----------------------------------------------->
		this._selectedList = null;
		this._selectedOrder = {};
		this._unselectedList = null;

		this.configureUI();
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	configureUI: function() {
		
		// Unselected list title
		this._unselectedListTitle.innerHTML = this.unselectedListTitleText; 
		// Selected list title
		this._selectedListTitle.innerHTML = this.selectedListTitleText;
		
		var self = this;
		
		// Unselected list box
		this._unselectedList = new ListBox({
			height: this.listBoxHeight,
			width: this.listBoxWidth,
			margin: "0px",
			sortField: this.sortField,
			iconMap: this.iconMap,
			ondblClick: function(entry) {
				self._handleEventDoubleClickedUnselected(entry);
			}
		});
		this._unselectedList.domNode.style.borderTop = "0";
		this._unselectedFilterBox = new jazz.ui.FilterBox({
			onChange: dojo.hitch(this, this._filterBoxChange, this._unselectedList)
		});
		this._unselectedFilterBox.domNode.style.width = this.listBoxWidth;
		this._unselectedFilterBox.domNode.style.height = String(this._FILTERBOX_HEIGHT - 1) + "px";
		this._unselectedFilterBox.domNode.style.border = "1px solid #7F9DB9";
		this._unselectedListWidget.appendChild(this._unselectedFilterBox.domNode);
		this._unselectedListWidget.appendChild(this._unselectedList.domNode);
		
		// Selected list box		
		this._selectedList = new ListBox({
			height: this.listBoxHeight,
			width: this.listBoxWidth,
			margin: "0px",
			sortField: this.sortField,
			sortOrder: this.sortSelected ? 2 : 0,
			iconMap: this.iconMap,
			onClick: dojo.hitch(this,this._updateMoveButtons),
			ondblClick: function(entry) {
				self._handleEventDoubleClickedSelected(entry);
			}
		});
		this._selectedList.domNode.style.borderTop = "0";
		this._selectedFilterBox = new jazz.ui.FilterBox({
			onChange: dojo.hitch(this, this._filterBoxChange, this._selectedList)
		});
		this._selectedFilterBox.domNode.style.width = this.listBoxWidth;
		this._selectedFilterBox.domNode.style.height = String(this._FILTERBOX_HEIGHT - 1) + "px";
		this._selectedFilterBox.domNode.style.border = "1px solid #7F9DB9";
		this._selectedListWidget.appendChild(this._selectedFilterBox.domNode);
		this._selectedListWidget.appendChild(this._selectedList.domNode);

		// Add button
		this.connect(this._addButton, "onclick", this._handleEventAddButtonClicked);
		// Add all button
		this.connect(this._addAllButton, "onclick", this._handleEventAddAllButtonClicked);
		// Remove button
		this.connect(this._removeButton, "onclick", this._handleEventRemoveButtonClicked);
		// Remove all button
		this.connect(this._removeAllButton, "onclick", this._handleEventRemoveAllButtonClicked);
		if (this.sortSelected) {
			this.connect(this._moveUpButton, "onclick", this._handleMoveUpButtonClicked);
			this.connect(this._moveDownButton, "onclick", this._handleMoveDownButtonClicked);
			if (dojo.isIE <= 8) {
				this.connect(this._moveUpButton, "dblclick", this._handleMoveUpButtonClicked);
				this.connect(this._moveDownButton, "dblclick", this._handleMoveDownButtonClicked);
			}
			this._updateMoveButtons();
			this._moveButtons.style.display = "block";
		}
	},

	/* 
	 * Private Method
	 *
	 * @param String value the value entered in the filterbox widget
	 */
	_filterBoxChange: function(filterList, value) {
		if (!filterList) { return; }
		if (this.sortSelected && filterList === this._selectedList)
			this._updateMoveButtons();
		var elements = filterList.getAllEntries();
		var selection = {};
		dojo.forEach(filterList.getSelection(), function(entry) {
			selection[entry.id] = 1;
		});
		filterList.setSelection(filterList.getSelection(), false);
		var newSelection = [];
		var firstVisible;
		for (var i = 0, len = elements.length; i < len; i++) {
			if (this._shouldFilterElement(elements[i], value)) {
				filterList.showEntry(elements[i], false);
			} else {
				if (elements[i].id in selection) {
					newSelection.push(elements[i]);
				}
				if (!firstVisible) { firstVisible = elements[i]; }
				filterList.showEntry(elements[i], true);
			}
		}
	},
	
	_shouldFilterElement: function(elt, value) {
		// TODO: Handle HTML character entities?
		// e.g. &amp;
		// Work Item: 151326
		return elt.label.toLowerCase().indexOf(value) === -1;
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object entry
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 * @return void
 	 */	
	_handleEventDoubleClickedUnselected: function(entry) {
		// Add entry to selected list
		this._selectedList.addEntry(entry);
		this._updateSelectedOrder();
		// Remove entry from unselected list
		this._unselectedList.removeEntry(entry);

		this._updateCount();
		if (this._shouldFilterElement(entry, this._selectedFilterBox.value)) {
			this._selectedList.showEntry(entry, false);
		}
		if (this.sortSelected) this._updateMoveButtons();
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object entry
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 * @return void
 	 */	
	_handleEventDoubleClickedSelected: function(entry) {
		// Add entry to unselected list
		this._unselectedList.addEntry(entry);
		// Remove entry from selected list
		this._selectedList.removeEntry(entry);
		this._updateSelectedOrder();
		if (this._shouldFilterElement(entry, this._unselectedFilterBox.value)) {
			this._unselectedList.showEntry(entry, false);
		}

		this._updateCount();
		if (this.sortSelected) this._updateMoveButtons();
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return void
 	 */	
	_handleEventAddButtonClicked: function(event) {
		// Cancel event if default action is performed
		event.preventDefault();
		// Get selected entries
		var entries = this._unselectedList.getSelection();
		// Validate entries
		if (entries.length === 0) {
			// NO-OP
			return;
		}
		// Add entries to selected list
		this._selectedList.addEntries(entries);
		this._updateSelectedOrder();
		// Remove entries from unselected list
		this._unselectedList.removeEntries(entries);
		// Hide entry if it should be filtered
		dojo.forEach(entries, dojo.hitch(this, function(entry) {
			if (this._shouldFilterElement(entry, this._selectedFilterBox.value)) {
				this._selectedList.showEntry(entry, false);
			}
		}));
		this._updateCount();
		if (this.sortSelected) this._updateMoveButtons();
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return void
 	 */	
	_handleEventAddAllButtonClicked: function(event) {
		// Cancel event if default action is performed
		event.preventDefault();
		// Get all entries
		var entries = this._unselectedList.getAllEntries();
		// Validate entries
		if (entries.length === 0) {
			// NO-OP
			return;
		}
		// Remove elements that aren't matched by the filter box
		var filterText = this._unselectedFilterBox.value;
		entries = dojo.filter(entries, dojo.hitch(this, function(entry) {
			return !this._shouldFilterElement(entry, filterText);
		}));
		// Add entries to selected list
		this._selectedList.addEntries(entries);
		this._updateSelectedOrder();
		// Remove entries from unselected list
		this._unselectedList.removeEntries(entries);
		// Hide filtered entries
		dojo.forEach(entries, dojo.hitch(this, function(entry) {
			if (this._shouldFilterElement(entry, this._selectedFilterBox.value)) {
				this._selectedList.showEntry(entry, false);
			}
		}));
		this._updateCount();
		if (this.sortSelected) this._updateMoveButtons();
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return void
 	 */	
	_handleEventRemoveButtonClicked: function(event) {
		// Cancel event if default action is performed
		event.preventDefault();
		// Get selected entries
		var entries = this._selectedList.getSelection();
		// Validate entries
		if (entries.length === 0) {
			// NO-OP
			return;
		}
		// Add entries to unselected list
		this._unselectedList.addEntries(entries);
		// Remove entries from selected list
		this._selectedList.removeEntries(entries);
		this._updateSelectedOrder();
		// Hide entry if it should be filtered
		dojo.forEach(entries, dojo.hitch(this, function(entry) {
			if (this._shouldFilterElement(entry, this._unselectedFilterBox.value)) {
				this._unselectedList.showEntry(entry, false);
			}
		}));

		this._updateCount();
		if (this.sortSelected) this._updateMoveButtons();
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return void
 	 */	
	_handleEventRemoveAllButtonClicked: function(event) {
		// Cancel event if default action is performed
		event.preventDefault();
		// Get all entries
		var entries = this._selectedList.getAllEntries();
		// Validate entries
		if (entries.length === 0) {
			// NO-OP
			return;
		}
		// Remove elements that aren't matched by the filter box
		var filterText = this._selectedFilterBox.value;
		entries = dojo.filter(entries, dojo.hitch(this, function(entry) {
			return !this._shouldFilterElement(entry, filterText);
		}));
		// Add entries to unselected list
		this._unselectedList.addEntries(entries);
		// Remove entries from selected list
		this._selectedList.removeEntries(entries);
		this._selectedOrder = {};
		dojo.forEach(entries, dojo.hitch(this, function(entry) {
			if (this._shouldFilterElement(entry, this._unselectedFilterBox.value)) {
				this._unselectedList.showEntry(entry, false);
			}
		}));

		this._updateCount();
		if (this.sortSelected) this._updateMoveButtons();
	},
	
	_handleMoveUpButtonClicked: function(event) {
		var selected = this._selectedList.getSelection();
		if (selected.length !== 1) return;
		selected = selected[0];
		var index = this._selectedOrder[selected.id];
		if (index === 0) return;
		index--;
		for (var id in this._selectedOrder) {
			if (this._selectedOrder[id] === index) {
				this._selectedOrder[id]++;
				break;
			}
		}
		this._selectedOrder[selected.id] = index;
		this._selectedList.sort(dojo.hitch(this,this._selectedComparator));
		this._updateMoveButtons();
	},
	
	_handleMoveDownButtonClicked: function(event) {
		var selected = this._selectedList.getSelection();
		if (selected.length !== 1) return;
		selected = selected[0];
		var index = this._selectedOrder[selected.id];
		if (index === this._selectedList.getAllEntries().length-1) return;
		index++;
		for (var id in this._selectedOrder) {
			if (this._selectedOrder[id] === index) {
				this._selectedOrder[id]--;
				break;
			}
		}
		this._selectedOrder[selected.id] = index;
		this._selectedList.sort(dojo.hitch(this,this._selectedComparator));
		this._updateMoveButtons();
	},

	/*
 	 * Public Method
 	 * 
 	 * @param Array entries : Object
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 * @return void
 	 */	
	setSelectedEntries: function(entries) {
		this._selectedList.removeAllEntries();
		this._selectedList.addEntries(entries);
		this._updateSelectedOrder();
		this._updateCount();
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return Array : Object
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 */	
	getSelectedEntries: function() {
		return this._selectedList.getAllEntries();
	},

	/*
 	 * Public Method
 	 * 
 	 * @param Array entries : Object
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 * @return void
 	 */	
	setUnselectedEntries: function(entries) {
		this._unselectedList.removeAllEntries();
		this._unselectedList.addEntries(entries);
		this._updateCount();
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return Array : Object
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 */	
	getUnselectedEntries: function() {
		return this._unselectedList.getAllEntries();
	},

	/*
 	 * Public Method
 	 * 
 	 * @param boolean show
 	 * @return void
 	 */	
	showLoading: function(show) {
		this._unselectedList.showLoading(show);
		this._selectedList.showLoading(show);
	},	

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_updateCount: function() {
		
		var sCount = this._selectedList.getEntryCount();
		var uCount = this._unselectedList.getEntryCount();
		
		this._selectionCount.innerHTML = sCount;
		this._totalCount.innerHTML = sCount + uCount;
		
	},
	
	_updateSelectedOrder: function() {
		if (!this.sortSelected) return;
		var entries = this._selectedList.getAllEntries();
		this._selectedOrder = {};
		for (var i = 0; i < entries.length; i++) {
			this._selectedOrder[entries[i].id] = i;
		}
	},
	
	_updateMoveButtons: function() {
		var allowUp = true, allowDown = true, selection = this._selectedList.getSelection();
		if (this._selectedFilterBox.value || selection.length !== 1) {
			allowUp = allowDown = false;
		} else {
			selection = selection[0];
			if (this._selectedOrder[selection.id] === 0) allowUp = false;
			if (this._selectedOrder[selection.id] === this._selectedList.getAllEntries().length-1) allowDown = false;
		}
		this._moveUpButton.disabled = allowUp ? "" : "disabled";
		this._moveDownButton.disabled = allowDown ? "" : "disabled";
	},
	
	_selectedComparator: function(entry1, entry2) {
		return this._selectedOrder[entry1.id] - this._selectedOrder[entry2.id];
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 * @overrides super
 	 */		
	destroy: function() {
		// Destroy widgets
		this._selectedList.destroy();
		this._unselectedList.destroy();
		
		// Call super
		this.inherited(arguments);
	}
	
});

})();
