/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.internal.TruncatedTextNode");

dojo.require("jazz.util.ResizeNotifier");
dojo.require("jazz.util.html");
(function() {
dojo.declare("jazz.ui.internal.TruncatedTextNode", [dijit._Widget, dijit._Templated], {
	templateString: "<div class=\"jazz-ui-TruncatedTextNode\"><div class=\"truncateNode\" dojoattachpoint=\"contentNode\"></div></div>",
	value: "",
	startup: function() {
		this._queueTruncate(true);
		if (!this._windowResizeEvent) {
			this._windowResizeEvent = jazz.util.ResizeNotifier.register({
				context: this,
				callback: this._queueTruncate
			});	
		}
		this.inherited(arguments);
	},
	uninitialize: function() {
		jazz.util.ResizeNotifier.unregister(this._windowResizeEvent);
	},
	_setValueAttr: function(value) {
		this.value = value;
		dojo.place(document.createTextNode(value), this.contentNode, "only");
		if (this._started) this._queueTruncate(true);
	},
	/*
	 * Handler for onresize truncate call.
	 * Use yielding so that UI is not hung in IE6/7 during truncate's while loop.
	 */ 
	_queueTruncate: function(force) {
		if (!this.truncateTimeout) {
			this.truncateTimeout = setTimeout(dojo.hitch(this,this._truncate,force),300);
		}
	},
	_truncate: function(force) {
		this.truncateTimeout = null;
		if (!this.contentNode) return;
		var node = this.contentNode, 
			title = this.value,
			htmlEscape = jazz.util.html.escape;
		// force is used when the title has been changed and the truncate algorithm needs to be run
		// even if the clientWidth is the same
		// force == true is used since this function is called from many places, some of which
		// pass in parameters
		if (force == true || (this._titleIsTruncated && this.domNode.clientWidth != this._prevWidth)) {
			node.innerHTML = htmlEscape(title);
			this._titleIsTruncated = false;
			if (node.title) {
				// Remove title attribute to prevent confusion over the accessible name
				node.removeAttribute("title");
			} 
		}
		var headerHeight = this.domNode.clientHeight;
		if (this.contentNode.clientHeight > headerHeight + 5) {
			while (this.contentNode.clientHeight > headerHeight + 5) {
				if (this.contentNode.clientHeight > headerHeight * 2) {
					title = title.substring(0,Math.floor(title.length/1.75));
				} else {
					title = title.substring(0,title.length-2);
				}
				while (title.lastIndexOf(" ") == title.length-1)
					title = title.substring(0,title.length-1);
				node.innerHTML = htmlEscape(title) + "...";
				if (node.innerHTML.length <= 6) break;
			}
			node.title = this.value;
			this._titleIsTruncated = true;
			this._prevWidth = this.domNode.clientWidth;
		}
	}
});
})();
