/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ensemble.internal.WidgetFactory");

dojo.require("jazz.core.loader");
dojo.require("jazz.core.RegistryFactory");

(function() {

var RegistryFactory = jazz.core.RegistryFactory;

var _factoriesByType = {};
var _factoryDeferredsByType = {};

/*
 * Creates Widget instances from state data.
 */
jazz.ensemble.internal.WidgetFactory = {
	
	/*
	 * Creates a new Widget instance based on the given widget state,
	 * returns it asynchronously.
	 */
	/* Deferred */ createWidget: function(/* object */ state, /* WidgetContext */ context, /* boolean */ readOnly, /* string */ uuid) {
		// prepare args
		var dfd = new dojo.Deferred();
		var args = {
			context: context,
			definition: state.getDefinition(),
			memento: state.getMemento(),
			readOnly: readOnly,
			uuid: uuid
		};
		
		// call type-specific factory
		var type = state.getType();
		var dfd2 = _getFactory(type);
		dfd2.addCallback(function(/* object */ factory) {
			var widget = factory.createWidget(args);
			dfd.callback(widget);
			return factory;
		});
		dfd2.addErrback(function(/* Error */ e) {
			dfd.errback(e);
			return e;
		});
		return dfd;
	}
};

function _getFactory(/* string */ type) {
	// check if already loaded
	var factory = _factoriesByType[type];
	if (factory) {
		var dfd = new dojo.Deferred();
		dfd.callback(factory);
		return dfd;
	}
	return _loadFactory(type);
}

function _getFactoryClass(/* string */ type) {
	var registry = RegistryFactory.getRegistry();
	var extPt = registry.getExtensionPoint("net.jazz.web.ensemble.widgetSystems");
	if (extPt) {
		var exts = extPt.getExtensions();
		for (var i=0;i<exts.length;i++) {
			var cfgElems = exts[i].getConfigurationElements();
			for (var j=0;j<cfgElems.length;j++) {
				var cfgElem = cfgElems[j];
				if (cfgElem.getName() == "widgetSystem" && cfgElem.getAttribute("type") == type) {
					var widgetFactory = cfgElem.getAttribute("widgetFactory");
					if (widgetFactory) {
						return widgetFactory;
					}
				}
			}
		}
	}
}

function _loadFactory(/* string */ type) {
	// check if already loading
	var dfd = _factoryDeferredsByType[type];
	if (dfd) {
		return dfd;
	}
	// load it
	dfd = new dojo.Deferred();
	_factoryDeferredsByType[type] = dfd;
	var className = _getFactoryClass(type);
	if (className) {
		try {
			// load the class
			jazz.core.loader.load_async(className, function() {
				var clazz = dojo.getObject(className);
				try {
					// instantiate and update cache
					var factory = new clazz();
					_factoriesByType[type] = factory;
					delete _factoryDeferredsByType[type];
					dfd.callback(factory);
				}
				catch (/* Error */ e) {
					dfd.errback(e);
				}
			});
		}
		catch (/* Error */ e) {
			dfd.errback(e);
		}
	}
	else {
		dfd.errback("No widget factory registered for widget type '" + type + "'");
	}
	return dfd;
}

})();