/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ensemble.internal.catalog.AddExternalWidgetPane");

/*
 * Imports 
 */
dojo.require("dojo.i18n"); 

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.ensemble.internal.catalog.AddWidgetButton");

dojo.requireLocalization("jazz.ensemble.internal", "WidgetCatalogMessages");

/*
 * 
 */

(function() {

/*
 * Aliases 
 */
var AddWidgetButton = jazz.ensemble.internal.catalog.AddWidgetButton;

/*
 * Constants 
 */
var TYPE_GADGET = "g";
var TYPE_IWIDGET = "i";

/*
 * 
 */
dojo.declare("jazz.ensemble.internal.catalog.AddExternalWidgetPane",
	[dijit._Widget, dijit._Templated], {

	// Properties //----------------------------------------------------------->

	catalog: null,
	type: null,
	
	// ------------------------------------------------------------------------>
	
	/*
	 * Templates
	 */	
	templatePath: dojo.moduleUrl("jazz.ensemble", "internal/catalog/templates/AddExternalWidgetPane.html"),
	
	/*
 	 * @param Object args
 	 * 
 	 * @return AddExternalWidgetPane
 	 */
	constructor: function(args) {
		
		// Messages //--------------------------------------------------------->

		this.msg = dojo.i18n.getLocalization("jazz.ensemble.internal", "WidgetCatalogMessages");
		
		if (args.type === TYPE_GADGET) {
			// Gadget
			this._msgTitle = this.msg.addGadgetTitle;
			this._msgInstruction = this.msg.addGadgetInstruction;
			this._sampleUrl = "http://www.domainname.com/gadgets/gadget.xml";
		} else {
			// iWidget
			this._msgTitle = this.msg.addIWidgetTitle;
			this._msgInstruction = this.msg.addIWidgetInstruction;
			this._sampleUrl = "http://www.domainname.com/iwidgets/iwidget.xml";
		}
		
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */	
	postCreate: function() {

		this.inherited(arguments);

		// Instance Variables //----------------------------------------------->

		this._addWidgetButton = null;

		// Initialization //--------------------------------------------------->

		this._initFields();
		this._initHandlers();
		
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initFields: function() {
		this._textSelected = false;
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initHandlers: function() {
		this._initBackLink();
		this._initAddWidgetButton();
		this._initUrlInputText();
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initBackLink: function() {
		
		this.connect(this._backLink, "onclick", function(event) {
			if (event) {
				dojo.stopEvent(event);
			}
			
			this.catalog._showWidgetResultsPane();
			
			return true;
		});	
	
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initAddWidgetButton: function() {
		
		this._addWidgetButton = new AddWidgetButton({
			catalog: this.catalog,
			onAddWidgetHandler: dojo.hitch(this, this._addWidget)
		});
		this._addWidgetButtonSite.appendChild(this._addWidgetButton.domNode);				
		this._addWidgetButton.attr("enabled", false);
		
	},
	
	/*
	 * @param void
	 *
	 * @return Object
	 */
	_addWidget: function() {
		var deferred;
		
		if (!this._urlInputText.value ||
			!this._isValidUrl(this._urlInputText.value)) {
			deferred = new dojo.Deferred();
			deferred.errback(new Error(this.msg.invalidUrl));
			return deferred;
		}
		
		var data;
		if (this.type === TYPE_GADGET) {
			// Gadget
			data = this._getGadgetData();
		} else {
			// iWidget
			data = this._getIWidgetData();
		}

		deferred = this.catalog._addWidgetToMashUp(data);
		return deferred;
	},

	/*
	 * @param {String} url The URL to validate
	 *
	 * @return Boolean Whether or not the `url` param is a valid RFC 3986 HTTP or HTTPS URI
	 */
	_isValidUrl: function(url) {
		return url.match(new RegExp('^(http(s)?://)([^/?#\\s]*)([^?#\\s]*)(\\?([^\\s#]*))?(#(\\S*))?$'));
	},

	/*
 	 * @param void
 	 * 
 	 * @return Object
 	 */		
	_getGadgetData: function() {

		var url = this._urlInputText.value;
		var widgetID = TYPE_GADGET + 
						":" + 
						url;
		var data = {
			url: widgetID
		};

		return data;		
	},

	/*
 	 * @param void
 	 * 
 	 * @return Object
 	 */		
	_getIWidgetData: function() {
		
		var url = this._urlInputText.value;
		var widgetID = TYPE_IWIDGET + 
						":" + 
						url;
		var data = {
			url: widgetID
		};

		return data;		
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initUrlInputText: function() {
		
		this._showSampleUrl();
		
		this.connect(this._urlInputText, "onkeydown", this._handleEventOnKeyDownUrlInput);
		this.connect(this._urlInputText, "onkeyup", this._handleEventOnKeyUpUrlInput);
		this.connect(this._urlInputText, "onpaste", this._handleEventOnPasteUrlInput);
		this.connect(this._urlInputText, "onfocus", this._handleEventOnFocusUrlInput);
		this.connect(this._urlInputText, "onblur", this._handleEventOnBlurUrlInput);
	},
	
	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */		
	_handleEventOnKeyUpUrlInput: function(event) {
		this._addWidgetButton.attr("enabled", this._urlInputText.value);
		return true;
	},

	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */		
	_handleEventOnKeyDownUrlInput: function(event) {
		this._urlInputText.style.color = "";
		if (this._urlInputText.value && event.keyCode === dojo.keys.ENTER) {
			this._addWidgetButton.handleAddWidget(dojo.hitch(this, this._addWidget));
		}
		return true;
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_showSampleUrl: function() {
		if (!this._urlInputText.value) {
			this._urlInputText.style.color = "#9E9E9E";	
			this._urlInputText.value = this._sampleUrl;
		}
	},

	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */		
	_handleEventOnPasteUrlInput: function(event) {
		this._textSelected = true;
		this._urlInputText.style.color = "";
		this._addWidgetButton.attr("enabled", true);
		return true;
	},

	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */		
	_handleEventOnFocusUrlInput: function(event) {
		if (event) {
			dojo.stopEvent(event);
		}

		if (this._textSelected) {
			return true;
		}
		
		var value = this._urlInputText.value;

		if (value === this._sampleUrl) {
			this._urlInputText.value = "";
		} else if (value) {
			this._urlInputText.select();
			this._textSelected = true;
		}
			
		return true;
	},

	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */		
	_handleEventOnBlurUrlInput: function(event) {
		if (event) {
			dojo.stopEvent(event);
		}

		this._textSelected = false;
		this._showSampleUrl();
			
		return true;
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	destroy: function() {
		
		if (this._addWidgetButton) {
			this._addWidgetButton.destroy();
			this._addWidgetButton = null;
		}
		
		this.inherited(arguments);
	}

});

})();
