/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ensemble.internal.catalog.AddWidgetButton");

/*
 * Imports 
 */
dojo.require("dojo.i18n"); 
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.ui.toolbar.Button");

dojo.requireLocalization("jazz.ensemble.internal", "WidgetCatalogMessages");

(function() {

/*
 * Aliases 
 */
var MessageBox = jazz.ui.MessageBox;
var STATUS_TYPES = jazz.ui.MessageBox.STATUS_TYPES;

/*
 * Class 
 */
dojo.declare("jazz.ensemble.internal.catalog.AddWidgetButton",
	[dijit._Widget, dijit._Templated], {

	// Properties //----------------------------------------------------------->

	onAddWidgetHandler: null,
	highlightNode: null,
	catalog: null,
	enabled: true, 

	// ------------------------------------------------------------------------>

	/*
 	 * Template
 	 */
	templatePath: dojo.moduleUrl("jazz.ensemble", "internal/catalog/templates/AddWidgetButton.html"),
	
	/*
 	 * @param Object args
 	 * 
 	 * @return AddWidgetButton 
 	 */
	constructor: function(args) {
		
		// Messages //------------------------------------------------------------->

		this.msg = dojo.i18n.getLocalization("jazz.ensemble.internal", "WidgetCatalogMessages");

		// Images //----------------------------------------------------------->

		this._addWidgetIcon = dojo.moduleUrl("jazz.ensemble", "internal/graphics/widget-catalog/images/add-widget.gif").toString();
		
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	postCreate: function() {

		this.inherited(arguments);

		// Instance Variables //----------------------------------------------->

		this._messageBox = null;
		this._fadeTimeout = null;

		// Initialization //--------------------------------------------------->

		this._initFields();
		
	},	

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initFields: function() {
		this._initAddWidgetButton();
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initAddWidgetButton: function() {
		this._addWidgetButton = new jazz.ui.toolbar.Button({
			label: this.msg.addWidget,
			onClick: dojo.hitch(this, this._handleEventOnClickAddWidget),
			imgSrc: this._addWidgetIcon
		});
		dojo.addClass(this._addWidgetButton.focusNode, "j-action-secondary");
		this._addWidgetButton.placeAt(this._addWidget);
	},

	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */		
	_handleEventOnClickAddWidget: function(event) {
		if (event) {
			dojo.stopEvent(event);
		}

		if (!this.onAddWidgetHandler) {
			return true;
		}

		this.handleAddWidget(this.onAddWidgetHandler);
			
		return true;
	},

	/*
 	 * @param Function handler
 	 * 
 	 * @return Boolean
 	 */		
	handleAddWidget: function(handler) {

		this._showAddingMessage();
		
		var deferred = handler();

		deferred.addCallback(dojo.hitch(this, function(status) {
			this._showSuccessMessage();
			return status; 
		}));
		deferred.addErrback(dojo.hitch(this, function(errorObject) {
			this._showErrorMessage(errorObject);
			return errorObject; 
		}));		
			
		return deferred;
	},

	/*
	 * @param void
	 *
	 * @return void
	 */
	_showAddingMessage: function() {
		// Show message area
		var messageBox = this._getMessageBox();
		messageBox.showMessage({ 
		   type: STATUS_TYPES.INFO,
		   summary: this.msg.statusAdding,
		   closable: false
		});
	},		
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_showSuccessMessage: function() {
		this._showStatusMessage(STATUS_TYPES.OK, this.msg.statusAdded);
	},

	/*
 	 * @param Error errorObject
 	 * 
 	 * @return void
 	 */		
	_showErrorMessage: function(errorObject) {
		this._showStatusMessage(STATUS_TYPES.ERROR, this.msg.statusFailed);
		var summary = this._getErrorSummary(errorObject);
		this.catalog._showStatusMessage(STATUS_TYPES.ERROR, summary);
	},

	/*
 	 * @param Number type
 	 * @param String summary
 	 * 
 	 * @return void
 	 */		
	_showStatusMessage: function(type, summary) {
		
		// Show message area
		var messageBox = this._getMessageBox();
		messageBox.showMessage({ 
		   type: type,
		   summary: summary,
		   closable: false
		});
		
		this._displayHighlightArea();
		this._abortFadeTimeout();
		this._fadeTimeout = setTimeout(
			dojo.hitch(this, this._doFadeOut), 3000);
	},
	
	_setEnabledAttr: function(enabled){
		if (this._addWidgetButton) {
			this._addWidgetButton.attr("enabled", enabled);
		}
	},

	/*
 	 * @param Error errorObject
 	 * 
 	 * @return String
 	 */
	_getErrorSummary: function(errorObject) {
		
		if (errorObject) {
			if (errorObject.status === 403) {
				return dojo.string.substitute(this.msg.errTemplate, [this.msg.errAddWidget, this.msg.errPermission]);
			} else if (errorObject.message) {
				return dojo.string.substitute(this.msg.errTemplate, [this.msg.errAddWidget, errorObject.message]);
			}
		} 
		
		return this.msg.errAddWidget;
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_displayHighlightArea: function() {
		if (this.highlightNode) {
			// Show background highlight
			dojo.style(this.highlightNode, {opacity: 1});
			if (dojo.isIE) {
				// Show background highlight
				this.highlightNode.style.display = "block";
			}
		}
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_doFadeOut: function() {

		var duration = 1000;
		
		if (this.highlightNode) {
			var backgroundFadeAnimation = dojo.animateProperty({
				node: this.highlightNode, 
				duration: duration,
				properties:	{
					opacity: {
						start: 1, 
						end: 0
					}
				}
			});
			backgroundFadeAnimation.play();
		}
		
		this._getMessageBox().hideMessage(true);
	},

	/*
 	 * @param void
 	 * 
 	 * @return MessageBox
 	 */		
	_getMessageBox: function() {
		if (!this._messageBox) {
			this._messageBox = new MessageBox();
      		this._messageAreaNode.appendChild(this._messageBox.domNode);
		}
		return this._messageBox;
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_abortFadeTimeout: function() {
		if (this._fadeTimeout) {
			clearTimeout(this._fadeTimeout);
			this._fadeTimeout = null;
		}		
	},
		
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	destroy: function() {
		
		if (this._messageBox) {
			this._messageBox.destroy();
			this._messageBox = null;
		}
		
		if (this._addWidgetButton) {
			this._addWidgetButton.destroy();
			delete this._addWidgetButton;
		}
		
		this._abortFadeTimeout();
		
		this.inherited(arguments);
	}
	
});

})();
