/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ensemble.internal.catalog.CatalogService");

dojo.require("jazz.ensemble.internal.util.Xml");
dojo.require("jazz.ensemble.internal.util.XmlBindingFactory");
dojo.require("jazz.client.xhr");
dojo.require("dojo.DeferredList");


(function()
{
var namespace  		 	= jazz.ensemble.internal.util.Xml.namespace;
var XmlBindingFactory 	= jazz.ensemble.internal.util.XmlBindingFactory;
var xml				 	= jazz.ensemble.internal.util.Xml;
var CatalogService		= jazz.ensemble.internal.catalog.CatalogService;

	/**
	 * load and consolidate always returns a dojo.deferred
	 * This will take either an array or a single uri 
	 * 
	 * This returns deferred. The callback will return the results of all the uris
	 * 
	 * {
	 *    results: [uri]=Error // null if no error
	 *  
	 *    catalog:
	 *    {
	 *    	entries:[0..n],
	 *      origins:[0..n],
	 *      categories:[0..n]
	 *    }
	 * }  
	 */
jazz.ensemble.internal.catalog.CatalogService.load = function(uris)
{
	// make it an array if it isn't already
	if (!dojo.isArray(uris)) uris = [uris];

	var results 		= [];
	var catalogs    	= [];
	var deferredList	= [];
	
	dojo.forEach(uris, function (uri){
		
		var deferred = jazz.client.xhrGet({handleAs: "xml", url: uri});
		
		deferred.addCallback(function(doc){
			results[uri]=null; // null means no errors
			catalogs.push(CatalogService.parseWidgetCatalog(doc, uri));
		});
	

		deferred.addErrback(function(error){
	   		results[uri]=error;
		});
		
		deferredList.push(deferred);
	});
	
	
	/**
	 * this is the deferred we return to our consumer
	 * we fire it after all of the list has returned
	 */
	var finalDeferred =  new dojo.Deferred();

	/**
	 * deferredList returns after all the lists are done
	 */
	(new dojo.DeferredList(deferredList)).addCallback(function(resultList){
		
		finalDeferred.callback({"results": results,
								"catalog": CatalogService.consolidateWidgetCatalogs(catalogs) 
								});
	}); 

	return finalDeferred;
}

jazz.ensemble.internal.catalog.CatalogService.consolidateWidgetCatalogs = function(catalogs)
{
	if (!dojo.isArray(catalogs)) catalogs = [catalogs];

	var entryHash	    = {};
	var categoryHash 	= {};
	var originHash		= {};

	dojo.forEach(catalogs, function(catalog){
	
		dojo.forEach(catalog.entries, 	function (entry)	{entryHash[entry.uri] 	= entry;  });
		dojo.forEach(catalog.categories,function (category)	{categoryHash[category] = true;	  });
		dojo.forEach(catalog.origins, 	function (origin)	{originHash[origin.uri] = origin; });
	
	});
	
	var entries = [], categories = [], origins=[];
	
	for (var uri 		in entryHash)	{entries.	push(entryHash[uri]);	}
	for (var category 	in categoryHash){categories.push(category);			}
	for (var uri 		in originHash)	{origins.	push(originHash[uri]);	}

	return 	{"entries"	:entries, "categories":categories, "origins"	:origins};
};

 
jazz.ensemble.internal.catalog.CatalogService.parseWidgetCatalog = function(doc, rootUri)
{
	var error = xml.getParseError(doc);
	if (error) throw new Error(error);

	var docElement = XmlBindingFactory.createBinding(doc.documentElement, CATALOG_SCHEMA);
	
	var content = {origins:[], categories:[], entries:[]};

	var categoryHash = {} // a unique list of categories
	
	dojo.forEach(docElement.entries, function(entry){
		var id	= entry.id;
		var uri	= entry.uri;

		if (uri){
			entry.uri=uri;
		}
		else if (!uri && id){
			entry.uri=rootUri+"/"+id;
		}
		else{
			// TODO Remove this and fail silently 
			console.warn("ERROR: Catalog entry without URI or id:", entry.title);
			return;
		}
		dojo.forEach(entry.categories, function(c){
			categoryHash[c]=1;
		});
		
		entry.title 		= entry.title||"";
		entry.type 			= entry.type||"";
		entry.description 	= entry.description||"";
		entry.widgetUri 	= entry.widgetUri||"";
		entry.author 		= entry.author||"";
		entry.icon 			= entry.icon||"";
		entry.preview 		= entry.preview||"";
		entry.thumbnail 	= entry.thumbnail||"";
		entry.origin 		= entry.origin||"";

		content.entries.push(entry);
	});

	dojo.forEach(docElement.origins, function(o){
		content.origins.push({title:o.title, uri:o.uri});
	});

	for (var c in categoryHash){
		content.categories.push(c);	
	}
	categoryHash = null;

	return content;
}


var PREFERENCE_SCHEMA =
[
	{
		property:      	"name",
		type:     		XmlBindingFactory.STRING,
		nodeType: 		XmlBindingFactory.ATTRIBUTE,
		attrName: 		"name",
		attrNS:   		namespace.ju
	},
	{
		property: 		"value",
		type: 	 		XmlBindingFactory.STRING,
		nodeType: 		XmlBindingFactory.ATTRIBUTE,
		attrName: 		"value",
		attrNS:   		namespace.ju
	}
];

var ENTRY_SCHEMA = 
[
	{
		property: 		"id",
		type: 			XmlBindingFactory.STRING,
		nodeType: 		XmlBindingFactory.ATTRIBUTE,
		attrName:		"id",
		attrNS:			namespace.xml
	},
	{
		property: 		"uri",
		type: 			XmlBindingFactory.STRING,
		nodeType: 		XmlBindingFactory.ATTRIBUTE,
		attrName:		"about",
		attrNS:			namespace.rdf
	},
	{
		property: 		"icon",
		type: 			XmlBindingFactory.STRING,
		nodeType: 		XmlBindingFactory.CHILD_ELEMENT_ATTRIBUTE,
		elemName:		"icon",
		elemNS:			namespace.ju,
		attrName: 		"resource",
		attrNS: 		namespace.rdf
	},
	{
		property: 		"preview",
		type: 			XmlBindingFactory.STRING,
		nodeType: 		XmlBindingFactory.CHILD_ELEMENT_ATTRIBUTE,
		elemName:		"preview",
		elemNS:			namespace.ju,
		attrName: 		"resource",
		attrNS: 		namespace.rdf
	},
	{
		property: 		"thumbnail",
		type: 			XmlBindingFactory.STRING,
		nodeType: 		XmlBindingFactory.CHILD_ELEMENT_ATTRIBUTE,
		elemName:		"thumbnail",
		elemNS:			namespace.ju,
		attrName: 		"resource",
		attrNS: 		namespace.rdf
	},
	{
		property: 		"origin",
		type: 			XmlBindingFactory.STRING,
		nodeType: 		XmlBindingFactory.CHILD_ELEMENT_ATTRIBUTE,
		elemName:		"origin",
		elemNS:			namespace.ju,
		attrName: 		"resource",
		attrNS: 		namespace.rdf
	},
	{
		property: 		"categories",
		type: 			XmlBindingFactory.STRING,
		nodeType: 		XmlBindingFactory.CHILD_ELEMENT_TEXT,
		elemName:		"category",
		elemNS:			namespace.ju,
		multiValued: 	true
	},
	{
		property: 		"description",
		type: 			XmlBindingFactory.STRING,
		nodeType: 		XmlBindingFactory.CHILD_ELEMENT_TEXT,
		elemName:		"description",
		elemNS:			namespace.dublin
	},
	{
		property: 		"title",
		type: 			XmlBindingFactory.STRING,
		nodeType: 		XmlBindingFactory.CHILD_ELEMENT_TEXT,
		elemName:		"title",
		elemNS:			namespace.dublin
	},
	{
		property: 		"author",
		type: 			XmlBindingFactory.STRING,
		nodeType: 		XmlBindingFactory.CHILD_ELEMENT_TEXT,
		elemName:		"author",
		elemNS:			namespace.ju
	},
	{
		property: 		"type",
		type: 			XmlBindingFactory.STRING,
		nodeType: 		XmlBindingFactory.CHILD_ELEMENT_TEXT,
		elemName:		"type",
		elemNS:			namespace.dublin
	},
	{
		property: 		"widgetUri",
		type: 			XmlBindingFactory.STRING,
		nodeType: 		XmlBindingFactory.CHILD_ELEMENT_TEXT,
		elemName:		"widgetUri",
		elemNS:			namespace.ju
	},
	{
		property:     	"preferences",
		type:     		XmlBindingFactory.OBJECT,
		nodeType: 		XmlBindingFactory.CHILD_ELEMENT,
		elemName: 		"preference",
		elemNS:   		namespace.ju,
		schema:   		PREFERENCE_SCHEMA,
		multiValued: 	true
	}
];


var ENTRY_ORIGIN = 
[
	{
		property: 		"uri",
		type: 			XmlBindingFactory.STRING,
		nodeType: 		XmlBindingFactory.ATTRIBUTE,
		attrName:		"about",
		attrNS:			namespace.rdf
	},
	{
		property: 		"title",
		type: 			XmlBindingFactory.STRING,
		nodeType: 		XmlBindingFactory.CHILD_ELEMENT_TEXT,
		elemName:		"title",
		elemNS:			namespace.dublin
	}
];

var CATALOG_SCHEMA =
[
	{
		property:     	"entries",
		type:     		XmlBindingFactory.OBJECT,
		nodeType: 		XmlBindingFactory.CHILD_ELEMENT,
		elemName: 		"catalog-entry",
		elemNS:   		namespace.ju,
		schema:   		ENTRY_SCHEMA,
		multiValued: 	true
	},
	{
		property:      	"origins",
		type:     		XmlBindingFactory.OBJECT,
		nodeType: 		XmlBindingFactory.CHILD_ELEMENT,
		elemName: 		"origin",
		elemNS:   		namespace.ju,
		schema:   		ENTRY_ORIGIN,
		multiValued: 	true
	}
];


})();

