/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ensemble.internal.catalog.FilterElementBox");

/*
 * Imports 
 */
dojo.require("dojo.i18n"); 

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.ensemble.internal.catalog.CatalogUtility");

dojo.requireLocalization("jazz.ensemble.internal", "WidgetCatalogMessages");

(function() {

/*
 * Aliases 
 */
var CatalogUtility = jazz.ensemble.internal.catalog.CatalogUtility;

/*
 * Class 
 */
dojo.declare("jazz.ensemble.internal.catalog.FilterElementBox",
	[dijit._Widget, dijit._Templated], {

	// Properties //----------------------------------------------------------->

	label: "",
	filterId: "",
	removable: true,
	onRemoveClick: null,

	// ------------------------------------------------------------------------>

	/*
 	 * Template
 	 */
	templatePath: dojo.moduleUrl("jazz.ensemble", "internal/catalog/templates/FilterElementBox.html"),
	
	/*
 	 * @param Object args
 	 * 
 	 * @return FilterElementBox 
 	 */
	constructor: function(args) {
		
		// Messages //------------------------------------------------------------->

		this.msg = dojo.i18n.getLocalization("jazz.ensemble.internal", "WidgetCatalogMessages");
		
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	postCreate: function() {

		this.inherited(arguments);

		// Instance Variables //----------------------------------------------->

		this._removeButtonHandlers = [];

		// Initialization //--------------------------------------------------->

		this._initLabel();
		this._initRemoveButton();
		
	},	

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initLabel: function() {
		var textNode = document.createTextNode(this.label);
		this._label.appendChild(textNode);		
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initRemoveButton: function() {
		
		if (this.removable) {
			// Show X
			CatalogUtility._setNodeVisibility(this._removeButtonSite, true);
			this.domNode.style.paddingRight = "";
			this._removeButton.src = dojo.moduleUrl("jazz.ensemble", "internal/graphics/widget-catalog/images/close.gif");
			this._updateRemoveButtonWAIName();
			
			// Add handlers
			this._removeButtonHandlers.push(this.connect(this._removeButton, "onclick", this._handleEventOnClickRemoveButton));		
			this._removeButtonHandlers.push(this.connect(this._removeButton, "onmouseover", this._handleEventOnMouseOverRemoveButton));
			this._removeButtonHandlers.push(this.connect(this._removeButton, "onmouseout", this._handleEventOnMouseOutRemoveButton));		
		} else {
			// Hide X
			CatalogUtility._setNodeVisibility(this._removeButtonSite, false);
			this.domNode.style.paddingRight = "7px";
			
			// Remove handlers
			for (var _i = 0; _i < this._removeButtonHandlers.length; _i++) {
				var handler = this._removeButtonHandlers[_i];
				this.disconnect(handler);
			}
		}

	},

	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */	
	_handleEventOnClickRemoveButton: function(event) {
		if (event) {
			dojo.stopEvent(event);
		}
		
		if (this.onRemoveClick) {
			this.onRemoveClick(this.label);
		}
		
		return true;
	},

	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */	
	_handleEventOnMouseOverRemoveButton: function(event) {
		if (event) {
			dojo.stopEvent(event);
		}
		// Highlight on
		this._removeButton.src = dojo.moduleUrl("jazz.ensemble", "internal/graphics/widget-catalog/images/close-hov.gif");
		
		return true;
	},

	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */	
	_handleEventOnMouseOutRemoveButton: function(event) {
		if (event) {
			dojo.stopEvent(event);
		}
		// Highlight off
		this._removeButton.src = dojo.moduleUrl("jazz.ensemble", "internal/graphics/widget-catalog/images/close.gif");
		
		return true;
	},

	/*
 	 * @param String label
 	 * 
 	 * @return void
 	 */		
	_updateLabel: function(label) {
		this.label = label;
		var textNode = document.createTextNode(this.label);
		this._label.replaceChild(textNode, this._label.firstChild);
		if (this.removable) {
			this._updateRemoveButtonWAIName();
		}
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_updateRemoveButtonWAIName: function() {
		var accessibleName = dojo.string.substitute(this.msg.removeFilterElement, [this.label]);
		dijit.setWaiState(this._removeButton, "label", accessibleName);
		this._removeButton.title = this.msg.remove;
	},

	/*
 	 * @param Boolean removable
 	 * 
 	 * @return void
 	 */		
	_setRemovable: function(removable) {
		
		if (removable === this.removable) {
			return;
		}
		
		this.removable = removable;
		this._initRemoveButton();
		
	},
		
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	destroy: function() {
		
		this.inherited(arguments);
	}
	
});

})();
