/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ensemble.internal.catalog.WidgetCatalog");

/*
 * TODO: MP
 */
// Redo default Foundation thumbnail image to include blue J instead of green J 
// Limit provider filter element label length with ... if too long
// Limit filter elements length to prevent collision with drawer close control?
// Filter element box - remove dependency on catalog?
// Show msg helping user configure friends
// Add message pane to catalog similar to error pane? Get rid of message div in widget results pane?
// Create absolute error message area in filter element space that is closable
// Add descriptions to categories?
// Check server versions and handle accordingly
// Figure out strategy of when to sort on the server vs. the client (categories and entries)
// Defer load of widget entry data ONLY needed when added? (e.g. preferences, trim, showBackground, etc.)
// Create a shared catalog constants static class
// Create a widgetResultsPage widget
// Fix bug: Click on add external widget page does not abort request

/*
 * Imports 
 */
dojo.require("dojo.i18n"); 

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.app.internal.applicationAbout");
dojo.require("jazz.app.ua.PageHoverHelp");

dojo.require("jazz.ui.FilterBox");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.ui.internal.Select");
dojo.require("jazz.util.ResizeNotifier");

dojo.require("jazz.ensemble.internal.catalog.AddExternalWidgetPane");
dojo.require("jazz.ensemble.internal.catalog.CatalogUtility");
dojo.require("jazz.ensemble.internal.catalog.ErrorUtility");
dojo.require("jazz.ensemble.internal.catalog.WidgetDetailsPane");
dojo.require("jazz.ensemble.internal.catalog.WidgetResultsPane");

dojo.require("net.jazz.ajax.BootstrapProperties");

dojo.requireLocalization("jazz.ensemble.internal", "WidgetCatalogMessages");

(function() {

/*
 * Aliases 
 */
var FilterBox = jazz.ui.FilterBox;
var MessageBox = jazz.ui.MessageBox;
var STATUS_TYPES = jazz.ui.MessageBox.STATUS_TYPES;

var AddExternalWidgetPane = jazz.ensemble.internal.catalog.AddExternalWidgetPane;
var WidgetResultsPane = jazz.ensemble.internal.catalog.WidgetResultsPane;
var WidgetDetailsPane = jazz.ensemble.internal.catalog.WidgetDetailsPane;
var CatalogUtility = jazz.ensemble.internal.catalog.CatalogUtility;
var applicationAbout = jazz.app.internal.applicationAbout;
var PageHoverHelp = jazz.app.ua.PageHoverHelp;
 
var BootstrapProperties = net.jazz.ajax.BootstrapProperties;

var ErrorUtility = jazz.ensemble.internal.catalog.ErrorUtility;

/*
 * Constants 
 */
var WIDGETS_PER_PAGE = 6;
var TYPE_GADGET = "g";
var TYPE_IWIDGET = "i";

/*
 * Class
 */
dojo.declare("jazz.ensemble.internal.catalog.WidgetCatalog",
	[dijit._Widget, dijit._Templated], {

	// Properties //----------------------------------------------------------->

	provider: null,
	onAddWidget: null,
	
	// ------------------------------------------------------------------------>
	
	/*
	 * Templates
	 */	
	templatePath: dojo.moduleUrl("jazz.ensemble", "internal/catalog/templates/WidgetCatalog.html"),
	
	widgetsInTemplate: true,
	
	/*
 	 * @param Object args
 	 * 
 	 * @return WidgetCatalog
 	 */
	constructor: function(args) {
		
		// Messages //--------------------------------------------------------->
		
		this.msg = dojo.i18n.getLocalization("jazz.ensemble.internal", "WidgetCatalogMessages");

		// Images //----------------------------------------------------------->

		this._gadgetIconSrc = dojo.moduleUrl("jazz.ensemble", "internal/graphics/widget-catalog/images/icon_gadget.gif");
		this._iwidgetIconSrc = dojo.moduleUrl("jazz.ensemble", "internal/graphics/widget-catalog/images/icon_iwidget.png");
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */	
	postCreate: function() {

		this.inherited(arguments);

		// Instance Variables //----------------------------------------------->

		this._messageBox = null;

		// Origins
		
		this._selectedOriginIndex = 0;
		this._origins = [];		
		
		// Categories
		
		this._selectedCategory = null;
		this._categoryIndex = {};

		// Panes
		
		this._selectedPane = null;

		this._widgetResultsPane = null;
		this._widgetDetailsPane = null;
		this._addGadgetPane = null;
		this._addIWidgetPane = null;

		// Filter Elements
		
		this._originFilterElement = null;
		this._categoryFilterElement = null;

		// Filter Box
		
		this._filterBox = null;
		this._filterTimeout = null;

		// Initialization //--------------------------------------------------->

		this._init();
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_onResize: function() {
		var sel = this._selector;
		if (sel && sel.domNode) {
			var w = dojo.position(this._catalogPickerSite).w;
			if (w > 0) {
				dojo.style(sel.domNode, 'maxWidth', (w - 5) + 'px');
			}
		}
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_init: function() {
		
		if (this._validateParameters() === false) {
			ErrorUtility.logError(new Error(this.msg.errInvalidConfiguration), this.id);
			this._showErrorMessage(this.msg.errInvalidConfiguration);
			return;
		}
		
		// Add Gadget Pane
		this._initAddGadgetLink();
		// Add iWidget Pane
		this._initAddIWidgetLink();
		// Widget Results Pane
		this._initWidgetResultsPane();
		// Filter Box
		this._initFilterBox();		
		
		// Origins
		var deferred = this._initOrigins();
		deferred.addCallback(dojo.hitch(this, function(origins) {
			// Categories
			this._initCategories();
			
			return origins; 
		}));		
		
	},

	/*
 	 * @param void
 	 * 
 	 * @return Boolean
 	 */		
	_validateParameters: function() {
		
		if (!this.provider) {
			return false;
		} else if (!this.onAddWidget) {
			return false;
		}
		
		return true;
	},

	/*
 	 * @param void
 	 * 
 	 * @return dojo.Deferred
 	 */		
	_initOrigins: function() {
		
		var deferred = this.provider.getOrigins();
		
		deferred.addCallback(dojo.hitch(this, function(origins) {
			this._processOrigins(origins);
			return origins; 
		}));
		deferred.addErrback(dojo.hitch(this, function(errorObject) {
			this._showLoadingMessage(false);
			ErrorUtility.logError(errorObject, this.id);
			this._showErrorMessage(this.msg.noOrigins);
			return errorObject; 
		}));
		
		return deferred;
	},

	/*
 	 * @param Array origins
 	 * 
 	 * @return void
 	 */		
	_processOrigins: function(origins) {
		this._origins = origins;
		if (this._origins.length === 0) {
			this._showLoadingMessage(false);
			ErrorUtility.logError(new Error(this.msg.noOrigins), this.id);
			this._showErrorMessage(this.msg.noOrigins);
			// TODO: MP: Abort initialization?
			return;
		}	
		// Origins >= 1
		
		var optionElements = {};
		var friendServers = [];
		var otherServers = [];
		for (var index = 0; index < this._origins.length; index++) {
			var origin = this._origins[index];
			var originName = origin.title;
			var originUri = origin.uri;			
			if (!originName) {
				ErrorUtility.logError(new Error("Encountered a origin without a title"), this.id);
				continue;
			} else if (!originUri) {
				ErrorUtility.logError(new Error("Encountered a origin without a uri"), this.id);
				continue;
			}
			if (origin.instanceName) {
				originName += '&nbsp;<span style="color: gray">(' + origin.instanceName + ')</span>';
			}
			var info = applicationAbout.get(originUri.replace(/rootservices$/, 'application-about'));
			if (!info || !info.icon) info = {'icon': dojo.moduleUrl('jazz.ensemble','internal/graphics/widget-catalog/images/friend_server_icon.gif').toString()};
			var optionElement = {
				value: index,
				label: originName,
				icon: info.icon
			};			
			if (originUri === BootstrapProperties.FRONTSIDE_URL+'/rootservices') {
				optionElement.here = 1;
			}

			if (origin.isFriend) {
				friendServers.push(optionElement);
			} else {
				otherServers.push(optionElement);
			}
		}
		var byLabel = function(a, b) {
			if (a.here) return -1;
			if (b.here) return 1;
			if (a.label.toLowerCase() < b.label.toLowerCase()) return -1;
			if (a.label.toLowerCase() > b.label.toLowerCase()) return 1;
			return 0;
		};
		if (otherServers.length !== 0) {
			var first = otherServers[0];
			otherServers.sort(byLabel);
			optionElements[this.msg.applications] = otherServers;
		}
		if (friendServers.length !== 0) {
			if (!first) first = friendServers[0];
			friendServers.sort(byLabel);
			optionElements[this.msg.friends] = friendServers;
		 }
		
		this._selector = new jazz.ui.internal.Select({
			defaultEntry: first,
			menuItems: optionElements,
			onChange: dojo.hitch(this, this._onSelectChange)
		}).placeAt(this._catalogPickerSite, "only");

		if (!this._resizeHandler) {
			this._resizeHandler = jazz.util.ResizeNotifier.register({context: this, callback: this._onResize, force: true});
		}
		this._onResize();
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return Object
 	 */		
	_onSelectChange: function(entry) {
		this._selectedOriginIndex = entry.value;
		this._initCategories();
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return Object
 	 */		
	_getSelectedOrigin: function() {
		if ((this._origins instanceof Array) === false) {
			return null;
		}
		return this._origins[this._selectedOriginIndex];
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_addCategories: function() {
		if (this._getCategoriesArea()) {
			this._replaceCategories();
		} else {
			this._appendCategories();
		}
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_appendCategories: function() {
		var categoryContainer = this._initializeCategoriesArea();
		this._categorySite.appendChild(categoryContainer);
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_replaceCategories: function() {
		var oldContainer = this._getCategoriesArea();
		var newContainer = this._initializeCategoriesArea();
		this._categorySite.replaceChild(newContainer, oldContainer);
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_removeCategories: function() {
		var oldContainer = this._getCategoriesArea();
		this._categorySite.removeChild(oldContainer);
	},	
	
	/*
 	 * @param void
 	 * 
 	 * @return Element
 	 */		
	_initializeCategoriesArea: function() {
		
		var categories = this._getSortedCategories();
		var categoryContainer = document.createElement("div");
		dijit.setWaiRole(categoryContainer, "listbox");
		
		for (var index = 0; index < categories.length; index++) {
			var category = categories[index];
			this._createCategory(category, categoryContainer);
		}
		
		return categoryContainer;
	},

	/*
 	 * @param void
 	 * 
 	 * @return Element
 	 */		
	_getCategoriesArea: function() {
		return this._categorySite.firstChild;
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_shutdownCategories: function() {
		this._deleteCategories(this._categoryIndex);
		this._selectedCategory = null;
		this._categoryIndex = {};
	},

	/*
 	 * @param Object index
 	 * 
 	 * @return void
 	 */		
	_deleteCategories: function(index) {
		for (var name in index) {
			if (index.hasOwnProperty(name)) {
				var category = index[name];
				this.disconnect(category.handler);
				this.disconnect(category.keyPressHandler);
			} 
		}
	},

	/*
 	 * @param void
 	 * 
 	 * @return dojo.Deferred
 	 */		
	_initCategories: function() {
		
		var originUri = this._getSelectedOrigin().uri;
		var deferred = this.provider.getCategories(originUri);
		this._showLoadingMessage(true);
		deferred.addCallback(dojo.hitch(this, function(categories) {
			this._showLoadingMessage(false);
			this._setupCategories(categories);
			this._addCategories();
			this._fireEventOnClickCategory(this.msg.all);
			return categories; 
		}));
		deferred.addErrback(dojo.hitch(this, function(errorObject) {
			this._showLoadingMessage(false);
			ErrorUtility.logError(errorObject, this.id);
			this._showErrorMessage(this.msg.getCategoriesFailed);
			this._shutdownCategories();
			this._removeCategories();
			return errorObject; 
		}));
		
		return deferred;
	},

	/*
 	 * @param Array categories
 	 * 
 	 * @return void
 	 */		
	_setupCategories: function(categories) {

		// TEST
		//console.debug(categories);

		this._shutdownCategories();

		this._categoryIndex[this.msg.all] = {}; 

		for (var count = 0; count < categories.length; count++) {
			var category = categories[count];
			if (!category) {
				ErrorUtility.logError(new Error("Undefined category entry encountered"), this.id);
				continue;
			}
			this._categoryIndex[category] = {};
		}

		// TEST
		//console.debug(this._categoryIndex);
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initFilterBox: function() {
		
		this._filterBox = new FilterBox({});
		this._filterBoxSite.appendChild(this._filterBox.domNode);
		// TODO: MP: BUG: This is firing on initialization
		this._filterBox.onChange = dojo.hitch(this, this._handleEventOnChangeFilterBox);
		this._filterBox.onEnter = dojo.hitch(this, this._handleEventOnEnterFilterBox);
		
	},

	/*
 	 * @param String value
 	 * 
 	 * @return void
 	 */		
	_handleEventOnChangeFilterBox: function(value) {
		
		this._abortFilterBoxRequest();
		
		this._filterTimeout = setTimeout(
			dojo.hitch(this, this._fireEventFilterByText), 
			300);
		
	},

	/*
	 * @return void
	 */
	_handleEventOnEnterFilterBox: function() {
		var originUri = this._getSelectedOrigin().uri;
		var category = this._selectedCategory;
		var filter = this._filterBox.value;

		if (category === this.msg.all) {
			category = null;
		}
		
		var deferred = this.provider.getEntries(originUri, category, filter, 0, 1);
		deferred.addCallback(dojo.hitch(this, function(results) {
			if (results.total > 0) {			
				this._addWidgetToMashUp(results.entries[0]);
			}
			return results;
		}));
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_abortFilterBoxRequest: function() {
		if (this._filterTimeout) {
			clearTimeout(this._filterTimeout);
			this._filterTimeout = null;
		}		
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_fireEventFilterByText: function() {
		
		this._filterTimeout = null;
		
		// Update widget results pane
		var originUri = this._getSelectedOrigin().uri;
		var category = this._selectedCategory;
		var filter = this._filterBox.value;
		
		this._renderWidgetResultsPage(originUri, category, filter);		
	},

	/*
 	 * @param void
 	 * 
 	 * @return Array
 	 */		
	_getSortedCategories: function() {
		
		var sortedCategories = [];
		
		for (var name in this._categoryIndex) {
			if (this._categoryIndex.hasOwnProperty(name)) {
				sortedCategories.push(name);
			} 
		}
		
		sortedCategories.sort(dojo.hitch(this, this._sortCategories));

		return sortedCategories;
	},

	/*
 	 * @param String name1
 	 * @param String name2
 	 * 
 	 * @return Number
 	 */
	_sortCategories: function(name1, name2) {
		
		if (name1 === this.msg.all) {
			// Make 'All' category always the first category
			return -1;
		}
		
		var categoryName1 = name1.toLowerCase();
		var categoryName2 = name2.toLowerCase();
		
		if (categoryName1 < categoryName2) {
			// Less than
			return -1;
		} else if (categoryName1 > categoryName2) {
			// Greater than
			return 1;
		}
		// Equal
		return 0;
	},

	/*
 	 * @param String categoryName
 	 * @param Element parentElement
 	 * 
 	 * @return void
 	 */		
	_createCategory: function(categoryName, parentElement) {

		if (!categoryName) {
			ErrorUtility.logError(new Error("Undefined category name encountered"), this.id);
			return;
		} else if ((categoryName in this._categoryIndex) === false) {
			ErrorUtility.logError(new Error("Category not found in index: " + categoryName), this.id);
			return;
		} 
		
		// Anchor instead of div to avoid :hover selector perf problems
		var categoryElement = document.createElement("a");
		categoryElement.href = "#";
		dojo.style(categoryElement, 'display', 'block');

		dijit.setWaiRole(categoryElement, "option");
		dijit.setWaiState(categoryElement, "label", categoryName);
		dijit.setWaiState(categoryElement, "selected", "false");
		
		categoryElement.tabIndex = 0;
		dojo.addClass(categoryElement, "category");

		var nameTextNode = document.createTextNode(categoryName);
		categoryElement.appendChild(nameTextNode);
		
		var handler = this.connect(categoryElement, "onclick", function(event) {
			if (event) {
				dojo.stopEvent(event);
			}
			
			this._fireEventOnClickCategory(categoryName);
			
			return true;
		});

		var keyPressHandler = this.connect(categoryElement, "onkeypress", function(event) {
			if (!event) {
				return false;
			}
			
			if (event.keyCode === dojo.keys.ENTER) {
				this._fireEventOnClickCategory(categoryName);
			}
			
			return true;
		});

		parentElement.appendChild(categoryElement);

		this._categoryIndex[categoryName].domNode = categoryElement;
		this._categoryIndex[categoryName].handler = handler;
		this._categoryIndex[categoryName].keyPressHandler = keyPressHandler;
		
	},

	/*
 	 * @param String categoryName
 	 * 
 	 * @return void
 	 */
	_fireEventOnClickCategory: function(categoryName) {

		if (categoryName === this._selectedCategory) {
			// Nothing to do
			return;
		}

		// Remove old selection
		this._setCategorySelection(this._selectedCategory, this._categoryIndex, false);
		// Add new selection
		this._setCategorySelection(categoryName, this._categoryIndex, true);

		this._clearFilterBoxContents();
		
		// Update widget results pane
		var originUri = this._getSelectedOrigin().uri;
		this._renderWidgetResultsPage(originUri, categoryName, null);
		
	},

	/*
 	 * @param String originUri
 	 * @param String category
 	 * @param String filter
 	 * 
 	 * @return void
 	 */		
	_renderWidgetResultsPage: function(originUri, category, filter) {
		
		if (category === this.msg.all) {
			category = null;
		}
		
		var deferred = this._widgetResultsPane.renderPage(this.provider, originUri, category, filter);
		
		deferred.addCallback(dojo.hitch(this, function(results) {
			this._showWidgetResultsPane();
			return results; 
		}));
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */
	_clearFilterBoxContents: function() {
		if (this._filterBox &&
			this._filterBox.value) {
			this._filterBox.clear();
		}
	},

	/*
 	 * @param String categoryName
 	 * @param Object index
 	 * @param Boolean select
 	 * 
 	 * @return void
 	 */		
	_setCategorySelection: function(categoryName, index, select) {

		if (!categoryName) {
			return;
		} else if ((categoryName in index) === false) {
			return;
		} 

		var domNode = index[categoryName].domNode;
		
		if (select) {
			if (categoryName !== this._selectedCategory) {
				dojo.addClass(domNode, "selected");
				dijit.setWaiState(domNode, "selected", "true");
				this._selectedCategory = categoryName;
			}
		} else {
			if (categoryName === this._selectedCategory) {
				dojo.removeClass(domNode, "selected");
				dijit.setWaiState(domNode, "selected", "false");
				this._selectedCategory = null;				
			}
		}

	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initAddGadgetLink: function() {
		
		this.connect(this._addGadgetLink, "onclick", function(event) {
			if (event) {
				dojo.stopEvent(event);
			}

			if (!this._addGadgetPane) {
				this._addGadgetPane = new AddExternalWidgetPane({
					catalog: this,
					type: TYPE_GADGET
				});
				this._contentArea.appendChild(this._addGadgetPane.domNode);
			}

			this._setSelectedPane(this._addGadgetPane);
			
			return true;
			
		});
		
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initAddIWidgetLink: function() {
		
		this.connect(this._addIWidgetLink, "onclick", function(event) {
			if (event) {
				dojo.stopEvent(event);
			}

			if (!this._addIWidgetPane) {
				this._addIWidgetPane = new AddExternalWidgetPane({
					catalog: this,
					type: TYPE_IWIDGET
				});
				this._contentArea.appendChild(this._addIWidgetPane.domNode);
			}

			this._setSelectedPane(this._addIWidgetPane);
			
			return true;
			
		});
		
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initWidgetResultsPane: function() {
		this._widgetResultsPane = new WidgetResultsPane({
			catalog: this,
			widgetsPerPage: WIDGETS_PER_PAGE
		});
		this._contentArea.appendChild(this._widgetResultsPane.domNode);
		
		this._selectedPane = this._widgetResultsPane;
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_showWidgetResultsPane: function() {
		this._setSelectedPane(this._widgetResultsPane);
	},

	/*
 	 * @param Widget pane
 	 * 
 	 * @return void
 	 */		
	_setSelectedPane: function(pane) {
		
		this._showLoadingMessage(false);
		
		// Note: Pending requests are not aborted
		// There exists a chance for a category / entry service call to hijack 
		// the add external widget pane
		
		// De-select old pane 		
		if (this._selectedPane) {
			CatalogUtility._setNodeVisibility(this._selectedPane.domNode, false);
		}
		// Select new pane
		this._selectedPane = pane;
		CatalogUtility._setNodeVisibility(this._selectedPane.domNode, true);
	},

	/*
 	 * @param Object data
 	 * 
 	 * @return void
 	 */		
	_showWidgetDetailsPane: function(data) {
		
		if (this._widgetDetailsPane) {
			// Update
			this._widgetDetailsPane._updateFields(data);
		} else {
			// New
			this._widgetDetailsPane = new WidgetDetailsPane({
				catalog: this,
				data: data
			});
			this._contentArea.appendChild(this._widgetDetailsPane.domNode);
		}
		
		this._setSelectedPane(this._widgetDetailsPane);
	},

	/*
 	 * @param Number type
 	 * @param String summary
 	 * 
 	 * @return void
 	 */		
	_showStatusMessage: function(type, summary) {

		var messageBox = this._getMessageBox();
		messageBox.showMessage({ 
			type: type,
			summary: summary,
			closable: true
		});
	},

	/*
 	 * @param void
 	 * 
 	 * @return MessageBox
 	 */		
	_getMessageBox: function() {
		if (!this._messageBox) {
			this._messageBox = new MessageBox().placeAt(this._messageBoxSite);
		}
		return this._messageBox;
	},

	/*
 	 * @param String message
 	 * 
 	 * @return void
 	 */		
	_showErrorMessage: function(message) {
		
		this._setNodeText(this._errorMessagePane, message);
		
		this._setSelectedPane({
			domNode: this._errorMessagePane
		});
	},

	/*
 	 * @param Boolean show
 	 * 
 	 * @return void
 	 */		
	_showLoadingMessage: function(show) {
		// TODO: MP: Fade-out loading message
		CatalogUtility._setNodeVisibility(this._loadingMessage, show);
	},

	/*
 	 * @param Element node
 	 * @param String text
 	 * 
 	 * @return void
 	 */		
	_setNodeText: function(node, text) {

		var textNode = document.createTextNode(text);

		if (node.firstChild) {
			node.replaceChild(textNode, node.firstChild);
		} else {
			node.appendChild(textNode);
		}
	},

	/*
 	 * @param Object data
 	 * 
 	 * @return dojo.Deferred
 	 */
	_addWidgetToMashUp: function(data) {
		// TEST
		//console.debug(data);
		
		var origin = this._getSelectedOrigin();
		
		var deferred = this.onAddWidget(data, origin.uri, origin.local);
		
		deferred.addCallback(dojo.hitch(this, function(status) {
			return status; 
		}));
		deferred.addErrback(dojo.hitch(this, function(errorObject) {
			ErrorUtility.logError(errorObject, this.id);
			return errorObject; 
		}));		
	
		return deferred;
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	destroy: function() {
		if (this._selector) {
			this._selector.destroy();
			this._selector = null;
		}
		if (this._resizeHandler) {
			jazz.util.ResizeNotifier.unregister(this._resizeHandler);
			this._resizeHandler = null;
		}
		if (this._widgetResultsPane) {
			this._widgetResultsPane.destroy();
			this._widgetResultsPane = null;
		}
		if (this._widgetDetailsPane) {
			this._widgetDetailsPane.destroy();
			this._widgetDetailsPane = null;
		}
		if (this._addGadgetPane) {
			this._addGadgetPane.destroy();
			this._addGadgetPane = null;
		}
		if (this._addIWidgetPane) {
			this._addIWidgetPane.destroy();
			this._addIWidgetPane = null;
		}
		if (this._filterBox) {
			this._filterBox.destroy();
			this._filterBox = null;
		}
		if (this._messageBox) {
			this._messageBox.destroy();
			this._messageBox = null;
		}
		if (this._selectHelp) {
			this._selectHelp.destroy();
			this._selectHelp = null;
		}
		if (this._widgetsHelp) {
			this._widgetsHelp.destroy();
			this._widgetsHelp = null;
		}
		
		// Provider
		
		if (this.provider) {
			this.provider.destroy();
			this.provider = null;
		}

		// Categories
		
		this._shutdownCategories();
		
		// Filter Box
		
		this._abortFilterBoxRequest();
		
		
		this.inherited(arguments);
	}

});

})();
