/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ensemble.internal.catalog.WidgetDetailsPane");

/*
 * Imports 
 */
dojo.require("dojo.i18n"); 

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.ensemble.internal.catalog.CatalogUtility");
dojo.require("jazz.ensemble.internal.catalog.AddWidgetButton");

dojo.requireLocalization("jazz.ensemble.internal", "WidgetCatalogMessages");

/*
 * 
 */

(function() {

/*
 * Aliases 
 */
var CatalogUtility = jazz.ensemble.internal.catalog.CatalogUtility;
var AddWidgetButton = jazz.ensemble.internal.catalog.AddWidgetButton;

/*
 * Constants
 */
var GADGET_TYPE = "org.opensocial.gadget";

/*
 * 
 */
dojo.declare("jazz.ensemble.internal.catalog.WidgetDetailsPane",
	[dijit._Widget, dijit._Templated], {

	// Properties //----------------------------------------------------------->

	catalog: null,
	data: null,
	
	// ------------------------------------------------------------------------>
	
	/*
	 * Templates
	 */	
	templatePath: dojo.moduleUrl("jazz.ensemble", "internal/catalog/templates/WidgetDetailsPane.html"),
	
	/*
 	 * @param Object args
 	 * 
 	 * @return WidgetDetailsPane
 	 */
	constructor: function(args) {
		
		// Messages //--------------------------------------------------------->

		this.msg = dojo.i18n.getLocalization("jazz.ensemble.internal", "WidgetCatalogMessages");

		// Images //----------------------------------------------------------->

		this._gadgetUrlIcon = dojo.moduleUrl("jazz.ensemble", "internal/graphics/widget-catalog/images/hyperlink.gif");
		
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */	
	postCreate: function() {

		this.inherited(arguments);
		
		// Instance Variables //----------------------------------------------->

		this._addWidgetButton = null;

		// Initialization //--------------------------------------------------->
		
		this._initFields();
		this._initHandlers();
		
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initFields: function() {
		this._initPreview();
		this._initTitle();
		this._initAuthor();
		this._initDescription();
		this._initGadgetURL();
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initHandlers: function() {
		this._initBackLink();
		this._initAddWidgetButton();
	},

	/*
 	 * @param Object data
 	 * 
 	 * @return void
 	 */		
	_updateFields: function(data) {
		this.data = data;
		this._initFields();
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initBackLink: function() {
		
		this.connect(this._backLink, "onclick", function(event) {
			if (event) {
				dojo.stopEvent(event);
			}
			
			this.catalog._showWidgetResultsPane();
			
			return true;
		});	
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initPreview: function() {

		var preview = this.data.preview;
		var title = this.data.title;
		
		if (!preview) {
			CatalogUtility._setNodeVisibility(this._screenshot, false);
			CatalogUtility._setNodeVisibility(this._noScreenshot, true);
		} else {
			CatalogUtility._setNodeVisibility(this._noScreenshot, false);
			CatalogUtility._setNodeVisibility(this._screenshot, true);
			this._screenshot.src = preview;
			this._screenshot.alt = title;
		}
		
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initTitle: function() {
		this._setNodeText(this._title, this.data.title);
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initAuthor: function() {
		
		if (this.data.author) {
			CatalogUtility._setNodeVisibility(this._authorSite, true);
			this._setNodeText(this._author, this.data.author);
		} else {
			CatalogUtility._setNodeVisibility(this._authorSite, false);
		}
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initDescription: function() {
		
		var description = this.data.description;
		if (!description) {
			description = this.msg.noDescription;
		}
		
		this._setNodeText(this._description, description);
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initGadgetURL: function() {

		if (this.data.type === GADGET_TYPE) {
			this._gadgetUrl.href = this.data.widgetUri;
			CatalogUtility._setNodeVisibility(this._gadgetUrlSite, true);
		} else {
			CatalogUtility._setNodeVisibility(this._gadgetUrlSite, false);
		}
		
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initAddWidgetButton: function() {

		this._addWidgetButton = new AddWidgetButton({
			catalog: this.catalog,
			onAddWidgetHandler: dojo.hitch(this, function(status) {
				var deferred = this.catalog._addWidgetToMashUp(this.data);
				return deferred;
			})
		});
		this._addWidgetButtonSite.appendChild(this._addWidgetButton.domNode);				

	},

	/*
 	 * @param Element node
 	 * @param String text
 	 * 
 	 * @return void
 	 */		
	_setNodeText: function(node, text) {
		// TODO: MP: Catalog utility function?
		var textNode = document.createTextNode(text);

		if (node.firstChild) {
			node.replaceChild(textNode, node.firstChild);
		} else {
			node.appendChild(textNode);
		}
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	destroy: function() {

		if (this._addWidgetButton) {
			this._addWidgetButton.destroy();
			this._addWidgetButton = null;
		}
		
		this.inherited(arguments);
	}

});

})();
