/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ensemble.internal.catalog.WidgetEntry");

/*
 * Imports 
 */
dojo.require("dojo.i18n"); 
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.ensemble.internal.catalog.AddWidgetButton");

dojo.requireLocalization("jazz.ensemble.internal", "WidgetCatalogMessages");

(function() {

/*
 * Aliases 
 */
var AddWidgetButton = jazz.ensemble.internal.catalog.AddWidgetButton;

/*
 * Constants 
 */

/*
 * Class 
 */
dojo.declare("jazz.ensemble.internal.catalog.WidgetEntry",
	[dijit._Widget, dijit._Templated], {

	// Properties //----------------------------------------------------------->

	data: null,
	catalog: null,

	// ------------------------------------------------------------------------>

	/*
 	 * Template
 	 */
	templatePath: dojo.moduleUrl("jazz.ensemble", "internal/catalog/templates/WidgetEntry.html"),
	
	/*
 	 * @param Object args
 	 * 
 	 * @return WidgetEntry 
 	 */
	constructor: function(args) {
		
		// Messages //------------------------------------------------------------->

		this.msg = dojo.i18n.getLocalization("jazz.ensemble.internal", "WidgetCatalogMessages");
		
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	postCreate: function() {

		this.inherited(arguments);

		// Instance Variables //----------------------------------------------->

		this._addWidgetButton = null;

		// Initialization //--------------------------------------------------->

		this._initFields();
		// TODO: MP: Make widget reusable / updatable
		// TODO: MP: Remember to not add connects when updating
		
	},	

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initFields: function() {
		
		this._initThumbnail();
		this._initTitle();
		this._initDescription();
		this._initAddWidgetButton();

	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initThumbnail: function() {

		var thumbnail = this.data.thumbnail;
		if (!thumbnail) {
			var preview = this.data.preview;
			if (preview) {
				this._thumbnail.src = preview;
			} else {
				this._thumbnail.src = dojo.moduleUrl("jazz.ensemble", "internal/graphics/widget-catalog/images/foundation-thumbnail.png");
			}
		} else {
			this._thumbnail.src = thumbnail;
		}
		this._thumbnail.alt = this.data.title;
		
		this.connect(this._thumbnailLink, "onclick", this._handleEventOnClickDetailsLink);			

	},

	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */		
	_handleEventOnClickDetailsLink: function(event) {
		if (event) {
			dojo.stopEvent(event);
		}

		this.catalog._showWidgetDetailsPane(this.data);
			
		return true;
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initTitle: function() {
		this._setNodeText(this._title, this.data.title);
		
		this.connect(this._title, "onclick", this._handleEventOnClickDetailsLink);			
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initDescription: function() {
		
		var description = this.data.description;
		if (!description) {
			description = this.msg.noDescription;
		}
		
		description = this._truncateDescription(description);
		
		this._setNodeText(this._description, description);
	},

	/*
 	 * @param String description
 	 * 
 	 * @return String
 	 */		
	_truncateDescription: function(description) {
		
		// PRE: Description is defined
		
		var descCharLimit = this._getDescCharLimit();
		
		if (description.length < descCharLimit) {
			return description;
		}
		var ellipsis = "...";
		var dotLimit = descCharLimit - ellipsis.length;
		var truncatedText = description.substring(0, dotLimit);
		
		// Search backwards for the first space
		var spaceIndex = truncatedText.lastIndexOf(" ");
		if (spaceIndex < 0) {
			// No spaces
			return truncatedText + ellipsis;
		}
		var resultText = truncatedText.substring(0, spaceIndex) + " " + ellipsis;
		
		return resultText;
		
	},

	/*
 	 * @param void
 	 * 
 	 * @return Number
 	 */		
	_getDescCharLimit: function() {
		if (djConfig.locale.match(/^ko/)) {
			// Korean
			return 65;
		} else if (djConfig.locale.match(/^ja/)) {
			// Japanese
			return 50;
		} else if (djConfig.locale.match(/^zh/)) {
			// Chinese
			return 50;
		} else if (djConfig.locale.match(/^ru/)) {
			// Russian
			return 80;
		}
		// Rest
		return 100;
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initAddWidgetButton: function() {
		
		var highlightNode = this._entryBackground;
		this._addWidgetButton = new AddWidgetButton({
			catalog: this.catalog,
			onAddWidgetHandler: dojo.hitch(this, function(status) {
				var deferred = this.catalog._addWidgetToMashUp(this.data);
				return deferred;
			}),
			highlightNode: highlightNode
		});
		this._addWidgetButtonSite.appendChild(this._addWidgetButton.domNode);
		
	},

	/*
 	 * @param Element node
 	 * @param String text
 	 * 
 	 * @return void
 	 */		
	_setNodeText: function(node, text) {
		// TODO: MP: Catalog utility function?
		var textNode = document.createTextNode(text);

		if (node.firstChild) {
			node.replaceChild(textNode, node.firstChild);
		} else {
			node.appendChild(textNode);
		}
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	destroy: function() {
		
		if (this._addWidgetButton) {
			this._addWidgetButton.destroy();
			this._addWidgetButton = null;
		}
		
		this.inherited(arguments);
	}
	
});

})();
