/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ensemble.internal.catalog.WidgetResultsPane");

/*
 * Imports 
 */
dojo.require("dojo.i18n"); 

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.ui.PaginationControl");

dojo.require("jazz.ensemble.internal.catalog.WidgetEntry");
dojo.require("jazz.ensemble.internal.catalog.ErrorUtility");
dojo.require("jazz.ensemble.internal.catalog.CatalogUtility");

dojo.requireLocalization("jazz.ensemble.internal", "WidgetCatalogMessages");

/*
 * 
 */

(function() {

/*
 * Aliases 
 */
var PaginationControl = jazz.ui.PaginationControl;
var WidgetEntry = jazz.ensemble.internal.catalog.WidgetEntry;
var ErrorUtility = jazz.ensemble.internal.catalog.ErrorUtility;
var CatalogUtility = jazz.ensemble.internal.catalog.CatalogUtility;

/*
 * 
 */
dojo.declare("jazz.ensemble.internal.catalog.WidgetResultsPane",
	[dijit._Widget, dijit._Templated], {

	// Properties //----------------------------------------------------------->

	catalog: null,
	widgetsPerPage: 6,
	
	// ------------------------------------------------------------------------>
	
	/*
	 * Templates
	 */	
	templatePath: dojo.moduleUrl("jazz.ensemble", "internal/catalog/templates/WidgetResultsPane.html"),
	
	/*
 	 * @param Object args
 	 * 
 	 * @return WidgetResultsPane
 	 */
	constructor: function(args) {
		
		// Messages //------------------------------------------------------------->
		
		this.msg = dojo.i18n.getLocalization("jazz.ensemble.internal", "WidgetCatalogMessages");

	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */	
	postCreate: function() {

		this.inherited(arguments);

		// Instance Variables //----------------------------------------------->

		this._pages = [];
		this._entries = [];
		this._selectedPageIndex = 0;
		this._currentPage = null;
		this._paginationControl = null;

		this._provider = null;
		this._originUri = null;
		this._category = null;
		this._filter = null;

		// Initialization //--------------------------------------------------->

		
	},

	/*
 	 * @param Number total
 	 * 
 	 * @return Element
 	 */		
	_createPaginationControl: function(total) {
		this._paginationControl = new PaginationControl({
			total: total,
			increment: this.widgetsPerPage,
			onNextClick: dojo.hitch(this, this._handleEventOnNextClick),
			onPreviousClick: dojo.hitch(this, this._handleEventOnPreviousClick)
		});
		return this._paginationControl.domNode;
	},

	/*
 	 * @param PaginationControl widget
 	 * @param Number start
 	 * @param Number end
 	 * 
 	 * @return void
 	 */		
	_handleEventOnNextClick: function(widget, start, end) {
		
		var currentPage = this._getCurrentPage();
		var index = this._getPageCalculatedIndex(start, end);
		var nextPage = this._getPage(index);

		// Make the next page the current page
		this._selectedPageIndex = index;

		if (nextPage) {
			this.catalog._showLoadingMessage(false);			
			// Hide the current page
			if (currentPage) {
				CatalogUtility._setNodeVisibility(currentPage, false);
			}
			// Show the next page (cached)
			CatalogUtility._setNodeVisibility(nextPage, true);
			this._setCurrentPage(nextPage);
			
			return;
		}
		
		// New page
		var deferred = this._provider.getEntries(this._originUri, this._category, this._filter, start - 1, this.widgetsPerPage);
		var pageIndex = this._getPageIndex();

		this.catalog._showLoadingMessage(true);
		deferred.addCallback(dojo.hitch(this, function(results) {
			this.catalog._showLoadingMessage(false);
			this._processNextResults(results, pageIndex);
			return results; 
		}));
		deferred.addErrback(dojo.hitch(this, function(errorObject) {
			this.catalog._showLoadingMessage(false);
			this._processGetEntriesFailure(errorObject);
			return errorObject; 
		}));	

	},

	/*
 	 * @param Error errorObject
 	 * 
 	 * @return void
 	 */		
	_processGetEntriesFailure: function(errorObject) {
		ErrorUtility.logError(errorObject, this.id);
		ErrorUtility.logError(new Error("Error encountered retrieving widget entries from provider"), this.id);
		this.catalog._showErrorMessage(this.msg.getEntriesFailed);
	},

	/*
 	 * @param Object results
 	 * @param Number pageIndex
 	 * 
 	 * @return void
 	 */		
	_processNextResults: function(results, pageIndex) {
		
		if (results.total === 0) {
			this._processGetEntriesFailure(new Error("No widget results found"));
			return;
		}

		var currentPage = this._getCurrentPage();
		var nextPage = this._createPage(results.entries);

		// Hide the current page
		if (currentPage) {
			CatalogUtility._setNodeVisibility(currentPage, false);
		}
		// Show the next page (new)
		this._addPage(nextPage, pageIndex);
		
	},

	/*
 	 * @param PaginationControl widget
 	 * @param Number start
 	 * @param Number end
 	 * 
 	 * @return void
 	 */		
	_handleEventOnPreviousClick: function(widget, start, end) {

		var currentPage = this._getCurrentPage();
		var index = this._getPageCalculatedIndex(start, end);
		var previousPage = this._getPage(index);

		// Make the previous page the current page
		this._selectedPageIndex = index;

		if (previousPage) {
			this.catalog._showLoadingMessage(false);			
			// Hide the current page
			if (currentPage) {
				CatalogUtility._setNodeVisibility(currentPage, false);
			}
			// Show the previous page (cached)
			CatalogUtility._setNodeVisibility(previousPage, true);
			this._setCurrentPage(previousPage);
			
			return;
		}

		// New page
		var deferred = this._provider.getEntries(this._originUri, this._category, this._filter, start - 1, this.widgetsPerPage);
		var pageIndex = this._getPageIndex();

		this.catalog._showLoadingMessage(true);
		deferred.addCallback(dojo.hitch(this, function(results) {
			this.catalog._showLoadingMessage(false);
			this._processPreviousResults(results, pageIndex);
			return results; 
		}));
		deferred.addErrback(dojo.hitch(this, function(errorObject) {
			this.catalog._showLoadingMessage(false);
			this._processGetEntriesFailure(errorObject);
			return errorObject; 
		}));	

	},

	/*
 	 * @param Object results
 	 * @param Number pageIndex
 	 * 
 	 * @return void
 	 */		
	_processPreviousResults: function(results, pageIndex) {
		
		if (results.total === 0) {
			this._processGetEntriesFailure(new Error("No widget results found"));
			return;
		}

		var currentPage = this._getCurrentPage();
		var previousPage = this._createPage(results.entries);

		// Hide the current page
		if (currentPage) {
			CatalogUtility._setNodeVisibility(currentPage, false);
		}
		// Show the previous page (new)
		this._addPage(previousPage, pageIndex);
		
	},

	/*
 	 * @param Object provider
 	 * @param String originUri
 	 * @param String category
 	 * @param String filter
 	 * 
 	 * @return dojo.Deferred
 	 */		
	renderPage: function(provider, originUri, category, filter) {
		
		this._provider = provider;
		this._originUri = originUri;
		this._category = category;		
		this._filter = filter;
		
		var deferred = this._provider.getEntries(this._originUri, this._category, this._filter, 0, this.widgetsPerPage);

		this.catalog._showLoadingMessage(true);
		deferred.addCallback(dojo.hitch(this, function(results) {
			this.catalog._showLoadingMessage(false);
			this._processInitialResults(results);
			return results; 
		}));
		deferred.addErrback(dojo.hitch(this, function(errorObject) {
			this.catalog._showLoadingMessage(false);
			this._processGetEntriesFailure(errorObject);
			return errorObject; 
		}));	
		
		return deferred;
	},

	/*
 	 * @param Object results
 	 * 
 	 * @return void
 	 */		
	_processInitialResults: function(results) {
		
		this._deletePages();
		this._deletePaginationControl();
		
		if (results.total > 0) {
			// Show widget results
			CatalogUtility._setNodeVisibility(this._messageArea, false);

			var firstPage = this._createPage(results.entries);
			this._addPage(firstPage, 0);
		
			var paginationElement = this._createPaginationControl(results.total);
			this._footer.appendChild(paginationElement);
		} else {
			// Show no results message
			CatalogUtility._setNodeVisibility(this._messageArea, true);
		}
	},

	/*
 	 * @param String pageIndex
 	 * 
 	 * @return Element
 	 */		
	_getPage: function(pageIndex) {
		if (pageIndex < 0) {
			return null;
		} else if (pageIndex >= this._pages.length) {
			return null;
		}
		
		return this._pages[pageIndex];
	},

	/*
 	 * @param void
 	 * 
 	 * @return Number
 	 */		
	_getPageIndex: function() {
		return this._selectedPageIndex;
	},

	/*
 	 * @param void
 	 * 
 	 * @return Element
 	 */		
	_getCurrentPage: function() {
		return this._currentPage; 
	},

	/*
 	 * @param Element pageElement
 	 * 
 	 * @return void
 	 */		
	_setCurrentPage: function(pageElement) {
		this._currentPage = pageElement; 
	},

	/*
 	 * @param start
 	 * @param end 
 	 * 
 	 * @return page index for the provided range
 	 */	
	_getPageCalculatedIndex: function(start, end) {
		return Math.floor(start/this.widgetsPerPage);
	},

	/*
 	 * @param Array entries
 	 * 
 	 * @return Element
 	 */		
	_createPage: function(entries) {
		
		var index;
		var entryCount = entries.length;
		var widgetsPerRow = 3;
		
		var pageElement = document.createElement("div");
		
		// PRE: At least one entry
		
		var firstRowWidgets;
		var secondRowWidgets;
		if (entryCount <= widgetsPerRow) {
			firstRowWidgets = entries;
			secondRowWidgets = null;
		} else {
			firstRowWidgets = entries.slice(0, widgetsPerRow);
			secondRowWidgets = entries.slice(widgetsPerRow, entryCount);
		}

		// Row 1
		var firstRowElement = this._createRow(firstRowWidgets);
		pageElement.appendChild(firstRowElement);
		
		// Row 2
		if (secondRowWidgets) {
			var secondtRowElement = this._createRow(secondRowWidgets);
			pageElement.appendChild(secondtRowElement);
		}		
		
		return pageElement;
	},

	/*
 	 * @param Element pageElement
 	 * @param Element pageIndex
 	 * 
 	 * @return void
 	 */		
	_addPage: function(pageElement, pageIndex) {
		
		if (pageIndex < 0) {
			ErrorUtility.logError(new Error("Invalid page index: " + pageIndex), this.id);
			return;
		}
		
		if (!this._widgetsSite.firstChild) {
			var container = document.createElement("div");
			this._widgetsSite.appendChild(container);
		}
		
		this._pages[pageIndex] = pageElement;
		this._widgetsSite.firstChild.appendChild(pageElement);
		
		this._setCurrentPage(pageElement);
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_deletePages: function() {

		// Entries
		for (var xIndex = 0; xIndex < this._entries.length; xIndex++) {
			var entry = this._entries[xIndex];
			entry.destroy();
		}
		this._entries = [];
		
		// Pages (DOM references)
		this._pages = [];
		this._selectedPageIndex = 0;
		this._currentPage = null;

		// Pages (on DOM)
		if (this._widgetsSite.firstChild) {
			this._widgetsSite.removeChild(this._widgetsSite.firstChild);
		}
		
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_deletePaginationControl: function() {
		if (this._paginationControl) {
			this._paginationControl.destroy();
			this._paginationControl = null;
		}
	},

	/*
 	 * @param Array entries
 	 * 
 	 * @return Element
 	 */		
	_createRow: function(entries) {
		
		var rowElement = document.createElement("div");
		dojo.addClass(rowElement, "w-row");
		
		for (var _i = 0; _i < entries.length; _i++) {
			var entry = entries[_i];
			var widgetElement = this._createEntry(entry);
			rowElement.appendChild(widgetElement);
		}

		return rowElement;
	},

	/*
 	 * @param Object entry
 	 * 
 	 * @return Element
 	 */		
	_createEntry: function(entry) {
		
		var widgetElement = document.createElement("div");
		dojo.addClass(widgetElement, "widget");
		
		var widgetEntry = new WidgetEntry({
			catalog: this.catalog,
			data: entry
		});
		widgetElement.appendChild(widgetEntry.domNode);

		this._addEntry(widgetEntry);

		return widgetElement;
	},

	/*
 	 * @param WidgetEntry widgetEntry
 	 * 
 	 * @return void
 	 */		
	_addEntry: function(widgetEntry) {
		this._entries.push(widgetEntry);
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	destroy: function() {
		
		this._deletePages();
		this._deletePaginationControl();
		
		this.inherited(arguments);
	}

});

})();
