/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("jazz.ensemble.internal.client.RestClient");

dojo.require("dojox.xml.parser");

dojo.require("jazz.ensemble.internal.client.HttpConstants");
dojo.require("jazz.client.xhr");

var HttpConstants = jazz.ensemble.internal.client.HttpConstants;

/*
 * An HTTP client that works with resource objects. Resource objects contain
 * the contents as well as metadata (last modified, content type, etc) from
 * headers.
 */
jazz.ensemble.internal.client.RestClient = {
	
	HANDLE_AS_XML: "xml",
	
	/*
	 * HEADs the resource at the given URL
	 */
	/* Deferred */ headResource: function(/* string */ url, /* object */ headers) {
		var dfd = new dojo.Deferred();
		var args = {
			url: url,
			load: function(/* any */ content, /* dojo.__XhrArgs */ xhrArgs) {
				var xhr = xhrArgs.xhr;
				var resource = {
					contentType: xhr.getResponseHeader(HttpConstants.CONTENT_TYPE),
					eTag: xhr.getResponseHeader(HttpConstants.ETAG),
					lastModified: xhr.getResponseHeader(HttpConstants.LAST_MODIFIED),
					lastModifiedUser: xhr.getResponseHeader(HttpConstants.LAST_MODIFIED_USER),
					lastModifiedXsd: xhr.getResponseHeader(HttpConstants.LAST_MODIFIED_XSD),
					securityContext: xhr.getResponseHeader(HttpConstants.SECURITY_CONTEXT_HEADER),
					url: url
				};
				dfd.callback(resource);
			},
			error: function(/* Error */ e) {
				dfd.errback(e);
			}
		};
		if (headers) {
			args.headers = headers;
		}
		jazz.client.xhrHead(args);
		return dfd;
	},
	
	/*
	 * DELETEs the resource at the given URL.
	 */
	/* Deferred */ deleteResource: function(/* string */ url) {
		var dfd = new dojo.Deferred();
		var args = {
			url: url,
			load: function(/* any */ content, /* dojo.__XhrArgs */ xhrArgs) {
				dfd.callback();
			},
			error: function(/* Error */ e) {
				dfd.errback(e);
			}
		};
		jazz.client.xhrDelete(args);
		return dfd;
	},
	
	/*
	 * GETs the resource at the given URL.
	 */
	/* Deferred */ getResource: function(/* string */ url, /* string */ handleAs, /* object */ headers) {
		var dfd = new dojo.Deferred();
		var args = {
			url: url,
			handleAs: handleAs,
			load: function(/* any */ content, /* dojo.__XhrArgs */ xhrArgs) {
				var xhr = xhrArgs.xhr;
				var resource = {
					content: content,
					contentType: xhr.getResponseHeader(HttpConstants.CONTENT_TYPE),
					eTag: xhr.getResponseHeader(HttpConstants.ETAG),
					lastModified: xhr.getResponseHeader(HttpConstants.LAST_MODIFIED),
					lastModifiedUser: xhr.getResponseHeader(HttpConstants.LAST_MODIFIED_USER),
					lastModifiedXsd: xhr.getResponseHeader(HttpConstants.LAST_MODIFIED_XSD),
					securityContext: xhr.getResponseHeader(HttpConstants.SECURITY_CONTEXT_HEADER),
					url: url
				};
				dfd.callback(resource);
			},
			error: function(/* Error */ e) {
				dfd.errback(e);
			}
		};
		if (headers) {
			args.headers = headers;
		}
		jazz.client.xhrGet(args);
		return dfd;
	},
	
	/*
	 * PUTs the given resource at its url.
	 */
	/* Deferred */ putResource: function(/* object */ resource) {
		var putData = resource.content;
		if (dojo.isIE) {
			/*
			 * 113794: IE automatically overrides the Content-Type header to "text/xml"
			 * when a Document is used as putData. Convert to string beforehand so we get
			 * the right content type.
			 */
			putData = dojox.xml.parser.innerXML(putData);
		}
		var dfd = new dojo.Deferred();
		var args = {
			url: resource.url,
			headers: _getRequestHeaders(resource),
			putData: putData,
			load: function(/* any */ content, /* dojo.__XhrArgs */ xhrArgs) {
				// update resource
				var xhr = xhrArgs.xhr;
				dojo.mixin(resource, {
					eTag: xhr.getResponseHeader(HttpConstants.ETAG),
					lastModified: xhr.getResponseHeader(HttpConstants.LAST_MODIFIED),
					lastModifiedUser: xhr.getResponseHeader(HttpConstants.LAST_MODIFIED_USER),
					lastModifiedXsd: xhr.getResponseHeader(HttpConstants.LAST_MODIFIED_XSD)
				});
				dfd.callback(resource);
			},
			error: function(/* Error */ e, /* dojo.__IoCallbackArgs */ args) {
				if (args && dojo.isObject(e)) {
					var xhr = args.xhr;
					if (xhr) {
						var statusText = xhr.statusText;
						if (statusText) {
							e.statusText = statusText;
						}
					}
				}
				dfd.errback(e);
			}
		};
		jazz.client.xhrPut(args);
		return dfd;
	}
};

/*
 * Returns an object containing the headers to be used for a request containing
 * the given resource.
 */
function _getRequestHeaders(/* Resource */ resource) {
	var headers = {};
	headers[HttpConstants.CONTENT_TYPE] = resource.contentType;
	if (resource.eTag) {
		headers[HttpConstants.IF_MATCH] = resource.eTag;
	}
	if (resource.lastModified) {
		headers[HttpConstants.IF_UNMODIFIED_SINCE] = resource.lastModified;
	}
	if (resource.securityContext) {
		headers[HttpConstants.SECURITY_CONTEXT_HEADER] = resource.securityContext;
	}
	return headers;
}

})();
