/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ensemble.internal.layout.Column");

dojo.require("jazz.ui.dnd.DraggableContainer");
dojo.require("jazz.ui.dnd.DraggableManager");
dojo.require("jazz.util.ResizeNotifier");
dojo.require("dijit._Container");

(function() {
	var draggableManager = jazz.ui.dnd.draggableManager();
	
	dojo.declare("jazz.ensemble.internal.layout.Column", [dijit._Widget, dijit._Container], {
		index: null,
		onChange: null,
		readOnly: false,
		relativeTo: null,
		onAddWidget: null,
		onRemoveWidget: null,
		layout: null,
		
		_column: null,
		
		postCreate: function() {
			dojo.addClass(this.domNode, "column" + (this.index+1));
			this._column = new jazz.ensemble.internal.layout._Column(this.domNode, {
				accept: ["widget"],
				enabled: !this.readOnly,
				columnIndex: this.index, 
				relativeTo: this.relativeTo,
				onStateChange: this.onChange,
				onAddWidget: this.onAddWidget,
				onRemoveWidget: this.onRemoveWidget,
				layout: this.layout
			});
		},
		add: function(item, index) {
			var args = {index: 0, handle: item.handle, type: "widget"};
			if (index !== null)	args.index = index;
			this._column.add(item, args);
		},
		remove: function(item) {
			var widget = dijit.byNode(item.domNode);
			this._column.remove(widget.getIndexInParent());
		},
		setEnabled: function(/* boolean */ value) {
			this._column.setEnabled(value);
		},
		destroy: function() {
			this._column.destroy();
			this.inherited(arguments);
		}
	});
	
	dojo.declare("jazz.ensemble.internal.layout._Column", jazz.ui.dnd.DraggableContainer, {
		constructor: function(domNode, params){
			this._disabled = true;   //default to disabled, until setEnabled(true) is called below
			this.useAvatar = false;
			this.index = params.columnIndex;
			this.relativeTo = params.relativeTo;
			this.containerNode = this.domNode = domNode;
			this.onStateChange = params.onStateChange;
			this.onAddWidget = params.onAddWidget;
			this.onRemoveWidget = params.onRemoveWidget;
			this.layout = params.layout;
			dojo.addClass(domNode,"jazz-ensemble-internal-layout-Column");
			if (this.index===0) {
				this.avatarClass = "column1";
			}
			this.setEnabled(params.enabled);
		},
		dragStopEvent: function(event) {
			if (event.target) {
				var pn = event.target.parentNode;
				if (pn) {
					var ppn = event.target.parentNode.parentNode;
					if (dojo.hasClass(pn, 'header') || (ppn && dojo.hasClass(ppn, 'header'))) {
						dojo.stopEvent(event);
					}
				}
			}
			this.inherited(arguments);
		},
		onMouseDown: function(event) {
			this.inherited(arguments);
			var widget = draggableManager.widget;
			if (widget && widget.isZoomed) {
				var preCoords = dojo.coords(event.currentTarget, true);
				var coords = dojo.coords(widget.zoomPlaceholder, true);
				var offsetX = event.pageX - preCoords.x;
				var offsetY = event.pageY - preCoords.y; 
				if (offsetX > coords.w - 80) {
					offsetX = coords.w - 80;
				}
				draggableManager.initialMouseCoords.offsetX = offsetX;
				draggableManager.initialMouseCoords.offsetY = offsetY;
			}
		},
		onStartDrag: function() {
			var cell = dojo.byId("content-cell");
			if (cell) cell.style.overflow = "visible";
			var widget = draggableManager.widget;
			if (widget && widget.isZoomed) {
				draggableManager.widget.toggleZoom(null, true);
				var coords = dojo.coords(draggableManager.widget.domNode,true);
				draggableManager.widgetCoords.h = coords.h;
				draggableManager.widgetCoords.w = coords.w;
			}
		},
		/* Move
		 * 
		 * return: cancelDrag - true/false 
		 * 
		 */
		move: function(fromIndex, toContainer, toIndex) {
			var fromContainer = draggableManager.fromContainer;
			var widgets = fromContainer.widgets;
			if (toContainer.layout != fromContainer.layout && toContainer.moveTo) {
				var widget = widgets[fromIndex];
				var success = toContainer.moveTo(widget, toIndex);	
				if (success) {
					this.onRemoveWidget(dijit.byNode(widget.domNode));
				} else {
					throw new Error();
				}
			} else {
				widget = widgets.splice(fromIndex,1)[0];
				toContainer.node.insertBefore(widget.domNode, this.getPlaceholder());
				toContainer.widgets.splice(toIndex,0,widget);			
				var cols = [fromContainer, toContainer];
				for (var j = 0; j < cols.length; j++) {
					this.updateIndices(cols[j]);
					if (fromContainer == toContainer) break;
				}
				this.onStateChange();
			}
		},

		moveTo: function(widget, toIndex) {
			var w = dijit.byNode(widget.domNode);
			this.onAddWidget(w.state.elem, this.node, toIndex);
			return true;
		},

		onDndStop: function(success) {
			this.inherited(arguments);
			var cell = dojo.byId("content-cell");
			if (cell) cell.style.overflow = "";
			dojo.publish("jazz.util.ResizeNotifier.resizeEvent");
		}
	});
})();