/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ensemble.internal.layout.ColumnLayout");

dojo.require("dijit._Templated");
dojo.require("jazz.ensemble.internal.Layout");
dojo.require("jazz.ensemble.internal.Schema");
dojo.require("jazz.ensemble.internal.layout.Column");
dojo.require("jazz.ensemble.internal.layout.ResizableColumns");
dojo.require("jazz.ensemble.internal.util.Xml");
dojo.require("jazz.ensemble.internal.util.XmlBindingFactory");

(function() {

var Column = jazz.ensemble.internal.layout.Column;
var ResizableColumns = jazz.ensemble.internal.layout.ResizableColumns;
var Schema = jazz.ensemble.internal.Schema;
var Xml = jazz.ensemble.internal.util.Xml;
var XmlBindingFactory = jazz.ensemble.internal.util.XmlBindingFactory;

var LAYOUT_ID_1 = "jazz.ensemble.layout.1column";
var LAYOUT_ID_2 = "jazz.ensemble.layout.2column";
var LAYOUT_ID_3 = "jazz.ensemble.layout.3column";

/*
 * A multi-column layout.
 * 
 */
dojo.declare("jazz.ensemble.internal.layout.ColumnLayout", jazz.ensemble.internal.Layout, {
	
	"class": "jazz-ensemble-layout-column",
	
	type: null,
	onStateChange: null,
	onAddWidget: null,
	onRemoveWidget: null,
	
	_data: null,
	_columns: null,
	_columnStates: null,
	_itemsById: null,
	_numColumns: null,
	
	postCreate: function() {
		this._itemsById = {};
		switch (this.type) {
			case LAYOUT_ID_1:
				this._numColumns = 1;
				break;
			case LAYOUT_ID_2:
				this._numColumns = 2;
				break;
			case LAYOUT_ID_3:
				this._numColumns = 3;
				break;
		}
		
		// create lookup map
		for (var i = 0; i < this.items.length; i++) {
			var item = this.items[i];
			dojo.addClass(item.domNode, "normal");
			this._itemsById[item.id] = item;
		}
		delete this.items;
		
		// use binding to work with XML DOM
		var data = this._data = XmlBindingFactory.createBinding(this.data, Schema.COLUMN_LAYOUT_DATA);
		var columnCollection = data.getColumns();
		var columnStates = this._columnStates = columnCollection.getColumns();
		
		if (this._numColumns > 1) {
			var widths = new Array();
			for (i = 0; i < this._numColumns; i++) {
				widths.push(columnStates[i].getWidth());
			}
			// create resizable columns widget
			this._resizableColumns = new ResizableColumns({
				disabled: this.readOnly,
				onChange: dojo.hitch(this,this._onChangeWidths),
				widths: widths
			});
			this.domNode.appendChild(this._resizableColumns.domNode);			
		}
		
		// create columns from 2D array
		var columns = this._columns = new Array();
	 	var colClass;
	 	switch(this._numColumns) {
			case 1:
				colClass = "one-col";
				break;
			case 2:
				colClass = "two-col";
				break;
			default:
				colClass = "three-col";
		}
	 	for (i = 0; i < this._numColumns; i++) {
			var column = new Column({
				"class": colClass,
				index: i,
				readOnly: this.readOnly,
				relativeTo: this.domNode, 
				onChange: dojo.hitch(this,this._onChangeWidgets),
				state: columnStates[i],
				onAddWidget: dojo.hitch(this,this._onAddWidget),
				onRemoveWidget: dojo.hitch(this,this._onRemoveWidget),
				layout: this
			});
			dojo.addClass(column.domNode, "normal");
			var widgetsCollection = columnStates[i].getWidgets();
			var widgets = widgetsCollection.getWidgets();
			for (var j = 0; j < widgets.length; j++) {
				if (widgets[j]) column.add(this._itemsById[widgets[j]]);
			}
			columns.push(column);
		} 
	},
	_onAddWidget: function(elem, container, pos) {
		this.onAddWidget(elem, container, pos);
	},
	_onRemoveWidget: function(elem) {
		this.onRemoveWidget(elem);
	},
	startup: function() {
		if (this._resizableColumns) {
			this._resizableColumns.heightNode = this.domNode.parentNode.parentNode;
			this._updateResizableColumns();
			this._resizableColumns.startup(this._columns);
		} else if (this._columns[0]) {
			dojo.place(this._columns[0].domNode, this.domNode);
		}
	},
	add: function(/* object */ item, /* string|node? */ ref, /* number? */ pos) {
		var columnIndex = 0;
		var widgetIndex = 0;
		if (ref != null) {
			if (ref.nodeType) {
				//domNode
				var col = dijit.byNode(ref);
				var columnIndex = col.index;
				if (pos === null) {
					var widgetIndex = ref.childNodes.length;
				} else if (typeof pos == "number") {
					widgetIndex = pos;
				}
			} else if (dojo.isString(ref)) {
				//refId string
				refItem = this._itemsById[ref];
				columnIndex = dijit.byNode(refItem.domNode).getParent().index;
				widgetIndex = refItem.index;
			}
		}
		
		dojo.addClass(item.domNode, "normal");
		this._columns[columnIndex].add(item, widgetIndex);
		this._itemsById[item.id] = item;
		this._updateWidgets();
	},
	
	/*
	 * Sets the maximized state of the item with the given id. If true, the
	 * item will be maximized to take up the entire content area of the layout,
	 * else it will be restored to its original size.
	 */
	setMaximized: function(/* string */ id, /* boolean */ maximized) {
		var item = this._itemsById[id];
		if (item) {
			var node = item.domNode;
			var colNode = node.parentNode;
			var column = dijit.byNode(colNode);
			if (maximized) {
				// turn off dnd while maximized
				column.setEnabled(false);
				// turn off resizeable columns while maximized
				if (this._resizableColumns) this._resizableColumns.attr("disabled", true);
				// signal the other widgets that they're hidden
				for (var i in this._itemsById) {
					if (i != id) {
						var widget = dijit.byNode(this._itemsById[i].domNode);
						widget.onHide();
					}
				}
				dojo.removeClass(node, "normal");
				dojo.addClass(node, "maximized");
				dojo.removeClass(colNode, "normal");
				dojo.addClass(colNode, "maximized");
				dojo.addClass(this.domNode, "maximized");
			}
			else {
				column.setEnabled(!this.readOnly);
				dojo.removeClass(node, "maximized");
				dojo.addClass(node, "normal");
				dojo.removeClass(colNode, "maximized");
				dojo.addClass(colNode, "normal");
				dojo.removeClass(this.domNode, "maximized");
				this._updateResizableColumns();
				
				// signal the other widgets that they're visible again
				setTimeout(dojo.hitch(this, function() {
					for (var i in this._itemsById) {
						if (i != id) {
							var widget = dijit.byNode(this._itemsById[i].domNode);
							widget.onShow();
						}
					}
				}), 0);
			}
		}
	},
	
	remove: function(/* string */ id) {
		var item = this._itemsById[id];
		if (item) {
			var widget = dijit.byNode(item.domNode);
			widget.getParent().remove(item);
			this._updateWidgets();
			delete this._itemsById[id];
		}
	},
	
	/* Private functions */
	
	_onChangeWidgets: function() {
		this._updateWidgets();
		this.onStateChange();
	},
	_onChangeWidths: function(/* number[] */ widths) {
		this._updateWidths(widths);
		this.onStateChange();
	},
	_updateWidgets: function() {
		for (var i = 0; i < this._columns.length; i++) {
			var widgets = [];
			var children = this._columns[i].getChildren();
			dojo.forEach(children, function(w) {
				widgets.push(w.widgetId);
			});
			var widgetsCollection = this._columnStates[i].getWidgets();
			widgetsCollection.setWidgets(widgets);
		}
		this._updateResizableColumns();
	},
	_updateResizableColumns: function() {
		var rc = this._resizableColumns;
		if (!rc) return;
		var hasWidgets = false;
		for (var i = 0; i < this._columns.length; i++) {
			var widgets = [];
			var children = this._columns[i].getChildren();
			if (children.length > 0) {
				hasWidgets = true;
				break;
			}
		}
		
		if (hasWidgets) {
			if (rc.attr("disabled") != this.readOnly) {
				rc.attr("disabled", this.readOnly);	
			}
		} else {
			rc.attr("disabled", true);
		}
	},
	_updateWidths: function(/* number[] */ widths) {
		for (var i=0;i<widths.length;i++) {
			// retain only 2 decimal places to save space
			var width = Math.round(widths[i] * 100) / 100;
			this._columnStates[i].setWidth(width);
		}
	},
	uninitialize: function() {
		while(this._columns.length > 0) {
			this._columns.pop().destroy();
		}
		this._resizableColumns && this._resizableColumns.destroy();
	}
});

// public static
dojo.mixin(jazz.ensemble.internal.layout.ColumnLayout, {
	
	ID_1: LAYOUT_ID_1,
	ID_2: LAYOUT_ID_2,
	ID_3: LAYOUT_ID_3,
	
	/*
	 * A static method on the Layout subclass that provides the default
	 * layout data for an empty layout of this type.
	 */
	getDefaultData: function(/* Element */ parentElem) {
		var elem = Xml.createElementNS(parentElem.ownerDocument, Xml.namespace.jw, "LayoutData");
		var layoutData = XmlBindingFactory.createBinding(elem, Schema.COLUMN_LAYOUT_DATA, "\t\t\t\t\t\t");
		var columnCollection = layoutData.createColumns();
		columnCollection.setParseType("Collection");
		layoutData.setColumns(columnCollection);
		var columns = [];
		for (var i=0;i<3;i++) {
			var column = columnCollection.createColumn();
			column.setWidth(i == 0 ? "34" : "33");
			var widgetCollection = column.createWidgets();
			widgetCollection.setParseType("Collection");
			column.setWidgets(widgetCollection);
			columns.push(column);
		}
		columnCollection.setColumns(columns);
		return elem;
	}
});

})();