/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ensemble.internal.layout.ResizableColumns");

dojo.require("dijit._Widget");
dojo.require("jazz.ui.underlay");
dojo.require("jazz.util.ResizeNotifier");

(function() {
var ResizeNotifier = jazz.util.ResizeNotifier;
var underlay = jazz.ui.underlay;

dojo.declare("jazz.ensemble.internal.layout.ResizableColumns", [dijit._Widget], {

	/* Defaults */
	columns: null,		//widgets or object with domNode property, ex. {domNode: node}
	widths: null,		//%
	minWidth: 180,
	fadeInDuration: 150,
	fadeOutDuration: 300,
	disabled: false,
	heightNode: null,
	
	// callbacks
	onChange: null,
		
	/* Internal vars */
	_sliders: null,
	_currentWidth: 0,
	_initWidths: false,
	
	postCreate: function() {
		this.domNode.className = "jazz-ensemble-internal-layout-ResizableColumns";
		this._resizeHandle = ResizeNotifier.register({context: this, callback: this._onResize});
	},
	startup: function(columns) {
		if (columns) {
			this.columns = columns;
		} else if (!this.columns){
			console.error("Error: No columns passed to ResizableColumns");
			return;
		}
		if (!this.widths || this.widths.length != this.columns.length) {
			this.widths = new Array();
			var size = 100 / this.columns.length;
			for (var i = 0; i < this.columns.length; i++) {
				this.widths[i] = size;
			}
			this._initWidths = true;
		}
		this.update();
		this.inherited(arguments);
	},
	_setDisabledAttr: function (disabled) {
		this.disabled = disabled;
		if (disabled) {
			this._clearSliders();
		} else if (this._started) {
			this.update();
		}
	},
	_onResize: function() {
		this.update();
	},
	update: function(isDrag) {
		this._currentWidth = dojo.marginBox(this.domNode).w;
		var hasSliders = this._sliders && this._sliders.length > 0;
		if (this._currentWidth) this._validateWidths(isDrag);
		if (this.disabled && hasSliders) {
			this._clearSliders();
			hasSliders = false;
		}
		for (var i = 0; i < this.columns.length; i++) {
			var col = this.columns[i];
			if (col && col.domNode) {
				if (!col.domNode.parentNode || col.domNode.parentNode != this.domNode) {
					this.domNode.appendChild(col.domNode);
				}
				col.domNode.style.width = this.widths[i] + "%";
				if (!this.disabled && i != this.columns.length - 1) {
					if (hasSliders) {
						this._updateSlider(col.domNode,i);
					} else {
						this._addSlider(col.domNode,i);
					}
				}
			}
		}
		if (this._initWidths) {
			isDrag = true;
			this._initWidths = false;
		}
		this._updateHeight();
		if (isDrag && this.onChange && dojo.isFunction(this.onChange)) {
			this.onChange(this.widths);
		}
	},
	_validateWidths: function(isDrag) {
		if (this._cachedWidths && isDrag !== true) {
			this.widths = this._cachedWidths;
		} else {
			this._cachedWidths = dojo.clone(this.widths);
		}
		var newWidths = new Array();
		var newWidthsDiff = 0, newWidthsCount = 0, unchangedWidthsCount = 0;
		for (var i = 0; i < this.widths.length; i++) {
			if (this.widths[i] / 100 * this._currentWidth < this.minWidth) {
				newWidths[i] = this.minWidth / this._currentWidth * 100;
				newWidthsDiff += Math.abs(this.widths[i] - newWidths[i]);
				newWidthsCount++;
			} else {
				newWidths[i] = 0;
				unchangedWidthsCount++;
			}
		}
		var scalingFactor = unchangedWidthsCount ? newWidthsDiff / unchangedWidthsCount : 0;
		var sum = 0; 
		for (var j = 0; j < newWidths.length; j++) {
			if (newWidths[j] == 0) {
				newWidths[j] = this.widths[j] - scalingFactor;
			} else {
				newWidths[j] = newWidths[j];
			}
			sum += newWidths[j];
		}
		this.widths = newWidths;
	},
	_startDragging: function(e) {
		dojo.stopEvent(e);
		this._currentSlider = e.target.index;
		dojo.addClass(document.body, "jazz-ui-SideNavigationLayout-dragCursor");
		if (this._dragListeners) {
			for (var i = 0; i < this._dragListeners.length; i++) {
				this.disconnect(this._dragListeners[i]);
			}
		}
		this._dragListeners = [
			this.connect(dojo.doc, "onmousemove", "_drag"),
			this.connect(document.body, "onmouseout", "_drag"),
			this.connect(document.body, "onmouseup", "_stopDragging"),
			this.connect(document.body, "ondragstart", "_cancelEvent"),
			this.connect(document.body, "onselectstart", "_cancelEvent"),
			this.connect(document.body, "onmouseleave", "_stopDragging")
		];
		this._isDragging = true;
		this._lastX = e.pageX;
		this._delta = 0;
		this._pixelize();
		jazz.ui.underlay.show({opacity: 0, zIndex: 2000});
	},
	_pixelize: function(){
		this._pixelCoords = new Array();
		for (var i = 0; i < this.columns.length; i++) {
			var col = this.columns[i].domNode;
			var width = dojo.marginBox(col).w + 1;
			col.style.width = width + "px";
			this._pixelCoords.push(width);
		}
	},
	_drag: function(e){
		var i = this._currentSlider;
		var col = this.columns[i];
		var col2 = this.columns[i+1];
		this._delta = e.pageX - this._lastX;
		if (this._pixelCoords[i] + this._delta <= this.minWidth) {
			this._delta = this.minWidth - this._pixelCoords[i];
		} else if (this._pixelCoords[i+1] - this._delta <= this.minWidth) {
			this._delta = this._pixelCoords[i+1] - this.minWidth;
		}
		col.domNode.style.width = this._pixelCoords[i] + this._delta + "px";
		col2.domNode.style.width = this._pixelCoords[i+1] - this._delta + "px";
		this._updateSlider(col.domNode,i);
		this._updateHeight();
	},
	_stopDragging: function(e) {
		dojo.removeClass(document.body, "jazz-ui-SideNavigationLayout-dragCursor");
		if (this._dragListeners) {
			for (var i = 0; i < this._dragListeners.length; i++) {
				this.disconnect(this._dragListeners[i]);
			}
			delete this._dragListeners;
		}
		this._pixelCoords[this._currentSlider] += this._delta;
		this._pixelCoords[this._currentSlider+1] -= this._delta;
		this._isDragging = false;
		var slider = this._sliders[this._currentSlider];
		if (slider) this._doFade(slider,dojo.fadeOut);
		this._currentSlider = null;
		var totalWidth = 0;
		for (var i = 0; i < this._pixelCoords.length; i++) {
			totalWidth += this._pixelCoords[i];
		}
		for (var i = 0; i < this.widths.length; i++) {
			this.widths[i] = this._pixelCoords[i] / totalWidth * 100;
		}
		jazz.ui.underlay.hide();
		ResizeNotifier.poke();
		this.update(true);
	},
	_cancelEvent: function(event) {
		dojo.stopEvent(event);
	},
	_addSlider: function(colNode,i) {
		if (!this._sliders) this._sliders = new Array();
		var div = document.createElement("div");
		div.className = "jazz-ensemble-internal-layout-ResizableColumns-Slider";
		this.connect(div,"onmousedown",this._startDragging);
		this.connect(div,"onmouseover",this._showSlider);
		this.connect(div,"onmouseout",this._hideSlider);
		this.domNode.appendChild(div);
		var coords = dojo.marginBox(colNode);
		div.style.left = coords.l + coords.w - 6 + "px";
		dojo.style(div,{opacity: 0});
		div.index = i;
		this._sliders[i] = div;
	},
	uninitialize: function() {
		if (this._currentFade && this._currentFade.status() == "playing") {
			this._currentFade.stop();
		}
		if (this._resizeHandle) {
			ResizeNotifier.unregister(this._resizeHandle);
		}
	},
	_clearSliders: function() {
		while(this._sliders && this._sliders.length > 0) {
			var slider = this._sliders.pop();
			if (slider.parentNode) slider.parentNode.removeChild(slider);
		}
	},
	_updateSlider: function(colNode, index) {
		var coords = dojo.marginBox(colNode);
		if (this._sliders) {
			if (this._sliders[index]) {
				this._sliders[index].style.left = coords.l + coords.w - 6 + "px";
			} else {
				this._addSlider(coords, index);
			}
		}
	},
	_showSlider: function(event) {
		this._updateHeight();
		this._doFade(event.target,dojo.fadeIn);
	},
	_hideSlider: function(event) {
		this._doFade(event.target,dojo.fadeOut);
	},
	_doFade: function(node, fadeFunc) {
		if (!this._isDragging && dojo.some(this._sliders, function(node){return node})) {
			if (this._currentFade && this._currentFade.status() == "playing") {
				if(node == this._currentFade.node) {
					this._currentFade.stop();
				} else {
					dojo.style(this._currentFade.node, {opacity: this._currentFade.properties.opacity.end});
				}
			}
			var fadeArgs = {
				node: node,
				duration: (fadeFunc == dojo.fadeOut) ? this.fadeOutDuration : this.fadeInDuration,
				onEnd: dojo.hitch(this,function(){
					this._currentFade = null;
				})
			};
			this._currentFade = fadeFunc(fadeArgs);
			this._currentFade.play();
		}		
	},
	_updateHeight: function() {
		if (!this.heightNode) return;
		var height = dojo.contentBox(this.heightNode).h;
		if (height > 10) height -= 10;
		if (this._currentSlider) {	
			var slider = this._sliders[this._currentSlider];
			if (slider) slider.style.height = height + "px";
		} else {
			if (this._sliders) {
				for (var i = 0; i < this._sliders.length; i++) {
					this._sliders[i].style.height = height + "px";
				}
			}
		}
	}
});
})();