/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ensemble.internal.settings.AppearancePanel");

dojo.require("jazz.ensemble.internal.settings.SettingsPanel");
dojo.require("jazz.ui.FilterBox");

dojo.require("dojo.i18n");
dojo.requireLocalization("jazz.ensemble.internal", "Messages");

(function() {

var SettingsPanel = jazz.ensemble.internal.settings.SettingsPanel;
var TrimStyles = jazz.ui.StyledBox.TrimStyles;

var TRIM_BY_ID = {
	"blue": TrimStyles.BLUE,
	"gray": TrimStyles.GRAY,
	"gold": TrimStyles.GOLD,
	"green": TrimStyles.GREEN,
	"teal": TrimStyles.TEAL,
	"purple": TrimStyles.PURPLE,
	"orange": TrimStyles.ORANGE,
	"none": TrimStyles.TRIMLESS
};

var TYPES = dojo.clone(SettingsPanel.TYPES);
dojo.mixin(TYPES, {
	COLOR: "COLOR",
	TITLE: "TITLE"
});

var DEFAULT_TRIM = "blue";
var TRIM_BY_ID = {
	"blue": TrimStyles.BLUE,
	"gray": TrimStyles.GRAY,
	"gold": TrimStyles.GOLD,
	"green": TrimStyles.GREEN,
	"teal": TrimStyles.TEAL,
	"purple": TrimStyles.PURPLE,
	"orange": TrimStyles.ORANGE,
	"none": TrimStyles.TRIMLESS
};

var messages = dojo.i18n.getLocalization("jazz.ensemble.internal", "Messages");

dojo.declare("jazz.ensemble.internal.settings.AppearancePanel",SettingsPanel,{

	postCreate: function() {
		this.init();
		this.inherited(arguments);
	},
	
	init: function() {
		var title = this.container.state.getTitle();
		this.settings = [{
			label: messages.appearTitleTitle,
			id: 'title',
			value: this.container.state.isCustomTitle() ? title : "",
			description: messages.appearTitleDesc,
			type: TYPES.TITLE,
			defaultText: this.container._latestTitle || (!this.container.state.isCustomTitle() ? title : "" ) || this.container.getDefaultTitle()
		},{
			label: messages.appearTrimTitle,
			id: 'trim',
			value: this.container.state.getTrim() || "blue",
			description: messages.appearTrimDesc,
			type: TYPES.COLOR
		},{
			label: messages.appearBackTitle,
			id: 'showBackground',
			value: this.container.state.getShowBackground() ? "true" : "false",
			description: messages.appearBackDesc,
			type: TYPES.BOOLEAN
		}];
	},
	
	onChange: function(id, value) {
		if (id === "showBackground") {
			this.container.setBackground(value === "true" ? true : false);
		}
	},
	
	createFormField: function(setting, field) {
		switch(setting.type) {
		case TYPES.TITLE:
			var inputDiv = dojo.create("div", null, field)
			var textBox = new jazz.ui.FilterBox({value: setting.value, defaultText: setting.defaultText, onChange: dojo.hitch(this,this._handleTitleChange), _maintainCase: true, _clearTooltip: messages.appearRevertTitle});
			inputDiv.appendChild(textBox.domNode);
			return;
		case TYPES.COLOR:
			var inputDiv = document.createElement("div");
			dojo.addClass(inputDiv,"color-picker");
			
			var colorArr = ["blue","gray","gold","green","teal","purple","orange","none"];
			for (var i = 0; i < colorArr.length; i++) {
				var div = document.createElement("div");
				div.className = colorArr[i]+"-option";
				div.value = colorArr[i];
				inputDiv.appendChild(div);
				if (setting.value === colorArr[i]) {
					dojo.addClass(div,"selected");
					this.selectedColor = div;
				}
			}
			this.connect(inputDiv, "onclick", this._handleColorChange);
			field.appendChild(inputDiv);
			return;
		}
		return this.inherited(arguments);
	},
	_handleTitleChange: function(value) {
		this.settings[this.settingsMap['title']].value = value;
	},
	_handleColorChange: function(event) {
		if (!event.target || !event.target.value) return;
		dojo.addClass(event.target,"selected");
		if (this.selectedColor) {
			dojo.removeClass(this.selectedColor,"selected");	
		}
		this.selectedColor = event.target;
		var value = event.target.value;
		this.settings[this.settingsMap['trim']].value = value;
		this.container.setTrim(TRIM_BY_ID[value]);
		dojo.publish("jazz.util.ResizeNotifier.resizeEvent");
	}
});
jazz.ensemble.internal.settings.AppearancePanel.TYPES = TYPES;
jazz.ensemble.internal.settings.AppearancePanel.TRIM_BY_ID = TRIM_BY_ID;
jazz.ensemble.internal.settings.AppearancePanel.DEFAULT_TRIM = DEFAULT_TRIM;

})();
