/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ensemble.internal.settings.SlideoutPanel");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.requireLocalization("jazz.ensemble.internal", "Messages");

(function() {

var messages = dojo.i18n.getLocalization("jazz.ensemble.internal", "Messages");

/*
 * A panel that slides open/closed, and shows arbitrary options as well as ok
 * and cancel buttons. The panel is initially invisible, and becomes visible when
 * open() is called.
 */
dojo.declare("jazz.ensemble.internal.settings.SlideoutPanel", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("jazz.ensemble", "internal/settings/templates/SlideoutPanel.html"),
	
	_messages: messages,
	
	animate: true,
	
	postCreate: function() {
		if (this.animate) dojo.style(this._slideoutDiv, {height: "0px"});
	},
	
	/*
	 * Make the panel visible and slide it open.
	 */
	open: function() {
		this.domNode.style.display = "";
		if (this.animate) {
			if (this._animation) this._animation.stop();
			this._animation = dojo.fx.wipeIn({node: this._slideoutDiv, duration: 250});
			this._isClosing = false;
			this.connect(this._animation, "onEnd", dojo.hitch(this, function() {
				delete this._animation;
			}));
			this._animation.play();
		}
	},
	
	/*
	 * Slide closed and make invisible. The callback will be called when the
	 * sliding animation has finished.
	 */
	close: function(/* function */ callback) {
		if (this.animate) {
			if (this._animation) this._animation.stop();
			this._animation = dojo.fx.wipeOut({node: this._slideoutDiv, duration: 250});
			this._isClosing = true;
			this.connect(this._animation, "onEnd", dojo.hitch(this, function() {
				this.domNode.style.display = "none";
				if (callback) {
					callback();
				}
				delete this._animation;
			}));
			this._animation.play();
		} else {
			this.domNode.style.display = "none";
		}
	},
	
	toggle: function() {
		var display = dojo.style(this.domNode, "display");
		if (display === "block" && !this._isClosing) {
			this.close();
		} else {
			this.open();
		}
	}
});

})();
