/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.app.internal.util.DiscoveredService");

dojo.require("jazz.app.internal.util.Rootservices");
dojo.require("net.jazz.ajax.BootstrapProperties");
dojo.require("jazz.ensemble.internal.util.XmlBindingFactory");
dojo.require("jazz.app.internal.util.XhrUtil");
dojo.require("dojo.DeferredList");

(function() {
	
var namespace  		 	= jazz.ensemble.internal.util.Xml.namespace;
var XhrUtil				= jazz.app.internal.util.XhrUtil;
var Rootservices        = jazz.app.internal.util.Rootservices;
var XmlBindingFactory 	= jazz.ensemble.internal.util.XmlBindingFactory;
var xml				 	= jazz.ensemble.internal.util.Xml;

dojo.declare("jazz.app.internal.util.RootServiceDiscoveredService", null, {

	namespace 	: null,
	service		: null,
	local 		: false,   /* true if it is a local service */
	title		: null,    /* taken from the friends.cfg    */
	rootservice : null,
	urlSuffix	: "",
	serviceUrl  : null,
	
	_result		: null,

	constructor: function(args) {
		if (!args) throw new Error("jazz.app.internal.util.DiscoveredService No arguments passed to constructor");

		this.namespace 		= args.namespace;
		this.service		= args.service;
		this.title			= args.title;
		this.rootservice 	= args.rootservice;
		this.local 			= args.local||this.local; // local is optional
		this.urlSuffix		= args.urlSuffix||this.urlSuffix;
	},

	isLoaded : function (){
		return this._result&&true;
	},
	
	getResult : function(){
		return this._result;
	},
	
	
	/**
	 *	args.refresh - forces a refresh of the servces without this it will return the cached deferred
	 *  args.passive - don't force a call if we meet an Oauth Challenge
	 */
	call : function(args){
		args = args || {};

		if (this._result && !args.refresh) return XhrUtil.immediateCallback(this._result);;

		var _this 		= this;
		var deferred 	= new dojo.Deferred();
		var passive  	= args.passive;
		
		/**
		 * everything starts with loading the root services document
		 */
		var rootDeferred = this.rootservice.load(passive);
		
		rootDeferred.addCallback(function(root){
			/* handle a passive call */
			if (!root.isLoaded()){
				deferred.callback(null);
				return;
			}
			
			_this.serviceUrl 	= root.getServiceUrl(_this.namespace, _this.service);
			
			if (!_this.serviceUrl) throw new Error("jazz.app.internal.util.DiscoveredService: could not find service: "+_this.namespace+":"+_this.service);

			if (_this.urlSuffix)
			{
				_this.serviceUrl += _this.urlSuffix;
			}

		  	var serviceDeferred = XhrUtil.getXml(_this.serviceUrl, {passive: passive});
		  	
		  	serviceDeferred.addCallback(function(document){
				/* handle a passive call */
				if (!document){
					deferred.callback(null);
					return;
				}
		  		_this._result 	= document;
		  		deferred.callback(document);
		  	});
			
			serviceDeferred.addErrback(function(error){deferred.errback(error);});
		});

		rootDeferred.addErrback(function(error){deferred.errback(error);});

		return deferred;
	}
});

jazz.app.internal.util.DiscoveredService = jazz.app.internal.util.RootServiceDiscoveredService;

/**
 * 	 discovers all the services that are available locally and on all the friends.
 *   args.passiveAutoCall	: passively calls all serivices on load
 *   args.urlSuffix         : append this suffex to the services url
 */

jazz.app.internal.util.DiscoveredService.loadRootserviceServices = function (ns, serviceName, args){
	
	args = args || {};
	
	var deferred = new dojo.Deferred();

	/**
	 * load the root
	 */
	var rootDeferred = Rootservices.loadLocal();
	
	rootDeferred.addCallback(function(root){

		var services = [];
		
		/**
		 * first our root it must be first .. 
		 */
		services.push(new jazz.app.internal.util.DiscoveredService({
			namespace 	: ns,
			service		: serviceName,
			local 		: true,
			urlSuffix	: args.urlSuffix,
			title		: root.getTitle(),
			rootservice : root
		}));
		/**
		 * now load the friends
		 */
		var friendDeferred = root.loadFriends();
		friendDeferred.addCallback(function(friends){
			dojo.forEach(friends, function(friend) {
				
				if (friend.rootServices == root.getAbout()) {
					//filter out self-referencing friends links
					return;
				}
				services.push(new jazz.app.internal.util.DiscoveredService({
					namespace   : ns,
					service     : serviceName,
					title		: friend.title,
					urlSuffix	: args.urlSuffix,
					rootservice : new Rootservices(friend.rootServices)
				}));
			});

			/**
			 * if we are not preloading then tell the deferred that we are done hurrah!!
			 */
			if (!args.passiveAutoCall){
				deferred.callback(services);
			}
			else{
				/**
				 * passively preload the services
				 */
				var deferreds = []

				dojo.forEach(services, function(aService){
					deferreds.push(aService.call({passive:true})); 
				});
				
				new dojo.DeferredList(deferreds).addCallbacks( 
					function()		{deferred.callback(services);},
					function(error)	{deferred.errback(error);	 }
				);
			}
		});
		
		friendDeferred.addErrback(function(error){deferred.errback(error);});
	});
	
	rootDeferred.addErrback(function(error) {deferred.errback(error);});
	
	return deferred;
}


/**
 * this enables me to create test cases for discovery. 
 * use this value for the next discovery query.
 */
var testQuery=null;
jazz.app.internal.util.DiscoveredService._setTempTestDiscoveryQuery = function(tq){
	testQuery = tq;
};

/**
 * 	 discovers all the services that are available locally and on all the friends.
 *   args.passiveAutoCall	: passively calls all serivices on load
 *   args.urlSuffix         : append this suffex to the services url
 */

jazz.app.internal.util.DiscoveredService.loadDiscoveryServices = function(serviceUri, schema, includeExternal){
	if (dojo.isString(serviceUri)){
		var types = "type=" + encodeURIComponent(serviceUri);
	} else if (dojo.isArray(serviceUri)){
		//serviceUri is an array of objects with attributes {uri, schema}
		types = "";
		for (var i = 0; i < serviceUri.length; i++) {
			if (i > 0) types += "&";
			types += "type=" + encodeURIComponent(serviceUri[i]);
		}
	}
	var discoveryQuery = net.jazz.ajax.BootstrapProperties.DISCOVERYSERVICE_URL+"?"+types;
	
	if (includeExternal)
		discoveryQuery = discoveryQuery +"&scope=includeExternal";
	
	/**
	 * make discoveredService testable
	 */
	if (testQuery){
		discoveryQuery=testQuery;
		testQuery=null;
	}

	var deferred = jazz.client.xhrGet(	{
    	url				 : discoveryQuery,
		handleAs		 : "xml",
		headers			 : { "accept": "application/rdf+xml" }
	});
	
	deferred.addCallback(function(doc){
		if (schema) {
			return XmlBindingFactory.createBinding(doc.documentElement, schema);
		}
	});
	return deferred;
}

jazz.app.internal.util.DiscoveredService.loadAll = jazz.app.internal.util.DiscoveredService.loadRootserviceServices;

})();
