/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.app.internal.util.Rootservices");

dojo.require("jazz.ensemble.internal.util.Xml");
dojo.require("jazz.ensemble.internal.util.XmlBindingFactory");
dojo.require("jazz.client.xhr");
dojo.require("jazz.app.internal.util.XhrUtil");


dojo.require("jazz.app.proxy");
dojo.require("jazz.app.auth");

dojo.require("dojox.xml.parser");

(function() {
var XhrUtil				= jazz.app.internal.util.XhrUtil;
var namespace  		 	= jazz.ensemble.internal.util.Xml.namespace;
var XmlBindingFactory 	= jazz.ensemble.internal.util.XmlBindingFactory;
var xml				 	= jazz.ensemble.internal.util.Xml;

var documentCache={};

dojo.declare("jazz.app.internal.util.Rootservices", null, {
	
	url          : null,
	_document    : null,
	_friendsList : null,
	_title       : null,
	
	constructor: function(url) {
		this.url = url;
		this._document = documentCache[url];
	},

	load : function(passive){
		if (this.isLoaded()) return XhrUtil.immediateCallback(this);
		
		var _this = this;
		
		var deferred = XhrUtil.getXml(this.url, {schema: ROOTSERVICES_SCHEMA, passive: passive});
		
		deferred.addCallback(function(doc){
			/* handle a passive call */
			if (doc) {
			   _this._document = doc;
			   documentCache[_this.url] = doc;
			}
			
			return _this;
		});
		
		return deferred;
	},
	
	getTitle: function() {
		if (!this.isLoaded()){
			log.error("jazz.app.internal.util.Rootservices.getTitle : not loaded");
			return null;
		}
		return this._document.title;
	},
	
	getPublisher : function(){
		if (!this.isLoaded()){
			log.error("jazz.app.internal.util.Rootservices.getPublisher : not loaded");
			return null;
		}
		return this._document.publisher;
	},
	
	getAbout: function() {
		if (!this.isLoaded()){
			log.error("jazz.app.internal.util.Rootservices.getAbout : not loaded");
			return null;
		}
		return this._document.about;
	},
	getServiceUrl : function (ns, name){
		if (!this.isLoaded()){
			console.error("jazz.app.internal.util.Rootservices.getServiceUrl : not loaded");
			return null;
		}

		var element = xml.getChildByNameNS(this._document.elem, ns, name);
		
		if (!element){
			console.warn("jazz.app.internal.util.Rootservices.getServiceUrl: no service for: ", ns, name);
			return null;
		}
		
		var service =  xml.getAttributeNS(element, namespace.rdf, "resource");
		
		if (!service) console.warn("jazz.app.internal.util.Rootservices.getServiceUrl: no service for: ", ns, name);
		
		return service;
	},
	
	/*
	 * must be called after this is loaded
	 * 
	 * returns in a deferred
	 * [
	 *    {
	 *    	title: "title",
	 *    	rootServices : "http://someservice.com"
	 *    }
	 *  ]
	 */
	loadFriends : function(passive)	{
		if (!this.isLoaded()) return XhrUtil.immediateErrback(new Error("jazz.app.Rootservices.loadFriends rootservice not loaded"));
				
		if (this._friendsList) return XhrUtil.immediateCallback(this._friendsList);

		var deferred = XhrUtil.getXml(this._document.friends, {schema: FRIENDS_LIST_SCHEMA, passive:passive}); 

		var _this = this;
		deferred.addCallback(function (result){ 
			if (!result) return null;
			dojo.forEach(result.friends, function(f){
				f.title = decodeURI(f.title);
			});
			_this._friendsList 	= result.friends;
			return _this._friendsList;
		});
		
		return deferred;
	},
	
	isLoaded : function(){
		return (this._document&&true);
	},
	
	getCurrentUser : function()
	{
		if (!this.isLoaded()){
			console.error("jazz.app.internal.util.Rootservices.getCurrentUser : not loaded");
			return null;
		}
		
		return this._document.currentUser;
	},
	
	getViewletServiceRoot : function()
	{
		if (!this.isLoaded()){
			console.error("jazz.app.internal.util.Rootservices.getViewletServiceRoot : not loaded");
			return null;
		}
		
		return this._document.viewletServiceRoot;
	},
	
	getViewletWebUIRoot : function()
	{
		if (!this.isLoaded()){
			console.error("jazz.app.internal.util.Rootservices.getViewletWebUIRoot : not loaded");
			return null;
		}
		
		return this._document.viewletWebUIRoot;
	},
		
	getHomeMenuIcon : function()
	{
		if (!this.isLoaded()){
			console.error("jazz.app.internal.util.Rootservices.getHomeMenuIcon : not loaded");
			return null;
		}
		
		return this._document.homeMenuIcon;
	}	
});

var localRoot = null;

jazz.app.internal.util.Rootservices.loadLocal = function() 
{
	if (localRoot) return XhrUtil.immediateCallback(localRoot);
	
	var deferred = jazz.app.internal.util.Rootservices.load (net.jazz.ajax._contextRoot + "/rootservices");

	deferred.addCallback(function (root) {
		if (root) localRoot = root;	
	});

	return deferred;
}

jazz.app.internal.util.Rootservices.load = function(url, passive)
{
	return (new jazz.app.internal.util.Rootservices(url)).load(passive);
}


var JD_NS 	= "http://jazz.net/xmlns/prod/jazz/discovery/1.0/";
var JFS_NS 	= "http://jazz.net/xmlns/prod/jazz/jfs/1.0/";
var JDB_NS 	= "http://jazz.net/xmlns/prod/jazz/dashboard/1.0/";
var OSLC_NS = "http://open-services.net/ns/core#";


var ROOTSERVICES_SCHEMA =
[
	{
		property:		"currentUser",
		type:			XmlBindingFactory.STRING,
		nodeType: 		XmlBindingFactory.CHILD_ELEMENT_ATTRIBUTE,
		elemName:		"currentUser",
		elemNS:			JFS_NS,
		attrName: 		"resource",
		attrNS: 		namespace.rdf
	},
	{
		property:		"title",
		type:			XmlBindingFactory.STRING,
		nodeType: 		XmlBindingFactory.CHILD_ELEMENT_TEXT,
		elemName:		"title",
		elemNS:			xml.namespace.dublin
	},
	{
		property:	"about",
		type: 		XmlBindingFactory.STRING,
		nodeType: 	XmlBindingFactory.ATTRIBUTE,
		attrName: 	"about",
		attrNS: 	namespace.rdf
	},
	{
		property: 		"friends",
		type: 			XmlBindingFactory.STRING,
		nodeType: 		XmlBindingFactory.CHILD_ELEMENT_ATTRIBUTE,
		elemName:		"friends",
		elemNS:			JD_NS,
		attrName: 		"resource",
		attrNS: 		namespace.rdf
	},
	{
		property: 		"publisher",
		type: 			XmlBindingFactory.STRING,
		nodeType: 		XmlBindingFactory.CHILD_ELEMENT_ATTRIBUTE,
		elemName:		"publisher",
		elemNS:			OSLC_NS,
		attrName: 		"resource",
		attrNS: 		namespace.rdf
	},
	{
		property: 		"viewletServiceRoot",
		type: 			XmlBindingFactory.STRING,
		nodeType: 		XmlBindingFactory.CHILD_ELEMENT_ATTRIBUTE,
		elemName:		"viewletServiceRoot",
		elemNS:			JD_NS,
		attrName: 		"resource",
		attrNS: 		namespace.rdf
	},
	{
		property: 		"viewletWebUIRoot",
		type: 			XmlBindingFactory.STRING,
		nodeType: 		XmlBindingFactory.CHILD_ELEMENT_ATTRIBUTE,
		elemName:		"viewletWebUIRoot",
		elemNS:			JD_NS,
		attrName: 		"resource",
		attrNS: 		namespace.rdf
	},
	{
		property: 		"homeMenuIcon",
		type: 			XmlBindingFactory.STRING,
		nodeType: 		XmlBindingFactory.CHILD_ELEMENT_ATTRIBUTE,
		elemName:		"homeMenuIcon",
		elemNS:			JDB_NS,
		attrName: 		"resource",
		attrNS: 		namespace.rdf
	}
];


var FRIENDS_SCHEMA =
[
	{
		property:		"rootServices",
		type: 			XmlBindingFactory.STRING,
		nodeType: 		XmlBindingFactory.CHILD_ELEMENT_ATTRIBUTE,
		elemName:		"rootServices",
		elemNS:			JD_NS,
		attrName: 		"resource",
		attrNS: 		namespace.rdf
	},
	{
		property: 		"title",
		type: 			XmlBindingFactory.STRING,
		nodeType: 		XmlBindingFactory.CHILD_ELEMENT_TEXT,
		elemName:		"title",
		elemNS:			namespace.dublin
	}
];

var FRIENDS_LIST_SCHEMA =
[
{
	property:       "friends",
	type:     		XmlBindingFactory.OBJECT,
	nodeType: 		XmlBindingFactory.CHILD_ELEMENT,
	elemName: 		"Friend",
	elemNS:   		JD_NS,
	schema:   		FRIENDS_SCHEMA,
	multiValued: 	true
}
]

})();
