/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ensemble.internal.util.Xml");

(function() {

/*
 * A namespace-aware, cross-browser XML utility.
 * 
 * Limitations:
 * - Does not support default namespaces, e.g. xmlns="...". You must declare explicit
 *   prefixes, e.g. xmlns:prefix="...".
 * - Only supports and generates namespace declarations on the root document element,
 *   not on child elements.
 */
jazz.ensemble.internal.util.Xml = {
	
	NodeType: {
		ELEMENT: 1,
		ATTRIBUTE: 2,
		TEXT: 3, 
		COMMENT: 8,
		DOCUMENT: 9
	},
	
	namespace: {
		xml: 	"http://www.w3.org/XML/1998/namespace",
		rdf: 	"http://www.w3.org/1999/02/22-rdf-syntax-ns#",
		dublin: "http://purl.org/dc/terms/",
		jdb: 	"http://jazz.net/xmlns/prod/jazz/dashboard/1.0/",
		jd:		"http://jazz.net/xmlns/prod/jazz/discovery/1.0/",
		jw: 	"http://jazz.net/xmlns/prod/jazz/widget/1.0/",
		ju: 	"http://jazz.net/ns/ui#",
		juentry:"http://jazz.net/ns/ui/entry#",
		oslc:	"http://open-services.net/ns/core#",
		jfs:	"http://jazz.net/xmlns/foundation/1.0/",
		lpa: 	"http://jazz.net/xmlns/foundation/lpa/1.0/"
	},
	
	/*
	 * Creates a new element with the given namespace in the specified document.
	 * Aquires a namespace prefix in the given context (if namespace is specified).
	 */
	createElementNS: function(/* Document */ doc, /* string */ ns, /* string */ name) {
		ns = ns || null;
		var docElem = doc.documentElement;
		var qName = _qName(docElem, ns, name);
		if (!qName) {
			qName = _newNSPrefix(docElem, ns) + ":" + name;
		}
		if (doc.createElementNS) {
			return doc.createElementNS(ns, qName);
		}
		else {
			// browser is not namespace aware
			return doc.createElement(qName);
		}
	},
	
	/*
	 * Returns the attribute with the given name and namespace under the specified
	 * element, or undefined if the element doesn't have that attribute.
	 */
	/* string */ getAttributeNS: function(/* Element */ elem, /* string */ ns, /* string */ name) {
		var node = this.getAttributeNodeNS(elem, ns, name);
		if (node) {
			return node.nodeValue;
		}
	},

	/*
	 * Returns the attribute node with the given name and namespace under the specified
	 * element, or undefined if the element doesn't have that attribute.
	 */
	/* Node */ getAttributeNodeNS: function(/* Element */ elem, /* string */ ns, /* string */ name) {
		ns = ns || null;
		if (elem.hasAttributeNS && elem.getAttributeNS) {
			if (elem.hasAttributeNS(ns, name)) {
				return elem.getAttributeNodeNS(ns, name);
			}
		}
		else {
			// browser is not namespace aware
			var qName = _qName(elem, ns, name);
			if (qName) {
				return elem.getAttributeNode(qName);
			}
		}
	},

	/*
	 * Returns the first child element with the given name and namespace under the specified
	 * element, or undefined if such a child element does not exist.
	 */
	/* Element */ getChildByNameNS: function(/* Element */ elem, /* string */ ns, /* string */ name) {
		ns = ns || null;
		var doc = elem.ownerDocument;
		if (doc.evaluate) {
			var xpath = ns ? ("ns:" + name) : name;
			var nsResolver = dojo.partial(_resolveNS, ns);
			var child = doc.evaluate(xpath, elem, nsResolver, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
			if (child) {
				return child;
			}
		}
		else {
			// IE
			var childNodes = elem.childNodes;
			var len = childNodes.length;
			for (var i=0;i<len;i++) {
				var node = childNodes[i];
				if (node.nodeType == 1 && this.isElementNameNS(node, ns, name)) {
					return node;
				}
			}
		}
	},
	
	/*
	 * Returns all child elements (in the order they appear) with the given name and namespace
	 * under the specified element, or empty array if there are none.
	 */
	/* Element[] */ getChildrenByNameNS: function(/* Element */ elem, /* string */ ns, /* string */ name) {
		ns = ns || null;
		var array = [];
		var doc = elem.ownerDocument;
		if (doc.evaluate) {
			var xpath = ns ? ("ns:" + name) : name;
			var nsResolver = dojo.partial(_resolveNS, ns);
			var iter = doc.evaluate(xpath, elem, nsResolver, XPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
			var child;
			while (child = iter.iterateNext()) {
				array.push(child);
			}
		}
		else {
			// IE xpath
			var qName = _qName(elem, ns, name);
			if (qName) {
				var nodes = elem.selectNodes(qName);
				for (var i=0;i<nodes.length;i++) {
					array.push(nodes[i]);
				}
			}
		}
		return array;
	},
	
	/*
	 * Returns whether or not the element has the given name and namespace.
	 */
	/* boolean */ isElementNameNS: function(/* Element */ elem, /* string */ ns, /* string */ name) {
		return elem.nodeName == _qName(elem, ns || null, name);
	},
	
	/*
	 * Returns the text node content directly under the given element, or empty string "" if there
	 * is no text. Assumes that the element contains only text nodes.
	 */
	/* string */ getTextValue: function(/* Element */ elem) {
		return elem.textContent || elem.text || "";
	},
	
	/*
	 * Removes the given attribute with namespace from the specified element.
	 */
	removeAttributeNS: function(/* Element */ elem, /* string */ ns, /* string */ name) {
		ns = ns || null;
		if (elem.removeAttributeNS) {
			elem.removeAttributeNS(ns, name);
		}
		else {
			// browser is not namespace aware
			var qName = _qName(elem, ns, name);
			if (qName) {
				elem.removeAttribute(qName);
			}
		}
	},
	
	/*
	 * Removes all child elements directly under the given one with the given name and
	 * namespace.
	 */
	removeChildrenByNameNS: function(/* Element */ elem, /* string */ ns, /* string */ name) {
		var children = this.getChildrenByNameNS(elem, ns || null, name);
		for (var i=0;i<children.length;i++) {
			elem.removeChild(children[i]);
		}
	},
	
	/*
	 * Sets a new value on the given attribute of the specified element. Creates a new
	 * attribute if none exists with that name.
	 */
	setAttributeNS: function(/* Element */ elem, /* string */ ns, /* string */ name, /* string */ value) {
		ns = ns || null;
		var qName = _qName(elem, ns, name);
		if (!qName) {
			qName = _newNSPrefix(elem, ns) + ":" + name;
		}
		if (elem.setAttributeNS) {
			elem.setAttributeNS(ns, qName, value);
		}
		else {
			// browser is not namespace aware
			elem.setAttribute(qName, value);
		}
	},
	
	/*
	 * returns the parse error and null if none.
	 */
	
	getParseError : function (document)
	{
		if (typeof(DOMParser) != "undefined") {
			if (document.documentElement && (document.documentElement.localName == 'parsererror'))
			{
			      return document.documentElement.childNodes[0].nodeValue;
			}
			else
			    return null;
		}
		else if (typeof(ActiveXObject) != "undefined") {
			if (document.parseError && document.parseError.errorCode)
				return document.parseError.reason;
			else
			    return null;
		}
		return null;
	},

	
	/*
	 * Sets the text content directly under the given element. Assumes that the
	 * element contains only text nodes.
	 */
	/* string */ setTextValue: function(/* Element */ elem, /* string */ value) {
		elem[dojo.isIE ? "text" : "textContent"] = value;
	}
};

/* Private methods */

var Xml = jazz.ensemble.internal.util.Xml;

/*
 * Generates a new namespace prefix for the given namespace uri, and sets it on the
 * document.
 */
function _newNSPrefix(/* Node */ node, /* string */ uri) {
	// find unused prefix
	for (var i=1;;i++) {
		var prefix = "ns" + i;
		if (!_nsURI(node, prefix)) {
			// set it on document element and return it
			node.ownerDocument.documentElement.setAttribute("xmlns:" + prefix, uri);
			return prefix;
		}
	}
}

/*
 * Returns the prefix for the given namespace at the given node context.
 */
function _nsPrefix(/* Node */ node, /* string */ uri) {
	if (uri == Xml.namespace.xml) {
		return "xml";
	}
	var docElem = node.ownerDocument.documentElement;
	if (docElem.lookupPrefix) {
		return docElem.lookupPrefix(uri);
	}
	var attrs = docElem.attributes;
	for (var i=0;i<attrs.length;i++) {
		var attr = attrs[i];
		var name = attr.name;
		if (name.indexOf("xmlns:") == 0 && attr.value == uri) {
			return name.substring(6);
		}
	}
	// traverse up the DOM to find the nearest ns declaration
	var elem = node;
	while (elem && elem != docElem) {
		var attrs = elem.attributes;
		for (var i=0;i<attrs.length;i++) {
			var attr = attrs[i];
			var name = attr.name;
			if (name.indexOf("xmlns:") == 0 && attr.value == uri) {
				return name.substring(6);
			}
		}
		elem = elem.parentNode;
	}
}

/*
 * Returns the uri for the given namespace prefix at the given node context.
 * For browsers that are not namespace-aware.
 */
function _nsURI(/* Node */ node, /* string */ prefix) {
	var qName = "xmlns:" + prefix;
	var attr = node.ownerDocument.documentElement.getAttributeNode(qName);
	if (attr) {
		return attr.nodeValue;
	}
}

/*
 * A simple namespace resolver that resolves a single prefix "ns" to the given
 * namespace, and nothing else.
 */
function _resolveNS(/* string */ ns, /* string */ prefix) {
	return prefix == "ns" ? ns : null;	
}

/*
 * Returns the qualified name of the given local name and namespace in the context
 * of the given node.
 */
function _qName(/* Node */ node, /* string */ ns, /* string */ localName) {
	if (ns) {
		var prefix = _nsPrefix(node, ns);
		if (!prefix) {
			// undefined prefix; return undefined
			return;
		}
		return prefix + ":" + localName;
	}
	return localName;
}

})();
