/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.app.web.internal.welcome.WelcomePage");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("jazz.app.context");

dojo.require("com.ibm.team.app.web.internal.welcome.WelcomeUtil");
dojo.require("com.ibm.team.app.web.internal.welcome.ErrorUtility");
dojo.require("com.ibm.team.process.web.ui.internal.ProcessAreaSelectionDialog");

dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.requireLocalization("com.ibm.team.app.web.internal.welcome", "WelcomePageMessages");

(function() {

var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var WelcomeUtil = com.ibm.team.app.web.internal.welcome.WelcomeUtil;
var ErrorUtility = com.ibm.team.app.web.internal.welcome.ErrorUtility;
var ProcessAreaSelectionDialog = com.ibm.team.process.web.ui.internal.ProcessAreaSelectionDialog;
var ActionRegistry = null; 

dojo.declare("com.ibm.team.app.web.internal.welcome.WelcomePage",	[dijit._Widget, dijit._Templated], {
		
	templatePath: dojo.moduleUrl("com.ibm.team.app.web", "internal/welcome/templates/WelcomePage.html"),

	constructor: function(args) {
		this._frameName = "_welcomeFrame";
		this.messages = dojo.i18n.getLocalization("com.ibm.team.app.web.internal.welcome", "WelcomePageMessages");
	},
	
	postCreate: function() {
		this._resizeIntervalID = null;

		// Perform a late binding to the action registry to ensure that the 
		// work bench is instantiated  
		ActionRegistry = PlatformUI.getWorkbench().getActionRegistry();
		ActionRegistry.registerAction("com.ibm.team.app.viewWelcome", this, "viewWelcome");
		ActionRegistry.registerAction("com.ibm.team.app.projectAction", this, "projectAction");
	},
	
	destroy: function() {
		
		this._clearResizeIntervalID();
		
		this.inherited(arguments);
	},		
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_clearResizeIntervalID: function() {
		if (this._resizeIntervalID) {
			clearInterval(this._resizeIntervalID);
			this._resizeIntervalID = null;
		}		
	},		
	
	viewWelcome: function(args) {
		var dfd = WelcomeUtil.getWelcomeUrl();
		dfd.addCallback(dojo.hitch(this, function (url) {
			if (url) {
				this._welcomeFrame.src = url;
				
				if (!this._resizeIntervalID) {
					this._resizeIntervalID = setInterval(dojo.hitch(this, function () {
						try {
							if (this._welcomeFrame.height != frames[this._frameName].document.documentElement.scrollHeight) {
								this._welcomeFrame.height = frames[this._frameName].document.documentElement.scrollHeight;
							}
						} catch (errorObject) {
							this._clearResizeIntervalID();
							ErrorUtility.logError(errorObject, this.id);
						}
					}), 1000);
				}
				
				this.connect(frames[this._frameName], "onload", function () {
					PlatformUI.getWorkbench().setTitle(frames[this._frameName].document.title);
				});
				this.connect(frames[this._frameName], "onmouseover", function (event) {
					if (event.target.tagName.toLowerCase() === "a") {
						if (!event.target.target) {
							event.target.target = "_parent";
						}
					}
				});
			}
		}));
	},
	
	projectAction: function(args) {
		var projectArea = jazz.app.context.get();
		if (!args.actionID) {
			console.warn("com.ibm.team.app.projectAction executed with no actionID argument - forwarding back to com.ibm.team.app.viewWelcome");
			dojo.hash("action=com.ibm.team.app.viewWelcome", true);
			return;
		}
		if (!projectArea || !projectArea.name) {
			// there is no project area, open a picker
			this.selectionDialog = new ProcessAreaSelectionDialog({
				projects: true,
				onOk: function(project) {
					// TODO: we should not hard code this url structure, it will only work for RTC
					var href = net.jazz.ajax._contextRoot + "/web/projects/" + encodeURIComponent(project.name) + "#action=" + args.actionID;
					location.replace(href);
				},
				onCancel: function() {
					dojo.hash("action=com.ibm.team.app.viewWelcome", true);
				}
			})
		} else {
			// there is a project area, just forward the request
			dojo.hash("action=" + args.actionID, true);
		}
	}
});

})();
