/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.app.web.internal.welcome.WelcomeUtil");

dojo.require("jazz.client.xhr");

(function() {


com.ibm.team.app.web.internal.welcome.WelcomeUtil = {
	
	_doWelcomeCheck: true,
	_isWelcomeFound: false,
	
	isWelcomeEnabled: function () {
		var dfd = new dojo.Deferred();
		
		if (!this._doWelcomeCheck) {
			dfd.callback(this._isWelcomeFound);
			return dfd;
		}
		
		this._doWelcomeCheck = false;
		
		var infoCenterRoot = dojo.getObject("net.jazz.ajax._infoCenterRoot");
		
		if (!infoCenterRoot) {
			dfd.callback(this._isWelcomeFound);
			return dfd;
		}

		var xhrGetUrl = infoCenterRoot + "/welcomeFinder";
		var xhrDfd = jazz.client.xhrGet({
			url: xhrGetUrl,
			headers: {
				'x-jazz-welcome-noredirect': true
			}
		});
		xhrDfd.addCallback(dojo.hitch(this, function (xhrReq) {
			this._welcomeFound(xhrReq);
			dfd.callback(this._isWelcomeFound);
		}, xhrDfd.ioArgs.xhr));
		xhrDfd.addErrback(dojo.hitch(this, function () {
			// Fallback: Try Dojo XHR Get request
			// Web UI proxy fails to proxy the Web Welcome request to the same domain with a 403 error in the RM application.
			// e.g. https://clmwb.ibm.com:9443/rm/proxy?uri=https%3A%2F%2Fclmwb.ibm.com%3A9443%2Fclmhelp%2FwelcomeFinder
			// See: 130928: "The proxy is not configured to allow the request" error when accessing new welcome page.
			this._isWelcomeEnabled2(dfd, xhrGetUrl);
		}));

		return dfd;
	},

	_isWelcomeEnabled2: function (masterDeferred, xhrGetUrl) {
		if (djConfig.isDebug) {
			console.log("Fall back to Dojo XHR GET request");
		}		
		var deferred = dojo.xhrGet({
			url: xhrGetUrl,
			headers: {
				"x-jazz-welcome-noredirect": "true",
				"X-Requested-With": null
			},
			handleAs: "text",
			load: dojo.hitch(this, function(data, ioargs) {
				this._welcomeFound(ioargs.xhr);
				masterDeferred.callback(this._isWelcomeFound);
			}),
			error: dojo.hitch(this, function(errorObject, ioargs) {
				masterDeferred.callback(this._isWelcomeFound);
			})
		});
		return deferred;
	},
	
	_welcomeFound: function(xhrReq) {
		this._welcomeUrl = new dojo._Url(xhrReq.getResponseHeader("Location"));
		this._isWelcomeFound = true;
	},	
	
	getWelcomeUrl: function () {
		var dfd = new dojo.Deferred();
		var enabledDfd = this.isWelcomeEnabled();
		enabledDfd.addCallback(dojo.hitch(this, function (enabled) {
			if (enabled) {
				dfd.callback(this._welcomeUrl);
			} else {
				dfd.callback(null);
			}
		}));
		return dfd;
	}
};
})();
