/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("jazz.util.moduleResource");

/**
 *  Fully qualifies a module-relative path. Takes a string of the following form:
 *      <module name>/<path>
 *  where the module name is a segment containing at least one '.', and returns a 
 *  dojo.moduleUrl(<module name>, <path>)
 * 
 *  It returns just the given "modulePath" argument if:
 *      1) not a string
 *      2) begins with a '/'
 *      3) has no '.'
 *  
 *  PROVISIONAL: This API is considered provisional and may be changed or removed 
 *  in the future with minimal or no warning.
 * @param {String} modulePath The module-relative path to a resource.
 */
jazz.util.moduleResource =  function(/*string*/ modulePath) {
	if (typeof(modulePath) != "string") 
		return modulePath;
	
	// if it's an absolute path, or we don't find a '/', return.
	var slash = modulePath.indexOf("/");
	if (slash <= 0) 
		return modulePath;
	
	// if we don't see a "." in the module path then we don't believe its a module
	var module = modulePath.substr(0, slash);
	if (module.indexOf(".") < 0 || !dojo.exists(module)) 
		return modulePath;
	
	return dojo.moduleUrl(module, modulePath.substr(slash+1, modulePath.length)).toString();
};

})();