/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.toolbar.MenuButton");

dojo.require("jazz.ui.toolbar.TwoStateButton");
dojo.require("jazz.ui.MenuPopup");
dojo.require("dijit._Container");

(function() {


/**
 * @name jazz.ui.toolbar.MenuButton
 * @class <p>Menu Button for the toolbar</p>
 * 	<p>PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * 	future with minimal or no warning.</p>
 * @constructor
 * @extends jazz.ui.toolbar.TwoStateButton
 * @extends jazz.ui.MenuPopup
 */ 
dojo.declare("jazz.ui.toolbar.MenuButton", [jazz.ui.toolbar.TwoStateButton, jazz.ui.MenuPopup], {
	
	/**
	 * URL of an image to use as the button's caret.  The URL may be absolute, relative or 
	 * module-relative. The value is passed through jazz.util.moduleResource, so, for
	 * example, jazz.ui/graphics/myimage.png will resolve properly.
	 * @type String
	 * @default "jazz.ui/graphics/Toolbar/black-caret.gif"
	 */
	caretSrc: "jazz.ui/graphics/Toolbar/black-caret.gif",
	
	/**
	 * Specify whether the button should include a caret (down arrow) automatically. 
	 * @type Boolean
	 * @default true
	 */
	injectCaret: true,
	
	// Internals
	
	classPressed: "open-menu",
	
	postMixInProperties: function(){
		if (!this.menuProvider){
			this.containerNode = dojo.create("div",{});
			this.menuProvider = this._containerMenuProvider;
		}	
		this.inherited(arguments);
	},
	postCreate: function() {
		this.inherited(arguments);
		this._updateClass();
		this.connect(this, "open", function() {
			if (this._menuOpen && !this.attr("pressed")) this.attr("pressed", true);
		});
		this.connect(this, "close", function() {
			if (this.attr("pressed")) this.attr("pressed", false);
		});
	},
	_containerMenuProvider: function(){
		if (!this._menu) this._menu = dojo.query("> [widgetId]", this.containerNode).map(dijit.byNode)[0]; // Widget[]
		return this._menu;
	},
	_updateLabel: function(label) {
		this.inherited(arguments);
		this._updateClass();
	},
	_updateImage: function(src) {
		this.inherited(arguments);
		this._updateClass();
	},
	_updateClass: function(){
		var bool = (!this.pressed && !this.label && !this.imgSrc) || (this.pressed && !this.labelPressed && !this.imgSrc && !this.imgSrcPressed);
		dojo.toggleClass(this.domNode, "caret-only", bool);
	}
});
})();
