/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.toolbar.TwoStateButton");

dojo.require("jazz.ui.toolbar.Button");
dojo.require("jazz.util.moduleResource");
dojo.require("jazz.util.html");

(function() {

var CLASS_NAME = "jazz-ui-toolbar-Button";

/**
 * Constructs a new two-state toolbar button. The button toggles on and off.
 * See superclass jazz.ui.toolbar.TwoStateButton for inherited API. 
 * 
 * All fields may be initialized in the widget constructor, or
 * can be changed after construction by using the attr function.
 * 
 * @name jazz.ui.toolbar.TwoStateButton
 * @class <p>A two-state toolbar button.</p>
 * 	<p>PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * 	future with minimal or no warning.</p>
 * @constructor
 * @extends jazz.ui.toolbar.Button
 */ 
dojo.declare("jazz.ui.toolbar.TwoStateButton", [jazz.ui.toolbar.Button], {
	
	/**
	 * Class to add on the button domNode when the button is in the pressed state.
	 * 
	 * @type String
	 * @default ""
	 */
	classPressed: "",


	/**
	 * URL of a 16x16 image to use as the button's icon when in the pressed state.  The URL 
	 * may be absolute, relative or module-relative. The value is passed through jazz.util.moduleResource, 
	 * so, for example, jazz.ui/graphics/myimage.png will resolve properly.
	 * 
	 * @type String
	 * @default ""
	 */
	imgSrcPressed: "",

	/**
	 * Label to be displayed when the button is in the pressed state.
	 * 
	 * @type String
	 * @default ""
	 */	
	labelPressed: "",

	/**
	 * Tooltip to be displayed when the button is in the pressed stated.
	 * 
	 * @type String
	 * @default ""
	 */		
	titlePressed: "",

	/**
	 * Specifies whether the button is pressed or not.
	 * 
	 * @type Boolean
	 * @default false
	 */
	pressed: false,
	
	_onClick: function(event) {
		if (!this.attr("enabled")) {
			dojo.stopEvent(event);
			return;
		}
		if (this.href == "#") event.preventDefault();
		
		//toggle the pressed state
		var pressed = !this.pressed;
		this.attr("pressed", pressed);
		event.pressed = pressed;

		this.onClick(event);
	},
	_setPressedAttr: function(pressed) {
		var isPressed = this.pressed;
		if ((isPressed&&pressed) || (!(isPressed||pressed))) return;
		if (pressed) {
			dojo.addClass(this.domNode, CLASS_NAME + "-pressed");
			this._updateImage(this.imgSrcPressed||this.imgSrc);
			if (this._created && this.labelPressed) this._updateLabel(this.labelPressed);
			if (this.titlePressed) this.focusNode.title = this.titlePressed;
		} else {
			dojo.removeClass(this.domNode, CLASS_NAME + "-pressed");
			this._updateImage(this.imgSrc);
			if (this._created) this._updateLabel(this.label);
		}
		this.pressed = pressed;
	},
	_setImgSrcPressedAttr: function(attr){
		var src = jazz.util.moduleResource(attr);
		if (this.pressed) this._updateImage(src);
		this.imgSrcPressed = src; 
	},
	_setImgSrcAttr: function(attr){
		var src = jazz.util.moduleResource(attr);
		if (!this.pressed || !this.imgSrcPressed) this._updateImage(src);
		this.imgSrc = src;
	},
	_setLabelAttr: function(label){
		this.label = label;
		if (!this.pressed || !this.labelPressed) this._updateLabel(label);
	},
	_setLabelPressedAttr: function(label) {
		this.labelPressed = label;
		if (!label) label = this.label;
		if (this.pressed) this._updateLabel(label);
	},
	_setClassPressedAttr: function(classPressed) {
		if (this.pressed) {
			if (classPressed != this.classPressed) {
				dojo.removeClass(this.domNode, this.classPressed);
				dojo.addClass(this.domNode, classPressed);	
			}
		}
		this.classPressed = classPressed;
	},
	_setTitlePressedAttr: function(titlePressed) {
		if (this.pressed) {
			this.domNode.title = titlePressed;
		}
		this.titlePressed = titlePressed;
	}
});
})();
