/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("jazz.app.ua.HelpBox");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.ui.StyledBox");
dojo.require("jazz.app.ua.internal.HelpDOMNode");

/*
 * jazz.app.ua.HelpBox
 * 
 * This widget presents help content in a styled box.  The help content can either
 * be pulled from an information center, or it can be provided in a custom DOM node.
 * This widget extends jazz.ui.StyledBox.  Refer to the jazz.ui.StyledBox
 * documentation for additional options and API
 * 
 * To enable help content in a Jazz Team Server, add the <jd:infoCenter> property to your
 * root services document.  For example:
 * 
 * <jd:infocenterRoot rdf:resource="http://publib.example.com/infocenter/rtc/v1r0m1" />
 * 
 * @since 0.7
 */

dojo.declare("jazz.app.ua.HelpBox", [jazz.ui.StyledBox], {
	
	/*
	 * Defaults: Pass any of these properties into the constructor to set them.
	 * 
	 * The UA team recommends keeping these default options for the styled box
	 * in order to maintain a consistent look and feel for help content across
	 * the products.
	 *  
	 * See jazz.ui.StyledBox for more options
	 */
	
	trim: jazz.ui.styledBox.TrimStyles.GOLD,
	showBackground: true,
	headerHoverState: false,
	primaryTitle: "",						
	
	/*
	 * The constructor expects either uaKey or contentNode to be defined.
	 * 
	 * uaKey is a string that is a key into the information center.  If uaKey is defined, the
	 * widget will retrieve help content from the information center based on that key.
	 * 
	 * contentNode is a DOM node.  If contentNode is specified, the help content that is displayed
	 * on hover will be the contents of this DOM node.
	 */
	
	constructor: function (args) {
		this.uaKey = args.uaKey;		// enter a uaKey
		if (args.uaKey) {
			this.helpDOMNode = new jazz.app.ua.internal.HelpDOMNode({
				uaKey: this.uaKey
			});
			this.contentNode = this.helpDOMNode.domNode;
		} else {
			this.contentNode = args.contentNode || document.createElement("div");
		}
		this.contentNode.style.padding = "0px 7px 7px 7px";
	},
	
	destroy: function() {
		
		if (this.helpDOMNode) {
			this.helpDOMNode.destroy();
			this.helpDOMNode = null;
		}
		
		this.inherited(arguments);
	}
	
});	
})();
