/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("jazz.app.ua.PageHoverHelp");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.xml.parser");

dojo.require("jazz.ui.HoverPopup");
dojo.require("jazz.app.ua.internal.HelpHoverPopup");

/*
 * jazz.app.ua.PageHoverHelp
 * 
 * This widget provides a hover-over help icon that is meant to provide page-level
 * help content.  This widget should generally not be used more than once on a single
 * page.  The help content can either be pulled from an information center, or it can
 * be provided in a custom DOM node.  Typical placement of this widget is at the top
 * of a page to the right of the page title.
 * 
 * To enable help in a Jazz Team Server, add the <jd:infoCenter> property to your
 * root services document.  For example:
 * 
 * <jd:infocenterRoot rdf:resource="http://publib.example.com/infocenter/rtc/v1r0m1" />
 * 
 * @since 0.7
 */

dojo.declare("jazz.app.ua.PageHoverHelp", [dijit._Widget, dijit._Templated], {
	templateString: "<img src='${_imageSrc}' dojoAttachPoint='_helpImg' style='margin-right:5px' />",
	_imageSrc: dojo.moduleUrl("jazz.app", "ua/images/page-help.gif"),
	_imageHoverSrc: dojo.moduleUrl("jazz.app", "ua/images/page-help-hov.gif"),
	
	/*
	 * uaKey is a string that is a key into the information center.  If uaKey is defined, the
	 * widget will retrieve help content from the information center based on that key.
	*/
	uaKey: "",

	/*
	 * contentNode is a DOM node.  If contentNode is specified, the help content that is displayed
	 * on hover will be the contents of this DOM node.
	*/
	contentNode: null,


	/*
	 * contentString is a string.  If contentString is defined, then the literal contents of that
	 * string will be displayed on hover.  No custom styling can be provided when using contentString.
	 * If any HTML markup is required, use contentNode instead.
	*/
	contentString: null,
	
	postCreate: function () {
		this.connect(this._helpImg, "onmouseover", function (event) {
			event.currentTarget.src = this._imageHoverSrc;
		});
		this.connect(this._helpImg, "onmouseout", function (event) {
			event.currentTarget.src = this._imageSrc;
		});
		if (this.uaKey) {
			this.hoverPopup = new jazz.app.ua.internal.HelpHoverPopup({
				uaKey: this.uaKey,
				around: this._helpImg
			});
		} else {
			if (!this.contentNode) {
				this.contentNode = document.createElement("div");
				this.contentNode.appendChild(document.createTextNode(this.contentString));
			}
			this.hoverPopup = new jazz.ui.HoverPopup({
				around: this._helpImg,
				createContents: dojo.hitch(this, function (popup) {
					return this.contentNode;
				})
			});
		}
	},
	
	destroy: function() {
		
		if (this.hoverPopup) {
			this.hoverPopup.destroy();
			this.hoverPopup = null;
		}
		
		this.inherited(arguments);
	}
	
});
})();
