/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("jazz.app.ua.TextHoverHelp");
 
dojo.require("dijit._Widget");
dojo.require("jazz.ui.HoverPopup");
dojo.require("jazz.app.ua.internal.HelpHoverPopup");

/*
 * jazz.app.ua.TextHoverHelp
 * 
 * This widget provides a help popup when the user hovers over some text.  A common
 * use is to provide help when a user hovers over an input field's label, but it can
 * be used for any text.  The help content can either be pulled from an information
 * center, or it can be provided in a custom DOM node or string.
 * 
 * To enable help in a Jazz Team Server, add the <jd:infoCenter> property to your
 * root services document.  For example:
 * 
 * <jd:infocenterRoot rdf:resource="http://publib.example.com/infocenter/rtc/v1r0m1" />
 * 
 * @since 0.7
 */

dojo.declare("jazz.app.ua.TextHoverHelp", [dijit._Widget], {
	/*
	 * The constructor expects one of uaKey, helpContentNode, or helpContentString to be defined.
	 * It also expects either contentNode or contentString to be defined.
	 * 
	 * uaKey is a string that is a key into the information center.  If uaKey is defined, the
	 * widget will retrieve help content from the information center based on that key.
	 * 
	 * helpContentNode is a DOM node.  If helpContentNode is specified, the help content that is displayed
	 * on hover will be the contents of this DOM node.
	 * 
	 * helpContentString is a string.  If helpContentString is defined, then the literal contents of that
	 * string will be displayed on hover.  No custom styling can be provided when using helpContentString.
	 * If any HTML markup is required, use contentNode instead.
	 * 
	 * contentNode is a DOM node.  This is the DOM node the hover help will be attached to.
	 * 
	 * contentString is a string.  The widget will created a new DOM node containing this string, and the
	 * hover help will be attached to that DOM node.
	 */
	constructor: function (args) {
		this.uaKey = args.uaKey;
		this.helpContentNode = args.helpContentNode;
		this.helpContentString = args.helpContentString;
		this.contentNode = args.contentNode;
		this.contentString = args.contentString;
	},
	postCreate: function () {
		if (this.contentNode) {
			this.domNode = this.contentNode;
		} else if (this.contentString) {
			this.domNode = document.createElement("div");
			this.domNode.appendChild(document.createTextNode(this.contentString));
		}
		if (this.uaKey) {
			this.hoverPopup = new jazz.app.ua.internal.HelpHoverPopup({
				uaKey: this.uaKey,
				around: this.domNode
			});
		} else {
			if (!this.helpContentNode) {
				this.helpContentNode = document.createElement("div");
				this.helpContentNode.appendChild(document.createTextNode(this.helpContentString));
			}
			this.hoverPopup = new jazz.ui.HoverPopup({
				around: this.domNode,
				createContents: dojo.hitch(this, function (popup) {
					return this.helpContentNode;
				})
			});
		}
		dojo.toggleClass(this.domNode, "jazz-app-ua-TextHoverHelp", true);
	},
	
	destroy: function() {
		
		if (this.hoverPopup) {
			this.hoverPopup.destroy();
			this.hoverPopup = null;
		}
		
		this.inherited(arguments);
		
	}
	
});
})();