/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.CustomPreferenceProvider");

(function(){

/*
 * The interface that custom preference providers must implement.
 * 
 * CAUTION: Must remain backward compatible for cross-version interoperability.
 * Dashboard framework must check for existence of new methods added after 0.7.
 * 
 * PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * future with minimal or no warning.
 * 
 * @since 0.6
 */	
dojo.declare("com.ibm.team.dashboard.web.ui.CustomPreferenceProvider",
	null, {
	
	/*
	 * Returns a screen-friendly representation of the preference given the stored value.
	 * Will be applied as innerHTML and, therefore, can contain tags. If the label is not
	 * yet available, null should be returned and the framework will start a polling loop.
	 */
	getLabel: function(value){
		return null;
	},
	
	/*
	 * Returns the label that should be used while the appropriate label is being loaded.
	 * Will be called when getLabel(value) returns null;
	 */
	getLoadingLabel: function() {
		return "";
	},
	
	/*
	 * Creates the chooser widget in the provided parent. 
	 * 
	 * When a selection is made in the chooser it should call the function passed in
	 * the onOk parameter with the new preference value as the first parameter and
	 * the preference id as the second parameter (e.g. onOk(value, prefId)).
	 * 
	 * @param Function onOk(String value, String prefId)
	 * @param Object settings
	 * 				. Object preferences
	 *					. <Type-1> <Preference-ID-1>
	 *					. <Type-2> <Preference-ID-2>
	 *					. ...
	 *					. <Type-N> <Preference-ID-N>
	 *				. Number scope
	 *				. String scopeItemId
	 *				. boolean showBackground
	 *				. String title
	 *				. Number trim
	 *				. boolean userTitle
	 * @param Object viewlet
	 * @return void
	 */
	createChooser: function(onOk, settings, viewlet) {
	},
	
	/*
	 * Called when the user saves the settings area. This can be used by preference providers
	 * that need to perform some additional action on save.
	 * 
	 * @since 0.7
	 */
	save: function() {
	}
});

})();
