/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.Dashboard");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.DashboardError");
dojo.require("com.ibm.team.dashboard.web.ui.internal.DashboardContainer");
dojo.require("com.ibm.team.dashboard.web.ui.internal.DashboardInitializationData");
dojo.require("com.ibm.team.dashboard.web.ui.internal.ViewletPage");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.process.web.client.internal.ProcessClient");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.dashboard.web.util.DashboardUtil");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.Observable");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ServiceTracker");
dojo.require("jazz.core.loader")
dojo.require("jazz.ensemble.internal.Schema");
dojo.require("jazz.ensemble.internal.client.RestClient");
dojo.require("jazz.ensemble.internal.client.HttpConstants");
dojo.require("jazz.ensemble.internal.util.Xml");
dojo.require("jazz.ensemble.internal.util.XmlBindingFactory");
dojo.require("jazz.util.ResizeNotifier");
dojo.require("jazz.ui.StyledBox");
dojo.require("jazz.ui.Dialog");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

var HttpConstants = jazz.ensemble.internal.client.HttpConstants;
var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var DashboardInitializationData = com.ibm.team.dashboard.web.ui.internal.DashboardInitializationData;
var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
var Schema = jazz.ensemble.internal.Schema;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServiceTracker = com.ibm.team.dashboard.web.ui.internal.util.ServiceTracker;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var DashboardContainer = com.ibm.team.dashboard.web.ui.internal.DashboardContainer;
var DashboardError = com.ibm.team.dashboard.web.ui.internal.DashboardError;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var DashboardUtil = com.ibm.team.dashboard.web.util.DashboardUtil;
var Observable = com.ibm.team.dashboard.web.ui.internal.util.Observable;
var RestClient = jazz.ensemble.internal.client.RestClient;
var ViewletPage = com.ibm.team.dashboard.web.ui.internal.ViewletPage;
var Xml = jazz.ensemble.internal.util.Xml;
var XmlBindingFactory = jazz.ensemble.internal.util.XmlBindingFactory;

var messages;

/**
 * An embeddable dashboard widget which allows any page to contain a dashboard. The
 * widget is given a URL to a dashboard resource, where it will GET and PUT to load
 * and persist the state of the dashboard.
 * 
 * @see https://jazz.net/wiki/bin/view/Main/DashboardResourceAPI
 * @see https://jazz.net/wiki/bin/view/Main/DashboardServiceAPI
 * @since 0.7
 * 
 * NOTE: This is provisional API. Clients using this API do so with the awareness that
 * this class may be removed or substantially changed at future times without warning.
 */
dojo.declare("com.ibm.team.dashboard.web.ui.Dashboard",
	[Observable, dijit._Widget, dijit._Templated], {
	
	templateString: "<div class=\"com-ibm-team-dashboard-web-ui-internal-Dashboard\"></div>",
	
	/**
	 * (Required)
	 * The URL of the dashboard resource containing the state of the dashboard
	 * to display. e.g. "https://example.com/jts/dashboards/123"
	 * 
	 * @see https://jazz.net/wiki/bin/view/Main/DashboardResourceAPI
	 * @see https://jazz.net/wiki/bin/view/Main/DashboardServiceAPI
	 */
	url: null,
	
	/**
	 * A callback that will be fired when the dashboard is dirtied. It will be provided
	 * one parameter which indicates if the dashboard is dirty or clean.
	 */
	onDirtyStateChange: function(isDirty) {},
	
	/**
	 * (Required for SCOPE_PROJECT_AREA, SCOPE_TEAM_AREA, Optional for SCOPE_CONTRIBUTOR)
	 * The name of the project area containing the dashboard, e.g. "Jazz Foundation".
	 * 
	 * @deprecated use the url parameter to specify which dashboard resource to load
	 */
	projectArea: null,
	
	/**
	 * (Required) The scope of the dashboard; one of the following properties from
	 * DashboardConstants: SCOPE_PROJECT_AREA, SCOPE_TEAM_AREA.
	 * 
	 * @deprecated use the url parameter to specify which dashboard resource to load
	 */
	scope: null,
	
	/**
	 * (Required for SCOPE_TEAM_AREA) This is required if scope is SCOPE_TEAM_AREA,
	 * where it is an array of team names; a path from a root team to the target team,
	 * e.g. ["Jazz Foundation Development", "Web UI", "Dashboard"]. The use of this
	 * param may be	expanded for other scopes in the future.
	 * 
	 * @deprecated use the url parameter to specify which dashboard resource to load
	 */
	scopeItem: null,
	
	/**
	 * Constructs a new dashboard widget. The required parameters listed above must be
	 * passed in as properties to the first argument, as well as any optional arguments.
	 * 
	 * e.g.
	 * var myDashboard = new Dashboard({
	 *    url: "https://example.com/jts/dashboards/123"
	 * });
	 * myDomNode.appendChild(myDashboard.domNode);
	 */
	constructor: function(args) {
		if (!messages) {
			messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
		}
	},
	
	/**
	 * Destroys the dashboard widget. Should be called when the dashboard is removed
	 * from a page and no longer needed.
	 */
	/* void */ destroy: function() {
		if (this._redirectTimeout) {
			clearTimeout(this._redirectTimeout);
			delete this._redirectTimeout;
		}
		if (this._serviceTracker) {
			var requests = this._serviceTracker.getActiveRequests();
			for (var i=0;i<requests.length;i++) {
				requests[i].abort();
			}
		}
		if (this.navigateAwayHandle) {
			PlatformUI.getWorkbench().unregisterBodyClickHandler(this.navigateAwayHandle);
		}
		if (this.onBeforeUnloadHandle){
			PlatformUI.getWorkbench().unregisterOnBeforeUnloadHandler(this.onBeforeUnloadHandle);	
		}
		if (this._contentWidget) {
			this._contentWidget.destroy();
			delete this._contentWidget;
		}
		this.inherited(arguments);
	},
	
	/**
	 * Returns a boolean indicating if the dashboard is currently dirty.
	 */
	isDirty: function() {
		if (this._contentWidget && this._contentWidget.isDirty) return this._contentWidget.isDirty();
		return false;
	},
	
	/**
	 * Returns a boolean indicating if the dashboard is currently unsaved. Will be true if the dashboard
	 * is dirty or if a save is in progress.
	 */
	isUnsaved: function() {
		if (this._contentWidget && this._contentWidget.isUnsaved) return this._contentWidget.isUnsaved();
		return false;
	},
	
	/*------------------------------------------------------------------------
	 * INTERNAL CODE BELOW - DO NOT CALL THIS CODE.
	 * 
	 * If you need additional functionality, please request API via work item.
	 *-----------------------------------------------------------------------*/
	
	/**
	 * Only applicable to contributor scope dashboards. If false, the "Sharing Options"
	 * button will be hidden.
	 */
	allowSharing: false,
	
	allowSetDefault: false,

	/* void */ showTab: function(/* string */ tab) {
		if (this._contentWidget instanceof DashboardContainer) {
			if (tab != this.tab) {
				this.tab = tab;
				if (tab == null) {
					this._contentWidget.showViewletPage(0);
				} else {
					this._contentWidget.showViewletPageById(tab);
				}
			}
			this.setActive(true);
		}
	},
	
	/* void */ setActive: function(active) {
		if (this._contentWidget instanceof DashboardContainer) {
			this._contentWidget.setActive(active);
		}
	},
	
	/* void */ postCreate: function() {
		var args = {tab: this.tab};
		if (this.url) {
			args.id = this.url.substring(this.url.lastIndexOf("/") + 1);
		}
		else {
			switch (this.scope) {
				case DashboardConstants.SCOPE_TEAM_AREA:
					args.team = this.scopeItem.join("/");
					break;
				case DashboardConstants.SCOPE_CONTRIBUTOR:
					// internal usage only
					for (var i in this.scopeItem) {
						args[i] = this.scopeItem[i];
					}
					break;
			}
		}
		this.viewDashboard(args, this.projectArea);
	},
	
	/* object */ getBundle: function() {
		return this._contentWidget && this._contentWidget.dashboardBundle;
	},
	
	/* ServiceTracker */ getServiceTracker: function() {
		if (!this._serviceTracker) {
			this._serviceTracker = new ServiceTracker();
		}
		return this._serviceTracker;
	},
	
	/* string */ getTitle: function(includeTab) {
		if (this._contentWidget && this._contentWidget instanceof DashboardContainer) {
			var title = this._contentWidget.getTitle();
			if (includeTab && this._contentWidget.currentPage && this._contentWidget.currentPage.page) {
				var tabTitle = this._contentWidget.currentPage.page.getTitle();
				return dojo.string.substitute(messages.dashboardTitleWithTab, [title, tabTitle]);
			}
			return title;
		}
		return "";
	},
	
	/* boolean */ isVisible: function() {
		return this._isVisible;
	},
	
	/* String */ handleOnBeforeUnload: function(event) {
		if (this._redirectTimeout) {
			clearTimeout(this._redirectTimeout);
			delete this._redirectTimeout;
		}
		// if the current content widget is a dashboard, ask it for an appropriate message
		if (this._contentWidget && this._contentWidget.dashboardResource)
			return this._contentWidget.handleOnBeforeUnload();
		return false;
	},
	
	/* boolean */ promptNavigateAway: function() {
		var doNavigate = true
		if (this._contentWidget && this._contentWidget instanceof DashboardContainer && this._contentWidget.dashboardResource) {
			doNavigate = this._contentWidget.promptNavigateAway();
		}
		if (doNavigate) {
			this._isVisible = false;
		}
		return doNavigate;	
	},
	
	viewDashboard: function(args, projectArea) {
		var dfd = DashboardInitializationData.load();
		dfd.addCallback(dojo.hitch(this, this.viewDashboard2, args, projectArea));
	},
	
	viewDashboard2: function(args, projectArea) {
		this._isVisible = true;
		this._checkNavigateAwayHandle();
		
		var tab = args.tab;
		delete args.tab;
		delete args.action;
		
		var serviceObj = {
			dashboard: this,
			args: args,
			tab: tab,
			success: function(/* ResponseDTO */ responseDTO) {
				switch(responseDTO.status) {
					case "MIGRATION_REQUIRED":
					case "MIGRATION_IN_PROGRESS":
						this.dashboard._showMigrationPage(args, projectArea);
						break;
					case "NOT_FOUND":
						this.dashboard._createError({
							title: messages.errorTitleNotFound,
							message: messages.errorNotFound
						});
						break;
					case "MOVED":
						var linkMarkup = [
							"<a href=\"",
							responseDTO.msg,
							"\">",
							responseDTO.msg,
							"</a>"
						].join("");
						this.dashboard._createError({
							title: messages.movedTitle,
							message: dojo.string.substitute(messages.movedMsg, [linkMarkup])
						});
						// redirect in 10 seconds
						this.dashboard._redirectTimeout = setTimeout(dojo.hitch(this, function() {
							// clearTimeout doesn't always work on IE7
							if (this.dashboard._redirectTimeout) {
								location.replace(responseDTO.msg);
							}
						}), 10000);
						break;
					case "ERROR":
						// invalid request or server error
						this.dashboard._createError({message: responseDTO.msg});
						break;
					case "OK":
					default:
						// store license permissions as global
						dojo.setObject("com.ibm.team.dashboard.internal.permissions", responseDTO.data.permissions);
						var dashboardUrl = responseDTO.data.dashboardUrl;
						var contentWidget = this.dashboard._contentWidget;
						if (contentWidget.dashboardUrl == dashboardUrl) {
							if (contentWidget.tabs) {
								contentWidget.showViewletPageById(this.tab);
							}
							dojo.publish("jazz.util.ResizeNotifier.resizeEvent");
							this.dashboard.propertyChanged("onload");
							this.dashboard._isLoaded = true;
						}
						else {
							if (contentWidget && contentWidget.dashboardUrl) {
								this.dashboard._createContentWidget(this.tab);
							}
							var headers = {};
							headers[HttpConstants.ACCEPT] = HttpConstants.CT_APPLICATION_RDF_XML;
							// Add a bogus query param to make the RDF URL distinct from the HTML page
							//    See: 130601
							if (dashboardUrl.indexOf("?") > -1)
								dashboardUrl += "&";
							else
								dashboardUrl += "?";
							dashboardUrl += "contentType=application%2Frdf%25xml";
							var dfd = RestClient.getResource(dashboardUrl, RestClient.HANDLE_AS_XML, headers);
							dfd.addCallback(dojo.hitch(this, function(/* Resource */ resource) {
								var bundle = responseDTO.data;
								bundle.dashboard = resource;
								resource.binding = XmlBindingFactory.createBinding(resource.content.documentElement, SCHEMA_DASHBOARD);
								this.dashboard._initializeContentWidget(bundle);
								dojo.publish("jazz.util.ResizeNotifier.resizeEvent");
								this.dashboard.propertyChanged("onload");
								this.dashboard._isLoaded = true;
								return resource;
							}));
							dfd.addErrback(dojo.hitch(this, function(/* Error */ e) {
								this.dashboard._createError({message: messages.errorRetrieving + "<br/><br/>" + e.message});
								return e;
							}));
						}
						break;
				}
			},
			failure: function(/* Error */ error) {
				var actionLink;
				var msg;
				if (error) {
					switch (error.status) {
						case 0:
							error.message = messages.connectionFailure;
							break;
						case 401:
							// assume it's the popup blocker
							actionLink = {
								label: messages.tryAgain,
								icon: dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/Dashboard/icons/action_refresh.gif").toString(),
								onclick: dojo.hitch(this, function() {
									this.dashboard.viewDashboard(args, projectArea);
								})
							};
							msg = error.message;
							break;
						case 403:
							error.message = messages.errorNoPermission;
							break;
					}
				}
				if (!msg) {
					msg = messages.errorRetrieving + "<br/><br/>" + error.message;
				}
				this.dashboard._createError({
					message: msg,
					actionLink: actionLink
				});
			}
		};
		this._createContentWidget(tab);
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		DashboardClient.getDashboard(args.id, args.team, projectArea, srh, this.getServiceTracker());
	},
	
	setCurrentDeleted: function () {
		this._contentWidget.deleted = true;
	},
	// ensure onload callbacks are fired if they are added after load completes
	// see work item 73132
	addListener: function(callback, property) {
		this.inherited(arguments);
		if (property == "onload" && this._isLoaded)
			callback("onload");
	},
	_createContentWidget: function(/* string */ dashboardTabId) {
		if (this._contentWidget){
			this._destroyContentWidget();
		}
		if (!dashboardTabId)
			dashboardTabId = "tab1";
		this._contentWidget = new DashboardContainer({dashboardWidget: this, selectedTabId: dashboardTabId, allowSharing: this.allowSharing, allowSetDefault: this.allowSetDefault});
		this._contentWidget.addListener(dojo.hitch(this, this._onTabChange), "tab");
		this._contentWidget.addListener(dojo.hitch(this, this._bubblePropertyChange), "title");
		this._contentWidget.addListener(dojo.hitch(this, this.propertyChanged), "onsave");
		this._contentWidget.addListener(dojo.hitch(this, this.propertyChanged), "ondelete");
		this._setContent(this._contentWidget.domNode);
		this._setDirtyConnect = this.connect(this._contentWidget,"setDirty",dojo.hitch(this,function(isDirty) {
			this.onDirtyStateChange(typeof isDirty == "undefined" ? true : isDirty);
		}));
	},
	_onTabChange: function(/* string */ property, /* string */ tab) {
		this.tab = tab;
		this.propertyChanged(property, tab);
	},
	_bubblePropertyChange: function(/* string */ property, /* string */ arg) {
		this.propertyChanged(property, arg);
	},
	_initializeContentWidget: function(/* DashboardBundleDTO */ bundle) {
		if (this._contentWidget && this._contentWidget.setContent) {
			this._contentWidget.setContent(bundle);
		}
	},
	_createError: function(/* object */ args) {
		// clear old content first
		if (this._contentWidget) {
			this._destroyContentWidget();	
		}
		this._contentWidget = new DashboardError(args);
		this._contentWidget.populate();
		this._setContent(this._contentWidget.domNode);
	},
	_setContent: function(/* Node */ node) {
		this.domNode.innerHTML = "";
		this.domNode.appendChild(node);
	},
	_destroyContentWidget: function() {
		if (!this._contentWidget) return;
		if (this._setDirtyConnect) {
			this.disconnect(this._setDirtyConnect);
			delete this._setDirtyConnect;
		}
		this._contentWidget.destroy();
		this._contentWidget = null;
	},
	_checkNavigateAwayHandle: function() {
		if (!this.onBeforeUnloadHandle){
			this.onBeforeUnloadHandle = PlatformUI.getWorkbench().registerOnBeforeUnloadHandler(dojo.hitch(this, this.handleOnBeforeUnload), /*priority*/ 10000);	
		}
	},
	_showMigrationPage: function(/* object */ args, /* string */ projectArea) {
		// lazy load
		var className = "com.ibm.team.dashboard.web.ui.internal.MigrationStatusPage";
		var MigrationStatusPage = dojo.getObject(className);
		if (!MigrationStatusPage) {
			jazz.core.loader.load_async(className, dojo.hitch(this, this._showMigrationPage, args, projectArea));
			return;
		}
		
		// clear old content first
		if (this._contentWidget) {
			this._destroyContentWidget();	
		}
		this._contentWidget = new MigrationStatusPage({
			handleRefresh: dojo.hitch(this, this.viewDashboard, args, projectArea)
		});
		this._setContent(this._contentWidget.domNode);
	}
});

var SCHEMA_DASHBOARD = [
	{
		get: "getAbout",
		set: "setAbout",
		type: XmlBindingFactory.STRING,
		nodeType: XmlBindingFactory.ATTRIBUTE,
		attrName: "about",
		attrNS: Xml.namespace.rdf
	},
	{
		get: "getRelation",
		set: "setRelation",
		type: XmlBindingFactory.STRING,
		nodeType: XmlBindingFactory.ATTRIBUTE,
		elemName: "relation",
		elemNS: Xml.namespace.dublin
	},
	{
		get: "getType",
		set: "setType",
		type: XmlBindingFactory.STRING,
		nodeType: XmlBindingFactory.CHILD_ELEMENT_TEXT,
		elemName: "type",
		elemNS: Xml.namespace.dublin
	},
	{
		get: "getTitle",
		set: "setTitle",
		type: XmlBindingFactory.STRING,
		nodeType: XmlBindingFactory.CHILD_ELEMENT_TEXT,
		elemName: "title",
		elemNS: Xml.namespace.dublin
	},
	{
		get: "getPages",
		set: "setPages",
		type: XmlBindingFactory.OBJECT,
		nodeType: XmlBindingFactory.CHILD_ELEMENT,
		elemName: "pages",
		elemNS: Xml.namespace.jw,
		schema: [
			{
				get: "getPages",
				set: "setPages",
				create: "createPage",
				type: XmlBindingFactory.OBJECT,
				nodeType: XmlBindingFactory.CHILD_ELEMENT,
				elemName: "Page",
				elemNS: Xml.namespace.jw,
				multiValued: true,
				schema: ViewletPage.SCHEMA
			}
		]
	}
];

})();
