/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("net.jazz.ajax.ui.PlatformUI");

dojo.require("net.jazz.ajax.ui.Workbench");
dojo.require("jazz.core.loader");
dojo.require("jazz.core.RegistryFactory");

net.jazz.ajax.ui.PlatformUI = {
	
	// summary:
	//		central object of the Jazz Web UI framework
	// description:
	//		Used to create and run the web workbench and to locate the running
	//		web workbench. Clients must call all methods in a 'static' manner,
	//		e.g.:
	//
	//		net.jazz.ajax.ui.PlatformUI.getWorkbench();
	//
	//		This is an object, not a class, and therefore cannot be
	//		instantiated.
	// since:
	//		0.5
	
	createAndRunWorkbench: function(applicationId) {
		// summary:
		//		creates and runs the application specified by 'applicationId'
		// description:
		//		he Jazz Ajax Framework calls this function exactly once at the
		//		beginning of the web page's lifecycle
		// since:
		//		0.5
		
		this._workbench = new net.jazz.ajax.ui.Workbench({id: applicationId});
		
		this._applicationId = applicationId;
		var RegistryFactory = jazz.core.RegistryFactory;
		var extensionPoint = RegistryFactory.getRegistry().getExtensionPoint("net.jazz.ajax.applications");
		var self = this;
		dojo.forEach(extensionPoint.getExtensions(), function(extension) {
			dojo.forEach(extension.getConfigurationElements(), function(configElement) {
				if (configElement.getName() !== "application")
					return;
				/*
				 * The ID should now be set on the configElement, but for backwards compatibility,
				 * continue to check the extension's ID too
				 */
				if (configElement.getAttribute("id") != applicationId && extension.getUniqueIdentifier() != applicationId)
					return;
				var appClassName = configElement.getAttribute("jsclass");
				
				// This load_async should be a no-op because the code should be loaded already.
				jazz.core.loader.load_async(appClassName, function() {
					var appConstructor = dojo.getObject(appClassName);
					new appConstructor({
						workbench: self._workbench
					});
				});
			});
		});
	},
	
	getWorkbench: function() {
		// summary:
		//		Returns a reference to the running the web workbench. Fails if
		//		the web workbench has not been created yet.
		// since:
		//		0.5
		return this._workbench;
	},
	
	//
	// Internal methods; do not call directly
	//
	
	_getApplicationId: function() {
		return this._applicationId;
	}
};