/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.AllShared");

dojo.require("dojo.i18n");

dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.util.DashboardUtil");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.dashboard.web.client.internal.PersonalDashboardClient");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ServiceTracker");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("jazz.ui.FilterBox");
dojo.require("jazz.util.html");
dojo.require("jazz.ui.StyledTable");
dojo.require("jazz.ui.ResourceLink");
dojo.require("jazz.ui.PaginationControl");
dojo.require("jazz.ui.toolbar.Button");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.util.i18n");

dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

var HTML = jazz.util.html;
var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var DashboardUtil = com.ibm.team.dashboard.web.util.DashboardUtil;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var PersonalDashboardClient = com.ibm.team.dashboard.web.client.internal.PersonalDashboardClient;
var ServiceTracker = com.ibm.team.dashboard.web.ui.internal.util.ServiceTracker;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var FilterBox = jazz.ui.FilterBox;
var StyledTable = jazz.ui.StyledTable;
var ResourceLink = jazz.ui.ResourceLink;
var PaginationControl = jazz.ui.PaginationControl;

var PlatformUI = net.jazz.ajax.ui.PlatformUI;

var messages;

var PAGE_SIZE = 50;

dojo.declare("com.ibm.team.dashboard.web.ui.internal.AllShared",
	[dijit._Widget, dijit._Templated], {
	
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/AllShared.html"),
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
		this.allPersonalDashboards = messages.allPersonalDashboards;
		this.allDashboardsPrev = messages.allDashboardsPrev;
		this.allDashboardsNext = messages.allDashboardsNext;
	},
	postCreate: function() {
		this._totalSharedDashboards = 0;
		this._filterBox = new FilterBox({onChange: dojo.hitch(this,this._doFilter)});
		this._filterArea.appendChild(this._filterBox.domNode);
		this._pagination.onNextClick = dojo.hitch(this, this._handleNext);
		this._pagination.onPreviousClick = dojo.hitch(this, this._handlePrevious);
		this._pagination.increment = PAGE_SIZE;
		this._table = new StyledTable({columns: [{label: messages.allPersonalColumnTitle, width: "50%", sortable: true}, {label: messages.allPersonalColumnAuthor, width: "50%", sortable: true}],
								trim: jazz.ui.StyledTable.TrimStyles.PRIMARY_HEADLESS,
								handleSort: dojo.hitch(this,this._handleSort),
								sortBy: 0});
		this._resultsArea.appendChild(this._table.domNode);
		this._currentFilter = "";
		this._reset();
		this._runPersonalQuery();
		this._runSharedQuery();
	},
	
	_reset: function() {
		this.page = 0;
		this._sharedDashboards = [];
	},
	
	_handleNext: function() {
		this.page++;
		if (this._nextPage && this._nextPage.page === this.page) {
			this._runSharedQuery(this._nextPage.url);
			delete this._nextPage;
		}
		else
			this._renderResults();
	},
	
	_handlePrevious: function() {
		this.page--;
		this._renderResults();
	},
	
	_handleSort: function() {
		this._reset();
		this._sortPersonal(this._personalDashboards);
		this._sortPersonal(this._matchingPersonalDashboards);
		this._runSharedQuery();
	},
	
	_handleDashboardCreated: function(url) {
		if (PlatformUI && !PlatformUI.getWorkbench().shouldVetoNavigation(url)) {
			/* Forward the browser to the new personal dashboard */
			location.href = url;
		} else {
			/* We've been veto'd, so simply display a message to indicate the dashboard 
			 * has been successfully created */
			this._createDashboardInProgress = false;
			this._messageBox.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.OK,
				summary: this.messages.createPersonalDashboardSuccessful,
				closable: true
			});
		}
	},
		
	_handleCreateDashboardError: function(error) {
		this._messageBox.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
			summary: error.message,
			closable: true, 
			errorObject: error.errorObj
		});
		this._createDashboardInProgress = false;
	},
	
	_handleCreateDashboard: function() {
		if (this._createDashboardInProgress) {
			/* Prevent users from clicking the 'Create' link if there is already an
			 * ongoing request */
			return;
		}
		this._createDashboardInProgress = true;
		this._messageBox.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
			summary: this.messages.createPersonalDashboardMessage,
			closable: false
		});
	
		var dfd = PersonalDashboardClient.createDashboard();
		dfd.addErrback(dojo.hitch(this, this._handleCreateDashboardError));
		dfd.addCallback(dojo.hitch(this, this._handleDashboardCreated));
	},
	
	_runPersonalQuery: function() {
		var deferred = PersonalDashboardClient.get();
		deferred.addCallback(dojo.hitch(this, function(/* LeftNavDTO */ result) {
			this._contributor = result.contributor;
			// don't modify original
			this._personalDashboards = this._matchingPersonalDashboards = this._sortPersonal(dojo.clone(result.personalDashboards) || []);
			if (this._pendingSharedResult)
				this._pendingSharedResult();
		}));
		deferred.addErrback(function(errorObj) {
			dojo.publish(Constants.serverError, [error]);
		});
	},
	
	_runSharedQuery: function(/* string */ pageUrl) {
		PlatformUI.getWorkbench().showMessage(messages.allPersonalSearching);
		if (this._runQueryTimeout) {
			clearTimeout(this._runQueryTimeout);
			delete this._runQueryTimeout;
		}
		var orderBy = this._table.sortBy ? "author" : "title";
		var orderByAscending = this._table.sortByAscending;
		var serviceObject = {
			page: this.page,
			filter: this._currentFilter,
			self: this,
			success: function(results) {
				this.self._totalSharedDashboards = results.totalResults || 0;
				if (!this.self._personalDashboards) {
					this.self._pendingSharedResult = dojo.hitch(this,this.success,results);
					return;
				}
				if (this.filter === this.self._currentFilter && results.dashboards) {
					var offset = PAGE_SIZE * this.page;
					for (var i = 0; i < results.dashboards.length; i++) {
						this.self._sharedDashboards[offset + i] = results.dashboards[i];
					}
				}
				this.self._nextPage = results.nextPage ? {
					page: this.page + 1,
					url: results.nextPage
				} : null;
				this.self._renderResults();
				if (!pageUrl)
					this.self._updatePageCount();
				PlatformUI.getWorkbench().hideMessageArea();
			},
			failure: function(error) {
				dojo.publish(Constants.serverError, [error]);
			}
		};
		var srh = new ServiceResponseHandler(serviceObject, "success", "failure");
		DashboardClient.getSharedDashboards(this._currentFilter || null, PAGE_SIZE, pageUrl, orderBy, orderByAscending, true, srh, this._getServiceTracker());
	},
	
	_renderResults: function() {
		this._destroyResourceLinks();
		this._resourceLinks = [];
		this._table.clearBody();
		var results = [], start = PAGE_SIZE * this.page, i, lastPersonal;
		if (this._matchingPersonalDashboards.length > start) {
			for (i = start; i < this._matchingPersonalDashboards.length && i < start + PAGE_SIZE; i++) {
				results.push(this._matchingPersonalDashboards[i]);
			}
			if (i === this._matchingPersonalDashboards.length) {
				lastPersonal = results.length - 1;
			}
		}
		if (results.length < PAGE_SIZE && this._sharedDashboards) {
			start = Math.max(PAGE_SIZE * this.page - this._matchingPersonalDashboards.length, 0);
			var max = PAGE_SIZE - results.length + start;
			for (i = start; i < this._sharedDashboards.length && i < max; i++) {
				var db = this._sharedDashboards[i];
				results.push(this._sharedDashboards[i]);
			}
		}
		if (results && results.length > 0) {
			for (i = 0; i < results.length; i++) {
				this._table.tbody.appendChild(this._createRow(results[i], lastPersonal === i));
			}
		} else {
			this._table.tbody.appendChild(this._createNoneMessage());
		}
	},
	
	_createRow: function(dashboard, lastPersonal) {
		var row = document.createElement("tr");
		var td = document.createElement("td");
		var iconFile = null;
		if (dashboard["default"]) {
			iconFile = "default-dash.png";
		}
		else {
			iconFile = dashboard.shared ? "shared-dash.png" : "my-dash.png";
		}
		var iconUrl = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/Dashboard/icons/" + iconFile);
		td.innerHTML = "<img class='dashboard-icon' src='" + iconUrl + "'/><a href='" + DashboardUtil.getDashboardHref(DashboardConstants.SCOPE_CONTRIBUTOR, dashboard.id) + "'>" + HTML.escape(dashboard.title) + "</a>";
		row.appendChild(td);
		var td2 = document.createElement("td");
		var user = dashboard.user || this._contributor;
		var link = new ResourceLink({
			uri: (net.jazz.ajax._dataServerRoot || Util.getContextPath(true)) + "/users/" + encodeURIComponent(user.userId),
			presentationUri: "#action=com.ibm.team.repository.viewUserProfile&itemId=" + user.itemId,
			label: HTML.escape(user.name),
			retainLinkText: true
		});
		this._resourceLinks.push(link);
		td2.appendChild(link.domNode);
		row.appendChild(td2);
		if (lastPersonal)
			td.className = td2.className = "last-personal";
		return row;
	},
	
	_createNoneMessage: function() {
		var row = document.createElement("tr");
		var td = document.createElement("td");
		td.colSpan = 2;
		var link = jazz.util.i18n.addMessage(td, messages.allPersonalNone)[0];
		this.connect(link, "onclick", this._handleCreateDashboard);
		row.appendChild(td);
		dojo.create("img", {src: dojo.moduleUrl("com.ibm.team.dashboard.web","graphics/Dashboard/icons/create-dash.png").toString(), style: "margin: 0px 2px 0px 3px; vertical-align: sub"}, link, "first");
		return row;
	},
	
	_sortPersonal: function(dashboards) {
		dashboards.sort(dojo.hitch(this,function(a,b) {
			// default is always first; exempt from sorting
			if (a["default"]) {
				return -1;
			}
			else if (b["default"]) {
				return 1;
			}
			else {
				var result = a.title.localeCompare(b.title);
			}
			if (this._table.sortBy === 0 && !this._table.sortByAscending)
				result = result * (-1);
			return result;
		}));
		return dashboards;
	},
	
	_doFilter: function(filter) {
		// set a timeout to query with the new filter
		// need to ensure 0 is passed to _runQuery or setTimeout will pass some other value
		if (this._handleFilterTimeout) {
			clearTimeout(this._handleFilterTimeout);
			delete this._handleFilterTimeout;
		}
		this._handleFilterTimeout = setTimeout(dojo.hitch(this,this._handleFilter, filter),300);
		
	},
	
	_handleFilter: function(filter) {
		this._currentFilter = filter;
		this._reset();
		if (!this._currentFilter || this._contributor.name.toLowerCase().indexOf(this._currentFilter) >= 0) {
			this._matchingPersonalDashboards = this._personalDashboards;
		} else {
			this._matchingPersonalDashboards = [];
			for (var i = 0; i < this._personalDashboards.length; i++) {
				var db = this._personalDashboards[i];
				if (db.title.toLowerCase().indexOf(this._currentFilter) >= 0) {
					this._matchingPersonalDashboards.push(db);
				}
			}
		}
		this._runSharedQuery();
	},
	
	_updatePageCount: function() {
		var total = this._totalSharedDashboards;
		if (this._matchingPersonalDashboards)
			total += this._matchingPersonalDashboards.length;
		this._pagination.setTotal(total);
	},
	
	_getServiceTracker: function() {
		if (!this._serviceTracker)
			this._serviceTracker = new ServiceTracker();
		return this._serviceTracker;
	},
	_destroyResourceLinks: function() {
		if (this._resourceLinks) {
			for (var i = 0; i < this._resourceLinks.length; i++)
				this._resourceLinks[i].destroy();
		}
	},
	destroy: function() {
		// abort any remaining requests
		if (this._serviceTracker) {
			var requests = this._serviceTracker.getActiveRequests();
			for (var i=0;i<requests.length;i++) {
				requests[i].abort();
			}
		}
		this._destroyResourceLinks();
		if (this._filterBox) this._filterBox.destroy();
		if (this._table) this._table.destroy();
		if (this._help) this._help.destroy();
		this.inherited(arguments);
	}
	
});

})();
