/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.AllTeamAreas");

dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("com.ibm.team.process.web.client.internal.ProcessClient");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.util.DashboardUtil");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("jazz.ui.FilterBox");
dojo.require("jazz.util.html");
dojo.require("jazz.ui.ResourceLink");
dojo.require("jazz.app.context");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

var HTML = jazz.util.html;
var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var DashboardUtil = com.ibm.team.dashboard.web.util.DashboardUtil;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var FilterBox = jazz.ui.FilterBox;
var ResourceLink = jazz.ui.ResourceLink;

var messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

dojo.declare("com.ibm.team.dashboard.web.ui.internal.AllTeamAreas",
	[dijit._Widget, dijit._Templated], {
		
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/AllTeamAreas.html"),
	
	constructor: function() {
		this.allProjectDash = dojo.string.substitute(messages.allProjectDash, [jazz.app.context.get().name]);
		this.allTeamsProj = messages.allTeamsProj;
	},
	postCreate: function() {
		this._filterBox = new FilterBox({onChange: dojo.hitch(this,this._doFilter), enabled: false});
		this._filterArea.appendChild(this._filterBox.domNode);
		var context = jazz.app.context.get();
		this.uriRoot = (net.jazz.ajax._dataServerRoot || Util.getContextPath(true)) + "/process/project-areas/" + context.itemId;
		var serviceObject = {
			self: this,
			success: function(result) {
				if (result) {
					result.uri = this.self.uriRoot;
					this.self._areas = [result];
				}
				this.self._populate();
			},
			failure: function(error) {
				dojo.publish(com.ibm.team.dashboard.web.ui.internal.Constants.serverError, [error]);
			}
		};
		var srh = new ServiceResponseHandler(serviceObject, "success", "failure");
		ProcessClient.getProjectHierarchy({uuid: context.itemId}, srh);
		
		// Accessibility
		dijit.setWaiRole(this._resultsArea, "tree");
		dijit.setWaiState(this._resultsArea, "expanded", "true");
		dijit.setWaiState(this._resultsArea, "disabled", "true");
	},
	
	_populate: function() {
		var project = this._areas[0];
		if (project.archived) return;
		project.node = this._createRow(project, "#action=com.ibm.team.dashboard.viewDashboard", 0);
		project.name = project.name.toLowerCase();
		if (project.children && project.children.length > 0) {
			this._sortAreas(project.children);
			this._parseTeams(project.children, 1, []);
		}
		this._filterBox.setEnabled(true);
	},
	
	_parseTeams: function(teams, indent, pathSegs) {
		for (var i = 0; i < teams.length; i++) {
			var team = teams[i];
			if (team.archived) continue;
			team.uri = this.uriRoot + "/team-areas/" + team.itemId;
			var thisPath = dojo.clone(pathSegs);
			thisPath.push(team.name);
			team.node = this._createRow(team, DashboardUtil.getDashboardHref(DashboardConstants.SCOPE_TEAM_AREA, thisPath), indent);
			team.name = team.name.toLowerCase();
			if (team.children)
				this._parseTeams(team.children, indent+1, thisPath);
		}
	},
	
	_createRow: function(area, href, indent) {
		var result = document.createElement("div");
		dijit.setWaiRole(result, "treeitem");
		dijit.setWaiState(result, "expanded", "true");
		dijit.setWaiState(result, "disabled", "true");
		var level = indent + 1;
		dijit.setWaiState(result, "level", level);
		
		result.className = "resultRow";
		if (indent) {
			result.style.paddingLeft = (1 + (indent * 2)) + "em";
		} else {
			result.style.paddingLeft = "1em";
		}
		if (href) {
			var rl = new ResourceLink({
				uri: area.uri,
				presentationUri: href,
				label: HTML.escape(area.name),
				retainLinkText: true
			});
			result.appendChild(rl.domNode);
		} else {
			result.appendChild(document.createTextNode(area));
		}
		this._resultsArea.appendChild(result);
		return result;
	},
	
	_doFilter: function(filter) {
		if (this._currentFilter && this._currentFilter == filter)
			return;
		this._currentFilter = filter;
		if (this._areas) {
			if (this._currentFilter.length == 0) 
				this._clearFilter(this._areas);
			else {
				var hits = this._filterAreas(this._areas);
				if (!hits) {
					if (!this._noHitsRow) {
						this._noHitsRow = this._createRow(messages.allTeamsNoneMatch);
					}
					this._noHitsRow.style.display = "block";
				} else if (this._noHitsRow)
					this._noHitsRow.style.display = "none";
			}
		}
	},
	
	_filterAreas: function(areas) {
		var result = false;
		for (var i = 0; i < areas.length; i++) {
			if (areas[i].archived) continue;
			var childHits = false;
			if (areas[i].children && areas[i].children.length > 0)
				childHits = this._filterAreas(areas[i].children);
			result = this._testFilterRow(areas[i], childHits) || result;
		}
		return result;
	},
	
	_testFilterRow: function(area, childHits) {
		if (childHits || area.name.indexOf(this._currentFilter) != -1) {
			area.node.style.display = "block";
			return true;
		}
		area.node.style.display = "none";
		return false;
	},
	
	_clearFilter: function(areas) {
		if (this._noHitsRow)
			this._noHitsRow.style.display = "none";
		for (var i = 0; i < areas.length; i++) {
			if (areas[i].archived) continue;
			areas[i].node.style.display = "block";
			if (areas[i].children && areas[i].children.length > 0)
				this._clearFilter(areas[i].children);
		}
	},
	
	_sortAreas: function(areas) {
		areas.sort(function(a,b) {
			return a.name < b.name ? -1 : 1;
		});
		for (var i = 0; i < areas.length; i++) {
			if (areas[i].archived) continue;
			if (areas[i].children && areas[i].children.length > 0) 
				this._sortAreas(areas[i].children);
		}
	}
	
});

})();
