/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.CatalogDrawer");

/*
 * Imports 
 */
dojo.require("dojo.i18n"); 
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.ensemble.internal.catalog.WidgetCatalog");

dojo.require("com.ibm.team.dashboard.web.ui.internal.catalog.CatalogServicesProvider");
dojo.require("com.ibm.team.dashboard.web.ui.internal.catalog.AddWidgetHelper");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "CatalogDrawerMessages");

(function() {

/*
 * Aliases 
 */
var WidgetCatalog = jazz.ensemble.internal.catalog.WidgetCatalog;
var CatalogServicesProvider = com.ibm.team.dashboard.web.ui.internal.catalog.CatalogServicesProvider;
var AddWidgetHelper = com.ibm.team.dashboard.web.ui.internal.catalog.AddWidgetHelper;

/*
 * Class 
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.CatalogDrawer",
	[dijit._Widget, dijit._Templated], {

	// Properties //----------------------------------------------------------->

	onAddWidget: null,
	onCloseCatalog: null,

	// ------------------------------------------------------------------------>

	/*
 	 * Template
 	 */
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/CatalogDrawer.html"),
	
	/*
 	 * @param Object args
 	 * 
 	 * @return CatalogDrawer 
 	 */
	constructor: function(args) {
		
		// Messages //------------------------------------------------------------->

		this.msg = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "CatalogDrawerMessages");
		
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	postCreate: function() {

		this.inherited(arguments);

		// Instance Variables //----------------------------------------------->

		this._catalog = null;
		this._catalogServiceProvider = new CatalogServicesProvider();
		this._addWidgetHelper = null;

		// Initialization //--------------------------------------------------->

		this._initCloseButtonSite();
		this._initWidgetCatalog();
		
	},	

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */
	_getAddWidgetHelper: function() {
		if (this._addWidgetHelper) {
			return this._addWidgetHelper;
		}
		
		this._addWidgetHelper = new AddWidgetHelper({
			onAddWidget: this.onAddWidget
		});
		return this._addWidgetHelper;
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */
	_initWidgetCatalog: function() {
		this._catalog = new WidgetCatalog({
			provider: this._catalogServiceProvider,
			onAddWidget: dojo.hitch(this, this._addWidgetToDashboard)
		});
		this._catalogSite.appendChild(this._catalog.domNode);
	},

	/*
 	 * @param Object data
 	 * @param String rootServices
 	 * @param Boolean local
 	 * 
 	 * @return dojo.Deferred
 	 */
	_addWidgetToDashboard: function(data, rootServices, local) {
		
		var deferred = this._getAddWidgetHelper().addWidget(data, rootServices, local);
	
		return deferred;		
	},	

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */
	_initCloseButtonSite: function() {
		this._closeButton.src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/CatalogDrawer/images/close.gif");
		
		this.connect(this._closeButton, "onclick", this._handleEventOnClickCloseButton);		
		this.connect(this._closeButton, "onmouseover", this._handleEventOnMouseOverCloseButton);
		this.connect(this._closeButton, "onmouseout", this._handleEventOnMouseOutCloseButton);		
	},	

	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */	
	_handleEventOnClickCloseButton: function(event) {
		if (event) {
			dojo.stopEvent(event);
		}
		
		// Close catalog
		if (this.onCloseCatalog) {
			this.onCloseCatalog();
		}
		
		return true;
	},

	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */	
	_handleEventOnMouseOverCloseButton: function(event) {
		if (event) {
			dojo.stopEvent(event);
		}
		// Highlight on
		this._closeButton.src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/CatalogDrawer/images/close-hov.gif");
		
		return true;
	},

	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */	
	_handleEventOnMouseOutCloseButton: function(event) {
		if (event) {
			dojo.stopEvent(event);
		}
		// Highlight off
		this._closeButton.src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/CatalogDrawer/images/close.gif");
		
		return true;
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	destroy: function() {
		
		if (this._catalog) {
			this._catalog.destroy();
			this._catalog = null;
		}
		
		if (this._catalogServiceProvider) {
			this._catalogServiceProvider.destroy();
			this._catalogServiceProvider = null;
		}
		
		if (this._addWidgetHelper) {
			this._addWidgetHelper.destroy();
			this._addWidgetHelper = null;
		}
		
		this.inherited(arguments);
	}
	
});

})();
