/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.ContributorGroupSelectionDialog");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("dojox.xml.parser");

dojo.require("jazz.ui.Dialog");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal", "ContributorGroupSelectionDialogMessages");

(function() {

var DELAY = 500;
var KEY_ESCAPE = 27;
var KEY_RETURN = 13;
var KEY_ENTER = 14;
var KEY_SPACE = 32;
var MAX_INT = 2147483647;
var SEARCH_LIMIT = 250;
var REST_CONTEXT_ROOT = "jazz/";
var REST_USER_GROUP_URL = "user-groups/";

var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	
dojo.declare("com.ibm.team.repository.web.ui.internal.ContributorGroupSelectionDialog", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/templates/ContributorGroupSelectionDialog.html"),
	
	constructor: function (args) {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal", "ContributorGroupSelectionDialogMessages");
		this.showingFirstMsg = dojo.string.substitute(this.messages.showingFirst, [ SEARCH_LIMIT ]);
		if (args == null) {
			args = {};
		}
		this.width = args.width || "450px";
		var properties = {
			"dialogTitle" : this.messages.selectUserGroup,
			"width" : "450px",
			"closable": true,
			"filterOutUserGroups" : [ ],
			"searchLimit" : SEARCH_LIMIT
		};
		for (var prop in properties) {
			if (args[prop] != null) {
				this[prop] = args[prop];
			}
			else {
				this[prop] = properties[prop];
			}
		}
		this.selectedUserGroup = null;
	},
	
	postCreate: function () {
		this.searchCount = 0;
		this.userGroupSelector.size = 15;
		this.keyStrokeFilter = this.connect(document, "onkeypress", function (e) {
			if (e.keyCode === 8 && e.target !== this.searchText) {
				dojo.stopEvent(e);
			}
		});
		
		//add ok/close buttons
		this.okButton.appendChild(document.createTextNode(this.messages.ok));
		this.closeButton.appendChild(document.createTextNode(this.messages.close));
		this.okButton.style.display = "";
		this.closeButton.style.display = "";
			
		this.userGroupSelector.multiple = false;
		this.dialog = new jazz.ui.Dialog({
			contentNode: this.contentNode,
			primaryTitle: this.dialogTitle,
			width: this.width,
			closable: this.closable
		});
		/*this.connect(this.searchText,"onkeypress","handleSearchChange");
		this.connect(this.searchText,"onchange","handleSearchChange");
		this.connect(this.showAllLink,"onclick", function (e) {
			dojo.stopEvent(e);
			this.searchText.value = "*";
			this._doSearch();
		});*/
		this.connect(this.closeButton,"onclick", function (e) {
			dojo.stopEvent(e);
			this.dialog.close();
		});
		this.connect(this.okButton,"onclick","_onAddClose");
		this.connect(this.userGroupSelector,"ondblclick","_handleDoubleClick");
		this.connect(this.userGroupSelector,"onkeydown","_handleKeyDown");
		// compensate for the IE bug that prevents us from disabling options
		if (dojo.isIE) {
			this.connect(this.userGroupSelector,"onchange", function(e) {
				var selectedIndices = [];
				var disabledSelected = false;
				
				if (this.userGroupSelector.selectedIndex >= 0) {
					if (this.userGroupSelector.options[this.userGroupSelector.selectedIndex].disabled) {
						if (this.userGroupSelector.savedSelectedIndex >= 0) {
							this.userGroupSelector.selectedIndex = this.userGroupSelector.savedSelectedIndex;
						}
						else {
							this.userGroupSelector.selectedIndex = -1;
						}
					}
					else {
						this.userGroupSelector.savedSelectedIndex = this.userGroupSelector.selectedIndex;
					}
				}
				
			});
		}
		this.connect(this.userGroupSelector,"onchange", function () {
			var selected = false;
			this.selectedUserGroup = null;
			for (var i=0; i < this.userGroupSelector.options.length; i++) {
				if (this.userGroupSelector.options[i].selected) {
					selected = true;
					this.selectedUserGroup = {
						itemId: this.userGroupSelector.options[i].itemId,
						name: this.userGroupSelector.options[i].name,
						summary: this.userGroupSelector.options[i].summary
					};
				}
			}
			this.okButton.disabled = !selected;
		});
		this.connect(this.dialog,"onClose","onClose");
		//this.searchText.focus();
		//this.searchText.disabled = true;
		var self = this;
		var serviceObj = {
			success: function (obj) {
				self._loadQueryResults(obj);
			},
			failure: function (errorObj) {
				self._queryError(errorObj);
			}
		}
		var srh= new ServiceResponseHandler(serviceObj, "success", "failure");
		this.runContributorGroupSearch({}, srh);
	},
	
	_handleDoubleClick: function (e) {
		if (this.selectedUserGroup) {
			this._onAddClose(e);
		}
	},
	
	_handleKeyDown: function (e) {
		if ((e.keyCode == KEY_RETURN || e.keyCode == KEY_ENTER) && this.selectedUserGroup) {
			this._onAddClose(e);
		}
	},
	
	handleSearchChange: function (e) {
		if(e.keyCode == KEY_ESCAPE) {
			return;
		}
		if (e.keyCode == KEY_RETURN || e.keyCode == KEY_ENTER) {
			if (this.selectedUserGroup == null && this.userGroupSelector.options.length > 0) {
				var index = 0;
				while (index < this.userGroupSelector.options.length && this.userGroupSelector.options[index].disabled) {
					index++;
				}
				if (index != this.userGroupSelector.options.length) {
					this.userGroupSelector.options[index].selected = true;
					this.selectedUserGroup = {
						itemId: this.userGroupSelector.options[index].itemId,
						name: this.userGroupSelector.options[index].name,
						summary: this.userGroupSelector.options[index].summary
					};
				}
			}
			if (this.selectedUserGroup) {
				this._onAddClose(e);
			}
		}
		else {
			this._doCancellableSearch();
		}
	},
	
	_onAddClose: function (e) {
		dojo.stopEvent(e);
		this.onAdd(this.selectedUserGroup);
		this.dialog.close();
	},
	
	_onAdd: function (e) {
		dojo.stopEvent(e);
		this.onAdd(this.selectedUserGroup);
	},
	
	// an equivalent to onAdd that can be connected to.
	onOK: function (selectedUsers) {},
	
	onAdd: function (selectedUserGroup) {
		this.selectedUserGroup = selectedUserGroup;
		this.okButton.disabled = true;
		for (var i=0; i < this.userGroupSelector.options.length; i++) {
			if (this.userGroupSelector.options[i].selected) {
				this.userGroupSelector.options[i].selected = false;
			}
		}
		if (this.onOK)
			this.onOK(this.selectedUserGroup);
	},
	
	onClose: function () {
		this.disconnect(this.keyStrokeFilter);
	},
	
	_doCancellableSearch: function() {
		if (this._currentId) {
			clearTimeout(this._currentId);
		}
		this._currentId= setTimeout(dojo.hitch(this, function () {this._doSearch();}), DELAY);
	},
	
	_doSearch: function () {
		// if there's no input, do nothing
		var value = this.searchText.value;
		if (dojo.trim(value) === "") {
			this._clearStatusMessage();
			return;
		}
		var params;

		value = value.replace(/\*/g,"%");
		value = value.replace(/\?/g,"_");
		params = {
			sortBy: "name",
			searchTerm: value,
			searchField: "name",
			pageSize: this.searchLimit,
			hideUnassigned: true,
			filterOutUsers: this.filterOutUsers
		};
		
		if (this.latestSearchTerm != null && this.latestSearchTerm === value) {
			// don't re-run the same search twice
			return;
		}
		this.latestSearchTerm = value;

		this._setStatusMessage(this.messages.runningSearch);
		this.searchCount++;
		var closureSearchCount = this.searchCount;
		var self = this;
		var serviceObj = {
			success: function (obj) {
				if (closureSearchCount === self.searchCount) {
					self._loadQueryResults(obj);
				}
			},
			failure: function (errorObj) {
				self._queryError(obj);
			}
		}
		var srh= new ServiceResponseHandler(serviceObj, "success", "failure");
		this.runContributorGroupSearch(params, srh);
	},
	
	runContributorGroupSearch: function (params, srh) {
		var url = this._getServerUrl() + "/" + REST_CONTEXT_ROOT + REST_USER_GROUP_URL;
		var xhrArgs = {
	        url: url
	    }
			
		var deferred = jazz.client.xhrGet(xhrArgs);
		deferred.addCallback(function(response){
			net.jazz.ajax.core.internal.util.callContributedCode(function() {
				srh.doCallback(response);
			});
		});
		deferred.addErrback(function(response){
			net.jazz.ajax.core.internal.util.callContributedCode(function() {
				srh.doErrorCallback(response);
			});
		});
	},
	
	_loadQueryResults: function (returnJson) {
		var resultSet = dojo.fromJson(returnJson);
		// compensate for the IE bug that prevents us from disabling options
		if (dojo.isIE) {
			this.userGroupSelector.savedSelected = [];
		}
		if (dojo.isIE < 7) {
			this.userGroupSelector.options.length = 0;
		}
		else {
			dojox.xml.parser.removeChildren(this.userGroupSelector);
		}
		if (resultSet != null) {
			this.okButton.disabled = true;
			for (var i=0; i < resultSet.length; i++) {
				var disp = this.getDispValue(resultSet[i]);
				var option = new Option(disp, resultSet[i].itemId);
				option.itemId = resultSet[i].itemId;
				option.name = resultSet[i].name;
				option.summary = resultSet[i].summary;
				if (dojo.isIE) {
					this.userGroupSelector.options[this.userGroupSelector.options.length] = option;
				}
				else {
					this.userGroupSelector.appendChild(option);
				}
			}
			if (resultSet.length === SEARCH_LIMIT) {
				this.searchLimitMsg.style.display = "block";
			}
			else {
				this.searchLimitMsg.style.display = "none";
			}
		}
		else {
			this.okButton.disabled = true;
		}
		this._clearStatusMessage();
	},
	
	getDispValue: function (elem) {
		return elem.name;
	},
	
	_queryError: function (err) {
		this._setStatusMessage(err.message, true);
	},
	
	_clearStatusMessage: function() {
		//this.searchStatus.style.visibility = "hidden";
	},
	
	_setStatusMessage: function(msg, isError) {
		//this.searchStatus.style.visibility = "visible";
		//dojox.xml.parser.replaceChildren(this.searchStatus, document.createTextNode(msg));
		//this.searchStatus.className= isError ? "ErrorStatus" : "NormalStatus";
	},
	_getServerUrl: function(){
		var location = document.location.href;
		var separator = "/"; //$NON-NLS-1$
		var path = location.split(separator); //$NON-NLS-1$
		var serverAddress;
		if (path.length > 0) {
			serverAddress = path.shift() + separator + separator;
			path.shift(); //empty from the double //
			serverAddress = serverAddress + path.shift();
		}
		return serverAddress;
	}
});

})();