/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.ContributorMemento");

dojo.require("com.ibm.team.dashboard.web.ui.IMemento");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");

(function() {

var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var IMemento = com.ibm.team.dashboard.web.ui.IMemento;

/*
 * A memento that stores viewlet state data for a particular user, e.g. customized
 * appearance. It is stored in a ViewletContributorInfo helper of a
 * DashboardContributorInfo item.
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.ContributorMemento",
	IMemento, {
	
	constructor: function(/* ViewletContributorInfo */ viewletInfo, /* ViewletContainer */ container) {
		this._viewletInfo = viewletInfo;
		this._container = container;
		
		// init working variable
		if (viewletInfo.memento) {
			// already has one; read it
			this._memento = dojo.fromJson(viewletInfo.memento);
		}
		else {
			// create a default empty one
			this._memento = new Object();
		}
	},
	
	/*
	 * Returns the object or primitive stored under the given key for this
	 * memento, or undefined if there is none.
	 */
	/* object or primitive */ get: function(/* string */ key) {
		return this._memento[key];
	},
	
	/*
	 * Stores the given value under the given key in this memento.
	 */
	set: function(/* string */ key, /* object or primative */ value) {
		this._memento[key] = value;
		this._viewletInfo.memento = dojo.toJson(this._memento);
		this._container.propertyChanged(Constants.PROPERTY_CONTRIBUTOR_INFO);
	}
});

})();