/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.CustomizeViewlet");

dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");

dojo.require("jazz.app.internal.applicationAbout");

dojo.require("dojo.i18n");
dojo.require("jazz.util.i18n");
dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "CustomizeMessages");

(function(){
    var Viewlet = com.ibm.team.dashboard.web.ui.Viewlet;
	var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
    
    var ID_JTS = "http://jazz.net/application/jts";
    var ID_CCM = "http://jazz.net/application/ccm";
    var ID_QM = "http://jazz.net/application/qm";
    var ID_RM = "http://jazz.net/application/rm";
    
    var messages;
    
    /*
     * The Customize Viewlet class.
     */
    dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.CustomizeViewlet", Viewlet, {
    
        templatePath: dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web", "ui/internal/templates/CustomizeViewlet.html"),
        
        constructor: function(args){
            messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "CustomizeMessages");
            this.loading = messages.loading;
            this.intro = messages.intro;
            this.drag = messages.drag;
        },
        
        init: function(){
            this._loading.style.display = "none";
            this._recommendedWidgets.style.display = "none";
			this._setWidgetListState(false);
			this.connect(this._widgetListTitle, "onclick", dojo.hitch(this, this._toggleWidgetList));
			var span = jazz.util.i18n.addMessage(this._fWidgets, messages.widgets)[1];
			span.innerHTML = "<img class=\"icon\" src=\""+this._addWidgetImgSrc+"\">";
			span = jazz.util.i18n.addMessage(this._fTabs, messages.tabs)[1];
			span.innerHTML = "<img class=\"icon\" src=\""+this._addTabImgSrc+"\">";
        },
        
        _toggleLoading: function(loading){
            if (loading) {
                this._loading.style.display = "block";
                this._content.style.display = "none";
            }
            else {
                this._loading.style.display = "none";
                this._content.style.display = "block";
            }
        },
        
        
        refresh: function(){
            this._toggleLoading(true);
            var abouts = jazz.app.internal.applicationAbout.get();
			//var abouts = this._createFakeAbouts();
            this._toggleLoading(false);
            if (this._createWidgetLists(abouts)) 
                this._recommendedWidgets.style.display = "block";
        },
		
		_createFakeAbouts: function () {
			var abouts = new Array();
			abouts["rm"] = { identifier: ID_RM, title: "Requirements Management"};
			abouts["qm"] = { identifier: ID_QM, title: "Quality Management"};
			abouts["ccm"] = { identifier: ID_CCM, title: "Configuration Management"};		
			return abouts;
		},
		
		_toggleWidgetList: function(e) {
			e.preventDefault();
			this._setWidgetListState(dojo.hasClass(this._widgetList, "hidden"));
		},
		
		_setWidgetListState: function(expanded) {
			if (expanded) {
				dojo.removeClass(this._widgetList, "hidden");
				dojo.addClass(this._widgetListTitle, "expanded");
				this._widgetListHead.style.marginBottom = "0px";		
				this._widgetListTitle.innerHTML = messages.hideRecommended;
			}
			else {
				dojo.addClass(this._widgetList, "hidden");
				dojo.removeClass(this._widgetListTitle, "expanded");			
				this._widgetListHead.style.marginBottom = "10px";
				this._widgetListTitle.innerHTML = messages.showRecommended;
			}
		},
        
        _createWidgetLists: function(abouts){
            dojo.empty(this._widgetList);
			// Don't show widgets if in a project or team dashboard
			var scope = this.getScope();
			if (scope === DashboardConstants.SCOPE_PROJECT_AREA ||
				scope === DashboardConstants.SCOPE_TEAM_AREA) {
				return false;
			}
            var count = 0;
            
            for (var x in abouts) {
                var about = abouts[x];
                
                switch (about.identifier) {
					/*
                    case ID_JTS:
                        this._widgetList.appendChild(this._createWidgetList(about.title, 
						[	messages.wJtsStatus	]));
                        count++;
                        break;
                        */
                    case ID_RM:
                        this._widgetList.appendChild(this._createWidgetList(about.title,
						 [ 	messages.wRmReviews, 
						 	messages.wRmComments, 
							messages.wRmRequirements, 
							messages.wRmRecentChanges]));
                        count++;
                        break;
                        
                    case ID_CCM:
                        this._widgetList.appendChild(this._createWidgetList(about.title,
						 [ 	messages.wCcmMyWorkItemChanges, 
						 	messages.wCcmWorkItems, 
							messages.wCcmBlocking, 
							messages.wCcmOpenPriority]));
                        count++;
                        break;
                        
                    case ID_QM:
                        this._widgetList.appendChild(this._createWidgetList(about.title,
						 [	messages.wQmMyTasks, 
						 	messages.wQmDefects, 
						 	messages.wQmExecutionStatus, 
							messages.wQmCoverage]));
                        count++;
                        break;
                }
            }
            return count > 0;
        },
        
        _createWidgetList: function(title, names){
            var node = document.createElement("div");
            var tnode = document.createElement("div");
            tnode.innerHTML = title;
			tnode.className="app-name";
            node.appendChild(tnode);
            var list = document.createElement("ul");
            for (var i = 0; i < names.length; i++) {
                var name = names[i];
                var li = document.createElement("li");
                li.innerHTML = name;
                list.appendChild(li);
            }
            node.appendChild(list);
            return node;
        },
        
        //mappings of image variables used in the viewlet template
        _addWidgetImgSrc: dojo.moduleUrl("com.ibm.team.dashboard.web",
		 										"graphics/DashboardMini/add-widget.png"),
        _addTabImgSrc: dojo.moduleUrl("com.ibm.team.dashboard.web",
		 										"graphics/MetaNav/icons/add-tab.png"),												
        _chevronUpImgSrc: dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web",
								"ui/internal/templates/graphics/chev-up.gif"),
        _chevronDownImgSrc: dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web",
								"ui/internal/templates/graphics/chev-dwn.gif")																		
    });
    
})();
