/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.dashboard.web.ui.internal.DashboardComponentMenu");

dojo.require("jazz.app._ComponentMenu");
dojo.require("jazz.app.context");
dojo.require("jazz.ui.internal.MenuItemGroup");
dojo.require("jazz.util.html");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.dashboard.web.util.DashboardUtil");
dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var DashboardUtil = com.ibm.team.dashboard.web.util.DashboardUtil;
var HTML = jazz.util.html;
var MenuItemGroup = jazz.ui.internal.MenuItemGroup;
var WrapHints = jazz.ui.MegaMenu.WrapHints;

var messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

/*
 * The Web UI page (vertical) containing the dashboard and side navigation bar.
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.DashboardComponentMenu", jazz.app._ComponentMenu, {
	
	postCreate: function() {
		dojo.addClass(this.domNode, "com-ibm-team-dashboard-ComponentMenu");
		var projectName = jazz.app.context.get().name;
		var escName = HTML.escape(projectName);
		this._projectDashboards = new MenuItemGroup({
			menu: this,
			groupTitle: messages.projectDashboards,
			items: [{
				label: escName,
				icon: dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/Dashboard/icons/project-dashboard.gif"),
				href: DashboardUtil.getDashboardHref(DashboardConstants.SCOPE_PROJECT_AREA, projectName)
			}, {
				label: dojo.string.substitute(messages.allProjectDash, [escName]),
				icon: dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/Dashboard/icons/all-projects.png"),
				href: "#action=com.ibm.team.dashboard.viewAllDashboards",
				wrapHint: WrapHints.NO_WRAP
			}]
		});
		dojo.publish("/jazz/dashboard/initMenu", [this]);
		this.subscribe("/jazz/dashboard/view", this._addRecent); // pass bundle
		this.subscribe("/jazz/dashboard/dirty", this.onDirtyStateChange); // pass bundle and dirty state
	},
	initRecent: function(recent) {
		var items = [];
		outer: for(var i = recent.length - 1; i >= 0; i--) {
			var item = this._createMenuItem(recent[i]);
			for (var j = 0; j < items.length; j++) {
				if (items[j].href == item.href) continue outer;
			}
			items.push(item);
		}
		if (this._recentDashboards) {
			this._recentDashboards.attr("items",items);
			this._updateRecentCount();
		} else if (items.length > 0) {
			this._createRecentSection(items);
		} else {
			return;
		}
		this.updateSelected();
	},
	_createRecentSection: function(items) {
		if (!this._recentDashboards) this._recentDashboards = new MenuItemGroup({
			menu: this,
			wrapHint: WrapHints.FORCE_WRAP,
			groupTitle: "<b>"+messages.recentDashboards+"</b> (" + (items ? items.length : 0) + ")",
			items: items || []
		});
	},
	_addRecent: function(/* object */ bundle) {
		var item = this._createMenuItem(bundle);
		return this._addRecentItem(item);
	},
	_addRecentItem: function(/* object */ item) {
		if (!this._recentDashboards) {
			this._createRecentSection();
		}
		var items = this._recentDashboards.attr("items");
		for (var i = 0; i < items.length; i++) {
			if (items[i].href == item.href) {
				this._recentDashboards.remove(items[i]).destroy();
				break;
			}
		}
		while (items.length > 4) {
			this._recentDashboards.remove(items[0]).destroy();
		}
		this._recentDashboards.addItemByArgs(item,0);
		this._updateRecentCount();
		this.updateSelected();
		return true;
	},
	_createMenuItem: function(/* object */ bundle, /* boolean */ dirty) {
		var dashboard = bundle.dashboard.binding;
		var name = dashboard.getTitle(),
			href = Util.getDashboardHref(bundle),
			iconUrl;
		switch(bundle.scope) {
			case DashboardConstants.SCOPE_CONTRIBUTOR:
				if (bundle.dashboard.securityContext.indexOf(Constants.CONTEXT_PUBLIC) >= 0)
					iconUrl = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/Dashboard/icons/shared-dash.png");
				else
					iconUrl = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/Dashboard/icons/my-dash.png");
				break;
			case DashboardConstants.SCOPE_TEAM_AREA:
				iconUrl = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/Dashboard/icons/all-team-dash.png");
				break;
			case DashboardConstants.SCOPE_PROJECT_AREA:
				iconUrl = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/Dashboard/icons/project-dashboard.gif");
				break;
		}
		var label = HTML.escape(name);
		if (dirty) {
			label = "<span class='j-dirty'>* " + label + "</span>";
		}
		return {
			label: label,
			icon: iconUrl,
			href: href,
			url: bundle.dashboardUrl
		};
	},
	_updateRecentCount: function() {
		if (!this._recentDashboards) return 0;
		var length = this._recentDashboards.attr("items").length;
		this._recentDashboards.attr("groupTitle", "<b>" + messages.recentDashboards + "</b> (" + length + ")");
		return length;
	},
	onDirtyStateChange: function(/* object */ bundle, /* boolean */ isDirty) {
		if (isDirty) {
			if (this._recentDashboards) {
				var items = this._recentDashboards.attr("items");
				var href = Util.getDashboardHref(bundle);
				for (var i = 0; i < items.length; i++) {
					if (items[i].href == href) {
						this._recentDashboards.remove(items[i]);
						break;
					}
				}
				if (!this._updateRecentCount()) {
					this._recentDashboards.remove();
					this._recentDashboards.destroy();
					delete this._recentDashboards;
				}
			}
			if (!this._unsavedDashboards) {
				this._unsavedDashboards = new MenuItemGroup({
					menu: this,
					wrapHint: this._recentDashboards ? WrapHints.NO_WRAP : WrapHints.FORCE_WRAP,
					groupTitle: "<b>" + messages.unsavedDashboards + "</b> (1)",
					items: [this._createMenuItem(bundle, true)]
				});
			}
		} else if (this._unsavedDashboards) {
			items = this._unsavedDashboards.attr("items");
			if (items && items.length) {
				this._unsavedDashboards.remove(items[0]);
				this._addRecentItem(this._createMenuItem(bundle));
			}
			this._unsavedDashboards.remove();
			this._unsavedDashboards.destroy();
			delete this._unsavedDashboards;
		}
	},
	getSelected: function() {
		var items = this.getItems();
		var sel = [];
		for (var i = 0; i < items.length; i++) {
			var item = items[i];
			if (item && item.href) {
				var href = location.href;
				if (href.indexOf(item.href) == href.length - item.href.length) {
					sel.push(item);
				}
			}
		}
		return sel;
	},
	uninitialize: function() {
		if (this._projectDashboards) this._projectDashboards.destroy();
		if (this._unsavedDashboards) this._unsavedDashboards.destroy();
		if (this._recentDashboards) this._recentDashboards.destroy();
	}
});
})();