/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.DashboardContainer");

dojo.require("dijit.form.TextBox");
dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("dojox.xml.parser");

dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.DashboardInitializationData");
dojo.require("com.ibm.team.dashboard.web.ui.internal.MetaNav");
dojo.require("com.ibm.team.dashboard.web.ui.internal.ViewletPage");
dojo.require("com.ibm.team.dashboard.web.ui.internal.dnd.DraggableTabs");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ItemNameRegistry");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.Observable");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ServiceTracker");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.CatalogDrawer");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("jazz.ensemble.internal.LayoutFactory");
dojo.require("jazz.ensemble.internal.client.RestClient");
dojo.require("jazz.ensemble.internal.util.XmlBindingFactory");
dojo.require("jazz.ui.MessageBox");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

//
// Aliases
//
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var DashboardInitializationData = com.ibm.team.dashboard.web.ui.internal.DashboardInitializationData;
var ItemNameRegistry = com.ibm.team.dashboard.web.ui.internal.util.ItemNameRegistry;
var LayoutFactory = jazz.ensemble.internal.LayoutFactory;
var MetaNav = com.ibm.team.dashboard.web.ui.internal.MetaNav;
var Observable = com.ibm.team.dashboard.web.ui.internal.util.Observable;
var DraggableTabs = com.ibm.team.dashboard.web.ui.internal.dnd.DraggableTabs;
var CatalogDrawer = com.ibm.team.dashboard.web.ui.internal.CatalogDrawer;
var RestClient = jazz.ensemble.internal.client.RestClient;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServiceTracker = com.ibm.team.dashboard.web.ui.internal.util.ServiceTracker;
var ViewletPage = com.ibm.team.dashboard.web.ui.internal.ViewletPage;
var XmlBindingFactory = jazz.ensemble.internal.util.XmlBindingFactory;

var MINUTE = 60000;
var STALE_CHECK_INTERVAL = MINUTE * 2;

var messages;

/**
 * The widget for the content area (right side) of the dashboard.
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.DashboardContainer",
	[Observable, dijit._Widget, dijit._Templated], {
	
	// Variables passed into constructor
	selectedTabId: null,
	allowSharing: false, // optional
	allowSetDefault: false, // optional
	
	// Variables available after setContent is called
	autoSave: null,
	dashboardWidget: null,
	dashboardBundle: null,
	dashboardResource: null,
	editMode: null,
	serviceTracker: new ServiceTracker(),
	tabs: null,
	viewletPages: null,
	
	// Internal variables
	_dashboardActive: false,
	
	// Template init
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/DashboardContainer.html"),
	
	/*
	 * Early lifecycle functions
	 */
	
	constructor: function(args) {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
	},
	postCreate: function(){
		this.metaNav = new com.ibm.team.dashboard.web.ui.internal.MetaNav({}, this.metaNav); 
	},
	/*
	 * Returns the dashboard's current title.
	 */
	getTitle: function() {
		if (this.dashboardResource) {
			return this.dashboardResource.binding.getTitle();
		}
	},
	/*
	 * Sets the dashboard content to the given model object containing
	 * the complete state of the dashboard. Called by Dashboard widget 
	 * once DashboardContainer.domNode is appended to layout.
	 */
	setContent: function(/* DashboardBundleDTO */ dashboardBundle) {
		this.dashboardBundle = dashboardBundle;
		this.dashboardResource = dashboardBundle.dashboard;
		this.viewletPages = [];
		
		this._dashboardActive = true;
		
		// set editMode and autoSave attributes
		if(this.dashboardBundle.permissions.modify) {
			if (this.isPrivateContributorDashboard()) {
				this.editMode = Constants.EDIT_AUTO_SAVE;
				this.autoSave = true; 
			} else {
				this.editMode = Constants.EDIT_DEFERRED_SAVE;
				this.autoSave = false; 
			}
		} else {
			this.editMode = Constants.READ_ONLY;
			this.autoSave = null;
		}
		
		//yield before instantiating the rest of the UI
		setTimeout(dojo.hitch(this,this._setContent2),0);
	},
	_setContent2: function() {
		if (!this._isDestroyed) {
			//initialize the metanav
			this.metaNav.addListener(dojo.hitch(this, this._onDashboardChange), Constants.PROPERTY_DASHBOARD);
			this.metaNav.init({
				dashboardContainer: this,
				dashboardBundle: this.dashboardBundle,
				editMode: this.editMode,
				selectedTabId: this.selectedTabId,
				allowSharing: this.allowSharing,
				allowSetDefault: this.allowSetDefault
			});
			
			//create new DraggableTabs container
			var div = document.createElement("div");
			this._tabs.appendChild(div);
			this.tabs = new DraggableTabs(div, {
				parent: this,
				accept: ["tab"],
				dashboardContainer: this,
				dashboardBundle: this.dashboardBundle,
				dashboardResource: this.dashboardResource,
				editing: this.isEditable()
			});
			this.tabs.addListener(dojo.hitch(this, this._onDashboardChange), Constants.PROPERTY_DASHBOARD);
			
			var pages = this.dashboardResource.binding.getPages().getPages();
			for (var i=0;i<pages.length;i++) {
				var page = pages[i];
				var id = page.getId();
				this.tabs.add(null, {
					title: page.getTitle(),
					id: id,
					index: i,
					page: page,
					type: "tab"
				});
			}
			
			//show the tab bar and the loading text
			this._tabWrapper.style.display = "block";
			this.tabs.startup(this.selectedTabId);
			
			this.setEditMode(this.editMode, true);
			var selectedTab = this.tabs.selectedTab;
			this.showViewletPage(selectedTab.index);
			this.isInitialized = true;
		}
	},
	
	/*
	 * Public Functions
	 */
	isUnsaved: function() {
		return this._saveInTransit || this.isDirty() || this._isEditingContents();
	},
	isDirty: function() {
		return this._isDirty ? true : false;
	},
	isEditable: function() {
		return (this.editMode == Constants.EDIT_AUTO_SAVE || this.editMode == Constants.EDIT_DEFERRED_SAVE);
	},
	isPrivateContributorDashboard: function() {
		var contributorUri = DashboardInitializationData.getAuthenticatedContributorUri();
		return (this.dashboardBundle.scope == DashboardConstants.SCOPE_CONTRIBUTOR
			&& this.dashboardResource.securityContext == contributorUri);
	},
	isVisible: function() {
		return this.dashboardWidget.isVisible();
	},
	handleOnBeforeUnload: function() {
		if (this._saveInTransit) {
			if (this.autoSave) {
				// If the dashboard is set to auto-save and the user tries to reload while the
				// save is ongoing, display the appropriate feedback so they know when the 
				// save is complete.
				this._showSaveInProgressMessage(true);
			}
			return messages.navigateAwaySaveInProgressGeneric;
		} else if (!this.autoSave && this.isDirty()) {
			return messages.unsavedChangesGeneric;
		} else if (this._isEditingContents()) {
			return messages.editingContentsGeneric;
		}
		return false;
	},
	promptNavigateAway: function() {
		if (this.deleted) return true;
		var inDashboards = this._dashboardActive;
		
		var confirmText = inDashboards ? messages.navigateAwayPrefixDashboards : messages.navigateAwayPrefixGeneric;
		if (this._saveInTransit) {
			if (this.autoSave) {
				// If the dashboard is set to auto-save and the user tries to reload while the
				// save is ongoing, display the appropriate feedback so they know when the 
				// save is complete.
				this._showSaveInProgressMessage(true);
			}
			confirmText += (inDashboards ? messages.navigateAwaySaveInProgressDashboards : messages.navigateAwaySaveInProgressGeneric) + messages.mayDiscard;
			var suffix = messages.navigateAwaySuffixSaveInProgress;
		} else {
			if (!this.autoSave && this.isDirty()) {
				confirmText += inDashboards ? messages.unsavedChangesDashboards : messages.unsavedChangesGeneric;
			} else if (this._isEditingContents()) {
				confirmText += inDashboards ? messages.editingContentsDashboards : messages.editingContentsGeneric;;
			}
			confirmText += messages.willDiscard;
			suffix = messages.navigateAwaySuffixUnsaved;
		}
		
		confirmText += inDashboards ? messages.navigateAwaySuffixDashboards : suffix; 

		return confirm(confirmText);
	},
	_toggleCatalog: function(event) {
		if (event){
			event.preventDefault();
		}
		if (this._catalogExpanded) {
			this.closeCatalog();
		} else {
			this.openCatalog();
		}
	},
	openCatalog: function(event) {
		if (event){
			event.preventDefault();
		}
		if (this._catalogExpanded && this._catalogDrawerSite.style.display != "none") {
			return;
		}
		this._createCatalog();
		this._catalogExpanded = true;
		
		var args = {node: this._catalogDrawerSite, duration: 200};
		var catalogWipeInAnimation = dojo.fx.wipeIn(args);
		this.connect(catalogWipeInAnimation, "onEnd", this._afterCatalogWipeInAnimation);	
		catalogWipeInAnimation.play();
		
		this._selectCatalogLink();
		dojo.publish("jazz.util.ResizeNotifier.resizeEvent");
	},

	closeCatalog: function() {
		if (!this._catalogExpanded) {
			return;
		}
		this._catalogExpanded = false;
		
		var args = {node: this._catalogDrawerSite, duration: 200};
		var catalogWipeOutAnimation = dojo.fx.wipeOut(args);
		catalogWipeOutAnimation.play();
		this.connect(catalogWipeOutAnimation, "onEnd", this._afterCatalogWipeOutAnimation);
	},

	_createCatalog: function() {
		if (this._catalogContent.hasChildNodes()) {
			return;
		} 
		
		this._catalogDrawer = new CatalogDrawer({
			onAddWidget: dojo.hitch(this, this._addWidgetToTab),
			onCloseCatalog: dojo.hitch(this, this.closeCatalog)
		}); 

		// Add the new child node
		this._catalogContent.appendChild(this._catalogDrawer.domNode);
	},
	
	_addWidgetToTab: function(/* object */ data) {
		// Get current tab
		var viewletPage = this.getCurrentViewletPage();
		if (!viewletPage) {
			// Error
			return;
		}		
		// Add the widget to the current tab
		viewletPage.addWidget(data);
	},
	
	_afterCatalogWipeInAnimation: function() {
	},
	
	_afterCatalogWipeOutAnimation: function() {
		this._showCatalogLink();
		dojo.publish("jazz.util.ResizeNotifier.resizeEvent");		
	},

	_selectCatalogLink: function() {
		dojo.addClass(this._catalogLink, "emphasis");
	},	
	
	_showCatalogLink: function() {
		this._catalogLink.style.display = "";
		dojo.removeClass(this._catalogLink, "emphasis");
	},	
	
	_hideCatalogLink: function() {
		this._catalogLink.style.display = "none";	
	},

	
	// prompts the users and reloads dashboard based on choice.
	refresh: function(event) {
		if (event) 
			dojo.stopEvent(event);
		if (this.isDirty() && !confirm(messages.confirm)) {  //if the dashboard is dirty, prompt the user that they will lose changes
			return;	
		}
		if (this.errorState || !this.dashboardResource || this.isDirty()) {
			this.reload();
		} else {
			this._refresh();
		}
	},
	// forces a reload of the current dashboard - doesn't check for dirty state
	reload: function() {
		var params = Util.getDashboardParams(this.dashboardBundle, this.tabs.selectedTabId);
		params.force = true;
		this.dashboardWidget.viewDashboard(params, this.dashboardWidget.projectArea);
	},
	duplicateTab: function(event) {
		if (event)
			dojo.stopEvent(event);
		this.handleCloneTab(false);
	},
	copyTab: function(event) {
		if (event)
			dojo.stopEvent(event);
		this.handleCloneTab(true);
	},
	handleCloneTab: function(copy) {
		if (this.currentPage) {
			var pageElem = this.currentPage.page.elem;
			if (copy) {
				// store the page XML in storage
				try {
					localStorage["net-jazz-ajax-clipboard"] = dojox.xml.parser.innerXML(pageElem);
				} catch (e) {
					alert(messages.featureNotSupported);
				}
			}
			else {
				var newPageElem = pageElem.cloneNode(true);
				var index = dojo.indexOf(this.viewletPages, this.currentPage);
				this.addPage(newPageElem, index+1);
			}
		}
	},
	saveDashboard: function() {
		// to ensure fresh state, only allow one save at a time
		if (this._saveInTransit) {
			this._savePending = true;
		}
		else {
			if (!this.autoSave) {
				// Don't show the 'save in progress' message if the dashboard is set to auto-save
				this._showSaveInProgressMessage(true);
			}
			this._doSave();
		}
	},
	setActive: function(active) {
		this._dashboardActive = active;
		if (this._blockerDiv) this._blockerDiv.style.display = active ? "block" : "none";
		if (this.zoomDiv) this.zoomDiv.style.display = active ? "block" : "none";
		if (this.currentPage) this.currentPage.setActive(active);
	},	
	setDirty: function(isDirty) {
		if (typeof isDirty == "undefined") isDirty = true;
		this._isDirty = isDirty;
		this.metaNav.setDirty(isDirty);
		if (isDirty && !this.autoSave && !this._stalenessCheckInterval) {
			this._stalenessCheckInterval = setInterval(dojo.hitch(this, this._checkStaleness), STALE_CHECK_INTERVAL);
		}
	},
	setEditMode: function (editMode, noPropagate) {
		this.editMode = editMode;
		if (this.isEditable()) {
			this.autoSave = (editMode == Constants.EDIT_AUTO_SAVE) ? true : false; 
			if (this.dashboardBundle && this.dashboardBundle.permissions.addTab) {
				this._createNewTabLink();
			}
			this._showCatalogLink();
			if (!noPropagate) this.tabs.setEditMode(true);
		} else {
			this._removeNewTabLink();
			this._hideCatalogLink();
			if (!noPropagate) this.tabs.setEditMode(false);
		}
		
		if (!noPropagate) {
			this.metaNav.setEditMode(editMode);
			var viewletPages = this.viewletPages;
			if (viewletPages) {
				for (var i = 0; i < viewletPages.length; i++) {
					if (viewletPages[i]) viewletPages[i].setEditMode(this.isEditable());
				}
			}
		}
	},
	
	addPage: function(/* Element */ elem, tabIndex) {
		var newPage = XmlBindingFactory.createBinding(elem, ViewletPage.SCHEMA);
		newPage.setId(this._generatePageId());
		this.doCreateViewletPage(newPage, tabIndex);
	},
	
	/*
	 *   Public functions: Viewlet page management
	 */
	createViewletPage: function(event){
		if (event) {
			dojo.stopEvent(event);
		}		
		// create the page elem
		var dashboard = this.dashboardResource.binding;
		var pageCollection = dashboard.getPages();
		var newPage = pageCollection.createPage();
		newPage.setId(this._generatePageId());
		newPage.setTitle(messages.newTab);
		var widgetCollection = newPage.createWidgets();
		widgetCollection.setParseType("Collection");
		newPage.setWidgets(widgetCollection);
		newPage.setLayout(LayoutFactory.createDefaultLayoutState(newPage));
		this.doCreateViewletPage(newPage);
	},
	doCreateViewletPage: function(/* object */ page, index) {
		if (typeof index !== "number")
			index = this.tabs.widgets.length;
		var dashboard = this.dashboardResource.binding;
		var pageCollection = dashboard.getPages();
		var pages = pageCollection.getPages();
		pages.splice(index, 0, page);
		pageCollection.setPages(pages);
		
		this.viewletPages.splice(index,0,null); // create a space in the viewletPages array, so the page will be reinitialized
		this._onDashboardChange();
		this.tabs.add(null,{title: page.getTitle(), id: page.getId(), index: index, page: page, type: "tab"});
		this.tabs._selectTabAction(this.tabs.widgets[index]);
		this.tabs.focusCurrentTab();
	},
	deleteViewletPage: function(index) {
		if (this.viewletPages[index]) {
			this.viewletPages[index].destroy();
		}
		this.viewletPages.splice(index, 1);
		var dashboard = this.dashboardResource.binding;
		var pageCollection = dashboard.getPages();
		var pages = pageCollection.getPages();
		pages.splice(index, 1);
		pageCollection.setPages(pages);
		this._onDashboardChange();
		this.showViewletPage(0);
	},
	moveViewletPage: function(startIndex, endIndex) {
		//update viewletpage widgets array
		var startPage = this.viewletPages[startIndex];
		this.viewletPages.splice(startIndex,1);
		this.viewletPages.splice(endIndex, 0, startPage);
		//update model
		var dashboard = this.dashboardResource.binding;
		var pageCollection = dashboard.getPages();
		var pages = pageCollection.getPages();
		startPage = pages[startIndex];
		pages.splice(startIndex,1);
		pages.splice(endIndex, 0, startPage);
		pageCollection.setPages(pages);
		this._onDashboardChange();
	},
	isPageActive: function(pageId) {
		return this._dashboardActive && pageId == this.selectedTabId;
	},
	refreshCurrentTab: function() {
		this.setPageDirty(this.tabs.selectedTab.index);
		this.showViewletPage(this.tabs.selectedTab.index);		
	},
	restoreDashboard: function() {
		var projectAreaItemId = jazz.app.context.get().itemId;
		var srh = new ServiceResponseHandler(this, "_restoreDashboard2", "_logError");
		DashboardClient.restoreDashboard(this.dashboardResource.url, projectAreaItemId, this.dashboardBundle.scope, this.dashboardBundle.scopeItem.itemId, srh, this.serviceTracker);
	},
	_restoreDashboard2: function(/* ResponseDTO */ responseDTO) {
		if (responseDTO.status == "OK") {
			this.propertyChanged("title", responseDTO.msg);
			this.reload();
		}
		else {
			alert(dojo.string.substitute(messages.restoreError, [responseDTO.msg]));
		}
	},
	setPageDirty: function(index) {
		if (this.viewletPages && this.viewletPages[index]){
			this.viewletPages[index].dirty = true;	
		}
	},
	showViewletPage: function(index) {
		var init = false;
		var vp = this.viewletPages[index];
		this.tabs.selectTab(index);
		if (this.currentPage && this.currentPage != this.viewletPages[index]) {
			this.currentPage.setActive(false);
			if (this.currentPage.domNode) {
				this.currentPage.domNode.style.display = "none";
			}
		}
		if (!vp || vp.dirty) {
			if (vp && vp.destroyRendering) {
				vp.destroyRendering();
			}
			this._viewletPageWrapper.appendChild(this._createViewletPage(this.tabs.widgets[index].page, index).domNode);
			init = true;
		} else {
			vp.domNode.style.display = "";
		}
		this.currentPage = this.viewletPages[index];
		this.selectedTabId = this.currentPage.pageId;
		if (init) {
			this.currentPage.init();
		}
	},
	showViewletPageById: function(pageId) {
		var pages = this.dashboardResource.binding.getPages().getPages();
		for (var i=0;i<pages.length;i++) {
			if (pages[i] && pages[i].getId() === pageId) {
				this.showViewletPage(i);
				break;
			}
		}
	},
	
	/*
	 * Private Functions
	 */
	
	_createViewletPage: function(/* object */ page, /* Integer */ index) {
		if (this.viewletPages[index]) {
			this.viewletPages[index].destroy();
		}
		this.viewletPages[index] = new ViewletPage({page: page, pageId: page.getId(), dashboardBundle: this.dashboardBundle, dashboardContainer: this});
		this.viewletPages[index].addListener(dojo.hitch(this, this._onDashboardChange), Constants.PROPERTY_DASHBOARD);
		return this.viewletPages[index];
	},
	_doSave: function() {
		this._saveInTransit = true;
		var dfd = RestClient.putResource(this.dashboardResource);
		dfd.addCallback(dojo.hitch(this, this._handleSave));
		dfd.addErrback(dojo.hitch(this, this._handleSaveError));
		
		if (this._stalenessCheckInterval) {
			clearInterval(this._stalenessCheckInterval);
			delete this._stalenessCheckInterval;
		}
	},
	/*
	 * Destroys all viewlet widgets so that they can be garbage collected.
	 */
	destroy: function() {
		this._isDestroyed = true;
		// abort any remaining requests
		if (this._serviceTracker) this._serviceTracker.abortActiveRequests();
		// are we in the middle of loading?
		if (this._yieldTimeout) {
			// cancel loading
			clearTimeout(this._yieldTimeout);
			delete this._yieldTimeout;
		}
		if (this._stalenessCheckInterval) {
			clearInterval(this._stalenessCheckInterval);
			delete this._stalenessCheckInterval;
		}
		if (this._currentTab)
			this._currentTab.destroy();
		if (this.viewletPages) {
			for (var i = 0; i < this.viewletPages.length; i++) {
				if (this.viewletPages[i]) {
					this.viewletPages[i].destroy();
					this.viewletPages[i] = null;
				}
			}
		}
		if (this._hideTimeout) {
			delete this._hideTimeout;
		}
		this.metaNav.destroy();
		if (this.tabs) {
			this.tabs.destroy();	
		}
		if (this._catalogDrawer) {
			this._catalogDrawer.destroy();
			this._catalogDrawer = null;
		}
		this.inherited(arguments);
		this._isDestroyed = true;
	},
	_isEditingContents: function() {
		//traverse the viewlet pages looking for viewlets that are in "Edit Contents" mode
		var viewletPages = this.viewletPages;
		if (viewletPages) {
			for (var i = 0; i < viewletPages.length; i++) {
				if (viewletPages[i]) {
					var columns = this.viewletPages[i].columns;
					for (var j = 0; columns && j < columns.length; j++) {
						var widgets = this.viewletPages[i].columns[j].widgets;
						for (var k = 0; widgets && k < widgets.length; k++) {
							var widget = widgets[k];
							if (widget && widget._viewlet) {
								if (widget._viewlet.isEditing()) {
									return true;
								}
							}
						}
					}
				}
			}
		}
		return false;
	},
	/*
	 * Utility empty callback for success responses we don't need to
	 * react to.
	 */
	_empty: function() {
		
	},	
	/*
	 * Utility method to log remote errors when calling services.
	 */	
	_logError: function(errorObj) {
		console.error("DashboardContainer.js remote error: " + errorObj.message);
	},
	
	/*
	 * Handles save PUT response.
	 */
	_handleSave: function(/* any */ obj) {
		this._saveInTransit = false;
		if (this._savePending) {
			this._savePending = false;
			this._doSave();
		} else {
			if (this.showSaveNotification || !this.autoSave) {
				// Always show the save successful notice for a manual save operation.
				// In auto-save mode, only show the 'Save successful' message if the
				// user has tried to navigate away from the Dashboard while a save is
				// is progress.  
				this.metaNav.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.OK,
					summary: messages.saveComplete,
					closable: true
				});
				if (this._hideTimeout) {
					// Clear the existing timeout if one is active
					clearTimeout(this._hideTimeout);
					delete this._hideTimeout;
				}
				this._hideTimeout = setTimeout(dojo.hitch(this, "_hideSavedNotification"), 5000);
			}
			this.propertyChanged("onsave");
		}
		return obj;
	},
	
	_hideSavedNotification: function() {
		this.showSaveNotification = false;
		if (this.metaNav) {
			this.metaNav.hideMessage(true);
		}
	},
	
	/*
	 * Handles save error response.
	 */
	_handleSaveError: function(/* Error */ error) {
		dojo.publish(Constants.serverError, [error]);
		this._logError(error);
		this._saveInTransit = false;
	},
	
	/*
	 * Generates a new page id that is not currently used in this container.
	 */
	_generatePageId: function() {
		var dashboard = this.dashboardResource.binding;
		var pageCollection = dashboard.getPages();
		var pages = pageCollection.getPages();
		var max = 1;
		for (var i=0;i<pages.length;i++) {
			var id = pages[i].getId();
			if (id.length > 1) {
				try {
					var num = parseInt(id.substring(1));
					if (num > max) {
						max = num;
					}
				}
				catch (/* Error */ e) {
					// not of the form "_#", skip
				}
			}
		}
		// use the next id
		return "_" + (max + 1);
	},
	
	/*
	 * Called after the user has modified the dashboard.
	 */
	_onDashboardChange: function() {
		if (this._staleDashboardWarning) {
			/* The gentle dashboard out of date notification is currently displayed. Now
			 * that the dashboard was dirtied, we need to notify the user with an angry message 
			 */
			this._displayConcurrentModificationError();
			// Stop the timer to stop polling for changes 
			if (this._stalenessCheckInterval) {
				clearInterval(this._stalenessCheckInterval);
				delete this._stalenessCheckInterval;
			}
			return;						
		}
		if (this.autoSave) {
			this.saveDashboard();
		}
		else {
			this.setDirty(true);
		}
	},
	
	_refresh: function() {
		if (this.viewletPages) {
			for (var i = 0; i < this.viewletPages.length; i++) {
				this.setPageDirty(i);
			}
		}
		this.refreshCurrentTab();
	},
	_createNewTabLink: function() {
		//create new tab link
		if (!this._newTabLink) {
			var newTabLink = document.createElement("a");
			newTabLink.className = "add-new-tab";
			newTabLink.href = "#";
			newTabLink.title = messages.addNewTab;
			var newTabImg = document.createElement("img");
			newTabImg.src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/MetaNav/icons/add-tab.png");
			newTabImg.alt = messages.addNewTab;
			newTabLink.appendChild(newTabImg);
			this._tabs.firstChild.appendChild(newTabLink);
			this._newTabLinkEvent = this.connect(newTabLink,"onclick",this.createViewletPage);
			this._newTabLink = newTabLink;
		}
	},
	_removeNewTabLink: function(){
		if (this._newTabLink && this._newTabLink.parentNode) {
			this.disconnect(this.newTabLinkEvent);
			this._newTabLink.parentNode.removeChild(this._newTabLink);
			this._newTabLink = null;
		}
	},
	_showFirebugWarning: function() {
		if (this._firebugWarning) return;
		this._firebugWarning = true;
		if (window.console && window.console.firebug) {
			this.metaNav.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.WARNING,
				summary: this.messages.firebugWarning
			});	
		}
	},
	_showBlockerDiv: function(opacity, zIndex) {
		if (!this._blockerDiv) {
			this._blockerDiv = document.createElement("div");
			this._blockerDiv.className = "jazz-ui-Dialog-blockerDiv";			
		}

		var de = document.documentElement;
		var height = de.clientHeight;
		var scrollHeight = de.scrollHeight;
		
		this._blockerDiv.style.height = ((scrollHeight > height) ? scrollHeight : height) + "px";
		
		if (opacity == null) {
			opacity = 0.06;
		}
		var args = {opacity: 0, top: 0, left: 0}; 
		args.zIndex = zIndex ? zIndex : 0;
		dojo.style(this._blockerDiv,args);

		var appendTo = dojo.byId("content-cell") || document.body;
		appendTo.appendChild(this._blockerDiv);
		
		if (opacity !== 0) {
			var blockerAnim = dojo.animateProperty({node: this._blockerDiv, duration: 280,
				properties:	{opacity: {start: 0, end: opacity}
			}});
			
			blockerAnim.play();
		}
	},
	_hideBlockerDiv: function() {
		if (this._blockerDiv && this._blockerDiv.parentNode)  {
			this._blockerDiv.parentNode.removeChild(this._blockerDiv);
		}
	},
	
	getCurrentViewletPage: function() {
		if (!this.currentPage) {
			return null;
		}
		return this.currentPage;
	},		
	
	_showSaveInProgressMessage: function(/* boolean */ showSaveCompleteMessage) {
		this.metaNav.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
			summary: messages.saveInProgress,
			closable: true
		});
		this.showSaveNotification = showSaveCompleteMessage;
	},
	
	_checkStaleness: function() {
		if (this.dashboardBundle.dashboardUrl) {
			var dfd = RestClient.headResource(this.dashboardBundle.dashboardUrl, {});
			dfd.addCallback(dojo.hitch(this, this._handleDashboardHeadRequest));
			dfd.addErrback(dojo.hitch(this, this._handleDashboardHeadError));
		} 
	},
	
	_displayConcurrentModificationError: function() {
		dojo.publish(Constants.serverError, [{
			type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
			status: 409, // Status code 409 indicates a conflict
			customMessage: messages.errorConcurrentModification,
			closeable: false
		}]);		
	},
	
	_handleDashboardHeadError: function(/* Error */ error) {
		if (error.status == 0 || error.status == 12029) {
			/* Check for loss of connectivity and don't cancel the timer. FF/Chrome/Safari all return 0 as a status 
			 * in this case, IE returns 12029 */
			return;			
		}
		if (error.status == 408 || error.status == 504) {
			/* Check to see if the request has timed out (408 "Request Timeout", 504 "Gateway Timeout")
			 * W3 HTTP spec says "The client MAY repeat the request without modifications at any later time.",
			 * so lets not cancel the staleness check timer
			 */			
			return;		
		}
		
		// Stop the timer to stop polling for changes 
		if (this._stalenessCheckInterval) {
			clearInterval(this._stalenessCheckInterval);
			delete this._stalenessCheckInterval;
		}
	},
	
	_handleDashboardHeadRequest: function(/* resource */ result) {
		if (this._isDestroyed) return;
		if (this.dashboardResource.eTag != result.eTag) {
			
			if (this.isDirty()) {
				// Conflicting dashboard change has been detected -> display error message to user 
				// and notify them that they need to reload and discard changes
				this._displayConcurrentModificationError();
			} else {
				// The dashboard has been updated, so gently notify the user that
				// they are viewing an out-of-date bashboard. This gentle message
				// will turn into a scary one if they user dirties their dashboard
				dojo.publish(Constants.serverError, [{
					type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
					status: 409, // Status code 409 indicates a conflict
					customMessage: messages.errorConcurrentModificationNonDirty,
					closeable: false
				}]);
				this._staleDashboardWarning = true;
			}
			
			// Stop the timer to stop polling for changes 
			if (this._stalenessCheckInterval) {
				clearInterval(this._stalenessCheckInterval);
				delete this._stalenessCheckInterval;
			}
		}

	},

	_addViewletImgSrc: dojo.moduleUrl("com.ibm.team.dashboard.web","graphics/DashboardMini/add-widget.png")
});
})();
