/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.dashboard.web.ui.internal.DashboardInitializationData");

dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.repository.web.client.init.InitializationData");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var InitializationData 	= com.ibm.team.repository.web.client.init.InitializationData;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;

var _dfd;
var _initData;

/*
 * Provides initialization data for Dashboards.
 */
com.ibm.team.dashboard.web.ui.internal.DashboardInitializationData = {
	
	/*
	 * Returns an object representing the currently authenticated user.
	 * The object contains the following properties:
	 * - {string} name: Name of the user
	 * - {string} itemId: The repository item ID of the contributor item
	 * - {string} userId: The user's ID
	 */
	getAuthenticatedContributor: function() {
		_checkInit();
		return _initData.currentContributor;
	},
	
	/*
	 * Returns the currently authenticated contributor's URI, or undefined if not
	 * available.
	 * 
	 * see:
	 * 123756: Include current user URL in initialization data
	 */
	getAuthenticatedContributorUri: function() {
		_checkInit();
		return _initData.currentContributorUri;
	},
	
	/*
	 * Returns the dashboard service URI.
	 */
	getDashboardServiceUri: function() {
		_checkInit();
		return _initData.dashboardServiceUri;
	},
	
	/*
	 * Returns the dashboard query service URI.
	 */
	getDashboardQueryServiceUri: function() {
		_checkInit();
		return _initData.dashboardQueryServiceUri;
	},
	
	/*
	 * Loads the initialization data.
	 */
	/* Deferred */ load: function() {
		if (_dfd) {
			return _dfd;
		}
		if (!_initData) {
			try {
				_initData = InitializationData.get("com.ibm.team.dashboard.service.dashboardInitializationData");
			}
			catch (e) {
				// "real" fronting apps don't have initialization data
				// fall back to explicit fetch
			}
		}
		if (_initData) {
			var dfd = new dojo.Deferred();
			dfd.callback();
			return dfd;
		}
		_dfd = new dojo.Deferred();
		var serviceObj = {
			success: function(/* InitDataDTO */ initData) {
				_initData = initData;
				_dfd.callback();
				_dfd = null;
			},
			failure: function(/* Error */ e) {
				_dfd.errback(e);
			}
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		DashboardClient.getInitData(srh);
		return _dfd;
	}
};

function _checkInit() {
	if (!_initData) {
		throw new Error("You must load DashboardInitializationData before using");
	}
}

})();
