/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.DashboardPage");

dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.dashboard.web.ui.Dashboard");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.AllShared");
dojo.require("com.ibm.team.dashboard.web.ui.internal.AllTeamAreas");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.DashboardContainer");
dojo.require("com.ibm.team.dashboard.web.util.DashboardUtil");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("dojo.hash");

dojo.require("jazz.ensemble.internal.client.RestClient");
dojo.require("jazz.app.context");

dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

var AllShared = com.ibm.team.dashboard.web.ui.internal.AllShared;
var AllTeamAreas = com.ibm.team.dashboard.web.ui.internal.AllTeamAreas;
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var Dashboard = com.ibm.team.dashboard.web.ui.Dashboard;
var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var DashboardContainer = com.ibm.team.dashboard.web.ui.internal.DashboardContainer;
var DashboardUtil = com.ibm.team.dashboard.web.util.DashboardUtil;
var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var RestClient = jazz.ensemble.internal.client.RestClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var Util = com.ibm.team.dashboard.web.util.internal.Util;

var messages;
var recent = [];
var unsaved = true;

/*
 * The Web UI page (vertical) containing the dashboard and side navigation bar.
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.DashboardPage",
	[dijit._Widget, dijit._Templated], {
	
	templateString: "<div class=\"com-ibm-team-dashboard-web-ui-internal-DashboardPage\"></div>",
	
	constructor: function() {
		if (!messages) {
			messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
		}
	},
	
	postCreate: function() {
		this.subscribe("/jazz/dashboard/initMenu", this._initMenu);

		this._contentDiv = this.domNode;

		// register actions
		var workbench = PlatformUI.getWorkbench();
		var actionRegistry = workbench.getActionRegistry();
		actionRegistry.registerAction("com.ibm.team.dashboard.viewDashboard", this, "viewDashboard");
		actionRegistry.registerAction("com.ibm.team.dashboard.viewAllDashboards", this, "viewAllDashboards");
		actionRegistry.registerAction("com.ibm.team.dashboard.viewDefaultDashboard", this, "viewDefaultDashboard");
		this.navigateAwayHandle = workbench.registerBodyClickHandler(dojo.hitch(this, this._handleNavigate), /*priority*/ 10000);
	},
	
	_initMenu: function(menu) {
		if (recent) {
			menu.initRecent(recent);
			recent = null;
		}
		if (typeof unsaved === "object") {
			menu.onDirtyStateChange(unsaved, true);
			unsaved = null;
		}
	},
	
	destroy: function() {
		if (this._allDashboards) {
			this._allDashboards.destroy();
		}
		this.destroyDashboard();
		this.inherited(arguments);
	},
	
	destroyDashboard: function() {
		if (this._dashboard) {
			this.disconnect(this._onDirtyStateConnect);
			dojo.publish("/jazz/dashboard/dirty", [this._dashboard.getBundle(), false]);
			PlatformUI.getWorkbench().setPageDirty("com.ibm.team.dashboard",false);
			this._dashboard.destroy();
			delete this._dashboard;
		}
	},
	
	_handleNavigate: function(urlObj) {
		var winLoc = new dojo._Url(window.location.href);
		if ((!urlObj.host || !urlObj.scheme || (urlObj.host === winLoc.host && urlObj.scheme === winLoc.scheme)) &&   	// if the target is to the same domain and
				urlObj.fragment && ((!urlObj.host && !urlObj.path) || (urlObj.path && urlObj.path === winLoc.path)) &&	// the target is in the web ui and
				this._dashboard instanceof Dashboard) { 																// we have a dashboard showing
				
			if (urlObj.fragment.indexOf("com.ibm.team.dashboard") === -1) {												// if the target is not in dashboards
				// if we are leaving dashboards, allow the transition and notify the dashboard
				if (winLoc.fragment.indexOf("com.ibm.team.dashboard") !== -1)
					this._dashboard.setActive(false);		
			} else if (!this.isDashboardShowing(this.getScopeInfoFromArgs(dojo.queryToObject(urlObj.fragment))) && 		// if the target is in dashboards and is not the current dashboard and
					this._dashboard.isUnsaved()) {																		// the dashboard has unsaved changes	
				var doNavigate = this._dashboard.promptNavigateAway();
				if (!doNavigate) {
					dojo.hash(this._currentDashboardHash);
					return true;
				}
			}
		}
		return false;
	},
	
	getScopeInfoFromArgs: function(args) {
		var result = {};
		if (args.action === "com.ibm.team.dashboard.viewAllDashboards") return result;
		if (args.id) {
			result.scope = DashboardConstants.SCOPE_CONTRIBUTOR;
			result.scopeItem = {id: args.id};
		}
		else {
			if (args.team) {
				result.scope = DashboardConstants.SCOPE_TEAM_AREA;
				result.scopeItem = args.team.split("/");
			}
			else {
				result.scope = DashboardConstants.SCOPE_PROJECT_AREA;
			}
		}
		return result;
	},
	
	isDashboardShowing: function(scopeInfo) {
		if (!this._dashboard) return false;
		return scopeInfo.scope === this._dashboard.scope && dojo.toJson(scopeInfo.scopeItem) === dojo.toJson(this._dashboard.scopeItem) && jazz.app.context.get().name === this._dashboard.projectArea;
	},
	
	viewAllDashboards: function(args) {
		this.destroyDashboard();
		var clazz;
		if (args.tab == 1) {
			clazz = AllShared;
		} else {
			clazz = AllTeamAreas;
		}
		if (clazz && !(this._contentWidget instanceof clazz)) {
			if (this._allDashboards) {
				this._allDashboards.destroy();
			}
			this._allDashboards = new clazz();
			this._setContent(this._allDashboards);
			this._updateBrowserTitle();
		}
	},
	
	viewDashboard: function(args) {
		this._currentDashboardHash = dojo.hash();
		
		var scopeInfo = this.getScopeInfoFromArgs(args);
		// check if already showing
		if (this.isDashboardShowing(scopeInfo)) {
			// same dashboard
			if (args.tab) // if specified, switch to the appropriate tab
				this._dashboard.showTab(args.tab);
			else // or just activate it if not
				this._dashboard.setActive(true);
		} else {
			if (this._dashboard) {
				// different dashboard; destroy old one
				this.destroyDashboard();
			}
			this._dashboard = new Dashboard({
				scope: scopeInfo.scope,
				scopeItem: scopeInfo.scopeItem,
				projectArea: jazz.app.context.get().name,
				tab: args.tab
			});
			this._dashboard.addListener(dojo.hitch(this, this._onLoad), "onload");
			this._dashboard.addListener(dojo.hitch(this, this._onDelete), "ondelete");
			this._dashboard.addListener(dojo.hitch(this, this._onTabChange), "tab");
			this._dashboard.addListener(dojo.hitch(this, this._onTitleChange), "title");
			this._setContent(this._dashboard);
			this._onDirtyStateConnect = this.connect(this._dashboard,"onDirtyStateChange",this._onDirtyStateChange);
		}

		if (this._title) {
			// "touch" the title so that it doesn't revert between navigations
			PlatformUI.getWorkbench().setTitle(this._title);
		}
	},
	
	viewDefaultDashboard: function(args) {
		this._replaceURL("#action=com.ibm.team.dashboard.viewDashboard");
	},
		
	/* string */ _getCurrentHref: function() {
		if (this._dashboard) {
			var scopeItem = this._dashboard.scopeItem;
			if (this._dashboard.scope == DashboardConstants.SCOPE_CONTRIBUTOR) {
				scopeItem = this._dashboard.scopeItem.id;
			}
			return DashboardUtil.getDashboardHref(this._dashboard.scope, scopeItem, this._dashboard.tab);
		} else if (this._contentWidget instanceof AllShared) {
			return Util.getAllDashboardsHref(1);
		} else if (this._contentWidget instanceof AllTeamAreas) {
			return Util.getAllDashboardsHref(0);
		}
	},
	
	_onTitleChange: function(/* string */ property, /* string */ title) {
		var bundle = this._dashboard.getBundle();
		dojo.publish("/jazz/dashboard/rename",[bundle.dashboardUrl, title]);
		this._updateBrowserTitle();
	},
	
	_onDirtyStateChange: function(isDirty) {
		PlatformUI.getWorkbench().setPageDirty("com.ibm.team.dashboard",isDirty);
		var bundle = this._dashboard.getBundle();
		if (unsaved) {
			if (isDirty) unsaved = bundle;
			else if (unsaved.dashboardUrl === bundle.dashboardUrl) unsaved = true;
		}
		dojo.publish("/jazz/dashboard/dirty", [bundle, isDirty]);
	},
	
	_onLoad: function() {
		if (this._dashboard) {
			var bundle = this._dashboard.getBundle();
			dojo.publish("/jazz/dashboard/view", [bundle]);
			if (recent) 
				recent.push(bundle);
			this._updateBrowserTitle();
		}
	},
	
	_onDelete: function() {
		// delete the resource
		var url = this._dashboard.getBundle().dashboardUrl;
		var dfd = RestClient.deleteResource(url);
		dfd.addCallback(dojo.hitch(this, function() {
			// navigate to another dashboard
			dojo.publish("/jazz/dashboard/delete", [url])
			this._dashboard.setCurrentDeleted();
			this.viewDefaultDashboard();
		}));
		dfd.addErrback(dojo.hitch(this, function(/* Error */ e) {
			console.error(e);
		}));
	},
	
	_onTabChange: function(/* string */ propertyId, /* string */ tab) {
		this._updateBrowserTitle();
		this._updateURL();
	},
	
	_setContent: function(/* Widget */ widget) {
		if (this._contentDiv.firstChild != widget.domNode) {
			// swap in new widget
			if (this._contentDiv.firstChild) {
				this._contentDiv.replaceChild(widget.domNode, this._contentDiv.firstChild);
			}
			else {
				this._contentDiv.appendChild(widget.domNode);
			}
			this._contentWidget = widget;
		}
	},
	
	_updateBrowserTitle: function() {
		var contentWidget = this._contentWidget;
		if (contentWidget && contentWidget instanceof Dashboard) {
			var title = contentWidget.getTitle(true);
		} else if (contentWidget && contentWidget instanceof AllShared) {
			title = messages.allPersonalTitle;
		} else if (contentWidget && contentWidget instanceof AllTeamAreas) {
			title = messages.allTeamAreasTitle;
		} else {
			title = messages.dashboardsGenericTitle;
		}
		title = dojo.string.substitute(messages.projectDashboardTitle, [title]);
		this._title = title;
		PlatformUI.getWorkbench().setTitle(title);
	},
	_updateURL: function() {
		if (this._dashboard && this._dashboard.getBundle()) {
			var curHref = this._getCurrentHref();
			if (curHref.charAt(0) == "#") {
				dojo.hash(curHref);
			} else {
				location.href = curHref;
			}
		}
	},
	_replaceURL: function(/* string */ url) {
		dojo.hash(url,true);
	}
});
})();