/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.FeedViewlet");

dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.dashboard.viewlets.web.client.internal.FeedClient");
dojo.require("com.ibm.team.dashboard.web.ui.PagedTableViewlet");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("jazz.util.format");
dojo.require("jazz.util.i18n");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");

dojo.require("jazz.ui.HoverPopup");
dojo.require("jazz.util.date");
dojo.require("jazz.util.html");

dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "FeedMessages");

(function() {

/*
 * Aliases
 */
var DateTime = jazz.util.date;
var FeedClient = com.ibm.team.dashboard.viewlets.web.client.internal.FeedClient;
var PagedTableViewlet = com.ibm.team.dashboard.web.ui.PagedTableViewlet;
var HoverPopup = jazz.ui.HoverPopup;
var HTML = jazz.util.html;
var I18nUtil = jazz.util.i18n;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var FormatUtil = jazz.util.format;
var Util = com.ibm.team.dashboard.web.util.internal.Util;

/*
 * Constants
 */
var PREFERENCE_ID_URL = "url";
var PREFERENCE_ID_USER = "user";
var PREFERENCE_ID_PASSWORD = "password";
var PREFERENCE_ITEMS_TO_SHOW = "items";
var SERVICE_ID = "com.ibm.team.dashboard.viewlets.service.internal.feed.IFeedService";

var messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "FeedMessages");

/*
 * A viewlet that displays a news feed.
 */
dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.FeedViewlet",
	PagedTableViewlet, {
	
	/*
	 * the css class for this viewlet; consumed by superclass.
	 */
	contentClass: "com-ibm-team-dashboard-viewlets-web-ui-internal-FeedViewlet",
	
	constructor: function() {
		this._entryCount = -1;
		this._columnCount = 1;
		this._hoverPopups = [];
		this._feedTooltips = [];
	},
	
	/*
	 * Initializes the feed viewlet.
	 */
	init: function() {
		this._trackReadMarks = this.hasContributorState();
		if (this._trackReadMarks) {
			this.getSite().addMenuItem(messages.markAllRead, dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web", "graphics/feeds/icons/menu-mark-read.png").toString(), dojo.hitch(this, this._markAll, true));
			this.getSite().addMenuItem(messages.markAllUnread, dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web", "graphics/feeds/icons/menu-mark-unread.png").toString(), dojo.hitch(this, this._markAll, false));
		}
		this.inherited(arguments);
	},
	
	/*
	 * Called to add the Tooltip to the given entry.
	 */
	createTooltip: function(linkNode, entry) {
		// attach tooltip
		return new HoverPopup({
			around: [linkNode],
			createContents: dojo.hitch(this, this._createTooltip, entry),
			onOpen: dojo.hitch(this, this._onTooltipOpen, entry),
			onClose: dojo.hitch(this, this._onTooltipClose),
			isEnabled: dojo.hitch(this, function() {
				return !this.isZoomed();
			})
		});
	},
	
	uninitialize: function() {
		for (var i=0;i<this._hoverPopups.length;i++) {
			this._hoverPopups[i].destroy();
		}
		this._hoverPopups = [];
		for (var i=0;i<this._feedTooltips.length;i++) {
			this._feedTooltips[i].destroy();
		}
		this._feedTooltips = [];
	},
	
	/*
	 * Called by super class when entries are needed.
	 */
	fetch: function (/* ServiceResponseHandler */ srh, /* number */ startIndex, /* number */ count, /* boolean */ entries, /* boolean */ details) {
		delete this._errorMsgNode;
		var url = this.getPreference(PREFERENCE_ID_URL);
		if (!url || url.length == 0) {
			if (!this._noFeedMsg) {
				// this part always shows
				this._noFeedMsg = document.createElement("span");
				this._noFeedMsg.appendChild(document.createTextNode(messages.noURL + " "));
				
				// this part only shows when dashboard in edit mode
				this._noFeedExtraMsg = document.createElement("span");
				var links = I18nUtil.addMessage(this._noFeedExtraMsg, messages.editSettings);
				this.connect(links[0], "onclick", dojo.hitch(this, function(e) {
					e.preventDefault();
					this.getSite().toggleSettings();
				}));
				this._noFeedExtraMsg.className = "editModeText";
				this._noFeedMsg.appendChild(this._noFeedExtraMsg);
				if (!this.getSite().isDashboardEditing()) {
					this._noFeedExtraMsg.style.display = "none";
				}
			}
			this._errorMsgNode = this._noFeedMsg;
		}
		var items = this.getPreference(PREFERENCE_ITEMS_TO_SHOW);
		if (!items || items < 1) {
			this._errorMsgNode = document.createTextNode(messages.errorItems)
		}
		
		// display error msg if there was an error
		if (this._errorMsgNode) {
			this._entryCount = 0;
			this._updateSecondaryTitle();
			srh.doCallback();
		}
		else {
			// fetch icon if new url
			if (this._lastURL != url) {
				this._lastURL = url;
				this.getSite().setIcon(Util.getServiceURL(SERVICE_ID, "icon", {url: url}));
			}
			// fetch feed
			var user = this.getPreference(PREFERENCE_ID_USER);
			var password = this.getPreference(PREFERENCE_ID_PASSWORD);
			var serviceObj = {
				_srh: srh,
				_viewlet: this,
				_entries: entries,
				_details: details,
				success: function(/* FeedResponseDTO */ response) {
					this._viewlet.handleFetch(response, this._entries, this._details, this._srh);
				},
				error: function(/* Error */ error) {
					this._srh.doErrorCallback(error);
				}
			};
			var srh2 = new ServiceResponseHandler(serviceObj, "success", "error");
			FeedClient.getFeed(url, user, password, null, this.getReferenceItemId(), startIndex, count, entries, details, !this._feedInfo, srh2, this.getServiceTracker());
		}
	},
	
	/*
	 * Called when an entry's tooltip is needed. Subclasses should override.
	 */
	fetchTooltip: function(/* EntryDTO */ entry, /* ServiceResponseHandler */ srh) {
		// fetch details of the specific item
		var url = this.getPreference(PREFERENCE_ID_URL);
		var user = this.getPreference(PREFERENCE_ID_USER);
		var password = this.getPreference(PREFERENCE_ID_PASSWORD);
		FeedClient.getFeed(url, user, password, entry.items[0].linkURL, null, null, null, true, true, false, srh, this.getServiceTracker());
	},
	
	/*
	 * Returns info for the current feed, or null if there is none or we don't
	 * have it yet.
	 */
	/* FeedInfoDTO */ getFeedInfo: function() {
		return this._feedInfo;
	},
	
	/*
	 * Returns the number of items to show per page. Called by superclass, and to
	 * be overriden by subclasses.
	 */
	/* number */ getItemsPerPage: function() {
		return this.getPreference(PREFERENCE_ITEMS_TO_SHOW);
	},
	
	/*
	 * Returns the message to display when there are no feed items to show.
	 */
	/* string */ getNoItemsMessage: function() {
		return messages.noItems;
	},
	
	/*
	 * Returns the message to display when there is no data available,
	 * including error messages.
	 */
	/* Node */ getNoResultsMessage: function() {
		if (this._errorMsgNode) {
			return this._errorMsgNode;
		}
		return document.createTextNode(this.getNoItemsMessage());
	},
	
	/*
	 * Returns the id of the latest reference feed item. The reference item is the
	 * first item of a feed at a given time (index 0).
	 */
	/* string */ getReferenceItemId: function() {
		if (this._trackReadMarks) {
			var readState = this.getContributorState().get("read");
			if (readState && readState[0]) {
				return readState[0];
			}
		}
	},
	
	/*
	 * Returns the total number of results in the feed; called by superclass.
	 */
	/* number */ getResultCount: function() {
		return this._entryCount;
	},
	
	/*
	 * Returns an appropriate URL for the viewlet's title link.
	 */
	/* string */ getTitleHyperlinkURL: function() {
		return this._feedInfo ? this._feedInfo.linkURL : "#";
	},
	
	/*
	 * Handles a feed request's response. Subclasses should use this handler
	 * for feed responses.
	 */
	handleFetch: function(/* FeedResponseDTO */ response, /* boolean */ entries, /* boolean */ details, /* ServiceResponseHandler */ srh) {
		// was there an error?
		if (response.errorMsg) {
			// show multi-line plain text error msg
			this._errorMsgNode = document.createElement("span");
			this._errorMsgNode.className = "errorText";
			this._errorMsgNode.innerHTML = HTML.escape(response.errorMsg).replace(/\n/g, "<br/>");
			this._entryCount = 0;
			this._updateSecondaryTitle();
			srh.doCallback();
		}
		else {
			// if response had feed info, store it
			if (response.info) {
				this.setFeedInfo(response.info);
			}
			
			// update result count
			this._entryCount = response.entryCount;
			this._updateSecondaryTitle();

			// update read marks
			if (this._trackReadMarks) {
				this._loadReadMarks(response);
			}
			
			// package results
			if (response.entryCount > 0) {
				var result = new Object();
				if (entries) {
					result.entries = response.entries;
					if (this._hasIcons(result.entries)) {
						this._columnCount = 2;
					}
				}
				if (details) {
					result.details = response.entries;
				}
				srh.doCallback(result);
			}
			else {
				// superclass will show no entries msg
				srh.doCallback();
			}
		}
	},
	
	/*
	 * Called by platform to see if viewlet has any zoomable content.
	 */
	/* boolean */ hasZoomableContent: function() {
		return this._entryCount > 0;
	},
	
	/*
	 * Renders the details for the given entry.
	 */
	/* Node[] */ renderDetails: function(/* EntryDTO */ entry) {
		var columns = new Array();
		var td = document.createElement("td");
		td.className = "details";
		if (this._columnCount > 1) {
			columns.push(document.createElement("td"));
			td.colSpan = this._columnCount - 1;
		}
		var buf = [];
		var items = entry.items;
		for (var i=0;i<items.length;i++) {
			var item = items[i];
			buf.push("<div class=\"item\">");
			if (item.date) {
				buf.push("<div class=\"date\">" + DateTime.formatDif(new Date(item.date), true, true) + "</div>");
			}
			if (item.description) {
				buf.push(HTML.removeScript(item.description));
			}
			else {
				buf.push("<span class=\"description-empty\">(" + messages.noDesc + ")</span>");
			}
			buf.push("</div>");
		}
		td.innerHTML = buf.join("");
		
		// set style if read
		if (this._trackReadMarks) {
			dojo.addClass(td, this._isRead(entry) ? "read-details" : "unread-details");
		}
		
		// mark read immediately after an item is zoomed
		this._markRead(entry, false);
		columns.push(td);
		return columns;
	},
	
	/*
	 * Renders an entry for the given entry.
	 */
	/* Node[] */ renderEntry: function(/* EntryDTO */ entry, /* EntryDTO */ entryDetails, /* boolean */ zoomed) {
		var columns = new Array();
		var firstItem = entry.items[0];
		
		// render icon if exists
		if (firstItem.iconURL) {
			var td = document.createElement("td");
			td.className = "iconCell";
			var div = document.createElement("div");
			div.className = "iconContainer";
			
			var iconImg = document.createElement("img");
			iconImg.className = "icon";
			iconImg.src = firstItem.iconURL;
			div.appendChild(iconImg);
			
			if (firstItem.overlay) {
				var overlayImg = document.createElement("img");
				overlayImg.className = "overlay";
				var overlay = firstItem.overlay;
				if (overlay.indexOf("../") == 0) {
					// backward compatibility - prior to 1.0.0.1, overlay format was:
					// "../<web_bundle>/<path>"
					overlayImg.src = overlay;
				}
				else {
					// new format - since 1.0.0.1
					// "<web_bundle>/<path>"
					var index = overlay.indexOf("/");
					var bundle = overlay.substring(0, index);
					var path = overlay.substring(index + 1); 
					overlayImg.src = dojo.moduleUrl(bundle, path);
				}
				
				if (firstItem.overlayClass) {
					dojo.addClass(overlayImg, firstItem.overlayClass);
				}
				div.appendChild(overlayImg);
			}
			
			td.appendChild(div);
			columns.push(td);
		}
		
		// render count for multi-item entries
		var td = document.createElement("td");
		var count = entry.items.length;
		if (count > 1) {
			var countSpan = document.createElement("span");
			countSpan.className = "count";
			countSpan.innerHTML = "[" + count + "] ";
			td.appendChild(countSpan);
		}
		
		// render link
		var a = document.createElement("a");
		if (!firstItem.disableLink)
			a.href = firstItem.linkURL;
		else
			a.href = "#";
		a.innerHTML = FormatUtil.insertWordBreaks(HTML.removeScript(firstItem.title), 20);
		if (this.getPreference("newWindow")) {
			a.target="_blank";
		}
		
		var widget = this.createTooltip(a, entry);
		if (widget)
			this._hoverPopups.push(widget);
		
		a.style.zoom = "normal"; // Bug #44703
		td.appendChild(a);
		columns.push(td);
		
		if (firstItem.disableLink) {
			this.connect(a, "onclick", function(event) {
				event.preventDefault();
				widget.open();
			});
		}
		
		// set style if read
		if (this._trackReadMarks) {
			var className = this._isRead(entry) ? "read-entry" : "unread-entry";
			if (firstItem.critical) {
				className = "critical-" + className;
			}
			for (var i=0;i<columns.length;i++) {
				dojo.addClass(columns[i], className);
			}
		}

		// render date
		if (!zoomed && firstItem.date) {
			var separatorSpan = document.createElement("span");
			separatorSpan.innerHTML = "&nbsp ";
			td.appendChild(separatorSpan);
			var dateSpan = document.createElement("span");
			dateSpan.className = "date";
			dateSpan.innerHTML = DateTime.formatDif(new Date(firstItem.date), false, true);
			td.appendChild(dateSpan);
		}
		
		entry.columns = columns;
		return columns;
	},

	/*
	 * Set number of entries back to initial state on each refresh.
	 */
	refresh: function() {
		this._entryCount = -1;
		this.inherited(arguments);
		var page = this.getContributorState().get("page");
		if (page) {
			// value is of the form "<timeStamp>|<pageNumber>"
			page = page.split("|");
			// only honour the state if it's less than 60 minutes old
			if (parseInt(page[0],10) + 3600000 > new Date().getTime()) {
				this.setCurrentPage(parseInt(page[1],10))
			}
		}		
	},
	
	onPageChange: function(newPage) {
		var page = new Date().getTime() + "|" + newPage;
		this.getContributorState().set("page", page);
	},

	/*
	 * Sets the number of columns this viewlet had.
	 */
	setColumnCount: function(/* number */ columnCount) {
		this._columnCount = columnCount;
	},
	
	/*
	 * Called when the dashboard switches to or from edit mode.
	 */
	setDashboardEditing: function(editing) {
		// if the extra no feed text is visible or cached
		if (this._noFeedExtraMsg) {
			// show it only if dashboard is editing
			this._noFeedExtraMsg.style.display = editing ? "inline" : "none";
		}
	},
	
	/*
	 * Sets the info about the current feed; to be called by subclasses when
	 * they get the info.
	 */
	setFeedInfo: function(/* FeedInfoDTO */ feedInfo) {
		this._feedInfo = feedInfo;
	},
	
	/*
	 * Sets whether or not the viewlet should track read/unread marks. Must
	 * be called during initialization.
	 */
	setTrackReadMarks: function(/* boolean */ value) {
		this._trackReadMarks = value;
	},
	
	/*
	 * Refresh or update depending on which settings changed; called by platform.
	 */
	settingsChanged: function(oldSettings, newSettings) {
		if (newSettings.preferences.url != oldSettings.preferences.url
			|| newSettings.preferences.user != oldSettings.preferences.user
			|| newSettings.preferences.password != oldSettings.preferences.password) {
			this.reset(true);
		} else if (newSettings.preferences.items != oldSettings.preferences.items) {
			this.reset(false);
		} else {
			this.update();
		}
	},
	
	/*
	 * Update secondary title on update.
	 */
	update: function() {
		this.inherited(arguments);
		this._updateSecondaryTitle();
	},
	
	/*
	 * Returns whether or not the given entries have icons.
	 */
	/* boolean */ _hasIcons: function(/* EntryDTO[] */ entries) {
		if (entries) {
			for (var i=0;i<entries.length;i++) {
				var items = entries[i].items;
				for (var j=0;j<items.length;j++) {
					if (items[j].iconURL) {
						return true;
					}
				}
			}
		}
		return false;
	},
	
	/*
	 * Returns whether or not the given entry is read.
	 */
	_isRead: function(/* EntryDTO */ entry) {
		var items = entry.items;
		for (var i=0;i<items.length;i++) {
			if (!this._readMarks[items[i].index]) {
				return false;
			}
		}
		return true;
	},
	
	/*
	 * Marks all items in this feed as either read or unread.
	 */
	_markAll: function(/* boolean */ read) {
		if (this._trackReadMarks) {
			for (var i=0;i<this._readMarks.length;i++) {
				this._readMarks[i] = read;
			}
			this._unreadCount = read ? 0 : this._readMarks.length;
			this.update();
			this._updateSecondaryTitle();
			this._saveReadMarks();
		}
	},

	/*
	 * Marks the given entry as read, updates styles, and saves contributor
	 * memento.
	 * 
	 * @param entry the entry to mark as read
	 * @param immediate whether or not to update the UI immediately
	 */
	_markRead: function(/* EntryDTO */ entry, /* boolean */ immediate) {
		if (this._trackReadMarks) {
			var items = entry.items;
			for (var i=0;i<items.length;i++) {
				var item = items[i];
				if (!this._readMarks[item.index]) {
					this._readMarks[item.index] = true;
					this._unreadCount--;
				}
			}
			if (immediate) {
				var isCritical = entry.items.length > 0 ? entry.items[0].critical : false;
				for (var i=0;i<entry.columns.length;i++) {
					var td = entry.columns[i];
					dojo.removeClass(td, "unread-entry");
					dojo.removeClass(td, "critical-unread-entry");
					dojo.addClass(td, isCritical ? "critical-read-entry" : "read-entry");
				}
				this._updateSecondaryTitle();
			}
			this._saveReadMarks();
		}
	},
	
	/*
	 * Called when a hover tooltip is created for the first time.
	 */
	_createTooltip: function(/* EntryDTO */ entry, /* HoverPopup */ hoverPopup) {
		var feedTooltip = new com.ibm.team.dashboard.viewlets.web.ui.internal.FeedTooltip({entry: entry, feedViewlet: this, hoverPopup: hoverPopup});
		this._feedTooltips.push(feedTooltip);
		return feedTooltip.domNode;
	},
	
	/*
	 * Called when the hover tooltip has been opened.
	 */
	_onTooltipOpen: function(/* EntryDTO */ entry, /* HoverPopup */ hoverPopup, /* Date */ lastOpened) {
		// stop refreshing as long as the hover is visible
		this.getSite().setAutoRefresh(false);
		
		// don't show tooltips when zoomed (info already visible)
		if (!this.isZoomed()) {
			if (this._trackReadMarks) {
				this._markRead(entry, true);
			}
		}
	},
	
	/*
	 * Called when the hover tooltip has been closed.
	 */
	_onTooltipClose: function() {
		// ok safe to refresh again
		this.getSite().setAutoRefresh(true);
	},
	
	/*
	 * Updates the viewlet's secondary title based on the number of total
	 * items and unread items.
	 */
	_updateSecondaryTitle: function() {
		this.updateTotalPageFooter();
		var heading = "";
		if (this._trackReadMarks && this._unreadCount > 0) {
			heading = "(" + dojo.string.substitute(messages.titleNew, [ this._unreadCount ]) + ")";
		}
		this.getSite().setSecondaryTitle(heading);
	},
	
	/*
	 * Loads or updates the boolean array of read/unread marks for the items
	 * in this viewlet's feed.
	 */
	_loadReadMarks: function (/* FeedResponseDTO */ response) {
		if (this._trackReadMarks) {
			var state = this.getContributorState().get("read");
			this._unreadCount = 0;
			this._referenceItemId = response.ref;
			this._readMarks = new Array(response.itemCount);
			if ((!response.refIndex && response.refIndex != 0) || response.refIndex == -1 || !state) {
				for (var i = 0; i < this._readMarks.length; i++) {
					this._readMarks[i] = false;
					this._unreadCount++;
				}
			}
			else {
				var prevIndexes = this._unserialize(state[1]);
				for (var i = 0; i < response.refIndex; i++) {
					this._readMarks[i] = false;
					this._unreadCount++;
				}
				for (var i = response.refIndex; i < this._readMarks.length; i++) {
					if (prevIndexes[i - response.refIndex]) {
						this._readMarks[i] = true;
					}
					else {
						this._readMarks[i] = false;
						this._unreadCount++;
					}
				}
			}
			this._updateSecondaryTitle();
		}
	},
	
	/*
	 * Saves the read/unread marks in the contributor memento.
	 */
	_saveReadMarks: function() {
		if (this._trackReadMarks) {
			var newState = [this._referenceItemId, this._serialize(this._readMarks)];
			this.getContributorState().set("read", newState);
			// store version for future migration needs
			if (this.getContributorState().get("version") != 1) {
				this.getContributorState().set("version", 1);
			}
		}
	},
	
	/*
	 * Serializes an array of booleans into a compact form containing the number
	 * of elements and a series of indexes where the read/unread state changes, starting
	 * from false (delta modulation).
	 * 
	 * This is effective because feed items typically come in bunches of read and unread
	 * items. Typically it's a bunch of newer unread item followed by a bunch of older read
	 * ones.
	 * 
	 * e.g.
	 * [true, true, false, false] -> [4, 2]
	 * [false, false, false] -> [3]
	 * [true, true, true, true, true] -> [5, 0]
	 * [true, false, true, false] -> [4, 1, 2, 3, 4]
	 */
	/* number[] */ _serialize: function(/* boolean[] */ array) {
		var serialized = [array.length];
		var currentState = false;
		for (var i=0;i<array.length;i++) {
			if (array[i] != currentState) {
				serialized.push(i);
				currentState = array[i];
			}
		}
		return serialized;
	},
	
	/*
	 * Unserializes a previously serialized array of booleans (see this._serialize()).
	 */
	/* boolean[] */ _unserialize: function(/* number[] */ serialized) {
		var unserialized = new Array(serialized[0]);
		var currentIndex = 1;
		var currentState = false;
		for (var i=0;i<unserialized.length;i++) {
			if (currentIndex < serialized.length && i == serialized[currentIndex]) {
				currentState = !currentState;
				currentIndex++;
			}
			unserialized[i] = currentState;
		}
		return unserialized;
	}
});

/*
 * The feed tooltip that appears when you hover on a news item.
 */
dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.FeedTooltip",
	[dijit._Widget, dijit._Templated], {
	
	templateString: "<div class=\"com-ibm-team-dashboard-viewlets-web-ui-internal-FeedTooltip\"><span class=\"loading\">" + messages.loading + "</span></div>",
	
	postCreate: function() {
		// fetch then display
		var srh = new ServiceResponseHandler(this, "_handleFetchSuccess", "_handleFetchError");
		this.feedViewlet.fetchTooltip(this.entry, srh);
	},
	
	_handleFetchSuccess: function(/* ResponseDTO */ response) {
		if (response.errorMsg) {
			this.domNode.innerHTML = "<span>" + messages.errorTooltip + ":<br/>" + response.errorMsg + "</span>";
		}
		else {
			this._currentPage = 0;
			// Create and populate the main content div
			var mainDiv = document.createElement("div");
			this.domNode.replaceChild(mainDiv, this.domNode.firstChild);
			this._createContents(response.entries, mainDiv);
			// we have changed the dom, trigger the popup positioning logic
			this.hoverPopup.position();
		}
	},
	
	_handleFetchError: function(/* Error */ error) {
		this.domNode.innerHTML = "<span>" + messages.errorTooltip + ":<br/>" + error + "</span>";
	},
	
	/* Node */ _createContents: function(/* EntryDTO[] */ entries, /* Node */ mainDiv) {
		if (entries && entries.length == 1) {
			var entry = entries[0];
			var items = entry.items;
			var num = items.length;
			
			// create header (page controls)
			this.headerDiv = document.createElement("div");
			this.headerDiv.className = "header-" + (num > 1 ? "multi" : "single");
			
			this._prevA = document.createElement("a");
			this._prevA.className = "prev prev-disabled";
			this._prevA.href = "#";
			this._prevA.title = messages.prevPage;
			this.connect(this._prevA, "onclick", dojo.hitch(this, this._handlePrevPage));
			this.connect(this._prevA, "ondblclick", dojo.hitch(this, this._handlePrevPage));
			this.headerDiv.appendChild(this._prevA);

			this._pagenumDiv = document.createElement("div");
			this._pagenumDiv.className = "pagenum";
			this._pagenumDiv.innerHTML = "1/" + items.length;
			this.headerDiv.appendChild(this._pagenumDiv);
			
			this._nextA = document.createElement("a");
			this._nextA.className = "next" + (num == 1 ? " next-disabled" : "");
			this._nextA.href = "#";
			this._nextA.title = messages.nextPage;
			this.connect(this._nextA, "onclick", dojo.hitch(this, this._handleNextPage));
			this.connect(this._nextA, "ondblclick", dojo.hitch(this, this._handleNextPage));
			this.headerDiv.appendChild(this._nextA);
			mainDiv.appendChild(this.headerDiv);
			
			// create body
			this._bodyDiv = document.createElement("div");
			this._bodyDiv.className = "body-" + (num > 1 ? "multi" : "single");
			this._pages = this._createTooltipPages(items);
			mainDiv.appendChild(this._bodyDiv);

			this._maxHeightVal = -1;
			this._maxWidthVal = -1;

			dojo.forEach(this._pages, dojo.hitch(this, 
				function(entry, i) {
					// We need to prevent the hover popup from jumping around as the user switches
					// through the pages. The size of each page is needed so that we can assign
					// the content size div to be the same for all pages. 
					//  
					// Here we render the page with opacity 0 (transparent) so the user 
					// doesn't see the page, and then get it's size. 
					dojo.addClass(entry, "transparent");
					if (i == 0) {
						this._bodyDiv.appendChild(entry);
					} else {
						var oldChild = this._bodyDiv.firstChild;
						this._bodyDiv.replaceChild(entry, this._bodyDiv.firstChild);
						dojo.removeClass(oldChild, "transparent");
					}
					
					// Check if a scroll bar will be needed for this page. If the height
					// of the page is larger than the max height of the hoverpop, then we know
					// a horizontal scrollbar will be required. Reduce the width of the content
					// div (not the parent container) so that the scroll bar does not cover any
					// content
					if (this._pages.length > 1 
						&& entry.offsetHeight > this.hoverPopup.maxHeight) {
						dojo.style(entry, "width", "96%");
					}
					
					// Get the size of the current entry.  
					this._maxHeightVal = entry.offsetHeight > this._maxHeightVal ? entry.offsetHeight : this._maxHeightVal;
					this._maxWidthVal = entry.offsetWidth > this._maxWidthVal ? entry.offsetWidth : this._maxWidthVal;
				}
			));
			dojo.removeClass(this._bodyDiv.firstChild, "transparent");
					
			// Adjust the width and height of the parent container div so that it is consistant for
			// all pages. Size does not need to be adjusted if the hover contains only 1 page 
			// because the hover will be sized correctly for that page. 
			if (this._pages.length > 1) {
				var pageHeight = this._maxHeightVal + this.headerDiv.offsetHeight + 5;
				var firstHeight = Math.min(pageHeight, this.hoverPopup.maxHeight);
				dojo.style(this._bodyDiv.parentNode, "height", firstHeight + "px");

				var firstWidth = Math.min(this._maxWidthVal, this.hoverPopup.maxWidth);
				dojo.style(this._bodyDiv.parentNode, "width", firstWidth + "px");
			}
			this._bodyDiv.replaceChild(this._pages[0], this._bodyDiv.firstChild);
		}
		else {
			mainDiv.innerHTML = "<span class=\"description-empty\">(" + messages.noDesc + ")</span>";
		}
		return mainDiv;
	},
	
	/* Node[] */ _createTooltipPages: function(/* ItemDTO[] */ items) {
		var pages = [];
		for (var i=0;i<items.length;i++) {
			var page = this._createTooltipPage(items[i]);
			pages.push(page);
		}
		return pages;
	},
	
	/* Node */ _createTooltipPage: function(/* ItemDTO */ item) {
		var pageDiv = document.createElement("div");
		var buf = [];
		buf.push("<div class=\"title\">");
		if (!item.disableLink) {
			buf.push("<a href=\"" + item.linkURL + "\">");
		}
		buf.push(HTML.removeScript(item.title));
		if (!item.disableLink) {
			buf.push("</a>");
		}
		buf.push("</div>");
		buf.push("<div class=\"page-content\">");		
		if (item.date) {
			buf.push("<div class=\"date\">" + dojo.string.substitute(messages.onDate, [ DateTime.formatDif(new Date(item.date), true, true) ]) + ":</div>");
		}
		if (item.description) {
			buf.push(HTML.removeScript(item.description));
		}
		else {
			buf.push("<span class=\"description-empty\">(" + messages.noDesc + ")</span>");
		}
		buf.push("</div>");		
		pageDiv.innerHTML = buf.join("");
		return pageDiv;
	},
	
	_handlePrevPage: function(/* Event */ event) {
		dojo.stopEvent(event);
		if (this._currentPage > 0) {
			this._showPage(--this._currentPage);
		}
	},
	
	_handleNextPage: function(/* Event */ event) {
		dojo.stopEvent(event);
		if (this._currentPage < (this._pages.length - 1)) {
			this._showPage(++this._currentPage);
		}
	},
	
	_showPage: function(/* number */ pageNum) {
		// swap in new page div
		this._bodyDiv.replaceChild(this._pages[pageNum], this._bodyDiv.firstChild);

		if (this._pages[pageNum].offsetHeight <= this.hoverPopup.maxHeight ) {
			var pageHeight = this._maxHeightVal + this.headerDiv.offsetHeight + 5;
			var height = Math.min(pageHeight, this.hoverPopup.maxHeight);
			dojo.style(this._bodyDiv.parentNode, "height", height + "px");				
		} else {
			var pageHeight = this._pages[pageNum].offsetHeight + this.headerDiv.offsetHeight + 5;
			dojo.style(this._bodyDiv.parentNode, "height", pageHeight + "px");
		}
		
		// update page numbers
		this._pagenumDiv.innerHTML = (pageNum + 1) + "/" + this._pages.length;
		
		// update page button enablement
		var func = (pageNum == 0) ? dojo.addClass : dojo.removeClass;
		func(this._prevA, "prev-disabled");
		func = (pageNum >= (this._pages.length - 1)) ? dojo.addClass : dojo.removeClass;
		func(this._nextA, "next-disabled");
	}
});

})();
