/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.GadgetAdapter");

dojo.require("dijit._Templated");

dojo.require("jazz.ensemble.Widget");
dojo.require("jazz.opensocial.Gadget");
dojo.require("jazz.ensemble.internal.settings.SettingsPanel");

(function() {

var SettingsPanel = jazz.ensemble.internal.settings.SettingsPanel;

var PREFERENCE_TYPE_MAP = {
	"bool": SettingsPanel.TYPES.BOOLEAN,
	"string": SettingsPanel.TYPES.STRING,
	"enum": SettingsPanel.TYPES.LIST,
	"list": SettingsPanel.TYPES.STRING
};

dojo.declare("com.ibm.team.dashboard.web.ui.internal.GadgetAdapter", [jazz.ensemble.Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/GadgetAdapter.html"),
	
	postCreate: function() {
		this.gadget = new jazz.opensocial.Gadget({
			url: this.definition,
			prefs: this.context.getPreferences(),
			onTitle: dojo.hitch(this.context,this.context.setTitle),
			onGadgetMetadata: dojo.hitch(this,this.onGadgetMetadata),
			moduleId: this.uuid
		});
		this.connect(this.gadget, "onGadgetPrefs", dojo.hitch(this, this._onGadgetPrefs));
		this._gadgetContent.appendChild(this.gadget.domNode);
	},
	
	getDefaultTitle: function() {
		if (this.metadata)
			return this.metadata.getTitle();
		return this.inherited(arguments);
	},
	
	refresh: function() {
		this.gadget._renderCurrentView();
	},
	
	onGadgetMetadata: function(metadata) {
		this.metadata = metadata;
		var views = metadata.getViews();
		if (views) {
			for (var i=0;i<views.length;i++) {
				if (views[i].name == "canvas") {
					this.context.setMaximizeHandler(dojo.hitch(this, this._handleMaximize));
					break;
				}
			}
		}
		var ups = metadata.getUserPrefs();
		for (var i = 0; i < ups.length; i++) {
			if (PREFERENCE_TYPE_MAP[ups[i].datatype]) {
				this.context.setSettingsHandler(dojo.hitch(this,this._handleSettings));
				break;
			}
		}
	},
	
	_handleMaximize: function(/* boolean */ maximize) {
		if (maximize) {
			this._originalView = this.gadget._view;
		}
		this.gadget.navigateTo(maximize ? "canvas" : this._originalView);
	},
	_handleSettings: function() {
		// if not already open
		if (!this._settingsPanel) {
			this._openSettings();
		}
	},
	_handleSettingsOk: function() {
		var settings = this._settingsPanel.settings, oldSettings = this._settingsPanel.oldSettings;
		var changes = {}, hasChanges = false;
		for (var i = 0; i < settings.length; i++) {
			var setting = settings[i];
			changes[setting.id] = setting.value;
			if (setting.value !== oldSettings[i].value) {
				hasChanges = true;
			}
		}
		if (hasChanges) {
			// preserve preferences not in the settings area (hidden prefs)
			var newPrefs = dojo.mixin(this.context.getPreferences(),changes);
			this.context.setPreferences(newPrefs);
			// TODO: Using an internal to update the prefs and then an internal to refresh, need a better way
			this.gadget._prefs = newPrefs;
			this.refresh();
		}
		this._handleSettingsCancel();
	},
	_handleSettingsCancel: function() {
		this._settingsPanel.close(dojo.hitch(this, this._handleSettingsClose));
	},
	_handleSettingsClose: function() {
		this._settingsContainer.removeChild(this._settingsPanel.domNode);
		this._settingsPanel.destroyRecursive();
		delete this._settingsPanel;
	},
	_onGadgetPrefs: function(/* object */ delta) {
		var prefs = this.context.getPreferences();
		dojo.mixin(prefs, delta);
		this.context.setPreferences(prefs);
	},
	_openSettings: function() {
		var prefs = this.context.getPreferences();
		var ups = this.metadata.getUserPrefs();
		var settings = [];
		for (var i = 0; i < ups.length; i++) {
			var up = ups[i];
			var type = PREFERENCE_TYPE_MAP[up.datatype];
			if (!type)
				continue;
			var value = prefs[up.name];
			var setting = {
				id: up.name,
				label: up.displayName,
				required: up.required,
				type: type,
				value: typeof value === "undefined" || value === null ? up.defaultValue : value
			};
			if (type === SettingsPanel.TYPES.LIST) {
				var options = [];
				var enums = up.enumValues;
				for (var j = 0; j < enums.length; j++) {
					options.push({
						label: enums[j].displayValue,
						value: enums[j].value
					});
				}
				setting.options = options;
			}
			settings.push(setting);
		}
		this._settingsPanel = new SettingsPanel({
			onOk: dojo.hitch(this,this._handleSettingsOk),
			onCancel: dojo.hitch(this, this._handleSettingsCancel),
			settings: settings,
			readOnly: this.readOnly
		});
		this._settingsContainer.appendChild(this._settingsPanel.domNode);
		this._settingsPanel.open();
	}
	
});

})();
