/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.GadgetViewlet");

dojo.require("dojo.i18n"); 
dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");

dojo.require("jazz.app.proxy");
dojo.require("jazz.opensocial.Gadget");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

var Util = com.ibm.team.dashboard.web.util.internal.Util;
var Viewlet = com.ibm.team.dashboard.web.ui.Viewlet;
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;

var messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

/*
 * The OpenSocial Gadget adapter viewlet class.
 * 
 * @since 0.7
 */

dojo.declare("com.ibm.team.dashboard.web.ui.internal.GadgetViewlet",
	Viewlet, {
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/GadgetViewlet.html"),

	_normalView: "default",
	
	init: function(){
		if (this._definition.error) {
			throw new Error(this._definition.error);
		}
		var v;
		if (this._definition.views) {
			this._viewInfo = {};
			for (var i = 0; i < this._definition.views.length; i++) {
				// TODO: Perhaps we should do this on the server, not here
				v = this._definition.views[i]
				if (v.name == "home") {
					this._normalView = "home";
				}
				this._viewInfo[v.name] = v;
			}
		}
		this._addSourceMenuItem();
	},
	
	update: function() {
		var z = this.isZoomed();
		if (this._gadget && z !== this._wasZoomed) {
			this._wasZoomed = z;
			this._gadget.navigateTo(z?"canvas":this._normalView);
		}
	},

	refresh: function() {
		if (!this._gadget) {
			this._gadget = new jazz.opensocial.Gadget({
				url: this._definition.url,
				prefs: this._getUserPrefsParams(true),
				onTitle: dojo.hitch(this,this._CallbackSetTitle),
				view: this._normalView,
//				_renderUrl: true, // hack to skip the metadata request
				_initialTitle: this._definition.name,
				_scrolling: this._definition.scrollable,
				_viewInfo: this._viewInfo
			}, this._content);
			this.connect(this._gadget, "onGadgetPrefs", dojo.hitch(this.getSite(),this.getSite()._setPreferences));
		} else {
			// TODO: need refresh API
			this._gadget._renderCurrentView();
		}
		this._wasZoomed = this.isZoomed();
	},
	
	getTitleHyperlinkURL: function() {
		return this._definition.titleUrl;
	},	
	
	_CallbackSetTitle: function(title) {
		// don't persist titles for OpenSocial gadgets
		// gadgets with custom titles are responsible for storing them and setting on each load
		this.getSite().setTitle(title, false);
	},

	_addSourceMenuItem: function() {
		this.getSite().addMenuItem(messages.viewGadgetSource, null, dojo.hitch(this, function() {
			window.open(this._definition.url);
		}));
	},

	_getGadgetServerBase: function() {
		// TODO: is it correct to assume data server is gadget server (could be on web ui server)?
		var root = net.jazz.ajax._dataServerRoot;
		if (!root) {
			var url = new dojo._Url(location.href);
			root = url.scheme + "://" + url.host + (url.port ? ":" + url.port : "");
		}
		return root + "/gadgets/";
	},
	
	_getParentUrl: function() {
		return document.location.protocol + '//' + document.location.host;
	},	

	// TODO: Remove this
	// orphaned, this is done by Gadget.js now
	_getIframeUrl: function() {
  		return this._getGadgetServerBase() + 'ifr?' + 
		'container='+"default" +
		'&lang=' + "default" +
		'&country=' + "default"+
		'&parent=' + encodeURIComponent(this._getParentUrl()) +
		'&view=' + this._getCurrentView() +
		'&mid=' + "1" + 
		this._getUserPrefsParams() + 
		// TODO: Should we be generating the secure token or should the framework?
		'&st=' + this._getSecureToken() +
		'&url='+encodeURIComponent(this._definition.url);
		/*
      'container=' + this._CONTAINER +
      '&mid=' +  this.id +
      '&nocache=' + gadgets.container.nocache_ +
      '&country=' + gadgets.container.country_ +
      '&lang=' + gadgets.container.language_ +
      '&view=' + gadgets.container.view_ +
      (this.specVersion ? '&v=' + this.specVersion : '') +
      (gadgets.container.parentUrl_ ? '&parent=' + encodeURIComponent(gadgets.container.parentUrl_) : '') +
      (this.debug ? '&debug=1' : '') +
      this.getAdditionalParams() +
      '&url=' + encodeURIComponent(this.specUrl) +
      '#rpctoken=' + this.rpcToken +
      (this.viewParams ?
          '&view-params=' +  encodeURIComponent(gadgets.json.stringify(this.viewParams)) : '') +
      (this.hashData ? '&' + this.hashData : '');
      */
	},
	settingsChanged: function(oldSettings, newSettings) {
		// TODO: Using an internal to update the prefs and then an internal to refresh, need a better way
		this._gadget._prefs = this._getUserPrefsParams(true);
		this.getSite().refresh();
		// TODO: Is there a way to change prefs live rather than doing a full refresh of the iframe?
//		var newPrefs = newSettings.preferences;
//		var oldPrefs = oldSettings.preferences;
//		var args = [];
//		for (var id in newPrefs) {
//			if (oldPrefs[id] !== newPrefs[id]) {
//				args.push(id);
//				args.push(newPrefs[id]);
//			}
//		}
//		if (args.length > 0) {
//			if (!this._prefs)
//				this._prefs = new gadgets.Prefs();
//			this._prefs.set.apply(this._prefs,args);
//		}
	},
	
	_getSecureToken: function() {
		if (!this._secureToken) {
			var pieces = ['john.doe', 'john.doe', '0', 'shindig', escape(escape(this._definition.url)), '0', '0'];
			this._secureToken = pieces.join(':');
		}
		return this._secureToken;
	},

	// TODO: Remove this
	// orphaned, this is handled by calling Gadget.js API in the update method
	_getCurrentView: function() {
		return this.isZoomed()?"canvas":this._normalView;
	},
	
	_getUserPrefsParams: function(/* boolean */ asObject) {
		var result = asObject ? {} : [];
		var pdefs = this._definition.preferenceDefinitions;
	
		if (pdefs) {
			for (var i = 0; i < pdefs.length; i++) {
				var pdef = pdefs[i];
				
				var value = this._getNondefaultPreference(pdef);
				if (value !== null) {
					if (asObject)
						result[pdef.id] = value;
					else
						result.push('&up_' + encodeURIComponent(pdef.id) + '=' + encodeURIComponent(value));
				}
			}
		}
		return asObject ? result : result.join("");
	},
	
	_getNondefaultPreference: function(prefDef) {
		var prefs = this._helper.preferences, value;
		if (prefs) {
			for (var j=0;j<prefs.length;j++) {
				if (prefs[j].id == prefDef.id) {
					value = prefs[j].value;
					break;
				}
			}
		}
		if (typeof value !== "undefined") {
				return Util._cast(prefDef.preferenceType, value);
		}
		return null;
	}
});

})();