/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2011, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.HeadlinesViewlet");

/*
 * Imports 
 */
dojo.require("com.ibm.team.dashboard.viewlets.web.client.internal.HeadlinesClient");
dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.URLUtility");
dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");
dojo.require("jazz.util.ResizeNotifier");
dojo.require("jazz.util.i18n");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("dijit.Editor");
dojo.require("dijit.form.Button");
dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("jazz.util.html");

// Editor Plug-ins
dojo.require("dijit._editor.plugins.TextColor");
dojo.require("dijit._editor.plugins.LinkDialog");
dojo.require("dijit._editor.plugins.ViewSource");
dojo.require("dijit._editor.plugins.FullScreen");
dojo.require("dojox.editor.plugins.PrettyPrint");

dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "HeadlinesMessages");

(function() {
	
/*
 * Aliases 
 */
var HeadlinesClient = com.ibm.team.dashboard.viewlets.web.client.internal.HeadlinesClient;
var ResizeNotifier = jazz.util.ResizeNotifier;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var URLUtility = com.ibm.team.dashboard.viewlets.web.ui.internal.utils.URLUtility;
var Viewlet = com.ibm.team.dashboard.web.ui.Viewlet;
var I18nUtil = jazz.util.i18n;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var Editor = dijit.Editor;

var SERVICE_IMG_ID = "com.ibm.team.dashboard.common.internal.service.IDashboardImageService";
var RICH_TEXT_EDITOR_ID = "hv_rte_";
var IMAGE_POSITION_LEFT = "display-image-left";
var IMAGE_POSITION_RIGHT = "display-image-right";

/*
 * Class 
 */
dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.HeadlinesViewlet",
	Viewlet, {
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web", "ui/internal/templates/HeadlinesViewlet.html"),
	
	_STATE_PAGES: "pages",
	
	constructor: function(args) {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "HeadlinesMessages");
	},

	init: function() {
		this._editContentsLink = I18nUtil.addMessage(this._editModeMessage, this.messages.noContentEdit)[0];
		
		// Instance Variables //--------------------------------------------------->
		
		this._colors = ["blue","gray","gold","green","teal","purple","orange"];
		this._editor = null;
		this._predefinedImages = null;
		
		this._currentPageIndex = 0;
		
		this._loadDisplayNodes();
		this._loadDisplayArea();
		
		this._pageData = [];
		
		this.connect(this._editContentsLink,"onclick", this._doEdit);
		this.connect(this._shiftLeft, "onclick", this._doShiftLeft);
		this.connect(this._shiftRight, "onclick", this._doShiftRight);
		
		this._addHandlersTitle();
		this._addHandlersImageUrl();
		this._addHandlersImagePosition();
		
		// Resize events
		this._dashboardResizeEvent = ResizeNotifier.register({context: this, callback: this._updateHeight});
	},	

	_doPollingLoop: function(iterations) {
		if (this._pollingTimeout) {
			clearTimeout(this._pollingTimeout);
			delete this._pollingTimeout;
		}
		if (this._active && iterations <= 10) {
			iterations && this._updateHeight();
			this._pollingTimeout = setTimeout(dojo.hitch(this, this._doPollingLoop, iterations++), 1500);
		}
	},

	_addHandlersTitle: function() {
		this.connect(this._title, "onchange", function(event) {
			if (event) {
				dojo.stopEvent(event);
			}
			
			this._updateTitleData(this._title.value);
			
			return true;
		});
	},

	_addHandlersImagePosition: function() {
		this.connect(this._imagePosition, "onchange", function(event) {
			
			if (!event) {
				return false;
			} else if (!event.currentTarget) {
				return false;
			} else if (!event.currentTarget.value) {
				return false;
			}

			dojo.stopEvent(event);

			var imagePosition = event.currentTarget.value;
			this._updateImagePositionData(imagePosition);
			
			return true;
		});
	},
	
	_addHandlersImageUrl: function() {
		this.connect(this._predefImages, "onchange", this._doChangePredefinedImage);
		this.connect(this._predefImages, "onkeyup", this._doChangePredefFromKey);
		this.connect(this._imageURL, "onchange", this._doChangeCustomImage);
	},	

	_getImageUrl: function(url) {
		if (url.indexOf("://") === -1) {
			return Util.getServiceURL(SERVICE_IMG_ID, null, {path: url});
		}
		return url;
	},	
	
	_doChangePredefFromKey: function(event) {
			switch (event.keyCode) {
				case dojo.keys.UP_ARROW:
				case dojo.keys.DOWN_ARROW:
					this._doChangePredefinedImageCommon();
					break;
				default:
					return;
			}
	},

	_doChangePredefinedImage: function(event) {
		
		if (event) {
			dojo.stopEvent(event);
		}
		this._doChangePredefinedImageCommon();
	},

	_doChangePredefinedImageCommon: function() {
		
		if (!this._predefinedImages) {
			return;
		} 
		
		var imageUrl = null;
		var index = this._predefImages.selectedIndex;
		if (index > 0){
			imageUrl = this._predefinedImages[index - 1].url;
		}
		
		this._imageURL.value = "";
		this._updateImagePreview(imageUrl);
		this._updateImageUrlData();
		
	},
	
	_doChangeCustomImage: function(event) {
		
		if (event) {
			dojo.stopEvent(event);
		}

		this._predefImages.selectedIndex = 0;
		this._updateImagePreview(this._imageURL.value);
		this._updateImageUrlData();
	},
	
	_loadDisplayArea: function() {
		
		// Update navigation links
		this._rebuildCounter();
		
		// Hide the editor area
		this._editorArea.style.display = "none";
		
		if (this._isEmpty()) {
			// Show the message area
			this._messageArea.style.display = "block";
			if (this._site.isDashboardEditing()) {
				this._editModeMessage.style.display = "inline";
			} else {
				this._editModeMessage.style.display = "none";
			}
			this._displayArea.style.display = "none";
			this._navArea.style.display = "none";
		} else {
			// Show the display area
			this._displayArea.style.display = "block";
			this._messageArea.style.display = "none";
			this._navArea.style.display = "block";
			this._updateNavigation();
			this._updateHeight();
		}
	},
	
	_createPageNode: function(page) {
		var node = document.createElement("div");
		node.className = "headline-page";
		var el = document.createElement("div");
		el.className="display-title";
		if (typeof page.titleColor != "undefined") {
			dojo.addClass(node,this._colors[page.titleColor]);
		}
		// user can bypass script check on save, so always sanitize
		el.innerHTML = jazz.util.html.removeScript(page.title);
		node.appendChild(el);
		el = document.createElement("div");
		el.className="display-text";
		if (page.image) {
			var cname = IMAGE_POSITION_LEFT;
			if (page.imagePosition) {
				cname = page.imagePosition;
			}
			var img = document.createElement("img");
			img.onload = dojo.hitch(this, this._updateHeight);
			img.src = this._getImageUrl(page.image);
			img.className = cname;
			el.appendChild(img);
		}

		var htmlContainer = document.createElement("div");
		// user can bypass script check on save, so always sanitize
		htmlContainer.innerHTML = jazz.util.html.removeScript(page.text);
		el.appendChild(htmlContainer);
		if (htmlContainer.innerHTML.indexOf("<img") !== -1) {
			// Poll to make sure height gets updated when body image loads
			this._doPollingLoop(0);
		}

		node.appendChild(el);
		return node;
	},
		
	_rebuildCounter: function() {
		this._removeAllChildren(this._navCounter);
		var pages = this._getPageData();
		if (!pages) {
			return;
		}
		for (var i=0; i<pages.length; i++) {
			var link = document.createElement("a");
			var obj = {
				viewlet: this,
				index: -(i+2),
				func: function(event) {
					this.viewlet._doCounterNav(event, this.index);
				}
			};
			this.connect(link, "onclick", dojo.hitch(obj, obj.func));
			link.innerHTML=i+1;
			link.href = "#";
			link.title = pages[i].title;
			this._navCounter.appendChild(link);
		}
		this._updateCounter();
	},
	
	_updateNavigation: function(leftNode, rightNode) {
		var pages = this._getPageData();
		if (!pages || pages.length < 2) {
			this._navArea.style.display = "none";
			this._shiftArea.style.display = "none";
		} else {
			this._navArea.style.display = "block";
			this._shiftArea.style.display = "block";		
		}
	},

	_doEdit: function(event) {
		dojo.stopEvent(event);
		this._cancelAnimation();
		this.getSite().editContents();
		this._scheduleAnimation();
	},
	
	_cancelAnimation: function() {
		if (this._timeoutEvent)
			clearTimeout(this._timeoutEvent);
		if (this._animQueued)
			clearTimeout(this._animQueued);
		this._timeoutEvent = null;			
		this._animQueued = null;
		dojo.removeClass(this._playPause,"pause");
		dojo.addClass(this._playPause,"play");
		this._playPause.title = this.messages.play;
	},
	_toggleAnimation: function(e){
		dojo.stopEvent(e);
		if (dojo.hasClass(this._playPause,"pause")){
			this._cancelAnimation();
		} else {
			this._scheduleAnimation();
		}
	},
	
	_doShiftLeft: function(event) {
		dojo.stopEvent(event);
		this._doShift(false);

	},
	
	_doShiftRight: function(event) {
		dojo.stopEvent(event);
		this._doShift(true);
	},
	
	_doShift: function(right) {

		var pages = this._getPageData();
		
		if (!pages) {
			return;
		}

		if ((right && this._currentPageIndex === pages.length - 1) || 
			(!right && this._currentPageIndex === 0)) {
			return;
		}

		var page = pages[this._currentPageIndex];
		pages.splice(this._currentPageIndex, 1);

		var previousIndex = this._currentPageIndex;
		
		if (right) 
			this._currentPageIndex++;
		else
			this._currentPageIndex--;
			
		// Insert at a new position
		pages.splice(this._currentPageIndex, 0, page);
		
		// Update titles
		var links = this._navCounter.childNodes;
		var tempTitle = links[previousIndex].title;
		links[previousIndex].title = links[this._currentPageIndex].title;
		links[this._currentPageIndex].title = tempTitle;
		
		this._doNav(0);
	},

	_doNav: function(next) {

		if (next == 1) {
			this._currentPageIndex++;
		} else if (next == -1) {
			this._currentPageIndex--;
		} else if (next < -1) {
			// random access to a page
			this._currentPageIndex = -(next + 2);
		}
		
		if (this.isEditing()) {
			this._updateEditorAreaContent();
			this._updateNavigation(this._shiftLeft, this._shiftRight);	
			this._playPause.style.display = "none";
		} else {
			this._updateDisplayAreaContent();
		}
		
		this._updateCounter();
		this._scheduleAnimation();
	},

	_updateDisplayAreaContent: function() {
		
		var pageNodes = this._getPageNodes();
		
		if (!pageNodes || pageNodes.length === 0) {
			return;
		}
		
		for (var index = 0; index < pageNodes.length; index++) {
			var pageNode = pageNodes[index];
			if (index === this._currentPageIndex) {
				pageNode.style.visibility = "visible";
			} else {
				pageNode.style.visibility = "hidden";
			}
		}
	},

	_updateHeight: function() {
		if (!this._active) {
			this._doHeightOnActive = true;
			return;
		}
		// Add in an artificial delay to ensure the display nodes are on the DOM
		setTimeout(dojo.hitch(this, this._updateHeightHelper), 100);
	},

	_updateHeightHelper: function() {
		if (!this.domNode) return;
		var pageNodes = this._getPageNodes();
		var maxHeight = 0;

		for (var index = 0; index < pageNodes.length; index++) {
			var node = pageNodes[index];
			var nodeHeight = node.clientHeight;

			if (nodeHeight > maxHeight) {
				maxHeight = nodeHeight;
			}
		}
		
		if (!maxHeight) {
			maxHeight = "85";
		}
		
		this._displayArea.style.height = maxHeight + "px";
	},

	_scheduleAnimation: function() {
		if (!this.domNode) return;
		this._playPause.style.display = "none";
		if (this.getPreference("slideshow") === false) {
			return;
		}
		if (this._getPageNumber()<2)
			return;
		this._cancelAnimation();
		if (!this.isEditing()) {
			this._playPause.style.display = "block";
			var interval = this.getPreference("interval");
			if (interval <= 0 || interval > 30)
				interval = 3;
			dojo.addClass(this._playPause,"pause");
			dojo.removeClass(this._playPause,"play");
			this._playPause.title = this.messages.pause;
			this._timeoutEvent = setTimeout(dojo.hitch(this, this._loop), interval*1000);
		}
	},
	
	_getPageNumber: function() {
		var pages = this._getPageData();
		if (!pages)
			return 0;
		return pages.length;
	},
	
	_loop: function () {
		//Check things happened between the animation was scheduled and now
		if (this.isEditing() || 
			(this.getPreference("slideshow") === false)) {
			return;		
		}

		this._timeoutEvent = undefined;
		this._doNextInLoop(true);
	},
	
	_doNextInLoop: function(forward) {
		var pages = this._getPageData();
		if (!pages)
			return;
		var nextIndex;
		if (forward) {
			if (this._currentPageIndex == pages.length - 1) {
				// return
				nextIndex = 0;
			}
			else 
				nextIndex = this._currentPageIndex + 1;
		}
		else {
			// back
			if (this._currentPageIndex === 0) {
				// return
				nextIndex = pages.length-1;
			}
			else 
				nextIndex = this._currentPageIndex - 1;			
		}
		this._doNav(-(nextIndex+2));
		if (nextIndex === 0) {
			if (!this.getPreference("continuousSlideshow")) {
				this._cancelAnimation();
			}
		}
	},
	
	_doCounterNav: function(event, index) {
		dojo.stopEvent(event);
		this._doNav(index);
	},
	
	_updateCounter: function() {
		var links = this._navCounter.childNodes;
		for (var i = 0; i < links.length; i++) {
			links[i].className = (i == this._currentPageIndex) ? "current" : "";
		}
	},
	
	_doAddPage: function(event) {
		if (event)
			dojo.stopEvent(event);
		var pages = this._getPageData();
		var newIndex = pages.length;
		var page = this._createPageData();
		pages.push(page);
		
		this._currentPageIndex = newIndex;
		this._doNav(0); 
		this._rebuildCounter();
	},
	
	_doRemovePage: function(event) {
		
		var pages = this._getPageData();
		
		if(event)
			dojo.stopEvent(event);
		if (pages.length == 1) {
			var makeNewPage = true;
		}
		if (!confirm(dojo.string.substitute(this.messages.confirmDelete, [this._currentPageIndex + 1]))) {
			return;
		}
		
		if (pages && pages[this._currentPageIndex])
			pages.splice(this._currentPageIndex, 1);
		if (pages.length === 0) {
			this._doAddPage();
		} else {
			if (this._currentPageIndex > pages.length - 1)
				this._currentPageIndex = pages.length - 1;
			this._doNav(0);
			this._rebuildCounter();	
		}
	},
	
	validateEditedContent: function() {
		// Get the title and edited contents
		var title = new String(this._title.value);
		var content = this._editor.attr("value");
		
		// Check for insecure content
		if (jazz.util.html.hasScript(title) || jazz.util.html.hasScript(content)) {
			alert(this.messages.errorScript);
			return false;
		}
		return true;
	},
	
	preSaveState: function() {
		// Save button was pressed in edit mode
		this._saved = true;
		this.getViewletState().set(this._STATE_PAGES, this._pageData);
	},	
	
	update: function() {
		// Determine whether we are entering or leaving edit mode
		this._site.getActionArea().innerHTML = "";

		if (this.isEditing()) {
			this._loadEditorState();
			this._loadEditorArea();
			this._addActions();
			this._cancelAnimation();
			this._playPause.style.display = 'none';
		} else {
			if (this._saved) {
				this._loadDisplayNodes();
			}
			this._loadDisplayArea();
			this._scheduleAnimation();
			this._saved = false;
		}
	},
	
	_loadEditorState: function() {
		
		this._pageData = this.getViewletState().get(this._STATE_PAGES);
		if (this._pageData) {
			return;
		}
		
		this._pageData = [];
		this._pageData[0] = this._createPageData();
	},

	_getPageData: function() {
		if (this.isEditing()) {
			return this._pageData;
		} 
		return this.getViewletState().get(this._STATE_PAGES);
	},

	_getPageNodes: function() {
		var displayNode = this._displayArea && this._displayArea.firstChild;
		if (!displayNode) {
			return [];
		} else if (!displayNode.hasChildNodes()) {
			return [];
		}
		
		var pageNodes = [];
		var children = displayNode.childNodes;
		for (var index = 0; index < children.length; index++) {
			var child = children[index];
			pageNodes.push(child);
		}

		return pageNodes;
	},
	
	_createPageData: function() {
		var page = new Object();
		page.title = this.messages.defaultTitle;
		page.text = "<div style=\"margin:5px;font-size:10pt;\">\n   " +
					this.messages.defaultBody +
					"\n</div>";
		
		page.titleColor = 0;
		page.imagePosition = IMAGE_POSITION_LEFT;
		page.imageId = null;
		page.image = null;
		
		return page;
	},
	
	_loadDisplayNodes: function() {

		this._currentPageIndex = 0;

		var pageData = this._getPageData();
		if (!pageData) {
			return;
		}

		var newContent = document.createElement("div");
		var oldContent = this._displayArea.firstChild;

		for (var index = 0; index < pageData.length; index++) {
			var page = pageData[index];
			var pageNode = this._createPageNode(page);
			
			if (index === this._currentPageIndex) {
				pageNode.style.visibility = "visible";
			} else {
				pageNode.style.visibility = "hidden";
			}			
			
			newContent.appendChild(pageNode);
		}
				
		this._displayArea.replaceChild(newContent, oldContent);	

	},
	
	_loadEditorArea: function() {
		
		// Create the rich text editor
		this._createEditor();		
		
		// Add content to editor area
		this._loadPredefinedImages();
		this._updateEditorAreaContent();

		// Show the editor area
		this._editorArea.style.display = "";
		// Hide the display area
		this._displayArea.style.display = "none";
		// Hide the message area
		this._messageArea.style.display = "none";
		// Show the nav area
		this._navArea.style.display = "block";
		this._playPause.style.display = "none";
		
		this._updateNavigation(this._shiftLeft, this._shiftRight);
	},

	/*
 	 * @param String html
 	 * 
 	 * @return String
 	 */          
	_sanitizeContent: function(html) {
		var sanitizedHTML = jazz.util.html.removeScript(html);
		return sanitizedHTML;
	},		

	_createEditor: function() {
	
		if (this._editor) {
			return;
		}

		this._editor = new Editor({
			id: this._getRichTextEditorID(),
			height: "150px",
			onChange: dojo.hitch(this, this._handleEventEditorChange),
			plugins: [],
			contentPostFilters: [this._sanitizeContent],
			contentPreFilters: [this._sanitizeContent],
			extraPlugins: [
				{
					name: 'viewsource'
				},			
				{
					name: 'fullscreen',
					zIndex: 999
				},				
				{
					name: '|'
				},				
				{
					name: 'bold'
				},				
				{
					name: 'italic'
				},				
				{
					name: 'underline'
				},				
				{
					name: '|'
				},				
				{
					name: 'foreColor'
				},				
				{
					name: 'hiliteColor'
				},				
				{
					name: '|'
				},				
				{
					name: 'justifyLeft'
				},				
				{
					name: 'justifyCenter'
				},				
				{
					name: 'justifyRight'
				},				
				{
					name: '|'
				},				
				{
					name: 'insertOrderedList'
				},				
				{
					name: 'insertUnorderedList'
				},				
				{
					name: '|'
				},				
				{
					name: 'indent'
				},				
				{
					name: 'outdent'
				},				
				{
					name: '|'
				},				
				{
					name: 'createLink'
				},				
				{
					name: 'insertImage'
				},				
				{
					name: 'prettyprint'
				}	
			]
 		}, this._rteArea);

	},		

	_handleEventEditorChange: function(newContent) {
		this._updateTextData();
	},

	_getRichTextEditorID: function() {
		return RICH_TEXT_EDITOR_ID + this.id;	
	},	
	
	_addActions: function() {
		var actionArea = this.getSite().getActionArea();
		var doc = actionArea.ownerDocument;
		
		var addPageWrapper = doc.createElement("div");
		addPageWrapper.className = "action add";
		var addPageLink = doc.createElement("a");
		addPageLink.innerHTML = this.messages.addPage;
		addPageLink.href = "#";
		addPageWrapper.appendChild(addPageLink);
		actionArea.appendChild(addPageWrapper);
		this.connect(addPageLink, "onclick", this._doAddPage);
		
		var deletePageWrapper = doc.createElement("div");
		deletePageWrapper.className = "action delete";
		var deletePageLink = doc.createElement("a");
		deletePageLink.innerHTML = this.messages.deletePage;
		deletePageLink.href = "#";
		deletePageWrapper.appendChild(deletePageLink);
		actionArea.appendChild(deletePageWrapper);
		this.connect(deletePageLink, "onclick", this._doRemovePage);
	},

	_selectColor: function(event) {
		if (!event.target) return;
		var className = event.target.className;
		var color;
		switch(className) {
			case "blue-option": 
				color = this.BLUE;
				break;
			case "gray-option":
				color = this.GRAY;
				break;
			case "gold-option":
				color = this.GOLD;
				break;
			case "green-option":
				color = this.GREEN;
				break;
			case "teal-option":
				color = this.TEAL;
				break;
			case "purple-option":
				color = this.PURPLE;
				break;
			case "orange-option":
				color = this.ORANGE;
				break;
			default:
				color = -1;
				break;
		}
		
		if (color === -1) {
			return;
		}
		
		this._updateSelectedColorNode(event.target);
		this._updateSelectedColorData(color);
		
	},

	_getCurrentPage: function() {
		var pages = this._getPageData();
		if (!pages) {
			return null;
		}
		var page = pages[this._currentPageIndex];
		if (!page) {
			return null;
		}
		return page;
	},

	_updateSelectedColorNode: function(colorNode) {
		
		if (!colorNode) {
			return;
		}
		
		// Remove selection
		if (this._selectedColorNode) {
			dojo.removeClass(this._selectedColorNode, "selected");	
		}
		
		// Add selection
		dojo.addClass(colorNode, "selected");
		this._selectedColorNode = colorNode;
	},

	_updateSelectedColorData: function(colorData) {
		var page = this._getCurrentPage();
		if (!page) {
			return;
		}
		// Update data
		page.titleColor = colorData;
	},

	_updateTitleNode: function(title) {
		this._title.value = title;
	},

	_updateTitleData: function(title) {
		var page = this._getCurrentPage();
		if (!page) {
			return;
		}
		
		var result = dojo.query('a[class="current"]', this._navCounter);
		if ((result instanceof Array) &&
			(result.length > 0)) {
			var link = result[0];
			dojo.attr(link, 'title', title);
		}
		
		// Update data
		page.title = title;		
	},
	
	_updateTextNode: function(html) {
		// User can bypass script check on save, so always sanitize
		html = jazz.util.html.removeScript(html);
		// Update editor contents   
		this._editor.attr("value", html);		
	},

	_updateTextData: function() {
		var page = this._getCurrentPage();
		if (!page) {
			return;
		}
		// Update data
		page.text = this._editor.attr("value");
	},

	_updateImagePositionNode: function(imagePosition) {

		this._imagePosition.selectedIndex = 0;
		var options = this._imagePosition.options;
		for (var index = 0; index < options.length; index++) {
			var option = options[index];
			if (option.value === imagePosition) {
				option.selected = true;
				return;
			}
		}
	},

	_updateImagePositionData: function(imagePosition) {
		var page = this._getCurrentPage();
		if (!page) {
			return;
		}
		// Update data
		if (imagePosition) {
			page.imagePosition = imagePosition;
		} else {
			page.imagePosition = IMAGE_POSITION_LEFT;
		}
	},

	_updateImageUrlNode: function(imageId, imageUrl) {
		
		if (!imageId && !imageUrl) {
			// No image
			this._predefImages.selectedIndex = 0;
			this._imageURL.value = "";
		} else if (!imageUrl) {
			// Error
			this._predefImages.selectedIndex = 0;
			this._imageURL.value = "";
		} else if (!imageId) {
			// Custom image
			this._predefImages.selectedIndex = 0;
			this._imageURL.value = imageUrl;
		} else {
			// Predefined image
			this._predefImages.selectedIndex = 0;
			var options = this._predefImages.options;
			for (var index = 0; index < options.length; index++) {
				if (options[index].value === imageId) {
					options[index].selected = true;
					break;
				}
			}

			this._imageURL.value = "";
		}
		
		this._updateImagePreview(imageUrl);		
	},

	_updateImageUrlData: function() {
		var page = this._getCurrentPage();
		if (!page) {
			return;
		}
		// Update data
		if (this._imageURL.value) {
			page.imageId = null;
			page.image = this._imageURL.value;
			return;
		} 
		
		var index = this._predefImages.selectedIndex;
		
		if (index > 0){
			page.imageId = this._predefImages.options[index].value;
			var url = this._predefinedImages[index - 1].url;
			page.image = url;
			return;
		}
		
		page.imageId = null;
		page.image = null;
	},

	setDashboardEditing: function(editing) {
		if (editing) {
			this._editModeMessage.style.display = "inline";
		} else {
			this._editModeMessage.style.display = "none";
		}
	},
	
	_isEmpty: function() {
		var pages = this._getPageData();
		return (!pages || pages.length === 0);
	},
	
	_updateEditorAreaContent: function() {
		
		var page = this._getCurrentPage();
		
		// Title
		this._updateTitleNode(page.title);
		// Color	
		this._updateSelectedColorNode(this._getColorNode(page.titleColor));
		// Text
		this._updateTextNode(page.text);
		// Image
		this._updateImageUrlNode(page.imageId, page.image);
		// Image Position
		this._updateImagePositionNode(page.imagePosition);
	},
	
	_getColorNode: function(colorData) {
		if (typeof(colorData) === "undefined") {
			return null;
		} else if (colorData < 0) {
			return null;
		} else if (colorData > (this._colors.length - 1)) {
			return null;
		}
		return this._colorPicker.childNodes[colorData];
	},	
	
	_updateImagePreview: function(imageUrl) {
		if (imageUrl) {
			this._imagePreview.style.display = "";
			this._imagePreview.src = this._getImageUrl(imageUrl);
		} else {
			this._imagePreview.style.display = "none";
		}
	},
	
	_loadPredefinedImages: function() {
		this._removeAllChildren(this._predefImages);
		var noimage = document.createElement("option");
		noimage.innerHTML = ""; 
		noimage.value="none";
		this._predefImages.appendChild(noimage);
		var serviceObj = {
			viewlet: this,
			success: function(images) {
				var combo = this.viewlet._predefImages;	
				this.viewlet._predefinedImages= images.images;			
				for (var i=0; i<images.images.length; i++) {
					var id = images.images[i].id;
					var name = images.images[i].name;
					combo.options[combo.options.length] = new Option(name, id);
				}
				var pages = this.viewlet._getPageData();
				var page = pages[this.viewlet._currentPageIndex];				
				
				this.viewlet._updateImageUrlNode(page.imageId, page.image);
			},
			failure: function(errorObj) {
				this.viewlet.getSite().error(errorObj);
			}
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		HeadlinesClient.getHeadlineImages(srh, this.getServiceTracker());		
	},

	_removeAllChildren: function(node) {
		// Remove all child nodes if any are defined
		while (node.childNodes.length > 0) {
			node.removeChild(node.firstChild);
		}
	},	
	
	refresh: function() {
		this.getSite().doneRefresh();
		this._scheduleAnimation();
		this._updateHeight();
	},
	
	settingsChanged: function(oldSettings, newSettings) {
		if (newSettings.preferences.slideshow != oldSettings.preferences.slideshow) {
			this.update();
		}
	},
	
	setActive: function(active) {
		this._active = active;
		if (active) {
			// start the animation
			this._animQueued = setTimeout(dojo.hitch(this,this._scheduleAnimation), 1000);
			if (this._doHeightOnActive) {
				delete this._doHeightOnActive;
				this._updateHeightHelper();
			}
		} else {
			// stop the animation
			this._cancelAnimation();
		}
	},
	destroy: function() {
		ResizeNotifier.unregister(this._dashboardResizeEvent);
		if (this._pollingTimeout) {
			clearTimeout(this._pollingTimeout);
			delete this._pollingTimeout;
		}
		this.inherited(arguments);
	},
	
	BLUE: 0,
	GRAY: 1,
	GOLD: 2,
	GREEN: 3,
	TEAL: 4,
	PURPLE: 5,
	ORANGE: 6
});


})();

