/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.IWidgetViewlet");

dojo.require("dojo.i18n");
dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");

dojo.require("jazz.iwidget._IWidget");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function()
{
	var Util = com.ibm.team.dashboard.web.util.internal.Util;
	var Viewlet = com.ibm.team.dashboard.web.ui.Viewlet;
	var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
	
	var messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
	
	/*
	 * Lotus edits preferences in a dialog. Edit mode of some of the default widgets
	 * assumes that this style will be set when we are in edit mode.
	 */
	var EDIT_MODE_STYLE = "lotusDialog edit";
	
	dojo.declare("com.ibm.team.dashboard.web.ui.internal.IWidgetViewlet", [Viewlet, jazz.iwidget._IWidget], {
	
		templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/IWidgetViewlet.html"),
		
		/**
		 * cant trust this._zoom must keep track ourself
		 */
		_isZoomed: false,
		
		setEditing: function(/* boolean */isEditing)
		{
			this.inherited(arguments);
			
			if (isEditing) 
			{
				this.zoomViewlet(true);
				this.setMode(jazz.iwidget.mode.EDIT);
			}
		},
		
		zoomViewlet: function(zoom)
		{
         /**
          * if zoom is the same then we don't do anything
          */
         if ((zoom && this._isZoomed) || (!zoom && !this._isZoomed)) return;

         /**
          * we must not be the same so we have to toggle
          */			
			this._viewletContainer.toggleZoom();
		},
		
		/**
		 * call back from Viewlet when our zoom state changes. 
		 * We have to keep track because this._zoom is unreliable
		 */
		setZoomed: function(z)
		{
			this._isZoomed = z;
		},
		
		refresh: function()
		{
			if (!this._iwidget) 
			{
				this._createIWidget(this._content, this._definition.url, this._getPreferences());
			}
			else 
			{
				/**
				 * todo: figure out how to do a refresh perhaps widget.onview?
				 */
				this._readPreferencesFromDashboard();
			}
		},
		
		onModeChange: function(newMode)
		{
			switch (newMode)
			{
				/**
				 * when switching back into view mode and the editor is up so close it
				 */
			case jazz.iwidget.mode.VIEW:
				if (this.isEditing())
				{
					this.zoomViewlet(false);
					this.getSite()._closeEditor(false);
					dojo.removeClass(this.domNode, EDIT_MODE_STYLE);
				}
				break;
			case jazz.iwidget.mode.EDIT:
				dojo.addClass(this.domNode, EDIT_MODE_STYLE);
				break;
			}
		},
		
		/**
		 * called when iWidget saves its attributes.
		 * transfer iWidget attributes to Dashboard.
		 */
		onSaveAttributes: function()
		{
			this._writePreferencesToDashboard();
		},
		
		_writePreferencesToDashboard: function()
		{
			if (!this._iwidget) return;
			
			var attributes = this._iwidget.getAttributes();
			
			var map = {};
			dojo.forEach(attributes.getAllNames(), function(name)
			{
				map[name] = attributes.getItemValue(name);
			});
			this.getSite()._setPreferences(map);
		},
		
		/**
		 * called when dashboard writes its settings.
		 * transfer these to the iWidget
		 */
		settingsChanged: function()
		{
			this._readPreferencesFromDashboard();
		},
		
		_readPreferencesFromDashboard: function()
		{
			if (!this._iwidget) return;
			
			var attributes = this._iwidget.getAttributes();
			var map = this._getPreferences();
			
			dojo.forEach(attributes.getAllNames(), function(name)
			{
				if (name in map) 
				{
					attributes.setItemValue(name, map[name]);
				}
				else 
				{
					console.warn("IWidgetViewlet.settingsChanged : attribute ", name, " not a dashboard perference.");
				}
			});
		},
		
		
		_getPreferences: function()
		{
			var result = {};
			var pdefs = this._definition.preferenceDefinitions;
			
			if (pdefs) 
			{
				for (var i = 0; i < pdefs.length; i++) 
				{
					var pdef = pdefs[i];
					
					var value = this._getNondefaultPreference(pdef);
					if (value !== null) 
					{
						result[pdef.id] = value;
					}
				}
			}
			return result;
		},
		
		_getNondefaultPreference: function(prefDef)
		{
			var prefs = this._helper.preferences, value;
			if (prefs) 
			{
				for (var j = 0; j < prefs.length; j++) 
				{
					if (prefs[j].id == prefDef.id) 
					{
						value = prefs[j].value;
						break;
					}
				}
			}
			if (typeof value !== "undefined") 
			{
				return Util._cast(prefDef.preferenceType, value);
			}
			return null;
		}
	});
	
})();
