/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.LegacyViewletSite");

dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.ContributorMemento");
dojo.require("com.ibm.team.dashboard.web.ui.internal.ViewletMemento");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ItemNameRegistry");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");

(function(){

var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var ContributorMemento = com.ibm.team.dashboard.web.ui.internal.ContributorMemento;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var ItemNameRegistry = com.ibm.team.dashboard.web.ui.internal.util.ItemNameRegistry;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var ViewletMemento = com.ibm.team.dashboard.web.ui.internal.ViewletMemento;

/*
 * A ViewletSite implementation that is used in old containers in order
 * to communicate with N-1 containers.
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.LegacyViewletSite",
	null, {
	
	constructor: function(args) {
		this._container = args.container;
		this._dashboardItem = args.dashboardItem;
		this._definition = args.definition;
		this._viewlet = args.viewlet;
		this._viewletHelper = args.viewletHelper;
		this._loading = false;
	},

	/*
	 * Adds a menu item to the viewlet menu.
	 * 
	 * @param label the text shown for the menu item.
	 * @param icon (optional) either an image URL or a css class name specifying
	 * 		a background-image property for the menu item's icon, or null for none.
	 * 		The use of css class is deprecated as it does not work with iframed
	 * 		viewlets (because the css is not available outside the iframe).
	 * @param callback the function to call when the user activates the menu item. The scope
	 * 		(i.e. 'this') will be the viewlet adding the menu item.
	 */
	addMenuItem: function(/* string */ label, /* string */ icon, /* function */ callback) {
		var entry = {label: label, onClick: dojo.hitch(this._container._viewlet, callback)};
		if (icon) {
			// ensure string
			icon = icon + "";
			// check if it's a url or a css class
			if (icon.indexOf("/") != -1) {
				// url iff it has a slash
				entry.icon = icon;
			}
			else {
				// must be a css class; say it's deprecated
				dojo.deprecated("com.ibm.team.dashboard.web.ui.ViewletSite.addMenuItem(label, iconClass, callback)", "Use an icon URL for the second parameter instead of a css class");
				entry.iconClass = icon;
			}
		}
		this._container.addMenuItem(entry);
	},
	
	/*
	 * Returns true or false based on if the current server license level allows viewing of the
	 * indicated dashboard type.
	 * 
	 * @param scope a number representing one of the constants for the scope of the dashboard
	 * @since 0.6.1
	 */
	/* boolean */ canViewDashboard: function(/* number */ scope) {
		var perms = this._container.dashboardBundle.permissions;
		if (scope == DashboardConstants.SCOPE_PROJECT_AREA)
			return perms.viewProject;
		if (scope == DashboardConstants.SCOPE_TEAM_AREA)
			return perms.viewTeam;
		if (scope == DashboardConstants.SCOPE_CONTRIBUTOR) {
			return perms.viewPersonal;
		}
		return false;
	},
	
	/*
	 * Reports a serious error which may impact the normal operation of the
	 * dashboard or other viewlets (e.g. connection failure, server down).
	 * 
	 * @param error An Error object representing the problem.
	 * @since 0.7
	 */
	error: function(/* Error */ error) {
		dojo.publish(Constants.serverError, [error]);
	},
	
	/*
	 * Returns a per-user memento for persisting viewlet state info that can be
	 * different for each user, e.g. appearance.
	 */
	/* IMemento */ getContributorState: function() {
		if (!this._contributorMemento) {
			var viewletInfo = this._container._viewletContributorInfo;
			this._contributorMemento = new ContributorMemento(viewletInfo, this._container);
		}
		return this._contributorMemento;
	},
	
	/*
	 * Returns the viewlet preference by the given id.
	 */
	/* string */ getPreference: function(/* string */ id) {
		var prefDefs = this._definition.preferenceDefinitions;
		if (prefDefs) {
			for (var i=0;i<prefDefs.length;i++) {
				var prefDef = prefDefs[i];
				if (prefDef.id == id) {
					// first check the viewlet helper
					var value;
					var prefs = this._viewletHelper.preferences;
					if (prefs) {
						for (var j=0;j<prefs.length;j++) {
							if (prefs[j].id == id) {
								value = prefs[j].value;
								break;
							}
						}
					}
					if (value || value == false) {
						return this._cast(value, prefDef.preferenceType);
					}
					// fall back to definition default
					if (prefDef.defaultValue) {
						return this._cast(prefDef.defaultValue, prefDef.preferenceType);
					}
					// no value found
					return null;
				}
			}
		}
		return null;
	},
	
	/*
	 * Returns the viewlet's scope. If the viewlet is set to automatic scope,
	 * then the dashboard's scope is returned.
	 */
	/* string */ getScope: function() { 
		var scope = this._viewletHelper.scope;
		// if the viewlet is not scope sensitive, always return the dashboard scope
		if (!this._definition.scopeSensitive ||
				(scope != DashboardConstants.SCOPE_CONTRIBUTOR && scope != DashboardConstants.SCOPE_TEAM_AREA && scope != DashboardConstants.SCOPE_PROJECT_AREA))
			scope = this._dashboardItem.scope;
		switch (this._definition.appliesTo) {
			case Constants.SCOPE_APPLIES_TO_TEAMS:
				return DashboardConstants.SCOPE_TEAM_AREA;
			case Constants.SCOPE_APPLIES_TO_CONTRIBUTORS:
				return DashboardConstants.SCOPE_CONTRIBUTOR;
			case Constants.SCOPE_APPLIES_TO_PROJECTS:
				return DashboardConstants.SCOPE_PROJECT_AREA;
			case Constants.SCOPE_APPLIES_TO_PROCESS:
				if (scope == DashboardConstants.SCOPE_CONTRIBUTOR) {
					return DashboardConstants.SCOPE_PROJECT_AREA;
				}
		}
		return scope;
	},
	
	/*
	 * Returns the viewlet's scope item. If the viewlet's scope item is set
	 * to automatic, then a default will be assigned as follows:
	 * if the scope is contributor, the authenticated user will be used
	 * if the scope is project or team, the dashboard's scope item
	 * will be used.
	 */
	/* object */ getScopeItem: function() {
		// special case to handle unconfigured viewlets that were copied and have scopes that don't apply
		if (typeof(this._viewletHelper.scope) != 'undefined' && !Util.isScopeApplicible(this._viewletHelper.scope, this._definition)) {
			// if the viewlet is scopable, null (a.k.a. not scoped) is a valid case and should be maintained after copying
			if (this._definition.scopeSensitive) {
				return null;
			}
			// if the viewlet is not scopable we should return the dashboard scope, so fall through to default
		}
		// ignore the viewlet helper if the viewlet is not scope sensitive, in this case always defer to the dashboard
		if (this._definition.scopeSensitive) {
			var scopeItem = this._viewletHelper.scopeItem;
			if (scopeItem) {
				var name = ItemNameRegistry.getName(scopeItem.itemId);
				if (name) {
					scopeItem.name = name;
				}
				return scopeItem;
			}
		}
		if (Util.isLocal(this._viewletHelper))
			return this._getDefaultItem(this.getScope());
		// if the viewlet is not local, return null since dashboard scopes do not apply, viewlets must handle this case
		return null;
	},
	
	/*
	 * Returns the scope setting that indicates if subteams should be considered
	 * part of the viewlets scope. Only applicible when the scope is set to a team
	 * area and only to viewlets who define include-subteams as "yes" or "yes-and-check"
	 * in their definition.
	 */
	/* boolean */ getScopeSubteams: function() {
		var value = this._viewletHelper.scopeSubteams;
		if (!value && value != false)
			value = this._definition.scopeSubteams == "yes-and-check";
		return value;
	},
	
	/*
	 * Returns a memento for persisting viewlet state info that is the same for
	 * all users, e.g. viewlet content.
	 */
	/* IMemento */ getViewletState: function() {
		if (!this._viewletMemento) {
			var memento;
			var jsonMemento = this._viewletHelper.memento;
			if (jsonMemento) {
				// viewlet has existing memento
				memento = dojo.fromJson(jsonMemento);
			}
			else {
				// create a new one
				memento = this._viewlet.getDefaultState();
			}
			this._viewletMemento = new ViewletMemento(memento, this._viewletHelper, this._container);
		}
		return this._viewletMemento;
	},
	
	/*
	 * Enables or disables automatic periodic refreshing of viewlets. Viewlets
	 * may wish to temporarily disable automatic refresh if it disrupts the
	 * normal user flow when, for example, the user is reading a hover tooltip or
	 * editing content. Normal auto-refresh will resume as soon as it is re-enabled.
	 */
	setAutoRefresh: function(/* boolean */ value) {
		this._container.setAutoRefresh(value);
	},
	
	/*
	 * Sets the viewlet icon to the image pointed to by the given URL.
	 * 
	 * @param url the URL of the image file.
	 */
	setIcon: function(/* string */ url) {
		this._container.setIcon(url);
	},
	
	/*
	 * Called to set loading state of viewlet.  Displays progress icon when loading.
	 */
	setLoading: function(loadingStatus) {
		if (this._container._iconImage) {
			if (loadingStatus) {
				this._container._iconImage.style.display = "none";
				dojo.addClass(this._container._headerIcon, "com-ibm-team-dashboard-progressIcon");
			}
			else {
				this._container._iconImage.style.display = "inline";
				dojo.removeClass(this._container._headerIcon, "com-ibm-team-dashboard-progressIcon");
			}
		}
		this._loading = loadingStatus;
	},
	
	/*
 	 * Public Method
 	 * 
 	 * Indicates whether the viewlet is loading or not
 	 *  
 	 * @param void
 	 * @return boolean true = loading; false = not loading
 	 */	
	isLoading: function() {
		return this._loading;
	},	
	
	/*
	 * Can be called by viewlets to programatically open or close the settings area.
	 */
	toggleSettings: function() {
		this._container.toggleSettings();
	},
	
	/*
	 * Can be called by viewlets to programatically enter edit mode.
	 */
	editContents: function() {
		this._container.startEditing();
	},
	
	/*
	 * Called by viewlets to update the dynamic title. Some viewlets only know the title
	 * when the SettingsArea is closed. These viewlets should indicate that the title
	 * needs to be persisted by setting the persist flag to true. Note that this results
	 * in a dashboard modification and will make the Dashboard dirty, thus it should
	 * only be done in tandem with another action that dirties the dashboard, such as
	 * changing the viewlet settings.
	 */
	setTitle: function(/* string */ title, /* boolean */ persist) {
		// ignore the call if the user has overriden the title
		if (!this._viewletHelper.userTitle) {
			// update the title on the container
			this._container.setTitle(title, persist);
		}
	},
	
	setSecondaryTitle: function(/* string */ title) {
		this._container.setSecondaryTitle(title);
	},
	
	getTitle: function() {
		return this._viewletHelper.title;
	},
	
	getSecondaryTitle: function() {
		return this._container.getSecondaryTitle();
	},
	
	getActionArea: function() {
		return this._container.actionArea;
	},
	
	isDashboardEditing: function() {
		return this._container.editing;
	},
	
	isActive: function() {
		return this._container.viewletPage.isActive;
	},
	
	/*
 	 * Public Method
 	 * 
 	 * Returns the height of the viewlet's content area
 	 * 
 	 * @param void
 	 * @return Number
 	 */		
	getHeight: function() {
		if (dojo.isIE == 6)
			//directly sniff height for IE6 (dojo.style doesn't support IE6 properly)
			return this._container._contentWrapper.clientHeight;
		else
			return dojo.style(this._container._contentWrapper, "height");
	},
	
	/*
 	 * Public Method
 	 * 
 	 * Returns the width of the viewlet's content area
 	 * 
 	 * @param void
 	 * @return Number
 	 */		
	getWidth: function() {
		if (dojo.isIE == 6) {
			//look at header width instead (because IE6 gives wrong width if content is busting out the side)
			var width = this._container._header.clientWidth - 5; 
			
			//for trimless viewlet case
			if (width < 0) width = this._container._contentWrapper.clientWidth - 40;
			
			return width;
		} else
			return dojo.style(this._container._contentWrapper, "width");
	},
	
	doneRefresh: function() {
		this._container.isRefreshing--;
		if (this._container.isRefreshing < 0)
			this._container.isRefreshing = 0;
	},
	
	refresh: function() {
		this._container.refreshViewlet(true);
	},
	
	navigateAwayEvent: function(eventObj, workbenchNavigation) {
		return this._container.viewletPage.dashboardContainer.dashboardWidget.navigateAwayEvent(eventObj, workbenchNavigation);
	},
	
	/*
	 * Returns a properly typed primitive according to the type specified.
	 * E.g. if you pass in a string "true" and the preferenceType is "BOOLEAN",
	 * it will return a boolean with the value true.
	 */
	/* any */ _cast: function(/* string */ value, /* string */ preferenceType) {
		if (preferenceType == "INTEGER") {
			var parsedValue = parseInt(value);
			if (isNaN(parsedValue)) {
				return null;
			}
			return parsedValue;
		}
		else if (preferenceType == "BOOLEAN") {
			if (value == true || value == false)
				return value;
			return value == "true" ? true: false;
		}
		return value;
	},
	
	_getDefaultItem: function(scope) {
		if (this._container._isLocal()) {
			var defaults = this._dashboardBundle.defaults;
			switch (scope) {
				case DashboardConstants.SCOPE_CONTRIBUTOR:
					return defaults.contributor;
				case DashboardConstants.SCOPE_TEAM_AREA:
					return defaults.team;
				case DashboardConstants.SCOPE_PROJECT_AREA:
					return defaults.project;
			}
		}
	}
});
	
})();
