/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.LicenseInfoWidget");
dojo.require("com.ibm.team.repository.web.client.internal.licensing"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.ui.internal.events");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal", "LicenseInfoWidgetMessages");

var messages;

var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var checkExpiringCALs = com.ibm.team.repository.web.client.internal.licensing.checkExpiringCALs;
var getExpirationInfo = com.ibm.team.repository.web.client.internal.licensing.getExpirationInfo;
var getLicenseInfo = com.ibm.team.repository.web.client.internal.licensing.getLicenseInfo;

dojo.declare("com.ibm.team.repository.web.ui.internal.LicenseInfoWidget", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/templates/LicenseInfoWidget.html"),
	
	postCreate: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal", "LicenseInfoWidgetMessages");
		
		dojo.subscribe(com.ibm.team.repository.web.ui.internal.events.licenseChange, this, "_showLicenseInfo");
		
		this.serverMode = dojo.getObject("com.ibm.team.repository.server.mode");
		this.isJTS = (this.serverMode == "APPLICATION") ? false : true;
		if(!this.isJTS) {
			//if this fails, continue loading CAL Info using the _contextRoot as a fallback for the licenseURL
			var srh1 = new ServiceResponseHandler(this, "handleJTSURLResponse", "getCALInfo");
			ServerStatusClient.getConfiguredJazzTeamServerURI(srh1);	
		} else {
			this.getCALInfo();
		}
	},
	
	handleJTSURLResponse: function(response) {
		this.jtsURL = response || null;
		this.getCALInfo();
	},

	getCALInfo: function() {
		var self = this;
		setTimeout(function() {
			getLicenseInfo(new ServiceResponseHandler(self, "_showLicenseInfo", "_showLicenseInfoErr"));
		}, 4000);
	},
	
	_showLicenseInfo: function(licenseInfo) {
		var disabledReason = dojo.getObject("serverLicense.disabledReason", false, licenseInfo || "");
		var expirationTime = dojo.getObject("serverLicense.expirationTime", false, licenseInfo || "");
		var isTrial = licenseInfo.serverLicense.trial;
		
		//Find out if we have expiring CALs
		var minDaysRemaining;
		var licenseUrl = (this.jtsURL || net.jazz.ajax._contextRoot) + "/admin#action=com.ibm.team.repository.admin.manageLicenses"; 
		var anyExpiringCALS = false;
		var types = licenseInfo.types || [];
		var combined = types;
		if (licenseInfo && licenseInfo.floatingSetupStatus) {
			var floatingTypes = licenseInfo.floatingLicenseTypes || [];
        	combined = types.concat(floatingTypes);
		}
		dojo.some(combined, function(type){
			if (type.expirationTime) {
				anyExpiringCALS = true;
				//Find out how many days till it expires
				var expirationInfo = getExpirationInfo(type.expirationTime);
				if (expirationInfo.isExpired) {
					minDaysRemaining = -1;
					return;
				} else {
					var days = expirationInfo.daysRemaining;
					if (days < minDaysRemaining || !minDaysRemaining) {
						minDaysRemaining = days;
					}
				}
			}
		});
		
		// no expiration time for the server license
		if (!expirationTime && !disabledReason) {
			this.expirationInfo.innerHTML = "";
			this.expirationWrapper.style.display = "none";
			
			//check to see if any CALs are expiring
			if (anyExpiringCALS) {
				this._displayExpiringCAL(minDaysRemaining,licenseUrl);
			}
			
		} else if(disabledReason) {
			this.expirationInfo.className = "expirationWarning";
			this.expirationInfo.innerHTML = "<a href='" + licenseUrl + "'>" + disabledReason + "</a>";
		} else {
			var expirationInfo = getExpirationInfo(expirationTime);

			if (anyExpiringCALS && expirationInfo.daysRemaining > minDaysRemaining && !expirationInfo.isExpired) {
				//a CAL expires first
				this._displayExpiringCAL(minDaysRemaining,licenseUrl);
			}
			else {
				//server expires first
				var purchaseLink = "<a href='" + licenseUrl + "'>";
				this.expirationInfo.innerHTML = dojo.string.substitute(isTrial? this.messages.trialLicense : this.messages.permLicense, [purchaseLink, "</a> "]);
				if (expirationInfo.isExpired || expirationInfo.daysRemaining <= 14) {
					this.expirationInfo.className = "expirationWarning";
				}
				else {
					this.expirationInfo.className = "expirationInfo";
				}
				if (expirationInfo.isExpired) {
					//server has expired. see if CAL is also expired
					if(minDaysRemaining == -1) {
						//cal also expired. display comined message
						var calPurchaseLink = "<a href='" + licenseUrl + "'>";	
						this.expirationInfo.innerHTML = dojo.string.substitute(this.messages.bothExpired, [ purchaseLink, "</a> ", calPurchaseLink, "</a> "]);
					} else {
						this.expirationInfo.innerHTML = dojo.string.substitute(this.messages.hasExpired, [this.expirationInfo.innerHTML]);	
					}
				}
				else 
					if (expirationInfo.daysRemaining === 0) {
						this.expirationInfo.innerHTML = dojo.string.substitute(this.messages.expiresToday, [this.expirationInfo.innerHTML]);
					}
					else 
						if (expirationInfo.daysRemaining === 1) {
							this.expirationInfo.innerHTML = dojo.string.substitute(this.messages.expiresInOneDay, [this.expirationInfo.innerHTML]);
						}
						else {
							var daysLeft = expirationInfo.daysRemaining;
							this.expirationInfo.innerHTML = dojo.string.substitute(this.messages.expiresInDays, [this.expirationInfo.innerHTML, daysLeft]);
						}
				this.expirationWrapper.style.display = "inline";
			}
		}
		
	},
	_showLicenseInfoErr: function(errorObj) {
		console.warn(dojo.string.substitute(this.messages.noLicenseInfo, [ errorObj.message ]));
	},
	
	_displayExpiringCAL: function(minDaysRemaining,licenseUrl) {
		var purchaseLink = "<a href='" + licenseUrl + "'>";	
		this.expirationInfo.innerHTML = dojo.string.substitute(this.messages.calLicense, [ purchaseLink, "</a> "]);
		this.expirationInfo.className = (minDaysRemaining < 14) ? "expirationWarning" : "expirationInfo";
				
		if (minDaysRemaining === -1) { //expired
			this.expirationInfo.innerHTML = dojo.string.substitute(this.messages.hasExpired, [this.expirationInfo.innerHTML]);
		} else if (minDaysRemaining === 0) {
			this.expirationInfo.innerHTML = dojo.string.substitute(this.messages.expiresToday, [ this.expirationInfo.innerHTML ]);
		} else if (minDaysRemaining === 1) {
			this.expirationInfo.innerHTML = dojo.string.substitute(this.messages.expiresInOneDay, [ this.expirationInfo.innerHTML ]);
		} else {
			var daysLeft = minDaysRemaining;
			this.expirationInfo.innerHTML = dojo.string.substitute(this.messages.expiresInDays, [ this.expirationInfo.innerHTML, daysLeft ]);
		}
		this.expirationWrapper.style.display = "inline";
	}
	
});

})();