/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.MementoAdapter");

(function() {

/*
 * A serializable memento backed by a javascript object.
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.MementoAdapter", null, {
	
	obj: null,
	onStateChange: null,
	
	/*
	 * Constructs a new memento based on the given object on the
	 * obj property.
	 */
	constructor: function(/* object */ args) {
		dojo.mixin(this, args);
		this.obj = dojo.clone(args.obj);
	},
	
	/*
	 * Returns the value for the given key.
	 */
	get: function(/* string */ key) {
		var obj = this._workingCopy || this.obj;
		return obj[key];
	},
	
	/*
	 * Returns a serializable javascript object containing the state of
	 * the memento.
	 */
	getState: function() {
		return dojo.clone(this.obj);
	},
	
	/*
	 * Sets the value for the given key.
	 */
	set: function(/* string */ key, /* any */ value) {
		var obj = this._workingCopy || this.obj;
		obj[key] = value;
		if (!this._workingCopy) {
			this.onStateChange();
		}
	},
	
	/*
	 * Start editing a temporary copy of the memento. Changes can be
	 * committed or canceled via stopEditing.
	 */
	startEditing: function() {
		this._workingCopy = dojo.clone(this.obj);
	},
	
	/*
	 * Stop editing the temporary copy of the memento and either commit
	 * the changes or discard them.
	 */
	stopEditing: function(/* boolean */ commit) {
		if (commit) {
			this.obj = this._workingCopy;
			this.onStateChange();
		}
		delete this._workingCopy;
	}
});

})();