/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.dashboard.web.ui.internal.MetaNav");

dojo.require("dijit.InlineEditBox");
dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("dojox.xml.parser");

dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.DashboardInitializationData");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.Observable");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.dashboard.web.util.DashboardUtil");

dojo.require("jazz.app.context");
dojo.require("jazz.app.ua.PageHoverHelp");
dojo.require("jazz.app.events");
dojo.require("jazz.client.xhr");
dojo.require("jazz.ui.Breadcrumbs");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.ui.SimpleToolbar");
dojo.require("jazz.util.html");
dojo.require("jazz.util.i18n");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

//
// Aliases
//
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var DashboardInitializationData = com.ibm.team.dashboard.web.ui.internal.DashboardInitializationData;
var HTML = jazz.util.html;
var Observable = com.ibm.team.dashboard.web.ui.internal.util.Observable;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var DashboardUtil = com.ibm.team.dashboard.web.util.DashboardUtil;
var PageHoverHelp = jazz.app.ua.PageHoverHelp;

var messages;

/**
 * The widget for the content area (right side) of the dashboard.
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.MetaNav",
	[dijit._Widget, dijit._Templated, Observable], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/MetaNav.html"),
	widgetsInTemplate: true,
	
	constructor: function(args) {
		messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
	},
	postCreate: function() {
		this.inherited(arguments);
		dojo.addClass(this._dashboardTitle, "loading-title");
		this._dashboardTitle.innerHTML = messages.loading;
		this._toolbar = new jazz.ui.SimpleToolbar({});
		this._headerLinks.appendChild(this._toolbar.domNode);
		this._breadcrumbs = new jazz.ui.Breadcrumbs({}, this._breadcrumbs);
	},
	init: function(args) {		
		if (this.isInitialized) {
			return;
		}
		
		//set dashboardBundle and setup other useful pointers
		this.dashboardBundle = args.dashboardBundle;
		this.dashboardBinding = this.dashboardBundle.dashboard.binding;
		this.dashboardContainer = args.dashboardContainer;
		this.allowSharing = args.allowSharing;
		this.allowSetDefault = args.allowSetDefault;
		
		this._dashboardTitle.innerHTML = "";
		dojo.removeClass(this._dashboardTitle, "loading-title");
		
		this._toolbar.addButton(this.dashboardContainer, this.dashboardContainer.refresh, {imgSrc: dojo.moduleUrl("com.ibm.team.dashboard.web","graphics/MetaNav/icons/refresh.gif"), label: messages.refresh});
		
		this.setEditMode(args.editMode);
		
		//initialize publish API for showing/hiding the message area (info, error, warning)
		this.subscribe(com.ibm.team.dashboard.web.ui.internal.Constants.serverError, "_showErrorMessage");
		this.subscribe(com.ibm.team.dashboard.web.ui.internal.Constants.hideServerError, "hideMessage");
		
		var perms = this.dashboardBundle.permissions;
		if (perms.viewTeam || perms.viewPersonal)
			this._addBreadcrumbs();
		else {
			this._breadcrumbs.domNode.style.display = "none";
			this.dashboardContainer._tabWrapper.style.marginTop = "3px";
		}
		
		// show a notice if scope item is archived
		if (this.dashboardBundle.isScopeArchived) {
			var msg;
			var scope = this.dashboardBundle.scope;
			switch(scope) {
				case DashboardConstants.SCOPE_CONTRIBUTOR:
					msg = messages.archivedContributor;
					break;
				case DashboardConstants.SCOPE_TEAM_AREA:
					msg = messages.archivedTeam;
					break;
				case DashboardConstants.SCOPE_PROJECT_AREA:
					msg = messages.archivedProject;
					break;
			}
			this.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
				summary: msg,
				closable: true
			});
		}
		
		var uakey = jazz.app.context.get().name ? 'com.ibm.team.dashboard.web.dash_proj_team' : 'com.ibm.team.dashboard.web.dash_personal';
		this._helpHover = new PageHoverHelp({"class": "dashboard-hover-help", uaKey: uakey});
		this._helpHover.placeAt(this._dashboardTitle.parentNode, 'last');
	},
	_showErrorMessage: function(errorObj) {
		this.dashboardContainer.errorState = true;
		var args = {
			type: errorObj.type ? errorObj.type : jazz.ui.MessageBox.STATUS_TYPES.ERROR,
			closable: errorObj.closeable
		};
		if (errorObj.status == 409 || errorObj.status == 412) {
			var buf = [];
			var msg = errorObj.customMessage
			if (!msg) {
				var isContributorScope = (this.dashboardBundle.scope == DashboardConstants.SCOPE_CONTRIBUTOR);
				msg = isContributorScope ? messages.errorConflictSelf : messages.errorConflict;
			}
			var parts = jazz.util.i18n.split(msg);
			for (var i=0;i<parts.length;i++) {
				var part = parts[i];
				if (part.id == 0) {
					buf.push("<a href=\"#\" id=\"com-ibm-team-dashboard-web-ui-internal-MetaNav-refreshLink\">");
					buf.push(part.text);
					buf.push("</a>");
				}
				else {
					buf.push(part.text);
				}
			}
			args.summary = buf.join("");
			this.showMessage(args);
			var a = dojo.byId("com-ibm-team-dashboard-web-ui-internal-MetaNav-refreshLink");
			this.connect(a, "onclick", dojo.hitch(this.dashboardContainer, this.dashboardContainer.refresh));
		}
		else if (errorObj.status > 0) {
			var buf = ["HTTP/1.1 ", errorObj.status];
			if (errorObj.statusText) {
				buf.push(" ");
				buf.push(errorObj.statusText);
			}
			args.summary = dojo.string.substitute(messages.errorSave, [buf.join("")]);
			var responseText = errorObj.responseText;
			if (responseText && responseText.length > 0) {
				responseText = responseText.replace(/\r?\n/g, "<br/>");
				responseText = responseText.replace(/\t/g, "&nbsp;&nbsp;&nbsp;&nbsp;");
				args.details = [{message: responseText}];
			}
			args.errorObject = errorObj;
			this.showMessage(args);
		}
		else {
			args.errorObject = errorObj;
			this.showMessage(args);
		}
	},
	_addBreadcrumbs: function() {
		var breadcrumbs = [];
		var defaults = this.dashboardBundle.defaults,
			context = jazz.app.context.get();
		if (context.name) {
			breadcrumbs.push({label: dojo.string.substitute(messages.allProjectDash,[jazz.app.context.get().name]), href: Util.getAllDashboardsHref()});			
		} else {
			breadcrumbs.push({label: messages.allPersonalDashboards, href: Util.getAllPersonalDashboardsHref()});
		}

		if (this.dashboardBundle.scope == DashboardConstants.SCOPE_TEAM_AREA) {
			breadcrumbs.push({label: defaults.project.name,	href: DashboardUtil.getDashboardHref(DashboardConstants.SCOPE_PROJECT_AREA, defaults.project.name)});
			var path = defaults.team.path;
			var curPath = [];
			for (var i=0;i<path.length-1;i++) {
				curPath.push(path[i]);
				breadcrumbs.push({
					label: path[i],
					href: DashboardUtil.getDashboardHref(DashboardConstants.SCOPE_TEAM_AREA, curPath)
				});
			}
		}
		
		this._breadcrumbs.segments = breadcrumbs;
		this._breadcrumbs.refresh();
	},
	destroy: function() {
		if (this._title) {
			this._title.destroy();
			delete this._title;
		}
		if (this._toolbar) {
			this._toolbar.destroy();
			delete this._toolbar;
		}
		if (this._breadcrumbs) {
			this._breadcrumbs.destroy();
			delete this._breadcrumbs;
		}
		if (this._helpHover) {
			this._helpHover.destroy();
			delete this._helpHover;
		}
		this.inherited(arguments);
	},	
	setEditMode: function(editMode) {
		var beforeMode = this.editMode;
		this.editMode = editMode;
		if (editMode != beforeMode) {
			if (editMode == Constants.EDIT_AUTO_SAVE || editMode == Constants.EDIT_DEFERRED_SAVE) {
				if (!this._title) {
					//create editable dashboard title
					var titleStr = this.dashboardBinding.getTitle();
					var node = document.createElement("span");
					node.appendChild(document.createTextNode(titleStr));
					this._dashboardTitle.appendChild(node);
					// value param is workaround for:
					// http://bugs.dojotoolkit.org/ticket/10653
			        this._title = new dijit.InlineEditBox({title:messages.clickToEdit, autoSave: true, width: "400px", noValueIndicator: "?", editorParams: {maxLength: "60"}, onChange: dojo.hitch(this, this._onTitleChange), value: titleStr}, node);
			        this._title.startup();					
				}
				this._toolbar.addSeparator(1);
				this.showSaveControls(true);
				this._setAutoSave();
			}
			else {
				if (this._title) {
					this._title.destroy();
					this._title = null;
				}
				this._dashboardTitle.appendChild(document.createTextNode(this.dashboardBinding.getTitle()));
				this.showSaveControls(false);
				if (typeof(this.beforeMode) != "undefined" && this.beforeMode != Constants.READ_ONLY) 
					this.dashboardContainer.setDirty(false);
			}
		}
	},
	showSaveControls: function (showControls){
		if(!this._editEvents) {
			this._editEvents = new Array();
		}
		if (showControls) {
			var id = new Date().getTime();
			var autoSave = document.createElement("div");
			autoSave.className = "auto-save";
			var autoSaveInput = document.createElement("input");
			autoSaveInput.type = "checkbox";
			autoSaveInput.id = "autoSave-" + id;
			
			var autoSaveLabel = document.createElement("label");
			autoSaveLabel.htmlFor = "autoSave-" + id;
			autoSaveLabel.innerHTML = messages.autoSave;
			autoSave.appendChild(autoSaveInput);
			autoSave.appendChild(autoSaveLabel);
			
			var save = document.createElement("input");
			save.type = "button";
			save.value = messages.save;
			save.className = "saveButton";
			save.disabled = "disabled";

			this._autoSave = this._toolbar.addWidget({domNode: autoSave, autoSaveInput: autoSaveInput});
			this._saveButton = this._toolbar.addWidget({domNode: save});
			if (typeof this.editMode != "undefined" && this.editMode == Constants.EDIT_AUTO_SAVE) 
				this._toolbar.hideItem(this._saveButton);
			
			this._editEvents.push(this.connect(save,"onclick",this._saveEditingHandler));
			this._editEvents.push(this.connect(autoSaveInput,"onclick",this._setAutoSave));
		} else {
			while(this._editEvents && this._editEvents.length > 0) {
				this.disconnect(this._editEvents.pop());	
			}
			if (this._saveButton) {
				this._toolbar.removeItem(this._saveButton);
			}
			if (this._autoSave) {
				this._toolbar.removeItem(this._autoSave);
			}
			this._saveButton = null;
			this._autoSave = null;
		}
		if (!this._separator && this.dashboardBundle && (this.dashboardBundle.permissions["modify"] || this.dashboardBundle.permissions["modify"]))
			this._separator = this._toolbar.addSeparator(0);
		if (this.allowSetDefault && !this._defaultButton && this.dashboardBundle.scope == DashboardConstants.SCOPE_CONTRIBUTOR && this.dashboardBundle && this.dashboardBundle.permissions["modify"]) {
			var isDefault = this.dashboardBundle.isDefaultDashboard;
			this._defaultButton = this._toolbar.addButton(this, this._setDefaultDashboard, {index: 0, imgSrc: dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/Dashboard/icons/default-dash.png"), label: messages[isDefault ? "defaultDisabled" : "default"]});
			if (isDefault) {
				this._toolbar.disableButton(this._defaultButton);
			}
		}
		if (this.allowSharing && !this._shareButton && this.dashboardBundle.scope == DashboardConstants.SCOPE_CONTRIBUTOR && this.dashboardBundle && this.dashboardBundle.permissions["modify"]) {
			this._shareButton = this._toolbar.addButton(this,this._shareDashboard,{imgSrc: dojo.moduleUrl("com.ibm.team.dashboard.web","graphics/Dashboard/icons/shared-dash.png"), imgSrcPressed: dojo.moduleUrl("com.ibm.team.dashboard.web","graphics/Dashboard/icons/unshared-dash.png"), buttonType: jazz.ui.SimpleToolbar.ButtonTypes.TWO_STATE, label: messages.sharing, index: 0});
			var contributorUri = DashboardInitializationData.getAuthenticatedContributorUri();
			var resource = this.dashboardContainer.dashboardResource;
			var isShared = resource.securityContext != contributorUri;
			this._toolbar.setPressed(this._shareButton, isShared);
		}
		if (!this._pasteButton && this.dashboardBundle && this.dashboardBundle.permissions["modify"])
			this._pasteButton = this._toolbar.addButton(this,this._paste,{index: 0, imgSrc: dojo.moduleUrl("com.ibm.team.dashboard.web","graphics/Dashboard/icons/paste_edit.gif"), label: messages.paste});
		if (!this._deleteButton && this.dashboardBundle.scope == DashboardConstants.SCOPE_CONTRIBUTOR && this.dashboardBundle && this.dashboardBundle.permissions["delete"]) {
			this._deleteButton = this._toolbar.addButton(this,this._deleteDashboard,{index: 0, imgSrc: dojo.moduleUrl("com.ibm.team.dashboard.web","graphics/Dashboard/icons/delete.gif"), label: messages.del});
		}
		// Remove the 'Restore Default' button from the dashboard. It will be re-visited as part
		// of enhancement 149850 (Dashboard history and restore previous state)
		//if (!this._restoreButton && this.dashboardBundle && this.dashboardBundle.permissions["modify"]) {
		//	this._restoreButton = this._toolbar.addButton(this,this._restoreDashboard,{index: 0, imgSrc: dojo.moduleUrl("com.ibm.team.dashboard.web","graphics/Dashboard/icons/restore-default-dash.png"), label: messages.restoreDefault});
		//}
		
	},
	
	_paste: function(event) {
		if (event) 
			dojo.stopEvent(event);
		try {
			var data = localStorage["net-jazz-ajax-clipboard"];
		} catch(e) {
			alert(messages.featureNotSupported);
			return;
		}
		if (!data) {
			alert(messages.clipboardEmpty);
			return;
		}
		var document = dojox.xml.parser.parse(data);
		var elem = document.documentElement;
		var ownerDocument = this.dashboardBinding.elem.ownerDocument;
		if (ownerDocument.importNode) {
			elem = ownerDocument.importNode(elem, true);
		}
		if (elem.nodeName.indexOf("Page") != -1) {
			this.dashboardContainer.addPage(elem);
		} else {
			this.dashboardContainer.currentPage.addWidgetFromElement(elem);
		}
	},
	_setAutoSave: function(event) {
		if((event && event.target.checked) || (!event && this.dashboardContainer.autoSave)) {
			if (event) {
				this.dashboardContainer.setEditMode(Constants.EDIT_AUTO_SAVE, true);
				this.editMode = Constants.EDIT_AUTO_SAVE;
			}
			var wasDirty = this.dashboardContainer.isDirty();
			this._toolbar.hideItem(this._saveButton);
			this.dashboardContainer.setDirty(false);
			this.dashboardContainer.autoSave = true;
			this._autoSave.autoSaveInput.checked = "checked";
			if (wasDirty) {
				this.propertyChanged(Constants.PROPERTY_DASHBOARD);
			}
		} else {
			if (event) {
				this.dashboardContainer.setEditMode(Constants.EDIT_DEFERRED_SAVE, true);
				this.editMode = Constants.EDIT_DEFERRED_SAVE;
			}
			this._toolbar.showItem(this._saveButton);
			this.dashboardContainer.autoSave = false;
		}
	},
	_setDefaultDashboard: function() {
		if (confirm(messages.defaultConfirm)) {
			var dashboardUrl = this.dashboardBundle.dashboardUrl;
			// keep the flag in the dashboard bundle up to date, so we can check it for navigate away logic
			this.dashboardBundle.isDefaultDashboard = true;
			var headers = {};
			headers["Content-Type"] = "text/plain";
			jazz.client.xhrPut({
				url: Util.getServiceRoot() + "/dashboard-user-prefs/default-dashboard",
				headers: headers,
				putData: dashboardUrl,
				error: dojo.hitch(this, this._showErrorMessage)
			});
			this._toolbar.disableButton(this._defaultButton);
			this._toolbar.updateButton(this._defaultButton, {label: messages.defaultDisabled});
			dojo.publish(jazz.app.events.HOME_MENU_REFRESH);
			dojo.publish("/jazz/dashboard/setDefault", [dashboardUrl]);
		}
	},
	_saveEditingHandler: function(event){
		if (event) 
			dojo.stopEvent(event);
		this.dashboardContainer.saveDashboard();
		this.dashboardContainer.setDirty(false);
		if (this.editMode == Constants.EDIT_DEFERRED_SAVE) {
			this.dashboardContainer.propertyChanged("title", this.dashboardBinding.getTitle());
		}
	},
	_shareDashboard: function(event) {
		if (event) 
			dojo.stopEvent(event);
		var isShared = !this._toolbar.isPressed(this._shareButton);
		if (this.editMode == Constants.EDIT_DEFERRED_SAVE && this.dashboardContainer.isDirty()) {
			alert(messages.alertShareDirty);
			// undo the toggle
			this._toolbar.setPressed(this._shareButton, isShared);
			return;
		}
		var msg = isShared ? messages.confirmUnshare : messages.confirmShare;
		if (confirm(msg)) {
			var newSecurityContext;
			if (isShared) {
				newSecurityContext = DashboardInitializationData.getAuthenticatedContributorUri();
			}
			else {
				newSecurityContext = Constants.CONTEXT_PUBLIC;
			}
			this.dashboardContainer.dashboardResource.securityContext = newSecurityContext;
			if (this.editMode == Constants.EDIT_AUTO_SAVE) {
				this.propertyChanged(Constants.PROPERTY_DASHBOARD);
				if (!isShared) {
					this._autoSave.autoSaveInput.checked = false;
					this._setAutoSave({target: this._autoSave.autoSaveInput});
				}
			}
			else if (this.editMode == Constants.EDIT_DEFERRED_SAVE) {
				if (!isShared) {
					this.dashboardContainer.setEditMode(Constants.EDIT_AUTO_SAVE, true);
					this.propertyChanged(Constants.PROPERTY_DASHBOARD);
					this.dashboardContainer.setEditMode(Constants.EDIT_DEFERRED_SAVE, true);
				} else {
					this._autoSave.autoSaveInput.checked = "checked";
					this._setAutoSave({ target: this._autoSave.autoSaveInput });
					this.propertyChanged(Constants.PROPERTY_DASHBOARD);
				}
			}
			// broadcast change
			this.dashboardContainer.dashboardWidget.propertyChanged("shared", !isShared);
		}
		else {
			// undo the toggle
			this._toolbar.setPressed(this._shareButton, isShared);
		}
	},
	_deleteDashboard: function(event) {
		if (event) 
			dojo.stopEvent(event);
		var message = dojo.string.substitute(messages.delConfirmPrivate, [this.dashboardBinding.getTitle() || ""]);
		if (!confirm(message))
			return;
		this.dashboardContainer.propertyChanged("ondelete");	
	},
	_restoreDashboard: function(event) {
		if (event) 
			dojo.stopEvent(event);
		var message = dojo.string.substitute(messages.restoreDashboard);
		if (!confirm(message))
			return;
		this.dashboardContainer.restoreDashboard();
	},
	/*
	 * Called when the user changes the dashboard title.
	 */
	_onTitleChange: function(/* string */ newTitle) {
		this.dashboardBinding.setTitle(newTitle);
		this.propertyChanged(Constants.PROPERTY_DASHBOARD);
		if (this.editMode != Constants.EDIT_DEFERRED_SAVE) {
			this.dashboardContainer.propertyChanged("title", newTitle);	
		}
	},
	showMessage: function(args) {
		this._messageBox.showMessage(args);
	},
	hideMessage: function(/* boolean */ fadeAnimation) {
		this._messageBox.hideMessage(fadeAnimation);
		this.dashboardContainer.errorState = false;
	},
	_onChange: function(/* string */ property) {
		// notify listeners that the content has changed
		this.propertyChanged(property);
	},
	setDirty: function (isDirty) {
		if (isDirty && !this.dashboardContainer.autoSave) {
			if (!this.dirtyIcon) {
				var span = document.createElement("span");
				span.innerHTML = "*";
				span.className = "dirtyStar";
				span.title = messages.dashboardModified;
				this._dashboardTitle.appendChild(span);
				this.dirtyIcon = span;	
			}
			this._saveButton.domNode.disabled = "";
		} else {
			if (this.dirtyIcon && this.dirtyIcon.parentNode) {
				this.dirtyIcon.parentNode.removeChild(this.dirtyIcon);
			}
			this.dirtyIcon = null;
			
			if (this._saveButton)
				this._saveButton.domNode.disabled = "disabled";
		}
	}
});

})();
