/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("com.ibm.team.dashboard.web.ui.internal.MigrationStatusPage");

dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.ProgressBar");
dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ServiceTracker");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("jazz.util.i18n");
dojo.require("net.jazz.ajax.BootstrapProperties");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

var BootstrapProperties = net.jazz.ajax.BootstrapProperties;
var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var ProgressBar = dijit.ProgressBar;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServiceTracker = com.ibm.team.dashboard.web.ui.internal.util.ServiceTracker;

var _messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

var REFRESH_DELAY_INITIAL = 2000;
var REFRESH_DELAY_PROGRESS = 9000;

/*
 * An page that appears in the dashboard content area to inform the user about
 * dashboard migration status and to allow migration to be started.
 * 
 * You must supply a handleRefresh function param in the constructor, which
 * will be called when the user clicks to refresh when finished or an error occurs.
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.MigrationStatusPage",
	[dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/MigrationStatusPage.html"),
	loading: _messages.loading,
	handleRefresh: null,
	
	_autoRefresh: false,
	_refreshTimeout: null,
	
	postCreate: function() {
		this._serviceTracker = new ServiceTracker();
		this._scheduleRefresh(0);
	},
	
	destroy: function() {
		// stop refreshing
		this._setAutoRefresh(false);
		// abort any remaining requests
		if (this._serviceTracker) {
			var requests = this._serviceTracker.getActiveRequests();
			for (var i=0;i<requests.length;i++) {
				requests[i].abort();
			}
		}
		if (this._progressBar) {
			this._progressBar.destroyRecursive();
		}
		this.inherited(arguments);
	},
	
	_refresh: function() {
		var srh = new ServiceResponseHandler(this, "_handleRefreshResponse", "_showError");
		DashboardClient.getMigrationStatus(null, srh, this.serviceTracker);
	},
	
	_scheduleRefresh: function(/* number */ time) {
		this._refreshTimeout = setTimeout(dojo.hitch(this, this._refresh), time);
	},
	
	_handleRefreshResponse: function(/* MigrationStatusDTO */ statusDTO) {
		// IE7 clearTimeout doesn't always work; check if it was deleted
		if (!this._refreshTimeout) {
			return;
		}
		delete this._refreshTimeout;
		
		// dispatch to specific handler
		var refreshDelay;
		switch (statusDTO.status) {
			case "NOT_STARTED":
				this._handleNotStarted(statusDTO);
				refreshDelay = REFRESH_DELAY_INITIAL;
				break;
			case "IN_PROGRESS":
				this._handleInProgress(statusDTO);
				this._setAutoRefresh(true); // make sure it's on
				refreshDelay = REFRESH_DELAY_PROGRESS;
				break;
			case "DONE":
				this._handleDone(statusDTO);
				this._setAutoRefresh(false);
				break;
			case "ERROR":
				this._handleError(statusDTO);
				this._setAutoRefresh(false);
				break;
		}
		
		this._currentStatus = statusDTO.status;
		
		// keep refreshing if auto-refresh
		if (this._autoRefresh) {
			this._scheduleRefresh(refreshDelay);
		}
	},
	
	_handleNotStarted: function(/* MigrationStatusDTO */ statusDTO) {
		if (this._currentStatus != "NOT_STARTED") {
			// title
			this.titleDiv.innerHTML = _messages.migrationTitleNotStarted;
			this.titleDiv.className = "title";
			var a = dojo.create("a", {
				href: "#"
			}, this.titleDiv);
			a.innerHTML = ["<img class=\"refresh-button\" src=\"", dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/MetaNav/icons/refresh.gif"), "\">"].join("");
			this.connect(a, "onclick", this._handleRefresh);
			
			// primary message
			dojo.empty(this.contentDiv);
			var p = dojo.create("p");
			p.appendChild(document.createTextNode(_messages.migrationRequiredMsg));
			this.contentDiv.appendChild(p);
			
			// targetted admin/non-admin message
			if (statusDTO.isAdmin) {
				dojo.create("p", {
					innerHTML: "<span class=\"note\">" + _messages.migrationRequiredMsgAdmin1 + "</span>"
				}, this.contentDiv);
				dojo.create("p", {
					innerHTML: _messages.migrationRequiredMsgAdmin2
				}, this.contentDiv);
			}
			else {
				dojo.create("p", {
					innerHTML: _messages.migrationRequiredMsgNonAdmin,
					className: "emphasized"
				}, this.contentDiv);
			}
			
			// button
			var button = dojo.create("button", {
				innerHTML: _messages.migrationButton
			});
			if (!statusDTO.isAdmin) {
				button.disabled = "disabled";
			}
			this.connect(button, "onclick", function(){
				// prevent multiple clicks
				button.disabled = "disabled";
				this._handleClickMigrate();
			});
			this.contentDiv.appendChild(button);
		}
	},
	
	_handleInProgress: function(/* MigrationStatusDTO */ statusDTO) {
		if (this._currentStatus != "IN_PROGRESS") {
			// title
			this.titleDiv.innerHTML = _messages.migrationTitleInProgress;
			this.titleDiv.className = "title";
			
			// progress
			this.contentDiv.innerHTML = [
				"<p>", _messages.migrationProgressMsg, "</p>",
				"<img class=\"inline-icon\" src=\"", dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/Dashboard/icons/spinner.gif"), "\">",
				"<span class=\"subtle\">", _messages.migrationProgress, "</span>"
			].join("");
			
			if (this._progressBar) {
				this._progressBar.destroyRecursive();
			}
			this._progressBar = new ProgressBar({progress: statusDTO.completed || 0, maximum: statusDTO.total || 1});
			dojo.addClass(this._progressBar.domNode, "progressBar");
			this.contentDiv.appendChild(this._progressBar.domNode);
			this._progressBar.startup();
			
			this._progressDetailsDiv = dojo.create("div", {className: "subtle"});
			var buf = ["<span class=\"subtle\">"];
			if (statusDTO.total) {
				// if we know the total, show progress
				buf.push(dojo.string.substitute(_messages.migrationProgress2, [statusDTO.completed, statusDTO.total]));
			}
			else {
				// don't know total yet; show "Initializing..."
				buf.push(_messages.migrationProgressInit);
			}
			buf.push("</span>");
			this._progressDetailsDiv.innerHTML = buf.join("");
			this.contentDiv.appendChild(this._progressDetailsDiv);
		}
		else {
			// refresh existing content
			this._progressBar.update({progress: statusDTO.completed || 0, maximum: statusDTO.total || 1});
			var buf = ["<span class=\"subtle\">"];
			if (statusDTO.total) {
				// if we know the total, show progress
				buf.push(dojo.string.substitute(_messages.migrationProgress2, [statusDTO.completed, statusDTO.total]));
			}
			else {
				// don't know total yet; show "Initializing..."
				buf.push(_messages.migrationProgressInit);
			}
			buf.push("</span>");
			this._progressDetailsDiv.innerHTML = buf.join("");
		}
	},
	
	_handleDone: function(/* MigrationStatusDTO */ statusDTO) {
		// title
		this.titleDiv.innerHTML = [
			"<img class=\"status-icon\" src=\"", dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/Dashboard/images/status-success.png"), "\">",
			_messages.migrationTitleSuccess
		].join("");
		this.titleDiv.className = "title";
		
		// content
		this.contentDiv.innerHTML = [
			"<p>", _messages.migrationSuccessMsg, "<br/>", dojo.string.substitute(_messages.migrationSuccessMsg2, [statusDTO.total, Math.round(statusDTO.duration/1000)]), "</p>"
		].join("");
		
		var a = dojo.create("a", {href: "#"});
		a.innerHTML = [
			"<img class=\"inline-icon\" src=\"", dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/Dashboard/icons/my-dash.png"), "\">",
			_messages.migrationShowDashboard
		].join("");
		this.connect(a, "onclick", this._handleRefresh);
		this.contentDiv.appendChild(a);
	},
	
	_handleError: function(/* MigrationStatusDTO */ statusDTO) {
		// title
		this.titleDiv.innerHTML = [
				"<img class=\"status-icon\" src=\"", dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/Dashboard/images/status-error.png"), "\">",
				_messages.migrationTitleError
		].join("");
		this.titleDiv.className = "title";
		
		// refresh link
		var a = dojo.create("a", {href: "#"}, this.titleDiv);
		a.innerHTML = [
			"<img class=\"refresh-button\" src=\"", dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/MetaNav/icons/refresh.gif"), "\">"
		].join("");
		this.connect(a, "onclick", this._handleRefresh);
		
		// error message
		var buf = [
			"<p>",
			_messages.migrationErrorsMsg, "<br/>"
		];
		buf.push("</p><p>");
		buf.push(statusDTO.isAdmin ? _messages.migrationErrorsMsg3Admin : _messages.migrationErrorsMsg3NonAdmin);
		buf.push("</p>");
		this.contentDiv.innerHTML = buf.join("");
		
		// retry button
		var button = dojo.create("button", {innerHTML: _messages.migrationTryAgain});
		if (!statusDTO.isAdmin) {
			button.disabled = "disabled";
		}
		this.connect(button, "onclick", function() {
			// prevent multiple clicks
			button.disabled = "disabled";
			this._handleClickMigrate();
		});
		this.contentDiv.appendChild(button);
		
		// retry and ignore errors button
		var button2 = dojo.create("button", {innerHTML: _messages.migrationOverride});
		if (!statusDTO.isAdmin) {
			button2.disabled = "disabled";
		}
		this.connect(button2, "onclick", this._handleClickMigrateIgnoreErrors);
		this.contentDiv.appendChild(button2);
		
		// details
		var errors = statusDTO.errors;
		if (errors && errors.length >= 1) {
			var pre = dojo.create("pre", {className: "details"});
			var buf = [
				_messages.migrationErrorDetails,
				"\n"
			];
			for (var i=0;i<errors.length;i++) {
				buf.push("\n");
				buf.push(errors[i]);
			}
			
			// server only sends first 3 errors.. there are probably more
			if (errors.length == 3) {
				buf.push("\n");
				buf.push(_messages.migrationErrorMore);
			}
			
			// IE ignores newlines in pre when using innerHTML
			if (dojo.isIE) {
				pre.innerText = buf.join("");
			}
			else {
				pre.innerHTML = buf.join("");
			}
			this.contentDiv.appendChild(pre);
		}
	},
	
	_handleRefresh: function(e) {
		if (e) {
			dojo.stopEvent(e);
		}
		this.handleRefresh();
	},
	
	_showError: function(/* Error */ e) {
		// populate details
		if (e.status == 401) {
			// assume 401 means popup blocker
			// update title
			this.titleDiv.innerHTML = [
					"<img class=\"status-icon\" src=\"", dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/Dashboard/images/status-warning.png"), "\">",
					_messages.migrationTitlePopupBlocker
			].join("");
			this.titleDiv.className = "title";
			this.contentDiv.innerHTML = e.message;
			var div = dojo.create("div", {className: "actionLinks"}, this.contentDiv);
			var a = dojo.create("a", {href: "#", innerHTML: _messages.tryAgain}, div);
			this.connect(a, "onclick", this._handleRefresh);
			dojo.create("img", {src: dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/Dashboard/icons/action_refresh.gif").toString()}, a, "first");
		}
		else {
			// update title
			this.titleDiv.innerHTML = [
					"<img class=\"status-icon\" src=\"", dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/Dashboard/images/status-error.png"), "\">",
					_messages.migrationTitleError
			].join("");
			this.titleDiv.className = "title";
		
			this.contentDiv.innerHTML = "";
			var parts = jazz.util.i18n.split(_messages.migrationError);
			for (var i=0;i<parts.length;i++) {
				var part = parts[i];
				if (part.id == "0") {
					var a = dojo.create("a", {href: "#", innerHTML: part.text});
					this.connect(a, "onclick", this._handleRefresh);
					this.contentDiv.appendChild(a);
				}
				else {
					this.contentDiv.appendChild(document.createTextNode(part.text));
				}
			}
			
			var pre = dojo.create("pre", {className: "details"});
			var buf = [
				_messages.migrationErrorDetails,
				"\n"
			];
			if (e.status) {
				buf.push("\n");
				buf.push(dojo.string.substitute(_messages.migrationErrorStatus, [e.status]));
			}
			if (e.message) {
				buf.push("\n");
				buf.push(e.message);
			}
			if (e.errorClass) {
				buf.push("\n");
				buf.push("\n");
				buf.push(e.errorClass);
				if (e.exception) {
					for (var i=0;i<e.exception.length;i++) {
						buf.push("\n  at ");
						buf.push(e.exception[i]);
					}
				}
			}
			// IE ignores newlines in pre when using innerHTML
			if (dojo.isIE) {
				pre.innerText = buf.join("");
			}
			else {
				pre.innerHTML = buf.join("");
			}
			this.contentDiv.appendChild(pre);
		}
	},
	
	_setAutoRefresh: function(/* boolean */ value) {
		if (value != this._autoRefresh) {
			if (value === true) {
				this._scheduleRefresh(REFRESH_DELAY_INITIAL);
			}
			else {
				if (this._refreshTimeout) {
					clearTimeout(this._refreshTimeout);
					delete this._refreshTimeout;
				}
			}
			this._autoRefresh = value;
		}
	},
	
	_handleClickMigrate: function() {
		var srh = new ServiceResponseHandler(this, "_handleMigrateResponse", "_showError");
		DashboardClient.migrateDashboards(BootstrapProperties.FRONTSIDE_URL, false, srh, this.serviceTracker);
		this._setAutoRefresh(true);
	},
	
	_handleClickMigrateIgnoreErrors: function() {
		if (confirm(_messages.migrateOverrideWarn)) {
			var srh = new ServiceResponseHandler(this, "_handleMigrateResponse", "_showError");
			DashboardClient.migrateDashboards(BootstrapProperties.FRONTSIDE_URL, true, srh, this.serviceTracker);
			this._setAutoRefresh(true);
		}
	},
	
	_handleMigrateResponse: function() {
	},
	
	_handleOverrideResponse: function() {
		this._handleRefresh();
	}
});

})();
