/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("net.jazz.ajax.ui.internal.OpenSocialJazz");

dojo.require("jazz.util.internal.login");
dojo.require("jazz.ui.underlay");

(function() {

if (!dojo.getObject("gadgets.io.makeRequest"))
	return;

var gio = gadgets.io;

var _baseUrl = (function(){
	var baseURI = document.baseURI;
	if (!baseURI) {
		var baseTags = document.getElementsByTagName('base');
		baseURI = (baseTags && baseTags[0].href) || document.location.href;
	}
	return new dojo._Url(baseURI);
})();
var _baseHost = _baseUrl.scheme + "://" + _baseUrl.authority;

// set the appropriate content related headers before making a gadget xhr call
function addContent(args, params) {
	var headers = args.headers || {};
	// In dojo.xhr, contentType overrides the value specified in headers
	if (args.contentType) {
		headers["Content-Type"] = args.contentType;
	} else {
		headers["Content-Type"] = "application/x-www-form-urlencoded";
	}
	if (args.handleAs && args.handleAs.toLowerCase() == "json") {
		params.CONTENT_TYPE = "JSON";
		headers["Accept"] = "text/json";
	}
	params.HEADERS = headers;
	if (args._jazzXHR) {
		params.AUTHORIZATION = "OAUTH";
		params.REFRESH_INTERVAL = 0;
	}
};

function fixUrl(url) {
	if (url.charAt(0) === "/")
		url = _baseHost + url;
	var result;
	while (url !== (result = url.replace(/\/[^\/]+\/\.\.\//g, "/")))
		url = result;
	return result;
};

var oldXhr = dojo.xhr;

dojo.xhr = function xhr(method, args) {
	var params = {};
	if (args.sync) {
		args.url = gadgets.io.getProxyUrl(fixUrl(args.url));
		return oldXhr(method, args);
	} else {
		addContent(args, params);
		params.METHOD = method;
		switch(method) {
			case "GET":
			case "HEAD":
				params.REFRESH_INTERVAL = 0;
				break;
			case "POST":
				if (args.postData || args.content)
					params.POST_DATA = args.postData || gio.encodeValues(args.content);
				break;
		}
		
		var url = fixUrl(args.url);
		var deferred = new dojo.Deferred();
		deferred.ioArgs = {
			args : args,
			url: args.url,
			xhr: new XHRImpl()
		};
		gio.makeRequest(url,function(response) {
			if (!response.oauthApprovalUrl && response.oauthError)
				deferred.errback(new Error(response.oauthErrorText));
			else
				deferred.callback(response.data || response);
		}, params);
		
		var load = args.load, err = args.error, handle = args.handle;
		if(load && dojo.isFunction(load))
			deferred.addCallback(function(value){
				return load.call(args, value);
			});
		if(err && dojo.isFunction(err))
			deferred.addErrback(function(value){
				return load.call(args, value);
			});
		if(handle && dojo.isFunction(handle))
			deferred.addBoth(function(value){
				return handle.call(args, value);
			});
	
		return deferred;
	}
};

// dummy implementation of XMLHttpRequest with just the methods and fields used in jazz.client.xhr 
function XHRImpl(){
	this.status = 0;
	this.getResponseHeader = function(headerName){
	}
};

var isWinOpen = false;

net.jazz.ajax.ui.internal.OpenSocialJazz = function(request, response, reqMgr) {
	// check to see if the response contained an OAuth authorization request
	var oauthUrlStr = response.dojoResponse
			&& response.dojoResponse.oauthApprovalUrl;
	//TODO check for reponse.dojoResponse.oauthError & oauthErrorText
	if(!oauthUrlStr)
		return false;
	jazz.ui.underlay.show({
		zIndex: 1000,
		animate: false,
		opacity: 1,
		stopEvents: true,
		color: "white"
	});

	var loginDiv = dojo.create("div", {className: "net-jazz-ajax-ui-internal-OpenSocialJazz-login"}, document.body);
	var link = jazz.util.internal.login.insertLoginLink(loginDiv, _baseUrl.authority);
	var linkConnect = dojo.connect(link, "onclick", null, function(e) {
		e.preventDefault();
		if (!isWinOpen) {
			// if there is not already an oauth popup open, then launch a new window
			// for the user to accept or reject the authorization request
			isWinOpen = true;
			var win = window.open(oauthUrlStr, "oauthPopup", "location=1,status=1,scrollbars=1,width=650,height=500");
			function replayRequest () {
				var deferred = jazz.client.xhr(request.method, request.args);
				deferred.addCallback(function (resp) {
					request.remoteDeferred.ioArgs = deferred.ioArgs;
					request.clientDeferred.callback(resp);
				});
				deferred.addErrback(function (err) {
					request.remoteDeferred.ioArgs = deferred.ioArgs;
					request.clientDeferred.errback(err);
				});
			}
			
			function onPopupClose() {
				if(win.closed) {
					clearInterval(onPopupCloseInterval);
					win = null;
					isWinOpen = false;
					dojo.disconnect(linkConnect);
					loginDiv.parentNode.removeChild(loginDiv);
					jazz.ui.underlay.hide();
					replayRequest();
				}
			}
			var onPopupCloseInterval = setInterval(onPopupClose, 100);
		}
	});
	return true;
};

net.jazz.ajax.ui.internal.OpenSocialJazz.RequestInterceptor = function(request) {
	if (request.args)
		request.args._jazzXHR = true;
}

})();
