/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.dashboard.web.ui.internal.PersonalDashboardComponentMenu");

dojo.require("jazz.app._ComponentMenu");
dojo.require("jazz.app.context");
dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.dashboard.web.client.internal.PersonalDashboardClient");
dojo.require("com.ibm.team.dashboard.web.util.DashboardUtil");
dojo.require("com.ibm.team.repository.web.client.session.Session");

dojo.require("jazz.ui.internal.MenuItemGroup");
dojo.require("jazz.util.html");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var PersonalDashboardClient = com.ibm.team.dashboard.web.client.internal.PersonalDashboardClient;
var DashboardUtil = com.ibm.team.dashboard.web.util.DashboardUtil;
var HTML = jazz.util.html;
var messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
var getAuthenticatedContributor = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;
var MenuItemGroup = jazz.ui.internal.MenuItemGroup;
var WrapHints = jazz.ui.MegaMenu.WrapHints;
var RESOURCE_PATH = "/dashboards";

/*
 * The Web UI page (vertical) containing the dashboard and side navigation bar.
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.PersonalDashboardComponentMenu", jazz.app._ComponentMenu, {
	
	postCreate: function() {
		dojo.addClass(this.domNode, "com-ibm-team-dashboard-ComponentMenu");
		this._myDashboards = new MenuItemGroup({
			menu: this,
			groupTitle: messages.currentPersonal
		});
		this._getLeftNavInfo();
		dojo.publish("/jazz/dashboard/initMenu", [this]);
		this.subscribe("/jazz/dashboard/rename", this._renameDashboard); // pass url, new title
		this.subscribe("/jazz/dashboard/delete", this._deleteDashboard); // pass url
		this.subscribe("/jazz/dashboard/setDefault", this._setDefaultDashboard); // pass url
		this.subscribe("/jazz/dashboard/share", this._shareDashboard); // pass url, shared
		this.subscribe("/jazz/dashboard/view", this._addRecent); // pass bundle
		this.subscribe("/jazz/dashboard/create", this._doAddPersonalDashboard); // pass url, title
		this.subscribe("/jazz/dashboard/dirty", this.onDirtyStateChange); // pass bundle and dirty state
	},
	initRecent: function(recent) {
		var items = [];
		outer: for(var i = recent.length - 1; i >= 0; i--) {
			var item = this._createMenuItem(recent[i]);
			for (var j = 0; j < items.length; j++) {
				if (items[j].href == item.href) continue outer;
			}
			items.push(item);
		}
		if (this._recentDashboards) {
			this._recentDashboards.attr("items",items);
			this._updateRecentCount();
		} else if (items.length > 0) {
			this._createRecentSection(items);
		} else {
			return;
		}
		this.updateSelected();
	},
	_createRecentSection: function(items) {
		if (!this._recentDashboards) this._recentDashboards = new MenuItemGroup({
			menu: this,
			wrapHint: WrapHints.FORCE_WRAP,
			groupTitle: "<b>"+messages.recentDashboards+"</b> (" + (items ? items.length : 0) + ")",
			items: items || []
		});
	},
	_addRecent: function(/* object */ bundle) {
		var item = this._createMenuItem(bundle);
		return this._addRecentItem(item);
	},
	_addRecentItem: function(/* object */ item) {
		if (!this._recentDashboards) {
			this._createRecentSection();
		}
		var items = this._recentDashboards.attr("items");
		for (var i = 0; i < items.length; i++) {
			if (items[i].href == item.href) {
				this._recentDashboards.remove(items[i]).destroy();
				break;
			}
		}
		while (items.length > 4) {
			this._recentDashboards.remove(items[0]).destroy();
		}
		this._recentDashboards.addItemByArgs(item,0);
		this._updateRecentCount();
		this.updateSelected();
		return true;
	},
	_createMenuItem: function(/* object */ bundle, /* boolean */ dirty) {
		var dashboard = bundle.dashboard.binding;
		var name = dashboard.getTitle(),
			href = Util.getDashboardHref(bundle),
			iconUrl;
		switch(bundle.scope) {
			case DashboardConstants.SCOPE_CONTRIBUTOR:
				if (bundle.isDefaultDashboard) {
					iconUrl = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/Dashboard/icons/default-dash.png");
				}
				else {
					var securityContext = bundle.dashboard.securityContext;
					if (securityContext) {
						if (securityContext.indexOf(Constants.CONTEXT_PUBLIC) >= 0)
							iconUrl = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/Dashboard/icons/all-dash.png");
						else
							iconUrl = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/Dashboard/icons/my-dash.png");
					}
				}
				break;
			case DashboardConstants.SCOPE_TEAM_AREA:
				iconUrl = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/Dashboard/icons/all-team-dash.png");
				break;
			case DashboardConstants.SCOPE_PROJECT_AREA:
				iconUrl = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/Dashboard/icons/project-dashboard.gif");
				break;
		}
		var label = HTML.escape(name);
		if (dirty) {
			label = "<span class='j-dirty'>* " + label + "</span>";
		}
		return {
			label: label,
			icon: iconUrl,
			href: href,
			url: bundle.dashboardUrl
		};
	},
	_updateRecentCount: function() {
		if (!this._recentDashboards) return 0;
		var length = this._recentDashboards.attr("items").length;
		this._recentDashboards.attr("groupTitle", "<b>"+messages.recentDashboards+"</b> ("+length+")");
		return length;
	},
	onDirtyStateChange: function(/* object */ bundle, /* boolean */ isDirty) {
		if (isDirty) {
			if (this._recentDashboards) {
				var items = this._recentDashboards.attr("items");
				var href = Util.getDashboardHref(bundle);
				for (var i = 0; i < items.length; i++) {
					if (items[i].href == href) {
						this._recentDashboards.remove(items[i]);
						break;
					}
				}
				if (!this._updateRecentCount()) {
					this._recentDashboards.remove();
					this._recentDashboards.destroy();
					delete this._recentDashboards;
				}
			}
			if (!this._unsavedDashboards) {
				this._unsavedDashboards = new MenuItemGroup({
					menu: this,
					wrapHint: this._recentDashboards ? WrapHints.NO_WRAP : WrapHints.FORCE_WRAP,
					groupTitle: "<b>"+messages.unsavedDashboards+"</b> (1)",
					items: [this._createMenuItem(bundle, true)]
				});
			}
		} else if (this._unsavedDashboards) {
			items = this._unsavedDashboards.attr("items");
			if (items && items.length) {
				this._unsavedDashboards.remove(items[0]);
				this._addRecentItem(this._createMenuItem(bundle));
			}
			this._unsavedDashboards.remove();
			this._unsavedDashboards.destroy();
			delete this._unsavedDashboards;
		}
	},
	getSelected: function() {
		var items = this.getItems(),
			sel = [],
			hash = dojo.hash(),
			href = (hash && hash.length > 0) ? hash : location.pathname,
			length = href.length;
		for (var i = 0; i < items.length; i++) {
			var item = items[i];
			if (item && item.href) {
				if (item.href.indexOf(href, item.href.length - href.length) !== -1) {
					sel.push(item);
				}
			}
		}
		return sel;
	},
	_getLeftNavInfo: function() {
		var deferred = PersonalDashboardClient.get();
		deferred.addCallback(dojo.hitch(this, function(/* LeftNavDTO */ navDTO) {
			this._navInfo = navDTO;
			this._populate();
			this.updateSelected();
		}));
		deferred.addErrback(function(errorObj) {
			console.error(Constants.LeftNav.LEFT_NAV_INFO_ERROR + errorObj.message);
		});
	},
	_populate: function() {
		var navDTO = this._navInfo,
			items = [];
		if (navDTO.personalDashboards) {
			for (var i = 0; i < navDTO.personalDashboards.length; i++) {
				var descriptor = navDTO.personalDashboards[i];
				var url = descriptor.url;
				var id = url.substring(url.lastIndexOf("/") + 1);
				var icon = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/Dashboard/icons/" + (descriptor["default"] ? "default-dash.png" : (descriptor.shared ? "shared-dash.png" : "my-dash.png")));
				items.push({
					label: HTML.escape(descriptor.title),
					icon: icon,
					href: DashboardUtil.getDashboardHref(DashboardConstants.SCOPE_CONTRIBUTOR, id),
					url: url
				});
			}
		}
		items.push({
			label: messages.allPersonalDashboards,
			icon: dojo.moduleUrl("com.ibm.team.dashboard.web","graphics/Dashboard/icons/all-dash.png"),
			href: Util.getAllPersonalDashboardsHref(),
			wrapHint: WrapHints.NO_WRAP
		});
		this._myDashboards.attr("items",items);

		if (navDTO.personalPermissions.create) {
			var args = {
				menu: this,
				groupTitle: messages.createDashboard,
				items: [{
					label: messages.createPersonalDashboard,
					icon: dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/Dashboard/icons/create-dash.png"),
					onClick: PersonalDashboardClient.newDashboard
				}]
			}
			if (this._recentDashboards) {
				args.insertIndex = this._recentDashboards.getIndexInParent();
			}
			this._createDashboards = new MenuItemGroup(args);
		}
	},

	_doAddPersonalDashboard: function(/* string */ url, /* string */ title, /* boolean */ shared) {
		var id = url.substring(url.lastIndexOf("/") + 1);
		var index = this._myDashboards.items.length - 1;
		this._myDashboards.addItemByArgs({
			label: HTML.escape(title),
			icon: shared ? dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/Dashboard/icons/shared-dash.png") : dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/Dashboard/icons/my-dash.png"),
			href: DashboardUtil.getDashboardHref(DashboardConstants.SCOPE_CONTRIBUTOR, id),
			url: url
		}, (index >= 0) ? index : null);
	},
	_renameDashboard: function(/* string */ url, /* string */ newTitle) {
		var items = this.findMenuItemsByUrl(url);
		if (items.length > 0) {
			for (var i = 0; i < items.length; i++) {
				items[i].attr("label", HTML.escape(newTitle));
			}
		}
	},
	_deleteDashboard: function(/* string */ url) {
		// if the deleted dashboard was selected, select the default dashboard
		var items = this.findMenuItemsByUrl(url);
		if (items.length > 0) {
			for (var i = 0; i < items.length; i++) {
				if (items[i].__menuItemGroup) {
					items[i].__menuItemGroup.remove(items[i]).destroy();
				} else {
					this.remove(items[i]).destroy();
				}
			}
		}
		var rd = this._recentDashboards;
		var recentItems = rd.attr("items");
		if (!dojo.isArray(recentItems) || recentItems.length == 0) {
			rd.remove();
			rd.destroy();
			this._recentDashboards = null;
		}
		this._updateRecentCount();
		this.updateSelected();
	},
	_setDefaultDashboard: function(/* string */ url) {
		// move new default to top
		item = this.findMenuItemsByUrl(url)[0];
		this._myDashboards.remove(item);
		this._myDashboards.addItemByArgs(item, "start");
		
		// update icons
		var defaultIcon = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/Dashboard/icons/default-dash.png").toString();
		var normalIcon = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/Dashboard/icons/my-dash.png").toString();
		var items = this.getItems();
		for (var i=0;i<items.length;i++) {
			var item = items[i];
			if (item.url == url) {
				// set new default icon
				item.attr("icon", defaultIcon);
			}
			else if (item.icon == defaultIcon) {
				// unset previous default
				item.attr("icon", normalIcon);
			}
		}
	},
	_shareDashboard: function(/* string */ url, /* boolean */ shared) {
		var items = this.findMenuItemsByUrl(url);
		if (items.length > 0) {
			var icon = dojo.moduleUrl("com.ibm.team.dashboard.web", shared ? "graphics/Dashboard/icons/shared-dash.png" : "graphics/Dashboard/icons/my-dash.png");
			for (var i = 0; i < items.length; i++) {
				items[i].attr("icon", icon);
			}
		}
	},
	/* domNode */ findMenuItemsByUrl: function(/* string */ url){
		var items = this._menu.getChildren();
		if (this._recentDashboards) {
			var recentItems = this._recentDashboards.attr("items");
			if (dojo.isArray(recentItems)) {
				items = items.concat(recentItems);
			}
		}
		var returnArr = [];
		for (var i = 0; i < items.length; i++) {
			if (items[i].url == url) {
				returnArr.push(items[i]);
			}
		}
		return returnArr;
	},
	uninitialize: function() {
		if (this._myDashboards) this._myDashboards.destroy();
		if (this._createDashboards) this._createDashboards.destroy();
		if (this._unsavedDashboards) this._unsavedDashboards.destroy();
		if (this._recentDashboards) this._recentDashboards.destroy();
	}
});
})();