/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.PersonalDashboardPage");

dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.dashboard.web.ui.Dashboard");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.AllShared");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.DashboardUpdate");
dojo.require("com.ibm.team.dashboard.web.util.DashboardUtil");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.dashboard.web.client.internal.PersonalDashboardClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("dojo.hash");

dojo.require("jazz.ensemble.internal.client.RestClient");
dojo.require("jazz.app.context");
dojo.require("jazz.app.events");

dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

var AllShared = com.ibm.team.dashboard.web.ui.internal.AllShared;
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var Dashboard = com.ibm.team.dashboard.web.ui.Dashboard;
var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var DashboardUpdate = com.ibm.team.dashboard.web.ui.internal.DashboardUpdate;
var DashboardUtil = com.ibm.team.dashboard.web.util.DashboardUtil;
var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var RestClient = jazz.ensemble.internal.client.RestClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var PersonalDashboardClient = com.ibm.team.dashboard.web.client.internal.PersonalDashboardClient;

var messages;
var recent = [];
var unsaved = true;
var RESOURCE_PATH = "/dashboards";

/*
 * The Web UI page (vertical) containing the dashboard and side navigation bar.
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.PersonalDashboardPage", [dijit._Widget, dijit._Templated], {
	
	templateString: "<div class=\"com-ibm-team-dashboard-web-ui-internal-DashboardPage\"></div>",
	
	constructor: function() {
		if (!messages) {
			messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
		}
	},
	
	postCreate: function() {
		this.subscribe("/jazz/dashboard/initMenu", this._initMenu);

		this._contentDiv = this.domNode;

		// register actions
		var workbench = PlatformUI.getWorkbench();
		var resourceRegistry = workbench.getResourceRegistry();
		resourceRegistry.registerHandlerForPattern(RESOURCE_PATH, this, "viewDashboard");

		var actionRegistry = workbench.getActionRegistry();
		actionRegistry.registerAction("com.ibm.team.dashboard.viewDefaultPersonalDashboard", this, "viewDashboard");
		
		this.navigateAwayHandle = workbench.registerBodyClickHandler(dojo.hitch(this, this._handleNavigate), /*priority*/ 10000);
	},
	
	_handleNavigate: function(urlObj, replace) {
		var winLoc = new dojo._Url(window.location.href);
		if ((!urlObj.host || !urlObj.scheme || (urlObj.host === winLoc.host && urlObj.scheme === winLoc.scheme)) &&   	// if the target is to the same domain and
				urlObj.fragment && ((!urlObj.host && !urlObj.path) || (urlObj.path && urlObj.path === winLoc.path)) &&	// the target is in the web ui and
				this._dashboard instanceof Dashboard) { 																// we have a dashboard showing
			
			winPath = this._getResourcePathFromUrl(winLoc);
			urlPath = this._getResourcePathFromUrl(urlObj);
			// check if the url is targeting the default dashboard
			var isDefault = urlPath.search(/\/dashboards\/?$/) !== -1;
			if (!urlPath || urlPath.indexOf(RESOURCE_PATH) === -1) {	// if the target is not in dashboards
				// if we are leaving dashboards, allow the transition and notify the dashboard
				if (winPath && winPath.indexOf(RESOURCE_PATH) === 0)
					this._dashboard.setActive(false);		
			} else if (((!isDefault && this._currentDashboardPath !== urlPath) || (isDefault && !this._dashboard.getBundle().isDefaultDashboard)) &&	// if the target is in dashboards and is not the current dashboard and
					this._dashboard.isUnsaved()) {																					// the dashboard has unsaved changes	
				var doNavigate = this._dashboard.promptNavigateAway();
				if (!doNavigate) {
					dojo.hash(this._currentDashboardPath, replace);
					return true;
				}
			}
		}
		return false;
	},
	
	_getResourcePathFromUrl: function(urlObj) {
		var hash = urlObj.fragment,
			path = urlObj.path;
		if (hash && hash.length > 0) {
			path = urlObj.fragment;
		}
		// strip off the context and the tab if they are present
		return path.replace(Util.getContextPath(),"").replace(/\/tab[^\/]+\/?$/,"");
	},
	
	_initMenu: function(menu) {
		if (recent) {
			menu.initRecent(recent);
			recent = null;
		}
		if (typeof unsaved === "object") {
			menu.onDirtyStateChange(unsaved, true);
			unsaved = null;
		}
	},
	
	destroy: function() {
		if (this._allDashboards) {
			this._allDashboards.destroy();
		}
		if (this._dashboardUpdate) {
			this._dashboardUpdate.destroy();
		}
		this.destroyDashboard();
		this.inherited(arguments);
	},
	
	destroyDashboard: function() {
		if (this._dashboard) {
			this.disconnect(this._onDirtyStateConnect);
			dojo.publish("/jazz/dashboard/dirty", [this._dashboard.getBundle(), false]);
			PlatformUI.getWorkbench().setPageDirty("com.ibm.team.dashboard.personal",false);
			this._dashboard.destroy();
			delete this._dashboard;
		}
	},
	
	_getInfoFromUrl: function(){
		var url = location.href;
		var urlObj = new dojo._Url(url),
			args = {},
			hash = urlObj.fragment,
			path = urlObj.path;
		if (hash && hash.length > 0) {
			if (path.indexOf(hash, path.length - hash.length) >= 0) { //if path ends with hash
				//strip off the hash and continue
				dojo.hash("",true);
			}
			path = Util.getContextPath() + urlObj.fragment;
		}
		
		// ends with /all?
		if (path.indexOf("/all") == (path.length - 4)) {
			return "all";
		}
		// dashboard update?
		if (path.indexOf("/update") == (path.length - 7)) {
			return "update";
		}
		// default dashboard?
		if (path.search(/\/dashboards\/?$/) != -1) {
			return null;
		}
		// add the resource path without the context
		args.resourcePath = path.replace(Util.getContextPath(),"");
		// extract the tab id
		var matches = path.match(/\/tab([^\/]+)\/?$/);
		if (matches && matches.length >= 1) {
			args.tab = matches[1];
			path = path.replace(/\/tab[^\/]+\/?$/, "");
		}
		// strip tab to get dashboard URL
		args.url = url.replace(/(https?:\/\/[^\/?#]+).*/, "$1" + path);
		return args;
	},
	viewDashboard: function() {
		var args = this._getInfoFromUrl();
		if (!args) {
			this._getDefaultDashboard(true, true);
		} else {
			this._currentDashboardPath = args.resourcePath;
			this._showDashboard(args);
		}
	},
	_getDefaultDashboard: function(/* boolean */ create, /* boolean */ replace) {
		var deferred = PersonalDashboardClient.get();
		deferred.addCallback(dojo.hitch(this, function(info){
			if (info && info.personalDashboards && info.personalDashboards[0]) {
				dojo.hash(this._hashify(info.personalDashboards[0].url), replace);
			} else {
				if (create) {
					// no dashboards; create one automatically
					var dfd = PersonalDashboardClient.createDashboard();
					dfd.addCallback(dojo.hitch(this, function(/* string */ url) {
						dojo.hash(this._hashify(url), replace);
						return url;
					}));
					dfd.addErrback(this, function(/* Error */ e) {
						dojo.hash(Util.getAllPersonalDashboardsHash(), replace);
						return e;
					});
				} else {
					dojo.hash(Util.getAllPersonalDashboardsHash(), replace);
				}
			}
		}));
		deferred.addErrback(dojo.hitch(this,function(e){
			dojo.hash(Util.getAllPersonalDashboardsHash(), replace);
		}));
	},
	_hashify: function(url) {
		return url.substring(url.indexOf(RESOURCE_PATH+"/"));
	},
	_showDashboard: function(args) {
		if (args == "all") {
			this._showAllPersonalDashboards();
			return;
		}
		else if (args == "update") {
			this._showDashboardUpdate();
			return;
		}
		
		if (this._allDashboards) {
			this._allDashboards.destroy();
			delete this._allDashboards;
		}
		if (this._dashboardUpdate) {
			this._dashboardUpdate.destroy();
			delete this._dashboardUpdate;
		}
		
		// check if already showing
		if (this._dashboard) {
			if (this._dashboard.url != args.url) {
				// different dashboard; destroy old one
				this.destroyDashboard();
			} else {
				// same dashboard - pass the current tab, if specified.
				this._dashboard.showTab(args.tab);
			}
		}
		var context = jazz.app.context.get();
		context.dashboard = args.url;
		jazz.app.context.set(context);

		// create new one if needed
		if (!this._dashboard) {
			this._dashboard = new Dashboard({
				url: args.url,
				tab: args.tab,
				allowSetDefault: true,
				allowSharing: true
			});
			this._dashboard.addListener(dojo.hitch(this, this._onLoad), "onload");
			this._dashboard.addListener(dojo.hitch(this, this._onDelete), "ondelete");
			this._dashboard.addListener(dojo.hitch(this, this._onSave), "onsave");
			this._dashboard.addListener(dojo.hitch(this, this._onTabChange), "tab");
			this._dashboard.addListener(dojo.hitch(this, this._onTitleChange), "title");
			this._dashboard.addListener(dojo.hitch(this, this._onSharedChange), "shared");
			this._setContent(this._dashboard);
		}

		if (this._title) {
			// "touch" the title so that it doesn't revert between navigations
			PlatformUI.getWorkbench().setTitle(this._title);
		}	
		this._onDirtyStateConnect = this.connect(this._dashboard,"onDirtyStateChange",this._onDirtyStateChange);
	},
	_showAllPersonalDashboards: function() {
		this.destroyDashboard();
		if (!(this._contentWidget instanceof AllShared)) {
			if (this._allDashboards) {
				this._allDashboards.destroy();
			}
			this._allDashboards = new AllShared();
			this._setContent(this._allDashboards);
			this._updateBrowserTitle();
			var context = jazz.app.context.get();
			context.dashboard = Util.getContextPath(true) + "/dashboards/all";
			jazz.app.context.set(context);
		}
	},
	_showDashboardUpdate: function() {
		this.destroyDashboard();
		if (!(this._contentWidget instanceof DashboardUpdate)) {
			if (this._dashboardUpdate) {
				this._dashboardUpdate.destroy();
			}
			this._dashboardUpdate = new DashboardUpdate();
			this._setContent(this._dashboardUpdate);
			this._updateBrowserTitle();
			var context = jazz.app.context.get();
			context.dashboard = Util.getContextPath(true) + "/dashboards/update";
			jazz.app.context.set(context);
		}
	},
	
	_onTitleChange: function(/* string */ property, /* string */ title) {
		var bundle = this._dashboard.getBundle();
		dojo.publish("/jazz/dashboard/rename",[bundle.dashboardUrl, title]);
		this._updateBrowserTitle();
		this._hmRefresh = true;
	},

	_onSharedChange: function(property, isShared) {
		var url = this._dashboard.getBundle().dashboardUrl;
		dojo.publish("/jazz/dashboard/share", [url, isShared]);
		this._hmRefresh = true;
	},

	_onSave: function() {
		if (this._hmRefresh) {
			dojo.publish(jazz.app.events.HOME_MENU_REFRESH);
			delete this._hmRefresh;
		}
	},
	
	_onDirtyStateChange: function(isDirty) {
		PlatformUI.getWorkbench().setPageDirty("com.ibm.team.dashboard.personal",isDirty);
		var bundle = this._dashboard.getBundle();
		if (unsaved) {
			if (isDirty) unsaved = bundle;
			else if (unsaved.dashboardUrl === bundle.dashboardUrl) unsaved = true;
		}
		dojo.publish("/jazz/dashboard/dirty", [bundle, isDirty]);
	},
	
	_onLoad: function() {
		if (this._dashboard) {
			var bundle = this._dashboard.getBundle();
			dojo.publish("/jazz/dashboard/view", [bundle]);
			if (recent) 
				recent.push(bundle);
			this._updateBrowserTitle();
			this._updateURL();
		}
	},
	
	_onDelete: function() {
		// delete the resource
		var url = this._dashboard.getBundle().dashboardUrl;
		var dfd = RestClient.deleteResource(url);
		dfd.addCallback(dojo.hitch(this, function() {
			// navigate to another dashboard
			dojo.publish("/jazz/dashboard/delete", [url])
			dojo.publish(jazz.app.events.HOME_MENU_REFRESH);
			this._dashboard.setCurrentDeleted();
			this._getDefaultDashboard();
		}));
		dfd.addErrback(dojo.hitch(this, function(/* Error */ e) {
			console.error(e);
		}));
	},
	
	_onTabChange: function(/* string */ propertyId, /* string */ tab) {
		this._updateBrowserTitle();
		this._updateURL();
	},
	
	_setContent: function(/* Widget */ widget) {
		if (this._contentDiv.firstChild != widget.domNode) {
			// swap in new widget
			if (this._contentDiv.firstChild) {
				this._contentDiv.replaceChild(widget.domNode, this._contentDiv.firstChild);
			}
			else {
				this._contentDiv.appendChild(widget.domNode);
			}
			this._contentWidget = widget;
		}
	},
	
	_updateBrowserTitle: function() {
		var contentWidget = this._contentWidget;
		var pageTemplate = dijit.byId("jazz_ui_PageTemplate_0");
		if (contentWidget && contentWidget instanceof Dashboard) {
			var title = contentWidget.getTitle(true);
			pageTemplate.attr("bannerTitle", contentWidget.getTitle());
		} else {
			if (contentWidget && contentWidget instanceof AllShared) {
				title = messages.allPersonalTitle;
			} else if (contentWidget && contentWidget instanceof DashboardUpdate) {
				title = messages.dashboardUpdateTitle;
			} else {
				title = messages.personalDashboards;
			}
			pageTemplate.attr("bannerTitle", title);
		}
		title = dojo.string.substitute(messages.personalDashboardTitle, [title]);
		this._title = title;
		PlatformUI.getWorkbench().setTitle(title);
	},
	_updateURL: function() {
		if (this._dashboard && this._dashboard.getBundle()) {
			var url = this._dashboard.url, 
				id = url.substring(url.lastIndexOf("/") + 1);
			if (id) {
				var href = RESOURCE_PATH + "/" + id,
					tab = this._dashboard.tab;
				if (tab) {
					try {
						var firstPageId = this._dashboard.getBundle().dashboard.binding.getPages().getPages()[0].getId();
					} catch (e) {
						// eat the error and continue.
					}
					var tabParam = "/tab" + tab, hash = dojo.hash();
					if (!firstPageId || firstPageId != tab || hash.indexOf(tabParam, hash.length - tabParam.length) !== -1) { 
						//include the tab param unless we're on the first tab or if first tab param is already on the current url
						href += tabParam;
					}
				}
				var path = location.pathname;
				if (path.indexOf(href, path.length - href.length) !== -1) { // path.endsWith(href)
					//if the path ends with the correct URL, strip off the hash
					dojo.hash() && dojo.hash("");
				} else {
					//overload the hash with the current dashboard url
					dojo.hash(href);
				}
			}
		}
	}
});
})();