/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.process.web.ui.internal.ProcessAreaSelectionDialog"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("jazz.ui.Dialog"); //$NON-NLS-1$
dojo.require("jazz.ui.ModeledTree"); //$NON-NLS-1$
dojo.require("jazz.ui.tree.IconItem"); //$NON-NLS-1$
dojo.require("jazz.util.html"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.client.internal.ProcessClient"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
(function(){

	dojo.setObject("com.ibm.team.process.web.ui.internal.processAreaSelectionDialog", { //$NON-NLS-1$
		ALL_PROJECTS_ID: "allProjects" //$NON-NLS-1$
	});
	
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
	var ALL_PROJECTS_ID = com.ibm.team.process.web.ui.internal.processAreaSelectionDialog.ALL_PROJECTS_ID;
	
	var messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	dojo.declare("com.ibm.team.process.web.ui.internal.ProcessAreaSelectionDialog", null, { //$NON-NLS-1$
		constructor: function(params){
			if (!dojo.isFunction(params.onOk)) {
				throw new Error("Missing function for 'onOk' parameter."); //$NON-NLS-1$
			}
			if (!params.teams && !params.projects) {
				throw new Error("Missing acceptable process area types. Must set 'teams' or 'projects parameter to true."); //$NON-NLS-1$
			} else if (params.teams && !params.projects) {
				var widgetParams = {
					primaryTitle: messages.ProcessAreaSelectionDialog_selectTeam
				};
			} else if (!params.teams && params.projects) {
				widgetParams = {
					primaryTitle: messages.ProcessAreaSelectionDialog_selectProject
				};
			} else {
				widgetParams = {
					primaryTitle: messages.ProcessAreaSelectionDialog_selectArea
				};
			}
			
			var onOk = dojo.hitch(this, function(attributes){
				params.onOk(attributes);
				this.floatingPane.close();
			});
			var onCancel = dojo.hitch(this, function(){
				if (params.onCancel)
					params.onCancel();
				this.floatingPane.close();
			});
			
			this.contentWidget = new com.ibm.team.process.web.ui.internal.ProcessAreaSelectionDialogContent({
				onOk: onOk,
				onCancel: onCancel,
				filterProjectArea: params.filterProjectArea,
				teams: params.teams,
				projects: params.projects,
				selection: params.selection,
				showAllOption: params.showAllOption
			});
			widgetParams.contentNode = this.contentWidget.domNode;
			widgetParams.destroyContents = true;
			this.floatingPane = new jazz.ui.Dialog(widgetParams);
			this.contentWidget.init();
		}
		
	});
	
	dojo.declare("com.ibm.team.process.web.ui.internal.ProcessAreaSelectionDialogContent", //$NON-NLS-1$
 [dijit._Widget, dijit._Templated], {
	
		templateString: '<div style="padding: 1em 1em 1em 1em;">' + //$NON-NLS-1$
		'<div dojoAttachPoint="_treeNode" class="com-ibm-team-process-ProcessAreaSelectionDialog" style="height: 20em; background-color: #FFFFFF; border: 1px solid #CCCCCC;"></div>' + //$NON-NLS-1$
		'<div style="padding: 1em 0em; float: right;">' + //$NON-NLS-1$
		'<button dojoAttachPoint="_okCmd" disabled="true">${_okMessage}</button>' + //$NON-NLS-1$
		'<button dojoAttachPoint="_cancelCmd">${_cancelMessage}</button>' + //$NON-NLS-1$
		'</div>' + //$NON-NLS-1$
		' </div>', //$NON-NLS-1$
		constructor: function(params){
			this._okMessage = messages.ProcessAreaSelectionDialog_ok;
			this._cancelMessage = messages.ProcessAreaSelectionDialog_cancel;
		},
		
		postCreate: function(){
			this.connect(this._cancelCmd, "onclick", this.onCancel); //$NON-NLS-1$
		},
		
		init: function(){
			var srh = new ServiceResponseHandler(this, "_setProcessAreas", "_handleError"); //$NON-NLS-1$ //$NON-NLS-2$
			ProcessClient.getProcessAreaHierarchies(srh);
		},
		
		_createTree: function(dataStore){
			if (this.showAllOption) {
				dataStore.splice(0, 0, {
					name: messages.ProcessAreaSelectionDialog_allProjects,
					itemId: ALL_PROJECTS_ID
				});
			}
			var tree = new jazz.ui.ModeledTree({
				model: dataStore,
				createTreeItem: dojo.hitch(this, function(obj){
					return new jazz.ui.tree.IconItem({
						label: jazz.util.html.escape(obj.name),
						iconClass: obj.itemId == ALL_PROJECTS_ID ? "allProjectsIcon" : //$NON-NLS-1$
 this._projectAreaSet[obj.itemId] ? "projectIcon" : "teamIcon", //$NON-NLS-1$ //$NON-NLS-2$
						getId: dojo.hitch(this, this._getTreeItemId, obj)
					});
				}),
				onDblClick: dojo.hitch(this, this._doOnOk),
				onEnter: dojo.hitch(this, this._doOnOk),
				onSelectionChanged: dojo.hitch(this, this._validateSelection)
			});
			this._tree = tree;
			this._treeNode.appendChild(tree.domNode);
			tree.expandTo(this.selection, this._getTreeItemId);
			tree.setSelection(tree.getItem(this.selection));
			this.connect(this._okCmd, "onclick", this._doOnOk); //$NON-NLS-1$
		},
		
		_getTreeItemId: function(obj){
			return obj.itemId;
		},
		
		_filter: function(processAreaHierarchy){
		
			var filter = this.filterProjectArea;
			if (!filter) {
				return processAreaHierarchy;
			}
			var filtered = [];
			for (var i = 0; i < processAreaHierarchy.length; i++) {
				if (filter && processAreaHierarchy[i].itemId == filter) {
					filtered.push(processAreaHierarchy[i]);
				}
			}
			return filtered.length > 0 ? filtered : processAreaHierarchy;
		},
		
		_sort: function(arr){
			arr.sort(this._compareAreas);
			for (var i = 0; i < arr.length; i++) {
				if (arr[i].children) {
					this._sort(arr[i].children);
				}
			}
		},
		
		_compareAreas: function(a, b){
			return a.name.localeCompare(b.name);
		},
		
		_createProjectAreaSet: function(processAreaHierarchy){
			var set = {};
			if (this.showAllOption) {
				set[ALL_PROJECTS_ID] = true;
			}
			for (var i = 0; i < processAreaHierarchy.length; i++) {
				set[processAreaHierarchy[i].itemId] = true;
			}
			return set;
		},
		_setProcessAreas: function(processAreaHierarchy){
			if (!processAreaHierarchy) {
				this._handleNoProjects();
				return;
			}
			this._projectAreaSet = this._createProjectAreaSet(processAreaHierarchy);
			if (!this.teams && this.projects) {
				for (var i = 0; i < processAreaHierarchy.length; i++) {
					processAreaHierarchy[i].children = [];
				}
			}
			processAreaHierarchy = this._filter(processAreaHierarchy);
			this._sort(processAreaHierarchy);
			this._createTree(processAreaHierarchy);
		},
		
		_handleNoProjects: function() {			 
		 	this._treeNode.innerHTML = "<div class='noProjects'>" + messages.ProcessAreaSelectionDialog_noProjects + "</div>"; //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		_doOnOk: function(){
			var result = this._tree.getSelection();
			if (result && result.length > 0) {
				var id = result[0].getId();
				result = dojo.clone(this._tree.getModelObject(id));
				result.project = this._projectAreaSet[id] ? true : false;
				this.onOk(result);
			}
		},
		
		_validateSelection: function(selection){
			if (this._isValidSelection(selection)) {
				this._okCmd.disabled = false;
			} else {
				this._okCmd.disabled = true;
			}
		},
		
		_isValidSelection: function(selection){
			if (selection && selection.length > 0) {
				var id = selection[0].getId();
				if (this._projectAreaSet[id]) {
					return this.projects;
				} else {
					return this.teams;
				}
			}
			return false;
		},
		
		_handleError: function(error){
			console.error(error);
		},
		
		uninitialize: function() {
			if (this._tree)
				this._tree.destroyRecursive();
			this.inherited(arguments);
		}
		
	});
	
})();
