/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.ProjectsViewlet");

dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");
dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.ServersPreferenceProvider");

dojo.require("jazz.ui.PagedTable");
dojo.require("jazz.ui.ResourceLink");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.util.internal.login");
dojo.require("jazz.util.i18n");
dojo.require("jazz.util.html");
dojo.require("jazz.app.internal.applicationAbout");
dojo.require("jazz.app.internal.util.DiscoveredService");
dojo.require("jazz.app.internal.util.XhrUtil");
dojo.require("jazz.app.internal.schema.HomeMenu");
dojo.require("jazz.ensemble.internal.util.Xml");

dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "ProjectsMessages");

(function(){

var ServersPreferenceProvider = com.ibm.team.dashboard.viewlets.web.ui.internal.utils.ServersPreferenceProvider;

var ResourceLink = jazz.ui.ResourceLink;
var DiscoveredService = jazz.app.internal.util.DiscoveredService;
var XhrUtil = jazz.app.internal.util.XhrUtil;
var HomeMenuSchema = jazz.app.internal.schema.HomeMenu;
var namespace = jazz.ensemble.internal.util.Xml.namespace;
var htmlEscape = jazz.util.html.escape;

var MENU_SECTION_PROJECT   = "http://jazz.net/ns/ui/menuSection#project";
var MENU_USAGE_ALL    = "http://jazz.net/ns/ui/menuUsage#all";

var messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "ProjectsMessages");

dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.ProjectsViewlet", com.ibm.team.dashboard.web.ui.Viewlet, {

	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web", "ui/internal/templates/ProjectsViewlet.html"),

	init: function() {
		if (this.getPreference("showHomeMenuUA") === "true") {
			this._mb = new jazz.ui.MessageBox({
				onClose: dojo.hitch(this,function() {
					this.setPreference("showHomeMenuUA",false);
				})
			});
			this._mb.domNode.style.marginBottom = "5px";
			this._mb.placeAt(this.domNode,"first");
			var imgHtml = "<img style='vertical-align: bottom;' src='" + dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web","graphics/projects/icons/home-menu_obj.gif") + "'/>";
			this._mb.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
				summary: dojo.create("div",{innerHTML: dojo.string.substitute(messages.homeMenuUA,[imgHtml])}),
				closable: true
			});
		}
	},

	refresh: function() {
		this._deletePagedTable();
		dojo.addClass(this._messages,"loading");
		this._messages.innerHTML = messages.loading;
		dojo.style(this._messages,{display: ""});
		this.services = [];
		this.availableDeferred = new dojo.Deferred();
	    var deferred = DiscoveredService.loadDiscoveryServices(namespace.ju+"HomeMenu", HomeMenuSchema.discovery);

		deferred.addCallback(dojo.hitch(this, function(services){
			if (!this.domNode) return;
			dojo.style(this._messages,{display: "none"});
			this._messages.innerHTML = "";
			dojo.removeClass(this._messages,"loading");
			if (!services.HomeMenu)	{
				console.warn("No HomeMenu entries found.");
				return;
			}
			var servers = this.getPreference("servers");
			servers = servers ? dojo.fromJson(servers) : {};
			this.available = [];
			dojo.forEach(services.HomeMenu, function(service){
				if (service.menuSection === MENU_SECTION_PROJECT){
					this.available.push(service);
					if (!servers || !servers[service.contextRoot])
						this.services.push(service);
				}
			},this);
			this.services.sort(function(a, b){if (a.title==b.title) return 0;return (a.title<b.title)?-1:1;});
			this.availableDeferred.callback(this.available);
			if (this.services.length) {
				dojo.forEach(this.services, function(service){
					this._callService(service, true)
				}, this);
			} else {
				this._showNoServers();
			}
		}));
		
		deferred.addErrback(function(error){
			console.warn("[HomeMenu] Service unreachable: ", error);
		}); 
	},
	
	settingsChanged: function(oldSettings, newSettings) {
		if (oldSettings.preferences.servers !== newSettings.preferences.servers) {
			this.getSite().refresh();
		} else if (oldSettings.preferences.items !== newSettings.preferences.items && this.pagedTable) {
			this.pagedTable.setItemsPerPage(newSettings.preferences.items);
			// need to flush the entry cache since it actually contains the domNode, this will not trigger a new server trip
			this.pagedTable.reset(true);
		}
	},
	
	_callService: function(service, passive) {
		var def = XhrUtil.getXml(service.service, {schema:HomeMenuSchema.menuService, passive:passive});
		def.addCallback(function (doc){
			service.results = [];
			service.projectCount = 0;
			if (doc) {
				dojo.forEach(doc.MenuItems, function(item){
					if (!item.usage)
						service.projectCount++;
					if (!item.usage || item.usage === MENU_USAGE_ALL)
						service.results.push(item);
				});
				if (!service.projectCount) {
					service.results.splice(0,0,"noProjects");
				}
			}
			else {
				service.results.push("authRequired");
			}
		});
		def.addErrback(function(error) {
			service.results = ["error"];
			service.projectCount = 0;
			// fail silently if user closed the auth popup
			if (!error || error.dojoType !== "cancel")
				console.error(error);
		});
		def.addBoth(dojo.hitch(this,this._updateUI));
	},
	
	_updateUI: function() {
		if (!this.domNode) return;
		var totalResults = 0, projectCount = 0;
		for (var i = 0; i < this.services.length; i++) {
			if (this.services[i].results) {
				totalResults += this.services[i].results.length;
				if (this.services[i].projectCount)
					projectCount += this.services[i].projectCount;
			}
		}
		if (!this.pagedTable) {
			this.pagedTable = new jazz.ui.PagedTable({
				fetch: dojo.hitch(this,this._fetch),
				renderEntry: function(entry) {return entry}, // the rendering is actually done in fetch
				resultCount: totalResults,
				itemsPerPage: this.getPreference("items")
			});
			this.domNode.appendChild(this.pagedTable.domNode);
		} else {
			this.pagedTable.setResultCount(totalResults);
			this.pagedTable.refresh(true);
		}
		this.getSite().setSecondaryTitle("(" + projectCount + ")");
	},
	
	_fetch: function(startIndex, count) {
		var index = 0, endIndex = startIndex + count, result = [], deferred = new dojo.Deferred(), lastService, row;
		var abouts = jazz.app.internal.applicationAbout.get();
		for (var i = 0; i < this.services.length && index < endIndex; i++) {
			var service = this.services[i]
			if (!service.results)
				continue;
			var about = abouts[service.applicationAbout];
			if (!about || about.error) about = {title: service.contextRoot};
			var offset = 0;
			if (index < startIndex) {
				if (index + service.results.length <= startIndex) {
					index += service.results.length;
					continue;
				} else {
					offset = startIndex - index;
					index = startIndex;
				}
			}
			for (var j = offset; j < service.results.length && index < endIndex; j++) {
				var item = service.results[j];
				var td = dojo.create("td");
				if (lastService !== service.contextRoot) {
					var title = "<span class='main-heading'>" + htmlEscape(about.title) + "</span> (" + htmlEscape(service.publisher) + ")";
					if (service.results.length > 0 && service.results[0] !== "authRequired")
						title += " <span class='sub-heading'>(" + service.projectCount + ")</span>";
					var div = dojo.create("div", {innerHTML: title, classname: lastService ? "heading padded" : "heading"}, td);
					if (this.getSite().isDashboardEditing()) {
						var hideLink = dojo.create("a", {classname: "hide-link", innerHTML: messages.hide, href: "#"}, div, "first");
					}
					lastService = service.contextRoot;
				}
				index++;
				if (item === "authRequired") {
					row = dojo.create("div", {classname: "row login-row", innerHTML: "<img class='icon' src='" + dojo.moduleUrl("jazz.app", "graphics/HomeMenu/login-arrow.png") + "' />"}, td);
					var loginLink = jazz.util.internal.login.insertLoginLink(dojo.create("span",{classname: "label"}, row), false);
				} else if (item === "noProjects") {
					row = dojo.create("div", {classname: "row info-row", innerHTML: messages.noProjects}, td);
				} else if (item === "error") {
					row = dojo.create("div", {classname: "row error-row", innerHTML: messages.serverError}, td);
				} else {
					row = dojo.create("div", {classname: "row",innerHTML: "<img class='icon' src='" + (item.icon || service.defaultIcon || about.icon) + "' />"}, td);
					var span = dojo.create("span", {classname: "label"}, row);
					if (item.usage === MENU_USAGE_ALL /*apply to projects for now too...*/ || !item.usage) {
						span.innerHTML = "<a href='" + item.link + "'>" + htmlEscape(item.title) + "</a>";
					} else {
						var rl = new ResourceLink({uri: item.link, label: item.title});
						row.appendChild(rl.domNode);
					}
					if (index === endIndex && j + 1 < service.results.length) {
						row = dojo.create("div", {classname: "row more", innerHTML: "..."}, td);
					}
				}
				if (dojo.isIE === 7) {
					// programmatic table cell population doesn't work in IE7
					// have to innerHTML it all and then find the links again
					td.innerHTML = td.innerHTML;
					var links = td.getElementsByTagName("a");
					if (hideLink) {
						hideLink = links[0];
					}
					if (loginLink) {
						loginLink = links[hideLink ? 1 : 0];
					}
				}
				if (hideLink) {
					this.connect(hideLink, "onclick", dojo.hitch(this,this._hideServer,service));
				}
				if (loginLink) {
					this.connect(loginLink,"onclick",dojo.hitch(this,function(service,e) {
						e.preventDefault();
						this._callService(service,false);
					}, service));
				}
				hideLink = null; loginLink = null;
				result.push([td]);
			}
		}
		if (row)
			dojo.addClass(row, "bottom-row")
		deferred.callback({entries: result});
		return deferred;
	},
	
	_hideServer: function(service, e) {
		e.preventDefault();
		var servers = this.getPreference("servers");
		servers = servers ? dojo.fromJson(servers) : {};
		servers[service.contextRoot] = true;
		this.setPreference("servers",dojo.toJson(servers));
		for (var i = 0; i < this.services.length; i++) {
			if (service.contextRoot === this.services[i].contextRoot) {
				this.services.splice(i,1);
				break;
			}
		}
		if (this.services.length) {
			this._updateUI();
		} else {
			this._showNoServers();
		}
	},
	
	_showNoServers: function() {
		this._deletePagedTable();
		if (this.available.length) {
			var prefsLink = jazz.util.i18n.addMessage(this._messages, messages.filteredServers)[0];
			this.connect(prefsLink,"onclick",function(e) {
				e.preventDefault();
				this.getSite().toggleSettings();
			});
		} else {
			this._messages.innerHTML = messages.noServers;
		}
		dojo.style(this._messages,{display: ""});
	},
	
	_deletePagedTable: function() {
		if (this.pagedTable) {
			this.domNode.removeChild(this.pagedTable.domNode);
			this.pagedTable.destroy();
			delete this.pagedTable;
		}
	},
	
	getPreferenceProvider: function() {
		if (!this._prefProvider) {
			this._prefProvider = new ServersPreferenceProvider({viewlet: this});
		}
		return this._prefProvider;
	},
	
	_getAvailableServices: function() {
		return this.availableDeferred;
	},
	
	uninitialize: function() {
		if (this.pagedTable) this.pagedTable.destroy();
		if (this._mb) this._mb.destroy();
	}
});

})();
