/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.SearchProvider");

dojo.require("jazz.ui._SearchProvider")

dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.dashboard.web.util.DashboardUtil");
dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("jazz.util.html");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

/*
 * Aliases
 */
var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var DashboardUtil = com.ibm.team.dashboard.web.util.DashboardUtil;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var PlatformUI = net.jazz.ajax.ui.PlatformUI;

var html = jazz.util.html;

var _messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

var SCOPE_ALL = {id: "all", label: _messages.searchScopeAll, icon: dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/Dashboard/icons/all-dash.png").toString()};
var SCOPE_TEAMS = {id: "teams", label: _messages.searchScopeAllTeams, icon: dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/Dashboard/icons/all-team-dash.png").toString()};
var SCOPE_PERSONAL = {id: "contributors", label: _messages.allPersonalDashboards, icon: dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/Dashboard/icons/all-dash.png").toString()};

/*
 * The quick search section at the top of the left nav.
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.SearchProvider", [jazz.ui._SearchProvider], {
	scopes: [SCOPE_ALL, SCOPE_TEAMS, SCOPE_PERSONAL],
	tooltip: _messages.searchTooltip,
	noResultsLabel: _messages.searchNoResultsLabel,
	onUpdate: function(args) {
		if (this._cachedResult && this._cachedResult.query === args.query && this._cachedResult.scope === args.scope) {
			args.callback(this._createResults(this._cachedResult.data, args.maxResults, args.page));
			return;
		}
		// call search service
		var serviceObj = {
			_self: this,
			_query: args.query,
			_callback: args.callback,
			_errorCallback: args.errback,
			success: function(/* ResponseDTO */ responseDTO){
				if (responseDTO.status == "OK") {
					this._self._cachedResult = {
						query: this._query,
						data: responseDTO.data,
						scope: args.scope
					};
					this._callback(this._self._createResults(responseDTO.data, args.maxResults, args.page));
				}
				else {
					this._errorCallback(responseDTO.msg);
				}
			},
			failure: function(/* Error */ error) {
				this._errorCallback(error.message);
			}
		};
		var projectAreaItemId = jazz.app.context.get().itemId;
		var includeTeams = (args.scope != SCOPE_PERSONAL.id);
		var includeContributors = (args.scope != SCOPE_TEAMS.id);
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		DashboardClient.search(args.query, projectAreaItemId, includeTeams, includeContributors, srh);
	},
	
	/* array */ _createResults: function(/* SearchResultsDTO */ resultsDTO, /* number */ maxResults, /* number */ page) {
		var results = [];
		var teams = resultsDTO.teams;
		var shared = resultsDTO.shared;
		var total = 0;
		var offset = (page-1) * maxResults;
		if ((teams && teams.length > 0) || (shared && shared.length > 0)) {
			if (teams) {
				total += teams.length;
				for (var i = offset; i < teams.length && results.length < maxResults; i++)
					results.push(this._createResult(teams[i], DashboardConstants.SCOPE_TEAM_AREA));
				offset += results.length - teams.length;
			}
			if (shared) {
				total += shared.length;
				for (var i=offset;i<shared.length && results.length < maxResults;i++)
					results.push(this._createResult(shared[i], DashboardConstants.SCOPE_CONTRIBUTOR));
			}
		}
		return {
			paging: true,
			totalResults: total,
			results: results
		};
	},

	/* object */ _createResult: function(/* SearchResultDTO */ resultDTO, /* string */ scope) {
		// resultDTO.name for team/project results, title for user results
		var label = resultDTO.name || resultDTO.title, iconUrl;
		if (scope === DashboardConstants.SCOPE_CONTRIBUTOR)
			iconUrl = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/Dashboard/icons/" + (resultDTO.shared ? "shared-dash.png" : "my-dash.png")).toString();
		else
			iconUrl = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/Dashboard/icons/all-team-dash.png").toString();
		var result = { label: label, uri: resultDTO.url || DashboardUtil.getDashboardHref(scope, eval(resultDTO.id)), icon: iconUrl};
		return result;
	}
});
com.ibm.team.dashboard.web.ui.internal.SearchProvider.SCOPE_ALL = SCOPE_ALL;
com.ibm.team.dashboard.web.ui.internal.SearchProvider.SCOPE_TEAMS = SCOPE_TEAMS;
com.ibm.team.dashboard.web.ui.internal.SearchProvider.SCOPE_PERSONAL = SCOPE_PERSONAL;

})();
