/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.ServerStatusViewlet");

dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");

dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "ServerStatusMessages");

(function() {

/*
 * Aliases
 */
var ServiceRequest = com.ibm.team.repository.web.transport.ServiceRequest;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
var Viewlet = com.ibm.team.dashboard.web.ui.Viewlet;

var SERVICE_URI = "com.ibm.team.repository.service.internal.IServerStatusRestService";
var METHOD_GET_SERVER_INFO = "getServerInfo";
var METHOD_GET_SERVER_STATUS = "getServerStatus";

var messages;

/*
 * A viewlet that summarizes the repository server's status.
 */
dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.ServerStatusViewlet",
	Viewlet, {
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web", "ui/internal/templates/ServerStatusViewlet.html"),

	constructor: function() {
		messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "ServerStatusMessages");
		this._loading = messages.loading;
	},

	/*
	 * Returns an appropriate URL for the viewlet's title link.
	 */
	/* string */ getTitleHyperlinkURL: function() {
		return this.getWebURL() + "/admin#action=jazz.viewPage&id=com.ibm.team.repository.server";
	},
	
	refresh: function() {
		this.getSite().setLoading(true);
		delete this._error;
		delete this._serverInfoDTO;
		delete this._serverStatusDTO;
		
		var handleSuccess = dojo.hitch(this, function() {
			if (!this.isDestroyed() && this._serverInfoDTO && this._serverStatusDTO) {
				this.update();
				this.getSite().setLoading(false);
				this.getSite().doneRefresh();
			}
		});
		
		var handleFailure = dojo.hitch(this, function() {
			if (!this.isDestroyed()) {
				this.update();
				this.getSite().setLoading(false);
				this.getSite().doneRefresh();
				this.getSite().error(error);
			}
		});
		
		// fetch server info
		var serviceObj = {
			_viewlet: this,
			success: function(/* ServerInfoDTO */ serverInfoDTO) {
				this._viewlet._serverInfoDTO = serverInfoDTO;
				handleSuccess();
			},
			failure: function(/* Error */ error) {
				this._viewlet._error = error;
				handleFailure();
			}
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		var req = new ServiceRequest(SERVICE_URI, METHOD_GET_SERVER_INFO);
		TeamServerClient.invokeService(req, srh);
		
		// fetch server status
		var serviceObj = {
			_viewlet: this,
			success: function(/* ServerStatusDTO */ serverStatusDTO) {
				this._viewlet._serverStatusDTO = serverStatusDTO;
				handleSuccess();
			},
			failure: function(/* Error */ error) {
				this._viewlet._error = error;
				handleFailure();
			}
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		var req = new ServiceRequest(SERVICE_URI, METHOD_GET_SERVER_STATUS);
		TeamServerClient.invokeService(req, srh);
	},
	
	update: function() {
		if (this._error) {
			var errorSpan = document.createElement("span");
			errorSpan.innerHTML = "<span class=\"error\">" + messages.error + ": " + this._error.message + "</span>";
			this._content.replaceChild(errorSpan, this._content.firstChild);
		}
		else {
			var table = document.createElement("table");
			dijit.setWaiRole(table, "presentation");
			var tbody = document.createElement("tbody");
			this._addRow(messages.database, this._getDB(), tbody);
			this._addRow(messages.services, this._getServices(), tbody);
			this._addRow(messages.memory, this._getMemory(), tbody);
			this._addRow(messages.version, this._getVersion(), tbody);
			this._addRow(messages.uptime, this._getUptime(), tbody);
			table.appendChild(tbody);
			this._content.replaceChild(table, this._content.firstChild);
		}
	},
	
	_addRow: function(/* string */ label, /* string|Node */ content, /* Node */ tbody) {
		var tr = document.createElement("tr");
		var td = document.createElement("td");
		td.className = "label";
		td.innerHTML = label + ":&nbsp;&nbsp;";
		tr.appendChild(td);
		td = document.createElement("td");
		if (typeof(content) == "string") {
			td.innerHTML = content;
		}
		else {
			td.appendChild(content);
		}
		tr.appendChild(td);
		tbody.appendChild(tr);
	},
	
	_formatSizeInBytes: function(/* number */ size) {
		size = Math.floor(size/1024);
		var kb = size % 1024;
		size = Math.floor(size/1024);
		var mb = size;
		return mb < 1 ? kb + messages.kilobyte : mb + messages.megabyte;
	},

	_getDB: function() {
		if (this._serverInfoDTO.databaseAvailable) {
			return "<span class=\"ok-status\">" + messages.connected + "</span>";
		}
		else {
			return "<a class=\"error-status\" href=\"" + this.getWebURL() + "/admin\">" + messages.unavailable + "</a>";
		}
	},
	
	_getMemory: function() {
		var info = this._serverInfoDTO;
		var percent = Math.round(info.freeMemory*100 / info.totalMemory);
		var allocated = this._formatSizeInBytes(info.totalMemory);
		var max = this._formatSizeInBytes(info.maxMemory);
		return dojo.string.substitute(messages.memoryInfo, [ percent, allocated, max ]);
	},
	
	_getServices: function() {
		// number of failing services
		if (!this._serverStatusDTO.servicesInitSuccess) {
			return "<span class=\"ok-status\">" + messages.ok + "</span>";
		}
		else {
			return "<a class=\"error-status\" href=\"" + this.getWebURL() + "/admin\">" + messages.failures + "</a>";
		}
	},
	
	_getUptime: function() {
		var t = this._serverInfoDTO.uptime;
		t = Math.floor(t/1000);
		t = Math.floor(t/60);
		if (t < 1){
			return messages.lessThanOne;
		}
		var m = t % 60;
		t = Math.floor(t/60);
		var h = t % 24;
		t = Math.floor(t/24);
		var d = t;
		var str = "";
		if (d > 0) {
			str += d + " " + (d == 1 ? messages.day : messages.days);
		}
		if (h > 0) {
			str += (str.length > 0 ? ", " : "") + h + " " + (h == 1 ? messages.hour : messages.hours);
		}
		if (m > 0 && d == 0) {
			str += (str.length > 0 ? ", " : "") + m + " " + (m == 1 ? messages.minute : messages.minutes);
		}
		return str;
	},
	
	_getVersion: function() {
		var version = this._serverInfoDTO.version;
		var buildId = this._serverInfoDTO.buildId;
		if (version && buildId) {
			// both are available
			return version + " (" + buildId + ")";
		}
		else if (version) {
			// only version available
			return version;
		}
		else if (buildId) {
			// only build id available
			return buildId;
		}
		else {
			// none available
			return messages.notAvailable;
		}
	}
});

})();
