/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2010, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.StaticViewlet");

/*
 * Imports 
 */
dojo.require("dojo.i18n"); 
dojo.require("dijit.Editor");

dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");
dojo.require("jazz.util.i18n");

dojo.require("jazz.util.html");

// Editor Plug-ins
dojo.require("dijit._editor.plugins.TextColor");
dojo.require("dijit._editor.plugins.LinkDialog");
dojo.require("dijit._editor.plugins.ViewSource");
dojo.require("dijit._editor.plugins.FullScreen");
dojo.require("dojox.editor.plugins.PrettyPrint");
dojo.require("dijit._editor.plugins.ToggleDir");

dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "StaticMessages");

(function() {

/*
 * Aliases 
 */
var Editor = dijit.Editor;
var Viewlet = com.ibm.team.dashboard.web.ui.Viewlet;
var I18nUtil = jazz.util.i18n;

/*
 * Constants 
 */
var RICH_TEXT_EDITOR_ID = "sh_rte_";
var STATE_HTML = "html";

/*
 * TODOs
 */

/*
 * Class 
 */
dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.StaticViewlet",
	Viewlet, {

	// Properties //----------------------------------------------------------->

	// ------------------------------------------------------------------------>

	/*
	 * Templates
	 */		
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web", "ui/internal/templates/StaticViewlet.html"),

	/*
 	 * @param Object args
 	 * 
 	 * @return StaticViewlet
 	 */
	constructor: function(args) {
		
		// Messages //------------------------------------------------------------->
		
		this.messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "StaticMessages");
		
	},
		
	/*
 	 * @param void
 	 * 
 	 * @overrides super
 	 * @return void
 	 */	
	init: function() {

		this.inherited(arguments);
		
		// Instance Variables //--------------------------------------------------->

		this._editor = null;
		this._defaultHTMLContent = "<div style=\"margin:5px;font-size:10pt;\">\n   " +
									this.messages.msgDefineContentHere +
									"\n</div>";

		// Initialization //--------------------------------------------------->
		
		this._loadDisplayArea();

	},	
	
	/*
 	 * @param void
 	 * 
 	 * @return boolean
	 * @overrides super
 	 */		
	validateEditedContent: function() {
		// Get the edited contents
		var html = this._editor.attr("value");
		// Check for insecure content
		if (jazz.util.html.hasScript(html)) {
			alert(this.messages.errorScript);
			return false;
		}
		return true;
	},		
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	preSaveState: function() {
		// Save button was pressed in edit mode
		// Update the display HTML using the contents of the editor
		var html = this._editor.attr("value");
		// Save the display HTML
		this.getViewletState().set(STATE_HTML, html);
	},	
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	update: function() {
		// Determine whether we are entering or leaving edit mode
		if (this.isEditing()) {
			// Enter
			// Load the editor area
			this._loadEditorArea();
		} else {
			// Leave
			// Load the display HTML
			this._loadDisplayArea();
		}
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_loadEditorArea: function() {

		// Create the Editor
		this._createEditor();
		
		// Get the saved HTML contents
		var html = this.getViewletState().get(STATE_HTML);
		// Declare some initial content if there is no html defined
		if (!html) {
			html = this._defaultHTMLContent;
		}
		
		// User can bypass script check on save, so always sanitize
		html = jazz.util.html.removeScript(html);
		
		// Update editor contents   
		this._editor.attr("value", html);

		this._updateVisibility(this._displayArea, false);
		this._updateVisibility(this._editorArea, true);
		
	},		

	/*
 	 * @param String html
 	 * 
 	 * @return String
 	 */          
	_sanitizeContent: function(html) {
		var sanitizedHTML = jazz.util.html.removeScript(html);
		return sanitizedHTML;
	},		
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */          
	_createEditor: function() {
	
		if (this._editor) {
			return;
		}

		this._editor = new Editor({
			id: this._getRichTextEditorID(),
			height: "200px",
			plugins: [],
			contentPostFilters: [this._sanitizeContent],
			contentPreFilters: [this._sanitizeContent],
			extraPlugins: [
				{
					name: 'viewsource'
				},			
				{
					name: 'fullscreen',
					zIndex: 999
				},				
				{
					name: '|'
				},				
				{
					name: 'bold'
				},				
				{
					name: 'italic'
				},				
				{
					name: 'underline'
				},				
				{
					name: '|'
				},				
				{
					name: 'foreColor'
				},				
				{
					name: 'hiliteColor'
				},				
				{
					name: '|'
				},				
				{
					name: 'justifyLeft'
				},				
				{
					name: 'justifyCenter'
				},				
				{
					name: 'justifyRight'
				},				
				{
					name: '|'
				},				
				{
					name: 'insertOrderedList'
				},				
				{
					name: 'insertUnorderedList'
				},				
				{
					name: '|'
				},				
				{
					name: 'indent'
				},				
				{
					name: 'outdent'
				},				
				{
					name: '|'
				},				
				{
					name: 'createLink'
				},				
				{
					name: 'insertImage'
				},				
				{
					name: 'prettyprint'
				},
				{
					name: '|'
				},				
				{
					name: 'toggleDir' 
				}		
			]
 		}, this._editorArea);

	},		



	/*
 	 * @param void
 	 * 
 	 * @return String
 	 */	
	_getRichTextEditorID: function() {
		return RICH_TEXT_EDITOR_ID + this.id;	
	},	
	
	/*
 	 * @param Element node
 	 * @param boolean show
 	 * 
 	 * @return void
 	 */	
	_updateVisibility: function(node, show) {
		if (show) {
			node.style.display = "block";
		} else {
			node.style.display = "none";
		}
	},		

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_loadDisplayArea: function() {
		
		var html = this.getViewletState().get(STATE_HTML);
		var div;

		if (!html) {
			div = this._getActionEditContents();			
		} else {
			div = document.createElement("div");
			// User can bypass script check on save, so always sanitize
			div.innerHTML = jazz.util.html.removeScript(html);
		}

		this._setHTMLNode(this._displayArea, div);

		this._updateVisibility(this._editorArea, false);		
		this._updateVisibility(this._displayArea, true);

	},
	
	/*
 	 * @param void
 	 * 
 	 * @return Element
 	 */	
	_getActionEditContents: function() {

		var div = document.createElement("div");
		div.style.padding = "5px";

		if (this.getSite().isDashboardEditing()) {
			// The user has permission to edit the Dashboard
			// Add action
			var links = I18nUtil.addMessage(div, this.messages.msgNoContentDefinedAction);
			var link = links[0];
			link.href = "#";
			this.connect(link, "onclick", this._handleEventOnClickEditContent);		

		} else {
			// The user does not have the permission to edit the Dashboard
			// Do not add action
			var fallbackMessage = document.createTextNode(this.messages.msgNoContentDefined);
			div.appendChild(fallbackMessage);			
		}
		
		return div;
	},	

	/*
 	 * @param Event event
 	 * 
 	 * @return void
 	 */		
	_handleEventOnClickEditContent: function(event) {
		if (event) {
			dojo.stopEvent(event);
		}
		
		this.getSite().editContents();
	},	
	
	/*
 	 * @param Element parent
 	 * @param Element child
 	 * 
 	 * @return void
 	 */		
	_setHTMLNode: function(parent, child) {

		if (parent.firstChild) {
			parent.replaceChild(child, parent.firstChild);
		} else {
			parent.appendChild(child);
		}
	},	
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */	
	refresh: function() {
		this.getSite().doneRefresh();
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	destroy: function() {
		
		if (this._editor) {
			this._editor.destroy();
			this._editor = null;
		}
		
		this.inherited(arguments);
	}	

});


})();

