/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.StyledTableItemList"); //$NON-NLS-1$
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dijit._Templated"); //$NON-NLS-1$
	dojo.require("dojox.xml.parser"); //$NON-NLS-1$
	dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$
	dojo.require("jazz.ui.StyledTable"); //$NON-NLS-1$
	dojo.require("jazz.app.ua.PageHoverHelp"); //$NON-NLS-1$
	dojo.require("jazz.ui.Toolbar"); //$NON-NLS-1$	
	dojo.require("jazz.ui.toolbar.Button"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper"); //$NON-NLS-1$
	
	function isNode(o){
		return o.nodeType && o.cloneNode;
	}
	
	function getRow(evt){
		var e = evt.target;
		while (e.parentNode) {
			if (e.tagName === "TR") { //$NON-NLS-1$
				return e;
			}
			e = e.parentNode;
		}
		return null;
	}
	
	dojo.declare("com.ibm.team.process.web.ui.internal.StyledTableItemList", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/templates/StyledTableItemList.html"), //$NON-NLS-1$ //$NON-NLS-2$
		_connectHandles: null,
		constructor: function(args) {
			this._connectHandles = [];
		},
		
		addLink: function(actionHref, label, imgSrc, onClickHandler){
			
			if (!this._actionToolbar) {
				this._actionToolbar = new jazz.ui.Toolbar();
			}
			var button = new jazz.ui.toolbar.Button({
				href : actionHref,
				imgSrc :  imgSrc,
				label : label,
				onClick : onClickHandler !== undefined ? onClickHandler : this.onClick
				
			});
			button.setDisabled = dojo.hitch(button, function(disable) {
				this.attr("enabled", !disable); //$NON-NLS-1$
			});
			this._actionToolbar.addChild(button);
			this.linkArea.appendChild(this._actionToolbar.domNode);
			return button;
		},
				
		createTable: function(styledTableArgs){
			
			this._destroyStyledTable();
			var defaultArgs = {
				trim: jazz.ui.StyledTable.TrimStyles.PRIMARY_HEADLESS,
				showHover: true
			};
			dojo.mixin()
			this.styledTable = new jazz.ui.StyledTable(dojo.mixin(defaultArgs, styledTableArgs));
			this.tbody = this.styledTable.tbody;
			this.tableContents.appendChild(this.styledTable.domNode);
			
		},
		
		addRow: function(actionHref, /*Array*/ columns){
			var row = document.createElement("tr"); //$NON-NLS-1$
			if (actionHref != null) {
				row.__jazzActionHref = "#action=" + actionHref; //$NON-NLS-1$
			}
			this.tbody.appendChild(row);
			for (var i = 0; i < columns.length; i++) {
				var td = row.insertCell(i);
				if (i === 0 && row.__jazzActionHref != null) {
					var link = document.createElement("a"); //$NON-NLS-1$
					link.href = row.__jazzActionHref;
					this._connectHandles.push(dojo.connect(link, "onfocus", this._handleFocus)); //$NON-NLS-1$
					this._connectHandles.push(dojo.connect(link, "onblur", this._handleBlur)); //$NON-NLS-1$
					td.appendChild(link);
					td = link;
				}
				var o = columns[i];
				if (dojo.isString(o)) {
					td.appendChild(document.createTextNode(o));
				} else if (isNode(o)) {
					td.appendChild(o);
				}
			}
			return row;
		},
			
		handleError: function(errorObj){
            UIHelper.runSafe(dojo.hitch(this, function(){
            
            	this.header.showMessage({
            		errorObject: errorObj
            	});
            }));
		},
		
		setHelpHooks: function(uaKey){
			if(uaKey){
				this.helpWidget = new jazz.app.ua.PageHoverHelp({
					uaKey: uaKey
				});
				this.helpHooks.appendChild(this.helpWidget.domNode);
			}
		},	
				
		_handleFocus: function(e){
			var row = getRow(e);
			if (row && (!row.collapsable || e.target.tagName === "A")) { //$NON-NLS-1$
				dojo.toggleClass(row, "selected", true); //$NON-NLS-1$
			}
		},
		
		_handleBlur: function(e){
			var row = getRow(e);
			if (row && (!row.collapsable || e.target.tagName === "A")) { //$NON-NLS-1$
				dojo.toggleClass(row, "selected", false); //$NON-NLS-1$
			}
		},
		
		_destroyStyledTable: function() { 
			if (this.styledTable) {
				this.styledTable.destroyRecursive();
				dojox.xml.parser.removeChildren(this.tableContents);
				delete this.styledTable;
				delete this.tbody;
			}		
		},
		
		_disconnectHandles: function(){
			dojo.forEach(this._connectHandles, function(handle) {
				dojo.disconnect(handle);
			});
			this._connectHandles = [];
		},
		
		uninitialize: function(){
			this._disconnectHandles();
			this._destroyStyledTable();
			if (this._actionToolbar) {
				this._actionToolbar.destroyRecursive();
				delete this._actionToolbar;
			}
		}
		
	});
	
})();
