/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.ViewletAdapter");

dojo.require("dijit._Templated");

dojo.require("dojo.i18n"); 
dojo.require("dojo.string");

dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");
dojo.require("com.ibm.team.dashboard.web.ui.internal.ViewletSiteAdapter");
dojo.require("com.ibm.team.dashboard.web.ui.internal.settings.ScopePreferenceProvider");
dojo.require("com.ibm.team.dashboard.web.ui.internal.settings.ViewletSettingsPanel");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.FilterPanel");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ErrorReport");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.IFrameContainer");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ItemNameRegistry");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.Observable");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ServiceTracker");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ViewletRegistry");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("jazz.app.proxy");
dojo.require("jazz.core.loader");
dojo.require("jazz.ensemble.Widget");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var ErrorReport = com.ibm.team.dashboard.web.ui.internal.util.ErrorReport;
var IFrameContainer = com.ibm.team.dashboard.web.ui.internal.util.IFrameContainer;
var ItemNameRegistry = com.ibm.team.dashboard.web.ui.internal.util.ItemNameRegistry;
var Observable = com.ibm.team.dashboard.web.ui.internal.util.Observable;
var ScopePreferenceProvider = com.ibm.team.dashboard.web.ui.internal.settings.ScopePreferenceProvider;
var ViewletSettingsPanel = com.ibm.team.dashboard.web.ui.internal.settings.ViewletSettingsPanel;
var FilterPanel = com.ibm.team.dashboard.web.ui.internal.util.FilterPanel;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServiceTracker = com.ibm.team.dashboard.web.ui.internal.util.ServiceTracker;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var Viewlet = com.ibm.team.dashboard.web.ui.Viewlet;
var ViewletRegistry = com.ibm.team.dashboard.web.ui.internal.util.ViewletRegistry;
var ViewletSiteAdapter = com.ibm.team.dashboard.web.ui.internal.ViewletSiteAdapter;
var Widget = jazz.ensemble.Widget;

var STATE_LOADING = 0;
var STATE_LOGIN = 1;
var STATE_ACTIVE = 2;
var STATE_ERROR = 3;

/*
 * For n-1 compatibility, we override these methods in the viewlet base
 * class with this server's implementation to ensure that we have the latest
 * code. The old code viewlet code assumed the existence of a viewlet helper,
 * etc, which no longer exists.
 */
var VIEWLET_OVERRIDE_METHODS = {
	getContributorState: true,
	getViewletState: true,
	hasContributorState: true,
	getPreference: true,
	getScope: true,
	getScopeItem: true,
	getScopeSubteams: true
};

var messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

/*
 * (static) Observables to listen for remote viewlet log-in, to allow viewlets
 * from the same origin to load as soon as one logs in.
 */
var _loginTrackersByOrigin;

/*
 * Adapts Viewlets to function as Widgets, allowing them to be placed in a
 * widget panel.
 * 
 * Viewlet URIs follow the scheme described at:
 * https://jazz.net/wiki/bin/view/Main/ViewletDefinitionURIs
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.ViewletAdapter", [Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/ViewletAdapter.html"),
	
	local: null,
	viewlet: null,
	
	_animation: null,
	_messages: messages,
	_isDestroyed: false,
	_settingsPanel: null,
	_serviceRoot: null,
    _serviceTracker: null,
	_viewletDefinition: null,
	_webUIRoot: null,
	_yieldTimeout: null,
	
	constructor: function(args) {
		var definitionUrl = args.definition;
		this._webUIRoot = Util.getWebUIRoot(definitionUrl);
		this._serviceRoot = Util.getServiceRoot(definitionUrl);
		this.local = Util.isLocalDefinition(definitionUrl);
	},
	
	postCreate: function() {
		this._icon = this.context.getIcon();
		if (!this.local) {
			var toolbar = this.context.getToolbar();
			if (toolbar) {
				var origin = this._webUIRoot;
				this._shortOrigin = origin;
				this._remoteIcon = toolbar.addWidget({
					domNode: dojo.create("div",{
						"class": "com-ibm-team-dashboard-web-ui-ViewletAdapter-remote-icon-disconnected",
						title: dojo.string.substitute(messages.disconnected,[messages.remoteServer,origin])
					})
				}, {index: 0});
			}
		}
		// yield
		this._yieldTimeout = setTimeout(dojo.hitch(this, this._loadViewlet, true), 0);
	},
	
	getDefaultTitle: function() {
		if (this._viewletDefinition)
			return this._viewletDefinition.name;
		return this.inherited(arguments);
	},
	
	getHeight: function() {
		return dojo.style(this._contentWrapper, "height");
	},
	
	/*
	 * Returns the CustomPreferenceProvider implementation class for the special
	 * viewlet scope preference. In the iframed remote viewlet case, this is a
	 * class from inside the iframe.
	 */
	getScopePreferenceProviderClass: function() {
		if (this._iframeContainer) {
			return this._iframeContainer.getScopePreferenceProviderClass();
		}
		return ScopePreferenceProvider;
	},
	
	getWidth: function() {
		return dojo.style(this._contentWrapper, "width");
	},
	
	/*
	 * Returns the window containing the viewlet.
	 */
	getWindow: function() {
		if (this._iframeContainer) {
			var iframe = this._iframeContainer.getIFrame();
			if (iframe) {
				return iframe.contentWindow;
			}
		}
		return window;
	},
	
	handleEditContent: function() {
		if (!this.viewlet.isEditing()) {
			this._startEditing();
		}
	},
	
	/*
	 * @see _Widget#onShow
	 */
	onShow: function() {
		this._setActive(true);
	},
	
	/*
	 * @see _Widget#onHide
	 */
	onHide: function() {
		this._setActive(false);
	},
	
	refresh: function() {
		this.viewlet && this.viewlet.refresh();
	},
	
	setAutoRefresh: function(/* boolean */ value) {
		if (this._viewletDefinition.refreshInterval > 0) {
			if (value && !this._refreshTimeout) {
				var timeSinceLastRefresh = new Date().getTime() - this._lastRefreshTime;
				var timeUntilNextRefresh = (this._viewletDefinition.refreshInterval * 1000) - timeSinceLastRefresh;
				// always wait at least 5 sec before an auto-refresh to avoid disruptions
				if (timeUntilNextRefresh < 5000) {
					timeUntilNextRefresh = 5000;
				}
				// resume auto-refresh
				this._refreshTimeout = setTimeout(dojo.hitch(this.viewlet, "refresh"), timeUntilNextRefresh);
			}
			else if (!value && this._refreshTimeout) {
				// suspend auto-refresh
				if (this._refreshTimeout) {
					clearTimeout(this._refreshTimeout);
					delete this._refreshTimeout;
				}
			}
		}
	},
	
	toggleSettings: function() {
		if (!this._settingsPanel) {
			this._openSettings();
		}
		else {
			this._settingsPanel.close(dojo.hitch(this, this._handleSettingsClose));
		}
	},
	
	uninitialize: function() {
		// kill auto-refresh
		if (this._refreshTimeout) {
			clearTimeout(this._refreshTimeout);
		}
		// kill any remaining timeouts
		if (this._yieldTimeout) {
			// cancel yield
			clearTimeout(this._yieldTimeout);
		}
		// abort any remaining requests
		if (this._serviceTracker) {
			var requests = this._serviceTracker.getActiveRequests();
			for (var i=0;i<requests.length;i++) {
				requests[i].abort();
			}
		}
		// set inactive
		this._setActive(false);
		// destroy the widgets
		if (this.viewlet) {
			try {
				this.viewlet.destroyRecursive();
			} catch (e) {
				// iframe may be in the process of unloading; ignore
			}
		}
		if (this._iframeContainer) {
			this._iframeContainer.destroy();
		}
		if (this._errorReport) {
			this._errorReport.destroy();
		}
		if (this.filterPanel) {
			this.filterPanel.destroy();
		}
		this._edit = null;
		this._close = null;
		this._isDestroyed = true;
	},
	
	updateTitleURL: function() {
		if (this.viewlet) {
			var url = this.viewlet.getTitleHyperlinkURL();
			this.context.setTitleURL(url);
		}
	},
	
	_addLoginTracker: function() {
		if (!_loginTrackersByOrigin) {
			_loginTrackersByOrigin = new Object();
		}
		var tracker = _loginTrackersByOrigin[this._webUIRoot];
		if (!tracker) {
			tracker = _loginTrackersByOrigin[this._webUIRoot] = new Observable();
		}
		this._loginCallback = dojo.hitch(this, function() {
			this._removeLoginTracker();
			this._loadViewlet(false);
		});
		tracker.addListener(this._loginCallback, "login");
	},
	
	_removeLoginTracker: function() {
		if (this._webUIRoot && _loginTrackersByOrigin && _loginTrackersByOrigin[this._webUIRoot]) {
			_loginTrackersByOrigin[this._webUIRoot].removeListener(this._loginCallback);
			delete this._loginCallback;
		}
	},
	
	_getServiceTracker: function() {
		if (!this._serviceTracker) {
			this._serviceTracker = new ServiceTracker();
		}
		return this._serviceTracker;
	},
	
	_getWebURL: function() {
		return this.local ? Util.getContextPath(true) : this._webUIRoot;
	},
	
	_handleAutoRefresh: function() {
		if (this._active) {
			this.refresh();
		}
		else {
			// try again next interval
			this._refreshTimeout = setTimeout(dojo.hitch(this, this._handleAutoRefresh), this._viewletDefinition.refreshInterval * 1000);
		}
	},
	
	_handleMaximize: function(/* boolean */ maximized) {
		this.viewlet.setZoomed(maximized);
		if (maximized) {
			dojo.addClass(this.domNode, "zoomed");
		}
		else {
			dojo.removeClass(this.domNode, "zoomed");
		}
		// notify viewlet that a zoom operation has completed
		this.viewlet.postZoom();	
		this.viewlet.getSite().setLoading(false);
	},
	
	_handlePreMaximize: function(/* boolean */ maximized) {
		// notify viewlet that a zoom operation is about to occur
		this.viewlet.preZoom();
	},
	
	_handleSettings: function() {
		// if not already open
		if (!this._settingsPanel) {
			this._openSettings();
		}
	},
	
	_handleSettingsCancel: function() {
		this._settingsPanel.close(dojo.hitch(this, this._handleSettingsClose));
	},
	
	_handleSettingsClose: function() {
		this._settingsContainer.removeChild(this._settingsPanel.domNode);
		this._settingsPanel.destroyRecursive();
		delete this._settingsPanel;
	},
	
	_hasSettings: function() {
		// true if viewlet is scopable or has any non-hidden prefs
		if (this._viewletDefinition.scopeSensitive) {
			return true;
		}
		var defns = this._viewletDefinition.preferenceDefinitions;
		if (defns) {
			for (var i=0;i<defns.length;i++) {
				if (defns[i].preferenceType != "HIDDEN") {
					return true;
				}
			}
		}
		return false;
	},
	
	/*
	 * Loading step 1
	 * Fetch viewlet definition
	 */
	_loadViewlet: function(/* boolean */ passive) {
		// 94229: clearTimeout doesn't always work on IE7
		if (this._isDestroyed) {
			return;
		}
		delete this._yieldTimeout;
		
		// check for a compatiblity URL
		var compatUrl = _getCompatibilityUrl(this.definition);
		var serviceObj = {
			_self: this,
			_onResponse: function() {
				// notify any login-waiting viewlets to load
				if (!this._self.local && this._self._loginCallback) {
					this._self._removeLoginTracker();
					_loginTrackersByOrigin[this._self._webUIRoot].propertyChanged("login");
				}
			},
			success: function(/* ViewletDefinitionDTO[] */ definitions) {
				if (this._self._isDestroyed) {
					return;
				}
				this._onResponse();
				if (definitions && definitions.length > 0) {
					this._self._viewletDefinition = definitions[0];
					var context = this._self.context;
					// reset title and icon after login
					if (context.getIcon() != this._self._icon) {
						context.setIcon(this._self._icon);
					}
					if (context.getSecondaryTitle() != "") {
						context.setSecondaryTitle("");
					}
					this._self._loadViewlet2();
				}
				else {
					// couldn't find definition
					var msg = dojo.string.substitute(messages.viewletDefNotFound, [this._self.definition]);
					this._self._showError(msg);
				}
			},
			failure: function(/* Error */ error) {
				if (this._self._isDestroyed) {
					return;
				}
				if (error) {
					if (compatUrl && (error.status == 403 || error.status == 404)) {
						this._self.definition = compatUrl;
						var iconUrl = this._self.context.getIcon();
						if (iconUrl) {
							var iconCompatUrl = _getCompatibilityUrl(iconUrl);
							if (iconCompatUrl) {
								this._self._icon = jazz.app.proxy.encodeProxiedURI(iconCompatUrl);
							}
						}
						this._self._loadViewlet(passive);
					}
					else if (error.status == 401) {
						this._self._showLogin();
					}
					else if (error.status == 504) {
						this._onResponse();
						this._self._showError(error.message, messages.remoteConnectionError);
					}
					else if (error.dojoType == "popupBlocked") {
						this._self._showPopupBlocked(error);
					}
					else if (error.status == 403) {
						this._onResponse();
						this._self._showError(error.responseText || error.message, messages.remoteConnectionError);
					}
					else {
						this._onResponse();
						this._self._showError(error);
					}
				}
				else {
					this._onResponse();
					this._self._showError(error);
				}
			}
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		ViewletRegistry.getDefinition(this.definition, passive, srh, this._getServiceTracker());
	},
	
	/*
	 * Loading step 2
	 * Fetch scope item name
	 */
	_loadViewlet2: function() {
		if (this._isDestroyed) {
			return;
		}
		var prefs = this.context.getPreferences();
		var itemId = (prefs && prefs["_scopeItem"]);
		if (itemId) {
			var name = ItemNameRegistry.getName(itemId);
			if (!name) {
				// don't have it; fetch it
				var serviceObj = {
					_this: this,
					_itemId: itemId,
					success: function(/* string */ name) {
						// add to registry
						ItemNameRegistry.addName({itemId: this._itemId, name: name});
						this._this._loadViewlet3();
					},
					failure: function(/* Error */ error) {
						// skip name and continue without it
						this._this._loadViewlet3();
					}
				};
				var dataServerRoot = this.local ? null : this._serviceRoot;
				var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
				DashboardClient.getItemName(prefs["_scope"], itemId, dataServerRoot, srh, this._getServiceTracker());
			}
			else {
				this._loadViewlet3();
			}
		}
		else {
			this._loadViewlet3();
		}
	},
	
	/*
	 * Loading step 3
	 * Load & instantiate viewlet widget
	 */
	_loadViewlet3: function() {
		if (this._isDestroyed) {
			return;
		}
		// create site
		var site = new ViewletSiteAdapter({
			context: this.context,
			viewletAdapter: this,
			viewletDefinition: this._viewletDefinition
		});
		if (this.local) {
			// load code & css
			jazz.core.loader.load_async(this._viewletDefinition.widget, dojo.hitch(this, function() {
				if (!this._isDestroyed) {
					try {
						// create and add the viewlet (the content)
						var viewletConstructor = eval(this._viewletDefinition.widget);
						this.viewlet = new viewletConstructor({
							site: site,
							webURL: Util.getContextPath(true)
						});
						this._bodyContainer.replaceChild(this.viewlet.domNode, this._bodyContainer.firstChild);
						this._loadViewlet4();
					} 
					catch (/* Error */ error) {
						this._showError(error);
					}
				}
			}));
		}
		else {
			// find the page container, which marks the iframe "bust out" bounds
			var pageNode = this.domNode;
			while (true) {
				if (pageNode.id == "content-cell" || dojo.hasClass(pageNode, "content-table") || !pageNode.parentNode) {
					break;
				}
				pageNode = pageNode.parentNode;
			}
			
			// remote viewlet; iframe it
			this._iframeContainer = new IFrameContainer({
				active: !this.collapsed,
				className: this._viewletDefinition.widget,
				webUIRoot: this._webUIRoot,
				serviceRoot: this._serviceRoot,
				args: {
					site: site
				},
				onLoad: dojo.hitch(this, function(/* Widget */ widget) {
					// 94229: clearTimeout doesn't always work on IE7
					if (!this._isDestroyed) {
						this.viewlet = widget;
						// n-1 compatibility; old Viewlet created its own site,
						// but new one expects one to be passed in. Override the
						// created one here
						this.viewlet._site = site;
						site.viewlet = this.viewlet;
						// n-1 compatibility; override these methods on the
						// viewlet base class to ensure we have the latest code
						// (old code assumed existence of viewlet helper, etc)
						for (var i in VIEWLET_OVERRIDE_METHODS) {
							this.viewlet[i] = Viewlet.prototype[i];
						}
						this._updateRemoteIndicator(true);
						this._loadViewlet4();
					}
				}),
				pageNode: pageNode
			});
			this._bodyContainer.replaceChild(this._iframeContainer.domNode, this._bodyContainer.firstChild);
		}
		if (this._viewletDefinition.zoomable) {
			this.context.setMaximizeHandler(dojo.hitch(this, this._handleMaximize));
			this.context.setPreMaximizeHandler(dojo.hitch(this, this._handlePreMaximize));
		}
	},
	
	/*
	 * Loading step 4
	 * Finish configuring viewlet, perform initial refresh
	 */
	_loadViewlet4: function() {
		if (this._isDestroyed) {
			return;
		}
		// enable filtering
		if (this._viewletDefinition.filterable && !this.filterPanel) {
			this.filterPanel = new FilterPanel();
			this._filterContainer.appendChild(this.filterPanel.domNode);
			this.context.getToolbar().addButton(this.filterPanel,this.filterPanel.toggle,{
				label: messages.filter,
				className: "com-ibm-team-dashboard-web-ui-ViewletAdapter-filter",
				index: this.local ? 0 : 1
			});
		}
		
		// enable edit content
		if (!this.readOnly && this._viewletDefinition.editable) {
			this.context.setEditContentHandler(dojo.hitch(this, this.handleEditContent));
		}
		
		// setup auto-refresh
		if (this._viewletDefinition.refreshInterval > 0) {
			this.connect(this.viewlet, "refresh", "_onRefresh");
		}
		// clear the viewlet menu in case this is a re-load
		//delete this._contributedMenuItems;
		// allow viewlets to initialize themselves before the first refresh
		this.viewlet.init();
		
		// inform viewlet that it is active
		this._setActive(!this.collapsed);
		
		// register settings handler
		if (this._hasSettings()) {
			this.context.setSettingsHandler(dojo.hitch(this, this._handleSettings));
		}
		
		// trigger initial refresh
		this.refresh();
	},
	
	_onLogin: function(/* Event */ event) {
		if (event) {
			dojo.stopEvent(event);
		}
		this._loadViewlet(false);
	},
	
	/*
	 * Called whenever the viewlet's refresh method is called (via
	 * this.connect). Resets the auto-refresh timer.
	 */
	_onRefresh: function() {
		if (this._refreshTimeout) {
			clearTimeout(this._refreshTimeout);
			delete this._refreshTimeout;
		}
		// recall the last refresh time so that we can properly suspend and resume
		// auto-refresh without disrupting the normal schedule
		this._lastRefreshTime = new Date().getTime();
		this._refreshTimeout = setTimeout(dojo.hitch(this, this._handleAutoRefresh), this._viewletDefinition.refreshInterval * 1000);
	},
	
	_openSettings: function() {
		this._settingsPanel = new ViewletSettingsPanel({
			onCancel: dojo.hitch(this, this._handleSettingsCancel),
			context: this.context,
			scopePreferenceProviderClass: this.getScopePreferenceProviderClass(),
			viewlet: this.viewlet,
			viewletAdapter: this,
			viewletDefinition: this._viewletDefinition, 
			readOnly: this.readOnly
		});
		this._settingsContainer.appendChild(this._settingsPanel.domNode);
		this._settingsPanel.open();
	},
	
	_setActive: function(/* boolean */ value) {
		if (value !== this._active) {
			this._active = value;
			if (this._iframeContainer) {
				this._iframeContainer.setActive(value);
			}
			if (this.viewlet) {
				this.viewlet.setActive(value);
			}
		}
	},
	
	_showError: function(/* Error/string */ error, /* string */ msg) {
		if (this._isDestroyed) {
			return;
		}
		// error icon
		this.iconPath = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/Dashboard/icons/error-10.gif");
		this.context.setIcon(this.iconPath);
		
		// error message w/show details
		var args = {error: error};
		if (msg) {
			args.msg = msg;
		}
		this._errorReport = new ErrorReport(args);
		this._bodyContainer.innerHTML = "";
		this._bodyContainer.appendChild(this._errorReport.domNode);
		// TODO: menu
		//this._toggleMenu.parentNode.removeChild(this._toggleMenu);
	},
	
	_showMessage: function(/* string */ msg) {
		if (this._isDestroyed) {
			return;
		}
		var div = document.createElement("div");
		div.className = "msg";
		div.appendChild(document.createTextNode(msg));
		this._bodyContainer.replaceChild(div, this._bodyContainer.firstChild);
		// TODO: menu
		//this._toggleMenu.parentNode.removeChild(this._toggleMenu);
	},
	
	_showLogin: function() {
		// show login required icon
		this.iconPath = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/ViewletAdapter/icons/login.gif");
		this.context.setIcon(this.iconPath);
		this.context.setSecondaryTitle(messages.viewletNotLoggedIn);
		
		// listen for log-ins for this origin
		this._addLoginTracker();
		
		// show log-in section
		var div = document.createElement("div");
		div.className = "login";
		div.innerHTML = "<div class=\"message\">" + dojo.string.substitute(messages.viewletLogin, [this._webUIRoot]) + "</div>";
		
		// add log-in link
		var linkDiv = document.createElement("div");
		linkDiv.className = "link";
		var a = document.createElement("a");
		a.href = "#";
		a.appendChild(document.createTextNode(messages.viewletLogin2));
		this.connect(a, "onclick", "_onLogin");
		linkDiv.appendChild(a);		
		div.appendChild(linkDiv);
		
		this._bodyContainer.replaceChild(div, this._bodyContainer.firstChild);
		this._state = STATE_LOGIN;
	},
	
	_showPopupBlocked: function(/* Error */ e) {
		// warning icon
		this.iconPath = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/Dashboard/icons/popup-blocked.gif");
		this.context.setIcon(this.iconPath);
		this.context.setSecondaryTitle("");
		
		var buf = ["<div class=\"msg\">", e.message, "</div>"];
		this._bodyContainer.innerHTML = buf.join("");
		var linkDiv = dojo.create("div", {className: "link"}, this._bodyContainer.firstChild);
		var a = dojo.create("a", {href: "#", innerHTML: messages.tryAgain}, linkDiv);
		this.connect(a, "onclick", "_onLogin");
	},
	
	_startEditing: function(event) {
		if (event) {
			dojo.stopEvent(event);
		}
		this.setAutoRefresh(false);
		// TODO: container should handle this
		/*if (this.collapsed) {
			this.toggleCollapsed();
		}*/
		
		// lock height
		var height = this.domNode.clientHeight;
		this.domNode.style.height = height + "px";
		
		// show button panel
		this._editButtonArea.style.display = "block";
		this.editingButtons = [];
		this.editingButtons.push(this.connect(this._saveEdits, "onclick", this._stopEditing));
		this.editingButtons.push(this.connect(this._cancelEdits, "onclick", this._stopEditing));
		
		// switch to edit mode
		this.viewlet.getViewletState().startEditing();
		this.viewlet.setEditing(true);
		
		// enlarge/shrink
		var anim = dojo.animateProperty({node: this.domNode, duration: 200,
			properties:
				{height: {end: this._settingsContainer.clientHeight + this._contentWrapper.clientHeight + this._editButtonArea.clientHeight, unit: "px"}
			}});
		anim.play();
		this.connect(anim, "onEnd", function() {
			this.domNode.style.height = "";
			dojo.publish("jazz.util.ResizeNotifier.resizeEvent");
		});
	},
	
	_stopEditing: function(e) {
		// collapsing viewlet while in edit mode calls this method
		var saving = false;
		// if the event handler exists, then a button was explicitly pressed
		if (!e) {
			if(!confirm(dojo.string.substitute(messages.stopEditingConfirm, [this.context.getTitle()]))) {
				return false;
			}
		} else {
			var saving = e.target == this._saveEdits;
		}
		if (saving) {
			// "OK" button was clicked
			// validate edited content
			var validContent = this.viewlet.validateEditedContent();
			// check if the content is valid
			if (validContent == false) {
				// Content is not valid.  Stay in edit mode
				return;
			}
			// content is valid. leave edit mode
			// notify viewlet that edited contents is about to be saved
			this.viewlet.preSaveState();
		}
		// lock height
		var height = this.domNode.clientHeight;
		this.domNode.style.height = height + "px";
		
		// hide button panel
		this._editButtonArea.style.display = "none";
		while(this.editingButtons.length > 0) {
			this.disconnect(this.editingButtons.pop());
		}
		
		// switch out of edit mode
		this.viewlet.getViewletState().stopEditing(saving);
		this.viewlet.setEditing(false);
		
		// enlarge/shrink
		var anim = dojo.animateProperty({node: this.domNode, duration: 200, 
			properties:
				{height: {end: this._settingsContainer.clientHeight + this._contentWrapper.clientHeight + this._editButtonArea.clientHeight, unit: "px"}
			}});
		anim.play();
		this.connect(anim,"onEnd",function() {
			this.domNode.style.height = "";
			this.setAutoRefresh(true);
			dojo.publish("jazz.util.ResizeNotifier.resizeEvent");
		});	
		if (!e) {
			return true;
		}
	},
	
	_updateRemoteIndicator: function(/* boolean */ value) {
		var node = this._remoteIcon.domNode;
		if (value === true) {
			node.title = dojo.string.substitute(messages.connected,[messages.remoteServer, this._shortOrigin]);
			node.className = "com-ibm-team-dashboard-web-ui-ViewletAdapter-remote-icon-connected";
		}
		else {
			node.title = dojo.string.substitute(messages.disconnected,[messages.remoteServer, this._shortOrigin]);
			node.className = "com-ibm-team-dashboard-web-ui-ViewletAdapter-remote-icon-disconnected";
		}
	}
});

function _getCompatibilityUrl(/* string */ url) {
	var query = new dojo._Url(url).query;
	if (query) {
		var matches = query.match(/compatUrl=([^&]*)/);
		if (matches && matches.length >= 2) {
			return decodeURIComponent(matches[1]);
		}
	}
}

})();
