/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.ViewletMemento");

dojo.require("com.ibm.team.dashboard.web.ui.IMemento");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");

(function() {

var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var IMemento = com.ibm.team.dashboard.web.ui.IMemento;

/*
 * A memento that stores viewlet data shared by all users, e.g. content. It is
 * stored in the Viewlet helper of a Dashboard item.
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.ViewletMemento",
	IMemento, {

	constructor: function(/* object */ memento, /* ViewletHelper */ viewletHelper, /* ViewletContainer */ container) {
		this._memento = memento;
		this._container = container;
		this._viewletHelper = viewletHelper;
	},
	
	startEditing: function() {
		this._workingCopy = dojo.clone(this._memento);
	},
	
	stopEditing: function(/* boolean */ commit) {
		if (commit) {
			this._memento = this._workingCopy;
			this._save();
		}
		delete this._workingCopy;
	},

	_save: function() {
		this._viewletHelper.memento = dojo.toJson(this._memento);
		this._container.propertyChanged(Constants.PROPERTY_DASHBOARD);
	},
	
	get: function(/* string */ key) {
		var memento = this._workingCopy ? this._workingCopy : this._memento;
		if (memento[key])
			return memento[key];
		return null;
	},
	
	set: function(/* string */ key, /* object OR primative */ value) {
		var memento = this._workingCopy ? this._workingCopy : this._memento;
		memento[key] = value;
		if (!this._workingCopy)
			this._save();
	}
});

})();