/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.ViewletPage");

dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.Observable");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("dojo.i18n"); 
dojo.require("dojo.string");

dojo.require("jazz.ensemble.WidgetPanel");
dojo.require("jazz.ensemble.internal.Schema");
dojo.require("jazz.ensemble.internal.util.Xml");
dojo.require("jazz.ensemble.internal.util.XmlBindingFactory");
dojo.require("jazz.util.i18n");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

//
// Aliases
//
var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var Observable = com.ibm.team.dashboard.web.ui.internal.util.Observable;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var Schema = jazz.ensemble.internal.Schema;
var WidgetPanel = jazz.ensemble.WidgetPanel;
var Xml = jazz.ensemble.internal.util.Xml;
var XmlBindingFactory = jazz.ensemble.internal.util.XmlBindingFactory;

var messages;

/**
 * A page of viewlets (contents of one dashboard tab).
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.ViewletPage",
	[dijit._Widget, dijit._Templated, Observable], {
	
	// Variables passed into constructor
	page: null,					//required
	pageId: null,				//required
	dashboardContainer: null,	//required
	dashboardBundle: null,		//required
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/ViewletPage.html"),
	
	constructor: function(args) {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
	},
	postCreate: function() {
		this._emptyTabAddViewletLink = jazz.util.i18n.addMessage(this._showInEditMode, dojo.string.substitute(messages.noViewletsAdd, [ messages.addViewletText ]))[0];
		this._emptyTabAddViewletEvent = dojo.connect(this._emptyTabAddViewletLink,"onclick",this.dashboardContainer,this.dashboardContainer.openCatalog);
		this.updateEmptyPageMessage(true);
		this.editing = (this.dashboardContainer.editMode != Constants.READ_ONLY) ? true : false;
		if (!this.editing) this._showInEditMode.style.display = "none"; 
	},
	init: function() {
		this._yieldTimeout = setTimeout(dojo.hitch(this, this.init2), 0);
		this.isActive = true;
	},
	init2: function() {
		if (!this._isDestroyed) {
			delete this._yieldTimeout;
			this._createWidgetPanel();
			this._loadingText.style.display ="none";
		}
	},
	_createWidgetPanel: function() {
		var properties = {
			"com.ibm.team.dashboard.scope": this.dashboardBundle.scope,
			"com.ibm.team.dashboard.scopeItem": this.dashboardBundle.scopeItem,
			"com.ibm.team.dashboard.defaults": this.dashboardBundle.defaults
		};
		// only contributor dashboards with the current user owner are not shared
		var shared = (this.dashboardBundle.scope != DashboardConstants.SCOPE_CONTRIBUTOR) || (this.dashboardBundle.contributorItemId != this.dashboardBundle.scopeItem.itemId);
		this._widgetPanel = new WidgetPanel({
			allowMaximize: true,
			properties: properties,
			readOnly: !this.dashboardBundle.permissions.modify,
			shared: shared,
			state: this.page.elem,
			onStateChange: dojo.hitch(this, this._onStateChange),
			uuid: this.dashboardBundle.dashboardUrl
		});
		this._tabArea.appendChild(this._widgetPanel.domNode);
		this._widgetPanel.startup();
		this.updateEmptyPageMessage();
	},
	isVisible: function() {
		return (this.dashboardContainer.currentPage == this) && this.dashboardContainer.isVisible();
	},
	setEditMode: function(editing) {
		this.editing = editing;
		if (editing) {
			this._showInEditMode.style.display = "";
		} else {
			this._showInEditMode.style.display = "none";
		}
	},
	setActive: function(/* boolean */ active) {
		if (this.isActive != active) {
			this.isActive = active;
			if (this._widgetPanel) {
				if (active) {
					this._widgetPanel.onShow();
				}
				else {
					this._widgetPanel.onHide();
				}
			}
		}
	},
	setLayout: function(/* string */ type) {
		this._widgetPanel.setLayout(type);
	},
	addWidget: function(/* object */ data) {
		var widgetElem = this._widgetPanel.createWidget();
		var widget = XmlBindingFactory.createBinding(widgetElem, Schema.WIDGET, "\t\t\t\t");
		
		widget.setType(data.type);
		widget.setDefinition(data.widgetUri);
		widget.setTitle(data.title);
		if (data.icon) {
			widget.setIcon(data.icon);
		}
		
		var preferences = data.preferences;
		if (preferences instanceof Array && preferences.length > 0) {
			var prefs = [];
			var widgetPreferences = widget.createPreferences();
			widgetPreferences.setParseType("Collection");
			widget.setPreferences(widgetPreferences);
			for (var i = 0; i < data.preferences.length; i++) {
				var name = data.preferences[i].name;
				var value = data.preferences[i].value;
				var pref = widgetPreferences.createPreference();
				pref.setName(name);
				pref.setValue(value);
				prefs.push(pref);
			}
			widgetPreferences.setPreferences(prefs);
		}		
		this._widgetPanel.addWidget(widgetElem);
		this.updateEmptyPageMessage();
		this.propertyChanged(Constants.PROPERTY_DASHBOARD);
	},
	addWidgetFromElement: function(/* Element */ elem) {
		this._widgetPanel.addWidget(elem);
		this.propertyChanged(Constants.PROPERTY_DASHBOARD);
	},
	removeWidget: function(/* string */ id) {
		this._widgetPanel.removeWidget(id);
		this.updateEmptyPageMessage();
	},
	
	updateEmptyPageMessage: function(isLoading) {
		// check if there are any widgets on this page
		var widgets = this.page.getWidgets().getWidgets();
		var isEmpty = (widgets.length == 0);
		if (isLoading && isEmpty) this._loadingText.style.display = "none";
		this._setEmptyTabMessage(isEmpty);
	},
	
	_onStateChange: function() {
		this.propertyChanged(Constants.PROPERTY_DASHBOARD);
		this.updateEmptyPageMessage();
	},
	
	_setEmptyTabMessage: function(isVisible) {
		if (isVisible) {
			this._emptyTabMessage.style.display = "block";	
		} else {
			this._emptyTabMessage.style.display = "none";
		}	
	},
	/*
	 * Destroys all viewlet widgets so that they can be garbage collected.
	 */
	destroy: function() {
		this._isDestroyed = true;
		// are we in the middle of loading?
		if (this._yieldTimeout) {
			// cancel loading
			clearTimeout(this._yieldTimeout);
			delete this._yieldTimeout;
		}
		if (this._emptyTabAddViewletEvent) dojo.disconnect(this._emptyTabAddViewletEvent);
		if (this._widgetPanel) this._widgetPanel.destroy();
		this.inherited(arguments);
	},
	refreshPage: function() {
		this.dashboardContainer._refreshCurrentTab();
	}
});

// public static
dojo.mixin(com.ibm.team.dashboard.web.ui.internal.ViewletPage, {
	
	SCHEMA: [
		{
			get: "getId",
			set: "setId",
			type: XmlBindingFactory.STRING,
			nodeType: XmlBindingFactory.ATTRIBUTE,
			attrName: "ID",
			attrNS: Xml.namespace.rdf
		},
		{
			get: "getTitle",
			set: "setTitle",
			type: XmlBindingFactory.STRING,
			nodeType: XmlBindingFactory.CHILD_ELEMENT_TEXT,
			elemName: "title",
			elemNS: Xml.namespace.dublin
		}
	].concat(Schema.WIDGET_PANEL)
});

})();
