/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.ViewletSiteAdapter");

dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ItemNameRegistry");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");

dojo.require("com.ibm.team.dashboard.web.ui.internal.MementoAdapter");

(function(){

var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var ItemNameRegistry = com.ibm.team.dashboard.web.ui.internal.util.ItemNameRegistry;
var MementoAdapter = com.ibm.team.dashboard.web.ui.internal.MementoAdapter;
var Util = com.ibm.team.dashboard.web.util.internal.Util;

/*
 * A ViewletSite implementation that forwards calls to the WidgetContext.
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.ViewletSiteAdapter",
	null, {
	
	context: null,
	viewletAdapter: null,
	viewletDefinition: null,
	
	_contributorMemento: null,
	_loading: false,
	_viewletMemento: null,
	
	constructor: function(/* object */ args) {
		dojo.mixin(this, args);
		// N-1 compatibility; old trend reports viewlet relied on this
		// no-longer-valid internal property
		this._container = {
			dashboardBundle: {
				permissions: {
					modify: !this.context.readOnly
				}
			}
		};
	},
	
	/*
	 * Adds a menu item to the viewlet menu.
	 * 
	 * @param label the text shown for the menu item.
	 * @param icon (optional) either an image URL or a css class name specifying
	 * 		a background-image property for the menu item's icon, or null for none.
	 * 		The use of css class is deprecated as it does not work with iframed
	 * 		viewlets (because the css is not available outside the iframe).
	 * @param callback the function to call when the user activates the menu item.
	 * 		The scope (i.e. 'this') will be the viewlet adding the menu item.
	 */
	addMenuItem: function(/* string */ label, /* string */ icon, /* function */ callback) {
		var item = {
			label: label,
			icon: icon,
			onClick: dojo.hitch(this.viewletAdapter.viewlet, callback)
		};
		var newItems = this.context.menuContributions.concat([item]);
		this.context.setMenuContributions(newItems);
	},
	
	/*
	 * Returns true or false based on if the current server license level allows
	 * viewing of the indicated dashboard type.
	 * 
	 * @param scope a number representing one of the constants for the scope of
	 * 		the dashboard
	 */
	canViewDashboard: function(/* number */ scope) {
		var perms = dojo.getObject("com.ibm.team.dashboard.internal.permissions");
		if (perms) {
			switch (scope) {
				case DashboardConstants.SCOPE_PROJECT_AREA:
					return perms.viewProject;
				case DashboardConstants.SCOPE_TEAM_AREA:
					return perms.viewTeam;
				case DashboardConstants.SCOPE_CONTRIBUTOR:
					return perms.viewPersonal;
			}
		}
		return false;
	},
	
	/*
	 * Signals the container that the viewlet is done refreshing.
	 */
	doneRefresh: function() {
		if (this.viewletDefinition.titleAsHyperlink) {
			this.viewletAdapter.updateTitleURL();
		}
	},
	
	/*
	 * Can be called by viewlets to programatically enter edit mode.
	 */
	editContents: function() {
		if (!this.viewletAdapter.readOnly) {
			this.viewletAdapter.handleEditContent();
		}
	},
	
	/*
	 * Reports a serious error which may impact the normal operation of the
	 * dashboard or other viewlets (e.g. connection failure, server down).
	 * 
	 * @param error An Error object representing the problem.
	 */
	error: function(/* Error */ error) {
		// TODO: implement this
		this.viewletAdapter.updateTitleURL();
		console.error(error);
	},
	
	/*
	 * Returns the action area of the container.
	 */
	getActionArea: function() {
		return this.viewletAdapter.actionArea;
	},
	
	getFilterBox: function() {
		return this.viewletAdapter.filterPanel ? this.viewletAdapter.filterPanel.filterBox : null;
	},
	
	/*
	 * Returns a memento that is personalized to the current user and will not
	 * be shared with other users. This can be used to store data that is specific
	 * to the user but should not be visible to other users.
	 * 
	 * This is available only when the user is the owner of the dashboard. To check
	 * whether or not it is available, call hasContributorState(). If unavailable,
	 * any data stored will the memento object will not be persisted.
	 */
	/* IMemento */ getContributorState: function() {
		// 117990: Discontinue viewlet contributor info for shared dashboards
		if (this.context.isShared()) {
			if (!this._contributorMemento) {
				this._contributorMemento = new MementoAdapter({
					obj: {},
					onStateChange: function() {
						// do nothing; contributor state for shared dashboards
						// is discontinued as per 117990.
					}
				});
			}
			return this._contributorMemento;
		}
		return this.getViewletState();
	},
	
	/*
	 * Returns the default scope item for the given scope.
	 */
	getDefaultItem: function(scope) {
		if (this.viewletAdapter.local) {
			var defaults = this.context.properties["com.ibm.team.dashboard.defaults"];
			if (defaults) {
				switch (scope) {
					case DashboardConstants.SCOPE_CONTRIBUTOR:
						return defaults.contributor;
					case DashboardConstants.SCOPE_TEAM_AREA:
						return defaults.team;
					case DashboardConstants.SCOPE_PROJECT_AREA:
						return defaults.project;
				}
			}
		}
	},
	
	/*
 	 * Returns the height of the viewlet's content area
 	 */		
	getHeight: function() {
		return this.viewletAdapter.getHeight();
	},
	
	/*
	 * Returns the preference with the given id.
	 */
	/* string */ getPreference: function(/* string */ id) {
		var prefs = this.context.getPreferences();
		var pref = (prefs && prefs[id]);
		var defs = this.viewletDefinition.preferenceDefinitions;
		if (defs) {
			for (var i=0;i<defs.length;i++) {
				if (defs[i].id == id) {
					return Util._cast(defs[i].preferenceType, typeof pref !== "undefined" ? pref : defs[i].defaultValue);
				}
			}
		}
	},
	/*
	 * Sets the preference with the given id.
	 */
	setPreference: function(/* string */ id, /* string */ value) {
		if (id) {
			var prefs = this.context.getPreferences();
			prefs[id] = value;
			this.context.setPreferences(prefs);
		}
	},
	
	/*
	 * Returns the viewlet scope. Scope data is stored in special hidden
	 * preferences.
	 */
	/* string */ getScope: function() {
		var prefs = this.context.getPreferences();
		var scope = prefs && prefs["_scope"];
		
		// if the viewlet is not scope sensitive, always return the dashboard scope
		if (!this.viewletDefinition.scopeSensitive || (scope != DashboardConstants.SCOPE_CONTRIBUTOR && scope != DashboardConstants.SCOPE_TEAM_AREA && scope != DashboardConstants.SCOPE_PROJECT_AREA)) {
			scope = this.context.properties["com.ibm.team.dashboard.scope"];
		}
		switch (this.viewletDefinition.appliesTo) {
			case Constants.SCOPE_APPLIES_TO_TEAMS:
				return DashboardConstants.SCOPE_TEAM_AREA;
			case Constants.SCOPE_APPLIES_TO_CONTRIBUTORS:
				return DashboardConstants.SCOPE_CONTRIBUTOR;
			case Constants.SCOPE_APPLIES_TO_PROJECTS:
				return DashboardConstants.SCOPE_PROJECT_AREA;
			case Constants.SCOPE_APPLIES_TO_PROCESS:
				if (scope == DashboardConstants.SCOPE_CONTRIBUTOR) {
					return DashboardConstants.SCOPE_PROJECT_AREA;
				}
		}
		return scope;
	},
	
	/*
	 * Returns the viewlet scope item. Scope data is stored in special hidden
	 * preferences.
	 */
	/* object */ getScopeItem: function() {
		var prefs = this.context.getPreferences();
		var scope = prefs && prefs["_scope"];
		
		// special case to handle unconfigured viewlets that were copied and have scopes that don't apply
		if (typeof(scope) != 'undefined' && !Util.isScopeApplicible(scope, this.viewletDefinition)) {
			// if the viewlet is scopable, null (a.k.a. not scoped) is a valid case and should be maintained after copying
			if (this.viewletDefinition.scopeSensitive) {
				return null;
			}
			// if the viewlet is not scopable we should return the dashboard scope, so fall through to default
		}
		// ignore the viewlet helper if the viewlet is not scope sensitive, in this case always defer to the dashboard
		if (this.viewletDefinition.scopeSensitive) {
			var itemId = prefs && prefs["_scopeItem"];
			if (itemId) {
				var scopeItem = {itemId: itemId};
				var name = ItemNameRegistry.getName(itemId);
				if (name) {
					scopeItem.name = name;
				}
				return scopeItem;
			}
		}
		if (this.viewletAdapter.local) {
			return this.getDefaultItem(this.getScope());
		}
		// if the viewlet is not local, return null since dashboard scopes do not apply, viewlets must handle this case
		return null;
	},
	
	/*
	 * Returns the scope setting that indicates if subteams should be considered
	 * part of the viewlets scope. Only applicible when the scope is set to a
	 * team area and only to viewlets who define include-subteams as "yes" or
	 * "yes-and-check" in their definition.
	 */
	/* boolean */ getScopeSubteams: function() {
		var prefs = this.context.getPreferences();
		var value = (prefs && prefs["_scopeSubteams"]);
		if (!value) {
			return (this.viewletDefinition.scopeSubteams == "yes-and-check");
		}
		return Util._cast(Constants.SettingsArea.FIELD_BOOLEAN, value);
	},
	
	/*
	 * Returns the viewlet's secondary title.
	 */
	getSecondaryTitle: function() {
		return this.context.getSecondaryTitle();
	},
	
	/*
	 * Returns the viewlet's primary title.
	 */
	getTitle: function() {
		return this.context.getTitle();
	},
	
	/*
	 * Returns a memento for persisting viewlet state info that is the same for
	 * all users, e.g. viewlet content.
	 */
	/* IMemento */ getViewletState: function() {
		if (!this._viewletMemento) {
			var obj;
			var mementoStr = this.context.getMemento();
			if (typeof(mementoStr) == "string") {
				// deserialize in the viewlet's window to ensure that it can correctly
				// identify arrays and other objects from its own window
				var window = this.viewletAdapter.getWindow();
				obj = window.dojo.fromJson(mementoStr);
			}
			this._viewletMemento = new MementoAdapter({
				obj: obj || this.viewletAdapter.viewlet.getDefaultState(),
				onStateChange: dojo.hitch(this, function() {
					this.context.setMemento(Util.toJson(this._viewletMemento.getState()));
				})
			});
		}
		return this._viewletMemento;
	},
	
	/*
 	 * Returns the width of the viewlet's content area
 	 */
	getWidth: function() {
		return this.viewletAdapter.getWidth();
	},
	
	/*
	 * Returns whether or not this viewlet allows contributor state to be
	 * stored for the current user. Viewlets that enable extra UI if contributor
	 * state is available should check this first.
	 */
	hasContributorState: function() {
		return !this.context.isShared();
	},
	
	/*
	 * Returns whether or not the viewlet is currently active.
	 */
	isActive: function() {
		return this.viewletAdapter._active;
	},
	
	/*
	 * Returns whether or not the current dashboard is being edited.
	 */
	isDashboardEditing: function() {
		return !this.viewletAdapter.readOnly;
	},
	
	/*
  	 * Indicates whether the viewlet is loading or not
 	 */	
	isLoading: function() {
		return this._loading;
	},	
	
	navigateAwayEvent: function(eventObj, workbenchNavigation) {
		// do we still need this?
		return true;
	},
	
	/*
	 * Refreshes the viewlet.
	 */
	refresh: function() {
		this.viewletAdapter.refresh();
	},
	
	/*
	 * Enables or disables automatic periodic refreshing of viewlets. Viewlets
	 * may wish to temporarily disable automatic refresh if it disrupts the
	 * normal user flow when, for example, the user is reading a hover tooltip or
	 * editing content. Normal auto-refresh will resume as soon as it is re-enabled.
	 */
	setAutoRefresh: function(/* boolean */ value) {
		this.viewletAdapter.setAutoRefresh(value);
	},
	
	/*
	 * Sets the viewlet icon to the image pointed to by the given URL.
	 * 
	 * @param url the URL of the image file.
	 */
	setIcon: function(/* string */ url) {
		this.context.setIcon(url);
	},
	
	/*
	 * Called to set loading state of viewlet.  Displays progress icon when loading.
	 */
	setLoading: function(loadingStatus) {
		this._loading = loadingStatus;
	},
	
	/*
	 * Sets the secondary viewlet title.
	 */
	setSecondaryTitle: function(/* string */ title) {
		this.context.setSecondaryTitle(title);
		this.viewletAdapter.updateTitleURL();
	},
	
	/*
	 * Called by viewlets to update the dynamic title. The title will be updated in the
	 * widget's state but will not dirty the widget panel. Thus it will only be persisted
	 * if a save occurs for another reason.
	 */
	setTitle: function(/* string */ title, /* boolean */ persist) {
		this.context.setTitle(title);
		this.viewletAdapter.updateTitleURL();
	},
	
	/*
	 * Can be called by viewlets to programatically open or close the settings area.
	 */
	toggleSettings: function() {
		this.viewletAdapter.toggleSettings();
	}
});
	
})();
