/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.about.AboutJazzTeamServer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.date");
dojo.require("dojo.date.locale");
dojo.require("dojo.date.stamp");
dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("dojox.xml.parser");
dojo.require("jazz.ui.Dialog");
dojo.require("jazz.ui.MessageBox");
dojo.require("net.jazz.ajax.BootstrapProperties");
dojo.require("jazz.app.internal.applicationAbout");
dojo.require("com.ibm.team.repository.web.client.internal.licensing");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.team.repository.web.ui.internal.Util");
dojo.require("com.ibm.team.repository.web.client.internal.HttpHeaders");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.about", "AboutJazzTeamServerMessages");

(function() {

var getAuthenticatedContributor = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var LicenseClient = com.ibm.team.repository.web.client.internal.licensing;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var STATUS_TYPES = jazz.ui.MessageBox.STATUS_TYPES;
var MessageBox = jazz.ui.MessageBox;
var Util = com.ibm.team.repository.web.ui.internal.Util;
var removeChildren = dojox.xml.parser.removeChildren;
var HttpConstants = com.ibm.team.repository.web.client.internal.HttpConstants;
var BootstrapProperties = net.jazz.ajax.BootstrapProperties;
var JTS_IDENTIFIER = "http://jazz.net/application/jts";
var JTS_PRODUCT_ID = "com.ibm.team.jazz.foundation.server";
	
dojo.declare("com.ibm.team.repository.web.ui.internal.about.AboutJazzTeamServer", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/about/templates/AboutJazzTeamServer.html"),
	
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.about", "AboutJazzTeamServerMessages");
		this.forUpdatesAndInfoMsg = dojo.string.substitute(this.messages.forUpdatesAndInfo, ["<a href='http://jazz.net' target='_blank'>http://jazz.net</a>"])
		this.headers = com.ibm.team.repository.web.client.internal.HttpHeaders._getHeaders();
		this.headers[HttpConstants.ACCEPT] = HttpConstants.CT_TEXT_JSON;
	},
	
	postCreate: function() {
		this.header = new MessageBox();
		this.headerArea.appendChild(this.header.domNode);
		
		this.ibmLogo.src = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/images/ibm_logo_sm.gif");
		this.ibmLogo.style.marginTop = "32px";
		this.jazzLogo.src = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/about/images/jazz-mark.png");
		this.jazzLogo.style.marginTop = "18px";
		this.eclipseLogo.src = dojo.moduleUrl("com.ibm.team.repository.web","ui/internal/about/images/BuiltOnEclipse_Logo.png");
		this.eclipseLogo.style.marginTop = "15px";
		this.javaLogo.src = dojo.moduleUrl("com.ibm.team.repository.web","ui/internal/about/images/sun-java-comp-logo.png");
		
		//See if we are running as an Application, or on the JTS/Standalone
		this.serverMode = dojo.getObject("com.ibm.team.repository.server.mode");
		this.isJTS = (this.serverMode == "APPLICATION") ? false : true;
		this.latestAbouts = jazz.app.internal.applicationAbout.get();
		this.reload();
	},
	
	reload: function() {
		this.header.showMessage({
	        type: STATUS_TYPES.INFO,
	        summary: this.messages.loading
	    });
		
		var srh = new ServiceResponseHandler(this,"cbGetAllProductInfo","displayErrorDialog");
		ServerStatusClient.getAllProductInfo(srh);
		
		if (this.isJTS) {
			//JTS ABOUT PAGE
			if (Util.userIsAdmin()) {
				var licKeyLink = "<a target=\"_blank\" href=\"" + net.jazz.ajax._contextRoot + "/admin#action=com.ibm.team.repository.admin.manageLicenses\">";
				var licenseKeyMgmt = dojo.string.substitute(this.messages.licenseKeyManagement, [licKeyLink, "</a>"]);
				this.licenseKeyManagementDiv.innerHTML = dojo.string.substitute(this.messages.toManageLicenses, [licenseKeyMgmt]);
				dojo.removeClass(this.licenseKeyManagementDiv, "noDisplay");
			}
			
			if(BootstrapProperties.APP_VERSION) {
				this.jtsVersion.innerHTML = this.messages.version + ": " + BootstrapProperties.APP_VERSION;
			}
			
			var srh1 = new ServiceResponseHandler(this, "cbGetLicenseInfo", "displayErrorDialog");
			LicenseClient.getLicenseInfo(srh1);	
		}
		else {
			//APPLICATION ABOUT PAGE
			this.appTitle.innerHTML = dojo.string.substitute(this.messages.aboutApplication, [ BootstrapProperties.APP_NAME || "" ]);
			if(BootstrapProperties.APP_VERSION) {
				this.appVersion.innerHTML = this.messages.version + ": " + BootstrapProperties.APP_VERSION;
			}
			dojo.removeClass(this.applicationSection, "noDisplay");
			var srh1 = new ServiceResponseHandler(this, "handleJTSURLResponse", "displayErrorDialog");
			ServerStatusClient.getConfiguredJazzTeamServerURI(srh1);
		}
		this.showDialog();
	},
	
	handleJTSURLResponse: function(response) {
		var self = this;
		if(response) {
			this.configuredJTS = response;

			//Now proxy to get JTS Server Information
			var deferred = this.getProductsFromJTS(this.configuredJTS);
			deferred.addCallback(function(productInfo) {
				self.buildProductTable(productInfo, self.componentsTBody);
			});
			deferred.addErrback(function(errObj) {
				var msg = self.messages.cantGetProductInfoJTS + " " + self.messages.makeSureJTSConnected
				self._handleError(errObj, msg);
				var tr = dojo.create("tr", {}, self.componentsTBody);
				dojo.create("td", { colSpan: 3, className: "noRowsYet", innerHTML: self.messages.unableToShowJTSInfo }, tr);
			});
			
			//Now proxy to get the JTS License Info
			var d2 = this.getLicenseInfoFromJTS(this.configuredJTS);
			d2.addCallback(function(licenseInfo) {
				self.licenseInfo = licenseInfo;
				//Now proxy to get the JTS CAL Assignments
				var d3 = self.getAssignedLicensesFromJTS(self.configuredJTS);
				d3.addCallback(function(calAssigments) {
					self.cbGetAssignedLicenses(calAssigments);
				});
				d3.addErrback(function(errObj) {
					self._handleCALAssignFail(errObj);
				});
			});
			d2.addErrback(function(errObj) {
				self._handleCALAssignFail(errObj);
			});
			
			var jtsVersion = this.getJTSVersion();
			if(jtsVersion) {
				this.jtsVersion.innerHTML = this.messages.version + ": " + jtsVersion;	
			}

		} else {		
			//Show an error about not being connected to a a JTS
			this.header.showMessage({
	            type: STATUS_TYPES.ERROR,
	            summary: this.messages.noJTSAvailable
	        });
			//Show the "unable to show..." rows
			var tr = dojo.create("tr", {}, this.installedCALBody);
			dojo.create("td", { colSpan: 4, className: "noRowsYet", innerHTML: this.messages.unableToShowCALs }, tr);
			tr = dojo.create("tr", {}, this.componentsTBody);
			dojo.create("td", { colSpan: 3, className: "noRowsYet", innerHTML: this.messages.unableToShowJTSInfo }, tr);
		
		}
	},
	
	_handleCALAssignFail: function(errObj) {
		var msg = this.messages.cantGetCALAssignments + " " + this.messages.makeSureJTSConnected
		this._handleError(errObj, msg);
		var tr = dojo.create("tr", {}, this.installedCALBody);
		dojo.create("td", { colSpan: 4, className: "noRowsYet", innerHTML: this.messages.unableToShowCALs }, tr);
	},
	
	getProductsFromJTS: function(jtsURL) {
		var endpoint = "service/com.ibm.team.repository.service.internal.IProductRegistryRestService/allProductInfo";
		var deferred = new dojo.Deferred();
		var self = this;
		if(jtsURL) {
			jazz.client.xhrGet({
				url: this._buildServiceURL(jtsURL, endpoint),
				handleAs: "json",
				headers: this.headers,
				load: function(response) {
					var response = self._demarshallResponse(response);
					deferred.callback(response);
				},
				error: function(errObject) {
					deferred.errback(errObject);
				}
			});	
		}
		return deferred;
	},
	
	getLicenseInfoFromJTS: function(jtsURL) {
		var endpoint = "service/com.ibm.team.repository.service.internal.ILicenseAdminRestService/licenseInfo";
		var deferred = new dojo.Deferred();
		var self = this;
		if(jtsURL) {
			jazz.client.xhrGet({
				url: this._buildServiceURL(jtsURL, endpoint),
				handleAs: "json",
				headers: this.headers,
				load: function(response) {
					var response = self._demarshallResponse(response);
					deferred.callback(response);
				},
				error: function(errObject) {
					deferred.errback(errObject);
				}
			});	
		}
		return deferred;
	},
	
	getAssignedLicensesFromJTS: function(jtsURL) {
		var userId = getAuthenticatedContributor().userId;
		var endpoint = "service/com.ibm.team.repository.service.internal.ILicenseAdminRestService/assignedLicenses?userId=" + userId
		var deferred = new dojo.Deferred();
		var self = this;
		if(jtsURL) {
			jazz.client.xhrGet({
				url: this._buildServiceURL(jtsURL, endpoint),
				handleAs: "json",
				headers: this.headers,
				load: function(response) {
					var response = self._demarshallResponse(response);
					deferred.callback(response);
				},
				error: function(errObject) {
					deferred.errback(errObject);
				}
			});	
		}
		return deferred;
	},
	
	getJTSVersion: function() {
		for(i in this.latestAbouts) {
			var contextRoot = this.latestAbouts[i].contextRoot;
			if(this.latestAbouts[i].identifier == JTS_IDENTIFIER && contextRoot.indexOf(this.configuredJTS) != -1) {
				return this.latestAbouts[i].version;
			}
		}
		return null;
	},
	
	_buildServiceURL: function(jts,endpoint) {	
		var lastChar = jts.charAt(jts.length-1);
		if(lastChar != '/') jts += "/";
		return jts + endpoint;
	},
	
	_demarshallResponse: function(evaldObj) {
		return com.ibm.team.repository.web.transport.TeamServerClient._demarshallResponse(evaldObj);
	},
	
	cbGetAllProductInfo: function(prodInfo) {
		var tbody = this.isJTS ? this.componentsTBody : this.appInfoTbody;
		if(prodInfo) {
			this.buildProductTable(prodInfo, tbody);
		} else {
			var row = dojo.create("tr", {}, tbody);
			dojo.create("td", { colSpan: 3, className: "noRowsYet", innerHTML: this.messages.noAppInfo }, row);
		}
		if(!this.dialogHasError) {
			this.header.hideMessage();	
		}
	},
	
	cbGetLicenseInfo: function(licenseInfo) {
		this.licenseInfo = licenseInfo;	
		var userId = getAuthenticatedContributor().userId;
		var srh2 = new ServiceResponseHandler(this, "cbGetAssignedLicenses", "displayErrorDialog");
		LicenseClient.getAssignedLicenses(srh2, {userId: userId});
	},
	
	cbGetAssignedLicenses: function(calAssignments) {
		this.assignmentIdMap = {};
		var self = this;
		dojo.forEach(calAssignments || [], function(cal) {
			self.assignmentIdMap[cal.id] = true;
		});
		this.displayLicenseData();
	},
	
	sortProducts: function (products) {	
		return products.sort(function(a, b){
			var aName = a.name.toLowerCase();
			var bName = b.name.toLowerCase();
			return aName > bName ? 1 : -1;
		});
	},
	
	buildProductTable: function(prodInfo, tbody) {
		dojo.empty(tbody);
		function col(row, node){
		    if (node === null || node === undefined) {
		        node = "-";
		    }
		    var td = dojo.create("td", {}, row);
		    if (dojo.isString(node)) {
		        node = document.createTextNode(node);
		    }
		    td.appendChild(node);
		    return td;
		}
		prodInfo = this.sortProducts(prodInfo);
		var jtsRows = [];
		dojo.forEach(prodInfo, function(product) {
			var row = dojo.create("tr");
			col(row,product.name);
			col(row,product.version);
			col(row,product.buildId);
			if(product.id == JTS_PRODUCT_ID) {
				jtsRows.push(row);
			} else {
				tbody.appendChild(row);
			}
			dojo.forEach(product.patches, function(patch){			
				var row = dojo.create("tr");
				dojo.toggleClass(row , "patch" , true);
				var td = col(row,patch.name);
				dojo.toggleClass(td , "patchName" , true);
				col(row,patch.version);
				col(row,patch.buildId);
				if(product.id == JTS_PRODUCT_ID) {
					jtsRows.push(row);
				} else {
					tbody.appendChild(row);
				}
			});
		});
		//Always show JTS libraries last (#154782)
		dojo.forEach(jtsRows, function(row) {
			tbody.appendChild(row);
		});
	},
	
	displayLicenseData: function() {
		var info = this.licenseInfo;		
		this.assignedCALHeader.innerHTML = dojo.string.substitute(this.messages.installedCALKeysToUser, [getAuthenticatedContributor().name]);
		var count = 0;
		if (info && info.types) {
			removeChildren(this.installedCALBody);
			var self = this;
			dojo.forEach(info.types, function(cal) {
				var isAssignedToUser = self.assignmentIdMap[cal.id];
				if(isAssignedToUser) {
					count++;
					var tr = dojo.create("tr", {}, self.installedCALBody);
					dojo.create("td", { innerHTML: cal.productName || "" }, tr);
					var versionTd = dojo.create("td", {}, tr);
					versionTd.appendChild(document.createTextNode(cal.version || ""));
					dojo.create("td", { innerHTML: cal.name || "" }, tr);
					var variant = LicenseClient.getVariantString(cal);
					dojo.create("td", { innerHTML: variant || "" }, tr);
				}
			});
		}
		if(count == 0) {
			var tr = dojo.create("tr", {}, this.installedCALBody);
			dojo.create("td", { colSpan: 4, className: "noRowsYet", innerHTML: this.messages.noCalsAssigned }, tr);
		}
		this.dialog.updatePosition();
		if(!this.dialogHasError) {
			this.header.hideMessage();	
		}
	},
	
	_handleError: function(err, message) {
		this.dialogHasError = true;
		this.header.showMessage({
            errorObject: err,
            summary: message
        });
	},
	
	displayErrorDialog: function(err) {
		this._handleError(err, this.messages.errorFetchingInformation);
	},
	
	showDialog: function () {
		this.dialog = new jazz.ui.Dialog({
			contentNode: this.contentNode,
			primaryTitle: this.dialogTitle,
			width: "800px",
			closeOnBlur: true
		});
	}
});

})();