/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.AdminApplication");

dojo.require("jazz.app._AjaxApplication");
dojo.require("jazz.app._ComponentMenu");
dojo.require("jazz.app.auth");
dojo.require("jazz.app.entryPoints");
dojo.require("net.jazz.ajax.ui.internal.PageRegistry");
dojo.require("jazz.app.internal.applicationAbout");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.AdminMenu");
dojo.require("com.ibm.team.repository.web.client.init.InitializationData");
dojo.require("com.ibm.team.repository.web.client.internal.RepositoryClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");
dojo.require("com.ibm.team.repository.web.ui.internal.about.AboutJazzTeamServer");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.NotAuthorizedView");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "AdminApplicationMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var InitializationData      = com.ibm.team.repository.web.client.init.InitializationData;
var RepositoryClient 		= com.ibm.team.repository.web.client.internal.RepositoryClient;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var AdminClient 			= com.ibm.team.repository.web.client.internal.AdminClient;
var Session 				= com.ibm.team.repository.web.client.session;
var messages 				= dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "AdminApplicationMessages"); //$NON-NLS-1$ //$NON-NLS-2$
var JTS_IDENTIFIER 			= "http://jazz.net/application/jts";

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.AdminApplication", jazz.app._AjaxApplication, {
	
	iconLarge: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/images/admin.png"),
	
	constructor: function() {
		if (net.jazz.ajax._hasLocalRepository) {
			this.getUserRoles();
		} else {
			this.redirectToJTSHome();
		}
		jazz.app.auth.setLogoutUrl(net.jazz.ajax._contextRoot + "/service/com.ibm.team.repository.service.internal.ILogoutRestService");
		dojo.addClass(document.body,"com-ibm-team-repository-web-admin-body");
	},	
	
	_fetchInitData: function (){
		var srh = new ServiceResponseHandler(this, "_handleInitData", "_fetchInitDataFailed");
		RepositoryClient.getInitializationData(srh);
	},
	
	getUserRoles: function () {
		var srh= new ServiceResponseHandler(this, "authRequestPassed", "authRequestFailed");		
		AdminClient.getUserRoles(srh);
	},
	
	authRequestPassed: function(response) {
		if (response.userId) {
			dojo.setObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_USERID", response.userId);
		}
		var roles = response.roles;
		for (var i=0; i < roles.length; i++) {
			if (roles[i] === "JazzAdmins") {
				this._userIsAdmin = true;
				break;
			}
		}
		this._fetchInitData();
	},
	
	authRequestFailed: function(errorObject) {
		this.setupNoContributorApplicationUI();
		this.showNotAuthorizedError();
	},
	
	redirectToJTSHome: function() {
		jazz.app.entryPoints.get("JazzTeamServerHome").addCallback(dojo.hitch(this,function(url){
			if (url && url != location.href) {
				location.replace(url);
			} else {
				this.setupNoContributorApplicationUI();
				this.ui.attr("bannerTitle", messages.applicationAdmin);
				this.onFailure(null, messages.invalidAdmin);
			}
		}));
	},
	
	showNotAuthorizedError: function() {
		var notAuth = new com.ibm.team.repository.web.ui.internal.admin.NotAuthorizedView();
		this.onFailure(null, notAuth.domNode);
	},
		
	_handleInitData: function(initData) {
		Session.init(initData);	
		var contrib = Session.getAuthenticatedContributor();
		if (contrib) {
			this.config.user = {
				name: contrib.name,
				userId: contrib.userId
			};	
		} else {
			this._handleNoContributor();
		}
			
		if (this._userIsAdmin == null || !this._userIsAdmin) {
			this._suppressAdminOnlyPages();
		} 
		
		this.config.buttons.push({
			label: messages.admin,
			className: "admin-menu",
			align: "right",
			menuProvider: dojo.hitch(this,this._adminMenuProvider)
		});
		
		this.config.buttons.push({
			label: messages.help,
			className: "help-menu",
			align: "right",
			menuProvider: dojo.hitch(this,this._helpMenuProvider)
		});
		
		var appAbout = jazz.app.internal.applicationAbout.getCurrent() || {};
		var aboutStr = this._aboutStr = (appAbout.identifier == JTS_IDENTIFIER) ? messages.aboutServer : messages.aboutApp;
		var node = dojo.create("a", { href: "#", innerHTML: this.ui.attr("title"), title: aboutStr });
		dojo.connect(node, "onclick", this, this._openAboutBox);
		this.ui.attr("title", node);
		
		if (com.ibm.team.repository.web.logouturi) {
			this.config.logout = { href: com.ibm.team.repository.web.logouturi };
		}

		this.start();
		
		//Check the server mode via the initData and update the Server tab label and Banner	
		var serverMode = dojo.getObject("com.ibm.team.repository.server.mode");
		if(serverMode == "APPLICATION") {
			var appTitle = "";
			if(appAbout.title) {
				appTitle = " - " + appAbout.title;
				if(appAbout.instanceName) {
					appTitle += " (" + appAbout.instanceName + ")";
				}
			}
			this.ui.attr("navbar").overridePageName("com.ibm.team.repository.server", messages.application);
			this.ui.attr("bannerTitle", messages.applicationAdmin + appTitle);
			this.workbench._setDefaultTitle(messages.applicationAdmin + " - " + (appAbout.title || "Rational Jazz Team Server")); //$NON-NLS-1$
		} else {
			this.ui.attr("bannerTitle", messages.serverAdmin);
			this.workbench._setDefaultTitle(messages.serverAdmin + " - Rational Jazz Team Server"); //$NON-NLS-1$
		}
	},
	
	_handleNoContributor: function () {
		// handle case where there is no DB so contributor isn't in init data
		var userId = dojo.getObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_USERID");
		if (userId) {
			this.config.user = {
				name: userId,
				userId: userId
			};
		} else {
			// fake out the Logout path since we don't know the unauthorized user
			this.config.user = {
				name: "",
				userId: ""
			};
		}
	},
	
	_selectContext: function(contextObj) {
		var curUrl = new dojo._Url(location.href);
		if (curUrl.query) {  //check for debug mode
			var query = "?" + curUrl.query;	
		} else {
			var query = "";
		}
		var encodedName = encodeURIComponent(contextObj.name);
		return net.jazz.ajax._contextRoot + "/web/projects/" + encodedName + query;
	},
	
	_fetchInitDataFailed: function(errorObject) {
		// If the user got this far, the user is authenticated as a member of a Jazz J2EE group but we can't access InitData
		this.setupNoContributorApplicationUI();
		
		// Forbidden users see unauth message
		if (errorObject.status === 403) {
			// this is probably an archived user or admin
			this.showNotAuthorizedError();
			return;
		}
		
		// If init data failed, the server must be in a bad state.  So show all pages to JazzAdmins only
		if ((this._userIsAdmin != null && this._userIsAdmin == true)) {
			this.start();
		} else {
			this.onFailure(errorObject);
		}
	},
	
	setupNoContributorApplicationUI: function () {
		this._handleNoContributor();
	},
	
	_suppressAdminOnlyPages: function() {
		dojo.setObject("com.ibm.team.repository.admin.suppressPage", true);
		dojo.setObject("com.ibm.team.repository.web.suppressedPages", {
			"com.ibm.team.repository.web.admin": ["com.ibm.team.repository.server", "com.ibm.team.repository.users"] 
		});
		dojo.setObject("com.ibm.team.repository.web.noAdminAuth", "not authorized");
	},

	getAdminPages: function (){
		var pageIDs = [];
		var pages = net.jazz.ajax.ui.internal.PageRegistry.getPages();
		dojo.forEach(pages, function(page){
			pageIDs.push(page.id);
		});
		return pageIDs;
	},
	_adminMenuProvider: function() {
		return new com.ibm.team.repository.web.ui.internal.admin.AdminMenu();	
	},
	_helpMenuProvider: function() {
		var menu = new jazz.app._ComponentMenu();
		var helpuri = dojo.getObject("com.ibm.team.repository.web.helpuri");
		if (helpuri) {
			this.helpuri = helpuri;
			menu.addItemByArgs({label: messages.helpContents, icon: dojo.moduleUrl("jazz.app","graphics/JazzUI/icons/systemhelp.gif"), onClick: dojo.hitch(this,this._openHelpContents)});
		}
		menu.addItemByArgs({label: this._aboutStr, onClick: dojo.hitch(this,this._openAboutBox)});
		this.contributeToHelpMenu(menu);
		return menu;
	},
	contributeToHelpMenu: function(menu) {
		// to be implemented by subclass - see work item 118683
	},
	_openHelpContents: function(event) {
		if (this.helpuri)
			window.open(this.helpuri,'help');
	},
	_openAboutBox: function(e){
		if (e) e.preventDefault();
		var appAbout = jazz.app.internal.applicationAbout.getCurrent() || {},
			aboutStr = (appAbout.identifier == JTS_IDENTIFIER) ? messages.aboutServer : messages.aboutApp;
		new com.ibm.team.repository.web.ui.internal.about.AboutJazzTeamServer({dialogTitle: aboutStr});
	}
});

})();