/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.AdminMenu");

dojo.require("jazz.app._ComponentMenu");
dojo.require("jazz.app.context");
dojo.require("jazz.app.internal.applicationAbout");
dojo.require("jazz.app.entryPoints");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "AdminApplicationMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "AdminApplicationMessages"); //$NON-NLS-1$ //$NON-NLS-2$
var JTS_IDENTIFIER = "http://jazz.net/application/jts";

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.AdminMenu", jazz.app._ComponentMenu, {
	getSelected: function() {
		var items = this.getItems();
		var hash = dojo.hash();
		var locHref = location.href;
		for (var i = 0; i < items.length; i++) {
			var item = items[i];
			var href = item.href;
			if (href) {
				if ((hash && href.indexOf(hash) >= 0) || locHref.indexOf(href) == 0) {
					return item;
				}
			}
		}
	},
	postCreate: function() {
		var roles = dojo.getObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR_ROLES");
		if (roles) {
			for (var i=0; i < roles.length; i++) {
				if (roles[i] === "JazzAdmins") {
					this.isAdmin = true;
				}
				if (roles[i] === "JazzProjectAdmins") {
					this.isProjectAdmin = true;
				}
			}
		}
		var mode = dojo.getObject("com.ibm.team.repository.server.mode");
		if (!mode || this.isAdmin || this.isProjectAdmin) {
			this._loading = this.addLoading();
			var deferred = jazz.app.entryPoints.get(["CreateLifecycleProject", "LifecycleProjectAdmin"]);
			deferred.addCallback(dojo.hitch(this, this._populateMenu));
		} else {
			this._populateMenu();
		}
	},
	_populateMenu: function(lpaUrls) {
		if (!this.domNode) return;
		var curUrl = new dojo._Url(location.href),
			query = curUrl.query ? "?" + curUrl.query : "",
			adminBase = dojo.baseUrl + "../../admin" + query + "#",
			mode = dojo.getObject("com.ibm.team.repository.server.mode"),
			context = jazz.app.context.get(),
			appAdmin = !!context.applicationAdmin,
			hasRepo = net.jazz.ajax._hasLocalRepository;
		if (appAdmin) {
			adminBase = "#";
		}
		if (this._loading) {
			this.remove(this._loading);
			this._loading.destroy();
			this._loading = null;
		}
		if (hasRepo) {
			this.addTitle((mode == "APPLICATION" || appAdmin) ? messages.applicationAdmin : messages.jtsAdministration);
			if (context && context.itemId) {
				this.addItemByArgs({
					label: messages.manageThisProject,
					href: adminBase + "action=com.ibm.team.process.editProjectArea&itemId=" + context.itemId
				});
				this.addSeparator();
			}
			if (this.isAdmin) {
				if (!appAdmin) {
					this.addItemByArgs({
						label: (mode == "APPLICATION") ? messages.manageApplication : messages.manageServer,
						href: adminBase + "action=jazz.viewPage&id=com.ibm.team.repository.server"
					});
				}
				this.addItemByArgs({
					label: messages.manageUsers,
					href: adminBase + "action=jazz.viewPage&id=com.ibm.team.repository.users"
				});
			}
			var hideProcessPages = dojo.getObject("com.ibm.team.process.service.hidePagesWebUIInitializationDataService");
			if ((appAdmin || mode == "APPLICATION" || mode == "STANDALONE") && !hideProcessPages) {
				this.addItemByArgs({
					label: messages.manageProjectAreas,
					href: adminBase + "action=jazz.viewPage&id=com.ibm.team.process.ProjectAreaManagement"
				});
				this.addItemByArgs({
					label: messages.manageTemplates,
					href: adminBase + "action=jazz.viewPage&id=com.ibm.team.process.ProcessTemplateManagement"
				});
			}
			if (this.isAdmin) {
				this.addItemByArgs({
					label: messages.manageReports,
					href: adminBase + "action=jazz.viewPage&id=com.ibm.team.reports.reportsManagementPage"
				});
			}
		}

		var abouts = jazz.app.internal.applicationAbout.get();
		for (var i in abouts) {
			var about = abouts[i];
			if (about.identifier == JTS_IDENTIFIER) {
				if (!hasRepo || appAdmin || mode == "APPLICATION") {
					this.addTitle(messages.jtsAdministration);
				} else {
					this.addSeparator();
				}
				if (lpaUrls) {
					if (lpaUrls[0]) {
						this.addItemByArgs({
							label: messages.createLPA,
							icon: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/images/create-lpa_obj.png").toString(),
							href: lpaUrls[0]
						});
					}
					if (lpaUrls[1]) {
						this.addItemByArgs({
							label: messages.manageLPA,
							icon: dojo.moduleUrl("jazz.app", "graphics/HomeMenu/lpa_obj.gif").toString(),
							href: lpaUrls[1]
						});
					}
				}
				this.addItemByArgs({
					label: messages.jtsHome,
					icon: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/images/admin_obj.png").toString(),
					href: about.contextRoot + "/admin"
				});
				break;
			}
		}
	}
});
})();