/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.dashboard.web.ui.internal.admin.DashboardAdminPage");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.app.auth");
dojo.require("jazz.client.xhr");
dojo.require("jazz.ui.MessageBox");

var DASHBOARD_MIGRATION_SERVICE = "dashboard-migration";

var CMD_UPDATE_URLS = "updateUrls";

dojo.declare("com.ibm.team.dashboard.web.ui.internal.admin.DashboardAdminPage",
		[dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/DashboardAdminPage.html"),
	
	postCreate: function() {
		this._messageBox = new jazz.ui.MessageBox();
		this.output.appendChild(this._messageBox.domNode);
	},
	
	updateUrls: function() {
		if (!confirm("WARNING: This operation may potentially modify all dashboard resources and the dashboard storage areas for all applications. If used incorrectly, this could result in dashboards not working. Please ensure that you have BACKED UP your databases before proceeding.\n\nClick OK to proceed, or Cancel to abort.")) {
			return;
		}
		
		// read the fields
		var buf = [];
		for (var i=1;i<=5;i++) {
			var oldUrl = this["oldUrl" + i].value;
			if (oldUrl) {
				buf.push("oldUrl" + i + "=" + encodeURIComponent(oldUrl));
			}
			var newUrl = this["newUrl" + i].value;
			if (newUrl) {
				buf.push("newUrl" + i + "=" + encodeURIComponent(newUrl));
			}
		}
		
		// disable UI to prevent multiple simultaneous updates
		this.setEnabled(false);
		
		// call migration service
		jazz.client.xhrPost({
		    url: DASHBOARD_MIGRATION_SERVICE + "/" + CMD_UPDATE_URLS,
			postData: buf.join("&"),
		    load: dojo.hitch(this, function(response) {
				this._messageBox.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.OK,
					summary: "Update completed successfully. Please shut down your JTS and run repotools-jts -reindex before continuing."
				});
				this.setEnabled(true);
		    }),
			error: dojo.hitch(this, function(/* Error */ error, args) {
				error.message = args.xhr.responseText;
				this._messageBox.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: "An error occured while updating URLs. Please check the JTS server log for details.",
					errorObject: error
				});
				this.setEnabled(true);
			})
		});
		
		this._messageBox.showMessage({
			summary: "Updating dashboards... <br/>This may take a while to complete, depending on how many dashboards need to be processed. Please wait. <img src=\"" + dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/Dashboard/icons/spinner.gif") + "\">"
		});
	},
	
	setEnabled: function(/* boolean */ value) {
		for (var i=1;i<=5;i++) {
			this["oldUrl" + i].disabled = !value;
			this["newUrl" + i].disabled = !value;
		}
		this.button.disabled = !value;
	}
});

})();
