/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.ItemList");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.xml.parser");
dojo.require("net.jazz.ajax.ui.PlatformUI");

function isNode(o){
	return o.nodeType && o.cloneNode;
}

function getRow(evt){
	var e = evt.target;
	while(e.parentNode){
		if(e.tagName === "TR"){
			return e;
		}
		e = e.parentNode;
	}
	return null;
}

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.ItemList", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/templates/ItemList.html"),
	
	constructor: function(args) {
		//do stuff
	},

	addNewButton: function(actionHref, label){
		var button = document.createElement("button");
		button.appendChild(document.createTextNode(label));
		button.__jazzActionHref = "#action=" + actionHref;
		this.newItemButtonArea.appendChild(button);
		return this.connect(button, "onclick", "_handleNewItemButtonClick");
	},
	
	setHeaderRow: function(/*Array*/labels){
		if(!this.headerRowLine || !this.headerRow) {
			return;
		}
		dojox.xml.parser.removeChildren(this.headerRowLine);
		dojox.xml.parser.removeChildren(this.headerRow);
		var self = this;
		dojo.forEach(labels, function(label){
			var th = document.createElement("th");
			th.appendChild(document.createTextNode(label));
			self.headerRow.appendChild(th);
			th = document.createElement("th");
			if (dojo.isIE) {
				th.appendChild(document.createTextNode(" "));
			}
			self.headerRowLine.appendChild(th);
		});
	},
	
	addRow: function(actionHref, /*Array*/columns){
		var row = document.createElement("tr");
		if (actionHref != null) {
			row.__jazzActionHref = "#action=" + actionHref;
		}
		this.tbody.appendChild(row);
		for(var i=0; i<columns.length; i++){
			var td = row.insertCell(i);
			if(i===0 && row.__jazzActionHref != null){
				var link = document.createElement("a");
				link.href = row.__jazzActionHref;
				this.connect(link, "onfocus", "_handleFocus");
				this.connect(link, "onblur", "_handleMouseOut");
				td.appendChild(link);
				td = link;
			}
			var o = columns[i];
			if(dojo.isString(o)){
				td.appendChild(document.createTextNode(o));
			} else if(isNode(o)){
				td.appendChild(o);
			}
		}
		return row;
	},
	
	handleError: function(errorObj){
		this.header.showMessage({
			errorObject: errorObj
		});
	},
	
	_handleNewItemButtonClick: function(e){
		if(e.target.__jazzActionHref){
			window.location.href = e.target.__jazzActionHref;
		}
	},
	
	_handleRowClick: function(e){
		var row = getRow(e);
		if (row.__jazzActionHref != null) {
			if (e.target.tagName === "A") {
				window.location.href = row.__jazzActionHref;
			}
		}
	},
	
	_handleFocus: function(e) {
		var row = getRow(e);
		if (row && (!row.collapsable || e.target.tagName === "A")) {
			dojo.toggleClass(row, "selected", true);
		}
	},
	
	_handleMouseOver: function(e) {
		//ensure that no other rows are "selected" because of keyboard focus
		var rows = this.tbody.rows;
		var i;
		for (i=0; i<rows.length; i++) {
			dojo.toggleClass(rows.item(i), "selected", false);
		}
		
		//highlight the current row
		var row = getRow(e);
		if (row && (!row.collapsable || e.target.tagName === "A")) {
			dojo.toggleClass(row, "selected", true);
		}
	},
	
	_handleMouseOut: function(e){
		var row = getRow(e);
		if (row && (!row.collapsable || e.target.tagName === "A")) {
			dojo.toggleClass(row, "selected", false);
		}
	}

});

})();