/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.NotAuthorizedView");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("dojo.i18n");
dojo.require("jazz.app.auth");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin", "NotAuthorizedViewMessages");

var getAuthenticatedUserId  = com.ibm.team.repository.web.client.session.getAuthenticatedUserId;

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.NotAuthorizedView", [dijit._Widget, dijit._Templated], {

	templateString: "<div class=\"com-ibm-team-repository-admin-noAuth\" dojoAttachPoint=\"notAuthArea\"></div>",

	applicationName: "Jazz Team Server Admin UI", 
	
	// custom messages that will overwrite the default messages if defined.
	customNoUserMessage: null,
	customNotAuthorizedMessage: null,
	
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin", "NotAuthorizedViewMessages");
	},

	postCreate: function() {
		var loginUser = getAuthenticatedUserId();

		if (loginUser == "UNKNOWN") {
			// Inform the user that their ID is unknown.
			// Then either use a generic message informing them that they
			// are not authorized or use a custom message if it was defined as a parameter
			var accessMessage = this.messages.unknownUser;
			accessMessage += "<br /><br />";
			accessMessage += (this.customNoUserMessage || dojo.string.substitute(this.messages.noUser, [ this.applicationName ]));
		
		} else {
			var accessMessage;
			if(this.customNotAuthorizedMessage) {
				accessMessage = this.customNotAuthorizedMessage;
			} else {
				var userFormated = "<strong>" + getAuthenticatedUserId() + "</strong>";
				accessMessage = dojo.string.substitute(this.messages.notAuthorized, [ userFormated, this.applicationName ]);	
			}
		}

		this.notAuthArea.innerHTML = accessMessage; 
		
		if(this.showLogout) {
			var div = dojo.create("div", { className: "logoutMessage" }, this.notAuthArea);
			var logoutLink = dojo.create("a", { innerHTML: this.messages.logout, href: "#" }, div);
			this.connect(logoutLink, "onclick", function (e) {
				dojo.stopEvent(e);
				var dfd = jazz.app.auth.logout();
				dfd.addCallback(function(){ window.location.reload(); });
			});
		}
	}
	
});

