/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
    
dojo.provide("com.ibm.rdm.fronting.server.web.ui.internal.admin.ServerPage");
    
dojo.require("dijit._Widget");
dojo.require("dojo.i18n");
dojo.require("jazz.ui.SideNavigationLayout");
dojo.require("com.ibm.rdm.fronting.server.web.ui.internal.admin.ServerStatus");
dojo.require("com.ibm.rdm.fronting.server.web.ui.internal.admin.Counters");
dojo.require("com.ibm.rdm.fronting.server.web.ui.internal.admin.ViewLogs");
dojo.require("com.ibm.rdm.fronting.server.web.ui.internal.admin.StorageAreas");
dojo.require("com.ibm.rdm.fronting.server.web.ui.internal.admin.AdvancedProperties");
dojo.require("com.ibm.rdm.fronting.server.web.ui.internal.admin.Debug");
dojo.requireLocalization("com.ibm.rdm.fronting.server.web.ui.internal.admin", "AdminApplicationMessages");
    
var ACTION_SETS = [
   	{ labelKey: 'status', actions: [
   		{labelKey: 'statusSummary', shortId: "serverStatus", widget: "ServerStatus"},
   		{labelKey: 'statistics', shortId: "statistics", widget: "Counters"},
//   		{labelKey: 'docGen', shortId: "docGen", widget: "DocumentGeneration"}
   	]},
   	{ labelKey: 'configuration', actions: [
   		{labelKey: 'storageAreas', shortId: "storageAreas", widget: "StorageAreas"},
   		{labelKey: 'logging', shortId: "logging", widget: "ViewLogs"},
   		{labelKey: 'advancedProperties', shortId: "advancedProperties", widget: "AdvancedProperties"}
   	]},
   	{ labelKey: 'debug', actions: [
   		{labelKey: 'debugService', shortId: "debug", widget: "Debug"}
   	]}
];

var ActionRegistry = null;
var PREFIX         = "com.ibm.rdm.fronting.server.web.";
var SERVER_PACKAGE = "com.ibm.rdm.fronting.server.web.ui.internal.admin.";
var ACTION_MAP     = {};
    
dojo.declare("com.ibm.rdm.fronting.server.web.ui.internal.admin.ServerPage", dijit._Widget, {
        
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.rdm.fronting.server.web.ui.internal.admin", "AdminApplicationMessages"); //$NON-NLS-1$ //$NON-NLS-2$
    },

    postCreate: function() {
        //Set the root style for the Server page and all of its sub-pages
		this.domNode.className = "com-ibm-team-repository-admin";
		
		//Instantiate the side-nav layout UI and append it to the page
		this._appView = new jazz.ui.SideNavigationLayout();
		this.domNode.appendChild(this._appView.domNode);
		
		//Register default action and create side-nav hyperlinks
		ActionRegistry = getWorkbench().getActionRegistry();
		ActionRegistry.registerAction("com.ibm.rdm.fronting.server.web._defaultAction", this, "_defaultAction");
   
		/* To add a new page, you must:
		 *  1) create the widget
		 *  2) register the action in plugin.xml
		 *  3) add the action and page to the ACTION_SETS variable
		 *  4) add the new page as an import (requires)
		 * 
		 *  For additional examples, see JF\com.ibm.team.repository.web.jts\resources\ui\internal\admin\server\ServerPage.js
		 */
		
		//Register the remaining actions upfront before we can fire any XHR requests (prevents action=null errors)
		var self = this;
		dojo.forEach(ACTION_SETS, function(actionSet) {
			dojo.forEach(actionSet.actions, function(action){
				self._registerAction(action);
			});
		});
		
		this._processActionSets();
    },
    
	_processActionSets: function() {
		var self = this;
		this.allLinks = [];
		dojo.forEach(ACTION_SETS, function(actionSet) {
			self._createNavSection(actionSet);
		});
	},
       
	_createNavSection: function(actionSet) {
		var self = this;
		var navSection = this._appView.addNavigationSection(document.createElement("div"), this.messages[actionSet.labelKey], true);
		dojo.forEach(actionSet.actions, function(action){
			var id = PREFIX + action.shortId;
			var link = navSection.addSectionRow(self.messages[action.labelKey], "#action=" + id);
			link.__jazzActionId = action.shortId;
			self.allLinks.push(link);
			navSection.addArtifactCommand(link,self.messages.refresh,dojo.hitch(self,self._onRefreshClick),"admin-refresh-command");
		});
	},
	
    _defaultAction: function() {
    	//By default, show the first section from the left-nav
		this._handleAction({action: PREFIX + ACTION_SETS[0].actions[0].shortId});
	},
    
	_onRefreshClick: function(e) {
		dojo.stopEvent(e);
		this.reloadCurrent();
	},
	
	reloadCurrent: function() {
		this._previousActionEntry = this._currentActionEntry || null;
		this._showCurrentWidget(/*isReload*/true);
	},
	
	_registerAction: function(action){
		var id = PREFIX + action.shortId;
		ACTION_MAP[action.shortId] = action;
		ActionRegistry.registerAction(id, this, "_handleAction");
	},
	
	_handleAction: function(args) {
		if(!args.action) {
			console.debug("ServerPage.js#_handleAction: missing required param 'args.action'");
			return;
		}
		
		// cache the previous entry to see if the widget should be destroyed or
		// not
		this._previousActionEntry = this._currentActionEntry || null;
		
		// parse out the short id from the fully-qualified id
		var shortId = args.action.substring(args.action.lastIndexOf(".") + 1);
		this._currentActionEntry = ACTION_MAP[shortId];
		this._currentArgs = args;
		this._currentArgs.parent = this;

		// set the title for the action
		getWorkbench().setTitle(this.messages[this._currentActionEntry.labelKey]);
		
		// highlight the selected link and unhighlight the others
		dojo.forEach(this.allLinks, function(link){
			if (link.__jazzActionId === shortId)
				dojo.addClass(link, "selected");
			else
				dojo.removeClass(link, "selected");
		});
		
		// call the actual method implementation
		this._showCurrentWidget();
	},
	
	_showCurrentWidget: function(isReload) {
		// save a reference to the prior content widget
		this._previousContentWidget = this._currentContentWidget || null;
		
		if(this._currentActionEntry.cached) {
			this._showCachedWidget(isReload);
		} else {
			this._showUncachedWidget();
		}
		
		// help the garbage collector do its job
		if(this._previousContentWidget && !this._previousActionEntry.cached) {
			this._previousContentWidget.destroyRecursive();
		}
	},
	
	_showCachedWidget: function(isReload) {
		var className = this._currentActionEntry.widget;
		if(this._widgetCache[className]) {
			// get widget from the cache if it exists
			this._currentContentWidget = this._widgetCache[className];
		} else {
			// otherwise instantiate it and add it to the cache
			var constructor = dojo.getObject(SERVER_PACKAGE + className);
			this._currentContentWidget = new constructor(this._currentArgs);
			dojo.addClass(this._currentContentWidget.domNode, "com-ibm-team-repository-admin-contentArea");
			this._widgetCache[className] = this._currentContentWidget;
		}
		if(isReload) {
			this._currentContentWidget.reload(this._currentArgs);
		} else {
			this._currentContentWidget.show(this._currentArgs);
			this._appView.setMainContent(this._currentContentWidget.domNode);
		}
	},
	
	_showUncachedWidget: function() {
		var constructor = dojo.getObject(SERVER_PACKAGE + this._currentActionEntry.widget);	
		this._currentContentWidget = new constructor(this._currentArgs);
		dojo.addClass(this._currentContentWidget.domNode, "com-ibm-team-repository-admin-contentArea");
		this._appView.setMainContent(this._currentContentWidget.domNode);
	}
});
  
function getWorkbench() {
	return net.jazz.ajax.ui.PlatformUI.getWorkbench();
}

})();