/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
    
dojo.provide("com.ibm.rdm.fronting.server.web.ui.internal.admin.ServerStatus");
    
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("jazz.ui.MessageBox");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.DataTableContainer");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.rdm.fronting.server.web.ui.internal.admin.ServiceClient");
dojo.require("com.ibm.rdm.fronting.server.web.util.domutils");

dojo.requireLocalization("com.ibm.rdm.fronting.server.web.ui.internal.admin", "AdminApplicationMessages");

var ServiceClient = com.ibm.rdm.fronting.server.web.ui.internal.admin.ServiceClient;
var domutils = com.ibm.rdm.fronting.server.web.util.domutils;
var MessageBox = jazz.ui.MessageBox;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var DataTableContainer = com.ibm.team.repository.web.ui.internal.admin.DataTableContainer;

var JTS_PRODUCT_ID = "com.ibm.team.jazz.foundation.server";
var STATUS_OK_IMG_SRC = dojo.moduleUrl("jazz.ui", "graphics/MessageBox/success_obj.gif");
var STATUS_ERR_IMG_SRC = dojo.moduleUrl("jazz.ui", "graphics/MessageBox/error_obj.gif");

dojo.declare("com.ibm.rdm.fronting.server.web.ui.internal.admin.ServerStatus", [dijit._Widget, dijit._Templated], {
    
    templatePath: dojo.moduleUrl("com.ibm.rdm.fronting.server.web", "ui/internal/admin/templates/ServerStatus.html"),
	_dataTables: [],
	_tooltips: [],
    
    constructor: function(){
        this.messages = dojo.i18n.getLocalization("com.ibm.rdm.fronting.server.web.ui.internal.admin", "AdminApplicationMessages");
    },
    
    postCreate: function(){
        this.header = new MessageBox();
        this.headerArea.appendChild(this.header.domNode);
        this._pendingResponseCount = 0;

		this.reload();
    },
    
    reload: function(){
        this.header.hideMessage();           
		
		this._fetchAndDisplayPublicURI();
		this._fetchAndDisplayInfo();
		this._fetchAndDisplayDiagnostics();
    },
    
	_fetchAndDisplayPublicURI: function() {
    	this._emptyIfDomExists(this.serverStatusTable);
    	this.handleLoadingData();
		var srh = new ServiceResponseHandler(this, "displayPublicUrl", "handleErrorReponse");
		ServiceClient.getServiceConfiguration(srh);
	},
	
    _fetchAndDisplayInfo: function(){
		this._emptyIfDomExists(this.infoArea);
        this.handleLoadingData();
        var srh = new ServiceResponseHandler(this, "displayInfo", "handleErrorReponse");
        ServiceClient.getRootServices(srh);
        
        this._fetchAndDisplayProductInfo();
    },
    
    _fetchAndDisplayProductInfo: function(){
        this.handleLoadingData();
        var srh = new ServiceResponseHandler(this, "displayProductInfo", "handleErrorReponse");
        ServiceClient.getAllProductInfo(srh);
    },
   
	_fetchAndDisplayDiagnostics: function() {
    	this._emptyIfDomExists(this.serverDiagnosticsTable);
    	this.handleLoadingData();
		var srh = new ServiceResponseHandler(this, "displayDiagnostics", "handleErrorReponse");
		ServiceClient.getServerHealth(srh);
	},
    
	displayPublicUrl: function(serviceInfo) {
		var label = ServiceClient.getPublicURI(serviceInfo);

		var pDiv = dojo.create("div", {}, this.infoArea);
        var heading = {
            label: this.messages.serverStatus
        };
        
        var dataBlock = [{
            label: this.messages.publicURL,
            value: dojo.create("a", { innerHTML: label, href: label })
        }];
    	
        this._appendInfoBlock(pDiv, heading, dataBlock);
        
		this.handleDataLoaded();
	},
	
    /**
     * Show the version
     */
    displayInfo: function(data){ 	
    	
		//parse response	
    	var version = domutils.parseResponse(data, "buildVersion", "http://open-services.net/xmlns/rm/1.0/", "oslc_rm");
    	
    	var pDiv = dojo.create("div", {}, this.infoArea);
        var heading = {
            label: this.messages.buildInformation
        };
        
        var dataBlock = [{
            label: this.messages.name,
            value: version
        }];
    	
        this.versionTable = this._appendInfoBlock(pDiv, heading, dataBlock);
    	
        this.handleDataLoaded();
    },
    
    displayDiagnostics: function(data){ 	
    	
    	//create section
    	var pDiv = dojo.create("div", {}, this.diagnosticsArea);
        var heading = {
            label: this.messages.diagnostics
        };
    	
        this.diagnosticsTable = this._appendInfoBlock(pDiv, heading, null);
      	
        for (var check in data) {
        	if (data.hasOwnProperty(check)) {
        		var label = check; 
        		var status = "OK";
        		
        		for (var sub in data[check]) {
                	if (data[check].hasOwnProperty(sub)) {
                		if (data[check][sub] !== "OK") {
                			status = sub + " (" + data[check][sub] + ")";
                			break;
                		}
                	}
                }
        		
        		this._setStatus(this.diagnosticsTable, label, status);
        	}
        }

        this.handleDataLoaded();
    },
    
    displayProductInfo: function(prodInfo){
		//If productInfoArea is undefined, the DOM has been destroyed already (page switched).
		if(!this.infoArea) return;
		
		if(!this.versionTable) return;
		
		var self = this;
		
        if (prodInfo) {
            this.prodInfo = prodInfo;
            dojo.forEach(this.prodInfo, function(product){
            	if(product.id == JTS_PRODUCT_ID) {
    				self._insertRow(self.versionTable, {
    					label: product.name,
	                    value: product.version + " (" + product.buildId + ")"
    	            });
				}
				
                dojo.forEach(product.patches, function(patch){
                	if(product.id == JTS_PRODUCT_ID) {
                		self._insertRow(self.versionTable, {
                			label: patch.name,
	                    	value: patch.version + " (" + patch.buildId + ")",
	                    	labelStyle: "patch patchName",
	                    	style: "patch"
        	            });
					}
                });
            });
            
        }
        this.handleDataLoaded();
    },
    
    _appendInfoBlock: function(node, heading, infoList){
        var infoTable = new DataTableContainer({
            heading: heading
        });
		this._dataTables.push(infoTable);
		dojo.forEach(infoList, function(item) {
			this._insertRow(infoTable, item);
		}, this);
		var infoBlock = dojo.create("div", { className: "stats-block" }, node);
        infoBlock.appendChild(infoTable.domNode);
        return infoTable;
    },
    
    _insertRow: function(table, item){
        var rowData = {
            0: {
                content: item.label,
                // Allows the label to have additional styles other than default
                style: "rank tight double" + (item.labelStyle ? " " + item.labelStyle : "")
            },
            1: {
                content: item.value,
                style: item.style
            }
        };
        table.addRow(rowData, 2);
    },
    
	_emptyIfDomExists: function(node) {
		if(node) {
			node.innerHTML = "";
			return true;
		} else {
			return false;
		}
	},
    
    _setStatus: function(table, label, statusString) {
    	var node = dojo.create("div");
		var img = dojo.create("img", { className: "status-icon" }, node);
		dojo.create("span", { innerHTML: statusString }, node);
		
		//Set the correct status icon
		switch (statusString) {
		case "OK": 
			this._setStatusIcon(img, "success");
			break;
		default: 
			this._setStatusIcon(img, "error");
		}
		
        this._insertRow(table, {
			label: label,
        	value: node
        });
    },
    
	_setStatusIcon: function(image, status) {
		if(!image) return;
		switch (status) {
			case "error": 
				image.src = STATUS_ERR_IMG_SRC;
				image.alt = this.messages.status_error;
				break;
			case "success": 
				image.src = STATUS_OK_IMG_SRC;
				image.alt = this.messages.status_ok;
				break;
			default: 
				//Set the default to OK
				image.src = STATUS_OK_IMG_SRC;
				image.alt = this.messages.status_ok;
		}
		dojo.removeClass(image, "noDisplay");
	},
	
	handleLoadingData: function() {
        this._pendingResponseCount++;
        getWorkbench().showMessage(this.messages.loadingData);
    },
    
    handleDataLoaded: function() {
        this._pendingResponseCount--;
        if (this._pendingResponseCount === 0) {
            getWorkbench().hideMessageArea();
        }
    },
    
    handleErrorReponse: function(errorObj) {		
        this._displayErrorMessage(this.messages.errorFetchingServerStatus, errorObj);
        this.handleDataLoaded();
    },
    
    _displayErrorMessage: function(message, errorObj) {  
        if (errorObj){
			errorObj.message = errorObj.message ? errorObj.message: message;
		} else{
			var errorObj={
				message: message,
				status: STATUS_TYPES.ERROR
			}
		}
		
        this.header.showMessage({
            errorObject: errorObj,
			summary: message
        });
    }
    
});
    
function getWorkbench(){
    return net.jazz.ajax.ui.PlatformUI.getWorkbench();
}

})();
