/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
    
dojo.provide("com.ibm.rdm.fronting.server.web.ui.internal.admin.StorageAreas");
    
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("jazz.ui.MessageBox");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.DataTableContainer");

dojo.requireLocalization("com.ibm.rdm.fronting.server.web.ui.internal.admin", "AdminApplicationMessages");

var ServiceClient = com.ibm.rdm.fronting.server.web.ui.internal.admin.ServiceClient;
var domutils = com.ibm.rdm.fronting.server.web.util.domutils;
var MessageBox = jazz.ui.MessageBox;
var DataTableContainer = com.ibm.team.repository.web.ui.internal.admin.DataTableContainer;
var runType;

var CONFIGURATION_VERSION = "3"; //$NON-NLS-1$
var DOORSRP_REQUEST_TYPE = "private"; //$NON-NLS-1$

dojo.declare("com.ibm.rdm.fronting.server.web.ui.internal.admin.StorageAreas", [dijit._Widget, dijit._Templated], {
    
    templatePath: dojo.moduleUrl("com.ibm.rdm.fronting.server.web", "ui/internal/admin/templates/StorageAreas.html"),
	_dataTables: [],
	_tooltips: [],
    
    constructor: function(){
        this.messages = dojo.i18n.getLocalization("com.ibm.rdm.fronting.server.web.ui.internal.admin", "AdminApplicationMessages");
    },
    
    postCreate: function(){
        this.header = new MessageBox();
        this.headerArea.appendChild(this.header.domNode);

		this.connect(this.replaceStorageAreasButton, "onclick", "reconfigureOperation"); //$NON-NLS-1$ $NON-NLS-2$
		this.connect(this.reindexStorageAreasButton, "onclick", "reindexOperation"); //$NON-NLS-1$ $NON-NLS-2$
		this.replaceStorageAreasButton.disable = true;
		this.reindexStorageAreasButton.disable = true;

		this.domOperation1 = this._addOperationDiv();
		this.domOperation2 = this._addOperationDiv();		
		this.domOperation3 = this._addOperationDiv();	
		this.reload();
    },
    
    reload: function(){
        this.header.hideMessage();
        this._hideIcons();

		this._checkConfigurationVersion();
		
		this._displayReplaceStorageAreas();
		this._displayReindexStorageAreas();
    },
    
	_displayReplaceStorageAreas: function() {
    	this._emptyIfDomExists(this.configureArea);
    	
    	var pDiv = dojo.create("div", {}, this.configureArea);
        var heading = {
            label: this.messages.replaceStorageAreas
        };
        
        var dataBlock = [
        	{
            	label: "",
            	value: this.messages.replaceStorageAreasInstructions    
        	},
        	{
            	label: "",
            	value: this.replaceStorageAreasButton    
        	},
        	{
            	label: this.domOperation1,
            	value: this.messages.replaceStorageAreasStep1    
        	},
        	{
            	label: this.domOperation2,
            	value: this.messages.replaceStorageAreasStep2   
        	}
        ];
    	
		var infoTable = new DataTableContainer({
			heading: heading
        });
		this._dataTables.push(infoTable);
		dojo.forEach(dataBlock, function(item) {
			this._insertRow(infoTable, item);
		}, this);
		var infoBlock = dojo.create("div", { className: "config-block" }, pDiv);
        infoBlock.appendChild(infoTable.domNode);
	},
	
    _displayReindexStorageAreas: function(){
		this._emptyIfDomExists(this.reindexArea);
		
		var pDiv = dojo.create("div", {}, this.reindexArea);

        var heading = {
            label: this.messages.reindexStorageAreas
        };

		        
        var dataBlock = [
        	{
            	label: "",
            	value: this.messages.reindexStorageAreasInstructions
        	},
        	{
            	label: "",
            	value: this.reindexStorageAreasButton
        	},
        	{
            	label: this.domOperation3,
            	value: this.messages.reindexStorageAreasStep1	
        	}
        ];
    	
		var infoTable = new DataTableContainer({
			heading: heading
        });
		this._dataTables.push(infoTable);
		dojo.forEach(dataBlock, function(item) {
			this._insertRow(infoTable, item);
		}, this);
		var infoBlock = dojo.create("div", { className: "config-block" }, pDiv);
        infoBlock.appendChild(infoTable.domNode);
    },

    _insertRow: function(table, item){
        var rowData = {
            0: {
                content: item.label,
                // Allows the label to have additional styles other than default
                //style: "rank tight double" + (item.labelStyle ? " " + item.labelStyle : "")
            },
            1: {
                content: item.value,
                style: item.style
            }
        };
        table.addRow(rowData, 2);
    },
    
	_emptyIfDomExists: function(node) {
		if(node) {
			node.innerHTML = "";
			return true;
		} else {
			return false;
		}
	},
    
    //Creates a new DIV element for the operation and appends it to the DOM
	//Returns: a handle to the DIV element
	_addOperationDiv: function() {
		var div = dojo.create("div");   //$NON-NLS-1$ $NON-NLS-2$
		dojo.create("p", { className: "icon" }, div);   //$NON-NLS-1$ $NON-NLS-2$
		return div;
	},
    
    _checkConfigurationVersion: function() {
		var self = this;
		this.replaceStorageAreasButton.disabled = true;					
		this.reindexStorageAreasButton.disabled = true;
		this._displayMsgInfo(this.messages.testingConfiguration);
			
		var result = jazz.client.xhrGet({
			url: net.jazz.ajax._contextRoot + "/project-resources/configure",
			handleAs: "text", //$NON-NLS-1$
			headers: {
		    	"Accept": "*",  //$NON-NLS-1$ $NON-NLS-2$
		    	"DoorsRP-Request-Type": DOORSRP_REQUEST_TYPE //$NON-NLS-1$ 
		    },
		    preventCache: "true",
			load: function(data, ioargs){
				if (data == CONFIGURATION_VERSION) {
					//Configuration found. Can reconfigure, rebuild index
					self.replaceStorageAreasButton.disabled = false;					
					self.reindexStorageAreasButton.disabled = false;
					self._displayMsgOK(self.messages.configurationOK);					
				} else {
					//Configuration found, but version does not match. Need to finalize app first
					self._displayMsgWarning(self.messages.configurationNotOK);		
				}
			},
			error: function(error, ioargs){
				self._displayMsgError(dojo.string.substitute(self.messages.configurationNotFound, [ioargs.xhr.status]));
			}	        
		});	
	},
	
   	reconfigureOperation: function(e) {
		//Disable buttons to prevent clicking while processing		
		this.replaceStorageAreasButton.disabled = true;
		this.reindexStorageAreasButton.disabled = true;	
		this._hideOperationsMessages();
		this._hideIcons();
        		
		this._displayMsgInfo(this.messages.reconfigureProgress);		

		//this.runType = 1;
		this.operation1(this.messages.reconfigureOperation, 1);	
	},
	
	reindexOperation: function(e) {
		//Disable buttons to prevent clicking while processing
		this.replaceStorageAreasButton.disabled = true;		
		this.reindexStorageAreasButton.disabled = true;
		this._hideOperationsMessages();
		this._hideIcons();
		        				
		this._displayMsgInfo(this.messages.rebuildProgress);		
		
		//this.runType = 2;		
		this.operation2(this.messages.rebuildOperation, 2);		
	},
	   
	//Operation 1: Reconfigure
	operation1: function(operationTypeMessage, runType) {
		var self = this;	
		var domNode = this.domOperation1;
		dojo.removeClass(domNode, "icon");		
		dojo.addClass(domNode, "icon-running");

		var dConfigure = this.reconfigure();	 
		dConfigure.addCallback(function(response){
			dojo.removeClass(domNode, "icon-running");			
			dojo.addClass(domNode,"icon-ok");				
			self.operation2(operationTypeMessage, runType);
		});
		dConfigure.addErrback(function(errObject){
			self._displayMsgError(dojo.string.substitute(self.messages.operationFailed, [operationTypeMessage]));
			dojo.removeClass(domNode, "icon-running");			
			dojo.addClass(domNode,"icon-error");						
		});
	},
	
	//Operation 2: Rebuild
	operation2: function(operationTypeMessage, runType) {	
		var self = this;
		var domNode;
		if (runType == 1) {
			domNode = this.domOperation2;
		} else {
			domNode = this.domOperation3;			
		}
		dojo.removeClass(domNode, "icon");		
		dojo.addClass(domNode, "icon-running");
		var dRebuild = this.rebuild();
		dRebuild.addCallback(function(response){
			//reenable buttons
			self.replaceStorageAreasButton.disabled = false;		
			self.reindexStorageAreasButton.disabled = false;
			self._displayMsgOK(dojo.string.substitute(self.messages.operationComplete, [operationTypeMessage]));
			dojo.removeClass(domNode, "icon-running");			
			dojo.addClass(domNode,"icon-ok");				
		});
		dRebuild.addErrback(function(errObject){
			self._displayMsgError(dojo.string.substitute(self.messages.operationFailed, [operationTypeMessage]));
			dojo.removeClass(domNode, "icon-running");			
			dojo.addClass(domNode,"icon-error");									
		});
	},
	
	reconfigure: function() {
		var deferred = new dojo.Deferred();
		jazz.client.rawXhrPost({
			url: net.jazz.ajax._contextRoot + "/configure?setup=true", //$NON-NLS-1$
			headers: {
		    	"DoorsRP-Request-Type": DOORSRP_REQUEST_TYPE //$NON-NLS-1$ 
		    },
			load: function(response) {
				deferred.callback(response);
			},
			error: function(errObject) {
				deferred.errback(errObject);		
			}
		});	
		return deferred;
	},
	
	rebuild: function() {
		var deferred = new dojo.Deferred();
		jazz.client.rawXhrPost({
			url: net.jazz.ajax._contextRoot + "/configure?rebuild=true", //$NON-NLS-1$
			headers: {
		    	"DoorsRP-Request-Type": DOORSRP_REQUEST_TYPE //$NON-NLS-1$ 
		    },
			load: function(response) {
				deferred.callback(response);
			},
			error: function(errObject) {
				deferred.errback(errObject);		
			}
		});	
		return deferred;
	},
	
	_hideOperationsMessages: function() {
		this._hideOperationMessages(this.domOperation1);
		this._hideOperationMessages(this.domOperation2);
		this._hideOperationMessages(this.domOperation3);		
	},
	
	_hideOperationMessages: function(node) {
		dojo.removeClass(node, "ok");  //$NON-NLS-1$
		dojo.removeClass(node, "error");  //$NON-NLS-1$	
		dojo.removeClass(node, "running");  //$NON-NLS-1$
	},

	_hideIcons: function() {
		this._hideIcon(this.domOperation1);
		this._hideIcon(this.domOperation2);
		this._hideIcon(this.domOperation3);				
	},

	_hideIcon: function(node) {
		if (dojo.hasClass(node, "icon-running")) {
			dojo.removeClass(node, "icon-running");			
		}
		if (dojo.hasClass(node, "icon-ok")) {		
			dojo.removeClass(node, "icon-ok");
		}
		if (dojo.hasClass(node, "icon-error")) {		
			dojo.removeClass(node, "icon-error");
		}				
	},
	
	_displayMsgOK: function(message) {
		this.header.showMessage({
			type: MessageBox.STATUS_TYPES.OK, 		
			summary: message
		});
	},

	_displayMsgInfo: function(message) {
		this.header.showMessage({
			type: MessageBox.STATUS_TYPES.INFO, 		
			summary: message
		});
	},

	_displayMsgError: function(message) {
		this.header.showMessage({
			type: MessageBox.STATUS_TYPES.ERROR, 		
			summary: message
		});
	},

	_displayMsgWarning: function(message) {
		this.header.showMessage({
			type: MessageBox.STATUS_TYPES.WARNING, 		
			summary: message
		});
	},
   
    _displayErrorMessage: function(message, errorObj) {  
        if (errorObj){
			errorObj.message = errorObj.message ? errorObj.message: message;
		} else{
			var errorObj={
				message: message,
				type: MessageBox.STATUS_TYPES.ERROR
			}
		}
		
        this.header.showMessage({
            errorObject: errorObj,
			summary: message
        });
    }
    
});
    
function getWorkbench(){
    return net.jazz.ajax.ui.PlatformUI.getWorkbench();
}

})();
