/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.process_events.ManageCrossAppEvents"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("dojox.xml.parser"); //$NON-NLS-1$
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dijit._Templated"); //$NON-NLS-1$ 
	dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$       
	dojo.require("com.ibm.team.process.web.ui.internal.admin.process_events.DisplayRegListeners"); //$NON-NLS-1$ 
	dojo.require("com.ibm.team.process.web.ui.internal.admin.process_events.RegisterListenerWizard"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.process_events.QuickRegistrationForm"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.process_events", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$    
	var PlatformUI = net.jazz.ajax.ui.PlatformUI;
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var DisplayRegListeners = com.ibm.team.process.web.ui.internal.admin.process_events.DisplayRegListeners;
	var RegisterListenerWizard = com.ibm.team.process.web.ui.internal.admin.process_events.RegisterListenerWizard;
	var QuickRegistrationForm = com.ibm.team.process.web.ui.internal.admin.process_events.QuickRegistrationForm;
	
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.process_events.ManageCrossAppEvents", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		//  summary:
		//      This embedable ui widget used for managing cross application event listeners.  It gets the registered event 
		//      listeners from the given rootservice .
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/process_events/templates/ManageCrossAppEvents.html"), //$NON-NLS-1$ //$NON-NLS-2$
		_rootservices: null,
		_displayRegListenerWidget: null,
		_registerListenerWizard: null,
		
		constructor: function(){
			var searchString = location.search.replace(/^\?/, ''); //$NON-NLS-1$ // remove leading '?'
			var filterArguments = dojo.queryToObject(searchString);
			this._rootservices = filterArguments.rootservices;
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.process_events", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		postCreate: function(){
			this._showListeners();
		},
		
		_showListeners: function(){
			var onAdd = dojo.hitch(this, this.addListenerWizard);
			var onQuickAdd = dojo.hitch(this, this.quickAddListenerDialog);
			dojox.xml.parser.removeChildren(this._manageCrossAppEvent);
			if (!this._displayRegListenerWidget) {
				this._displayRegListenerWidget = new DisplayRegListeners({
					rootservices: this._rootservices,
					onAdd: onAdd,
					onQuickAdd: onQuickAdd
				});
			}
			this._manageCrossAppEvent.appendChild(this._displayRegListenerWidget.domNode);
		},
		
		
		//  summary:
		//      This method is to invoke the register listener wizard.
		addListenerWizard: function(event){
			dojo.stopEvent(event);
			dojox.xml.parser.removeChildren(this._manageCrossAppEvent);
			var onCancel = dojo.hitch(this, this._showListeners);
			var onFinish = dojo.hitch(this, function(newItem){
				if (newItem) {
					this._addListener(newItem);
				}
				this._showListeners();
			});
			
			if (this._registerListenerWizard) {
				this._registerListenerWizard.destroyRecursive();
			}
			
			this._registerListenerWizard = new RegisterListenerWizard({
				rootservices: this._rootservices,
				onCancel: onCancel,
				onFinish: onFinish
			});
			this._manageCrossAppEvent.appendChild(this._registerListenerWizard.domNode);
		},
		
		
		_addListener: function(newItem){
			this._displayRegListenerWidget.handleAddListenerSuccess(newItem);
			console.debug(newItem);
		},
		
		//  summary:
		//      This method is to invoke the Quick Add Listener Dialog		
		quickAddListenerDialog: function(event){
			dojo.stopEvent(event);
			var onSuccess = dojo.hitch(this, function(newItem){
				if (newItem) {
					this._displayRegListenerWidget.handleAddListenerSuccess(newItem);
				}
				this._dialog.close();
				this._dialog.destroyRecursive();
			});
			var onCancel = dojo.hitch(this, function(newItem){
				this._dialog.close();
				this._dialog.destroyRecursive();
			});
			var quickRegistrationForm = new QuickRegistrationForm({
				rootservices: this._rootservices,
				onSuccess: onSuccess,
				onCancel: onCancel
			});
			this._dialog = new jazz.ui.Dialog({
				contentNode: quickRegistrationForm.domNode,
				primaryTitle: this.messages.ManageCrossAppEvents_1,
				width: '31em', //$NON-NLS-1$
				height: '20em', //$NON-NLS-1$
				destroyContents: true
			});
			quickRegistrationForm.reSize();
		},
		
		
		
		uninitialize: function(){
			delete this._rootservices;
			delete this.messages;
			if (this._displayRegListenerWidget) {
				this._displayRegListenerWidget.destroyRecursive();
				delete this._displayRegListenerWidget;
			}
			if (this._registerListenerWizard) {
				this._registerListenerWizard.destroyRecursive();
				delete this._registerListenerWizard;
			}
			if (this._dialog) {
				this._dialog.destroyRecursive();
				delete this._dialog;
			}
			delete this._showWizard;
			delete this._dialog;
			this.inherited(arguments);
		}
		
	});
	
})();
