/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.process_events.selector.EventSourceSelectorWidget"); //$NON-NLS-1$
	
	dojo.require("dijit._Widget");  //$NON-NLS-1$
	dojo.require("dijit._Templated"); //$NON-NLS-1$
	dojo.require("dojox.xml.parser"); //$NON-NLS-1$
	dojo.require("jazz.ui.MessageBox"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.client.internal.ProcessClient"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.DeferredModel"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper"); //$NON-NLS-1$	
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.ProcessTree"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.process_events.CrossAppEventClient"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.process_events.selector.EventSourcesReadStore"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.process_events.selector", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$	
	
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var CrossAppEventClient = com.ibm.team.process.web.ui.internal.admin.process_events.CrossAppEventClient;
	var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
	var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
	var MessageBox = jazz.ui.MessageBox;
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.process_events.selector.EventSourceSelectorWidget", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
	
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/process_events/selector/templates/EventSourceSelectorWidget.html"), //$NON-NLS-1$, //$NON-NLS-2$
		widgetsInTemplate: true,
		
		constructor: function(){
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.process_events.selector", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$			
		},
		
		loadEventSources: function(eventSourcesUrl){
			var callback = dojo.hitch(this, function(eventSources){
				var hasEventSources = (eventSources && eventSources.length > 0);
				this.onFetchEventSources(hasEventSources);
                if (hasEventSources) {
                    eventSources.sort(function(eventSource1, eventSource2){
                    	return ProcessClient.stringCompare(eventSource1.name, eventSource2.name);
                    });
                	var eventSourcesTreeNodes = [];
                	dojo.forEach(eventSources, function(eventSource){
                		var eventSourceTreeNode = {};
                		eventSourceTreeNode.name = [eventSource.name];
                		eventSourceTreeNode.summary = [eventSource.summary];
                		eventSourceTreeNode.details = [eventSource.details];
                		eventSourceTreeNode.typeId = [eventSource.typeId];
                		eventSourceTreeNode.url = [eventSource.url];
                		var date = UIHelper.getUniqueTimeStamp();
                		eventSourceTreeNode.id = [eventSource.url + date.getTime()];
                		eventSourceTreeNode.eventTypes = [eventSource.eventTypes];
                		eventSourceTreeNode.type = ProcessConstants.TYPE_EVENT_SOURCE;
                		if (eventSource.children && eventSource.children.length > 0) {
                			var childEventSources = eventSource.children;
                			eventSourceTreeNode.children = [];
                			dojo.forEach(childEventSources, function(childEventSource){
                				var childEventSourceTreeNode = {};
                				childEventSourceTreeNode.reference = [childEventSource.reference];
                				childEventSourceTreeNode.name = [ProcessConstants.TYPE_EVENT_SOURCE_NOT_LOADED];
                				childEventSourceTreeNode.type = [ProcessConstants.TYPE_EVENT_SOURCE_NOT_LOADED];
                				childEventSourceTreeNode.parentEventSourceNodeId = eventSourceTreeNode.id;
                				childEventSourceTreeNode.url = [childEventSource.reference];
                				var date = UIHelper.getUniqueTimeStamp();
                				childEventSourceTreeNode.id = [childEventSource.reference + date.getTime()];
                				eventSourceTreeNode.children.push(childEventSourceTreeNode);
                			});
                		}
                		eventSourcesTreeNodes.push(eventSourceTreeNode);
                	});
					this._createEventSourceTree(eventSourcesTreeNodes);
                } else {
					dojox.xml.parser.removeChildren(this._eventSourceTreeDiv);
					var divNode = document.createElement("div"); //$NON-NLS-1$
					var textNode = document.createTextNode(this.messages.EventSourceSelectorWidget_3);
					divNode.className = "infoMsg"; //$NON-NLS-1$;
					this._eventSourceTreeDiv.appendChild(divNode);
					divNode.appendChild(textNode);
				}
			});
			var errback = dojo.hitch(this, function(error){
				dojox.xml.parser.removeChildren(this._eventSourceTreeDiv);
				this.messageBox.showMessage({
					type: MessageBox.ERROR,
					summary: this.messages.EventSourceSelectorWidget_1,
					errorObject: error
				});
			});
			CrossAppEventClient.getEventSources(eventSourcesUrl, callback, errback);
			dojox.xml.parser.removeChildren(this._eventSourceTreeDiv);
			var divNode = document.createElement("div"); //$NON-NLS-1$
			var textNode = document.createTextNode(this.messages.EventSourceSelectorWidget_2);
			divNode.className = "infoMsg"; //$NON-NLS-1$;
			this._eventSourceTreeDiv.appendChild(divNode);
			divNode.appendChild(textNode);
		},
		
		_createEventSourceTree: function(eventSources){
			dojox.xml.parser.removeChildren(this._eventSourceTreeDiv);
			this._destroyEventSourceTree();
            var getEventSourcesErrback = dojo.hitch(this, function(/* Error */error){
            	this.messageBox.showMessage({
            		type: MessageBox.ERROR,
            		summary: this.messages.EventSourceSelectorWidget_1,
            		errorObject: error
            	});
            });
			var store = new com.ibm.team.process.web.ui.internal.admin.process_events.selector.EventSourcesReadStore({
					data: {
						identifier: "id", //$NON-NLS-1$
						label: "name", //$NON-NLS-1$
						items: eventSources
					}
				},
				getEventSourcesErrback
			);
				
			var model = new com.ibm.team.process.web.ui.internal.admin.projects.DeferredModel({
				store: store,
				childrenAttrs: ["children"] //$NON-NLS-1$
			});
			var self = this;
            this._eventSourceTree = new com.ibm.team.process.web.ui.internal.admin.projects.ProcessTree({
            	persist: false,
            	model: model,
            	showRoot: false,
            	
            	focusChanged: function(node){
            		var type = this.model.store.getValue(node.item, "type"); //$NON-NLS-1$
            		if (type === ProcessConstants.TYPE_EVENT_SOURCE_NOT_LOADED) {//nested eventSource
            			self._selectedEventSource = undefined;
            		} else if (type === ProcessConstants.TYPE_EVENT_SOURCE) {
            			if (self._selectedEventSource != node.item) {
            				self._selectedEventSource = node.item;
            				self.onSelectEventSource({
            					eventTypes: this.model.store.getValue(node.item, "eventTypes"), //$NON-NLS-1$
            					url: this.model.store.getValue(node.item, "url") //$NON-NLS-1$
            				});
            			}
            		}
            	}
            });
 			this._eventSourceTreeDiv.appendChild(this._eventSourceTree.domNode);
		},
		
		onSelectEventSource: function(selectedEventSource){
		
		},
		
		onFetchEventSources: function(hasEventSources){
			
		},
		
		uninitialize: function(){
			this._destroyEventSourceTree();
			this.inherited(arguments);
		},
        
        _destroyEventSourceTree: function(){
        	if (this._onClickHandle) {
        		dojo.disconnect(this._onClickHandle);
        	}
        	if (this._eventSourceTree) {
        		this._eventSourceTree.destroyRecursive();
        		delete this._eventSourceTree;
        	}
        }
	});
})();
