/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.process_events.selector.EventSourcesReadStore"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("dojo.data.ItemFileReadStore"); //$NON-NLS-1$
	dojo.require("jazz.client.xhr"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.client.internal.ProcessClient"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.process_events.CrossAppEventClient"); //$NON-NLS-1$
	
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var CrossAppEventClient = com.ibm.team.process.web.ui.internal.admin.process_events.CrossAppEventClient;
	var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
	var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.process_events.selector.EventSourcesReadStore", dojo.data.ItemFileReadStore, { //$NON-NLS-1$
	
		constructor: function(args, getEventSourcesErrback){
			if(getEventSourcesErrback){
				this.getEventSourcesErrback = getEventSourcesErrback;
			}
		},
		
		isItemLoaded: function(item){
			var type = this.getValue(item, "type"); //$NON-NLS-1$
			if (type === ProcessConstants.TYPE_EVENT_SOURCE_NOT_LOADED) {
				return false;
			}
			return true;
		},
		
		/**
		 * Overload of ItemFileReadStore#loadItem.
		 * See Read#loadItem for details on parameter etc.
		 */
		loadItem: function(keywordArgs){
			var item = keywordArgs.item;
			this._assertIsItem(item);
			
			var parentEventSourceTreeNode;
			var parentEventSourceNodeId = this.getValue(item, "parentEventSourceNodeId"); //$NON-NLS-1$
			this.fetch({
				query: {
					id: "*" + parentEventSourceNodeId //$NON-NLS-1$
				},
				queryOptions: {
					deep: true
				},
				onItem: function(eventSource, request){
					parentEventSourceTreeNode = eventSource;
				}
			});
			var callback = dojo.hitch(this, function(eventSources){
				this._populateEventSources(eventSources, item, parentEventSourceTreeNode);
				if (keywordArgs.onItem) {
					var scope = keywordArgs.scope ? keywordArgs.scope : dojo.global;
					keywordArgs.onItem.call(scope, parentEventSourceTreeNode);
				}
			});
			var errback = dojo.hitch(this, function(error){
				if(this.getEventSourcesErrback){
					this.getEventSourcesErrback(error);
				}
			});
			CrossAppEventClient.getEventSources(this.getValue(item, "reference"), callback, errback); //$NON-NLS-1$
		},
		
		_populateEventSources: function(eventSources, stubItem, parentEventSourceTreeNode){
			if (!eventSources) {
				return;
			}
            eventSources.sort(function(eventSource1, eventSource2){
            	return ProcessClient.stringCompare(eventSource1.name, eventSource2.name);
            });		
			var firstChild = true;
			dojo.forEach(eventSources, function(eventSource){
				var eventSourceTreeNode;
				if (firstChild) {
					eventSourceTreeNode = stubItem;
				} else {
					eventSourceTreeNode = {};
				}
				eventSourceTreeNode.name = [eventSource.name];
				eventSourceTreeNode.summary = [eventSource.summary];
				eventSourceTreeNode.details = [eventSource.details];
				eventSourceTreeNode.typeId = [eventSource.typeId];
				eventSourceTreeNode.url = [eventSource.url];
				var date = UIHelper.getUniqueTimeStamp();
				eventSourceTreeNode.id = [eventSource.url + date.getTime()];
				eventSourceTreeNode.eventTypes = [eventSource.eventTypes];
				eventSourceTreeNode.type = [ProcessConstants.TYPE_EVENT_SOURCE];
				if (eventSource.children && eventSource.children.length > 0) {
					var childEventSources = eventSource.children;
					eventSourceTreeNode.children = [];
					dojo.forEach(childEventSources, function(childEventSource){
						var childEventSourceTreeNode = {};
						childEventSourceTreeNode.reference = [childEventSource.reference];
						childEventSourceTreeNode.name = [ProcessConstants.TYPE_EVENT_SOURCE_NOT_LOADED];
						childEventSourceTreeNode.type = [ProcessConstants.TYPE_EVENT_SOURCE_NOT_LOADED];
						childEventSourceTreeNode.parentEventSourceNodeId = eventSourceTreeNode.id;
						childEventSourceTreeNode.url = [childEventSource.reference];
						var date = UIHelper.getUniqueTimeStamp();
						childEventSourceTreeNode.id = [childEventSource.reference + date.getTime()];
						this._arrayOfAllItems.push(childEventSourceTreeNode);
						childEventSourceTreeNode[this._storeRefPropName] = this;
						childEventSourceTreeNode[this._itemNumPropName] = (this._arrayOfAllItems.length - 1); //Last one pushed in is the new item
						eventSourceTreeNode.children.push(childEventSourceTreeNode);
					}, this);
				}
				if (!firstChild) {
					parentEventSourceTreeNode.children.push(eventSourceTreeNode);
					this._arrayOfAllItems.push(eventSourceTreeNode);
					eventSourceTreeNode[this._storeRefPropName] = this;
					eventSourceTreeNode[this._itemNumPropName] = (this._arrayOfAllItems.length - 1); //Last one pushed in is the new item
				} else {
					firstChild = false;
				}
			}, this);
		}
	});
})();
