/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){

	dojo.provide("com.ibm.team.process.web.ui.internal.admin.process_events.selector.EventTypeSelectorWidget"); //$NON-NLS-1$
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dijit._Templated"); //$NON-NLS-1$	
	dojo.require("dijit.tree.ForestStoreModel"); //$NON-NLS-1$
	dojo.require("dojo.data.ItemFileReadStore"); //$NON-NLS-1$
	dojo.require("dojox.xml.parser"); //$NON-NLS-1$
	dojo.require("jazz.ui.MessageBox"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.client.internal.ProcessClient"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.ProcessTree"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.process_events.CrossAppEventClient"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.process_events.selector", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$			
	
	var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var CrossAppEventClient = com.ibm.team.process.web.ui.internal.admin.process_events.CrossAppEventClient;
	var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
	var MessageBox = jazz.ui.MessageBox;
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.process_events.selector.EventTypeSelectorWidget", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/process_events/selector/templates/EventTypeSelectorWidget.html"), //$NON-NLS-1$, //$NON-NLS-2$
		widgetsInTemplate: true,
		
		constructor: function(){
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.process_events.selector", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$			
		},
		
		loadEventTypes: function(eventTypesUrl){
            if (!eventTypesUrl) {
            	dojox.xml.parser.removeChildren(this._eventTypeTreeDiv);
				var divNode = document.createElement("div"); //$NON-NLS-1$
				var textNode = document.createTextNode(this.messages.EventTypeSelectorWidget_3);
				divNode.className = "infoMsg"; //$NON-NLS-1$
				this._eventTypeTreeDiv.appendChild(divNode);
				divNode.appendChild(textNode);
				return;
            }
			var callback = dojo.hitch(this, function(eventTypes){
				if (eventTypes && eventTypes.length > 0) {
                    eventTypes.sort(function(eventType1, eventType2){
                    	return ProcessClient.stringCompare(eventType1.name, eventType2.name);
                    });
					var eventTypesTreeNodes = [];
					var eventTypeCategoryTreeNodes = [];
					dojo.forEach(eventTypes, function(eventType){
						var eventTypeTreeNode = {};
						eventTypeTreeNode.name = [eventType.name];
						eventTypeTreeNode.summary = [eventType.summary];
						eventTypeTreeNode.url = [eventType.url];
						eventTypeTreeNode.type = [ProcessConstants.TYPE_EVENT_TYPE];
						if (eventType.category && eventType.category.length > 0) {
							var eventTypeCategoryTreeNode = {};
							if (eventTypeCategoryTreeNodes[eventType.category]) {
								eventTypeCategoryTreeNode = eventTypeCategoryTreeNodes[eventType.category];
							} else {
								eventTypeCategoryTreeNode.name = [eventType.category];
								var date = UIHelper.getUniqueTimeStamp();
								eventTypeCategoryTreeNode.url = [eventType.category + date.getTime()];
								eventTypeCategoryTreeNode.type = [ProcessConstants.TYPE_EVENT_TYPE_CATEGORY];
								eventTypeCategoryTreeNode.children = [];
								eventTypesTreeNodes.push(eventTypeCategoryTreeNode);
								eventTypeCategoryTreeNodes[eventType.category] = eventTypeCategoryTreeNode;
							}
							eventTypeCategoryTreeNode.children.push(eventTypeTreeNode);
						} else {
							eventTypesTreeNodes.push(eventTypeTreeNode);
						}
					});
                    eventTypesTreeNodes.sort(function(eventTypeTreeNode1, eventTypeTreeNode2){
                    	return ProcessClient.stringCompare(eventTypeTreeNode1.name[0], eventTypeTreeNode2.name[0]);
                    });
					this._createEventTypeTree(eventTypesTreeNodes);
				} else {
					dojox.xml.parser.removeChildren(this._eventTypeTreeDiv);
					var divNode = document.createElement("div"); //$NON-NLS-1$
					var textNode = document.createTextNode(this.messages.EventTypeSelectorWidget_4);
					divNode.className = "infoMsg"; //$NON-NLS-1$
					this._eventTypeTreeDiv.appendChild(divNode);
					divNode.appendChild(textNode);
				}
			});
			var errback = dojo.hitch(this, function(error){
				dojox.xml.parser.removeChildren(this._eventTypeTreeDiv);
				this.messageBox.showMessage({
					type: MessageBox.ERROR,
					summary: this.messages.EventTypeSelectorWidget_1,
					errorObject: error
				});
			});
			CrossAppEventClient.getEventTypes(eventTypesUrl, callback, errback);
			dojox.xml.parser.removeChildren(this._eventTypeTreeDiv);
			var divNode = document.createElement("div"); //$NON-NLS-1$
			var textNode = document.createTextNode(this.messages.EventTypeSelectorWidget_2);
			divNode.className = "infoMsg"; //$NON-NLS-1$
			this._eventTypeTreeDiv.appendChild(divNode);
			divNode.appendChild(textNode);
		},
		
		_createEventTypeTree: function(eventTypesTreeNodes){
			dojox.xml.parser.removeChildren(this._eventTypeTreeDiv);
			this._destroyEventTypeTree();
			var store = new dojo.data.ItemFileReadStore({
				data: {
					identifier: "url", //$NON-NLS-1$
					label: "name", //$NON-NLS-1$
					items: eventTypesTreeNodes
				}
			});
			var model = new dijit.tree.ForestStoreModel({
				store: store,
				childrenAttrs: ["children"] //$NON-NLS-1$
			});
			var self = this;
            this._eventTypeTree = new com.ibm.team.process.web.ui.internal.admin.projects.ProcessTree({
            	persist: false,
            	model: model,
            	showRoot: false,
            	
            	focusChanged: function(node){
            		var type = this.model.store.getValue(node.item, "type"); //$NON-NLS-1$
            		if (type === ProcessConstants.TYPE_EVENT_TYPE_CATEGORY) {//nested eventType
            			self._selectedEventType = undefined;
            		} else if (type === ProcessConstants.TYPE_EVENT_TYPE) {
            			if (self._selectedEventType != node.item) {
            				self._selectedEventType = node.item;
            				self.onSelectEventType({
            					url: this.model.store.getValue(node.item, "url") //$NON-NLS-1$
            				});
            			}
            		}
            	}
            });

			this._eventTypeTreeDiv.appendChild(this._eventTypeTree.domNode);
		},
		
		onSelectEventType: function(selectedEventType){
		
		},
		
        uninitialize: function(){
			this._destroyEventTypeTree();
			this.inherited(arguments);
        },
		
		_destroyEventTypeTree: function(){
        	if (this._onClickHandle) {
        		dojo.disconnect(this._onClickHandle);
        	}
		    if (this._eventTypeTree) {
        		this._eventTypeTree.destroyRecursive();
        		delete this._eventTypeTree;
        	}
		}
		
	});
})();
