/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){

	dojo.provide("com.ibm.team.process.web.ui.internal.admin.process_events.wizard.EventTypeWizardStep"); //$NON-NLS-1$
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dijit._Templated"); //$NON-NLS-1$	
	dojo.require("jazz.opensocial.Gadget");//$NON-NLS-1$
	dojo.require("jazz.ui.wizard.WizardStep"); //$NON-NLS-1$
	dojo.require("jazz.ui.wizard.WizardStepHeader"); //$NON-NLS-1$
	dojo.require("jazz.ui.MessageBox"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.process_events.CrossAppEventClient"); //$NON-NLS-1$ 
	dojo.require("com.ibm.team.process.web.ui.internal.admin.process_events.selector.EventTypeSelector"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.process_events.wizard", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var CrossAppEventClient = com.ibm.team.process.web.ui.internal.admin.process_events.CrossAppEventClient;
	var EventTypeSelector = com.ibm.team.process.web.ui.internal.admin.process_events.selector.EventTypeSelector;
	var MessageBox = jazz.ui.MessageBox;
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.process_events.wizard.EventTypeWizardStep", [jazz.ui.wizard.WizardStep, dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/process_events/wizard/templates/EventTypeWizardStep.html"), //$NON-NLS-1$, //$NON-NLS-2$
		widgetsInTemplate: true,
		_selectedRootservices: null,
		_eventTypesGadget: null,
		_disableNext: true,
		_reload: false,
		
		constructor: function(){
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.process_events.wizard", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			this.stepTitle = this.messages.EventTypeWizardStep_1;
		},
		
		postCreate: function(){
			this.header = new jazz.ui.wizard.WizardStepHeader({
				stepTitle: "" //$NON-NLS-1$
			});
			this._stepHeaderArea.appendChild(this.header.domNode);
		},
		
		setEventSrc: function(eventSrcSelected){
			// Display EventTypes from remote server or application server embedding the cross app event management page from JTS
			// In this scenario eventSrcSelected will have eventTypesGadgetSpecificationUrl attribute and not eventTypes attribute
			if (eventSrcSelected && eventSrcSelected.eventTypesGadgetSpecificationUrl && eventSrcSelected.selectedRootServices && this._eventTypesGadgetSpecificationUrl !== eventSrcSelected.eventTypesGadgetSpecificationUrl) {
				this._selectedRootservices = eventSrcSelected.selectedRootServices;
				this._eventTypesGadgetSpecificationUrl = eventSrcSelected.eventTypesGadgetSpecificationUrl;
				delete this._eventTypes;
				this._reload = true;
			} else if (eventSrcSelected && eventSrcSelected.eventTypes && eventSrcSelected.selectedRootServices && this._eventTypes !== eventSrcSelected.eventTypes) {
				// Display EventTypes from local server i.e. server hosting the EventTypes selector UI
				// In this scenario eventSrcSelected will have evenTypes attribute and not eventTypesGadgetSpecificationUrl attribute
				this._selectedRootservices = eventSrcSelected.selectedRootServices;
				this._eventTypes = eventSrcSelected.eventTypes;
				delete this._eventTypesGadgetSpecificationUrl;
				this._reload = true;
			}
		},
		
		focusStep: function(){
			if (this._reload) {
				this._reload = false;
				try {
					this._displayEventTypeSelector();
					this._onEventTypeDeSelect();
				}catch(e){
					this._reload = true;
				}								
			}
		},
		
		//  Description:
		//    This method embeds the event type selector widget
		_displayEventTypeSelector: function(){
			dojox.xml.parser.removeChildren(this._eventTypesGadgetContainerDiv);
			if (this._eventTypesGadget) {
				this._eventTypesGadget.destroyRecursive();
				delete this._eventTypesGadget;
			}
			if(this._onSelectEventTypeConnect) {
				this.disconnect(this._onSelectEventTypeConnect);
				delete this._onSelectEventTypeConnect;
			}
			// Display EventTypes from local server i.e. server hosting the EventTypes selector UI
			if (this._eventTypes) {
				var callback = dojo.hitch(this, function(eventTypeURL){
					if(eventTypeURL){
						this.onEventTypeSelect(eventTypeURL);
					}
				});
				// Directly instantiate the EventTypeSelector widget
                this._eventTypesGadget = new EventTypeSelector({
                	eventTypesURL: this._eventTypes
                });
				this._onSelectEventTypeConnect = this.connect(this._eventTypesGadget, "onSelectEventType", callback); //$NON-NLS-1$
				this._eventTypesGadgetContainerDiv.appendChild(this._eventTypesGadget.domNode);
			} else {
				// Display EventTypes from remote server or application server embedding the cross app event management page from JTS
				var dataCallback = dojo.hitch(this, function(eventTypeURL){
					if (eventTypeURL) {
						this.onEventTypeSelect(eventTypeURL);
					}
				});
				// Embed the EventTypesGadget using OpenSocial Gadgets API
				this._eventTypesGadget = new jazz.opensocial.Gadget({
					url: this._eventTypesGadgetSpecificationUrl
				});
				this._eventTypesGadgetContainerDiv.appendChild(this._eventTypesGadget.domNode);
				gadgets.rpc.register(ProcessConstants.RPCSERVICE_EVENTTYPE_SELECTION_CHANGED, dataCallback);
			}
		},
		
		onEventTypeSelect: function(eventTypeURL){
			this._disableNext = false;
			this.wizard.updateNavigationControls();
		},
		
		_onEventTypeDeSelect: function(){
			this._disableNext = true;
			this.wizard.updateNavigationControls();
		},
		
		canFlipToNextPage: function(){
			if (this._disableNext) {
				return false;
			} else {
				return true;
			}
		},
		
		uninitialize: function(){
			delete this.messages;
			delete this._selectedRootservices;
			delete this._reload;
			delete this._eventTypesGadgetSpecificationUrl;
			delete this._eventTypes;
			if (this._eventTypesGadget) {
				this._eventTypesGadget.destroyRecursive();
				delete this._eventTypesGadget;
			}
			this.inherited(arguments);
		}
		
	});
})();
