/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.process.web.ui.internal.admin.process_templates.ExportTemplateDialog"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dojox.xml.parser"); //$NON-NLS-1$
dojo.require("dijit.ProgressBar"); //$NON-NLS-1$
dojo.require("dijit.form.CheckBox"); //$NON-NLS-1$
dojo.require("jazz.ui.Dialog");//$NON-NLS-1$
dojo.require("jazz.ui.StyledTable");//$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.client.internal.userRegistry");//$NON-NLS-1$
(function(){

	var ExportTemplateDialogMessages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.process_templates", "Messages");//$NON-NLS-1$ //$NON-NLS-2$
	var DEFAULT_INTRO_TEXT = ExportTemplateDialogMessages.ExportTemplateDialog_2;
	var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
	var SERVICE_URL = TeamServerClient._getServicePrefix() + "com.ibm.team.process.internal.service.web.IProcessDefinitionFileService/";//$NON-NLS-1$
	var _currentRequest;
	var _WIDTH_IN_EM = 70;
	var processExportKey;
	var _configDataTable;
	var StyledTable = jazz.ui.StyledTable;
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.process_templates.ExportTemplateDialog", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/process_templates/templates/ExportTemplateDialog.html"),//$NON-NLS-1$ //$NON-NLS-2$ 
		constructor: function(args){
			if (!args) {
				args = {};
			}
			this.width = _WIDTH_IN_EM + "em"; //$NON-NLS-1$
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.process_templates", "Messages");//$NON-NLS-1$ //$NON-NLS-2$
		},
		
		postCreate: function(){
			var self = this;
			this.dialog = new jazz.ui.Dialog({
				contentNode: this.contentNode,
				primaryTitle: this.messages.ExportTemplateDialog_1,
				width: this.width,
				destroyContents: true,
				onClose: function(){
					if (self.processExportKey) {
						self._invokeService(self.serviceUrl + "/cancelExport?itemId=" + self.itemId + "&exportKey=" + self.processExportKey, null, null); //$NON-NLS-1$ //$NON-NLS-2$
					}
				}
			});
			
			this._topText.innerHTML = DEFAULT_INTRO_TEXT;
			this.okButton.disabled = true;
			dojo.toggleClass(this._configDataDiv, "none", true); //$NON-NLS-1$
			var fetchConfigDataCallback = function(response){
				var jsonResponse = dojo.fromJson(response);
				if (jsonResponse && jsonResponse.data && jsonResponse.data.length && jsonResponse.data.length > 0) {
					dojo.toggleClass(self._configDataDiv, "none", false); //$NON-NLS-1$
					dojo.toggleClass(self._configDataDiv, "block", true); //$NON-NLS-1$
					self.okButton.disabled = false;
					self._topText.innerHTML = self.messages.ExportTemplateDialog_3;
					self.processExportKey = jsonResponse.exportKey;
					self._clearStatusMessage();
					self._createTable(jsonResponse.data);
				} else {
					self.dialog.close();
					window.location = self.serviceUrl + "?itemId=" + self.itemId; //$NON-NLS-1$
				}
			};
			
			var fetchConfigDataErrorCallback = function(error){
				var responseText = "";//$NON-NLS-1$
				if (error.responseText) {
					responseText = error.responseText;
				} else if (error.message) {
					responseText = error.message;
				} else {
					responseText = error;
				}
				
				self._setStatusMessage(responseText, true);
				
			};
			
			this._invokeService(this.serviceUrl + "/configData?itemId=" + this.itemId + "&owningApplicationKey=" + this.applicationKey, fetchConfigDataCallback, fetchConfigDataErrorCallback); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this.cancelButton, "onclick", function(e){ //$NON-NLS-1$
				dojo.stopEvent(e);
				if (this.intervalId) {
					clearInterval(this.intervalId);
				}
				if (_currentRequest) {
					_currentRequest.cancel();
				}
				if (window.stop) {
					window.stop();
				}
				
				this.dialog.close();
			});
			this.connect(this.okButton, "onclick", "_onOk"); //$NON-NLS-1$ //$NON-NLS-2$ 
		},
		
		_createRowFromArray: function(data, isValid){
			var tr = document.createElement("tr");//$NON-NLS-1$
			for (var i = 0; i < data.length; i++) {
				var td = document.createElement("td");//$NON-NLS-1$
				if (!isValid) {
					td.className = "invalidConfigDataRow";//$NON-NLS-1$
				}
				td.appendChild(data[i]);
				tr.appendChild(td);
			}
			return tr;
		},
		
		_createRow: function(config){
			var dataType = config._configDataTypeCol;
			var dataTypeId = config._configDataTypeIdCol;
			var dataUrl = config._configDataUrlCol;
			var isValid = config._isValid;
			var data = [];
			
			// Selection
			var selectionCheckbox = document.createElement("input"); //$NON-NLS-1$
			data.push(selectionCheckbox);
			selectionCheckbox.type = "checkbox"; //$NON-NLS-1$
			selectionCheckbox.checked = isValid;
			selectionCheckbox.disabled = !isValid;
			selectionCheckbox.setAttribute("dataTypeId", dataTypeId); //$NON-NLS-1$
			
			// Configuration Data Type            
			var label = document.createElement("label"); //$NON-NLS-1$
			data.push(label);
			label.innerHTML = dataType;
			
			// Configuration Data Url            
			label = document.createElement("label"); //$NON-NLS-1$
			data.push(label);
			label.innerHTML = dataUrl;
			
			// Exportable            
			label = document.createElement("label"); //$NON-NLS-1$
			data.push(label);
			label.innerHTML = isValid;
			
			return this._createRowFromArray(data, isValid);
		},
		
		_createTable: function(jsonData){
			this._configDataTable = new StyledTable({
				columns: [{
					label: "",//$NON-NLS-1$
					width: "2%", //$NON-NLS-1$
					sortable: false
				}, {
					label: this.messages.ExportTemplateDialog_4,
					width: "25%", //$NON-NLS-1$
					sortable: false
				}, {
					label: this.messages.ExportTemplateDialog_5,
					width: "63%", //$NON-NLS-1$
					sortable: false
				}, {
					label: this.messages.ExportTemplateDialog_7,
					width: "10%", //$NON-NLS-1$
					sortable: false
				}],
				sortBy: 1,
				sortByAscending: false,
				topBorder: true,
				trim: jazz.ui.styledTable.TrimStyles.GOLD
			});
			
			this._configDataDiv.appendChild(this._configDataTable.domNode);
			dojo.toggleClass(this._configDataTable.domNode, "configDataTable", true);//$NON-NLS-1$
			var self = this;
			dojo.forEach(jsonData, function(item){
				var row = self._createRow(item);
				self._configDataTable.tbody.appendChild(row);
			});
			
			if (dojo.isIE) {
				this._initializeSelection();
			}
			
		},
		
		_initializeSelection: function() {
			var inputs = this._configDataTable.tbody.getElementsByTagName("input"); //$NON-NLS-1$
			for (var i = 0; i < inputs.length; i++) {
				inputs[i].checked = !inputs[i].disabled;
			}
		},
		
		_onOk: function(e){
			this.okButton.disabled = true;
			if (e) {
				dojo.stopEvent(e);
			}
			
			var excludedList = ""; //$NON-NLS-1$
			var inputs = this._configDataTable.tbody.getElementsByTagName("input"); //$NON-NLS-1$
			for (var i = 0; i < inputs.length; i++) {
				var checked = inputs[i].checked;
				if (!checked) {
					excludedList = excludedList + inputs[i].getAttribute("dataTypeId") + ";"; //$NON-NLS-1$ //$NON-NLS-2$	
				}
			}
			window.location = this.serviceUrl + "?itemId=" + this.itemId + "&exportKey=" + this.processExportKey + "&excluded=" + excludedList; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			this.processExportKey = null;
			this.dialog.close();
			
		},
		
		_handleErrorOnUpload: function(response, ioArgs){
			if (!response.dojoType === "cancel") { //$NON-NLS-1$
				clearInterval(this.intervalId);
				this._setStatusMessage(response, true);
			}
		},
		
		_clearStatusMessage: function(){
			dojo.toggleClass(this.status, "hidden", true);//$NON-NLS-1$
		},
		
		_setStatusMessage: function(msg, isError){
			dojo.toggleClass(this.status, "hidden", false);//$NON-NLS-1$
			dojox.xml.parser.replaceChildren(this.status, document.createTextNode(msg));
			this.status.className = isError ? "ErrorStatus" : "NormalStatus";//$NON-NLS-1$ //$NON-NLS-2$
		},
		
		_handleCheckingConfigData: function(response, ioArgs){
			// if find any configuration data in this process template
			// prompt a page to allow user to choose the configuration data included/excluded in/from the target template
			// TODO
			this._displayDataSelection();
			
		},
		
		_invokeService: function(url, successCallback, errorCallback){
			var encodedUrl = encodeURI(url);
			var args = {
				url: encodedUrl,
				headers: {
					"accept": "*" //$NON-NLS-1$ //$NON-NLS-2$
				}
			};
			var deferred = jazz.client.xhrGet(args);
			
			deferred.addCallback(successCallback);
			deferred.addErrback(errorCallback);
			
			this._currentRequest = deferred;
		}
		
	});
	
})();
