/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.process.web.ui.internal.admin.project_move.ProjectMove"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient"); //$NON-NLS-1$
dojo.require("jazz.ui.wizard.WizardStepHeader"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.client.internal.ProcessClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.client.internal.ProjectMoveClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.client.internal.ProjectMoveProgressClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.ui.internal.admin.project_move.ErrorUtility"); //$NON-NLS-1$

dojo.require("jazz.ui.ListBox"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.project_move", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() {

var getAuthenticatedContributor = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
var ProjectMoveClient = com.ibm.team.process.web.client.internal.ProjectMoveClient;
var ProjectMoveProgressClient = com.ibm.team.process.web.client.internal.ProjectMoveProgressClient;
var WizardStepHeader = jazz.ui.wizard.WizardStepHeader;
var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
var ErrorUtility = com.ibm.team.process.web.ui.internal.admin.project_move.ErrorUtility;
var STATUS_TYPES = WizardStepHeader.STATUS_TYPES;
var messages;

var ListBox = jazz.ui.ListBox;

// Project move states

var STATE_UNKNOWN = "UNKNOWN"; //$NON-NLS-1$

// Defined in projectmove.ecore
var STATE_NOT_STARTED = "NOT_STARTED"; //$NON-NLS-1$
var STATE_IN_PROGRESS = "IN_PROGRESS"; //$NON-NLS-1$
var STATE_COMPLETED = "COMPLETED"; //$NON-NLS-1$
var STATE_CANCELLED = "CANCELLED"; //$NON-NLS-1$
var STATE_SUSPENDED = "SUSPENDED"; //$NON-NLS-1$
var STATE_FAILED = "FAILED"; //$NON-NLS-1$

// Project phases

var PHASE_UNKNOWN = "UNKNOWN"; //$NON-NLS-1$

// Defined in projectmove.ecore
var PHASE_INIITALIZE_MOVE_IN_SOURCE = "INIITALIZE_MOVE_IN_SOURCE"; //$NON-NLS-1$
var PHASE_INIITALIZE_MOVE_IN_TARGET = "INIITALIZE_MOVE_IN_TARGET"; //$NON-NLS-1$
var PHASE_ANALYZE_ITEMS = "ANALYZE_ITEMS"; //$NON-NLS-1$
var PHASE_COPYING_ITEMS = "COPYING_ITEMS"; //$NON-NLS-1$
var PHASE_COPYING_DATA = "COPYING_DATA"; //$NON-NLS-1$
var PHASE_POST_PROCESSING_TARGET_ITEMS = "POST_PROCESSING_TARGET_ITEMS"; //$NON-NLS-1$
var PHASE_POST_PROCESSING_SOURCE_ITEMS = "POST_PROCESSING_SOURCE_ITEMS"; //$NON-NLS-1$
var PHASE_DELETE_ITEMS = "DELETE_ITEMS"; //$NON-NLS-1$
var PHASE_DELETE_DATA = "DELETE_DATA"; //$NON-NLS-1$
var PHASE_FINALIZE_MOVE_IN_SOURCE = "FINALIZE_MOVE_IN_SOURCE"; //$NON-NLS-1$
var PHASE_FINALIZE_MOVE_IN_TARGET = "FINALIZE_MOVE_IN_TARGET"; //$NON-NLS-1$
var PHASE_REMOTE_LOGIN = "REMOTE_LOGIN"; //$NON-NLS-1$

// Project move operations

var OPERATION_MOVE = 1;
var OPERATION_ANALYZE = 2;


var REFRESH_INTERVAL = 3000; // 3 seconds

dojo.declare("com.ibm.team.process.web.ui.internal.admin.project_move.ProjectMove", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
	
	templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/project_move/templates/ProjectMove.html"), //$NON-NLS-1$ //$NON-NLS-2$
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.project_move", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
		
		this._autoRefreshImgSrc = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/project_move/images/refresh-status.gif").toString(); //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	postCreate: function() {
		
		this._messageHeader = new WizardStepHeader({ 
			stepTitle: this.messages.ProjectMove_1,
			reset: dojo.hitch(this,"refresh") //$NON-NLS-1$
		});
		this.headerArea.appendChild(this._messageHeader.domNode);
		
		var authContrib = getAuthenticatedContributor();
		this._loggedInUser = null;
		if (authContrib) {
			this._loggedInUser = authContrib.userId;
		}

		this._refreshTimeOutID = null;
		this._projectAreaListBox = null;
		this._operationInProgress = false;
		
		// Initiliaze
		
		this._init();
		
	},
	
	_init: function() {
		this._connectEventHandlers();
		this._initStateMap();
		this._initPhaseMap();
		this._initProjectAreaListBox();
		this._initTargetRepositoriesSelector();		
		this._initStatusTable(true);
	},		

	_connectEventHandlers: function() {
		// Button
		this.connect(this._moveButton, "onclick", this._handleEventOnClickMoveButton); //$NON-NLS-1$
		
		// Status table row
		this.connect(this._statusTableRow, "onclick", this._handleEventOnClickTableRow); //$NON-NLS-1$
		
	},	

	_handleEventOnClickTableRow: function(event) {

		if (event) {
			dojo.stopEvent(event);
		}

		var statusItemId = this._hiddenStatusItemId.value;

		if (!statusItemId) {
			ErrorUtility.alertError(new Error(this.messages.ProjectMove_ErrorNoStatusItemID, this.id));
			return false;
		}

		location.href = "#action=com.ibm.team.process.projectMoveResults&itemId=" + statusItemId; //$NON-NLS-1$
		
		return true;
		
	},	

	_showAutoRefreshImage: function(on) {
		if (on) {
			this._statusAutoRefreshImage.style.display = ""; //$NON-NLS-1$
		} else {
			this._statusAutoRefreshImage.style.display = "none"; //$NON-NLS-1$
		}
	},		
	
	_initStateMap: function() {
		this._stateMap = {};
		
		this._stateMap[STATE_UNKNOWN] = {
			label: messages.ProjectMove_Unknown,
			image: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/project_move/images/unknown.gif").toString() //$NON-NLS-1$ //$NON-NLS-2$
		};		
		this._stateMap[STATE_NOT_STARTED] = {
			label: messages.ProjectMove_Pending,
			image: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/project_move/images/notstarted.gif").toString() //$NON-NLS-1$ //$NON-NLS-2$
		};		
		this._stateMap[STATE_IN_PROGRESS] = {
			label: messages.ProjectMove_InProgress,
			image: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/project_move/images/inprogress.gif").toString() //$NON-NLS-1$ //$NON-NLS-2$
		};		
		this._stateMap[STATE_COMPLETED] = {
			label: messages.ProjectMove_Completed,
			image: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/project_move/images/complete.gif").toString() //$NON-NLS-1$ //$NON-NLS-2$
		};		
		this._stateMap[STATE_CANCELLED] = {
			label: messages.ProjectMove_Cancelled,
			image: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/project_move/images/cancelled.gif").toString() //$NON-NLS-1$ //$NON-NLS-2$
		};		
		this._stateMap[STATE_SUSPENDED] = {
			label: messages.ProjectMove_Suspended,
			image: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/project_move/images/suspended.gif").toString() //$NON-NLS-1$ //$NON-NLS-2$
		};		
		this._stateMap[STATE_FAILED] = {
			label: messages.ProjectMove_Failed,
			image: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/project_move/images/failed.gif").toString() //$NON-NLS-1$ //$NON-NLS-2$
		};		
		
	},		

	_initPhaseMap: function() {
		this._phaseMap = {};

		this._phaseMap[PHASE_UNKNOWN] = {
			label: messages.ProjectMove_PhaseUnknown
		};		
		this._phaseMap[PHASE_INIITALIZE_MOVE_IN_SOURCE] = {
			label: messages.ProjectMove_PhaseInitMoveSource
		};		
		this._phaseMap[PHASE_INIITALIZE_MOVE_IN_TARGET] = {
			label: messages.ProjectMove_PhaseInitMoveTarget
		};		
		this._phaseMap[PHASE_ANALYZE_ITEMS] = {
			label: messages.ProjectMove_PhaseAnalyzeItems
		};		
		this._phaseMap[PHASE_COPYING_ITEMS] = {
			label: messages.ProjectMove_PhaseCopyItems
		};		
		this._phaseMap[PHASE_COPYING_DATA] = {
			label: messages.ProjectMove_PhaseCopyData
		};		
		this._phaseMap[PHASE_POST_PROCESSING_TARGET_ITEMS] = {
			label: messages.ProjectMove_PhasePostProcessTargetItems
		};		
		this._phaseMap[PHASE_POST_PROCESSING_SOURCE_ITEMS] = {
			label: messages.ProjectMove_PhasePostProcessSourceItems
		};		
		this._phaseMap[PHASE_DELETE_ITEMS] = {
			label: messages.ProjectMove_PhaseDeleteItems
		};		
		this._phaseMap[PHASE_DELETE_DATA] = {
			label: messages.ProjectMove_PhaseDeleteData
		};		
		this._phaseMap[PHASE_FINALIZE_MOVE_IN_SOURCE] = {
			label: messages.ProjectMove_PhaseFinalizeMoveSource
		};		
		this._phaseMap[PHASE_FINALIZE_MOVE_IN_TARGET] = {
			label: messages.ProjectMove_PhaseFinalizeMoveTarget
		};		
		this._phaseMap[PHASE_REMOTE_LOGIN] = {
			label: messages.ProjectMove_PhaseRemoteLogin
		};		
		
	},		
	
	_initProjectAreaListBox: function() {
		
		this._projectAreaListBox = new ListBox({
							margin: "0px", //$NON-NLS-1$
							width: "240px", //$NON-NLS-1$
							height: "120px", //$NON-NLS-1$
							allowMultipleSelections: true,
							iconMap: [
								dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/images/project.gif").toString() //$NON-NLS-1$  //$NON-NLS-2$
							]
						});
		this._projectAreaListBoxNode.appendChild(this._projectAreaListBox.domNode);

		this._projectAreaListBox.showLoading(true);
		
		this._refreshProjectAreaListBox();

	},	

	_refreshProjectAreaListBox: function() {
		
		if (!this._projectAreaListBox) {
			return;
		}
		
		var deferred = new dojo.Deferred();
		deferred.addCallback(dojo.hitch(this, function(projectAreas) {
			this._processProjectAreas(projectAreas);
			return projectAreas; 
		}));
		deferred.addErrback(dojo.hitch(this, function(errorObject) {
			this._handleServiceError(errorObject, messages.ProjectMove_ErrorRefreshListBox);
			return errorObject; 
		}));		
		
		this._callRESTGetProjectAreas(deferred, this._loggedInUser);
		
	},	
	
	_callRESTGetProjectAreas: function(deferred, userId) {
		
		var serviceObject = {
			success: dojo.hitch(this, function(dto) {
				deferred.callback(dto);
			}),
			failure: dojo.hitch(this, function(errorObject) {
				deferred.errback(errorObject);
			})
		};

		var handler = new ServiceResponseHandler(serviceObject, "success", "failure"); //$NON-NLS-1$ //$NON-NLS-2$
		
		ProcessClient.getAllProjectAreas(handler, userId);

	},
	
	_processProjectAreas: function(projectAreas) {
		
		if (!(projectAreas instanceof Array) ||
			(projectAreas.length === 0)) {

			this._messageHeader.showMessage({ 
				type: STATUS_TYPES.ERROR,
				summary: messages.ProjectMove_ErrorNoProjectAreasFound
			});			

			this._projectAreaListBox.removeAllEntries();
			this._projectAreaListBox.showLoading(false);

			return;
		}
		
		var entries = [];
		
		for (var index = 0; index < projectAreas.length; index++) {
			var projectArea = projectAreas[index];
			var projectAreaItemId = projectArea.itemId;
			var projectAreaName = projectArea.name;
			if (!projectAreaItemId) {
				// Error
				continue;
			} else if (!projectAreaName) {
				// Error
				continue;
			}
			entries.push({
				label: projectAreaName,
				value: projectAreaItemId,
				id: projectAreaItemId,
				type: 0
			});
		}

		this._projectAreaListBox.removeAllEntries();

		if (entries.length === 0) {
			this._messageHeader.showMessage({ 
				type: STATUS_TYPES.ERROR,
				summary: messages.ProjectMove_ErrorNoProjectAreasFound
			});			

			this._projectAreaListBox.showLoading(false);

			return;
		}
		
		this._projectAreaListBox.addEntries(entries);
		this._projectAreaListBox.showLoading(false);
		
	},	
	
	_getSelectedProjectAreas: function() {
		
		var entries = this._projectAreaListBox.getSelection();
		
		if (entries.length === 0) {
			this._messageHeader.showMessage({ 
				type: STATUS_TYPES.ERROR,
				summary: messages.ProjectMove_NoProjectArea
			});			
			return null;
		}
		
		var itemIds = [];
		
		for (var index = 0; index < entries.length; index++) {
			var entry = entries[index];
			itemIds.push(entry.value);
		}
		
		return itemIds.join(","); //$NON-NLS-1$
		
	},	
	
	_initStatusTable: function(poll) {
		this._refreshStatusTable(poll);
	},		
	
	_refreshStatusTable: function(poll) {
		
		this._showAutoRefreshImage(true);
		
		var deferred = new dojo.Deferred();
		deferred.addCallback(dojo.hitch(this, function(dto) {
			this._processProjectMoveStatus(dto);
			return dto; 
		}));
		if (poll) {
			deferred.addCallback(dojo.hitch(this, function(dto) {
				this._checkProjectMoveStatus(dto);
				return dto; 
			}));
		}
		
		deferred.addErrback(dojo.hitch(this, function(errorObject) {
			this._handleServiceError(errorObject, messages.ProjectMove_ErrorRefreshStatusTable);
			return errorObject; 
		}));		
		
		this._callRESTGetProjectMoveStatus(deferred);
		
		return deferred;
	},		

	_callRESTGetProjectMoveStatus: function(deferred) {
		
		var serviceObject = {
			success: dojo.hitch(this, function(dto) {
				deferred.callback(dto);
			}),
			failure: dojo.hitch(this, function(errorObject) {
				deferred.errback(errorObject);
			})
		};

		var handler = new ServiceResponseHandler(serviceObject, "success", "failure"); //$NON-NLS-1$ //$NON-NLS-2$
		
		ProjectMoveProgressClient.getProjectMoveStatus(handler);

	},

	_callRESTGetIsProjectMoveInProgress: function(deferred) {
		
		var serviceObject = {
			success: dojo.hitch(this, function(dto) {
				deferred.callback(dto);
			}),
			failure: dojo.hitch(this, function(errorObject) {
				deferred.errback(errorObject);
			})
		};

		var handler = new ServiceResponseHandler(serviceObject, "success", "failure"); //$NON-NLS-1$ //$NON-NLS-2$
		
		ProjectMoveProgressClient.getIsProjectMoveInProgress(handler);

	},

	_processProjectMoveStatus: function(dto) {
		
		if (!dto) {
			// Error
			return;
		}
		
		// State
		
		var state = dto.state;
		var stateLabel;
		var image;
		if (!state || 
			((state in this._stateMap) === false)) {
			
			ErrorUtility.logWarning(new Error("Unknown state encountered: " + state), this.id); //$NON-NLS-1$
			if (!state) {
				stateLabel = this._stateMap[STATE_UNKNOWN].label;
			} else {
				stateLabel = state;
			}
			image = this._stateMap[STATE_UNKNOWN].image;
		} else {
			stateLabel = this._stateMap[state].label;
			image = this._stateMap[state].image;
		}
		this._statusImage.src = image;
		this._statusImage.alt = stateLabel;
		this._statusState.innerHTML = stateLabel;

		// Phase
		
		var phase = dto.phase;
		var phaseLabel;
		if (!phase || 
			((phase in this._phaseMap) === false)) {
			
			ErrorUtility.logWarning(new Error("Unknown phase encountered: " + phase), this.id); //$NON-NLS-1$
			if (!phase) {
				phaseLabel = this._phaseMap[PHASE_UNKNOWN].label;
			} else {
				phaseLabel = phase;
			}
		} else {
			phaseLabel = this._phaseMap[phase].label;
		}
		this._statusPhase.innerHTML = phaseLabel;

		// Target repository

		var targetRepository = dto.targetRepository;
		if (!targetRepository) {
			targetRepository = ""; //$NON-NLS-1$
		}
		this._statusTargetRepository.innerHTML = targetRepository;
		
		// Status Item ID
		
		var statusItemId = dto.statusItemId;
		if (!statusItemId) {
			ErrorUtility.logError(new Error("Undefined status item ID encountered"), this.id); //$NON-NLS-1$
			this._hiddenStatusItemId.value = ""; //$NON-NLS-1$
		} else {
			this._hiddenStatusItemId.value = statusItemId;
		}
		
	},	
	
	_finalizeProjectMoveStatus: function(dto) {
		
		this._showAutoRefreshImage(false);
		
		if (!dto) {
			// Error
			return;
		}
		
		var state = dto.state;
		
		if (this._operationInProgress) {
			if (state === STATE_COMPLETED) {
				// Update status message
				this._messageHeader.showMessage({ 
					type: STATUS_TYPES.OK,
					summary: messages.ProjectMove_StatusMoveOperationComplete
				});	
				// TODO: Base message on operation type
				// "Project move analysis complete";
            	// "Project move operation complete";
			
				// Update project areas
				this._refreshProjectAreaListBox();
			} else if (state === STATE_FAILED) {
				// Update status message
				this._messageHeader.showMessage({ 
					type: STATUS_TYPES.ERROR,
					summary: messages.ProjectMove_StatusMoveOperationFailed
				});	
			} else if (state === STATE_CANCELLED) {
				this._messageHeader.hideMessage();
			} else if (state === STATE_SUSPENDED) {
				this._messageHeader.hideMessage();
			}
		}
		
	},		
	
	_checkProjectMoveStatus: function(dto) {
		
		if (!dto) {
			// Error
			return;
		}
		
		var deferred = new dojo.Deferred();
		deferred.addCallback(dojo.hitch(this, function(status) {
			this._scheduleStatusRefresh(status);
			return status; 
		}));
		deferred.addErrback(dojo.hitch(this, function(errorObject) {
			this._handleServiceError(errorObject, messages.ProjectMove_ErrorProjectMoveStatusCheck);
			return errorObject; 
		}));		
		
		this._callRESTGetIsProjectMoveInProgress(deferred);
		
	},			

	_scheduleStatusRefresh: function(status) {
		
		if (status === "false") { //$NON-NLS-1$
			// Refresh table one last time with the latest status
			var deferred = this._refreshStatusTable(false);
			deferred.addCallback(dojo.hitch(this, function(dto) {
				this._finalizeProjectMoveStatus(dto);
				this._operationInProgress = false;
				return dto; 
			}));			
			
			return;
		}
		
		this._operationInProgress = true;
		
		// TODO: Generalize project move status message to apply to both move
		// and analysis operations
		this._messageHeader.showMessage({ 
			type: STATUS_TYPES.INFO,
			summary: messages.ProjectMove_StatusMoveOperationInProgress
		});	
					
		if (this._refreshTimeOutID) {
			clearTimeout(this._refreshTimeOutID);
		}
		
		this._refreshTimeOutID = setTimeout(dojo.hitch(this, function() {
			this._refreshStatusTable(true);
		}), REFRESH_INTERVAL);
		
	},		

	_resetStatusTable: function() {
		
		// State
		
		var stateLabel = this._stateMap[STATE_NOT_STARTED].label;
		this._statusImage.src = this._stateMap[STATE_NOT_STARTED].image;
		this._statusImage.alt = stateLabel;
		this._statusState.innerHTML = stateLabel;

		// Phase
		
		this._statusPhase.innerHTML = ""; //$NON-NLS-1$

		// Target repository

		this._statusTargetRepository.innerHTML = ""; //$NON-NLS-1$
		
		// Status Item ID
		
		this._hiddenStatusItemId.value = ""; //$NON-NLS-1$
		
	},		
	
	refresh: function() {
		
		this._messageHeader.hideMessage();
		
		this._refreshStatusTable(true);
		this._refreshProjectAreaListBox();
		this._refreshTargetRepositoriesSelector();

	},
	
	_analyzeMove: function(checkedProjects, targetRepository) {	
	
		var deferred = new dojo.Deferred();
		deferred.addCallback(dojo.hitch(this, function(itemId) {
			this._analyzeRequestSent();
			return itemId; 
		}));
		deferred.addErrback(dojo.hitch(this, function(errorObject) {
			this._handleServiceError(errorObject, messages.ProjectMove_ErrorProjectMoveAnalysis);
			return errorObject; 
		}));		
		
		var params = {
			projectAreas: checkedProjects,
			targetRepository: targetRepository
		};		
		
		this._callRESTPostAnalyzeProjectsToMove(deferred, params);		
		
	},
	
	_callRESTPostAnalyzeProjectsToMove: function(deferred, params) {
		
		var serviceObject = {
			success: dojo.hitch(this, function(itemId) {
				deferred.callback(itemId);
			}),
			failure: dojo.hitch(this, function(errorObject) {
				deferred.errback(errorObject);
			})
		};

		var handler = new ServiceResponseHandler(serviceObject, "success", "failure"); //$NON-NLS-1$ //$NON-NLS-2$

		ProjectMoveClient.postAnalyzeProjectsToMove(handler, params);

	},	
	
	_analyzeRequestSent: function() {		
		this._refreshStatusTable(true);
	},
	
	_handleServiceError: function(errorObj, summary) {
		
		this._showAutoRefreshImage(false);
		
		ErrorUtility.logError(errorObj, this.id);
		
		var params = {
			type: STATUS_TYPES.ERROR, 
			errorObject: errorObj
		};
		
		if (summary) {
			params.summary = summary;
		}
		
		this._messageHeader.showMessage(params);
	},
	
	_getSelectedServer: function() {
		if (this._serverSelector.options.length === 0) {
			this._messageHeader.showMessage({ 
				type: STATUS_TYPES.ERROR,
				summary: messages.ProjectMove_NoTargetRepository
			});
			return false;
		}
		return this._serverSelector.options[this._serverSelector.selectedIndex].resource || ""; //$NON-NLS-1$
	},
	
	_handleEventOnClickMoveButton: function(event) {

		if (event) {
			dojo.stopEvent(event);
		}

		// Validate project areas
		var checkedProjects = this._getSelectedProjectAreas();
		if (!checkedProjects) {
			return false;
		}
		
		// Validate target repository
		var targetRepository = this._getSelectedServer();
		if (!targetRepository) {
			this._messageHeader.showMessage({ 
				type: STATUS_TYPES.ERROR,
				summary: messages.ProjectMove_NoTargetRepository
			});
			return false;
		}

		targetRepository = targetRepository.replace("/rootservices", ""); //$NON-NLS-1$  //$NON-NLS-2$

		// Reset status
		this._messageHeader.hideMessage();
		this._messageHeader.showMessage({ 
			type: STATUS_TYPES.INFO,
			summary: messages.ProjectMove_2
		});	
		this._resetStatusTable();

		// Run operation
		if (this._analyzeCheckbox.checked) {
			// Run analysis
			this._analyzeMove(checkedProjects, targetRepository);
		} else {
			// Run analysis + project move
			this._moveProjects(checkedProjects, targetRepository);
		}
		
		return true;
		
	},	
	
	_moveProjects: function(checkedProjects, targetRepository) {

		var deferred = new dojo.Deferred();
		deferred.addCallback(dojo.hitch(this, function(itemId) {
			this._moveRequestSent();
			return itemId; 
		}));
		deferred.addErrback(dojo.hitch(this, function(errorObject) {
			this._handleServiceError(errorObject, messages.ProjectMove_ErrorProjectMoveOperation);
			return errorObject; 
		}));		
		
		var params = {
			projectAreas: checkedProjects,
			targetRepository: targetRepository
		};	
		
		this._callRESTPostMoveProjects(deferred, params);		
		
	},
	
	_moveRequestSent: function() {
		this._refreshStatusTable(true);
	},
	
	_callRESTPostMoveProjects: function(deferred, params) {
		
		var serviceObject = {
			success: dojo.hitch(this, function(dto) {
				deferred.callback(dto);
			}),
			failure: dojo.hitch(this, function(errorObject) {
				deferred.errback(errorObject);
			})
		};

		var handler = new ServiceResponseHandler(serviceObject, "success", "failure"); //$NON-NLS-1$ //$NON-NLS-2$

		ProjectMoveClient.postMoveProjects(handler, params);

	},		
	
	_initTargetRepositoriesSelector: function() {
		this._refreshTargetRepositoriesSelector();
	},	

	_refreshTargetRepositoriesSelector: function() {
		this._getServers();
	},	
	
	_getServers: function(){
		var index = window.location.hash.toLowerCase().indexOf("testdiscovery=true"); //$NON-NLS-1$
		var rootServicesXml;
		if (index > -1) {
			rootServicesXml = "processtest/rootservices.xml"; //$NON-NLS-1$
		} else {
			rootServicesXml = ProcessConstants.ROOTSERVICESXML;
		}
		var deferred = jazz.client.xhrGet({
			url: rootServicesXml,
			headers: {
				"accept": ProcessConstants.CONTENT_TYPE_APPLICATION_XML //$NON-NLS-1$
			},
			handleAs: "xml", //$NON-NLS-1$
			load: dojo.hitch(this, this._readServers),
			error: dojo.hitch(this, function(errorObject) {
				this._handleServiceError(errorObject, messages.ProjectMove_7);
				return errorObject;
			})
		});
	},
		
	_readServers: function(rootServicesXml){
		if (!rootServicesXml) {
			return;
		}
		var children = rootServicesXml.childNodes;
		var descriptionNode = UIHelper.getNode(children, "Description", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
		var descriptionNodeChildren = descriptionNode.childNodes;

		var friendsNode = UIHelper.getNode(descriptionNodeChildren, "friends", ProcessConstants.NAMESPACE_JD); //$NON-NLS-1$ 
		var friendsResourceNode = UIHelper.getAttributeNode(friendsNode, "resource", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
		var friendsResourceURL = friendsResourceNode.value;
		var deferred = jazz.client.xhrGet({
			url: friendsResourceURL,
			headers: {
				"accept": ProcessConstants.CONTENT_TYPE_APPLICATION_XML //$NON-NLS-1$
			},
			handleAs: "xml", //$NON-NLS-1$
			load: dojo.hitch(this, this._populateServers),
			error: dojo.hitch(this, function(errorObject) {
				this._handleServiceError(errorObject, messages.ProjectMove_6);
				return errorObject;
			})
		});
	},
		
	_populateServers: function(friendsListingXml){
		if (!friendsListingXml) {
			return friendsListingXml;
		}
			
		var children = friendsListingXml.childNodes;
		var rdfNode = UIHelper.getNode(children, "RDF", ProcessConstants.NAMESPACE_RDF);  //$NON-NLS-1$
		var allFriends = UIHelper.getNodes(rdfNode.childNodes, "Friend", ProcessConstants.NAMESPACE_JD); //$NON-NLS-1$
		if (allFriends.length > 0) {
			
			this._disableActions(false);
			
			var servers = [];
			dojo.forEach(allFriends, function(friend){
				var friendChildren = friend.childNodes;
				var titleNode = UIHelper.getNode(friendChildren, "title", ProcessConstants.NAMESPACE_DC);  //$NON-NLS-1$
				var friendTitle = this._getServerTitle(titleNode.firstChild.data);
				var rootServicesNode = UIHelper.getNode(friendChildren, "rootServices", ProcessConstants.NAMESPACE_JD);  //$NON-NLS-1$
				var rootServicesResourceNode = UIHelper.getAttributeNode(rootServicesNode, "resource", ProcessConstants.NAMESPACE_RDF);  //$NON-NLS-1$
				var anOption = new Option(friendTitle, friendTitle);
				anOption.resource = rootServicesResourceNode.value;
				servers.push(anOption);
			}, this);
			servers.sort(function(option1, option2){
				return com.ibm.team.process.web.client.internal.ProcessClient.stringCompare(option1.value, option2.value);
			});
			var options = this._serverSelector.options;
			
			// Remove old entries if any
			if (options.length > 0) {
				for (var index1 = 0; index1 < options.length; index1++) {
					options[index1] = null;   
				}
			}
			
			// Add new entries
			for (var index2 = 0; index2 < servers.length; index2++) {
				var serverOption = servers[index2];
				options[index2] = serverOption;   
			}
			
			// Select the first server
			this._serverSelector.selectedIndex = 0;
		} else {
			this._handleNoServersDefined();
		}
		return friendsListingXml;
	},
		
	_handleNoServersDefined: function() {
		
		this._removeAllChildren(this._serverSelector);
		
		var optionName = messages.ProjectMove_NoServersFound;
		
		var optionElement = document.createElement("option"); //$NON-NLS-1$
		optionElement.value = optionName;
		optionElement.selected = "selected"; //$NON-NLS-1$
			
		var nameText = document.createTextNode(optionName);
		optionElement.appendChild(nameText);
			
		this._serverSelector.appendChild(optionElement);
		
		this._messageHeader.showMessage({ 
			type: STATUS_TYPES.INFO,
			summary: messages.ProjectMove_9
		});

		this._disableActions(true);

	},

	_disableActions: function(disable) {
		this._moveButton.disabled = disable;
	},

	_removeAllChildren: function(node) {
		// Remove all child nodes if any are defined
		while (node.childNodes.length > 0) {
			node.removeChild(node.firstChild);
		}
	},	

	_getServerTitle: function(title) {
		if (!title) {
			return messages.ProjectMove_Unknown;
		}
		// BEFORE:
		// Server%20Local%202
		// AFTER:
		// Server Local 2
		return decodeURIComponent(title);
	},
		
	destroy: function() {
		
		if (this._projectAreaListBox) {
			this._projectAreaListBox.destroy();
			this._projectAreaListBox = null;
		}
		
		if (this._refreshTimeOutID) {
			clearTimeout(this._refreshTimeOutID);
			this._refreshTimeOutID = null;
		}
		
		this.inherited(arguments);
	}		
	
});	

})();

