/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.process.web.ui.internal.admin.project_move.ProjectMoveResults"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient"); //$NON-NLS-1$
dojo.require("jazz.ui.wizard.WizardStepHeader"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.client.internal.ProcessClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.client.internal.ProjectMoveClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.client.internal.ProjectMoveProgressClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.ui.internal.admin.project_move.ErrorUtility"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.project_move", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() {

var getAuthenticatedContributor = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
var ProjectMoveClient = com.ibm.team.process.web.client.internal.ProjectMoveClient;
var ProjectMoveProgressClient = com.ibm.team.process.web.client.internal.ProjectMoveProgressClient;
var WizardStepHeader = jazz.ui.wizard.WizardStepHeader;
var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
var ErrorUtility = com.ibm.team.process.web.ui.internal.admin.project_move.ErrorUtility;
var STATUS_TYPES = WizardStepHeader.STATUS_TYPES;
var messages;


dojo.declare("com.ibm.team.process.web.ui.internal.admin.project_move.ProjectMoveResults", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
	
	templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/project_move/templates/ProjectMoveResults.html"), //$NON-NLS-1$ //$NON-NLS-2$
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.project_move", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	postCreate: function() {
		
		// Initialize
		
		if (this._isValidParameters()) {
			this._init();
		} else {
			// TODO: Externalize
			dojo.publish(net.jazz.ajax.internal.events.viewErrorPage, ["Missing parameter: itemId", "", true]); //$NON-NLS-1$ //$NON-NLS-2$
		}
		
	},
	
	_isValidParameters: function() {
		// Item ID
		if (this.itemId) {
			return true;
		}
		return false;
	},		
	
	_init: function() {
		this._initMessageHeader();
		this._initReportItemsTable();
	},		
	
	_initMessageHeader: function() {
		this._messageHeader = new WizardStepHeader({ 
			stepTitle: this.messages.ProjectMoveResults_AnalysisReport,
			reset: dojo.hitch(this, "refresh") //$NON-NLS-1$
		});
		this._headerArea.appendChild(this._messageHeader.domNode);
	},	
	
	_initReportItemsTable: function() {
		// TEST
		this._getReportItems();
	},		
	
	_getReportItems: function() {
		
		var deferred = new dojo.Deferred();
		deferred.addCallback(dojo.hitch(this, function(dto) {
			this._processReportItems(dto);
			return dto; 
		}));
		deferred.addErrback(dojo.hitch(this, function(errorObject) {
			// TODO: Externalize
			this._handleServiceError(errorObject, "Failed to retrieve report items"); //$NON-NLS-1$
			return errorObject; 
		}));		
		
		var params = {
			id: this.itemId,
			// TEST
			component: "com.ibm.team.repository",  //$NON-NLS-1$
			firstReport: 0,
			reportCount: 20
		};			
		
		this._callRESTGetReportItems(deferred, params);
		
		return deferred;
	},		

	_callRESTGetReportItems: function(deferred, params) {
		
		var serviceObject = {
			success: dojo.hitch(this, function(dto) {
				deferred.callback(dto);
			}),
			failure: dojo.hitch(this, function(errorObject) {
				deferred.errback(errorObject);
			})
		};

		var handler = new ServiceResponseHandler(serviceObject, "success", "failure"); //$NON-NLS-1$ //$NON-NLS-2$
		
		ProjectMoveProgressClient.getReportItems(handler, params);

	},	

	_processReportItems: function(dto) {
		// TEST
		console.debug(dto);
	},

	_handleServiceError: function(errorObj, summary) {
		
		ErrorUtility.logError(errorObj, this.id);
		
		var params = {
			type: STATUS_TYPES.ERROR, 
			errorObject: errorObj
		};
		
		if (summary) {
			params.summary = summary;
		}
		
		this._messageHeader.showMessage(params);
	},
	
	refresh: function() {
		
		this._messageHeader.hideMessage();
		
		// TODO
		alert("Refresh"); //$NON-NLS-1$
		
	},
		
	destroy: function() {
		
		
		this.inherited(arguments);
	}		
	
});	

})();

