/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.AddProjectLinkDialog"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dojox.xml.parser"); //$NON-NLS-1$
dojo.require("jazz.client.xhr"); //$NON-NLS-1$
dojo.require("jazz.app.proxy"); //$NON-NLS-1$
dojo.require("jazz.util.error"); //$NON-NLS-1$
dojo.require("dojo.DeferredList"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.client.internal.ProcessClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper");//$NON-NLS-1$
dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.DeferredLoadReadStore");//$NON-NLS-1$
dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.DeferredModel");//$NON-NLS-1$
dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.ProcessTree");//$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");//$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
var AddProjectLinkDialog = com.ibm.team.process.web.ui.internal.admin.projects.AddProjectLinkDialog;
var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;

var CSSClass = {
	SERVICE_PROVIDER: 'serviceProviderIcon', //$NON-NLS-1$
	SERVICE_PROVIDER_CATALOG: 'serviceProviderCatalogIcon', //$NON-NLS-1$
	NO_SERVICE_PROVIDERS: 'noServiceProvidersIcon' //$NON-NLS-1$
};

(function(){

	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.AddProjectLinkDialog", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/AddProjectLinkDialog.html"), //$NON-NLS-1$ //$NON-NLS-2$
		widgetsInTemplate: true,
		
		constructor: function(arg){
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			this.projectAreaItemId = arg.projectAreaItemId; //item id of the project area adding the link to
			this.projectAreaName = arg.projectAreaName; //the name of the project area adding the link to
			this.originalProjectAreaName = arg.originalProjectAreaName; //the name of the project area before any edits
			this.firstServerChange = true;
			this._owningApplicationKey = arg.owningApplicationKey;
		},
		
		postCreate: function(){
			this.originalHeaderHTML = this.dialogHeading.innerHTML;
			
			this._setDialogEnabled(false, this.messages.AddProjectLinkDialog_Loading);
			
			// This is the root call to initiate project linking.
			// After checking if OSLC core is enabled, we evaluate the link types.
			this._getIsOSLCCoreEnabled();
			
			this.previewText.appendChild(document.createTextNode(this.projectAreaName));
			
			this.connect(this.serverSelector, "onchange", "_updateForServerChange"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this.serviceSelector, "onchange", "_updateForServiceChange"); //$NON-NLS-1$ //$NON-NLS-2$
			this.finishButton.disabled = true;
		},
			
		_setDialogEnabled: function(enable, message){
			this.serverSelector.disabled = !enable;
			this.serviceSelector.disabled = !enable;
			this.serviceProvidersTree.disabled = !enable;
			this.dialogHeading.innerHTML = message;
		},
		
		startup: function(){
			com.ibm.team.process.web.ui.internal.admin.projects.UIHelper.resizeI18NButtons(this.dialogFooter, [this.finishButton, this.cancelButton]);
			this.inherited(arguments);
		},
		
		_createServiceProvidersTree: function(serviceProviders){
			dojox.xml.parser.removeChildren(this.serviceProvidersTree);
			if (this._serviceProvidersTree) {
				this._serviceProvidersTree.destroyRecursive();
				delete this._serviceProvidersTree;
			}
			var self = this;
			var store = new com.ibm.team.process.web.ui.internal.admin.projects.DeferredLoadReadStore({
				data: {
					identifier: "id", //$NON-NLS-1$
					label: "name", //$NON-NLS-1$
					items: serviceProviders
				},
				oslcCoreEnabled: this._oslcCoreEnabled
			});
			var model = new com.ibm.team.process.web.ui.internal.admin.projects.DeferredModel({
				store: store
			});
			
			this._serviceProvidersTree = new com.ibm.team.process.web.ui.internal.admin.projects.ProcessTree({
				persist: false,
				model: model,
				showRoot: false,
				
				getIconClass: function(item){
					if (item && !item.root) {
						var type = this.model.store.getValue(item, "type"); //$NON-NLS-1$
						if (type === ProcessConstants.SERVICE_PROVIDER_TYPE) {
							return CSSClass.SERVICE_PROVIDER;
						} else if (type === ProcessConstants.SERVICE_PROVIDER_CATALOG_TYPE) {
							return CSSClass.SERVICE_PROVIDER_CATALOG;
						} else if (type === ProcessConstants.SERVICE_PROVIDER_NOT_LOADED_TYPE || type === ProcessConstants.SERVICE_PROVIDER_ERROR_TYPE) {
							return CSSClass.NO_SERVICE_PROVIDERS;
						}
					}
				},
				
				focusChanged: function(node){
					var type = this.model.store.getValue(node.item, "type"); //$NON-NLS-1$
					if (type === ProcessConstants.SERVICE_PROVIDER_CATALOG_TYPE) {//nested catalog
						self._selectedServiceProvider = undefined;
						self.finishButton.disabled = true;
					} else if (type === ProcessConstants.SERVICE_PROVIDER_ERROR_TYPE || type === ProcessConstants.SERVICE_PROVIDER_NOT_LOADED_TYPE) { //$NON-NLS-1$
						self._selectedServiceProvider = undefined;
						self.finishButton.disabled = true;
					} else {
						self._selectedServiceProvider = node.item;
						self.finishButton.disabled = false;
					}
					
					self._updateLinkPreview();
				}
			});
			// Upon creating a tree from a store in dojo, it seems to pre-select the top node in the tree.
			// Remove the pre-selection.
			if (this._serviceProvidersTree.lastFocused) {
				this._serviceProvidersTree.lastFocused.setSelected(false);
			}
			this.serviceProvidersTree.appendChild(this._serviceProvidersTree.domNode);
		},
		
		_getServerURL: function(){
			if (this.serverSelector.selectedIndex === -1 || this.serverSelector.options.length === 0) {
				return ProcessConstants.EMPTY;
			}
			return this.serverSelector.options[this.serverSelector.selectedIndex].resource;
		},
		
		_getServiceProviders: function(){
			delete this._selectedServiceProvider;
			dojox.xml.parser.removeChildren(this.serviceProvidersTree);
			var serverURL = this._getServerURL();
			if (serverURL.length === 0) {
				this.finishButton.disabled = true;
				return;
			}
			
			var deferred = jazz.client.xhrGet({
				url: serverURL,
				headers: {
					"accept": ProcessConstants.CONTENT_TYPE_APPLICATION_XML //$NON-NLS-1$
				},
				handleAs: "xml", //$NON-NLS-1$
				load: dojo.hitch(this, this._readServiceProvidersResponse),
				error: dojo.hitch(this, function(error){
					this._handleErrorGettingRemote(deferred, error, this.messages.AddProjectLinkDialog_FailedServiceProviderCatalogResource);
					return error;
				})
			});
		},

		_collectServiceProviderCatalogs: function(descriptionNodeChildren){
			// Summary:
			//	Collects the catalogs with valid associations.
			
			// list of associations which are possible with the selected application.
			this._validAssociations = [];
			var serviceProviderCatalogs = [];
			var catalogs = this._getAllUniqueServiceProviderCatalogs(descriptionNodeChildren);
			for (var i = 0; i < this.linkTypes.length; i++) {
				var linkType = this.linkTypes[i];
				var tagName = UIHelper.getTagNameForLinkType(linkType);
				var namespace = UIHelper.getNamespaceForLinkType(linkType);
				var coreNamespace = UIHelper.getTargetOSLCCoreNamespaceForLinkType(linkType);
				dojo.some(catalogs, dojo.hitch(this, function(catalog){
					if(catalog.tagName === tagName && (catalog.namespace === coreNamespace || catalog.namespace === namespace)) {
						this._validAssociations.push(linkType);
						serviceProviderCatalogs.push(catalog);
					}
					return (catalog.tagName === tagName && (catalog.namespace === coreNamespace || catalog.namespace === namespace));
				}));
			}
			return serviceProviderCatalogs;
		},
		
		_readServiceProvidersResponse: function(serviceProviderXml){
			var children = serviceProviderXml.childNodes;
			var descriptionNode = UIHelper.getNode(children, "Description", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
			var uniqueCatalogs = [];
			this.serviceProviders = [];			
			var serviceProviderCatalogs = this._collectServiceProviderCatalogs(descriptionNode.childNodes);
			// return if there are no catalogs with possible associations.
			if (serviceProviderCatalogs.length === 0) {
				this._handleServerChange();
				return;
			}

			this._readServiceProvidersCallback = dojo.hitch (this, function (/*Array*/serviceProviders, /*Object*/catalog){
				// summary: Callback to be invoked after getting the service providers
				// from the given catalog.
				if (serviceProviders.length > 0) {
					if (this.serviceProviders.length === 0) {
						this.serviceProviders = serviceProviders;
					} else {
						// Filter out the duplicate service providers.
						// A service provider is duplicate when it has same type, parent catalog name and url.
						dojo.forEach(serviceProviders, function(serviceProvider){
							var duplicate = false;
							for (var i = 0; i < this.serviceProviders.length; i++) {
								var alreadyIncluded = this.serviceProviders[i];
								if (alreadyIncluded.type[0] === serviceProvider.type[0] && alreadyIncluded.serviceProviderCatalogTagName[0] ===  serviceProvider.serviceProviderCatalogTagName[0]) {
									if (alreadyIncluded.type[0] === ProcessConstants.SERVICE_PROVIDER_CATALOG_TYPE && alreadyIncluded.serviceProviderCatalogURL[0] === serviceProvider.serviceProviderCatalogURL[0]) {
										duplicate = true;
										break;
									}
									if (alreadyIncluded.type[0] === ProcessConstants.SERVICE_PROVIDER_TYPE && alreadyIncluded.presentationUrl[0] === serviceProvider.presentationUrl[0]) {
										duplicate = true;
										break;
									}
								}
							}
							if (!duplicate) {
								this.serviceProviders.push(serviceProvider);
							}
						}, this);
					}
				}				
			});
			
			this._populateReadServiceProvidersResponse = dojo.hitch(this, function(/*Array of service providers*/options, /*Catalog Object*/catalog){
				//Summary: Populates the service providers in UI after all
				// the catalogs have been parsed.
				//
				// It is connected to the callback method to keep tab of the 
				// catalogs which have been parsed.
				// This method makes sure that we wait till we get the service 
				// providers from all the catalogs. Some catalogs can have 
				// service providers with external references, we need to wait
				// to get information from such service providers as well.

				// Mark the catalogs as parsed in the list.
				var foundCatalog = dojo.some(uniqueCatalogs, function(item){
					if (item.tagName === catalog.tagName) {
						item.parsed = true;
					}
					return item.tagName === catalog.tagName;
				}, this);
								
				// Check if all the catalogs have been parsed.
				var haveAllCatalogsParsed = dojo.every(uniqueCatalogs, function(item){
					return item.parsed === true;
				}, this);
				
				// build the UI
				if (haveAllCatalogsParsed) {
					if (this.serviceProviders.length === 0) {
						var serviceProviders = [];
						var serviceProviderTreeNode = {};
						serviceProviderTreeNode.name = [this.messages.AddProjectLinkDialog_NoServiceProviders];
						serviceProviderTreeNode.id = [this.messages.AddProjectLinkDialog_NoServiceProviders + new Date().getTime()];
						serviceProviderTreeNode.type = [ProcessConstants.SERVICE_PROVIDER_ERROR_TYPE];
						serviceProviders.push(serviceProviderTreeNode);
						this._populateServiceSelector(this._validAssociations);
						this._createServiceProvidersTree(serviceProviders);
						return;
					} else {
						this.serviceProviders.sort(function(provider1, provider2){
							return ProcessClient.stringCompare(provider1.name[0], provider2.name[0]);
						});
						this._handleServerChange();
					}
				}
			});
			this.connect(this, "_readServiceProvidersCallback", "_populateReadServiceProvidersResponse"); //$NON-NLS-1$ //$NON-NLS-2$
			var unique = {};
			dojo.every(serviceProviderCatalogs, function(serviceProviderCatalog){
				var catalog = {};
				catalog.tagName = serviceProviderCatalog.tagName;
				// Filter out the duplicate service provider catalogs.
				if(!unique[catalog.tagName]) {
					unique[catalog.tagName] = true;
					uniqueCatalogs.push(catalog);
				} else {
					return true;
				}
				
				var matchingServiceProviderCatalogResource = serviceProviderCatalog.resourceNode;
				var header = {"accept": ProcessConstants.CONTENT_TYPE_APPLICATION_XML};  //$NON-NLS-1$
				if (this._oslcCoreEnabled) {
					header[ProcessConstants.OSLC_VERSION_HEADER] = ProcessConstants.OSLC_CORE_VERSION;
				}
				var deferred = jazz.client.xhrGet({
					url: matchingServiceProviderCatalogResource,
					headers: header,
					handleAs: "xml", //$NON-NLS-1$
					load: dojo.hitch(this, function(serviceProviderCatalogXML, ioArgs){
						var oslcResponseVersion = ioArgs.xhr.getResponseHeader(ProcessConstants.OSLC_VERSION_HEADER);
						this.dialogHeading.innerHTML = this.originalHeaderHTML;
						this._determineServiceProviders(serviceProviderCatalogXML, oslcResponseVersion, this._readServiceProvidersCallback, catalog);
					}),
					error: dojo.hitch(this, function(error){
						this._handleErrorGettingRemote(deferred, error, this.messages.AddProjectLinkDialog_FailedMatchingServiceProviderCatalogResource);
						return error;
					})
				});
				return true;
			}, this);
		},
		
		_evalProjectLinks: function(){
			// Summary: Populate Add Link dialog with available service providers
			//			based on source project area's link types.
			
			// 			Start with finding source project area's link types.
			//			Find available service providers only if source link types
			//			are available.
			var index = window.location.hash.toLowerCase().indexOf("testdiscovery=true"); //$NON-NLS-1$
			var rootServicesXml;
			if (index > -1) {
				rootServicesXml = "processtest/rootservices.xml"; //$NON-NLS-1$
			} else {
				rootServicesXml = ProcessConstants.ROOTSERVICESXML;
			}
			var deferred = jazz.client.xhrGet({
				url: rootServicesXml,
				headers: {
					"accept": ProcessConstants.CONTENT_TYPE_APPLICATION_XML //$NON-NLS-1$
				},
				handleAs: "xml", //$NON-NLS-1$
				load: dojo.hitch(this, this._determineLinkTypes),
				error: dojo.hitch(this, function(error){
					this._handleErrorGettingRemote(deferred, error, this.messages.AddProjectLinkDialog_FailedRootServicesResource);
					return error;
				})
			});
		},
		
		_reportErrorFetchingRemote: function(errorMessage, contextHeaderMessage){
			var iconPath = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/images/warning.gif"); //$NON-NLS-1$ //$NON-NLS-2$ 
			var imgHtml = "<img src=\"" + iconPath + "\"\/>"; //$NON-NLS-1$ //$NON-NLS-2$
			var brHtml = "<br />"; //$NON-NLS-1$
			if (!contextHeaderMessage) {
				contextHeaderMessage = this.messages.AddProjectLinkDialog_ErrorFetchingRemote;
			}
			
			this.dialogHeading.innerHTML = imgHtml + " " + contextHeaderMessage + brHtml + brHtml + errorMessage; //$NON-NLS-1$
			this.finishButton.disabled = true;
		},
		
		_readServers: function(foundationRootServicesNodes){
			var friendsNode = UIHelper.getNode(foundationRootServicesNodes, "friends", ProcessConstants.NAMESPACE_JD); //$NON-NLS-1$ 
			var friendsResourceNode = UIHelper.getAttributeNode(friendsNode, "resource", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
			var friendsResourceURL = friendsResourceNode.value;
			var deferred = jazz.client.xhrGet({
				url: friendsResourceURL,
				headers: {
					"accept": ProcessConstants.CONTENT_TYPE_APPLICATION_XML //$NON-NLS-1$
				},
				handleAs: "xml", //$NON-NLS-1$
				load: dojo.hitch(this, this._populateServers),
				error: dojo.hitch(this, function(error){
					this._handleErrorGettingRemote(deferred, error, this.messages.AddProjectLinkDialog_FailedFriendsResource);
					return error;
				})
			});
		},
		
		_determineServiceCatalogs: function(rootServicesNodes){
			// Summary: From root services find the available registered catalogs.
			
			var tagNames = [ProcessConstants.ServiceProviderTypes.CM, ProcessConstants.ServiceProviderTypes.RM, ProcessConstants.ServiceProviderTypes.QM, ProcessConstants.ServiceProviderTypes.SCM, ProcessConstants.ServiceProviderTypes.AM];
			var namesSpaces = [ProcessConstants.NAMESPACE_OSLC_CM, ProcessConstants.NAMESPACE_OSLC_RM, ProcessConstants.NAMESPACE_OSLC_QM, ProcessConstants.NAMESPACE_OSLC_SCM, ProcessConstants.NAMESPACE_OSLC_AM];
			var catalogs = [];
			for (var i = 0; i < tagNames.length; i++) {
				var tagName = tagNames[i];
				var namespace = namesSpaces[i];
				var serviceProviderCatalog = UIHelper.getNode(rootServicesNodes, tagName, namespace);
				if (serviceProviderCatalog) {
					var catalog = {};
					var catalogResourceNode = UIHelper.getAttributeNode(serviceProviderCatalog, "resource", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
					catalog.resourceNode = catalogResourceNode.value;
					catalog.tagName = tagName;
					catalog.namespace = namespace;
					catalogs.push(catalog);
				}
			}
			return catalogs;
		},
		
		_determineServiceCatalogsOSLCCoreStyle: function(rootServicesNodes) {
			// Summary:
			// Gets the service provider catalogs from provided root services document.
			// Only catalogs published in V2 syntax are parsed here.
			var catalogs = [];
			var oslcCatalogNodes = UIHelper.getNodes(rootServicesNodes, ProcessConstants.ELEM_OSLC_CATALOGS, ProcessConstants.NAMESPACE_JD);
			dojo.forEach(oslcCatalogNodes, function(oslcCatalogNode){
				var spCatalogNodes = UIHelper.getNodes(oslcCatalogNode.childNodes, ProcessConstants.ELEM_SERVICE_PROVIDER_CATALOG, ProcessConstants.NAMESPACE_OSLC_CORE);
				dojo.forEach(spCatalogNodes, function(spCatalogNode){
					var catalogRef = UIHelper.getAttributeNodeValue(spCatalogNode, ProcessConstants.XMLATTRIBUTE_ABOUT, ProcessConstants.NAMESPACE_RDF);
					var catalogDomainNode = UIHelper.getNode(spCatalogNode.childNodes, ProcessConstants.ELEM_DOMAIN, ProcessConstants.NAMESPACE_OSLC_CORE);
					var catalogDomainRef = UIHelper.getAttributeNodeValue(catalogDomainNode, ProcessConstants.XMLATTRIBUTE_RESOURCE, ProcessConstants.NAMESPACE_RDF);
					if (catalogRef && catalogDomainRef){
						var catalog = {};
						catalog.resourceNode = catalogRef;
						catalog.tagName = UIHelper.getTagNameForNamespace(catalogDomainRef);
						catalog.namespace = catalogDomainRef;
						catalogs.push(catalog);
					}
				});
			});
			return catalogs;
		},
		
		_evalLinkTypes: function(catalogs, foundationRootServicesNodes){
			//Summary: Evaluate link types of current project area from the available catalogs
			
			// No need to proceed if there are no registered catalogs. 
			if (catalogs.length === 0) {
				this._handleProjectLinkProblem(this.messages.AddProjectLinkDialog_ServerCatalogNotFound);
				return;
			}
			
			this.linkTypes = [];
			this._sourceProjectAreaInfo = {};
			if (this.originalProjectAreaName) {
				this._getDetermineLinkTypesDeferred(catalogs, foundationRootServicesNodes);
			} else { //new project area (not saved yet.).
				// In this case, there won't be project area in catalog.
				// Pick up link types from all available catalogs.
				if (catalogs.length > 0) {
					while (catalogs.length > 0) {
						var aCatalog = catalogs.pop();
						var catalogLinkTypes = UIHelper.getLinkTypesForTagName(aCatalog.tagName);
						for (var k = 0; k < catalogLinkTypes.length; k++) {
							this.linkTypes.push(catalogLinkTypes[k]);
						}
					}
					// Find remote servers only if there are link types.
					if (this.linkTypes.length !== 0) {
						// There won't be any associations unless an application is selected.
						this._populateServiceSelector(this._validAssociations);
						this._setDialogEnabled(true, this.originalHeaderHTML);
						this._readServers(foundationRootServicesNodes);
					} else {
						// No need to proceed if there are no link types.
						this._handleProjectLinkProblem(this.messages.AddProjectLinkDialog_ServerProviderNotFound);
					}
				}
			}
		},
		
		_determineLinkTypes: function(rootServicesXml){
			if (!rootServicesXml) {
				return;
			}
			var children = rootServicesXml.childNodes;
			var descriptionNode = UIHelper.getNode(children, "Description", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
			var foundationRootServicesNodes = descriptionNode.childNodes;
			
			if (this._owningApplicationKey === ProcessConstants.JTS_OWNING_APPLICATION_KEY) {
				var catalogs = this._getAllUniqueServiceProviderCatalogs(foundationRootServicesNodes);
				this._evalLinkTypes(catalogs, foundationRootServicesNodes);
			} else {
				this._determineLinkTypesFromFrontingServer(foundationRootServicesNodes);
			}
		},
		
		_determineLinkTypesFromFrontingServer: function(foundationServicesRootElem){
			// Summary: Finds link types of the current project area in it's owning application.
			var handleFrontingAppsObject = {
				success: dojo.hitch(this, function(response){
					var friend = dojo.fromJson(response);
					// Check if the response is not empty.
					if (dojo.exists("id", friend)) { //$NON-NLS-1$
						var deferred = jazz.client.xhrGet({
							url: friend.rootServices,
							headers: {
								"accept": ProcessConstants.CONTENT_TYPE_APPLICATION_XML //$NON-NLS-1$
							},
							handleAs: "xml", //$NON-NLS-1$
							load: dojo.hitch(this, function(rootServicesXml){
								if (!rootServicesXml) {
									return;
								}
								var children = rootServicesXml.childNodes;
								var descriptionNode = UIHelper.getNode(children, "Description", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
								var friendRootServicesNodes = descriptionNode.childNodes;
								var catalogs = this._getAllUniqueServiceProviderCatalogs(friendRootServicesNodes);
								this._evalLinkTypes(catalogs, foundationServicesRootElem);
							}),
							error: dojo.hitch(this, function(error){
								this._handleErrorGettingRemote(deferred, error, this.messages.AddProjectLinkDialog_FailedRootServicesResource);
							})
						});
					}
				}),
				failure: dojo.hitch(this, function(error){
					this._handleErrorGettingRemote(deferred, error, this.messages.AddProjectLinkDialog_FailedFriendsResource);
				})
			};
			
			var srh = new ServiceResponseHandler(handleFrontingAppsObject, "success", "failure"); //$NON-NLS-1$ //$NON-NLS-2$
			var args = {
				owningApplicationKey: this._owningApplicationKey
			};
			ProcessClient.getApplication(args, srh);
		},
		
		_getDetermineLinkTypesDeferred: function(catalogs, foundationRootServicesNodes){
			// Summary: Walk through the available catalogs and find the matching project area
			// 			From this match you have the namespace and tag name.
			// 			We know all the of the possible namespaces and tagnames 
			//			so look for them out of the rootservices.
			//TODO: this will change when link types are extensible.
			if (catalogs.length === 0) {
				return;
			}

			this._evalSourceLinkTypes = dojo.hitch(this, function(/*Array of service providers*/options, /*Object - parsed catalog*/catalog){
				// summary: Callback to be invoked after getting the service providers
				// from the given catalog.
				// This is to determine the link types of the source service provider.
				for (var j = 0; j < options.length; j++) {
					var serviceProviderCatalogOption = options[j];
					if (serviceProviderCatalogOption.type[0] === ProcessConstants.SERVICE_PROVIDER_CATALOG_TYPE) {
						// walk through the catalog recursively
						catalog.deferredCount++;
						var httpHeader = {"accept": ProcessConstants.CONTENT_TYPE_APPLICATION_XML};  //$NON-NLS-1$
						if (this._oslcCoreEnabled) {
							httpHeader[ProcessConstants.OSLC_VERSION_HEADER] = ProcessConstants.OSLC_CORE_VERSION;
						}
						var evalSourceLinkTypesDeferred = jazz.client.xhrGet({
							url: serviceProviderCatalogOption.serviceProviderCatalogURL[0],
							headers: httpHeader,
							handleAs: "xml", //$NON-NLS-1$
							load: dojo.hitch(this, function(serviceProviderCatalogXML, ioArgs){
								var oslcResponseVersion = ioArgs.xhr.getResponseHeader(ProcessConstants.OSLC_VERSION_HEADER);
								this._determineServiceProviders(serviceProviderCatalogXML, oslcResponseVersion, this._evalSourceLinkTypes, catalog);
							}),
							error: dojo.hitch(this, function(error){
								this._handleErrorGettingRemote(evalSourceLinkTypesDeferred, error, this.messages.AddProjectLinkDialog_FailedMatchingServiceProviderCatalogResource);
								return error;
							})
						});
					} else if (serviceProviderCatalogOption.type[0] === ProcessConstants.SERVICE_PROVIDER_TYPE){
						if (this.originalProjectAreaName === serviceProviderCatalogOption.name[0]) {
							// Hold current project's service provider details.
							// It will be used for creating back link during save operation.
							
							// There can be multiple entries of current project area
							// in different catalogs. Hold all of them and pick the
							// right one at the time of saving based on the link type chosen.
							if (!this._sourceProjectAreaInfo[catalog.tagName]) {
								this._sourceProjectAreaInfo[catalog.tagName] = serviceProviderCatalogOption;
							}
							//add to the linkTypes collection
							var catalogLinkTypes = UIHelper.getLinkTypesForTagName(catalog.tagName);
							for (var k = 0; k < catalogLinkTypes.length; k++) {
								this.linkTypes.push(catalogLinkTypes[k]);
							}
						}
					}
				}
			});
			
			this._getServersForAssociations = dojo.hitch(this, function(options, catalog){
				//Summary: Gets the servers for associations after all
				// the catalogs have been parsed and link types of current
				// project area has been determined.
				//
				// It is connected to the callback method for getting source link
				// types in order to keep tab of the catalogs which have been parsed.
				// This method makes sure that we wait till we get the service 
				// providers from all the catalogs. Some catalogs can have 
				// service providers with external references, we need to wait
				// to get information from such service providers.
				
				// Mark the catalog as parsed in the list.	
				var foundCatalog = dojo.some(catalogs, function(item){
					if (item.tagName === catalog.tagName) {
						if (catalog.deferredCount) {
							if (catalog.deferredCount === 0) {
								item.linkTypesParsed = true;
							} else {
								catalog.deferredCount--;
							}
						} else {
							item.linkTypesParsed = true;
						}
					}
					return item.tagName === catalog.tagName;
				}, this);
				
				// Check if all the catalogs have been parsed.
				var haveAllCatalogsParsed = dojo.every(catalogs, function(item){
					return item.linkTypesParsed === true;
				}, this);
				
				//Proceed only if all the catalogs have been parsed.
				if (haveAllCatalogsParsed) {
					// Find remote servers only if there are link types.
					if (this.linkTypes.length !== 0) {
						// There won't be any associations unless an application is selected.
						this._populateServiceSelector(this._validAssociations);
						this._setDialogEnabled(true, this.originalHeaderHTML);
						this._readServers(foundationRootServicesNodes);
					} else {
						this._handleProjectLinkProblem(this.messages.AddProjectLinkDialog_ServerProviderNotFound);
					}
				}
			});
			
			this.connect(this, "_evalSourceLinkTypes", "_getServersForAssociations"); //$NON-NLS-1$ //$NON-NLS-2$

			dojo.forEach(catalogs, function(catalog){
				var header = {"accept": ProcessConstants.CONTENT_TYPE_APPLICATION_XML}; //$NON-NLS-1$
				if (this._oslcCoreEnabled) {
					header[ProcessConstants.OSLC_VERSION_HEADER] = ProcessConstants.OSLC_CORE_VERSION;
				}
				catalog.deferredCount = 0;				
				var deferred = jazz.client.xhrGet({
					url: catalog.resourceNode,
					headers: header,
					handleAs: "xml", //$NON-NLS-1$
					load: dojo.hitch(this, function(serviceProviderCatalogXML, ioArgs){
						var oslcResponseVersion = ioArgs.xhr.getResponseHeader(ProcessConstants.OSLC_VERSION_HEADER);
						this._determineServiceProviders(serviceProviderCatalogXML, oslcResponseVersion, this._evalSourceLinkTypes, catalog);
					}),
					error: dojo.hitch(this, function(error){
						this._handleErrorGettingRemote(deferred, error, this.messages.AddProjectLinkDialog_FailedServiceProviderCatalog);
						return error;
					})
				});
			}, this);
		},
		
		_populateServiceSelector: function(linkTypes){
			this.serviceSelector.options.length = 0;
			var services = this._createServiceOptions(linkTypes);
			var optionArray = this.serviceSelector.options;
			dojo.forEach(services, function(serviceOption){
				//appendChild(anOption) does not work on IE7, so modify array directly
				optionArray[optionArray.length] = serviceOption;
			});
		},
		
		_createServiceOptions: function(linkTypes){
			var services = [];
			// We won't have valid link types unless an application is selected.
			if (!linkTypes || linkTypes.length === 0) {
				var serviceOption = new Option(this.messages.AddProjectLinkDialog_None, this.messages.AddProjectLinkDialog_None);
				services.push(serviceOption);
			} else {
				dojo.forEach(linkTypes, function(linkType){
					var label;
					if (UIHelper.isUsesLinkType(linkType)) {
						label = dojo.string.substitute(this.messages.AddProjectLinkDialog_ServiceProviderOptionName, [this.messages.AddProjectLinkDialog_UsesLinkTypeName, UIHelper.getLinkTypeLabel(linkType, this.messages)]);
					} else {
						label = dojo.string.substitute(this.messages.AddProjectLinkDialog_ServiceProviderOptionName, [this.messages.AddProjectLinkDialog_ProvidesLinkTypeName, UIHelper.getLinkTypeLabel(linkType, this.messages)]);
					}
					var serviceOption = new Option(label, label);
					serviceOption.linkType = linkType;
					services.push(serviceOption);
				}, this);
			}
			services.sort(function(option1, option2){
				return ProcessClient.stringCompare(option1.value, option2.value);
			});
			return services;
		},
		
		_populateServers: function(friendsListingXml){
			if (!friendsListingXml) {
				return friendsListingXml;
			}
			
			var children = friendsListingXml.childNodes;
			var rdfNode = UIHelper.getNode(children, "RDF", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
			var allFriends = UIHelper.getNodes(rdfNode.childNodes, "Friend", ProcessConstants.NAMESPACE_JD); //$NON-NLS-1$
			var index = window.location.hash.toLowerCase().indexOf("testdiscovery=true"); //$NON-NLS-1$
			var myRootServicesURL;
			if (index > -1) {
				myRootServicesURL = document.location.protocol + "//" + document.location.host + net.jazz.ajax._contextRoot + "/processtest/rootservices.xml"; //$NON-NLS-1$ //$NON-NLS-2$
			} else {
				myRootServicesURL = document.location.protocol + "//" + document.location.host + ProcessConstants.ROOTSERVICESXML; //$NON-NLS-1$
			}
			var servers = [];
			if (allFriends.length > 0) {
				//Ensure we have ourself as a friend
				var friendOfMyself = false;
				dojo.forEach(allFriends, function(friend){
					var friendChildren = friend.childNodes;
					var titleNode = UIHelper.getNode(friendChildren, "title", ProcessConstants.NAMESPACE_DC); //$NON-NLS-1$
					var friendTitle = titleNode.firstChild.data;
					friendTitle = decodeURIComponent(friendTitle);
					var rootServicesNode = UIHelper.getNode(friendChildren, "rootServices", ProcessConstants.NAMESPACE_JD); //$NON-NLS-1$
					var rootServicesResourceNode = UIHelper.getAttributeNode(rootServicesNode, "resource", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
					var hostname = this._getHostname(rootServicesResourceNode.value);
					var applicationName = dojo.string.substitute(this.messages.AddProjectLinkDialog_ApplicationName, [friendTitle, hostname]);
					var anOption = new Option(applicationName, applicationName);
					anOption.resource = rootServicesResourceNode.value;
					if (rootServicesResourceNode.value === myRootServicesURL) {
						friendOfMyself = true;
					}
					servers.push(anOption);
				}, this);
				if (!friendOfMyself && this._owningApplicationKey === ProcessConstants.JTS_OWNING_APPLICATION_KEY) {
					var hostname = this._getHostname(myRootServicesURL);
					var applicationName = dojo.string.substitute(this.messages.AddProjectLinkDialog_ApplicationName, [this.messages.AddProjectLinkDialog_LocalApplicationName, hostname]);
					var anOption = new Option(applicationName, applicationName);
					anOption.resource = myRootServicesURL;
					servers.push(anOption);
				}
				
				servers.sort(function(option1, option2){
					return ProcessClient.stringCompare(option1.value, option2.value);
				});
				var selectOption = new Option(this.messages.AddProjectLinkDialog_SelectServer, this.messages.AddProjectLinkDialog_SelectServer);
				selectOption.resource = ProcessConstants.EMPTY;
				servers.unshift(selectOption);
				
			} else if (this._owningApplicationKey === ProcessConstants.JTS_OWNING_APPLICATION_KEY) { 
			    //Always be friends of yourself
				hostname = this._getHostname(myRootServicesURL);
				applicationName = dojo.string.substitute(this.messages.AddProjectLinkDialog_ApplicationName, [this.messages.AddProjectLinkDialog_LocalApplicationName, hostname]);
				anOption = new Option(applicationName, applicationName);
				anOption.resource = myRootServicesURL;
				servers.push(anOption);
				selectOption = new Option(this.messages.AddProjectLinkDialog_SelectServer, this.messages.AddProjectLinkDialog_SelectServer);
				selectOption.resource = ProcessConstants.EMPTY;
				servers.unshift(selectOption);
			}
			var optionArr = this.serverSelector.options;
			dojo.forEach(servers, function(serverOption){
				//appendChild(anOption) does not work on IE7, so modify array directly
				optionArr[optionArr.length] = serverOption;
			}, this);
			return friendsListingXml;
		},
		
		_handleProjectLinkProblem: function(errMsg){
			dojo.toggleClass(this._dialogBody, "hidden", true); //$NON-NLS-1$
			dojo.toggleClass(this.dialogHeading, "hidden", true); //$NON-NLS-1$
			dojo.toggleClass(this.finishButton, "hidden", true); //$NON-NLS-1$
			if (errMsg) {
				this._dialogHeadingProblem.innerHTML = errMsg;
			}
			dojo.toggleClass(this._dialogHeadingProblem, "hidden", false); //$NON-NLS-1$
		},
		
		_determineServiceProvidersOSLCCore: function(/* DOM Object*/ serviceProviderCatalogXml, /* Array */ providers, /* Array */providerCatalogs, /* Array */providerRefs, /* Array */ providerCatalogRefs ){
			// Summary: Determines the service providers and service provider
			// catalogs from the provided catalog xml based on oslc core specs.
			//
			// The evaluated service providers and catalogs are pushed in the
			// provided 'providers', 'providerCatalogs', 'providerRefs' 
			// and 'providerCatalogRefs' arrays.
			// providers - array of service providers with inline information
			// providerCatalogs - array of nested catalogs with inline information
			// providerRefs - array of service provider with external references
			// providerCatalogRefs - array of nested catalogs with external references.
			
			var root = serviceProviderCatalogXml.documentElement;
			var children = root.childNodes;
			var serviceProviderCatalog = UIHelper.getNode(children, "ServiceProviderCatalog", ProcessConstants.NAMESPACE_OSLC_CORE); //$NON-NLS-1$
			children = serviceProviderCatalog.childNodes; //children of ServiceProviderCatalog
			var oslcServiceProviders = UIHelper.getNodes(children, "serviceProvider", ProcessConstants.NAMESPACE_OSLC_CORE); //$NON-NLS-1$
			dojo.forEach(oslcServiceProviders, function(oslcServiceProvider){
				var serviceProviderRef = UIHelper.getAttributeNodeValue(oslcServiceProvider, "resource", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
				if (serviceProviderRef) {
					providerRefs.push(serviceProviderRef);
				} else {
					var serviceProvider = UIHelper.getNode(oslcServiceProvider.childNodes, "ServiceProvider", ProcessConstants.NAMESPACE_OSLC_CORE); //$NON-NLS-1$
					if (serviceProvider) {
						providers.push(serviceProvider);
					}
				}
			});
			var oslcServiceProviderCats = UIHelper.getNodes(children, "serviceProviderCatalog", ProcessConstants.NAMESPACE_OSLC_CORE); //$NON-NLS-1$
			dojo.forEach(oslcServiceProviderCats, function(oslcServiceProviderCat){
				var serviceProviderCatalogRef = UIHelper.getAttributeNodeValue(oslcServiceProviderCat, "resource", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
				if (serviceProviderCatalogRef) {
					providerCatalogRefs.push(serviceProviderCatalogRef);
				} else {
					var serviceProviderCatalog = UIHelper.getNode(oslcServiceProviderCat.childNodes, "ServiceProviderCatalog", ProcessConstants.NAMESPACE_OSLC_CORE); //$NON-NLS-1$
					if (serviceProviderCatalog) {
						providerCatalogs.push(serviceProviderCatalog);
					}
				}
			});
		},
		
		_determineServiceProviders: function(/*DOM Node*/serviceProviderCatalogXml, /*String*/oslcSpecVersion, /*function*/callback, /*object*/catalog){
			//Summary: Determines the service providers of a given catalog.
			var serviceProviders = [];
			var providers = [];
			var providerCatalogs = [];
			var providerRefs = [];
			var providerCatalogRefs = [];
			if (this._oslcCoreEnabled && oslcSpecVersion && oslcSpecVersion === ProcessConstants.OSLC_CORE_VERSION) {
				this._determineServiceProvidersOSLCCore(serviceProviderCatalogXml, providers, providerCatalogs, providerRefs, providerCatalogRefs);
			} else {
				var children = serviceProviderCatalogXml.childNodes;
				var serviceProviderCatalog = UIHelper.getNode(children, "ServiceProviderCatalog", ProcessConstants.NAMESPACE_OSLC); //$NON-NLS-1$
				children = serviceProviderCatalog.childNodes; //children of ServiceProviderCatalog
				var entries = UIHelper.getNodes(children, "entry", ProcessConstants.NAMESPACE_OSLC); //$NON-NLS-1$
				dojo.forEach(entries, function(entry){
					var serviceProvider = UIHelper.getNode(entry.childNodes, "ServiceProvider", ProcessConstants.NAMESPACE_OSLC); //$NON-NLS-1$
					if (serviceProvider) {
						providers.push(serviceProvider);
					}
					var serviceProviderCatalog = UIHelper.getNode(entry.childNodes, "ServiceProviderCatalog", ProcessConstants.NAMESPACE_OSLC); //$NON-NLS-1$
					if (serviceProviderCatalog) {
						providerCatalogs.push(serviceProviderCatalog);
					}
				});
			}
			dojo.forEach(providers, function(serviceProvider){
				var serviceProviderTreeNode = {};
				UIHelper.configureServiceProviderTreeNode(serviceProviderTreeNode, serviceProvider, oslcSpecVersion, catalog.tagName);
				serviceProviders.push(serviceProviderTreeNode);
			});

			dojo.forEach(providerCatalogs, function(serviceProviderCatalog){
				var serviceProviderCatalogNode = {};
				var stubNode = {};
				UIHelper.configureServiceProviderCatalogTreeNode(serviceProviderCatalogNode, stubNode, this.messages.AddProjectLinkDialog_NoServiceProvidersSpecified, serviceProviderCatalog, catalog.tagName);
				serviceProviders.push(serviceProviderCatalogNode);
			}, this);
			
			dojo.forEach(providerCatalogRefs, function(catalogRef){
				var serviceProviderCatalogNode = {};
				var stubNode = {};
				UIHelper.configureServiceProviderCatalogTreeNode(serviceProviderCatalogNode, stubNode, this.messages.AddProjectLinkDialog_NoServiceProvidersSpecified, serviceProviderCatalog, catalog.tagName, catalogRef, this.messages.AddProjectLinkDialog_NestedCatalogName);
				serviceProviders.push(serviceProviderCatalogNode);				
			}, this);
			
			// Invoke the callback immediately if there are no service providers 
			// with external reference. Else, get service providers from all  
			// the external references and then invoke the callback function.
			if (providerRefs.length === 0) {
				callback(serviceProviders, catalog);
			} else {
				var deferreds = [];
				dojo.forEach(providerRefs, dojo.hitch(this, function(serviceProviderRef){
					var header = {
						"accept": ProcessConstants.CONTENT_TYPE_APPLICATION_XML //$NON-NLS-1$
					};
					if (this._oslcCoreEnabled) {
						header[ProcessConstants.OSLC_VERSION_HEADER] = ProcessConstants.OSLC_CORE_VERSION;
					}
					var deferred = jazz.client.xhrGet({
						url: serviceProviderRef,
						headers: header,
						handleAs: "xml", //$NON-NLS-1$
						load: dojo.hitch(this, function(serviceProviderXML){
							var rootRDF = serviceProviderXML.documentElement;
							var children = rootRDF.childNodes;
							var serviceProvider = UIHelper.getNode(children, "ServiceProvider", ProcessConstants.NAMESPACE_OSLC_CORE); //$NON-NLS-1$
							var serviceProviderTreeNode = {};
							UIHelper.configureServiceProviderTreeNode(serviceProviderTreeNode, serviceProvider, oslcSpecVersion, catalog.tagName);
							serviceProviders.push(serviceProviderTreeNode);
						}),
						error: dojo.hitch(this, function(error){
							this._handleErrorGettingRemote(deferred, error, this.messages.AddProjectLinkDialog_FailedServiceProviderCatalog);
							return error;
						})
					});
					deferreds.push(deferred);
				}));
				var deferredList = new dojo.DeferredList(deferreds);
				deferredList.addCallback(function(){
					callback(serviceProviders, catalog);
				});
			}
		},
		
		_updateForServerChange: function(){
			if (this.firstServerChange) {
				var optionArr = this.serverSelector.options;
				optionArr[0] = null; //remove the Select Server... option
				this.firstServerChange = false;
			}
			this._currentLink = null;
			this.finishButton.disabled = true;
			this._getServiceProviders();
			this._updateLinkPreview();
		},
		
		_handleServerChange: function(){
			// Summary: Populate service selector with the valid associations possible.
			// with the selected application. If there are no valid associations possible, 
			// display a message.
			this.dialogHeading.innerHTML = this.originalHeaderHTML;
			this._populateServiceSelector(this._validAssociations);
			if (this._validAssociations.length !== 0) {
				this._updateForServiceChange();
			} else {
				var serviceProviders = [];
				var serviceProviderTreeNode = {};
				serviceProviderTreeNode.name = this.messages.AddProjectLinkDialog_TargetSupportedCatalogsNotFound;
				serviceProviderTreeNode.id = this.messages.AddProjectLinkDialog_TargetSupportedCatalogsNotFound;
				serviceProviderTreeNode.type = ProcessConstants.SERVICE_PROVIDER_ERROR_TYPE;
				serviceProviders.push(serviceProviderTreeNode);
				this._createServiceProvidersTree(serviceProviders);
			}
		},
		
		_updateForServiceChange: function(){
			delete this._selectedServiceProvider;
			this.finishButton.disabled = true;

			var linkType = this.serviceSelector.options[this.serviceSelector.selectedIndex].linkType;
			this._currentLink = {};
			this._currentLink.linkType = linkType;
			this._currentLink.linkLabel = UIHelper.getLinkTypeLabel(linkType, this.messages);
			
			if (!this.firstServerChange) {
				this._createServiceProvidersTree(this._getFilteredServiceProvidersByTagName(this.serviceProviders));
			}
			this._updateLinkPreview();
		},
		
		_updateLinkPreview: function(){
			//Summary: Displays the preview text on the dailog.
			//	Text formats: 
			//	<current project area> uses <selected association> from <the selected service provider>
			//	Or <current project area> provides <selected association> to <the selected service provider>
			dojox.xml.parser.removeChildren(this.previewText);
			var previewText = this.projectAreaName;
			if (this._currentLink) {
				// Append the image if the association has been selected.
				this.previewText.appendChild(UIHelper.getProjectLinkImage(this._currentLink.linkType, "linkPreviewImage")); //$NON-NLS-1$
				var serviceProvider = this._getSelectedServiceProvider();
				var textWithTarget, textWithoutTarget;
				var usesServices = UIHelper.isUsesLinkType(this._currentLink.linkType);
				if (usesServices) {
					textWithTarget = this.messages.AddProjectLinkDialog_PreviewTextWithTarget_Uses;
					textWithoutTarget = this.messages.AddProjectLinkDialog_PreviewTextWithoutTarget_Uses;
				} else {
					textWithTarget = this.messages.AddProjectLinkDialog_PreviewTextWithTarget_Provides;
					textWithoutTarget = this.messages.AddProjectLinkDialog_PreviewTextWithoutTarget_Provides;
				}
				// Display the correct message based on whether service provider has been selected.
				if (serviceProvider && !serviceProvider.isError) {
					previewText = dojo.string.substitute(textWithTarget, [this.projectAreaName, this._currentLink.linkLabel, serviceProvider.name]);
				} else {
					previewText = dojo.string.substitute(textWithoutTarget, [this.projectAreaName, this._currentLink.linkLabel]);
				}
			}
			this.previewText.appendChild(document.createTextNode(previewText));
		},
		
		_handleErrorGettingRemote: function(deferred, error, contextErrorHeading){
			if (error.message) {
				if (error.dojoType === "popupBlocked") { //$NON-NLS-1$
					this._reportErrorFetchingRemote(error.message, contextErrorHeading);
				} else {
					this._reportErrorFetchingRemote(decodeURIComponent(jazz.util.error.getDefaultXhrErrMsg(deferred)), contextErrorHeading);
				}
			} else if (error.responseText) {
				this._reportErrorFetchingRemote(error.responseText, contextErrorHeading);
			} else {
				this._reportErrorFetchingRemote(this.messages.AddProjectLinkDialog_NoErrorDetails, contextErrorHeading);
			}
		},
		
		getAssociatedProject: function(){
			var associatedProjectLink = {};
			var associatedServiceProviderInfo = {};
			var selectedServiceProvider = this._getSelectedServiceProvider();
			associatedServiceProviderInfo.targetUrl = selectedServiceProvider.url[0];
			associatedServiceProviderInfo.presentationUrl = selectedServiceProvider.presentationUrl[0];
			associatedServiceProviderInfo.consumerRegistryUrl = selectedServiceProvider.consumerRegistryUrl[0];
			associatedServiceProviderInfo.servicesUrl = selectedServiceProvider.servicesUrl[0];
			associatedServiceProviderInfo.label = selectedServiceProvider.name[0];
			associatedServiceProviderInfo.serverURL = this._getServerURL();
			associatedProjectLink.projectAreaInfo = associatedServiceProviderInfo;
			associatedProjectLink.sourceProjectAreaInfo = this._sourceProjectAreaInfo;
			associatedProjectLink.linkType = this._currentLink.linkType;
			associatedProjectLink.linkLabel = this._currentLink.linkLabel;
			return associatedProjectLink;
		},
		
		_getSelectedServiceProvider: function(){
			return this._selectedServiceProvider;
		},
		
		_getHostname: function(url){
			var urlsegments = url.split("//"); //$NON-NLS-1$
			if (urlsegments.length < 2) {
				return ProcessConstants.EMPTY;
			}
			urlsegments = urlsegments[1].split("/"); //$NON-NLS-1$
			if (urlsegments.length < 2) {
				return ProcessConstants.EMPTY;
			}
			urlsegments = urlsegments[0].split(":"); //$NON-NLS-1$
			if (urlsegments.length <= 2) {
				return urlsegments[0];
			}
			return ProcessConstants.EMPTY;
		},
		
		_getIsOSLCCoreEnabled: function() {
			// summary: Checks if OSLC Core is enabled or not and evaluates
			// the project links after that.
			var handleResponse = {
				success: dojo.hitch(this, function(response){
					this._oslcCoreEnabled = response === "true"; //$NON-NLS-1$
					this._evalProjectLinks();
				}),
				failure: dojo.hitch(this, function(response){
					this._oslcCoreEnabled = false;
					this._evalProjectLinks();
				})
			};
			
			var srh = new ServiceResponseHandler(handleResponse, "success", "failure"); //$NON-NLS-1$ //$NON-NLS-2$			
			ProcessClient.getIsOslcCoreEnabled(srh);
		},
		
		_getFilteredServiceProvidersByTagName: function(serviceProviders) {
			var linkType = this.serviceSelector.options[this.serviceSelector.selectedIndex].linkType;
			var tagName = UIHelper.getTagNameForLinkType(linkType);
			var newServiceProviders = [];
			for (var i = 0; i < serviceProviders.length; i++) {
				serviceProvider = serviceProviders[i];
				if (serviceProvider.serviceProviderCatalogTagName[0] === tagName) {
					newServiceProviders.push(serviceProvider);
				}
			}
			return newServiceProviders;
		},
		
		_getAllUniqueServiceProviderCatalogs: function(rootServicesNodes) {
			// Get all service provider catalog entries with the given services node list.
	 		// Currently, each type of service provider catalog can have only one entry.
	 		// If one service provider catalog has duplicated entries, the one defined in core(v2)
	 		// format will be chosen. 
			var preCoreCatalogs = this._determineServiceCatalogs(rootServicesNodes);
			var coreCatalogs = this._determineServiceCatalogsOSLCCoreStyle(rootServicesNodes);
			var existedCatalogs = UIHelper.removeDuplicatedCatalogs(coreCatalogs);
			dojo.forEach(preCoreCatalogs, function(catalog){
				if(!UIHelper.isDuplicatedCatalog(catalog, existedCatalogs)) {
					existedCatalogs.push(catalog);
				}
			}, this);
			return existedCatalogs;
		},
		
		uninitialize: function() {
			if (this._serviceProvidersTree) {
				this._serviceProvidersTree.destroyRecursive();
				delete this._serviceProvidersTree;
			}
		}
	});
	
})();
