/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
 
(function() {
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.ConfigurationDataEditor"); //$NON-NLS-1$
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dijit._Templated"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("dojox.xml.parser"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.ConfigurationDataTree"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.ConfigurationDataStore"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.ConfigurationDataPerTypeEditor"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.ConfigurationDataEditor", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/ConfigurationDataEditor.html"), //$NON-NLS-1$ //$NON-NLS-2$
		constructor: function(args){
			this._processAreaEditor = args.editor; //hold the outer project area editor
			this._owningApplicationKey = args.owningApplicationKey;
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		postCreate: function(){
			this.refresh();
		},
		refresh: function() {
			this.clearState();
			this.editor.addSaveParticipant(this, "com.ibm.team.process.web.ui.internal.admin.projects.ConfigurationDataEditor"); //$NON-NLS-1$
			this.createConfigurationDataTree();
			dojox.xml.parser.removeChildren(this.configurationDataTree);
			this.configurationDataTree.appendChild(this._configurationPointTree.domNode);
			this._updateConfigurationDataDescription(this.messages.ConfigurationDataStore_ConfigurationDataRootDescription);
		},
		
		_updateConfigurationDataDescription: function(message) {
			dojox.xml.parser.removeChildren(this._configurationDataDetailEditor);
			var div = this._createConfigurationDataDescription(message);
			this._configurationDataDetailEditor.appendChild(div);	
		},
		
		clearState: function(){
			this.editor.removeSaveParticipant("com.ibm.team.process.web.ui.internal.admin.projects.ConfigurationDataEditor"); //$NON-NLS-1$
			for (var itemId in this.configurationPointCache) {
				this.configurationPointCache[itemId].destroyRecursive();
				this.configurationPointCache[itemId] = null;
			}
			this.configurationPointCache = {};
			
			if (this._configurationPointTree) {
				this._configurationPointTree.destroyRecursive();
				delete this._configurationPointTree;
			}
			
		},
		
		createConfigurationDataTree: function() {
			//get the configuration point category
			//get the configuration points
			var self = this;
			this._configurationPointTree = new com.ibm.team.process.web.ui.internal.admin.projects.ConfigurationDataTree({
				model: self._getModel(),
				focusChanged: function(node){
					self.onConfigurationPoint(node.item, node);
				}
			});
			this._configurationPointTree.domNode.setAttribute("aria-label", this.messages.ConfigurationDataEditorHTML_ConfigurationHeading); //$NON-NLS-1$ 
		},
		
		_getModel: function(){
			var self = this;
			return new dijit.tree.ForestStoreModel({
				store: self._getConfigurationDataStore(),
				childrenAttrs: ["items"] //$NON-NLS-1$
			});
		},
		
		onConfigurationPoint: function(item,node) {
			var editor = {
				configurationPoint: item,
				changesPending: function(){
					return false;
				},
				saved: function(saveResponse){
				},
				aboutToBeSaved: function(saveObj){
				},
				destroyRecursive: function(){
				}
			};
			var itemId = this._configurationPointTree.model.store.getValue(item,"id"); //$NON-NLS-1$ 
			if (!this.configurationPointCache[itemId]) {
				dojox.xml.parser.removeChildren(this._configurationDataDetailEditor);
				var itemType = this._configurationPointTree.model.store.getValue(item,"type"); //$NON-NLS-1$ 
				if(itemType === ProcessConstants.ConfigurationData.CATEGORY) {
					var div = this._createConfigurationDataDescription(item.description);
					editor.domNode = div;
				} else if(itemType === ProcessConstants.ConfigurationData.ROOT) {
					var div = this._createConfigurationDataDescription(item.description);
					editor.domNode = div;
				} else if(itemType === ProcessConstants.ConfigurationData.CONFIGURATION_DATA_TYPE) {
					editor = this._createConfigurationDataPresentationEditor(item);
				}
				this.configurationPointCache[itemId] = editor;
			} else if(this._currentNode !== node) {
				dojox.xml.parser.removeChildren(this._configurationDataDetailEditor);
				editor = this.configurationPointCache[itemId];
			}
			this._currentNode = node;
			if(editor.domNode) {
				if(editor.showConfigurationPointPresentation) {
					editor.showConfigurationPointPresentation(item);
				} 
				
				this._configurationDataDetailEditor.appendChild(editor.domNode);
			}
		},
		
		_getConfigurationDataStore: function() {
			var args = {};
			args._processAreaEditor = this.editor;
			args.owningApplicationKey = this._owningApplicationKey;
			args.errorCallback = dojo.hitch(this._processAreaEditor, this._processAreaEditor.handleServerResponseError);
			return new com.ibm.team.process.web.ui.internal.admin.projects.ConfigurationDataStore(args);
		},
		
		_createConfigurationDataDescription: function(message) {
			var div = document.createElement("div"); //$NON-NLS-1$
			dojo.addClass(div, "configurationDataDescription"); //$NON-NLS-1$
			div.innerHTML = message;
			return div;
		},
		
		_createConfigurationDataPresentationEditor: function(item) {
			var configurationDataPerTypeEditor = new com.ibm.team.process.web.ui.internal.admin.projects.ConfigurationDataPerTypeEditor(
				{
					configurationPointId: item.id,
					configurationPointStore: this._configurationPointTree.model.store,
					editor: this.editor
				}
			); 
			return configurationDataPerTypeEditor;
		},
		
		changesPending: function() {
			return this._configurationPointTree.model.store.isDirty();
		},
		
		aboutToBeSaved: function(saveObj) {
			var store = this._configurationPointTree.model.store;
		 	var getConfigurationDataTypes = function(items,request) {
				for(var i = 0;i < items.length;i ++) {
					if(!saveObj.jsonConfigurationDataTypeInstances) {
						saveObj.jsonConfigurationDataTypeInstances = [];
					}
					var item = {};
					item.id = store.getValue(items[i],"id"); //$NON-NLS-1$
					item.instanceUrl = store.getValue(items[i], "instanceUrl"); //$NON-NLS-1$
					item.isFinal = store.getValue(items[i], "isFinal"); //$NON-NLS-1$
					saveObj.jsonConfigurationDataTypeInstances.push(dojo.toJson(item));
				}
			};
			this._configurationPointTree.model.store.fetch({
				query: {
						type:ProcessConstants.ConfigurationData.CONFIGURATION_DATA_TYPE,
					   	instanceUrlChanged:true
					   },
				queryOptions: {deep: true},
				onComplete: getConfigurationDataTypes});
		},
		
		saved: function(saveResponse) {
			
			if(this._configurationPointTree.model.store.isDirty()) {
				var self = this;
				var getConfigurationDataTypes = function(items,request) {
					for(var i = 0;i < items.length;i ++) {
						if(self._configurationPointTree.model.store.getValue(items[i],"instanceUrlChanged")) { //$NON-NLS-1$
							self._configurationPointTree.model.store.setValue(items[i],"instanceUrlChanged",false); //$NON-NLS-1$
						}
					}
					self._configurationPointTree.model.store.save();
				};
				this._configurationPointTree.model.store.fetch({
					query: {
							type:ProcessConstants.ConfigurationData.CONFIGURATION_DATA_TYPE,
						   	instanceUrlChanged:true
						   },
					queryOptions: {deep: true},
					onComplete: getConfigurationDataTypes});
			}
		},
		
		uninitialize: function(){
			this.editor.removeSaveParticipant("com.ibm.team.process.web.ui.internal.admin.projects.ConfigurationDataEditor"); //$NON-NLS-1$
			for (var itemId in this.configurationPointCache) {
				this.configurationPointCache[itemId].destroyRecursive();
				this.configurationPointCache[itemId] = null;
			}
			
			delete this.configurationPointCache;
			if (this._configurationPointTree) {
				this._configurationPointTree.destroyRecursive();
				delete this._configurationPointTree;
			}
			if(this._processAreaEditor) {
				delete this._processAreaEditor;
			}
			this.inherited(arguments);
		}
		
	});
})();