/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
 
(function() {
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.ConfigurationDataPerTypeEditor"); //$NON-NLS-1$
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dijit._Templated"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("dojox.xml.parser"); //$NON-NLS-1$
	dojo.require("jazz.ui.RemoteChooserContainer"); //$NON-NLS-1$
	dojo.require("jazz.ui.RemoteChooserResponder"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.ConfigurationDataPerTypeEditor", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/ConfigurationDataPerTypeEditor.html"), //$NON-NLS-1$ //$NON-NLS-2$
		constructor: function(args){
			this._processAreaEditor = args.editor; //hold the outer project area editor
			this.configurationPointStore = args.configurationPointStore;
			this.configurationPointId = args.configurationPointId; 
			var configurationPoint;
			this.configurationPointStore.fetchItemByIdentity( {
					identity: this.configurationPointId,
					onItem: function(item, request){
						configurationPoint = item;
					}
				});
			this.configurationPoint = configurationPoint;
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		postCreate: function(){
			var instanceUrl = this.configurationPointStore.getValue(this.configurationPoint,"instanceUrl"); //$NON-NLS-1$
			if (!instanceUrl) { //final enabled only there are instances already existed
				this.finalCheckBox.disabled = true;
			} else {
				this.finalCheckBox.disabled = false;
			}
			this.connect(this.finalCheckBox, "onclick", "_onFinalClick"); //$NON-NLS-1$ //$NON-NLS-2$
			this.refresh();
		},
		refresh: function() {
			//
		},
		
		_onFinalClick: function() {
			var isFinal = this.finalCheckBox.checked;
			this.configurationPointStore.setValue(this.configurationPoint, "isFinal", isFinal); //$NON-NLS-1$
			this.configurationPointStore.setValue(this.configurationPoint,"instanceUrlChanged",true); //$NON-NLS-1$
			this.editor.setDirty(this.editor.changesPending());
		},
		
		getRemoteInstanceUrl: function(item) {
			var remoteUrl = ""; //$NON-NLS-1$
			var instanceUrl = this.configurationPointStore.getValue(item,"instanceUrl"); //$NON-NLS-1$
			if(instanceUrl) {
				remoteUrl = instanceUrl;
			} else {
				var factoryUrl = this.configurationPointStore.getValue(item,"factoryUrl"); //$NON-NLS-1$
				if (factoryUrl) {
					remoteUrl = factoryUrl;
				}
			}
			return remoteUrl;
		},

		
		showConfigurationPointPresentation: function(item) {
			var self = this;
			var callback = function(data) {
				var isWindowName = !(dojo.isSafari >= 4 || dojo.isFF >= 3); // && dojo.isIE < 8;	
	    		if (isWindowName) {
	    			dojox.xml.parser.removeChildren(self.remoteChooserContainerDiv);
	                if (self.remoteChooserContainer) {
	                	self.remoteChooserContainer.destroyRecursive();
	                }
	    			var placeholder = document.createElement("div"); //$NON-NLS-1$
	    			self.remoteChooserContainerDiv.appendChild(placeholder);
	    			self.remoteChooserContainer = new jazz.ui.RemoteChooserContainer({
	    				remoteLocation: data,
	    				onData: callback
	    			}, placeholder);
	    		}
				self.saveInstanceUrl(item,data);
			};
			
			
			var remoteUrl = "";//$NON-NLS-1$
			remoteUrl = this.getRemoteInstanceUrl(item);
			this.showRemoteInstanceUrl(item);
			var isFinal = this.configurationPointStore.getValue(item, "isFinal"); //$NON-NLS-1$
			this.finalCheckBox.checked = isFinal;
			if(this.remoteChooserContainer) {
				this.remoteChooserContainer.destroyRecursive();
			}
			var placeholder = document.createElement("div"); //$NON-NLS-1$
			var parentNode =  this.remoteChooserContainerDiv;
			dojox.xml.parser.removeChildren(parentNode);
			parentNode.appendChild(placeholder);
			this.remoteChooserContainer = new jazz.ui.RemoteChooserContainer({
    				remoteLocation: remoteUrl,
    				onData: callback
    			}, placeholder);
	 	},
			
		saveInstanceUrl: function(item,instanceUrl) {
			this.configurationPointStore.setValue(item,"instanceUrl",instanceUrl); //$NON-NLS-1$
			this.configurationPointStore.setValue(item,"instanceUrlChanged",true); //$NON-NLS-1$
			this.editor.setDirty(this.editor.changesPending());
			this.showRemoteInstanceUrl(item);
			this.finalCheckBox.disabled = false;
		},
		
		showRemoteInstanceUrl: function(item) {
			var remoteUrl = this.getRemoteInstanceUrl(item);
			// this.instanceUrlSpan.innerHTML = remoteUrl;
			dojo.attr(this.instanceUrlSpan, "innerHTML", remoteUrl); //$NON-NLS-1$
		},
		
		changesPending: function() {
			return this.configurationPointStore.isDirty();
		},
		aboutToBeSaved: function(saveObj) {
			
		},
		saved: function(saveResponse) {
			
		},
		
		uninitialize: function() {
			if(this._processAreaEditor) {
				delete this._processAreaEditor;
			}
			
			if(this.remoteChooserContainer) {
				this.remoteChooserContainer.destroyRecursive();
			}
			
			this.inherited(arguments);
		}
		
		
	});
})();