/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.ConfigurationDataStore"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("dojo.data.ItemFileWriteStore"); //$NON-NLS-1$
	dojo.require("jazz.core.RegistryFactory"); //$NON-NLS-1$
	dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.ConfigurationDataClient"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.client.internal.ProcessClient"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
	var ConfigurationDataClient = com.ibm.team.process.web.ui.internal.admin.projects.ConfigurationDataClient;
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var RegistryFactory = jazz.core.RegistryFactory;
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.ConfigurationDataStore", dojo.data.ItemFileWriteStore, { //$NON-NLS-1$
		constructor: function(args) {
			this._jsonData = {
					identifier: 'id', //$NON-NLS-1$
					label: 'name', //$NON-NLS-1$
					items: []
			};
			this._processAreaEditor = args._processAreaEditor;
			this._owningApplicationKey = args.owningApplicationKey;
			this._errorCallback = args.errorCallback;
			this.configurationPointCategoryArray = [];
			this.configurationPointArray = [];
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			this._initialize();	
		},
		_initialize: function() {
			this._addConfigurationPointRootItem();
			this._populateConfigurationPointTree();
		},
		_addConfigurationPointRootItem: function() {
			this.newItem({
				id: ProcessConstants.ConfigurationData.ROOT,
				name: this.messages.ConfigurationDataStore_ConfigurationDataRootName,
				description: this.messages.ConfigurationDataStore_ConfigurationDataRootDescription,
				type: ProcessConstants.ConfigurationData.ROOT
			});
			this.save();
		},
		
		_populateConfigurationPointTree: function() {
			this._calculateConfigurationPointCategory();
		},
		
		_buildConfigurationPointTree: function(configurationPointCategoryArray,configurationPointArray, configurationPointInstanceArray) {
			configurationPointCategoryArray = this._normalizeConfigurationPointCategoryArray(configurationPointCategoryArray);
			var self = this;
			this.fetchItemByIdentity( {
				identity: ProcessConstants.ConfigurationData.ROOT,
				onItem: function(item, request) {
					dojo.forEach(configurationPointCategoryArray, function(configurationPointCategory){
						var newChildItem = self.newItem(
							{
								id: configurationPointCategory.id,
								name: configurationPointCategory.name,
								description: configurationPointCategory.description,
								type: ProcessConstants.ConfigurationData.CATEGORY,
								parent: item.id
							},
							{
								parent: item,
								attribute: "items" //$NON-NLS-1$
							}
						);
					});
				}
			}); 
			
			configurationPointArray = this._normalizeConfigurationPointArray(configurationPointArray);
			for(var i = 0; i < configurationPointArray.length; i++) {
				var configurationPointCategory = this._getConfigurationCategory(configurationPointCategoryArray,configurationPointArray[i]);
				var configurationPointInstanceInfo = this._getConfigurationPointInstanceInfo(configurationPointArray[i],configurationPointInstanceArray);
				var configurationPointInstanceUrl = configurationPointInstanceInfo.instanceUrl;
				var configurationPointInstanceIsFinal = configurationPointInstanceInfo.isFinal;
				if(configurationPointInstanceIsFinal === "true") { //$NON-NLS-1$
					configurationPointInstanceIsFinal = true; 
				} else {
					configurationPointInstanceIsFinal = false;
				}
				var self = this;
				this.fetchItemByIdentity ({
					identity: configurationPointCategory.id,
					onItem: function(item,request) {
						var newChildItem = self.newItem(
							{
									id: configurationPointArray[i].id,
									name: configurationPointArray[i].name,
									description: configurationPointArray[i].description,
									categoryId: configurationPointArray[i].categoryId,
									factoryUrl: configurationPointArray[i].factoryUrl,
									instanceUrl: configurationPointInstanceUrl,
									isFinal: configurationPointInstanceIsFinal,
									type: ProcessConstants.ConfigurationData.CONFIGURATION_DATA_TYPE,
									instanceUrlChanged: false,
									parent: item.id
							},
							{
									parent: item,
									attribute: "items" //$NON-NLS-1$
							}
						);
					}
				    }
				);
			}
			this.save();
		},
		
		_calculateConfigurationPointCategory: function() {
			ConfigurationDataClient.getConfigurationPointCategories(this._owningApplicationKey, dojo.hitch(this,this._populateConfigurationDataCategories),dojo.hitch(this,this._handleConfigurationDataCategoriesFecthError));
		},
		
		_populateConfigurationDataCategories: function(configurationPointCategoryXMLResponse) {
			var i = 0;
			this.configurationPointCategoryArray = [];
			var configurationPointCategoriesDocument = dojox.xml.DomParser.parse(configurationPointCategoryXMLResponse);
			var nodeList = configurationPointCategoriesDocument.childNodes;
			if (!nodeList || nodeList.length === 0) {
				return [];
			}
			var configurationPointCategoriesNodeList = UIHelper.getNodes(nodeList,"configuration-data-categories");//$NON-NLS-1$
			if( !configurationPointCategoriesNodeList || configurationPointCategoriesNodeList.length === 0) {
				return [];
			}
			var configurationPointCategoryNodeList = UIHelper.getNodes(configurationPointCategoriesNodeList[0].childNodes,"configuration-data-category");//$NON-NLS-1$				 
			var i = 0;
			var self = this;
			dojo.forEach(configurationPointCategoryNodeList, function(configurationPointCategoryNode){
				var configurationPointCategory = {};
				var configurationPointCategoryId = "";//$NON-NLS-1$
				var configurationPointCategoryName = "";//$NON-NLS-1$	
				if(configurationPointCategoryNode.attributes[0].localName === "id") { //$NON-NLS-1$
					configurationPointCategoryId = configurationPointCategoryNode.attributes[0].nodeValue;
					configurationPointCategoryName = configurationPointCategoryNode.attributes[1].nodeValue;
				} else {
					configurationPointCategoryName = configurationPointCategoryNode.attributes[0].nodeValue;
					configurationPointCategoryId = configurationPointCategoryNode.attributes[1].nodeValue;
				}
				var configurationPointCategoryChildNodes = configurationPointCategoryNode.childNodes;
				var descriptionNode = UIHelper.getNode(configurationPointCategoryChildNodes,"description"); //$NON-NLS-1$
				var description = "";//$NON-NLS-1$
				
				if(descriptionNode) {
					if(descriptionNode.childNodes && descriptionNode.childNodes.length > 0) {
						description = descriptionNode.childNodes[0].nodeValue;
					} 
				}
				configurationPointCategory.id = configurationPointCategoryId;
				configurationPointCategory.name = configurationPointCategoryName;
				configurationPointCategory.description = description;
				self.configurationPointCategoryArray[i++] = configurationPointCategory; 
		  });
		  	var uncategorizedCategory = {
				id: "uncategorized",   //$NON-NLS-1$
				name: this.messages.ConfigurationDataStore_UncategorizedName,  
				type: ProcessConstants.ConfigurationData.CATEGORY,
				description: this.messages.ConfigurationDataStore_UncategorizedDescription
				
			};
			this.configurationPointCategoryArray[i] = uncategorizedCategory;
			ConfigurationDataClient.getConfigurationPoints(this._owningApplicationKey, dojo.hitch(this,this._populateConfigurationPoint),dojo.hitch(this,this._handleConfiguratioPointFetchError));
		},
		
		_populateConfigurationPoint: function(configurationPointXMLResponse) {
				var configurationPointsDocument = dojox.xml.DomParser.parse(configurationPointXMLResponse);
				var nodeList = configurationPointsDocument.childNodes;
				if (!nodeList || nodeList.length === 0) {
					return;
				}
				var configurationPointsNodeList = UIHelper.getNodes(nodeList,"configuration-data-types");//$NON-NLS-1$
				if (!configurationPointsNodeList || configurationPointsNodeList.length === 0) {
					return;
				}
				var configurationPointNodeList = UIHelper.getNodes(configurationPointsNodeList[0].childNodes,"configuration-data-type");//$NON-NLS-1$				 
				this.onfigurationPointArray = [];
				var i = 0;
				var self = this;
				dojo.forEach(configurationPointNodeList, function(configurationPointNode){
					var configurationPoint = {};
					var configurationPointId = "";//$NON-NLS-1$
					var configurationPointName = "";//$NON-NLS-1$	
					if(configurationPointNode.attributes[0].localName === "id") { //$NON-NLS-1$
						configurationPointId = configurationPointNode.attributes[0].nodeValue;
						configurationPointName = configurationPointNode.attributes[1].nodeValue;
					} else {
						configurationPointName = configurationPointNode.attributes[0].nodeValue;
						configurationPointId = configurationPointNode.attributes[1].nodeValue;
					}
					var configurationPointChildNodes = configurationPointNode.childNodes;
					var factoryUrlNode = UIHelper.getNode(configurationPointChildNodes,"factoryUrl"); //$NON-NLS-1$
					var categoryIdNode = UIHelper.getNode(configurationPointChildNodes,"categoryId"); //$NON-NLS-1$
					var descriptionNode = UIHelper.getNode(configurationPointChildNodes,"description"); //$NON-NLS-1$
					var factoryUrl = ""; //$NON-NLS-1$
					var description = "";//$NON-NLS-1$
					var categoryId = ""; //$NON-NLS-1$
					if (factoryUrlNode) {
						factoryUrl = factoryUrlNode.childNodes[0].nodeValue;
					}
					if (categoryIdNode) {
						if(categoryIdNode.childNodes && categoryIdNode.childNodes.length > 0) {
							categoryId = categoryIdNode.childNodes[0].nodeValue;
						} 
					}
					if(descriptionNode) {
						if(descriptionNode.childNodes && descriptionNode.childNodes.length > 0) {
							description = descriptionNode.childNodes[0].nodeValue;
						} 
					}
					configurationPoint.description = description;
					configurationPoint.factoryUrl = factoryUrl;
					configurationPoint.categoryId = categoryId;
					configurationPoint.id = configurationPointId;
					configurationPoint.name = configurationPointName;
					self.configurationPointArray[i++] = configurationPoint;
				});
			ConfigurationDataClient.getConfigurationPointInstances(dojo.hitch(this,this._populateConfigurationPointInstanceUrl),dojo.hitch(this,this._handleConfiguratioPointInstanceUrlFetchError),this._processAreaEditor.projectAreaItemId);	
		},
		
		_populateConfigurationPointInstanceUrl: function(configurationPointInstanceXMLResponse) {
			var configurationPointInstanceArray = [];
			var configurationPointInstancesDocument = dojox.xml.DomParser.parse(configurationPointInstanceXMLResponse);
				var nodeList = configurationPointInstancesDocument.childNodes;
				if (!nodeList || nodeList.length === 0) {
					return;
				}
				var configurationPointInstancesNodeList = UIHelper.getNodes(nodeList,"configuration-data-instances");//$NON-NLS-1$
				if (!configurationPointInstancesNodeList || configurationPointInstancesNodeList.length === 0) {
					return;
				}
				var configurationPointInstanceNodeList = UIHelper.getNodes(configurationPointInstancesNodeList[0].childNodes,"configuration-data-instance");//$NON-NLS-1$				 
				var i = 0;
				var self = this;
				dojo.forEach(configurationPointInstanceNodeList, function(configurationPointInstanceNode){
					var configurationPointInstance = {};
					var configurationPointId = "";//$NON-NLS-1$
					var configurationDataInstanceIsFinal = false;
					var configurationPointInstanceUrl = "";//$NON-NLS-1$	
					var nodeInfo = self._getConfigurationPointInstanceNodeInfo(configurationPointInstanceNode);
					configurationPointId = nodeInfo.configurationPointId;
					configurationDataInstanceIsFinal = nodeInfo.configurationDataInstanceIsFinal;
					instanceUrl = nodeInfo.instanceUrl;
					configurationPointInstance.configurationPointId = configurationPointId;
					configurationPointInstance.instanceUrl = instanceUrl;
					configurationPointInstance.isFinal = configurationDataInstanceIsFinal;
					var b = self._isConfigurationPointInstanceInvalid(configurationPointInstance);
					if(b) {
						return;
					}
					configurationPointInstanceArray[i++] = configurationPointInstance;
				});
			
			
			this._buildConfigurationPointTree(this.configurationPointCategoryArray,this.configurationPointArray, configurationPointInstanceArray);			
		},
		
		_getConfigurationPointInstanceNodeInfo: function(configurationPointInstanceNode) {
			var nodeInfo = [];
			var attributeName = configurationPointInstanceNode.attributes[0].localName;
			if(attributeName === "configurationDataTypeId" ) {//$NON-NLS-1$
				nodeInfo.configurationPointId = configurationPointInstanceNode.attributes[0].nodeValue;
			} else if(attributeName === "configurationDataInstanceIsFinal") {//$NON-NLS-1$
				nodeInfo.configurationDataInstanceIsFinal = configurationPointInstanceNode.attributes[0].nodeValue;
			}
			attributeName = configurationPointInstanceNode.attributes[1].localName;
			if(attributeName === "configurationDataTypeId" ) {//$NON-NLS-1$
				nodeInfo.configurationPointId = configurationPointInstanceNode.attributes[1].nodeValue;
			} else if(attributeName === "configurationDataInstanceIsFinal") {//$NON-NLS-1$
				nodeInfo.configurationDataInstanceIsFinal = configurationPointInstanceNode.attributes[1].nodeValue;
			}
			
			var configurationPointInstanceChildNodes = configurationPointInstanceNode.childNodes;
			var instanceUrlNode = UIHelper.getNode(configurationPointInstanceChildNodes,"configurationDataInstanceUrl"); //$NON-NLS-1$
	
			var instanceUrl = ""; //$NON-NLS-1$
			if (instanceUrlNode) {
				instanceUrl = instanceUrlNode.childNodes[0].nodeValue;
			}
			nodeInfo.instanceUrl = instanceUrl;
			if(nodeInfo.configurationPointId && nodeInfo.configurationDataInstanceIsFinal) {
				return nodeInfo;
			}
			attributeName = configurationPointInstanceNode.attributes[2].localName;
			if(attributeName === "configurationDataTypeId" ) {//$NON-NLS-1$
				nodeInfo.configurationPointId = configurationPointInstanceNode.attributes[2].nodeValue;
			} else if(attributeName === "configurationDataInstanceIsFinal") {//$NON-NLS-1$
				nodeInfo.configurationDataInstanceIsFinal = configurationPointInstanceNode.attributes[2].nodeValue;
			}
			return nodeInfo;
		},
		
		_normalizeConfigurationPointCategoryArray: function(configurationPointCategoryArray) {
			var result = [];
			var k = 0;
			for(var i = 0; i < configurationPointCategoryArray.length; i++) {
				var b = this._isConfigurationPointCategoryInvalid(configurationPointCategoryArray[i]);
				if(b) {
					continue;
				}
				var j = 0;
				for(j = 0; j < result.length; j++) {
					if(result[j].id === configurationPointCategoryArray[i].id) {
						break;
					}
				}
				if(j === result.length) {
					result[k++] = configurationPointCategoryArray[i];
				}
			}
			return result;
		},
		
		_isConfigurationPointCategoryInvalid: function(configurationPointCategory) {
			if(!configurationPointCategory || !configurationPointCategory.id
					|| !configurationPointCategory.name 
					|| dojo.string.trim(configurationPointCategory.id) === ""  //$NON-NLS-1$
					|| dojo.string.trim(configurationPointCategory.name) === "") { //$NON-NLS-1$
				return true;	
			}
			return false;
		},
		
		_normalizeConfigurationPointArray: function(configurationPointArray) {
			var result = [];
			var k = 0;
			for(var i = 0; i < configurationPointArray.length; i++) {
				var b = this._isConfigurationPointInvalid(configurationPointArray[i]);
				if(b) {
					continue;
				}
				var j = 0;
				for(j = 0; j < result.length; j++) {
					if(result[j].id === configurationPointArray[i].id) {
						break;
					}
				}
				if(j === result.length) {
					result[k++] = configurationPointArray[i];
				}
			}
			return result;
		},
		
		_isConfigurationPointInvalid: function(configurationPoint) {
			if(!configurationPoint || !configurationPoint.id
					|| !configurationPoint.name 
					|| dojo.string.trim(configurationPoint.id) === ""  //$NON-NLS-1$
					|| dojo.string.trim(configurationPoint.name) === "") { //$NON-NLS-1$
				return true;	
			}
			return false;
		},
		
		_isConfigurationPointInstanceInvalid : function(configurationPointInstance) {
			if(!configurationPointInstance || !configurationPointInstance.configurationPointId
					 || !configurationPointInstance.instanceUrl 
					 || dojo.string.trim(configurationPointInstance.configurationPointId) === "" //$NON-NLS-1$
					 || dojo.string.trim(configurationPointInstance.instanceUrl) === "") { //$NON-NLS-1$
				return true;		 	
			}
			return false; 
					
				  
		},
		
		_getConfigurationCategory: function(configurationPointCategoryArray,configurationPoint) {
			for(var i = 0;i < configurationPointCategoryArray.length; i++) {
				if(configurationPointCategoryArray[i].id === configurationPoint.categoryId) {
					return configurationPointCategoryArray[i];
				}
			}
			return configurationPointCategoryArray[configurationPointCategoryArray.length -1];
		},
		
		_getConfigurationPointInstanceInfo: function(configurationPoint,configurationPointInstanceArray) {
			var configurationDataInstanceInfo = [];
			configurationDataInstanceInfo.instanceUrl = ""; //$NON-NLS-1$
			configurationDataInstanceInfo.isFinal = false;
			if( !configurationPointInstanceArray || configurationPointInstanceArray.length === 0) {
				return configurationDataInstanceInfo;
			}
			for(var i = 0 ; i< configurationPointInstanceArray.length; i++) {
				if(configurationPoint.id === configurationPointInstanceArray[i].configurationPointId) {
					configurationDataInstanceInfo.instanceUrl = configurationPointInstanceArray[i].instanceUrl;
					configurationDataInstanceInfo.isFinal = configurationPointInstanceArray[i].isFinal;
					return  configurationDataInstanceInfo;
				}
			}
			return configurationDataInstanceInfo;
		},
		
		_handleConfigurationDataCategoriesFecthError: function(error) {
			this.configurationPointCategoryArray = [];
			if(this.isDirty()) {
				this.save();
			}
			var message = this.messages.ConfigurationDataStore_ErrorFetchConfigurationDataCategories;
			this._errorCallback(error, message);
		},
		
		_handleConfiguratioPointFetchError: function(error) {
			if(this.isDirty()) {
				this.save();
			}
			var message = this.messages.ConfigurationDataStore_ErrorFetchConfigurationDataTypes;
			this._errorCallback(error, message);
		},
		
		_handleConfiguratioPointInstanceUrlFetchError: function(error) {
			if(this.isDirty()) {
				this.save();
			}
			var message = this.messages.ConfigurationDataStore_ErrorFetchConfigurationDataInstances;
			this._errorCallback(error, message);
		}

});})();