/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.DeferredLoadReadStore"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dojo.data.ItemFileReadStore"); //$NON-NLS-1$
dojo.require("jazz.client.xhr"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper");//$NON-NLS-1$
var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;

dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.DeferredLoadReadStore", dojo.data.ItemFileReadStore, { //$NON-NLS-1$
	constructor: function(params){
		this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
		this._oslcCoreEnabled = params.oslcCoreEnabled;
	},
	
	isItemLoaded: function(item){
		var type = this.getValue(item, "type"); //$NON-NLS-1$
		if (type === ProcessConstants.SERVICE_PROVIDER_NOT_LOADED_TYPE) {
			return false;
		}
		return true;
	},
	
	/**
	 * Overload of ItemFileReadStore#loadItem.
	 * See Read#loadItem for details on parameter etc.
	 */
	loadItem: function(keywordArgs){
		var item = keywordArgs.item;
		this._assertIsItem(item);
		
		var serviceProviderCatalogNode;
		var serviceProviderCatalogId = this.getValue(item, "serviceProviderCatalogId"); //$NON-NLS-1$
		this.fetch({
			query: {
				id: "*" + serviceProviderCatalogId //$NON-NLS-1$
			},
			queryOptions: {
				deep: true
			},
			onItem: function(catalog, request){
				serviceProviderCatalogNode = catalog;
			}
		});
		var serviceProviderResource = this.getValue(serviceProviderCatalogNode, "serviceProviderCatalogURL"); //$NON-NLS-1$
		var serviceProviderCatalogTagName = this.getValue(serviceProviderCatalogNode, "serviceProviderCatalogTagName") //$NON-NLS-1$
		var header = {"accept": ProcessConstants.CONTENT_TYPE_APPLICATION_XML};  //$NON-NLS-1$
		if (this._oslcCoreEnabled) {
			header[ProcessConstants.OSLC_VERSION_HEADER] = ProcessConstants.OSLC_CORE_VERSION;
		}
		var deferred = jazz.client.xhrGet({
			url: serviceProviderResource,
			headers: header,
			handleAs: "xml", //$NON-NLS-1$
			load: dojo.hitch(this, function(serviceProviderCatalogXML, ioArgs){
				var oslcResponseVersion = ioArgs.xhr.getResponseHeader(ProcessConstants.OSLC_VERSION_HEADER);
				this._populateServiceProviders(serviceProviderCatalogXML, item, serviceProviderCatalogNode, keywordArgs, oslcResponseVersion, serviceProviderCatalogTagName);
				return serviceProviderCatalogXML;
			}),
			error: dojo.hitch(this, function(error){
				item.name = [this.messages.DeferredLoadReadStore_Error];
				item.type = ProcessConstants.SERVICE_PROVIDER_ERROR_TYPE;
				//the default error handler dumps to the console
				if (keywordArgs.onError) {
					var scope = keywordArgs.scope ? keywordArgs.scope : dojo.global;
					keywordArgs.onError.call(scope, error);
				}
				if (keywordArgs.onItem) {
					scope = keywordArgs.scope ? keywordArgs.scope : dojo.global;
					keywordArgs.onItem.call(scope, serviceProviderCatalogNode);
				}
				return error;
			})
		});
	},
	
	_determineServiceProvidersOSLCCore: function(/* DOM Object*/ serviceProviderCatalogXml, /* Array */ providers, /* Array */providerCatalogs, /* Array */providerRefs, /* Array */providerCatalogRefs){
		// summary: determines the service providers and service provider
		// catalogs from the provided catalog xml based on oslc core specs.
		// The evaluated service providers and catalogs are pushed in the
		// provided 'providers', 'providerCatalogs', 'providerRefs' 
		// and 'providerCatalogRefs' arrays.
		// providers - array of service providers with inline information
		// providerCatalogs - array of nested catalogs with inline information
		// providerRefs - array of service provider with external references
		// providerCatalogRefs - array of nested catalogs with external references.
		
		var root = serviceProviderCatalogXml.documentElement;
		var children = root.childNodes;
		var serviceProviderCatalog = UIHelper.getNode(children, "ServiceProviderCatalog", ProcessConstants.NAMESPACE_OSLC_CORE); //$NON-NLS-1$
		children = serviceProviderCatalog.childNodes; //children of ServiceProviderCatalog
		var oslcServiceProviders = UIHelper.getNodes(children, "serviceProvider", ProcessConstants.NAMESPACE_OSLC_CORE); //$NON-NLS-1$
		dojo.forEach(oslcServiceProviders, function(oslcServiceProvider){
			var serviceProviderRef = UIHelper.getAttributeNodeValue(oslcServiceProvider, "resource", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
			if (serviceProviderRef) {
				providerRefs.push(serviceProviderRef);
			} else {
				var serviceProvider = UIHelper.getNode(oslcServiceProvider.childNodes, "ServiceProvider", ProcessConstants.NAMESPACE_OSLC_CORE); //$NON-NLS-1$
				if (serviceProvider) {
					providers.push(serviceProvider);
				}
			}
		});

		var oslcServiceProviderCats = UIHelper.getNodes(children, "serviceProviderCatalog", ProcessConstants.NAMESPACE_OSLC_CORE); //$NON-NLS-1$
		dojo.forEach(oslcServiceProviderCats, function(oslcServiceProviderCat){
			var serviceProviderCatalogRef = UIHelper.getAttributeNodeValue(oslcServiceProviderCat, "resource", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
			if (serviceProviderCatalogRef) {
				providerCatalogRefs.push(serviceProviderCatalogRef);
			} else {
				var serviceProviderCatalog = UIHelper.getNode(oslcServiceProviderCat.childNodes, "ServiceProviderCatalog", ProcessConstants.NAMESPACE_OSLC_CORE); //$NON-NLS-1$
				if (serviceProviderCatalog) {
					providerCatalogs.push(serviceProviderCatalog);
				}
			}
		});
	},
		
	_populateServiceProviders: function(serviceProviderCatalogXml, stubItem, parentServiceProviderCatalogTreeNode, keywordArgs, oslcSpecVersion, serviceProviderCatalogTagName){
		if (!serviceProviderCatalogXml) {
			return;
		}
		var serviceProviders = [];
		var providers = [];
		var providerCatalogs = [];
		var providerRefs = [];
		var providerCatalogRefs = [];
		if (this._oslcCoreEnabled && oslcSpecVersion && oslcSpecVersion === ProcessConstants.OSLC_CORE_VERSION) {
			this._determineServiceProvidersOSLCCore(serviceProviderCatalogXml, providers, providerCatalogs, providerRefs, providerCatalogRefs);
		} else {
			var children = serviceProviderCatalogXml.childNodes;
			var serviceProviderCatalog = UIHelper.getNode(children, "ServiceProviderCatalog", ProcessConstants.NAMESPACE_OSLC); //$NON-NLS-1$
			children = serviceProviderCatalog.childNodes; //children of ServiceProviderCatalog
			var entries = UIHelper.getNodes(children, "entry", ProcessConstants.NAMESPACE_OSLC); //$NON-NLS-1$
			dojo.forEach(entries, function(entry){
				var serviceProvider = UIHelper.getNode(entry.childNodes, "ServiceProvider", ProcessConstants.NAMESPACE_OSLC); //$NON-NLS-1$
				if (serviceProvider) {
					providers.push(serviceProvider);
				}
				var serviceProviderCatalog = UIHelper.getNode(entry.childNodes, "ServiceProviderCatalog", ProcessConstants.NAMESPACE_OSLC); //$NON-NLS-1$
				if (serviceProviderCatalog) {
					providerCatalogs.push(serviceProviderCatalog);
				}
			});
		}
			
		var firstChild = true;
		dojo.forEach(providers, function(serviceProvider){
			var serviceProviderTreeNode;
			if (firstChild) {
				serviceProviderTreeNode = stubItem;
			} else {
				serviceProviderTreeNode = {};
			}
			UIHelper.configureServiceProviderTreeNode(serviceProviderTreeNode, serviceProvider, oslcSpecVersion, serviceProviderCatalogTagName);
			
			if (!firstChild) {
				parentServiceProviderCatalogTreeNode.children.push(serviceProviderTreeNode);
				this._arrayOfAllItems.push(serviceProviderTreeNode);
				serviceProviderTreeNode[this._storeRefPropName] = this;
				serviceProviderTreeNode[this._itemNumPropName] = (this._arrayOfAllItems.length - 1); //Last one pushed in is the new item
			} else {
				firstChild = false;
			}
		}, this);
		
		// Local function to parse the service provider catalogs to build UI.
		var parseServiceProviderCatalogs = dojo.hitch(this, function(serviceProviderCatalogNode, stubNode){
			this._arrayOfAllItems.push(serviceProviderCatalogNode);
			serviceProviderCatalogNode[this._storeRefPropName] = this;
			serviceProviderCatalogNode[this._itemNumPropName] = (this._arrayOfAllItems.length - 1); //Last one pushed in is the new item
			parentServiceProviderCatalogTreeNode.children.push(serviceProviderCatalogNode);
			
			this._arrayOfAllItems.push(stubNode);
			stubNode[this._storeRefPropName] = this;
			stubNode[this._itemNumPropName] = (this._arrayOfAllItems.length - 1); //Last one pushed in is the new item
			serviceProviders.push(serviceProviderCatalogNode);
		});
		
		dojo.forEach(providerCatalogs, function(serviceProviderCatalog){
			var serviceProviderCatalogNode = {};
			var stubNode = {};
			UIHelper.configureServiceProviderCatalogTreeNode(serviceProviderCatalogNode, stubNode, this.messages.DeferredLoadReadStore_NoServiceProvidersSpecified, serviceProviderCatalog, serviceProviderCatalogTagName);
			parseServiceProviderCatalogs(serviceProviderCatalogNode, stubNode);
		}, this);

		dojo.forEach(providerCatalogRefs, function(catalogRef){
			var serviceProviderCatalogNode = {};
			var stubNode = {};
			UIHelper.configureServiceProviderCatalogTreeNode(serviceProviderCatalogNode, stubNode, this.messages.AddProjectLinkDialog_NoServiceProvidersSpecified, serviceProviderCatalog, serviceProviderCatalogTagName, catalogRef, this.messages.AddProjectLinkDialog_NestedCatalogName);
			parseServiceProviderCatalogs(serviceProviderCatalogNode, stubNode);
		}, this);
		
		// Local function to parse all the service providers to build UI.
		var buildServiceProviderUI = dojo.hitch(this, function(){
			this._arrayOfAllItems[parentServiceProviderCatalogTreeNode[this._itemNumPropName]] = parentServiceProviderCatalogTreeNode;
			if (serviceProviders.length > 0) {
				serviceProviders.sort(function(provider1, provider2){
					return com.ibm.team.process.web.client.internal.ProcessClient.stringCompare(provider1.name[0], provider2.name[0]);
				});
			} else {
				serviceProviders = [];
				var serviceProviderTreeNode = {};
				serviceProviderTreeNode.name = [this.messages.DeferredLoadReadStore_NoServiceProviders];
				serviceProviderTreeNode.id = [this.messages.DeferredLoadReadStore_NoServiceProviders + new Date().getTime()];
				serviceProviderTreeNode.type = [ProcessConstants.SERVICE_PROVIDER_ERROR_TYPE];
				serviceProviders.push(serviceProviderTreeNode);
			}
			if (keywordArgs.onItem) {
				var scope = keywordArgs.scope ? keywordArgs.scope : dojo.global;
				keywordArgs.onItem.call(scope, parentServiceProviderCatalogTreeNode);
			}
		});
		
		// If there are no external service providers, build the UI immediately.
		// Else, get all the external providers and then build the UI.
		if (providerRefs.length === 0) {
			buildServiceProviderUI();
		} else {
			var deferreds = [];
			dojo.forEach(providerRefs, dojo.hitch(this, function(serviceProviderRef){
				var header = {
					"accept": ProcessConstants.CONTENT_TYPE_APPLICATION_XML //$NON-NLS-1$
				};
				if (this._oslcCoreEnabled) {
					header[ProcessConstants.OSLC_VERSION_HEADER] = ProcessConstants.OSLC_CORE_VERSION;
				}
				var deferred = jazz.client.xhrGet({
					url: serviceProviderRef,
					headers: header,
					handleAs: "xml", //$NON-NLS-1$
					load: dojo.hitch(this, function(serviceProviderXML){
						var rootRDF = serviceProviderXML.documentElement;
						var children = rootRDF.childNodes;
						var serviceProvider = UIHelper.getNode(children, "ServiceProvider", ProcessConstants.NAMESPACE_OSLC_CORE); //$NON-NLS-1$
						var serviceProviderTreeNode;
						if (firstChild) {
							serviceProviderTreeNode = stubItem;
						} else {
							serviceProviderTreeNode = {};
						}
						UIHelper.configureServiceProviderTreeNode(serviceProviderTreeNode, serviceProvider, oslcSpecVersion, serviceProviderCatalogTagName);
						if (!firstChild) {
							parentServiceProviderCatalogTreeNode.children.push(serviceProviderTreeNode);
							this._arrayOfAllItems.push(serviceProviderTreeNode);
							serviceProviderTreeNode[this._storeRefPropName] = this;
							serviceProviderTreeNode[this._itemNumPropName] = (this._arrayOfAllItems.length - 1); //Last one pushed in is the new item
						} else {
							firstChild = false;
						}
					}),
					error: dojo.hitch(this, function(error){
						item.name = [this.messages.DeferredLoadReadStore_Error];
						item.type = ProcessConstants.SERVICE_PROVIDER_ERROR_TYPE;
						//the default error handler dumps to the console
						if (keywordArgs.onError) {
							var scope = keywordArgs.scope ? keywordArgs.scope : dojo.global;
							keywordArgs.onError.call(scope, error);
						}
						if (keywordArgs.onItem) {
							scope = keywordArgs.scope ? keywordArgs.scope : dojo.global;
							keywordArgs.onItem.call(scope, serviceProviderCatalogNode);
						}
						return error;
					})
				});
				deferreds.push(deferred);
			}));
			var deferredList = new dojo.DeferredList(deferreds);
			deferredList.addCallback(function(){
				buildServiceProviderUI();
			});
		}
	}
});
