/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.IterationDialog"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dijit.form.Button"); //$NON-NLS-1$
dojo.require("dijit.form.TextBox"); //$NON-NLS-1$
dojo.require("dijit.form.FilteringSelect"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dojox.xml.parser"); //$NON-NLS-1$
dojo.require("dojo.data.ItemFileWriteStore"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper");//$NON-NLS-1$
dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.DateTimeTextBox"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$

var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
(function(){
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.IterationTypesSelect", [dijit.form.FilteringSelect], { //$NON-NLS-1$
		tabIndex: 1000,
		isValid: function(){
			if (this.attr('displayedValue') == ProcessConstants.EMPTY) { //$NON-NLS-1$
				return false;
			}
			return this.inherited(arguments);
		},
		validate: function(/*Boolean*/isFocused){
			var isValid = this.inherited(arguments);
			this.dialog._validateData();
			return isValid;
		}
		
	});
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.IterationDialog", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/IterationDialog.html"), //$NON-NLS-1$ //$NON-NLS-2$
		widgetsInTemplate: true,
		
		constructor: function(args){
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		postCreate: function(){
			this.iterationTypesSelect = new com.ibm.team.process.web.ui.internal.admin.projects.IterationTypesSelect({
				store: this._createIterationTypesStore(),
				dialog: this
			});
			this.iterationTypesCell.appendChild(this.iterationTypesSelect.domNode);
			
			this.inputFields = [this.identifier, this.displayName, this.startDate, this.endDate];
			this.identifier.attr('intermediateChanges', true); //$NON-NLS-1$
			var self = this;
			this.identifier.onChange = function(newValue){
				self._validateData();
			};
			this.startDate.attr('intermediateChanges', true); //$NON-NLS-1$
			this.startDate.onChange = function(newValue){
				self._validateData();
				
			};
			this.endDate.attr('intermediateChanges', true); //$NON-NLS-1$
			this.endDate.onChange = function(newValue){
				self._validateData();
				
			};
			
			this._populateDialog();
		},
		
		startup: function(){
			com.ibm.team.process.web.ui.internal.admin.projects.UIHelper.resizeI18NButtons(this.dialogFooter, [this.finishButton, this.cancelButton]);
			this.inherited(arguments);
		},
		
		_validateData: function(){
			if (this._isValidIdentifier() && this._isValidDateRange() && this.iterationTypesSelect.isValid()) {
				this.finishButton.disabled = false;
			} else {
				this.finishButton.disabled = true;
			}
			
		},
		
		_isValidDateRange: function(){
			var startDate = this.startDate.attr('value'); //$NON-NLS-1$
			var endDate = this.endDate.attr('value'); //$NON-NLS-1$
			if (startDate !== null && !this.startDate.isValid()) {
				return false; // Start date must contain a valid date
			}
			if (endDate === null) {
				return true; // Always ok if there is no end date
			}
			if (startDate === null && endDate !== null) {
				return false; // Can't have end date without start date
			}
			
			return endDate >= startDate; // End date must be greater than or equal to the start date
		},
		
		_isValidIdentifier: function(){
			var identifier = this.identifier.attr('value'); //$NON-NLS-1$
			return identifier && dojo.string.trim(identifier).length > 0;
		},
		
		_populateDialog: function(){
			if (!this._iterationHasIterationType()) {
				this.iterationTypesSelect.attr("displayedValue", this.messages.IterationDialog_IterationTypeNone); //$NON-NLS-1$
			} else {
				var select = this.iterationTypesSelect;
				select.store.fetchItemByIdentity({
					identity: this.iterationTypeItemId,
					onItem: function(item, request){
						var iterationName = select.store.getValue(item, "name"); //$NON-NLS-1$
						select.attr("displayedValue", iterationName); //$NON-NLS-1$
					}
				});
				
			}
			this.scheduledReleaseCheckbox.checked = this.hasDeliverable;
			for (var i = 0; i < this.inputFields.length; i++) {
				this.inputFields[i].attr('value', this.initialValues[i]); //$NON-NLS-1$
			}
		},
		
		_iterationHasIterationType: function(){
			if (this.isNewIteration) {
				return false;
			} else if (this.iterationTypeItemId) {
				var iterationType = null;
				this.iterationTypesSelect.store.fetchItemByIdentity({
					identity: this.iterationTypeItemId,
					onItem: function(item, request){
						iterationType = item;
					}
				});
				return iterationType === null ? false : true;
			}
			return false;
		},
		
		changesPending: function(){
			if (this.isNewIteration === true) {
				return true;
			}
			
			for (var i = 0; i < this.inputFields.length; i++) {
				if (this._compare(this.inputFields[i].attr("value"), this.initialValues[i]) !== 0) { //$NON-NLS-1$
					return true;
				}
			}
			
			if (this.hasDeliverable !== this.scheduledReleaseCheckbox.checked) {
				return true;
			}
			
			if (this.iterationTypeItemId !== this.getSelectedIterationTypeId()) {
				return true;
			}
			return false;
		},
		
		getSelectedIterationTypeId: function(){
			var store = this.iterationTypesSelect.store;
			return store.getIdentity(this.iterationTypesSelect.item);
		},
		
		_compare: function(newValue, oldValue){
			if (newValue > oldValue) {
				return 1;
			} else if (newValue < oldValue) {
				return -1;
			} else {
				return 0;
			}
		},
		
		_createIterationTypesStore: function(){
			var iterationTypeItems = [];
			this.timelineStore.fetch({
				query: {
					enabledContext: ProcessConstants.PermissionContext.ITERATION_TYPE
				},
				queryOptions: {
					deep: true
				},
				onComplete: dojo.hitch(this.timelineStore, function(items){
					iterationTypeItems = items;
				})
			});
			
			iterationTypeItems.sort(dojo.hitch(this.timelineStore, function(iterationTypeItem1, iterationTypeItem2){
				var name1 = this.getValue(iterationTypeItem1, "name"); //$NON-NLS-1$
				var name2 = this.getValue(iterationTypeItem2, "name"); //$NON-NLS-1$
				return name1.localeCompare(name2);
			}));
			
			var jsonData = {
				data: {
					identifier: 'id', //$NON-NLS-1$
					label: 'name', //$NON-NLS-1$
					items: []
				}
			};
			var iterationTypesStore = new dojo.data.ItemFileWriteStore(jsonData);
			for (var i = 0; i < iterationTypeItems.length; i++) {
				iterationTypesStore.newItem({
					id: this.timelineStore.getIdentity(iterationTypeItems[i]),
					name: this._getIterationTypeLabel(iterationTypeItems[i]) 
				});
			}
			
			// Add none last in the list
			this._noneItem = iterationTypesStore.newItem({
				id: com.ibm.team.process.web.common.internal.ProcessConstants.ITERATION_TYPE_NONE,
				name: this.messages.IterationDialog_IterationTypeNone
			});
			return iterationTypesStore;
			
		},
		
		_getIterationTypeLabel: function(iterationTypeItem) {
			var label = this.timelineStore.getValue(iterationTypeItem, "name");//$NON-NLS-1$
			if (this.timelineStore.isArchivedItem(iterationTypeItem)) {
				label = dojo.string.substitute(this.messages.IterationDialog_IterationTypeArchivedLabel, [label]);
			}
			return label;
		},
		
		close: function() {
			// Workitem 110877..on IE 7 the change handlers are being called
			// after the controls have been destroyed.  Make them do nothing.
			var emptyChangeHandler = function(newValue){
				// Intentionally do nothing
			};
			this.identifier.onChange = this.startDate.onChange = this.endDate.onChange = emptyChangeHandler
			
		}
		
		
	});
	
})();
