/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsPerRoleEditor"); //$NON-NLS-1$
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dijit._Templated"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("dojox.xml.parser"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.ProcessTree"); //$NON-NLS-1$
	dojo.require("dojo.data.ItemFileWriteStore"); //$NON-NLS-1$
	dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$
	dojo.require("com.ibm.team.repository.web.client.init.InitializationData"); //$NON-NLS-1$
	dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.client.projectarea"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.client.internal.ProcessClient"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.require("jazz.ui.Dialog"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	var PermissionsPerRoleEditor = com.ibm.team.process.web.ui.internal.admin.projects.PermissionsPerRoleEditor;
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsPerRoleEditor", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/PermissionsPerRoleEditor.html"), //$NON-NLS-1$ //$NON-NLS-2$
		
		constructor: function(args){
			this.roles = [];
			
			this.fStore=  {
				identifier: ProcessConstants.PermissionDTO.ID,
				label: ProcessConstants.PermissionDTO.NAME,
				items: []
			};
			this.store = args.store;
			this.roles = args.roles;
			this.processItemScopeId = args.processItemScopeId;
			this.editor = args.editor;
			var processItemScope;
			this.store.fetchItemByIdentity({
				identity: args.processItemScopeId,
				onItem: function(item, request){
					processItemScope = item;
				}
			});
			this.processItemScope = processItemScope;
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			this.permissionTrees = {};
			this.addedRoles = args.addedRoles;
			this.removedRoles = args.removedRoles;
			this._owningApplicationKey = args.owningApplicationKey;
		},
		
		
		postCreate: function(){
			dojo.toggleClass(this.finalCheckBoxDiv, "hidden", false); //$NON-NLS-1$
			this.connect(this.finalCheckBox, "onclick", "_onFinalClick"); //$NON-NLS-1$ //$NON-NLS-2$
			if (this._getProcessItemScopeValue("overrideAllowed") === true && this._getProcessItemScopeValue("enabledContext") > ProcessConstants.PermissionContext.PROJECT) { //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ 
				dojo.toggleClass(this.overrideButton, "hidden", false); //$NON-NLS-1$
				this.connect(this.overrideButton, "onclick", "_onOverrideClick"); //$NON-NLS-1$ //$NON-NLS-2$
			}
			this.connect(this.permissionDeniedTextArea, "onkeyup", "_onPermissionDeniedChange"); //$NON-NLS-1$ //$NON-NLS-2$
			this._createRoles();
			
			this.rolesList.disabled = true;
			var srh = new ServiceResponseHandler(this, "_populatePermissionExtensions", "handlePermissionExtensionsFetchError"); //$NON-NLS-1$ //$NON-NLS-2$
			ProcessClient.getPermissionExtensions({
				context: this._getProcessItemScopeValue("context"), //$NON-NLS-1$
				processAreaItemId: this.editor.itemId,
				owningApplicationKey: this._owningApplicationKey
			}, srh);
		},
		
		refresh: function(){
			this._refresh(false);
		},
		
		refreshRoles: function(){
			this._refresh(true);
		},
		
		_refresh: function(refreshRoles){
			this.roles = this.editor.getAllProcessRoles();
			var selectedIndex = this.rolesList.selectedIndex;
			if (selectedIndex == -1) {
				selectedIndex = 0;
			}
			this._clearRoles();
			if (!refreshRoles) {
				this.permissionTrees = {};
			}
			this._createRoles();
			if (selectedIndex < this.rolesList.options.length) {
				this.rolesList.selectedIndex = selectedIndex; //restore the previously selected index
			} else {
				this.rolesList.selectedIndex = 0;
			}
			if (refreshRoles) {
				//clear out any permission trees for deleted roles
				var permissionTrees = [];
				dojo.forEach(this.roles, function(role){
					var permissionTree = this.permissionTrees[role.id];
					if (permissionTree) {
						permissionTrees[role.id] = permissionTree;
					}
				}, this);
				
				this.permissionTrees = permissionTrees;
			}
			this.updateForRolesListSelectionChange();
		},
		
		_clearRoles: function(){
			this.rolesList.selectedIndex = -1;
			this.rolesList.options.length = 0;
		},
		
		_createRoles: function(){
			this.populateRolesSelect();
			if (this.rolesList.options.length > 0) {
				this.rolesList.options[0].selected = true;
				this.connect(this.rolesList, "onchange", "updateForRolesListSelectionChange"); //$NON-NLS-1$ //$NON-NLS-2$
			}
		},
		
		updateForRolesListSelectionChange: function(){
			this.finalCheckBox.checked = false;
			this._disableFinalCheckBox(true);
			this.permissionDeniedTextArea.value = ""; //$NON-NLS-1$
			this._disableDenialExplanation(true);
			this.overrideButton.disabled = true;
			if (this._createPermissionsEditTree(this._getRoleId())) {
				this._populatePermissions(this._getRoleId());
			}
			
		},
		
		populateRolesSelect: function(){
			dojo.forEach(this.roles, function(role){
				var anOption = new Option(this._getRoleLabel(role), role.id);
				//this.rolesList.appendChild(anOption); does not work on IE7
				var optionArr = this.rolesList.options;
				optionArr[optionArr.length] = anOption;
			}, this);
		},
		
		_getRoleLabel: function(role){
			if (role.id == ProcessConstants.DEFAULT_ROLE_ID) {
				return dojo.string.substitute(this.messages.PermissionsPerRoleEditor_DefaultRoleIdLabel, [role.label]);
			} else if (role.label && dojo.string.trim(role.label) !== 0) {
				return role.label;
			} else {
				return role.id;
			}
			
		},
		
		_getRoleId: function(){
			return this.rolesList.options[this.rolesList.selectedIndex].value;
		},
		
		_getProcessItemScopeValue: function(attribute){
			return this.store.getValue(this.processItemScope, attribute);
		},
		
		_populatePermissionExtensions: function(data){
			this.fStore.items = data;
			data.sort(function(a, b){
				return ProcessClient.stringCompare(a.name, b.name);
			});
			for (var i = 0; i < data.length; i++) {
				var category = data[i];
				category.selected = false;
				category.configured = false;
				if (category.items && category.items.length > 0) {
					category.items.sort(function(a, b){
						return ProcessClient.stringCompare(a.name, b.name);
					});
					for (var j = 0; j < category.items.length; j++) {
						var operation = category.items[j];
						operation[ProcessConstants.PermissionConfigurationDTO.FINAL_OP] = false;
						operation[ProcessConstants.PermissionConfigurationDTO.DENIAL_EXPLANATION] = ""; //$NON-NLS-1$
						this._processOperation(operation, category);
					}
				}
			}
			this._createPermissionsEditTree(this._getRoleId());
			this._populatePermissions(this._getRoleId());
		},
		
		_processOperation: function(operation, category){
			operation.selected = false;
			operation.configured = false;
			operation.id = category.id + ProcessConstants.PERMISSION_PATH_SEPARATOR + operation.id;
			if (operation.items && operation.items.length > 0) {
				operation.items.sort(function(a, b){
					return ProcessClient.stringCompare(a.name, b.name);
				});
				for (var i = 0; i < operation.items.length; i++) {
					this._processOperation(operation.items[i], category);
				}
			}
		},
		
		handlePermissionExtensionsFetchError: function(error){
			this.editor.handleServerResponseError(error, this.messages.PermissionsPerRoleEditor_ErrorFetchingPermissionExtensions);
			this.loadingCompleteCallback();
		},
		
		
		_populatePermissions: function(roleId){
			if (roleId) {
				this.rolesList.disabled = true;
				var args = {
					processAreaItemId: this.editor.itemId,
					processAreaContext: this.editor.context,
					context: this._getProcessItemScopeValue("enabledContext"), //$NON-NLS-1$
					contextId: this._getProcessItemScopeValue("contextId"), //$NON-NLS-1$
					roleId: roleId,
					owningApplicationKey: this._owningApplicationKey
				};
				var srh = new ServiceResponseHandler(this, "_processEnabledPermissions", "_handlePermissionsFetchError"); //$NON-NLS-1$ //$NON-NLS-2$
				ProcessClient.getPermissions(args, srh);
			} else {
				this.loadingCompleteCallback();
			}
		},
		
		_handlePermissionsFetchError: function(error){
			this.editor.handleServerResponseError(error, this.messages.PermissionsPerRoleEditor_ErrorFetchingPermissions);
			this.rolesList.disabled = false;
			this.permissionTrees[this._getRoleId()].setDisabled(false);
			this.loadingCompleteCallback();
			this.editor.stopLoading();
		},
		
		_processEnabledPermissions: function(actions){
			if (actions && actions.length > 0) {
				var self = this;
				
				dojo.forEach(actions, function(action){
				
					self._permissionsTree.model.store.fetch({
						query: {
							id: "*" + action.id //$NON-NLS-1$
						},
						queryOptions: {
							deep: true
						},
						onItem: function(item, request){
							// Only operations support the final attribute
							if (self._permissionsTree.model.store.hasAttribute(item, ProcessConstants.PermissionConfigurationDTO.FINAL_OP)) {
								self._permissionsTree.model.store.setValue(item, ProcessConstants.PermissionConfigurationDTO.FINAL_OP, action.finalOp);
							}
							// Only operations support the denial explanation attribute
							if (self._permissionsTree.model.store.hasAttribute(item, ProcessConstants.PermissionConfigurationDTO.DENIAL_EXPLANATION)) {
								self._permissionsTree.model.store.setValue(item, ProcessConstants.PermissionConfigurationDTO.DENIAL_EXPLANATION, action.denialExplanation);
							}
							self._permissionsTree._selectItem(item, action.enabled, true);
						}
					});
					
				});
			}
			this._permissionsTree.model.store.save();
			this.rolesList.disabled = false;
			this.permissionTrees[this._getRoleId()].setDisabled(false);
			this.loadingCompleteCallback();
			this.editor.stopLoading();
		},
		
		_disableFinalCheckBox: function(disable){
			this.finalCheckBox.disabled = disable;
			dojo.toggleClass(this.finalCheckBoxSpan, "disabled", disable); //$NON-NLS-1$
		},
		
		_disableDenialExplanation: function(disable){
			this.permissionDeniedTextArea.disabled = disable;
		},
		
		_onFinalClick: function(event){
			var focusedNode = this._permissionsTree.lastFocused;
			if (focusedNode) {
				this._permissionsTree.model.store.setValue(focusedNode.item, ProcessConstants.PermissionConfigurationDTO.CONFIGURED, true);
				this._permissionsTree.model.store.setValue(focusedNode.item, ProcessConstants.PermissionConfigurationDTO.FINAL_OP, this.finalCheckBox.checked);
				this.overrideButton.disabled = false;
				this.editor.setDirty(this.editor.changesPending());
				this._permissionsTree.highlightNode(focusedNode);
			}
		},
		
		_onOverrideClick: function(event){
			var focusedNode = this._permissionsTree.lastFocused;
			if (focusedNode) {
				this._permissionsTree._selectItem(focusedNode.item, false, false);
				this.finalCheckBox.checked = false;
				this._permissionsTree.model.store.setValue(focusedNode.item, ProcessConstants.PermissionConfigurationDTO.FINAL_OP, false);
				this._permissionsTree.model.store.setValue(focusedNode.item, ProcessConstants.PermissionConfigurationDTO.DENIAL_EXPLANATION, ""); //$NON-NLS-1$
				this._permissionsTree.model.store.setValue(focusedNode.item, "overrideCleared", true); //$NON-NLS-1$
				this.permissionDeniedTextArea.value = ""; //$NON-NLS-1$
				this.overrideButton.disabled = true;
				this.editor.setDirty(this.editor.changesPending());
				this._permissionsTree.highlightNode(focusedNode);
			}
		},
		
		_onPermissionDeniedChange: function(event){
			var focusedNode = this._permissionsTree.lastFocused;
			if (focusedNode) {
				var permissionDeniedExplanation = this._permissionsTree.model.store.getValue(focusedNode.item, ProcessConstants.PermissionConfigurationDTO.DENIAL_EXPLANATION);
				if (!permissionDeniedExplanation) {
					// If the permission denied explanation is not set in the store
					// make an empty string so we can do a valid comparison below
					permissionDeniedExplanation = ProcessConstants.EMPTY;
				}
				if (permissionDeniedExplanation !== event.target.value) {
					this._permissionsTree.model.store.setValue(focusedNode.item, ProcessConstants.PermissionConfigurationDTO.CONFIGURED, true);
					this._permissionsTree.model.store.setValue(focusedNode.item, ProcessConstants.PermissionConfigurationDTO.DENIAL_EXPLANATION, event.target.value);
					this.editor.setDirty(this.editor.changesPending());
					this._permissionsTree.highlightNode(focusedNode);
				}
			}
		},
		
		_createPermissionsEditTree: function(roleId){
			var treeCreated = false;
			dojox.xml.parser.removeChildren(this.permittedActionsTree);
			if (!this.permissionTrees[roleId]) {
				treeCreated = true;
				this.editor.startLoading(this.messages.PermissionsPerRoleEditor_LoadingPermissions);
				
				var aStore = dojo.clone(this.fStore);
				var stores = new dojo.data.ItemFileWriteStore({
					data: aStore
				});
				var model = new dijit.tree.ForestStoreModel({
					childrenAttrs: ["items"], //$NON-NLS-1$
					store: stores
				});
				var self = this;
				this.permissionTrees[roleId] = new com.ibm.team.process.web.ui.internal.admin.projects.ProcessTree({
					disabled: true,
					model: model,
					showRoot: false,
					labelAttr: ProcessConstants.PermissionDTO.NAME,
					childrenAttr: [ProcessConstants.PermissionDTO.ITEMS],
					getIconClass: function(item){
						if (item && !item.root) {
							if (this.model.store.hasAttribute(item, ProcessConstants.PermissionConfigurationDTO.SELECTED)) {
								var selected = this.model.store.getValue(item, ProcessConstants.PermissionConfigurationDTO.SELECTED);
								if (selected === true) {
									return "dijitCheckBoxIcon"; //from tundra.css //$NON-NLS-1$
								} else {
									return "dijitCheckBox"; //$NON-NLS-1$
								}
							} else {
								return "partialCheckBoxIcon"; //from ProcessPerRoleEditor.css //$NON-NLS-1$
							}
						}
						return "dijitCheckBox"; //$NON-NLS-1$
					},
					
					getLabelClass: function(item){
						if (item && !item.root) {
							if (this.model.store.hasAttribute(item, ProcessConstants.PermissionConfigurationDTO.CONFIGURED) &&
							this.model.store.getValue(item, ProcessConstants.PermissionConfigurationDTO.CONFIGURED) === true &&
							self._getProcessItemScopeValue("overrideAllowed") === true) { //$NON-NLS-1$ //$NON-NLS-2$
								return "configuredPermission"; //$NON-NLS-1$
							} else {
								return "unconfiguredPermission"; //$NON-NLS-1$
							}
						}
						
					},
					
					_selectItem: function(item, selected, configured){
						this.model.store.setValue(item, ProcessConstants.PermissionConfigurationDTO.SELECTED, selected);
						this.model.store.setValue(item, ProcessConstants.PermissionConfigurationDTO.CONFIGURED, configured);
						this._selectChildren(this.model.store.getValues(item, ProcessConstants.PermissionDTO.ITEMS), selected);
						var parentId = this._getParentId(item);
						this._parentSelectionForChildChange(parentId, selected);
					},
					
					_selectChildren: function(children, select){
						if (children === undefined) {
							return;
						}
						var self = this;
						dojo.forEach(children, function(child){
							self.model.store.setValue(child, ProcessConstants.PermissionConfigurationDTO.SELECTED, select);
							self._selectChildren(self.model.store.getValues(child, ProcessConstants.PermissionDTO.ITEMS), select);
						});
					},
					
					_getParentId: function(item){
						var id = this.model.store.getValue(item, ProcessConstants.PermissionDTO.ID);
						var ids = id.split(ProcessConstants.PERMISSION_PATH_SEPARATOR);
						if (ids.length > 0) {
							ids.pop();
							return ids.join(ProcessConstants.PERMISSION_PATH_SEPARATOR);
						}
						
					},
					
					_parentSelectionForChildChange: function(parentId, toSelected){
						if (parentId === undefined || parentId.length === 0) {
							return;
						}
						var self = this;
						this.model.store.fetchItemByIdentity({
							identity: parentId,
							onItem: function(item, request){
								var children = self.model.store.getValues(item, ProcessConstants.PermissionDTO.ITEMS);
								var selectedChild = toSelected;
								if (dojo.some(children, function(child){
									var selected = self.model.store.getValue(child, ProcessConstants.PermissionConfigurationDTO.SELECTED);
									return (selected != toSelected || selected === undefined);
								})) {
									self.model.store.unsetAttribute(item, ProcessConstants.PermissionConfigurationDTO.SELECTED);
								} else {
									self.model.store.setValue(item, ProcessConstants.PermissionConfigurationDTO.SELECTED, toSelected);
								}
								self._parentSelectionForChildChange(self._getParentId(item), toSelected);
							}
						});
					},
					
					_deselectAll: function(){
						var self = this;
						this.model.store.fetch({
							query: {
								id: "*" //$NON-NLS-1$
							},
							queryOptions: {
								deep: true
							},
							onItem: function(item, request){
								self.model.store.setValue(item, ProcessConstants.PermissionConfigurationDTO.SELECTED, false);
								self.model.store.setValue(item, ProcessConstants.PermissionConfigurationDTO.CONFIGURED, false);
								if (self.model.store.hasAttribute(item, ProcessConstants.PermissionConfigurationDTO.FINAL_OP)) {
									self.model.store.setValue(item, ProcessConstants.PermissionConfigurationDTO.FINAL_OP, false);
								}
								if (self.model.store.hasAttribute(item, ProcessConstants.PermissionConfigurationDTO.DENIAL_EXPLANATION)) {
									self.model.store.setValue(item, ProcessConstants.PermissionConfigurationDTO.DENIAL_EXPLANATION, ""); //$NON-NLS-1$
								}
							}
						});
						
					},
					
					_changesPending: function(){
						return this.model.store.isDirty();
					},
					
					focusChanged: function(node){
						if (this.model.store.hasAttribute(node.item, ProcessConstants.PermissionConfigurationDTO.FINAL_OP)) {
							self.finalCheckBox.checked = this.model.store.getValue(node.item, ProcessConstants.PermissionConfigurationDTO.FINAL_OP);
							self._disableFinalCheckBox(false);
						} else {
							self.finalCheckBox.checked = false;
							self._disableFinalCheckBox(true);
						}
						
						var configured = this.model.store.getValue(node.item, ProcessConstants.PermissionConfigurationDTO.CONFIGURED);
						if (self._getProcessItemScopeValue("overrideAllowed") === true && configured === true) { //$NON-NLS-1$ //$NON-NLS-2$
							self.overrideButton.disabled = false;
						} else {
							self.overrideButton.disabled = true;
						}
						
						
						if (this.model.store.hasAttribute(node.item, ProcessConstants.PermissionConfigurationDTO.DENIAL_EXPLANATION)) {
							var permissionDenied = this.model.store.getValue(node.item, ProcessConstants.PermissionConfigurationDTO.DENIAL_EXPLANATION);
							if (permissionDenied) {
								self.permissionDeniedTextArea.value = permissionDenied;
							} else {
								self.permissionDeniedTextArea.value = ""; //$NON-NLS-1$
							}
							self._disableDenialExplanation(false);
						} else {
							self.permissionDeniedTextArea.value = ""; //$NON-NLS-1$
							self._disableDenialExplanation(true);
						}
						
						dojox.xml.parser.removeChildren(self.description);
						
						var description = this.model.store.getValue(node.item, ProcessConstants.PermissionDTO.DESCRIPTION);
						if (description) {
							self.description.appendChild(document.createTextNode(description));
						}
					},
					
					_handleIconSelected: function(node){
						//update the iconNode to emulate a checkbox tree
						//change an attribute in the store to represent the checked state
						var selected = this.model.store.getValue(node.item, ProcessConstants.PermissionConfigurationDTO.SELECTED); //$NON-NLS-1$
						if (selected === undefined || selected === false) {
							this._selectItem(node.item, true, true);
						} else {
							this._selectItem(node.item, false, true);
						}
						self.editor.setDirty(self.editor.changesPending());
						self.overrideButton.disabled = false;
					},
										
					onClicked: function(/*TreeNode*/ nodeWidget, /*Event*/ e){
						var domElement = e.target;
						if (domElement == nodeWidget.iconNode) {
							this._handleIconSelected(nodeWidget);
						}
					},
												
					enterKeyChanged: function(/*Object*/message, /*Event*/ evt){ // from Tree.js
						this._handleIconSelected(message.node);
					},
					
					highlightNode: function(node){
						dojo.addClass(node.labelNode, "dijitTreeLabelFocused"); //$NON-NLS-1$
					}
				});
				
			}
			
			this._permissionsTree = this.permissionTrees[roleId];
			this._permissionsTree.domNode.setAttribute("aria-label", this.messages.PermissionsPerRoleEditorHTML_PermittedActionsHeading) //$NON-NLS-1$
			this.permittedActionsTree.appendChild(this._permissionsTree.domNode);
			return treeCreated;
			
		},
		
		changesPending: function(){
			for (var i = 0; i < this.roles.length; i++) {
				var permissionTree = this.permissionTrees[this.roles[i].id];
				if (permissionTree && permissionTree.model.store.isDirty()) {
					return true;
				}
			}
			return false;
		},
		
		_getActionPaths: function(id){
			var actionPaths = [""]; //$NON-NLS-1$
			var operationId;
			var ids = id.split(ProcessConstants.PERMISSION_PATH_SEPARATOR);
			if (ids.length > 0) {
				ids.shift(); //remove category
				operationId = ids.shift(); //remove operationId
				actionPaths[0] = ids.join(ProcessConstants.PERMISSION_PATH_SEPARATOR); //action path
			}
			var returnValue = {};
			returnValue.actionPaths = actionPaths;
			returnValue.operationId = operationId;
			return returnValue;
		},
		
		saved: function(saveResponse){
			dojo.forEach(this.roles, function(role){
				var permissionTree = this.permissionTrees[role.id];
				if (permissionTree && permissionTree.model.store.isDirty()) {
					permissionTree.model.store.save();
				}
			}, this);
		},
		
		aboutToBeSaved: function(saveObj){
			for (var i = 0; i < this.roles.length; i++) {
				var roleId = this.roles[i].id;
				var permissionTree = this.permissionTrees[roleId];
				if (permissionTree && permissionTree.model.store.isDirty()) {
					var grantedOperationChanges = {}; //operationId to operationChanges
					var revokedOperationChanges = {}; //operationId to array of action paths
					if (this._getProcessItemScopeValue("enabledContext") == ProcessConstants.PermissionContext.PROJECT) { //$NON-NLS-1$
						var jsonProjectConfigurationPermissions = {};
						this.configureChangedPermissionsObjectForTeamAndProjectConfiguration(jsonProjectConfigurationPermissions, grantedOperationChanges, revokedOperationChanges, roleId);
					} else if (this._getProcessItemScopeValue("enabledContext") == ProcessConstants.PermissionContext.TEAM) { //$NON-NLS-1$
						var jsonTeamConfigurationPermissions = {};
						this.configureChangedPermissionsObjectForTeamAndProjectConfiguration(jsonTeamConfigurationPermissions, grantedOperationChanges, revokedOperationChanges, roleId);
					} else if (this._getProcessItemScopeValue("enabledContext") == ProcessConstants.PermissionContext.DEV_LINE) { //$NON-NLS-1$
						var jsonDevelopmentLinePermissions = {};
						this.configureChangedPermissionsObjectForDevLineAndIterationTypes(jsonDevelopmentLinePermissions, grantedOperationChanges, revokedOperationChanges, roleId);
					} else if (this._getProcessItemScopeValue("enabledContext") == ProcessConstants.PermissionContext.ITERATION_TYPE) { //$NON-NLS-1$
						var jsonIterationTypePermissions = {};
						this.configureChangedPermissionsObjectForDevLineAndIterationTypes(jsonIterationTypePermissions, grantedOperationChanges, revokedOperationChanges, roleId);
					} else if (this._getProcessItemScopeValue("enabledContext") == ProcessConstants.PermissionContext.ITERATION) { //$NON-NLS-1$
						var jsonIterationPermissions = {};
						var grantedPermissions = {};
						
						var grantedDevLineToIterationPaths = {};
						var grantedIterationPathsToRoleToOperations = {};
						var rolesToOperations = {};
						rolesToOperations[roleId] = grantedOperationChanges;
						
						var fullPath = this._getProcessItemScopeValue("contextId"); //$NON-NLS-1$
						var ids = fullPath.split(ProcessConstants.PERMISSION_PATH_SEPARATOR);
						if (ids.length > 0) {
							var devLineId = null;
							if (this._fromProjectAreaEditor() === true) {
								devLineId = ids.shift();
							} else {
								devLineId = ids[0];
							}
							var iterationPath = ids.join(ProcessConstants.PERMISSION_PATH_SEPARATOR);
						}
						grantedIterationPathsToRoleToOperations[iterationPath] = rolesToOperations;
						grantedDevLineToIterationPaths[devLineId] = grantedIterationPathsToRoleToOperations;
						
						grantedPermissions.developmentLineToIterationPaths = grantedDevLineToIterationPaths;
						
						var revokedPermissions = {};
						var revokedDevLineToIterationPaths = {};
						var revokedIterationPathsToRoleToPaths = {};
						
						var revokedRolesToOperations = {};
						revokedRolesToOperations[roleId] = revokedOperationChanges;
						
						revokedIterationPathsToRoleToPaths[iterationPath] = revokedRolesToOperations;
						revokedDevLineToIterationPaths[devLineId] = revokedIterationPathsToRoleToPaths;
						
						revokedPermissions.developmentLineToIterationPaths = revokedDevLineToIterationPaths;
						
						jsonIterationPermissions.granted = grantedPermissions;
						jsonIterationPermissions.revoked = revokedPermissions;
					}
					var self = this;
					permissionTree.model.store.fetch({
						query: {
							id: "*" //$NON-NLS-1$
						},
						queryOptions: {
							deep: true
						},
						onItem: function(item, request){
						
							var itemStore = permissionTree.model.store;
							var itemIsNotDirty = !itemStore.isDirty(item);
							var configured = itemStore.getValue(item, ProcessConstants.PermissionConfigurationDTO.CONFIGURED);
							var overrideCleared = itemStore.getValue(item, "overrideCleared"); //$NON-NLS-1$
							if (itemIsNotDirty || (configured !== true && overrideCleared === false)) {
								return;
							}
							var granted = itemStore.getValue(item, ProcessConstants.PermissionConfigurationDTO.SELECTED);
							var id = itemStore.getValue(item, ProcessConstants.PermissionDTO.ID);
							var ids = id.split(ProcessConstants.PERMISSION_PATH_SEPARATOR);
							if (ids.length > 1) {
								if (granted === true) { //item is checked
									self.determineActionPaths(item, grantedOperationChanges, permissionTree);
								} else if (granted === false) { //item is unchecked
									self.determineActionPaths(item, revokedOperationChanges, permissionTree);
								} // else item is partially selected
							} else { //top level grouping element
								var operations = itemStore.getValues(item, ProcessConstants.PermissionDTO.ITEMS);
								dojo.forEach(operations, function(operation){
									if (granted === true) { // item is checked
										self.determineActionPaths(operation, grantedOperationChanges, permissionTree);
									} else if (granted === false) { // item is unchecked
										self.determineActionPaths(operation, revokedOperationChanges, permissionTree);
									} // else item is partially selected
								});
							}
						}
					});
					
					//TODO prune out empty entries....e.g. if only granting, do not marshal revoke
					if (jsonProjectConfigurationPermissions) {
						if (!saveObj.jsonProjectConfigurationPermissions) {
							saveObj.jsonProjectConfigurationPermissions = [];
						}
						saveObj.jsonProjectConfigurationPermissions.push(dojo.toJson(jsonProjectConfigurationPermissions));
					}
					if (jsonTeamConfigurationPermissions) {
						if (!saveObj.jsonTeamConfigurationPermissions) {
							saveObj.jsonTeamConfigurationPermissions = [];
						}
						saveObj.jsonTeamConfigurationPermissions.push(dojo.toJson(jsonTeamConfigurationPermissions));
					}
					if (jsonDevelopmentLinePermissions) {
						if (!saveObj.jsonDevelopmentLinePermissions) {
							saveObj.jsonDevelopmentLinePermissions = [];
						}
						saveObj.jsonDevelopmentLinePermissions.push(dojo.toJson(jsonDevelopmentLinePermissions));
					}
					if (jsonIterationTypePermissions) {
						if (!saveObj.jsonIterationTypePermissions) {
							saveObj.jsonIterationTypePermissions = [];
						}
						saveObj.jsonIterationTypePermissions.push(dojo.toJson(jsonIterationTypePermissions));
					}
					if (jsonIterationPermissions) {
						if (!saveObj.jsonIterationPermissions) {
							saveObj.jsonIterationPermissions = [];
						}
						saveObj.jsonIterationPermissions.push(dojo.toJson(jsonIterationPermissions));
					}
				}
			}
		},
		
		configureChangedPermissionsObjectForDevLineAndIterationTypes: function(changedPermissions, grantedOperationChanges, revokedOperationChanges, roleId){
		
			var grantedPermissions = {};
			var rolesToOperations = {};
			rolesToOperations[roleId] = grantedOperationChanges;
			var grantedIdToRolesToOperations = {};
			grantedIdToRolesToOperations[this._getProcessItemScopeValue("contextId")] = rolesToOperations; //$NON-NLS-1$
			grantedPermissions.idToRolesToOperations = grantedIdToRolesToOperations;
			
			var revokedPermissions = {};
			var revokedRolesToOperations = {};
			revokedRolesToOperations[roleId] = revokedOperationChanges;
			var revokedIdToRolesToOperations = {};
			revokedIdToRolesToOperations[this._getProcessItemScopeValue("contextId")] = revokedRolesToOperations; //$NON-NLS-1$
			revokedPermissions.idToRolesToOperations = revokedIdToRolesToOperations;
			
			changedPermissions.granted = grantedPermissions;
			changedPermissions.revoked = revokedPermissions;
		},
		
		configureChangedPermissionsObjectForTeamAndProjectConfiguration: function(changedPermissions, grantedOperationChanges, revokedOperationChanges, roleId){
		
			var grantedPermissions = {};
			var rolesToOperations = {};
			rolesToOperations[roleId] = grantedOperationChanges;
			grantedPermissions.rolesToOperations = rolesToOperations;
			
			var revokedPermissions = {};
			var revokedRolesToOperations = {};
			revokedRolesToOperations[roleId] = revokedOperationChanges;
			revokedPermissions.rolesToOperations = revokedRolesToOperations;
			
			changedPermissions.granted = grantedPermissions;
			changedPermissions.revoked = revokedPermissions;
		},
		
		determineActionPaths: function(item, allOperationChanges, permissionTree){
			var itemStore = permissionTree.model.store;
			var itemId = itemStore.getValue(item, ProcessConstants.PermissionDTO.ID);
			var actionPaths = null;
			var operationChanges = [];
			var operationChange = {};
			
			var returnValue = this._getActionPaths(itemId);
			var operationId = returnValue.operationId;
			var overrideCleared = itemStore.getValue(item, "overrideCleared"); //$NON-NLS-1$
			if (itemStore.hasAttribute(item, ProcessConstants.PermissionConfigurationDTO.SELECTED)) { //not partially selected
				actionPaths = returnValue.actionPaths;
			}
			if (overrideCleared === true) {
				operationChange.overrideCleared = overrideCleared;
			} else {
				if (itemStore.hasAttribute(item, ProcessConstants.PermissionConfigurationDTO.FINAL_OP)) {
					operationChange["final"] = itemStore.getValue(item, ProcessConstants.PermissionConfigurationDTO.FINAL_OP); //$NON-NLS-1$				
				}
				if (itemStore.hasAttribute(item, ProcessConstants.PermissionConfigurationDTO.DENIAL_EXPLANATION)) {
					operationChange.explanation = itemStore.getValue(item, ProcessConstants.PermissionConfigurationDTO.DENIAL_EXPLANATION);
				}
			}
			
			var alreadyDefinedOperationChanges = allOperationChanges[operationId];
			if (alreadyDefinedOperationChanges) {
				for (var i = 0; i < alreadyDefinedOperationChanges.length; i++) {
					operationChanges.push(alreadyDefinedOperationChanges[i]);
				}
			}
			operationChange.actionPaths = actionPaths;
			
			operationChanges.push(operationChange);
			allOperationChanges[operationId] = operationChanges;
		},
		
		_fromProjectAreaEditor: function(){
			return this.editor.declaredClass == "com.ibm.team.process.web.ui.internal.admin.projects.ProjectAreaEditor"; //$NON-NLS-1$
		},
		
		uninitialize: function(){
			for (var roleEditorId in this.permissionTrees) {
				this.permissionTrees[roleEditorId].destroyRecursive();
				this.permissionTrees[roleEditorId] = null;
			}
			delete this.permissionTrees;
			delete this.editor;
			delete this.roles;
			this.inherited(arguments);
		}
	});
})();
