/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeEditor"); //$NON-NLS-1$
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dijit._Templated"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("dojox.xml.parser"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeTree"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeStore"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsPerRoleEditor"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeModel"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeEditor", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/PermissionsScopeEditor.html"), //$NON-NLS-1$ //$NON-NLS-2$
		constructor: function(args){
			this._processAreaEditor = args.editor;
			this._owningApplicationKey = args.owningApplicationKey;
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		postCreate: function(){
			this._actionsBar = new jazz.ui.SimpleToolbar();
			this._showArchivedButton = this._actionsBar.addButton(this, this._handleShowArchivedClick, {
				label: this.messages.TimelineEditor_ShowArchivedTooltip,
				imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/milestone_archived_obj.gif"), //$NON-NLS-1$ //$NON-NLS-2$
				buttonType: jazz.ui.SimpleToolbar.ButtonTypes.TWO_STATE
			});
			this.actionsBarDiv.appendChild(this._actionsBar.domNode);
			
			this.refresh();
		},
		
		refresh: function(){
			this.clearState();
			
			this.editor.addSaveParticipant(this, "com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeEditor"); //$NON-NLS-1$
			// Create the scope tree
			this._createScopeTree();
			dojox.xml.parser.removeChildren(this.permissionsScopeTree);
			this.permissionsScopeTree.appendChild(this._scopeTree.domNode);
			
			this._updatePermissionSection(dojo.string.substitute(this.messages.PermissionScopeEditor_PermissionSectionIntro, ["<b>", "</b>"])); //$NON-NLS-1$ //$NON-NLS-2$);
		},
		
		clearState: function(){
			this.editor.removeSaveParticipant("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeEditor"); //$NON-NLS-1$
			for (var scopeEditorId in this.permissionScopeCache) {
				this.permissionScopeCache[scopeEditorId].destroyRecursive();
				this.permissionScopeCache[scopeEditorId] = null;
			}
			this.permissionScopeCache = {};
			
			if (this._scopeTree) {
				this._scopeTree.destroyRecursive();
				delete this._scopeTree;
			}
			delete this._currentScopeNode;
		},
		
		_updatePermissionSection: function(message){
			dojox.xml.parser.removeChildren(this._permissionsPerRoleEditor);
			var div = this._createPermissionInfoSection(message);
			this._permissionsPerRoleEditor.appendChild(div);
		},
		
		_createPermissionInfoSection: function(message){
			var div = document.createElement("div"); //$NON-NLS-1$
			dojo.addClass(div, "scopeDescription"); //$NON-NLS-1$
			div.innerHTML = message;
			return div;
		},
		
		_createScopeTree: function(){
			this._model = this._getModel();
			this._updateArchivedQuery();
		
			var self = this;
			this._scopeTree = new com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeTree({
				model: this._model,
				editorContext: self._fromProjectAreaEditor() ? ProcessConstants.EditorContext.PROJECT : ProcessConstants.EditorContext.TEAM,
				focusChanged: function(node){
					self.onPermissions(node.item, node);
				}
			});
			this._scopeTree.domNode.setAttribute("aria-label", this.messages.PermissionsScopeEditorHTML_ConfigurationHeading); //$NON-NLS-1$
		},
		
		_createTimelinesMessage: function(){
			var message = this.messages.PermissionScopeEditor_TimelinesNodeMessage;
			message += this._getRuntimeLookupMessage();
			
			return this._createPermissionInfoSection(message);
			
		},
		
		_createIterationTypesMessage: function(){
			var message = dojo.string.substitute(this.messages.PermissionScopeEditor_IterationTypesNodeMessage, ["<b>", "</b>"]); //$NON-NLS-1$ //$NON-NLS-2$
			message += this._getRuntimeLookupMessage();
			
			return this._createPermissionInfoSection(message);
			
		},
		
		_createIterationsMessage: function(){
			var message = this.messages.PermissionScopeEditor_IterationsNodeMessage;
			message += this._getRuntimeLookupMessage();
			
			return this._createPermissionInfoSection(message);
			
		},
		
		_getRuntimeLookupMessage: function(){
			var listItem = "<li>${0}</li>"; //$NON-NLS-1$
			var message = "<br></br><br></br>"; //$NON-NLS-1$
			message += this.messages.PermissionScopeEditor_RuntimeLookupHeader;
			message += "<ul>"; //$NON-NLS-1$
			message += dojo.string.substitute(listItem, [this.messages.PermissionScopeEditor_RuntimeLookupListItem1]);
			message += dojo.string.substitute(listItem, [this.messages.PermissionScopeEditor_RuntimeLookupListItem2]);
			message += dojo.string.substitute(listItem, [this.messages.PermissionScopeEditor_RuntimeLookupListItem3]);
			message += dojo.string.substitute(listItem, [this.messages.PermissionScopeEditor_RuntimeLookupListItem4]);
			message += "</ul>"; //$NON-NLS-1$
			return message;
		},
		
		_getModel: function(){
			return new com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeModel({
				store: this.editor._getProcessAreaStore(),
				childrenAttrs: ["items"] //$NON-NLS-1$
			});
		},
		
		activateTab: function(){
			for (var scopeEditorId in this.permissionScopeCache) {
				// Refresh the roles anytime the tab activates
				// since the roles are not in the same store
				var permissionEditor = this.permissionScopeCache[scopeEditorId];
				permissionEditor.refreshRoles();
				
				var store = this._scopeTree.model.store;
				var processItemScope;
				store.fetchItemByIdentity({
					identity: store.getIdentity(permissionEditor.processItemScope),
					onItem: function(item, request){
						processItemScope = item;
					}
					
				});
				
				if (!processItemScope) {
					// Must be a new scope that was deleted so
					// delete the permissionEditor for it
					permissionEditor.destroyRecursive();
					delete this.permissionScopeCache[scopeEditorId];
					
					if (this._currentScopeNode && this._currentScopeNode.item === permissionEditor.processItemScope) {
						// The tree was focused on the deleted node
						// Go back to the original intro page
						this._updatePermissionSection(dojo.string.substitute(this.messages.PermissionScopeEditor_PermissionSectionIntro, ["<b>", "</b>"])); //$NON-NLS-1$ //$NON-NLS-2$);
					}
				}
				
			}
			if (this._currentScopeNode && this._scopeTree.model.store.getValue(this._currentScopeNode.item, "archived") && !this._actionsBar.isPressed(this._showArchivedButton)) { //$NON-NLS-1$
				// Tree was focused on node just archived, 
				// and tree is not showing archived
				this._clearFocus();
			}
		},
		
		onPermissions: function(item, node){
			var editor = {
				processItemScope: item,
				changesPending: function(){
					return false;
				},
				saved: function(saveResponse){
				},
				aboutToBeSaved: function(saveObj){
				},
				refreshRoles: function(){
				
				},
				destroyRecursive: function(){
				}
			};
			var id = this._scopeTree.model.getIdentity(item);
			if (!this.permissionScopeCache[id]) {
				dojox.xml.parser.removeChildren(this._permissionsPerRoleEditor);
				if (id === ProcessConstants.PermissionsScopeContext.ITERATION_TYPES) {
					editor.domNode = this._createIterationTypesMessage();
				} else if (id === ProcessConstants.PermissionsScopeContext.DEV_LINES) {
					editor.domNode = this._createTimelinesMessage();
				} else if (id === ProcessConstants.PermissionsScopeContext.ITERATIONS) {
					editor.domNode = this._createIterationsMessage();
				} else {
					this._processAreaEditor.startLoading(this.messages.PermissionScopeEditor_LoadingPermissions);
					this._scopeTree.setDisabled(true);
					editor = new com.ibm.team.process.web.ui.internal.admin.projects.PermissionsPerRoleEditor({
						store: this._scopeTree.model.store,
						roles: this._processAreaEditor.getAllProcessRoles(),
						addedRoles: this._processAreaEditor.addedRoles,
						removedRoles: this._processAreaEditor.removedRoles,
						processItemScopeId: id,
						editor: this._processAreaEditor,
						owningApplicationKey: this._owningApplicationKey,
						loadingCompleteCallback: dojo.hitch(this, this._stopLoading)
					});
				}
				this.permissionScopeCache[id] = editor;
			} else if (this._currentScopeNode !== node) {
				// Only do this if the focused node in the tree changed
				dojox.xml.parser.removeChildren(this._permissionsPerRoleEditor);
				editor = this.permissionScopeCache[id];
			}
			this._currentScopeNode = node;
			if (editor.domNode) {
				this._permissionsPerRoleEditor.appendChild(editor.domNode);
			}
		},
		
		_startLoading: function() {
			this._processAreaEditor.startLoading(this.messages.PermissionScopeEditor_LoadingPermissions);
			this._scopeTree.setDisabled(true);
		},
		
		_stopLoading: function() {
			this._processAreaEditor.stopLoading();
			this._scopeTree.setDisabled(false);
		},
		
		changesPending: function(){
			for (var scopeEditorId in this.permissionScopeCache) {
				if (this.permissionScopeCache[scopeEditorId].changesPending()) {
					return true;
				}
			}
			return false;
		},
		
		saved: function(saveResponse){
			for (var scopeEditorId in this.permissionScopeCache) {
				this.permissionScopeCache[scopeEditorId].saved();
			}
		},
		
		aboutToBeSaved: function(saveObj){
			for (var scopeEditorId in this.permissionScopeCache) {
				this.permissionScopeCache[scopeEditorId].aboutToBeSaved(saveObj);
			}
		},
		
		_fromProjectAreaEditor: function(){
			return this._processAreaEditor.declaredClass == "com.ibm.team.process.web.ui.internal.admin.projects.ProjectAreaEditor"; //$NON-NLS-1$
		},
		
		_handleShowArchivedClick: function(event){
			dojo.stopEvent(event);
			this._updateArchivedQuery();
			this._scopeTree.update();
			if (this._scopeTree.isArchivedItemSelected()) {
				this._clearFocus();
			}
		},
		
		_updateArchivedQuery: function(){
			if (!this._actionsBar.isPressed(this._showArchivedButton)) {
				this._model.query.archived = false;
			} else {
				delete this._model.query.archived;
			}
		},
		
		_clearFocus: function() {
			this._scopeTree.lastFocused.setSelected(false);
			this._scopeTree.lastFocused = null;
			delete this._currentScopeNode;
			this._updatePermissionSection(dojo.string.substitute(this.messages.PermissionScopeEditor_PermissionSectionIntro, ["<b>", "</b>"])); //$NON-NLS-1$ //$NON-NLS-2$);
		},
		
		uninitialize: function(){
			this.editor.removeSaveParticipant("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeEditor"); //$NON-NLS-1$
			for (var scopeEditorId in this.permissionScopeCache) {
				this.permissionScopeCache[scopeEditorId].destroyRecursive();
				this.permissionScopeCache[scopeEditorId] = null;
			}
			
			delete this.permissionScopeCache;
			if (this._scopeTree) {
				this._scopeTree.destroyRecursive();
				delete this._scopeTree;
			}
			delete this._processAreaEditor;
			this.inherited(arguments);
		}
		
	});
})();
