/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

(function(){
	
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeModel"); //$NON-NLS-1$
	dojo.require("dijit.Tree"); //$NON-NLS-1$
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeModel", dijit.tree.ForestStoreModel, { //$NON-NLS-1$
		
		query: {
			archived: false
		},
		
		/*
		 * Constructs the Permissions model
		 * @param {Object} args The args are expected to contain the following:
		 *  {
		 *    store: the store the model should use for it's data
		 *    childrenAttrs: list of attribute names that specify that item's children
		 *  }
		 */
		constructor: function(args){
		},
		
		// Overriden to hide archived children nodes when tree is created/
		// re-created, if hiding is the current setting, and to force
		// requery of root children
		getChildren: function(/*dojo.data.Item*/ parentItem, /*function(items)*/ callback, /*function*/ onError){
			if (parentItem === this.root) {
				delete this.root.children;
			} else if (this.query.archived === false) {
				var store = this.store;
				var oldCallback = callback;
				// only call callback on non-archived items
				callback = function(items){
					items = dojo.filter(items, function(item) {
            			return !store.getValue(item, "archived"); //$NON-NLS-1$
        			});
					oldCallback(items);
				};
			}
			
			this.inherited(arguments);
		},
		
		// Calls getChildren() on all nodes in the tree
		_refreshModel: function(){
			var items = [this.root];
			while (items.length > 0) {
				var item = items.shift();
				var self = this;
				this.getChildren(item, function(newChildren){
					item.children = newChildren;
					self.onChildrenChange(item, newChildren);
				}, null);
				items = items.concat(item.children);
			}
		},
		
		mayHaveChildren: function(item){
			if (this.query.archived === false) {
				return dojo.some(this.store.getValues(item, "items"), function(child){ //$NON-NLS-1$
					return !this.store.getValue(child, "archived"); //$NON-NLS-1$
				}, this);
			} else {
				return this.inherited(arguments);
			}
		},
		
		// Overridden to hide archived nodes and show unarchived nodes
		// immediately after they are changed
		onSetItem: function(item, attribute, oldValue, newValue){
			this.inherited(arguments);
			if (attribute === "archived") { //$NON-NLS-1$
				var parent = this._getParent(item);
				var self = this;
				this.getChildren(parent, function(newChildren){
					parent.children = newChildren;
					self.onChildrenChange(parent, newChildren);
				}, null);
			}
		},
		
		_getParent: function(item){
			return this.store.getParent(item);
		}
	});
	
})();
