/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeTree"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.ProcessTree"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	
	var ContextCSSClass = {
		PROJECT: 'projectConfigIcon', //$NON-NLS-1$
		TEAM: 'teamConfigIcon', //$NON-NLS-1$
		TIMELINES: 'timelinesConfigIcon', //$NON-NLS-1$
		TIMELINE: 'timelineConfigIcon', //$NON-NLS-1$
		TIMELINE_ARCHIVED: 'timelineConfigIconArchived', //$NON-NLS-1$
		PROJECT_TIMELINE: 'projectTimelineConfigIcon', //$NON-NLS-1$
		PROJECT_TIMELINE_ARCHIVED: 'projectTimelineConfigIconArchived', //$NON-NLS-1$
		ITERATION_TYPES: 'iterationTypesConfigIcon', //$NON-NLS-1$
		ITERATION_TYPE: 'iterationTypeConfigIcon', //$NON-NLS-1$
		ITERATION_TYPE_ARCHIVED: 'iterationTypeConfigIconArchived', //$NON-NLS-1$
		ITERATIONS: 'iterationConfigIcon', //$NON-NLS-1$
		ITERATION: 'iterationConfigIcon', //$NON-NLS-1$
		ITERATION_ARCHIVED: 'iterationConfigIconArchived', //$NON-NLS-1$
		CURRENT_ITERATION: 'currentIterationConfigIcon', //$NON-NLS-1$
		CURRENT_ITERATION_ARCHIVED: 'currentIterationConfigIconArchived', //$NON-NLS-1$
		CURRENT_ITERATION_LABEL: 'currentIterationLabel', //$NON-NLS-1$
		COMPLETED_ITERATION: 'completedIterationConfigIcon', //$NON-NLS-1$
		COMPLETED_ITERATION_ARCHIVED: 'completedIterationConfigIconArchived', //$NON-NLS-1$
		PERMISSIONS: 'permissionsConfigIcon' //$NON-NLS-1$
	};
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeTree", com.ibm.team.process.web.ui.internal.admin.projects.ProcessTree, { //$NON-NLS-1$
		labelAttr: 'name', //$NON-NLS-1$
		childrenAttr: ['items'], //$NON-NLS-1$
		showRoot: false,
		persist: false,
		/*
		 * Constructs the Timeline tree instance
		 * @param {Object} args The args are expected to contain the following:
		 *  {
		 *    model: the model the tree should use for it's data
		 *    editorContext: the editor context tree is being used in... a value in ProcessConstants.EditorContext
		 *    onClick: the function to call when a node in the tree is clicked on
		 *  }
		 */
		constructor: function(args){
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		getLabel: function(item){
			var label = this._getDisplayLabel(item);
			if (item.enabledContext == ProcessConstants.PermissionContext.DEV_LINE) {
				if (item.projectTimeline == "true") { //$NON-NLS-1$
					label = dojo.string.substitute(this.messages.PermissionsScopeTree_TimelineDecoratorText, [label, this.messages.PermissionsScopeTree_ProjectTimelineDecoratorText]);
				}
			}
			return label;
		},
		
		_getDisplayLabel: function(item){
			if (item && !item.root) {
				var store = this.model.store;
				var name = dojo.trim(store.getValue(item, "name")); //$NON-NLS-1$
				if (!name || name.length === 0) {
					name = store.getValue(item, "contextId"); //$NON-NLS-1$
					if (store.isIteration(item)) {
						name = this.getIterationId(name);
					}
				}
				return name;
			}
		},
		
		getIterationId: function(contextId){
			var segments = contextId.split(ProcessConstants.PERMISSION_PATH_SEPARATOR);
			return segments[segments.length - 1];
		},
		
		getLabelClass: function(item, opened){
			var store = this.model.store;
			if (item.enabledContext == ProcessConstants.PermissionContext.ITERATION) {
				if (store.hasAttribute(item, "current") && store.getValue(item, "current") === true) { //$NON-NLS-1$ //$NON-NLS-2$
					if (store.hasAttribute(item, "leaf") && store.getValue(item, "leaf") === true) { //$NON-NLS-1$ //$NON-NLS-2$
						return ContextCSSClass.CURRENT_ITERATION_LABEL;
					}
				}
			}
		},
		
		getIconClass: function(item){
			if (item && !item.root) {
				var store = this.model.store;
				var id = this.model.getIdentity(item);
				if (id === ProcessConstants.PermissionsScopeContext.PROJECT) {
					return ContextCSSClass.PROJECT;
				} else if (id === ProcessConstants.PermissionsScopeContext.TEAM) {
					if (this.editorContext === ProcessConstants.EditorContext.PROJECT) {
						return ContextCSSClass.TEAM;
					} else {
						return ContextCSSClass.PERMISSIONS;
					}
				} else if (id === ProcessConstants.PermissionsScopeContext.ITERATION_TYPES) {
					return ContextCSSClass.ITERATION_TYPES;
				} else if (id === ProcessConstants.PermissionsScopeContext.DEV_LINES) {
					return ContextCSSClass.TIMELINES;
				} else if (id === ProcessConstants.PermissionsScopeContext.ITERATIONS) {
					return ContextCSSClass.ITERATION;
				}
				
				if (store.hasAttribute(item, "enabledContext")) { //$NON-NLS-1$
					var context = store.getValue(item, "enabledContext"); //$NON-NLS-1$
					if (context === ProcessConstants.PermissionContext.ITERATION_TYPE) {
						if (store.getValue(item, "archived") === true) { //$NON-NLS-1$
							return ContextCSSClass.ITERATION_TYPE_ARCHIVED;
						} else {
							return ContextCSSClass.ITERATION_TYPE;
						}
					} else if (context === ProcessConstants.PermissionContext.DEV_LINE) {
						if (store.hasAttribute(item, "projectTimeline") && store.getValue(item, "projectTimeline") === true) { //$NON-NLS-1$ //$NON-NLS-2$
							if (store.getValue(item, "archived") === true) {  //$NON-NLS-1$
								return ContextCSSClass.PROJECT_TIMELINE_ARCHIVED;
							} else {
								return ContextCSSClass.PROJECT_TIMELINE;
							}
						} else {
							if (store.getValue(item, "archived") === true) {  //$NON-NLS-1$
								return ContextCSSClass.TIMELINE_ARCHIVED;
							} else {
								return ContextCSSClass.TIMELINE;
							}
						}
					} else if (context === ProcessConstants.PermissionContext.ITERATION) {
						if (store.hasAttribute(item, "current") && store.getValue(item, "current") === true) { //$NON-NLS-1$ //$NON-NLS-2$
							if (store.isArchivedItem(item)) { 
								return ContextCSSClass.CURRENT_ITERATION_ARCHIVED;
							} else {
								return ContextCSSClass.CURRENT_ITERATION;
							}
						} else if (store.hasAttribute(item, "completed") && store.getValue(item, "completed") === true) { //$NON-NLS-1$ //$NON-NLS-2$
							if (store.isArchivedItem(item)) { 
								return ContextCSSClass.COMPLETED_ITERATION_ARCHIVED;
							} else {
								return ContextCSSClass.COMPLETED_ITERATION;
							}
						} else if (store.isArchivedItem(item)) {
							return ContextCSSClass.ITERATION_ARCHIVED;
						} else {
							return ContextCSSClass.ITERATION;
						}
					}
				}
			}
			
		},
		
		focusItem: function(item){
			/* 
			 * According to dojo doc this works if the node is visible
			 * which in our current situation is always true
			 * we may want to think about setting this in the future using the alternate
			 * path setting mechanism the tree now provides which would wait
			 * for any item loading if necessary.  We don't currently use the
			 * the new deferred loading mechanism in the tree though and it would
			 * be some work to convert.
			 */
			this.attr("selectedItem", item); //$NON-NLS-1$
		},
		
		isArchivedItem: function(item){
			return this.model.store.isArchivedItem(item);
		},
		
		isArchivedItemSelected: function(){
			return this.lastFocused && this.model.store.isItem(this.lastFocused.item) && this.isArchivedItem(this.lastFocused.item);
		},
		
		// Refreshes tree without rebuilding, and makes sure that the node
		// selected before is still selected after
		update: function(){
			var beforeFocused = this.lastFocused;

			this.model._refreshModel();
			
			var afterFocused = this.lastFocused;
			if (afterFocused && afterFocused !== beforeFocused) {
				afterFocused.setSelected(false);
				this.lastFocused = beforeFocused;
			}
		},
				
		uninitialize: function(){
			delete this.model;
			this.inherited(arguments);
		}
		
	});
})();
