/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *
 *******************************************************************************/
dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.ProcessProviderDialog"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$

var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;

(function(){
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.ProcessProviderDialog", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/ProcessProviderDialog.html"),//$NON-NLS-1$ //$NON-NLS-2$
		widgetsInTemplate: true,
		
		constructor: function(args){
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages");//$NON-NLS-1$ //$NON-NLS-2$	
			this._owningApplicationKey = args.owningApplicationKey;
			this._projectAreaItemId = args.projectAreaItemId;
		},
		
		postCreate: function(){
			this._setDialogEnabled(false, this.messages.ProcessProviderDialog_Loading);
			this._getProcesses();
		},
		
		startup: function(){
			UIHelper.resizeI18NButtons(this.dialogFooter, [this.okButton, this.cancelButton]);
			this.inherited(arguments);
		},
		
		_setDialogEnabled: function(enable, message){
			this.processSelector.disabled = !enable;
			this.dialogHeading.innerHTML = message;
			this.okButton.disabled = !this._shouldEnableOkButton(enable);
		},
		
		_shouldEnableOkButton: function(enable){
			if (enable) {
				return !this.processSelector.disabled && this.processSelector.selectedIndex > -1;
			}
			return false;
		},
		
		_getProcesses: function(){
			var srh = new ServiceResponseHandler(this, "_populateProcessProviders", "_handleFetchError"); //$NON-NLS-1$ //$NON-NLS-2$
            ProcessClient.getAllProcessProviders({
            	owningApplicationKey: this._owningApplicationKey
            }, srh);
		},
		
		_populateProcessProviders: function(processes){
			if (processes) {
				this._populateOptions(processes);
				this._setDialogEnabled(true, this.messages.ProcessProviderDialog_Prompt);
			} else {
				this._handleNoProcessProviders();
			}
		},
		
		_populateOptions: function(processes){
			var processesToShow = new Array();
			//first get rid of the project area that this provider will provide for (if it is in the collection)
			dojo.forEach(processes, function(process){
				if (process.itemId !== this._projectAreaItemId) {
					processesToShow.push(process);
				}
			}, this);
			
			var newOptions = dojo.map(processesToShow, function(process){
                var option = new Option(process.name, process.itemId);
                option.process = process;
                return option;
			}, this);
			
			newOptions.sort(function(a, b){
				return a.text.localeCompare(b.text);
			});
			
			dojo.forEach(newOptions, function(option){
				if (dojo.isIE) {
					this.processSelector.options[this.processSelector.options.length] = option;
				} else {
					this.processSelector.appendChild(option);
				}
			}, this);
			this.connect(this.processSelector, "onchange", this._handleSelection); //$NON-NLS-1$
		},
		
		_handleSelection: function(){
			this.okButton.disabled = !this._shouldEnableOkButton(true);
			this.selectedProcess = this.processSelector.options[this.processSelector.selectedIndex].process;
		},
		
		_handleNoProcessProviders: function(){
			dojo.toggleClass(this._dialogBody, "hidden", true); //$NON-NLS-1$
			dojo.toggleClass(this.dialogHeading, "hidden", true); //$NON-NLS-1$
			dojo.toggleClass(this.okButton, "hidden", true); //$NON-NLS-1$
			dojo.toggleClass(this._dialogHeadingNoProcessProviders, "hidden", false); //$NON-NLS-1$
		},
		
		_handleFetchError: function(error){
			if (error.message) {
				this._reportErrorFetchingRemote(error.message);
			} else if (error.responseText) {
				this._reportErrorFetchingRemote(error.responseText);
			} else {
				this._reportErrorFetchingRemote(this.messages.ProcessProviderDialog_NoErrorDetails);
			}
		},
		
		_reportErrorFetchingRemote: function(errorMessage, contextHeaderMessage){
			var iconPath = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/images/warning.gif"); //$NON-NLS-1$ //$NON-NLS-2$ 
			var imgHtml = "<img src=\"" + iconPath + "\"\/>"; //$NON-NLS-1$ //$NON-NLS-2$
			var brHtml = "<br />"; //$NON-NLS-1$
			if (!contextHeaderMessage) {
				contextHeaderMessage = this.messages.ProcessProviderDialog_ErrorFetchingRemote;
			}
			
			this.dialogHeading.innerHTML = imgHtml + " " + contextHeaderMessage + brHtml + brHtml + errorMessage; //$NON-NLS-1$
			this.okButton.disabled = true;
		}
		
	});
})();
