/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.ProcessRoleDialog"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dijit.form.Button"); //$NON-NLS-1$
dojo.require("dijit.form.TextBox"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dojox.xml.parser"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper");//$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
(function(){
	var defaultRoleLabel;
	var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
	
	function getSelectedIndices(list){
		var selected = [];
		for (var i = 0; i < list.options.length; i++) {
			if (list.options[i].selected) {
				selected.push(i);
			}
		}
		return selected;
	}
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.ProcessRoleDialog", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/ProcessRoleDialog.html"), //$NON-NLS-1$ //$NON-NLS-2$
		widgetsInTemplate: true,
		
		constructor: function(args){
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			this.allRoles = args.allRoles;
			this.member = args.member;
		},
		
		postCreate: function(){
			this.availableRoles.opposite = this.selectedRoles;
			this.selectedRoles.opposite = this.availableRoles;
			this.populateObj();
			this.connect(this.availableRoles, "onchange", "handleListChange"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this.availableRoles, "ondblclick", "handleAddDoubleClick"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this.selectedRoles, "onchange", "handleListChange"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this.selectedRoles, "ondblclick", "handleRemoveDoubleClick"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this.addButton, "onclick", "handleAdd"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this.removeButton, "onclick", "handleRemove"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this.upButton, "onclick", "handleUp"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this.downButton, "onclick", "handleDown"); //$NON-NLS-1$ //$NON-NLS-2$
			this.updateButtonState();
		},
		
		startup: function(){
			this.resizeAllButtons();
			this.inherited(arguments);
		},
		
		populateObj: function(){
			dojox.xml.parser.removeChildren(this.assignProcessRoles);
			dojox.xml.parser.removeChildren(this.availableRoles);
			dojox.xml.parser.removeChildren(this.selectedRoles);
			this.assignProcessRoles.appendChild(document.createTextNode(dojo.string.substitute(this.messages.ProcessRoleDialog_2, [this.member.name])));
			var note = dojo.string.substitute("<strong>${0}</strong>", [this.messages.ProcessRoleDialog_3]); //$NON-NLS-1$
			this.orderingNote.innerHTML = dojo.string.substitute(this.messages.ProcessRoleDialog_4, [note]);
			this.selectedRoleMap = {};
			for (var i = 0; i < this.member.processRoles.length; i++) {
				this.selectedRoleMap[this.member.processRoles[i].id] = true;
				var option = document.createElement("option"); //$NON-NLS-1$
				option.value = this.member.processRoles[i].id;
				if (this.member.processRoles[i].id == "default") { //$NON-NLS-1$
					// squirrel away the label so we can pass it back to the project area 
					// editor unmodified. 
					this.defaultRoleLabel = this.member.processRoles[i].label;
					option.appendChild(document.createTextNode(dojo.string.substitute(this.messages.ProcessRoleDialog_1, [this.member.processRoles[i].label])));
					option.frozen = true;
				} else {
					option.appendChild(document.createTextNode(this.member.processRoles[i].label));
				}
				option.description = this.member.processRoles[i].description;
				option.setAttribute("aria-describedby", this.id+"_description"); //$NON-NLS-1$ //$NON-NLS-2$
				this.selectedRoles.appendChild(option);
			}
			for (i = 0; i < this.allRoles.length; i++) {
				if (this.selectedRoleMap[this.allRoles[i].id]) {
					continue;
				}
				option = document.createElement("option"); //$NON-NLS-1$
				option.value = this.allRoles[i].id;
				option.appendChild(document.createTextNode(this.allRoles[i].label));
				option.description = this.allRoles[i].description;
				option.setAttribute("aria-describedby", this.id+"_description"); //$NON-NLS-1$ //$NON-NLS-2$
				this.availableRoles.appendChild(option);
			}
		},
		
		handleListChange: function(e){
			for (var i = 0; i < e.target.opposite.options.length; i++) {
				e.target.opposite.options[i].selected = false;
			}
			this.updateButtonState();
			var selectedItems = getSelectedIndices(e.target);
			dojox.xml.parser.removeChildren(this.description);
			if (selectedItems.length == 1) {
				this.description.appendChild(document.createTextNode(e.target.options[selectedItems[0]].description));
			}
		},
		
		handleAddDoubleClick: function(e){
			dojo.stopEvent(e);
			if (dojo.isIE) {
				var select = e.currentTarget;
				if (select.options.selectedIndex != -1) {
					this.handleAdd(e);
				}
			} else if (e.target.tagName.toUpperCase() === "OPTION" && e.target.frozen !== true) { //$NON-NLS-1$
				this.handleAdd(e);
			}		
		},
		
		handleAdd: function(e){
			dojo.stopEvent(e);
			if (dojo.isIE) {
				this.selectedRoles.style.display = "none"; //$NON-NLS-1$
			}
			var indices = getSelectedIndices(this.availableRoles);
			for (var i = indices.length - 1; i >= 0; i--) {
				var option = this.availableRoles.options[indices[i]];
				this.availableRoles.removeChild(option);
				this.selectedRoles.insertBefore(option, this.selectedRoles.options[0]);
			}
			this.updateButtonState();
			if (dojo.isIE) {
				this.selectedRoles.style.display = "block"; //$NON-NLS-1$
			}
		},
		
		handleRemoveDoubleClick: function(e){
			dojo.stopEvent(e);
			if (dojo.isIE) {
				var select = e.currentTarget;
				if (select.options.selectedIndex != -1 && select.options[select.options.selectedIndex].frozen !== true) { //$NON-NLS-1$
					this.handleRemove(e);
				}
			} else if (e.target.tagName.toUpperCase() === "OPTION" && e.target.frozen !== true) { //$NON-NLS-1$
				this.handleRemove(e);
			}
		},
		
		handleRemove: function(e){
			dojo.stopEvent(e);
			if (dojo.isIE) {
				this.availableRoles.style.display = "none"; //$NON-NLS-1$
			}
			var indices = getSelectedIndices(this.selectedRoles);
			for (var i = indices.length - 1; i >= 0; i--) {
				var option = this.selectedRoles.options[indices[i]];
				this.selectedRoles.removeChild(this.selectedRoles.options[indices[i]]);
				this.availableRoles.insertBefore(option, this.availableRoles.options[0]);
			}
			this.updateButtonState();
			if (dojo.isIE) {
				this.availableRoles.style.display = "block"; //$NON-NLS-1$
			}
		},
		
		handleUp: function(e){
			dojo.stopEvent(e);
			if (dojo.isIE) {
				this.selectedRoles.style.display = "none"; //$NON-NLS-1$
			}
			var index = getSelectedIndices(this.selectedRoles)[0];
			var option = this.selectedRoles.options[index];
			this.selectedRoles.removeChild(this.selectedRoles.options[index]);
			this.selectedRoles.insertBefore(option, this.selectedRoles.options[index - 1]);
			this.updateButtonState();
			if (dojo.isIE) {
				this.selectedRoles.style.display = "block"; //$NON-NLS-1$
			}
		},
		
		handleDown: function(e){
			dojo.stopEvent(e);
			if (dojo.isIE) {
				this.selectedRoles.style.display = "none"; //$NON-NLS-1$
			}
			var index = getSelectedIndices(this.selectedRoles)[0];
			var option = this.selectedRoles.options[index];
			this.selectedRoles.removeChild(this.selectedRoles.options[index]);
			this.selectedRoles.insertBefore(option, this.selectedRoles.options[index + 1]);
			this.updateButtonState();
			if (dojo.isIE) {
				this.selectedRoles.style.display = "block"; //$NON-NLS-1$
			}
		},
		
		getChosenRoles: function(e){
			var roles = [];
			for (var i = 0; i < this.selectedRoles.options.length; i++) {
				var option = this.selectedRoles.options[i];
				roles.push({
					//if the role id is default, use the label we stored (without 
					//the other text we added for display)
					label: option.value === "default" ? this.defaultRoleLabel : option.text,//$NON-NLS-1$
					id: option.value,
					description: option.description
				});
			}
			return roles;
		},
		/*
		 * resize the buttons in this dialog
		 */
		resizeAllButtons: function(){
			this.resizeButtons([this.upButton, this.downButton], 0);
			this.resizeButtons([this.finishButton, this.cancelButton], 0);
			this.resizeButtons([this.addButton, this.removeButton], 0);
		},
		
		/*
		 * resize all of the buttons in the dialog to accomodate their labels.
		 * buttons - array of buttons with labels set
		 * defaultWidth - the width of the buttons before this method resizes them.
		 */
		resizeButtons: function(buttons, defaultWidth){
			var words = [];
			for (var i = 0; i < buttons.length; i++) {
				words[i] = buttons[i].innerHTML;
			}
			var newWidth = UIHelper.getMaxStringDisplayWidthInEms(words, this.tester);
			if (newWidth > defaultWidth) {
				dojo.forEach(buttons, function(button){
					UIHelper.styleI18NButton(button, newWidth);
				});
			}
		},
		updateButtonState: function(){
			var availableIndices = getSelectedIndices(this.availableRoles);
			var selectedIndices = getSelectedIndices(this.selectedRoles);
			this.addButton.disabled = availableIndices.length === 0;
			this.removeButton.disabled = selectedIndices.length === 0 || this.selectedRoles.options[this.selectedRoles.options.length - 1].selected;
			if (selectedIndices.length == 1) {
				this.upButton.disabled = selectedIndices[0] === 0 || this.selectedRoles.options[selectedIndices[0]].frozen;
				this.downButton.disabled = selectedIndices[0] == this.selectedRoles.options.length - 2 || this.selectedRoles.options[selectedIndices[0]].frozen;
			} else {
				this.upButton.disabled = true;
				this.downButton.disabled = true;
			}
		}
	});
	
})();
