/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.ProcessRoleEmbeddedEditor"); //$NON-NLS-1$

	dojo.require("dojo.i18n"); //$NON-NLS-1$	
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dijit._Templated"); //$NON-NLS-1$
	dojo.require("jazz.ui.MessageBox"); //$NON-NLS-1$
	dojo.require("jazz.ui.RemoteChooserResponder"); //$NON-NLS-1$
	dojo.require("jazz.client.xhr"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.ProcessRoleDialog"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.client.internal.ProcessClient"); //$NON-NLS-1$
	dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$
	

	dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$	

	var ProcessRoleDialog = com.ibm.team.process.web.ui.internal.admin.projects.ProcessRoleDialog;
	var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	var MessageBox = jazz.ui.MessageBox;

	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.ProcessRoleEmbeddedEditor", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/ProcessRoleEmbeddedEditor.html"), //$NON-NLS-1$ //$NON-NLS-2$
	
		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			
			// This class helps in communication with the consumer of this widget.
			// It communicates when Save and Cancel actions are clicked on the widget.  
			this._responder = new jazz.ui.RemoteChooserResponder();			
			// Get the provided arguments from the query string.
			// Member and Roles resource 'must' be provided to this widget.
			var filterArguments = dojo.queryToObject(window.location.search.slice(1));
			this._memberUrl = filterArguments.memberUrl;
			this._rolesUrl = filterArguments.rolesUrl;		
		},
		
		postCreate: function(){
			this.inherited(arguments);			
			this._messageBox = new MessageBox();
            this._header.appendChild(this._messageBox.domNode);			
			// Member's resource and Roles' resource are mandatory for this widget
			// If not present, throw an error.
			if(!this._memberUrl) {
				var err = new Error(this.messages.ProcessRoleEmbeddedEditor_MemberUrlNotFound);
				this._messageBox.showMessage({
					type: MessageBox.STATUS_TYPES.ERROR,
					errorObject: err
				});
			} else if(!this._rolesUrl){
				var err = new Error(this.messages.ProcessRoleEmbeddedEditor_RolesUrlNotFound);
				this._messageBox.showMessage({
					type: MessageBox.STATUS_TYPES.ERROR,
					errorObject: err
				});
			} else {
				// Gets the member and roles' information from the provided resources
				// and builds the Process Role Editor UI.
				this._getResourcesAndBuildUI();				
			}
		},
		
		_getResourcesAndBuildUI: function () {
			this._getMemberName();
		},
		
		_getMemberName: function() {
			var parms = {};
			parms.searchField = "userId"; //$NON-NLS-1$
			parms.searchTerm = this._memberUrl.substring(this._memberUrl.lastIndexOf("/")+1); //$NON-NLS-1$
			var srh = new ServiceResponseHandler(this, "_getContributorsCallback", "_getContributorsErrorCallback"); //$NON-NLS-1$ //$NON-NLS-2$
			ProcessClient.getContributors(parms, srh); 
		},
		
		_getContributorsCallback: function(resultSet) {
			this.memberName = resultSet.elements[0].name;
			this._getProcessRoles();
		}, 
		
		_getContributorsErrorCallback: function(error) {
			var contribId = this._memberUrl.substring(this._memberUrl.lastIndexOf("/") + 1); //$NON-NLS-1$
			contribId = decodeURIComponent(contribId);
			this._messageBox.showMessage({
				type: MessageBox.STATUS_TYPES.ERROR,
				summary: dojo.string.substitute(this.messages.ProcessRoleEmbeddedEditor_ErrorFetchingContributor, [contribId]),
				errorObject: error
			});
		}, 
		
		_getProcessRoles: function() {
			// Summary: 
			//		Gets the process roles and then gets the member from it's
			// 		callback.
			var callback = dojo.hitch(this, function(roles){
				this._getMember(roles)
			});
			var errBack = dojo.hitch(this, function(error){
				this._messageBox.showMessage({
					type: MessageBox.STATUS_TYPES.ERROR,
					summary: dojo.string.substitute(this.messages.ProcessRoleEmbeddedEditor_ErrorFetchingRoles, [this._rolesUrl]),
					errorObject: error
				});
			}) ;
			ProcessClient.getProcessRoles(this._rolesUrl, callback, errBack);
		},
		
		_getMember: function(/* Array */ roles){
			// Summary:
			// 		Gets the member and builds UI in the callback.
			//		Process roles have already been queried and parsed. 
			var callback = dojo.hitch(this, function(member){
				
				// Get the member's role information from the available all process
				// roles.
				for(var i = 0; i < member.processRoles.length; i++) {
					var processRole = member.processRoles[i];
					for(var k = 0; k < roles.length; k++) {
						var role = roles[k];
						if(processRole.roleUrl === role.url) {
							processRole.id = role.id;
							processRole.label = role.label;
							processRole.description = role.description;
							break;
						}
					}
				}
				member.name = this.memberName;
				// Build the UI.
				var args = {};
				args.allRoles = roles;
				args.member = member;
				this._embedProcessRoleDialog(args);
			});
			var errBack = dojo.hitch(this, function(error){
				this._messageBox.showMessage({
					type: MessageBox.STATUS_TYPES.ERROR,
					summary: dojo.string.substitute(this.messages.ProcessRoleEmbeddedEditor_ErrorFetchingMember, [this._memberUrl]),
					errorObject: error
				});
			}) ;
			ProcessClient.getMember(this._memberUrl, callback, errBack);		
		},
		
		_embedProcessRoleDialog: function(args){
			// Summary:
			//		Instantiates ProcessRoleDialog widget with required information
			//		and embeds it in this Dom node.
			this._processRoleDialog = new ProcessRoleDialog(args);
			this._processRoleDialogDiv.appendChild(this._processRoleDialog.domNode);
			this._processRoleDialog.startup();
			this.connect(this._processRoleDialog.finishButton, "onclick", "_handleSave"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this._processRoleDialog.cancelButton, "onclick", "_handleCancel");//$NON-NLS-1$ //$NON-NLS-2$
		},
		
		_handleSave: function() {
			var success = dojo.hitch(this, function(response) {
				this._messageBox.showMessage({
					type: MessageBox.STATUS_TYPES.OK,
					summary: this.messages.ProcessRoleEmbeddedEditor_SaveSuccess
				});
				
				// Prepare the JSON response and pass it to the consumer.
				var response = this._prepareJsonResponse(this._memberUrl);
				this._responder.sendResponse(response);				
			});
			
			var fail = dojo.hitch(this, function(error) {
				this._messageBox.showMessage({
					type: MessageBox.STATUS_TYPES.ERROR,
					summary: this.messages.ProcessRoleEmbeddedEditor_SaveFailed,
					errorObject: error
				});
			});
			
			var args = {};
			args.chosenRoles = this._processRoleDialog.getChosenRoles();
			args.memberUrl =  this._memberUrl;
			ProcessClient.putProcessRoleAssignments(args,success,fail);
			
		},
		
		_handleCancel: function() {
			// Prepare the JSON response and pass it to the consumer.			
			var response = this._prepareJsonResponse();
			this._responder.sendResponse(response);
		},
		
		_prepareJsonResponse: function(/* String */memberUrl){
			//Summary:
			//	Prepares the JSON response that is passed to the consumer when
			//	user clicks on Save or Cancel.
			//
			//	Save response: {"oslc:results":[{"rdf:resource":"https://127.0.0.1:9444/jazz/process/project-areas/_IwmocC0aEeCnGckhoQd2jw/members/TestJazzUser1"}]}
			//	Cancel response: {"oslc:results":[]}
			var OSLC_RESULTS = "oslc:results"; //$NON-NLS-1$
			var RDF_RESOURCE = "rdf:resource"; //$NON-NLS-1$
			var response = {};
			response[OSLC_RESULTS] = [];			
			if(memberUrl){
				var member = {};
				member[RDF_RESOURCE] = memberUrl;
				response[OSLC_RESULTS][0] = member;
			}
			return dojo.toJson(response);
		},
				
		uninitialize: function() {
            if (this._messageBox) {
            	this._messageBox.destroyRecursive();
            	delete this._messageBox;
            }			
			if (this._processRoleDialog) {
				this._processRoleDialog.destroyRecursive();
				delete this._processRoleDialog;
			}
			this.inherited(arguments);
		}
	
	});
})();
