/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.ProjectAreaEditor"); //$NON-NLS-1$
	dojo.require("jazz.ui.ResourceLink"); //$NON-NLS-1$
	dojo.require("jazz.core.loader"); //$NON-NLS-1$
	dojo.require("jazz.opensocial.Gadget"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.AbstractProcessAreaEditor"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.AccessControlEditor"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.AddProjectLinkDialog"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.TimelineEditor"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper");//$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.ConfigurationDataEditor"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.ProcessProviderDialog"); //$NON-NLS-1$
	dojo.require("jazz.ui.Dialog"); //$NON-NLS-1$
	dojo.require("jazz.client.xhr"); //$NON-NLS-1$
	dojo.require("jazz.util.error"); //$NON-NLS-1$
	dojo.require("dojo.date"); //$NON-NLS-1$
	dojo.require("dojo.date.locale"); //$NON-NLS-1$
	dojo.require("dojox.xml.parser"); //$NON-NLS-1$
	var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
	var ResourceLink = jazz.ui.ResourceLink;
	var ON_CLICK = "onclick"; //$NON-NLS-1$
	var TEAM_AREA_EDITOR = "com.ibm.team.process.editTeamArea"; //$NON-NLS-1$
	var PROJECT_AREA_EDITOR = "com.ibm.team.process.editProjectArea"; //$NON-NLS-1$
	var enterpriseProcessSettings = {
		IS_ENTERPRISE_PROCESS: "is", //$NON-NLS-1$
		USES_ENTERPRISE_PROCESS: "uses", //$NON-NLS-1$
		NONE: "none" //$NON-NLS-1$
	};
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.ProjectAreaEditor", com.ibm.team.process.web.ui.internal.admin.projects.AbstractProcessAreaEditor, { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/ProjectAreaEditor.html"), //$NON-NLS-1$ //$NON-NLS-2$
		initialize: function(args){
			this.projectAreaItemId = args.itemId;
			this.context = ProcessConstants.EditorContext.PROJECT;
			this._projectAreaIconSrc = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/projectAreaIcon.png"); //$NON-NLS-1$ //$NON-NLS-2$
			this._projectAreaIconClass = "itemIcon"; //$NON-NLS-1$
			if (!this.isNew) {
				this._projectAreaIconClass = "itemIcon hideIcon"; //$NON-NLS-1$
			}
			this.allProcessDefinitions = [];
			
			this.accessControl = {};
			this.accessControl.publicVisible = false;
			this.accessControl.memberVisible = true;
			this.accessControl.specificUserVisible = false;
			
			this.projectAreaLinks = [];
			this.existingProjectAreaLinks = [];
			this.newProjectAreaLinks = [];
			this.removedProjectAreaLinks = [];
			
			this.enterpriseProcessResourceLink = null;//used to save the RL to the enterprise process in case it changes.
			
			this._backlinkDeletionWarning = false;
				
			this._expandedIconSrc = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/grey-twistie-open.png").toString(); //$NON-NLS-1$ //$NON-NLS-2$
			this._collapsedIconSrc = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/grey-twistie-closed.png").toString(); //$NON-NLS-1$ //$NON-NLS-2$
			this._breadcrumbIconSrc = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/launch_obj.gif").toString(); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		postCreate: function(){
			this.inherited(arguments);
			this._createLinksTables();
		},
		
		_setProcessAreaIcon: function(isArchived){
			dojo.removeClass(this.projectAreaIcon, "hideIcon"); //$NON-NLS-1$
			if (!isArchived) {
				this.projectAreaIcon.src = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/projectAreaIcon.png"); //$NON-NLS-1$ //$NON-NLS-2$
			} else {
				this.projectAreaIcon.src = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/projectAreaIcon_archived.png"); //$NON-NLS-1$ //$NON-NLS-2$
			}
		},
		
		toggleViewEditMode: function(mode){
			this.inherited(arguments);
		},
		
		_handleGetIsConfigurationDataEnabledSuccess: function(response) {
			response = dojo.fromJson(response);
			if (response.isEnabled) {
				this.createTab(this.messages.ProjectAreaEditor_ConfigurationDataTabLabel, ProcessConstants.TabEnum.CONFIGURATIONDATA, this._createConfigurationData, ProcessConstants.TabEnum.ACCESS_CONTROL, undefined, ProcessConstants.UAKey.CONFIGURATION_DATA_TAB);
			}
		},
		
		_handleGetIsConfigurationDataEnabledFailure: function(response) {
			//Do nothing, NOT create and show the configuration data tab
		},
		
		createCustomTabs: function(){
			this.createTab(this.messages.ProjectAreaEditor_TimelineTabLabel, ProcessConstants.TabEnum.TIMELINE, this._createTimelineEditor, ProcessConstants.TabEnum.ROLES, undefined, ProcessConstants.UAKey.TIMELINES_TAB);
			this.createTab(this.messages.ProjectAreaEditor_AccessControlTabLabel, ProcessConstants.TabEnum.ACCESS_CONTROL, this._createAccessControl, undefined, undefined, ProcessConstants.UAKey.ACCESSCONTROL_TAB);
			if (this._isEmbedded) {
				var srh = new ServiceResponseHandler(this, "_handleGetIsConfigurationDataEnabledSuccess", "_handleGetIsConfigurationDataEnabledFailure"); //$NON-NLS-1$ //$NON-NLS-2$
				ProcessClient.getIsConfigurationDataEnabled(srh);
			}
			this._loadRegistryTabs();
		},
		
		customSave: function(saveObj){
		
			// TODO Don't like this...this is really the job of the access control
			// tab to handle but since it doesn't exist until after we create 
			// the project have to initialize here instead.	
			if (this.isNew) {
				saveObj.publicVisible = false;
				saveObj.memberVisible = true;
				saveObj.specificUserVisible = false;
			}
			
			// Workitem 74324
			// Don't like this either...this is really the job of the access control
			// tab to handle but since the save participant is not added until the
			// the tab is actually clicked on...we have to initialize here.
			if (saveObj.publicVisible === undefined) {
				saveObj.publicVisible = this.accessControl.publicVisible;
				saveObj.memberVisible = this.accessControl.memberVisible;
				saveObj.specificUserVisible = this.accessControl.specificUserVisible;
			}
			
			if (this.linksHaveChanged()) {
				this._postProjectAreaLinks(saveObj);
//				if (!this.isNew) { //cannot clear until we have a chance to post
//					// the back link with the UUID of the new project area
//					this.newProjectAreaLinks = [];
//					this.removedProjectAreaLinks = [];
//				}
			}
			this._saveEnterpriseProcessSetting(saveObj);
		},
		
		_postProjectAreaLinks: function(saveObj){
			var projectAreasLinksUrl = this._getProjectAreaLinksURL();
			if (this.newProjectAreaLinks.length > 0 || this.removedProjectAreaLinks.length > 0) {
				var jsonLinks = {};
			}
			if (this.newProjectAreaLinks.length > 0) {
				var addedLinks = [];
				jsonLinks.add = addedLinks;
			}
			if (this.removedProjectAreaLinks.length > 0) {
				var removedLinks = [];
				jsonLinks.remove = removedLinks;
			}
			var createTrace = this.messages.ProjectAreaEditor_CreatingBackLinks;
			dojo.forEach(this.newProjectAreaLinks, function(projectAreaLink){
				var projectAreaInfo = projectAreaLink.projectAreaInfo;
				var newLink = {};
				newLink.label = projectAreaInfo.label;
				newLink.linkType = projectAreaLink.linkType;
				newLink.targetUrl = projectAreaInfo.targetUrl;
				newLink.targetServicesUrl = projectAreaInfo.servicesUrl;
				if (projectAreaInfo.consumerRegistryUrl) {
					newLink.consumerRegistryUrl = projectAreaInfo.consumerRegistryUrl;
				}
				
				addedLinks.push(newLink);
				
				var backLinkConsumerRegistryUrl = this._getBackLinkConsumerRegistryUrl(projectAreaLink);
				if (backLinkConsumerRegistryUrl && !this.isNew) {
					if (this._isSelfLinking(projectAreaLink)) {
						// add this back link to new link list if the link type is not "related-to"
						//"related-to" is the peer of itself so no need to create the back link if self linking
						if (projectAreaLink.linkType !== ProcessConstants.LinkTypes.RELATED_TO) {
							addedLinks.push(this._getBacklinking(projectAreaLink));
						}
					} else {
						createTrace = "\n\t" + dojo.string.substitute(this.messages.ProjectAreaEditor_CreatingBackLinkOfType, [projectAreaLink.linkType, projectAreaInfo.label]); //$NON-NLS-1$
						this.postBackLink(projectAreaLink, createTrace);
					}
				}
			}, this);
			
			var deleteTrace = this.messages.ProjectAreaEditor_DeletingBackLinks;
			dojo.forEach(this.removedProjectAreaLinks, function(projectAreaLink){
				removedLinks.push(projectAreaLink.linkUrl);
				
				//delete the back link
				var targetUrl = this._getThisTargetUrl();
				var targetConsumerRegistryUrl = this._getBackLinkConsumerRegistryUrl(projectAreaLink);
				if (targetConsumerRegistryUrl) {
					if (this._isSelfLinking(projectAreaLink)) {
						// add this back link to removed link list if the link type is not "related-to"
						//"related-to" is the peer of itself so no need to create the back link if self linking
						if (projectAreaLink.linkType !== ProcessConstants.LinkTypes.RELATED_TO) {
							var deleteSelfBackLinkUrl = this._getSelfBackLinkUrl(projectAreaLink);
							if (deleteSelfBackLinkUrl) {
								removedLinks.push(deleteSelfBackLinkUrl);
							}
						}
					} else {
						var linkType = UIHelper.getPeerLinkType(projectAreaLink.linkType);
						var deleteBackLinkUrl = targetConsumerRegistryUrl + "?type=" + linkType + "&resource=" + targetUrl; //$NON-NLS-1$ //$NON-NLS-2$
						deleteTrace += "\n\t" + dojo.string.substitute(this.messages.ProjectAreaEditor_DeletingBackLinkOfType, [linkType, projectAreaLink.projectAreaInfo.label]); //$NON-NLS-1$
						var deferred = jazz.client.xhrDelete({
							url: deleteBackLinkUrl,
							handleAs: "xml", //$NON-NLS-1$
							load: dojo.hitch(this, function(){
								deleteTrace += "\n\t" + dojo.string.substitute(this.messages.ProjectAreaEditor_DeletedBackLink, [linkType, projectAreaLink.projectAreaInfo.label]); //$NON-NLS-1$
								this.handleServerResponseSuccess();
							}),
							error: dojo.hitch(this, function(error){
								var listing = projectAreaLink.projectAreaInfo.label + "," + linkType; //$NON-NLS-1$
								deleteTrace += "\n\t" + dojo.string.substitute(this.messages.ProjectAreaEditor_ErrorDeletingBackLinkWithReason, [listing]); //$NON-NLS-1$
								this._configureErrorData(error, deleteTrace);
								this._handleDeleteProjectAreaLinkWarning(deferred, error);
								return error;
							})
						});
					}
				}
			}, this);
			saveObj.jsonLinks = dojo.toJson(jsonLinks);
		},
		
		postBackLink: function(projectAreaLink, createTrace){
			var servicesUrl;
			var consumerRegistryUrl;
			// Check if source project area information is available as a 
			// service provider before proceeding.
			// New project area (not saved yet) will not have this information.
			// And in that case, back link can't be created.
			var tagName = UIHelper.getTagNameForLinkType(UIHelper.getPeerLinkType(projectAreaLink.linkType));
			if (projectAreaLink.sourceProjectAreaInfo[tagName] && projectAreaLink.sourceProjectAreaInfo[tagName].presentationUrl) {
				var url = projectAreaLink.sourceProjectAreaInfo[tagName].presentationUrl[0];
				var fullPath = url.split("/"); //$NON-NLS-1$
				if (fullPath.length > 0) {
					if (fullPath.pop() === this.projectAreaItemId) {
						servicesUrl = projectAreaLink.sourceProjectAreaInfo[tagName].servicesUrl[0];
						consumerRegistryUrl = projectAreaLink.sourceProjectAreaInfo[tagName].consumerRegistryUrl[0];
					}
				}
			}
			if (!servicesUrl) {
				var error = {};
				error.message = this.messages.ProjectAreaEditor_NoMatchingProjectAreaForLink;
				this._handlePostProjectAreasLinkWarning(null, error);
				return;
			}
			var backLinkConsumerRegistryUrl = this._getBackLinkConsumerRegistryUrl(projectAreaLink);
			var deferred = jazz.client.rawXhrPost({
				url: backLinkConsumerRegistryUrl,
				contentType: "application/xml; charset=utf-8", //$NON-NLS-1$
				handleAs: "xml", //$NON-NLS-1$
				postData: this._getProjectAreaBackLinkPostData(projectAreaLink, servicesUrl, consumerRegistryUrl),
				load: dojo.hitch(this, this.handleServerResponseSuccess),
				error: dojo.hitch(this, function(error){
					createTrace += "\n\t" + dojo.string.substitute(this.messages.ProjectAreaEditor_FailedToPostBackLink, [projectAreaLink.linkType, backLinkConsumerRegistryUrl]); //$NON-NLS-1$
					this._configureErrorData(error, createTrace);
					this._handlePostProjectAreasLinkWarning(deferred, error);
					return error;
				})
			});
		},
		
		_configureErrorData: function(error, trace){
			error.errorData = {};
			error.errorData.identifier = "com.ibm.team.process.projectLinks"; //$NON-NLS-1$
			error.errorData.internalReportInfos = [];
			error.errorData.internalReportInfos[0] = {};
			error.errorData.internalReportInfos[0].identifier = "com.ibm.team.process.internal.linksException"; //$NON-NLS-1$
			error.errorData.internalReportInfos[0].data = trace;
		},
		
		_handleDeleteProjectAreaLinkWarning: function(deferred, error){
			if (error.message) {
				var errorHeader = this.messages.ProjectAreaEditor_ErrorDeletingBackLink;
				error.message = decodeURIComponent(jazz.util.error.getDefaultXhrErrMsg(deferred));
			} else if (error.responseText) {
				errorHeader = dojo.string.substitute(this.messages.ProjectAreaEditor_ErrorDeletingBackLinkWithReason, [error.responseText]);
			} else {
				var message = error.message;
				error.message = error.trace;
				error.message += ProcessConstants.BR + message;
				errorHeader = this.messages.ProjectAreaEditor_ErrorDeletingBackLink;
			}
			this._backlinkDeletionWarning = true;
			this.handleServerResponseWarning(error, errorHeader);
		},
		
		_getBackLinkConsumerRegistryUrl: function(projectAreaLink){
			return projectAreaLink.projectAreaInfo.consumerRegistryUrl;
		},
		
		_getProjectAreaBackLinkPostData: function(projectAreaLink, servicesUrl, consumerRegistryUrl){
			var label = this._toXmlEncoded(this.name.value);
			var linkType = UIHelper.getPeerLinkType(projectAreaLink.linkType);
			var targetUrl = this._getThisTargetUrl();
			return this._getPostData(linkType, label, targetUrl, servicesUrl, consumerRegistryUrl);
		},
		
		_toXmlEncoded: function(plainText){
			var xmlText = plainText;
			// do & case first to avoid double substitutions
			xmlText = xmlText.replace(/&/g, "&amp;"); //$NON-NLS-1$ 
			xmlText = xmlText.replace(/\"/g, "&quot;"); //$NON-NLS-1$
			// skip &apos;
			xmlText = xmlText.replace(/</g, "&lt;"); //$NON-NLS-1$
			xmlText = xmlText.replace(/>/g, "&gt;"); //$NON-NLS-1$
			return xmlText;
		},
		
		_getPostData: function(linkType, label, targetUrl, servicesUrl, consumerRegistryUrl){
			var postData = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" + //$NON-NLS-1$
			"<jp:link \n" + //$NON-NLS-1$
			"xmlns:rdf=\"" + //$NON-NLS-1$
			ProcessConstants.NAMESPACE_RDF +
			"\"\n" + //$NON-NLS-1$
			"xmlns:jp=\"" + //$NON-NLS-1$
			ProcessConstants.NAMESPACE_JP +
			"\"\n" + //$NON-NLS-1$
			"xmlns:dc=\"" + //$NON-NLS-1$
			ProcessConstants.NAMESPACE_DC +
			"\"\n" + //$NON-NLS-1$ 
			"xmlns:oslc=\"" + //$NON-NLS-1$
			ProcessConstants.NAMESPACE_OSLC +
			"\">\n" + //$NON-NLS-1$
			"<jp:link-type>" + //$NON-NLS-1$
			linkType +
			"</jp:link-type>\n" + //$NON-NLS-1$
			"<oslc:ServiceProvider>\n" + //$NON-NLS-1$
			"<dc:title>" + //$NON-NLS-1$
			label +
			"</dc:title>\n" + //$NON-NLS-1$
			"<oslc:details rdf:resource=\"" + //$NON-NLS-1$
			targetUrl +
			"\"/>\n" + //$NON-NLS-1$
			"<oslc:services rdf:resource=\"" + //$NON-NLS-1$
			servicesUrl +
			"\"/>\n"; //$NON-NLS-1$
			if (consumerRegistryUrl) {
				postData += "<jp:consumerRegistry rdf:resource=\"" + //$NON-NLS-1$
				consumerRegistryUrl +
				"\"/>\n"; //$NON-NLS-1$	
			}
			
			postData += "</oslc:ServiceProvider>\n" + //$NON-NLS-1$
			"</jp:link>"; //$NON-NLS-1$
			return postData;
		},
		
		_getThisTargetUrl: function(){
			var serverAddress = this._getServerUrl();
			return serverAddress + net.jazz.ajax._contextRoot + "/process/project-areas/" + this.projectAreaItemId; //$NON-NLS-1$
		},
		
		_handlePostProjectAreasLinkWarning: function(deferred, error){
			if (error && error.message) {
				var errorHeader = this.messages.ProjectAreaEditor_ErrorCreatingBackLink;
				if (deferred) {
					error.message = decodeURIComponent(jazz.util.error.getDefaultXhrErrMsg(deferred));
				}
			} else if (error && error.responseText) {
				errorHeader = dojo.string.substitute(this.messages.ProjectAreaEditor_ErrorCreatingBackLinkWithReason, [error.responseText]);
			} else {
				errorHeader = this.messages.ProjectAreaEditor_ErrorCreatingBackLink;
			}
			this.handleServerResponseWarning(error, errorHeader);
		},
		
		_createAccessControl: function(){
		
			var editor = new com.ibm.team.process.web.ui.internal.admin.projects.AccessControlEditor({
				editor: this
			});
			
			this.accessControlEditor = editor;
			this.twoColumnMainColumn.appendChild(editor.domNode);
			return editor;
		},
		
		_createTimelineEditor: function(){
		
			var editor = new com.ibm.team.process.web.ui.internal.admin.projects.TimelineEditor({
				editor: this
			});
			
			this.timelineEditor = editor;
			this.twoColumnMainColumn.appendChild(editor.domNode);
			editor.startup();
			return editor;
		},
		
		_createConfigurationData: function(){
			var editor = new com.ibm.team.process.web.ui.internal.admin.projects.ConfigurationDataEditor({
				editor: this,
				owningApplicationKey: this._owningApplicationKey
			});
			this.configurationDataEditor = editor;
			this.twoColumnMainColumn.appendChild(editor.domNode);
			return editor;
		},
		
		getPrimaryFieldLabel: function(){
			return this.messages.ProjectAreaEditor_7;
		},
		
		getFetchErrorMessage: function(error){
			if (error.status == ProcessConstants.SC_FORBIDDEN) {
				return this.messages.ProjectAreaEditor_ErrorNoReadAccess;
			} else {
				return this.messages.ProjectAreaEditor_6;
			}
		},
		
		populateProcessAreaData: function(data){
			var args = arguments;
			UIHelper.runSafe(dojo.hitch(this, function(){
				this.inherited(args);
				
				this.processLocale = data.locale;
				
				this.accessControl = {};
				this.accessControl.publicVisible = data.publicVisible;
				this.accessControl.memberVisible = data.memberVisible;
				this.accessControl.specificUserVisible = data.specificUserVisible;
				
				this.isProcessChanged = false;
				
				// TODO I don't like this....need to load the data
				// independently on that tab otherwise the timing is off
				// old data shows up from the project since this async
				// method may not have ran by the time saved() has been called.
				if (this.accessControlEditor) {
					this.accessControlEditor.refresh();
				}
				// TODO Same as above...need to share the same data otherwise
				// have a potential timing issue in saved().
				if (this.timelineEditor) {
					this.timelineEditor.refresh();
				}
				if (this.permissionEditor) {
					this.permissionEditor.refresh(); //update the roles
				}
				if (this.configurationDataEditor) {
					this.configurationDataEditor.refresh();
				}
				this._cacheEnterpriseProcessSetting(data);
				if (!this.isNew) {
					this.populateEnterpriseProcessSection(data);
				}
				
				this.populateProcessDescription(data.processDTO);
				this.navigateToInitTab();
			}));
		},
		
		clearState: function(refreshClicked){
			this.inherited(arguments);
			this.projectAreaLinks = [];
			this.newProjectAreaLinks = [];
			this.removedProjectAreaLinks = [];
			this.existingProjectAreaLinks = [];
			this._backlinkDeletionWarning = false;
			
			//clear the state enterprise buttons
			var buttons = [this.enterpriseProcessNotUsedButton, this.enterpriseProcessIsButton, this.enterpriseProcessUsesButton];
			for (var i = 0; i < buttons.length; i++) {
				buttons[i].changePending = false;
				buttons[i].checked = false;
				buttons[i].savedValue = "off"; //$NON-NLS-1$
			}
			this._existingProcessProvider = null;
			dojo.toggleClass(this.usesEnterpriseProcessSection, "hidden", true); //$NON-NLS-1$
		},
		
		getSaveErrorMessage: function(err){
			return this.messages.ProjectAreaEditor_31;
		},
		
		_disableCheckBox: function(checkbox, span, disable){
			checkbox.disabled = disable;
			dojo.toggleClass(span, "disabled", disable); //$NON-NLS-1$
		},
		
		populateFilteredProcesses: function(e){
			this.localeSelector.disabled = this.localeFilterToggle.disabled || !this.localeFilterToggle.checked;
			
			var definitionsToShow;
			
			if (this.localeFilterToggle.checked) {
				var locale = this.localeSelector.options[this.localeSelector.selectedIndex].value;
				definitionsToShow = dojo.filter(this.allProcessDefinitions, function(definition){
					return locale === definition.locale;
				});
			} else {
				definitionsToShow = this.allProcessDefinitions;
			}
			
			this.processSelector.options.length = 0;
			
			var newOptions = dojo.map(definitionsToShow, function(definition){
				var option = new Option(this.getTranslatedProcessDefinitionName(definition), definition.itemId);
				option.translatedSummary = this.getTranslatedProcessDefinitionSummary(definition);
				option.locale = definition.locale;
				option.displayableLocale = definition.displayableLocale;
				return option;
			}, this);
			
			newOptions.sort(function(a, b){
				return a.text.localeCompare(b.text);
			});
			
			dojo.forEach(newOptions, function(option){
				if (dojo.isIE) {
					this.processSelector.options[this.processSelector.options.length] = option;
				} else {
					this.processSelector.appendChild(option);
				}
			}, this);
			
			this.processSelector.selectedIndex = 0;
			this.displaySelectedProcessInformation();
		},
		
		populateProcessSelector: function(data, isReused, e){
			if (data !== null && data.length > 0) {
				this.processDescription.style.display = ProcessConstants.NONE;
				this.processSelection.style.display = ProcessConstants.BLOCK;
				
				this.allProcessDefinitions.length = 0;
				this.populateProcessDefinitions(data, isReused, e);
				this.populateFilteredProcesses();
				
			} else {
				// no process definitions, so add the "deploy definitions" controls
				dojox.xml.parser.removeChildren(this.processDescription);
				this.processDescription.style.display = ProcessConstants.BLOCK;
				this.processSelection.style.display = ProcessConstants.NONE;
				
				this.processDescription.appendChild(document.createTextNode(this.messages.ProjectAreaEditor_13));
				this.processDescription.appendChild(document.createElement(ProcessConstants.BR));
				this.processDescription.appendChild(document.createElement(ProcessConstants.BR));
				var requiredFlag = document.createElement("span"); //$NON-NLS-1$
				requiredFlag.className = "j-required"; //$NON-NLS-1$
				requiredFlag.appendChild(document.createTextNode("* ")); //$NON-NLS-1$
				this.processDescription.appendChild(requiredFlag);
				var link = document.createElement("a"); //$NON-NLS-1$
				link.href = "#"; //$NON-NLS-1$
				link.appendChild(document.createTextNode(this.messages.ProjectAreaEditor_14));
				this.connect(link, ON_CLICK, "deployPredefinedProcessDefinitions"); //$NON-NLS-1$ //$NON-NLS-2$
				this.deployPredefinedTemplatesLink = link;
				this.processDescription.appendChild(link);
			}
		},
		
		populateProcessDefinitions: function(data, isReused, e){
			// populate the 'allProcessDefinitions' list
			dojo.forEach(data, function(processDef){
				if (processDef.supportedLocales && processDef.supportedLocales.length > 0) {
					for (var i = 0; i < processDef.supportedLocales.length; i++) {
						var localeDefEntry = dojo.clone(processDef);
						localeDefEntry.locale = processDef.supportedLocales[i];
						localeDefEntry.displayableLocale = processDef.displayableLocales[i];
						this.allProcessDefinitions.push(localeDefEntry);
					}
				} else {
					this.allProcessDefinitions.push(processDef);
				}
			}, this);
			
			// populate the list of supported locales
			this.localeSelector.options.length = 0;
			var localeNames = [];
			var supportedLocales = [];
			dojo.forEach(data, function(processDefinitionDTO){
				if (processDefinitionDTO.supportedLocales) {
					for (var i = 0; i < processDefinitionDTO.supportedLocales.length; i++) {
						var locale = processDefinitionDTO.supportedLocales[i];
						if (dojo.indexOf(localeNames, locale) == -1) {
							localeNames.push(locale);
							supportedLocales.push({
								localeDisplayName: processDefinitionDTO.displayableLocales[i],
								localeName: locale
							});
						}
					}
				}
			}, this);
			
			supportedLocales.sort(function(a, b){
				return a.localeDisplayName.localeCompare(b.localeDisplayName);
			});
			
			dojo.forEach(supportedLocales, function(locale){
				// add the supported locale to locale selector
				var option = new Option(locale.localeDisplayName, locale.localeName);
				if (dojo.isIE) {
					this.localeSelector.options[this.localeSelector.options.length] = option;
				} else {
					this.localeSelector.appendChild(option);
				}
			}, this);
			
			
			this.connectHandles.push(dojo.connect(this.localeFilterToggle, ON_CLICK, this, this.populateFilteredProcesses));
			this.connectHandles.push(dojo.connect(this.localeSelector, "onchange", this, this.populateFilteredProcesses)); //$NON-NLS-1$
			this.connectHandles.push(dojo.connect(this.processSelector, "onchange", this, this.displaySelectedProcessInformation)); //$NON-NLS-1$			
			if (supportedLocales.length > 0) {
				this.enableLocale(true);
				// attempt to select the supported locale closest to the current client locale
				localeNames = dojo.map(supportedLocales, function(elem){
					return elem.localeName;
				});
				
				var localeIndex = this.getSupportedClientLocale(localeNames);
				this.setLocaleFilterChecked(localeIndex);
				this.localeSelector.selectedIndex = localeIndex > 0 ? localeIndex : 0;
			} else {
				this.enableLocale(false);
				this.localeFilterToggle.checked = false;
			}
		},
		
		setLocaleFilterChecked: function(localeIndex){
			if (localeIndex == -1) {
				this.localeFilterToggle.checked = false;
				localeIndex = 0;
			} else {
				this.localeFilterToggle.checked = true;
			}
		},
		
		enableLocale: function(isEnabled){
			this.localeFilterToggle.disabled = !isEnabled;
			this.localeSelector.disabled = !isEnabled;
			if (isEnabled) {
				dojo.removeClass(this.localeFilterText, "disabled"); //$NON-NLS-1$
			} else {
				dojo.addClass(this.localeFilterText, "disabled"); //$NON-NLS-1$				
				// work around lack of min-width support
				if (dojo.isIE) {
					this.localeSelector.style.width = "10em"; //$NON-NLS-1$
				}
			}
		},
		
		deployPredefinedProcessDefinitions: function(e){
			dojo.stopEvent(e);
			if (dojo.hasClass(e.currentTarget, "disabled")) {//$NON-NLS-1$
				return;
			}
			//warn user that current templates may be overwritten
            if (!confirm(this.messages.ProjectAreaEditor_53)) {
            	//user pressed CANCEL
            	return;
            }
			dojo.addClass(e.currentTarget, "disabled"); //$NON-NLS-1$
			// Remove the href so link looks disabled
			e.currentTarget.removeAttribute("href"); //$NON-NLS-1$
			var srh = new ServiceResponseHandler(this, "handleDeployTemplateSuccess", "handleDeployTemplateError"); //$NON-NLS-1$ //$NON-NLS-2$
			this._displayPendingMessage(this.messages.ProjectAreaEditor_15);
			var queryArgs = {
				owningApplicationKey: this._owningApplicationKey
			};
			ProcessClient.deployPredefinedProcessDefinitions(queryArgs, srh);
		},
		
		handleDeployTemplateSuccess: function(){
			this.clearProcessSelector();
			var srh = new ServiceResponseHandler(this, "handleFetchProcessSuccess", "handleFetchProcessError"); //$NON-NLS-1$ //$NON-NLS-2$
			var queryArgs = {
				owningApplicationKey: this._owningApplicationKey
			};
			ProcessClient.getAllProcessDefinitions(queryArgs, srh);
		},
		
		handleDeployTemplateError: function(error){
			this.handleServerResponseError(error, this.messages.ProjectAreaEditor_5, [ProcessConstants.LICENSE_NOT_GRANTED_EXCEPTION]);
		},
		
		clearProcessSelector: function(){
			dojox.xml.parser.removeChildren(this.processDescription);
			this.processDescription.appendChild(document.createTextNode(this.messages.ProjectAreaEditor_17));
		},
		
		updateProcessDefinitionCache: function(data){
			this.processDefinitionCache = data;
			this.populateProcessSelector(data);
		},
		
		handleFetchProcessSuccess: function(data){
			this.updateProcessDefinitionCache(data);
			this.setDirty(this.changesPending());
			if (data && data.length > 0) {
				this._displayLongSuccessMessage(this.messages.ProjectAreaEditor_16);
			} else {
				var msg = dojo.string.substitute(this.messages.ProjectAreaEditor_54, ["#action=jazz.viewPage&id=com.ibm.team.process.ProcessTemplateManagement"]);//$NON-NLS-1$
				this._displayLongInfoMessage(msg);
				dojo.addClass(this.deployPredefinedTemplatesLink, "disabled");//$NON-NLS-1$
				this.deployPredefinedTemplatesLink.removeAttribute("href"); //$NON-NLS-1$
			}
		},

		handleFetchProcessError: function(error){
			this.handleServerResponseError(error, this.messages.ProjectAreaEditor_4, [ProcessConstants.LICENSE_NOT_GRANTED_EXCEPTION]);
		},
		
		populateProcessDescription: function(data){
			dojox.xml.parser.removeChildren(this.processDescription);
			this.processDescription.style.display = ProcessConstants.BLOCK;
			this.processSelection.style.display = ProcessConstants.NONE;
			
			if (data.name) {
				var div = document.createElement(ProcessConstants.DIV);
				div.style.marginBottom = "1em"; //$NON-NLS-1$
				div.appendChild(document.createTextNode(data.name));
				var hr = document.createElement("hr"); //$NON-NLS-1$
				hr.style.border = ProcessConstants.NONE;
				hr.style.borderTop = "1px solid #EEEEEE"; //$NON-NLS-1$
				div.appendChild(hr);
				this.processDescription.appendChild(div);
			}
			if (data.details) {
				div = document.createElement(ProcessConstants.DIV);
				div.appendChild(document.createTextNode(data.details));
				this.processDescription.appendChild(div);
			} else if (data.summary) {
				div = document.createElement(ProcessConstants.DIV);
				div.appendChild(document.createTextNode(data.summary));
				this.processDescription.appendChild(div);
			}
		},
		
		/**
		 * Check the list of supportedLocales for the closest to the current
		 * client locale. Look for an exact match first, then check for a supported
		 * locale with  the same language code as the current client locale.
		 *
		 * @param supportedLocales
		 *     An array of locale strings like "zh", "en-us", etc.
		 *
		 * @return the index in the specified list of the locale that was found
		 *     or -1 if no match was found
		 */
		getSupportedClientLocale: function(supportedLocales){
			var clientLocale = djConfig.locale;
			
			if (clientLocale) {
				// if the client locale has a direct match, return it
				if (dojo.indexOf(supportedLocales, clientLocale) != -1) {
					return dojo.indexOf(supportedLocales, clientLocale);
				}
				
				// we don't support the client locale directly, so see if we
				// support a locale with the same language code
				if (clientLocale.length > 2) {
					clientLocale = clientLocale.substring(0, 2);
				}
				
				var supportedLocale = -1;
				for (var i = 0; i < supportedLocales.length; i++) {
					if (supportedLocales[i] === clientLocale) {
						return i;
					}
					if (supportedLocales[i].indexOf(clientLocale) === 0) {
						supportedLocale = i;
					}
				}
				return supportedLocale;
			}
		},
		
		getTranslatedProcessDefinitionSummary: function(/*ProcessDefinitionDTO or ProcessProviderDTO*/definition){
			var translatedSummary;
			
			// if locale filtering is on, use the currently selected locale
			// if locale filtering is off, see if the definition entry has an
			// associated locale
			var locale;
			if (this.localeFilterToggle.checked) {
				locale = this.localeSelector.options[this.localeSelector.selectedIndex].value;
			} else {
				locale = definition.locale;
			}
			
			if (locale && definition.supportedLocales && definition.supportedLocales.length > 0) {
				if (dojo.indexOf(definition.supportedLocales, locale) != -1) {
					//ProcessProviderDTO does not have translatedSummaries
					if (definition.translatedSummaries) {
						translatedSummary = definition.translatedSummaries[dojo.indexOf(definition.supportedLocales, locale)];
					}
				}
			}
			
			// try the default summary
			if (!translatedSummary) {
				translatedSummary = definition.defaultSummary;
			}
			
			// default to the untranslated value
			if (!translatedSummary) {
				translatedSummary = definition.summary;
			}
			
			return translatedSummary;
		},
		
		getTranslatedProcessDefinitionName: function(/*ProcessDefinitionDTO or ProcessProviderDTO*/definition){
		
			var translatedName;
			
			// if locale filtering is on, use the currently selected locale
			// if locale filtering is off, see if the definition entry has an
			// associated locale
			var locale;
			if (this.localeFilterToggle.checked) {
				locale = this.localeSelector.options[this.localeSelector.selectedIndex].value;
			} else {
				locale = definition.locale;
			}
			
			if (locale && definition.supportedLocales && definition.supportedLocales.length > 0) {
				if (dojo.indexOf(definition.supportedLocales, locale) != -1) {
					//ProcessProviderDTO does not have translatedNames field
					if (definition.translatedNames) {
						translatedName = definition.translatedNames[dojo.indexOf(definition.supportedLocales, locale)];
					}
				}
			}
			
			// try the default name
			if (!translatedName) {
				translatedName = definition.defaultName;
			}
			
			// default to the untranslated value
			if (!translatedName) {
				translatedName = definition.name;
			}
			
			return translatedName;
		},
		
		displaySelectedProcessInformation: function(){
			dojox.xml.parser.removeChildren(this.processDescription);
			this.processDescription.appendChild(document.createTextNode(this.messages.ProjectAreaEditor_34));
			dojox.xml.parser.removeChildren(this.selectedProcessDescription);
			
			var translatedSummary = this.processSelector.options[this.processSelector.selectedIndex].translatedSummary;
			if (!translatedSummary) {
				translatedSummary = this.messages.ProjectAreaEditor_48;
			}
			this.selectedProcessDescription.appendChild(document.createTextNode(translatedSummary));
			
			dojox.xml.parser.removeChildren(this.selectedProcessLocale);
			var localeString = this.processSelector.options[this.processSelector.selectedIndex].displayableLocale;
			if (!localeString || localeString.length === 0) {
				localeString = this.messages.ProjectAreaEditor_46;
			}
			this.selectedProcessLocale.appendChild(document.createTextNode(localeString));
		},
		
		customRefresh: function(refreshClicked){
			dojox.xml.parser.removeChildren(this.processDescription);
			dojox.xml.parser.removeChildren(this.lpaBreadcrumbs);
			if (this._linksForUsesTableDiv.styledTable) {
				dojox.xml.parser.removeChildren(this._linksForUsesTableDiv.styledTable.tbody);
			}
			if (this._linksForProvidesTableDiv.styledTable) {
				dojox.xml.parser.removeChildren(this._linksForProvidesTableDiv.styledTable.tbody);
			}
			this.projectAreaLinks = [];
			this.processDescription.appendChild(document.createTextNode(this.messages.ProjectAreaEditor_34));
			if (this.isNew) {
				this.name.value = this.messages.ProjectAreaEditor_1;
				var srh = new ServiceResponseHandler(this, "updateProcessDefinitionCache", "handleFetchProcessError"); //$NON-NLS-1$ //$NON-NLS-2$
				var queryArgs = {
					owningApplicationKey: this._owningApplicationKey
				};
				ProcessClient.getAllProcessDefinitions(queryArgs, srh);
			} else {
				srh = new ServiceResponseHandler(this, "populateProcessAreaData", "handleFetchError"); //$NON-NLS-1$ //$NON-NLS-2$
				ProcessClient.getProjectAreaByUUIDWithLimitedMembers({
					processAreaItemId: this.projectAreaItemId,
					maxMembers: this.fetchMaxMembers
				}, srh);
				
				this._refreshProcessAreaHierarchy();				
				this._fetchProjectAreaLinks();
				
				lpaSrh = new ServiceResponseHandler(this, "_showLpaBreadcrumbs", "_nullErrorHandler"); //$NON-NLS-1$ //$NON-NLS-2$
				ProcessClient.getLPAContainingProjectAreaUrl({
					uuid: this.projectAreaItemId
				}, lpaSrh);
			}
			
			if (refreshClicked) //need to double check  
			{
				if (this.accessControlEditor) {
					this.accessControlEditor.refresh();
				}
				if (this.timelineEditor) {
					this.timelineEditor.refresh();
				}
				for (tabId in this._tabs) {
					var tab = this._tabs[tabId];
					if (this.isActivatedExtensionTab(tab)) {
						tab.mainPage.refresh();
					}
				}
			}
			
			this.showBreadcrumbs();
		},
		
		_refreshProcessAreaHierarchy: function() {
			this._destroyHierarchyToolbars();
			dojox.xml.parser.removeChildren(this.teamAreaHierarchyContent);
			this.teamAreaHierarchyContent.appendChild(document.createTextNode(this.messages.ProjectAreaEditor_35));
			srh = new ServiceResponseHandler(this, "populateTeamAreaHierarchy", "handleTeamAreaHierarchyFetchError"); //$NON-NLS-1$ //$NON-NLS-2$
			ProcessClient.getProjectHierarchy({
				uuid: this.projectAreaItemId
			}, srh);

		},		
		
		_getProcessAreaStore: function(){
			if (!this.sharedStore) {
				var args = {
					editorContext: ProcessConstants.EditorContext.PROJECT,
					processAreaItemId: this.projectAreaItemId,
					errorContext: this,
					errorCallback: this._handleCreateStoreError
				};
				this.sharedStore = new com.ibm.team.process.web.ui.internal.admin.projects.TimelineStore(args);
			}
			return this.sharedStore;
		},
		
		// called from AbstractProcessAreaEditor.postCreate
		customConnect: function(){
		
			if (!this.isNew) {
				if (this._hideTeams) {
					this.teamAreaHierarchy.style.display = ProcessConstants.NONE;
				} else {
					this.teamAreaHierarchy.style.display = ProcessConstants.EMPTY;
				}
			}
			this.connect(this.linkAddButton, ON_CLICK, function(e){
				dojo.stopEvent(e);
				this.addProjectLink();
			});
			this.connect(this.usesLinksCollapseButton, ON_CLICK, this._onUsesLinksSectionCollapseButtonClick);
			this.connect(this.usesLinksCollapseArea, ON_CLICK, this._onUsesLinksSectionCollapseButtonClick);
			
			this.connect(this.providesLinksCollapseButton, ON_CLICK, this._onProvidesLinksSectionCollapseButtonClick);
			this.connect(this.providesLinksCollapseArea, ON_CLICK, this._onProvidesLinksSectionCollapseButtonClick);
			
			this.connect(this.changeProcessButton, ON_CLICK, function(e){
				dojo.stopEvent(e);
				this._createProcessProviderDialog();
			});
			this.connect(this.enterpriseProcessNotUsedButton, ON_CLICK, this._handleChangeEnterpriseProcessSetting);
			this.connect(this.enterpriseProcessIsButton, ON_CLICK, this._handleChangeEnterpriseProcessSetting);
			this.connect(this.enterpriseProcessUsesButton, ON_CLICK, this._handleChangeEnterpriseProcessSetting);
		},
		
		_createTeamArea: function() {
			var name = this.name.savedValue;
			if (!this._handleNavigate()) {
				document.location.href = "#action=" + TEAM_AREA_EDITOR + "&itemId=new&parentItemId=" + this.projectAreaItemId + "&projectAreaItemId=" + this.projectAreaItemId + "&parentName=" + encodeURIComponent(name) + "&parentType=PROJECT"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
			}
		},
		
		addProjectLink: function(){
			var addLink = new com.ibm.team.process.web.ui.internal.admin.projects.AddProjectLinkDialog({
				projectAreaItemId: this.projectAreaItemId,
				projectAreaName: this.name.value,
				originalProjectAreaName: this.name.savedValue,
				owningApplicationKey: this._owningApplicationKey
			});
			var self = this;
			var dialogWidget = new jazz.ui.Dialog({
				primaryTitle: this.messages.ProjectAreaEditor_AddProjectLinkDialogTitle,
				contentNode: addLink.domNode,
				width: dojo.isIE ? "30em" : "32em", //$NON-NLS-1$ //$NON-NLS-2$ 
				destroyContents: true
			});
			dialogWidget.addLinkObj = addLink;
			addLink.startup();
			this.connectHandles.push(dojo.connect(addLink.cancelButton, ON_CLICK, dialogWidget, function(e){
				dojo.stopEvent(e);
				this.close();
			}));
			this.connectHandles.push(dojo.connect(addLink.finishButton, ON_CLICK, dialogWidget, function(e){
				self._addNewProjectLink(this.addLinkObj.getAssociatedProject());
				this.close();
			}));
		},
		
		_addNewProjectLink: function(newProjectAssociation){
			var linksTable;
			if (UIHelper.isUsesLinkType(newProjectAssociation.linkType)) {
				linksTable = this._linksForUsesTableDiv;
				if (dojo.hasClass(this.usesLinksContentSection, "hidden")) { //$NON-NLS-1$
					this._toggleUsesSection();
				}
			} else {
				linksTable = this._linksForProvidesTableDiv;
				if (dojo.hasClass(this.providesLinksContentSection, "hidden")) { //$NON-NLS-1$
					this._toggleProvidesSection();
				}
			}
			//disable all other rows in both uses and provides table to highlight the new one
			var rows = this._linksForUsesTableDiv.styledTable.tbody.rows;
			for (var i = 0; i < rows.length; i++) {
				dojo.toggleClass(rows[i], ProcessConstants.CSSClass.TABLE_ROW_SELECTED, false);
			}
			rows = this._linksForProvidesTableDiv.styledTable.tbody.rows;
			for (var i = 0; i < rows.length; i++) {
				dojo.toggleClass(rows[i], ProcessConstants.CSSClass.TABLE_ROW_SELECTED, false);
			}
			var existingLink;
			for (var j = 0; j < this.existingProjectAreaLinks.length; j++) {
				var existingProjectAreaLink = this.existingProjectAreaLinks[j];
				if (newProjectAssociation.linkType === existingProjectAreaLink.linkType && newProjectAssociation.projectAreaInfo.targetUrl === existingProjectAreaLink.projectAreaInfo.targetUrl && newProjectAssociation.projectAreaInfo.servicesUrl === existingProjectAreaLink.projectAreaInfo.servicesUrl) {
					//attempting to add a link that already exists or was deleted and re-added between saves
					existingLink = existingProjectAreaLink;
					break;
				}
			}
			
			if (existingLink) { //removed and re-added between saves
				for (j = 0; j < this.removedProjectAreaLinks.length; j++) {
					if (existingLink === this.removedProjectAreaLinks[j]) {
						this.removedProjectAreaLinks.splice(j, 1);
						this.projectAreaLinks.push(existingLink);
						this._createProjectLinkRow(existingLink, true);
						this.setDirty(this.changesPending());
						return;
					}
				}
			} else {
				var duplicates = UIHelper.calculateDuplicateLabels(ProcessConstants.AllLinkTypes, this.messages);
				if (dojo.indexOf(duplicates, newProjectAssociation.linkLabel) !== -1) {
					newProjectAssociation.linkLabel = UIHelper.getQualifiedLinkTypeLabel(newProjectAssociation.linkType, this.messages);
				}
				if (!this.newProjectAreaLinks) {
					this.newProjectAreaLinks = [];
				}
				for (var k = 0; k < this.newProjectAreaLinks.length; k++) {
					var alreadyAddedNewProjectAreaLink = this.newProjectAreaLinks[k];
					if (newProjectAssociation.linkType == alreadyAddedNewProjectAreaLink.linkType && newProjectAssociation.projectAreaInfo.targetUrl == alreadyAddedNewProjectAreaLink.projectAreaInfo.targetUrl && newProjectAssociation.projectAreaInfo.servicesUrl == alreadyAddedNewProjectAreaLink.projectAreaInfo.servicesUrl) {
						//attempting to add a link that already added between saves
						return;
					}
				}
				this.projectAreaLinks.push(newProjectAssociation);
				this.newProjectAreaLinks.push(newProjectAssociation);
				this._createProjectLinkRow(newProjectAssociation, true);
				this.setDirty(this.changesPending());
				return;
			}
			var tr = this._getRowForLink(existingLink, linksTable);
			dojo.toggleClass(tr, ProcessConstants.CSSClass.TABLE_ROW_SELECTED, true);
		},
		
		_getRowForLink: function(existingLink, linksTable){
			var rows = linksTable.styledTable.tbody.rows;
			for (var i = 0; i < rows.length; i++) {
				var rowProjectLink = rows[i].projectAreaLink;
				if (existingLink == rowProjectLink) {
					return rows[i];
				}
			}
			return null;
		},
		
		populateProjectAreaLinks: function(){
			dojox.xml.parser.removeChildren(this._linksForUsesTableDiv.styledTable.tbody);
			dojox.xml.parser.removeChildren(this._linksForProvidesTableDiv.styledTable.tbody);
			if (this.projectAreaLinks) {
				for (var i = 0; i < this.projectAreaLinks.length; i++) {
					this._createProjectLinkRow(this.projectAreaLinks[i], false);
				}
			}
		},
		
		_createProjectLinkRow: function(projectAreaLink, select){
			var tr = document.createElement("tr"); //$NON-NLS-1$
			var projectAreaInfo = projectAreaLink.projectAreaInfo;
			tr.projectAreaLink = projectAreaLink;
			
			var td = document.createElement("td"); //$NON-NLS-1$
			td.className = "linkLabel"; //$NON-NLS-1$
			td.appendChild(UIHelper.getProjectLinkImage(projectAreaLink.linkType, "linkImage")); //$NON-NLS-1$
			td.appendChild(document.createTextNode(projectAreaLink.linkLabel));
			tr.appendChild(td);
			
			td = document.createElement("td"); //$NON-NLS-1$
			td.className = "linkServiceProvider"; //$NON-NLS-1$
			var link = document.createElement("a"); //$NON-NLS-1$
			link.href = projectAreaInfo.presentationUrl;
			link.appendChild(document.createTextNode(decodeURIComponent(projectAreaInfo.label)));
			td.appendChild(link);
			tr.appendChild(td);
			
			td = document.createElement("td"); //$NON-NLS-1$
			td.className = "iconNode"; //$NON-NLS-1$
			var trToolbar = new com.ibm.team.process.web.ui.internal.TRToolbar();
			td.appendChild(trToolbar.domNode);
			this._actionToolbars.push(trToolbar);
			this.addRemoveIcon(trToolbar, this._removeProjectAreaLink);
			tr.appendChild(td);
			
			this.connectHandles.push(dojo.connect(link, "onfocus", tr, function(e){ //$NON-NLS-1$
				dojo.toggleClass(this, ProcessConstants.CSSClass.TABLE_ROW_SELECTED, true);
			}));
			this.connectHandles.push(dojo.connect(link, "onblur", tr, function(e){ //$NON-NLS-1$
				dojo.toggleClass(this, ProcessConstants.CSSClass.TABLE_ROW_SELECTED, false);
			}));
			
			if (UIHelper.isUsesLinkType(projectAreaLink.linkType)) {
				this._linksForUsesTableDiv.styledTable.tbody.appendChild(tr);
			} else {
				this._linksForProvidesTableDiv.styledTable.tbody.appendChild(tr);
			}
			if (select) {
				dojo.toggleClass(tr, ProcessConstants.CSSClass.TABLE_ROW_SELECTED, true);
			}
		},
		
		_removeProjectAreaLink: function(e){
			dojo.stopEvent(e);
			var rowNode = e.target;
			while (rowNode.tagName !== "TR") { //$NON-NLS-1$
				rowNode = rowNode.parentNode;
			}
			var selectedLink = rowNode.projectAreaLink;
			
			if (!selectedLink.linkUrl) {
				for (var j = 0; j < this.newProjectAreaLinks.length; j++) {
					if (selectedLink === this.newProjectAreaLinks[j]) {
						this.newProjectAreaLinks.splice(j, 1);
						break;
					}
				}
			} else { //not a link that was just added
				this.removedProjectAreaLinks.push(selectedLink);
			}
			
			for (j = 0; j < this.projectAreaLinks.length; j++) {
				if (this.projectAreaLinks[j] === selectedLink) {
					this.projectAreaLinks.splice(j, 1);
					break;
				}
			}
			this.setDirty(this.changesPending());
			rowNode.parentNode.removeChild(rowNode);
		},
		
		_createLinksTables: function() {
			this._createLinksTable(this._linksForUsesTableDiv);
			this._createLinksTable(this._linksForProvidesTableDiv);
		},
		
		_createLinksTable: function(linksTable){
			this._destroyStyledTable(linksTable);
			
			var columns = [];
			columns.push({
				label: this.messages.ProjectAreaEditor_ProjectAreaLinkHeader2,
				className: "linkLabel", //$NON-NLS-1$
				sortable: false
			});
			columns.push({
				label: this.messages.ProjectAreaEditor_ProjectAreaLinkHeader,
				className: "linkServiceProvider", //$NON-NLS-1$
				sortable: false
			});
			columns.push({
				label: this.messages.ProjectAreaEditor_ProjectAreaLinkActionsHeader,
				// Work around for 127064	
				className: ProcessConstants.CSSClass.ACTION_COLUMN + " action-column-larger", //$NON-NLS-1$
				sortable: false
			});
			linksTable.styledTable = new jazz.ui.StyledTable({
				columns: columns,
				showHover: true
			});
			linksTable.appendChild(linksTable.styledTable.domNode);
		},
		
		_getProjectAreaLinksURL: function(){
			return this._getThisTargetUrl() + "/links"; //$NON-NLS-1$
		},
		
		_fetchProjectAreaLinks: function(){
			var projectAreasLinksUrl = this._getProjectAreaLinksURL();
			jazz.client.xhrGet({
				url: projectAreasLinksUrl,
				headers: {
					"accept": ProcessConstants.CONTENT_TYPE_APPLICATION_XML //$NON-NLS-1$
				},
				handleAs: "xml", //$NON-NLS-1$
				load: dojo.hitch(this, this._createProjectAreaLinks),
				error: dojo.hitch(this, this._handleErrorGettingProjectAreaLinks)
			});
		},
		
		_createProjectAreaLinks: function(xmlDocument){
			if (!xmlDocument) {
				return;
			}
			
			var children = xmlDocument.childNodes;
			var linksNode = UIHelper.getNode(children, "links", ProcessConstants.NAMESPACE_JP); //$NON-NLS-1$
			var linkChildren = linksNode.childNodes; //children of ServiceProviderCatalog
			var projectAreas = UIHelper.getNodes(linkChildren, "link", ProcessConstants.NAMESPACE_JP); //$NON-NLS-1$
			var count = projectAreas.length;
			var duplicates = UIHelper.calculateDuplicateLabels(ProcessConstants.AllLinkTypes, this.messages);
			dojo.forEach(projectAreas, function(projectAreaLink){
				var url = projectAreaLink.getAttribute("projectArea");//$NON-NLS-1$
				var presentationUrl = url;
				var projectAreaLinkChildren = projectAreaLink.childNodes;
				
				var urlNode = UIHelper.getNode(projectAreaLinkChildren, "url", ProcessConstants.NAMESPACE_JP); //$NON-NLS-1$
				var linkUrl = urlNode.firstChild.data;
				
				var linkTypeNode = UIHelper.getNode(projectAreaLinkChildren, "link-type", ProcessConstants.NAMESPACE_JP); //$NON-NLS-1$
				var linkType = linkTypeNode.firstChild.data;
				
				var serviceProviderNode = UIHelper.getNode(projectAreaLinkChildren, "ServiceProvider", ProcessConstants.NAMESPACE_OSLC); //$NON-NLS-1$
				var serviceProviderChildren = serviceProviderNode.childNodes;
				
				var titleNode = UIHelper.getNode(serviceProviderChildren, "title", ProcessConstants.NAMESPACE_DC); //$NON-NLS-1$
				var label = titleNode.firstChild.data;
				
				var detailsNode = UIHelper.getNode(serviceProviderChildren, "details", ProcessConstants.NAMESPACE_OSLC); //$NON-NLS-1$
				var attributeNode = UIHelper.getAttributeNode(detailsNode, "resource", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
				var targetUrl = attributeNode.value;
				
				var servicesURLNode = UIHelper.getNode(serviceProviderChildren, "services", ProcessConstants.NAMESPACE_OSLC); //$NON-NLS-1$
				attributeNode = UIHelper.getAttributeNode(servicesURLNode, "resource", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
				var servicesUrl = attributeNode.value;
				
				var consumerRegistryNode = UIHelper.getNode(serviceProviderChildren, "consumerRegistry", ProcessConstants.NAMESPACE_JP); //$NON-NLS-1$
				attributeNode = UIHelper.getAttributeNode(consumerRegistryNode, "resource", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
				var consumerRegistryUrl;
				if (attributeNode) { //a consumer registry does not have to be provided to have a valid link
					consumerRegistryUrl = attributeNode.value;
				}
				
				var associatedProjectLink = {};
				var associatedProjectAreaInfo = {};
				associatedProjectLink.projectAreaInfo = associatedProjectAreaInfo;
				associatedProjectAreaInfo.label = label;
				associatedProjectAreaInfo.targetUrl = targetUrl;
				associatedProjectAreaInfo.servicesUrl = servicesUrl;
				associatedProjectAreaInfo.presentationUrl = targetUrl;
				associatedProjectAreaInfo.consumerRegistryUrl = consumerRegistryUrl;
				associatedProjectLink.linkLabel = UIHelper.getLinkTypeLabel(linkType, this.messages);
				if (dojo.indexOf(duplicates, associatedProjectLink.linkLabel) !== -1) {
					associatedProjectLink.linkLabel = UIHelper.getQualifiedLinkTypeLabel(linkType, this.messages);
				}
				associatedProjectLink.linkType = linkType;
				associatedProjectLink.linkUrl = linkUrl;
				
				this.projectAreaLinks.push(associatedProjectLink);
				this.existingProjectAreaLinks.push(associatedProjectLink);
				
				count = count - 1;
				if (count === 0) {
					this.populateProjectAreaLinks();
				}
				
			}, this);
		},
		
		_handleErrorGettingProjectAreaForLink: function(error){
			if (error.responseText) {
				var errorHeader = dojo.string.substitute(this.messages.ProjectAreaEditor_ErrorLoadingProjectAreaForLinkWithReason, [error.responseText]);
			} else {
				errorHeader = this.messages.ProjectAreaEditor_ErrorLoadingProjectAreaForLink;
			}
			this.handleServerResponseError(error, errorHeader);
		},
		
		_handleErrorGettingProjectAreaLinks: function(error){
			if (error.responseText) {
				var errorHeader = dojo.string.substitute(this.messages.ProjectAreaEditor_ErrorLoadingLinksWithReason, [error.responseText]);
			} else {
				errorHeader = this.messages.ProjectAreaEditor_ErrorLoadingLinks;
			}
			this.handleServerResponseError(error, errorHeader);
		},
		
		customChangesPending: function(){
			if (this.linksHaveChanged()) {
				return true;
			}
			if (this.isProcessChanged) {
				return true;
			}
			
			if (!this.isNew) {
				var buttons = [this.enterpriseProcessNotUsedButton, this.enterpriseProcessIsButton, this.enterpriseProcessUsesButton];
				for (var i = 0; i < buttons.length; i++) {
					if (buttons[i].changePending === true) {
						return true;
					}
				}
			}
			
			return this.inherited(arguments);
		},
		
		linksHaveChanged: function(){
			return this.newProjectAreaLinks.length > 0 || this.removedProjectAreaLinks.length > 0;
		},
		
		getDeferredSaveCount: function(response){
			if (!this.isNew || response) {
				//return the total number of deferred calls that will occur
				var count = 1; //repository save of the project area and link creation
				if (response) {
					//the repository save has already occurred for the new area
					count = 0;
				}
				
				if (this.discardPending) {
					count++; //delete of the process description
				}
				
				dojo.forEach(this.newProjectAreaLinks, function(projectAreaLink){
					//new back links POST
					var backLinkConsumerRegistryUrl = this._getBackLinkConsumerRegistryUrl(projectAreaLink);
					if (backLinkConsumerRegistryUrl && !this._isSelfLinking(projectAreaLink)) {
						count++;
					}
				}, this);
				
				dojo.forEach(this.removedProjectAreaLinks, function(projectAreaLink){
					//remove back links DELETE
					var backLinkConsumerRegistryUrl = this._getBackLinkConsumerRegistryUrl(projectAreaLink);
					if (backLinkConsumerRegistryUrl && !this._isSelfLinking(projectAreaLink)) {
						count++;
					}
				}, this);
				
				if (this._tabs) {
					for (tabId in this._tabs) {
						var tab = this._tabs[tabId];
						if (this.isActivatedExtensionTab(tab)) {
							if (tab.mainPage.changesPending()) {
								count++;
							}
						}
					}
				}
				
				return count;
			} else {//for new areas we do the repo save and on success post the links
				return 1;
			}
		},
		
		handleSaveResponse: function(response){
			if (this.isNew && response && this.newProjectAreaLinks.length > 0) {
				this.deferredSaveCount = this.getDeferredSaveCount(response);
				//do not have to deal with removed as since it is a new project
				//area there cannot be any removed links
				this.projectAreaItemId = response;
				var createTrace = this.messages.ProjectAreaEditor_CreatingBackLinks;
				dojo.forEach(this.newProjectAreaLinks, function(projectAreaLink){
					var backLinkConsumerRegistryUrl = this._getBackLinkConsumerRegistryUrl(projectAreaLink);
					if (backLinkConsumerRegistryUrl) {
						createTrace = "\n\t" + dojo.string.substitute(this.messages.ProjectAreaEditor_CreatingBackLinkOfType, [projectAreaLink.linkType, projectAreaLink.projectAreaInfo.label]); //$NON-NLS-1$
						this.postBackLink(projectAreaLink, createTrace);
					}
				}, this);
				this.newProjectAreaLinks = [];
			}
			
			this.inherited(arguments);
		},
		
		_handleChangeProcess: function(processProvider){
			this.newProcessProvider = processProvider.itemId;
			if (this._existingProcessProvider !== this.newProcessProvider) {
				this.isProcessChanged = true;
			} else {
				this.isProcessChanged = false;
			}
			this._showUsesEnterpriseProcessSection(processProvider);
			this.setDirty(this.changesPending());
		},
		
		_createProcessProviderDialog: function(){
			var processProviderDialog = new com.ibm.team.process.web.ui.internal.admin.projects.ProcessProviderDialog({
				owningApplicationKey: this._owningApplicationKey,
				projectAreaItemId: this.projectAreaItemId
			});
			
			var dialogWidget = new jazz.ui.Dialog({
				primaryTitle: this.messages.ProjectAreaEditor_ProcessProviderDialogTitle,
				contentNode: processProviderDialog.domNode,
				// IE can't auto compute the size based on the width of the children
				width: dojo.isIE ? "30em" : "32em", //$NON-NLS-1$  //$NON-NLS-2$
				destroyContents: true,
				editor: this
			});
			dialogWidget.processProviderDialog = processProviderDialog;
			processProviderDialog.startup();
			this.connectHandles.push(dojo.connect(processProviderDialog.cancelButton, "onclick", dialogWidget, function(e){ //$NON-NLS-1$
				dojo.stopEvent(e);
				this.close();
			}));
			this.connectHandles.push(dojo.connect(processProviderDialog.okButton, "onclick", dialogWidget, function(e){//$NON-NLS-1$
				dojo.stopEvent(e);
				this.editor._handleChangeProcess(this.processProviderDialog.selectedProcess);
				this.close();
			}));
			return dialogWidget;
		},
		
		_handleChangeEnterpriseProcessSetting: function(e, processProvider){
			if (e.target === this.enterpriseProcessIsButton) {
				if (e.target.checked) {
					//display warning and only continue if user clicks OK
					if (!confirm(this.messages.ProjectAreaEditor_EnterpriseProcessVisibilityPrompt)) {
						// Workitem 139360...stopEvent() doesn't work as expected on IE7
						// rollback the checked status to the latest default status
						this.enterpriseProcessNotUsedButton.checked = this.enterpriseProcessNotUsedButton.defaultChecked;
						this.enterpriseProcessIsButton.checked = this.enterpriseProcessIsButton.defaultChecked;
						this.enterpriseProcessUsesButton.checked = this.enterpriseProcessUsesButton.defaultChecked;
						return;
					}
				}
			}
			
			var buttons = [this.enterpriseProcessNotUsedButton, this.enterpriseProcessIsButton, this.enterpriseProcessUsesButton];
			for (var i = 0; i < buttons.length; i++) {
				buttons[i].changePending = false;
				buttons[i].defaultChecked = false;
			}
			
			// Workitem 80966...IE 7 needs the defaultChecked state set in the line
			// above and below or it will not properly set the radio button
			// when switching tabs
			e.target.defaultChecked = e.target.checked;
			
			if (e.target === this.enterpriseProcessUsesButton) {
				if (e.target.checked) {
					this._showUsesEnterpriseProcessSection(processProvider);
				}
			} else {
				if (e.target.checked) {
					dojo.toggleClass(this.usesEnterpriseProcessSection, "hidden", true); //$NON-NLS-1$
					this.isProcessChanged = false;
				}
			}
			
			if (e.target.value != e.target.savedValue) {
				e.target.changePending = true;
				this.setDirty(true);
			} else {
				e.target.changePending = false;
				if (!this.changesPending()) {
					this.setDirty(false);
				}
			}
			
		},
		
		_showUsesEnterpriseProcessSection: function(processProvider){
			var processProviderText = this.messages.ProjectAreaEditor_NoProcessProviderName;
			var processProviderId = "#"; //$NON-NLS-1$
			var link;
			if (processProvider) {
				processProviderText = processProvider.name;
				processProviderId = processProvider.itemId;
				link = document.createElement("a"); //$NON-NLS-1$
				dojox.xml.parser.textContent(link, processProviderText);
				link.href = this._getServerUrl() + net.jazz.ajax._contextRoot + "/process/project-areas/" + processProviderId;//$NON-NLS-1$
				var presentationUri = "#action=" + PROJECT_AREA_EDITOR + "&itemId=" + processProviderId; //$NON-NLS-1$ //$NON-NLS-2$
				if (this.enterpriseProcessResourceLink) {
					this.enterpriseProcessResourceLink.destroy();
				}
				var rl = new ResourceLink({
					'presentationUri': presentationUri, //$NON-NLS-1$
					'lazyFetch': true //$NON-NLS-1$
				}, link);
				this.enterpriseProcessResourceLink = rl;
			} else {
				link = document.createElement("label"); //$NON-NLS-1$
				dojox.xml.parser.textContent(link, processProviderText);
			}
			
			dojox.xml.parser.removeChildren(this.usesEnterpriseProcess);
			this.usesEnterpriseProcess.appendChild(link);
			dojo.toggleClass(this.usesEnterpriseProcessSection, "hidden", false); //$NON-NLS-1$
		},
		
		_cacheEnterpriseProcessSetting: function(data){
			if (data.isInternalProcessProvider) {
				this.savedEnterpriseProcessSetting = enterpriseProcessSettings.IS_ENTERPRISE_PROCESS;
			} else if (data.processProviderDTO) {
				this.savedEnterpriseProcessSetting = enterpriseProcessSettings.USES_ENTERPRISE_PROCESS;
			} else {
				this.savedEnterpriseProcessSetting = enterpriseProcessSettings.NONE;
			}
		},
		
		populateEnterpriseProcessSection: function(data){
			dojo.removeClass(this.enterpriseProcessSection, "hidden");//$NON-NLS-1$ 
			if (data.internalIsProcessProvider) {
				this._setChecked(this.enterpriseProcessIsButton);
			} else if (data.processProviderDTO) {
				this._existingProcessProvider = data.processProviderDTO.itemId;
				this.newProcessProvider = null;
				this._setChecked(this.enterpriseProcessUsesButton);
				this._showUsesEnterpriseProcessSection(data.processProviderDTO);
			} else {
				this._setChecked(this.enterpriseProcessNotUsedButton);
			}
		},
		
		_setChecked: function(radioButton){
			radioButton.checked = true;
			// Workitem 80966...IE 7 needs the defaultChecked state set too
			// or it will not properly set the radio button upon tab creation
			radioButton.defaultChecked = true;
			radioButton.savedValue = "on"; //$NON-NLS-1$
		},
		
		_saveEnterpriseProcessSetting: function(saveObj){
			if (this.enterpriseProcessNotUsedButton.checked) {
				if (this.savedEnterpriseProcessSetting === enterpriseProcessSettings.IS_ENTERPRISE_PROCESS) {
					saveObj.isProcessProvider = false;
				} else if (this.savedEnterpriseProcessSetting === enterpriseProcessSettings.USES_ENTERPRISE_PROCESS) {
					saveObj.internalProcessProviderUUID = ProcessConstants.NONE;
				}
			} else if (this.enterpriseProcessIsButton.checked) {
				saveObj.isProcessProvider = true;
			} else if (this.enterpriseProcessUsesButton.checked) {
				if (this.savedEnterpriseProcessSetting === enterpriseProcessSettings.USES_ENTERPRISE_PROCESS) {
					if (this.isProcessChanged) {
						saveObj.internalProcessProviderUUID = this.newProcessProvider;
					} else {
						//Is already using an enterprise process
						saveObj.internalProcessProviderUUID = this._existingProcessProvider;
					}
				} else {
					//newly set to use an enterprise process
					saveObj.internalProcessProviderUUID = this.newProcessProvider;
				}
			}
		},
		
		_isSelfLinking: function(projectAreaLink) {
			var targetProjectAreaInfo = projectAreaLink.projectAreaInfo;
			var url = targetProjectAreaInfo.presentationUrl;
			var fullPath = url.split("/"); //$NON-NLS-1$
			if (fullPath.length > 0) {
				if (fullPath.pop() === this.projectAreaItemId) {
					return true;
				}
			}
			return false;
		},
		
		_getBacklinking: function(projectAreaLink) {
			// this method is only available for the new added project links
			var backLink = {};
			backLink.label = this._toXmlEncoded(this.name.value);
			backLink.linkType = UIHelper.getPeerLinkType(projectAreaLink.linkType);
			backLink.targetUrl = this._getThisTargetUrl();
			var tagName = UIHelper.getTagNameForLinkType(backLink.linkType);
			backLink.targetServicesUrl = projectAreaLink.sourceProjectAreaInfo[tagName].servicesUrl[0];
			var backLinkConsumerRegistryUrl = this._getBackLinkConsumerRegistryUrl(projectAreaLink);
			if (backLinkConsumerRegistryUrl) {
				backLink.consumerRegistryUrl = backLinkConsumerRegistryUrl;
			}
			return backLink;
		},

		_getSelfBackLinkUrl: function(projectAreaLink) {
			if (this._isSelfLinking(projectAreaLink)) {
				var linkType = UIHelper.getPeerLinkType(projectAreaLink.linkType);
				if (this.existingProjectAreaLinks.length > 0) {
					for (var i = 0; i < this.existingProjectAreaLinks.length; i++) {
						var existingProjectAreaLink = this.existingProjectAreaLinks[i];
						if (existingProjectAreaLink.linkType === linkType) {
							var targetUrl = existingProjectAreaLink.projectAreaInfo.targetUrl;
							var segs = targetUrl.split("/"); //$NON-NLS-1$
							if (segs[segs.length -1] === this.projectAreaItemId) {
								return existingProjectAreaLink.linkUrl;
							}
						}
					}
				}
			}
			return null;
		},
		
		_onUsesLinksSectionCollapseButtonClick: function(event) {
			dojo.stopEvent(event);
			this._toggleUsesSection();
		},
		
		_toggleUsesSection: function() {
			if (dojo.hasClass(this.usesLinksContentSection, "hidden")) { //$NON-NLS-1$
				this.usesLinksCollapseImage.src = this._expandedIconSrc;
				dojo.addClass(this.providesLinksCollapseArea, "linkSectionCollapseAreaShowBorder"); //$NON-NLS-1$
			} else {
				this.usesLinksCollapseImage.src = this._collapsedIconSrc;
				dojo.removeClass(this.providesLinksCollapseArea, "linkSectionCollapseAreaShowBorder"); //$NON-NLS-1$
			}
			dojo.toggleClass(this.usesLinksContentSection, "hidden"); //$NON-NLS-1$
		},
		
		_onProvidesLinksSectionCollapseButtonClick: function(event) {
			dojo.stopEvent(event);
			this._toggleProvidesSection();
		},
		
		_toggleProvidesSection: function() {
			if (dojo.hasClass(this.providesLinksContentSection, "hidden")) { //$NON-NLS-1$
				this.providesLinksCollapseImage.src = this._expandedIconSrc;
			} else {
				this.providesLinksCollapseImage.src = this._collapsedIconSrc;
			}
			dojo.toggleClass(this.providesLinksContentSection, "hidden"); //$NON-NLS-1$
		},

		customCanSave: function(){
			if (this.enterpriseProcessUsesButton.checked && (this._existingProcessProvider === this.newProcessProvider)) {
				return false;
			}
			return true;
		},
		
		customValidateRequired: function(saveData){
			if (this.enterpriseProcessUsesButton.checked && !saveData.internalProcessProviderUUID) {
				return this.messages.ProjectAreaEditor_EnterpriseProcessEmptyError;
			}
		},
		
		_showLpaBreadcrumbs: function(lpaProjectsData) {
			//since this is a callback, check that dom node exists before trying to access it
			if (lpaProjectsData && lpaProjectsData.length > 0 && this.lpaBreadcrumbs) {
				dojo.removeClass(this.lpaBreadcrumbs, "hidden");//$NON-NLS-1$
				var intro = this.getLpaIntroMessage(lpaProjectsData);
				var introElement = document.createTextNode(intro);
				this.lpaBreadcrumbs.appendChild(introElement);				
				dojo.forEach(lpaProjectsData, function(lpaProject){
					var breadcrumbText = lpaProject.name;
					var breadcrumbUrl = lpaProject.url;
					var anchor = document.createElement("a");//$NON-NLS-1$
					anchor.setAttribute("href", breadcrumbUrl);//$NON-NLS-1$
					anchor.appendChild(document.createTextNode(breadcrumbText));
					
					var icon = document.createElement("img");//$NON-NLS-1$
					icon.setAttribute("src", this._breadcrumbIconSrc); //$NON-NLS-1$
					
					this.lpaBreadcrumbs.appendChild(anchor);
					this.lpaBreadcrumbs.appendChild(icon);
				}, this);
			}
		},
		
		getLpaIntroMessage: function(lpaProjectsData) {
			var message = this.messages.ProjectAreaEditor_LpaBreadcrumbIntroSingle;
			if (lpaProjectsData.length > 1) {
				message = dojo.string.substitute(this.messages.ProjectAreaEditor_LpaBreadcrumbIntroMulti, [lpaProjectsData.length]);
			}	
			return message;
		},
		
		//error handler that just writes error to console
		_nullErrorHandler: function(err){
			console.warn(dojo.string.substitute("[ProjectAreaEditor] ${0} getting LPA breadcrumbs: ${1} ", [err.errorClass, err.message]));//$NON-NLS-1$
		},
		
		uninitialize: function(){
			if (this.accessControlEditor) {
				this.accessControlEditor.destroyRecursive();
				delete this.accessControlEditor;
			}
			if (this.timelineEditor) {
				this.timelineEditor.destroyRecursive();
				delete this.timelineEditor;
			}
			
			if (this.configurationDataEditor) {
				this.configurationDataEditor.destroyRecursive();
				delete this.configurationDataEditor;
			}
			
			this._destroyStyledTable(this._linksForUsesTableDiv);
			this._destroyStyledTable(this._linksForProvidesTableDiv);
			
			if (this.enterpriseProcessResourceLink) {
				this.enterpriseProcessResourceLink.destroyRecursive();
				this.enterpriseProcessResourceLink = null;
			}
			
			this.inherited(arguments);
		},
		
		getTabUAKey: function(tabKey) {
			if (tabKey === ProcessConstants.TabEnum.OVERVIEW) {
				return ProcessConstants.UAKey.OVERVIEW_TAB;
			} else if (tabKey === ProcessConstants.TabEnum.ROLES) {
				return ProcessConstants.UAKey.ROLES_TAB;
			} if (tabKey === ProcessConstants.TabEnum.PERMISSIONS) {
				return ProcessConstants.UAKey.PERMISSIONS_TAB;
			} 
			return undefined;
		},
		
		getRegistryTabUAKey: function(extensionElement) {
			var uaKey = extensionElement.getAttribute(ProcessConstants.ProjectAreaExtensionPage.ATTRIBUTE_UA_KEY);
			return uaKey;
		},
		
		isNeedRefreshWithWarning: function() {
			return this._backlinkDeletionWarning;
		}
	});
})();
