/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.ProjectAreaManagementPage"); //$NON-NLS-1$
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$
	dojo.require("jazz.ui.SideNavigationLayout"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.ManageProjectAreas"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.ManageArchivedProjectAreas"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.ProjectAreaEditor"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.TeamAreaEditor"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	var ActionRegistry;
	var PlatformUI = net.jazz.ajax.ui.PlatformUI;
	var SideNavigationLayout = jazz.ui.SideNavigationLayout;
	var ManageProjectAreas = com.ibm.team.process.web.ui.internal.admin.projects.ManageProjectAreas;
	var ManageArchivedProjectAreas = com.ibm.team.process.web.ui.internal.admin.projects.ManageArchivedProjectAreas;
	var ProjectAreaEditor = com.ibm.team.process.web.ui.internal.admin.projects.ProjectAreaEditor;
	var TeamAreaEditor = com.ibm.team.process.web.ui.internal.admin.projects.TeamAreaEditor;
	var PREFIX = "com.ibm.team.process."; //$NON-NLS-1$
	var ProjectAreaManagementPageMessages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	var SERVER_PACKAGE = "com.ibm.team.process.web.ui.internal.admin.projects."; //$NON-NLS-1$
	var ProjectAreaManagementPage = com.ibm.team.process.web.ui.internal.admin.projects.ProjectAreaManagementPage;
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	
	var ACTION_MAP = {};
	var ACTION_SETS = [{
		label: ProjectAreaManagementPageMessages.ProjectAreaManagementPage_1,
		actions: [{
			label: ProjectAreaManagementPageMessages.ProjectAreaManagementPage_2,
			shortId: "manageProjectAreas", //$NON-NLS-1$
			aliases: ["editProjectArea", "editTeamArea"], //$NON-NLS-1$ //$NON-NLS-2$
			widget: "ManageProjectAreas", //$NON-NLS-1$
			showInNav: true
		}, {
			label: ProjectAreaManagementPageMessages.ProjectAreaManagementPage_3,
			shortId: "manageArchivedProjectAreas", //$NON-NLS-1$
			aliases: ["editArchivedProjectArea"], //$NON-NLS-1$
			widget: "ManageArchivedProjectAreas", //$NON-NLS-1$
			showInNav: true
		}, {
			label: ProjectAreaManagementPageMessages.ProjectAreaManagementPage_4,
			shortId: "editProjectArea", //$NON-NLS-1$
			widget: "ProjectAreaEditor", //$NON-NLS-1$
			showInNav: false
		}, {
			label: ProjectAreaManagementPageMessages.ProjectAreaManagementPage_5,
			shortId: "editArchivedProjectArea", //$NON-NLS-1$
			widget: "ProjectAreaEditor", //$NON-NLS-1$
			showInNav: false
		}, {
			label: ProjectAreaManagementPageMessages.ProjectAreaManagementPage_6,
			shortId: "editTeamArea", //$NON-NLS-1$
			widget: "TeamAreaEditor", //$NON-NLS-1$
			showInNav: false
		}, {
			label: ProjectAreaManagementPageMessages.ProjectAreaManagementPage_9,
			shortId: "usersInProjectArea", //$NON-NLS-1$
			widget: "Members", //$NON-NLS-1$
			showInNav: false
		}]
	}];
	
	function getWorkbench(){
		return PlatformUI.getWorkbench();
	}
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.ProjectAreaManagementPage", dijit._Widget, { //$NON-NLS-1$
		
		constructor: function(){
			// Get owning application key from the URL. Embedding fronting
			// application must provide it's application key.
			// If not present, then it means that request has been made by JTS 
			// itself. In that case, set the application key of JTS.
			var _queryParams = dojo.queryToObject(window.location.search.slice(1));
			if (_queryParams.owningApplicationKey && _queryParams.owningApplicationKey !== ProcessConstants.JTS_OWNING_APPLICATION_KEY) {
				this._owningApplicationKey = _queryParams.owningApplicationKey;
				this._isEmbedded = true;
			} else {
				this._owningApplicationKey = ProcessConstants.JTS_OWNING_APPLICATION_KEY;
			}
			
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		reloadCurrent: function(){
			this._previousActionEntry = this._currentActionEntry || null;
			this._showCurrentWidget(/*isReload*/true);
		},
		
		postCreate: function(){
			this.inherited(arguments);
			
			this._widgetCache = {};
			this.userEditorCache = {};
			
			this.domNode.className = "com-ibm-team-repository-admin"; //$NON-NLS-1$
			this._appView = new jazz.ui.SideNavigationLayout();
			this.domNode.appendChild(this._appView.domNode);
			
			// register actions and create hyperlinks
			ActionRegistry = getWorkbench().getActionRegistry();
			ActionRegistry.registerAction(PREFIX + "_projectAreasDefaultAction", this, "_projectAreasDefaultAction"); //$NON-NLS-1$ //$NON-NLS-2$
			var self = this;
			this.allLinks = [];
			dojo.forEach(ACTION_SETS, function(actionSet){
				self._createNavSection(actionSet);
			});
		},
		
		_createNavSection: function(actionSet){
			var self = this;
			var navSection = this._appView.addNavigationSection(document.createElement("div"), actionSet.label, true); //$NON-NLS-1$
			dojo.forEach(actionSet.actions, function(action){
				var id = PREFIX + action.shortId;
				ACTION_MAP[action.shortId] = action;
				if (action.showInNav) {
					var link = navSection.addSectionRow(action.label, "#action=" + id); //$NON-NLS-1$
					link.__jazzActionId = action.shortId;
					self.allLinks.push(link);
					navSection.addArtifactCommand(link, self.messages.ProjectAreaManagementPage_10, dojo.hitch(self, self._onNavSectionClick), "process-management-refresh-command"); //$NON-NLS-1$
				}
				ActionRegistry.registerAction(id, self, "_handleAction"); //$NON-NLS-1$
			});
		},
		
		_projectAreasDefaultAction: function(){
			// by default, show the first section from the left-nav
			this._handleAction({
				action: PREFIX + ACTION_SETS[0].actions[0].shortId
			});
		},
		
		_handleAction: function(args){
			if (!args.action) {
				console.debug(this.messages.ProjectAreaManagementPage_8);
				return;
			}
			
			// cache the previous entry to see if the widget should be destroyed or
			// not
			this._previousActionEntry = this._currentActionEntry || null;
			
			// parse out the short id from the fully-qualified id
			var shortId = args.action.substring(args.action.lastIndexOf(".") + 1); //$NON-NLS-1$
			this._currentActionEntry = ACTION_MAP[shortId];
			this._currentArgs = args;
			this._currentArgs.parent = this;
			this._currentArgs.owningApplicationKey = this._owningApplicationKey;
			this._currentArgs.hideTeams = this._hideTeams;
			if (this._isEmbedded) {
				this._currentArgs.isEmbedded = this._isEmbedded;
			}
			
			// set the title for the action
			getWorkbench().setTitle(this._currentActionEntry.label);
			
			// highlight the selected link and unhighlight the others
			dojo.forEach(this.allLinks, function(link){
				if (link.__jazzActionId === shortId) {
					dojo.toggleClass(link, "selected", true); //$NON-NLS-1$
				} else {
					if (link.aliases) {
						var found = false;
						for (var i = 0; i < link.aliases.length; i++) {
							if (link.aliases[i] === shortId) {
								dojo.toggleClass(link, "selected", true); //$NON-NLS-1$
								found = true;
								break;
							}
						}
						if (!found) {
							dojo.toggleClass(link, "selected", false); //$NON-NLS-1$
						}
					} else {
						dojo.toggleClass(link, "selected", false); //$NON-NLS-1$
					}
				}
			});
			
			// call the actual method implementation
			if (!this._currentActionEntry.simpleAction || !this._currentContentWidget ||
				this._currentContentWidget.id.replace(/_\d+$/, "").replace(/_/g, ".") !== SERVER_PACKAGE + this._currentActionEntry.widget) { //$NON-NLS-1$ //$NON-NLS-2$
				this._showCurrentWidget();
			}
			if (this._currentActionEntry.simpleAction) {
				this._currentActionEntry.simpleAction(args, this._currentContentWidget);
			}
		},
		
		_onNavSectionClick: function(e){
			dojo.stopEvent(e);
			this.reloadCurrent();
		},
		
		_showCurrentWidget: function(isReload){
			if (this._currentActionEntry.widget === "ProjectAreaEditor" && this._isEmbedded && this._hideTeams === undefined) { //$NON-NLS-1$
				// Determine whether to show team areas or not by the absence/presence
				// of the service entry "jp:hideTeamAreas" in the rootservices 
				// of current fronting applicatoin. This configuration is only valid 
				// if the foundation server is running in JTS mode.
				// "this._hideTeams === undefined" means that this argument has not been initialized.
				// This argument should be initialized for only one time.
				var successCallback = dojo.hitch(this, function(isHideTeams){
					this._hideTeams = isHideTeams;
					this._currentArgs.hideTeams = this._hideTeams;
					this._doShowCurrentWidget(isReload);
				});

				var	errorCallback = dojo.hitch(this, function(error){
					throw new Error(error);
				});
				
				this._getHideTeamAreas(successCallback, errorCallback);
			} else {
				this._doShowCurrentWidget(isReload);
			}
		},
		
		_doShowCurrentWidget: function(isReload){
			// save a reference to the prior content widget
			this._previousContentWidget = this._currentContentWidget || null;
			
			if (this._currentActionEntry.cached) {
				this._showCachedWidget(isReload);
			} else {
				this._showUncachedWidget();
			}
			
			// help the garbage collector do its job
			if (this._previousContentWidget && !this._previousActionEntry.cached) {
				this._previousContentWidget.destroyRecursive();
			}
		},
		
		_showCachedWidget: function(isReload){
			var className = this._currentActionEntry.widget;
			if (this._widgetCache[className]) {
				// get widget from the cache if it exists
				this._currentContentWidget = this._widgetCache[className];
			} else {
				// otherwise instantiate it and add it to the cache
				var constructor = dojo.getObject(SERVER_PACKAGE + className);
				this._currentContentWidget = new constructor(this._currentArgs);
				dojo.addClass(this._currentContentWidget.domNode, "com-ibm-team-repository-admin-contentArea"); //$NON-NLS-1$
				this._widgetCache[className] = this._currentContentWidget;
			}
			if (isReload) {
				this._currentContentWidget.reload(this._currentArgs);
			} else {
				this._currentContentWidget.show(this._currentArgs);
				this._appView.setMainContent(this._currentContentWidget.domNode);
			}
		},
		
		_showUncachedWidget: function(){
			var constructor = dojo.getObject(SERVER_PACKAGE + this._currentActionEntry.widget);
			this._currentContentWidget = new constructor(this._currentArgs);
			dojo.addClass(this._currentContentWidget.domNode, "com-ibm-team-repository-admin-contentArea"); //$NON-NLS-1$
			this._appView.setMainContent(this._currentContentWidget.domNode);
		},

		_getHideTeamAreas: function(/* Function */ successCallback, /* Function */ errorCallback) {
			// Checking the absence/presence of the service entry "jp:hideTeamAreas" 
			// in the rootservices of current fronting applicatoin.
			// And then invoke the callback function with a boolean parameter. 
			// If this the service entry does exist in rootservices, 
			// the boolean parameter is set to true. Otherwise, the parameter 
			// is set to false.
			
			var handleError = function(error) {
				if (errorCallback) {
					errorCallback(error);
				}
			};
			
			var getHideTeamAreas = function(rootServicesNode) {
				// Checking the absence/presence of the service entry "jp:hideTeamAreas" 
				// in the rootservices of current fronting applicatoin.
				var hideTeamAreasEntryNode = rootServicesNode.childrenByNameNS("hideTeamAreas", ProcessConstants.NAMESPACE_JP); //$NON-NLS-1$	
				var hideTeams = false;
				if (hideTeamAreasEntryNode.length > 0) {
					hideTeams = true;
				}
				return hideTeams;
			};

			var handleRootServices = function(rootServices) {
				var	success = function(rootServicesNode){
					var isHideTeams = getHideTeamAreas(rootServicesNode);
					if (successCallback) {
						successCallback(isHideTeams);
					}
				};
				
				ProcessClient.getRootServices(rootServices, success, handleError);			
			};
			
			var serviceResponseHandler = {};
			serviceResponseHandler.handleFriend = function(response) {
				var friend = dojo.fromJson(response);
				// Check if the response is not empty.
				if (dojo.exists("id", friend)) { //$NON-NLS-1$
					handleRootServices(friend.rootServices);
				} 
			}; 
			serviceResponseHandler.errorCallback = handleError;
			var args = {
				owningApplicationKey: this._owningApplicationKey
			};
			var srh = new ServiceResponseHandler(serviceResponseHandler, "handleFriend", "errorCallback"); //$NON-NLS-1$ //$NON-NLS-2$
			ProcessClient.getApplication(args, srh);
		}
		
	});
	
})();
