/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.TeamAreaEditor"); //$NON-NLS-1$
	dojo.require("dojox.string.BidiComplex"); //$NON-NLS-1$
	dojo.require("dojox.xml.parser"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.AbstractProcessAreaEditor"); //$NON-NLS-1$
	dojo.require("jazz.ui.ResourceLink"); //$NON-NLS-1$
	dojo.require("jazz.opensocial.Gadget"); //$NON-NLS-1$
	dojo.require("jazz.client.xhr"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	var ResourceLink = jazz.ui.ResourceLink;
	var PROJECT_AREA_EDITOR = "com.ibm.team.process.editProjectArea"; //$NON-NLS-1$
	var TEAM_AREA_EDITOR = "com.ibm.team.process.editTeamArea"; //$NON-NLS-1$
	var MSG_COOKIE_NAME = "com.ibm.team.process.editTeamArea#saveMsg"; //$NON-NLS-1$
	var NEW_MEMBERS_COOKIE_NAME = "com.ibm.team.process.web.ui.internal.admin.projects.ProcessAreaEditor#newMembers"; //$NON-NLS-1$
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.TeamAreaEditor", com.ibm.team.process.web.ui.internal.admin.projects.AbstractProcessAreaEditor, { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/TeamAreaEditor.html"), //$NON-NLS-1$ //$NON-NLS-2$
		initialize: function(args){
			this.context = ProcessConstants.EditorContext.TEAM;
			if (!this.isNew) {
				this.itemId = args.itemId;
			}
			this.projectAreaItemId = args.projectAreaItemId;
			this.parentItemId = args.parentItemId;
			this.parentName = args.parentName;
			this.parentType = args.parentType;
			this._teamAreaIconSrc = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/teamAreaIcon.png"); //$NON-NLS-1$ //$NON-NLS-2$
			this._teamAreaIconClass = "itemIcon"; //$NON-NLS-1$
			if (!this.isNew) {
				this._teamAreaIconClass = "itemIcon hideIcon"; //$NON-NLS-1$
			}
		},
		
		_setProcessAreaIcon: function(isArchived){
			dojo.removeClass(this.teamAreaIcon, "hideIcon"); //$NON-NLS-1$
			if (!isArchived) {
				this.teamAreaIcon.src = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/teamAreaIcon.png"); //$NON-NLS-1$ //$NON-NLS-2$
			} else {
				this.teamAreaIcon.src = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/teamAreaIcon_archived.png"); //$NON-NLS-1$ //$NON-NLS-2$
			}
		},
		
		// called from AbstractProcessAreaEditor.postCreate 
		customConnect: function(){
		
			this.connect(this.name, "onchange", function(){ //$NON-NLS-1$
				if (this.hierarchyNameDisplay) {
					dojox.xml.parser.removeChildren(this.hierarchyNameDisplay);
					if (this.name.value == ProcessConstants.EMPTY) {
						this.hierarchyNameDisplay.appendChild(document.createTextNode(this.messages.TeamAreaEditor_1));
					} else {
						this.hierarchyNameDisplay.appendChild(document.createTextNode(this.name.value));
					}
				}
			});
		},
		
		_createTeamArea: function() {
			if (!this._handleNavigate()) {
				document.location.href = "#action=" + TEAM_AREA_EDITOR + "&itemId=new&parentItemId=" + this.itemId + "&projectAreaItemId=" + this.projectAreaItemId + "&parentType=TEAM"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
			}
		},
		
		customRefresh: function(refreshClicked){
			dojox.xml.parser.removeChildren(this.developmentLineDiv);
			this.developmentLineDiv.appendChild(document.createTextNode(this.messages.TeamAreaEditor_3));
			
			if (this.isNew) {
				this.name.value = this.messages.TeamAreaEditor_4;
				this.populateDevelopmentLineSelector();
				this._refreshProcessAreaHierarchy();
				
				setTimeout(dojo.hitch(this.name, function(){
					try {
						this.select();
					} catch (error) {
						// If the text can't be selected...ignore the error
					}
				}), 250);
			} else {
				dojox.xml.parser.removeChildren(this.teamAreaHierarchyContent);
				this.teamAreaHierarchyContent.appendChild(document.createTextNode(this.messages.TeamAreaEditor_2));
				srh = new ServiceResponseHandler(this, "populateProcessAreaData", "handleFetchError"); //$NON-NLS-1$ //$NON-NLS-2$
				ProcessClient.getTeamAreaByUUIDWithLimitedMembers({
					processAreaItemId: this.itemId,
					maxMembers: this.fetchMaxMembers
				}, srh);
			}
			
			if (refreshClicked) { //need to double check  
				for (tabId in this._tabs) {
					var tab = this._tabs[tabId];
					if (this.isActivatedExtensionTab(tab)) {
						tab.mainPage.refresh();
					}
				}
			}
		},
		
		_refreshProcessAreaHierarchy: function() {
			this._destroyHierarchyToolbars();
			dojox.xml.parser.removeChildren(this.teamAreaHierarchyContent);
			this.teamAreaHierarchyContent.appendChild(document.createTextNode(this.messages.TeamAreaEditor_2));
				var srh = new ServiceResponseHandler(this, "populateProcessHierarchy", "handleTeamAreaHierarchyFetchError"); //$NON-NLS-1$ //$NON-NLS-2$
				ProcessClient.getProjectHierarchy({
					uuid: this.projectAreaItemId
				}, srh);
		},
		
		clearState: function(refreshClicked){
			this.inherited(arguments);
			this.selectedMembers = {};
			this.selectedAdmins = {};
			this.parentProcessArea = null;
			dojox.xml.parser.removeChildren(this.teamAreaHierarchyContent);
			dojox.xml.parser.removeChildren(this.developmentLineDiv);
		},
		
		getFetchErrorMessage: function(error){
			if (error.status == ProcessConstants.SC_FORBIDDEN) {
				return this.messages.TeamAreaEditor_ErrorNoReadAccess;
			} else {
				return this.messages.TeamAreaEditor_7;
			}
		},
		
		getPrimaryFieldLabel: function(){
			return this.messages.TeamAreaEditor_8;
		},
		
		populateProcessAreaData: function(data){
			var args = arguments;
			UIHelper.runSafe(dojo.hitch(this, function(){
				this.inherited(args);
				
				this.projectAreaName = data.processHierarchy.name;
				
				// TODO I don't like this....need to force a refresh after
				//we are sure that the new data has been async loaded
				if (this.permissionEditor) {
					this.permissionEditor.refresh(); //update the roles
				}
				
				var teamArray = this.getAncestorsAsArray(data.processHierarchy);
				teamArray.shift();
				this.teamAreaPath = dojo.map(teamArray, "return '/'.concat(item.name)").join(ProcessConstants.EMPTY); //$NON-NLS-1$
				this.populateProcessHierarchy(data.processHierarchy);
				this.populateDevelopmentLine(data.developmentLine);
				
				this.navigateToInitTab();
			}));
		},
		
		createInvitationPropertiesString: function(user, repoLink){
			return dojo.string.substitute("teamRepository=${0}\nuserId=${1}\nuserName=${2}\nprojectAreaName=${3}\nteamAreaPath=${4}", [repoLink, user.userId, user.name, this.projectAreaName, this.teamAreaPath]); //$NON-NLS-1$
		},
		
		getInvitationLink: function(host){
			return dojo.string.substitute("${0}/web/projects/${1}", [host, this.projectAreaName]); //$NON-NLS-1$;
		},
		
		customTeamInvitationParams: function(params){
			params.teamAreaUuid = this.itemId;
		},
		
		populateDevelopmentLine: function(data){
			dojox.xml.parser.removeChildren(this.developmentLineDiv);
			if (data && data.label) {
				var label = data.label;
				if (data.projectTimeline === true) {
					label += " " + this.messages.Timeline_DecoratorText; //$NON-NLS-1$
				}
				this.developmentLineDiv.appendChild(document.createTextNode(label));
			} else {
				this.developmentLineDiv.appendChild(document.createTextNode(this.messages.TeamAreaEditor_17));
			}
		},
		
		populateDevelopmentLineSelector: function(){
			dojox.xml.parser.removeChildren(this.developmentLineDiv);
			if (this.parentProcessArea) {
				if (this.parentProcessArea.type == "projectArea") { //$NON-NLS-1$
					var srh = new ServiceResponseHandler(this, "developmentLineSelectorCallback", "handleProjectDevLineQueryError"); //$NON-NLS-1$ //$NON-NLS-2$
					ProcessClient.getProjectDevelopmentLines({
						uuid: this.parentProcessArea.itemId
					}, srh);
				} else if (this.parentProcessArea.type == "teamArea") { //$NON-NLS-1$
					srh = new ServiceResponseHandler(this, "developmentLineDisplayCallback", "handleTeamDevLineQueryError"); //$NON-NLS-1$ //$NON-NLS-2$
					ProcessClient.getTeamAreaByUUIDWithLimitedMembers({
						processAreaItemId: this.parentProcessArea.itemId,
						maxMembers: this.fetchMaxMembers
					}, srh);
				}
			} else {
				this.developmentLineDiv.appendChild(document.createTextNode(this.messages.TeamAreaEditor_18));
			}
		},
		
		handleProjectDevLineQueryError: function(error){
			this.handleServerResponseError(error, this.messages.TeamAreaEditor_19);
		},
		
		handleTeamDevLineQueryError: function(error){
			this.handleServerResponseError(error, this.messages.TeamAreaEditor_20);
		},
		
		developmentLineSelectorCallback: function(data){
			dojox.xml.parser.removeChildren(this.developmentLineDiv);
			if (data && data.length > 0) {
				var select = document.createElement("select"); //$NON-NLS-1$
				select.setAttribute("title", this.messages.TeamAreaEditorHTML_12); //$NON-NLS-1$
				this.developmentLineSelector = select;
				for (var i = 0; i < data.length; i++) {
					var option = new Option(data[i].label, data[i].itemId);
					if (data[i].summary && data[i].summary != ProcessConstants.EMPTY) {
						option.description = data[i].summary;
					} else if (data[i].description && data[i].description != ProcessConstants.EMPTY) {
						option.description = data[i].description;
					} else {
						option.description = this.messages.TeamAreaEditor_21;
					}
					if (dojo.isIE) {
						select.options[select.options.length] = option;
					} else {
						select.appendChild(option);
					}
				}
				this.developmentLineDiv.appendChild(select);
			} else {
				this.developmentLineDiv.appendChild(document.createTextNode(this.messages.TeamAreaEditor_22));
			}
		},
		
		developmentLineDisplayCallback: function(data){
			this.populateDevelopmentLine(data.developmentLine);
		},
		
		populateProcessHierarchy: function(project){
			this._project = project;
			dojox.xml.parser.removeChildren(this.teamAreaHierarchyContent);
			this.showBreadcrumbs();
            var processAreaData = {
				processAreaName: project.name,
				processAreaItemId: project.itemId,
				archived: project.archived,
				isProjectArea: true
			};
			var div = this._createTeamHierarchyElement(processAreaData, false, true);
			div.archived = project.archived;
			this.teamAreaHierarchyContent.appendChild(div);
			var isNewParent = false;
			if (this.isNew && this._project.itemId === this.parentItemId) {
				this.parentProcessArea = {
					name: this._project.name,
					itemId: this._project.itemId,
					type: "projectArea" //$NON-NLS-1$
				};
				isNewParent = true;
			}
			if (this._project.children && this._project.children.length > 0) {
				this.populateRemainingHierarchy(this._project.children, 1, isNewParent, div);
			} else {
				if (isNewParent) {
					this.populateRemainingHierarchy([], 1, true, div);
				}
			}
			this._displayArchivedChildren();
			if (this.isNew) {
				this.populateDevelopmentLineSelector();
			}
		},
		
		populateRemainingHierarchy: function(teams, indent, includeNewTeam, parentProcessArea){
			teams = teams.sort(function(a, b){
				return ProcessClient.stringCompare(a.name, b.name);
			});
			dojo.forEach(teams, dojo.hitch(this, function(team){
                if (this.itemId === team.itemId) {
					this._isArchived = team.archived;
					this.updateProcessHierarchyButtons(team.archived);
					// update this editor's icon in case its team just changed archived status
					this._setProcessAreaIcon(team.archived);
				}
				var processAreaData = {
					processAreaName: team.name,
					processAreaItemId: team.itemId,
					parentProcessArea: parentProcessArea,
					archived: team.archived,
					isProjectArea: false
				};
				var div = this._createTeamHierarchyElement(processAreaData, !parentProcessArea.archived, this.itemId !== team.itemId);
				if (!this.isNew && this.itemId === team.itemId) {
					this.hierarchyNameDisplay = div;
				}
				div.style.paddingLeft = (15 * indent) + "px"; //$NON-NLS-1$
				div.archived = team.archived;
				this.teamAreaHierarchyContent.appendChild(div);
				var isNewParent = false;
				if (this.isNew && team.itemId === this.parentItemId) {
					this.parentProcessArea = {
						name: team.name,
						itemId: team.itemId,
						type: "teamArea" //$NON-NLS-1$
					};
					isNewParent = true;
				}
				if (team.children && team.children.length > 0) {
					this.populateRemainingHierarchy(team.children, indent + 1, isNewParent, div);
				} else {
					if (isNewParent) {
						this.populateRemainingHierarchy([], indent + 1, true, div);
					}
				}
			}));
			if (includeNewTeam) {
                var processAreaData = {
					processAreaName: this.messages.TeamAreaEditor_23,
					processAreaItemId: this.messages.TeamAreaEditor_23,
					parentProcessArea: parentProcessArea,
					archived: false,
					isProjectArea: false
				};
				var div = this._createTeamHierarchyElement(processAreaData, false, false);
				this.hierarchyNameDisplay = div;
				div.style.paddingLeft = (15 * indent) + "px"; //$NON-NLS-1$
				this.teamAreaHierarchyContent.appendChild(div);
			}
		},

		// returns an array representing the process hierarchy directly above this team area
		getAncestorsAsArray: function(processArea){
			var foundParentOfNewTeam = this.isNew && this.parentItemId === processArea.itemId;
			var foundThisTeam = !this.isNew && this.itemId === processArea.itemId;
			var hasChildren = processArea.children && processArea.children.length > 0;
			
			// Return an array containing this process area so it can bubble up recursively
			// The recursive callers will insert their process area at the front of this array so
			// we get a complete ancestry array in the end 
			if (foundParentOfNewTeam || foundThisTeam) {
				return [processArea];
			} else if (hasChildren) {
				for (var i = 0; i < processArea.children.length; i++) {
					var curLine = this.getAncestorsAsArray(processArea.children[i]);
					// If the current child team exists along the ancestry path, we know this element
					// also belongs along the ancestry path.  Add the current processArea to the front
					// of the ancestry path and bubble up to the next recursive caller.
					if (curLine) {
						curLine.unshift(processArea);
						return curLine;
					}
				}
			} else {
				// We didn't find what we were looking for down this branch.
				// Return null so the recursive callers will ignore this path.
				return null;
			}
		},
		
		customAddBreadcrumbSegments: function(breadcrumbSegments){
			var hierarchy = this.getAncestorsAsArray(this._project);
			for (var i = 0; i < hierarchy.length; i++) {
				// Don't include the current element in the breadcrumb hierarchy
				if (!this.isNew && i === hierarchy.length - 1) {
					continue;
				}
				var segment = {};
				if (i === 0) {
					segment.href = "#action=" + PROJECT_AREA_EDITOR + "&itemId=" + hierarchy[i].itemId; //$NON-NLS-1$ //$NON-NLS-2$
				} else {
					segment.href = "#action=" + TEAM_AREA_EDITOR + "&itemId=" + hierarchy[i].itemId + "&projectAreaItemId=" + this.projectAreaItemId; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				}
				segment.label = dojox.string.BidiComplex.createDisplayString(hierarchy[i].name);
				breadcrumbSegments.push(segment);
			}
		},
		
		doSave: function(saveObj, serviceResponseHandler){
			ProcessClient.postTeamArea(saveObj, serviceResponseHandler);
			this._displayPendingMessage(this.messages.TeamAreaEditor_26);
		},
		
		configureSaveObject: function(){
			var saveObj = {
				itemId: this.itemId
			};
			if (!this.isNew) {
				saveObj.stateId = this.stateId;
			}
			for (var i = 0; i < this.inputFields.length; i++) {
				if (this.inputFields[i].changePending) {
					saveObj[this.inputFields[i].name] = this.inputFields[i].value;
				}
			}
			saveObj.jsonMembers = dojo.toJson(this.pendingMemberChanges);
			saveObj.jsonAdmins = dojo.toJson(this.pendingAdminChanges);
			if (this.isNew && this.parentProcessArea) {
				saveObj.jsonParentProcessArea = dojo.toJson(this.parentProcessArea);
			}
			if (this.developmentLineSelector && this.developmentLineSelector.value) {
				saveObj.developmentLineUuid = this.developmentLineSelector.value;
			}
						
			this.generateChangeParameters(saveObj);
			this.generateRoleChanges(saveObj);
			
			return saveObj;
		},
		
		validateRequired: function(saveData){
			if (saveData.name == ProcessConstants.EMPTY || (this.isNew && !saveData.name)) {
				this.togglePrimaryField(ProcessConstants.EditorMode.EDIT);
				return this.messages.TeamAreaEditor_27;
			}
			
			if (saveData.name && saveData.name.indexOf('/') >= 0) { //$NON-NLS-1$
				this.togglePrimaryField(ProcessConstants.EditorMode.EDIT);
				return this.messages.TeamAreaEditor_35;
			}
			
			if (this.isNew && (!saveData.jsonParentProcessArea || saveData.jsonParentProcessArea == ProcessConstants.EMPTY)) {
				return this.messages.TeamAreaEditor_28;
			}
		},
		
		handleSaveResponse: function(response){
		
			if (this.discardPending) {
				this._deleteProcessDescription(response);
			}
			
			this._saveExtensionTabs();
			
			if (this.pendingMemberChanges.add && this.pendingMemberChanges.add.length > 0) {
				dojo.cookie(NEW_MEMBERS_COOKIE_NAME, dojo.toJson(this.pendingMemberChanges.add));
			}
			
			if (this.isNew && response) {
				dojo.cookie(MSG_COOKIE_NAME, this.messages.TeamAreaEditor_29);
				this._unregisterHandlers();
				document.location.href = "#action=" + TEAM_AREA_EDITOR + "&itemId=" + response + "&projectAreaItemId=" + this.projectAreaItemId; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			} else {
				this.handleServerResponseSuccess(response);
				dojo.forEach(this._saveParticipants, function(saveParticipant){
					saveParticipant.saved(response);
				});
			}
		},
		
		checkForSuccessMsg: function() {
			var msgCookie = dojo.cookie(MSG_COOKIE_NAME);
			if (msgCookie) {
				this._displayLongSuccessMessage(msgCookie);
				var now = new Date();
				now.setTime(now.getTime() - 1);
				dojo.cookie(MSG_COOKIE_NAME, ProcessConstants.EMPTY, {
					expires: now
				});
			}
		},
		
		getSaveErrorMessage: function(err){
			return this.messages.TeamAreaEditor_31;
		},
		
		enableSaveButton: function(isDirty){
			if (isDirty && this._canSave()) {
				this.saveButton.disabled = false;
			} else {
				this.saveButton.disabled = true;
			}
		},
		
		_getProcessAreaStore: function(){
			var args = {
				editorContext: ProcessConstants.EditorContext.TEAM,
				processAreaItemId: this.itemId,
				errorContext: this,
				errorCallback: this._handleCreateStoreError
			};
			return new com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeStore(args);
		},
		
		createCustomTabs: function(){
			this._loadRegistryTabs();
		},
		
		_createForCurrentEditor: function(createForTeamAreaEditor){
			return createForTeamAreaEditor;
		},
		
		getDeferredSaveCount: function(response){
			if (!this.isNew || response) {
				var count = 1;
				if (this._tabs) {
					for (tabId in this._tabs) {
						var tab = this._tabs[tabId];
						if (this.isActivatedExtensionTab(tab)) {
							if (tab.mainPage.changesPending()) {
								count++;
							}
						}
					}
				}
				if (this.discardPending) {
					count++; //delete of the process description
				}
				return count;
			} else {
				return 1;
			}
		},

		getTabUAKey: function(tabKey) {
			if (tabKey === ProcessConstants.TabEnum.OVERVIEW) {
				return ProcessConstants.UAKey.OVERVIEW_TEAMAREA_TAB;
			} else if (tabKey === ProcessConstants.TabEnum.ROLES) {
				return ProcessConstants.UAKey.ROLES_TEAMAREA_TAB;
			} if (tabKey === ProcessConstants.TabEnum.PERMISSIONS) {
				return ProcessConstants.UAKey.PERMISSIONS_TEAMAREA_TAB;
			} 
			return undefined;
		},
		
		getRegistryTabUAKey: function(extensionElement) {
			var uaKey = extensionElement.getAttribute(ProcessConstants.ProjectAreaExtensionPage.ATTRIBUTE_UA_KEY_TEAM_AREA);
			return uaKey;
		}
		
	});
	
})();
