/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.TimelineStore"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeStore"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var PENDING_TIMELINE_ITEM_ID = "com.ibm.team.process.web.ui.internal.admin.projects.TimelineStore.pending"; //$NON-NLS-1$
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.TimelineStore", com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeStore, { //$NON-NLS-1$
		/*
		 * Constructs the Timeline store
		 * @param {Object} args  No special args at this level see base class for any required args
		 */
		constructor: function(args){
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		_createPendingItem: function(){
			this.newItem({
				id: PENDING_TIMELINE_ITEM_ID,
				name: this.messages.TimelineStore_PendingTimelineLabel,
				enabledContext: ProcessConstants.PermissionContext.DEV_LINE,
				archived: false
			});
			
		},
		
		_deletePendingItem: function(){
			var pendingItem;
			this.fetchItemByIdentity({
				identity: PENDING_TIMELINE_ITEM_ID,
				onItem: function(item, request){
					pendingItem = item;
				}
			});
			
			if (pendingItem) {
				this.deleteItem(pendingItem);
				this.save();
			}
			
		},
		
		addNewTimeline: function(timeline){
			var self = this;
			// Can be only one project time line so unmark any others in the store
			if (timeline.projectTimeline === true) {
				this.clearProjectTimeline();
			}
			
			
			// Add the new timeline
			var newChildItem;
			this.fetchItemByIdentity({
				identity: ProcessConstants.PermissionsScopeContext.DEV_LINES,
				onItem: function(item, request){
					newChildItem = self.newItem({
						name: timeline.name,
						id: ProcessConstants.EditorMode.NEW + new Date().getTime(),
						stateId: ProcessConstants.EditorMode.NEW,
						parent: item.id,
						context: ProcessConstants.PermissionContext.TEAM,
						contextId: timeline.id,
						enabledContext: ProcessConstants.PermissionContext.DEV_LINE,
						overrideAllowed: true,
						projectTimeline: timeline.projectTimeline,
						startDate: timeline.startDate,
						endDate: timeline.endDate,
						archived: false
					}, {
						parent: item,
						attribute: "items" //$NON-NLS-1$
					});
					
				}
			});
			return newChildItem;
			
		},
		
		updateTimeline: function(timeline, timelineData){
			// Can be only one project time line so unmark any others in the store
			if (timelineData.projectTimeline === true) {
				this.clearProjectTimeline();
			}
			var contextId = this.getValue(timeline, "contextId"); //$NON-NLS-1$
			this.setValue(timeline, "contextId", timelineData.contextId); //$NON-NLS-1$
			if (contextId !== timelineData.contextId) {
				// Context id changed...recompute for the children
				this._recomputeContextIdForChildren(timelineData.contextId, this.getValues(timeline, ProcessConstants.PermissionDTO.ITEMS));
			}
			this.setValue(timeline, "name", timelineData.name); //$NON-NLS-1$
			this.setValue(timeline, "startDate", timelineData.startDate); //$NON-NLS-1$
			this.setValue(timeline, "endDate", timelineData.endDate); //$NON-NLS-1$
			this.setValue(timeline, "projectTimeline", timelineData.projectTimeline);//$NON-NLS-1$
		},
		
		_recomputeContextIdForChildren: function(parentContextId, children){
			if (children === undefined) {
				return;
			}
			dojo.forEach(children, function(child){
				var iterationContextId = this._getIterationId(this.getValue(child, "contextId")); //$NON-NLS-1$
				iterationContextId = parentContextId + ProcessConstants.PERMISSION_PATH_SEPARATOR + iterationContextId;
				this.setValue(child, "contextId", iterationContextId); //$NON-NLS-1$
				this._recomputeContextIdForChildren(iterationContextId, this.getValues(child, ProcessConstants.PermissionDTO.ITEMS));
			}, this);
		},
		
		_getIterationId: function(contextId){
			var segments = contextId.split(ProcessConstants.PERMISSION_PATH_SEPARATOR);
			return segments[segments.length - 1];
		},
		
		setProjectTimeline: function(timeline){
			// Can be only one project time line so unmark any others in the store			
			this.clearProjectTimeline();
			this.setValue(timeline, "projectTimeline", true);//$NON-NLS-1$
		},
		
		clearProjectTimeline: function(){
			this.fetch({
				query: {
					projectTimeline: true
				},
				queryOptions: {
					deep: true
				},
				onComplete: dojo.hitch(this, function(items){
					for (var i = 0; i < items.length; i++) {
						this.setValue(items[i], "projectTimeline", false); //$NON-NLS-1$
					}
				})
			});
		},
		
		addNewIteration: function(iteration){
			var self = this;
			var newChildItem;
			this.fetchItemByIdentity({
				identity: iteration.parentId,
				onItem: function(item, request){
					newChildItem = self.newItem({
						name: iteration.name,
						id: ProcessConstants.EditorMode.NEW + new Date().getTime(),
						stateId: ProcessConstants.EditorMode.NEW,
						parent: item.id,
						context: ProcessConstants.PermissionContext.TEAM,
						contextId: iteration.id,
						enabledContext: ProcessConstants.PermissionContext.ITERATION,
						overrideAllowed: true,
						startDate: iteration.startDate,
						endDate: iteration.endDate,
						completed: iteration.completed,
						current: iteration.current,
						leaf: iteration.leaf,
						hasDeliverable: iteration.hasDeliverable,
						iterationTypeItemId: iteration.iterationTypeItemId,
						archived: false
					
					}, {
						parent: item,
						attribute: "items" //$NON-NLS-1$
					});
					
				}
			});
			return newChildItem;
			
		},
		
		updateIteration: function(iteration, iterationData){
			var contextId = this.getValue(iteration, "contextId"); //$NON-NLS-1$
			if (contextId !== iterationData.contextId) {
				var parent = this.getParent(iteration);
				var parentContextId = this.getValue(parent, "contextId"); //$NON-NLS-1$
				contextId = parentContextId + ProcessConstants.PERMISSION_PATH_SEPARATOR + iterationData.contextId;
				this.setValue(iteration, "contextId", contextId); //$NON-NLS-1$
				this._recomputeContextIdForChildren(contextId, this.getValues(iteration, ProcessConstants.PermissionDTO.ITEMS));
			}
			this.setValue(iteration, "name", iterationData.name); //$NON-NLS-1$
			this.setValue(iteration, "startDate", iterationData.startDate); //$NON-NLS-1$
			this.setValue(iteration, "endDate", iterationData.endDate); //$NON-NLS-1$
			this.setValue(iteration, "hasDeliverable", iterationData.hasDeliverable);//$NON-NLS-1$
			this.setValue(iteration, "iterationTypeItemId", iterationData.iterationTypeItemId);//$NON-NLS-1$
		},
		
		setCurrentIteration: function(iteration){
			// Can be only one current iteration in the development line			
			this.clearCurrentIteration(iteration);
			
			// Set the new iteration and it's parent iterations as the current iteration
			this.setValue(iteration, "current", true);//$NON-NLS-1$
			this.setValue(iteration, "leaf", true);//$NON-NLS-1$
			var parent = this.getParent(iteration);
			while (parent && this.isIteration(parent)) {
				this.setValue(parent, "current", true);//$NON-NLS-1$
				parent = this.getParent(parent);
			}
			
			// Mark the iterations before the current iteration as complete
			var timeline = parent;
			this._markCompletedIterations(this.getValues(timeline, ProcessConstants.PermissionDTO.ITEMS));
			
			// Mark all children of the current iteration as not complete
			this._markNonCompletedIterations(this.getValues(iteration, ProcessConstants.PermissionDTO.ITEMS));
			
			
			// Mark all children that follow the current iteration in the timeline as not complete
			var iterations = this.getValues(timeline, ProcessConstants.PermissionDTO.ITEMS);
			if (iterations) {
				for (var i = 0; i < iterations.length; i++) {
					iteration = iterations[i];
					if (this.getValue(iteration, "current") === true) { //$NON-NLS-1$
						break;
					}
				}
				var nonCompleteIterations = iterations.slice(i);
				this._markNonCompletedIterations(nonCompleteIterations);
			}
		},
		
		_markCompletedIterations: function(iterations){
			if (iterations === undefined) {
				return;
			}
			for (var i = 0; i < iterations.length; i++) {
				var iteration = iterations[i];
				if (this.getValue(iteration, "current") === true) { //$NON-NLS-1$
					// Only iterations before the current are marked as completed
					break;
				} else if (this.getValue(iteration, "completed") !== true) { //$NON-NLS-1$
					this.setValue(iteration, "completed", true); //$NON-NLS-1$
					this._markCompletedIterations(this.getValues(iteration, ProcessConstants.PermissionDTO.ITEMS));
				}
			}
			
		},
		
		_markNonCompletedIterations: function(iterations){
			if (iterations === undefined) {
				return;
			}
			for (var i = 0; i < iterations.length; i++) {
				var iteration = iterations[i];
				if (this.getValue(iteration, "completed") === true) { //$NON-NLS-1$
					this.setValue(iteration, "completed", false); //$NON-NLS-1$
					this._markNonCompletedIterations(this.getValues(iteration, ProcessConstants.PermissionDTO.ITEMS));
				}
			}
			
		},
		
		clearCurrentIteration: function(iteration){
			var timeline = this.getTimelineForIteration(iteration);
			var currentIteration;
			this.fetch({
				query: {
					current: true,
					leaf: true
				},
				queryOptions: {
					deep: true
				},
				onComplete: dojo.hitch(this, function(items){
					for (var i = 0; i < items.length; i++) {
						if (timeline === this.getTimelineForIteration(items[i])) {
							currentIteration = items[i];
						}
					}
				})
			});
			if (currentIteration) {
				this.setValue(currentIteration, "current", false);//$NON-NLS-1$
				this.setValue(currentIteration, "leaf", false);//$NON-NLS-1$
				var parent = this.getParent(currentIteration);
				while (parent && this.isIteration(parent)) {
					this.setValue(parent, "current", false);//$NON-NLS-1$
					parent = this.getParent(parent);
				}
			}
		}
		
	});
	
})();
