/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.TimelineTree"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeTree"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var TimlineTreeCSSClass = {
		NEW_PROCESS_ITEM: 'newProcessItem' //$NON-NLS-1$
	};
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.TimelineTree", [com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeTree], { //$NON-NLS-1$
		
		getLabel: function(item){
			var label = this._getDisplayLabel(item);
			if (item.enabledContext == ProcessConstants.PermissionContext.DEV_LINE || item.enabledContext == ProcessConstants.PermissionContext.ITERATION) {
				var dateDecorator = this._getDateDecorator(item);
				if (item.projectTimeline == "true") { //$NON-NLS-1$
					label = dojo.string.substitute(this.messages.TimelineTree_TimelineDecoratorText, [label, this.messages.TimelineTree_ProjectTimelineDecoratorText, dateDecorator]);
				} else {
					label = dojo.string.substitute(this.messages.TimelineTree_IterationDecoratorText, [label, dateDecorator]);
				}
			}
			return label;
		},
		
		getLabelClass: function(item, opened){
			if (item && !item.root) {
				var store = this.model.store;
				if (store.isTimeline(item) || store.isIteration(item)) { //$NON-NLS-1$
					if (this.isNewItem(item)) {
						return TimlineTreeCSSClass.NEW_PROCESS_ITEM;
					}
				}
				
				return this.inherited(arguments);
				
			}
		},
		
		addNewTimeline: function(timeline){
			if (this.lastFocused && this.lastFocused.labelNode) {
				this.lastFocused.setSelected(false);
			}
			var newTimeline = this.model.store.addNewTimeline(timeline);
			return newTimeline;
		},
		
		addNewIteration: function(iteration){
			if (this.lastFocused && this.lastFocused.labelNode) {
				this.lastFocused.setSelected(false);
				if (!this.lastFocused.isExpanded) {
					this.lastFocused.expand();
				}
			}
			var parentItem = this.getFocusedItem();
			var parentId = this.model.getIdentity(parentItem);
			iteration.parentId = parentId;
			var parentContextId = this.model.store.getValue(parentItem, "contextId"); //$NON-NLS-1$
			iteration.id = parentContextId + ProcessConstants.PERMISSION_PATH_SEPARATOR + iteration.id;
			//  Fatty doesn't change the state of the icon until after save...should we?
			//  Commenting out until we decide
//			iteration.current = this.model.store.getValue(parentItem, "current") ? true : false; //$NON-NLS-1$
//			iteration.completed = this.model.store.getValue(parentItem, "completed") ? true : false; //$NON-NLS-1$
//			if (this.model.store.getValue(parentItem, "leaf")) { //$NON-NLS-1$
//				iteration.leaf = true;
//				this.model.store.setValue(parentItem, "leaf", false); //$NON-NLS-1$
//			} else {
//				iteration.leaf = false;
//			}
			var newIteration = this.model.store.addNewIteration(iteration);
			return newIteration;
		},
		
		updateIteration: function(iteration, iterationData) {
			var store = this.model.store;
			//  No Edit support for the id right now so don't update it unless the iteration is new. 
			//  Especially since we stripped off the entire context path for existing
			if (!this.isNewItem(iteration)) {
				iterationData.contextId = store.getValue(iteration, "contextId"); //$NON-NLS-1$
			}
			store.updateIteration(iteration, iterationData);
		},

		deleteSelectedItem: function(){
			if (this.lastFocused) {
				this.model.store.deleteItem(this.lastFocused.item);
			}
		},
		
		archiveSelectedItem: function() {
			if (this.lastFocused) {
				this.model.store.archiveItem(this.lastFocused.item);
			}
		},
		
		unarchiveSelectedItem: function() {
			if (this.lastFocused) {
				this.model.store.unarchiveItem(this.lastFocused.item);
			}
		},
		
		getFocusedItem: function(){
			if (this.lastFocused) {
				return this.lastFocused.item;
			}
			return null;
		},
		
		isNewItem: function(item){
			return this.model.store.isNewItem(item);
		},
		
		isAnyAncestorArchived: function(item){
			var parent = this.model.store.getParent(item);
			while (parent) {
				if (this.model.store.getValue(parent, "archived")) { //$NON-NLS-1$
					return true;
				}
				parent = this.model.store.getParent(parent);
			}
			return false;
		},
		
		_getDateDecorator: function(item){
			var dateDecorator = ""; //$NON-NLS-1$
			var store = this.model.store;
			var startDate;
			var endDate;
			if (store.hasAttribute(item, "startDate")) { //$NON-NLS-1$	
				startDate = this._formatDate(store.getValue(item, "startDate")); //$NON-NLS-1$
				if (store.hasAttribute(item, "endDate")) { //$NON-NLS-1$	
					endDate = this._formatDate(store.getValue(item, "endDate")); //$NON-NLS-1$
				}
			}
			if (startDate && startDate !== this.messages.TimelineTree_DateUnknown) {
				dateDecorator = dojo.string.substitute(this.messages.TimelineTree_DateDecorator, [startDate, endDate]);
			}
			return dateDecorator;
		},
		
		_formatDate: function(date){
			if (date) {
				return dojo.date.locale.format(date, {
					selector: "date", //$NON-NLS-1$
					formatLength: "short", //$NON-NLS-1$
					fullYear: true
				});
			} else {
				return this.messages.TimelineTree_DateUnknown;
			}
		},
		
		uninitialize: function(){
			// Place holder for now
			this.inherited(arguments);
		}
			
	});
})();
