/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper");//$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	
	function isNode(o){
		return o && o.nodeType && o.cloneNode;
	}
	
	com.ibm.team.process.web.ui.internal.admin.projects.UIHelper = {
	
		/*
		 * Retrieves the width of the longest string in the array of
		 * strings passed to it. Can be used to determine the correct size for buttons.
		 * Returns the width in EM.
		 */
		getMaxStringDisplayWidthInEms: function(stringArray, testNode, pxMinWidth){
			if (!isNode(testNode)) {
				return -1;
			}
			var tester = document.createElement("div"); //$NON-NLS-1$
			dojo.style(tester, "position", "absolute"); //$NON-NLS-1$ //$NON-NLS-2$
			dojo.style(tester, "left", "-10000px"); //$NON-NLS-1$ //$NON-NLS-2$
			dojo.style(tester, "pad", "0"); //$NON-NLS-1$ //$NON-NLS-2$
			dojo.style(tester, "border", "0"); //$NON-NLS-1$ //$NON-NLS-2$
			testNode.appendChild(tester);
			var width = pxMinWidth || -1;
			
			// Find the longest string
			dojo.forEach(stringArray, function(sample){
				var sampleDiv = document.createElement("div"); //$NON-NLS-1$
				dojo.style(sampleDiv, "float", "right"); //$NON-NLS-1$ //$NON-NLS-2$
				sampleDiv.innerHTML = sample;
				tester.appendChild(sampleDiv);
				width = sampleDiv.clientWidth > width ? sampleDiv.clientWidth : width;
				tester.removeChild(sampleDiv);
				sampleDiv = null;
			});
			
			// Compute pixels per EM
			tester.style.height = '100em'; //$NON-NLS-1$
			var emSize = tester.offsetHeight / 100;
			
			testNode.removeChild(tester);
			
			return width / emSize;
		},
		
		/*
		 *  Helper to style the button for I18N support.  Adds the desired pad
		 *  to the width and sets the button width in em.  Removes any left
		 *  or right pad on the button to ensure that it doesn't eat into the
		 *  content space
		 */
		styleI18NButton: function(button, emWidth, emPad){
			if (emPad === undefined) {
				emPad = 1.5; //  Set a default pad
			}
			var width = emWidth + emPad + "em"; //$NON-NLS-1$
			dojo.style(button, "width", width); //$NON-NLS-1$
			dojo.style(button, "paddingLeft", 0); //$NON-NLS-1$
			dojo.style(button, "paddingRight", 0); //$NON-NLS-1$
		},
		
		/*
		 * Resize the buttons in the array to accomodate the width of their labels.
		 *
		 * testNode - The node that can be used to calculate the required button width
		 * buttons - The array of buttons with their labels set
		 * pxMinWidth - The optional min width of the buttons in pixels
		 * emPad - The option left and right pad to give the button on each side of label
		 */
		resizeI18NButtons: function(testNode, buttons, pxMinWidth, emPad){
			var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
			var labels = [];
			for (var i = 0; i < buttons.length; i++) {
				labels[i] = buttons[i].innerHTML;
			}
			var newWidth = UIHelper.getMaxStringDisplayWidthInEms(labels, testNode, pxMinWidth);
			dojo.forEach(buttons, function(button){
				UIHelper.styleI18NButton(button, newWidth, emPad);
			});
			return newWidth;
		},
		
		/*
		 * Invokes a function catching all errors and logging them to the console.  In the 
		 * event the execution environment does not include a console then the error
		 * is simply rethrown.
		 *
		 * aFunction - the function to invoke in a safe manner
		 */
		runSafe: function(aFunction) {
			try {
				aFunction();
			} catch(error){
				if (console && console.error) {
					console.error(error);
				} else {
					throw error;
				}				
			}
		},
		
		/*
		 * Returns the namespace constant for the supplied link type.
		 *
		 * linkType - one of ProcessConstants.LinkTypes
		 */
		getNamespaceForLinkType: function(linkType){
			switch (linkType) {
				case ProcessConstants.LinkTypes.IMPLEMENTS:
					return ProcessConstants.NAMESPACE_OSLC_RM;
				case ProcessConstants.LinkTypes.IMPLEMENTED_BY:
					return ProcessConstants.NAMESPACE_OSLC_CM;
				case ProcessConstants.LinkTypes.VALIDATES:
					return ProcessConstants.NAMESPACE_OSLC_RM;
				case ProcessConstants.LinkTypes.VALIDATED_BY:
					return ProcessConstants.NAMESPACE_OSLC_QM;
				case ProcessConstants.LinkTypes.TESTS:
					return ProcessConstants.NAMESPACE_OSLC_CM;
				case ProcessConstants.LinkTypes.TESTED_BY:
					return ProcessConstants.NAMESPACE_OSLC_QM;
				case ProcessConstants.LinkTypes.RELATED_TO:
					return ProcessConstants.NAMESPACE_OSLC_CM;
				case ProcessConstants.LinkTypes.TRACKS:
					return ProcessConstants.NAMESPACE_OSLC_QM;
				case ProcessConstants.LinkTypes.TRACKED_BY:
					return ProcessConstants.NAMESPACE_OSLC_CM;
				case ProcessConstants.LinkTypes.TRACKS_CHANGES:
					return ProcessConstants.NAMESPACE_OSLC_SCM;
				case ProcessConstants.LinkTypes.CHANGES_TRACKED_BY:
					return ProcessConstants.NAMESPACE_OSLC_CM;
				case ProcessConstants.LinkTypes.TRACKS_RM:
					return ProcessConstants.NAMESPACE_OSLC_RM;
				case ProcessConstants.LinkTypes.TRACKED_BY_CM:
					return ProcessConstants.NAMESPACE_OSLC_CM;
				case ProcessConstants.LinkTypes.ELABORATES:
					return ProcessConstants.NAMESPACE_OSLC_RM
				case ProcessConstants.LinkTypes.ELABORATED_BY:
					return ProcessConstants.NAMESPACE_OSLC_AM;
				case ProcessConstants.LinkTypes.ARCHITECTS:
					return ProcessConstants.NAMESPACE_OSLC_CM;
				case ProcessConstants.LinkTypes.ARCHITECTED_BY:
					return ProcessConstants.NAMESPACE_OSLC_AM;
				case ProcessConstants.LinkTypes.VALIDATES_AM:
					return ProcessConstants.NAMESPACE_OSLC_AM;
				case ProcessConstants.LinkTypes.VALIDATED_BY_AM:
					return ProcessConstants.NAMESPACE_OSLC_QM;
			}
		},
		
		getTargetOSLCCoreNamespaceForLinkType: function(linkType){
			// Summary: 
			// 	Returns the OSLC Core Namespace of the TARGET for the supplied link type.
			// 	linkType - one of ProcessConstants.LinkTypes
			switch (linkType) {
				case ProcessConstants.LinkTypes.IMPLEMENTS:
					return ProcessConstants.NAMESPACE_OSLC_CORE_RM;
				case ProcessConstants.LinkTypes.IMPLEMENTED_BY:
					return ProcessConstants.NAMESPACE_OSLC_CORE_CM;
				case ProcessConstants.LinkTypes.VALIDATES:
					return ProcessConstants.NAMESPACE_OSLC_CORE_RM;
				case ProcessConstants.LinkTypes.VALIDATED_BY:
					return ProcessConstants.NAMESPACE_OSLC_CORE_QM;
				case ProcessConstants.LinkTypes.TESTS:
					return ProcessConstants.NAMESPACE_OSLC_CORE_CM;
				case ProcessConstants.LinkTypes.TESTED_BY:
					return ProcessConstants.NAMESPACE_OSLC_CORE_QM;
				case ProcessConstants.LinkTypes.RELATED_TO:
					return ProcessConstants.NAMESPACE_OSLC_CORE_CM;
				case ProcessConstants.LinkTypes.TRACKS:
					return ProcessConstants.NAMESPACE_OSLC_CORE_QM;
				case ProcessConstants.LinkTypes.TRACKED_BY:
					return ProcessConstants.NAMESPACE_OSLC_CORE_CM;
				case ProcessConstants.LinkTypes.TRACKS_CHANGES:
					return ProcessConstants.NAMESPACE_OSLC_CORE_SCM;
				case ProcessConstants.LinkTypes.CHANGES_TRACKED_BY:
					return ProcessConstants.NAMESPACE_OSLC_CORE_CM;
				case ProcessConstants.LinkTypes.TRACKS_RM:
					return ProcessConstants.NAMESPACE_OSLC_CORE_RM;
				case ProcessConstants.LinkTypes.TRACKED_BY_CM:
					return ProcessConstants.NAMESPACE_OSLC_CORE_CM;
				case ProcessConstants.LinkTypes.ELABORATES:
					return ProcessConstants.NAMESPACE_OSLC_CORE_RM;
				case ProcessConstants.LinkTypes.ELABORATED_BY:
					return ProcessConstants.NAMESPACE_OSLC_AM;
				case ProcessConstants.LinkTypes.ARCHITECTS:
					return ProcessConstants.NAMESPACE_OSLC_CORE_CM;
				case ProcessConstants.LinkTypes.ARCHITECTED_BY:
					return ProcessConstants.NAMESPACE_OSLC_AM;
				case ProcessConstants.LinkTypes.VALIDATES_AM:
					return ProcessConstants.NAMESPACE_OSLC_AM;
				case ProcessConstants.LinkTypes.VALIDATED_BY_AM:
					return ProcessConstants.NAMESPACE_OSLC_CORE_QM;
			}
		},
				
		getTagNameForNamespace: function(namespace){
			// Summary: 
			//	Returns the tag name for the given namespace.
			switch(namespace){
				case ProcessConstants.NAMESPACE_OSLC_CM:
					return ProcessConstants.ServiceProviderTypes.CM;
				case ProcessConstants.NAMESPACE_OSLC_CORE_CM:
					return ProcessConstants.ServiceProviderTypes.CM;
				case ProcessConstants.NAMESPACE_OSLC_RM:
					return ProcessConstants.ServiceProviderTypes.RM;
				case ProcessConstants.NAMESPACE_OSLC_CORE_RM:
					return ProcessConstants.ServiceProviderTypes.RM;
				case ProcessConstants.NAMESPACE_OSLC_QM:
					return ProcessConstants.ServiceProviderTypes.QM;
				case ProcessConstants.NAMESPACE_OSLC_CORE_QM:
					return ProcessConstants.ServiceProviderTypes.QM;
				case ProcessConstants.NAMESPACE_OSLC_SCM:
					return ProcessConstants.ServiceProviderTypes.SCM;
				case ProcessConstants.NAMESPACE_OSLC_CORE_SCM:
					return ProcessConstants.ServiceProviderTypes.SCM;
				case ProcessConstants.NAMESPACE_OSLC_AM:
					return ProcessConstants.ServiceProviderTypes.AM;
			}
		},
		
		
		/*
		 * Returns the tag name for the supplied link type.
		 *
		 * linkType - one of ProcessConstants.LinkTypes
		 */
		getTagNameForLinkType: function(linkType){
			switch (linkType) {
				case ProcessConstants.LinkTypes.IMPLEMENTS:
					return ProcessConstants.ServiceProviderTypes.RM;
				case ProcessConstants.LinkTypes.IMPLEMENTED_BY:
					return ProcessConstants.ServiceProviderTypes.CM;
				case ProcessConstants.LinkTypes.VALIDATES:
					return ProcessConstants.ServiceProviderTypes.RM;
				case ProcessConstants.LinkTypes.VALIDATED_BY:
					return ProcessConstants.ServiceProviderTypes.QM;
				case ProcessConstants.LinkTypes.TESTS:
					return ProcessConstants.ServiceProviderTypes.CM;
				case ProcessConstants.LinkTypes.TESTED_BY:
					return ProcessConstants.ServiceProviderTypes.QM;
				case ProcessConstants.LinkTypes.RELATED_TO:
					return ProcessConstants.ServiceProviderTypes.CM;
				case ProcessConstants.LinkTypes.TRACKS:
					return ProcessConstants.ServiceProviderTypes.QM;
				case ProcessConstants.LinkTypes.TRACKED_BY:
					return ProcessConstants.ServiceProviderTypes.CM;
				case ProcessConstants.LinkTypes.TRACKS_CHANGES:
					return ProcessConstants.ServiceProviderTypes.SCM;
				case ProcessConstants.LinkTypes.CHANGES_TRACKED_BY:
					return ProcessConstants.ServiceProviderTypes.CM;
				case ProcessConstants.LinkTypes.TRACKS_RM:
					return ProcessConstants.ServiceProviderTypes.RM;
				case ProcessConstants.LinkTypes.TRACKED_BY_CM:
					return ProcessConstants.ServiceProviderTypes.CM;
				case ProcessConstants.LinkTypes.ELABORATES:
					return ProcessConstants.ServiceProviderTypes.RM
				case ProcessConstants.LinkTypes.ELABORATED_BY:
					return ProcessConstants.ServiceProviderTypes.AM;
				case ProcessConstants.LinkTypes.ARCHITECTS:
					return ProcessConstants.ServiceProviderTypes.CM;
				case ProcessConstants.LinkTypes.ARCHITECTED_BY:
					return ProcessConstants.ServiceProviderTypes.AM;
				case ProcessConstants.LinkTypes.VALIDATES_AM:
					return ProcessConstants.ServiceProviderTypes.AM;
				case ProcessConstants.LinkTypes.VALIDATED_BY_AM:
					return ProcessConstants.ServiceProviderTypes.QM;
			}
		},
		
		getLinkTypesForTagName: function(serviceProviderTag){
			if (serviceProviderTag == ProcessConstants.ServiceProviderTypes.CM) {
				return [ProcessConstants.LinkTypes.TESTED_BY, ProcessConstants.LinkTypes.IMPLEMENTS, ProcessConstants.LinkTypes.RELATED_TO, ProcessConstants.LinkTypes.TRACKS, ProcessConstants.LinkTypes.TRACKS_CHANGES, ProcessConstants.LinkTypes.TRACKS_RM, ProcessConstants.LinkTypes.ARCHITECTED_BY];
			} else if (serviceProviderTag == ProcessConstants.ServiceProviderTypes.QM) {
				return [ProcessConstants.LinkTypes.TESTS, ProcessConstants.LinkTypes.VALIDATES, ProcessConstants.LinkTypes.TRACKED_BY, ProcessConstants.LinkTypes.VALIDATES_AM];
			} else if (serviceProviderTag == ProcessConstants.ServiceProviderTypes.RM) {
				return [ProcessConstants.LinkTypes.IMPLEMENTED_BY, ProcessConstants.LinkTypes.VALIDATED_BY, ProcessConstants.LinkTypes.TRACKED_BY_CM, ProcessConstants.LinkTypes.ELABORATED_BY];
			} else if (serviceProviderTag == ProcessConstants.ServiceProviderTypes.SCM) {
				return [ProcessConstants.LinkTypes.CHANGES_TRACKED_BY];
			} else if (serviceProviderTag == ProcessConstants.ServiceProviderTypes.AM) {
				return [ProcessConstants.LinkTypes.ELABORATES, ProcessConstants.LinkTypes.ARCHITECTS, ProcessConstants.LinkTypes.VALIDATED_BY_AM];
			}
			return [];
		},
		
		/*
		 * Returns the node from the collection that has the matching baseName and namespace URI.
		 * undefined is returned if no match is found.
		 *
		 * collection - the collection to search
		 * baseName - the name (local name) of the node to match
		 * namespaceURI - the namespace URI to match
		 */
		getNode: function(collection, baseName, namespaceURI){
			var matchingNode;
			dojo.forEach(collection, function(node){
				if ((node.baseName == baseName || node.localName == baseName) && (node.namespaceURI == namespaceURI || (node.namespaceURI === "" && namespaceURI === null))) { //$NON-NLS-1$
					matchingNode = node;
				}
			});
			return matchingNode;
		},
		
		/*
		 * Returns the nodes from the collection that have the matching baseName and namespace URI.
		 * An empty collection is returned if no matches are found.
		 *
		 * collection - the collection to search
		 * baseName - the name (local name) of the nodes to match
		 * namespaceURI - the namespace URI to match
		 */
		getNodes: function(collection, baseName, namespaceURI){
			var matchingNodes = [];
			dojo.forEach(collection, function(node){
				if ((node.baseName == baseName || node.localName == baseName) && (node.namespaceURI == namespaceURI || (node.namespaceURI === "" && namespaceURI === null))) { //$NON-NLS-1$
					matchingNodes.push(node);
				}
			});
			return matchingNodes;
		},
		
		/*
		 * Returns the attribute node of the node that has the matching baseName and namespace.
		 * undefined is returned if no match is found.
		 *
		 * node - the node whose attributes are to be searched
		 * name - the name of the attribute
		 * namespace - the namespace to match
		 */
		getAttributeNode: function(node, name, namespace){
			var attribute;
			if (dojo.isIE) {
				attribute = node.attributes.getQualifiedItem(name, namespace);
			} else {
				attribute = node.attributes.getNamedItemNS(namespace, name);
			}
			return attribute;
		},

		/*
		 * Returns the attribute value of the node that has the matching baseName and namespace.
		 * null is returned if no match is found.
		 *
		 * node - the node whose attributes are to be searched
		 * name - the name of the attribute
		 * namespace - the namespace to match
		 */
		getAttributeNodeValue: function(node, name, namespace){
			var attribute = this.getAttributeNode(node, name, namespace);
			if (attribute) {
				return attribute.value;
			} else {
				return null;
			}
		},
		
		/*
		 * Sets the required attributes of the specified service provider tree
		 * node from the XML specification.
		 *
		 * serviceProviderTreeNode - the node whose attributes are to be set
		 * serviceProvider - the DOM representation of the service provider
		 * oslcSpecVersion - OSLC spec version of the provided service provider
		 * serviceProviderCatalogTagName - Catalog tag name containing this service provider
		 */
		configureServiceProviderTreeNode: function(serviceProviderTreeNode, serviceProvider, oslcSpecVersion, serviceProviderCatalogTagName){
			var serviceProviderChildren = serviceProvider.childNodes;
			var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
			var titleNode = UIHelper.getNode(serviceProviderChildren, "title", ProcessConstants.NAMESPACE_DC); //$NON-NLS-1$
			var name = titleNode.firstChild.data;
			name = decodeURIComponent(name);
			
			var detailsNode = null;
			if (oslcSpecVersion && oslcSpecVersion === ProcessConstants.OSLC_CORE_VERSION) {
				detailsNode = UIHelper.getNode(serviceProviderChildren, "details", ProcessConstants.NAMESPACE_OSLC_CORE); //$NON-NLS-1$
			} else {
				detailsNode = UIHelper.getNode(serviceProviderChildren, "details", ProcessConstants.NAMESPACE_OSLC); //$NON-NLS-1$
			}
			var attributeNode = UIHelper.getAttributeNode(detailsNode, "resource", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
			var url = attributeNode.value;
			var presentationUrl = url;
			
			var servicesUrl = null;
			if (oslcSpecVersion && oslcSpecVersion === ProcessConstants.OSLC_CORE_VERSION) {
				attributeNode = UIHelper.getAttributeNode(serviceProvider, "about", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
				servicesUrl = attributeNode.value;
			} else {
				var servicesNode = UIHelper.getNode(serviceProviderChildren, "services", ProcessConstants.NAMESPACE_OSLC); //$NON-NLS-1$
				attributeNode = UIHelper.getAttributeNode(servicesNode, "resource", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
				servicesUrl = attributeNode.value;
			}
			
			var consumerRegistryNode = UIHelper.getNode(serviceProviderChildren, "consumerRegistry", ProcessConstants.NAMESPACE_JP); //$NON-NLS-1$
			var consumerRegistryUrl;
			if (consumerRegistryNode) {
				attributeNode = UIHelper.getAttributeNode(consumerRegistryNode, "resource", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
				consumerRegistryUrl = attributeNode.value;
			}
			
			var date = UIHelper.getUniqueTimeStamp();
			
			serviceProviderTreeNode.name = [name];
			serviceProviderTreeNode.id = [name + date.getTime()];
			serviceProviderTreeNode.type = [ProcessConstants.SERVICE_PROVIDER_TYPE];
			serviceProviderTreeNode.url = [url];
			serviceProviderTreeNode.presentationUrl = [presentationUrl];
			serviceProviderTreeNode.consumerRegistryUrl = [consumerRegistryUrl];
			serviceProviderTreeNode.servicesUrl = [servicesUrl];
			serviceProviderTreeNode.serviceProviderCatalogTagName = [serviceProviderCatalogTagName];
		},
		
		/*
		 * Sets the required attributes of the specified service provider
		 * catalog tree node from the XML specification. As well, the stub node
		 * is configure to be a placeholder for the potential children of the
		 * catalog that are fetched in a deferred manner.
		 *
		 * serviceProviderCatalogTreeNode - the node whose attributes are to be set
		 * stubTreeNode - the node to represent potential children of the catalog
		 * stubNodeName - the name to use for the stub node
		 * serviceProviderCatalog - the DOM representation of the service provider catalog
		 * serviceProviderCatalogTagName - Catalog TagName containing this nested catalog
		 * serviceProviderCatalogRef - Optional. External reference of the service provider catalog. 
		 * nestedCatalogName - Optional. Name of catalog in case of external reference.
		 */
		configureServiceProviderCatalogTreeNode: function(serviceProviderCatalogTreeNode, stubNode, stubNodeName, serviceProviderCatalog, serviceProviderCatalogTagName, serviceProviderCatalogRef, nestedCatalogName){
			serviceProviderCatalogTreeNode.type = [ProcessConstants.SERVICE_PROVIDER_CATALOG_TYPE];
			serviceProviderCatalogTreeNode.children = [];
			var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
			var date = UIHelper.getUniqueTimeStamp();			
			if (serviceProviderCatalogRef) {
				serviceProviderCatalogTreeNode.name = [nestedCatalogName];
				serviceProviderCatalogTreeNode.id = [ProcessConstants.SERVICE_PROVIDER_CATALOG_TYPE + date.getTime()];
				serviceProviderCatalogTreeNode.serviceProviderCatalogURL = [serviceProviderCatalogRef];
			} else {
				var serviceProviderCatalogChildren = serviceProviderCatalog.childNodes;
				var titleNode = UIHelper.getNode(serviceProviderCatalogChildren, "title", ProcessConstants.NAMESPACE_DC); //$NON-NLS-1$
				serviceProviderCatalogTreeNode.name = [titleNode.firstChild.data];
				serviceProviderCatalogTreeNode.id = [titleNode.firstChild.data + date.getTime()];
				var attributeNode = UIHelper.getAttributeNode(serviceProviderCatalog, "about", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
				serviceProviderCatalogTreeNode.serviceProviderCatalogURL = [attributeNode.value];
			}
			serviceProviderCatalogTreeNode.serviceProviderCatalogTagName = [serviceProviderCatalogTagName];
			date = UIHelper.getUniqueTimeStamp();
			stubNode.name = [stubNodeName];
			stubNode.id = [stubNodeName + date.getTime()];
			stubNode.type = [ProcessConstants.SERVICE_PROVIDER_NOT_LOADED_TYPE];
			stubNode.serviceProviderCatalogId = serviceProviderCatalogTreeNode.id;
			
			serviceProviderCatalogTreeNode.children.push(stubNode);
		},
		
		getUniqueTimeStamp: function(){
			var millis = 2;
			var date = new Date();
			var curDate = null;
			do {
				curDate = new Date();
			} while (curDate - date < millis);
			return curDate;
		},
		
		getLinkTypeLabel: function(linkType, messages){
			switch (linkType) {
				case ProcessConstants.LinkTypes.IMPLEMENTS:
					return messages.LinkType_Implements;
				case ProcessConstants.LinkTypes.IMPLEMENTED_BY:
					return messages.LinkType_Implemented;
				case ProcessConstants.LinkTypes.VALIDATES:
					return messages.LinkType_Validates;
				case ProcessConstants.LinkTypes.VALIDATED_BY:
					return messages.LinkType_Validated;
				case ProcessConstants.LinkTypes.TESTS:
					return messages.LinkType_Tests;
				case ProcessConstants.LinkTypes.TESTED_BY:
					return messages.LinkType_Tested;
				case ProcessConstants.LinkTypes.RELATED_TO:
					return messages.LinkType_RelatedTo;
				case ProcessConstants.LinkTypes.TRACKS:
					return messages.LinkType_Tracks;
				case ProcessConstants.LinkTypes.TRACKED_BY:
					return messages.LinkType_Tracked;
				case ProcessConstants.LinkTypes.TRACKS_CHANGES:
					return messages.LinkType_TracksChanges;
				case ProcessConstants.LinkTypes.CHANGES_TRACKED_BY:
					return messages.LinkType_ChangesTrackedBy;
				case ProcessConstants.LinkTypes.TRACKS_RM:
					return messages.LinkType_Tracks_RM;
				case ProcessConstants.LinkTypes.TRACKED_BY_CM:
					return messages.LinkType_Tracked_By_CM;
				case ProcessConstants.LinkTypes.ELABORATES:
					return messages.LinkType_Elaborates;
				case ProcessConstants.LinkTypes.ELABORATED_BY:
					return messages.LinkType_ElaboratedBy;
				case ProcessConstants.LinkTypes.ARCHITECTS:
					return messages.LinkType_Architects;
				case ProcessConstants.LinkTypes.ARCHITECTED_BY:
					return messages.LinkType_ArchitectedBy;
				case ProcessConstants.LinkTypes.VALIDATED_BY_AM:
					return messages.LinkType_ValidatedByAM;
				case ProcessConstants.LinkTypes.VALIDATES_AM:
					return messages.LinkType_ValidatesAM;
			}
			return linkType;
		},
		
		getQualifiedLinkTypeLabel: function(linkType, messages){
			var linkLabel = this.getLinkTypeLabel(linkType, messages);
			var peerLinkType = this.getPeerLinkType(linkType);
			var peerLinkLabel = this.getLinkTypeLabel(peerLinkType, messages);
			if (linkLabel.localeCompare(peerLinkLabel) === 0) {
				return linkLabel;
			}
			return dojo.string.substitute(messages.LinkType_QualifiedLabel, [linkLabel, peerLinkLabel]);
		},
		
		calculateDuplicateLabels: function(linkTypes, messages){
			var labels = [];
			var duplicates = [];
			for (var i = 0; i < linkTypes.length; i++) {
				var label = this.getLinkTypeLabel(linkTypes[i], messages);
				if (dojo.indexOf(labels, label) === -1) {
					labels.push(label);
				} else {
					duplicates.push(label);
				}
			}
			return duplicates;
		},
		
		getPeerLinkType: function(linkType){
			switch (linkType) {
				case ProcessConstants.LinkTypes.IMPLEMENTS:
					return ProcessConstants.LinkTypes.IMPLEMENTED_BY;
				case ProcessConstants.LinkTypes.IMPLEMENTED_BY:
					return ProcessConstants.LinkTypes.IMPLEMENTS;
				case ProcessConstants.LinkTypes.VALIDATES:
					return ProcessConstants.LinkTypes.VALIDATED_BY;
				case ProcessConstants.LinkTypes.VALIDATED_BY:
					return ProcessConstants.LinkTypes.VALIDATES;
				case ProcessConstants.LinkTypes.TESTS:
					return ProcessConstants.LinkTypes.TESTED_BY;
				case ProcessConstants.LinkTypes.TESTED_BY:
					return ProcessConstants.LinkTypes.TESTS;
				case ProcessConstants.LinkTypes.RELATED_TO:
					return ProcessConstants.LinkTypes.RELATED_TO;
				case ProcessConstants.LinkTypes.TRACKS:
					return ProcessConstants.LinkTypes.TRACKED_BY;
				case ProcessConstants.LinkTypes.TRACKED_BY:
					return ProcessConstants.LinkTypes.TRACKS;
				case ProcessConstants.LinkTypes.TRACKS_CHANGES:
					return ProcessConstants.LinkTypes.CHANGES_TRACKED_BY;
				case ProcessConstants.LinkTypes.CHANGES_TRACKED_BY:
					return ProcessConstants.LinkTypes.TRACKS_CHANGES;
				case ProcessConstants.LinkTypes.TRACKS_RM:
					return ProcessConstants.LinkTypes.TRACKED_BY_CM;
				case ProcessConstants.LinkTypes.TRACKED_BY_CM:
					return ProcessConstants.LinkTypes.TRACKS_RM;
				case ProcessConstants.LinkTypes.ELABORATES:
					return ProcessConstants.LinkTypes.ELABORATED_BY;
				case ProcessConstants.LinkTypes.ELABORATED_BY:
					return ProcessConstants.LinkTypes.ELABORATES;
				case ProcessConstants.LinkTypes.ARCHITECTS:
					return ProcessConstants.LinkTypes.ARCHITECTED_BY;
				case ProcessConstants.LinkTypes.ARCHITECTED_BY:
					return ProcessConstants.LinkTypes.ARCHITECTS;
				case ProcessConstants.LinkTypes.VALIDATES_AM:
					return ProcessConstants.LinkTypes.VALIDATED_BY_AM;
				case ProcessConstants.LinkTypes.VALIDATED_BY_AM:
					return ProcessConstants.LinkTypes.VALIDATES_AM;
			}
			return null;
		},
		
		getProjectLinkImage: function(linkType, className){
			var img = document.createElement("img"); //$NON-NLS-1$
			img.className = className;
			var module = "com.ibm.team.process.web"; //$NON-NLS-1$
			switch (linkType) {
				case ProcessConstants.LinkTypes.IMPLEMENTED_BY:
					img.src = dojo.moduleUrl(module, "ui/internal/admin/projects/images/implemented_by_obj.gif"); //$NON-NLS-1$
					break;
				case ProcessConstants.LinkTypes.IMPLEMENTS:
					img.src = dojo.moduleUrl(module, "ui/internal/admin/projects/images/implements_obj.gif"); //$NON-NLS-1$
					break;
				case ProcessConstants.LinkTypes.TESTED_BY:
					img.src = dojo.moduleUrl(module, "ui/internal/admin/projects/images/tested_by_obj.gif"); //$NON-NLS-1$
					break;
				case ProcessConstants.LinkTypes.TESTS:
					img.src = dojo.moduleUrl(module, "ui/internal/admin/projects/images/tests_obj.gif"); //$NON-NLS-1$
					break;
				case ProcessConstants.LinkTypes.VALIDATED_BY:
					img.src = dojo.moduleUrl(module, "ui/internal/admin/projects/images/verified_by_obj.gif"); //$NON-NLS-1$
					break;
				case ProcessConstants.LinkTypes.VALIDATES:
					img.src = dojo.moduleUrl(module, "ui/internal/admin/projects/images/verifies_obj.gif"); //$NON-NLS-1$
					break;
				case ProcessConstants.LinkTypes.RELATED_TO:
					img.src = dojo.moduleUrl(module, "ui/internal/admin/projects/images/related_to_obj.gif"); //$NON-NLS-1$
					break;
				case ProcessConstants.LinkTypes.TRACKS:
					img.src = dojo.moduleUrl(module, "ui/internal/admin/projects/images/tested_by_obj.gif"); //$NON-NLS-1$
					break;
				case ProcessConstants.LinkTypes.TRACKED_BY:
					img.src = dojo.moduleUrl(module, "ui/internal/admin/projects/images/tests_obj.gif"); //$NON-NLS-1$
					break;
				case ProcessConstants.LinkTypes.TRACKS_CHANGES:
					img.src = dojo.moduleUrl(module, "ui/internal/admin/projects/images/implemented_by_obj.gif"); //$NON-NLS-1$
					break;
				case ProcessConstants.LinkTypes.CHANGES_TRACKED_BY:
					img.src = dojo.moduleUrl(module, "ui/internal/admin/projects/images/implements_obj.gif"); //$NON-NLS-1$
					break;
				case ProcessConstants.LinkTypes.TRACKS_RM:
					img.src = dojo.moduleUrl(module, "ui/internal/admin/projects/images/tested_by_obj.gif"); //$NON-NLS-1$
					break;
				case ProcessConstants.LinkTypes.TRACKED_BY_CM:
					img.src = dojo.moduleUrl(module, "ui/internal/admin/projects/images/tests_obj.gif"); //$NON-NLS-1$
					break;
				case ProcessConstants.LinkTypes.ELABORATES:
					img.src = dojo.moduleUrl(module, "ui/internal/admin/projects/images/elaborates_obj.gif"); //$NON-NLS-1$
					break;
				case ProcessConstants.LinkTypes.ELABORATED_BY:
					img.src = dojo.moduleUrl(module, "ui/internal/admin/projects/images/elaborated_by_obj.gif"); //$NON-NLS-1$
					break;
				case ProcessConstants.LinkTypes.ARCHITECTS:
					img.src = dojo.moduleUrl(module, "ui/internal/admin/projects/images/architects_obj.gif"); //$NON-NLS-1$
					break;
				case ProcessConstants.LinkTypes.ARCHITECTED_BY:
					img.src = dojo.moduleUrl(module, "ui/internal/admin/projects/images/architected_by_obj.gif"); //$NON-NLS-1$
					break;
				case ProcessConstants.LinkTypes.VALIDATED_BY_AM:
					img.src = dojo.moduleUrl(module, "ui/internal/admin/projects/images/validated_by_am_obj.gif"); //$NON-NLS-1$
					break;
				case ProcessConstants.LinkTypes.VALIDATES_AM:
					img.src = dojo.moduleUrl(module, "ui/internal/admin/projects/images/validates_am_obj.gif"); //$NON-NLS-1$
					break;
			}
			
			img.alt = linkType;
			return img;
		},
		
		isUsesLinkType: function(linkType) {
			for (var i = 0; i < ProcessConstants.UsesLinkTypes.length; i++) {
				if (linkType === ProcessConstants.UsesLinkTypes[i]) {
					return true;
				}
			}
			return false;
		},
		
		removeDuplicatedCatalogs: function(catalogs) {
			// Remove the duplicated catalogs.
			var rtnCatalogs = [];
			dojo.forEach(catalogs, function(catalog){
				if(!this.isDuplicatedCatalog(catalog, rtnCatalogs)) {
					rtnCatalogs.push(catalog);
				}
			}, this);
			return rtnCatalogs;
		},
		
		isDuplicatedCatalog: function(catalog, catalogs) {
			// Check if the given catalog exists in the catalogs array.
			return dojo.some(catalogs, function(item){
				return catalog.tagName === item.tagName;
			});
		}

	};
})();
