/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.ActiveServicesView");

dojo.require("dijit._Widget");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.util")
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "ActiveServicesViewMessages");

dojo.require("jazz.ui.MessageBox");

var MessageBox = jazz.ui.MessageBox;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var formatTimeFromMilliseconds = com.ibm.team.repository.web.ui.internal.admin.util.formatTimeFromMilliseconds;
var replaceNewLineWithPaddedBreaks = com.ibm.team.repository.web.ui.internal.admin.util.replaceNewLineWithPaddedBreaks;

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.ActiveServicesView", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/ActiveServicesView.html"),
	
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "ActiveServicesViewMessages");
	},	
	postCreate: function() {
		this.header = new MessageBox();
		this.headerArea.appendChild(this.header.domNode);
		this._fetchAndDisplayServiceInfo();
	},
	
	refresh: function () {
		this.header.hideMessage();
		this._fetchAndDisplayServiceInfo();
	},
	
	_fetchAndDisplayServiceInfo: function() {
		var srh= new ServiceResponseHandler(this, "_displayActiveServicesInfo", "serviceError");
		ServerStatusClient.getActiveServiceInfo(srh)
	},


	serviceError: function(errorObj) {
		this.header.showMessage({
			errorObject: errorObj
		});
	},

	_displayActiveServicesInfo: function(activeServiceInfo) {
		dojox.xml.parser.removeChildren(this.contentArea);
		var servicesDiv = document.createElement("div");	
		var services = activeServiceInfo.services;
		//KMB TODO: sort service list
		if (services) {
			for (var i=0; i<services.length; i++) {
				this._appendActiveServiceSection(services[i], servicesDiv, activeServiceInfo.currentTime);
			}
		}
		this.contentArea.appendChild(servicesDiv);
		this.showTimestamp();
	},
	
	_appendActiveServiceSection: function(service, node, currentTime) {
		var elapsedTime = currentTime - service.startTime;
		elapsedTime = formatTimeFromMilliseconds(elapsedTime);
		elapsedTime = dojo.string.substitute(this.messages.runningTime, [ elapsedTime ]);
		var activeService = dojo.create("div", { className: "active-service" }, node);
		var serviceTitleDiv = dojo.create("div", { className: "service-title" }, activeService);
		var div = dojo.create("div", { className: "right-area" }, serviceTitleDiv);
		dojo.create("span", { className: this.messages.requester, innerHTML: this.messages.by + service.userId }, div);
		dojo.create("span", { className: "running-time", innerHTML: elapsedTime }, div);
		var serviceName = dojo.create("div", { className: "service-name twistyClosed", innerHTML: service.service + "." + service.method }, serviceTitleDiv);
		var traceDiv = dojo.create("div", { className: "service-trace noDisplay", innerHTML: replaceNewLineWithPaddedBreaks(service.stacktrace, true) }, activeService);
		serviceTitleDiv.onclick = function(){
			dojo.toggleClass(traceDiv, "noDisplay");
			dojo.toggleClass(serviceName, "twistyClosed");
			dojo.toggleClass(serviceName, "twistyOpen");
		}
	},
	
	showTimestamp: function() {
		this.loadedTimestampArea.innerHTML = "";
		var nowTime = dojo.date.locale.format(new Date());
		var timestamp = dojo.string.substitute(this.messages.loadedTimestamp, [nowTime]);
		this.loadedTimestampArea.appendChild(document.createTextNode(timestamp));
		dojo.toggleClass(this.loadedTimestampArea, "noDisplay", false);
	},
	
	uninitialize: function () {
		this.header.destroyRecursive();
		this.inherited(arguments);
    }
});

})();