/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.AdminHome");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.ui.MessageBox");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("dojo.i18n");
dojo.require("jazz.app.internal.util.DiscoveredService");
dojo.require("jazz.ensemble.internal.util.Xml");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.schema.AdminHome");
dojo.require("jazz.app.internal.applicationAbout");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "AdminHomeMessages");

(function() {
var messages;
var MessageBox = jazz.ui.MessageBox;
var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var AdminHomeSchema = com.ibm.team.repository.web.ui.internal.admin.server.schema.AdminHome;
var DiscoveredService = jazz.app.internal.util.DiscoveredService;
var namespace = jazz.ensemble.internal.util.Xml.namespace;
var XhrUtil = jazz.app.internal.util.XhrUtil;
var LPA_IDENTIFIER = "http://jazz.net/application/lpa";

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.AdminHome", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/AdminHome.html"),
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "AdminHomeMessages");
	},
	
	postCreate: function () {
		this.header = new MessageBox();
		this.stepHeaderArea.appendChild(this.header.domNode);
		
		//Find the user's role
		this.setUserRoles();
		
		//Register action
		var actionRegistry = PlatformUI.getWorkbench().getActionRegistry();
		actionRegistry.registerAction("com.ibm.team.repository.admin.adminHome", this, "reload");
	}, 
	
	reload: function () {
		this.header.hideMessage();
		var contextRoot = net.jazz.ajax._contextRoot;
		
		//If the user is not a JazzAdmin, show an INFO message
		if(!this.isAdmin) {
			this.header.showMessage({
				type: "INFO",
				summary: this.messages.jazzUsersPermissions 
			});
			dojo.addClass(this.serverAdmin, "disabled");
			dojo.addClass(this.userAdmin, "disabled");
		} else {
			//Set the various Admin links
			this.serverAdmin.href = contextRoot + "/admin#action=jazz.viewPage&id=com.ibm.team.repository.server";
			this.userAdmin.href = contextRoot + "/admin#action=jazz.viewPage&id=com.ibm.team.repository.users";	
		}
		this._reloadApplicationLinks();
	},
	
	_testLPA: function(abouts) {
		//Let's see if the LPA application is installed
		//Note: LPA link is still valid for JazzUsers, so don't disable it.
		for(i in abouts) {
			if(abouts[i].identifier == LPA_IDENTIFIER) {
				dojo.removeClass(this.lpaSection, "noDisplay");
				var parts = i.split("/application-about");
				this.lpaURL = parts[0];
				this.lifeCycleAdmin.href = this.lpaURL + "/web#/projects";
				break;
			}
		}
	},
	
	_reloadApplicationLinks: function () {
		//dojo.addClass(this._messages,"loading");
		//this._messages.innerHTML = messages.loading;
        dojo.empty(this.applicationLinks);		
		this.services = [];
	    var deferred = DiscoveredService.loadDiscoveryServices(namespace.jfs+"AdminHome", AdminHomeSchema.discovery);

		deferred.addCallback(dojo.hitch(this, function(services) {
			//dojo.removeClass(this._messages, "loading");
			if (!services.AdminHome)	{
				console.warn("No AdminHome entries found.");
				return;
			}
			var abouts = jazz.app.internal.applicationAbout.get();
			//Use this opportunity to test for the LPA presence
			this._testLPA(abouts);

			dojo.forEach(services.AdminHome, function(service) {
				service.about = abouts[service.applicationAbout];
				service.title = service.about.title + " (" + service.publisher + ")";
				this.services.push(service);
			}, this);
			if (this.services.length) {
				this.services.sort(function(a, b){if (a.title==b.title) return 0;return (a.title<b.title)?-1:1;});
				var oldAbout = null;				
				dojo.forEach(this.services, function(service){
					this._callService(service, true, oldAbout);
					oldAbout = service.about;
				}, this);
			} else {
				//this._showNoServers();
			}
		}));
		
		deferred.addErrback(function(error){
			console.warn("[AdminHome] Service unreachable: ", error);
		}); 		
	},

	_callService: function(service, passive, oldAbout) {
		var def = XhrUtil.getXml(service.service, {schema:AdminHomeSchema.adminHomeService, passive:passive});
		def.addCallback(dojo.hitch(this, function (doc) {
			service.results = [];
			service.entryCount = 0;
			if (doc) {
				if (doc.AdminHomeEntries && doc.AdminHomeEntries.entries) {
					dojo.forEach(doc.AdminHomeEntries.entries, function(item){
						service.entryCount++;
						service.results.push(item);
					}, this);
					if (!service.entryCount) {
						service.results.splice(0, 0, "noEntries");
					}
				}
			}
			else {
				service.results.push("authRequired");
			}
			this._updateUI(service, oldAbout);
		}));
		def.addErrback(function(error) {
			// fail silently if user closed the auth popup
			if (!error || error.dojoType !== "cancel") {
				console.error(error);
			}
		});
	},

	_updateUI: function (service, oldAbout) {
		if (service.about != oldAbout) {
			// insert an application row
			var appnode = dojo.create("div", { className: "app-type" }, this.applicationLinks);
			var inode = dojo.create("img", {}, appnode);
			inode.src = service.about.icon;
			dojo.create("span", { innerHTML: service.about.title }, appnode);
			if(service.about.instanceName) {
				dojo.create("span", { className: "instanceName", innerHTML: "(" + service.about.instanceName +")" }, appnode);
			}
		}
		for (var i=0; i<service.results.length; i++) {
			var entry = service.results[i];
			var enode = dojo.create("div", { className: "app-link" }, this.applicationLinks);
			var link = dojo.create("a", { href: this.isAdmin ? entry.href : "#", innerHTML: entry.title  }, enode);
			if (!this.isAdmin) {
				dojo.addClass(link, "disabled");
				this.connect(link, "onclick", function(e) {
					if(e) e.preventDefault();
				});
			}
		}
	},
	
	setUserRoles: function() {
		var roles = dojo.getObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR_ROLES");
		this.isAdmin = false;
		this.isProjectAdmin = false;
		if (roles) {
			for (var i=0; i < roles.length; i++) {
				if (roles[i] === "JazzAdmins") {
					this.isAdmin = true;
				}
				if (roles[i] === "JazzProjectAdmins") {
					this.isProjectAdmin = true;
				}
			}
		}
	}
});

})();