/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyComponent");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyService");

(function() {

var checkNotNull = net.jazz.ajax.toolbox.lang.checkNotNull;
var ServerPropertyService = com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyService;

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyComponent", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/ServerPropertyComponent.html"),
	
	postCreate: function() {
		com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyComponent.superclass.postCreate.apply(this);
		checkNotNull(this.componentInfo, "this.componentInfo");
		checkNotNull(this.displayIn, "this.displayIn");
		
		this._services = new Array();
		if (this.componentHeaderClass != null) {
			for (var i=0; i < this.componentHeaderClass.length; i++) {
				dojo.toggleClass(this._componentHeader,this.componentHeaderClass[i],true);
			}
		}
		this.domNode._defaultSortVal = this.componentInfo.name;
		/*
		 * This object should have an array of servicesInfo and an array of extensionServicesInfo that will be used to create
		 * a set of ServerPropertyService objects.  Each of the ServerPropertyService objects should be created with information
		 * for populating ServerPropertyElements
		 */
		if (this.servicesInfo) {
			var checkDups = new Object();
			for (var i=0; i < this.servicesInfo.length; i++) {
				var curClass = this.servicesInfo[i].service.implementationClassName;
				if (curClass != null) {
					if (checkDups[curClass] == null) {
						checkDups[curClass] = true;
					}
					else {
						this.servicesInfo.splice(i,1);
						i--;
					}
				}
			}
			for (var i=0; i < this.servicesInfo.length; i++) {
				if (this.servicesInfo[i].config && this.servicesInfo[i].config.properties) {
					var curService = new ServerPropertyService({
						"serviceInfo": this.servicesInfo[i].service,
						"configInfo": this.servicesInfo[i].config,
						"enableEdits": this.enableEdits,
						"serviceHeaderClass": this.serviceHeaderClass,
						"propertiesRowClass": this.propertiesRowClass,
						"displayIn": this._componentBody,
						"hiddenPropertyList": ["ws.allow.admin.access","server.webapp.url"]
					});
					this._services.push(curService);
					this.connect(curService, "_onServiceResponse", "_onServiceResponse");
					this.connect(curService, "_show", "_show");
					this.connect(curService, "_hide", "_checkHide");
					this.connect(curService, "_markChanged", "_markChanged");
					this.connect(curService, "_checkUnchanged", "_checkUnchanged");
				}
			}
		}
		// Add the extension services to the list of configurable services
		if (this.extensionServicesInfo) {
			for (var i=0; i < this.extensionServicesInfo.length; i++) {
				if (this.extensionServicesInfo[i].config && this.extensionServicesInfo[i].config.properties) {
					var curService = new ServerPropertyService({
						"serviceInfo": this.extensionServicesInfo[i].service,
						"configInfo": this.extensionServicesInfo[i].config,
						"enableEdits": this.enableEdits,
						"serviceHeaderClass": this.serviceHeaderClass,
						"propertiesRowClass": this.propertiesRowClass,
						"displayIn": this._componentBody
					});
					this._services.push(curService);
					this.connect(curService, "_onServiceResponse", "_onServiceResponse");
					this.connect(curService, "_show", "_show");
					this.connect(curService, "_hide", "_checkHide");
					this.connect(curService, "_markChanged", "_markChanged");
					this.connect(curService, "_checkUnchanged", "_checkUnchanged");
				}
			}
		}
		
		if (this._services.length > 0) {
			this._componentHeader.appendChild(document.createTextNode(this.componentInfo.name));
		}
	},
	
	// a function that can be connected to by implementing code in order to
	// take action when a response is received from the server
	_onServiceResponse: function () {},
	
	_show: function () {
		this.domNode.style.display = "";
		this._showing = true;
		
		if (!this._initialized) {
			var beforeNode = this.displayIn.firstChild;
			while (beforeNode != null && this._sortFunction(this.domNode, beforeNode) > 0) {
				beforeNode = beforeNode.nextSibling;
			}
			if (beforeNode == null) {
				this.displayIn.appendChild(this.domNode);
			}
			else {
				this.displayIn.insertBefore(this.domNode, beforeNode);
			}
			this._initialized = true;
		}
	},
	
	_sortFunction: function (a, b) {
		if (a._customSortVal != null || b._customSortVal != null) {
			if (a._customSortVal != null && b._customSortVal != null) {
				if (a._customSortVal < b._customSortVal) {
					return -1;
				}
				else if (a._customSortVal > b._customSortVal) {
					return 1;
				}
				else {
					return 0;
				}
			}
			else if (a._customSortVal == null) {
				return 1;
			}
			else {
				return -1;
			}
		}
		else {
			if (a._defaultSortVal < b._defaultSortVal) {
				return -1;
			}
			else if (a._defaultSortVal > b._defaultSortVal) {
				return 1;
			}
			else {
				return 0;
			}
		}
	},
	
	_checkHide: function () {
		var hide = true;
		for (var i=0; i < this._services.length; i++) {
			if (this._services[i]._showing) {
				hide = false;
				break;
			}
		}
		if (hide) {
			this._hide();
		}
	},
	
	_hide: function () {
		this.domNode.style.display = "none";
		this._showing = false;
	},
	
	_markChanged: function () {
		this._changed = true;
	},
	
	_checkUnchanged: function () {
		change = false;
		for (var i=0; i < this._services.length; i++) {
			if (this._services[i]._changed) {
				change = true;
				break;
			}
		}
		this._changed = change;
	},
	
	_getChanges: function () {
		var changes = new Array();
		var properties = new Array();
		for (var i = 0; i < this._services.length; i++) {
			if (this._services[i]._changed) {
				var changeObj = this._services[i]._getChanges();
				changes = changes.concat(changeObj["changes"]);
				properties = properties.concat(changeObj["properties"]);
			}
		}
		return {
			"changes": changes,
			"properties": properties
		};
	},
	
	getPendingServiceCalls: function () {
		var count = 0;
		for (var i=0; i < this._services.length; i++) {
			count += this._services[i].getPendingServiceCalls();
		}
		return count;
	},
	
	_destroyServices: function() {
		for (var i = 0; i < this._services.length; i++) {
			this._services[i].destroyRecursive();
		}
		this._services = [];
	},
	
	uninitialize: function(){
		this._destroyServices();
		
		this.inherited(arguments);
	}

});

})();