/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.WhitelistUtility");

dojo.require("dijit._Widget");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ConfigUtility");

(function() {

var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ConfigUtility = com.ibm.team.repository.web.ui.internal.admin.server.ConfigUtility;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var AUTH_CONFIG_SERVICE = "com.ibm.team.repository.internal.service.auth.impl.AuthConfigurationProperties";
var WHITELIST_PROPERTY_NAME = "whitelist";

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.WhitelistUtility", dijit._Widget, {

	onSuccess: null,
	onError: null,	
	
	constructor: function() {
		this.configUtil = new ConfigUtility();
	},
	
	addURL: function(/* String or Array */ newURL) {
		this._processURLs(newURL, true);
	},
	
	removeURL: function(/* String or Array */ removeURL) {
		this._processURLs(removeURL, false);
	},
	
	_processURLs: function(newURL, isAddingEntry) {
		var urls = [];
		if (dojo.isString(newURL)) {
			urls.push(newURL);
		} else if (dojo.isArray(newURL)) {
			urls = newURL;
		}
		this._getWhitelist(urls, isAddingEntry);
	},
	
	_getWhitelist: function(urlArray, isAddingEntry) {
		var handler = {
			success: dojo.hitch(this, function (serviceInfo) {
				if (serviceInfo.properties && serviceInfo.properties.length > 0) {
					var whitelist;
					//Find the whitelist property that we want
					dojo.forEach(serviceInfo.properties, function(prop) {
						if(prop.name == WHITELIST_PROPERTY_NAME) {
							whitelist = prop;
							return;
						}
					});
					//Build up an array of the whitelist contents
					this.whitelist = [];
					if(whitelist && whitelist.overrideValue) {
						this.whitelist = whitelist.overrideValue.split(",");
					}
				}
				
				if(this.whitelist && urlArray) {
					dojo.forEach(urlArray, function(url) {						
						var index = dojo.indexOf(this.whitelist,url);
						//Append our new URLs
						if(isAddingEntry) {
							if(index == -1) {
								this.whitelist.push(url);	
							}
						}
						//Remove our URLs 
						else {
							if(index != -1) {
								this.whitelist.splice(index, 1);
							}
						}
					}, this);
					this._saveChanges(this.whitelist);
				} else {
					this._handleError();
				}
			}),
			failure: dojo.hitch(this, "_handleError")
		};
		var srh = new ServiceResponseHandler(handler, "success", "failure");
		ServerStatusClient.getServiceConfiguration(srh, AUTH_CONFIG_SERVICE);
	},
	
	_saveChanges: function(whitelist) {
		var changeList = [{
			"name": WHITELIST_PROPERTY_NAME,
			"value": whitelist.join(","),
			"serviceImplementationName": AUTH_CONFIG_SERVICE
		}];
		var changes = {
			"changes": changeList,
			"properties": []
		};
		this.configUtil.save(changes, this, "_handleSuccess", "_handleError");
	},
	
	_handleSuccess: function() {
		if (this.onSuccess && typeof this.onSuccess == "function") {
			this.onSuccess();
		}
	},
	
	_handleError: function(errorObject) {
		if (this.onError && typeof this.onError == "function") {
			this.onError();
		}
	}

});

})();