/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.users.GroupEditor");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.string.BidiComplex");
dojo.require("dojo.string");
dojo.require("dojox.xml.parser");
dojo.require("dojo.i18n");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.ui.Breadcrumbs"); //$NON-NLS-1$
dojo.require("jazz.ui.StyledTable"); //$NON-NLS-1$
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("jazz.ui.ResourceLink"); //$NON-NLS-1$

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.ui.internal.TeamContributorSelectionDialog"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.users", "GroupEditorMessages");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.users", "ManageUserGroupsMessages");

var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var GROUP_EDITOR = "com.ibm.team.repository.editUserGroup";
var MSG_COOKIE_NAME = "com.ibm.team.repository.editGroup#saveMsg";
var KEY_RETURN = 13;
var KEY_ENTER = 14;
var MessageBox = jazz.ui.MessageBox;
var STATUS_TYPES = MessageBox.STATUS_TYPES;
var DEFAULT_DISPLAY_MAX = 5;
var ON_CLICK = "onclick";
var ResourceLink = jazz.ui.ResourceLink;
var REST_USER_GROUP_URL = "user-groups/";
var ACTION_PREFIX = "com.ibm.team.repository.editUser&itemId=";

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.users.GroupEditor", [dijit._Widget, dijit._Templated], {
	
	dirty: false,
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/templates/GroupEditor.html"), 
	
	constructor: function(args){
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.users", "GroupEditorMessages");
		this._manageGroupMessages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.users", "ManageUserGroupsMessages");
		this.isNew = args.itemId === "new";
		if (!this.isNew) {
			this.itemId = args.itemId;
		}
		
		this._userIconSrc = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/images/userIcon.png");
		this._refreshIconSrc = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/images/refresh16.gif"); //$NON-NLS-1$ //$NON-NLS-2$

		this._actionToolbars = [];
		this._resourceLinks = [];
	},
	
	postCreate: function () {
		
		this.header = new MessageBox();
		this.headerArea.appendChild(this.header.domNode);
		
		this.showingAllMembers = true;
		this.inputFields = [
			this.name,
			this.summary,
			this.description
		];
		
		// Field: Group Name
		this.connect(this.previewName,"onmouseover", function (e) {
			e.target.style.backgroundColor = "#FFFFCC";
		});
		this.connect(this.previewName,"onmouseout", function (e) {
			e.target.style.backgroundColor = "";
		});
		this.connect(this.previewName,"onfocus", function (e) {
			this.togglePrimaryField("Edit");
		});
		this.connect(this.name, "onblur", function (e) {
			this.togglePrimaryField("Preview");
		});
		
		// Field: Group Name
		// Field: Summary
		// Field: Description
		for (var i=0; i < this.inputFields.length; i++) {
			this.connect(this.inputFields[i],"onkeyup","handleChange");
			this.connect(this.inputFields[i],"onchange","handleChange");
		}
		
		this.connect(this.memberAddButton, ON_CLICK, function(e){ //$NON-NLS-1$
			dojo.stopEvent(e);
			if (!this.allMembers) {
				this.allMembers = [];
			}
			this.addUserDialog(this.allMembers, "addNewMembers"); //$NON-NLS-1$
		});
		
		this.connect(this.saveButton,"onclick","handleSave");
		
		this.connect(this.showAllMembersLink, ON_CLICK, function(e){
			dojo.stopEvent(e);
			this.showingAllMembers = true;
			this.showAllMembersLink.style.display = "none";
			this.showFewerMembersLink.style.display = "block";
			var membersTable = this.memberArea;
			for (var i = 0; i < membersTable.rows.length; i++) {
				membersTable.rows[i].style.display = "";
			}
		});
		this.connect(this.showFewerMembersLink, ON_CLICK, function(e){
			dojo.stopEvent(e);
			this.showingAllMembers = false;
			this.showAllMembersLink.style.display = "block";
			this.showFewerMembersLink.style.display = "none";
			var membersTable = this.memberArea;
			for (var i = DEFAULT_DISPLAY_MAX; i < membersTable.rows.length; i++) {
				membersTable.rows[i].style.display = "none";
			}
		});
		
		this.connectHandles = [];
		this._actionToolbars = [];
		
		this._showBreadcrumbs();
		this.refresh();
	},

	refresh: function () {
		if (!this.navigateAwayHandle) {
			this.navigateAwayHandle = PlatformUI.getWorkbench().registerBodyClickHandler(dojo.hitch(this, this._handleNavigate), /*priority*/ 10000);
			this.onBeforeUnloadHandle = PlatformUI.getWorkbench().registerOnBeforeUnloadHandler(dojo.hitch(this, this.navigateAwayEvent), /*priority*/ 10000);
		}
		this.clearState();
		
		if (this.isNew) {
			this.name.value = this.messages.defaultGroupName;
			this.togglePrimaryField("Edit");
			setTimeout(dojo.hitch(this, function () {
				this.name.select();
			}), 250);
		}
		else {
			this.getGroupInfo();
		}
		this.checkForSuccessMsg();
		
	},	
	
	_showBreadcrumbs: function(){
		if (this._breadCrumbsWidget) {
			this._breadCrumbsWidget.destroyRecursive();
			delete this._breadCrumbsWidget;
		}
		dojox.xml.parser.removeChildren(this._breadcrumbs);
		
		this._breadCrumbsWidget = new jazz.ui.Breadcrumbs({
			segments: [ {
				href: "#action=com.ibm.team.repository.manageUserGroups", //$NON-NLS-1$
				label: this._manageGroupMessages.userGroups
			} ]
		});
		
		this._breadcrumbs.appendChild(this._breadCrumbsWidget.domNode);
		
		
	},
	
	
	_handleNavigate: function(urlObj) {
		if (this.dirty) {
			if (confirm(this.messages.navigateAway + "\n\n" + this.messages.unsavedChanges + "\n\n" + this.messages.okOrCancel)) {
				PlatformUI.getWorkbench().unregisterBodyClickHandler(this.navigateAwayHandle);
				this.navigateAwayHandle = null;
				PlatformUI.getWorkbench().unregisterOnBeforeUnloadHandler(this.onBeforeUnloadHandle);
				this.onBeforeUnloadHandle = null;
				return false;
			}
			else {
				return true;
			}
		}
		return false;
	},
	
	navigateAwayEvent: function() {
		return this.dirty ? this.messages.unsavedChanges : false;
	},
	
	checkForSuccessMsg: function () {
		var msgCookie = dojo.cookie(MSG_COOKIE_NAME);
		if (msgCookie) {
			this.header.showMessage({
				summary: msgCookie,
				type: STATUS_TYPES.OK
			});		
			var now = new Date();
			now.setTime(now.getTime() - 1);
			dojo.cookie(MSG_COOKIE_NAME,"",{expires: now});
		}
	},
	
	getGroupInfo: function () {
		var srh = new ServiceResponseHandler(this, "populateGroupData", "handleGroupDataError");
		var url = this._getServerUrl() + net.jazz.ajax._contextRoot + "/" + REST_USER_GROUP_URL + this.itemId;
		var xhrArgs = {
	            url: url
	        }
			
		var deferred = jazz.client.xhrGet(xhrArgs);
		deferred.addCallback(function(response){
			net.jazz.ajax.core.internal.util.callContributedCode(function() {
				srh.doCallback(response);
			});
		});
		deferred.addErrback(function(response){
			net.jazz.ajax.core.internal.util.callContributedCode(function() {
				srh.doErrorCallback(response);
			});
		});
	},
	
	clearState: function () {
		this.name.value = "";
		this.summary.value = "";
		this.description.value = "";
		dojo.toggleClass(this.name, "highlightChange", false);
		dojo.toggleClass(this.summary, "highlightChange", false);
		dojo.toggleClass(this.description, "highlightChange", false);
		this.pendingMemberChanges = {};
		for (var i=0; i < this.inputFields.length; i++) {
			this.inputFields[i].changePending = false;
		}
		this._destroyActionToolbars();
		this._destroyResourceLinks();
		
	},
	
	addUserDialog: function(filterUsers, callbackFunction){
		var userFilter = [];
		for (var i = 0; i < filterUsers.length; i++) {
			userFilter.push({
				'itemId': filterUsers[i].itemId //$NON-NLS-1$
			});
		}
		var userPicker = new com.ibm.team.repository.web.ui.internal.TeamContributorSelectionDialog({
			'disableUsers': userFilter, //$NON-NLS-1$
			destroyContent: true
		});
		this.connect(userPicker, "onAdd", callbackFunction); //$NON-NLS-1$
	},
	
	addNewMembers: function(newMembers){
		if (newMembers) {
			if (newMembers.length > 0) {
				this.setDirty(true);
				if (!this.pendingMemberChanges.add) {
					this.pendingMemberChanges.add = [];
				}
			}

			for (var i = 0; i < newMembers.length; i++) {
				var member = newMembers[i];
				member.itemId = member.itemId;
				this.allMembers.push(member);
				this.pendingMemberChanges.add.push(member.itemId);
				if (this.pendingMemberChanges.remove) {
					for (var j = 0; j < this.pendingMemberChanges.remove.length; j++) {
						if (this.pendingMemberChanges.remove[j] === member.itemId) {
							this.pendingMemberChanges.remove.splice(j, 1);
							break;
						}
					}
				}
			}
		}
		this.populateMembers(this.allMembers);
	},
		
	populateMembers: function(members){
		this.populateUsers(members, this.showingAllMembers, "memberRemoveHandler", "userAddedHandler"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ 
		if (members && members.length > DEFAULT_DISPLAY_MAX) {
			if (this.showingAllMembers) {
				this.showFewerMembersLink.style.display = "block";
				this.showAllMembersLink.style.display = "none";
			} else {
				this.showFewerMembersLink.style.display = "none";
				this.showAllMembersLink.style.display = "block";
			}
		} else {
			this.showFewerMembersLink.style.display = "none";
			this.showAllMembersLink.style.display = "none";
		}
	},
	
	populateUsers: function(users, showAll, removeHandler, userAddedHandler){
		var tableDiv = this._membersDiv;
		this._destroyStyledTable(tableDiv);
		
		var columns = [];
		columns.push({
			label: this.messages.name,
			className: "groupMemberName", //$NON-NLS-1$
			width: "25%", //$NON-NLS-1$
			sortable: false
		});
		columns.push({
			label: this.messages.userId,
			className: "groupMemberId", //$NON-NLS-1$
			sortable: false
		});
		columns.push({
			label: this.messages.email,
			className: "groupMemberMail", //$NON-NLS-1$
			sortable: false
		});
		columns.push({
			label: this.messages.actions,
			className: "action-column", //$NON-NLS-1$,
			sortable: false
		});
		tableDiv.styledTable = new jazz.ui.StyledTable({
			columns: columns,
			showHover: true
		});
		tableDiv.appendChild(tableDiv.styledTable.domNode);
		
		if (users && users.length > 0) {
			for (var i = 0; i < users.length; i++) {
				tr = document.createElement("tr"); //$NON-NLS-1$
				tr.userObj = users[i];
				var td = document.createElement("td"); //$NON-NLS-1$
				td.className = "groupMemberName"; //$NON-NLS-1$
				var link = document.createElement("a"); //$NON-NLS-1$
				var presentationUri;
				if (users[i].archived) {
					presentationUri = "#action=" + ARCHIVED_USER_EDITOR + "&itemId=" + users[i].itemId; //$NON-NLS-1$ //$NON-NLS-2$
				} else {
					presentationUri = "#action=" + ACTION_PREFIX + users[i].itemId; //$NON-NLS-1$ //$NON-NLS-2$
				}
				var encodedUserId = encodeURIComponent(users[i].userId);
				link.href = this._getServerUrl() + net.jazz.ajax._contextRoot + "/users/" + encodedUserId; //$NON-NLS-1$ 
				var rl = new ResourceLink({
					'presentationUri': presentationUri, //$NON-NLS-1$
					'lazyFetch': true //$NON-NLS-1$
				}, link);
				this._resourceLinks.push(rl);
				
				link.appendChild(document.createTextNode(users[i].name));
				td.appendChild(link);
				
				this.connectHandles.push(dojo.connect(link, "onfocus", tr, function(e){ //$NON-NLS-1$
					dojo.toggleClass(this, "selected", true);
				}));
				this.connectHandles.push(dojo.connect(link, "onblur", tr, function(e){ //$NON-NLS-1$
					dojo.toggleClass(this, "selected", false);
				}));
				
				tr.appendChild(td);
				
				td = document.createElement("td"); //$NON-NLS-1$
				td.appendChild(document.createTextNode(users[i].userId));
				td.className = "groupMemberId"; //$NON-NLS-1$
				tr.appendChild(td);
				
				td = document.createElement("td"); //$NON-NLS-1$
				td.className = "groupMemberMail"; //$NON-NLS-1$
				if (users[i].emailAddress) {
					link = document.createElement("a"); //$NON-NLS-1$
					//Do not format the email address code, just the display String
					link.href = "mailto:" + users[i].emailAddress; //$NON-NLS-1$
					var emailAddress = dojox.string.BidiComplex.createDisplayString(users[i].emailAddress, "EMAIL"); //$NON-NLS-1$
					link.appendChild(document.createTextNode(emailAddress));
					this.connectHandles.push(dojo.connect(link, "onfocus", tr, function(e){ //$NON-NLS-1$
						dojo.toggleClass(this, "selected", true);
					}));
					this.connectHandles.push(dojo.connect(link, "onblur", tr, function(e){ //$NON-NLS-1$
						dojo.toggleClass(this, "selected", false);
					}));
					
					td.appendChild(link);
				} else if (dojo.isIE) {
					td.appendChild(document.createTextNode(" ")); //$NON-NLS-1$
				}
				tr.mailDisplay = td;
				
				tr.appendChild(td);
				td = document.createElement("td"); //$NON-NLS-1$
				var trToolbar = new com.ibm.team.repository.web.ui.internal.admin.users.TRToolbar();
				td.appendChild(trToolbar.domNode);
				this._actionToolbars.push(trToolbar);
				this.addRemoveIcon(trToolbar, removeHandler);
				tr.appendChild(td);
				if (!showAll && i >= DEFAULT_DISPLAY_MAX) {
					tr.style.display = "none";
				}
				tableDiv.styledTable.tbody.appendChild(tr);
			}
		}
	},
	
	addRemoveIcon: function(trToolbar, removeHandler){		
		var removeAction = {};
		removeAction.imgSrc = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/images/remove-larger.gif"); //$NON-NLS-1$  //$NON-NLS-2$
		removeAction.label = this.messages.deleteMember;
		trToolbar.addButton(this, removeHandler, removeAction);
	},
	
	memberRemoveHandler: function(e){
		dojo.stopEvent(e);
		if (!this.pendingMemberChanges.remove) {
			this.pendingMemberChanges.remove = [];
		}
		var rowNode = e.target;
		while (rowNode.tagName !== "TR") { //$NON-NLS-1$
			rowNode = rowNode.parentNode;
		}
		var selectedUser = rowNode.userObj;
		var itemId = selectedUser.itemId;
		
		this.pendingMemberChanges.remove.push(itemId);
		for (var j = 0; j < this.allMembers.length; j++) {
			if (itemId === this.allMembers[j].itemId) {
				this.allMembers.splice(j, 1);
				break;
			}
		}
		if (this.pendingMemberChanges.add) {
			for (j = 0; j < this.pendingMemberChanges.remove.length; j++) {
				if (this.pendingMemberChanges.add[j] === itemId) {
					this.pendingMemberChanges.add.splice(j, 1);
					break;
				}
			}
		}
		this.setDirty(this.changesPending());
		this.populateMembers(this.allMembers);
	},
	
	userAddedHandler: function(e){
		dojo.stopEvent(e);
		var rowNode = e.target;
		while (rowNode.tagName !== "TR") { //$NON-NLS-1$
			rowNode = rowNode.parentNode;
		}
		var selectedUser = rowNode.userObj;
		//send notification that user has been added to group
	},
	
	handleGroupDataError: function (err) {//keep for tying in users
		this.header.showMessage({
			summary: this.messages.errorFetchingGroupData,
			errorObject: err
		});
	},
	
	togglePrimaryField: function (mode) {
		if (mode === "Preview") {
			this.name.style.display = "none";
			this.previewName.innerHTML = "";
			this.previewName.appendChild(document.createTextNode(this.name.value || this.messages.userName));
			this.previewName.style.display = "";
		}
		else if (mode === "Edit") {
			this.previewName.style.display = "none";
			this.name.style.display = "";
			this.name.focus();
			this.name.select();
			this.header.hideMessage();
		}
	},
	
	populateGroupData: function (response) {
		var data = dojo.fromJson(response);
		this.name.value = data.name;
		this.name.savedValue = data.name;
		this.togglePrimaryField("Preview");
		this.summary.value = data.summary;
		this.summary.savedValue = data.summary;
		this.description.value = data.description;
		this.description.savedValue = data.description;
		this.allMembers = data.users;
		this.stateId = data.stateId;
		this.populateMembers(this.allMembers);
	},
	
	_getServerUrl: function(){
		var location = document.location.href;
		var separator = "/"; //$NON-NLS-1$
		var path = location.split(separator); //$NON-NLS-1$
		var serverAddress;
		if (path.length > 0) {
			serverAddress = path.shift() + separator + separator;
			path.shift(); //empty from the double //
			serverAddress = serverAddress + path.shift();
		}
		return serverAddress;
	},
	
	handleChange: function (e) {
		if (e.target.type == "checkbox") {
			if (e.target.checked != e.target.savedValue) {
				e.target.changePending = true;
				this.setDirty(true);
			}
			else {
				e.target.changePending = false;
				if (!this.changesPending()) {
					this.setDirty(false);
				}
			}
		}
		else {
			if (e.target.value != e.target.savedValue) {
				e.target.changePending = true;
				this.setDirty(true);
			}
			else {
				e.target.changePending = false;
				if (!this.changesPending()) {
					this.setDirty(false);
				}
			}
		}
	},

	/*
 	 * @param Boolean dirty
 	 * 
 	 * @return void
 	 */	
	setDirty: function (dirty) {
		
		if (dirty === this.dirty) {
			// Nothing to do
			return;
		}
		
		if (dirty) {
			// Show *
			this.modifiedFlag.style.display = "inline";
			// Enable save button
			this.saveButton.disabled = false;
		} else {
			// Hide *
			this.modifiedFlag.style.display = "none";
			// Disable save button
			this.saveButton.disabled = true;
		}

		this.dirty = dirty;
	},
	
	handleSave: function (e) {
		dojo.stopEvent(e);
		this.saveGroup();
	},
	
	saveGroup: function () {
		this.saveButton.disabled = true;
		// Item ID
		var saveObj = {
			itemId: this.itemId
		};
		
		// group Name
		// summary
		// description
		for (var i=0; i < this.inputFields.length; i++) {
			if (this.isNew || this.inputFields[i].changePending) {
				saveObj[this.inputFields[i].name] = //Do not have special characters
				dojox.string.BidiComplex.stripSpecialCharacters(this.inputFields[i].value);
			}
		}
		
		// Validate save object
		var msg = this.validateRequired(saveObj);
		if (msg != null && msg != "") {
			this.header.showMessage({
				summary: msg,
				type: STATUS_TYPES.ERROR
			});
			this.saveButton.disabled = false;
		} else {
			var srh = new ServiceResponseHandler(this, "handleSaveResponse", "handleSaveError");
			var url = this._getServerUrl() + net.jazz.ajax._contextRoot + "/" + REST_USER_GROUP_URL;
			var xhrArgs = {
				url: url,
				// ask the server to send JSON and expect form encoded data
				headers: {
					"accept": "text/json",
					"content-type": "application/x-www-form-urlencoded; charset=utf-8",
					"X-Jazz-CSRF-Prevent": dojo.cookie("JSESSIONID")
				},
				// ask Dojo to expect and automatically demarshall a JSON response body
				handleAs: "json"
			}
			var deferred;
			
			if (!this.isNew) { //update group
				saveObj.stateId = this.stateId;
				
				/*
				 * Since allMembers is kept up to date with what the current members should be during the addMember and removeMember handlers, 
				 * if there are pending member changes we can just blow away the current list in the Repository with this one, rather
				 * than calling POST or DELETE for each new/removed member.
				 */
				if ((this.pendingMemberChanges && this.pendingMemberChanges.add && this.pendingMemberChanges.add.length > 0) ||
					(this.pendingMemberChanges && this.pendingMemberChanges.remove && this.pendingMemberChanges.remove.length > 0)) {
					saveObj.users = [];
					for (var index = 0; index < this.allMembers.length; index++) {
						saveObj.users.push(this.allMembers[index].itemId);
					}
				}
				
				var jsonString = this._saveObjToJson(saveObj);
				xhrArgs.postData = jsonString;
				xhrArgs.url = xhrArgs.url + this.itemId;
				deferred = jazz.client.xhrPut(xhrArgs);
			}
			else { //create group
				if (this.pendingMemberChanges && this.pendingMemberChanges.add) {
					saveObj.users = this.pendingMemberChanges.add;
				}
				var jsonString = this._saveObjToJson(saveObj);
				xhrArgs.postData = jsonString;
				deferred = jazz.client.xhrPost(xhrArgs);
			}
			deferred.addCallback(function(response){
				net.jazz.ajax.core.internal.util.callContributedCode(function() {
					srh.doCallback(response);
				});
			});
			deferred.addErrback(function(response){
				net.jazz.ajax.core.internal.util.callContributedCode(function() {
					srh.doErrorCallback(response);
				});
			});
		}
	},
	
	/*
	 * dojo toJson doesn't actually make a JSON object, so we have to do it ourselves.
	 */
	_saveObjToJson: function(saveObj) {
		var groupAsJson = "{"; //$NON-NLS-1$
		if (saveObj.name)
			groupAsJson += "\"name\":\"" + saveObj.name + "\","; //$NON-NLS-1$ //$NON-NLS-2$
		if (saveObj.itemId)
			groupAsJson += "\"itemId\":\"" + saveObj.itemId + "\","; //$NON-NLS-1$ //$NON-NLS-2$
		if (saveObj.stateId)
			groupAsJson += "\"stateId\":\"" + saveObj.stateId + "\","; //$NON-NLS-1$ //$NON-NLS-2$
		if (saveObj.summary)
			groupAsJson += "\"summary\":\"" + saveObj.summary  + "\","; //$NON-NLS-1$ //$NON-NLS-2$
		if (saveObj.description)
			groupAsJson += "\"description\":\"" + saveObj.description  + "\","; //$NON-NLS-1$ //$NON-NLS-2$
		if (saveObj.users) {
			groupAsJson += "\"users\":"; //$NON-NLS-1$
			var usersAsString = "["; //$NON-NLS-1$
			var members = saveObj.users;
			for (var x = 0; x < members.length; x++) {
				
				usersAsString += "\"" + members[x] + "\"";
				
				if (x != members.length - 1) {
					usersAsString += ","; //$NON-NLS-1$
				}
			}
			usersAsString += "]"; //$NON-NLS-1$
			groupAsJson += usersAsString; //$NON-NLS-1$
		}
		if (groupAsJson.charAt(groupAsJson.length - 1) === ',') {
			groupAsJson = groupAsJson.slice(0, groupAsJson.length - 1);
		}
		groupAsJson +="}";
		return groupAsJson;
	},
	
	validateRequired: function (saveData) {
		//saveData.name will be undefined only if it has not been changed on an 
		//existing user group. Otherwise, make sure the name is not empty or all spaces
		if (saveData.name !== undefined && !dojo.trim(saveData.name)) {
			this.togglePrimaryField("Edit");
			return this.messages.groupMustHaveName;
		}
		if (saveData.summary === "" || (this.isNew && saveData.summary == null)) {
			this.summary.focus();
			return this.messages.groupMustHaveSummary;
		}
		if (saveData.description === "" || (this.isNew && saveData.description == null)) {
			this.description.focus();
			return this.messages.groupMustHaveDescription;
		}
	},
	
	handleSaveResponse: function(response) {
		// Undirty Overview page
		this.setDirty(false);
		
		if (this.isNew && response) {
			dojo.cookie(MSG_COOKIE_NAME, this.messages.groupCreatedSuccessfully);
			PlatformUI.getWorkbench().unregisterBodyClickHandler(this.navigateAwayHandle);
			this.navigateAwayHandle = null;
			PlatformUI.getWorkbench().unregisterOnBeforeUnloadHandler(this.onBeforeUnloadHandle);
			this.onBeforeUnloadHandle = null;
			document.location.href = "#action=" + GROUP_EDITOR + "&itemId=" + response.itemId;
			return;
		}

		this.pendingMemberChanges.add = [];
		this.pendingMemberChanges.remove = [];

		// Refresh Overview page (only)
		this.refresh();
		
		this.header.showMessage({
			summary: this.messages.groupSavedSuccessfully,
			type: STATUS_TYPES.OK
		});
	},
	
	handleSaveError: function (err) {
		this.saveButton.disabled = false;
		//service writes to responseText, so use that as the 
		//message if it is not empty
        if (err && err.responseText) {
            err.message = err.responseText;
        }   
		this.header.showMessage({
			summary: this.messages.groupSaveFailed,
			errorObject: err
		});
	},
	
	onRefresh: function(e){
		dojo.stopEvent(e);
		if (this.dirty && !confirm(this.messages.refreshWarning)) {
			return;
		}
		// Undirty Overview page
		this.setDirty(false);		
		// Refresh Overview page (only)
		this.refresh();
		this.header.hideMessage();
	},
	
	changesPending: function () {
		for (var i=0; i < this.inputFields.length; i++) {
			if (this.inputFields[i].changePending) {
				return true;
			}
		}
		if (this.pendingMemberChanges.add && this.pendingMemberChanges.add.length > 0) {
			return true;
		}
		if (this.pendingMemberChanges.remove && this.pendingMemberChanges.remove.length > 0) {
			return true;
		}
		return false;
	},
	
	uninitialize: function () {
		
		dojo.forEach(this.connectHandles, function(handle){
			dojo.disconnect(handle);
		});
		
		this._destroyActionToolbars();
		this._destroyResourceLinks();
		this._destroyStyledTable(this._membersDiv);
		
		if (this._breadCrumbsWidget) {
			this._breadCrumbsWidget.destroyRecursive();
			delete this._breadCrumbsWidget;
		}
		
		this.inherited(arguments);
	},
	
	_destroyActionToolbars: function(){
		for (var i = 0; i < this._actionToolbars.length; i++) {
			this._actionToolbars[i].destroyRecursive();
		}
		this._actionToolbars = [];
	},
	
	_destroyResourceLinks: function(){
		for (var i = 0; i < this._resourceLinks.length; i++) {
			this._resourceLinks[i].destroyRecursive();
		}
		this._resourceLinks = [];
	},
	
	_destroyStyledTable: function(tableDiv){
		if (tableDiv.styledTable) {
			tableDiv.styledTable.destroyRecursive();
			// IE 7 doesn't like delete here for some reason so just undefine it
			tableDiv.styledTable = undefined;
		}
		dojox.xml.parser.removeChildren(tableDiv);
	}
	
});

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.users.TRToolbar", jazz.ui.SimpleToolbar, { //$NON-NLS-1$
	addButton: function(context, callback, args){
		var button = this.inherited(arguments);
		button.domNode.callbackData = args.callbackData;
		
		this.connect(button.domNode, "onfocus", function(e){ //$NON-NLS-1$
			var row = e.target;
			while (row && row.tagName.toLowerCase() !== "tr") { //$NON-NLS-1$
				row = row.parentNode;
			}
			if (row) {
				dojo.toggleClass(row, "selected", true); //$NON-NLS-1$
			}
		});
		this.connect(button.domNode, "onblur", function(e){ //$NON-NLS-1$
			var row = e.target;
			while (row && row.tagName.toLowerCase() !== "tr") { //$NON-NLS-1$
				row = row.parentNode;
			}
			if (row) {
				dojo.toggleClass(row, "selected", false); //$NON-NLS-1$
			}
		});
		
		return button;
	}
});


})();