/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function () {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.users.ManageArchivedUsers");

dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("dojox.xml.parser");
dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.ItemList");
dojo.require("com.ibm.team.repository.web.ui.internal.common.Toolbar");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.users.UserHelper");
dojo.require("jazz.app.ua.PageHoverHelp");
dojo.require("jazz.ui.MessageBox");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.users", "ManageArchivedUsersMessages");

var AdminClient = com.ibm.team.repository.web.client.internal.AdminClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var MessageBox = jazz.ui.MessageBox;
var STATUS_TYPES = MessageBox.STATUS_TYPES;
var UserHelper = com.ibm.team.repository.web.ui.internal.admin.users.UserHelper;
var Toolbar = com.ibm.team.repository.web.ui.internal.common.Toolbar;
var PageHoverHelp = jazz.app.ua.PageHoverHelp;

function setText(e, text){
	e.appendChild(document.createTextNode(text));
}

var ACTION_PREFIX = "com.ibm.team.repository.editArchivedUser&itemId=";
var SEARCH_ACTION = "action=com.ibm.team.repository.searchArchivedUsers";
var PAGE_SIZE = 50;

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.users.ManageArchivedUsers", com.ibm.team.repository.web.ui.internal.admin.ItemList, {
	
	constructor: function () {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.users", "ManageArchivedUsersMessages");
	},
	
	postCreate: function() {
		this.header = new MessageBox();
		this.headerArea.appendChild(this.header.domNode);	
		
		this.userHelper = new UserHelper({
			messages: this.messages
		});
		
		//Append the context sensitive help
		var help = new PageHoverHelp({ uaKey: "com.ibm.jazz.repository.web.admin.ui.archivedUsers" });
		this.helpHover.appendChild(help.domNode);
		
		//Add the Table Controls table
		this.controls = this.userHelper.buildTableControls(this.tableControls);
		
		//Inject our CSS into the ItemList domNode
		dojo.addClass(this.itemListNode, "com-ibm-team-repository-users-manageUsers")
		
		setText(this.heading, this.messages.archivedUsers);
		setText(this.description, this.messages.pageDescription);
		setText(this.userPagesDescription, this.messages.usersPageDescription);
		this.tableHeading.parentNode.removeChild(this.tableHeading);
		this.newItemButtonArea.parentNode.removeChild(this.newItemButtonArea);
		this.headerRowLine.parentNode.removeChild(this.headerRowLine);
		
		this.searchText = this.userHelper.buildFilterBox(this.controls["col3"], this.messages.searchPlaceholder, this.messages.search, SEARCH_ACTION);
		
		this.setHeaderRow([this.messages.name, this.messages.userId, this.messages.email]);
		this.headerRow.cells[0].sortBy = "name";
		this.headerRow.cells[1].sortBy = "userId";
		this.headerRow.cells[2].sortBy = "emailaddress";
		dojo.toggleClass(this.headerRow, "sortable", true);
		this.connect(this.headerRow, "onclick", "updateSorting");
		this.connect(this.headerRow, "onmouseover", function (e) {
			dojo.toggleClass(e.target, "selected", true);
		});
		this.connect(this.headerRow, "onmouseout", function (e) {
			dojo.toggleClass(e.target, "selected", false);
		});
		
		//Modify the ItemList table to have an actions column
		dojo.create("th", { innerHTML: this.messages.actionsLabel, className: "actions" }, this.headerRow);
		dojo.addClass(this.tableContents, "actions-table");
		
		this.pageNumberContainer = document.createElement("div");
		this.pageNumberContainer.style.marginTop = "5px";
		this.pageNumberContainer.style.fontWeight = "normal";
		this.pageNumberContainer.style.display = "none";
		this.pageNumberContainer.appendChild(document.createTextNode(dojo.string.substitute(this.messages.showingFirstUsers, [ PAGE_SIZE ])));
		this.tableControls.appendChild(this.pageNumberContainer);
		
		this.refresh();
	},
	
	refresh: function () {
		this.clearState();
		if (this.latestSearch != null) {
			this.runUserSearch(this.latestSearch.searchTerm, this.latestSearch.pageNum);
		}
		else {
			this.runUserSearch("");
		}
	},
	
	show: function () {
		this.refresh();
	},
	
	reload: function () {
		this.refresh();
	},
	
	clearState: function () {
		if(this.searchText.value != this.messages.searchPlaceholder) {
			this.searchText.value = "";	
		}
		dojox.xml.parser.removeChildren(this.tbody);
		this.pageNumberContainer.style.display = "none";
		this.header.hideMessage();
		var tr = dojo.create("tr", {}, this.tbody);
		dojo.create("td", { className: "noRowsYet", colSpan: 4, innerHTML: this.messages.searchingForUsers }, tr);	
	},
	
	runUserSearch: function (searchTerm, pageNum, sortBy, sortOrder) {
		if (searchTerm == null) {
			searchTerm = "";
		}
		if (sortBy == null) {
			sortBy = "name";
		}
		if (sortOrder !== "desc") {
			sortOrder = "asc";
		}
		if (pageNum == null) {
			pageNum = 0;
		}
		this.latestSearch = {
			searchTerm: searchTerm,
			pageNum: pageNum,
			sortBy: sortBy,
			sortOrder: sortOrder
		};
		searchTerm = searchTerm.replace(/\*/g,"%");
		this.latestSearchTerm = searchTerm;
		this.latestSearchType = "searchTerm";
		this.queryPage = pageNum || 0;
		this.updatePagerDivLinks();
		dojox.xml.parser.removeChildren(this.tbody);
		var tr = dojo.create("tr", {}, this.tbody);
		dojo.create("td", { className: "noRowsYet", colSpan: 4, innerHTML: this.messages.searchingForUsers }, tr);
		var srh = new ServiceResponseHandler(this, "showUsers", "handleError");
		AdminClient.getContributors({searchTerm: searchTerm, pageSize: PAGE_SIZE, sortBy: sortBy, sortOrder: sortOrder, hideGuest: true, hideUnassigned: true, pageNum: pageNum, onlyArchivedUsers: true}, srh);
	},
	
	updateSorting: function (e) {
		if (this.latestSearch) {
			var sortByField = e.target.sortBy;
			var sortOrder;
			if (this.latestSearch.sortBy === sortByField) {
				if (this.latestSearch.sortOrder === "desc") {
					sortOrder = "asc";
				}
				else {
					sortOrder = "desc";
				}
			}
			else {
				sortOrder = "asc";
			}
			document.location.href = "#" + SEARCH_ACTION + "&searchTerm=" + encodeURIComponent(this.latestSearch.searchTerm) + "&pageNum=" +
					this.latestSearch.pageNum + "&sortBy=" + sortByField + "&sortOrder=" + sortOrder;
		}
	},
	
	showUsers: function (resultSet) {
		dojox.xml.parser.removeChildren(this.tbody);
		if (this.pagerDiv == null) {
			this.pagerDiv = this.userHelper.createPagerDiv();
			this.controls["col2"].appendChild(this.pagerDiv);
		}
		if (resultSet == null || resultSet.elements == null || resultSet.elements.length == 0) {
			var tr = dojo.create("tr");
			dojo.create("td", { className: "noRowsYet", colSpan: 4, innerHTML: this.messages.noMatchingUsers }, tr);
			tr.onclick = function (e) {
				dojo.stopEvent(e);
			};
			this.tbody.appendChild(tr);
			dojo.toggleClass(this.pagerDiv.prevSpan,"disabled", true);
			dojo.toggleClass(this.pagerDiv.nextSpan,"disabled", true);
			this.pagerDiv.rangeSpan.innerHTML = "0";
			this.pagerDiv.totalSpan.innerHTML = "0";
		}
		else {
			this.pagerDiv.rangeSpan.innerHTML = "";
			this.pagerDiv.totalSpan.innerHTML = "";
			if (resultSet.count > PAGE_SIZE) {
				var lowerBound = (this.queryPage * PAGE_SIZE) + 1;
				var upperBound = (this.queryPage + 1) * PAGE_SIZE;
				if (this.queryPage == 0) {
					dojo.toggleClass(this.pagerDiv.prevSpan,"disabled", true);
				}
				else {
					dojo.toggleClass(this.pagerDiv.prevSpan,"disabled", false);
				}
				if (upperBound >= resultSet.count) {
					upperBound = resultSet.count;
					dojo.toggleClass(this.pagerDiv.nextSpan,"disabled", true);
				}
				else {
					dojo.toggleClass(this.pagerDiv.nextSpan,"disabled", false);
				}
				this.pagerDiv.rangeSpan.appendChild(document.createTextNode(lowerBound + " - " + upperBound));
			}
			else {
				dojo.toggleClass(this.pagerDiv.prevSpan,"disabled", true);
				dojo.toggleClass(this.pagerDiv.nextSpan,"disabled", true);
				this.pagerDiv.rangeSpan.appendChild(document.createTextNode("1 - " + resultSet.count));
			}
			this.pagerDiv.totalSpan.appendChild(document.createTextNode(resultSet.count));
			dojo.forEach(resultSet.elements, dojo.hitch(this, function(user){
				var emailAddress = dojox.string.BidiComplex.createDisplayString(user.emailAddress,"EMAIL");
				var toolbar = new Toolbar();
				var unArchiveAction = {};
				unArchiveAction.imgSrc = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/images/unarchive_co.gif");
				unArchiveAction.label = this.messages.unarchiveUser;
				unArchiveAction.callbackData = {
					itemId: user.itemId
				};
				var callback = function(e){
					dojo.stopEvent(e);
					this.unarchiveUser(e.currentTarget.callbackData.itemId);
				};
				toolbar.addButton(this, callback, unArchiveAction);

				var row = this.addRow(ACTION_PREFIX + user.itemId, [user.name, user.userId, emailAddress, toolbar.domNode]);
				dojo.toggleClass(row.parentNode,"userTable",true);
			}));
		}
	},
	
	unarchiveUser: function (itemId) {
		var args = {
			"itemId": itemId,
			"archived": false
		};
		var srh = new ServiceResponseHandler(this, "unarchiveCallback", "handleError");
		AdminClient.postArchiveUser(args, srh);
	},
	
	unarchiveCallback: function (retVal) {
		var data = dojo.fromJson(retVal);
		if (data.errMessage) {
			this.header.showMessage({
				type: STATUS_TYPES.ERROR,
				summary: dojo.string.substitute(this.messages.errorUnarchiving, [data.errMessage])
			});
		}
		else {
			this.refresh();
		}
	},
	
	updatePagerDivLinks: function () {
		if (this.pagerDiv != null) {
			this.pagerDiv.prevLink.href = "#" + SEARCH_ACTION + "&" + this.latestSearchType + "=" + encodeURIComponent(this.latestSearchTerm) + "&" + "pageNum=" + (this.queryPage - 1);
			this.pagerDiv.nextLink.href = "#" + SEARCH_ACTION + "&" + this.latestSearchType + "=" + encodeURIComponent(this.latestSearchTerm) + "&" + "pageNum=" + (this.queryPage + 1);
		}
	}	
});

})();