/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function () {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.users.ManageUserCALs");

dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");
dojo.require("com.ibm.team.repository.web.client.internal.licensing");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.ui.internal.TeamContributorSelectionDialog");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.users.ReassignLicensesDialog");
dojo.require("com.ibm.team.repository.web.ui.internal.common.Toolbar");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.ItemList");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.users.UserHelper");
dojo.require("jazz.app.ua.PageHoverHelp");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.ui.Toolbar");
dojo.require("jazz.ui.toolbar.Button");
dojo.require("jazz.ui.toolbar.Separator");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.users", "ManageUserCALsMessages");

var AdminClient = com.ibm.team.repository.web.client.internal.AdminClient;
var LicenseClient = com.ibm.team.repository.web.client.internal.licensing;
var ItemList = com.ibm.team.repository.web.ui.internal.admin.ItemList;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var TeamContributorSelectionDialog = com.ibm.team.repository.web.ui.internal.TeamContributorSelectionDialog;
var UserHelper = com.ibm.team.repository.web.ui.internal.admin.users.UserHelper;
var MessageBox = jazz.ui.MessageBox;
var STATUS_TYPES = MessageBox.STATUS_TYPES;
var LICENSE_KEY_MANAGEMENT_ACTION = "#action=com.ibm.team.repository.admin.manageLicenses";
var SEARCH_ACTION = "action=com.ibm.team.repository.manageUserCALs";
var Toolbar = com.ibm.team.repository.web.ui.internal.common.Toolbar;
var PageHoverHelp = jazz.app.ua.PageHoverHelp;
var PAGE_SIZE = 50;
var PAGE_SIZES = [10, 50, 100, 150];
var messages;
var ReassignLicensesDialog = com.ibm.team.repository.web.ui.internal.admin.users.ReassignLicensesDialog;

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.users.ManageUserCALs", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/templates/ManageUserCALs.html"),
	widgetsInTemplate: true,
	
	constructor: function () {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.users", "ManageUserCALsMessages");
		this.pageSize = PAGE_SIZE;
	},
	
	postCreate: function() {
		this.header = new MessageBox();
		this.headerArea.appendChild(this.header.domNode);
		this.licenseMap = {};
		
		this.userHelper = new UserHelper({
			messages: this.messages
		});
		
		//Append the context sensitive help
		var help = new PageHoverHelp({ uaKey: "com.ibm.jazz.repository.web.admin.ui.clientAccessMgmt" });
		this.helpHover.appendChild(help.domNode);
		
		//Create a new ItemList & some hacks.
		this.itemList = new ItemList();
		this.itemList.setHeaderRow([this.messages.name, this.messages.userId, this.messages.email ]);
		dojo.create("th", { innerHTML: this.messages.actionsLabel, className: "actions" }, this.itemList.headerRow);
		this.itemListTable.appendChild(this.itemList.domNode);
		this.itemList.headerRowLine.parentNode.removeChild(this.itemList.headerRowLine);
		dojo.addClass(this.itemList.tableContents, "actions-table");
		dojo.addClass(this.itemList.navContainer, "noDisplay");
		
		this.assignButton.onClick = dojo.hitch(this, "onAssignLicenseClicked");
		this.reassignAllButton.onClick = dojo.hitch(this, "onReassignLicensesClicked");
		
		this.connect(this.choosePageSize, "onchange", "handlePageChanged");
		this.connect(this.licenseSelector, "onchange", "handleLicenseSelected");			
		this.addPageSizes();
	},
	
	refresh: function () {
		this.clearState();
		var srh = new ServiceResponseHandler(this, "getLicenseInfoCallback", "handleError");
		LicenseClient.getLicenseInfo(srh);		
	},
	
	reload: function () {
		this.refresh();
		this.runLatestSearch();
	},
	
	//Called by UsersPage.js
	show: function() {
		this.refresh();
	},
	
	onAssignLicenseClicked: function() {
		var userPicker = new TeamContributorSelectionDialog({ 'disableUsersWithLicense' : this.latestSearch.licenseName });
		this.connect(userPicker,"onAdd","assignLicenses");
	},
	
	onReassignLicensesClicked: function() {
		var license = this.licenseMap[this.selectedLicenseType];
		var licenseName;
		if (license) {
			var productName = license.productName;
			var name = license.name;
			var variantName = license.variantName;
			licenseName = productName + " - " + name + (variantName ? " (" + variantName + ")" : ""); 
		}
		var args = {
			sourceLicenseType: this.selectedLicenseType,
			sourceLicenseName: licenseName,
			licenses: this.availableForReassign
		}
		var dialog = new ReassignLicensesDialog(args);
		this.connect(dialog, "onChange", function() {
			this.reload();
		});
	},
	
	canReassignLicense: function(source,target){			
		var unlimited = target.total == -1;		
		if (!unlimited) {
			var available = target.total - target.used;
		}
		var disabled = target.disabledReason != null;				 		
		return !disabled && (unlimited || source.used <= available);		
	},
	
	//User can specify license type and pageNum in the URL to pre-load the page
	handleURLActions: function(licenseType, pageNum, pageSize) {
		this.pageSize = pageSize || PAGE_SIZE;
		this.addPageSizes();
		this.selectedLicenseType = licenseType;
		this.runLicenseSearch(this.selectedLicenseType, pageNum || 0);
	},
	
	clearState: function () {
		dojo.empty(this.itemList.tbody);
		this.header.hideMessage();
		var tr = dojo.create("tr", {}, this.itemList.tbody);
		dojo.create("td", { className: "noRowsYet", colSpan: 4, innerHTML: this.messages.selectAKeyFirst }, tr);
		this.licenseSelector.innerHTML = "";
		dojo.create("option", { innerHTML: this.messages.selectKey }, this.licenseSelector);
		this.assignButton.attr("enabled", false);
	},
	
	addPageSizes: function() {
		this.choosePageSize.innerHTML = "";
		dojo.forEach(PAGE_SIZES, function(size) {
			var option = dojo.create("option", {value: size, innerHTML: size, selected: size == this.pageSize }, this.choosePageSize);
		}, this);
	},
	
	handlePageChanged: function(e) {
		dojo.stopEvent(e);
		this.setPageSize(this.choosePageSize.value);
	},
	
	setPageSize: function(num) {
		this.pageSize = num || PAGE_SIZE;
		if(this.latestSearch) {
			this.latestSearch.pageNum = 0;
		}
		this.runLatestSearch();
	},
	
	handleLicenseSelected: function(e) {				
		dojo.stopEvent(e);
		window.location.href = "#" + SEARCH_ACTION + "&licenseType=" + this.licenseSelector.value;
		this.selectedLicenseType = this.licenseSelector.value;			
	},
	
	/**
	 * Looks to see if there are any licenses available to reassign to other than the selected one.  The crieria must be that there are 
	 * enough of the target licenses to accomplish this.
	 *
	 */ 
	updateReassignButton: function(){
		this.availableForReassign = [];
		if (this.latestSearch) {
			var sourceLicense = this.licenseMap[this.latestSearch.licenseName];
			if (sourceLicense) {
				if (this.licenseCountArea[sourceLicense.id]) {				
					// Only enable the button if there is something to do.. i.e licenses are used																	 
					if (this.licenseCountArea[sourceLicense.id]["used"] > 0) {
						for (var key in this.licenseMap) {
							var license = this.licenseMap[key];
							if (license && license.id != sourceLicense.id) {
								var canReassign =  this.canReassignLicense(sourceLicense, license);
								if (canReassign) {							
									this.availableForReassign.push(license);					
								}
							}
						}
					}
				}
			}
		}
		this.reassignAllButton.attr("enabled", this.availableForReassign.length > 0);
	},

					
	getLicenseInfoCallback: function (data) {
		if (data.types) {
			var self = this;
			var licenses = LicenseClient.sortCALs(data.types);
			dojo.forEach(licenses, function (license) {
				self.licenseMap[license.id] = license;
				var licenseInfo = LicenseClient.getClientAccessLicenseInfo(data, license);
				var displayName = licenseInfo.name;
				if (licenseInfo.variantName) {
					displayName += " (" + licenseInfo.variantName + ")";
				}
				var option = dojo.create("option", {id: license.id, value: license.id, innerHTML: displayName, 
					selected: self.selectedLicenseType == license.id  }, self.licenseSelector);
				
				self.licenseDescription[license.id] = license.description;
				
				if (license.expirationTime) {
					var obj = LicenseClient.getExpirationInfo(license.expirationTime);
					self.licenseExpiration[license.id] = obj.expiresOnString;
				} else {
					self.licenseExpiration[license.id] = "";
				}
				
				if (self.licenseCountArea[license.id]) {
					self.licenseCountArea[license.id]["disabled"] = licenseInfo.available === -2;
					self.licenseCountArea[license.id]["disabledReason"] = licenseInfo.available === -2 ? license.disabledReason : "";
					self.licenseCountArea[license.id]["unlimited"] = licenseInfo.available === -1;
					self.licenseCountArea[license.id]["total"] = license.trial ? license.max : license.total;
					if (self.latestSearch != null && self.latestSearch.licenseName == license.id) {
						self.showAvailableCount(licenseInfo.available);
					}
				}
				else {
					self.licenseCountArea[license.id] = {
						total: license.trial ? license.max : license.total,
						unlimited: licenseInfo.available === -1,
						disabled: licenseInfo.available === -2,
						disabledReason: licenseInfo.available === -2 ? license.disabledReason : "",
						variant: licenseInfo.variantName
					};
				}
			});
			this.updateHeaderLinks();
			// Update the reassign button with the latest search results 
			this.updateReassignButton();
			this.itemList.tbody.parentNode.style.display = "";
		}
		else {
			dojo.addClass(this.controls,"noDisplay");
			dojo.addClass(this.itemList.tbody.parentNode,"noDisplay");
			this.header.showMessage({
				type: STATUS_TYPES.INFO,
				summary: dojo.string.substitute(this.messages.noCALsInstalled, ["<a href=\"" + LICENSE_KEY_MANAGEMENT_ACTION + "\">", "</a>"])
			});
		}
		
	},
	
	updateHeaderLinks: function () {
		if (this.licenseSelector && this.licenseSelector.childNodes && this.latestSearch) {
			var activeLicense = this.latestSearch.licenseName;
			for (var i=0; i < this.licenseSelector.childNodes.length; i++) {
				if (this.licenseSelector.childNodes[i].licenseId == activeLicense) {
					dojo.toggleClass(this.licenseSelector.childNodes[i], "selected", true);
					this.licenseSelector.childNodes[i].disabled = true;
				}
				else {
					dojo.toggleClass(this.licenseSelector.childNodes[i], "selected", false);
					this.licenseSelector.childNodes[i].disabled = false;
				}
			}
			this.showLicenseDetails(activeLicense);
			this.updateLicenseButton(activeLicense);
			this.showAvailableCount(activeLicense);
			this.header.hideMessage();
		}
	},
	
	runLatestSearch: function () {
		if (this.latestSearch != null) {
			this.runLicenseSearch(this.latestSearch.licenseName, this.latestSearch.pageNum);
		}
	},
	
	runLicenseSearch: function (licenseName, pageNum) {
		if(!licenseName){
			this.updateReassignButton();	
		 	return;
		}
		this.showLicenseDetails(licenseName);
		this.updateLicenseButton(licenseName);
		if (licenseName != null) {
			this.latestSearch = {
				type: "license",
				licenseName: licenseName,
				pageNum: pageNum
			};
			this.updateHeaderLinks();						
			this.latestSearchTerm = licenseName;
			this.latestSearchType = "licenseType";
			this.queryPage = pageNum || 0;
			this.updatePagerDivLinks();
			dojo.empty(this.itemList.tbody);
			var tr = dojo.create("tr", {}, this.itemList.tbody);
			dojo.create("td", { className: "noRowsYet", colSpan: 4, innerHTML: this.messages.searchingForUsers }, tr);
			var srh = new ServiceResponseHandler(this, "showUsers", "handleError");
			AdminClient.getContributors({searchTerm: licenseName, isLicenseSearch: true, pageSize: this.pageSize, hideGuest: true, hideUnassigned: true, pageNum: pageNum}, srh);
		}
		
	},
	
	showLicenseDetails: function (licenseId) {
		this.licenseDescription.innerHTML = this.licenseDescription[licenseId] || "";
		this.licenseExpiration.innerHTML = this.licenseExpiration[licenseId] || "";
	},
	
	updateLicenseButton: function (licenseId) {
		if (this.licenseMap[licenseId]) {
			this.assignButton.innerHTML = dojo.string.substitute(this.messages.assignLicenses, [ this.licenseMap[licenseId].name ]);
		}
	},
	
	showAvailableCount: function (licenseId) {
		this.licenseCountArea.innerHTML = "";
		if (this.licenseCountArea[licenseId] && this.licenseCountArea[licenseId]["total"] != null &&
				this.licenseCountArea[licenseId]["used"] != null) {
			var unlimited = this.licenseCountArea[licenseId]["unlimited"];
			if (!unlimited) {
				var available = this.licenseCountArea[licenseId]["total"] - this.licenseCountArea[licenseId]["used"];
			}
			var disabled = this.licenseCountArea[licenseId]["disabled"];
			if (disabled) {
				var disabledReason = this.licenseCountArea[licenseId]["disabledReason"];
				this.header.showMessage({
					type: STATUS_TYPES.WARNING,
					summary: dojo.string.substitute(this.messages.calDisabled, [ disabledReason ])
				})	
				// set a flag if this license is disabled because it is assigned to more users than are allowed
				this.isOverAssigned = disabledReason != null && disabledReason.indexOf("CRJAZ0969I") === 0;
			}
			else {
				this.isOverAssigned = false;
				if (!unlimited) {
					this.licenseCountArea.innerHTML = dojo.string.substitute(this.messages.licensesAvailable, [available]);
				}
			}
			this.assignButton.attr("enabled", unlimited || (available && !disabled));
		}
	},
	
	showUsers: function (resultSet) {
		dojo.empty(this.itemList.tbody);
		if (this.pagerDiv == null) {
			this.pagerDiv = this.userHelper.createPagerDiv();
			this.pagerContainer.appendChild(this.pagerDiv);
			this.updatePagerDivLinks();
		}
		if (this.licenseCountArea[this.latestSearch.licenseName]) {
			this.licenseCountArea[this.latestSearch.licenseName]["used"] = resultSet.count;
			this.showAvailableCount(this.latestSearch.licenseName);
		}
		else {
			this.licenseCountArea[this.latestSearch.licenseName] = { used: resultSet.count };
		}
		if (resultSet == null || resultSet.elements == null || resultSet.elements.length == 0) {
			var tr = dojo.create("tr", {}, this.itemList.tbody);
			dojo.create("td", { className: "noRowsYet", colSpan: 4, innerHTML: this.messages.noMatchingUsers }, tr);
			tr.onclick = function (e) {
				dojo.stopEvent(e);
			};
			dojo.toggleClass(this.pagerDiv.prevSpan,"disabled", true);
			dojo.toggleClass(this.pagerDiv.nextSpan,"disabled", true);
			this.pagerDiv.rangeSpan.innerHTML = "0";
			this.pagerDiv.totalSpan.innerHTML = "0";
		}
		else {
			this.pagerDiv.rangeSpan.innerHTML = "";
			this.pagerDiv.totalSpan.innerHTML = "";
			if (resultSet.count > this.pageSize) {
				var lowerBound = (this.queryPage * this.pageSize) + 1;
				var upperBound = (this.queryPage + 1) * this.pageSize;
				dojo.toggleClass(this.pagerDiv.prevSpan,"disabled", this.queryPage == 0);
				
				if (upperBound >= resultSet.count) {
					upperBound = resultSet.count;
					dojo.toggleClass(this.pagerDiv.nextSpan,"disabled", true);
				}
				else {
					dojo.toggleClass(this.pagerDiv.nextSpan,"disabled", false);
				}
				this.pagerDiv.rangeSpan.innerHTML = lowerBound + " - " + upperBound;
			}
			else {
				dojo.toggleClass(this.pagerDiv.prevSpan,"disabled", true);
				dojo.toggleClass(this.pagerDiv.nextSpan,"disabled", true);
				this.pagerDiv.rangeSpan.innerHTML = "1 - " + resultSet.count;
			}
			this.pagerDiv.totalSpan.innerHTML = resultSet.count;
			dojo.forEach(resultSet.elements, dojo.hitch(this, function(user){
				var nameNode = dojo.create("div", { innerHTML: user.name });		
				var toolbar = new Toolbar();
				var unassignAction = {};
				unassignAction.imgSrc = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/images/delete_hov.gif");
				unassignAction.label = this.messages.unassignLicense;
				unassignAction.callbackData = {
					userId: user.userId
				};
				var callback = function(e){
					dojo.stopEvent(e);
					this.unassignLicense(e.currentTarget.callbackData.userId);
				};
				toolbar.addButton(this, callback, unassignAction);
				var row = this.itemList.addRow(null, [nameNode, user.userId, user.emailAddress, toolbar.domNode ]);
				dojo.toggleClass(row.parentNode,"userTable",true);
			}));
		}
		this.updateReassignButton();		
	},
	
	assignLicenses: function (newUsers) {
		if (newUsers != null) {
			var userIds = [];
			for (var i=0; i < newUsers.length; i++) {
				var user = newUsers[i];
				userIds.push(user.userId);
			}
			var args = {
				jsonUserIds: dojo.toJson(userIds),
				licenseId: this.latestSearch.licenseName
			};
			var srh = new ServiceResponseHandler(this, "assignLicensesCallback", "handleError");
			LicenseClient.assignLicenseMultiple(srh, args);
		}
	},
	
	assignLicensesCallback: function (response) {
		this.runLatestSearch();
		if (response) {
			var detailsObject = new Array();
			dojo.forEach(dojo.fromJson(response), function(error) {
				detailsObject.push({
					message: error.userId + ": " + error.message,
					code: STATUS_TYPES.ERROR
				});	
			});
			this.header.showMessage({
				type: STATUS_TYPES.ERROR,
				summary: this.messages.errorAssigning,
				details: detailsObject,
				closable: true
			});
		}
		else {
			this.header.hideMessage();
		}
	},
	
	unassignLicense: function (itemId) {
		var srh = new ServiceResponseHandler(this, "unassignLicenseCallback", "handleError");
		LicenseClient.unassignLicense(srh, { userId: itemId, licenseId: this.latestSearch.licenseName });
	},
	
	unassignLicenseCallback: function (response) {
		if (this.isOverAssigned) {
			this.reload();
		}
		else {
			this.runLatestSearch();
			this.header.hideMessage();
		}
	},
	
	updatePagerDivLinks: function () {
		if (this.pagerDiv) {
			this.pagerDiv.prevLink.href = "#" + SEARCH_ACTION + "&" + this.latestSearchType + "=" + encodeURIComponent(this.latestSearchTerm) + "&" + "pageNum=" + (this.queryPage - 1) + "&" + "pageSize=" + this.pageSize;
			this.pagerDiv.nextLink.href = "#" + SEARCH_ACTION + "&" + this.latestSearchType + "=" + encodeURIComponent(this.latestSearchTerm) + "&" + "pageNum=" + (this.queryPage + 1) + "&" + "pageSize=" + this.pageSize;
		}
	},
	
	handleError: function(errorObj){
		this.header.showMessage({
			errorObject: errorObj
		});
	}
});

})();