/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function () {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.users.ManageUserGroups");

dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("dojox.string.BidiComplex");
dojo.require("dojox.xml.parser");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.ItemList");
dojo.require("com.ibm.team.repository.web.ui.internal.common.Toolbar");
dojo.require("jazz.ui.MessageBox");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.users", "ManageUserGroupsMessages");

var getAuthenticatedContributor = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;
var AdminClient = com.ibm.team.repository.web.client.internal.AdminClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var MessageBox = jazz.ui.MessageBox;
var STATUS_TYPES = MessageBox.STATUS_TYPES;
var Toolbar = com.ibm.team.repository.web.ui.internal.common.Toolbar;
var REST_USER_GROUP_URL = "user-groups/";

function setText(e, text){
	e.appendChild(document.createTextNode(text));
}

var ACTION_PREFIX = "com.ibm.team.repository.editUserGroup&itemId=";
var PAGE_SIZE = 250;

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.users.ManageUserGroups", com.ibm.team.repository.web.ui.internal.admin.ItemList, {
	
	constructor: function () {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.users", "ManageUserGroupsMessages");
	},
	
	postCreate: function() {
		this.header = new MessageBox();
		this.headerArea.appendChild(this.header.domNode);	
		
		//Inject our CSS into the ItemList domNode (Shares the Manage Users styles since they are so similar)
		dojo.addClass(this.itemListNode, "com-ibm-team-repository-users-manageUsers")
			
		setText(this.heading, this.messages.userGroups);
		setText(this.description, this.messages.pageDescription);
		this.tableHeading.parentNode.removeChild(this.tableHeading);
		this.headerRowLine.parentNode.removeChild(this.headerRowLine);
		this.setHeaderRow([this.messages.name, this.messages.summary]);
		this.headerRow.cells[0].sortBy = "name";
		this.headerRow.cells[1].sortBy = "summary";
		dojo.toggleClass(this.headerRow, "sortable", true);
		this.connect(this.headerRow, "onclick", "updateSorting");
		this.connect(this.headerRow, "onmouseover", function (e) {
			dojo.toggleClass(e.target, "selected", true);
		});
		this.connect(this.headerRow, "onmouseout", function (e) {
			dojo.toggleClass(e.target, "selected", false);
		});
		
		//Modify the ItemList table to have an actions column
		dojo.create("th", { innerHTML: this.messages.actionsLabel, className: "actions" }, this.headerRow);
		dojo.addClass(this.tableContents, "toolbar-table");
		
		//Create new buttons in the topControls div
		this.newGroupButton = dojo.create("div", { className: "create", innerHTML: this.messages.createGroup }, this.topControls);
		this.connect(this.newGroupButton,"onclick", function (e) {
			dojo.stopEvent(e);
			window.location.href = "#action=" + ACTION_PREFIX + "new";
		});
		
		this.pageNumberContainer = document.createElement("div");
		this.pageNumberContainer.style.marginTop = "5px";
		this.pageNumberContainer.style.fontWeight = "normal";
		this.pageNumberContainer.style.display = "none";
		this.pageNumberContainer.appendChild(document.createTextNode(dojo.string.substitute(this.messages.showingFirstGroups, [ PAGE_SIZE ])));
		this.tableControls.appendChild(this.pageNumberContainer);
		
		this.refresh();
	},
	
	refresh: function () {
		this.header.hideMessage();
		this.clearState();
		var tr = dojo.create("tr", {}, this.tbody);
		dojo.create("td", { className: "noRowsYet", colSpan: 4, innerHTML: this.messages.searchingForGroups }, tr);	
	},
	
	show: function () {
		this.refresh();
	},
	
	reload: function () {
		this.refresh();
		this.getAllGroups();
	},
	
	clearState: function () {
		dojox.xml.parser.removeChildren(this.tbody);
		this.pageNumberContainer.style.display = "none";
		this.header.hideMessage();
	},
	
	getAllGroups: function () {
		var url = this._getServerUrl() + net.jazz.ajax._contextRoot + "/" + REST_USER_GROUP_URL;
		var srh = new ServiceResponseHandler(this, "showGroups", "handleError");
		var xhrArgs = {
	        url: url
	    }
			
		var deferred = jazz.client.xhrGet(xhrArgs);
		deferred.addCallback(function(response){
			net.jazz.ajax.core.internal.util.callContributedCode(function() {
				srh.doCallback(response);
			});
		});
		deferred.addErrback(function(response){
			net.jazz.ajax.core.internal.util.callContributedCode(function() {
				srh.doErrorCallback(response);
			});
		});
	},
	
	updateSorting: function (e) {
		//TODO: Implement sorting by name
	},
	
	showGroups: function (response) {
		var resultSet = dojo.fromJson(response);
		var amount = resultSet.length;
		dojox.xml.parser.removeChildren(this.tbody);
		if (resultSet == null || resultSet.length == 0) {
			var tr = document.createElement("tr");
			dojo.create("td", { className: "noRowsYet", colSpan: 4, innerHTML: this.messages.noMatchingGroup }, tr);
			tr.onclick = function (e) {
				dojo.stopEvent(e);
			};
			this.tbody.appendChild(tr);
		}
		else {
			dojo.forEach(resultSet, dojo.hitch(this, function(group){
				var toolbar = new Toolbar();
				var deleteAction = {};
				deleteAction.imgSrc = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/images/delete_hov.gif");
				deleteAction.label = this.messages.deleteGroup;
				deleteAction.callbackData = {
					itemId: group.itemId
				};
				var callback = function(e){
					dojo.stopEvent(e);
					var verify = confirm(this.messages.confirmDelete);
					if(verify) {
						this.deleteGroup(e.currentTarget.callbackData.itemId);	
					}
				};
				toolbar.addButton(this, callback, deleteAction);
				
				var row = this.addRow(ACTION_PREFIX + group.itemId, [group.name, group.summary, toolbar.domNode]);
				dojo.toggleClass(row.parentNode,"userTable",true);
			}));
		}
	},
	
	_getServerUrl: function(){
		var location = document.location.href;
		var separator = "/"; //$NON-NLS-1$
		var path = location.split(separator); //$NON-NLS-1$
		var serverAddress;
		if (path.length > 0) {
			serverAddress = path.shift() + separator + separator;
			path.shift(); //empty from the double //
			serverAddress = serverAddress + path.shift();
		}
		return serverAddress;
	},
	
	deleteGroup: function (groupId) {
		var url = this._getServerUrl() + net.jazz.ajax._contextRoot + "/" + REST_USER_GROUP_URL + groupId;
		var srh = new ServiceResponseHandler(this, "getAllGroups", "handleError");
		var xhrArgs = {
			url: url,
			// ask the server to send JSON and expect form encoded data
			headers: {
				"accept": "text/json",
				"content-type": "application/x-www-form-urlencoded; charset=utf-8",
				"X-Jazz-CSRF-Prevent": dojo.cookie("JSESSIONID")
			},
			// ask Dojo to expect and automatically demarshall a JSON response body
			handleAs: "json"
		}
			
		var deferred = jazz.client.xhrDelete(xhrArgs);
		deferred.addCallback(function(response){
			net.jazz.ajax.core.internal.util.callContributedCode(function() {
				srh.doCallback(response);
			});
		});
		deferred.addErrback(function(response){
			net.jazz.ajax.core.internal.util.callContributedCode(function() {
				srh.doErrorCallback(response);
			});
		});
	}
});

})();