/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function () {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.users.ManageUsers");

dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("dojox.string.BidiComplex");
dojo.require("dojox.xml.parser");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.ItemList");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.users.ImportUsersDialog");
dojo.require("com.ibm.team.repository.web.ui.internal.common.Toolbar");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.users.UserHelper");
dojo.require("jazz.ui.Toolbar");
dojo.require("jazz.ui.toolbar.Button");
dojo.require("jazz.ui.toolbar.Separator");
dojo.require("jazz.app.ua.PageHoverHelp");
dojo.require("jazz.ui.MessageBox");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.users", "ManageUsersMessages");

var getAuthenticatedContributor = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;
var AdminClient = com.ibm.team.repository.web.client.internal.AdminClient;
var ImportUsersDialog = com.ibm.team.repository.web.ui.internal.admin.users.ImportUsersDialog;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var MessageBox = jazz.ui.MessageBox;
var STATUS_TYPES = MessageBox.STATUS_TYPES;
var UserHelper = com.ibm.team.repository.web.ui.internal.admin.users.UserHelper;
var Toolbar = com.ibm.team.repository.web.ui.internal.common.Toolbar;
var PageHoverHelp = jazz.app.ua.PageHoverHelp;

function setText(e, text){
	e.appendChild(document.createTextNode(text));
}

var ACTION_PREFIX = "com.ibm.team.repository.editUser&itemId=";
var SEARCH_ACTION = "action=com.ibm.team.repository.searchUsers";
var PAGE_SIZE = 50;

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.users.ManageUsers", com.ibm.team.repository.web.ui.internal.admin.ItemList, {
	
	constructor: function () {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.users", "ManageUsersMessages");
		this._actionToolbars = [];
	},
	
	postCreate: function() {
		this.header = new MessageBox();
		this.headerArea.appendChild(this.header.domNode);		
		
		this.userHelper = new UserHelper({
			messages: this.messages
		});
		
		//Append the context sensitive help
		var help = new PageHoverHelp({ uaKey: "com.ibm.jazz.repository.web.admin.ui.activeUsers" });
		this.helpHover.appendChild(help.domNode);
				
		//Add the Table Controls table
		this.controls = this.userHelper.buildTableControls(this.tableControls);
		
		//Inject our CSS into the ItemList domNode
		dojo.addClass(this.itemListNode, "com-ibm-team-repository-users-manageUsers");
		
		setText(this.heading, this.messages.activeUsers);
		setText(this.description, this.messages.pageDescription);
		setText(this.userPagesDescription, this.messages.usersPageDescription);
		this.tableHeading.parentNode.removeChild(this.tableHeading);
		this.headerRowLine.parentNode.removeChild(this.headerRowLine);
		
		this.searchText = this.userHelper.buildFilterBox(this.controls["col3"], this.messages.searchPlaceholder, this.messages.searchUsers, SEARCH_ACTION);
		
		this.setHeaderRow([this.messages.name, this.messages.userId, this.messages.email]);
		this.headerRow.cells[0].sortBy = "name";
		this.headerRow.cells[1].sortBy = "userId";
		this.headerRow.cells[2].sortBy = "emailaddress";
		dojo.toggleClass(this.headerRow, "sortable", true);
		this.connect(this.headerRow, "onclick", "updateSorting");
		this.connect(this.headerRow, "onmouseover", function (e) {
			dojo.toggleClass(e.target, "selected", true);
		});
		this.connect(this.headerRow, "onmouseout", function (e) {
			dojo.toggleClass(e.target, "selected", false);
		});
		
		//Modify the ItemList table to have an actions column
		dojo.create("th", { innerHTML: this.messages.actionsLabel, className: "actions" }, this.headerRow);
		dojo.addClass(this.tableContents, "actions-table");
		
		//Create new buttons in the topControls div
		var toolbar = new jazz.ui.Toolbar();
		this.newUserButton = new jazz.ui.toolbar.Button({
			label: this.messages.createUser,
			imgSrc: "com.ibm.team.repository.web/ui/internal/admin/users/images/create-user_co.gif",
			onClick: function() {
				window.location.href = "#action=" + ACTION_PREFIX + "new";
			}
		});
		toolbar.addChild(this.newUserButton);
		toolbar.addChild(new jazz.ui.toolbar.Separator);
		this.importUserButton = new jazz.ui.toolbar.Button({
			label: this.messages.importUsers,
			imgSrc: "com.ibm.team.repository.web/ui/internal/admin/users/images/import-users_co.gif",
			onClick: dojo.hitch(this, function() {
				var dialog = new ImportUsersDialog();
				this.connect(dialog,"onOk","performImport");
			})
		});
		toolbar.addChild(this.importUserButton);
		this.topControls.appendChild(toolbar.domNode);

		var srh = new ServiceResponseHandler(this, "getUserRegistryInfoCallback", "handleError");
		AdminClient.getUserRegistryInfo(srh);
		
		this.pageNumberContainer = document.createElement("div");
		this.pageNumberContainer.style.marginTop = "5px";
		this.pageNumberContainer.style.fontWeight = "normal";
		this.pageNumberContainer.style.display = "none";
		this.pageNumberContainer.appendChild(document.createTextNode(dojo.string.substitute(this.messages.showingFirstUsers, [ PAGE_SIZE ])));
		this.controls["col1"].appendChild(this.pageNumberContainer);
		
		this.refresh();
	},
	
	refresh: function () {
		this.header.hideMessage();
		this.clearState();
		var tr = dojo.create("tr", {}, this.tbody);
		dojo.create("td", { className: "noRowsYet", colSpan:4, innerHTML: this.messages.searchingForUsers }, tr);
		this.runLatestSearch();
	},
	
	show: function () {
		this.refresh();
	},
	
	reload: function () {
		this.refresh();
	},
	
	clearState: function () {
		if(this.searchText.value != this.messages.searchPlaceholder) {
			this.searchText.value = "";	
		}
		dojox.xml.parser.removeChildren(this.tbody);
		this.pageNumberContainer.style.display = "none";
		this.header.hideMessage();
		this._destroyActionToolbars();
	},
	
	getUserRegistryInfoCallback: function (userRegistryInfo) {
		if (userRegistryInfo != null) {
			if (userRegistryInfo.queryable) {
				this.importUserButton.attr("enabled", true);
			}
		}
	},
	
	performImport: function (users) {
		if (users != null && users.length > 0) {
			for (var i=0; i < users.length; i++) {
				if (users[i].emailAddress == null || users[i].emailAddress == "") {
					users[i].emailAddress = this.messages.unknown;
				}
			}
			var srh = new ServiceResponseHandler(this, "handleImportCallback", "errorImportCallback");
			AdminClient.postMultipleNewContributors({jsonUserInfo: dojo.toJson(users)}, srh);
		}
	},
	
	handleImportCallback: function (response) {
		this.runUserSearch(this.latestSearchTerm || "");
		if (response == null) {
			this.header.showMessage({
				type: STATUS_TYPES.OK, 
				summary: this.messages.successfulImport
			});
		}
		else {
			var errObject = dojo.fromJson(response);
			var errNode = document.createElement("span");
			errNode.appendChild(document.createTextNode(this.messages.failedImport));
			errNode.appendChild(document.createElement("br"));
			for (var i=0; i < errObject.length; i++) {
				errNode.appendChild(document.createElement("br"));
				errNode.appendChild(document.createTextNode(errObject[i].userId + ": " + errObject[i].exception));
			}
			this.header.showMessage({
				type: STATUS_TYPES.ERROR, 
				summary: errNode
			});
		}
	},
	
	errorImportCallback: function (err) {
		this.refresh();
	},
	
	runLatestSearch: function () {
		if (this.latestSearch != null) {
			if (this.latestSearch.type === "user") {
				if(this.searchText.value != this.messages.searchPlaceholder) this.searchText.value = this.latestSearch.searchTerm;
				this.runUserSearch(this.latestSearch.searchTerm, this.latestSearch.pageNum);
			}
			else if (this.latestSearch.type === "license") {
				this.runLicenseSearch(this.latestSearch.licenseName, this.latestSearch.pageNum);
			}
			else {
				this.runUserSearch("");
			}
		}
		else {
			this.runUserSearch("");
		}
	},
	
	runUserSearch: function (searchTerm, pageNum, sortBy, sortOrder) {
		if (searchTerm == null) {
			searchTerm = "";
		}
		if (sortBy == null) {
			sortBy = "name";
		}
		if (sortOrder !== "desc") {
			sortOrder = "asc";
		}
		if (pageNum == null) {
			pageNum = 0;
		}
		this.latestSearch = {
			type: "user",
			searchTerm: searchTerm,
			pageNum: pageNum,
			sortBy: sortBy,
			sortOrder: sortOrder
		};
		searchTerm = searchTerm.replace(/\*/g,"%");
		this.latestSearchTerm = searchTerm;
		this.latestSearchType = "searchTerm";
		this.queryPage = pageNum || 0;
		this.updatePagerDivLinks();
		dojox.xml.parser.removeChildren(this.tbody);
		var tr = dojo.create("tr", {}, this.tbody);
		dojo.create("td", { className: "noRowsYet", colSpan: 4, innerHTML: this.messages.searchingForUsers }, tr);
		var srh = new ServiceResponseHandler(this, "showUsers", "handleError");
		AdminClient.getContributors({searchTerm: searchTerm, pageSize: PAGE_SIZE, sortBy: sortBy, sortOrder: sortOrder, hideAdmin: true, hideGuest: true, hideUnassigned: true, pageNum: pageNum, hideArchivedUsers: true}, srh);
	},
	
	updateSorting: function (e) {
		if (this.latestSearch && this.latestSearch.type === "user") {
			var sortByField = e.target.sortBy;
			var sortOrder;
			if (this.latestSearch.sortBy === sortByField) {
				if (this.latestSearch.sortOrder === "desc") {
					sortOrder = "asc";
				}
				else {
					sortOrder = "desc";
				}
			}
			else {
				sortOrder = "asc";
			}
			document.location.href = "#" + SEARCH_ACTION + "&searchTerm=" + encodeURIComponent(this.latestSearch.searchTerm) + "&pageNum=" +
					this.latestSearch.pageNum + "&sortBy=" + sortByField + "&sortOrder=" + sortOrder;
		}
	},
	
	runLicenseSearch: function (licenseName, pageNum) {
		this.latestSearch = {
			type: "license",
			licenseName: licenseName,
			pageNum: pageNum
		};
		if (licenseName == null) {
			licenseName = "";
		}
		this.latestSearchTerm = licenseName;
		this.latestSearchType = "licenseType";
		this.queryPage = pageNum || 0;
		this.updatePagerDivLinks();
		dojox.xml.parser.removeChildren(this.tbody);
		var tr = dojo.create("tr", {}, this.tbody);
		dojo.create("td", { className: "noRowsYet", colSpan: 4, innerHTML: this.messages.searchingForUsers }, tr);
		var srh = new ServiceResponseHandler(this, "showUsers", "handleError");
		AdminClient.getContributors({searchTerm: licenseName, isLicenseSearch: true, pageSize: PAGE_SIZE, hideAdmin: true, hideGuest: true, hideUnassigned: true, pageNum: pageNum}, srh);
	},
	
	showUsers: function (resultSet) {
		dojox.xml.parser.removeChildren(this.tbody);
		if (this.pagerDiv == null) {
			this.pagerDiv = this.userHelper.createPagerDiv();
			this.controls["col2"].appendChild(this.pagerDiv);
			this.updatePagerDivLinks();
		}
		if (resultSet == null || resultSet.elements == null || resultSet.elements.length == 0) {
			var tr = document.createElement("tr");
			dojo.create("td", { className: "noRowsYet", colSpan: 4, innerHTML: this.messages.noMatchingUsers }, tr);
			tr.onclick = function (e) {
				dojo.stopEvent(e);
			};
			this.tbody.appendChild(tr);
			dojo.toggleClass(this.pagerDiv.prevSpan,"disabled", true);
			dojo.toggleClass(this.pagerDiv.nextSpan,"disabled", true);
			this.pagerDiv.rangeSpan.innerHTML = "0";
			this.pagerDiv.totalSpan.innerHTML = "0";
		}
		else {
			this.pagerDiv.rangeSpan.innerHTML = "";
			this.pagerDiv.totalSpan.innerHTML = "";
			if (resultSet.count > PAGE_SIZE) {
				var lowerBound = (this.queryPage * PAGE_SIZE) + 1;
				var upperBound = (this.queryPage + 1) * PAGE_SIZE;
				dojo.toggleClass(this.pagerDiv.prevSpan,"disabled", this.queryPage == 0);
				if (upperBound >= resultSet.count) {
					upperBound = resultSet.count;
					dojo.toggleClass(this.pagerDiv.nextSpan,"disabled", true);
				}
				else {
					dojo.toggleClass(this.pagerDiv.nextSpan,"disabled", false);
				}
				this.pagerDiv.rangeSpan.innerHTML = lowerBound + " - " + upperBound;
			}
			else {
				dojo.toggleClass(this.pagerDiv.prevSpan,"disabled", true);
				dojo.toggleClass(this.pagerDiv.nextSpan,"disabled", true);
				this.pagerDiv.rangeSpan.innerHTML = "1 - " + resultSet.count;
			}
			this.pagerDiv.totalSpan.innerHTML = resultSet.count;
			var loggedInUser = getAuthenticatedContributor();
			dojo.forEach(resultSet.elements, dojo.hitch(this, function(user){
				var emailAddress = dojox.string.BidiComplex.createDisplayString(user.emailAddress,"EMAIL");
				var toolbar = new Toolbar();
				this._actionToolbars.push(toolbar);
				var archiveAction = {};
				archiveAction.imgSrc = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/images/archive_co.gif");
				archiveAction.label = this.messages.archiveUser;
				archiveAction.callbackData = {
					itemId: user.itemId
				};
				var callback = function(e){
					dojo.stopEvent(e);
					var verifyArchive = confirm(this.messages.confirmArchive);
					if(verifyArchive) {
						this.archiveUser(e.currentTarget.callbackData.itemId);	
					}
				};
				var archiveButton = toolbar.addButton(this, callback, archiveAction);
				
				//The logged in user cannot archive himself
				if (user.userId === loggedInUser.userId) {
					toolbar.disableButton(archiveButton);
				}
				var row = this.addRow(ACTION_PREFIX + user.itemId, [user.name, user.userId, emailAddress, toolbar.domNode]);
				dojo.toggleClass(row.parentNode,"userTable",true);
			}));
		}
	},
	
	archiveUser: function (itemId) {
		var args = {
			"itemId": itemId,
			"archived": true
		};
		var srh = new ServiceResponseHandler(this, "archiveCallback", "archiveErrorCallback");
		AdminClient.postArchiveUser(args, srh);
	},
	
	archiveCallback: function (retVal) {
		var data = dojo.fromJson(retVal);
		if (data.errMessage) {
			this.header.showMessage({
				type: STATUS_TYPES.ERROR, 
				summary: dojo.string.substitute(this.messages.errorArchiving, [data.errMessage])
			});
		}
		else {
			this.reload();
		}
	},
	
	archiveErrorCallback: function (errorObj) {
		this.header.showMessage({
			type: STATUS_TYPES.ERROR, 
			summary: this.messages.errorArchiving,
			errorObject: errorObj
		});
	},
	
	updatePagerDivLinks: function () {
		if (this.pagerDiv != null) {
			this.pagerDiv.prevLink.href = "#" + SEARCH_ACTION + "&" + this.latestSearchType + "=" + encodeURIComponent(this.latestSearchTerm) + "&" + "pageNum=" + (this.queryPage - 1);
			this.pagerDiv.nextLink.href = "#" + SEARCH_ACTION + "&" + this.latestSearchType + "=" + encodeURIComponent(this.latestSearchTerm) + "&" + "pageNum=" + (this.queryPage + 1);
		}
	},
	
	_destroyActionToolbars: function(){
		for (var i = 0; i < this._actionToolbars.length; i++) {
			this._actionToolbars[i].destroyRecursive();
		}
		this._actionToolbars = [];
	},
	
	uninitialize: function(){
		this._destroyActionToolbars();
		
		this.inherited(arguments);
	}

});

})();