/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.users.PasswordChangeDialog");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.TextBox");
dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.util.ui");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.users", "PasswordChangeDialogMessages");

(function() {

var MIN_BUTTON_WIDTH = 52;

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.users.PasswordChangeDialog", [dijit._Widget, dijit._Templated], {
		
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/templates/PasswordChangeDialog.html"),
	widgetsInTemplate: true,
	
	constructor: function (args) {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.users", "PasswordChangeDialogMessages");
		this.itemId = args.itemId;
		this.userId = args.userId;
		this.userName = args.userName;
		if (this.userName != null && this.userName != "") {
			this.dialogHeadingMsg = dojo.string.substitute(this.messages.dialogHeading, [ this.userName ]);
		} else {
			this.dialogHeadingMsg = this.messages.changePassword;
		}
	},
	
	postCreate: function () {
		this.header = new jazz.ui.MessageBox();
		this.headerArea.appendChild(this.header.domNode);
		this._updateButtonWidth();
		this.dialog = new jazz.ui.Dialog({
			contentNode: this.contentNode,
			primaryTitle: this.messages.changePassword,
			// FF can autosize the dialog but IE can't
			width: dojo.isIE ? "25em" : "" //$NON-NLS-1$
		});
	},
	
	showErrorMessage: function (msg) {
		this.header.showMessage({
			summary: msg,
			type: jazz.ui.MessageBox.STATUS_TYPES.ERROR
		});
	},
	
	clearErrorMessage: function() {
		this.header.hideMessage();
	},
	
	close: function () {
		this.dialog.close();
	},
	
	_updateButtonWidth: function() {
		var tester = dojo.create("div", { style: { fontSize: "1em" }}, document.body);
		var newWidth = jazz.util.ui.findMaxStringDisplayWidth([this.messages.cancel, this.messages.finish], tester, MIN_BUTTON_WIDTH);
		// add 23 pixels to accommodate button borders and margin
		var newWidthStyle = (newWidth + 23) + "px";
		this.finishButton.style.width = newWidthStyle;
		this.cancelButton.style.width = newWidthStyle;
		document.body.removeChild(tester);
	}
});

})();