/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.users.ProfileEditorPage");

/*
 * Imports 
 */
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");


(function() {

/*
 * Base class for all contributed profile editor pages.
 * Please see: https://jazz.net/wiki/bin/view/Main/TeamProcessUserSettings
 */
dojo.declare("com.ibm.team.repository.web.ui.internal.admin.users.ProfileEditorPage",
	[dijit._Widget, dijit._Templated], {

	// Properties //----------------------------------------------------------->
	
	/*
	 * The item UUID
	 */
	itemUUID: "",
	
	/*
	 * A flag that tracks whether this page is dirty and needs saving or not.
	 * 
	 * Important Note: This is a read-only property for subclasses.
	 * The value of this flag is changed indirectly by the page container
	 * through calls to setDirty() and doSave().
	 */
	dirty: false,

	/*
	 * A flag that tracks whether this page is active (visible) or not (hidden).
	 * 
	 * Important Note: This is a read-only property for subclasses.
	 * The value of this flag is changed indirectly by the page container.
	 */
	active: false,

	/*
	 * A flag that tracks whether this page is stale and needs refreshing or not.
	 * 
	 * When a page becomes stale, a refresh of it's content may be delayed in
	 * order to improve performance.  For example, there is no need to refresh
	 * pages that are currently not active (hidden).
	 */
	stale: false,
	
	/*
	 * A reference to this page's container
	 */
	pageContainer: null,

	/*
	 * The ID of this page
	 */
	pageID: "",
	
	/*
	 * The title of this page
	 */
	pageTitle: "",
	
	// Public Methods //------------------------------------------------------->
	
	/*
 	 * Initializes this page.
 	 * 
 	 * Called by this page's container.
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */	
	postCreate: function() {
	},

	/*
 	 * Destroys this page.
 	 * 
 	 * Called by this page's container.
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	destroy: function() {

		delete this.itemUUID;
		delete this.dirty;
		delete this.active;
		delete this.stale;
		// Note: Do not delete the page container (do not own)
		delete this.pageID;
		delete this.pageTitle;

		this.inherited(arguments);
	},

	/*
 	 * Notifies this page's container as to whether this page is dirty and 
 	 * requires it's input to be saved or not. 
 	 * 
 	 * Called by subclasses. 
 	 * 
 	 * @param Boolean dirty - The flag indicating whether this page is dirty or
 	 *        				  not
 	 * 
 	 * @return void
 	 */		
	setDirty: function(dirty) {
		this.pageContainer.setDirtyEditorPage(this.pageID, dirty);
	},

	/*
 	 * Saves this page's input and marks this page as not being dirty.
 	 * If an error occurs, this page remains dirty and the error message is 
 	 * displayed in this page's container.
 	 * 
 	 * Called by this page's container.  
 	 * 
 	 * @param Function callback - A callback used to notify the page container 
 	 * 							  whether the save operation succeeded or failed.
 	 * 
 	 * 		Callback method signature:
 	 *   
 	 * 				@param String pageID - The ID of this page
 	 * 				@param Error errorObject - An optional error object 
 	 * 										   specified only if the save 
 	 * 										   operation failed.
 	 *                                 
 	 * 				@return void
 	 * 
 	 * 				callback(pageID, errorObject)
 	 * 
 	 * 		e.g. Save operation succeeded
 	 * 		callback(this.pageID);
 	 * 
 	 * 		e.g. Save operation failed
 	 * 		callback(this.pageID, new Error("Failed to save editor page: " + 
 	 * 			this.pageTitle));
 	 * 
 	 * @return void
 	 */		
	doSave: function(callback) {
	},
	
	/*
 	 * Refreshes this page's content.
 	 * 
 	 * Called by this page's container.
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	refresh: function() {
	}	

	// Private Methods //------------------------------------------------------>

	
});

})();
