/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.users.ReassignLicensesDialog");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.TextBox");
dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.util.ui");
dojo.require("com.ibm.team.repository.web.client.internal.licensing");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.ItemList");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.users", "ReassignLicensesDialogMessages");

(function() {

var MIN_BUTTON_WIDTH = 52;
var ItemList = com.ibm.team.repository.web.ui.internal.admin.ItemList;
var LicenseClient = com.ibm.team.repository.web.client.internal.licensing;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
dojo.declare("com.ibm.team.repository.web.ui.internal.admin.users.ReassignLicensesDialog", [dijit._Widget, dijit._Templated], {
		
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/templates/ReassignLicensesDialog.html"),
	widgetsInTemplate: true,
	sourceLicenseType: null,
	licenses: null,
	refreshCallback: null,
	constructor: function (args) {		
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.users", "ReassignLicensesDialogMessages");
		this.dialogHeadingMsg = dojo.string.substitute(this.messages.dialogHeadingMsg, [ args.sourceLicenseName ])			
	},
	
	postCreate: function () {		
		this.header = new jazz.ui.MessageBox();
		this.headerArea.appendChild(this.header.domNode);
		this.createLicenseSelection();
		this._updateButtonWidth();
		this.dialog = new jazz.ui.Dialog({								
			contentNode: this.contentNode,
			primaryTitle: this.messages.reassignLicenses,
			width: dojo.isIE ? "74em" : ""						
		});
		this.finishButton.disabled = true;
		this.connect(this.cancelButton, "onclick", "onCancel");
		this.connect(this.finishButton, "onclick", "onFinish");
	},
	
	onCancel: function(){
		this.close();
	},
	
	onFinish: function(){
		var args = {
			sourceLicenseId : this.sourceLicenseType,
			targetLicenseId : this.targetLicenseId
		}
		var srh = new ServiceResponseHandler(this, "reassignLicensesCallback", "handleError");
		LicenseClient.reassignLicenses(srh , args);
	},
	
	handleError: function(errorObj){	
		this.header.showMessage({
			errorObject: errorObj
		});	
	},
	
	reassignLicensesCallback: function(response) {
		if (response != null && response != ""){
			var errors = dojo.fromJson(response);			
			var span = document.createElement("span");
			span.appendChild(document.createTextNode(dojo.string.substitute(this.messages.errorReassigning, [ "" ])));
			span.appendChild(document.createElement("br"));
			for (var i=0; i < errors.length; i++) {
				span.appendChild(document.createElement("br"));
				span.appendChild(document.createTextNode(errors[i].userId + ": " + errors[i].message));
			}
			
			this.showErrorMessage(span);
			this.finishButton.disabled = true;
		}
		else{			
			// Success, close the dialog
			this.onChange();
			this.close();
		}
	},
	
	onChange: function(){
		// Meant to be overriden
	},
	
	createLicenseSelection: function() {
		var label = dojo.create("label", {innerHTML: this.dialogHeadingMsg , style : "font-weight: heavy;"}, this.dialogBody);
	
		this.licenseTable = new ItemList();		
		dojo.style(this.licenseTable.navContainer , "display" , "none");
		
		this.licenseTable.setHeaderRow(["" , this.messages.productName , this.messages.name, this.messages.variant ]);
		this.licenseTable.headerRowLine.parentNode.removeChild(this.licenseTable.headerRowLine);
		this.dialogBody.appendChild(this.licenseTable.domNode);
		
		dojo.forEach(this.licenses , function(license){
				if(dojo.isIE == 7) {
					var radio = document.createElement("<input type=\"radio\" name=\"license\" />");				 
					radio.setAttribute("value", license.id);
				} else {
					var radio = dojo.create("input", { type: "radio", name: "license" , value: license.id});					
				}
				this.connect(radio, "onclick", "licenseSelected");				
				var row = this.licenseTable.addRow(null, [radio , license.productName, license.name, license.variantName]);
		}, this);		
	},
		
	licenseSelected: function(license){
		if (license.target.value != this.targetLicenseId)
		{
			this.clearErrorMessage();
			this.targetLicenseId = license.target.value;
			this.finishButton.disabled = false;
		}
	},
	
	showErrorMessage: function (msg) {
		this.header.showMessage({
			summary: msg,
			type: jazz.ui.MessageBox.STATUS_TYPES.ERROR
		});
	},
	
	clearErrorMessage: function() {
		this.header.hideMessage();
	},
	
	close: function () {
		this.dialog.close();
	},
	
	_updateButtonWidth: function() {
		var tester = dojo.create("div", { style: { fontSize: "1em" }}, document.body);
		var newWidth = jazz.util.ui.findMaxStringDisplayWidth([this.messages.cancel, this.messages.finish], tester, MIN_BUTTON_WIDTH);
		// add 23 pixels to accommodate button borders and margin
		var newWidthStyle = (newWidth + 23) + "px";
		this.finishButton.style.width = newWidthStyle;
		this.cancelButton.style.width = newWidthStyle;
		document.body.removeChild(tester);
	}
});

})();