/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.users.UserEditor");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.string.BidiComplex");
dojo.require("dojox.xml.parser");
dojo.require("dojox.validate.regexp");
dojo.require("dojo.io.iframe");
dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("jazz.ui.MessageBox");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("jazz.ui.HoverPopup");
dojo.require("jazz.core.RegistryFactory");
dojo.require("jazz.core.loader");

dojo.require("com.ibm.team.repository.web.ui.internal.admin.users.PasswordChangeDialog");

dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");
dojo.require("com.ibm.team.repository.web.client.internal.licensing");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.team.repository.web.ui.internal.Util");

dojo.require("com.ibm.team.repository.web.ui.internal.admin.users.SimpleTabContainer");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.users", "UserEditorMessages");

var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var HoverPopup = jazz.ui.HoverPopup;
var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
var LicenseAdminClient = com.ibm.team.repository.web.client.internal.licensing;
var AdminClient = com.ibm.team.repository.web.client.internal.AdminClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var USER_EDITOR = "com.ibm.team.repository.editUser";
var PHOTO_SERVICE_URI = TeamServerClient._getServicePrefix() + "com.ibm.team.repository.service.internal.IMemberPhotoService";
var MSG_COOKIE_NAME = "com.ibm.team.repository.editUser#saveMsg";
var FILE_UPLOAD_RESPONSE = "File Upload Response:";
var FILE_UPLOAD_ERROR = "File Upload Error:";
var KEY_RETURN = 13;
var KEY_ENTER = 14;
var REG_GROUP_ERROR_FLAG = "REG_GROUP_ERROR:";
var USER_NOT_FOUND_MSG_ID = "CRJAZ1231I";
var SimpleTabContainer = com.ibm.team.repository.web.ui.internal.admin.users.SimpleTabContainer;
var RegistryFactory = jazz.core.RegistryFactory;
var loader = jazz.core.loader;
var Util = com.ibm.team.repository.web.ui.internal.Util;
var MessageBox = jazz.ui.MessageBox;
var STATUS_TYPES = MessageBox.STATUS_TYPES;
var LICENSE_HELP_URL = "/clmhelp/topic/com.ibm.jazz.repository.web.admin.doc/topics/c_managing_licenses.html";
var _currentBindRequest;
var _currentUploadFrameNumber = 100;

function clearInlineError(elem) { 
	if(elem) {
		dojo.removeClass(elem,"inlineError");
		elem.innerHTML = "";
	}
};

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.users.UserEditor", [dijit._Widget, dijit._Templated], {
	
	container: null,
	overviewPageDirty: false,
	dirty: false,
	adminUser: false,
	ownerUser: false,
	_tooltips: [],
	
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/templates/UserEditor.html"),

	/*
	 * Static Variable Container
	 */
	_static: function() {}, 
	
	constructor: function(args){
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.users", "UserEditorMessages");
		this.isNew = args.itemId === "new";
		if (!this.isNew) {
			this.itemId = args.itemId;
		}
		this._defaultPhotoSrc = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/images/no-photo.png");
		this._removeIconSrc = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/images/remove.gif");
		this._removeIconHoverSrc = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/images/remove-hov.gif");
		this._revertIconSrc = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/images/restore.gif");
		this._revertIconHoverSrc = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/images/restore-hov.gif");
		this._userIconSrc = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/images/userIcon.png");
		this._infoIconSrc = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/images/info_ovr.gif");
		this._refreshIconSrc = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/images/refresh16.gif"); //$NON-NLS-1$ //$NON-NLS-2$
		
		this.adminUser = Util.userIsAdmin();
		this.ownerUser = this._isOwnerUser();

	},
	
	postCreate: function () {
		
		this.header = new MessageBox();
		this.headerArea.appendChild(this.header.domNode);
		
		this._updateFieldEnablement();
		
		this.inputFields = [
			this.name,
			this.userId,
			this.emailAddress
		];

		if ((this.adminUser) ||
			(this.ownerUser)) {
		
			// Field: Photo
			this.connect(this.photoFileLocation,"onchange","handlePhotoChange");
			// Field: Photo
			this.connect(this.photoRemoveAnchor,"onclick", function (e) {
				dojo.stopEvent(e);
				if (!this.photoRemoveButton.clickDisabled) {
					this.removePhoto();
				}
			});
			// Field: Photo
			this.connect(this.photoRevertAnchor,"onclick", function (e) {
				dojo.stopEvent(e);
				if (!this.photoRevertButton.clickDisabled) {
					this.revertPhoto();
				}
			});
	
			// Field: Save
			this.connect(this.saveButton,"onclick","handleSave");
			// Field: Photo
			this.connect(this.photoRevertButton, "onmouseover", function (e) {
				if (!e.target.clickDisabled) {
					e.target.src = this._revertIconHoverSrc;
				}
			});
			// Field: Photo
			this.connect(this.photoRevertAnchor, "onfocus", function (e) {
				if (!this.photoRevertButton.clickDisabled) {
					this.photoRevertButton.src = this._revertIconHoverSrc;
				}
			});
			// Field: Photo
			this.connect(this.photoRevertButton, "onmouseout", function (e) {
				e.target.src = this._revertIconSrc;
			});
			// Field: Photo
			this.connect(this.photoRevertAnchor, "onblur", function (e) {
				this.photoRevertButton.src = this._revertIconSrc;
			});
			// Field: Photo
			this.connect(this.photoRemoveButton, "onmouseover", function (e) {
				if (!e.target.clickDisabled) {
					e.target.src = this._removeIconHoverSrc;
				}
			});
			// Field: Photo
			this.connect(this.photoRemoveAnchor, "onfocus", function (e) {
				if (!this.photoRemoveButton.clickDisabled) {
					this.photoRemoveButton.src = this._removeIconHoverSrc;
				}
			});
			// Field: Photo
			this.connect(this.photoRemoveButton, "onmouseout", function (e) {
				e.target.src = this._removeIconSrc;
			});
			// Field: Photo
			this.connect(this.photoRemoveAnchor, "onblur", function (e) {
				this.photoRemoveButton.src = this._removeIconSrc;
			});
			
			// Field: User Name
			this._nameFieldHandlers = [];
			this._nameFieldHandlers.push(this.connect(this.name, "onkeypress", function (e) {
				if (e.keyCode === KEY_RETURN || e.keyCode === KEY_ENTER) {
					dojo.stopEvent(e);
					this.togglePrimaryField("Preview");
				}
			}));
			this._nameFieldHandlers.push(this.connect(this.previewName,"onmouseover", function (e) {
				e.target.style.backgroundColor = "#FFFFCC";
			}));
			this._nameFieldHandlers.push(this.connect(this.previewName,"onmouseout", function (e) {
				e.target.style.backgroundColor = "";
			}));
			this._nameFieldHandlers.push(this.connect(this.previewName,"onfocus", function (e) {
				this.togglePrimaryField("Edit");
			}));
			this._nameFieldHandlers.push(this.connect(this.name, "onblur", function (e) {
				this.togglePrimaryField("Preview");
			}));
			
			// Field: User Name
			// Field: User ID
			// Field: Email Address
			for (var i=0; i < this.inputFields.length; i++) {
				this.connect(this.inputFields[i],"onkeyup","handleChange");
				this.connect(this.inputFields[i],"onchange","handleChange");
			}
			// Field: Password
			this.connect(this.changePassword,"onclick","handlePasswordChange");
			
		}
		
		// Setup tabs
		this._initializeStaticVariables();
		this._loadTabDescriptors();
		this._initializeTabContainer();
		
		// Refresh Overview page (only)
		this.refresh();
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_updateFieldEnablement: function() {
		
		// OWNER USER
		
		// (a) Own Profile
		// Allowed to edit:
		// * User name
		// * Photo
		// * Email address
		// * Password
		// Not allowed to edit:
		// * User ID
		// * Repository Permissions
		// * Client Access Licenses
		
		// (b) Other Profile
		// Allowed to edit:
		// Not allowed to edit:
		// * User name
		// * Photo
		// * Email address
		// * Password
		// * User ID
		// * Repository Permissions
		// * Client Access Licenses
		
		// ADMIN USER
		
		// (a) Own Profile
		// Allowed to edit:
		// * User name
		// * Photo
		// * Email address
		// * Repository Permissions
		// * Client Access Licenses
		// * Password
		// Not allowed to edit:
		// * User ID (Existing user)
		
		// (b) Other Profile		
		// Allowed to edit:
		// * User name
		// * Photo
		// * Email address
		// * Repository Permissions
		// * Client Access Licenses
		// Not allowed to edit:
		// * User ID (Existing user)
		// * Password		
		
		if ((this.adminUser === false) &&
			(this.ownerUser === false)) {

			// Field: User Name
			dojo.addClass(this.previewName, "textFieldDisabled");
			dojo.addClass(this.name, "textFieldDisabled");
			this.name.disabled = true;
			
			// Field: Photo
			dojo.addClass(this.photoFileLocation, "textFieldDisabled");
			this.photoFileLocation.disabled = true;
			this.photoRemoveButton.disabled = true;
			this.photoRevertButton.disabled = true;
			
			// Field: Email Address
			dojo.addClass(this.emailAddress, "textFieldDisabled");
			this.emailAddress.disabled = true;
		}		
		
	},	

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initializeStaticVariables: function() {

		if (this._static._instanceCount) {
			this._static._instanceCount++;
			return;
		}

		this._static._instanceCount = 1;

		this._static._EDITOR_ID_USER = "com.ibm.team.repository.users";
		this._static._TAB_ID_OVERVIEW = "com.ibm.team.repository.users.overview";

		this._static._EXTENSION_POINT_PROFILE_PAGES = "com.ibm.team.repository.web.profileEditorPages";

		this._static._ELEMENT_PAGE = "page";
		
		this._static._ATTRIBUTE_TARGET_ID = "targetId";
		this._static._ATTRIBUTE_ID = "id";
		this._static._ATTRIBUTE_TITLE = "title";
		this._static._ATTRIBUTE_WIDGET = "widget";
		
		this._static._TAB_SAVE_TIMEOUT = 10000; // 10 s
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_loadTabDescriptors: function() {
		this._tabDescriptors = {};

		this._loadPredefinedTabs();
		
		if (this.isNew === false) {
			this._loadRegistryTabs();
		}
	},	

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_loadPredefinedTabs: function() {
		this._storeTabDescriptor(this._static._TAB_ID_OVERVIEW,
								 this.messages.overviewTabName,
								 "not.applicable");
	},

	/*
 	 * @param String id
 	 * @param String title
 	 * @param String widget
 	 * 
 	 * @return void
 	 */	
	_storeTabDescriptor: function(id, title, widget) {
		var tabDescriptor = {};
		tabDescriptor.id = id;
		tabDescriptor.title = title;
		tabDescriptor.widget = widget;
		
		this._tabDescriptors[tabDescriptor.id] = tabDescriptor;
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_loadRegistryTabs: function() {
		// Get extension registry
		var registry = RegistryFactory.getRegistry();
		if (!registry) {
			return;
		}
		// Get extension points
		var extensionsPoints = registry.getExtensionPoint(this._static._EXTENSION_POINT_PROFILE_PAGES);
		if (!extensionsPoints) {
			return;
		}
		// Get extensions
		var extensions = extensionsPoints.getExtensions();		
		if ((extensions instanceof Array === false) ||
			(extensions.length === 0)) {
			return;
		}
		// Get elements
		for (var index = 0; index < extensions.length; index++) {
			var elements = extensions[index].getConfigurationElements();
			if ((elements instanceof Array === false) ||
				(elements.length === 0)) {
				continue;
			}
			this._processTabElements(elements);
		}			
	},

	/*
 	 * @param Array elements
 	 * 
 	 * @return void
 	 */	
	_processTabElements: function(elements) {
		
		for (var index = 0; index < elements.length; index++) {
			var element = elements[index];
			if (element.getName() !== this._static._ELEMENT_PAGE) {
				continue;
			} else if (element.getAttribute(this._static._ATTRIBUTE_TARGET_ID) !== this._static._EDITOR_ID_USER) {
				continue;
			}
			// Get id
			var id = element.getAttribute(this._static._ATTRIBUTE_ID);
			if (!id) {
				continue;
			}
			// Get title
			var title = element.getAttribute(this._static._ATTRIBUTE_TITLE);
			if (!title) {
				continue;
			}
			// Get widget
			var widget = element.getAttribute(this._static._ATTRIBUTE_WIDGET);
			if (!widget) {
				continue;
			}
			
			// Store tab descriptor
			this._storeTabDescriptor(id, title, widget);
		}		

	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initializeTabContainer: function() {
		
		this._tabContentNodes = {};
		
		this._tabContainer = new SimpleTabContainer({
			callbackTabSwitch: dojo.hitch(this, this._handleEventTabSwitch)
		});
		this._tabContainerAP.appendChild(this._tabContainer.domNode);
		
		for (var tabID in this._tabDescriptors) {
			// Get tab descriptor
			var tabDescriptor = this._tabDescriptors[tabID];
			// Add tab
			this._tabContainer.addTab(tabDescriptor.id, 
									  tabDescriptor.title, 
									  dojo.hitch(this, this._getTabContentNode));			
		}
		// Select overview tab
		this._tabContainer.selectTab(this._static._TAB_ID_OVERVIEW);
	},

	/*
 	 * @param String newTabID
 	 * @param String oldTabID
 	 * 
 	 * @return void
 	 */	
	_handleEventTabSwitch: function(newTabID, oldTabID) {
		this.header.hideMessage();
		
		// New tab
		if (newTabID &&
			(newTabID in this._tabContentNodes)) {
			// Mark active
			this._tabContentNodes[newTabID].active = true;
			// Refresh tab if stale
			if (this._tabContentNodes[newTabID].stale) {
				this._tabContentNodes[newTabID].refresh();
				this._tabContentNodes[newTabID].stale = false;
			}
		}
		
		// Old tab
		if (oldTabID &&
			(oldTabID in this._tabContentNodes)) {
			// Mark inactive
			this._tabContentNodes[oldTabID].active = false;
		}
	},	

	/*
 	 * @param String tabID
 	 * 
 	 * @return Element
 	 */	
	_getTabContentNode: function(tabID) {
		
		if (tabID === this._static._TAB_ID_OVERVIEW) {
			// Predefined overview tab
			return this._overviewTabContent;
		} else if ((tabID in this._tabDescriptors) === false) {
			// Error
			return null;
		}
		// Extension registry tab
		// Create widget
		var widgetInstance;
		var tabDescriptor = this._tabDescriptors[tabID];
		
		try {
			widgetInstance = this._createTabContentWidget(tabDescriptor);
		} catch (errorObject) {
			var errorMessage = this._getErrorMessage(errorObject);
			// Log error in console
			console.error(errorMessage);
			// Embed error in UI
			var errorNode = this._createTabContentError(errorMessage);
			// Add tab content to editor
			this._tabContainerContentAP.appendChild(errorNode);
			
			return errorNode; 
		}

		// Add tab content to editor
		this._tabContainerContentAP.appendChild(widgetInstance.domNode);
		// Store tab content node
		this._tabContentNodes[tabID] = widgetInstance;
		
		return widgetInstance.domNode;
	},	

	/*
 	 * @param Object tabDescriptor
 	 * 
 	 * @return Widget
 	 */	
	_createTabContentWidget: function(tabDescriptor) {
		// Load widget JS, HTML & CSS		
		loader.load_sync(tabDescriptor.widget);
		// Instantiate widget
		var widgetConstructor = eval(tabDescriptor.widget);
		var tabContentWidget = new widgetConstructor({
			itemUUID: this.itemId,
			pageContainer: this,
			pageID: tabDescriptor.id,
			pageTitle: tabDescriptor.title
		});
		
		return tabContentWidget;
	},	

	/*
 	 * @param String errorMessage
 	 * 
 	 * @return Element
 	 */	
	_createTabContentError: function(errorMessage) {
		var div = document.createElement("div");
		div.style.margin = "10px";
		div.style.color = "red";
		div.innerHTML = errorMessage;
		return div;
	},	

	/*
 	 * @param Error
 	 * 
 	 * @return String
 	 */
	_getErrorMessage: function(errorObject) {
		
		var className = "";
		if (this.declaredClass) {
			className = this.declaredClass + ": ";
		} 
		
		var lineNumber = "";
		if (errorObject.lineNumber) {
			lineNumber = "(line " + errorObject.lineNumber + "): ";
		} 
		
		var message;
		if (errorObject.message) {
			message = errorObject.message;
		} else {
			message = "No error message defined";
		}
		
		var errorMessage = "[ERROR]: " +
							className +
							lineNumber +
							message;
							
		return errorMessage;
	},		
	
	/*
 	 * @param void
 	 * 
 	 * @return Boolean
 	 */
	_isOwnerUser: function() {
		if (com.ibm.team.repository.web.client.session.getAuthenticatedContributor().itemId === this.itemId) {
			return true;
		}
		return false;
	},		
	
	refresh: function () {
		if (!this.navigateAwayHandle) {
			this.navigateAwayHandle = PlatformUI.getWorkbench().registerBodyClickHandler(dojo.hitch(this, this._handleNavigate), /*priority*/ 10000);
			this.onBeforeUnloadHandle = PlatformUI.getWorkbench().registerOnBeforeUnloadHandler(dojo.hitch(this, this.navigateAwayEvent), /*priority*/ 10000);
		}
		this.clearState();
		
		if (this.isNew) {
			this.name.value = this.messages.userName;
			this.togglePrimaryField("Edit");
			setTimeout(dojo.hitch(this, function () {
				this.name.select();
			}), 250);
		}
		this.getUserRegistryInfo();
		this.checkForSuccessMsg();
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_refreshEditorPages: function() {
		for (var tabID in this._tabContentNodes) {
			if (this._tabContentNodes[tabID].active) {
				// Do refresh
				this._tabContentNodes[tabID].refresh();
				// Mark unstale
				this._tabContentNodes[tabID].stale = false;
			} else {
				// Mark stale
				this._tabContentNodes[tabID].stale = true;
			}
			// Mark undirty
			this.setDirtyEditorPage(tabID, false);
		}
	},	
	
	_handleNavigate: function(urlObj) {
		if (this.dirty) {
			if (confirm(this.messages.navigateAway + "\n\n" + this.messages.unsavedChanges + "\n\n" + this.messages.okOrCancel)) {
				PlatformUI.getWorkbench().unregisterBodyClickHandler(this.navigateAwayHandle);
				this.navigateAwayHandle = null;
				PlatformUI.getWorkbench().unregisterOnBeforeUnloadHandler(this.onBeforeUnloadHandle);
				this.onBeforeUnloadHandle = null;
				return false;
			}
			else {
				return true;
			}
		}
		return false;
	},
	
	navigateAwayEvent: function() {
		return this.dirty ? this.messages.unsavedChanges : false;
	},
	
	checkForSuccessMsg: function () {
		var msgCookie = dojo.cookie(MSG_COOKIE_NAME);
		if (msgCookie) {
			this.header.showMessage({
				summary: msgCookie,
				type: STATUS_TYPES.OK
			});		
			var now = new Date();
			now.setTime(now.getTime() - 1);
			dojo.cookie(MSG_COOKIE_NAME,"",{expires: now});
		}
	},
	
	clearState: function () {
		this.name.value = "";
		this.userId.value = "";
		this.emailAddress.value = "";
		this.changePassword.disabled = true;
		this.changePassword.style.display = "none";
		this.repoPermissions.innerHTML = "";
		dojox.xml.parser.removeChildren(this.photoArea);
		this.removePhotoFlag = false;
		this.photoFileLocation.value = "";
		dojo.toggleClass(this.name, "highlightChange", false);
		dojo.toggleClass(this.userId, "highlightChange", false);
		dojo.toggleClass(this.emailAddress, "highlightChange", false);
		dojo.toggleClass(this.photoFileLocation, "highlightChange", false);
		for (var i=0; i < this.inputFields.length; i++) {
			this.inputFields[i].changePending = false;
		}
		this._clickDisabled(this.photoRevertButton,this.photoRevertAnchor,true);
		
		// Hide info decorators
		this._nameInfoDec.style.display = "none";
		this._userIDInfoDec.style.display = "none";
		this._emailAddressInfoDec.style.display = "none";
		
		this._destroyHoverPopups();
		
		clearInlineError(this.photoErr);
	},
	
	getUserRegistryInfo: function () {
		var srh = new ServiceResponseHandler(this, "getUserRegistryInfoCallback", "handleRegistryInfoError");
		AdminClient.getUserRegistryInfo(srh);
	},
	
	handleRegistryInfoError: function (err) {
		/*
		 * The error message CRJAZ0799I indicates that the user registry is not supported.  We can't query or write to the external
		 * registry, but we should allow repository-only creation and editing of users.
		 */
		if (err.message.indexOf("CRJAZ0799I") !== -1) {
			var userRegistryInfo = {
				writable: false,
				queryable: false
			};
			this.getUserRegistryInfoCallback(userRegistryInfo);
		}
		else {
			this.header.showMessage({
				summary: this.messages.userRegistryError,
				errorObject: err
			});
		}
	},
	
	getUserRegistryInfoCallback: function (userRegistryInfo) {
		this.userRegistryInfo = {};
		this.userRegistryInfo.isWritable = userRegistryInfo.writable;
		this.userRegistryInfo.isQueryable = userRegistryInfo.queryable;
		this.userRegistryInfo.fields = {};
		if (userRegistryInfo.readOnlyAttributes != null && userRegistryInfo.readOnlyAttributes.length > 0) {
			for (var i = 0; i < userRegistryInfo.readOnlyAttributes.length; i++) {
				this.userRegistryInfo.fields[userRegistryInfo.readOnlyAttributes[i]] = { writable: false };
			}
		}
		for (i = 0; i < this.inputFields.length; i++) {
			if (this.userRegistryInfo.fields[this.inputFields[i].name] == null) {
				this.userRegistryInfo.fields[this.inputFields[i].name] = { writable: true };
			}
		}
		
		if (!this.isNew) {
			for (i=0; i < this.inputFields.length; i++) {
				if (!this.userRegistryInfo.fields[this.inputFields[i].name].writable) {
					dojo.addClass(this.inputFields[i], "textFieldDisabled");
					this.inputFields[i].disabled = true;
					if (this.adminUser || this.ownerUser) {
						this._makeFieldReadOnly(this.inputFields[i]);
					}
				}
			}
			if (userRegistryInfo.writable) {
				this.changePassword.style.display = "";
			} else if (this.adminUser || this.ownerUser) {				
				// Show "passive" pending message
				if(!this.header.messageArea.firstChild) {
					this.header.showMessage({
						summary: this.messages.userRegistryNotWritable,
						type: STATUS_TYPES.INFO
					});
				}			
			}
		}
		else {
			if (this.userRegistryInfo.isWritable) {
				this.header.showMessage({
					summary: this.messages.newDefaultPassword,
					type: STATUS_TYPES.WARNING
				});
			}
		}
		if(this.repoPermissions) {
			this.repoPermissions.innerHTML = "";
			this.repoPermissions.appendChild(document.createTextNode(this.messages.loading));
			var srh = new ServiceResponseHandler(this, "populateRoles", "handleRegistryGroupError");
			AdminClient.getAllRoles(srh);
		}
	},
	
	/*
 	 * @param Object field
 	 * 
 	 * @return void
 	 */	
	_makeFieldReadOnly: function(field) {
		if (field.name === "name") {
			// Name
			dojo.addClass(this.previewName, "textFieldDisabled");
			this._nameInfoDec.style.display = "inline";

			if (this._nameInfoHover) {
				this._nameInfoHover.destroy();
			}
			this._nameInfoHover = new HoverPopup({
				around: this._nameInfoDec,
				html: this.messages.readOnlyName,
				openDelay: 300
			});

			if (this._nameFieldHandlers) {
				for (var index = 0; index < this._nameFieldHandlers.length; index++) {
					var handler = this._nameFieldHandlers[index];
					if (handler) {
						this.disconnect(handler);
					}
				}
				delete this._nameFieldHandlers;
			}			
		} else if (field.name === "userId") {
			// User ID
			this._userIDInfoDec.style.display = "inline";

			if (this._userIDInfoHover) {
				this._userIDInfoHover.destroy();
			}
			this._userIDInfoHover = new HoverPopup({
				around: this._userIDInfoDec,
				html: this.messages.readOnlyUserID,
				openDelay: 300
			});
		} else if (field.name === "emailAddress") {
			// Email Address
			this._emailAddressInfoDec.style.display = "inline";
			
			if (this._emailAddressInfoHover) {
				this._emailAddressInfoHover.destroy();
			}
			this._emailAddressInfoHover = new HoverPopup({
				around: this._emailAddressInfoDec,
				html: this.messages.readOnlyEmailAddress,
				openDelay: 300
			});
		}
	},	
	
	getLicensingInfo: function () {
		var srh = new ServiceResponseHandler(this, "getLicensingInfoCallback", "handleLicensingInfoError");
		LicenseAdminClient.getLicenseInfo(srh);
	},
	
	getLicensingInfoCallback: function (data) {
		if(!this.licensesDiv) return;
		this.licenseInputs = [];
		this.licensesDiv.innerHTML = "";
		if (data.serverLicense) {
			this.isTrialLicense = data.serverLicense.trial;
		}
		if (data.types != null) {
			var licenses = LicenseAdminClient.sortCALs(data.types);
			for (var i=0; i < licenses.length; i++) {
				var licenseInfo = LicenseAdminClient.getClientAccessLicenseInfo(data, licenses[i]);
				var div = dojo.create("div", {}, this.licensesDiv);
				var checkbox = dojo.create("input", { type: "checkbox", tabIndex: "0", id: "checkBox_" + licenses[i].id, value: licenses[i].id }, div);

				var availableLicenses = licenseInfo.available;
				if (availableLicenses === 0 || availableLicenses === -2 || (this.adminUser === false)) {
					checkbox.disabled = true;
				}
				this.licenseInputs.push(checkbox);
				var txtDiv = dojo.create("div", { className: "labelContainer"}, div);
				var curLabel = dojo.create("label", { htmlFor: checkbox.id, innerHTML: licenseInfo.name + " " }, txtDiv);
				
				var variantAndCount = dojo.create("div", { className: "variantAndCount" }, txtDiv);
				var licenseCountSpan = dojo.create("span");
				if (availableLicenses != -1) {
					licenseCountSpan.appendChild(document.createTextNode(licenseInfo.simpleAvailabilityString));
				}
				
				licenseCountSpan.licenseCount = availableLicenses;
				licenseCountSpan.assignableToUserWithCALs = licenseInfo.assignableToUserWithCALs;
				licenseCountSpan.assignableToUserWithoutCALs = licenseInfo.assignableToUserWithoutCALs;
				checkbox.licenseCountSpan = licenseCountSpan;
				variantAndCount.appendChild(licenseCountSpan);
				
				//Add the variant if it exists
				if(licenseInfo.variantName) {
					if (availableLicenses != -1) {
						variantAndCount.appendChild(document.createTextNode(" - "));
					}
					variantAndCount.appendChild(document.createTextNode(licenseInfo.variantName));
				}

				//Show expiration date for CALs
				if (licenses[i].expirationTime) {
					var obj = LicenseAdminClient.getExpirationInfo(licenses[i].expirationTime);
					variantAndCount.appendChild(document.createTextNode(" (" + obj.expiresOnString + ")"));
				}

				dojo.toggleClass(div, "checkboxDiv", true);

				var span = document.createElement("span");
				this.licensesDiv.appendChild(span);
				var tooltip = new HoverPopup({
					around: curLabel.parentNode,
					html: licenses[i].description,
					openDelay: 300,
					closeDelay: 50,
					orientation: "L",
					persist: false
				}, span);
				this._tooltips.push(tooltip);
				
				if (this.adminUser) {
					this.connect(checkbox,"onclick","handleChange");
					this.connect(checkbox,"onclick", function(e) {
						if (e.target.licenseCountSpan.licenseCount != -1) {
							if (e.target.checked) {
								// Only update the available license count if the license is not "unlimited" or "disabled"
								if (e.target.licenseCountSpan.licenseCount !== -1 && e.target.licenseCountSpan.licenseCount !== -2) {
									e.target.licenseCountSpan.innerHTML = dojo.string.substitute(this.messages.simpleLicensesAvailable, [--e.target.licenseCountSpan.licenseCount]);
								}
								dojo.addClass(this.noLicenseWarning, "noDisplay");
							}
							else {
								// Only update the available license count if the license is not "unlimited" or "disabled"
								if (e.target.licenseCountSpan.licenseCount !== -1 && e.target.licenseCountSpan.licenseCount !== -2) {
									e.target.licenseCountSpan.innerHTML = dojo.string.substitute(this.messages.simpleLicensesAvailable, [++e.target.licenseCountSpan.licenseCount]);
								}
								var assigned = false;
								for (var i=0; i < this.licenseInputs.length; i++) {
									if (this.licenseInputs[i].checked) {
										assigned = true;
										break;
									}
								}
								dojo.toggleClass(this.noLicenseWarning, "noDisplay", assigned);
							}
						}
					});
				}
			}
			if (!this.isNew) {
				var args = {"userId": this.userId.savedValue};
				var srh = new ServiceResponseHandler(this, "getAssignedLicensesCallback", "handleAssignedLicensesError");
				LicenseAdminClient.getAssignedLicenses(srh, args);
			}
			else {
				srh = new ServiceResponseHandler(this, "getDefaultLicenseIdCallback","handleDefaultLicenseIdError");
				LicenseAdminClient.getDefaultLicenseId(srh, args);
			}
		}
		else {
			dojo.create("div", { style: "color:#999;", innerHTML: this.messages.noCALsFound}, this.licensesDiv);
			this.showLicenses();
		}
	},
	
	showLicenses: function() {
		if(this.licenseHelpMessage) {
			this.licenseHelpMessage.innerHTML = ""; //Hide the "Loading..." message
			dojo.removeClass(this.licensesDiv, "noDisplay");
			dojo.create("a", { innerHTML: this.messages.licenseHelpMessage, href: LICENSE_HELP_URL }, this.licenseHelpMessage);	
		}
	},
	
	getDefaultLicenseIdCallback: function (defaultLicenseId) {
		var data = null;
		var assigned = false;
		if (defaultLicenseId !== null && defaultLicenseId !== "") {
			data = [];
			var defaultLicense = {};
			defaultLicense.id = defaultLicenseId;
			data[0] = defaultLicense;
			assigned = true;
		}
		if(this.noLicenseWarning) {
			dojo.toggleClass(this.noLicenseWarning, "noDisplay", assigned);
		}
		this.getAssignedLicensesCallback(data);	
	}, 
	
	handleDefaultLicenseIdError: function (error) {
		//no problem if an error on fetching the default license id
		//set to show the message about having no CALs
		dojo.removeClass(this.noLicenseWarning, "noDisplay");
		this.showLicenses();
	}, 
	
	handleLicensingInfoError: function (err) {
		this.header.showMessage({
			summary: this.messages.licenseInfoError,
			errorObject: err
		});
	},
	
	handleRegistryGroupError: function (err) {
		this.header.showMessage({
			summary: this.messages.groupInfoError,
			errorObject: err
		});
	},
	
	populateRoles: function (data) {
		this.repoPermissions.innerHTML = "";
		this.roleInputs = [];
		if (!this.userRegistryInfo.isWritable) {
			var div = document.createElement("div");
			div.appendChild(document.createTextNode(this.messages.dsNotWritable));
			div.style.marginBottom = "1em";
			this.repoPermissions.appendChild(div);
		}
		for (var i=0; i < data.length; i++) {
			div = document.createElement("div");
			var checkbox = document.createElement("input");
			var checkBoxId = data[i].name + "CheckBox";
			checkbox.id = checkBoxId;
			checkbox.type = "checkbox";
			checkbox.tabIndex = "0";
			checkbox.name = "repoPermissions";
			checkbox.value = data[i].name;
			checkbox.savedValue = false;
			checkbox.style.marginRight = "0.5em";
			if (!this.userRegistryInfo.isWritable || 
				(this.adminUser === false)) {
				checkbox.disabled = true;
			} else {
				this.connect(checkbox,"onclick","handleChange");
			}
			this.roleInputs.push(checkbox);
			
			div.appendChild(checkbox);
			var boxLabel = document.createElement("label");
			boxLabel.appendChild(document.createTextNode(data[i].name));
			boxLabel.htmlFor = checkBoxId;
			checkbox.labelElem = boxLabel;
			div.appendChild(boxLabel);
			var span = document.createElement("span");
			div.appendChild(span);
			var tooltip = new HoverPopup({
				around: boxLabel.parentNode,
				html: data[i].description,
				openDelay: 300,
				orientation: "L"
			}, span);
			this._tooltips.push(tooltip);
			div.style.marginBottom = "0.3em";
			this.repoPermissions.appendChild(div);
		}
		if (!this.isNew) {
			var srh = new ServiceResponseHandler(this, "populateUserData", "handleUserDataError");
			AdminClient.getContributorByUUID({uuid: this.itemId}, srh);
		}
		else {
			var jazzUsersBox = dojo.byId("JazzUsersCheckBox");
			if (jazzUsersBox && this.userRegistryInfo && this.userRegistryInfo.isWritable) {
				jazzUsersBox.checked = true;
			}
			this.name.savedValue = "";
			this.userId.savedValue = "";
			this.emailAddress.savedValue = "";
			this.photoExists = false;
			this._clickDisabled(this.photoRemoveButton,this.photoRemoveAnchor,!this.photoExists);
			this.showSavedPhoto();
			this.getLicensingInfo();
		}
	},
	
	handleUserDataError: function (err) {
		this.header.showMessage({
			summary: this.messages.errorFetchingUserData,
			errorObject: err
		});
	},
	
	togglePrimaryField: function (mode) {
		if (mode === "Preview") {
			this.name.style.display = "none";
			this.previewName.innerHTML = "";
			this.previewName.appendChild(document.createTextNode(this.name.value || this.messages.userName));
			this.previewName.style.display = "";
		}
		else if (mode === "Edit") {
			this.previewName.style.display = "none";
			this.name.style.display = "";
			this.name.focus();
			this.name.select();
			this.header.hideMessage();
		}
	},
	
	populateUserData: function (data) {
		if(!this.name) return; //user navigated to another page
		this.showBreadcrumbs(data.archived);
		this.stateId = data.stateId;
		this.name.value = data.name;
		this.name.savedValue = data.name;
		this.togglePrimaryField("Preview");
		this.userId.value = data.userId;
		this.userId.savedValue = data.userId;
		var email = dojox.string.BidiComplex.createDisplayString(data.emailAddress,"EMAIL");
		this.emailAddress.value = email;
		this.emailAddress.savedValue = email;
		this.photoExists = data.photoExists;
		this._clickDisabled(this.photoRemoveButton,this.photoRemoveAnchor,!this.photoExists);
		this.showSavedPhoto();
		if (com.ibm.team.repository.web.client.session.getAuthenticatedContributor().userId == data.userId) {
			this.changePassword.disabled = false;
		}
		if (data.roles != null) {
			if (data.roles.length === 1 && data.roles[0].indexOf(REG_GROUP_ERROR_FLAG) === 0) {
				var message = data.roles[0].substring(REG_GROUP_ERROR_FLAG.length);
				if (message.indexOf(USER_NOT_FOUND_MSG_ID) === 0) {
					this.header.showMessage({
						summary: this.messages.userNotFound,
						type: STATUS_TYPES.WARNING
					});
				} else if (this.adminUser) {
					this.header.showMessage({
						summary: dojo.string.substitute(this.messages.problemFetchingRoles, [ message ]),
						type: STATUS_TYPES.WARNING
					});
				}
			}
			else {
				for (var i=0; i < data.roles.length; i++) {
					for (var j=0; j < this.roleInputs.length; j++) {
						if (this.roleInputs[j].value == data.roles[i]) {
							this.roleInputs[j].checked = true;
							this.roleInputs[j].savedValue = true;
						}					
					}
				}
			}
		}
		this.getLicensingInfo();
	},
	
	showBreadcrumbs: function (isArchived) {
		if (!this.container) {
			return;
		}
		this.container.updateBreadCrumbs(isArchived);
	},
	
	getAssignedLicensesCallback: function (data) {
		if (data == null) {
			data = [];
		}
		for (var i=0; i < this.licenseInputs.length; i++) {
			this.licenseInputs[i].savedValue = false;
			for (var j=0; j < data.length; j++) {
				if (data[j].id == this.licenseInputs[i].value) {
					this.licenseInputs[i].savedValue = true;
					this.licenseInputs[i].checked = true;
					if (this.adminUser) {
						this.licenseInputs[i].disabled = false;
					}
				}
			}
		}
		if (data.length === 0) {
			dojo.removeClass(this.noLicenseWarning, "noDisplay");
			for (i=0; i < this.licenseInputs.length; i++) {
				var assignableToUserWithoutCALs = this.licenseInputs[i].licenseCountSpan.assignableToUserWithoutCALs;
				var assignableToUserWithCALs = this.licenseInputs[i].licenseCountSpan.assignableToUserWithCALs;
				this.licenseInputs[i].disabled = !assignableToUserWithoutCALs || !this.adminUser;
				if (!assignableToUserWithoutCALs && assignableToUserWithCALs) {
					dojo.removeClass(this.serverLicenseLimitMessage, "noDisplay");
				}
			}
		}
		//Hide the "loading" message and display the div
		this.showLicenses();
	},
	
	handleAssignedLicensesError: function (err) {
		this.header.showMessage({
			summary: this.messages.errorFecthingAssignedLicenses,
			errorObject: err
		});
		//Hide the "loading" message and display the div
		this.showLicenses();
	},
	
	handlePasswordChange: function (e) {
		dojo.stopEvent(e);
		this.passwordChangeDialog = new com.ibm.team.repository.web.ui.internal.admin.users.PasswordChangeDialog(
				{userName: this.name.savedValue, userId: this.userId.savedValue, itemId: this.itemId});
		this.connect(this.passwordChangeDialog.cancelButton, "onclick", function (e) {
			dojo.stopEvent(e);
			this.passwordChangeDialog.close();
		});
		this.connect(this.passwordChangeDialog.finishButton, "onclick", function (e) {
			dojo.stopEvent(e);
			if (this.passwordChangeDialog.newPassword.value != this.passwordChangeDialog.confirmPassword.value) {
				this.passwordChangeDialog.showErrorMessage(this.messages.passwordNoMatch);
				this.passwordChangeDialog.newPassword.focus();
			}
			else {
				this.passwordChangeDialog.clearErrorMessage();
				var srh = new ServiceResponseHandler(this,"passwordChangeCallback","passwordChangeError");
				AdminClient.changeUserPassword({
					userId: this.passwordChangeDialog.userId,
					oldPassword: this.passwordChangeDialog.oldPassword.value,
					newPassword: this.passwordChangeDialog.newPassword.value
				}, srh);
			}
		});
	},
	
	passwordChangeCallback: function () {
		this.header.showMessage({
			summary: this.messages.passwordChangeSuccess,
			type: STATUS_TYPES.OK
		});
		this.passwordChangeDialog.close();
	},
	
	passwordChangeError: function (err) {
		if (err.status == 403 || err.status == 401) {
			this.passwordChangeDialog.showErrorMessage(dojo.string.substitute(this.messages.passwordChangeError, [ this.messages.incorrectPassword ]));
			this.passwordChangeDialog.oldPassword.focus();
		}
		else {
			this.passwordChangeDialog.showErrorMessage(dojo.string.substitute(this.messages.passwordChangeError, [ err.message ]));
		}
	},
	
	handleChange: function (e) {
		if (e.target.type == "checkbox") {
			if (e.target.checked != e.target.savedValue) {
				e.target.changePending = true;
				this.setDirtyOverviewPage(true);
			}
			else {
				e.target.changePending = false;
				if (!this.changesPending()) {
					this.setDirtyOverviewPage(false);
				}
			}
		}
		else {
			if (e.target.value != e.target.savedValue) {
				e.target.changePending = true;
				this.setDirtyOverviewPage(true);
			}
			else {
				e.target.changePending = false;
				if (!this.changesPending()) {
					this.setDirtyOverviewPage(false);
				}
			}
		}
	},

	/*
 	 * @param Boolean dirty
 	 * 
 	 * @return void
 	 */	
	setDirty: function (dirty) {
		
		if (dirty === this.dirty) {
			// Nothing to do
			return;
		}
		
		if (dirty) {
			// Show *
			this.modifiedFlag.style.display = "inline";
			// Enable save button
			this.saveButton.disabled = false;
		} else {
			// Hide *
			this.modifiedFlag.style.display = "none";
			// Disable save button
			this.saveButton.disabled = true;
		}

		this.dirty = dirty;
	},
	
	/*
 	 * @param Boolean dirty
 	 * 
 	 * @return void
 	 */	
	setDirtyOverviewPage: function(dirty) {
		// Update the dirty status of the overview tab
		this.overviewPageDirty = dirty;
		
		// Update the dirty status of the editor 
		if (dirty) {
			// At least one editor page is dirty
			// Mark dirty
			this.setDirty(true);
		} else if (this._isEditorPageDirty() === false) {
			// No editor pages are dirty
			// Mark undirty
			this.setDirty(false);
		}
	},

	/*
 	 * @param String tabID
 	 * @param Boolean dirty
 	 * 
 	 * @return void
 	 */	
	setDirtyEditorPage: function (tabID, dirty) {
		
		if ((tabID in this._tabContentNodes) === false) {
			return;
		}
		// Update the dirty status of the specified tab
		this._tabContentNodes[tabID].dirty = dirty;

		// Update the dirty status of the editor 
		if (dirty) {
			// At least one editor page is dirty
			// Mark dirty
			this.setDirty(true);
		} else if ((this._isEditorPageDirty() === false) &&
				   (this._isOverviewPageDirty() === false)) {
			// No editor pages are dirty
			// Mark undirty
			this.setDirty(false);
		}
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return Boolean
 	 */	
	_isEditorPageDirty: function () {
		for (var tabID in this._tabContentNodes) {
			if (this._tabContentNodes[tabID].dirty) {
				return true;
			}
		}
		return false;
	},	

	/*
 	 * @param void
 	 * 
 	 * @return Boolean
 	 */	
	_isOverviewPageDirty: function () {
		return this.overviewPageDirty; 
	},	
	
	setPhoto: function (uri, msg, highlightMsg) {
		if (uri) {
			this.photoArea.src = uri;
		}
		else {
			this.photoArea.src = this._defaultPhotoSrc;
		}
	},
	
	handlePhotoChange: function () {
		if (this.photoFileLocation.value != "") {			
			clearInlineError(this.photoErr);
			this.header.hideMessage();
			this.removePhotoFlag = false;
			this._performUpload(this.photoFileLocation.value);
			this._clickDisabled(this.photoRemoveButton,this.photoRemoveAnchor,false);
			this._clickDisabled(this.photoRevertButton,this.photoRevertAnchor,false);
			dojo.toggleClass(this.photoFileLocation, "highlightChange", true);
			this.setDirtyOverviewPage(true);
		}
		else {
			dojo.toggleClass(this.photoFileLocation, "highlightChange", false);
			if (!this.changesPending()) {
				this.setDirtyOverviewPage(false);
			}
			if (this.removePhotoFlag) {
				this.removePhoto();
				this._clickDisabled(this.photoRemoveButton,this.photoRemoveAnchor,true);
				this._clickDisabled(this.photoRevertButton,this.photoRevertAnchor,false);
			}
			else {
				this.showSavedPhoto();
				this._clickDisabled(this.photoRemoveButton,this.photoRemoveAnchor,!this.photoExists);
				this._clickDisabled(this.photoRevertButton,this.photoRevertAnchor,true);
				
			}
		}
	},
	
	_performUpload: function(fullPath /*e, isRestart*/) {
		this._isUploadInProgress = true;
		this._priorFullPath = fullPath;
		var parts= fullPath.split("\\");
		var fileName= parts[parts.length-1];

		var self= this;
		if(this._currentBindRequest != null) {
			this._currentBindRequest.cancel();
			this._currentBindRequest  = null;
		}
		
/*
 * Internet Explorer (with dojo) has a bug such that if you wait ten seconds after one file upload, the 
 * next one breaks. Resetting the iframe fixes the problem
 * Unfortunately this code reaches into iframe private variables.
 */
		if(dojo.io.iframe._frame) {
			document.body.removeChild(dojo.io.iframe._frame);				
		}
		frameName = "dojoIoIframe" + _currentUploadFrameNumber++;
		dojo.io.iframe._iframeName = frameName;
		dojo.io.iframe._frame = null;
		
		this._currentBindRequest = dojo.io.iframe.send({
			form: this._uploadForm,
			url: PHOTO_SERVICE_URI, //+ "?projectId=" + getCurrentProjectArea().itemId,
			content: {
					mimetype: "text/html",
					fileFields: "photoFileLocation"
				},
			handleAs:  "html",
			handle: function(response, ioArgs) {
				self.isUploadInProgress = "false";
				if(response instanceof Error){
					if(response.dojoType && response.dojoType == "cancel") {
						return; // this is a cancelled request, ignore it
					}
					self._currentBindRequest = null; // no need to cancel it any more
					alert(this.messages.invalidFileName);
					console.debug(dojo.string.substitute(this.messages.errorUploadingFile, [ response.message ]));
				}
				else {
					var responseMsg = response.body.innerHTML;
					if (responseMsg.indexOf(FILE_UPLOAD_RESPONSE) == 0) {
						var data = responseMsg.substring(FILE_UPLOAD_RESPONSE.length);
						var dataArray = data.split(",");
						// Make sure we have a success
						if (dataArray.length == 4) {
							var photoLocation = dataArray[0];
							var photoSize = dataArray[1];
							var photoContentType = dataArray[2];
							var photoChecksum = dataArray[3];
							self.photoFileLocation.savedLocation = photoLocation;
							self.photoFileLocation.savedFileSize = photoSize;
							self.photoFileLocation.savedContentType = photoContentType;
							self.photoFileLocation.savedChecksum = photoChecksum;
							self.removePhotoFlag = false;
							self.setPhoto(PHOTO_SERVICE_URI + "?photoLocation=" + photoLocation);							
						}
						else {
							// Means failure.  Set the location to null so that it will not be saved
							self.photoFileLocation.savedLocation = null;
							dojo.addClass(self.photoErr, "inlineError");
							self.photoErr.innerHTML = data;
							self.header.showMessage({
								summary: data,
								type: "ERROR"
							});
							
						}
					}
				}
			}		  
		});
	},
	
	showSavedPhoto: function () {
		if (this.photoExists) {
			// Tacking on the extra Math.random() number ensures that the photo does not get cached			
			this.setPhoto(PHOTO_SERVICE_URI + "?contributorItemId=" + this.itemId + "&" + Math.random());
		}
		else {
			this.setPhoto(null, this.messages.noPhotoOnFile);
		}
	},
	
	removePhoto: function (e) {
		if (e != null) {
			dojo.stopEvent(e);
		}
		this.photoFileLocation.value = "";
		this.photoFileLocation.savedLocation = null;
		this.photoFileLocation.savedFileSize = null;
		this.photoFileLocation.savedContentType = null;
		this.photoFileLocation.savedChecksum = null;
		dojo.toggleClass(this.photoFileLocation, "highlightChange", false);
		if (this.photoExists) {
			this.removePhotoFlag = true;
			this.setPhoto(null, this.messages.photoToBeRemoved,true);
			this._clickDisabled(this.photoRemoveButton,this.photoRemoveAnchor,true);
			this._clickDisabled(this.photoRevertButton,this.photoRevertAnchor,false);
			this.setDirtyOverviewPage(true);
		}
		else {
			this.showSavedPhoto();
			this._clickDisabled(this.photoRemoveButton,this.photoRemoveAnchor,true);
			this._clickDisabled(this.photoRevertButton,this.photoRevertAnchor,true);
			this.setDirtyOverviewPage(this.changesPending());
		}
	},
	
	revertPhoto: function (e) {
		if (e != null) {
			dojo.stopEvent(e);
		}
		this.removePhotoFlag = false;
		this.photoFileLocation.value = "";
		this.photoFileLocation.savedLocation = null;
		dojo.toggleClass(this.photoFileLocation, "highlightChange", false);
		this.showSavedPhoto();
		this._clickDisabled(this.photoRemoveButton,this.photoRemoveAnchor,!this.photoExists);
		this._clickDisabled(this.photoRevertButton,this.photoRevertAnchor,true);
		this.setDirtyOverviewPage(this.changesPending());
	},
	
	handleSave: function (e) {
		dojo.stopEvent(e);
		// If a user is editing their own account and they have revoked privilges from themselves, display a warning before continuing
		if (com.ibm.team.repository.web.client.session.getAuthenticatedContributor().userId == this.userId.savedValue) {
			for (var i=0; i < this.roleInputs.length; i++) {
				var adminChecked, usersChecked, adminRevoked, usersRevoked;
				if (this.roleInputs[i].value == "JazzAdmins") {
					if (this.roleInputs[i].checked) {
						adminChecked = true;
						adminRevoked = false;
					}
					else {
						adminChecked = false;
						if (this.roleInputs[i].savedValue) {
							adminRevoked = true;
						}
						else {
							adminRevoked = false;
						}
					}
				}
				if (this.roleInputs[i].value == "JazzUsers") {
					if (this.roleInputs[i].checked) {
						usersChecked = true;
						usersRevoked = false;
					}
					else {
						usersChecked = false;
						if (this.roleInputs[i].savedValue) {
							usersRevoked = true;
						}
						else {
							usersRevoked = false;
						}
					}
				}
			}
			var msg = "";
			if (adminRevoked) {
				if (!usersChecked) {
					msg = this.messages.revokeAdminUserWarning;
				}
				else {
					msg = this.messages.revokeAdminWarning;
				}
			}
			else if (!adminChecked && usersRevoked) {
				msg = this.messages.revokeUserWarning;
			}
			if (msg != "" && !confirm(msg)) {
				return;
			}
		}
		this.testLicenseSelections();
	},
	
	testLicenseSelections: function() {
		if (!this.adminUser) {
			//Only Admin users can change license assignements, so skip this step
			this.saveUser();
			return;
		}
		
		//Get all of the checked licenses
		var licenseChanges = new Array();
		if (this.licenseInputs) {
			for (var i=0; i < this.licenseInputs.length; i++) {
				if (this.licenseInputs[i].checked) {
					licenseChanges.push(this.licenseInputs[i].value);
				}
			}
		}
		
		if(licenseChanges.length > 0) {
			var args = { licenseIds: licenseChanges }
			LicenseAdminClient.postTestLicenseAssignments(new ServiceResponseHandler(this, "saveUser", "handleLicenseAssignError"), args);	
		} else {
			this.saveUser();
		}
	},
	
	handleLicenseAssignError: function(errorObj) {
		this.header.showMessage({
			errorObject: errorObj,
			summary: this.messages.licenseAssignedError
		});
	},

	saveUser: function () {
		this.saveButton.disabled = true;
		// Item ID
		var saveObj = {
			itemId: this.itemId
		};
		
		// State ID
        if (!this.isNew) {
            saveObj.stateId = this.stateId;
        }
		
		// User Name
		// User ID
		// Email Address
		for (var i=0; i < this.inputFields.length; i++) {
			if (this.isNew || this.inputFields[i].changePending) {
				saveObj[this.inputFields[i].name] = //Do not have special characters
					dojox.string.BidiComplex.stripSpecialCharacters(this.inputFields[i].value);
			}
		}
		
		if (this.adminUser) {
			// Repository Permissions
			var repoRoles = [];
			for (i=0; i < this.roleInputs.length; i++) {
				if (this.roleInputs[i].checked) {
					repoRoles.push(this.roleInputs[i].value);
				}
			}
			saveObj["jsonRoles"] = dojo.toJson(repoRoles);
		
			// Client Access Licenses
			var licenseChanges = {
				add: [],
				remove: []
			};
			if (this.licenseInputs) {
				for (i=0; i < this.licenseInputs.length; i++) {
					if (this.licenseInputs[i].changePending) {
						licenseChanges[(this.licenseInputs[i].checked ? "add" : "remove")].push(this.licenseInputs[i].value);
					} else if (this.isNew && this.licenseInputs[i].checked) { //set the default license id for a new user
						licenseChanges.add.push(this.licenseInputs[i].value);
					}
				}
			}
			saveObj["jsonLicenses"] = dojo.toJson(licenseChanges);
		}
		
		// Photo
		if (this.photoFileLocation.savedLocation != null) {
			saveObj["photoLocation"] = this.photoFileLocation.savedLocation;
			saveObj["photoSize"] = this.photoFileLocation.savedFileSize;
			saveObj["photoContentType"] = this.photoFileLocation.savedContentType;
			saveObj["photoChecksum"] = this.photoFileLocation.savedChecksum;
		}
		else if (this.removePhotoFlag) {
			saveObj["photoLocation"] = "";
		}
		
		// User registry write access
		if (!this.userRegistryInfo.isWritable ||
			(this.adminUser === false)) {
			saveObj["repoOnly"] = true;
		}
		
		// Validate save object
		var msg = this.validateRequired(saveObj);
		if (msg != null && msg != "") {
			this.header.showMessage({
				summary: msg,
				type: STATUS_TYPES.ERROR
			});
			this.saveButton.disabled = false;
		} else {
			var srh = new ServiceResponseHandler(this, "handleSaveResponse", "handleSaveError");
			AdminClient.postContributor(saveObj, srh);
			this.header.showMessage({
				summary: this.messages.saveInProgress,
				type: STATUS_TYPES.INFO
			});
		}
	},
	
	/*
 	 * @param String void
 	 * 
 	 * @return void
 	 */	
	_saveEditorPages: function() {
		
		this._tabSavePendingStatus = {};
		this._tabSaveErrorObject = null;
		
		for (var tabID in this._tabContentNodes) {
			if (this._tabContentNodes[tabID].dirty) {
				this._tabSavePendingStatus[tabID] = true;
				this._tabContentNodes[tabID].doSave(dojo.hitch(this, this._saveEditorPageCallback));
			}
		}
		
		this._startTimeOutSaveOperation();
	},

	/*
 	 * @param String tabID
 	 * @param Error errorObject
 	 * 
 	 * @return void
 	 */	
	_saveEditorPageCallback: function(tabID, errorObject) { 

		if (!this.domNode) {
			// Edge case
			// Notification too late, the editor has already been destroyed
			return;
		} else if ((tabID in this._tabSavePendingStatus) === false) {
			// Unexpected notification
			console.error(this._getErrorMessage(new Error("Unregistered tab ID encountered during save operation: " + tabID)));
			return;
		} 
		
		// Unregister tab from save operation monitoring
		delete this._tabSavePendingStatus[tabID];
		
		if (errorObject) {
			// Save operation failed for the tab
			console.error(this._getErrorMessage(errorObject));
			this._tabSaveErrorObject = errorObject;
		} else {
			// Save operation passed for the tab
			// Undirty tab
			this._tabContentNodes[tabID].dirty = false;
		}
		
		this._checkSaveOperationStatus();
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_checkSaveOperationStatus: function() {
		// Check the save operation status of each dirty tab
		for (var tabID in this._tabSavePendingStatus) {
			if (this._tabSavePendingStatus[tabID]) {
				// Incomplete
				// Save operation still in progress
				return;
			}
		}		

		this._stopTimeOutSaveOperation();

		if (this._tabSaveErrorObject) {
			// Save operation failed
			this.handleSaveError(this._tabSaveErrorObject);
		} else {
			// Save operation succeeded
			// Refresh all contributed editor pages
			this._refreshEditorPages();
			this.header.showMessage({
				summary: this.messages.userSavedSuccessfully,
				type: STATUS_TYPES.OK
			});
		}
	},		

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_startTimeOutSaveOperation: function() { 
		this._stopTimeOutSaveOperation();
	
		this._timeOutSaveOperationHandle = 
			setTimeout(dojo.hitch(this, this._timeOutSaveOperation), 
					   this._static._TAB_SAVE_TIMEOUT);		
	},	

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_stopTimeOutSaveOperation: function() { 
		if (this._timeOutSaveOperationHandle) {
			clearTimeout(this._timeOutSaveOperationHandle);
			delete this._timeOutSaveOperationHandle;
		}
	},
			
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_timeOutSaveOperation: function() {
		for (var tabID in this._tabSavePendingStatus) {
			var errorMessage = dojo.string.substitute(this.messages.errorTimeout, [this._tabDescriptors[tabID].title]);
			var errorObject = new Error(errorMessage);
			this.handleSaveError(errorObject);
			console.error(this._getErrorMessage(errorObject));
		}
	},	
	
	validateRequired: function (saveData) {
		if (saveData.name == "" || (this.isNew && saveData.name == null)) {
			this.togglePrimaryField("Edit");
			return this.messages.userMustHaveName;
		}
		if (saveData.userId == "" || (this.isNew && saveData.userId == null)) {
			this.userId.focus();
			return this.messages.userMustHaveID;
		}
		if (saveData.emailAddress == "" || (this.isNew && saveData.emailAddress == null)) {
			this.emailAddress.focus();
			return this.messages.userMustHaveEmail;
		}
		if (saveData.emailAddress) {
			// check to see if the email is valid
			var regex = new RegExp(dojox.validate.regexp.emailAddress());
			var match = regex.exec(saveData.emailAddress);
			if(!match) {
				return this.messages.invalidEmail;
			}
		}
	},
	
	handleSaveResponse: function(response) {
		// Undirty Overview page
		this.setDirtyOverviewPage(false);
		
		if (this.isNew && response) {
			dojo.cookie(MSG_COOKIE_NAME, this.messages.userCreatedSuccessfully);
			PlatformUI.getWorkbench().unregisterBodyClickHandler(this.navigateAwayHandle);
			this.navigateAwayHandle = null;
			PlatformUI.getWorkbench().unregisterOnBeforeUnloadHandler(this.onBeforeUnloadHandle);
			this.onBeforeUnloadHandle = null;
			document.location.href = "#action=" + USER_EDITOR + "&itemId=" + response;
			return;
		} 

		// Refresh Overview page (only)
		this.refresh();
		
		if (this._isEditorPageDirty()) {
			// Save all dirty contributed editor pages
			this._saveEditorPages();
		} else {
			this.header.showMessage({
				summary: this.messages.userSavedSuccessfully,
				type: STATUS_TYPES.OK
			});
		}
	},
	
	handleSaveError: function (err) {
		this.saveButton.disabled = false;
		this.header.showMessage({
			summary: this.messages.userSaveFailed,
			errorObject: err
		});
	},
	
	onRefresh: function(e){
		dojo.stopEvent(e);
		if (this.dirty && !confirm(this.messages.refreshWarning)) {
			return;
		}
		// Undirty Overview page
		this.setDirtyOverviewPage(false);		
		// Refresh Overview page (only)
		this.refresh();
		// Refresh all contributed editor pages
		this._refreshEditorPages();
		this.header.hideMessage();
	},
	
	changesPending: function () {
		if (this.removePhotoFlag) {
			return true;
		}
		if (this.photoFileLocation.value != "") {
			return true;
		}
		for (var i=0; i < this.inputFields.length; i++) {
			if (this.inputFields[i].changePending) {
				return true;
			}
		}
		if (this.roleInputs) {
			for (i=0; i < this.roleInputs.length; i++) {
				if (this.roleInputs[i].changePending) {
					return true;
				}
			}
		}
		if (this.licenseInputs) {
			for (i=0; i < this.licenseInputs.length; i++) {
				if (this.licenseInputs[i].changePending) {
					return true;
				}
			}
		}
		
		return false;
	},
	
	_destroyHoverPopups: function() {
		
		// Info decorator hover popups
		if (this._nameInfoHover) {
			this._nameInfoHover.destroy();
			delete this._nameInfoHover;
		}
		if (this._userIDInfoHover) {
			this._userIDInfoHover.destroy();
			delete this._userIDInfoHover;
		}
		if (this._emailAddressInfoHover) {
			this._emailAddressInfoHover.destroy();
			delete this._emailAddressInfoHover;
		}
		
		for (var i = 0; i < this._tooltips.length; i++) {
			this._tooltips[i].destroyRecursive();
		}
		this._tooltips = [];
		
	},
	
	uninitialize: function () {
		
		// Tab Save Status Flags
		if (this._tabSavePendingStatus) {
			delete this._tabSavePendingStatus;
		}
		
		// Tab Save Error Object
		if (this._tabSaveErrorObject) {
			delete this._tabSaveErrorObject;
		}
		
		// Poll Tab Save Status Handle
		this._stopTimeOutSaveOperation();
		
		var tabID;
		// Tab Descriptors
		if (this._tabDescriptors) {
			for (tabID in this._tabDescriptors) {
				if (this._tabDescriptors[tabID]) {
					delete this._tabDescriptors[tabID];
				}
			}
			delete this._tabDescriptors;
		}
		// Tab Content Nodes
		if (this._tabContentNodes) {
			for (tabID in this._tabContentNodes) {
				if (this._tabContentNodes[tabID]) {
					this._tabContentNodes[tabID].destroy();
					delete this._tabContentNodes[tabID];
				}
			}
			delete this._tabContentNodes;
		}
		// Tab Container
		if (this._tabContainer) {
			this._tabContainer.destroy();
			delete this._tabContainer;
		}
		
		this._destroyHoverPopups();
				
		this.inherited(arguments);
	},
	
	/**
	 * Set the enabled set for the click on the button and update the
	 * href of the parent anchor. 
	 */
	_clickDisabled: function(button,anchor,disabled){
		
		button.clickDisabled = disabled;
		
		if(disabled){
			anchor.removeAttribute("href");
		}
		else{
			anchor.href= "#";
		}
		
	}
});

})();