/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.users.UserEditorContainer");

/*
 * Imports 
 */
dojo.require("dojo.i18n"); 

dojo.require("com.ibm.team.repository.web.ui.internal.admin.users.UserEditor");
dojo.require("jazz.ui.Breadcrumbs");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.users", "UserEditorContainerMessages");


(function() {

/*
 * Aliases 
 */
var UserEditor = com.ibm.team.repository.web.ui.internal.admin.users.UserEditor;
var Breadcrumbs = jazz.ui.Breadcrumbs;

/*
 * Class 
 */
dojo.declare("com.ibm.team.repository.web.ui.internal.admin.users.UserEditorContainer",
	[dijit._Widget, dijit._Templated], {

	// Properties //----------------------------------------------------------->

	editor: null,
	breadCrumbs: null,
	itemId: null,
	
	/*
	 * Templates
	 */	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/templates/UserEditorContainer.html"),
	
	/*
	 * Static Variable Container
	 */
	_static: function() {}, 
		
	
	// Public Methods //------------------------------------------------------->

	/*
 	 * @param Object args
 	 * 
 	 * @return void
 	 */
	constructor: function(args) {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.users", "UserEditorContainerMessages");
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */	
	postCreate: function() {
		this.inherited(arguments);
		
		this._initializeStaticVariables();
		this._initializeEditor();
		this._initializeBreadCrumbs();
	},

	/*
 	 * @param Boolean isArchived
 	 * 
 	 * @return void
 	 */	
	updateBreadCrumbs: function(isArchived) {
		if(!this.breadCrumbs) return;
		var link = new Object();
		
		if (isArchived) {
			link.href = this._static._MANAGE_ARCHIVED_LINK;
			link.label = this.messages.archivedUsers;
		} else {
			link.href = this._static.MANAGE_ACTIVE_LINK;
			link.label = this.messages.activeUsers;
		}

		this.breadCrumbs.segments = [link];
		this.breadCrumbs.refresh();
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	destroy: function() {
		
		if (this.editor) {
			this.editor.destroy();
			delete this.editor;
		}

		if (this.breadCrumbs) {
			this.breadCrumbs.destroy();
			delete this.breadCrumbs;
		}

		if (this.itemId) {
			delete this.itemId;
		}
		
		this.inherited(arguments);
	},

	// Private Methods //------------------------------------------------------>
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initializeStaticVariables: function() {

		if (this._static._instanceCount) {
			this._static._instanceCount++;
			return;
		}

		this._static._instanceCount = 1;

		this._static._MANAGE_ARCHIVED_LINK = "#action=com.ibm.team.repository.manageArchivedUsers";
		this._static.MANAGE_ACTIVE_LINK = "#action=com.ibm.team.repository.manageUsers";

	},	
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initializeEditor: function() {
		this.editor = new UserEditor({
			itemId: this.itemId,
			container: this
		});
		this._editorContentSite.appendChild(this.editor.domNode);
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initializeBreadCrumbs: function() {
		this.breadCrumbs = new Breadcrumbs();
		this._breadCrumbsSite.appendChild(this.breadCrumbs.domNode);
	}	
	
});

})();
