/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function(){
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.users.UsersPage");

dojo.require("dijit._Widget");
dojo.require("dojo.i18n");
dojo.require("dojox.xml.parser");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("jazz.ui.SideNavigationLayout");

dojo.require("com.ibm.team.repository.web.ui.internal.admin.users.ManageUsers");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.users.ManageUserGroups");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.users.ManageArchivedUsers");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.users.ManageUserCALs");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.users.UserEditorContainer");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.users.GroupEditorContainer");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.users.ActiveFloatingLicensesView");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.NotAuthorizedView");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.users", "UsersPageMessages");

var messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.users", "UsersPageMessages");

var ActionRegistry;
var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var SideNavigationLayout = jazz.ui.SideNavigationLayout;
var ManageUsers = com.ibm.team.repository.web.ui.internal.admin.users.ManageUsers;
var ManageUserGroups = com.ibm.team.repository.web.ui.internal.admin.users.ManageUserGroups;
var ManageArchivedUsers = com.ibm.team.repository.web.ui.internal.admin.users.ManageArchivedUsers;
var ManageUserCALs = com.ibm.team.repository.web.ui.internal.admin.users.ManageUserCALs;
var UserEditorContainer = com.ibm.team.repository.web.ui.internal.admin.users.UserEditorContainer;

var PREFIX = "com.ibm.team.repository.";

var ACTION_SETS = [
	{ label: messages.userManagement, actions: [
		{label: messages.activeUsers, shortId: "manageUsers", aliases: ["searchUsers", "editUser"], widget: "ManageUsers", showInNav: true, cached: true,
			postShowAction: function (args, widget) {
				widget.runUserSearch("");
			}
		},
		{label: messages.activeUsers, shortId: "searchUsers", widget: "ManageUsers", showInNav: false, cached: true,
			simpleAction: function (args, widget) {
				var pageNum = args.pageNum || 0;
				pageNum = parseInt(pageNum);
				if (args.licenseType) {
					widget.runLicenseSearch(args.licenseType, pageNum);
				}
				else {
					if(args.searchTerm != "") {
						widget.searchText.value = args.searchTerm;
					}
					widget.runUserSearch(args.searchTerm, pageNum, args.sortBy, args.sortOrder);
				}
			}
		},
		{label: messages.archivedUsers, shortId: "manageArchivedUsers", aliases: ["searchArchivedUsers", "editArchivedUser"], widget: "ManageArchivedUsers", showInNav: true, cached: true},
		{label: messages.archivedUsers, shortId: "searchArchivedUsers", widget: "ManageArchivedUsers", showInNav: false, cached: true,
			simpleAction: function (args, widget) {
				var pageNum = args.pageNum || 0;
				pageNum = parseInt(pageNum);
				widget.searchText.value = args.searchTerm;
				widget.runUserSearch(args.searchTerm, pageNum, args.sortBy, args.sortOrder);
			}
		},
		{label: messages.editUser, shortId: "editUser", widget: "UserEditorContainer", showInNav: false},
		{label: messages.manageUserGroups, shortId: "manageUserGroups", widget: "ManageUserGroups", showInNav: true, isUserGroup: true, cached: true,
			postShowAction: function (args, widget) {
				widget.getAllGroups();
			}
		},
		{label: messages.editUserGroup, shortId: "editUserGroup", widget: "GroupEditorContainer", showInNav: false, isUserGroup: true},
		{label: messages.editArchivedUser, shortId: "editArchivedUser", widget: "UserEditorContainer", showInNav: false},
		{label: messages.calManagement, shortId: "manageUserCALs", widget: "ManageUserCALs", showInNav: true, cached: true, jtsOnly: true, 
			simpleAction: function (args, widget) {
				var pageNum = args.pageNum || 0;
				pageNum = parseInt(pageNum);
				var pageSize = parseInt(args.pageSize);
				widget.handleURLActions(args.licenseType, pageNum, pageSize);
			}
		},
		{label: messages.acquiredFloating, shortId: "activeFloatingLicenses", widget: "ActiveFloatingLicensesView",  showInNav: true, jtsOnly: true}

	]}
];
var ACTION_MAP = {};

function getWorkbench () {
	return PlatformUI.getWorkbench();
}

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.users.UsersPage", dijit._Widget, {
	
	reloadCurrent: function() {
		this._previousActionEntry = this._currentActionEntry || null;
		this._showCurrentWidget(/*isReload*/true);
	},

	postCreate: function () {
		this.inherited(arguments);
		
		this._widgetCache = {};
		this.userEditorCache = {};
		
		this.domNode.className = "com-ibm-team-repository-admin";
		
		this.serverMode = dojo.getObject("com.ibm.team.repository.server.mode");
		this.isJTS = (this.serverMode == "APPLICATION") ? false : true;
		
		//Hide user groups based on system setting and whether the server is in JTS mode or not
		//Hidden by default
		this.hideUserGroups = dojo.getObject("com.ibm.team.repository.hideUserGroupAdminUI");
		this.isThisJazzTeamServer = (this.serverMode == "JTS") ? true : false;
		
		this._appView = new SideNavigationLayout();
		this.domNode.appendChild(this._appView.domNode);
		
		// register actions and create hyperlinks
		ActionRegistry = getWorkbench().getActionRegistry();
		ActionRegistry.registerAction(PREFIX + "_defaultAction", this, "_defaultAction");
		
		this.allLinks = [];
		dojo.forEach(ACTION_SETS, function(actionSet) {
			if(this.isJTS || !actionSet.jtsOnly) {
				this._createNavSection(actionSet);
			}
		}, this);		
	},
	
	_createNavSection: function(actionSet) {
		var self = this;
		var navSection = this._appView.addNavigationSection(document.createElement("div"), actionSet.label, true);
		dojo.forEach(actionSet.actions, function(action){
			if ( (!self.isJTS && action.jtsOnly) || ( action.isUserGroup && (self.hideUserGroups || self.isThisJazzTeamServer) ) ) {
				return; //continue
			}
			var id = PREFIX + action.shortId;
			ACTION_MAP[action.shortId] = action;
			if (action.showInNav) {
				var link = navSection.addSectionRow(action.label, "#action=" + id);
				link.__jazzActionId = action.shortId;
				self.allLinks.push(link);
				navSection.addArtifactCommand(link, messages.refresh, dojo.hitch(self, self._onNavSectionClick), "admin-refresh-command");
			}
			ActionRegistry.registerAction(id, self, "_handleAction");
		});
	},
	
	_defaultAction: function() {
		// by default, show the first section from the left-nav
		this._handleAction({action: PREFIX + ACTION_SETS[0].actions[0].shortId});
	},
	
	_initAdminUI: function () {
		// by default, show the first section from the left-nav
		this._handleAction({action: PREFIX + ACTION_SETS[0].actions[0].shortId});
	},
	
	_handleAction: function(args) {
		if (!isAdmin()) {
			this._showUnauthorized();
			return;
		}

		if(!args.action) {
			console.debug("UsersPage.js#_handleAction: missing required param 'args.action'");
			return;
		}
		
		// cache the previous entry to see if the widget should be destroyed or
		// not
		this._previousActionEntry = this._currentActionEntry || null;
		
		// parse out the short id from the fully-qualified id
		var shortId = args.action.substring(args.action.lastIndexOf(".") + 1);
		this._currentActionEntry = ACTION_MAP[shortId];
		this._currentArgs = args;
		this._currentArgs.parent = this;
		
		// set the title for the action
		getWorkbench().setTitle(this._currentActionEntry.label);
		
		// highlight the selected link and unhighlight the others
		dojo.forEach(this.allLinks, function(link){
			if (link.__jazzActionId === shortId) {
				dojo.toggleClass(link, "selected", true);
			}
			else {
				if (link.aliases != null) {
					var found = false;
					for (var i=0; i < link.aliases.length; i++) {
						if (link.aliases[i] === shortId) {
							dojo.toggleClass(link, "selected", true);
							found = true;
							break;
						}
					}
					if (!found) {
						dojo.toggleClass(link, "selected", false);
					}
				}
				else {
					dojo.toggleClass(link, "selected", false);
				}
			}
		});
		
		if (!this._currentActionEntry.simpleAction || this._currentContentWidget == null ||
				this._currentContentWidget.id.replace(/_\d+$/,"").replace(/_/g,".") !== SERVER_PACKAGE + this._currentActionEntry.widget) {
			this._showCurrentWidget();
		}
		if (this._currentActionEntry.simpleAction) {
			this._currentActionEntry.simpleAction(args, this._currentContentWidget);
		}
		if (this._currentActionEntry.postShowAction != null) {
			this._currentActionEntry.postShowAction(args, this._currentContentWidget);
		}
	},
	
	_onNavSectionClick: function(e) {
			dojo.stopEvent(e);
			this.reloadCurrent();
	},
	
	_showCurrentWidget: function(isReload) {
		// save a reference to the prior content widget
		this._previousContentWidget = this._currentContentWidget || null;
		
		if(this._currentActionEntry.cached) {
			this._showCachedWidget(isReload);
		} else {
			this._showUncachedWidget();
		}
		
		// help the garbage collector do its job
		if(this._previousContentWidget && !this._previousActionEntry.cached) {
			this._previousContentWidget.destroyRecursive();
		}
	},
	
	_showCachedWidget: function(isReload) {
		var className = this._currentActionEntry.widget;
		if(this._widgetCache[className]) {
			// get widget from the cache if it exists
			this._currentContentWidget = this._widgetCache[className];
		} else {
			// otherwise instantiate it and add it to the cache
			var constructor = dojo.getObject(SERVER_PACKAGE + className);
			this._currentContentWidget = new constructor(this._currentArgs);
			dojo.addClass(this._currentContentWidget.domNode, "com-ibm-team-repository-admin-contentArea");
			this._widgetCache[className] = this._currentContentWidget;
		}
		if(isReload) {
			this._currentContentWidget.reload(this._currentArgs);
		} else {
			this._currentContentWidget.show(this._currentArgs);
			this._appView.setMainContent(this._currentContentWidget.domNode);
		}
	},
	
	_showUncachedWidget: function() {
		var constructor = dojo.getObject(SERVER_PACKAGE + this._currentActionEntry.widget);
		this._currentContentWidget = new constructor(this._currentArgs);
		dojo.addClass(this._currentContentWidget.domNode, "com-ibm-team-repository-admin-contentArea");
		this._appView.setMainContent(this._currentContentWidget.domNode);
	},
	
	_showUnauthorized: function(){
		dojox.xml.parser.removeChildren(this.domNode);
		if (!this._notAuth) {
			this._notAuth = new com.ibm.team.repository.web.ui.internal.admin.NotAuthorizedView();
		}	
		this.domNode.appendChild(this._notAuth.domNode);
	}
});

var SERVER_PACKAGE = "com.ibm.team.repository.web.ui.internal.admin.users.";

function isAdmin() {
	return com.ibm.team.repository.web.noAdminAuth ? false : true;
}

var UsersPage = com.ibm.team.repository.web.ui.internal.admin.users.UsersPage;

})();
