/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.util");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin", "utilMessages");
var messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin", "utilMessages");

function prepad2(number){
	if (number < 10){
		return "0" + number;
	}
	return number;
}

function prepad3(number){
	var result = prepad2(number);
	if (number < 100){
		result = "0" + result;
	} 
	return result;
}

function separateUnitLabels(label1, label2){
	var result = "";
	if (label1.length > 0) {
		result = label1;
	}
	if (label1.length > 0 && label2.length > 0){
		result = result + ", ";
	}
	if (label2.length > 0) {
		result = result + label2;
	}
	return result;
}

// considered deprecated.  use formatUnitLabelKeys instead.
// leaving here in case anyone uses?!
function formatUnitLabel(value, singularLabel, pluralLabel) {
	switch (value){
		case 0:
			return "";
		case 1:
			return "1 " + singularLabel;
		default:
			return value + " " + pluralLabel;		
	}
}

function formatUnitLabelKeys(value, singularLabelKey, pluralLabelKey) {
	switch (value){
		case 0:
			return "";
		case 1:
			return messages[singularLabelKey];
		default:
			return dojo.string.substitute(messages[pluralLabelKey], [ value ]);	
	}
}

function formatTimeFromMilliseconds(ms) {
	var time = ms;
	
	var milli = time%1000;
	var formatString = prepad3(milli);
	time = Math.floor(time/1000);
	
	var seconds = time%60;
	formatString = prepad2(seconds) + ":" + formatString;
	time = Math.floor(time/60);
	
	var minutes = time%60;
	formatString = prepad2(minutes) + ":" + formatString;
	time = Math.floor(time/60);
	
	var hours = time%24;
	formatString = prepad2(hours) + ":" + formatString;
	time = Math.floor(time/24);
	
	if (time > 0){
		var daysLabel = dojo.string.substitute(messages.labelDateDays, [ time ]);
		formatString =  daysLabel + ", " + formatString; 
	}
	return formatString;
}

com.ibm.team.repository.web.ui.internal.admin.util.formatTimeFromMilliseconds = formatTimeFromMilliseconds;

function formatDaysHoursMinsFromMilliseconds(ms) {
	var time = ms;
	var formatString = "";
	
	//drop ms
	time = Math.floor(time/1000);
	//drop sec
	time = Math.floor(time/60);
	
	var minutes = time%60;
	if (time < 1){
		return messages.labelDateUnderMinute;
	}
	formatString = formatUnitLabelKeys(minutes, "labelDateMinute", "labelDateMinutes");
	time = Math.floor(time/60);
	
	var hours = time%24;
	var hourLabel = formatUnitLabelKeys(hours, "labelDateHour", "labelDateHours");
	formatString = separateUnitLabels(hourLabel, formatString);
	time = Math.floor(time/24);
	var dayLabel = formatUnitLabelKeys(time, "labelDateDay", "labelDateDays");
	formatString = separateUnitLabels(dayLabel, formatString);
	
	return formatString;
}

com.ibm.team.repository.web.ui.internal.admin.util.formatDaysHoursMinsFromMilliseconds = formatDaysHoursMinsFromMilliseconds;

function formatSizeFromBytes(bytes) {
	var size = bytes;
	var CONVERSION = 1024;
	var formatString = "";
	
	//1 kilobytes = 1024 bytes
	var b = size%CONVERSION;
	size = Math.floor(size/CONVERSION);
	
	//1 megabytes = 1024 kilobytes
	var kb = size%CONVERSION;
	size = Math.floor(size/CONVERSION);
	
	var mb = size;
	
	if (mb < 1){
		formatString = dojo.string.substitute(messages.labelKBytes, [ kb ]);
	} else {
		formatString = dojo.string.substitute(messages.labelMBytes, [ mb ]);
	}
	return formatString;
}

com.ibm.team.repository.web.ui.internal.admin.util.formatSizeFromBytes = formatSizeFromBytes;

function replaceNewLineWithBreak(text){
	return text.replace(/\n/g, "<br>"); 
}

com.ibm.team.repository.web.ui.internal.admin.util.replaceNewLineWithBreak = replaceNewLineWithBreak;

function replaceNewLineWithPaddedBreaks(text, prepend){
	var result = text.replace(/\n/g, "<br>&nbsp;&nbsp;&nbsp;"); 
	if (prepend) {
		//pad with actual spaces so copy/paste of a stacktrace preserves some formatting
		result = "&nbsp;&nbsp;&nbsp;" + result;
	}
	return result;
}

com.ibm.team.repository.web.ui.internal.admin.util.replaceNewLineWithPaddedBreaks = replaceNewLineWithPaddedBreaks;

function arrayIndexOfObject(array, match) {
	var index = -1;
	if(array != null && match != null) {
		for (var i=0; i<array.length; i++) {
			if (match === array[i]) { return i; }
		}
	}
	return index;
}

com.ibm.team.repository.web.ui.internal.admin.util.arrayIndexOfObject = arrayIndexOfObject;

function arrayIndexWithPropertyOfObject(array, property, match) {
	var index = -1;
	if(array != null && match != null && property != null) {
		for (var i=0; i<array.length; i++) {
			if (match === array[i][property]) { return i; }
		}
	}
	return index;
}

com.ibm.team.repository.web.ui.internal.admin.util.arrayIndexWithPropertyOfObject = arrayIndexWithPropertyOfObject;


function arrayContainsObject(array, match) {
	if(arrayIndexOfObject(array, match) > -1) {
		return true;
	}
	return false;
}

com.ibm.team.repository.web.ui.internal.admin.util.arrayContainsObject = arrayContainsObject;

function convertDateStringToRelativeTimeString(dateString) {
	var needsAgo = false;
	var labelKey = "labelDateMinutes";
	var date = dojo.date.stamp.fromISOString(dateString);
	var diff = dojo.date.difference(new Date(), date, "minute");
	
	if (diff < 0) {
		diff = diff * -1;
		if (diff > 1) {
			labelKey = "labelDateMinutesAgo";
		} else {
			labelKey = "labelDateMinuteAgo";
		}
	} else if (diff > 0) {
		if (diff > 1) {
			labelKey = "labelDateMinutes";
		} else {
			labelKey = "labelDateMinute"; 
		}
	} else { // diff from minutes was zero, need seconds
		diff = dojo.date.difference(new Date(), date, "second");
		if (diff < 0) {
			diff = diff * -1;
			if (diff > 1) {
				labelKey = "labelDateSecondsAgo";
			} else {
				labelKey = "labelDateSecondAgo";
			}	
		} else if (diff > 0) {
			if (diff > 1) {
				labelKey = "labelDateSeconds";
			} else {
				labelKey = "labelDateSecond";
			}
		} else { // diff from seconds was zero, need milliseconds
			diff = dojo.date.difference(new Date(), date, "millisecond");
			if (diff < 0) {
				diff = diff * -1;
				if (diff > 1) {
					labelKey = "labelDateMillisecondsAgo";
				} else {
					labelKey = "labelDateMillisecondAgo"
				}
			} else if (diff > 1) {
				labelKey = "labelDateMilliseconds";
			} else {
				labelKey = "labelDateMillisecond";
			}
		}
	}
	if (diff > 1) {
		return dojo.string.substitute(messages[labelKey], [ diff ]); 
	} else {
		return messages[labelKey];
	}
}

com.ibm.team.repository.web.ui.internal.admin.util.convertDateStringToRelativeTimeString = convertDateStringToRelativeTimeString;

function getBirtDateValue(date) {
	return date ? "" + date.getDate() + "/" + (date.getMonth() + 1) + "/" + date.getFullYear() : null;
}

com.ibm.team.repository.web.ui.internal.admin.util.getBirtDateValue = getBirtDateValue;

})();