/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.catalog.AddWidgetHelper");

/*
 * Imports 
 */
dojo.require("dojo.i18n"); 

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.ensemble.internal.catalog.ErrorUtility");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.DiscoveryRegistry");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ViewletRegistry");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "AddWidgetHelperMessages");

(function() {

/*
 * Aliases 
 */
var ErrorUtility = jazz.ensemble.internal.catalog.ErrorUtility;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var DiscoveryRegistry = com.ibm.team.dashboard.web.ui.internal.util.DiscoveryRegistry;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var ViewletRegistry = com.ibm.team.dashboard.web.ui.internal.util.ViewletRegistry;

/*
 * Constants 
 */
var GADGET_TYPE = "org.opensocial.gadget";
var IWIDGET_TYPE = "com.ibm.iwidget";
var IMAGE_SERVICE = "/service/com.ibm.team.dashboard.common.internal.service.IDashboardImageService";
var PATH_ARG = "?path=";
var URL_ARG = "?url=";

/*
 * Class 
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.catalog.AddWidgetHelper",
	[dijit._Widget], {

	// Properties //----------------------------------------------------------->

	onAddWidget: null,

	// ------------------------------------------------------------------------>

	/*
 	 * @param Object args
 	 * 
 	 * @return AddWidgetHelper 
 	 */
	constructor: function(args) {
		this.msg = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "AddWidgetHelperMessages");
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	postCreate: function() {

		this.inherited(arguments);

		// Instance Variables //----------------------------------------------->

		this._remoteServerData = null;
		
	},	

	/*
 	 * @param Object data
 	 * @param String rootServices
 	 * @param Boolean local
 	 * 
 	 * @return dojo.Deferred
 	 */		
	addWidget: function(data, rootServices, local) {
		// TEST
		//console.debug(data);

		var deferred = new dojo.Deferred();
		
		if (local) {
			// Local
			this._callRESTAddWidget(deferred, data);
		} else {
			// Remote
			var deferred2 = this._getRemoteServerData(rootServices);
			deferred2.addCallback(dojo.hitch(this, function(serverData) {
				this._callRESTAddWidget(deferred, data);
				return serverData; 
			}));			
			deferred2.addErrback(dojo.hitch(this, function(errorObject) {
				deferred.errback(errorObject);
				return errorObject; 
			}));
		}
		
		return deferred;
	},

	/*
 	 * @param String rootServices
 	 * 
 	 * @return dojo.Deferred
 	 */		
	_getRemoteServerData: function(rootServices) {
		var deferred = new dojo.Deferred();
		
		if (this._remoteServerData) {
			deferred.callback(this._remoteServerData);
		} else {
			this._callRESTGetRemoteServerData(deferred, rootServices);
			deferred.addCallback(dojo.hitch(this, function(serverData) {
				this._remoteServerData = serverData;
				return serverData; 
			}));			
		}

		return deferred;
	},	

	/*
 	 * @param dojo.Deferred deferred
 	 * @param String rootServices
 	 * 
 	 * @return void
 	 */
	_callRESTGetRemoteServerData: function(deferred, rootServices) {
		
		var successCallback = dojo.hitch(this, function(serverData) {
			// TEST
			//console.debug(serverData);
			deferred.callback(serverData);
		});
		var errorCallback = dojo.hitch(this, function(errorObject) {
			ErrorUtility.logError(errorObject, this.id);
			ErrorUtility.logError(new Error(this.msg.errServerData), this.id);
			deferred.errback(errorObject);
		});
		
		DiscoveryRegistry.getServerData(
			rootServices, 
			false, 
			successCallback, 
			errorCallback);

	},

	/*
 	 * @param dojo.Deferred deferred
 	 * @param Object data
 	 * 
 	 * @return void
 	 */
	_callRESTAddWidget: function(deferred, data) {
		
		if (data.type === GADGET_TYPE) {
			// Internal Gadgets (catalog service document)
			this._processInternalWidget(deferred, data);
		} else if (data.type === IWIDGET_TYPE) {
			// Internal iWidgets (catalog service document)	
			// Note: No known use case
			this._processInternalWidget(deferred, data);
		} else {
			// External gadgets
			// External iWidgets
			// Viewlets
			this._getWidgetDefinition(deferred, data);
		}
	},

	/*
 	 * @param dojo.Deferred deferred
 	 * @param Object data
 	 * 
 	 * @return void
 	 */		
	_processInternalWidget: function(deferred, data) {

		// Data is being cached, making a copy ...
		var dataCopy = dojo.clone(data);

		// Use image service to resolve images through the proxy
		// Assumption: Absolute url to the icon
		if (dataCopy.icon) {
			dataCopy.icon = Util.getServiceRoot() + 
						IMAGE_SERVICE + 
						URL_ARG + 
						encodeURIComponent(dataCopy.icon);
		}


		if (this.onAddWidget) {
			this.onAddWidget(dataCopy);
		} else {
			this._processAddWidgetFailure(deferred, new Error(this.msg.errUndefinedHandler));
			// TODO: MP: Call deferred error callback?
			return;
		}
		
		deferred.callback(true);
		
	},

	/*
 	 * @param dojo.Deferred deferred
 	 * @param Object data
 	 * 
 	 * @return void
 	 */
	_getWidgetDefinition: function(deferred, data) {
		
		var serviceObject = {
			success: dojo.hitch(this, function(definitions) {
				// TEST
				//console.debug(definitions[0]);
				this._processWidgetDefinitions(deferred, definitions, data);
			}),
			failure: dojo.hitch(this, function(errorObject) {
				// BUG: Expected this to be called when no definitions were found ...
				this._processAddWidgetFailure(deferred, errorObject);
			})
		};

		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");

		// TEST
		//console.debug(data.url);

		ViewletRegistry.getDefinition(
			data.url || data.widgetUri, 
			false, 
			handler);		
	},

	/*
 	 * @param dojo.Deferred deferred
 	 * @param Array definitions
 	 * @param Object data
 	 * 
 	 * @return void
 	 */		
	_processWidgetDefinitions: function(deferred, definitions, data) {

		if ((definitions instanceof Array) === false) {
			var errorMessage = this.msg.errWidgetDefinition;
			if (data.title) {
				errorMessage = errorMessage + ": " + data.title;
			}
			var errorObject = new Error(errorMessage);
			this._processAddWidgetFailure(deferred, errorObject);
			return;
		}

		var definition = definitions[0];

		if (!data.title) {
			if (definition.name) {
				data.title = definition.name;
			} else if (this._isGadget(definition.id)) {
				data.title = this.msg.gadgetTitle;
			} else if (this._isIWidget(definition.id)) {
				data.title = this.msg.iWidgetTitle;
			} else {
				data.title = this.msg.viewletTitle;			
			}
		}
		
		if (!data.icon) {
			var serverRoot = this._remoteServerData ? this._remoteServerData.viewletServiceRoot : Util.getServiceRoot();
			data.icon = serverRoot + IMAGE_SERVICE + PATH_ARG + encodeURIComponent(definition.icon);
		}

		if (!data.type) {
			if (this._isGadget(definition.id)) {
				data.type = GADGET_TYPE;
				data.widgetUri = definition.url;
			} else if (this._isIWidget(definition.id)) {
				data.type = IWIDGET_TYPE;
				data.widgetUri = definition.url;
			}
		}
		
		_fixWidgetUris(data);

		if (this.onAddWidget) {
			this.onAddWidget(data);
		} else {
			this._processAddWidgetFailure(deferred, new Error(this.msg.errUndefinedHandler));
			// TODO: MP: Call deferred error callback?
			return;
		}
		
		deferred.callback(true);
		
	},

	/*
 	 * @param String id
 	 * 
 	 * @return Boolean
 	 */		
	_isGadget: function(id) {
		return (id.indexOf("g:") === 0);
	},	

	/*
 	 * @param String id
 	 * 
 	 * @return Boolean
 	 */		
	_isIWidget: function(id) {
		return (id.indexOf("i:") === 0);
	},	
	
	/*
 	 * @param dojo.Deferred deferred
 	 * @param Error errorObject
 	 * 
 	 * @return void
 	 */		
	_processAddWidgetFailure: function(deferred, errorObject) {
		ErrorUtility.logError(errorObject, this.id);
		deferred.errback(errorObject);
	},

	/*
 	 * @param Boolean local
 	 * 
 	 * @return String
 	 */
	_getRepositoryRoot: function(local) {
		
		if (local) {
			return null;
		} else if (!this._remoteServerData) {
			return null;
		}
		return this._remoteServerData.viewletServiceRoot;
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	destroy: function() {

		this.inherited(arguments);
	}
	
});

// RRC viewlet definition ids and their new paths to the 
// e.g. "com.ibm.example.viewlet" -> "/path/ExampleViewlet"
var DEFINITION_PATH_BY_ID = {
	"com.ibm.rdm.web.viewlet.MyProjects": "/web/com.ibm.rdm.web.viewlet/MyProjectsDefinition.json",
	"com.ibm.rdm.web.viewlet.RecentArtifacts": "/ViewletService/RecentRequirementsDefinition.json",
	"com.ibm.rdm.web.viewlet.RecentComments": "/web/com.ibm.rdm.web.viewlet/RecentCommentsDefinition.json",
	"com.ibm.rdm.web.viewlet.RecentRequirements": "/ViewletService/RecentRequirementsDefinition.json",
	"com.ibm.rdm.web.viewlet.MyReviews": "/web/com.ibm.rdm.web.viewlet/MyReviewsDefinition.json",
	"com.ibm.rdm.web.viewlet.ReqWithImplWorkItems": "/ViewletService/ReqWithImplWorkItemsDefinition.json",
	"com.ibm.rdm.web.viewlet.ReqWithValidatedTestCases": "/ViewletService/ReqWithImplWorkItemsDefinition.json",
	"com.ibm.web.rdm.viewlets.web.RRCwelcome": "/ViewletService/WelcomeDefinition.json"
};

// viewlet icons that need to be moved from the JTS to application
// e.g. "com.ibm.example.viewlet" -> "/path/exampleviewlet.gif"
var ICON_PATH_BY_ID = {
	"com.ibm.rdm.web.viewlet.MyProjects": "/web/com.ibm.rdm.web.viewlet/graphics/MyProjects/icons/obj16/rrc-project-viewlet.gif",
	"com.ibm.rdm.web.viewlet.RecentArtifacts": "/web/com.ibm.rdm.web.viewlet/graphics/MyProjects/icons/obj16/requirements.gif",
	"com.ibm.rdm.web.viewlet.RecentComments": "/web/com.ibm.rdm.web.viewlet/graphics/MyProjects/icons/obj16/comments.gif",
	"com.ibm.rdm.web.viewlet.RecentRequirements": "/web/com.ibm.rdm.web.viewlet/graphics/MyProjects/icons/obj16/requirements.gif",
	"com.ibm.rdm.web.viewlet.MyReviews": "/web/com.ibm.rdm.web.viewlet/graphics/MyProjects/icons/obj16/reviews.gif",
	"com.ibm.rdm.web.viewlet.ReqWithImplWorkItems": "/web/com.ibm.rdm.web.viewlet/graphics/MyProjects/icons/obj16/requirements.gif",
	"com.ibm.rdm.web.viewlet.ReqWithValidatedTestCases": "/web/com.ibm.rdm.web.viewlet/graphics/MyProjects/icons/obj16/requirements.gif",
	"com.ibm.web.rdm.viewlets.web.RRCwelcome": "/web/com.ibm.rdm.web.viewlet/welcome/images/Welcome_widget16.png"
};	

/*
 * 160182: Unable to view a remote Requirements viewlet on a dashboard after 2x to 3x migration
 * If the URL is to a legacy RRC 2.x viewlet, fix it up to the new 3.0.1 URL format and tack
 * on a compatibility param, so that it will continue to work before and after RRC upgrade to 3.0.1,
 * where RRC viewlet URLs change. This guards against any new broken URLs being added, which
 * we would otherwise need to do another pass through all dashboards to fix.
 * 
 * This hack can be removed after backward compatibility with RRC 2.x is no longer supported.
 * 
 * @param object data
 * @return void
 */
function _fixWidgetUris(data) {
	var widgetUri = data.widgetUri;
	var url = new dojo._Url(widgetUri);
	var query = url.query;
	if (query) {
		var params = dojo.queryToObject(query);
		var webUIRoot = params.webUIRoot;
		if (webUIRoot) {
			var definitionId = params.definitionIds;
			if (definitionId) {
				// is this one of the viewlets that moved in 3.0.1?
				var newDefPath = DEFINITION_PATH_BY_ID[definitionId];
				if (newDefPath) {
					// yes. fix definition and icon URLs
					data.widgetUri = webUIRoot + newDefPath + "?compatUrl=" + encodeURIComponent(widgetUri) + "&webUIRoot=" + encodeURIComponent(webUIRoot);
					var newIconPath = ICON_PATH_BY_ID[definitionId];
					if (newIconPath) {
						data.icon = webUIRoot + newIconPath + "?compatUrl=" + encodeURIComponent(data.icon);
					}
				}
			}
		}
	}
}

})();
