/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.catalog.CatalogServicesProvider");

/*
 * Imports 
 */
dojo.require("dijit._Widget");

dojo.require("com.ibm.team.dashboard.web.ui.internal.catalog.DashboardCatalogServiceProvider");
dojo.require("jazz.ensemble.internal.catalog.ErrorUtility");

(function() {

/*
 * Aliases 
 */
var DashboardCatalogServiceProvider = com.ibm.team.dashboard.web.ui.internal.catalog.DashboardCatalogServiceProvider;
var ErrorUtility = jazz.ensemble.internal.catalog.ErrorUtility;

/*
 * Constants 
 */

/*
 * Class 
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.catalog.CatalogServicesProvider",
	[dijit._Widget], {

	// Properties //----------------------------------------------------------->

	// ------------------------------------------------------------------------>

	/*
 	 * @param Object args
 	 * 
 	 * @return CatalogServicesProvider 
 	 */
	constructor: function(args) {
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	postCreate: function() {

		this.inherited(arguments);

		// Instance Variables //----------------------------------------------->

		this._catalogServiceProvider = new DashboardCatalogServiceProvider();

		this._servicesInitialized = false;
		this._catalogIndexInitialized = {};
		
		this._origins = [];
		this._originIndex = {};
		
		this._entriesByOriginIndex = {};
		this._entriesIndex = {};
	
		this._categoriesByOriginIndex = {};
		this._categoriesIndex = {};
		
		// Initialization //--------------------------------------------------->
		
	},	

	/*
 	 * @param void
 	 * 
 	 * @return dojo.Deferred
 	 */		
	_getServiceItems: function() {

		var deferred = this._catalogServiceProvider.loadCatalogs();

		deferred.addCallback(dojo.hitch(this, function(serviceItems) {
			this._processServiceItems(serviceItems);
			return serviceItems; 
		}));		
		
		deferred.addErrback(dojo.hitch(this, function(errorObject) {
			ErrorUtility.logError(errorObject, this.id);
			return errorObject; 
		}));		
		
		return deferred;

	},	

	/*
 	 * @param Object services
 	 * 
 	 * @return void
 	 */
	_processServiceItems: function(services) {
		
		if ((services instanceof Array) === false) {
			ErrorUtility.logError(new Error("Undefined catalog service items encountered."), this.id);
			return;
		}
		
		for (var index = 0; index < services.length; index++) {
			var serviceItem = services[index];

			// TEST
			//console.debug(serviceItem);
			
			if (!serviceItem) {
				ErrorUtility.logError(new Error("Undefined service item encountered."), this.id);
				continue;
			} 
			
			var title = serviceItem.title;
			
			if (!title) {
				ErrorUtility.logError(new Error("Service item with no title encountered."), this.id);
				continue;
			} 
			
			title = decodeURIComponent(title);
			
			var rootServicesUrl = serviceItem.rootservices.url;

			if (!rootServicesUrl) {
				ErrorUtility.logError(new Error("Service item with no root services document encountered."), this.id);
				continue;
			} 
			
			var local = serviceItem.local;
			
			// Fully qualify local root services document url
			if (local && rootServicesUrl.indexOf("://") === -1) {
				var url = new dojo._Url(location.href);
				var absoluteRootServicesUrl = url.scheme + "://" + url.host;
				if (url.port) {
					absoluteRootServicesUrl = absoluteRootServicesUrl + ":" + url.port;
				}
				rootServicesUrl = absoluteRootServicesUrl + rootServicesUrl;
			}
			
			this._origins.push({
				title: title,
				instanceName: serviceItem.instanceName,
				isFriend: serviceItem.isFriend,
				uri: rootServicesUrl,
				local: local
			});
			this._originIndex[rootServicesUrl] = serviceItem;
			this._catalogIndexInitialized[rootServicesUrl] = false;
			
		}
		
 		if (services.length === 0) {
			ErrorUtility.logError(new Error("No catalog service items found."), this.id);
			return;
		}

		this._servicesInitialized = true;
		
	},

	/*
 	 * @param Object catalog
 	 * @param String originUri
 	 * 
 	 * @return void
 	 */		
	_indexCatalog: function(catalog, originUri) {
		
		// TEST
		//console.debug(catalog);
		
		if (!catalog ||
			((catalog.entries instanceof Array) === false) ||
			((catalog.categories instanceof Array) === false)) {
			
			ErrorUtility.logError(new Error("Failed to load catalog."), this.id);	
			
			return;
		}		
		
		// Categories
		
		var catalogCategories = catalog.categories;
		for (var indexk = 0; indexk < catalogCategories.length; indexk++) {
			var categoryk = catalogCategories[indexk];
			this._categoriesIndex[categoryk] = true;
		}
		
		// Entries
		
		var entries = catalog.entries;
		
		for (var indexi = 0; indexi < entries.length; indexi++) {
			var entry = entries[indexi];
			
			if (!entry.title) {
				ErrorUtility.logError(new Error("Missing title. Catalog entry omitted: " + entry.uri), this.id);
				continue;
			} else if (!entry.uri) {
				ErrorUtility.logError(new Error("Missing uri. Catalog entry omitted: " + entry.title), this.id);
				continue;
			} else if (!entry.widgetUri) {
				ErrorUtility.logError(new Error("Missing widgetUri. Catalog entry omitted: " + entry.uri), this.id);
				continue;
			} else if (!entry.type) {
				ErrorUtility.logError(new Error("Missing type. Catalog entry omitted: " + entry.uri), this.id);
				continue;
			}
			
			if (this._entriesIndex.hasOwnProperty(entry.uri)) {
				var e1 = this._entriesIndex[entry.uri].title;
				var e2 = entry.title;
				ErrorUtility.logError(new Error("Duplicate entry uri encountered for widgets: '" + e1 + "', '" + e2 + "': " + entry.uri), this.id);
				// Make unique uri on the fly
				entry.uri = entry.uri + new Date().getTime();
				ErrorUtility.logWarning(new Error("Auto-generated new uri for widget: '" + e2 + "': " + entry.uri), this.id);
			}
			
			// Entries
			this._entriesIndex[entry.uri] = entry;
			
			// Entries By Origin
			if (!this._entriesByOriginIndex[originUri]) {
				this._entriesByOriginIndex[originUri] = [];
			}
			this._entriesByOriginIndex[originUri].push(entry.uri);
			
			
			var categories = entry.categories;
			if (categories) {
				
				if (!this._categoriesByOriginIndex[originUri]) {
					this._categoriesByOriginIndex[originUri] = {};
				}				
				
				for (var indexj = 0; indexj < categories.length; indexj++) {
					var category = categories[indexj];
					
					// Categories By Origin
					this._categoriesByOriginIndex[originUri][category] = true;
				}
			}
			
		}
		
		// TEST
		//console.log("Entries Index:");
		//console.debug(this._entriesIndex);
		//console.log("Entries By Origin Index:");
		//console.debug(this._entriesByOriginIndex);
		//console.log("Categories Index:");
		//console.debug(this._categoriesIndex);
		//console.log("Categories By Origin Index:");
		//console.debug(this._categoriesByOriginIndex);
		
	},	

	/*
 	 * @param void
 	 * 
 	 * @return dojo.Deferred
 	 */		
	getOrigins: function() {
		
		var deferred = new dojo.Deferred();
		
		if (this._servicesInitialized) {
			deferred.callback(this._origins);
			return deferred;
		}
		
		var deferred2 = this._getServiceItems();
		deferred2.addCallback(dojo.hitch(this, function(obj) {
			deferred.callback(this._origins);
		}));			
		deferred2.addErrback(dojo.hitch(this, function(errorObject) {
			deferred.errback(errorObject);	
			return errorObject; 
		}));
		
		return deferred;
	},	

	/*
 	 * @param String originUri
 	 * 
 	 * @return dojo.Deferred
 	 */		
	getCategories: function(originUri) {

		// TEST
		//console.debug(originUri);
		
		var deferred = new dojo.Deferred();
		
		if (this._servicesInitialized === false) {
			var deferred2A = this._getServiceItems();

			deferred2A.addCallback(dojo.hitch(this, function(serviceItems) {
				var deferred2B = this._getCatalogDocument(originUri);
				
				deferred2B.addCallback(dojo.hitch(this, function(status) {
					this._getCategoriesHelper(deferred, originUri);
					return status;
				}));			
				deferred2B.addErrback(dojo.hitch(this, function(errorObject) {
					deferred.errback(errorObject);	
					return errorObject; 
				}));
				
				return serviceItems;
			}));			
			deferred2A.addErrback(dojo.hitch(this, function(errorObject) {
				deferred.errback(errorObject);	
				return errorObject; 
			}));
			
			return deferred;

		} else if (!this._catalogIndexInitialized[originUri]) {
			
			var deferred3 = this._getCatalogDocument(originUri);

			deferred3.addCallback(dojo.hitch(this, function(status) {
				this._getCategoriesHelper(deferred, originUri);
				return status;
			}));			
			deferred3.addErrback(dojo.hitch(this, function(errorObject) {
				deferred.errback(errorObject);	
				return errorObject; 
			}));
			
			return deferred;
		}

		this._getCategoriesHelper(deferred, originUri);
		
		return deferred;
	},	

	/*
 	 * @param String originUri
 	 * 
 	 * @return dojo.Deferred
 	 */		
	_getCatalogDocument: function(originUri) {
		
		var masterDeferred = new dojo.Deferred();		
		
		masterDeferred.addCallback(dojo.hitch(this, function(status) {
			this._catalogIndexInitialized[originUri] = true;
			return status;
		}));		
		
		masterDeferred.addErrback(dojo.hitch(this, function(errorObject) {
			ErrorUtility.logError(errorObject, this.id);
			return errorObject; 
		}));		

		var serviceItem = this._originIndex[originUri];

		if (!serviceItem) {
			// Error
			masterDeferred.errback(new Error("Unknown origin URI encountered: " + originUri));	
			return masterDeferred;
		}
		
		var serviceItemDeferred = serviceItem.load();
		
		serviceItemDeferred.addCallback(dojo.hitch(this, function(catalog) {
			this._indexCatalog(catalog, originUri);
			masterDeferred.callback(true);
			return catalog;
		}));			
		
		serviceItemDeferred.addErrback(dojo.hitch(this, function(errorObject) {
			masterDeferred.errback(errorObject);	
			return errorObject; 
		}));
		
		return masterDeferred;
	},	

	/*
 	 * @param void
 	 * 
 	 * @return Array
 	 */		
	_getAllCategories: function() {
		var categories = [];
		for (var category in this._categoriesIndex) {
		    if (this._categoriesIndex.hasOwnProperty(category)) {
				categories.push(category);
		    }			
		}
		categories.sort();
		
		return categories;
	},

	/*
 	 * @param dojo.Deferred deferred
 	 * @param String originUri
 	 * 
 	 * @return void
 	 */		
	_getCategoriesHelper: function(deferred, originUri) {
		
		if (!originUri) {
			// All Categories
			deferred.callback(this._getAllCategories());
			return;
		}
		
		if ((originUri in this._categoriesByOriginIndex) === false) {
			// Error: Origin not found
			deferred.callback([]);
			return;
		}
		
		var categories = [];
		var categoryIndex = this._categoriesByOriginIndex[originUri];
		for (var category in categoryIndex) {
			if (categoryIndex.hasOwnProperty(category)) {
				categories.push(category);
			}
		}
		categories.sort();
		
		deferred.callback(categories);
	},	

	/*
 	 * @param String originUri
 	 * @param String category
 	 * @param String filter
 	 * @param String start
 	 * @param String count
 	 * 
 	 * @return dojo.Deferred
 	 */		
	getEntries: function(originUri, category, filter, start, count) {

		// TEST
		//console.debug(originUri);
		
		var deferred = new dojo.Deferred();
		var query = {
			originUri: originUri,
			category: category,
			filter: filter,
			start: start,
			count: count
		};
		
		if (this._servicesInitialized === false) {
			var deferred2A = this._getServiceItems();

			deferred2A.addCallback(dojo.hitch(this, function(serviceItems) {
				var deferred2B = this._getCatalogDocument(originUri);
				
				deferred2B.addCallback(dojo.hitch(this, function(status) {
					this._getEntriesHelper(deferred, query);
					return status;
				}));			
				deferred2B.addErrback(dojo.hitch(this, function(errorObject) {
					deferred.errback(errorObject);	
					return errorObject; 
				}));				
				
				return serviceItems;
			}));			
			deferred2A.addErrback(dojo.hitch(this, function(errorObject) {
				deferred.errback(errorObject);	
				return errorObject; 
			}));				
			
			return deferred;

		} else if (!this._catalogIndexInitialized[originUri]) {
			
			var deferred3 = this._getCatalogDocument(originUri);

			deferred3.addCallback(dojo.hitch(this, function(status) {
				this._getEntriesHelper(deferred, query);
				return status;
			}));			
			deferred3.addErrback(dojo.hitch(this, function(errorObject) {
				deferred.errback(errorObject);	
				return errorObject; 
			}));				

			return deferred;
		}

		this._getEntriesHelper(deferred, query);
		
		return deferred;
		
	},	

	/*
 	 * @param dojo.Deferred deferred
 	 * @param Object query
 	 * 
 	 * @return void
 	 */		
	_getEntriesHelper: function(deferred, query) {
		
		var originUri = query.originUri;
		var category = query.category;
		var filter = query.filter;
		
		var filteredResult;		
		
		if (originUri) {
			// Specific Origin
			filteredResult = this._getFilteredEntriesByOrigin(originUri);
		} else {
			// All Origins
			filteredResult = this._getAllEntries();
		}
			
		if (category) {
			// Specific Category
			filteredResult = this._getFilteredEntriesByCategory(filteredResult, category);
		}
		if (filter) {
			// Filter Text
			filteredResult = this._getFilteredEntriesByText(filteredResult, filter);
		}
		
		var resultsLength = filteredResult.total;

		// Check for results
		if (resultsLength === 0) {
			deferred.callback(filteredResult);
			return;
		}		
		
		// Sort the entries
		if (resultsLength > 1) {
			filteredResult.entries.sort(dojo.hitch(this, this._sortEntries));
		}
		
		// Validate start index
		var start = query.start;
		if (start > (resultsLength - 1)) {
			ErrorUtility.logError(new Error("Start index exceeds total widgets available: " + start), this.id);
			deferred.callback({
				entries: [],
				total: 0
			});
			return;
		}

		// Calculate end index
		var count = query.count;
		var end = start + count;
		if (end > resultsLength) {
			end = resultsLength;
		}
		
		// Return results page subset
		deferred.callback({
			entries: filteredResult.entries.slice(start, end),
			total: resultsLength
		});
	},	

	/*
 	 * @param void
 	 * 
 	 * @return Object
 	 */		
	_getAllEntries: function() {
		
		var entries = [];
		for (var entryID in this._entriesIndex) {
			if (this._entriesIndex.hasOwnProperty(entryID)) {
				var entry = this._entriesIndex[entryID];
				entries.push(entry);
			}
		}
		
		return {
			entries: entries,
			total: entries.length
		};		
	},

	/*
 	 * @param String originUri
 	 * 
 	 * @return Object
 	 */		
	_getFilteredEntriesByOrigin: function(originUri) {
		
		if ((originUri in this._entriesByOriginIndex) === false) {
			return {
				entries: [],
				total: 0
			};				
		}
		
		var entryUris = this._entriesByOriginIndex[originUri];
		var entries = [];
		
		for (var indexi = 0; indexi < entryUris.length; indexi++) {
			var entryUri = entryUris[indexi];
			if (entryUri in this._entriesIndex) {
				var entry = this._entriesIndex[entryUri];
				entries.push(entry);
			}
			// TODO: MP: else Error?
		}
		
		return {
			entries: entries,
			total: entries.length
		};		
	},

	/*
 	 * @param Object result
 	 * @param String category
 	 * 
 	 * @return Object
 	 */		
	_getFilteredEntriesByCategory: function(result, category) {
		
		if (result.total === 0) {
			return result;
		}

		var entries = result.entries;
		var filteredEntries = [];
		
		for (var indexi = 0; indexi < entries.length; indexi++) {
			var entry = entries[indexi];
			var categories = entry.categories;
			for (var indexj = 0; indexj < categories.length; indexj++) {
				var targetCategory = categories[indexj];
				// TODO: MP: Create an index for origins / categories
				if (targetCategory === category) {
					filteredEntries.push(entry);
					break;
				}
			}
		}
		
		return {
			entries: filteredEntries,
			total: filteredEntries.length
		};		
	},

	/*
 	 * @param Object result
 	 * @param String filter
 	 * 
 	 * @return Object
 	 */		
	_getFilteredEntriesByText: function(result, filter) {
		
		if (result.total === 0) {
			return result;
		}

		var filteredEntries = [];
		var pattern = new RegExp(filter, "i");
		
		var entries = result.entries;
		for (var index = 0; index < entries.length; index++) {
			var entry = entries[index];
			var title = entry.title;
			
			if (pattern.test(title)) {
				filteredEntries.push(entry);
			}
		}
		
		return {
			entries: filteredEntries,
			total: filteredEntries.length
		};		
	},

	/*
 	 * @param String entry1
 	 * @param String entry2
 	 * 
 	 * @return Number
 	 */
	_sortEntries: function(entry1, entry2) {
		
		// Note: Make spaces have less sorting precedence than letters
		// See: 135903: Widget Catalog:  Most frequently used widgets are difficult to find
		var title1 = entry1.title.toLowerCase().replace(/\s+/g, "~");
		var title2 = entry2.title.toLowerCase().replace(/\s+/g, "~");
		
		if (title1 < title2) {
			// Less than
			return -1;
		} else if (title1 > title2) {
			// Greater than
			return 1;
		}
		// Equal
		return 0;
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	destroy: function() {
		
		this.inherited(arguments);
	}
	
});

})();
