/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.catalog.DashboardCatalogServiceProvider");

(function(){

dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.dashboard.web.ui.internal.catalog.LegacyCatalogService");
dojo.require("jazz.ensemble.internal.catalog._CatalogServiceProvider");
dojo.require("jazz.ensemble.internal.catalog.CatalogService");
dojo.require("jazz.app.internal.util.Rootservices");
dojo.require("jazz.app.context");
dojo.require("jazz.app.internal.applicationAbout");


var DashboardClient 		= com.ibm.team.dashboard.web.client.internal.DashboardClient;
var Rootservices 			= jazz.app.internal.util.Rootservices;
var CatalogService 			= jazz.ensemble.internal.catalog.CatalogService;
var LegacyCatalogService 	= com.ibm.team.dashboard.web.ui.internal.catalog.LegacyCatalogService;
var applicationAbout        = jazz.app.internal.applicationAbout;


var JUI_NS = "http://jazz.net/ns/ui#";
var WIDGET_CATALOG_ROOTSERVICE = "widgetCatalog";
var PROCESS_CATALOG_SERVICE_URL = "/service/com.ibm.team.dashboard.common.internal.service.IProcessWidgetCatalogService/";
var ROOTSERVICES_PATH = "/rootservices";

function getCatalogServiceArgs(aboutService, rootservice, defaultName){
	if (rootservice) {
		var position = rootservice.lastIndexOf(ROOTSERVICES_PATH);
		if (position == rootservice.length-ROOTSERVICES_PATH.length) {
			var appUrl = rootservice.substring(0,position);
		}
		if (!aboutService){
			aboutService = applicationAbout.findAboutUriFromContext(appUrl);
		}
	}
	var about=null;
	if (aboutService) {
		about = applicationAbout.get(aboutService);
	}
	var args = {};
	if (about) {
		args.title = about.title;
		args.instanceName = about.instanceName;
		args.isFriend = false;
	} else if (rootservice) {
		args.title = defaultName;
		args.instanceName = appUrl;
		args.isFriend = true;
	} else {
		args.title = defaultName;
		args.isFriend = true;
	}
	return args;
}

dojo.declare("com.ibm.team.dashboard.web.ui.internal.catalog.DashboardCatalogServiceProvider", [jazz.ensemble.internal.catalog._CatalogServiceProvider], {
	
	_processCatalogUrl: null,
	
	constructor: function(){
		var projectAreaItemId = this._getCurrentProjectAreaItemId();
		if (projectAreaItemId) {
			this._processCatalogUrl = this._getServerRoot() + PROCESS_CATALOG_SERVICE_URL + projectAreaItemId;
		}
	},
	
	loadCatalogs: function(refresh){
		var _this = this;
		var deferred = new dojo.Deferred();
		var rootDeferred = Rootservices.loadLocal();
		
		rootDeferred.addCallback(function(root){
			var catalogs = []; 
			var localRootServicesUrl = root.getAbout();
			
			var args = getCatalogServiceArgs(root.getPublisher(), localRootServicesUrl, root.getTitle());
			args.processUrl = _this._processCatalogUrl;
			args.rootservices = root;
			catalogs.push(new DashboardLocalCatalogService(args));
			
			/**
			 * now load the friends
			 */
			var friendDeferred = root.loadFriends();
			
			friendDeferred.addCallback(function(friends){
				dojo.forEach(friends, function(friend){
					// ignore self-friend entries
					if (friend.rootServices != localRootServicesUrl) {
						var catalogArgs = getCatalogServiceArgs(null, friend.rootServices, friend.title);
						catalogArgs.rootservices = new Rootservices(friend.rootServices);
						catalogs.push(new DashboardCatalogService(catalogArgs));
					}
				});
				deferred.callback(catalogs);
			});
			
			friendDeferred.addErrback(function(errorObject) {
				deferred.errback(errorObject);
				return errorObject;
			});
		});
		
		rootDeferred.addErrback(function(errorObject) {
			deferred.errback(errorObject);
			return errorObject;
		});
		
		return deferred;
	},
	
	/*
	 * @param void
	 *
	 * @return String
	 */
	_getServerRoot: function(){
	
		if (typeof(this._serverRoot) === "string") {
			return this._serverRoot;
		}
		
		// e.g. Data server root:
		// Local: undefined
		// Remote: https://127.0.0.1:9444/jazz
		var root = dojo.getObject("net.jazz.ajax._dataServerRoot");
		if (!root) {
			// e.g. Context root:
			// Local: /jazz
			// Remote: https://127.0.0.1:9444/jazz
			root = dojo.getObject("net.jazz.ajax._contextRoot");
		}
		if (!root) {
			root = "";
		}
		this._serverRoot = root;
		
		return this._serverRoot;
	},
	
	_getCurrentProjectAreaItemId: function(){
		var context = jazz.app.context.get();
		return context ? context.itemId : null;
	}
	
});


dojo.declare("com.ibm.team.dashboard.web.ui.internal.catalog.DashboardCatalogService", [jazz.ensemble.internal.catalog._CatalogService], {

	title: "",
	instanceName: "",
	isFriend: false,
	rootservices: null,
	
	constructor: function(args) {
		dojo.mixin(this, args);
	},
	
	load: function(refresh){
		return loadRootserviceDefinedCatalog(this.rootservices);
	}
});


/**
 * represents the widget catalog at a particular location
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.catalog.DashboardLocalCatalogService", [jazz.ensemble.internal.catalog._CatalogService], {
	
	title: "",
	instanceName: "",
	isFriend: false,
	rootservices: null,
	processUrl: null,
	local: true,
	
	constructor: function(args){
		dojo.mixin(this, args);
	},
	
	/**
	 * returns a deferred the result of which is
	 * https://jazz.net/wiki/bin/view/Main/WidgetCatalogDiscoveryAPI
	 */
	load: function(refresh){
		var deferred = new dojo.Deferred();
		var _this = this;
		
		/**
		 * load the local catalog first then mix in the process catalog
		 */
		var localDeferred = loadRootserviceDefinedCatalog(this.rootservices);
		
		localDeferred.addCallback(function(localCatalog){
		
			if (!_this.processUrl) {
				deferred.callback(localCatalog);
			}
			else {
				var processDeferred = jazz.client.xhrGet({
					handleAs: "xml",
					url: _this.processUrl
				});
				
				processDeferred.addCallback(function(doc){
				
					deferred.callback(CatalogService.consolidateWidgetCatalogs([localCatalog, CatalogService.parseWidgetCatalog(doc, _this.processUrl)]));
				});
				
				processDeferred.addErrback(function(errorObject) {
					deferred.errback(errorObject);
					return errorObject;
				});
			}
		});
		localDeferred.addErrback(function(errorObject) {
			deferred.errback(errorObject);
			return errorObject;
		});
		
		return deferred;
	}
});

/**
 * this loads a catalog defined in a rootservice document.
 * @param {Object} rootservice
 */
function loadRootserviceDefinedCatalog(rootservice){
	var deferred = new dojo.Deferred();
	
	/**
	 * everything starts with loading the root services document
	 */
	var rootDeferred = rootservice.load();
	
	rootDeferred.addCallback(function(root){
	
		var widgetCatalogUrl = root.getServiceUrl(JUI_NS, WIDGET_CATALOG_ROOTSERVICE);
		
		/** this is a 3.0 service!! respond accordingly otherwise look for a 1.0 catalog
		 *
		 */
		if (widgetCatalogUrl) {
			var serviceDeferred = jazz.client.xhrGet({
				handleAs: "xml",
				url: widgetCatalogUrl
			});
			
			serviceDeferred.addCallback(function(doc) {
				try {
					var catalog = CatalogService.parseWidgetCatalog(doc, widgetCatalogUrl);
					deferred.callback(catalog);
				} catch (errorObject) {
					console.debug(errorObject);
					deferred.errback(errorObject);
				}
			});
			
			serviceDeferred.addErrback(function(errorObject) {
				deferred.errback(errorObject);
				return errorObject;
			});
		}
		else {
			/**
			 * ok this rootservice document does not have a widget catalog
			 * so lets *assume* it is has a legacy catalog
			 * This may not be a good assumption;
			 * there is a chance that this server does not have a catalog in this case we return null
			 */
			var viewletServiceRoot = root.getViewletServiceRoot();
			var viewletWebUIRoot = root.getViewletWebUIRoot();
			
			if (!(viewletServiceRoot && viewletWebUIRoot)) {
				deferred.callback(null);
				return;
			}
			
			var legacyDeferred = LegacyCatalogService.load(viewletServiceRoot, viewletWebUIRoot);
			legacyDeferred.addCallback(function(doc){
				deferred.callback(doc);
			});
			legacyDeferred.addErrback(function(errorObject) {
				deferred.errback(errorObject);
				return errorObject;
			});
		}
	});
	rootDeferred.addErrback(function(errorObject) {
		deferred.errback(errorObject);
		return errorObject;
	});
	
	return deferred;
	
}

var DashboardCatalogService = com.ibm.team.dashboard.web.ui.internal.catalog.DashboardCatalogService;
var DashboardLocalCatalogService = com.ibm.team.dashboard.web.ui.internal.catalog.DashboardLocalCatalogService;
	
})();
