/*******************************************************************************
 * Licensed Materials - Property of IBM (c) Copyright IBM Corporation 2009, 2011. All
 * Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.catalog.LegacyCatalogService");
dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");

(function()
{
var CatalogService 			= jazz.ensemble.internal.catalog.CatalogService;
var DashboardClient 		= com.ibm.team.dashboard.web.client.internal.DashboardClient;
var ServiceResponseHandler 	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var Util 					= com.ibm.team.dashboard.web.util.internal.Util;

var VIEWLET_SERVICE = "/service/com.ibm.team.dashboard.common.internal.service.IDashboardRestService/viewletDefinitions";
var IMAGE_SERVICE = "/service/com.ibm.team.dashboard.common.internal.service.IDashboardImageService";


com.ibm.team.dashboard.web.ui.internal.catalog.LegacyCatalogService.load = function(viewletServiceRoot, viewletWebUIRoot)
{
	var deferred = getRemoteViewletHierarchy(viewletServiceRoot);
	
	deferred.addCallback(function(doc)
	{
		var uriMaker = new EntryUriMaker(viewletServiceRoot);		
		var results = {entries:[], categories:[], origins:[]};
		dojo.forEach(doc.viewletCategories, function(category){
			readCategory(category, [], results, viewletServiceRoot, viewletWebUIRoot, uriMaker);
	    });	
		
		return results;	
	});
	
	return deferred;
};

function readCategory(viewletCategory, parentCategories, results, viewletServiceRoot, viewletWebUIRoot, uriMaker)
{
	results.categories.push(viewletCategory.name);

	var categories = dojo.clone(parentCategories);
	categories.push(viewletCategory.name);
	
	var imgSrcMaker = new ImageSrcMaker(viewletServiceRoot);

	dojo.forEach(viewletCategory.viewletEntries, function(e){
		var entry = {};
		
		entry.title         = e.title||"";
		entry.type          = "com.ibm.team.dashboard.viewlet";
		entry.description   = e.description||"";
		entry.icon          = imgSrcMaker.getImageSrc(e.icon) || "";
		entry.preview       = imgSrcMaker.getImageSrc(e.preview) || "";
		entry.thumbnail     = "";
		entry.author        = "";
		entry.uri           = uriMaker.make(e);
		entry.widgetUri     = createViewletWidgetUri(e.viewletDefId, viewletServiceRoot, viewletWebUIRoot);
		
		entry.categories    = categories;
		entry.preferences   = readPreferences(e.preferences);
		
		results.entries.push(entry);
	});

	/*
	 * recurse through the tree
	 */
	dojo.forEach(viewletCategory.viewletCategories, function(c){
		readCategory(c, categories, results, viewletServiceRoot, viewletWebUIRoot, uriMaker);
	});
}

function ImageSrcMaker(viewletServiceRoot) {
	
	this.serverRoot = viewletServiceRoot;
	
	this.getImageSrc = function(imagePath) {
	
		if (!imagePath) {
			return "";
		} else if (imagePath.indexOf("://") != -1) {
			// Absolute path
			return imagePath;
		} 
		
		// Relative path
		// e.g. com.ibm.team.reports.viewlets.web/resources/graphics/build/images/preview-build-duration.png
		
		return viewletServiceRoot + IMAGE_SERVICE + "?path=" + encodeURIComponent(imagePath);
	};
	
}

/**
 * Need to make a unique URI for each catalog entry.
 * If it has an id field that that is guarenteed to be unique. Otherwise use the viewletDefId.
 * If there is more than one viewletDefId, append a number to the second occurance.
 */
function EntryUriMaker(viewletServiceRoot){
	this.root 	= viewletServiceRoot;
	this.idhash = {};
	
	this.make = function(e)
	{
		// id
		if (e.id) {
			return this.root + "#" + e.id;
		}
		
		// ViewletDefId
		if (e.viewletDefId in this.idhash) { 
			this.idhash[e.viewletDefId]++;
		} else { 							  
			this.idhash[e.viewletDefId] = 0;
		}
		
		return this.root+VIEWLET_SERVICE+"#"+e.viewletDefId+(this.idhash[e.viewletDefId]?"-"+this.idhash[e.viewletDefId]:"");
	};
}

/**
 * create an URI for the viewlet (its widgetUri)
 * @see https://jazz.net/wiki/bin/view/Main/ViewletDefinitionURIs
 * 
 */
function createViewletWidgetUri(definitionId, serviceRoot, webUIRoot){
	
	return	serviceRoot + VIEWLET_SERVICE + 
			'?'+
	        'definitionIds='+definitionId+'&'+		 
	        'webUIRoot='	+encodeURIComponent(webUIRoot); 
	
}

function readPreferences(preferences){
	
	var results = [];
	dojo.forEach(preferences, function(p){
		results.push({name:p.id, value:p.value});
	});
	return results;
}

function getRemoteViewletHierarchy(viewletServiceRoot) {
	
	var deferred =  new dojo.Deferred();

	var serviceObject = {
		success: dojo.hitch(this, function(dto) {
			deferred.callback(dto);
		}),
		failure: dojo.hitch(this, function(errorObject) {
			deferred.errback(errorObject);
		})
	};

	var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
	
	var request = DashboardClient.getViewletHierarchy(
		null, 	// projectAreaName, 
		true, 	// remoteOnly  
		null, 	// scope, 
		null, 	// scopeItemId
		viewletServiceRoot, 
		handler);
	
	return deferred;
}

})();

