/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.dnd.DraggableTab");

dojo.require("com.ibm.team.dashboard.web.ui.internal.dnd.TabTarget");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("jazz.util.html");
dojo.require("jazz.ui.Menu");
dojo.require("jazz.ui.MenuPopup");
dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

var TabTarget = com.ibm.team.dashboard.web.ui.internal.dnd.TabTarget;
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var HTML = jazz.util.html;
var messages;

dojo.declare("com.ibm.team.dashboard.web.ui.internal.dnd.DraggableTab",
	[dijit._Widget, dijit._Templated], {
		
	tabTitle: null,
	editing: false,
	selected: false,
	draggableTabs: null,
	dashboardContainer: null,
	page: null,
	_menuOpen: false,
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/DraggableTab.html"),
	
	constructor: function(args) {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
	},
	postCreate: function() {
		if (!this.selected) this._registerSelectEvent(true);
		this.setEditMode(this.editing);
		dijit.setWaiRole(this.domNode, "tab");
	},
	setSelected: function(selected) {
		if (selected == this.selected) return;
		
		this.selected = selected;
		if (selected) {
			if (this.draggableTabs.selectedTab && !this.draggableTabs.selectedTab.isDestroyed)
				this.draggableTabs.selectedTab.setSelected(false);
			this._addTabDropDown();
			dojo.addClass(this.domNode, "selectedTab");
			this.draggableTabs.selectedTab = this;
			this._registerSelectEvent(false);
			if (!this._editableTitle) this._tabTitle.tabIndex = "0";
		} else {
			this._removeTabDropDown();
			this._registerSelectEvent(true);
			dojo.removeClass(this.domNode,"selectedTab");
			this._tabTitle.tabIndex = "-1";
		}
		this.setEditMode(this.editing);
	},
	setEditMode: function(editing) {
		this.editing = editing;
		this.setTitle();
		if (editing && !this.selected) {
			if (!this._tabTarget) {
				this._tabTarget = new TabTarget(this.domNode, {
					accept: ["widget"],
					index: this.index,
					dashboardContainer: this.dashboardContainer
				});
			}
		} else {
			if (this._tabTarget) {
				this._tabTarget.destroy();
				this._tabTarget = null;
			}
		}
	},
	setTitle: function(title) {
		if (!title) title = this.tabTitle;
		
		if (this.selected && this.editing) {
			if (!this._editableTitle) {
				this._tabTitle.innerHTML = "";
				
				var div = document.createElement("div");
				div.className = "editable-tab-title";
				div.innerHTML = HTML.escape(title);
		        this._editableTitle = new dijit.InlineEditBox({title:messages.clickToEdit, autoSave: true, width: "200px", noValueIndicator: "?", editorParams: {maxLength: "60"}, onChange: dojo.hitch(this, this._onTabTitleChange), value: title}, div);
		        this._tabTitle.appendChild(div);				
				this._editableTitle.startup();
				this._tabTitle.tabIndex = "-1";
				
				this._tabTitleCloseMenuEvent = this.connect(this._editableTitle, "edit", jazz.ui.MenuPopup.closeAll);
				this._tabTitleKeypressStopper = this.connect(this._editableTitle.domNode.parentNode,"onkeypress", this._stopBubble);
				this._tabTitleClickStopper = this.connect(this._editableTitle.domNode.parentNode,"onmousedown", this._stopBubble);			
			}
		} else {
			if (this._editableTitle) {
				this.disconnect(this._tabTitleCloseMenuEvent);
				this.disconnect(this._tabTitleKeypressStopper);
				this.disconnect(this._tabTitleClickStopper);
				this._tabTitleCloseMenuEvent = null;
				this._tabTitleKeypressStopper = null;
				this._tabTitleClickStopper = null;
				this._editableTitle.destroy();
				this._editableTitle = null;
			}
			this._tabTitle.innerHTML = HTML.escape(title);
		}
		this.tabTitle = title;
	},
	_registerSelectEvent: function(doRegister) {
		if (doRegister) {
			if (!this._selectTabHandle) {
				this._selectTabHandle = this.connect(this.domNode, "onclick", this._selectTabHandler); 
			}
		} else {
			this.disconnect(this._selectTabHandle);
			this._selectTabHandle = null;
		}
	},
	_addTabDropDown: function() {
		if (this._menuItemClickHandler) {
			this._removeTabDropDown();
		}
		dojo.addClass(this.domNode, "tabWithMenu");
		var accessibleName = dojo.string.substitute(this.messages.tabButtonName, [this.tabTitle]);
		this._menuLink = new jazz.ui.toolbar.MenuButton({
			menuProvider: dojo.hitch(this,this._tabDropDownProvider),
			title: accessibleName
		}).placeAt(this._menuWrapper, "only");
		this._menuWrapper.style.display = "block";
	},
	_removeTabDropDown: function() {
		if (this._menuLink) {
			this._menuLink.destroy();
			delete this._menuLink;
			dojo.removeClass(this.domNode, "tabWithMenu");
			this._menuWrapper.style.display = "none";
		}
	},
	_tabDropDownProvider: function() {
		//if the current tab is being edited, save it
		if (this._editableTitle && this._editableTitle.editing){
			this._editableTitle.save();
		}
		//Display menu
		this._createTabDropDown();
		return this._menu;
	},
	_createTabDropDown: function (){
		if (this._menu && this._menuIsEditing != this.editing) {
			this._menu.destroyRecursive();
			this._menu = null;
		}
		if (!this._menu) {
			// create items
			var menu = new jazz.ui.Menu();
			dojo.addClass(menu.domNode, "com-ibm-team-dashboard-tab-menu");
			if (this.editing) {
				menu.addChild(new dijit.MenuItem({label: messages.addViewletText, iconClass: 'addIcon', onClick: dojo.hitch(this.dashboardContainer, this.dashboardContainer.openCatalog)}));
				menu.addChild(new dijit.MenuSeparator());
				menu.addChild(new dijit.MenuItem({label: messages.rename, iconClass: 'renameIcon',	onClick: dojo.hitch(this, this._renameTab)}));
			}
			menu.addChild(new dijit.MenuItem({label: messages.refresh, iconClass: 'refreshIcon', onClick: dojo.hitch(this.dashboardContainer, this.dashboardContainer.refresh)}));
			menu.addChild(new dijit.MenuSeparator());
			menu.addChild(new dijit.MenuItem({label: messages.copyTab, iconClass: 'copyTabIcon', onClick: dojo.hitch(this.dashboardContainer, this.dashboardContainer.copyTab)}));
			if (this.editing) {
				menu.addChild(new dijit.MenuItem({label: messages.duplicateTab, iconClass: 'duplicateTabIcon', onClick: dojo.hitch(this.dashboardContainer, this.dashboardContainer.duplicateTab)}));
				this._menuDeleteItem = new dijit.MenuItem({label: messages.deleteText, iconClass: 'deleteIcon', onClick: dojo.hitch(this, this._deleteTab)});
				menu.addChild(this._menuDeleteItem);
				menu.addChild(new dijit.MenuSeparator());
				var menu2 = new jazz.ui.Menu();
				dojo.addClass(menu2.domNode, "com-ibm-team-dashboard-tab-menu");
				menu2.addChild(new dijit.MenuItem({label: messages.oneColumn, iconClass: 'oneColumnIcon', onClick: dojo.hitch(this,this._changeLayout, "jazz.ensemble.layout.1column")}));
				menu2.addChild(new dijit.MenuItem({label: messages.twoColumns, iconClass: 'twoColumnsIcon', onClick: dojo.hitch(this,this._changeLayout, "jazz.ensemble.layout.2column")}));
				menu2.addChild(new dijit.MenuItem({label: messages.threeColumns, iconClass: 'threeColumnsIcon', onClick: dojo.hitch(this,this._changeLayout, "jazz.ensemble.layout.3column")}));
				var iconClass = this._getColumnIconClass();
				var popupMenu = this._layoutMenuItem = new dijit.PopupMenuItem({label: messages.layout, iconClass: iconClass});
				popupMenu.popup = menu2;
				menu.addChild(popupMenu);
			}
			menu.startup();
			if (this.editing) {
				popupMenu.startup();
				menu2.startup();
				this._popupMenu = popupMenu;
			}
			this._menu = menu;
			this._menuIsEditing = this.editing;
		}
		// enable/disable items appropriately
		if (this.editing) {
			this._menuDeleteItem.domNode.style.display = (this.draggableTabs.widgets.length > 1) ? "" : "none";
		}
	},
	_onTabTitleChange: function(/* string */ newTitle) {
		if(!newTitle) return false;
		this.tabTitle = newTitle;
		this.page.setTitle(newTitle);
		this.draggableTabs.propertyChanged(Constants.PROPERTY_DASHBOARD);
		
		//workaround to innerHTML bug where extra <a> tag wrapper appears when it shouldn't.
		this._editableTitle.displayNode.innerHTML = "";
		this._editableTitle.displayNode.appendChild(document.createTextNode(newTitle));
	},
	destroy: function () {
		if (this._editableTitle) {
			this._editableTitle.destroy();
			delete this._editableTitle;
		}
		if (this._menuLink) {
			this._menuLink.destroy();
			delete this._menuLink;
		}
		if (this._menu) {
			this._menu.destroyRecursive();
			delete this._menu;
		}
		this.isDestroyed = true;
		this.inherited(arguments);
	},
	_selectTabHandler: function(event) {
		event.preventDefault();
		if (dojo.hasClass(event.currentTarget,"selectedTab")) return;
		
		//if the current tab is being edited, save it
		if (this._editableTitle && this._editableTitle.editing){
			this._editableTitle.save();
		}
		
		for (var i = 0; i < this.draggableTabs.widgets.length; i++) {
			if (event.currentTarget == this.draggableTabs.widgets[i].domNode) {
				this.draggableTabs._selectTabAction(this.draggableTabs.widgets[i]);
				break;
			}
		}
	},
	_deleteTab: function(event) {
		if (this.draggableTabs.remove(this.index))
			this.destroy();
	},
	_renameTab: function(event) {
		this._editableTitle._onClick(event);
	},
	_changeLayout: function(/* string */ type) {
		this.dashboardContainer.getCurrentViewletPage().setLayout(type);
		this.draggableTabs.propertyChanged(Constants.PROPERTY_DASHBOARD);
		
		// update current layout icon
		var iconClass = this._getColumnIconClass();
		this._layoutMenuItem.attr("iconClass", iconClass);
	},
	_getColumnIconClass: function(){
		var layoutProp = this.page.getLayout();
		var layout = layoutProp.getLayout();
		switch (layout.getType()) {
			case "jazz.ensemble.layout.1column":
				return "oneColumnIcon";
			case "jazz.ensemble.layout.2column":
				return "twoColumnsIcon";
			case "jazz.ensemble.layout.3column":
				return "threeColumnsIcon";
		}
	},
	_stopBubble: function(event){
		if (event) {
			if (this._editableTitle.editing) {
				event.cancelBubble = true;
			} else {
				return;
			}
		}
		return false;
	}
});

})();