/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.dnd.TabTarget");

dojo.require("dijit._Widget");

dojo.require("jazz.ensemble.internal.Schema");
dojo.require("jazz.ensemble.internal.util.XmlBindingFactory");
dojo.require("jazz.ui.dnd.DraggableContainer");
dojo.require("jazz.ui.dnd.DraggableManager");

dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");

(function() {
	
	var Schema = jazz.ensemble.internal.Schema;
	var XmlBindingFactory = jazz.ensemble.internal.util.XmlBindingFactory;
	
	var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
	var draggableManager = jazz.ui.dnd.draggableManager();
	
	dojo.declare("com.ibm.team.dashboard.web.ui.internal.dnd.TabTarget", jazz.ui.dnd.DraggableContainer, {
		constructor: function(node, params){
			this.isSource = false;
			this.index = params.index;
			this.dashboardContainer = params.dashboardContainer;
		},
		destroy: function() {
			var targets = draggableManager.targets;
			for (var i = 0; i < targets.length; i++ ) {
				if (targets[i] == this) {
					targets.splice(i,1);
					break;
				}	
			}
			this.inherited(arguments);
		},
		/* Move
		 * 
		 * return: cancelDrag - true/false 
		 * 
		 */
		move: function(fromIndex, toTab) {
			var dw = draggableManager.widget;
			var widgetCon = dijit.byId(dw.domNode.id);
			if (widgetCon) {
				var widget = widgetCon.widget;
				if (widget.viewlet && widget.viewlet.isEditing()) {
					draggableManager.stopForInput();
					if(!widget._stopEditing()) {
						return true;
					}
				}
			}
			var fromContainer = draggableManager.fromContainer,
				item = fromContainer.widgets[fromIndex],
				// find a free widget id in this tab
				pages = this.dashboardContainer.dashboardResource.binding.getPages().getPages(),
				toPage = pages[toTab.index],
				widgetCollection = toPage.getWidgets(),
				widgets = widgetCollection.getWidgets(),
				max = 0;
			for (var i=0;i<widgets.length;i++) {
				try {
					var widgetId = parseInt(widgets[i].getId().substring(1));
					if (widgetId > max) {
						max = widgetId;
					}
				}
				catch (/* Error */ e) {
					// not an integer; skip
				}
			}
			var widgetId = "_" + (max + 1),
				widget = dijit.byNode(item.domNode);
			
			// remove from source tab
			var viewletPage = this.dashboardContainer.getCurrentViewletPage();
			viewletPage.removeWidget(widget.widgetId);
			viewletPage.updateEmptyPageMessage();
			
			// add to destination tab
			widget.state.setId(widgetId);
			widgets.push(widget.state);
			widgetCollection.setWidgets(widgets);
			
			var column = XmlBindingFactory.createBinding(toPage.getLayout().getLayout().getData().getLayoutData(), Schema.COLUMN_LAYOUT_DATA).getColumns().getColumns()[0],
				toWidgetCollection = column.getWidgets(),
				toWidgets = toWidgetCollection.getWidgets();
			toWidgets.splice(0, 0, widgetId);
			toWidgetCollection.setWidgets(toWidgets);
			
			this.dashboardContainer._onDashboardChange();
			this.dashboardContainer.setPageDirty(toTab.index);
			this.updateIndices(fromContainer);
			widget.destroy();
			return false;
		},
		registerContainer: function() {
			draggableManager.targets.push(this);
		}
	});
})();