/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.provision.ProvisioningPage");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Dialog");
dojo.require("dijit.layout.SplitContainer");
dojo.require("dijit.layout.ContentPane");
dojo.require("jazz.ui.Menu");
dojo.require("dijit.Tree");
dojo.require("dojo.data.ItemFileWriteStore");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("com.ibm.team.repository.web.ui.internal.provision.ProvisioningStatusPanel");
dojo.require("com.ibm.team.repository.web.ui.internal.provision.ConnectWidget");
dojo.require("com.ibm.team.repository.web.client.internal.ProvisioningClient");
dojo.require("com.ibm.team.repository.web.transport.internal.ProvisionResponseHandler");
dojo.require("jazz.ui.SideNavigationLayout");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.NotAuthorizedView");

(function() {
	var PlatformUI = net.jazz.ajax.ui.PlatformUI;
	var checkNotNull = net.jazz.ajax.toolbox.lang.checkNotNull;
	var checkNotEmpty = net.jazz.ajax.toolbox.lang.checkNotEmpty;
	var ProvisioningClient = com.ibm.team.repository.web.client.internal.ProvisioningClient;

	dojo.declare("com.ibm.team.repository.web.ui.internal.provision.ProvisioningPage", dijit._Widget, {

			postCreate: function() {
				com.ibm.team.repository.web.ui.internal.provision.ProvisioningPage.superclass.postCreate.apply(this);
				PlatformUI.getWorkbench().getActionRegistry().registerAction("com.ibm.team.repository.web.provision.defaultAction", 
				                                                             function(args) {});
				
				if (com.ibm.team.repository.web.noAdminAuth) {
					this._notAuth = new com.ibm.team.repository.web.ui.internal.admin.NotAuthorizedView();
					this.domNode.appendChild(this._notAuth.domNode);
					return;
				}
				                                                            
				var treeData = {     
                    identifier: 'id',
                    label: 'name',   
	                items: [
	                    { 
	                    	id: 'rootnode', name: 'Jazz Server', type: 'root', icon: 'iconSiteObj',
	                          children:[
	                          	{ id: 'installed~', name:'Installed Jazz Components', type: 'status', icon: 'iconPsiteObj'}, 
	                          	{ id: 'available~', name:'Available Jazz Components', type: 'status', icon: 'iconPsiteObj'},
	                          	{ id: 'corebundles~', name:'Core Bundles', type: 'status', icon: 'iconEsiteObj'}
	                          ]
	                    }
                   	]
        		};
        		
        		this._mainApplicationView = new jazz.ui.SideNavigationLayout();
        		this._mainApplicationView._leftNav.style.width = "350px";
        		this._mainApplicationView._leftNavWidth = 350;
				this.domNode.className = "com-ibm-team-repository-admin"; 
        		this.domNode.appendChild(this._mainApplicationView.domNode);
        		
				this.treeSection = document.createElement("div");
				this._mainApplicationView.addNavigationSection(this.treeSection, "Server Components");
				
				this.provStatus = new com.ibm.team.repository.web.ui.internal.provision.ProvisioningStatusPanel();
				this._mainApplicationView.setMainContent(this.provStatus.domNode);
        		
		        this.treeStore = new dojo.data.ItemFileWriteStore({data: treeData});
        		this.tree = new dijit.Tree({           
                					id:'provTree',
                					store:this.treeStore,
                					query: {type:'root'},
                					labelAttr:"name",
                					getIconClass: function(item) {
                						if (item != null) {
		                            		return this.store.getValue(item, "icon");
                						}	
									}}, this.treeSection);
				dojo.subscribe(this.tree.id, this, "treeListener");
				
				var menu = new jazz.ui.Menu();
				
				this.connectMenuItem = new dijit.MenuItem({label: "Connect", disabled: true});
				menu.addChild(this.connectMenuItem);
				this.connect(this.connectMenuItem, "onClick", "showConnectDialog");
				
				this.disconnectMenuItem = new dijit.MenuItem({label: "Disconnect", disabled: true});
				menu.addChild(this.disconnectMenuItem);
				this.connect(this.disconnectMenuItem, "onClick", "disconnectAction");
				
				this.installFeatureMenuItem = new dijit.MenuItem({label: "Install", disabled: true});
				menu.addChild(this.installFeatureMenuItem);
				this.connect(this.installFeatureMenuItem, "onClick", "installAction");

				this.installAndStartFeatureMenuItem = new dijit.MenuItem({label: "Install and Start", disabled: true});
				menu.addChild(this.installAndStartFeatureMenuItem);
				this.connect(this.installAndStartFeatureMenuItem, "onClick", "installAndStartAction");
				/*
				this.updateFeatureMenuItem = new dijit.MenuItem({label: "Update", disabled: true});  
				menu.addChild(this.updateFeatureMenuItem);
				this.connect(this.updateFeatureMenuItem, "onClick", "updateAction");
				*/
				this.removeFeatureMenuItem = new dijit.MenuItem({label: "Remove", disabled: true});  
				menu.addChild(this.removeFeatureMenuItem);
				this.connect(this.removeFeatureMenuItem, "onClick", "removeAction");
				
				this.startMenuItem = new dijit.MenuItem({label: "Start", disabled: true});  
				menu.addChild(this.startMenuItem);
				this.connect(this.startMenuItem, "onClick", "startAction");
				
				this.stopMenuItem = new dijit.MenuItem({label: "Stop", disabled: true});  
				menu.addChild(this.stopMenuItem);
				this.connect(this.stopMenuItem, "onClick", "stopAction");
				
				this.statusMenuItem = new dijit.MenuItem({label: "Status", disabled: true});  
				menu.addChild(this.statusMenuItem);
				this.connect(this.statusMenuItem, "onClick", "statusAction");
				
				this.resetMenuItem = new dijit.MenuItem({label: "Reset", disabled: true});  
				menu.addChild(this.resetMenuItem);
				this.connect(this.resetMenuItem, "onClick", "resetAction");
				
				menu.bindDomNode(this.tree.domNode);
				
				var self = this;
				
				var getAvailableItem = function(item) {
					self.availableItem = item;
				}
				this.treeStore.fetchItemByIdentity({identity: "available~", onItem: getAvailableItem});
				
				var getCoreBundlesItem = function(item) {
					self.coreBundlesItem = item;
				}
				this.treeStore.fetchItemByIdentity({identity: "corebundles~", onItem: getCoreBundlesItem});
				
				var getInstalledItem = function(item) {
					self.installedItem = item;
				}
				this.treeStore.fetchItemByIdentity({identity: "installed~", onItem: getInstalledItem});
				
				var prh = new com.ibm.team.repository.web.transport.internal.ProvisionResponseHandler(this, "loadStatus", "requestFailed");
				ProvisioningClient.getStatus(true, true, false, null, null, prh);
				var prh = new com.ibm.team.repository.web.transport.internal.ProvisionResponseHandler(this, "displayInstallLog", "requestFailed");
				ProvisioningClient.getInstallLog(prh);
			},
			
			requestFailed: function(errorObj) {
				this.provStatus.setOutput("Request failed ["+errorObj.status+"]["+errorObj.statusText+"]");
			},
			
			loadStatus: function(response, args) {
				var self = this;
				this.connectionURL = response.connectionURL;
				this.treeStore.setValue(this.availableItem, "name", "Available Jazz Components ["+this.connectionURL+"]");
				
				if (args[0] == true) {
					var pInfo = {parent: this.coreBundlesItem, attribute: "children"};
					for (var i = 0; i < response.core.items.length; i++) {
						var newItem = this.treeStore.newItem(response.core.items[i], pInfo);
					}
				}
				if (args[1] == true) {
					var pInfo = {parent: this.installedItem, attribute: "children"};
					for (var i = 0; i < response.installed.items.length; i++) {
						var newItem = this.treeStore.newItem(response.installed.items[i], pInfo);
					}
				}
				if (args[2] == true) {
					var pInfo = {parent: this.availableItem, attribute: "children"};
					for (var i = 0; i < response.available.items.length; i++) {
						var newItem = this.treeStore.newItem(response.available.items[i], pInfo);
					}
				}
				if (args[3] && args[4]) {
					var loadFeatureStatus = function(item) {
						var pInfo = {parent: item, attribute: "children"};
						for (var i = 0; i < response.feature.items.length; i++) {
							var newItem = self.treeStore.newItem(response.feature.items[i], pInfo);
						}
					}
					this.treeStore.fetchItemByIdentity({identity: args[3], onItem: loadFeatureStatus});
				}
				this.saveTreeStore();
				if (this.connectionURL != "disconnected" && args[2] == false && args[3] == null) {
					var prh = new com.ibm.team.repository.web.transport.internal.ProvisionResponseHandler(this, "loadStatus", "requestFailed");
					ProvisioningClient.getStatus(false, false, true, null, null, prh);
				}
				this.tree.expand();               					
			},
			
			treeListener: function( message) {
				if (message.event=="execute") {
					this.selectedItem = message.item;
					this.selectedNode = message.node;
					this.connectMenuItem.setDisabled(true);
					this.disconnectMenuItem.setDisabled(true);
					this.installFeatureMenuItem.setDisabled(true);
					this.installAndStartFeatureMenuItem.setDisabled(true);
					//this.updateFeatureMenuItem.setDisabled(true);
					this.removeFeatureMenuItem.setDisabled(true);
					this.startMenuItem.setDisabled(true);
					this.stopMenuItem.setDisabled(true);
					this.statusMenuItem.setDisabled(true);
					this.resetMenuItem.setDisabled(true);
					
					if (this.selectedItem.type == 'root') {
						if (this.connectionURL == 'disconnected') {
							this.connectMenuItem.setDisabled(false);
							this.disconnectMenuItem.setDisabled(true);
						}
						else {
							this.connectMenuItem.setDisabled(true);
							this.disconnectMenuItem.setDisabled(false);
						}
						this.resetMenuItem.setDisabled(false);
					}
					else if (this.selectedItem.id == 'installed~') {
						this.startMenuItem.setDisabled(false);
						this.stopMenuItem.setDisabled(false);
						this.statusMenuItem.setDisabled(false);
					}
					else if (this.selectedItem.type == 'bundleStatus') {
						this.startMenuItem.setDisabled(false);
						this.stopMenuItem.setDisabled(false);
						this.statusMenuItem.setDisabled(false);
					}
					else if (this.selectedItem.type == 'installedFeature' || 
					         this.selectedItem.type == 'availableFeature' ||
					         this.selectedItem.type == 'subFeature') {
						if (this.selectedItem.children == undefined) {
							var prh = new com.ibm.team.repository.web.transport.internal.ProvisionResponseHandler(this, "loadStatus", "requestFailed");
							ProvisioningClient.getStatus(false, false, false, this.selectedItem.id, this.selectedItem.version, prh);
						}
						if (this.selectedItem.type == 'installedFeature') {
							this.startMenuItem.setDisabled(false);
							this.stopMenuItem.setDisabled(false);
							this.removeFeatureMenuItem.setDisabled(false);
						}
						else if (this.selectedItem.type == 'availableFeature') {
							this.installFeatureMenuItem.setDisabled(false);
							this.installAndStartFeatureMenuItem.setDisabled(false);
						}
					}
				}
			},
			
			showConnectDialog: function() {
				var workbenchNode = dojo.byId("workbench");
				var connectWidget = new com.ibm.team.repository.web.ui.internal.provision.ConnectWidget(null, workbenchNode);
				var dialogWidget = dijit.byId("com.ibm.team.repository.web.ui.internal.provision.ConnectWidgetDialog");
				if (dialogWidget == null) { 
					dialogWidget = new dijit.Dialog({id:"com.ibm.team.repository.web.ui.internal.provision.ConnectWidgetDialog", 
				                                     title: "Connect",
						execute: dojo.hitch(this, function(dialogArgs) {
							this.connectAction(dialogArgs.url, dialogArgs.userid, dialogArgs.password);	
						})
					}, connectWidget.domNode);
				}
				dialogWidget.show();
			},
			
			connectAction: function(url, userid, password) {
				var prh = new com.ibm.team.repository.web.transport.internal.ProvisionResponseHandler(this, "connected", "requestFailed");
				ProvisioningClient.connect(url, userid, password, prh);
			},
			
			connected: function(response, args) {
				if (response.error != undefined) {
					this.provStatus.setStatusMsg("Connection to ["+args[0]+"] failed");
					this.provStatus.setOutput(response.error);
				}
				else { 
					this.provStatus.setOutput("Connected to ["+args[0]+"]");
					var prh = new com.ibm.team.repository.web.transport.internal.ProvisionResponseHandler(this, "loadStatus", "requestFailed");
					ProvisioningClient.getStatus(false, false, true, null, null, prh);
				}
			},
						
			disconnectAction: function() {
				var prh = new com.ibm.team.repository.web.transport.internal.ProvisionResponseHandler(this, "disconnected", "requestFailed");
				ProvisioningClient.disconnect(prh);
			},
			
			disconnected: function(response, args) {
				if (response.error != undefined) {
					this.provStatus.setStatusMsg("Failed to disconnect");
					this.provStatus.setOutput(response.error);
				}
				else { 
					this.connectionURL = "disconnected";
					var self = this;
					var deleteAvailable = function(items, request) {
						self.treeStore.setValue(self.availableItem, "name", "Available Jazz Components [disconnected]");
						for (var i = 0; i < items.length; i++){
							self.deleteItem(items[i]);
						}
						self.saveTreeStore();
					}
					this.treeStore.fetch({query: {type:"availableFeature"}, queryOptions: {deep: true}, onComplete: deleteAvailable});
				}
			},
			
			saveTreeStore: function() {
				if (this.treeStore.isDirty()) {
					this.treeStore.save({scope: this, onComplete: this.onTreeStoreSaveComplete, onError: this.onTreeStoreSaveError});
				}
			},
			
			onTreeStoreSaveComplete: function() {
			},
			
			onTreeStoreSaveError: function(error) {
				console.debug("onTreeStoreSaveError ["+error+"]");
			},
			
			deleteItem: function(item) {
				if (item.children != undefined) {
					for (var i = 0; i < item.children.length; i++) {
						this.deleteItem(item.children[i]);
					}
				}
				this.treeStore.deleteItem(item);
			},
			
			installAction: function() {
				this.installFeature(false);	
			},
			
			installAndStartAction: function() {
				this.installFeature(true);	
			},
			
			installFeature: function(start) {
				this.provStatus.setStatusMsg("Installing feature ["+this.selectedItem.name+"].....");
				var prh = new com.ibm.team.repository.web.transport.internal.ProvisionResponseHandler(this, "featureInstalled", "requestFailed");
				ProvisioningClient.installFeature(this.selectedItem.featureId, this.selectedItem.version, start, prh);
			},
			
			featureInstalled: function(response, args) {
				if (response.error != undefined) {
					this.provStatus.setStatusMsg("Install of feature ["+this.selectedItem.name+"] failed");
					this.provStatus.setOutput(response.error);
				}
				else { 
					this.provStatus.setStatusMsg("Install of feature ["+this.selectedItem.name+"] complete");
					this.provStatus.setOutput(response.output);
					var self = this;
					var deleteFeature = function(item) {
						self.deleteItem(item);
						self.saveTreeStore();
						var pInfo = {parent: self.installedItem, attribute: "children"};
						var newItem = {id: item.id[0], 
						               name: item.name[0], 
						               type: "installedFeature", 
						               icon: "iconFeatureObj",
						               featureId: item.featureId[0],
						               version: item.version[0]};
						self.treeStore.newItem(newItem, pInfo);
						self.saveTreeStore();
					}
					this.treeStore.fetchItemByIdentity({identity: this.selectedItem.id, onItem: deleteFeature});
					if (response.dependencyFeatures != undefined) {
						for (var i = 0; i < response.dependencyFeatures.length; i++) {
							var id = "available~"+response.dependencyFeatures[i].id + "_" + response.dependencyFeatures[i].version;
							this.treeStore.fetchItemByIdentity({identity: id,
							                                    onItem: deleteFeature}); 	
						}
					}
					this.checkForRepository();
				}
			},
			
			updateAction: function() {
			},
			
			removeAction: function() {
				if (this.selectedItem.id == "installed~") {
					this.removeAll();
				}
				else if (this.selectedItem.type == "installedFeature") {
					this.removeFeature(this.selectedItem.featureId, this.selectedItem.version);	
				}
			},
			
			startAction: function() {
				if (this.selectedItem.type == "bundleStatus") {
					this.startBundle(this.selectedItem.id, this.selectedItem.name);
				}
				else if (this.selectedItem.type == "installedFeature") {
					this.startFeature(this.selectedItem.featureId, this.selectedItem.version);	
				}
				else if (this.selectedItem.id == "installed~") {
					this.startAll();
				}
				this.checkForRepository();
			},
			
			stopAction: function() {
				if (this.selectedItem.type == "bundleStatus") {
					this.stopBundle(this.selectedItem.id, this.selectedItem.name);
				}
				else if (this.selectedItem.type == "installedFeature") {
					this.stopFeature(this.selectedItem.featureId, this.selectedItem.version);	
				}
				else if (this.selectedItem.id == "installed~") {
					this.stopAll();
				}
				this.checkForRepository();
			},
			
			statusAction: function() {
				if (this.selectedItem.type == "bundleStatus") {
					var prh = new com.ibm.team.repository.web.transport.internal.ProvisionResponseHandler(this, "displayBundleStatus", "requestFailed");
					ProvisioningClient.getBundleStatus(this.selectedItem.id, prh);
				}
				else if (this.selectedItem.id == "installed~") {
					var prh = new com.ibm.team.repository.web.transport.internal.ProvisionResponseHandler(this, "displayInstallLog", "requestFailed");
					ProvisioningClient.getInstallLog(prh);
				}
				this.checkForRepository();
			},
			
			displayInstallLog: function(response, args) {
				this.provStatus.setStatusMsg("Profile Install Log");
				this.provStatus.setOutput(response.installLogStatus.log);	
			},
			
			displayBundleStatus: function(response, args) {
				this.provStatus.setStatusMsg("Bundle Status ["+response.bundleStatus.name+"]");
				this.provStatus.setOutput("Location: ["+response.bundleStatus.location+"]");
				this.provStatus.appendOutput("Dataroot: ["+response.bundleStatus.dataroot+"]");
				this.provStatus.appendOutput("Registered Services:", "label");
				for (var i = 0; i < response.bundleStatus.registeredServices.length; i++) {
					this.provStatus.appendOutput(response.bundleStatus.registeredServices[i].serviceName);	
				}
				this.provStatus.appendOutput("Services in use:", "label");
				for (var i = 0; i < response.bundleStatus.servicesInUse.length; i++) {
					this.provStatus.appendOutput(response.bundleStatus.servicesInUse[i].serviceName);	
				}
				this.provStatus.appendOutput("Exported Packages:", "label");
				for (var i = 0; i < response.bundleStatus.exportedPackages.length; i++) {
					this.provStatus.appendOutput(response.bundleStatus.exportedPackages[i].packageName);	
				}
				this.provStatus.appendOutput("Imported Packages:", "label");
				for (var i = 0; i < response.bundleStatus.importedPackages.length; i++) {
					this.provStatus.appendOutput(response.bundleStatus.importedPackages[i].packageName);	
				}
				this.provStatus.appendOutput("Required Bundles:", "label");
				for (var i = 0; i < response.bundleStatus.requiredBundles.length; i++) {
					this.provStatus.appendOutput(response.bundleStatus.requiredBundles[i].bundleName);	
				}
			},
						
			resetAction: function() {
				var prh = new com.ibm.team.repository.web.transport.internal.ProvisionResponseHandler(this, "displayResetStatus", "requestFailed");
				ProvisioningClient.requestReset(prh);
			},

			displayResetStatus: function(response, args) {
				this.provStatus.setStatusMsg("Reset requested");
				this.provStatus.setOutput(response.output);
			},
			
			startBundle: function(id , name) {
				this.provStatus.setStatusMsg("Starting bundle ["+name+"].....");
				var prh = new com.ibm.team.repository.web.transport.internal.ProvisionResponseHandler(this, "updateBundleStatus", "requestFailed");
				ProvisioningClient.startBundle(id, prh);
			},
			
			stopBundle: function(id , name) {
				this.provStatus.setStatusMsg("Stopping bundle ["+name+"].....");
				var prh = new com.ibm.team.repository.web.transport.internal.ProvisionResponseHandler(this, "updateBundleStatus", "requestFailed");
				ProvisioningClient.stopBundle(id, prh);
			},
			
			updateBundleStatus: function(response, args) {
				var name = response.bundleStatus.name;
				this.provStatus.setStatusMsg("Bundle ["+name+"] status updated");
				this.provStatus.setOutput(response.output);
				var icon = this.getBundleIcon(response.bundleStatus.state);
				var state = this.getBundleState(response.bundleStatus.state);
				var self = this;
				var setBundleStatus = function(item) {
					var oldIcon = self.treeStore.getValue(item, "icon");
					self.treeStore.setValue(item, "icon", icon);
					self.treeStore.setValue(item, "name", name+" ["+state+"]");
					self.saveTreeStore();
					dojo.removeClass(self.selectedNode.iconNode, oldIcon);	
					dojo.addClass(self.selectedNode.iconNode, icon);	
				}
				this.treeStore.fetchItemByIdentity({identity: args[0], onItem: setBundleStatus});
				this.checkForRepository();
			},
			
			startFeature: function(featureId, version) {
				this.provStatus.setStatusMsg("Starting Jazz Component ["+featureId+"_"+version+"].....");
				var prh = new com.ibm.team.repository.web.transport.internal.ProvisionResponseHandler(this, "featureStarted", "requestFailed");
				ProvisioningClient.startFeature(featureId, version, prh);
			},
			
			featureStarted: function(response, args) {
				this.provStatus.setStatusMsg("Start of Jazz Component complete");
				if (response.output)
					this.provStatus.setOutput(response.output);
				if (response.bundleStatusArray) {
					for (var i = 0; i < response.bundleStatusArray.length; i++) {
						this.provStatus.appendOutput(response.bundleStatusArray[i].msg);	
					}
				}
			},
			
			stopFeature: function(featureId, version) {
				this.provStatus.setStatusMsg(msg + "Stopping Jazz Component ["+featureId+"].....");
				var prh = new com.ibm.team.repository.web.transport.internal.ProvisionResponseHandler(this, "featureStopped", "requestFailed");
				ProvisioningClient.stopFeature(featureId, version, prh);
			},
			
			featureStopped: function(response, args) { 
				this.provStatus.setStatusMsg("Stop of Jazz Component complete");
				this.provStatus.setOutput(response.output);
			},
			
			startAll: function() {
				this.provStatus.setStatusMsg("Starting all Jazz Components.....");
				var prh = new com.ibm.team.repository.web.transport.internal.ProvisionResponseHandler(this, "startAllComplete", "requestFailed");
				ProvisioningClient.startAll(prh);
			},
			
			startAllComplete: function(response, args) {
				this.provStatus.setStatusMsg("Start of all Jazz Components complete");
				this.provStatus.setOutput(response.output);
			},
			
			stopAll: function() {
				this.provStatus.setStatusMsg("Starting all Jazz Components.....");
				var prh = new com.ibm.team.repository.web.transport.internal.ProvisionResponseHandler(this, "startAllComplete", "requestFailed");
				ProvisioningClient.stopAll(prh);
			},
			
			stopAllComplete: function(response, args) {
				this.provStatus.setStatusMsg("Stop of all Jazz Components complete");
				this.provStatus.setOutput(response.output);
			},
			
			removeFeature: function(featureId, version) {
				this.provStatus.setStatusMsg("Removing Jazz Component ["+featureId+"_"+version+"].....");
				var prh = new com.ibm.team.repository.web.transport.internal.ProvisionResponseHandler(this, "removeFeatureComplete", "requestFailed");
				ProvisioningClient.removeFeature(featureId, version, prh);
			},
			
			removeFeatureComplete: function(response, args) {
				if (response.error != undefined) {
					this.provStatus.setStatusMsg("Removal of Jazz Component ["+this.selectedItem.name+"] failed");
					this.provStatus.setOutput(response.error);
				}
				else { 
					this.provStatus.setStatusMsg("Removal of Jazz Component ["+this.selectedItem.name+"] complete");
					this.provStatus.setOutput(response.output);
					var self = this;
					var deleteFeature = function(item) {
						self.deleteItem(item);
						self.saveTreeStore();
						if (self.connectionURL != "disconnected") {
							var pInfo = {parent: self.availableItem, attribute: "children"};
							var newItem = {id: item.id[0], 
							               name: item.name[0], 
							               type: "availableFeature", 
							               icon: "iconFeatureObj",
							               featureId: item.featureId[0],
							               version: item.version[0]};
							self.treeStore.newItem(newItem, pInfo);
							self.saveTreeStore();
						}
					}
					this.treeStore.fetchItemByIdentity({identity: this.selectedItem.id, onItem: deleteFeature});
					if (response.dependencyFeatures != undefined) {
						for (var i = 0; i < response.dependencyFeatures.length; i++) {
							var id = "available~"+response.dependencyFeatures[i].id + "_" + response.dependencyFeatures[i].version;
							this.treeStore.fetchItemByIdentity({identity: id,
							                                    onItem: deleteFeature}); 	
						}
					}
					this.checkForRepository();
				}
			},
			
			removeAll: function() {
				this.provStatus.setStatusMsg("Removing all Jazz Components.....");
				var prh = new com.ibm.team.repository.web.transport.internal.ProvisionResponseHandler(this, "removeAllComplete", "requestFailed");
				ProvisioningClient.removeAll(prh);
			},
			
			removeAllComplete: function(response, args) {
				this.provStatus.setStatusMsg("Removal of all Jazz Components complete");
				this.provStatus.setOutput(response.output);
			},
			
			checkForRepository: function() {
				var prh = new com.ibm.team.repository.web.transport.internal.ProvisionResponseHandler(this, "updateRepositoryStatus", "requestFailed");
				ProvisioningClient.isRepositoryAvailable(prh);
			},
			
			updateRepositoryStatus: function(response, args) {
				dojo.publish(com.ibm.team.repository.web.isRepositoryAvailable, [response.repositoryAvailable]);					
			},
			
			getBundleIcon: function(state) {
				switch (state) {
					case 0x20:
						iconAsString = "iconActive";
						break;
					case 0x02:
						iconAsString = "iconInstalled";
						break;
					case 0x04:
						iconAsString = "iconResolved";
						break;
					case 0x08:
						iconAsString = "iconStarting";
						break;
					case 0x10:
						iconAsString = "iconStopping";
						break;
					default:
						iconAsString = "iconBundle";
						break;				
				}
				return iconAsString;
			},
			
			getBundleState: function(state) {
				var stateAsString = "UNKNOWN";
				switch (state) {
					case 0x20:
						stateAsString = "ACTIVE";
						break;
					case 0x01:
						stateAsString = "UNINSTALLED";
						break;
					case 0x02:
						stateAsString = "INSTALLED";
						break;
					case 0x04:
						stateAsString = "RESOLVED";
						break;
					case 0x08:
						stateAsString = "STARTING";
						break;
					case 0x10:
						stateAsString = "STOPPING";
						break;
				}
				
				return stateAsString;
			}
		});
 })();				