/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.settings.ViewletSettingsPanel");

dojo.require("jazz.ensemble.internal.settings.SettingsPanel");

dojo.require("com.ibm.team.dashboard.web.util.internal.ObfUtil");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ItemNameRegistry");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

var SettingsPanel = jazz.ensemble.internal.settings.SettingsPanel;
var ItemNameRegistry = com.ibm.team.dashboard.web.ui.internal.util.ItemNameRegistry;
var ObfUtil = com.ibm.team.dashboard.web.util.internal.ObfUtil;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;

var messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

var TYPES = dojo.clone(SettingsPanel.TYPES);
dojo.mixin(TYPES, {
	CUSTOM: "CUSTOM",
	PASSWORD: "PASSWORD",
	COMPUTED: "COMPUTED"
});

var PREFERENCE_TYPE_MAP = {};
PREFERENCE_TYPE_MAP[Constants.SettingsArea.FIELD_PASSWORD] = TYPES.PASSWORD;
PREFERENCE_TYPE_MAP[Constants.SettingsArea.FIELD_CUSTOM] = TYPES.CUSTOM;
PREFERENCE_TYPE_MAP[Constants.SettingsArea.FIELD_LIST] = TYPES.LIST;
PREFERENCE_TYPE_MAP[Constants.SettingsArea.FIELD_COMPUTED] = TYPES.COMPUTED;
PREFERENCE_TYPE_MAP[Constants.SettingsArea.FIELD_BOOLEAN] = TYPES.BOOLEAN;
PREFERENCE_TYPE_MAP[Constants.SettingsArea.FIELD_STRING] = TYPES.STRING;
PREFERENCE_TYPE_MAP[Constants.SettingsArea.FIELD_INTEGER] = TYPES.STRING;
PREFERENCE_TYPE_MAP[Constants.SettingsArea.FIELD_HIDDEN] = TYPES.HIDDEN;

dojo.declare("com.ibm.team.dashboard.web.ui.internal.settings.ViewletSettingsPanel",jazz.ensemble.internal.settings.SettingsPanel,{
	
	context: null,
	scopePreferenceProviderClass: null,
	viewlet: null,
	viewletAdapter: null,
	viewletDefinition: null,
	
	postCreate: function() {
		this._customPreferences = {};
		this._computedCombos = [];
		this.init();
		this.inherited(arguments);
	},
	
	onOk: function() {
		var changes = {}, hasChanges = false;
		for (var i = 0; i < this.settings.length; i++) {
			var setting = this.settings[i];
			var oldSetting = this.oldSettings[i];
			// N-1 compatibility; old scope pref provider still uses "scope"
			if (setting.id === "_scope" || setting.id === "scope") {
				changes._scopeItem = setting.value.itemId;
				changes._scope = setting.value.type;
				if (setting.value.itemId !== oldSetting.value.itemId) {
					// make sure the scope item name is always available (viewlets
					// expect it from getScopeItem().name)
					if (!ItemNameRegistry.getName(setting.value.itemId)) {
						// must be a remote viewlet; query the ItemNameRegistry
						// inside the iframe to get the name
						var w = this.viewletAdapter.getWindow();
						if (w) {
							var ItemNameRegistry2 = dojo.getObject("com.ibm.team.dashboard.web.ui.internal.util.ItemNameRegistry", false, w);
							if (ItemNameRegistry2 && ItemNameRegistry2.getName) {
								var name = ItemNameRegistry2.getName(setting.value.itemId);
								if (name) {
									ItemNameRegistry.addName({
										itemId: setting.value.itemId,
										name: name
									});
								}
							}
						}
					}
					hasChanges = true;
				}
			} else {
				changes[setting.id] = setting.value || "";
				if (setting.value !== oldSetting.value) {
					hasChanges = true;
				}
			}
		}
		if (hasChanges) {
			// preserve preferences not in the settings area (hidden prefs)
			this.context.setPreferences(dojo.mixin(this.context.getPreferences(),changes));
			this.viewlet.settingsChanged(this.oldWorkingCopy, this.workingCopy);
		}
		this.onCancel();
	},
	
	init: function() {
		var prefs = this.context.getPreferences();
		var prefDefs = this.viewletDefinition.preferenceDefinitions;
		this.workingCopy = { title: this.context.getTitle() };	// used to pass to Viewlet APIs in the format they expect
		this.settings = [];		// used as the model for SettingsPanel
		if (this._hasScope()) {
			var scope = this.viewlet.getScope();
			var scopeItemId = this.viewlet.getScopeItem() ? this.viewlet.getScopeItem().itemId : null;
			this.workingCopy.scope = scope;
			this.workingCopy.scopeItemId = scopeItemId;
			this.settings.push({
				id: "_scope",
				label: this._getScopeLabel(),
				type: TYPES.CUSTOM,
				required: true,
				value: {
					itemId: scopeItemId,
					type: scope
				}
			});
			if (this._hasScopeSubteams()) {
				var scopeSubteams = this.viewlet.getScopeSubteams();
				this.workingCopy.scopeSubteams = scopeSubteams;
				this.settings.push({
					id: "_scopeSubteams",
					label: messages.settingsScopeSub,
					type: TYPES.BOOLEAN,
					description: messages.settingsScopeSubDesc,
					value: scopeSubteams ? "true" : "false" // SettingsPanel expects strings
				});
			}
		}
		this.preferenceDefinitionsMap = {};
		if (this._hasPrefs()) {
			this.workingCopy.preferences = {};
			for (var i = 0; i < prefDefs.length; i++) {
				this.preferenceDefinitionsMap[prefDefs[i].id] = i;
				// use the viewlet to populate the working copy since it will include defaults and will use typed variables
				this.workingCopy.preferences[prefDefs[i].id] = this.viewlet.getPreference(prefDefs[i].id);
				var type = PREFERENCE_TYPE_MAP[prefDefs[i].preferenceType];
				if (!type)
					continue;
				var value = prefs[prefDefs[i].id];
				value = typeof value === "undefined" || value === null ? prefDefs[i].defaultValue : value;
				var setting = {
					id: prefDefs[i].id,
					label: prefDefs[i].name,
					description: prefDefs[i].description,
					type: type,
					required: prefDefs[i].required,
					value: value
				};
				if (type === TYPES.LIST)
					setting.options = prefDefs[i].options;
				this.settings.push(setting);
			}
		}
		this.oldWorkingCopy = dojo.clone(this.workingCopy);
	},
	
	onChange: function(id, value) {
		// N-1 compatibility; old scope pref provider still uses "scope"
		if (id == "_scope" || id == "scope") {
			if (value.itemId != this.workingCopy.scopeItemId) {
				this.workingCopy.scopeItemId = value.itemId;
				id = "scopeItem";
			}
			if (value.type != this.workingCopy.scope) {
				this.workingCopy.scope = value.type;
				id = "scope";
			}
		} else if (id == "_scopeSubteams") {
			this.workingCopy.scopeSubteams = Util._cast(Constants.SettingsArea.FIELD_BOOLEAN, value);
		} else {
			// viewlets expect types preferences, but SettingsPanel works with strings
			var prefDef = this.viewletDefinition.preferenceDefinitions[this.preferenceDefinitionsMap[id]];
			this.workingCopy.preferences[id] = Util._cast(prefDef.preferenceType,value);
		}
		this.settingChanged(id);
	},
	settingChanged: function(id) {
		var prefsToUpdate = this.viewlet.settingChanging(id, this.workingCopy);
		if (prefsToUpdate && prefsToUpdate.length) {
			for (var i = 0; i < prefsToUpdate.length; i++) {
				var prefId = prefsToUpdate[i];
				var def = this.viewletDefinition.preferenceDefinitions[this.preferenceDefinitionsMap[prefId]];
				if (!def) continue;
				var type = PREFERENCE_TYPE_MAP[def.preferenceType];
				if (type === TYPES.COMPUTED) {
					for (var j = 0; j < this._computedCombos.length; j++) {
						var cc = this._computedCombos[j];
						if (cc && prefId == cc.select.name) {
							cc.setting.value = cc.select.options[cc.select.selectedIndex].value;
							cc.field.innerHTML = "";
							this.createFormField(cc.setting, cc.field);
							this._computedCombos.splice(j,1);
							break;
						}
					}
				} else if (type === TYPES.CUSTOM) {
					var value = def.defaultValue;
					this.workingCopy.preferences[prefId] = value;
					this.settings[this.settingsMap[prefId]].value = value;
					this._updateCustomPreferenceLabel(prefId, value);
				}
			}
		}
	},
	
	_getScopeLabel: function() {
		switch (this.viewletDefinition.appliesTo) {
			case Constants.SCOPE_APPLIES_TO_TEAMS:
				return messages.settingsScopeTeam;
			case Constants.SCOPE_APPLIES_TO_CONTRIBUTORS:
				return messages.settingsScopeUser;
			case Constants.SCOPE_APPLIES_TO_PROJECTS:
				return messages.settingsScopeProject;
			case Constants.SCOPE_APPLIES_TO_PROCESS:
				return messages.settingsScopeProcess;
			default:
				return messages.settingsScopeAll;
		}
	},
	_hasScope: function() {
		return this.viewletDefinition.scopeSensitive;
	},
	_hasScopeSubteams: function() {
		return this.viewletDefinition.scopeSubteams == "yes" || this.viewletDefinition.scopeSubteams == "yes-and-check";
	},
	_hasPrefs: function() {
		var defs = this.viewletDefinition.preferenceDefinitions;
		if (!defs || defs.length==0) return false;

		// test for hidden
		for (var i=0; i<defs.length; i++) {
			if (defs[i].preferenceType!=Constants.SettingsArea.FIELD_HIDDEN)
				return true;
		}
		return false;
	},
	
	createFormField: function(setting, field) {
		switch(setting.type) {
		case TYPES.PASSWORD:
			var inputDiv = dojo.create("div",null,field);
			var textBox = new dijit.form.TextBox({type: "password",name: setting.id, value: setting.value, trim: true, id: this.id + "-" + setting.id});
			textBox.startup();
			textBox.domNode.className += " string";
			if (this.readOnly || setting.startDisabled)
				textBox.domNode.disabled = true;
			this.connect(textBox.domNode, "onchange", this._handlePasswordChange);
			// workaround for dojo bug: onchange not triggering on blur
			this.connect(textBox.domNode, "onblur", this._handlePasswordChange);
			inputDiv.appendChild(textBox.domNode);
			return textBox.domNode;
		case TYPES.CUSTOM:
			var inputDiv = dojo.create("div",{className: "custom-container"},field);
			var buttonSpan = dojo.create("span",{className: "custom-edit-button"},inputDiv);
			var label = dojo.create("span",{className: "custom-label"},inputDiv);
			if (!(setting.id in this._customPreferences))
				this._customPreferences[setting.id] = {};
			this._customPreferences[setting.id].label = label;
			this._updateCustomPreferenceLabel(setting.id, setting.value);
			var button = dojo.create("button",{name: setting.id, id: this.id + "-" + setting.id, innerHTML: messages.settingsPrefsEdit},buttonSpan);
			this.connect(button,"onclick", this._showCustomPreferenceChooser);
			if (this.readOnly || setting.startDisabled)
				button.disabled = true;
			return button;
		case TYPES.COMPUTED:
			var result = this.viewlet.getPreferenceOptions(setting.id, this.workingCopy);
			if (result) {
				if (result.computedDefault)
					setting.value = result.computedDefault;
				if (result.options)
					setting.options = result.options;
				else if (result.groups)
					setting.groups = result.groups;
				setting.type = TYPES.LIST;
				var select = this.inherited(arguments);
				setting.type = TYPES.COMPUTED;
				this._computedCombos.push({
					select: select,
					setting: setting,
					field: field
				});
				var value = select.options[select.selectedIndex].value;
				this.workingCopy.preferences[setting.id] = value;
				setting.value = value;
				return select;
			} else {
				var inputDiv = dojo.create("div", null, field);
				var select = dojo.create("select", {disabled: true}, inputDiv);
				var option = dojo.create("option", null, select);
				option.appendChild(document.createTextNode(messages.settingsLoadingOpts));
				setTimeout(dojo.hitch(this, function() {
					field.innerHTML = "";
					this.createFormField(setting, field);
					}), 250);
				return select;
			}
		}
		return this.inherited(arguments);
	},
	_showCustomPreferenceChooser: function(event){
		dojo.stopEvent(event);
		
		this._getPreferenceProvider(event.target.name).createChooser(
			dojo.hitch(this,this._handleCustomChange), 
			this.workingCopy,
			this.viewlet);
	},
	_updateCustomPreferenceLabel: function(id, value) {
		if (this._customPreferences[id]) {
			if (this._customPreferences[id].timeout)
				clearTimeout(this._customPreferences[id].timeout);
			var labelHTML = this._getPreferenceProvider(id).getLabel(value);
			if (!labelHTML) {
				labelHTML = this._getPreferenceProvider(id).getLoadingLabel();
				this._customPreferences[id].timeout = setTimeout(dojo.hitch(this, function() {
					this._updateCustomPreferenceLabel(id, value);
					}), 250);
			}

			var customElement = this._customPreferences[id].label; 
			customElement.innerHTML = labelHTML;
			
			if (this.readOnly) {
				// Any HTML snippet can be supplied by the custom preference provider
				// Disable all 'input' elements if the provided preference includes any
				dojo.query('input', customElement).forEach(
					function(inputElements){
						inputElements.disabled = 'disabled';
					}
				)
			}
		}
	},
	_getPreferenceProvider: function(id) {
		if (id == "_scope") {
			if (!this._scopeProvider) {
				// ask the viewlet for its scope preference provider
				// (handles remote viewlet case properly)
				var constructor = this.scopePreferenceProviderClass;
				this._scopeProvider = new constructor({
					appliesTo: this.viewletDefinition.appliesTo,
					preferenceId: "_scope"
				});
			}
			return this._scopeProvider;
		}
		return this.viewlet.getPreferenceProvider(id);
	},
	_handlePasswordChange: function(event) {
		var box = event.target;
		var value = ObfUtil.menc(box.value);
		this.settings[this.settingsMap[box.name]].value = value;
		this.onChange(box.name, value);
	},
	_handleCustomChange: function(value, id) {
		// N-1 compatibility; old scope pref provider still uses "scope"
		if (id == "scope") {
			id = "_scope";
		}
		this._updateCustomPreferenceLabel(id, value);
		this.settings[this.settingsMap[id]].value = value;
		this.onChange(id, value);
	}
	
});
com.ibm.team.dashboard.web.ui.internal.settings.ViewletSettingsPanel.TYPES = TYPES;

})();
