/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.util.IFrameContainer");

dojo.require("com.ibm.team.dashboard.web.util.internal.Util");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.app.proxy");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

var Util = com.ibm.team.dashboard.web.util.internal.Util;

dojo.declare("com.ibm.team.dashboard.web.ui.internal.util.IFrameContainer",
	[dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/util/IFrameContainer.html"),
	
	active: false,
	args: null,
	className: null,
	onLoad: null,
	pageNode: null,
	serviceRoot: null,
	webUIRoot: null,
	
	_iframe: null,
	_inner: null,
	_proxiedWebUIRoot: null,
	_proxiedServiceRoot: null,
	
	constructor: function() {
		this._messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
	},
	
	postCreate: function() {
		this._proxiedWebUIRoot = Util.getContextPath() + "/_iframeproxy/" + encodeURIComponent(encodeURIComponent(this.webUIRoot));
		this._proxiedServiceRoot = Util.getContextPath() + "/_iframeproxy/" + encodeURIComponent(encodeURIComponent(this.serviceRoot));
		this._iframe = document.createElement("iframe");
		this._iframe.src = this._proxiedWebUIRoot + "/_ajax-modules/com.ibm.team.dashboard.web.iframe/" + this.className + "?_proxyURL=" + encodeURIComponent(this._proxiedWebUIRoot) + (djConfig.isDebug ? "&debug=true" : "");
		this._iframe.onCreate = dojo.hitch(this, this._onCreate);
		this._iframe.frameBorder = "0";
		this._iframe.scrolling = "no";
		this._iframe.title = this._messages.iframeTitle;
		this._iframe.allowTransparency = "true";
		this._iframeDiv.appendChild(this._iframe);
	},
	
	uninitialize: function() {
		if (this._iframeHandle) {
			try {
				dijit.unregisterIframe(this._iframeHandle);
				delete this._iframeHandle;
			} catch (e) {
				//eat the error - this can throw exceptions in certain cases
			}
		}
	},
	/*
	 * Returns the iframe node contained in this widget.
	 */
	getIFrame: function() {
		return this._iframe;
	},
	
	/*
	 * Returns the class that implements CustomPreferenceProvider for the viewlet
	 * scope. This is exposed in order to allow the implementation class to
	 * be moved or renamed.
	 */
	/* CustomPreferenceProvider */ getScopePreferenceProviderClass: function() {
		return this._inner.getScopePreferenceProviderClass();
	},
	
	/*
	 * Sets whether or not the container is currently active (visible to the
	 * user on the current page).
	 */
	setActive: function(/* boolean */ value) {
		if (value != this.active) {
			try {
				this._inner.setActive(value);
			}
			catch (/* Error */ e) {
				// iframe may be in the process of reloading; ignore
			}
			this.active = value;
		}
	},
	
	_encodeProxiedURI: function(/* string */ uri) {
		// use the iframe proxy in these cases to support relative URIs
		if (uri.indexOf(this.webUIRoot) == 0) {
			return this._proxiedWebUIRoot + uri.substring(this.webUIRoot.length);
		}
		if (this.serviceRoot && (uri.indexOf(this.serviceRoot) == 0)) {
			return this._proxiedServiceRoot + uri.substring(this.serviceRoot.length);
		}
		// don't use the local dashboard server's proxy to talk to itself
		var localUrl = new dojo._Url(location.href);
		var path = localUrl.path;
		if (path.length > 0) {
			path = path.substring(0,path.substring(1).indexOf("/")+2);
		}
		localUrl = localUrl.scheme + "://" + localUrl.authority + path;
		if (uri.indexOf(localUrl) === 0)
			return uri;
		// this uses the local dashboard's proxy... we really shouldn't do this, but double proxying doesn't work
		return jazz.app.proxy.encodeProxiedURI(uri);
	},
	
	_onCreate: function(/* IFramedInnerContainer */ inner) {
		this._iframeHandle = dijit.registerIframe(this._iframe);
		// initialize iframe environment
		this._inner = inner;

		//clobber previous unload event if iframe refreshed without firing unload event (IE)
		if (this._unloadEvent) dojo.disconnect(this._unloadEvent);

		//Register onunload event to clean up iframe events when iframe is reloaded
		//without being destroyed.
		this._unloadEvent = dojo.connect(this._iframe.contentWindow, "onunload", this, function() {
			if (this._iframeHandle) {
				dijit.unregisterIframe(this._iframeHandle);
				delete this._iframeHandle;
			}
			dojo.disconnect(this._unloadEvent);
			delete this._unloadEvent;
		});

		this._inner.init({
			active: this.active,
			args: this.args,
			encodeProxiedURI: dojo.hitch(this, this._encodeProxiedURI),
			onBustOut: dojo.hitch(this, this._onBustOut),
			onBustIn: dojo.hitch(this, this._onBustIn),
			onLoad: dojo.hitch(this, this._onLoad),
			onResize: dojo.hitch(this, this._onResize),
			serviceRoot: this.serviceRoot,
			webUIRoot: this.webUIRoot
		});

		// hide the loading text and show the iframe in one atomic step
		this._container.className = "loaded";
	},
	
	_onLoad: function(/* Widget */ widget) {
		this.onLoad(widget);
	},
	
	_onBustOut: function() {
		// fix the iframeDiv to specific size to preserve layout
		var iframeDiv = this._iframeDiv;
		var iframeDivCoords = dojo.coords(iframeDiv);
		iframeDiv.style.width = iframeDivCoords.w + "px";
		iframeDiv.style.height = iframeDivCoords.h + "px";
		
		// fix contents to specific size so it doesn't grow with the iframe
		var innerCoords = this._iframe.contentWindow.dojo.coords(this._inner.domNode);
		this._inner.domNode.style.width = innerCoords.w + "px";
		this._inner.domNode.style.height = innerCoords.h + "px";
		
		// set iframe z-index
		this._iframe.style.zIndex = "500";
		
		// grow the iframe to full window
		this._updateFullPageSize();
		
		// turn off overflow: hidden/auto and push up z-index all the way up to the styled box
		this._overflowNodes = [];
		this._zIndexNodes = [];
		var node = this._iframeDiv;
		while (node) {
			var overflow = dojo.style(node, "overflow");
			var overflowX = dojo.style(node, "overflowX");
			var overflowY = dojo.style(node, "overflowY");
			if (overflow != "visible" || overflowX != "visible" || overflowY != "visible") {
				var entry = {node: node};
				if (overflow != "visible") {
					entry.overflow = node.style.overflow;
					node.style.overflow = "visible";
				}
				else {
					if (overflowX != "visible") {
						entry.overflowX = node.style.overflowX;
						node.style.overflowX = "visible";
					}
					if (overflowY != "visible") {
						entry.overflowY = node.style.overflowY;
						node.style.overflowY = "visible";
					}
				}
				this._overflowNodes.push(entry);
			}
			
			this._zIndexNodes.push({
				node: node,
				zIndex: node.style.zIndex
			});
			node.style.zIndex = "500";
			if (node.parentNode && node.parentNode == this.pageNode) {
				break;
			}
			node = node.parentNode;
		}
		
		// enable full-page auto-size
		this._updateFullPageHeightHandle = this.connect(window, "onresize", this._updateFullPageSize);
	},
	
	_onBustIn: function() {
		// disable full-page auto-size
		this.disconnect(this._updateFullPageHeightHandle);
		
		// reset overflows and zIndex on container nodes
		for (var i=0;i<this._overflowNodes.length;i++) {
			var entry = this._overflowNodes[i];
			if (typeof entry.overflow != "undefined") {
				entry.node.style.overflow = entry.overflow;
			}
			if (typeof entry.overflowX != "undefined") {
				entry.node.style.overflowX = entry.overflowX;
			}
			if (typeof entry.overflowY != "undefined") {
				entry.node.style.overflowY = entry.overflowY;
			}
		}
		for (var i = 0; i < this._zIndexNodes.length; i++) {
			var entry = this._zIndexNodes[i];
			entry.node.style.zIndex = entry.zIndex;
		}
		
		// reset iframe back to normal size
		this._iframe.style.position = "";
		this._iframe.style.margin = "";
		this._iframe.style.width = "";
		this._iframe.style.height = "";
		this._inner.domNode.style.padding = "";
		
		// reset iframe z-index
		this._iframe.style.zIndex = "";
		
		// reset content size
		this._inner.domNode.style.width = "100%";
		this._inner.domNode.style.height = "";
		
		// reset container size
		this._iframeDiv.style.width = "";
		this._iframeDiv.style.height = "";
	},
	
	_onResize: function(/* int */ height) {
		this._iframeDiv.style.height = height + "px";
	},
	
	_updateFullPageSize: function() {
		// calculate how much iframe has to grow in each direction to fill page
		var v = dojo.coords(this._iframeDiv, true); // viewlet coords
		var p = dojo.coords(this.pageNode, true);  // page coords
		var vp = dijit.getViewport(); // viewport coords
		vp = {l: 0, t: 0, w: vp.w, h: vp.h, x: vp.l, y: vp.t};  // normalize
		
		/*
		 * Bust-out bounds is the overlap (intersection) of page and viewport, but
		 * must always contain (union) the entire viewlet. This prevents negative values
		 * for the padding, which breaks the padding rule and the consequently the layout.
		 */
		var bustoutRect = _union(v, _intersection(p, vp));
		var top = v.y - bustoutRect.y;
		var right = (bustoutRect.x + bustoutRect.w) - (v.x + v.w);
		var bottom = (bustoutRect.y + bustoutRect.h) - (v.y + v.h);
		var left = v.x - bustoutRect.x;
		
		// grow the iframe to fill bustout rectangle
		this._iframe.style.position = "absolute";
		this._iframe.style.margin = (-top) + "px " + (-right) + "px " + (-bottom) + "px " + (-left) + "px";
		this._iframe.style.width = bustoutRect.w + "px";
		this._iframe.style.height = bustoutRect.h + "px";
		this._inner.domNode.style.padding = top + "px " + right + "px " + bottom + "px " + left + "px";
	}
});

/*
 * Returns a new rectangle representing the intersection of the two given rectangle objects.
 * Rectangles are objects with these properties:
 * - w: width (must be positive)
 * - h: height (must be positive)
 * - x: x coordinate
 * - y: y coordinate
 */
/* object */ function _intersection(/* object */ rect1, /* object */ rect2) {
	var r = Math.min(rect1.x + rect1.w, rect2.x + rect2.w);
	var b = Math.min(rect1.y + rect1.h, rect2.y + rect2.h);
	var x = Math.max(rect1.x, rect2.x);
	var y = Math.max(rect1.y, rect2.y);
	return {
		x: x,
		y: y,
		w: r - x,
		h: b - y
	};
}

/*
 * Returns a new rectangle representing the union of the two given rectangles. This
 * is the smallest rectangle that contains both given rectangles. Rectangles are
 * objects with these properties:
 * - w: width (must be positive)
 * - h: height (must be positive)
 * - x: x coordinate
 * - y: y coordinate
 */
/* object */ function _union(/* object */ rect1, /* object */ rect2) {
	var r = Math.max(rect1.x + rect1.w, rect2.x + rect2.w);
	var b = Math.max(rect1.y + rect1.h, rect2.y + rect2.h);
	var x = Math.min(rect1.x, rect2.x);
	var y = Math.min(rect1.y, rect2.y);
	return {
		x: x,
		y: y,
		w: r - x,
		h: b - y
	};
}

})();
