/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.util.IFrameInnerContainer");

dojo.require("com.ibm.team.dashboard.web.ui.internal.settings.ScopePreferenceProvider");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("dojo.i18n");

dojo.require("jazz.app.internal.ProxyRequestInterceptor");
dojo.require("jazz.app.proxy");
dojo.require("jazz.client.xhr");
dojo.require("jazz.core.loader");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "IFrameInnerContainerMessages");

(function() {

var CONTRIBUTOR_SERVICE_PATH = "/service/com.ibm.team.repository.common.internal.IContributorRestService/currentContributor";

var ScopePreferenceProvider = com.ibm.team.dashboard.web.ui.internal.settings.ScopePreferenceProvider;

var messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "IFrameInnerContainerMessages");

/*
 * The container widget inside the iframe that contains the desired widget.
 * Should only be used by IFrameContainer.
 * 
 * CAUTION: Must remain backward compatible for cross-version interoperability.
 * Dashboard framework must check for existence of new methods added after 0.7.
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.util.IFrameInnerContainer",
	[dijit._Widget, dijit._Templated], {
	
	templateString: "<div class=\"com-ibm-team-dashboard-web-ui-internal-util-IFrameInnerContainer\"><div class=\"loading\">" + messages.loading + "</div></div>",
	
	active: false,
	args: null,
	onBustOut: null,
	onBustIn: null,
	onLoad: null,
	onResize: null,
	
	_serviceRoot: null,
	_encodeProxiedURI: null,
	_isAutoResize: false,
	_isDestroyed: false,
	_bustOutCounter: 0,
	
	postCreate: function() {
		// notify parent that container is created
		if (frameElement.onCreate) {
			frameElement.onCreate(this);
		}
	},
	
	destroy: function() {
		this._isDestroyed = true;
		clearTimeout(this._onResizeTimeout);
		this.setActive(false);
		if (this._dialogDestroyConnect) {
			dojo.disconnect(this._dialogDestroyConnect);
			delete this._dialogDestroyConnect;
		}
		this.inherited(arguments);
	},
	
	/*
	 * Returns the scope preference provider class (constructor) for the
	 * viewlet in this iframe.
	 */
	getScopePreferenceProviderClass: function() {
		return ScopePreferenceProvider;
	},
	
	init: function(/* object */ args) {
		/* 
		 * args.args = args to pass into inner widget constructor. 
		 * Copy args.args so that it's created by the iframe's Object instead 
		 * of the parent window's Object.
		 */
		this.args = {};
		var constArgs = args.args;
		for (var arg in constArgs) {
			if (constArgs.hasOwnProperty(arg)) this.args[arg] = constArgs[arg];
		}
		this.args.webURL = args.webUIRoot;
		this.onBustOut = args.onBustOut;
		this.onBustIn = args.onBustIn;
		this.onLoad = args.onLoad;
		this.onResize = args.onResize;
		this._serviceRoot = args.serviceRoot;
		this._encodeProxiedURI = args.encodeProxiedURI;
		
		// config data server root
		if (this._serviceRoot) {
			dojo.setObject("net.jazz.ajax._dataServerRoot", this._serviceRoot);
		}
		
		// lazy update link targets to parent window
		this.connect(document.body, "onmouseover", this._onMouseOver);
		
		// set initial active/inactive
		this.setActive(args.active);
		
		// get the class name from the url path
		var path = location.pathname + "";
		var index = path.lastIndexOf("/");
		var className = path.substring(index + 1);
		
		// replace proxy URI building to work in our proxied iframe
		jazz.app.proxy.encodeProxiedURI = this._encodeProxiedURI;
		jazz.app.internal.ProxyRequestInterceptor.setEncodeProxiedURIFunction(this._encodeProxiedURI);
		
		// get current contributor
		var serviceRoot = dojo.getObject("net.jazz.ajax._dataServerRoot");
		if (serviceRoot) {
			jazz.client.xhrGet({
				url: serviceRoot + CONTRIBUTOR_SERVICE_PATH,
				handleAs: "json",
				headers: {Accept: "text/json"},
				handle: dojo.hitch(this, function(/* object */ response) {
					if (response) {
						// unwrap SOAP envelope
						var contributor = dojo.getObject("soapenv:Body.response.returnValue.value", false, response);
						if (contributor) {
							// to support com.ibm.team.repository.web.client.session API in remote viewlets
							// where no initialization data is loaded
							dojo.setObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR", contributor);
						}
					}
					this._loadWidget(className);
				})
			});
		}
		else {
			this._loadWidget(className);
		}
	},
	
	/*
	 * Sets whether or not the container is currently active (visible to the
	 * user on the current page).
	 */
	setActive: function(/* boolean */ value) {
		if (value != this.active) {
			// resize only while active
			this._setAutoResize(value);
			this.active = value;
		}
	},
	
	_loadWidget: function(/* string */ className) {
		jazz.core.loader.load_async(className, dojo.hitch(this, function() {			
			// has to be done after code is loaded
			this._applyWorkarounds();
			
			// create and append widget
			var clazz = dojo.getObject(className);
			var widget = new clazz(this.args);
			
			// use a parent div with css rules to ensure text cannot flow outside the viewlet trim on bustout
			dojo.create("div",{className: "wrapper"},this.domNode,"only").appendChild(widget.domNode);
			if (this.onLoad) {
				this.onLoad(widget);
			}
		}));
	},
	
	_setAutoResize: function(/* boolean */ value) {
		if (value != this._isAutoResize) {
			if (value == true) {
				this._onResize(true);
				this._onResizeHandle = this.connect(window, "onresize", this._onResize);
			} else {
				this.disconnect(this._onResizeHandle);
				clearTimeout(this._onResizeTimeout);
				delete this._onResizeTimeout;
			}
			this._isAutoResize = value;
		}
	},
	
	_onBustOut: function() {
		// use ref counting to account for hovers in dialogs, vice versa
		if (this._bustOutCounter++ == 0) {
			this._setAutoResize(false);
			
			// delegate to outer container
			if (this.onBustOut) {
				this.onBustOut();
			}
		}
	},

	_onBustIn: function() {
		// don't try to bust in after destroy
		if (!this._isDestroyed) {
			// use ref counting to account for hovers in dialogs, vice versa
			if (this._bustOutCounter >= 1 && this._bustOutCounter-- == 1) {
				// delegate to outer container
				if (this.onBustIn) {
					this.onBustIn();
				}
				this._setAutoResize(true);
			}
		}
	},
	
	_onMouseOver: function(/* Event */ event) {
		if (event) {
			var node = event.target;
			if (!node)
				return;
			if (node.nodeName !== "A" && node.parentNode)
				node = node.parentNode;
			if (node.nodeName == "A" && (!node.target || node.target.toLowerCase() === "_self")) {
				// open links in parent window
				node.target = "_parent";
			}
		}
	},
	
	_onResize: function(/* boolean */ force) {
		if (this._isDestroyed || !this.domNode) return;
		var pos = dojo.position(this.domNode);
		if (pos && (force || pos.h != this._h || pos.y != this._y)) {
			this._h = pos.h;
			this._y = pos.y;
			if (this.onResize) {
				this.onResize(pos.h + pos.y * 2);
			}
		}
		clearTimeout(this._onResizeTimeout);
		this._onResizeTimeout = setTimeout(dojo.hitch(this, this._onResize), 500);
	},
	
	_applyWorkarounds: function() {
		var dijitConnect = dijit._Widget.prototype.connect;
		dijit._Widget.prototype.connect = function() {
			var args = [], i, result = [];
			for (i = 0; i < arguments.length; i++)
				args[i] = arguments[i];
			result.push(dijitConnect.apply(this,args));
			if (args[0] === dojo.body()) {
				args[0] = window.parent.dojo.body();
				result.push(dijitConnect.apply(this,args));
			}
			return {iframeconnect:result};
		}
		var dijitDisconnect = dijit._Widget.prototype.disconnect;
		dijit._Widget.prototype.disconnect = function() {
			var arg = arguments[0];
			if (typeof arg === "object" && arg.iframeconnect) {
				var arr = arg.iframeconnect;
				for (var i = 0; i < arr.length; i++)
					dijitDisconnect.apply(this,[arr[i]]);
			} else {
				dijitDisconnect.apply(this,[arg]);
			}
		}
		// attach bust-out listeners
		var Dialog = dojo.getObject("jazz.ui.Dialog");
		if (Dialog) {
			var self = this;
			this.connect(Dialog.prototype, "buildRendering", this._onBustOut);
			this._dialogDestroyConnect = dojo.connect(Dialog.prototype, "destroyRendering", null, function() {
				// ignore multiple destroy calls
				if (!this._destroyed) {
					self._onBustIn();
				}
			});
		}
		var MasterPopup = dojo.getObject("jazz.ui.internal._MasterPopup");
		if (MasterPopup) {
			this.connect(MasterPopup.prototype, "hide", dojo.hitch(this, function() {
				setTimeout(dojo.hitch(this, this._onBustIn), 0);
			}));
		}
		var HoverPopup = dojo.getObject("jazz.ui.HoverPopup");
		if (HoverPopup) {
			var self = this;
			var openFunc = HoverPopup.prototype.open;
			HoverPopup.prototype.open = function() {
				// need to preemptively bust out for hover popups
				// don't bust out for disabled hover popups
				if (this.isEnabled()) {
					self._onBustOut();
				}
				openFunc.apply(this, arguments);
			};
		}
		
		if (net.jazz.ajax._proxyLegacyMode) {
			var ResourceLink = dojo.getObject("jazz.ui.ResourceLink");
			if (ResourceLink) {
				ResourceLink.setLazyFetch(true);
			}
		}
	}
});

})();
