/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.util.ServiceTracker");

(function() {

/*
 * Keeps track of active service calls registered with this tracker.
 * Useful for canceling service requests on destroy.
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.util.ServiceTracker",
	null, {
	
	/*
	 * Constructs a new tracker.
	 */
	constructor: function() {
		this._activeRequests = new Object();
		this._currentRequestId = 0;
	},

	/*
	 * Returns an array of previously registered request objects that
	 * are still active, i.e. response has not yet been received.
	 */
	/* object[] */ getActiveRequests: function() {
		var requests = new Array();
		for (var i in this._activeRequests) {
			requests.push(this._activeRequests[i]);
		}
		return requests;
	},
	
	/*
	 * Registers the given request object and associated response handler with
	 * the viewlet. The viewlet will automatically cancel any active requests
	 * on destroy.
	 */
	register: function(/* object */ req, /* ServiceResponseHandler */ srh) {
		/*
		 * Wrap the srh with our own that tracks which responses have
		 * been received (transparently).
		 */
		var wrapper = {
			_id: "req" + this._currentRequestId,
			_self: this,
			_doCallback: dojo.hitch(srh, srh.doCallback),
			_doErrorCallback: dojo.hitch(srh, srh.doErrorCallback),
			doCallback: function(returnVal) {
				delete this._self._activeRequests[this._id];
				this._doCallback(returnVal);
			},
			doErrorCallback: function(errorObj) {
				delete this._self._activeRequests[this._id];
				this._doErrorCallback(errorObj);
			}
		};
		srh.doCallback = dojo.hitch(wrapper, wrapper.doCallback);
		srh.doErrorCallback = dojo.hitch(wrapper, wrapper.doErrorCallback);
		this._activeRequests[this._currentRequestId++] = req;
	},
	abortActiveRequests: function() {
		for (var i in this._activeRequests) {
			this._activeRequests[i].abort();
		}
	}
});

})();
