/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.util.ViewletRegistry");

dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");

dojo.require("jazz.client.xhr");

(function() {

var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var inFlight = 0;
var queue = [];

/*
 * Provides access to viewlet definitions, with caching.
 */
com.ibm.team.dashboard.web.ui.internal.util.ViewletRegistry = {
	
	_deferredsByUrl: new Object(),
	
	/*
	 * Retrieves the viewlet definition object from the specified origin.
	 */
	/* void */ getDefinition: function(/* string */ url, /* boolean */ passive, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
		// check cache
		var deferred = this._deferredsByUrl[url];
		if (!deferred || (!passive && !deferred.success)) {
			deferred = _fetchDefinition(url, passive, tracker);
			this._deferredsByUrl[url] = deferred;
		}
		deferred.addCallback(function(/* object[] */ definitions) {
			deferred.success = true;
			srh.doCallback(definitions);
	    	return definitions;
		});
		deferred.addErrback(function(/* Error */ error) {
			srh.doErrorCallback(error);
	    	return error;
		});
	}
};

function _demarshallResponse(evaldObj) {
	var returnValue= evaldObj["soapenv:Body"].response.returnValue;
	
	var obj= null;
	
	if (returnValue.values) {          // array of EMF objects 
		obj = returnValue.values;
	} else if (returnValue.value) {    // single EMF object
		obj = returnValue.value;
	}
	
	return obj;
}

function decrementInFlight(response) {
	inFlight--;
	if (inFlight > 0) {
		var deferred = new dojo.Deferred();
		queue.push(function(){
			deferred.callback(response);
		});
		return deferred;
	} else {
		for (var k = 0; k < queue.length; k++) 
			queue[k]();
		queue = [];
	}
}

function watchInFlight(deferred, url) {
	if (!com.ibm.team.dashboard.web.util.internal.Util.isLocalDefinition(url))
		return;
	inFlight++;
	deferred.addBoth(decrementInFlight);
}

function _fetchDefinition(/* string */ url, /* boolean */ passive, /* ServiceTracker */ tracker) {
	if (url.search(/^(g|i):.+/) == 0) {
		var deferred = new dojo.Deferred();
		var serviceObj = {
			_deferred: deferred,
			success: function(/* object[] */ definitions) {
				this._deferred.callback(definitions);
			},
			failure: function(/* Error */ error) {
				if (!(error instanceof Error)) {
					var orig = error;
					error = new Error();
					dojo.mixin(error, orig);
				}
				this._deferred.errback(error);
			}
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		DashboardClient.getViewletDefinitions([url], null, passive, srh, tracker);
		return deferred;
	}
	else {
		var args = {
			url: url,
			headers: {
				"accept": "text/json",
				"X-com-ibm-team-configuration-versions": "LATEST",
				"X-Jazz-CSRF-Prevent": dojo.cookie("JSESSIONID")
			},
			handleAs: "json"
		};
		if (passive) {
			args.skipInterceptors = ["jazz.oauth"];
		}
		var dfd = jazz.client.xhrGet(args);
		watchInFlight(dfd, url);
		dfd.addCallback(function(/* any */ response) {
			return _demarshallResponse(response);
		});
		return dfd;
	}
}
	
})();
