/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.ServersPreferenceProvider");

dojo.require("com.ibm.team.dashboard.web.ui.CustomPreferenceProvider");
dojo.require("jazz.ui.SwapListDialog");
dojo.require("jazz.app.internal.applicationAbout");

dojo.require("dojo.string");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "ProjectsMessages");

(function() {

var messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "ProjectsMessages");

dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.ServersPreferenceProvider",
		com.ibm.team.dashboard.web.ui.CustomPreferenceProvider, {

	viewlet: null,
	
	constructor: function(args) {
		this.viewlet = args.viewlet;
	},

	getLabel: function(servers) {
		servers = servers ? dojo.fromJson(servers) : {};
		if (this._label)
			return this._label;
		else if (!this._labelDeferred) {
			this._labelDeferred = this.viewlet._getAvailableServices();
			this._labelDeferred.addCallback(dojo.hitch(this,function(services) {
				var filtered = 0, total = services.length;
				for(var i = 0; i < total; i++) {
					if (servers[services[i].contextRoot])
						filtered++;
				}
				if (filtered === 0) {
					this._label = this._buildLabel(messages.allVisible);
				} else if (filtered === total) {
					this._label = this._buildLabel(messages.noneVisible);
				} else {
					this._label = this._buildLabel(dojo.string.substitute(messages.someVisible,[total-filtered,total]));
				}
			}));
		}
		return this._label || null;
	},
	
	getLoadingLabel: function() {
		return this._buildLabel(messages.loading);
	},
	
	_buildLabel: function(text) {
		return "<input type='text' readonly='readonly' value='" + text + "'/>";
	},
	
	_getServers: function() {
		var servers = settings.preferences.servers;
		return servers ? dojo.fromJson(servers) : {};
	},
	
	createChooser: function(onOk, settings) {
		var servers = settings.preferences.servers;
		servers = servers ? dojo.fromJson(servers) : {};
		this.swapListDialog = new jazz.ui.SwapListDialog({
			dialogTitleText: messages.dialogTitle,
			unselectedListTitleText: messages.dialogUnselected,
			selectedListTitleText: messages.dialogSelected,
			onOK: dojo.hitch(this,this.dialogOk,onOk),
			iconMap: this._getIconMap()
		});

		this.swapListDialog.show();
		this.swapListDialog.showLoading(true);		
		
		var deferred = this.viewlet._getAvailableServices(), abouts = jazz.app.internal.applicationAbout.get();
		deferred.addCallback(dojo.hitch(this,function(services) {
			var selected = [], unselected = [];
			for (var i = 0; i < services.length; i++) {
				var service = services[i];
				var about = abouts[service.applicationAbout];
				if (!about || about.error) about = {title: service.contextRoot};
				var entry = {
					id: service.contextRoot,
					label: about.title + " (" + service.publisher + ")",
					value: service.contextRoot,
					type: this._getServiceType(service)
				};
				if (servers[entry.id])
					unselected.push(entry);
				else
					selected.push(entry);
			}
			this.swapListDialog.setSelectedEntries(selected);
			this.swapListDialog.setUnselectedEntries(unselected);
			this.swapListDialog.showLoading(false);
		}));
	},
	
	dialogOk: function(onOk) {
		delete this._label;
		delete this._labelDeferred;
		var entries = this.swapListDialog.getUnselectedEntries();
		var result = {};
		for (var i = 0; i < entries.length; i++) {
			result[entries[i].id] = true;
		}
		onOk(dojo.toJson(result),"servers");
	},
	
	_getIconMap: function() {
		var abouts = jazz.app.internal.applicationAbout.get(), iconMap = [], aboutsMap = {};
		for (var key in abouts) {
			var value = "";
			if (key && abouts[key] && abouts[key].icon)
				value = abouts[key].icon
			aboutsMap[key] = iconMap.push(value) - 1;
		}
		this.aboutsMap = aboutsMap;
		return iconMap;
	},
	
	_getServiceType: function(service) {
		return this.aboutsMap[service.applicationAbout];
	}

});

})();
