/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rdm.fronting.server.web.ui.internal.wizard.FinalizeStep");  //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("jazz.ui.Button"); //$NON-NLS-1$
dojo.require("jazz.ui.wizard.WizardStepHeader"); //$NON-NLS-1$
dojo.require("jazz.ui.wizard.WizardStep"); //$NON-NLS-1$
dojo.require("jazz.ui.styledBox.TrimStyles"); //$NON-NLS-1$
dojo.require("jazz.ui.StyledBox"); //$NON-NLS-1$
dojo.require("jazz.ui.MessageBox");  //$NON-NLS-1$

dojo.require("com.ibm.rdm.fronting.server.web.util.rrsutils"); //$NON-NLS-1$

dojo.require("dojo.i18n");  //$NON-NLS-1$
dojo.requireLocalization("com.ibm.rdm.fronting.server.web.ui.internal.wizard", "FinalizeStepMessages"); //$NON-NLS-1$ $NON-NLS-2$

(function() {
var messages;
var isFirstFinalize;
var WizardStepHeader = jazz.ui.wizard.WizardStepHeader;
var WizardStep = jazz.ui.wizard.WizardStep;
var MessageBox = jazz.ui.MessageBox;
var TrimStyles = jazz.ui.styledBox.TrimStyles;
var rrsutils = com.ibm.rdm.fronting.server.web.util.rrsutils;
var CONFIGURATION_VERSION = "3"; //$NON-NLS-1$
var DOORSRP_REQUEST_TYPE = "private"; //$NON-NLS-1$


dojo.declare("com.ibm.rdm.fronting.server.web.ui.internal.wizard.FinalizeStep", [WizardStep, dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.rdm.fronting.server.web", "ui/internal/wizard/templates/FinalizeStep.html"),  //$NON-NLS-1$ $NON-NLS-2$
	widgetsInTemplate: true,
	innerBoxTrim:  TrimStyles.EDITOR,
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.rdm.fronting.server.web.ui.internal.wizard", "FinalizeStepMessages");  //$NON-NLS-1$ $NON-NLS-2$
		this.stepTitle = messages.stepTitle;
	},

	postCreate: function() {
		this.header = new WizardStepHeader({stepTitle: this.stepTitle});
		this.stepHeaderArea.appendChild(this.header.domNode);
		this.footerMsg = new MessageBox();
		this.stepFooterMsgArea.appendChild(this.footerMsg.domNode);		
		
		//Set StyledBox titles
		this._setStepBoxTitle(this.step1, this.messages.step1Label, this.messages.step1Title);
		this._setStepBoxTitle(this.step2, this.messages.step2Label, this.messages.step2Title);
		this._setStepBoxTitle(this.step3, this.messages.step3Label, this.messages.step3Title);

		this.servicePrefix = rrsutils.getRRSUrl(document.location.href);
		
		//Controls whether or not the step can go next
		this._setCompletedState(false);
		
		this.hasAssignLicenseProcessed = false;
		this.hasFinalizeProcessed = false;
		this.hasRebuildProcessed = false;		
		this.step1Button.disabled = true;		
		this.step2Button.disabled = true;
		
		//Create Progress Section DOM
		this.domOperation1 = this._addOperationDiv(this.messages.operation1title, this.messages.operation1desc);
		this.domOperation2 = this._addOperationDiv(this.messages.operation2title, this.messages.operation2desc);
		this.domOperation3 = this._addOperationDiv(this.messages.operation3title, this.messages.operation3desc);

		this.connect(this.step1Button, "onclick", "finalizeOperation"); //$NON-NLS-1$ $NON-NLS-2$
		this.connect(this.step2Button, "onclick", "rebuildOperation"); //$NON-NLS-1$ $NON-NLS-2$
	},
	
	focusStep: function() {
		this.checkConfigurationVersion();
	},
	
	checkConfigurationVersion: function() {
		var self = this;

		this._setFirstFinalize(false);
		this._displayMsgInfo(this.messages.testingConfiguration);
		
		var qstr = document.location.search.substr(1);
		var url = this.servicePrefix + "/project-resources/configure";
		if (qstr) {
			url = url + "?" + qstr;
		}

		var result = jazz.client.xhrGet({
			url: url,
			handleAs: "text", //$NON-NLS-1$
			headers: {
		    	"Accept": "*",  //$NON-NLS-1$ $NON-NLS-2$
		    	"DoorsRP-Request-Type": DOORSRP_REQUEST_TYPE //$NON-NLS-1$ 
		    },
			load: function(data, ioargs){
				self._hideMsg();
						
				if (data == CONFIGURATION_VERSION) {
					//Configuration found. Can reconfigure, rebuild index, or click Next
					self.step1Button.disabled = false;					
					self.step2Button.disabled = false;
					self._setCompletedState(true);
					if (self.isEmbedded) {
						self._displayMsgOK(self.messages.noFinalizeNeeded);
					} else {
						self._displayMsgOK(self.messages.noFinalizeNeededStandalone);
					}
				} else {
					//Configuration found, but version does not match. Need to finalize app				
					self.step1Button.disabled = false;
					self.step2Button.disabled = true;
					self._setCompletedState(false);
					self._displayMsgWarning(self.messages.noVersionMatch);					
				}
			},
			error: function(error, ioargs){
				self._hideMsg();
				
                switch (ioargs.xhr.status) {
                case 404:
                case 410:
                	//Configuration not found. Need to finalize app, cannot rebuild index
                    self.step1Button.disabled = false;
					self.step2Button.disabled = true;
					self._setFirstFinalize(true);
					self._displayMsgOK(self.messages.configurationOK);					
                    break;
                case 401:
                	//User should be authenticated already, but in case something went wrong...                     
                	self.step1Button.disabled = true;
					self.step2Button.disabled = true;                	
					self._displayMsgError(self.messages.authenticationError);
                    break;
                case 403:
                	//User not authorized. Either CAL or existing storage with old consumer key                
                	self.step1Button.disabled = true;
                	self.step2Button.disabled = true;                	
					self._displayMsgError(self.messages.authorizationError);
                    break;
                default:
                	//Catch it all for other errors
                	self.step1Button.disabled = true;
                	self.step2Button.disabled = true;  
					self._displayMsgError(dojo.string.substitute(self.messages.unexpectedError, [ioargs.xhr.status]));                	              	
					break;
                }
				self._setCompletedState(false);				
			}	        
		});	
	},
		
	finalizeOperation: function(e) {
		dojo.stopEvent(e);

		//Hide and refresh messages out there		
		this._hideOperationsMessages();
		this._displayMsgInfo(this.messages.finalizeProgress);
		
		//Disable buttons to prevent clicking while processing		
		this.step1Button.disabled = true;	
		this.step2Button.disabled = true;
		
		this.operation1();
	},
	
	rebuildOperation: function(e) {
		dojo.stopEvent(e);

		//Hide and refresh messages out there				
		this._hideOperationsMessages();
		this._displayMsgInfo(this.messages.rebuildProgress);		

		//Disable buttons to prevent clicking while processing
		this.step1Button.disabled = true;
		this.step2Button.disabled = true;
		 
		this.operation3();		
	},
	
	//Operation 1: Assign License
	operation1: function() {
		var self = this;
		//Create DOM to display the progress
		var domNode = this.domOperation1;
		dojo.addClass(domNode, "running");		
		var dAssignLicense = this.assignLicense();
		dAssignLicense.addCallback(function(response){
			dojo.removeClass(domNode, "running");			
			dojo.addClass(domNode, "ok");  //$NON-NLS-1$
			self.hasAssignLicenseProcessed = true;			
			self.operation2();
		});
		dAssignLicense.addErrback(function(errObject){
			dojo.removeClass(domNode, "running");			
			self._showInlineError(domNode, self.messages.operationError1);
			self._handleError(self.messages.finalizeError, errObject);
		});
	},
	
	//Operation 2: Configure
	operation2: function() {
		var self = this;	
		var domNode = this.domOperation2;
		dojo.addClass(domNode, "running");		
		var dConfigure = this.configure();	 
		dConfigure.addCallback(function(response){
			dojo.removeClass(domNode, "running");			
			dojo.addClass(domNode, "ok");  //$NON-NLS-1$
			self.hasFinalizeProcessed = true;			
			self.operation3();
		});
		dConfigure.addErrback(function(errObject){
			dojo.removeClass(domNode, "running");			
			self._showInlineError(domNode, self.messages.operationError2);
			self._handleError(self.messages.finalizeError, errObject);
		});
	},
	
	//Operation 3: Rebuild
	operation3: function() {	
		var self = this;
		var domNode = this.domOperation3;
		dojo.addClass(domNode, "running");
		var dRebuild = this.rebuild();
		dRebuild.addCallback(function(response){
			dojo.removeClass(domNode, "running");
			dojo.addClass(domNode, "ok");  //$NON-NLS-1$
			self.hasRebuildProcessed = true;			
			if (self.isEmbedded) {
				self._displayMsgOK(self.messages.allComplete);
			} else {
				self._displayMsgOK(self.messages.allCompleteStandalone);				
			}
			//Re-enable buttons depending on what ran
			self._setCompletedState(true);			
			if (self.hasAssignLicenseProcessed && self.hasFinalizeProcessed) {
				//License assignment and finalize ran fine, but still disable the Finalize button to prevent a re-configure right away and show same behavoir as other CLM apps during setup.
				//To enable the button for re-configure, need to re-run the setup wizard
				self.step1Button.disabled = true;
			}
			if (self.hasRebuildProcessed) {
				self.step2Button.disabled = false;
			}			
		});
		dRebuild.addErrback(function(errObject){
			dojo.removeClass(domNode, "running");			
			self._showInlineError(domNode, self.messages.operationError3);
			self._handleError(self.messages.userSyncError, errObject);
		});
	},
	
	assignLicense: function() {
		var deferred = new dojo.Deferred();
		jazz.client.rawXhrPost({
			url: this.servicePrefix + "/configure?assignLicense=true", //$NON-NLS-1$
			headers: {
		    	"DoorsRP-Request-Type": DOORSRP_REQUEST_TYPE //$NON-NLS-1$ 
		    },
			load: function(response) {
				deferred.callback(response);
			},
			error: function(errObject) {
				deferred.errback(errObject);		
			}
		});	
		return deferred;
	},
	
	configure: function() {
		var deferred = new dojo.Deferred();
		jazz.client.rawXhrPost({
			url: this.servicePrefix + "/configure?setup=true", //$NON-NLS-1$
			headers: {
		    	"DoorsRP-Request-Type": DOORSRP_REQUEST_TYPE //$NON-NLS-1$ 
		    },
			load: function(response) {
				deferred.callback(response);
			},
			error: function(errObject) {
				deferred.errback(errObject);		
			}
		});	
		return deferred;
	},
	
	rebuild: function() {
		var deferred = new dojo.Deferred();
		var firstTime = "false"; //$NON-NLS-1$
		if (this._getFirstFinalize()) {
			firstTime = "true"; //$NON-NLS-1$
		}
		jazz.client.rawXhrPost({
			url: this.servicePrefix + "/configure?rebuild=true&first=" + firstTime, //$NON-NLS-1$
			headers: {
		    	"DoorsRP-Request-Type": DOORSRP_REQUEST_TYPE //$NON-NLS-1$ 
		    },
			load: function(response) {
				deferred.callback(response);
			},
			error: function(errObject) {
				deferred.errback(errObject);		
			}
		});	
		return deferred;
	},
	
	_hideOperationsMessages: function(node) {
		this._hideOperationMessages(this.domOperation1);
		this._hideOperationMessages(this.domOperation2);
		this._hideOperationMessages(this.domOperation3);
	},
	
	_hideOperationMessages: function(node) {
		dojo.removeClass(node, "ok");  //$NON-NLS-1$
		dojo.removeClass(node, "error");  //$NON-NLS-1$	
		dojo.removeClass(node, "running");  //$NON-NLS-1$
		this._hideInlineError(node);
	},
	
	_showInlineError: function(node, message) {
		dojo.addClass(node,"error");
		var errorDiv = node.getElementsByTagName("div");  //$NON-NLS-1$
		errorDiv[0].innerHTML = message;
		dojo.removeClass(errorDiv[0], "noDisplay");  //$NON-NLS-1$
	},
	
	_hideInlineError: function(node) {
		var errorDiv = node.getElementsByTagName("div");  //$NON-NLS-1$
		errorDiv[0].innerHTML = "";
		dojo.addClass(errorDiv[0], "noDisplay");  //$NON-NLS-1$	
	},	
	
	_handleError: function(message, response) {
		var errorObj = {};
		if (response && response.responseText) {
			try {
				var responseObj = dojo.fromJson(response.responseText);
				dojo.mixin(errorObj, {
					errorClass: responseObj.errorClass,
					message: responseObj.errorMessage,
					exception: responseObj.errorTrace,
					errorData: responseObj.errorData,
					nestedError: responseObj.nestedError
				});
			} 
			catch (/* Error */ e) {
				// for non-JSON responses, include it as a message
				errorObj.message = response.responseText;
			}
		}
		this.header.showMessage({
			errorObject: errorObj,
			summary: message
		});
		
		this.hasProcessed = false;
		this.step1Button.disabled = false;
	},

	_displayMsgOK: function(message) {
		this.header.showMessage({
			type: MessageBox.STATUS_TYPES.OK, 		
			summary: message
		});
		this.footerMsg.showMessage({
			type: MessageBox.STATUS_TYPES.OK, 		
			summary: message
		});		
		this._refreshPage();	
	},

	_displayMsgInfo: function(message) {
		this.header.showMessage({
			type: MessageBox.STATUS_TYPES.INFO, 		
			summary: message
		});
		this.footerMsg.showMessage({
			type: MessageBox.STATUS_TYPES.INFO, 		
			summary: message
		});		
		this._refreshPage();	
	},

	_displayMsgError: function(message) {
		this.header.showMessage({
			type: MessageBox.STATUS_TYPES.ERROR, 		
			summary: message
		});
		this.footerMsg.showMessage({
			type: MessageBox.STATUS_TYPES.ERROR, 		
			summary: message
		});
		this._refreshPage();
	},

	_displayMsgWarning: function(message) {
		this.header.showMessage({
			type: MessageBox.STATUS_TYPES.WARNING, 		
			summary: message
		});
		this.footerMsg.showMessage({
			type: MessageBox.STATUS_TYPES.WARNING, 		
			summary: message
		});
		this._refreshPage();
	},

	_hideMsg: function() {
		this.header.hideMessage();
		this.footerMsg.hideMessage();
		this._refreshPage();	
	},

	//Creates a new DIV element for the operation and appends it to the DOM
	//Returns: a handle to the DIV element
	_addOperationDiv: function(title, description) {
		var div = dojo.create("div", { className: "operation"}, this.operations);   //$NON-NLS-1$ $NON-NLS-2$
		dojo.create("h3", { className: "operationTitle", innerHTML: title}, div);  //$NON-NLS-1$ $NON-NLS-2$
		dojo.create("p", { className: "operationDesc", innerHTML: description}, div);   //$NON-NLS-1$ $NON-NLS-2$
		dojo.create("div", { className: "operationError noDisplay"}, div);    //$NON-NLS-1$ $NON-NLS-2$
		return div;
	},
	
	_setStepBoxTitle: function(node,number,title) {
		dojo.addClass(node._headerPrimary, "inner-step-title");  //$NON-NLS-1$
		node._headerPrimary.innerHTML = "<span class=\"step\">" + number + "</span><span>" + title + "</span>";  //$NON-NLS-1$ $NON-NLS-2$ $NON-NLS-3$
	},
	
	_setCompletedState: function(state) {
		this.isCompleted = state;
		this._refreshPage();
	},

	_refreshPage: function() {
		//refresh page navigation controls and resize iframe
		this.triggerButtonStateUpdate();
	},
	
	_setFirstFinalize: function(state) {
		//is this the first time finalize runs?
		this.isFirstFinalize = state;
	},

	_getFirstFinalize: function() {
		return this.isFirstFinalize;		
	},

	canFlipToPreviousPage: function() {
		return true;
	},
	 
	canFlipToNextPage: function() {
		return this.isCompleted;
	},
	
	canFinish: function() {
		return this.isCompleted && !this.isEmbedded;
	}

});

})();