/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009,2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.fronting.server.web.ui.internal.wizard.WizardApplication");

dojo.require("jazz.client.xhr");
dojo.require("jazz.ui.Wizard");
dojo.require("jazz.ui.wizard.WizardStep");

dojo.require("com.ibm.rdm.fronting.server.web.ui.internal.wizard.FinalizeStep");
dojo.require("com.ibm.rdm.fronting.server.web.util.rrsutils");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.rdm.fronting.server.web.ui.internal.wizard", "WizardApplicationMessages");

var messages;
var Wizard = jazz.ui.Wizard;
var WizardStep = jazz.ui.wizard.WizardStep;
var EmbedFinalize = com.ibm.rdm.fronting.server.web.ui.internal.wizard.FinalizeStep; 
var rrsutils = com.ibm.rdm.fronting.server.web.util.rrsutils;

dojo.declare("com.ibm.rdm.fronting.server.web.ui.internal.wizard.WizardApplication", dijit._Widget, {
	
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.rdm.fronting.server.web.ui.internal.wizard", "WizardApplicationMessages");
	},	

	postCreate: function () {
		this._startApp();
	}, 
	
	_startApp: function () {
		document.body.style.background = "none";
		document.title = this.messages.setupName;
		this.isEmbedded = isEmbedded(); // check if we run stand-alone or embedded as part as the CLM setup wizard
		this._createUI();
	},
	
	_createUI: function() {
		var self = this;

		//Flags to track authentication and redirections
		this.isAuthenticated = false;
		this.didRedirect = false;
				
		this.servicePrefix = rrsutils.getRRSUrl(document.location.href);
		
		var qstr = document.location.search.substr(1);
		var url = this.servicePrefix + "/configure/oauth";
		if (qstr) {
			url = url + "?" + qstr;
		}
			
		// make sure the user is authenticated when the wizard loads
		jazz.client.xhrGet({
			url: url,
			sync: true,
			skipInterceptors: ["jazz.oauth"],			
		    load: function(data, ioargs){
				self.isAuthenticated = true;		    	
			},
			error: function(error, ioargs){	
				if (ioargs.xhr.status === 401) {
					self.isAuthenticated = false;				
				    var oauthUrl = ioargs.xhr.getResponseHeader("X-jazz-web-oauth-url");
				    if (oauthUrl) {
						//Redirect to the authentication page					    
					    self.didRedirect = true;		
					    document.location.href = rrsutils.getRewrittenOAuthUrl(oauthUrl);
				    }
				} else {
					self.isAuthenticated = true;				
					console.log(error);				
				}
			} 
		});

		if (this.isAuthenticated) {
			//The request was authenticated. If the authentication tokens are still present, remove them and redirect again.
			this.didRedirect = this.redirectWOTokens();
		}
		
		if (this.didRedirect) {
			//A redirect kicked in, so don't continue with the finalize page. 
			//Redirect conditions:
			//1. The request failed to authenticate, and we did a redirect to the authentication page. 
			//   The authentication page in turn will redirect to this page. 
			//2. The request authenticated, but there were still authentication tokens. We removed the tokens and redirected again to this page.			
			return;
		}			
		
	 	this.content = dojo.create("div");

		dojo.addClass(this.domNode, "com-ibm-team-repository-web-setup-RootNode");		
		dojo.addClass(this.content, "com-ibm-team-repository-wizard");		

	 	// Only add header elements if we not run embedded
	 	if(!this.isEmbedded) {	 	
			var textHeader = document.createElement("div");
			textHeader.innerHTML= this.messages.setupHeading;
			textHeader.className = "setup-header";
			this.content.appendChild(textHeader);
		
			var jazzIcon = document.createElement("img");
			jazzIcon.src = dojo.moduleUrl("com.ibm.rdm.fronting.server.web", "ui/internal/images/jazz-identR.png");
			jazzIcon.className = "jazz-icon";
			this.content.appendChild(jazzIcon);
		
			var setupGraphic = document.createElement("img");
			setupGraphic.src = dojo.moduleUrl("com.ibm.rdm.fronting.server.web", "ui/internal/images/JTS-wizard-graphic.png");
			setupGraphic.className = "wizard-graphic";
			this.content.appendChild(setupGraphic);
		} else {
			dojo.addClass(this.content, "com-ibm-team-repository-wizard-embedded");
		}
				
		this.wizardBox = document.createElement("div");
		this.content.appendChild(this.wizardBox);
								
		var wizardParams = {
			fullPageContainer: this.wizardBox,
			stepListLayout: "vertical",			
			processFinish:  function(){
				// URL to switch to when wizard finishes & we are not running embedded: /{context root}/web
			 	if(!this.isEmbedded) {
					document.location.href = net.jazz.ajax._contextRoot + "/web";
			 	}					
            }
		};
				
		wizardParams.steps = this.initServerSetupSteps();
		var wizard = new Wizard(wizardParams);
		this.domNode.appendChild(this.content);
	},

	// If the 'request_token_secret' and 'oauth_token' query params are present in the URL
	// we remove them from the URL and redirect once more.
	// If the tokens are not present, we do nothing.
	redirectWOTokens: function() {
		var redirected = false;
		
		var curPage = location.href;
		var curPageUrl = new dojo._Url(curPage);
		
		//save the embedded step fragment

		var fullFragment = "";
		if (curPageUrl.fragment) {
			var fragment = "#" + curPageUrl.fragment;
			var fragmentStart = curPage.indexOf(fragment);
			fullFragment = curPage.substr(fragmentStart);
		}
				
		var nvps = rrsutils.getNVPs(curPageUrl.query);
		var newNvps = [];
		var found = false;
		for(var i=0; i<nvps.length; i++) {
			if (nvps[i].n === "request_token_secret" || nvps[i].n === "oauth_token") {
				found = true;
			} else {
				newNvps.push({n: nvps[i].n, v: nvps[i].v}); 
			}
		}
		
		if (found) {
			redirected = true;
			document.location.href = rrsutils.buildNewUrl(curPageUrl, newNvps) + fullFragment;
		}
		
		return redirected;	
	},
	
	initServerSetupSteps: function () {
		var steps = [ new EmbedFinalize({ isEmbedded: this.isEmbedded }) ];			
		return steps;
	}
	
});

function isEmbedded() {
	var qstr = window.location.search.substr(1);
	if (qstr.length) {
		var qparts = qstr.split("&");
		for (var x = 0; x < qparts.length; x++) {
			var tp = qparts[x].split("=");
			if (tp[0] == "embed") {			 
				return tp[1] === "true" ? true : false;
			}
		}
	}	
	return false;
}

})();