/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rdm.fronting.server.web.reqpromigration.ui.internal.wizard.WizardMigrationStatus");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.app.auth");
dojo.require("jazz.ui.Button");
dojo.require("jazz.ui.wizard.WizardStepHeader");
dojo.require("jazz.ui.wizard.WizardStep");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.rdm.fronting.server.web.reqpromigration.ui.internal.wizard", "WizardMessages");

dojo.require("com.ibm.rdm.fronting.server.web.reqpromigration.util.rrsutils");
dojo.require("com.ibm.rdm.fronting.server.web.reqpromigration.util.XhrUtils");

(function() {
var rrsutils = com.ibm.rdm.fronting.server.web.reqpromigration.util.rrsutils;
var XhrUtils = com.ibm.rdm.fronting.server.web.reqpromigration.util.XhrUtils;
var messages;
var WizardStepHeader = jazz.ui.wizard.WizardStepHeader;
var WizardStep = jazz.ui.wizard.WizardStep;
var success = false;
var timerMigrationStatusId;
var statusGetCounter = 0;
var statusGetCounterProcessed = 0;
dojo.declare("com.ibm.rdm.fronting.server.web.reqpromigration.ui.internal.wizard.WizardMigrationStatus", [WizardStep, dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.rdm.fronting.server.web.reqpromigration", "ui/internal/wizard/templates/MigrationStatus.html"),
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.rdm.fronting.server.web.reqpromigration.ui.internal.wizard", "WizardMessages");
		this.stepTitle = messages.stepMigrationStatus;
		this.stepTitleInProgress = messages.stepMigrationStatusProgress;
		this.stepTitleDone = messages.stepMigrationStatusDone;
	},
	
	postCreate: function(){
		this.header =  new WizardStepHeader({stepTitle: this.stepTitle});
		this.stepHeaderArea.appendChild(this.header.domNode);
	},

	focusStep: function () {
		var self = this;
		var rrsUrl = rrsutils.getRRSUrl(document.location.href);
		var qstr = window.location.search.substr(1);
		
		self.statusUrl = document.getElementById("statusUrl").value;
		
		timerMigrationStatusId = setInterval(function() { self.getRebuildStatus(); }, 1000);
	},
	
	getRebuildStatus: function() {
		statusGetCounter++;
		var self = this;
		var response;
		var rrsUrl = rrsutils.getRRSUrl(document.location.href);
		var reqProMigrationService = self.statusUrl;
		
		self.stepHeaderArea.removeChild(self.header.domNode);
		self.header =  new WizardStepHeader({stepTitle: self.stepTitleInProgress});
		self.stepHeaderArea.appendChild(self.header.domNode);
					
		//check the status of the configuration
  		var result = jazz.client.xhrGet({
			url: reqProMigrationService,
			sync:true,
			preventCache: true,
			handleAs : "xml",
			load: function(data, ioargs){
				statusGetCounterProcessed++;
				var remainingPkgs = self.parseMigrationStatusDataForPkgsCount(data);
				var statusMsg = self.parseMigrationStatusDataForStatusMsg(data);
				if (remainingPkgs === '0') {
					clearInterval(timerMigrationStatusId);
					self._showConnectionStatus("OK", statusMsg);
					self.stepHeaderArea.removeChild(self.header.domNode);
					self.header =  new WizardStepHeader({stepTitle: self.stepTitleDone});
					self.stepHeaderArea.appendChild(self.header.domNode);
				} else {
					self._showConnectionStatus("PROGRESS", statusMsg);
				}
				
			},
			error: function(error){
				if (self.XhrUtils.getResponseCode(error) != 404) {
					clearInterval(timerMigrationStatusId);
					self._showConnectionStatus("ERROR", error);
				}
			}	        
		});
		if ((statusGetCounterProcessed - statusGetCounter) > 10){
			//Stop it, could be done
			clearInterval(timerMigrationStatusId);
			self._showConnectionStatus("OK", statusMsg);
			self.header.stepTitle = self.stepTitleDone;
		}
		return result;
	},
	
	parseMigrationStatusDataForPkgsCount: function(data) {
		var remainingPkgs = 0;
		
		//parse response				
		var nodes = domutils.getElementsByTagNameNS(
				 data, 
				 "remainingPkgs",
				 domutils.RM_NAMESPACE, 
				 domutils.RM_QUALIFIER);
 
		for (var i=0; i<nodes.length; i++) {
			var node = nodes[i];
			remainingPkgs = (node.firstChild ? node.firstChild.nodeValue : "0");
		}	
		
		//remaining pkgs
		return remainingPkgs;
	},
	
	parseMigrationStatusDataForStatusMsg: function(data) {
		var statusMsg = 0;
		
		//parse response				
		var nodes = domutils.getElementsByTagNameNS(
				 data, 
				 "currentStatusMsg",
				 domutils.RM_NAMESPACE, 
				 domutils.RM_QUALIFIER);
 
		for (var i=0; i<nodes.length; i++) {
			var node = nodes[i];
			statusMsg = (node.firstChild ? node.firstChild.nodeValue : "");
		}	
		
		//current status msg
		statusMsg = statusMsg.replace(/NEWLINE/gi, '<BR>');
		return statusMsg;
	},
	
	_showConnectionStatus: function(type, statusMsg) {
		var responseNode = this.responseMessage;
		responseNode.className = "stepStatus statusMsg " + type;
		responseNode.innerHTML = statusMsg;
	},
	
	_setStatusIndication: function (type, summary, details) {
		var responseNode = this.responseMessage;
		
		// append summary to details
		var buf = "";
		buf = buf + "<h3>";
		buf = buf + summary
		buf = buf + "</h3>";
		buf = buf + "<br>";
		if(typeof(details) != "undefined") {
			buf = buf + details;
		}
		summary = buf;
		
		

		if (type != "INFO" && type != "NONE") {
			responseNode.innerHTML = summary;
			var indication = document.createElement("div");
			indication.className = "stepStatus statusMsg " + type;
			indication.innerHTML = summary;
		} else {
			dojo.toggleClass(responseNode, "noDisplay", false);
		}
	},
	
	canFinish: function() {
		return this.success;
	}		
});
function parseData (json) {

	var buf = "";

	// heading
	buf = buf + "<h3>";
	if (json.remainingTasks == 0) {
		if(typeof(json.errors) != "undefined") {
			buf = buf + messages.Migration_Failed;
		} else {
			buf = buf + messages.Migration_Completed;
		}
	} else {
		buf = buf + messages.Migration_Running;
	}
	buf = buf + "</h3>";
	
	buf = buf + "<br>";
	
	//errors
	if(typeof(json.errors) != "undefined") {
		for (var i = 0; i < json.errors.length; i++) {
			buf = buf + json.errors[i];
			buf = buf + "<br>";
		}
		buf = buf + "<br>";
	}
	
	//results
	for (var i = 0; i < json.results.length; i++) {
		buf = buf + json.results[i];
		buf = buf + "<br>";
	}
	
	return buf;//"Hi<br>";
}
})();