/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rdm.fronting.server.web.reqpromigration.ui.internal.wizard.WizardProjectOptions");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");
dojo.require("jazz.ui.Button");
dojo.require("jazz.ui.wizard.WizardStepHeader");
dojo.require("jazz.ui.wizard.WizardStep");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.rdm.fronting.server.web.reqpromigration.ui.internal.wizard", "WizardMessages");

dojo.require("com.ibm.rdm.fronting.server.web.reqpromigration.util.domutils");
dojo.require("com.ibm.rdm.fronting.server.web.reqpromigration.util.rrsutils");
dojo.require("com.ibm.rdm.fronting.server.web.reqpromigration.util.Headers");

(function() {
var domutils = com.ibm.rdm.fronting.server.web.reqpromigration.util.domutils;
var rrsutils = com.ibm.rdm.fronting.server.web.reqpromigration.util.rrsutils;
var messages;
var WizardStepHeader = jazz.ui.wizard.WizardStepHeader;
var WizardStep = jazz.ui.wizard.WizardStep;
var finished = false;
var timeout;
var rrsUrl;
var header;

dojo.declare("com.ibm.rdm.fronting.server.web.reqpromigration.ui.internal.wizard.WizardProjectOptions", [WizardStep, dijit._Widget, dijit._Templated], {
	templatePath: dojo.moduleUrl("com.ibm.rdm.fronting.server.web.reqpromigration", "ui/internal/wizard/templates/ProjectOptions.html"),
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.rdm.fronting.server.web.reqpromigration.ui.internal.wizard", "WizardMessages");
		this.stepTitle = messages.stepProjectOptions;
		this.stepTitleSubmitting = messages.stepProjectOptionsSubmitting;
		this.Headers = com.ibm.rdm.fronting.server.web.reqpromigration.util.Headers;
	},
	
	postCreate: function(){
		this.header =  new WizardStepHeader({stepTitle: this.stepTitle});
		this.stepHeaderArea.appendChild(this.header.domNode);
	},
	
	canFinish: function() {
		return true;
	},
	
	canFlipToNextPage: function() {
		return false;
	},
	
	onSelectReqTypes: function(e) {
		if(!e.target || e.target.tagName !== "INPUT") {return;}
		
		var reqTypes = new Array();
		var inputs = this.glossaryReqTypes.getElementsByTagName("input");
		var value = null;
		var counter = 0;
		for (var i=0; i<inputs.length; i++) {
			if (inputs[i].checked) {
				value = inputs[i].id;
				reqTypes[counter] = value;
				counter++;
			}
		}
		if(counter > 0){
			this.reqTypesAsGlossarry = reqTypes;
		}
	},
	
	onFinish: function() {
		var self =this;
		var rrsUrl = rrsutils.getRRSUrl(document.location.href);
		var reqProMigrationService = rrsUrl + "/reqpromigration";
	    projOptionsXML = self.getProjOptionsXML();	
	    projOptionsJSON = self.getProjOptionsJSON();	
		var requestHeaders = null;
    	requestHeaders = {'Content-Type':'application/json', 'Content-Length': projOptionsJSON.length};
		
		self.stepHeaderArea.removeChild(self.header.domNode);
		self.header =  new WizardStepHeader({stepTitle: self.stepTitleSubmitting});
		self.stepHeaderArea.appendChild(self.header.domNode);
		
		var result = jazz.client.xhrPost({
	  		url: reqProMigrationService,
	  		postData: projOptionsJSON,
	  		sync:false,
			preventCache: true,
            headers: {
                "Content-Type":	"application/json",
                "DoorsRP-Request-Type": "private",
                "Accept": "none"
            },
            load: function(data, ioargs){
	  			
	  			var ref = window.location + "";
				document.getElementById("statusUrl").value= self.getHeader(ioargs,self.Headers.LOCATION);
				
				//Strip out the current step information
				ref = ref.substring(0,ref.indexOf('#')); 
				
				//Append "next step" information
				ref += "#/steps/3";
				
				window.location = ref;
				console.log("Done submitting the migration request" + data);
	        },
	        error: function(error){
	          	console.log("Error occurred migrating the project" + error);
	        }	        
	  	});
	  	
	},
	
	getHeader: function(ioArgs, header) {
		var headerValue = null;
		try {
			headerValue = ioArgs.xhr.getResponseHeader(header);
		}
		catch(e) {
			console.error(e);
		}
		return headerValue;
	},
	
	getProjOptionsXML : function(){
		var projXML = '<?xml version="1.0" encoding="UTF-8"?>';
			projXML = projXML + '<' + domutils.RM_QUALIFIER + ':project';
			projXML = projXML+ ' xmlns:' + domutils.RM_QUALIFIER + '="'+domutils.RM_NAMESPACE + '">\n';
			projXML = projXML+ this._composeProjectNameXmlSubstring();
			projXML = projXML+ this._composeProjectGuidXmlSubstring();
			projXML = projXML+ this._composeBaselineLocXmlSubstring();
			projXML = projXML+ this._composeMigrateDiscsXmlSubstring();
			projXML = projXML+ '<glossaryReqTypes>';
			projXML = projXML+ this._composeGlossaryReqTypes();
			projXML = projXML+ '</glossaryReqTypes>';
			projXML = projXML+ '</' + domutils.RM_QUALIFIER + ':project>';
		return projXML;
	},
	
	getProjOptionsJSON : function(){
		var projJsonObj = '{"baselineLoc": "' + escape(document.getElementById('baselineLocation').value) + '", ';
    		projJsonObj = projJsonObj + '"projGuid": "' + document.getElementById('projGuid').value + '", ';
    		projJsonObj = projJsonObj + '"rrmProjName": "' + document.getElementById('rrmProjName').value + '", ';
    		projJsonObj = projJsonObj + '"reqProProjName": "' + document.getElementById('reqProProjName').value + '", ';
    		projJsonObj = projJsonObj + '"glossaryReqTypes": [ ';
    		
    		var numOfGlossaryReqTypes = 0;
    		
    		if(this.reqTypesAsGlossarry){
    			this.reqTypesAsGlossarry.length;
    		}
    		
    		for (var i=0; i<numOfGlossaryReqTypes; i++) {
				var reqTypeGUID = this.reqTypesAsGlossarry[i];
				projJsonObj = projJsonObj+ '{"id": "' + reqTypeGUID + '"} ';
				if(i < numOfGlossaryReqTypes - 1){
					projJsonObj = projJsonObj+ ', ';
				}
            }
		
			projJsonObj = projJsonObj+ ']}';
			
		return projJsonObj;
	},
	
	_composeProjectNameXmlSubstring: function(){
		return ('<'+ domutils.RM_QUALIFIER + ':name>'+ document.getElementById('rrmProjName').value + '</'+ domutils.RM_QUALIFIER + ':name>');
	},
	
	_composeProjectGuidXmlSubstring: function(){
		return ('<'+ domutils.RM_QUALIFIER + ':guid>'+ document.getElementById('projGuid').value + '</'+ domutils.RM_QUALIFIER + ':guid>');
	},
	
	_composeBaselineLocXmlSubstring: function(){
		return ('<'+ domutils.RM_QUALIFIER + ':baselineLocation>'+ document.getElementById('baselineLocation').value + '</'+ domutils.RM_QUALIFIER + ':baselineLocation>');
	},
	
	_composeGlossaryReqTypes: function(){
		var reqTypeIds = '';
		if(this.reqTypesAsGlossarry){
			for (var i=0; i<this.reqTypesAsGlossarry.length; i++) {
				reqTypeIds = reqTypeIds + '<reqTypeId>' + this.reqTypesAsGlossarry[i] + '</reqTypeId>\n'
			}
		}
		return reqTypeIds;
	},
	
	_composeMigrateDiscsXmlSubstring: function(){
		if(document.getElementById("migrateDiscussions").checked){
			return ('<'+ domutils.RM_QUALIFIER + ':migrateDiscs>true</'+ domutils.RM_QUALIFIER + ':migrateDiscs>');
		}
		else{
			return ('<'+ domutils.RM_QUALIFIER + ':migrateDiscs>false</'+ domutils.RM_QUALIFIER + ':migrateDiscs>');
		}
	},
	
	_composeProjectDescriptionXmlSubstring: function(){
		return(
			'<' + queryutils.JP_QUALIFIER + ':description>'
			+ this._projectArgs["description"]
            + '</' + queryutils.JP_QUALIFIER + ':description>\n'
		);
	},
	
	getRebuildStatus: function() {	
		var self = this;
		var response;
		var rrsUrl = rrsutils.getRRSUrl(document.location.href);
		var reqProMigrationService = rrsUrl + "/reqpromigration/status?";
		//check the status of the configuration
  		var result = jazz.client.xhrGet({
			url: reqProMigrationService,
			handleAs: "xml",
			headers:{"Accept": "application/rdf+xml",
					"DoorsRP-Request-Type": "private" 
			},
			load: function(data, ioargs){
				//parse response	
				var time = self.parseRebuildData(data);
				if (time === 0) {
					clearInterval(timerRebuildId);
					self._showConnectionStatus("OK", messages.Configuration_Rebuild_Completed);
				} else {
					self._showConnectionStatus("PROGRESS", messages.Configuration_Rebuild_Running, dojo.string.substitute(messages.Configuration_Rebuild_Progress, [time]));
				}
			},
			error: function(error){
				self._showConnectionStatus("ERROR", error);
			}	        
		});
		return response;
	},
		
	getStatus: function(rrsUrl) {
		var self = this;
		var response;
		//check the status of the configuration
  		var result = jazz.client.xhrGet({
			url: rrsUrl + "/migration?status",
			sync: true,
			headers: {
                "Accept": "application/json"
            },
			load: function(data, ioargs){
				response = data;
				return data;
			},
			error: function(error){
				self._showConnectionStatus("ERROR", messages.Migration_Failed, error);
			}	        
		});
		return response;
	},
	
	_showConnectionStatus: function(type, summary, details) {
		if (type != "NONE" && type != "PROGRESS") {
			var headerSummary = summary;
			if (type == "OK") {
				headerSummary = headerSummary;	
			}
		
			var detailsMessagesArray;
			if(typeof(details) != "undefined") { 
				var detailsArray = details.split("<br>");
				detailsMessagesArray = new Array();
	
				//Get the actual details message
				for (var i = 0; i < detailsArray.length; i++) { 
					var detailsMessage = {};
					detailsMessage.message = detailsArray[i];
					detailsMessagesArray[i] = detailsMessage;
				}
			}
			
			this.header.showStatus({type: type, summary: headerSummary, details: detailsMessagesArray})
		} else {
			this.header.clearMessages();
		}
		this._setStatusIndication(type, summary, details);
	}, 
	
	_setStatusIndication: function (type, summary, details) {
		
		// append summary to details
		var buf = "";
		buf = buf + "<h3>";
		buf = buf + summary
		buf = buf + "</h3>";
		buf = buf + "<br>";
		if(typeof(details) != "undefined") {
			buf = buf + details;
		}
		summary = buf;
		
		

		if (type != "INFO" && type != "NONE") {
			var indication = document.createElement("div");
			indication.className = "stepStatus statusMsg " + type;
			indication.innerHTML = summary;

			dojox.data.dom.removeChildren(this.statusIndicationArea);
			dojo.toggleClass(this.statusIndicationArea, "noDisplay", false);
			this.statusIndicationArea.appendChild(indication);
			if (type != "PROGRESS") {
				setTimeout(dojo.hitch(this, function(){
					dojo.toggleClass(this.statusIndicationArea, "noDisplay", true);
				}), 3500);
			}	
			
		} else {
			dojox.data.dom.removeChildren(this.statusIndicationArea);
			dojo.toggleClass(this.statusIndicationArea, "noDisplay", false);
		}
	},

	doStatus: function () {
		var doStatusSelf = this;
		var response = doStatusSelf.getStatus(doStatusSelf.rrsUrl);
		var json = eval('(' + response + ')');
		var progress = parseData(json);
		
		if (json.remainingTasks == 0) {
			doStatusSelf.finished = true;
			clearInterval(doStatusSelf.timeout);
			if(typeof(json.errors) != "undefined") {
				doStatusSelf._showConnectionStatus("ERROR", messages.Migration_Failed, progress);
			}
			else{
				doStatusSelf._showConnectionStatus("OK", messages.Migration_Completed, progress);
			}
			} else {
					doStatusSelf._showConnectionStatus("PROGRESS", messages.Migration_Running, progress);
			}	
	}
		
});

function parseData (json) {

	var buf = "";

	// heading
	buf = buf + "<h3>";

	buf = buf + "</h3>";
	
	buf = buf + "<br>";
	
	//errors
	if(typeof(json.errors) != "undefined") {
		for (var i = 0; i < json.errors.length; i++) {
			buf = buf + "<font color=\"red\">"+json.errors[i]+"</font>";
			buf = buf + "<br>";
		}
		buf = buf + "<br>";
	}
	
	//results
	for (var i = 0; i < json.results.length; i++) {
		buf = buf + json.results[i];
		buf = buf + "<br>";
	}
	
	return buf;//"Hi<br>";
}

})();