/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rdm.fronting.server.web.reqpromigration.ui.internal.wizard.WizardUploadBaseline");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.io.iframe");
dojo.require("jazz.ui.Button");
dojo.require("jazz.ui.wizard.WizardStepHeader");
dojo.require("jazz.ui.wizard.WizardStep");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.rdm.fronting.server.web.reqpromigration.ui.internal.wizard", "WizardMessages");

dojo.require("com.ibm.rdm.fronting.server.web.reqpromigration.util.domutils");
dojo.require("com.ibm.rdm.fronting.server.web.reqpromigration.util.rrsutils");

(function() {

var domutils = com.ibm.rdm.fronting.server.web.reqpromigration.util.domutils;
var rrsutils = com.ibm.rdm.fronting.server.web.reqpromigration.util.rrsutils;
var messages;
var WizardStepHeader = jazz.ui.wizard.WizardStepHeader;
var WizardStep = jazz.ui.wizard.WizardStep;
var isAuthorized = false;

dojo.declare(
    "com.ibm.rdm.fronting.server.web.reqpromigration.ui.internal.wizard.WizardUploadBaseline",
    [WizardStep,dijit._Widget, dijit._Templated],
{
    // summary:
    // Provides a dialog to upload artifacts to server. 
    templatePath: dojo.moduleUrl("com.ibm.rdm.fronting.server.web.reqpromigration", "ui/internal/wizard/templates/WizardUploadBaseline.html"),
    
    constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.rdm.fronting.server.web.reqpromigration.ui.internal.wizard", "WizardMessages");
		this.stepTitle = messages.stepIntroTitle;
	},
    
    postCreate: function() {
		var header =  new WizardStepHeader({stepTitle: this.stepTitle});
		this.stepHeaderArea.appendChild(header.domNode);
		
		//get base Url
		var curPage = new dojo._Url(document.location.href);
		var path = curPage.path.toString();
		var rrsUrl = curPage.toString().substring(0, curPage.toString().indexOf(path));

		this.checkOAuth();
		
		//handle form validation
		this.reqProBaselineArchive = this.uploadFileName.value;
		this.validate();
	},
    
	// Check to force the user to login if OAuth key has been set.  Must be synchronous.
	checkOAuth: function() {
		var rrsUrl = rrsutils.getRRSUrl(document.location.href);
		var self = this;
		
		jazz.client.xhrGet({
			url: rrsUrl + "/configure/oauth",
			sync: true,
			load: function(data, ioargs){
				if (200 == ioargs.xhr.status) {
		            self.isAuthorized = true;
		            self.triggerButtonStateUpdate();
		        }
			}	        
		});
	},
	
	//form events
	handleKeyUp: function(e) {
		if(e.target && e.target.tagName === "INPUT") {
			if(this.onKeyUpTimout){
				clearTimeout(this.onKeyUpTimout);
			}
			var f = dojo.hitch(this, this.validate);
			var timeout = 2000;
			if (e.target._jazzRow  && e.target._jazzRow.hasError) {
				timeout = 500;
			}
			this.onKeyUpTimout = setTimeout(f, timeout);
		}
	},
	
	//form events
	handleChange: function(e) {
		var selectedFile = this.uploadFileName.value;
    	
    	//IE passes the file path along with the file name. FF only passes the file name.
    	//We need to make sure we only capture the filename
    	
    	var lastForwardSlashIndex = selectedFile.lastIndexOf("/"); //Unix path separator
    	var lastBackSlashIndex = selectedFile.lastIndexOf("\\"); //Windows path separator
    	
    	if(lastForwardSlashIndex != -1){
    		selectedFile = selectedFile.substring(lastForwardSlashIndex + 1);
    	}else if(lastBackSlashIndex != -1){
    		selectedFile = selectedFile.substring(lastBackSlashIndex + 1);
    	}   	   	
    	this.reqProBaselineArchive = selectedFile;
		this.validate();
	},
	
	//check for empty fields
	validate: function() {
	
		var isValid = true;
//		if (this.dataFields) {
//			for (var i = 0; i < this.dataFields.length; i++) {
//				if (isValid) {
//					var input = this.dataFields[i];
//					isValid = rrsutils.validateURLHostField(input.value);
//				}
//			}
//		}
		
		var isEmpty = this.reqProBaselineArchive === "";
		if (isEmpty) {
			isValid = false;
		} 
		
		clearTimeout(this.onKeyUpTimout);
		
		var formStatus = {
			isComplete: isValid
		};
		this.setStatus(formStatus);
		return formStatus;
	},
	
    _onFileNameChange: function(evt){
    	
    	var selectedFile = this._uploadFilename.value;
    	
    	//IE passes the file path along with the file name. FF only passes the file name.
    	//We need to make sure we only capture the filename
    	
    	var lastForwardSlashIndex = selectedFile.lastIndexOf("/"); //Unix path separator
    	var lastBackSlashIndex = selectedFile.lastIndexOf("\\"); //Windows path separator
    	
    	if(lastForwardSlashIndex != -1){
    		selectedFile = selectedFile.substring(lastForwardSlashIndex + 1);
    	}else if(lastBackSlashIndex != -1){
    		selectedFile = selectedFile.substring(lastBackSlashIndex + 1);
    	}   	   	
    	this._artifactName.value = selectedFile;
    }
    ,
    
    // invalid data disables the next button
	setStatus: function(status) {
		var canClickNext = false;
		if (status.isComplete) {
			canClickNext = true;
		}
		this.enableNext = canClickNext;
		this.triggerButtonStateUpdate();
	},
	
	// WIZARD STEP
	canFlipToNextPage: function () {
		return (this.enableNext === true && this.isAuthorized === true);
	},
	
	onNext: function() {
		
		var rrsUrl = rrsutils.getRRSUrl(document.location.href);
	
		this.submitFormToMigrationService();
		
		var ref = window.location + "";
		
		//Strip out the current step information
		ref = ref.substring(0,ref.indexOf('#')); 
		
		//Append "next step" information
		ref += "#/steps/2";
		
		window.location = ref;
	},
	
	postJFSUrl: function(rrsUrl) {	
		var ret = true;
		var result = jazz.client.xhrPost({
	  		url: rrsUrl,
	  		sync: true,
	  		load: function(data, ioargs){
	  			if(data){
	  			//Need to do some here
	  			}
	        },
	        error: function(error){
                console.log(error);
	        	ret = false;
	        }	        
	  	});
		return ret;
	},
	
	updateProjOptions: function(response) {
		var projInfo =  eval('(' + response + ')'); 
		
		document.getElementById("reqProProjName").value=projInfo.projName;
		document.getElementById("rrmProjName").value=projInfo.projName;
		document.getElementById("projGuid").value=projInfo.projGuid;
		document.getElementById("baselineLocation").value=projInfo.baselineLoc;
		var reqTypes = projInfo.reqTypes;
		
		var reqTypesTable = document.getElementById("glossaryReqTypes");
		var reqTypesTableBody = reqTypesTable.tBodies[0];
		
		var trCounter = 0;
		var tr = document.createElement("tr");
		for (var i=0; i<reqTypes.length; i++) {
			trCounter++;
			var reqType = reqTypes[i];
			var tdLabel = document.createElement("td");
	        tdLabel.setAttribute("class","wizardTypeCell");
	        tdLabel.setAttribute("colspan","2");
	        var label = document.createElement("label");
	        var labelText = document.createTextNode(reqType.name);
	        label.appendChild(labelText);
	        tdLabel.appendChild(label);
	        
	        var reqTypeInput = document.createElement("input");
	        reqTypeInput.setAttribute("type","checkbox");
	        reqTypeInput.setAttribute("align","left");
	        reqTypeInput.setAttribute("id", reqType.id);
	        reqTypeInput.setAttribute("name","reqType_" + reqType.id);
	        //reqTypeInput.setAttribute("tabIndex","1");
	        
	        var tdInput = document.createElement("td");
	        tdLabel.setAttribute("class","input");
	        tdInput.appendChild(reqTypeInput);
	        
	        var tdSpacer = document.createElement("td");
	        tdSpacer.setAttribute("class","wizardTypeCell");
	        
	        tr.appendChild(tdLabel);
	        tr.appendChild(tdInput);
	        tr.appendChild(tdSpacer);
	        
	        reqTypesTableBody.appendChild(tr);
	        //Add two reqtypes in each row
	        if (trCounter == 2){
	        	tr = document.createElement("tr");
	        	trCounter = 0;
	        }
		}
        this.showProjOptions();
	},
	
	showProjOptions: function() {
		document.getElementById("uploadingBaselineStatus").className = "net-jazz-ajax-hidden";
		document.getElementById("projOptionsDiv").className = "boxContainerYellow";
	},
	
	updateUploadErrorMessage: function(response) {
		document.getElementById("uploadingBaselineStatus").innerHTML = "Error occurred while uploading/extracting the baseline" + response;
	},
	
    submitFormToMigrationService: function(){    	
        
       	var isEmpty = this.reqProBaselineArchive === "";
		if (isEmpty) {
			return false;
		} 
//       this.showMessage(this.messages.UploadStatus);
       
       var rrsUrl = rrsutils.getRRSUrl(document.location.href);
	   var uri = rrsUrl + "/reqpromigration/baselinearchive";
	   var self =this;
	   //Submit the form using an IFRAME
    	dojo.io.iframe.send({
    		form: "uploadForm",
    		//handleAs: "json",
    		url: uri,
    		headers: {
		    	"DoorsRP-Request-Type": "private" 
		    },
    		load: function(response, ioArgs){
    		   console.log(messages.UploadStatusDone);
    		   this.step1Response = response;
    		   self.updateProjOptions(response);
    	    },
    	    error: function(response, ioArgs){
    	      console.log("error");
    	      console.log(response, ioArgs);
    	      self.updateUploadErrorMessage(response);
    	    }    	
    	});
    },
    
    

});
})();

