/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.underlay");

(function() {
	
/* Private */

var _onResizeEvent, _onScrollEvent, _onClickEvent, _underlay,
	_visible = false;

/* Public */

jazz.ui.underlay = {
	/*
	 * Adds an underlay to the body.
	 *
	 * args:
	 * 		zIndex: corresponds to the z-index of the underlay (default: 0)
	 * 		opacity: the opacity of the underlay (default: 0.06)
	 * 		animate: specifies whether the underlay should fade in (default: true, unless opacity is 0)
	 * 		appendTo: specifies where to append the blockerDiv
	 * 		stopEvents: boolean whether clicks on underlay should pass through
	 * 		color: css value for the underlay's background color (default: grey with a slight gradient)
	 */
	show: function(args){
		if (!args) args = {};
		if (!_underlay) {
			_underlay = dojo.create("div", {id: "jazz-ui-underlay"});
		}
		_update();
		var animate = (args.animate == null) ? true : args.animate,
			opacity = (args.opacity == null) ? 0.06 : args.opacity,
			appendTo = args.appendTo || document.body;
			
		// set style of underlay, pre-animation
		dojo.style(_underlay, {
			opacity: animate && !_visible ? 0 : opacity,
			zIndex: args.zIndex ? args.zIndex : 0,
			top: 0,
			left: 0
		});
		if (args.color) {
			dojo.style(_underlay, {
				backgroundImage: "none",
				backgroundColor: args.color
			});
		}
		if (args.stopEvents) {
			 if (!_onClickEvent) _onClickEvent = dojo.connect(_underlay, "onclick", null, _onClick);
		} else {
			dojo.disconnect(_onClickEvent);
			_onClickEvent = null;
		}
		if (!_visible || appendTo != _underlay.parentNode) {
			dojo.place(_underlay, appendTo);
		}
		if (!_visible) {
			_visible = true;
			if (animate && opacity !== 0) dojo.animateProperty({node: _underlay, duration: 280, properties: { opacity: { start: 0, end: opacity }}}).play();
			if (!_onResizeEvent) _onResizeEvent = dojo.connect(window, "onresize", null, _update);
			if (!_onScrollEvent) _onScrollEvent = dojo.connect(document.documentElement, "onscroll", null, _update);
		}
	},
	
	/*
	 * Hides the underlay.
	 */
	hide: function(){
		if (_visible) {
			_onResizeEvent && dojo.disconnect(_onResizeEvent);
			_onScrollEvent && dojo.disconnect(_onScrollEvent);
			if (_underlay) {
				_onClickEvent && dojo.disconnect(_onClickEvent);
				var pn = _underlay.parentNode;
				pn && pn.removeChild(_underlay);
			}
			_underlay = _onClickEvent = _onResizeEvent = _onScrollEvent = null;
			_visible = false;
		}
	},
	/*
	 * Updates the height and width of the overlay to match the document size.
	 */	
	update: function() {
		_update();
	}
}
function _update() {
	if (_underlay) {
		var style = _underlay.style;
		style.height = "0";
		var de = document.documentElement, height = de.clientHeight, scrollHeight = de.scrollHeight ? de.scrollHeight : document.body.scrollHeight;
		if (dojo.isIE <= 8) {
			var scrollTop = de.scrollTop, viewport = dijit.getViewport(), docHeight = ((scrollHeight > height) ? scrollHeight : height), top = Math.max(scrollTop - 400, 0), newHeight = viewport.h + 800;
			if (top + newHeight > docHeight) {
				newHeight = docHeight - top;
			}
			//Opacity isn't applied on DOM nodes larger than 2048px high in certain IE8- configurations (work item 126556).
			var h = Math.min(newHeight, 2000);
			if (h != newHeight) {
				var diff = Math.floor((newHeight - h) / 2);
				if (top > diff) {
					top = top - diff;
				}
				newHeight = h;
				if (top + newHeight > docHeight) {
					newHeight = docHeight - top;
				}
			}
			style.top = top + "px";
			style.height = newHeight + "px";
		} else {
			style.height = ((scrollHeight > height) ? scrollHeight : height) + "px";
		}
	}
}
function _onClick(event)  {
	dojo.stopEvent(event);
}

})();