 /* Copyright IBM Corp. 2007, 2009.  All Rights Reserved.
  *
  * US Government Users Restricted Rights - Use, duplication or disclosure
  * restricted by GSA ADP Schedule Contract with IBM Corp.
  */
(function() {
dojo.provide("com.ibm.rdm.fronting.server.web.reqpromigration.util.XhrUtils");
dojo.require("com.ibm.rdm.fronting.server.web.reqpromigration.util.Headers");

var Headers = com.ibm.rdm.fronting.server.web.reqpromigration.util.Headers;

com.ibm.rdm.fronting.server.web.reqpromigration.util.XhrUtils = {
	
	///////////////////////////////////////////////////////////////////////////////////
	// API
	
	getResponseEtag: function(response, doNotStripQuotes) {
		var etag = null;
		try {
			etag = response.ioArgs.xhr.getResponseHeader("Etag");
			if(!doNotStripQuotes) {
				if (etag && etag.indexOf('"') == 0) {
					etag = etag.substring(1, etag.length-1);
				}
			}
		}
		catch(e) {
			console.error(e);
		}
		return etag;
	},
	
	getResponseCode: function(response) {
		var responseCode = null;
		try {
			responseCode = response.ioArgs.xhr.status;
		}
		catch(e) {
			console.error(e);
		}
		return responseCode;
	},
	
	getResponseURI: function(response) {
		var uri = null;
		try {
			uri = response.ioArgs.args.url;
		}
		catch(e) {
			console.error(e);
		}
		return uri;       
	},
	
	getHeader: function(response, header) {
		var headerValue = null;
		try {
			headerValue = response.ioArgs.xhr.getResponseHeader(header);
		}
		catch(e) {
			console.error(e);
		}
		return headerValue;
	},
	
	getResponseContentType: function(response) {
		var contentType = null;
		try {
			contentType = this.getNormalizedContentType(this.getHeader(response, Headers.CONTENT_TYPE));
		}
		catch(e) {
			console.error(e);
		}
		return contentType;
	},
	
	getNormalizedContentType: function(contentType) {
		// summary:
		//    Sometimes our content types are specified like this:
		//        application/x-com.ibm.sid.sketch+xml;charset=UTF-8
		//    and other times like this:
		//        application/x-com.ibm.sid.sketch+xml
		//    This function is called to prune, if needed, the string
		//    after the ";" ...
		var index = contentType.indexOf(';');
		return (index == -1 ? contentType : contentType.substring(0, index))
	},
	getResponseLocation: function(response) {
		var locationHeader = null;
		try {
			locationHeader = this.getHeader(response, Headers.LOCATION);
			if (locationHeader.indexOf("?") > 0) {
				locationHeader = locationHeader.substring(0, locationHeader.indexOf("?"));
			}
		}
		catch(e) {
			console.error(e);
		}
		return locationHeader;
	},
	
	getResponseLastModified: function(response) {
		var lastModified = null;
		try {
			lastModified = this.getHeader(response, Headers.LAST_MODIFIED);
			lastModified = dojo.date.stamp.fromISOString(dojo.date.stamp.toISOString(new Date(lastModified)));
		}
		catch(e) {
			console.error(e);
		}
		return lastModified;
	},

	getResponseDate: function(response) {
		var date = null;
		try {
			date = this.getHeader(response, Headers.DATE);
			date = dojo.date.stamp.fromISOString(dojo.date.stamp.toISOString(new Date(date)));
		}
		catch(e) {
			console.error(e);
		}
		return date;
	},
	
	getResponseLastModifiedBy: function(response) {
		var lastModifiedBy = null;
		try {
			lastModifiedBy = this.getHeader(response, Headers.LAST_MODIFIED_BY);
		}
		catch(e) {
			console.error(e);
		}
		return lastModifiedBy;
	}
};

})();
