/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.util.internal.Util");

dojo.require("com.ibm.team.dashboard.web.util.DashboardUtil");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");

dojo.require("jazz.app.proxy");

(function() {

/*
 * Aliases
 */
var DashboardUtil = com.ibm.team.dashboard.web.util.DashboardUtil;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;

var LOCAL_URL = new dojo._Url(location.href);

/*
 * General purpose utility methods.
 */
com.ibm.team.dashboard.web.util.internal.Util = {
	
	/*
	 * Returns the context path of the local Web UI, e.g. "/jazz".
	 * 
	 * CAUTION: Used by foundation viewlets.
	 */
	/* string */ getContextPath: function(/* boolean */ qualified) {
		// use internal JAF constant (see work item 48210)
		var result = dojo.getObject("net.jazz.ajax._contextRoot");
		if (qualified) {
			result = LOCAL_URL.scheme + "://" + LOCAL_URL.host + (LOCAL_URL.port ? ":" + LOCAL_URL.port : "") + result;
		}
		return result;
	},
	
	/*
	 * Given a definition URL, returns the definition ID.
	 * e.g. for:
	 *    "https://localhost:9443/jazz/service/com.ibm.team.dashboard.common.internal.service.IDashboardRestService/viewletDefinitions?definitionIds=com.ibm.team.dashboard.viewlets.web.feed&amp;webUIRoot=https%3A%2F%2Flocalhost%3A9443%2Fjazz",
	 * returns:
	 *    "com.ibm.team.dashboard.viewlets.web.feed".
	 */
	/* string */ getDefinitionId: function(/* string */ definitionURL) {
		var url = new dojo._Url(definitionURL);
		return url.query.match(/definitionIds=([^&]*)/)[1];
	},
	
	/*
	 * Returns the service root where traditional services are located. If a
	 * definition URL is provided, then the service root of that definition's
	 * server is returned, otherwise the current data server's root is returned.
	 * e.g. for:
	 *    "https://localhost:9443/jazz/service/com.ibm.team.dashboard.common.internal.service.IDashboardRestService/viewletDefinitions?definitionIds=com.ibm.team.dashboard.viewlets.web.feed&amp;webUIRoot=https%3A%2F%2Flocalhost%3A9443%2Fjazz",
	 * this would return:
	 *    "https://localhost:9443/jazz".
	 */
	/* string */ getServiceRoot: function(/* string? */ url) {
		if (url) {
			var index = url.indexOf("/service/com.ibm.team.dashboard.common.internal.service.IDashboardRestService");
			return url.substring(0, index);
		}
		var serviceRoot = dojo.getObject("net.jazz.ajax._dataServerRoot");
		if (!serviceRoot) {
			serviceRoot = this.getContextPath(true);
		}
		return serviceRoot;
	},
	
	/*
	 * Given a definition URL, returns the Web UI root.
	 * e.g. for:
	 *    "https://localhost:9443/jazz/service/com.ibm.team.dashboard.common.internal.service.IDashboardRestService/viewletDefinitions?definitionIds=com.ibm.team.dashboard.viewlets.web.feed&amp;webUIRoot=https%3A%2F%2Flocalhost%3A9443%2Fjazz",
	 * this would return:
	 *    "https://localhost:9443/jazz".
	 */
	/* string */ getWebUIRoot: function(/* string */ url) {
		var url = new dojo._Url(url);
		if (!url.query) {
			return "";
		}
		return decodeURIComponent(url.query.match(/webUIRoot=([^&]*)/)[1]);
	},
	
	/*
	 * Returns a service URL for the given service call,
	 * e.g. "https://localhost:9443/jazz/service/com.ibm.team.myservice.MyService/method?param1=value1"
	 * 
	 * CAUTION: Used by foundation viewlets.
	 */
	/* string */ getServiceURL: function(/* string */ serviceId, /* string */ method, /* object */ params, /* string */ dataServerRoot) {
		if (!dataServerRoot) {
			dataServerRoot = dojo.getObject("net.jazz.ajax._dataServerRoot");
			if (!dataServerRoot) {
				dataServerRoot = this.getContextPath();
			}
		}
		
		var buf = [dataServerRoot, "/service/", serviceId];
		
		if (method) {
			buf.push("/");
			buf.push(method);
		}
		if (params) {
			buf.push("?");
			buf.push(dojo.objectToQuery(params));
		}
		var url = buf.join("");
		var urlObj = new dojo._Url(url);
		return this._isDifferentOrigin(urlObj) ? jazz.app.proxy.encodeProxiedURI(url) : url;
	},
	
	/*
	 * Returns an HREF that can be used as a link to the given dashboard. Tab is optional,
	 * and if specified, will take you to that specific tab.
	 */
	/* string */ getDashboardHref: function(/* DashboardBundleDTO */ dashboardBundle, /* string */ tabId) {
		var scope = dashboardBundle.scope;
		var scopeItem;
		switch(scope) {
			case DashboardConstants.SCOPE_TEAM_AREA:
				scopeItem = dashboardBundle.defaults.team.path;
				break;
			case DashboardConstants.SCOPE_PROJECT_AREA:
				scopeItem = dashboardBundle.defaults.project.name;
				break;
			default:
				var url = dashboardBundle.dashboardUrl;
				scope = DashboardConstants.SCOPE_CONTRIBUTOR;
				scopeItem = url.substring(url.lastIndexOf("/") + 1);
				break;
		}
		return DashboardUtil.getDashboardHref(scope, scopeItem, tabId);
	},
	
	/* string */ getAllPersonalDashboardsHref: function() {
		return net.jazz.ajax._contextRoot + "/dashboards/all";
	},
	
	/* string */ getAllPersonalDashboardsHash: function() {
		return "#/dashboards/all";
	},
	
	/* string */ getAllDashboardsHref: function(/* number */ tab) {
		var href = "#action=" + DashboardConstants.ACTION_VIEW_ALL_DASHBOARDS;
		if (tab == 0 || tab == 1) {
			href += "&tab=" + tab;
		}
		return href;
	},
	
	/* string */ getDashboardUpdateHref: function() {
		return net.jazz.ajax._contextRoot + "/dashboards/update";
	},
	
	/*
	 * Returns an args object that can be passed to DashboardPage#viewDashboard to view
	 * the given dashboard. Tab is optional, and if specified, will take you to that
	 * specific tab.
	 */
	/* object */ getDashboardParams: function(/* DashboardBundleDTO */ dashboardBundle, /* string */ tabId) {
		var scope = dashboardBundle.scope;
		var teamPath = scope == DashboardConstants.SCOPE_TEAM_AREA ? dashboardBundle.defaults.team.path.join("/") : null;
		var url = dashboardBundle.dashboardUrl;
		var id = url.substring(url.lastIndexOf("/") + 1);
		return this._getDashboardParams(scope, id, teamPath, tabId);
	},
	
	/*
	 * Returns an args object that can be passed to DashboardPage#viewDashboard to view
	 * the given dashboard info. Tab is optional, and if specified, will take you to that
	 * specific tab.
	 */
	/* object */ _getDashboardParams: function(/* number */ scope, /* string */ id, /* string */ teamPath, /* string */ tabId) {
		var params = new Object();
		switch(scope) {
			case DashboardConstants.SCOPE_CONTRIBUTOR:
				params[DashboardConstants.URL_PARAM_ID] = decodeURIComponent(id);
				break;
			case DashboardConstants.SCOPE_TEAM_AREA:
				params[DashboardConstants.URL_PARAM_TEAM] = decodeURIComponent(teamPath);
				break;
		}
		if (tabId) {
			params[DashboardConstants.URL_PARAM_TAB] = decodeURIComponent(tabId);
		}
		return params;
	},
	
	/*
	 * Returns the name of the dashboard's scope item.
	 */
	/* string */ getScopeItemName: function(/* DashboardBundleDTO */ bundle) {
		var itemId = bundle.dashboard.scopeItem.itemId;
		var itemNames = bundle.itemNames;
		for (var i=0;i<itemNames.length;i++) {
			if (itemNames[i].itemId == itemId) {
				return itemNames[i].name;
			}
		}
	},
	
	/*
	 * Returns whether or not the given viewlet is from the local server.
	 */
	/* boolean */ isLocal: function(/* object */ viewletHelper) {
		return this.isLocalOrigin(viewletHelper.origin);
	},
	
	/*
	 * Returns whether or not the given viewlet definition URL is from the
	 * local server or not.
	 */
	/* boolean */ isLocalDefinition: function(/* string */ url) {
		var webUIRoot = this.getWebUIRoot(url);
		if (!webUIRoot) {
			return false;
		}
		var localWebUIRoot = dojo.getObject("net.jazz.ajax._viewletWebUIRoot") || (this.getContextPath(true));
		return this._urlEquals(webUIRoot, localWebUIRoot);
	},
	
	/*
	 * Returns whether or not the given rootservices URL is from the local server
	 * or not.
	 */
	/* boolean */ isLocalOrigin: function(/* string */ rootServices) {
		if (!rootServices) {
			// old local viewlets don't have an origin set
			return true;
		}
		var localRootServices = dojo.getObject("net.jazz.ajax._rootServices") || (this.getContextPath(true) + "/rootservices");
		return this._urlEquals(rootServices, localRootServices);
	},
	
	/*
	 * Converts an associative array into an array of entries (reverse of toObject),
	 * i.e. { id1: value1, ... } -> [ {id: id1, value: value1}, ... ]
	 */
	/* object[] */ toEntryArray: function(/* object */ obj) {
		var entries = new Array();
		if (obj) {
			for (var i in obj) {
				entries.push({id:i, value:obj[i]});
			}
		}
		return entries;
	},
	
	/*
	 * A replacement for dojo.toJson that works with arrays from other
	 * windows. (dojo.isArray thinks they're objects with properties "0",
	 * "1", etc, which is wrong).
	 */
	/* string */ toJson: function(/* any */ value) {
		/*
		 * Temporarily replace dojo.isArray with one that works with arrays
		 * from other windows for the duration of this dojo.toJson call.
		 */
		var temp = dojo.isArray;
		dojo.isArray = _isArray;
		try {
			return dojo.toJson(value);
		}
		finally {
			dojo.isArray = temp;
		}
	},
	
	/*
	 * Converts an array of entries into an associative array (reverse of toEntryArray),
	 * i.e. [ {id: id1, value: value1}, ... ] -> { id1: value1, ... }
	 */
	/* object */ toObject: function(/* object[] */ array) {
		var obj = new Object();
		if (array) {
			for (var i=0;i<array.length;i++) {
				var entry = array[i];
				obj[entry.id] = entry.value;
			}
		}
		return obj;
	},
	
	/*
	 * Returns a boolean indicating if the specified scope applies to the provided definition.
	 */
	/* boolean */ isScopeApplicible: function(scope, definition) {
		if (!definition.scopeSensitive)
			return false;
		if (definition.appliesTo == Constants.SCOPE_APPLIES_TO_ALL)
			return true;
		switch (scope) {
			case DashboardConstants.SCOPE_CONTRIBUTOR:
				return definition.appliesTo == Constants.SCOPE_APPLIES_TO_CONTRIBUTORS;
			case DashboardConstants.SCOPE_TEAM_AREA:
				return definition.appliesTo == Constants.SCOPE_APPLIES_TO_TEAMS ||
						definition.appliesTo == Constants.SCOPE_APPLIES_TO_PROCESS;
			case DashboardConstants.SCOPE_PROJECT_AREA:
				return definition.appliesTo == Constants.SCOPE_APPLIES_TO_PROJECTS ||
						definition.appliesTo == Constants.SCOPE_APPLIES_TO_PROCESS;
		}
	},
	
	/*
	 * Copied from ProxyRequestInterceptor.js
	 */
	_isDifferentOrigin: function(url) {
		
		// optimize for the most common case (request to a non-complete URL)
		if(!url.host) {
			// if host isn't set, scheme and port don't even make sense
			return false;
		}
		
		this._normalizePort(url);
		
		// if any of these are different, it's different origin
		var props = ["scheme", "host", "port"];
		for(var i=0; i<props.length; i++) {
			var prop = props[i];
			if(url[prop] !== LOCAL_URL[prop]) {
				return true;
			}
		}
		
		// all properties are equal, same origin
		return false;
	},
	
	_normalizePort: function(url) {
		var scheme = url.scheme || "http";
		if(!url.port) {
			url.port = scheme === "http" ? "80" : "443";
		}
	},
	
	/*
	 * Returns whether or not the first URL is the same as the second, while
	 * being tolerant about scheme, hostname case, and explicit ports when not
	 * needed.
	 */
	/* boolean */ _urlEquals: function(/* string */ url1, /* string */ url2) {
		url1 = new dojo._Url(url1);
		url2 = new dojo._Url(url2);
		this._normalizePort(url1);
		this._normalizePort(url2);
		
		// check host case-insensitive
		if (url1.host.toLowerCase() != url2.host.toLowerCase()) {
			return false;
		}
		// check the rest of the relevant parts
		var props = ["scheme", "port", "path", "query", "fragment"];
		for(var i=0; i<props.length; i++) {
			var prop = props[i];
			if(url1[prop] !== url2[prop]) {
				return false;
			}
		}
		// all properties are equal, same URL
		return true;
	},
	
	_cast: function(type, value) {
		if (type === Constants.SettingsArea.FIELD_INTEGER) {
			var parsedValue = parseInt(value);
			if (isNaN(parsedValue)) {
				return null;
			}
			return parsedValue;
		}
		else if (type === Constants.SettingsArea.FIELD_BOOLEAN) {
			if (value === true || value === false)
				return value;
			return value === "true" ? true : false;
		}
		return value;
	}
};

function _isArray(/* any */ value) {
	return value && (typeof value == "array" || value instanceof Array || (typeof value == "object" && isFinite(value.length)));
}

})();