/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009,2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.fronting.server.web.util.rrsutils");

com.ibm.rdm.fronting.server.web.util.rrsutils = {
	
	getRRSUrl: function(location) {
		var curPage = new dojo._Url(location);
		var rrsUrl = curPage.toString();
		return rrsUrl.substring(0, rrsUrl.indexOf("/rmsetup"));
    },
    
    getUrlPrecedingSegment: function(location, currentSegment) {
        var curPage = new dojo._Url(location);
        var rrsUrl = curPage.toString();
        return rrsUrl.substring(0, rrsUrl.indexOf(currentSegment));
    },
    
	/*
	 * Verify that the host part of a url is valid
	 */
	validateURLHostField: function(inputValue) {
		var isValid = true;
		var isEmpty = inputValue === "";
		if (isEmpty) {
			isValid = false;
		} 
		
		if (isValid) {
			//validate host name
			var host = new dojo._Url(inputValue).host;
			if (!host) {
				isValid = false;
			}
		}
		return isValid;
	},
	
	getRewrittenOAuthUrl: function(oauthUrlStr) {
		var oauthUrl = new dojo._Url(oauthUrlStr);
		var oauthUrlNvps = this.getNVPs(oauthUrl.query);

		for (var i=0; i<oauthUrlNvps.length; i++) {
			var nvp = oauthUrlNvps[i];
			if(nvp.n === "oauth_callback") {
				var originalCallbackUrl = new dojo._Url(decodeURIComponent(nvp.v));
								
				var curPage = location.href;
				var curPageUrl = new dojo._Url(curPage);
				
				//save the embedded step fragment
				var fullFragment = "";
				if (curPageUrl.fragment) {
					var fragment = "#" + curPageUrl.fragment;
					var fragmentStart = curPage.indexOf(fragment);
					fullFragment = curPage.substr(fragmentStart);
				}
				
				var callbackNvps = this.getNVPs(originalCallbackUrl.query);
				if(curPageUrl.query) {
					callbackNvps = callbackNvps.concat(this.getNVPs(curPageUrl.query));
				}
				nvp.v = encodeURIComponent(this.buildNewUrl(curPageUrl, callbackNvps) + fullFragment);
			}
		}
		
		return this.buildNewUrl(oauthUrl, oauthUrlNvps);
	},
	
	buildNewUrl: function(dojoUrl, nvps) {
		var newUrl = dojoUrl.scheme + "://" + dojoUrl.host + (dojoUrl.port ? ":" + dojoUrl.port : "" ) + dojoUrl.path;
		var uniqueNvps = this.getUniqueNvps(nvps);
		for(var i=0; i<uniqueNvps.length; i++) {
			newUrl += (i === 0 ? "?" : "&") + uniqueNvps[i].n + "=" + uniqueNvps[i].v;
		}
		return newUrl;
	},
	
	getNVPs: function(query) {
		var nvps = [];
		if (query != null) {
			dojo.forEach(query.split("&"), function(encodedNvp) {
				var nvp = encodedNvp.split("=");
				nvps.push({n: nvp[0], v: nvp[1]}); 
			});
		}
		return nvps;
	},
	
	getUniqueNvps: function(nvps) {
		var uniqueNvps = [];
		for(var i=0; i<nvps.length; i++) {
			var exists = this.contains(uniqueNvps, nvps[i].n, nvps[i].v);
			if (!exists) {
				uniqueNvps.push({n: nvps[i].n, v: nvps[i].v}); 
			}
		}
		return uniqueNvps;
	},
	
	contains: function(nvps, n, v) {
 		for(var i=0; i<nvps.length; i++) {
			if(nvps[i].n === n && nvps[i].v === v){
				return true;
			}
		}
  		return false;
	}
};
})();