/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

(function() {
	
dojo.provide("jazz.ui.wizard.ExternalWizardStep");

dojo.require("jazz.ui.wizard.WizardStep");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

/*
 * Provides a generic Wizard Step that takes a source URL and displays it in an iFrame. 
 * It also handles communication between the parent wizard and the step in the iFrame. 
 */

var WizardStep = jazz.ui.wizard.WizardStep;
var STEPS_URI_PREFIX = "/steps/";

dojo.declare("jazz.ui.wizard.ExternalWizardStep", [WizardStep, dijit._Widget, dijit._Templated], {	

	templatePath: dojo.moduleUrl("jazz.ui", "templates/ExternalWizardStep.html"),
	isExternal: true,
	
	constructor: function(args) {
		this.stepTitle = args.displayName;
		this.srcURL = args.stepURL;
		this.customAllowFinish = args.overrideFinish;
	},
	
	setStepPosition: function(position) {
		this.position = position;
		
		//Build a parent URL to send to the iFrame
		var url = self.location.href;
		var i = url.indexOf("#");
		var newHash =  "#" + STEPS_URI_PREFIX + position;
		this.parentURL = i != -1 ? url.substr(0,i) + newHash : url + newHash;
	},
	
	focusStep: function() {
		if(!this.iFrame) {
			this.iFrame = dojo.create("iframe", { 
				src: this.srcURL + "&parent=" + encodeURIComponent(this.parentURL),    
				marginWidth: 0,    
				marginHeight: 0,    
				frameBorder: 0,    
				style: {        
					width: "100%",        
					height: "1000px",        
					border: "none",        
					overflowX: "hidden"    
				}
			}, this.iFrameContainer);
		}
	},

	checkNavigationStatus: function() {
		// Checks the URL to determine the state of the navigation controls
		var qstr = self.location.hash.substr(1);
		if (qstr.length) {
			var qparts = qstr.split("&");
			for (var x = 0; x < qparts.length; x++) {
				var tp = qparts[x].split("=");
				if (tp[0] == "cn") { //can go next
					this.iFrameCanNext = decodeURIComponent(tp[1])*1;
				}
				else if (tp[0] == "cp") { //can go previous
					this.iFrameCanPrevious = decodeURIComponent(tp[1])*1;
				}
				else if (tp[0] == "cf") { //can go finish
					this.iFrameCanFinish = decodeURIComponent(tp[1])*1;
				}
				else if (tp[0] == "h") { //body content height
					var h = parseInt(tp[1]) + 60; //extra adjustment to offset weird padding
					this.iFrame.style.height = h + "px";
				}
				else if (tp[0] == "pn") { //execute processNext
					//strip this value from the URL
					qparts.splice(x,1);
					self.location.hash = qparts.join("&");
					
					var yesGoNext = decodeURIComponent(tp[1])*1;
					if(yesGoNext) {
						this.iFrame.src = this.srcURL + "&parent=" + encodeURIComponent(self.location.href);	
						this.wizard.processNext();
					} 
				}
				else if (tp[0] == "pp") { //execute processPrevious
					//strip this value from the URL
					qparts.splice(x,1);
					self.location.hash = qparts.join("&");
					
					var yesGoPrev = decodeURIComponent(tp[1])*1;
					if(yesGoPrev) {
						this.iFrame.src = this.srcURL + "&parent=" + encodeURIComponent(self.location.href);	
						this.wizard.processPrevious();
					} 
				}
			}
		}
		if (this.wizard) {
			this.wizard.updateNavigationControls();
		}
	},
	
	canFlipToNextPage: function () {
		return this.iFrameCanNext;		
	},
	canFinish: function() {
		return this.iFrameCanFinish || this.customAllowFinish;
	},
	canCancel: function() {
		return true;
	},
	canFlipToPreviousPage: function() {
		return this.iFrameCanPrevious;
	},
	onNext: function() {
		this.iFrame.src = this.srcURL + "&next=true&parent=" + encodeURIComponent(self.location.href);	
	},
	onPrevious: function() {
		this.iFrame.src = this.srcURL + "&previous=true&parent=" + encodeURIComponent(self.location.href);	
	}
	
});		
})();