/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("jazz.ui.wizard.WizardStep");

/*
 * Provides the default implementation of a Wizard Step as expected by the Wizard.
 */

dojo.declare("jazz.ui.wizard.WizardStep", null, {
	

	// label displayed in wizard list
	stepTitle: "defaultTitle", 
	
	// Flags to specify whether UI features should be included for this step	
	includeNavigationUI: true, 
	enabled: false,  // makes the top navigation link active when the wizard is loaded
	inStepList: true,  // includes stepTitle in top navigation list
	
	// Functions to determine enabled/disabled state of navigation buttons
	canFlipToNextPage: function () {
		return this.validate().length === 0;
	},
	canFinish: function() {
		return false;
	},
	canCancel: function() {
		return true;
	},
	canFlipToPreviousPage: function() {
		return true;
	},
	
	triggerButtonStateUpdate: function() {
		if (this.wizard) {
			this.wizard.updateNavigationControls();
		}
	},
	
	// Functions to handle user interaction
	focusStep: function() {
		/*
		 * Intentionally left blank in default Wizard Step.  Step can provide 
		 * implementation of a function to be called when this step becomes in view.
		 */ 
	},
	
	trySelect: function(e) {
		dojo.stopEvent(e);
		if (this.wizard) {
			this.wizard.trySelectStep(this.position);
		}
	},
	
	// Functions which provide an optional hook function
	onNext: function() {
		/*
		 * Step can provide implementation of the customOnNext function to be called when Next is 
		 * processed on this step.  If onNext needs to be asynchronous, the individual step should
		 * have an onNext function and invoke wizard.processNext when ready.
		 */ 
		 if (this.customOnNext && dojo.isFunction(this.customOnNext)) {
         	this.customOnNext();
         }
		 this.wizard.processNext();
	},
	onCancel: function() {
		/*
		 * Step can provide implementation of the customOnCancel function to be called when Cancel is 
		 * processed on this step.  If onCancel needs to be asynchronous, the individual step should
		 * have an onCancel function and invoke wizard.processCancel when ready.
		 */ 
		 if (this.customOnCancel && dojo.isFunction(this.customOnCancel)) {
         	this.customOnCancel();
         }
		 this.wizard.processCancel();
	},
	onFinish: function() {
		/*
		 * Step can provide implementation of the customOnFinish function to be called when Finish is 
		 * processed on this step.  If onFinish needs to be asynchronous, the individual step should
		 * have an onFinish function and invoke wizard.processFinish when ready.
		 */ 
		 if (this.customOnFinish && dojo.isFunction(this.customOnFinish)) {
         	this.customOnFinish();
         }
		 this.wizard.processFinish();
	},
	onPrevious: function () {
		/*
		 * Step can provide implementation of the customOnPrevious function to be called when Previous is 
		 * processed on this step.  If onPrevious needs to be asynchronous, the individual step should
		 * have an onPrevious function and invoke wizard.processPrevious when ready.
		 */ 
		 if (this.customOnPrevious && dojo.isFunction(this.customOnPrevious)) {
         	this.customOnPrevious();
         }
		 this.wizard.processPrevious();
	},
	validate: function() {
		/*
		 * Intentionally returns blank array in default Wizard Step.  Step can 
		 * return errors, warnings if the state of this step is invalid.  
		 */ 
		return [];
	}, 
	
	finalStep: function () {
		if (this.position && this.wizard) {
			return this.position === this.wizard._steps.length;
		} 
		return false;
	}
});		
})();