/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("jazz.ui.wizard.WizardStepList");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

function getVerticalStep(step, index, active) {
	var stepTitle = step.stepTitle;
	if(!stepTitle) {
		console.error("Step title cannot be empty"); //$NON-NLS-1$
		return;
	}
	var div = dojo.create("div", { id: "stepNum_" + index, className: active ? "activeStep" : "inactiveStep" , title: stepTitle});
	dojo.create("span", { innerHTML: stepTitle }, div);
	step.connect(div, 'onclick', 'trySelect');
	return div;
}

function getHorizontalStep(step, index, enabled) {
	var stepTitle = step.stepTitle;
	if(!stepTitle) {
		console.error("Step title cannot be empty"); //$NON-NLS-1$
		return;
	}
	var html = "<span class=\"number\">" + index + "</span><span class=\"sectionLabel\">" + stepTitle +"</span>";
	if(enabled) {
		var link = dojo.create("a", { title: stepTitle, innerHTML: html });
		step.connect(link, 'onclick', 'trySelect');
		return link;	
	} else {
		return dojo.create("div", {className: "inactiveStep", title: stepTitle, innerHTML: html });	
	}
}

function createStepGroup(groupTitle, widget) {
	var div = dojo.create("div", {className: "stepGroup" });
	var section = dojo.create("div", {className: "section", innerHTML: groupTitle },div);
	widget.connect(section, "onclick", function(e) {
		dojo.stopEvent(e);
		dojo.toggleClass(div, "collapsed");
	});
	return div;
}

dojo.declare("jazz.ui.wizard.WizardStepList", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("jazz.ui", "templates/WizardStepList.html"),
	
	postCreate: function() {
		if(this.orientation.toLowerCase() == "horizontal") {
			this.isHorizontal = true;
			dojo.addClass(this.wizardSteps, "horizontal");
		} else {
			this.isHorizontal = false;
			dojo.addClass(this.wizardSteps, "vertical");
		}
		this.lastIndex = 0;
	},
	
	addStep: function(step, groupNode){
		var index = this.lastIndex + 1;
		this.lastIndex++;
		var stepSummary;
		if(this.isHorizontal) {
			stepSummary = getHorizontalStep(step, index, step.enabled);
		} else {
			stepSummary = getVerticalStep(step, index, step.enabled);
		}
		stepSummary.position = step.position;
		if(groupNode) {
			groupNode.appendChild(stepSummary);
			step._groupNode = groupNode;
		} else {
			this.wizardSteps.appendChild(stepSummary);
		}
		step._stepListDOM = stepSummary;
		return step;
	}, 
	
	removeStep: function(step) {
		if(step._groupNode) {
			step._groupNode.removeChild(step._stepListDOM);
		}
		else if(step && step.position > 0) {
			var elem = dojo.byId("stepNum_" + step.position);
			if(elem) {
				this.wizardSteps.removeChild(elem);
			}
		}
	},
	
	removeGroup: function(group) {
		if(group && group.isGroup) {
			this.wizardSteps.removeChild(group);
		}
	},
	
	enableStep: function (step) {
		var steps = this.wizardSteps.getElementsByTagName("DIV");
		var oldStep;
		for (var i=0; i < steps.length; i++) {
		 	if (steps[i].position === step.position) {
				oldStep = steps[i];
				break;			
			}
		}
		if(this.isHorizontal) {
			var newStep = getHorizontalStep(step, step.position, true);
			newStep.position = step.position;
			oldStep.parentNode.replaceChild(newStep, oldStep);
		} else {
			dojo.removeClass(oldStep, "inactiveStep");
			dojo.addClass(oldStep, "activeStep");
		}
		step.enabled = true;
	},
	
	selectStep: function(step) {
		if (!step.enabled) {
			this.enableStep(step);
		}
		if (this.isHorizontal) {
			var allLinks = this.wizardSteps.getElementsByTagName("A");
		} else {
			var allLinks = this.wizardSteps.getElementsByTagName("DIV");
		}
		for (var i = 0; i < allLinks.length; i++) {
			if (step.position === allLinks[i].position) {
				dojo.addClass(allLinks[i], "selected");
			} else {
				dojo.removeClass(allLinks[i], "selected");
			} 
		}
	},
	
	//Currently only vertical step lists are supported for step groups
	addStepGroup: function(name) {
		if (!this.isHorizontal && name && typeof name == "string") {
			var node = createStepGroup(name, this);
			node.isGroup = true;
			this.wizardSteps.appendChild(node);
			return node;
		}
	}

});

var WizardStepList = jazz.ui.wizard.WizardStepList;

})();